/*
 * File Schedules.cpp.
 *
 * This file is part of the source code of the software program
 * Vampire. It is protected by applicable
 * copyright laws.
 *
 * This source code is distributed under the licence found here
 * https://vprover.github.io/license.html
 * and in the source directory
 *
 * In summary, you are allowed to use Vampire for non-commercial
 * purposes but not allowed to distribute, modify, copy, create derivatives,
 * or use in competitions. 
 * For other uses of Vampire please contact developers for a different
 * licence, which we will make an effort to provide. 
 */
/**
 * @file Schedules.cpp
 * Implements class Schedules.
 * @since 09/15/2017
 * @author Martin Suda
 */

#include "Schedules.hpp"

using namespace Lib;
using namespace Shell;
using namespace CASC;

void Schedules::getLtb2014MzrSchedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category();
  unsigned prop = property.props();
  unsigned atoms = property.atoms();

  switch (cat) {
  case Property::NEQ:
    if (prop == 131075) {
      quick.push("dis+2_64_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=4:ptb=off:sos=on:spl=sat:sser=off:ssfp=100000:ssfq=1.4:ssnc=none_4");
      quick.push("dis+10_14_bs=off:cond=fast:drc=off:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=3.0:sac=on:spo=on:sp=occurrence_4");
      quick.push("dis+1011_24_cond=fast:drc=off:nwc=10:nicw=on:ptb=off:ssec=off:spl=sat_14");
      quick.push("dis-1010_5_bs=off:bsr=unit_only:cond=fast:ep=on:gsp=input_only:lcm=reverse:nwc=1:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=backtracking:sp=occurrence_54");
      quick.push("dis+1011_28_bs=off:cond=fast:drc=off:gs=on:nwc=1.1:ptb=off:ssec=off:spl=sat_8");
      quick.push("dis-1002_2:1_bs=off:drc=off:ep=on:fde=none:gsp=input_only:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=all:sp=occurrence_16");
      quick.push("dis+4_128_bs=off:drc=off:fde=none:nwc=1:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking:sp=reverse_arity_1");
      quick.push("dis+1011_7_bs=off:drc=off:ep=RS:fde=none:gsp=input_only:nwc=10:ptb=off:ssec=off:spl=sat:sser=off:ssfp=40000:ssfq=1.2:ssnc=none_6");
      quick.push("ott+10_50_bs=off:br=off:cond=fast:drc=off:flr=on:gs=on:nwc=4:nicw=on:ptb=off:ssec=off:sio=off:spl=sat:ssfq=2.0:ssnc=all:sp=reverse_arity:urr=on_2");
      quick.push("dis+11_4_ep=on:fde=none:lcm=reverse:nwc=10:nicw=on:ptb=off:ssec=off:sos=on:spo=on:spl=sat_42");
      quick.push("ins-11_24_bs=off:cond=fast:ep=RSTC:flr=on:gsp=input_only:gs=on:igbrr=0.7:igrr=1/4:igrpq=1.3:igwr=on:lcm=reverse:nwc=2.5:ptb=off:ssec=off:sio=off:spl=off:urr=on_3");
      quick.push("dis+11_24_bs=off:cond=on:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=1.7:ptb=off:ssec=off:spl=sat:sp=reverse_arity_26");
      quick.push("ott+2_40_bsr=unit_only:drc=off:fsr=off:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sio=off:urr=on_46");
      quick.push("dis+1011_128_bs=unit_only:cond=fast:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:spl=sat:sfv=off_244");
      quick.push("dis+11_5:1_bs=off:drc=off:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=all:sgo=on:sio=off:spo=on:spl=sat:sp=occurrence_213");
      quick.push("dis-1010_3:2_bs=off:bsr=unit_only:drc=off:ep=RST:fde=none:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=on:sgo=on:spo=on:spl=backtracking:sp=reverse_arity_262");
      quick.push("ott+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=sat_4");
    }
    else if (prop == 3) {
      if (atoms < 2500) {
        quick.push("dis+10_14_bs=off:cond=fast:drc=off:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=3.0:sac=on:spo=on:sp=occurrence_3");
        quick.push("ott-11_8:1_bd=off:bs=off:drc=off:ep=on:fde=none:lcm=predicate:nwc=3:ptb=off:ssec=off:sos=on:sio=off:urr=on_29");
        quick.push("dis-1002_5:4_bs=off:drc=off:ep=RST:flr=on:fde=none:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sgo=on:spo=on:spl=sat:sp=reverse_arity:updr=off_4");
        quick.push("ott+10_2:3_bd=off:bs=off:cond=fast:drc=off:fde=none:lcm=predicate:nwc=10:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking:sp=occurrence_2");
        quick.push("dis+11_3:1_bs=off:drc=off:ep=on:nwc=10:ptb=off:ssec=off:spl=sat:urr=on_8");
        quick.push("ins-10_40_bs=off:cond=fast:ep=RSTC:flr=on:gsp=input_only:gs=on:igbrr=0.7:igrr=1/4:igrp=4000:igrpq=1.2:igwr=on:lcm=reverse:nwc=1.7:ptb=off:ssec=off:sio=off:spl=off:urr=on_30");
        quick.push("dis+11_5_bs=off:cond=fast:drc=off:ep=on:fde=none:gsp=input_only:lcm=reverse:nwc=1:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking:sp=occurrence_14");
        quick.push("dis+1011_4:1_bs=off:bsr=on:drc=off:ep=on:flr=on:gsp=input_only:gs=on:lcm=reverse:nwc=2:ptb=off:ssec=off:sos=all:spo=on:spl=sat:ssfp=10000:ssnc=none:sp=occurrence_5");
        quick.push("ott+1011_5_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=2:ptb=off:ssec=off:sos=on:sio=off:sp=reverse_arity:updr=off_7");
        quick.push("ott+1011_2:3_bd=off:bs=off:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sac=on:sgo=on:sio=off:spl=backtracking:sfv=off_61");
        quick.push("ins-11_24_bs=off:cond=fast:ep=RSTC:flr=on:gsp=input_only:gs=on:igbrr=0.7:igrr=1/4:igrpq=1.3:igwr=on:lcm=reverse:nwc=2.5:ptb=off:ssec=off:sio=off:spl=off:urr=on_7");
        quick.push("dis+1011_64_bs=off:drc=off:ep=on:flr=on:nwc=2:ptb=off:ssec=off:spl=sat:sp=occurrence_48");
        quick.push("ott+1011_3_bs=off:drc=off:ep=on:fde=none:nwc=1.1:nicw=on:ptb=off:ssec=off:spl=sat_58");
        quick.push("dis+1011_28_bs=off:cond=fast:drc=off:gs=on:nwc=1.1:ptb=off:ssec=off:spl=sat_40");
        quick.push("dis+11_40_bs=off:bms=on:drc=off:fde=none:gs=on:lcm=predicate:nwc=1.3:sd=4:ss=axioms:sac=on:spo=on:sp=occurrence_1");
        quick.push("dis+1010_4:1_bs=off:bms=on:drc=off:ep=RST:nwc=4:sswn=on:sos=on:sgo=on:spo=on_1");
        quick.push("dis+1011_128_bs=unit_only:cond=fast:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:spl=sat:sfv=off_56");
        quick.push("ott+10_2_bd=off:bs=off:drc=off:ecs=on:flr=on:fsr=off:nwc=1.7:ssec=off:sac=on:sgo=on:sio=off_95");
        quick.push("dis+1011_2_bs=off:drc=off:ep=on:flr=on:nwc=1.1:nicw=on:ptb=off:ssec=off:spl=sat:urr=on:updr=off_154");
        quick.push("dis+1011_2_bs=off:cond=on:drc=off:ep=on:gsp=input_only:nwc=1.5:nicw=on:ptb=off:ssec=off:spl=sat:urr=on_152");
        quick.push("dis+11_8:1_bs=off:br=off:drc=off:ep=RST:nwc=1:ptb=off:ssec=off:sgo=on:sio=off:spo=on:spl=backtracking:urr=on_160");
        quick.push("ott+2_3_bd=off:cond=on:drc=off:flr=on:nwc=1.3:ptb=off:ssec=off:sac=on:sgo=on:sio=off:sfv=off:sp=reverse_arity:updr=off_216");
        quick.push("dis-1010_3:2_bs=off:bsr=unit_only:drc=off:ep=RST:fde=none:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=on:sgo=on:spo=on:spl=backtracking:sp=reverse_arity_28");
        quick.push("dis-1010_3:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.7:nicw=on:ptb=off:spl=sat:sser=off:ssfp=100000:ssfq=1.1:ssnc=none_39");
        quick.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:gsp=input_only:nwc=4:ptb=off:ssec=off:sos=all:sio=off:spo=on:urr=on:updr=off_40");
        quick.push("ott+1_1_bd=off:bs=off:bsr=unit_only:br=off:cond=on:drc=off:ep=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sd=4:ss=axioms:st=2.0:sio=off:urr=on_77");
      }
      else {
        quick.push("ott-1010_3_bd=off:bs=off:cond=on:drc=off:ep=on:fde=none:nwc=10:ptb=off:ssec=off:sd=3:ss=axioms:sos=on:sac=on:sio=off:spl=backtracking:sp=reverse_arity:urr=on_11");
        quick.push("dis-1010_3_cond=fast:drc=off:nwc=4:sac=on_19");
        quick.push("ott+1011_5_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=2:ptb=off:ssec=off:sos=on:sio=off:sp=reverse_arity:updr=off_13");
        quick.push("dis+11_5:1_bs=off:drc=off:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=all:sgo=on:sio=off:spo=on:spl=sat:sp=occurrence_29");
        quick.push("dis+4_8:1_bs=off:bsr=on:drc=off:fsr=off:fde=none:lcm=reverse:nwc=10:ptb=off:ssec=off:spl=sat:ssfp=10000:ssfq=1.0:ssnc=none:sp=reverse_arity_16");
        quick.push("dis+11_12_bs=off:drc=off:ep=on:flr=on:fsr=off:gsp=input_only:lcm=predicate:nwc=3:nicw=on:ptb=off:ssec=off:sos=on:spl=sat_1");
        quick.push("ott+11_3_bs=off:br=off:drc=off:nwc=1.1:nicw=on:ss=axioms:sos=on:sio=off:urr=on_2");
        quick.push("ott+1011_3_bs=off:drc=off:ep=on:fde=none:nwc=1.1:nicw=on:ptb=off:ssec=off:spl=sat_86");
        quick.push("ins-1010_8:1_bs=off:ep=RSTC:fsr=off:igbrr=1.0:igrr=1/128:igrp=200:igrpq=2.0:igwr=on:nwc=1.2:ptb=off:ssec=off:sos=on:sio=off:spl=off_86");
        quick.push("dis+10_14_bs=off:cond=fast:drc=off:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=3.0:sac=on:spo=on:sp=occurrence_115");
        quick.push("ott+1_10_bd=off:bs=off:drc=off:lcm=predicate:nwc=2.5:ptb=off:ssec=off:sos=on:spl=sat_113");
        quick.push("ott+11_2_bd=off:bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=10:ptb=off:ssec=off:spl=sat:sp=reverse_arity_246");
        quick.push("dis+1_8:1_bs=off:cond=on:drc=off:ep=RST:flr=on:fsr=off:gsp=input_only:lcm=reverse:nwc=3:ptb=off:ssec=off:sio=off:urr=on_259");
        quick.push("lrs+2_3:1_bs=off:br=off:drc=off:flr=on:lcm=predicate:nwc=10:ssec=off:stl=60:sgo=on:sio=off:spo=on:sp=occurrence:urr=on_260");
        quick.push("dis+2_4:1_bs=off:cond=on:drc=off:ecs=on:ep=on:gs=on:lcm=reverse:nwc=1.5:ssec=off:sos=on:sio=off:ssfp=1000:ssfq=1.1:ssnc=none:urr=on_43");
      }
    }
    else if (prop == 131079) {
      quick.push("dis+11_5:1_bs=off:drc=off:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=all:sgo=on:sio=off:spo=on:spl=sat:sp=occurrence_1");
      quick.push("dis+11_5:4_bs=off:bsr=on:cond=on:drc=off:fde=none:lcm=reverse:nwc=1.3:nicw=on:ptb=off:ssec=off:spl=sat:sscc=on:sser=off:ssfp=40000:ssfq=1.0_8");
      quick.push("dis+11_50_bs=off:cond=fast:drc=off:fde=none:gs=on:lcm=predicate:nwc=2:nicw=on:ssec=off:spo=on:sp=reverse_arity_1");
      quick.push("dis+11_5:1_bs=off:cond=fast:drc=off:nwc=10:sswn=on:sswsr=on_5");
      quick.push("ott+11_5_bs=off:ep=on:fde=none:nwc=1.5:nicw=on:ptb=off:ssec=off:sswsr=on:sac=on:sio=off:spo=on:spl=backtracking:sp=reverse_arity_79");
      quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1:ptb=off:ssec=off:sos=on:sgo=on:sio=off:sp=occurrence_18");
      quick.push("dis+1011_28_bs=off:cond=fast:drc=off:gs=on:nwc=1.1:ptb=off:ssec=off:spl=sat_11");
      quick.push("ott+1011_6_drc=off:ep=on:fde=none:gs=on:nwc=1.3:nicw=on:ptb=off:ssec=off:sswsr=on:sac=on:sio=off:spo=on:spl=backtracking:sfv=off_5");
      quick.push("ott+2_8:1_bd=off:bs=off:bsr=unit_only:bms=on:br=off:cond=fast:drc=off:nwc=1.5:ssec=off:sos=on:sio=off:urr=on:updr=off_7");
      quick.push("ott+1011_3_bs=off:bsr=unit_only:ep=on:nwc=2:ptb=off:ssec=off:spl=sat_27");
      quick.push("dis+1011_16_bd=off:bs=off:cond=on:drc=off:ep=on:fsr=off:gs=on:nwc=2.5:ptb=off:ssec=off:sgo=on:spo=on:spl=backtracking_5");
      quick.push("dis+1011_12_bs=off:drc=off:fsr=off:nwc=5:nicw=on:ptb=off:ssec=off:sos=on:spl=sat_28");
      quick.push("ott+11_2_bd=off:bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=10:ptb=off:ssec=off:spl=sat:sp=reverse_arity_185");
      quick.push("dis+1011_24_cond=fast:drc=off:nwc=10:nicw=on:ptb=off:ssec=off:spl=sat_129");
      quick.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:gsp=input_only:nwc=4:ptb=off:ssec=off:sos=all:sio=off:spo=on:urr=on:updr=off_91");
      quick.push("ott+11_2:3_bd=off:bs=unit_only:bsr=unit_only:cond=on:drc=off:fde=none:lcm=reverse:nwc=1.2:ptb=off:ssec=off:spl=sat_94");
      quick.push("ott+3_8:1_bd=off:bs=off:bsr=unit_only:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sos=all:spl=sat:sser=off:ssfp=10000:ssfq=1.4:ssnc=none_149");
      quick.push("ott+11_8:1_drc=off:ep=on:nwc=1:ptb=off:ssec=off:sac=on:spo=on:sp=occurrence_471");
      quick.push("dis+1011_4:1_bs=off:bsr=on:drc=off:ep=on:flr=on:gsp=input_only:gs=on:lcm=reverse:nwc=2:ptb=off:ssec=off:sos=all:spo=on:spl=sat:ssfp=10000:ssnc=none:sp=occurrence_1");
    }
    else if (prop == 1) {
      quick.push("ott+10_2:3_bd=off:bs=off:cond=fast:drc=off:fde=none:lcm=predicate:nwc=10:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking:sp=occurrence_1");
      quick.push("dis+10_6_bs=off:br=off:cond=on:drc=off:ep=RSTC:gs=on:nwc=1.1:nicw=on:ptb=off:ssec=off:sos=all:sio=off:spl=sat:urr=on_2");
      quick.push("dis+1011_16_bd=off:bs=off:cond=on:drc=off:ep=on:fsr=off:gs=on:nwc=2.5:ptb=off:ssec=off:sgo=on:spo=on:spl=backtracking_24");
      quick.push("ott+11_2_bd=off:bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=10:ptb=off:ssec=off:spl=sat:sp=reverse_arity_6");
      quick.push("ins+1_8:1_bs=off:cond=on:ep=RSTC:flr=on:gs=on:igbrr=0.8:igrr=1/4:igrp=1400:igrpq=1.05:igwr=on:nwc=2.5:ptb=off:ssec=off:sio=off:spl=off:urr=on_3");
      quick.push("ins+1011_3_bs=off:ep=RSTC:fde=none:gs=on:igbrr=1.0:igrr=1/64:igrp=100:igrpq=1.3:igwr=on:nwc=2.5:ptb=off:ssec=off:sos=on:sio=off:spl=off_2");
      quick.push("ott-11_40_bd=off:bs=off:drc=off:ecs=on:flr=on:fsr=off:lcm=predicate:nwc=1.5:nicw=on:ssec=off:sos=on_42");
      quick.push("dis+11_3_bs=off:bms=on:cond=fast:drc=off:gsp=input_only:lcm=reverse:nwc=3:ssec=off:sos=on:sac=on:spo=on:sp=occurrence_2");
      quick.push("lrs+10_20_bd=off:bs=off:cond=fast:drc=off:ecs=on:nwc=1.5:ptb=off:ssec=off:stl=30:sio=off:spo=on:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:updr=off_18");
      quick.push("ott-11_2:3_bd=off:bs=off:lcm=predicate:nwc=5:nicw=on:ptb=off:ssec=off:spl=sat:sp=reverse_arity_14");
      quick.push("dis+11_8:1_bs=off:br=off:drc=off:ep=RST:nwc=1:ptb=off:ssec=off:sgo=on:sio=off:spo=on:spl=backtracking:urr=on_29");
      quick.push("dis+11_5:1_bs=off:drc=off:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=all:sgo=on:sio=off:spo=on:spl=sat:sp=occurrence_123");
      quick.push("ott+10_28_bd=off:bs=off:drc=off:nwc=1.5:ptb=off:ssec=off:spl=sat_21");
      quick.push("dis+11_50_bs=off:drc=off:gsp=input_only:nwc=1.3:nicw=on:ptb=off:ssec=off:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none_35");
      quick.push("ott+11_50_bd=off:bs=off:drc=off:ep=on:lcm=reverse:nwc=1.7:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=sat:sp=occurrence_53");
      quick.push("dis-1010_7_bs=off:cond=fast:drc=off:fsr=off:gsp=input_only:gs=on:nwc=1.1:ptb=off:ssec=off:sio=off:spo=on:urr=on_21");
      quick.push("dis+10_1024_bs=off:cond=on:drc=off:flr=on:fsr=off:lcm=predicate:nwc=1.2:nicw=on:ptb=off:ssec=off:sac=on:spo=on_110");
      quick.push("dis+1011_40_bs=off:cond=on:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sos=on:sagn=off:sgo=on:sio=off:spl=sat:sser=off:ssfp=4000:ssnc=none_51");
      quick.push("lrs-1002_4:1_bs=off:gs=on:nwc=4:nicw=on:ssec=off:stl=60:sgo=on:spo=on:urr=on_96");
      quick.push("ott-1010_8:1_bd=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sos=on:spl=sat:ssfp=4000:ssfq=1.4:ssnc=none:urr=on_19");
      quick.push("lrs+1011_20_bd=off:bs=off:ecs=on:ep=on:fde=none:lcm=predicate:nwc=3:nicw=on:ssec=off:stl=60:sio=off:spl=off:urr=on_214");
      quick.push("dis+11_12_bs=off:drc=off:ep=on:flr=on:fsr=off:gsp=input_only:lcm=predicate:nwc=3:nicw=on:ptb=off:ssec=off:sos=on:spl=sat_142");
      quick.push("lrs+3_64_bs=off:drc=off:fsr=off:gsp=input_only:nwc=5:nicw=on:stl=60:sac=on:sgo=on_307");
      quick.push("ott+10_64_bd=off:bs=off:cond=fast:drc=off:flr=on:gsp=input_only:lcm=predicate:nwc=2:nicw=on:ptb=off:ssec=off:sac=on:spo=on_220");
      quick.push("dis+11_5_bs=off:cond=fast:drc=off:ep=on:fde=none:gsp=input_only:lcm=reverse:nwc=1:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking:sp=occurrence_1");
      quick.push("ott+3_8:1_bd=off:bs=off:bsr=unit_only:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sos=all:spl=sat:sser=off:ssfp=10000:ssfq=1.4:ssnc=none_6");
      quick.push("lrs+2_3:1_bs=off:br=off:drc=off:flr=on:lcm=predicate:nwc=10:ssec=off:stl=60:sgo=on:sio=off:spo=on:sp=occurrence:urr=on_15");
      quick.push("ott+10_2_bd=off:bs=off:drc=off:ecs=on:flr=on:fsr=off:nwc=1.7:ssec=off:sac=on:sgo=on:sio=off_68");
      quick.push("ott+11_2:3_bd=off:bs=unit_only:bsr=unit_only:cond=on:drc=off:fde=none:lcm=reverse:nwc=1.2:ptb=off:ssec=off:spl=sat_106");
    }
    else {
      quick.push("ott+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=sat_4");
      quick.push("ott+10_2:3_bd=off:bs=off:cond=fast:drc=off:fde=none:lcm=predicate:nwc=10:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking:sp=occurrence_3");
      quick.push("dis-1002_5:4_bs=off:drc=off:ep=RST:flr=on:fde=none:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sgo=on:spo=on:spl=sat:sp=reverse_arity:updr=off_10");
      quick.push("dis+11_5_bs=off:cond=fast:drc=off:ep=on:fde=none:gsp=input_only:lcm=reverse:nwc=1:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking:sp=occurrence_2");
      quick.push("dis+11_3_bs=off:bms=on:cond=fast:drc=off:gsp=input_only:lcm=reverse:nwc=3:ssec=off:sos=on:sac=on:spo=on:sp=occurrence_1");
      quick.push("ott+10_50_bs=off:br=off:cond=fast:drc=off:flr=on:gs=on:nwc=4:nicw=on:ptb=off:ssec=off:sio=off:spl=sat:ssfq=2.0:ssnc=all:sp=reverse_arity:urr=on_5");
      quick.push("dis-2_50_bs=off:drc=off:lcm=predicate:nwc=1.7:ptb=off:ssec=off:sio=off:spl=backtracking:sp=occurrence_6");
      quick.push("dis+11_24_bs=off:bms=on:cond=on:drc=off:ecs=on:fde=none:nwc=2.5:nicw=on:ssec=off:spo=on:sp=occurrence:urr=on_1");
      quick.push("ott+1011_3_bs=off:drc=off:ep=on:fde=none:nwc=1.1:nicw=on:ptb=off:ssec=off:spl=sat_11");
      quick.push("dis-1010_3:2_bs=off:bsr=unit_only:drc=off:ep=RST:fde=none:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=on:sgo=on:spo=on:spl=backtracking:sp=reverse_arity_22");
      quick.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:gsp=input_only:nwc=4:ptb=off:ssec=off:sos=all:sio=off:spo=on:urr=on:updr=off_3");
      quick.push("dis+10_5_bd=off:bs=off:cond=fast:ep=RST:gsp=input_only:lcm=predicate:nwc=5:ptb=off:ssec=off:sos=all:sac=on:sio=off:spl=backtracking:sfv=off:urr=on_10");
      quick.push("ott-1002_2:1_bd=off:bs=off:bsr=on:drc=off:ep=on:flr=on:fsr=off:lcm=predicate:nwc=2.5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:urr=on_51");
      quick.push("ins+1011_3_bs=off:ep=RSTC:fde=none:gs=on:igbrr=1.0:igrr=1/64:igrp=100:igrpq=1.3:igwr=on:nwc=2.5:ptb=off:ssec=off:sos=on:sio=off:spl=off_66");
    }
    break;

  case Property::HEQ:
    if (prop == 2) {
      quick.push("dis+1_2:1_bs=off:cond=on:drc=off:nwc=5:sos=on:sio=off:spo=on:sp=occurrence_2");
      quick.push("ott+1_3_bs=off:cond=on:drc=off:fde=none:nwc=3:ptb=off:ssec=off:sos=all:sio=off:spl=sat_60");
      quick.push("ott+11_64_bs=off:drc=off:ecs=on:gs=on:nwc=1.5:ssec=off:sio=off:spl=off_746");
      quick.push("ins+11_50_bs=off:drc=off:fde=none:igbrr=0.7:igrr=1/32:igrp=700:igrpq=1.2:igwr=on:nwc=3:ptb=off:sio=off:spl=off:sp=reverse_arity_1045");
    }
    else if (prop == 0) {
      quick.push("dis+2_8_bd=off:bs=off:ep=RST:fsr=off:lcm=reverse:nwc=3:nicw=on:ptb=off:ssec=off:spo=on:spl=backtracking:sfv=off_2");
      quick.push("lrs+11_1_bd=off:bs=off:cond=on:drc=off:gs=on:nwc=2:ptb=off:stl=30:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none_259");
      quick.push("lrs-1_2:1_bs=off:drc=off:gs=on:nwc=1.1:nicw=on:ssec=off:stl=50:sio=off:spl=off:updr=off_175");
      quick.push("ott+11_50_bs=off:cond=on:drc=off:flr=on:fde=none:lcm=predicate:nwc=2:nicw=on:ptb=off:sgo=on:sio=off:spl=backtracking:sp=occurrence:urr=on_1127");
    }
    else {
      quick.push("dis-1010_3_bs=off:drc=off:lcm=predicate:nwc=10:ptb=off:ssec=off:spo=on_2");
      quick.push("ott+11_50_bs=off:cond=on:drc=off:flr=on:fde=none:lcm=predicate:nwc=2:nicw=on:ptb=off:sgo=on:sio=off:spl=backtracking:sp=occurrence:urr=on_67");
      quick.push("dis+2_128_bs=off:drc=off:lcm=predicate:nwc=10:sac=on:sio=off:sp=occurrence_199");
      quick.push("ott+11_40_bd=off:bs=off:cond=on:drc=off:nwc=3:nicw=on:ptb=off:ssec=off:spl=sat:sp=reverse_arity_2");
      quick.push("lrs+11_1_bd=off:bs=off:cond=on:drc=off:gs=on:nwc=2:ptb=off:stl=30:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none_66");
      quick.push("lrs+11_4:1_bs=unit_only:cond=fast:drc=off:fde=none:lcm=predicate:nwc=10:nicw=on:ptb=off:stl=60:sac=on:spo=on:spl=sat:ssac=none:ssfp=1000:ssfq=1.0:ssnc=all_191");
      quick.push("lrs+2_14_bs=off:cond=fast:drc=off:nwc=10:ptb=off:stl=90:sac=on:sio=off:spo=on:spl=sat:ssac=none:ssfp=100000:ssfq=1.1:ssnc=all_dependent:sp=reverse_arity_679");
      quick.push("ott+10_64_bs=off:cond=on:drc=off:ecs=on:flr=on:gsp=input_only:nwc=5:nicw=on:ssec=off:spo=on:sp=reverse_arity:updr=off_367");
      quick.push("dis+11_7_bs=off:drc=off:lcm=reverse:nwc=5:ssec=off:sio=off:spl=off:updr=off_116");
    }
    break;

  case Property::PEQ:
    if (prop == 0) {
      if (atoms > 15) {
        quick.push("dis+11_8_bsr=unit_only:cond=on:drc=off:fsr=off:gs=on:nwc=1.1:nicw=on:ptb=off:ssec=off:sos=all:sio=off:spo=on:spl=sat:sser=off:ssfq=1.4:ssnc=none_87");
        quick.push("ott+1_3_bs=off:bms=on:cond=on:drc=off:nwc=2:sac=on:sio=off:updr=off_635");
        quick.push("ott+10_8:1_drc=off:ecs=on:ep=RSTC:gs=on:nwc=1.3:ssec=off:sac=on:sio=off_31");
        quick.push("dis+10_5:4_cond=on:gs=on:nwc=2:nicw=on:ptb=off:sos=all:sio=off:spl=sat:sscc=on:sser=off:ssfp=1000:ssfq=1.1:ssnc=none:sfv=off:sp=occurrence_324");
        quick.push("dis+4_50_bd=off:bs=unit_only:cond=on:drc=off:gs=on:lcm=predicate:nwc=5:nicw=on:ptb=off:sos=all:sio=off:spl=backtracking:urr=on_379");
        quick.push("dis+1_6_bd=off:flr=on:gs=on:nwc=1.5:ptb=off:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity_330");
        quick.push("dis+2_6_bd=off:drc=off:flr=on:fsr=off:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking_419");
      }
      else {
        quick.push("lrs+1_40_bs=unit_only:cond=on:drc=off:ecs=on:nwc=1:nicw=on:ptb=off:ssec=off:stl=30:sio=off:spo=on:spl=sat:ssfq=1.0:ssnc=none_140");
        quick.push("lrs-3_4_bsr=unit_only:cond=on:drc=off:fsr=off:fde=none:gs=on:nwc=1.2:ssec=off:stl=150:sgo=on:sio=off:urr=on:updr=off_492");
        quick.push("dis+10_10_bs=off:cond=fast:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:sio=off:spo=on:spl=sat:sser=off:ssnc=none_434");
      }
    }
    else if (prop == 1) {
      quick.push("dis+1_12_bs=off:drc=off:ep=on:nwc=1.3:nicw=on:sos=on:sio=off:spl=off_1");
      quick.push("ott+4_5_bd=off:bsr=unit_only:cond=fast:drc=off:fsr=off:gs=on:nwc=1.5:nicw=on:sac=on:sio=off_2");
      quick.push("dis+1_6_bd=off:flr=on:gs=on:nwc=1.5:ptb=off:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity_21");
      quick.push("ott+3_128_bs=off:br=off:cond=fast:drc=off:ep=on:nwc=1:nicw=on:ptb=off:sos=all:sio=off:spo=on:spl=backtracking:urr=on_108");
      quick.push("lrs-3_4_bsr=unit_only:cond=on:drc=off:fsr=off:fde=none:gs=on:nwc=1.2:ssec=off:stl=150:sgo=on:sio=off:urr=on:updr=off_1255");
      quick.push("dis+1_2_bd=off:bs=off:drc=off:flr=on:fsr=off:nwc=10:ptb=off:ssec=off:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_253");
      quick.push("dis+4_50_bd=off:bs=unit_only:cond=on:drc=off:gs=on:lcm=predicate:nwc=5:nicw=on:ptb=off:sos=all:sio=off:spl=backtracking:urr=on_142");
    }
    else if (prop == 2) {
      quick.push("ott+4_6_bs=off:bsr=on:cond=on:drc=off:fsr=off:nwc=3:ssec=off:sswn=on:sac=on:sio=off:sp=occurrence_120");
      quick.push("ott+3_20_bs=off:cond=fast:drc=off:fde=none:nwc=2:ptb=off:ssec=off:sgo=on:sio=off:spo=on:spl=sat:sp=occurrence_157");
      quick.push("ott+4_28_bs=off:cond=on:drc=off:fde=none:gs=on:nwc=1.1:ptb=off:ssec=off:sgo=on:sio=off:spo=on:spl=sat_164");
      quick.push("ott+2_1024_bs=off:drc=off:gsp=input_only:nwc=1.1:ptb=off:ssec=off:spl=sat:ssfq=1.4:ssnc=none:sp=occurrence_371");
    }
    else {
      quick.push("dis-1004_1024_bs=off:cond=fast:drc=off:ep=on:nwc=1.2_96");
      quick.push("lrs+11_2:3_bd=off:drc=off:flr=on:nwc=1.3:nicw=on:ptb=off:stl=90:sio=off:spl=sat:ssnc=none_106");
      quick.push("ins+4_3_cond=fast:flr=on:igbrr=0.6:igrr=1/128:igrp=700:igrpq=1.2:igwr=on:nwc=1.7:ptb=off:sio=off:spl=off:updr=off_116");
      quick.push("lrs-3_4_bsr=unit_only:cond=on:drc=off:fsr=off:fde=none:gs=on:nwc=1.2:ssec=off:stl=150:sgo=on:sio=off:urr=on:updr=off_1133");
    }
    break;

  case Property::HNE:
    if (prop == 8192) {
      if (atoms > 6) {
        quick.push("dis+11_1_bs=off:cond=fast:fsr=off:nwc=10:ptb=off:ssec=off:sio=off:spo=on:spl=sat_56");
        quick.push("ott+4_6_bs=off:bsr=unit_only:cond=fast:nwc=1.2:nicw=on:ptb=off:ssec=off:sio=off:spo=on:spl=sat_281");
        quick.push("dis+4_14_fsr=off:lcm=reverse:nwc=1:sac=on:sio=off_34");
        quick.push("lrs+10_3:1_bs=off:bsr=unit_only:cond=fast:nwc=10:ptb=off:ssec=off:stl=90:sio=off:spo=on_144");
        quick.push("ott+1011_10_bs=off:cond=on:nwc=3:ptb=off:ssec=off:sio=off:spl=backtracking_62");
        quick.push("lrs-1003_28_bs=unit_only:bms=on:cond=on:flr=on:fsr=off:gsp=input_only:nwc=1:nicw=on:ssec=off:stl=60:sio=off:spl=off:sp=occurrence_180");
        quick.push("dis+2_5:4_cond=fast:ecs=on:flr=on:nwc=1.5:ssec=off:sio=off:spl=off_576");
        quick.push("dis+1002_50_bs=off:fsr=off:lcm=predicate:nwc=1.2:nicw=on:ssec=off_392");
        quick.push("lrs+2_5:1_bs=off:cond=fast:flr=on:fsr=off:lcm=predicate:nwc=10:ptb=off:ssec=off:stl=90:spl=backtracking_588");
        quick.push("dis-1004_4_bs=off:bms=on:cond=on:nwc=2:sio=off_52");
      }
      else {
        quick.push("lrs+3_8_bs=unit_only:cond=fast:flr=on:nwc=4:nicw=on:ptb=off:ssec=off:stl=30:sio=off:spl=backtracking_124");
        quick.push("dis+11_64_bs=unit_only:bms=on:lcm=reverse:nwc=1.5:sio=off_33");
        quick.push("lrs+1011_64_bs=unit_only:bsr=unit_only:cond=fast:flr=on:nwc=1:ptb=off:stl=180:sio=off:spo=on:spl=backtracking_658");
        quick.push("lrs+1_1024_bs=off:bms=on:cond=on:ecs=on:nwc=1:nicw=on:ssec=off:stl=90:sac=on:sio=off_761");
      }
    }
    else {
      quick.push("ott+1011_10_bs=off:cond=on:nwc=3:ptb=off:ssec=off:sio=off:spl=backtracking_3");
      quick.push("dis+2_128_bs=off:bms=on:cond=on:ecs=on:gs=on:lcm=reverse:nwc=1.7:ssec=off:sos=on:sio=off_1");
      quick.push("dis+1010_10_bs=off:cond=fast:gs=on:nwc=1.5:ptb=off:ssec=off:sio=off_9");
      quick.push("dis+1_50_bs=off:cond=on:nwc=3:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=occurrence_142");
      quick.push("lrs+10_4:1_bs=off:cond=on:flr=on:nwc=10:ssec=off:stl=50:sio=off:spl=off:sp=reverse_arity_63");
      quick.push("dis+1011_20_bs=off:fsr=off:nwc=2:sio=off:spl=off_201");
      quick.push("ott+1_8:1_bs=off:br=off:cond=on:gs=on:nwc=3:ptb=off:ssec=off:sio=off:spl=sat:urr=on_227");
      quick.push("lrs+10_3:1_bs=off:bsr=unit_only:cond=fast:nwc=10:ptb=off:ssec=off:stl=90:sio=off:spo=on_4");
    }
    break;

  case Property::NNE:
    if (prop == 32768) {
      quick.push("dis+3_50_bs=off:fsr=off:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:spl=sat:updr=off_29");
      quick.push("dis+1011_128_bs=off:nwc=5:ptb=off:ssec=off:sswsr=on:sos=on:spl=sat_148");
      quick.push("lrs+4_1024_bs=off:flr=on:fsr=off:gs=on:lcm=reverse:nwc=1.2:nicw=on:ptb=off:ssec=off:stl=30:sos=on:sio=off:spo=on:spl=backtracking_240");
      quick.push("dis+1011_12_bs=off:cond=fast:ecs=on:flr=on:nwc=1:ssec=off_170");
      quick.push("dis+1011_128_bs=off:cond=fast:flr=on:fsr=off:lcm=reverse:nwc=2.5:nicw=on:ptb=off:ssec=off:sswsr=on:spl=sat_215");
      quick.push("ott-1002_2_bs=off:ecs=on:flr=on:lcm=predicate:nwc=2:nicw=on:ssec=off:sos=on:spo=on:sp=reverse_arity:updr=off_173");
      quick.push("dis+11_128_bs=off:cond=fast:flr=on:lcm=reverse:nwc=2:ptb=off:ssec=off:spl=sat_190");
    }
    else {
      quick.push("dis+1011_4_bs=off:gs=on:nwc=1.7:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spo=on:spl=backtracking_1");
      quick.push("dis+1_2:1_bs=off:bsr=unit_only:fsr=off:lcm=reverse:nwc=1.1:nicw=on:ptb=off:ssec=off:sio=off:spl=sat:sser=off:ssfq=2.0_14");
      quick.push("dis-2_14_bs=off:cond=fast:flr=on:lcm=predicate:nwc=1:nicw=on:ptb=off:ssec=off:sagn=off:sac=on:sio=off:spl=backtracking_39");
      quick.push("ott+1010_2_bs=off:gs=on:nwc=5:ptb=off:ssec=off:spl=sat_26");
      quick.push("dis+11_20_bs=off:fsr=off:gsp=input_only:lcm=reverse:nwc=1.3:ptb=off:ssec=off:spl=sat:sp=occurrence_13");
      quick.push("dis+1011_20_bs=off:gsp=input_only:nwc=1.3:nicw=on:ptb=off:ssec=off:sswsr=on:sos=on:sac=on:sio=off:spo=on:spl=sat_184");
      quick.push("dis+11_10_bs=off:flr=on:lcm=reverse:nwc=2.5:ptb=off:ssec=off:spl=sat_29");
      quick.push("lrs+1002_7_bs=off:cond=on:flr=on:fsr=off:gsp=input_only:gs=on:lcm=predicate:nwc=1.2:ptb=off:stl=30:spl=sat:sser=off:ssfp=40000:ssfq=2.0:ssnc=none_62");
      quick.push("dis+1_40_bs=off:cond=on:gsp=input_only:lcm=predicate:nwc=4:nicw=on:ptb=off:ssec=off:sgo=on:sio=off:spl=sat_114");
      quick.push("dis+1011_6_bs=off:nwc=1.2:ptb=off:ssec=off:spl=sat_179");
      quick.push("dis+4_12_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=10:ptb=off:ssec=off:spl=sat:sp=occurrence_184");
    }
    break;

  case Property::FEQ:
    if (prop == 1) {
      if (atoms > 2000000) {
        quick.push("dis+2_24_bs=off:ep=RSTC:flr=on:lcm=reverse:nwc=5:nicw=on:ssec=off:sac=on:sgo=on:spo=on:sfv=off_1179");
      }
      else {
        quick.push("ott+11_64_bd=off:bs=off:br=off:cond=fast:drc=off:ep=on:gsp=input_only:gs=on:nwc=2:ptb=off:ssec=off:sos=all:sio=off:spl=backtracking:urr=on_3");
        quick.push("dis-11_20_bs=off:cond=on:drc=off:ep=RS:nwc=1.7:nicw=on:ptb=off:ssec=off:sio=off:spl=sat:ssfq=2.0:ssnc=none_1");
        quick.push("dis+2_24_bs=off:cond=on:drc=off:fde=none:nwc=1.1:nicw=on:sd=10:ss=axioms:sio=off:spl=off:sp=reverse_arity_8");
        quick.push("ott+1_10_bs=off:cond=fast:drc=off:flr=on:lcm=predicate:nwc=10:ptb=off:ssec=off:sac=on:sio=off:spo=on:sp=occurrence:urr=on_7");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=5:ptb=off:ssec=off:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_1");
        quick.push("ott+11_12_cond=fast:drc=off:ecs=on:gs=on:nwc=1.2:nicw=on:ssec=off:sp=occurrence_1");
        quick.push("dis+2_24_bs=off:ep=RSTC:flr=on:lcm=reverse:nwc=5:nicw=on:ssec=off:sac=on:sgo=on:spo=on:sfv=off_197");
        quick.push("ott+1011_6_bs=unit_only:drc=off:ep=RST:nwc=10:ssec=off:sac=on:sgo=on:sio=off:spo=on_11");
        quick.push("dis+1011_1_bs=off:bsr=on:cond=fast:drc=off:flr=on:fde=none:gs=on:nwc=1.5:ptb=off:ssec=off:sos=on:sac=on:sio=off:urr=on_29");
        quick.push("dis+1_3:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=1.5:sos=on:sagn=off:sac=on:sio=off:spl=backtracking:sp=occurrence_46");
        quick.push("ott+1011_1_bs=off:cond=on:drc=off:ecs=on:nwc=2:nicw=on:ptb=off:ssec=off:spl=sat:ssfp=1000:ssfq=1.2:ssnc=none_10");
        quick.push("dis+1010_24_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=2.5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=occurrence_111");
        quick.push("dis-1010_2_bs=off:cond=fast:drc=off:nwc=1.7:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=backtracking:updr=off_240");
        quick.push("ott+11_2:3_bs=off:cond=on:drc=off:flr=on:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:spo=on:spl=sat:sp=reverse_arity:urr=on_196");
        quick.push("ott+1_5:1_bs=off:cond=fast:drc=off:fde=none:lcm=reverse:nwc=10:nicw=on:ptb=off:ssec=off:sos=on:spl=sat:sp=occurrence_306");
        quick.push("dis+1011_3_bd=off:bs=off:drc=off:ep=on:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:updr=off_8");
        quick.push("dis+2_4_bs=off:drc=off:ep=RST:flr=on:lcm=reverse:nwc=1.7:ptb=off:ssec=off:sos=on:spl=sat:sp=reverse_arity_12");
      }
    }
    else if (prop == 2) {
      if (atoms > 19) {
        quick.push("lrs+2_40_bs=off:cond=fast:drc=off:nwc=1.7:ptb=off:ssec=off:stl=30:sio=off:spo=on:spl=sat_35");
        quick.push("ott+2_5:1_bd=off:bs=off:drc=off:gsp=input_only:nwc=1.1:nicw=on:ptb=off:ssec=off:sd=3:ss=axioms:sos=on:spl=sat:urr=on_1");
        quick.push("lrs+10_8_bs=off:drc=off:gs=on:nwc=1.5:nicw=on:ptb=off:ssec=off:stl=60:sio=off:spo=on:spl=sat:updr=off_22");
        quick.push("dis+1011_3_bd=off:bs=off:drc=off:ep=on:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:updr=off_55");
        quick.push("lrs+11_50_bs=off:bms=on:cond=fast:drc=off:ecs=on:nwc=1.1:ssec=off:stl=90:urr=on_627");
        quick.push("ott+11_12_cond=fast:drc=off:ecs=on:gs=on:nwc=1.2:nicw=on:ssec=off:sp=occurrence_372");
        quick.push("ott+10_1024_bs=off:drc=off:nwc=1.7:ssec=off:sac=on:sio=off:sp=occurrence:updr=off_395");
        quick.push("lrs-1004_50_bs=off:br=off:cond=fast:drc=off:nwc=1.5:nicw=on:stl=60:sio=off:spl=off:urr=on:updr=off_273");
        quick.push("dis+11_3:1_bs=off:drc=off:ep=on:nwc=2.5:ptb=off:ssec=off:sos=all:sio=off:spl=sat:sp=reverse_arity_32");
      }
      else {
        quick.push("dis-1002_3_bs=off:cond=on:drc=off:gs=on:nwc=3:nicw=on:ssec=off:sgo=on:sio=off:sp=reverse_arity_34");
        quick.push("lrs+11_50_bs=off:bms=on:cond=fast:drc=off:ecs=on:nwc=1.1:ssec=off:stl=90:urr=on_6");
        quick.push("ott+2_6_bs=off:cond=fast:drc=off:fde=none:nwc=1.2:ptb=off:ssec=off:sio=off:spl=sat_112");
        quick.push("ott+3_28_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sac=on:sio=off_243");
        quick.push("ott-11_8:1_bd=off:bs=off:drc=off:lcm=reverse:nwc=3:sio=off:spl=off:urr=on_72");
        quick.push("ott+1_8_bs=off:drc=off:lcm=reverse:nwc=1:ptb=off:ssec=off:sos=on:sio=off:spo=on:spl=backtracking:sp=reverse_arity_55");
        quick.push("ott+1_50_bd=off:bs=unit_only:drc=off:nwc=1.1:nicw=on:ptb=off:sio=off:spl=sat:ssfp=10000:ssfq=1.0:ssnc=all_dependent:sp=reverse_arity_299");
        quick.push("lrs+3_24_bd=off:cond=on:drc=off:nwc=1.3:ptb=off:ssec=off:stl=30:sio=off:spl=sat:sp=occurrence_217");
        quick.push("ott+11_12_cond=fast:drc=off:ecs=on:gs=on:nwc=1.2:nicw=on:ssec=off:sp=occurrence_435");
        quick.push("ott+3_24_bs=unit_only:drc=off:ecs=on:fde=none:nwc=1.1:ssec=off:sos=all:sac=on_425");
      }
    }
    else if (prop == 131087) {
      if (atoms > 240000) {
        quick.push("ott+2_5:1_bd=off:bs=off:drc=off:gsp=input_only:nwc=1.1:nicw=on:ptb=off:ssec=off:sd=3:ss=axioms:sos=on:spl=sat:urr=on_44");
        quick.push("ott+1_4_bs=off:cond=on:drc=off:ep=on:flr=on:nwc=4:nicw=on:ptb=off:ssec=off:sswsr=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat_41");
        quick.push("ott+1011_8:1_bs=off:bsr=unit_only:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_176");
        quick.push("dis+1_4:1_bs=off:drc=off:ep=on:fde=none:lcm=reverse:nwc=2:nicw=on:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=15:ss=axioms:sos=on:spl=sat:sp=reverse_arity:updr=off_31");
        quick.push("dis-1010_14_bs=off:ep=RST:nwc=1.1:ptb=off:sd=1:ss=included:st=1.5:sos=on:sac=on:sio=off:spl=sat:ssfq=1.0:ssnc=none:sfv=off_35");
        quick.push("dis+1011_3_bs=off:drc=off:fde=none:gs=on:nwc=1.1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sagn=off:sac=on:sio=off:spl=sat_46");
        quick.push("dis-3_5:1_bs=off:drc=off:ecs=on:ep=on:fde=none:gsp=input_only:nwc=1.3:ssec=off:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sac=on:sio=off:sp=occurrence_38");
        quick.push("ott+3_32_bd=off:bs=off:drc=off:flr=on:lcm=predicate:nwc=1.1:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:st=3.0:sos=on:spl=sat:updr=off_31");
        quick.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=3:ptb=off:ssec=off:sswn=on:sswsr=on:sd=3:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence:updr=off_106");
        quick.push("dis+2_5:1_bd=off:bs=off:ep=RSTC:gs=on:lcm=reverse:nwc=1.2:nicw=on:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:sac=on:sio=off:sfv=off:sp=reverse_arity_39");
        quick.push("dis-1002_12_bs=off:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1:nicw=on:ptb=off:ssec=off:sswsr=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat_46");
        quick.push("dis+2_8:1_bs=off:drc=off:flr=on:lcm=reverse:nwc=1.5:ptb=off:ssec=off:sswn=on:sd=2:sgt=7:ss=axioms:sos=on:sio=off:spl=sat_44");
        quick.push("ott+1_8:1_bs=off:drc=off:ep=on:flr=on:gsp=input_only:nwc=1.5:nicw=on:ptb=off:ssec=off:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:sac=on:sio=off:spl=backtracking_51");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=5:ptb=off:ssec=off:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_132");
        quick.push("ott+1_2_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=4:ptb=off:ssec=off:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_135");
        quick.push("dis+3_2_bs=off:drc=off:ep=RST:fsr=off:nwc=1:ptb=off:ssec=off:sd=1:ss=axioms:st=1.2:sos=on:spl=sat:ssfq=2.0:ssnc=none:sfv=off_75");
        quick.push("ott+4_5:1_bd=off:bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=3:ssec=off:sd=3:ss=axioms:sos=on:sio=off:spl=off:urr=on_217");
        quick.push("dis-1002_5_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:ssec=off:sswn=on:sd=1:ss=included:sos=on:sagn=off:spl=sat:ssfq=1.4:ssnc=none:updr=off_47");
        quick.push("ott+1_8:1_bs=off:cond=fast:drc=off:ep=on:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=backtracking_49");
      }
      else if (atoms > 140000) {
        quick.push("dis+1_3:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=1.5:sos=on:sagn=off:sac=on:sio=off:spl=backtracking:sp=occurrence_45");
        quick.push("ott+1_8:1_bs=off:cond=fast:drc=off:ep=on:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=backtracking_46");
        quick.push("ott+4_3:1_bs=off:bms=on:br=off:drc=off:fde=none:nwc=1.7:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:sfv=off:urr=on_21");
        quick.push("ott-11_4:1_bs=off:bsr=on:cond=fast:drc=off:ecs=on:nwc=1.3:ssec=off:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sio=off_29");
        quick.push("ott-1010_4_bs=off:cond=fast:drc=off:fsr=off:gs=on:nwc=10:nicw=on:ptb=off:ssec=off:sd=4:ss=axioms:st=1.2:sos=on:sac=on:sio=off:spl=backtracking:sfv=off_19");
        quick.push("dis-1_1_bs=off:ep=RSTC:gs=on:nwc=1.1:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off_22");
        quick.push("dis+11_2:1_bs=off:ep=on:nwc=1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=15:ss=axioms:sos=on:spl=sat_35");
        quick.push("dis-1010_5:4_bs=off:bms=on:drc=off:nwc=10:sd=2:ss=axioms:st=1.5:sac=on:sio=off:updr=off_20");
        quick.push("ott+1011_8:1_bs=off:bsr=unit_only:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_39");
        quick.push("dis+1011_2_bs=off:cond=on:drc=off:gs=on:nwc=4:ptb=off:ssec=off:sd=1:ss=axioms:st=1.2:spl=sat:updr=off_23");
        quick.push("dis+1_2_bs=off:cond=on:drc=off:lcm=predicate:nwc=2:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=2.0:sagn=off:sac=on:sio=off:spl=sat:sser=off:ssnc=none:updr=off_35");
        quick.push("dis+2_5:1_bd=off:bs=off:ep=RSTC:gs=on:lcm=reverse:nwc=1.2:nicw=on:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:sac=on:sio=off:sfv=off:sp=reverse_arity_51");
        quick.push("dis-1002_5_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:ssec=off:sswn=on:sd=1:ss=included:sos=on:sagn=off:spl=sat:ssfq=1.4:ssnc=none:updr=off_172");
        quick.push("dis-1010_2_bs=off:cond=fast:drc=off:nwc=1.7:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=backtracking:updr=off_120");
        quick.push("dis+2_4_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.2:nicw=on:ptb=off:ssec=off:sswn=on:sd=2:sgt=2:ss=axioms:sos=on:sgo=on:spl=backtracking:sp=reverse_arity_241");
        quick.push("ott+1_2_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=4:ptb=off:ssec=off:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_162");
      }
      else if (atoms > 20000) {
        quick.push("ott+1_4_bs=off:cond=on:drc=off:ep=on:flr=on:nwc=4:nicw=on:ptb=off:ssec=off:sswsr=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat_16");
        quick.push("dis-1_1_bs=off:ep=RSTC:gs=on:nwc=1.1:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off_7");
        quick.push("ott+1_8:1_bs=off:cond=fast:drc=off:ep=on:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=backtracking_12");
        quick.push("ott+2_5:1_bd=off:bs=off:drc=off:gsp=input_only:nwc=1.1:nicw=on:ptb=off:ssec=off:sd=3:ss=axioms:sos=on:spl=sat:urr=on_16");
        quick.push("dis+1_3:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=1.5:sos=on:sagn=off:sac=on:sio=off:spl=backtracking:sp=occurrence_11");
        quick.push("dis-1002_5_bs=off:drc=off:ep=on:nwc=2:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=1.2:sos=on:sagn=off:spl=sat:ssfq=1.4:ssnc=none_21");
        quick.push("ott+1_5:1_bs=off:br=off:cond=fast:drc=off:nwc=1:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=5.0:sos=all:sio=off:spl=sat:urr=on_10");
        quick.push("dis+1_2_bs=off:cond=on:drc=off:lcm=predicate:nwc=2:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=2.0:sagn=off:sac=on:sio=off:spl=sat:sser=off:ssnc=none:updr=off_10");
        quick.push("dis+1010_128_bs=off:cond=fast:drc=off:ep=on:flr=on:gs=on:lcm=predicate:nwc=1.1:nicw=on:ptb=off:ssec=off:sswsr=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sac=on:sio=off:spo=on:spl=sat:sfv=off_14");
        quick.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=3:ptb=off:ssec=off:sswn=on:sswsr=on:sd=3:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence:updr=off_53");
        quick.push("dis+10_5:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=5:nicw=on:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_54");
        quick.push("dis-1010_14_bs=off:ep=RST:nwc=1.1:ptb=off:sd=1:ss=included:st=1.5:sos=on:sac=on:sio=off:spl=sat:ssfq=1.0:ssnc=none:sfv=off_11");
        quick.push("lrs+11_2:1_bs=off:cond=on:drc=off:ep=on:fde=none:gs=on:lcm=predicate:nwc=4:nicw=on:ptb=off:ssec=off:stl=30:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spo=on:spl=backtracking:urr=on_9");
        quick.push("dis-1010_1024_bs=off:cond=fast:drc=off:fde=none:lcm=reverse:nwc=4:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=sat_4");
        quick.push("dis+2_8:1_bs=off:drc=off:flr=on:lcm=reverse:nwc=1.5:ptb=off:ssec=off:sswn=on:sd=2:sgt=7:ss=axioms:sos=on:sio=off:spl=sat_10");
        quick.push("dis-3_5:1_bs=off:drc=off:ecs=on:ep=on:fde=none:gsp=input_only:nwc=1.3:ssec=off:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sac=on:sio=off:sp=occurrence_12");
        quick.push("dis+11_2:1_bs=off:ep=on:nwc=1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=15:ss=axioms:sos=on:spl=sat_12");
        quick.push("ott+1011_8:1_bs=off:bsr=unit_only:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_221");
        quick.push("ott+1_2_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=4:ptb=off:ssec=off:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_70");
        quick.push("dis-1010_40_bs=off:cond=on:drc=off:gs=on:nwc=3:nicw=on:ptb=off:ssec=off:sd=4:ss=axioms:st=1.5:sos=on:sio=off:spl=sat_11");
        quick.push("dis-1010_2_bs=off:cond=fast:drc=off:nwc=1.7:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=backtracking:updr=off_134");
        quick.push("dis+3_4_bs=off:drc=off:ep=RST:fsr=off:nwc=3:ptb=off:ssec=off:sd=1:ss=axioms:st=1.5:sos=on:spl=sat:sser=off:ssfq=1.4:ssnc=none:sfv=off_16");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=5:ptb=off:ssec=off:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_104");
        quick.push("ott+11_8:1_bs=off:drc=off:ecs=on:fsr=off:gs=on:lcm=reverse:nwc=2:nicw=on:ptb=off:ssec=off:sd=3:ss=axioms:sos=all:spl=sat:ssfq=2.0:ssnc=none_259");
        quick.push("dis+2_8:1_bd=off:bs=off:bsr=unit_only:drc=off:lcm=reverse:nwc=2.5:nicw=on:ptb=off:ssec=off:sswn=on:sswsr=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sp=reverse_arity_104");
        quick.push("dis-1010_5:4_bs=off:bms=on:drc=off:nwc=10:sd=2:ss=axioms:st=1.5:sac=on:sio=off:updr=off_221");
        quick.push("ott-3_50_bs=off:cond=fast:drc=off:flr=on:nwc=1.7:ptb=off:ssec=off:sd=2:ss=axioms:st=3.0:sos=on:sio=off:urr=on_10");
        quick.push("ott+4_5:1_bd=off:bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=3:ssec=off:sd=3:ss=axioms:sos=on:sio=off:spl=off:urr=on_135");
      }
      else if (atoms > 5000) {
        quick.push("lrs+11_2:1_bs=off:cond=on:drc=off:ep=on:fde=none:gs=on:lcm=predicate:nwc=4:nicw=on:ptb=off:ssec=off:stl=30:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spo=on:spl=backtracking:urr=on_9");
        quick.push("dis-1_2:3_bs=off:drc=off:ecs=on:ep=RST:fde=none:gs=on:nwc=4:nicw=on:ssec=off:sos=on:sio=off:spl=off_3");
        quick.push("dis+1_2_bs=off:cond=on:drc=off:lcm=predicate:nwc=2:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=2.0:sagn=off:sac=on:sio=off:spl=sat:sser=off:ssnc=none:updr=off_6");
        quick.push("dis-1010_2_bs=off:cond=fast:drc=off:nwc=1.7:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=backtracking:updr=off_3");
        quick.push("dis+1011_3_bs=off:drc=off:fde=none:gs=on:nwc=1.1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sagn=off:sac=on:sio=off:spl=sat_11");
        quick.push("ott+1_8:1_bs=off:cond=fast:drc=off:ep=on:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=backtracking_22");
        quick.push("ott-1010_4_bs=off:cond=fast:drc=off:fsr=off:gs=on:nwc=10:nicw=on:ptb=off:ssec=off:sd=4:ss=axioms:st=1.2:sos=on:sac=on:sio=off:spl=backtracking:sfv=off_4");
        quick.push("dis-1002_12_bs=off:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1:nicw=on:ptb=off:ssec=off:sswsr=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat_4");
        quick.push("ott+1_2_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=4:ptb=off:ssec=off:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_21");
        quick.push("dis-1010_5:4_bs=off:bms=on:drc=off:nwc=10:sd=2:ss=axioms:st=1.5:sac=on:sio=off:updr=off_7");
        quick.push("ott+1011_8:1_bs=off:bsr=unit_only:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_56");
        quick.push("dis+1_5:1_bs=off:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sswn=on:sd=1:ss=included:sagn=off:spl=sat:ssfq=1.4:ssnc=none_33");
        quick.push("dis+2_8:1_bd=off:bs=off:bsr=unit_only:drc=off:lcm=reverse:nwc=2.5:nicw=on:ptb=off:ssec=off:sswn=on:sswsr=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sp=reverse_arity_29");
        quick.push("dis-1003_2:1_bs=off:drc=off:ep=on:fsr=off:fde=none:nwc=3:ptb=off:ssec=off:sswn=on:sswsr=on:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_8");
        quick.push("dis+3_5_bs=off:drc=off:ep=RST:fsr=off:nwc=1.7:nicw=on:ptb=off:ssec=off:sd=1:ss=axioms:st=1.2:sos=on:spl=sat:ssfq=1.4:ssnc=none:sfv=off_8");
        quick.push("ott+1_4_bs=off:cond=on:drc=off:ep=on:flr=on:nwc=4:nicw=on:ptb=off:ssec=off:sswsr=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat_22");
        quick.push("ott+2_5:1_bd=off:bs=off:drc=off:gsp=input_only:nwc=1.1:nicw=on:ptb=off:ssec=off:sd=3:ss=axioms:sos=on:spl=sat:urr=on_23");
        quick.push("ott+1_5:1_bs=off:br=off:cond=fast:drc=off:nwc=1:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=5.0:sos=all:sio=off:spl=sat:urr=on_203");
        quick.push("dis+2_4_bs=off:drc=off:ep=RST:flr=on:lcm=reverse:nwc=1.7:ptb=off:ssec=off:sos=on:spl=sat:sp=reverse_arity_176");
        quick.push("dis+1011_2_bs=off:cond=on:drc=off:gs=on:nwc=4:ptb=off:ssec=off:sd=1:ss=axioms:st=1.2:spl=sat:updr=off_3");
        quick.push("ott-3_50_bs=off:cond=fast:drc=off:flr=on:nwc=1.7:ptb=off:ssec=off:sd=2:ss=axioms:st=3.0:sos=on:sio=off:urr=on_4");
        quick.push("dis-1002_5_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:ssec=off:sswn=on:sd=1:ss=included:sos=on:sagn=off:spl=sat:ssfq=1.4:ssnc=none:updr=off_18");
        quick.push("ott+11_8:1_bs=off:drc=off:ecs=on:fsr=off:gs=on:lcm=reverse:nwc=2:nicw=on:ptb=off:ssec=off:sd=3:ss=axioms:sos=all:spl=sat:ssfq=2.0:ssnc=none_76");
      }
      else  {
        quick.push("dis-1010_5:1_bsr=on:drc=off:fde=none:gsp=input_only:gs=on:nwc=4:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=sat:updr=off_15");
        quick.push("ott+1_2_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=4:ptb=off:ssec=off:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_25");
        quick.push("ott+2_8:1_bs=off:bsr=on:cond=fast:drc=off:fde=none:lcm=reverse:nwc=4:nicw=on:ptb=off:ssec=off:sos=on:spl=sat:sser=off:ssfp=100000:ssnc=none:urr=on_41");
        quick.push("dis+1011_2_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=4:ptb=off:ssec=off:sio=off:spl=backtracking:sp=occurrence_39");
        quick.push("ott+11_2:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:fde=none:gsp=input_only:lcm=reverse:nwc=2.5:ptb=off:ssec=off:spl=sat:sp=occurrence_23");
        quick.push("dis+1_4_bs=off:drc=off:ep=on:fde=none:lcm=reverse:nwc=5:ptb=off:ssec=off:sos=on:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sp=reverse_arity_1");
        quick.push("dis+11_1_bs=off:bms=on:drc=off:ep=RS:flr=on:fde=none:nwc=5:sos=on:sgo=on:sio=off_22");
        quick.push("dis+1011_5:1_bs=off:cond=fast:drc=off:ep=RS:nwc=5:nicw=on:ptb=off:ssec=off:sio=off:spo=on_19");
        quick.push("ott+1_10_bs=off:cond=fast:drc=off:flr=on:lcm=predicate:nwc=10:ptb=off:ssec=off:sac=on:sio=off:spo=on:sp=occurrence:urr=on_8");
        quick.push("dis+2_8_bs=off:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=1.7:ptb=off:ssec=off:sswn=on:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sp=occurrence_21");
        quick.push("ott+1011_8:1_bs=off:bsr=unit_only:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_285");
        quick.push("ott+1011_10_bd=off:bs=off:drc=off:ep=on:fde=none:gs=on:nwc=5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=reverse_arity_86");
        quick.push("dis+10_64_bs=off:cond=on:drc=off:ep=RS:nwc=1:ptb=off:ssec=off:sio=off:spo=on_30");
        quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ep=RST:flr=on:fde=none:lcm=reverse:nwc=10:sac=on:sio=off:urr=on_83");
        quick.push("dis+11_4:1_bs=off:drc=off:ep=RST:fde=none:lcm=reverse:nwc=5:nicw=on:ssec=off:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sgo=on:sio=off:sp=occurrence_154");
        quick.push("ins-11_8:1_bs=off:bsr=unit_only:igbrr=0.9:igrr=1/128:igrp=400:igrpq=1.5:igwr=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=all:spl=off:sp=occurrence:updr=off_184");
        quick.push("ott-1004_3:1_bs=off:br=off:cond=on:drc=off:nwc=2:nicw=on:ptb=off:ssec=off:sio=off:spo=on:spl=sat:urr=on_95");
        quick.push("dis+2_5:4_bs=off:br=off:cond=on:drc=off:fde=none:gsp=input_only:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sio=off:spl=backtracking:urr=on_139");
        quick.push("dis+2_1_bs=off:bsr=unit_only:drc=off:fsr=off:gsp=input_only:gs=on:lcm=reverse:nwc=5:ptb=off:ssec=off:sswn=on:sd=2:sgt=7:ss=axioms:st=1.2:sos=on:sio=off:spl=sat_180");
        quick.push("dis+1011_3_bd=off:bs=off:drc=off:ep=on:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:updr=off_258");
        quick.push("dis-1010_14_bs=off:ep=RST:nwc=1.1:ptb=off:sd=1:ss=included:st=1.5:sos=on:sac=on:sio=off:spl=sat:ssfq=1.0:ssnc=none:sfv=off_19");
        quick.push("ott+1_8:1_bd=off:bs=off:drc=off:fde=none:lcm=reverse:nwc=1.5:ptb=off:ssec=off:sos=on:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sp=reverse_arity_26");
      }
    }
    else if (prop == 0) {
      if (atoms <= 70) {
        quick.push("ott+10_3:1_bsr=unit_only:cond=on:fsr=off:lcm=reverse:nwc=1.5:nicw=on:ptb=off:ssec=off:sio=off:spo=on:spl=sat:urr=on:updr=off_21");
        quick.push("lrs+10_8_bs=off:drc=off:gs=on:nwc=1.5:nicw=on:ptb=off:ssec=off:stl=60:sio=off:spo=on:spl=sat:updr=off_11");
        quick.push("ott+2_8_bd=off:bs=off:bsr=unit_only:cond=on:flr=on:nwc=2:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:updr=off_59");
        quick.push("dis+1_16_bd=off:bs=off:cond=fast:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:urr=on_72");
        quick.push("lrs+2_40_bs=off:cond=fast:drc=off:nwc=1.7:ptb=off:ssec=off:stl=30:sio=off:spo=on:spl=sat_181");
        quick.push("ott-1003_2:1_bs=unit_only:bsr=unit_only:drc=off:nwc=1.2:ptb=off:ssec=off:sio=off:updr=off_52");
        quick.push("dis+2_12_drc=off:ep=RST:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sgo=on:spl=sat:sp=occurrence_57");
        quick.push("ott+4_64_bs=off:drc=off:flr=on:lcm=predicate:nwc=10:ptb=off:ssec=off:spl=sat:sp=reverse_arity_108");
        quick.push("ott+3_28_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sac=on:sio=off_89");
        quick.push("dis+1_2_bs=off:cond=on:drc=off:lcm=predicate:nwc=2:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=2.0:sagn=off:sac=on:sio=off:spl=sat:sser=off:ssnc=none:updr=off_97");
        quick.push("ott-11_8:1_bd=off:bs=off:drc=off:lcm=reverse:nwc=3:sio=off:spl=off:urr=on_105");
        quick.push("ott+11_3_bd=off:bs=off:gs=on:nwc=1.1:nicw=on:ptb=off:ssec=off:spl=sat:sser=off:ssfp=10000:ssfq=2.0_272");
        quick.push("ott+3_3_bs=off:bsr=unit_only:cond=fast:drc=off:fsr=off:fde=none:gs=on:nwc=2.5:nicw=on:ptb=off:ssec=off:sgo=on:spo=on:spl=backtracking:sp=occurrence_338");
      }
      else {
        quick.push("ott+1011_1_bs=off:cond=on:drc=off:ecs=on:nwc=2:nicw=on:ptb=off:ssec=off:spl=sat:ssfp=1000:ssfq=1.2:ssnc=none_18");
        quick.push("dis-4_3:2_cond=on:drc=off:gs=on:nwc=4:nicw=on:ptb=off:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence_2");
        quick.push("dis-1_6_bs=off:bsr=unit_only:cond=on:drc=off:fsr=off:gs=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=on:sio=off:spo=on:urr=on_24");
        quick.push("ott+1_10_bs=off:cond=fast:drc=off:flr=on:lcm=predicate:nwc=10:ptb=off:ssec=off:sac=on:sio=off:spo=on:sp=occurrence:urr=on_121");
        quick.push("dis-1002_5:1_bs=off:bms=on:drc=off:ecs=on:flr=on:gs=on:nwc=10:nicw=on:ssec=off:sac=on:sgo=on:urr=on_9");
        quick.push("ott+1_12_bs=off:br=off:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sio=off:spo=on:spl=sat:sser=off:ssfq=1.0:ssnc=none:urr=on_95");
        quick.push("ott-1004_3:2_br=off:drc=off:fsr=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=all:spl=sat:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_194");
        quick.push("lrs+1_20_bs=off:cond=fast:gs=on:nwc=4:ptb=off:ssec=off:sswsr=on:stl=30:sgt=20:ss=axioms:st=1.2:sos=on:sagn=off:sio=off:spo=on:spl=backtracking:updr=off_228");
        quick.push("ott+10_64_bd=off:bs=off:cond=on:drc=off:ecs=on:gs=on:nwc=1:ssec=off:sac=on:sio=off_267");
        quick.push("lrs-1004_50_bs=off:br=off:cond=fast:drc=off:nwc=1.5:nicw=on:stl=60:sio=off:spl=off:urr=on:updr=off_148");
        quick.push("lrs-3_10_drc=off:nwc=5:ptb=off:ssec=off:stl=30:spl=sat:sser=off:ssfp=4000:ssnc=none:urr=on_150");
        quick.push("lrs+1_4_bs=off:drc=off:ecs=on:ep=on:flr=on:fsr=off:lcm=predicate:nwc=1.3:nicw=on:ptb=off:ssec=off:stl=30:spl=sat:sser=off:ssfp=40000:ssfq=1.1:ssnc=none:sp=occurrence_154");
        quick.push("ott+11_10_bd=off:bs=off:cond=fast:drc=off:lcm=predicate:nwc=1:nicw=on:sac=on:sgo=on:urr=on_333");
        quick.push("dis+2_5:4_bs=off:br=off:cond=on:drc=off:fde=none:gsp=input_only:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sio=off:spl=backtracking:urr=on_3");
        quick.push("ott-1003_24_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:ptb=off:ssec=off:spl=sat:sser=off:ssfp=1000:ssnc=none:urr=on_51");
      }
    }
    else if (prop == 131075) {
      if (atoms > 3000) {
        quick.push("dis-1002_3_bs=off:cond=fast:drc=off:ep=RS:nwc=1.1:ptb=off:ssec=off:swb=on_22");
        quick.push("ott+1_8:1_bs=off:cond=fast:drc=off:ep=on:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=backtracking_5");
        quick.push("dis-1_1_bs=off:ep=RSTC:gs=on:nwc=1.1:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off_6");
        quick.push("ott+4_3:1_bs=off:bms=on:br=off:drc=off:fde=none:nwc=1.7:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:sfv=off:urr=on_14");
        quick.push("ott+1011_1_bs=off:cond=on:drc=off:ecs=on:nwc=2:nicw=on:ptb=off:ssec=off:spl=sat:ssfp=1000:ssfq=1.2:ssnc=none_69");
        quick.push("ott-1002_5:4_bs=off:drc=off:fde=none:nwc=1.7:ptb=off:ssec=off:sos=on:sac=on:sio=off:spo=on:spl=sat:urr=on_16");
        quick.push("dis+11_8:1_bs=off:br=off:cond=on:drc=off:ecs=on:ep=RST:fsr=off:nwc=1:ptb=off:ssec=off:sd=7:ss=axioms:st=2.0:sio=off:spo=on:spl=sat:ssnc=none:urr=on:updr=off_29");
        quick.push("dis+2_8:1_bs=off:drc=off:flr=on:lcm=reverse:nwc=1.5:ptb=off:ssec=off:sswn=on:sd=2:sgt=7:ss=axioms:sos=on:sio=off:spl=sat_36");
        quick.push("dis+2_8:1_bd=off:bs=off:bsr=unit_only:drc=off:lcm=reverse:nwc=2.5:nicw=on:ptb=off:ssec=off:sswn=on:sswsr=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sp=reverse_arity_62");
        quick.push("dis+1_3:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=1.5:sos=on:sagn=off:sac=on:sio=off:spl=backtracking:sp=occurrence_157");
        quick.push("lrs+1_4_bs=off:drc=off:ecs=on:ep=on:flr=on:fsr=off:lcm=predicate:nwc=1.3:nicw=on:ptb=off:ssec=off:stl=30:spl=sat:sser=off:ssfp=40000:ssfq=1.1:ssnc=none:sp=occurrence_164");
        quick.push("ott-1010_8:1_bs=off:flr=on:fsr=off:gs=on:nwc=4:ptb=off:ssec=off:sac=on:sio=off:sp=reverse_arity:urr=on_434");
        quick.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=2:ptb=off:ssec=off:spo=on:sp=occurrence_237");
      }
      else {
        quick.push("ott-1002_5:4_bs=off:drc=off:fde=none:nwc=1.7:ptb=off:ssec=off:sos=on:sac=on:sio=off:spo=on:spl=sat:urr=on_1");
        quick.push("dis-1010_5:4_bs=off:bms=on:drc=off:nwc=10:sd=2:ss=axioms:st=1.5:sac=on:sio=off:updr=off_1");
        quick.push("ott-1002_1024_bd=off:bs=off:cond=on:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=on:sgo=on:sio=off:spo=on:spl=backtracking:urr=on_8");
        quick.push("ott+1011_10_bd=off:bs=off:drc=off:ep=on:fde=none:gs=on:nwc=5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=reverse_arity_5");
        quick.push("dis+1011_1_bs=off:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:nicw=on:ptb=off:ssec=off:sgt=7:ss=axioms:sos=on:spl=sat_4");
        quick.push("dis-1002_5_bs=off:drc=off:ep=on:nwc=2:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=1.2:sos=on:sagn=off:spl=sat:ssfq=1.4:ssnc=none_4");
        quick.push("ott-11_4:1_bs=off:bsr=on:cond=fast:drc=off:ecs=on:nwc=1.3:ssec=off:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sio=off_3");
        quick.push("ott+3_3_bs=off:bsr=unit_only:cond=fast:drc=off:fsr=off:fde=none:gs=on:nwc=2.5:nicw=on:ptb=off:ssec=off:sgo=on:spo=on:spl=backtracking:sp=occurrence_14");
        quick.push("ott+11_8:1_bd=off:bs=unit_only:bsr=unit_only:drc=off:fsr=off:lcm=reverse:nwc=1.1:nicw=on:ptb=off:ssec=off:spl=sat:ssfq=2.0:ssnc=none_17");
        quick.push("ott+11_64_bd=off:bs=off:br=off:cond=fast:drc=off:ep=on:gsp=input_only:gs=on:nwc=2:ptb=off:ssec=off:sos=all:sio=off:spl=backtracking:urr=on_9");
        quick.push("dis+1010_7_bs=off:bsr=on:bms=on:drc=off:fsr=off:fde=none:gs=on:nwc=1.7:sac=on:sgo=on:sio=off:sp=occurrence_57");
        quick.push("ott+4_3:1_bs=off:bms=on:br=off:drc=off:fde=none:nwc=1.7:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:sfv=off:urr=on_1");
        quick.push("dis-1002_5_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:ssec=off:sswn=on:sd=1:ss=included:sos=on:sagn=off:spl=sat:ssfq=1.4:ssnc=none:updr=off_11");
        quick.push("ott-4_7_bs=off:drc=off:fde=none:gs=on:lcm=reverse:nwc=1.7:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=backtracking:urr=on_13");
        quick.push("ott+11_50_bd=off:bs=off:cond=fast:drc=off:ep=on:fsr=off:lcm=reverse:nwc=1:nicw=on:ptb=off:ssec=off:sos=on:spl=backtracking:sp=occurrence:updr=off_10");
        quick.push("dis-1_1_bs=off:ep=RSTC:gs=on:nwc=1.1:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off_1");
        quick.push("ott-11_8:1_bd=off:bs=off:drc=off:lcm=reverse:nwc=3:sio=off:spl=off:urr=on_118");
        quick.push("dis-1010_1024_bs=off:cond=fast:drc=off:fde=none:lcm=reverse:nwc=4:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=sat_1");
        quick.push("dis-1010_7_bs=off:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:updr=off_123");
        quick.push("ott-2_5:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:fde=none:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking_11");
        quick.push("dis+1011_2_bs=off:cond=on:drc=off:gs=on:nwc=4:ptb=off:ssec=off:sd=1:ss=axioms:st=1.2:spl=sat:updr=off_8");
        quick.push("dis+1011_3_bd=off:bs=off:drc=off:ep=on:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:updr=off_174");
        quick.push("ott-1010_24_bd=off:bs=off:drc=off:fsr=off:gs=on:nwc=4:nicw=on:ptb=off:ssec=off:spl=backtracking_76");
        quick.push("ins-11_8:1_bs=off:bsr=unit_only:igbrr=0.9:igrr=1/128:igrp=400:igrpq=1.5:igwr=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=all:spl=off:sp=occurrence:updr=off_67");
        quick.push("ott+1011_1_bs=off:cond=on:drc=off:ecs=on:nwc=2:nicw=on:ptb=off:ssec=off:spl=sat:ssfp=1000:ssfq=1.2:ssnc=none_190");
        quick.push("dis-1010_5:1_bsr=on:drc=off:fde=none:gsp=input_only:gs=on:nwc=4:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=sat:updr=off_170");
        quick.push("lrs-2_2:1_bs=off:br=off:cond=on:ep=on:fde=none:lcm=reverse:nwc=10:ptb=off:ssec=off:stl=60:sos=on:sio=off:spo=on:urr=on_327");
        quick.push("dis-1010_2_bs=off:cond=fast:drc=off:nwc=1.7:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=backtracking:updr=off_9");
        quick.push("dis+1010_24_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=2.5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=occurrence_65");
      }
    }
    else if (prop == 131073) {
      if (atoms > 200) {
        quick.push("dis+1011_2_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=4:ptb=off:ssec=off:sio=off:spl=backtracking:sp=occurrence_1");
        quick.push("ott-3_50_bs=off:cond=fast:drc=off:flr=on:nwc=1.7:ptb=off:ssec=off:sd=2:ss=axioms:st=3.0:sos=on:sio=off:urr=on_2");
        quick.push("ott-1010_4_bs=off:cond=fast:drc=off:fsr=off:gs=on:nwc=10:nicw=on:ptb=off:ssec=off:sd=4:ss=axioms:st=1.2:sos=on:sac=on:sio=off:spl=backtracking:sfv=off_2");
        quick.push("dis+1010_24_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=2.5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=occurrence_5");
        quick.push("dis+1_4:1_bs=off:drc=off:ep=on:fde=none:lcm=reverse:nwc=2:nicw=on:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=15:ss=axioms:sos=on:spl=sat:sp=reverse_arity:updr=off_1");
        quick.push("ott+1011_8:1_bs=off:bsr=unit_only:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_2");
        quick.push("ott+1_8:1_bs=off:drc=off:ep=on:flr=on:gsp=input_only:nwc=1.5:nicw=on:ptb=off:ssec=off:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:sac=on:sio=off:spl=backtracking_1");
        quick.push("dis+1_14_bs=off:br=off:drc=off:fde=none:gs=on:nwc=1.1:ssec=off:sd=2:ss=axioms:st=2.0:sac=on:sio=off:urr=on_2");
        quick.push("dis+1011_1_bs=off:bsr=on:cond=fast:drc=off:flr=on:fde=none:gs=on:nwc=1.5:ptb=off:ssec=off:sos=on:sac=on:sio=off:urr=on_20");
        quick.push("dis-1002_1024_bs=off:bms=on:cond=fast:drc=off:ecs=on:ep=on:lcm=predicate:nwc=3:nicw=on:ssec=off:sswn=on:sswsr=on:sac=on_1");
        quick.push("ins+1011_3:1_bs=off:cond=fast:ep=RSTC:igbrr=0.8:igrr=1/2:igrp=200:igrpq=1.0:igwr=on:nwc=2:ptb=off:ssec=off:sos=all:sio=off:spl=off_8");
        quick.push("ott-2_5:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:fde=none:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking_2");
        quick.push("dis+1011_14_bd=off:bs=off:drc=off:nwc=4:ptb=off:ssec=off:sswn=on:sac=on:spl=sat:sp=occurrence_16");
        quick.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:ptb=off:ssec=off:spl=sat_28");
        quick.push("lrs-1004_50_bs=off:br=off:cond=fast:drc=off:nwc=1.5:nicw=on:stl=60:sio=off:spl=off:urr=on:updr=off_5");
        quick.push("dis-1010_50_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:ptb=off:sswn=on:sswsr=on:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none_3");
        quick.push("ott-1004_3:1_bs=off:br=off:cond=on:drc=off:nwc=2:nicw=on:ptb=off:ssec=off:sio=off:spo=on:spl=sat:urr=on_8");
        quick.push("ott+11_12_cond=fast:drc=off:ecs=on:gs=on:nwc=1.2:nicw=on:ssec=off:sp=occurrence_1");
        quick.push("dis+1010_7_bs=off:bsr=on:bms=on:drc=off:fsr=off:fde=none:gs=on:nwc=1.7:sac=on:sgo=on:sio=off:sp=occurrence_2");
        quick.push("dis+1_64_bs=off:drc=off:fde=none:nwc=3:nicw=on:sos=on:sio=off:sp=reverse_arity:urr=on_16");
        quick.push("dis+11_2:1_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=2:nicw=on:ptb=off:ssec=off:sos=all:sac=on:sio=off:spo=on:spl=sat:sscc=on:sser=off:ssfq=2.0:sp=occurrence_18");
        quick.push("dis-1010_5:1_bsr=on:drc=off:fde=none:gsp=input_only:gs=on:nwc=4:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=sat:updr=off_18");
        quick.push("dis-1003_2:1_bs=off:drc=off:ep=on:fsr=off:fde=none:nwc=3:ptb=off:ssec=off:sswn=on:sswsr=on:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_2");
        quick.push("ins-1003_3_bs=off:ep=RSTC:flr=on:gsp=input_only:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=2.0:igwr=on:nwc=2:ptb=off:ssec=off:sos=on:sio=off:spl=off_2");
        quick.push("lrs+11_2:1_bs=off:cond=on:drc=off:ep=on:fde=none:gs=on:lcm=predicate:nwc=4:nicw=on:ptb=off:ssec=off:stl=30:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spo=on:spl=backtracking:urr=on_1");
        quick.push("dis+1_2_bs=off:cond=on:drc=off:lcm=predicate:nwc=2:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=2.0:sagn=off:sac=on:sio=off:spl=sat:sser=off:ssnc=none:updr=off_4");
        quick.push("lrs+1_14_bs=off:cond=fast:drc=off:nwc=1.3:nicw=on:ssec=off:stl=60:spo=on:updr=off_1");
        quick.push("dis-1002_3_bs=off:cond=fast:drc=off:ep=RS:nwc=1.1:ptb=off:ssec=off:swb=on_8");
        quick.push("dis+2_4_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.2:nicw=on:ptb=off:ssec=off:sswn=on:sd=2:sgt=2:ss=axioms:sos=on:sgo=on:spl=backtracking:sp=reverse_arity_16");
        quick.push("dis+1_3:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=1.5:sos=on:sagn=off:sac=on:sio=off:spl=backtracking:sp=occurrence_1");
        quick.push("dis+2_8_bs=off:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=1.7:ptb=off:ssec=off:sswn=on:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sp=occurrence_30");
        quick.push("dis-1002_5_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:ssec=off:sswn=on:sd=1:ss=included:sos=on:sagn=off:spl=sat:ssfq=1.4:ssnc=none:updr=off_13");
        quick.push("dis+10_128_bd=off:bs=off:drc=off:nwc=3:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking_29");
        quick.push("dis+2_5_bs=off:cond=fast:drc=off:nwc=1.3:nicw=on:ptb=off:ssec=off:ss=included:st=1.5:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_13");
        quick.push("ott+1_2_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=4:ptb=off:ssec=off:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_42");
        quick.push("ott+3_24_bs=unit_only:drc=off:ecs=on:fde=none:nwc=1.1:ssec=off:sos=all:sac=on_2");
        quick.push("dis+11_8:1_bs=off:br=off:cond=on:drc=off:ecs=on:ep=RST:fsr=off:nwc=1.1:ptb=off:ssec=off:sd=5:ss=axioms:st=3.0:sio=off:spo=on:spl=sat:ssfq=1.2:ssnc=none:urr=on_97");
        quick.push("dis-2_8:1_bs=off:drc=off:lcm=predicate:nwc=3:ptb=off:ssec=off:sswn=on:sswsr=on:sgt=10:ss=axioms:sos=on:spl=sat_93");
        quick.push("dis+1011_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=5:nicw=on:ssec=off:sswn=on:sp=reverse_arity_257");
        quick.push("ott-1010_8:1_bs=off:flr=on:fsr=off:gs=on:nwc=4:ptb=off:ssec=off:sac=on:sio=off:sp=reverse_arity:urr=on_1");
        quick.push("dis+1_2:1_bd=off:bs=off:bsr=on:drc=off:ep=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sswn=on:sd=1:ss=included:sos=on:sagn=off:sio=off:spo=on:spl=backtracking_1");
        quick.push("ott+3_28_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sac=on:sio=off_2");
        quick.push("dis+1011_1_bs=off:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:nicw=on:ptb=off:ssec=off:sgt=7:ss=axioms:sos=on:spl=sat_2");
        quick.push("ott+4_3:1_bs=off:bms=on:br=off:drc=off:fde=none:nwc=1.7:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:sfv=off:urr=on_2");
        quick.push("lrs-2_2:1_bs=off:br=off:cond=on:ep=on:fde=none:lcm=reverse:nwc=10:ptb=off:ssec=off:stl=60:sos=on:sio=off:spo=on:urr=on_4");
        quick.push("ott+2_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ecs=on:ep=RST:flr=on:fde=none:lcm=reverse:nwc=5:ssec=off:sac=on:sio=off:urr=on_4");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=5:ptb=off:ssec=off:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_8");
        quick.push("dis-3_5:1_bs=off:drc=off:ecs=on:ep=on:fde=none:gsp=input_only:nwc=1.3:ssec=off:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sac=on:sio=off:sp=occurrence_9");
        quick.push("ott+11_2:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:fde=none:gsp=input_only:lcm=reverse:nwc=2.5:ptb=off:ssec=off:spl=sat:sp=occurrence_12");
        quick.push("dis+1011_3_bs=off:drc=off:fde=none:nwc=10:nicw=on:ss=axioms:st=2.0:sac=on:sio=off_16");
        quick.push("dis+11_8_bs=off:cond=fast:ep=RST:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:ptb=off:sd=7:ss=axioms:st=2.0:sgo=on:spo=on:spl=sat:sser=off:ssfq=1.1:ssnc=none_16");
        quick.push("dis+11_8:1_bs=off:br=off:cond=on:drc=off:ecs=on:ep=RST:fsr=off:nwc=1:ptb=off:ssec=off:sd=7:ss=axioms:st=2.0:sio=off:spo=on:spl=sat:ssnc=none:urr=on:updr=off_18");
        quick.push("ott+1011_1_bs=off:cond=on:drc=off:ecs=on:nwc=2:nicw=on:ptb=off:ssec=off:spl=sat:ssfp=1000:ssfq=1.2:ssnc=none_19");
        quick.push("dis-1003_50_bs=off:cond=fast:drc=off:fde=none:lcm=predicate:nwc=1:nicw=on:ptb=off:ssec=off:sos=on:sgo=on:sio=off:spo=on:spl=sat_24");
        quick.push("dis+1011_3_bs=off:drc=off:fde=none:gs=on:nwc=1.1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sagn=off:sac=on:sio=off:spl=sat_24");
        quick.push("dis-1010_7_bs=off:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:updr=off_59");
        quick.push("dis+1010_32_cond=fast:drc=off:ep=RS:fsr=off:nwc=1.7:ptb=off:ssec=off:spl=sat_61");
        quick.push("ott-1002_1024_bd=off:bs=off:cond=on:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=on:sgo=on:sio=off:spo=on:spl=backtracking:urr=on_62");
        quick.push("dis-1010_1024_bs=off:cond=fast:drc=off:fde=none:lcm=reverse:nwc=4:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=sat_83");
      }
      else {
        quick.push("dis-1010_5:4_bs=off:bms=on:drc=off:nwc=10:sd=2:ss=axioms:st=1.5:sac=on:sio=off:updr=off_14");
        quick.push("ott-2_5:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:fde=none:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking_3");
        quick.push("ott-1004_3:1_bs=off:br=off:cond=on:drc=off:nwc=2:nicw=on:ptb=off:ssec=off:sio=off:spo=on:spl=sat:urr=on_9");
        quick.push("dis-1010_2_bs=off:cond=fast:drc=off:nwc=1.7:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=backtracking:updr=off_22");
        quick.push("dis+1_2_bs=off:cond=on:drc=off:lcm=predicate:nwc=2:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=2.0:sagn=off:sac=on:sio=off:spl=sat:sser=off:ssnc=none:updr=off_5");
        quick.push("dis+1011_3_bd=off:bs=off:drc=off:ep=on:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:updr=off_2");
        quick.push("ott-4_7_bs=off:drc=off:fde=none:gs=on:lcm=reverse:nwc=1.7:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=backtracking:urr=on_4");
        quick.push("ott+3_32_bd=off:bs=off:drc=off:flr=on:lcm=predicate:nwc=1.1:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:st=3.0:sos=on:spl=sat:updr=off_3");
        quick.push("ott+1011_5:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=backtracking:sp=occurrence_2");
        quick.push("dis+1010_24_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=2.5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=occurrence_116");
        quick.push("dis+2_4_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.2:nicw=on:ptb=off:ssec=off:sswn=on:sd=2:sgt=2:ss=axioms:sos=on:sgo=on:spl=backtracking:sp=reverse_arity_46");
        quick.push("dis+2_8_bs=off:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=1.7:ptb=off:ssec=off:sswn=on:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sp=occurrence_38");
        quick.push("lrs+1003_8:1_bs=off:drc=off:ep=on:fsr=off:gsp=input_only:gs=on:nwc=1.5:ptb=off:ssec=off:stl=30:sio=off:spl=sat_113");
        quick.push("ott+11_8:1_bs=off:drc=off:ecs=on:fsr=off:gs=on:lcm=reverse:nwc=2:nicw=on:ptb=off:ssec=off:sd=3:ss=axioms:sos=all:spl=sat:ssfq=2.0:ssnc=none_65");
        quick.push("dis+10_128_bd=off:bs=off:drc=off:nwc=3:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking_99");
        quick.push("dis+1011_3_bs=off:drc=off:fde=none:gs=on:nwc=1.1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sagn=off:sac=on:sio=off:spl=sat_121");
        quick.push("dis+1011_14_bd=off:bs=off:drc=off:nwc=4:ptb=off:ssec=off:sswn=on:sac=on:spl=sat:sp=occurrence_219");
        quick.push("ins+1011_3:1_bs=off:cond=fast:ep=RSTC:igbrr=0.8:igrr=1/2:igrp=200:igrpq=1.0:igwr=on:nwc=2:ptb=off:ssec=off:sos=all:sio=off:spl=off_4");
        quick.push("ott+11_12_cond=fast:drc=off:ecs=on:gs=on:nwc=1.2:nicw=on:ssec=off:sp=occurrence_11");
      }
    }
    else if (prop == 131081 || prop == 131072) {
      quick.push("dis+11_2:1_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=2:nicw=on:ptb=off:ssec=off:sos=all:sac=on:sio=off:spo=on:spl=sat:sscc=on:sser=off:ssfq=2.0:sp=occurrence_1");
      quick.push("ott+1_5:1_bs=off:cond=fast:drc=off:fde=none:lcm=reverse:nwc=10:nicw=on:ptb=off:ssec=off:sos=on:spl=sat:sp=occurrence_24");
      quick.push("dis-1010_7_bs=off:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:updr=off_11");
      quick.push("dis-1003_50_bs=off:cond=fast:drc=off:fde=none:lcm=predicate:nwc=1:nicw=on:ptb=off:ssec=off:sos=on:sgo=on:sio=off:spo=on:spl=sat_1");
      quick.push("ott-1010_8:1_bs=off:flr=on:fsr=off:gs=on:nwc=4:ptb=off:ssec=off:sac=on:sio=off:sp=reverse_arity:urr=on_6");
      quick.push("ott+1011_5:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=backtracking:sp=occurrence_141");
      quick.push("dis+1_20_bd=off:bs=off:drc=off:ep=on:flr=on:lcm=predicate:nwc=2:nicw=on:ptb=off:ssec=off:sswn=on:ss=included:st=5.0:sos=on:sagn=off:spl=sat:ssnc=none_28");
      quick.push("dis+1010_128_bs=off:cond=fast:drc=off:ep=on:flr=on:gs=on:lcm=predicate:nwc=1.1:nicw=on:ptb=off:ssec=off:sswsr=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sac=on:sio=off:spo=on:spl=sat:sfv=off_25");
      quick.push("dis+1011_3_bs=off:drc=off:fde=none:gs=on:nwc=1.1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sagn=off:sac=on:sio=off:spl=sat_31");
      quick.push("ins-11_8:1_bs=off:bsr=unit_only:igbrr=0.9:igrr=1/128:igrp=400:igrpq=1.5:igwr=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=all:spl=off:sp=occurrence:updr=off_160");
      quick.push("dis+1_16_bd=off:bs=off:cond=fast:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:urr=on_106");
      quick.push("lrs+1_4_bs=off:drc=off:ecs=on:ep=on:flr=on:fsr=off:lcm=predicate:nwc=1.3:nicw=on:ptb=off:ssec=off:stl=30:spl=sat:sser=off:ssfp=40000:ssfq=1.1:ssnc=none:sp=occurrence_96");
      quick.push("dis-1010_3_bs=off:drc=off:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sd=1:ss=axioms:st=3.0:sac=on:sio=off:spl=sat_190");
      quick.push("dis+1_3:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=1.5:sos=on:sagn=off:sac=on:sio=off:spl=backtracking:sp=occurrence_201");
      quick.push("dis+2_5:4_bs=off:br=off:cond=on:drc=off:fde=none:gsp=input_only:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sio=off:spl=backtracking:urr=on_219");
      quick.push("ott+1011_10_bd=off:bs=off:drc=off:ep=on:fde=none:gs=on:nwc=5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=reverse_arity_228");
      quick.push("ott-2_5:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:fde=none:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking_1");
    }
    else if (prop > 255) {
      quick.push("ott+1_2_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=4:ptb=off:ssec=off:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_21");
      quick.push("dis+1_16_bd=off:bs=off:cond=fast:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:urr=on_112");
      quick.push("dis+10_64_bd=off:bs=off:cond=fast:drc=off:ep=RSTC:flr=on:gsp=input_only:nwc=4:nicw=on:ptb=off:ssec=off:sswn=on:sd=10:ss=included:st=2.0:sos=all:spl=backtracking_56");
      quick.push("ott-1010_8:1_bs=off:flr=on:fsr=off:gs=on:nwc=4:ptb=off:ssec=off:sac=on:sio=off:sp=reverse_arity:urr=on_22");
      quick.push("dis-1_2:3_bs=off:drc=off:ecs=on:ep=RST:fde=none:gs=on:nwc=4:nicw=on:ssec=off:sos=on:sio=off:spl=off_23");
      quick.push("ott+1_8_bs=off:drc=off:lcm=reverse:nwc=1:ptb=off:ssec=off:sos=on:sio=off:spo=on:spl=backtracking:sp=reverse_arity_76");
      quick.push("dis-1010_3_bs=off:drc=off:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sd=1:ss=axioms:st=3.0:sac=on:sio=off:spl=sat_56");
      quick.push("dis-1010_7_bs=off:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:updr=off_227");
      quick.push("dis+1011_3_bs=off:drc=off:fde=none:nwc=10:nicw=on:ss=axioms:st=2.0:sac=on:sio=off_75");
      quick.push("dis+1002_5:1_bs=off:bms=on:drc=off:ep=on:flr=on:fde=none:gsp=input_only:nwc=1:nicw=on:ssec=off:sac=on:sio=off:updr=off_388");
      quick.push("dis+1_2:1_bs=off:bms=on:br=off:drc=off:gsp=input_only:gs=on:lcm=predicate:nwc=1:nicw=on:ssec=off:sd=3:sgt=7:ss=axioms:st=2.0:sos=on:spo=on:urr=on_202");
    }
    else {
      quick.push("dis+1011_14_bd=off:bs=off:drc=off:nwc=4:ptb=off:ssec=off:sswn=on:sac=on:spl=sat:sp=occurrence_7");
      quick.push("ins+10_4:1_bs=off:bsr=unit_only:cond=on:ep=RSTC:gs=on:igbrr=0.4:igrr=1/128:igrp=100:igrpq=1.3:igwr=on:nwc=1.1:ptb=off:ssec=off:sos=on:sio=off:spl=off:urr=on_2");
      quick.push("lrs+1003_8:1_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:fsr=off:gsp=input_only:gs=on:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:stl=90:sgo=on:sio=off:spl=backtracking:sp=occurrence:urr=on_7");
      quick.push("dis+1_3:1_bs=off:drc=off:fde=none:gs=on:nwc=1.7:ptb=off:ssec=off:sos=all:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence_1");
      quick.push("ott+11_12_cond=fast:drc=off:ecs=on:gs=on:nwc=1.2:nicw=on:ssec=off:sp=occurrence_4");
      quick.push("ott-1_2:3_bs=off:bms=on:cond=on:drc=off:fde=none:lcm=predicate:nwc=1.7:nicw=on:sd=5:ss=axioms:st=1.2:sio=off:urr=on_23");
      quick.push("dis+1_16_bd=off:bs=off:cond=fast:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:urr=on_2");
      quick.push("ott+2_8:1_bs=off:bsr=on:cond=fast:drc=off:fde=none:lcm=reverse:nwc=4:nicw=on:ptb=off:ssec=off:sos=on:spl=sat:sser=off:ssfp=100000:ssnc=none:urr=on_24");
      quick.push("dis+2_5_bs=off:cond=fast:drc=off:nwc=1.3:nicw=on:ptb=off:ssec=off:ss=included:st=1.5:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_1");
      quick.push("dis+11_16_bs=off:drc=off:gs=on:nwc=5:nicw=on:ptb=off:ssec=off:sos=on:sio=off:spo=on:spl=sat:sser=off:ssfp=1000:ssnc=none:sp=occurrence:urr=on_2");
      quick.push("lrs+1003_8:1_bs=off:drc=off:ep=on:fsr=off:gsp=input_only:gs=on:nwc=1.5:ptb=off:ssec=off:stl=30:sio=off:spl=sat_28");
      quick.push("dis+2_40_bs=off:drc=off:gs=on:lcm=predicate:nwc=5:ptb=off:ssec=off:sd=1:sgt=7:ss=axioms:sac=on:sgo=on:sio=off:spl=sat:sp=occurrence_2");
      quick.push("dis-1002_5:1_bs=off:bms=on:drc=off:ecs=on:flr=on:gs=on:nwc=10:nicw=on:ssec=off:sac=on:sgo=on:urr=on_1");
      quick.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:ptb=off:ssec=off:spl=sat_1");
      quick.push("dis-1010_3_bs=off:drc=off:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sd=1:ss=axioms:st=3.0:sac=on:sio=off:spl=sat_13");
      quick.push("dis-1010_7_bs=off:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:updr=off_41");
      quick.push("dis+1011_5_bs=off:bms=on:drc=off:fde=none:nwc=1.5:nicw=on:ssec=off:sswn=on:sswsr=on:sd=3:sgt=5:ss=axioms:st=3.0:sos=on:sagn=off:sgo=on:spo=on:sp=reverse_arity_21");
      quick.push("dis+2_128_bs=off:bms=on:drc=off:fde=none:lcm=reverse:nwc=1.3:nicw=on:ssec=off:sos=on:sio=off:spl=off_2");
      quick.push("dis+11_2:3_bs=off:bsr=unit_only:drc=off:gs=on:lcm=reverse:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=on:spl=sat:sp=reverse_arity:urr=on:updr=off_15");
      quick.push("ott+1011_8:1_bs=off:bsr=unit_only:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_236");
      quick.push("dis+1011_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=5:nicw=on:ssec=off:sswn=on:sp=reverse_arity_71");
      quick.push("ott+10_5:1_bd=off:bs=off:cond=fast:drc=off:gsp=input_only:lcm=reverse:nwc=4:nicw=on:ptb=off:ssec=off:ss=axioms:st=3.0:sos=on:sio=off:spo=on:spl=sat:urr=on_1");
      quick.push("ins+1011_3:1_bs=off:cond=fast:ep=RSTC:igbrr=0.8:igrr=1/2:igrp=200:igrpq=1.0:igwr=on:nwc=2:ptb=off:ssec=off:sos=all:sio=off:spl=off_232");
      quick.push("dis+3_4_bd=off:bs=off:cond=fast:drc=off:fsr=off:fde=none:gs=on:lcm=reverse:nwc=1.7:ptb=off:ssec=off:sos=on:spl=sat_39");
      quick.push("dis+1011_3_bs=off:drc=off:fde=none:gs=on:nwc=1.1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sagn=off:sac=on:sio=off:spl=sat_178");
      quick.push("dis+11_2:1_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=2:nicw=on:ptb=off:ssec=off:sos=all:sac=on:sio=off:spo=on:spl=sat:sscc=on:sser=off:ssfq=2.0:sp=occurrence_52");
      quick.push("lrs+3_8:1_bsr=on:cond=on:drc=off:flr=on:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:stl=90:sgo=on:sio=off:spl=backtracking:sp=reverse_arity:urr=on_370");
      quick.push("dis-1002_3_bs=off:cond=on:drc=off:gs=on:nwc=3:nicw=on:ssec=off:sgo=on:sio=off:sp=reverse_arity_21");
      quick.push("ott+1011_5:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=backtracking:sp=occurrence_73");
      quick.push("ott+1010_2:3_bs=off:drc=off:fde=none:nwc=1.2:sac=on_81");
    }
    break;

  case Property::FNE:
    if (prop == 0) {
      if (atoms > 400) {
        quick.push("ott+11_14_bs=off:drc=off:ep=RS:flr=on:fde=none:lcm=reverse:nwc=1.2:ptb=off:ssec=off:sio=off:spo=on_6");
        quick.push("dis+3_2_bs=off:gs=on:nwc=2.5:ptb=off:ssec=off:sac=on:spl=sat:sp=reverse_arity_1");
        quick.push("dis+3_7_bs=off:bms=on:cond=fast:ecs=on:fsr=off:lcm=reverse:nwc=1.1:ssec=off_54");
        quick.push("dis+1004_5_bs=off:nwc=4:nicw=on:ptb=off:ssec=off:sagn=off:spl=backtracking_129");
        quick.push("dis+11_40_bs=off:bsr=on:cond=on:fsr=off:lcm=reverse:nwc=2.5:ptb=off:ssec=off:sio=off:spl=sat:updr=off_249");
        quick.push("ott+1_10_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=2.5:ptb=off:ssec=off_347");
        quick.push("dis+1011_7_bs=off:nwc=4:nicw=on:ptb=off:ssec=off:sos=all:sio=off:spo=on:spl=sat:ssfq=1.4:ssnc=none:updr=off_128");
        quick.push("dis+1011_64_bs=off:gs=on:nwc=3:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=sat_141");
      }
      else {
        quick.push("dis+1011_64_bs=off:gs=on:nwc=3:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=sat_4");
        quick.push("dis+2_3:2_bs=off:cond=on:lcm=reverse:nwc=1:nicw=on:ptb=off:ssec=off:sio=off:spl=sat_1");
        quick.push("ott+10_2:1_bs=off:bsr=on:cond=fast:lcm=predicate:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=sat:sp=occurrence:urr=on_25");
        quick.push("dis+1011_7_bs=off:cond=on:fsr=off:gs=on:nwc=1:ptb=off:ssec=off:sos=all:spl=sat_63");
        quick.push("ott+1_40_bsr=on:cond=fast:gs=on:nwc=1.5:ptb=off:ssec=off:sio=off:spo=on:spl=sat:urr=on_104");
        quick.push("ott+10_4_bs=off:gsp=input_only:gs=on:lcm=reverse:nwc=10:nicw=on:ptb=off:ssec=off:sp=occurrence:urr=on_107");
        quick.push("tab+10_1_spl=off:tfsr=off:tgawr=4/1:tglr=1/20:tipr=off:tlawr=8/1_112");
        quick.push("dis+1010_10_bs=off:nwc=1.1:ptb=off:ssec=off:sio=off_147");
        quick.push("dis-1002_4_bs=off:bsr=unit_only:gsp=input_only:gs=on:nwc=3:nicw=on:ptb=off:ssec=off:spl=sat:urr=on:updr=off_255");
        quick.push("ott-1010_128_bs=off:cond=on:nwc=1.3:ssec=off:urr=on_401");
      }
    }
    else {
      quick.push("ott+11_4_bs=off:nwc=1.2:nicw=on:ptb=off:sd=10:ss=axioms:st=2.0:sos=all:sac=on:sio=off:spo=on:spl=sat:ssfp=1000:ssfq=1.4:ssnc=none:updr=off_43");
      quick.push("ott-1010_1024_bs=off:drc=off:fde=none:nwc=2:sos=on_100");
      quick.push("dis+10_7_bd=off:bs=off:cond=fast:drc=off:ep=R:flr=on:fde=none:nwc=10:nicw=on:ptb=off:ssec=off:sswn=on:sd=10:ss=included:st=1.5:spo=on:spl=backtracking_272");
      quick.push("tab+10_1_bms=on:sd=3:ss=axioms:st=1.2:sio=off:tbsr=off:tgawr=1/32:tglr=1/10:tipr=off:tlawr=3/1_47");
      quick.push("ott+11_14_bs=off:drc=off:ep=RS:flr=on:fde=none:lcm=reverse:nwc=1.2:ptb=off:ssec=off:sio=off:spo=on_69");
      quick.push("dis-1002_2:3_bd=off:bs=off:cond=fast:drc=off:fde=none:nwc=3:nicw=on:ptb=off:ssec=off:sac=on:sgo=on:sio=off:spl=backtracking_280");
      quick.push("dis+11_64_bs=off:cond=fast:gs=on:nwc=3:nicw=on:ptb=off:ss=axioms:st=1.2:sac=on:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none_88");
    }
    break;

  case Property::EPR:
    quick.push("ins+11_14_bs=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=4000:igrpq=1.5:igwr=on:lcm=predicate:nwc=3:ptb=off:ssec=off:spl=off:urr=on_55");
    quick.push("ott+10_8:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:spo=on:spl=sat:sser=off:ssnc=none:urr=on_166");
    quick.push("dis+1_8:1_bs=off:drc=off:flr=on:nwc=1.5:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.2_164");
    quick.push("ins+4_14_bs=off:cond=on:flr=on:igbrr=0.2:igrr=1/128:igrp=700:igrpq=1.2:igwr=on:nwc=1:ptb=off:ssec=off:spl=off:urr=on_41");
    quick.push("dis+1_20_bs=off:nwc=4:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=sat:ssfp=4000:ssfq=1.1:sp=occurrence:updr=off_285");
    quick.push("ott+1_24_bs=off:cond=fast:nwc=1.3:nicw=on:ptb=off:ssec=off:sac=on:spo=on:spl=sat:urr=on_191");
    quick.push("ins+1_5_bs=off:bsr=on:drc=off:gs=on:igbrr=0.0:igrr=1/32:igrp=400:igrpq=2.0:igwr=on:lcm=predicate:nwc=2:ptb=off:ssec=off:sio=off:spl=off:sp=reverse_arity:urr=on:updr=off_264");
    quick.push("dis+3_10_bs=off:drc=off:ecs=on:fsr=off:nwc=1.2:nicw=on:ssec=off:sio=off:spl=off_477");
    break;

  case Property::UEQ:
    if (prop == 0) {
      if (atoms > 11) {
        quick.push("lrs+10_20_bs=off:drc=off:fsr=off:nwc=1:stl=120:sp=occurrence_295");
        quick.push("lrs+10_8:1_nwc=1:stl=90:sfv=off:sp=reverse_arity_271");
        quick.push("ott+10_7_bs=off:drc=off:fsr=off:nwc=1.1:sp=occurrence_249");
        quick.push("lrs+10_128_bs=unit_only:drc=off:fsr=off:nwc=1.2:stl=120:sfv=off:sp=occurrence_709");
        quick.push("dis+10_128_bs=unit_only:drc=off:fsr=off:gsp=input_only:nwc=10_296");
      }
      else {
        quick.push("lrs+10_20_bs=off:drc=off:fsr=off:nwc=1:stl=120:sp=occurrence_357");
        quick.push("ott+10_8:1_bs=off:drc=off:fsr=off:fde=none:nwc=5_372");
        quick.push("ott+10_7_bs=off:drc=off:fsr=off:gsp=input_only:nwc=3:sp=occurrence_504");
        quick.push("ott+10_1_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.3_527");
      }
    }
    else if (prop == 2) {
      if (atoms > 18) {
        quick.push("ott+10_8_bd=off:bs=off:drc=off:fsr=off:fde=none:nwc=2:sp=occurrence_45");
        quick.push("dis+10_5_bs=off:drc=off:fsr=off:nwc=1.5:sp=occurrence_386");
        quick.push("ott+10_3_bs=off:drc=off:fsr=off:fde=none:gsp=input_only:nwc=1.3:sp=occurrence_205");
        quick.push("lrs+10_128_bs=unit_only:drc=off:fsr=off:nwc=1.2:stl=120:sfv=off:sp=occurrence_1139");
      }
      else {
        quick.push("ott+10_6_bs=off:drc=off:fsr=off:fde=none:nwc=1.2_108");
        quick.push("ott+10_1_bs=off:drc=off:fsr=off:fde=none:nwc=5_2");
        quick.push("lrs+10_20_bd=off:bs=unit_only:drc=off:fsr=off:nwc=1.2:stl=90_528");
        quick.push("ott+10_32_bs=off:bsr=on:drc=off:fsr=off:fde=none:nwc=1.3_170");
        quick.push("ott+10_8:1_bs=off:bsr=on:nwc=1.3_113");
        quick.push("ott+10_3_bs=off:drc=off:fsr=off:fde=none:gsp=input_only:nwc=1.3:sp=occurrence_200");
        quick.push("ott+10_50_bd=off:drc=off:fsr=off:nwc=1.1:sp=occurrence_283");
      }
    }
    else {
      quick.push("dis+10_128_bs=unit_only:drc=off:fsr=off:gsp=input_only:nwc=10_148");
      quick.push("dis+4_5:1_bs=off:br=off:ep=RSTC:fsr=off:nwc=3:ptb=off:sos=all:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_94");
      quick.push("lrs+10_128_bs=unit_only:drc=off:fsr=off:nwc=1.2:stl=120:sfv=off:sp=occurrence_610");
      quick.push("lrs+10_2:1_bd=off:bs=off:drc=off:fsr=off:nwc=2.5:stl=30_143");
    }
    break;
  }

  switch (cat) {
  case Property::NEQ:
    fallback.push("ott+1011_3_bs=off:bsr=unit_only:ep=on:nwc=2:ptb=off:ssec=off:spl=sat_300");
    fallback.push("dis-1002_2:1_bs=off:drc=off:ep=on:fde=none:gsp=input_only:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=all:sp=occurrence_300");
    fallback.push("ott+11_2_bd=off:bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=10:ptb=off:ssec=off:spl=sat:sp=reverse_arity_300");
    fallback.push("dis+11_5:1_bs=off:drc=off:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=all:sgo=on:sio=off:spo=on:spl=sat:sp=occurrence_300");
    fallback.push("dis+11_50_bs=off:drc=off:gsp=input_only:nwc=1.3:nicw=on:ptb=off:ssec=off:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none_300");
    fallback.push("dis+1011_24_cond=fast:drc=off:nwc=10:nicw=on:ptb=off:ssec=off:spl=sat_300");
    fallback.push("dis-1010_5_bs=off:bsr=unit_only:cond=fast:ep=on:gsp=input_only:lcm=reverse:nwc=1:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=backtracking:sp=occurrence_300");
    fallback.push("dis+10_14_bs=off:cond=fast:drc=off:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=3.0:sac=on:spo=on:sp=occurrence_300");
    fallback.push("ott+1011_3_bs=off:drc=off:ep=on:fde=none:nwc=1.1:nicw=on:ptb=off:ssec=off:spl=sat_300");
    fallback.push("lrs+2_3:1_bs=off:br=off:drc=off:flr=on:lcm=predicate:nwc=10:ssec=off:sgo=on:sio=off:spo=on:sp=occurrence:urr=on_600");
    fallback.push("ott+11_5_bs=off:ep=on:fde=none:nwc=1.5:nicw=on:ptb=off:ssec=off:sswsr=on:sac=on:sio=off:spo=on:spl=backtracking:sp=reverse_arity_300");
    fallback.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1:ptb=off:ssec=off:sos=on:sgo=on:sio=off:sp=occurrence_300");
    fallback.push("dis+11_8:1_bs=off:br=off:drc=off:ep=RST:nwc=1:ptb=off:ssec=off:sgo=on:sio=off:spo=on:spl=backtracking:urr=on_300");
    fallback.push("ott+11_8:1_drc=off:ep=on:nwc=1:ptb=off:ssec=off:sac=on:spo=on:sp=occurrence_600");
    fallback.push("dis+11_12_bs=off:drc=off:ep=on:flr=on:fsr=off:gsp=input_only:lcm=predicate:nwc=3:nicw=on:ptb=off:ssec=off:sos=on:spl=sat_300");
    fallback.push("lrs+11_20_bd=off:bs=off:drc=off:gs=on:nwc=2.5:spo=on_600");
    fallback.push("ins-1010_8:1_bs=off:ep=RSTC:fsr=off:igbrr=1.0:igrr=1/128:igrp=200:igrpq=2.0:igwr=on:nwc=1.2:ptb=off:ssec=off:sos=on:sio=off:spl=off_300");
    fallback.push("dis+2_64_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=4:ptb=off:sos=on:spl=sat:sser=off:ssfp=100000:ssfq=1.4:ssnc=none_300");
    fallback.push("ott+3_8:1_bd=off:bs=off:bsr=unit_only:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sos=all:spl=sat:sser=off:ssfp=10000:ssfq=1.4:ssnc=none_300");
    fallback.push("ott+11_2:3_bd=off:bs=unit_only:bsr=unit_only:cond=on:drc=off:fde=none:lcm=reverse:nwc=1.2:ptb=off:ssec=off:spl=sat_300");
    fallback.push("ott-1010_3_bd=off:bs=off:cond=on:drc=off:ep=on:fde=none:nwc=10:ptb=off:ssec=off:sd=3:ss=axioms:sos=on:sac=on:sio=off:spl=backtracking:sp=reverse_arity:urr=on_300");
    fallback.push("dis+1011_128_bs=unit_only:cond=fast:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:spl=sat:sfv=off_300");
    fallback.push("ott-1002_2:1_bd=off:bs=off:bsr=on:drc=off:ep=on:flr=on:fsr=off:lcm=predicate:nwc=2.5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:urr=on_300");
    fallback.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:gsp=input_only:nwc=4:ptb=off:ssec=off:sos=all:sio=off:spo=on:urr=on:updr=off_300");
    fallback.push("ins+1011_3_bs=off:ep=RSTC:fde=none:gs=on:igbrr=1.0:igrr=1/64:igrp=100:igrpq=1.3:igwr=on:nwc=2.5:ptb=off:ssec=off:sos=on:sio=off:spl=off_300");
    fallback.push("ott+10_50_bs=off:br=off:cond=fast:drc=off:flr=on:gs=on:nwc=4:nicw=on:ptb=off:ssec=off:sio=off:spl=sat:ssfq=2.0:ssnc=all:sp=reverse_arity:urr=on_300");
    fallback.push("ins-10_40_bs=off:cond=fast:ep=RSTC:flr=on:gsp=input_only:gs=on:igbrr=0.7:igrr=1/4:igrp=4000:igrpq=1.2:igwr=on:lcm=reverse:nwc=1.7:ptb=off:ssec=off:sio=off:spl=off:urr=on_300");
    fallback.push("dis-1010_3:2_bs=off:bsr=unit_only:drc=off:ep=RST:fde=none:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=on:sgo=on:spo=on:spl=backtracking:sp=reverse_arity_300");
    fallback.push("dis+11_50_bs=off:cond=fast:drc=off:fde=none:gs=on:lcm=predicate:nwc=2:nicw=on:ssec=off:spo=on:sp=reverse_arity_300");
    fallback.push("dis+10_1024_bs=off:cond=on:drc=off:flr=on:fsr=off:lcm=predicate:nwc=1.2:nicw=on:ptb=off:ssec=off:sac=on:spo=on_300");
    fallback.push("dis+11_24_bs=off:bms=on:cond=on:drc=off:ecs=on:fde=none:nwc=2.5:nicw=on:ssec=off:spo=on:sp=occurrence:urr=on_300");
    fallback.push("dis+11_5:1_bs=off:cond=fast:drc=off:nwc=10:sswn=on:sswsr=on_300");
    fallback.push("ott+10_2:3_bd=off:bs=off:cond=fast:drc=off:fde=none:lcm=predicate:nwc=10:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking:sp=occurrence_300");
    fallback.push("ott+10_28_bd=off:bs=off:drc=off:nwc=1.5:ptb=off:ssec=off:spl=sat_300");
    fallback.push("dis+1_5:1_bs=off:cond=on:drc=off:ep=RST:flr=on:fsr=off:lcm=reverse:nwc=10:ptb=off:ssec=off:sio=off:urr=on_300");
    fallback.push("ott-11_40_bd=off:bs=off:drc=off:ecs=on:flr=on:fsr=off:lcm=predicate:nwc=1.5:nicw=on:ssec=off:sos=on_300");
    fallback.push("ott+11_3_bs=off:br=off:drc=off:nwc=1.1:nicw=on:ss=axioms:sos=on:sio=off:urr=on_300");
    fallback.push("ott+1011_5_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=2:ptb=off:ssec=off:sos=on:sio=off:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_2_bs=off:cond=on:drc=off:ep=on:gsp=input_only:nwc=1.5:nicw=on:ptb=off:ssec=off:spl=sat:urr=on_300");
    fallback.push("dis+1011_2_bs=off:drc=off:ep=on:flr=on:nwc=1.1:nicw=on:ptb=off:ssec=off:spl=sat:urr=on:updr=off_300");
    fallback.push("dis+1011_64_bs=off:drc=off:ep=on:flr=on:nwc=2:ptb=off:ssec=off:spl=sat:sp=occurrence_300");
    fallback.push("dis+4_8:1_bs=off:bsr=on:drc=off:fsr=off:fde=none:lcm=reverse:nwc=10:ptb=off:ssec=off:spl=sat:ssfp=10000:ssfq=1.0:ssnc=none:sp=reverse_arity_300");
    fallback.push("dis-1010_3:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.7:nicw=on:ptb=off:spl=sat:sser=off:ssfp=100000:ssfq=1.1:ssnc=none_300");
    fallback.push("lrs+10_20_bd=off:bs=off:cond=fast:drc=off:ecs=on:nwc=1.5:ptb=off:ssec=off:sio=off:spo=on:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:updr=off_300");
    fallback.push("dis+11_5_bs=off:cond=fast:drc=off:ep=on:fde=none:gsp=input_only:lcm=reverse:nwc=1:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking:sp=occurrence_300");
    fallback.push("ott+2_8:1_bd=off:bs=off:bsr=unit_only:bms=on:br=off:cond=fast:drc=off:nwc=1.5:ssec=off:sos=on:sio=off:urr=on:updr=off_100");
    fallback.push("ott+10_2_bd=off:bs=off:drc=off:ecs=on:flr=on:fsr=off:nwc=1.7:ssec=off:sac=on:sgo=on:sio=off_300");
    fallback.push("dis+11_4_ep=on:fde=none:lcm=reverse:nwc=10:nicw=on:ptb=off:ssec=off:sos=on:spo=on:spl=sat_300");
    fallback.push("ott+2_3_bd=off:cond=on:drc=off:flr=on:nwc=1.3:ptb=off:ssec=off:sac=on:sgo=on:sio=off:sfv=off:sp=reverse_arity:updr=off_300");
    fallback.push("dis-1010_3_cond=fast:drc=off:nwc=4:sac=on_300");
    fallback.push("dis-2_50_bs=off:drc=off:lcm=predicate:nwc=1.7:ptb=off:ssec=off:sio=off:spl=backtracking:sp=occurrence_300");
    fallback.push("dis+1011_7_bs=off:drc=off:ep=RS:fde=none:gsp=input_only:nwc=10:ptb=off:ssec=off:spl=sat:sser=off:ssfp=40000:ssfq=1.2:ssnc=none_300");
    fallback.push("dis+1011_16_bd=off:bs=off:cond=on:drc=off:ep=on:fsr=off:gs=on:nwc=2.5:ptb=off:ssec=off:sgo=on:spo=on:spl=backtracking_300");
    fallback.push("ott+11_50_bd=off:bs=off:drc=off:ep=on:lcm=reverse:nwc=1.7:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=sat:sp=occurrence_300");
    fallback.push("dis+1011_4:1_bs=off:bsr=on:drc=off:ep=on:flr=on:gsp=input_only:gs=on:lcm=reverse:nwc=2:ptb=off:ssec=off:sos=all:spo=on:spl=sat:ssfp=10000:ssnc=none:sp=occurrence_100");
    fallback.push("lrs-1002_4:1_bs=off:gs=on:nwc=4:nicw=on:ssec=off:sgo=on:spo=on:urr=on_600");
    fallback.push("ott+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=sat_300");
    fallback.push("ott+1011_6_drc=off:ep=on:fde=none:gs=on:nwc=1.3:nicw=on:ptb=off:ssec=off:sswsr=on:sac=on:sio=off:spo=on:spl=backtracking:sfv=off_300");
    fallback.push("ott+10_64_bd=off:bs=off:cond=fast:drc=off:flr=on:gsp=input_only:lcm=predicate:nwc=2:nicw=on:ptb=off:ssec=off:sac=on:spo=on_300");
    fallback.push("ott+1011_2:3_bd=off:bs=off:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sac=on:sgo=on:sio=off:spl=backtracking:sfv=off_300");
    fallback.push("dis-1010_7_bs=off:cond=fast:drc=off:fsr=off:gsp=input_only:gs=on:nwc=1.1:ptb=off:ssec=off:sio=off:spo=on:urr=on_300");
    fallback.push("dis+11_5:4_bs=off:bsr=on:cond=on:drc=off:fde=none:lcm=reverse:nwc=1.3:nicw=on:ptb=off:ssec=off:spl=sat:sscc=on:sser=off:ssfp=40000:ssfq=1.0_300");
    fallback.push("lrs+1011_20_bd=off:bs=off:ecs=on:ep=on:fde=none:lcm=predicate:nwc=3:nicw=on:ssec=off:sio=off:spl=off:urr=on_600");
    fallback.push("ott+2_40_bsr=unit_only:drc=off:fsr=off:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sio=off:urr=on_300");
    fallback.push("ott+1_10_bd=off:bs=off:drc=off:lcm=predicate:nwc=2.5:ptb=off:ssec=off:sos=on:spl=sat_300");
    fallback.push("dis+10_5_bd=off:bs=off:cond=fast:ep=RST:gsp=input_only:lcm=predicate:nwc=5:ptb=off:ssec=off:sos=all:sac=on:sio=off:spl=backtracking:sfv=off:urr=on_100");
    fallback.push("dis+1011_12_bs=off:drc=off:fsr=off:nwc=5:nicw=on:ptb=off:ssec=off:sos=on:spl=sat_300");
    fallback.push("ott-11_8:1_bd=off:bs=off:drc=off:ep=on:fde=none:lcm=predicate:nwc=3:ptb=off:ssec=off:sos=on:sio=off:urr=on_300");
    break;

  case Property::HEQ:
  case Property::PEQ:
  case Property::EPR:
    fallback.push("dis+1_10_bs=off:fde=none:gsp=input_only:nwc=4:ptb=off:ssec=off:sac=on:sio=off:spl=sat_300");
    fallback.push("lrs+2_14_bs=off:cond=fast:drc=off:nwc=10:ptb=off:sac=on:sio=off:spo=on:spl=sat:ssac=none:ssfp=100000:ssfq=1.1:ssnc=all_dependent:sp=reverse_arity_900");
    fallback.push("ott+1_3_bs=off:bms=on:cond=on:drc=off:nwc=2:sac=on:sio=off:updr=off_900");
    fallback.push("ott+11_50_bs=off:cond=on:drc=off:flr=on:fde=none:lcm=predicate:nwc=2:nicw=on:ptb=off:sgo=on:sio=off:spl=backtracking:sp=occurrence:urr=on_1200");
    fallback.push("lrs-3_4_bsr=unit_only:cond=on:drc=off:fsr=off:fde=none:gs=on:nwc=1.2:ssec=off:sgo=on:sio=off:urr=on:updr=off_1500");
    fallback.push("ott+10_8:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:spo=on:spl=sat:sser=off:ssnc=none:urr=on_300");
    fallback.push("dis+1_6_bd=off:flr=on:gs=on:nwc=1.5:ptb=off:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity_600");
    fallback.push("ins+11_14_bs=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=4000:igrpq=1.5:igwr=on:lcm=predicate:nwc=3:ptb=off:ssec=off:spl=off:urr=on_300");
    fallback.push("dis+2_128_bs=off:drc=off:lcm=predicate:nwc=10:sac=on:sio=off:sp=occurrence_300");
    fallback.push("dis+1_8:1_bs=off:drc=off:flr=on:nwc=1.5:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.2_300");
    fallback.push("dis+11_7_bs=off:drc=off:lcm=reverse:nwc=5:ssec=off:sio=off:spl=off:updr=off_600");
    fallback.push("dis+4_50_bd=off:bs=unit_only:cond=on:drc=off:gs=on:lcm=predicate:nwc=5:nicw=on:ptb=off:sos=all:sio=off:spl=backtracking:urr=on_500");
    fallback.push("ott+2_1024_bs=off:drc=off:gsp=input_only:nwc=1.1:ptb=off:ssec=off:spl=sat:ssfq=1.4:ssnc=none:sp=occurrence_600");
    fallback.push("ott+1_24_bs=off:cond=fast:nwc=1.3:nicw=on:ptb=off:ssec=off:sac=on:spo=on:spl=sat:urr=on_300");
    fallback.push("dis+1_20_bs=off:nwc=4:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=sat:ssfp=4000:ssfq=1.1:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_2:3_bd=off:drc=off:flr=on:nwc=1.3:nicw=on:ptb=off:sio=off:spl=sat:ssnc=none_900");
    fallback.push("dis+10_5:4_cond=on:gs=on:nwc=2:nicw=on:ptb=off:sos=all:sio=off:spl=sat:sscc=on:sser=off:ssfp=1000:ssfq=1.1:ssnc=none:sfv=off:sp=occurrence_600");
    fallback.push("dis+1_12_bs=off:drc=off:ep=on:nwc=1.3:nicw=on:sos=on:sio=off:spl=off_300");
    fallback.push("lrs+11_4:1_bs=unit_only:cond=fast:drc=off:fde=none:lcm=predicate:nwc=10:nicw=on:ptb=off:sac=on:spo=on:spl=sat:ssac=none:ssfp=1000:ssfq=1.0:ssnc=all_600");
    fallback.push("dis+2_8_bd=off:bs=off:ep=RST:fsr=off:lcm=reverse:nwc=3:nicw=on:ptb=off:ssec=off:spo=on:spl=backtracking:sfv=off_300");
    fallback.push("lrs+11_1_bd=off:bs=off:cond=on:drc=off:gs=on:nwc=2:ptb=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none_300");
    fallback.push("dis+10_10_bs=off:cond=fast:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:sio=off:spo=on:spl=sat:sser=off:ssnc=none_600");
    fallback.push("ott+3_128_bs=off:br=off:cond=fast:drc=off:ep=on:nwc=1:nicw=on:ptb=off:sos=all:sio=off:spo=on:spl=backtracking:urr=on_300");
    fallback.push("ott+4_6_bs=off:bsr=on:cond=on:drc=off:fsr=off:nwc=3:ssec=off:sswn=on:sac=on:sio=off:sp=occurrence_300");
    fallback.push("ins+1_5_bs=off:bsr=on:drc=off:gs=on:igbrr=0.0:igrr=1/32:igrp=400:igrpq=2.0:igwr=on:lcm=predicate:nwc=2:ptb=off:ssec=off:sio=off:spl=off:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis-1010_3_bs=off:drc=off:lcm=predicate:nwc=10:ptb=off:ssec=off:spo=on_300");
    fallback.push("dis+1_2:1_bs=off:cond=on:drc=off:nwc=5:sos=on:sio=off:spo=on:sp=occurrence_300");
    fallback.push("ott+4_5_bd=off:bsr=unit_only:cond=fast:drc=off:fsr=off:gs=on:nwc=1.5:nicw=on:sac=on:sio=off_300");
    fallback.push("dis+11_8_bsr=unit_only:cond=on:drc=off:fsr=off:gs=on:nwc=1.1:nicw=on:ptb=off:ssec=off:sos=all:sio=off:spo=on:spl=sat:sser=off:ssfq=1.4:ssnc=none_200");
    fallback.push("ott+4_28_bs=off:cond=on:drc=off:fde=none:gs=on:nwc=1.1:ptb=off:ssec=off:sgo=on:sio=off:spo=on:spl=sat_300");
    fallback.push("ott+10_8:1_drc=off:ecs=on:ep=RSTC:gs=on:nwc=1.3:ssec=off:sac=on:sio=off_300");
    fallback.push("ott+3_20_bs=off:cond=fast:drc=off:fde=none:nwc=2:ptb=off:ssec=off:sgo=on:sio=off:spo=on:spl=sat:sp=occurrence_300");
    fallback.push("ins+4_14_bs=off:cond=on:flr=on:igbrr=0.2:igrr=1/128:igrp=700:igrpq=1.2:igwr=on:nwc=1:ptb=off:ssec=off:spl=off:urr=on_300");
    fallback.push("dis-1004_1024_bs=off:cond=fast:drc=off:ep=on:nwc=1.2_300");
    fallback.push("ins+4_3_cond=fast:flr=on:igbrr=0.6:igrr=1/128:igrp=700:igrpq=1.2:igwr=on:nwc=1.7:ptb=off:sio=off:spl=off:updr=off_300");
    fallback.push("ott+11_40_bd=off:bs=off:cond=on:drc=off:nwc=3:nicw=on:ptb=off:ssec=off:spl=sat:sp=reverse_arity_300");
    fallback.push("dis+3_10_bs=off:drc=off:ecs=on:fsr=off:nwc=1.2:nicw=on:ssec=off:sio=off:spl=off_600");
    fallback.push("ott+10_64_bs=off:cond=on:drc=off:ecs=on:flr=on:gsp=input_only:nwc=5:nicw=on:ssec=off:spo=on:sp=reverse_arity:updr=off_600");
    fallback.push("dis+2_6_bd=off:drc=off:flr=on:fsr=off:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking_600");
    fallback.push("ott+11_64_bs=off:drc=off:ecs=on:gs=on:nwc=1.5:ssec=off:sio=off:spl=off_900");
    fallback.push("ins+11_50_bs=off:drc=off:fde=none:igbrr=0.7:igrr=1/32:igrp=700:igrpq=1.2:igwr=on:nwc=3:ptb=off:sio=off:spl=off:sp=reverse_arity_1200");
    break;

  case Property::HNE:
  case Property::NNE:
    fallback.push("ott+4_6_bs=off:bsr=unit_only:cond=fast:nwc=1.2:nicw=on:ptb=off:ssec=off:sio=off:spo=on:spl=sat_300");
    fallback.push("dis+1011_64_bs=off:nwc=1.7:ptb=off:ssec=off:spl=sat_300");
    fallback.push("dis+1011_20_bs=off:fsr=off:nwc=2:sio=off:spl=off_300");
    fallback.push("lrs+1002_7_bs=off:cond=on:flr=on:fsr=off:gsp=input_only:gs=on:lcm=predicate:nwc=1.2:ptb=off:spl=sat:sser=off:ssfp=40000:ssfq=2.0:ssnc=none_300");
    fallback.push("dis+4_12_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=10:ptb=off:ssec=off:spl=sat:sp=occurrence_300");
    fallback.push("lrs+10_3:1_bs=off:bsr=unit_only:cond=fast:nwc=10:ptb=off:ssec=off:sio=off:spo=on_900");
    fallback.push("dis+1011_128_bs=off:nwc=5:ptb=off:ssec=off:sswsr=on:sos=on:spl=sat_300");
    fallback.push("lrs-1003_28_bs=unit_only:bms=on:cond=on:flr=on:fsr=off:gsp=input_only:nwc=1:nicw=on:ssec=off:sio=off:spl=off:sp=occurrence_600");
    fallback.push("lrs+1_1024_bs=off:bms=on:cond=on:ecs=on:nwc=1:nicw=on:ssec=off:sac=on:sio=off_900");
    fallback.push("dis+11_128_bs=off:cond=fast:flr=on:lcm=reverse:nwc=2:ptb=off:ssec=off:spl=sat_300");
    fallback.push("dis+1011_20_bs=off:gsp=input_only:nwc=1.3:nicw=on:ptb=off:ssec=off:sswsr=on:sos=on:sac=on:sio=off:spo=on:spl=sat_300");
    fallback.push("dis-2_14_bs=off:cond=fast:flr=on:lcm=predicate:nwc=1:nicw=on:ptb=off:ssec=off:sagn=off:sac=on:sio=off:spl=backtracking_300");
    fallback.push("lrs+3_8_bs=unit_only:cond=fast:flr=on:nwc=4:nicw=on:ptb=off:ssec=off:sio=off:spl=backtracking_300");
    fallback.push("dis+2_5:4_cond=fast:ecs=on:flr=on:nwc=1.5:ssec=off:sio=off:spl=off_600");
    fallback.push("dis+11_64_bs=unit_only:bms=on:lcm=reverse:nwc=1.5:sio=off_300");
    fallback.push("dis+1011_128_bs=off:cond=fast:flr=on:fsr=off:lcm=reverse:nwc=2.5:nicw=on:ptb=off:ssec=off:sswsr=on:spl=sat_300");
    fallback.push("dis+1011_6_bs=off:nwc=1.2:ptb=off:ssec=off:spl=sat_300");
    fallback.push("lrs+2_5:1_bs=off:cond=fast:flr=on:fsr=off:lcm=predicate:nwc=10:ptb=off:ssec=off:spl=backtracking_900");
    fallback.push("dis+11_20_bs=off:fsr=off:gsp=input_only:lcm=reverse:nwc=1.3:ptb=off:ssec=off:spl=sat:sp=occurrence_300");
    fallback.push("dis+11_1_bs=off:cond=fast:fsr=off:nwc=10:ptb=off:ssec=off:sio=off:spo=on:spl=sat_300");
    fallback.push("lrs+4_1024_bs=off:flr=on:fsr=off:gs=on:lcm=reverse:nwc=1.2:nicw=on:ptb=off:ssec=off:sos=on:sio=off:spo=on:spl=backtracking_300");
    fallback.push("dis+1_40_bs=off:cond=on:gsp=input_only:lcm=predicate:nwc=4:nicw=on:ptb=off:ssec=off:sgo=on:sio=off:spl=sat_300");
    fallback.push("dis+1011_12_bs=off:cond=fast:ecs=on:flr=on:nwc=1:ssec=off_300");
    fallback.push("dis+3_50_bs=off:fsr=off:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:spl=sat:updr=off_300");
    fallback.push("ott-1002_2_bs=off:ecs=on:flr=on:lcm=predicate:nwc=2:nicw=on:ssec=off:sos=on:spo=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1_2:1_bs=off:bsr=unit_only:fsr=off:lcm=reverse:nwc=1.1:nicw=on:ptb=off:ssec=off:sio=off:spl=sat:sser=off:ssfq=2.0_300");
    fallback.push("ott+1_8:1_bs=off:br=off:cond=on:gs=on:nwc=3:ptb=off:ssec=off:sio=off:spl=sat:urr=on_300");
    fallback.push("ott+1010_2_bs=off:gs=on:nwc=5:ptb=off:ssec=off:spl=sat_300");
    fallback.push("dis+1002_50_bs=off:fsr=off:lcm=predicate:nwc=1.2:nicw=on:ssec=off_600");
    fallback.push("lrs+1011_64_bs=unit_only:bsr=unit_only:cond=fast:flr=on:nwc=1:ptb=off:sio=off:spo=on:spl=backtracking_1800");
    break;

  case Property::FEQ:
    fallback.push("dis+1011_14_bd=off:bs=off:drc=off:nwc=4:ptb=off:ssec=off:sswn=on:sac=on:spl=sat:sp=occurrence_300");
    fallback.push("ott+1011_8:1_bs=off:bsr=unit_only:drc=off:ep=on:fde=none:nwc=1.3:ptb=off:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_300");
    fallback.push("dis+1010_24_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=2.5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=occurrence_300");
    fallback.push("ott+1_2_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=4:ptb=off:ssec=off:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_300");
    fallback.push("dis+1_16_bd=off:bs=off:cond=fast:drc=off:fsr=off:nwc=1.2:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:urr=on_300");
    fallback.push("lrs+4_20_cond=fast:drc=off:flr=on:lcm=predicate:nwc=1.2:nicw=on:ptb=off:ssec=off:sgo=on:sio=off:spl=backtracking:sp=occurrence_900");
    fallback.push("ott-1002_5:4_bs=off:drc=off:fde=none:nwc=1.7:ptb=off:ssec=off:sos=on:sac=on:sio=off:spo=on:spl=sat:urr=on_300");
    fallback.push("dis+1_5:1_bs=off:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sswn=on:sd=1:ss=included:sagn=off:spl=sat:ssfq=1.4:ssnc=none_300");
    fallback.push("dis-1002_3_bs=off:cond=fast:drc=off:ep=RS:nwc=1.1:ptb=off:ssec=off:swb=on_300");
    fallback.push("dis+2_24_bs=off:ep=RSTC:flr=on:lcm=reverse:nwc=5:nicw=on:ssec=off:sac=on:sgo=on:spo=on:sfv=off_1200");
    fallback.push("dis+1011_3_bs=off:drc=off:fde=none:gs=on:nwc=1.1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sagn=off:sac=on:sio=off:spl=sat_300");
    fallback.push("dis+2_8:1_bd=off:bs=off:bsr=unit_only:drc=off:lcm=reverse:nwc=2.5:nicw=on:ptb=off:ssec=off:sswn=on:sswsr=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sp=reverse_arity_300");
    fallback.push("ott+1010_2:3_bs=off:drc=off:fde=none:nwc=1.2:sac=on_300");
    fallback.push("dis+11_8:1_bs=off:br=off:cond=on:drc=off:ecs=on:ep=RST:fsr=off:nwc=1:ptb=off:ssec=off:sd=7:ss=axioms:st=2.0:sio=off:spo=on:spl=sat:ssnc=none:urr=on:updr=off_300");
    fallback.push("dis-1010_5:1_bsr=on:drc=off:fde=none:gsp=input_only:gs=on:nwc=4:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=sat:updr=off_300");
    fallback.push("dis+1_2_bs=off:cond=on:drc=off:lcm=predicate:nwc=2:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=2.0:sagn=off:sac=on:sio=off:spl=sat:sser=off:ssnc=none:updr=off_300");
    fallback.push("ott+1011_10_bd=off:bs=off:drc=off:ep=on:fde=none:gs=on:nwc=5:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:sp=reverse_arity_300");
    fallback.push("ins+1011_3:1_bs=off:cond=fast:ep=RSTC:igbrr=0.8:igrr=1/2:igrp=200:igrpq=1.0:igwr=on:nwc=2:ptb=off:ssec=off:sos=all:sio=off:spl=off_300");
    fallback.push("lrs+3_8:1_bsr=on:cond=on:drc=off:flr=on:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sgo=on:sio=off:spl=backtracking:sp=reverse_arity:urr=on_900");
    fallback.push("ott+1_8:1_bs=off:cond=fast:drc=off:ep=on:lcm=predicate:nwc=10:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=backtracking_300");
    fallback.push("ins-11_8:1_bs=off:bsr=unit_only:igbrr=0.9:igrr=1/128:igrp=400:igrpq=1.5:igwr=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=all:spl=off:sp=occurrence:updr=off_300");
    fallback.push("ott+2_6_bs=off:cond=fast:drc=off:fde=none:nwc=1.2:ptb=off:ssec=off:sio=off:spl=sat_300");
    fallback.push("dis-1_1_bs=off:ep=RSTC:gs=on:nwc=1.1:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off_300");
    fallback.push("dis+10_5:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=5:nicw=on:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_300");
    fallback.push("ott+1_12_bs=off:br=off:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sio=off:spo=on:spl=sat:sser=off:ssfq=1.0:ssnc=none:urr=on_300");
    fallback.push("dis-1010_7_bs=off:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:updr=off_300");
    fallback.push("dis-1010_3_bs=off:drc=off:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sd=1:ss=axioms:st=3.0:sac=on:sio=off:spl=sat_300");
    fallback.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=3:ptb=off:ssec=off:sswn=on:sswsr=on:sd=3:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence:updr=off_300");
    fallback.push("ott-1002_1024_bd=off:bs=off:cond=on:drc=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=on:sgo=on:sio=off:spo=on:spl=backtracking:urr=on_300");
    fallback.push("ott+2_5:1_bd=off:bs=off:drc=off:gsp=input_only:nwc=1.1:nicw=on:ptb=off:ssec=off:sd=3:ss=axioms:sos=on:spl=sat:urr=on_300");
    fallback.push("lrs+1003_8:1_bs=off:drc=off:ep=on:fsr=off:gsp=input_only:gs=on:nwc=1.5:ptb=off:ssec=off:sio=off:spl=sat_300");
    fallback.push("ott+4_3:1_bs=off:bms=on:br=off:drc=off:fde=none:nwc=1.7:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:sfv=off:urr=on_300");
    fallback.push("dis+2_5:4_bs=off:br=off:cond=on:drc=off:fde=none:gsp=input_only:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sio=off:spl=backtracking:urr=on_300");
    fallback.push("lrs+11_2:1_bs=off:cond=on:drc=off:ep=on:fde=none:gs=on:lcm=predicate:nwc=4:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spo=on:spl=backtracking:urr=on_300");
    fallback.push("ott-11_4:1_bs=off:bsr=on:cond=fast:drc=off:ecs=on:nwc=1.3:ssec=off:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sio=off_300");
    fallback.push("dis+2_4_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.2:nicw=on:ptb=off:ssec=off:sswn=on:sd=2:sgt=2:ss=axioms:sos=on:sgo=on:spl=backtracking:sp=reverse_arity_300");
    fallback.push("dis-1010_2_bs=off:cond=fast:drc=off:nwc=1.7:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=backtracking:updr=off_300");
    fallback.push("dis+1_3:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:sswn=on:sd=1:ss=included:st=1.5:sos=on:sagn=off:sac=on:sio=off:spl=backtracking:sp=occurrence_300");
    fallback.push("lrs+3_24_bd=off:cond=on:drc=off:nwc=1.3:ptb=off:ssec=off:sio=off:spl=sat:sp=occurrence_300");
    fallback.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=5:ptb=off:ssec=off:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_300");
    fallback.push("dis+1011_3_bd=off:bs=off:drc=off:ep=on:fde=none:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spo=on:spl=backtracking:updr=off_300");
    fallback.push("dis+1010_128_bs=off:cond=fast:drc=off:ep=on:flr=on:gs=on:lcm=predicate:nwc=1.1:nicw=on:ptb=off:ssec=off:sswsr=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sac=on:sio=off:spo=on:spl=sat:sfv=off_300");
    fallback.push("ott+1_8_bs=off:drc=off:lcm=reverse:nwc=1:ptb=off:ssec=off:sos=on:sio=off:spo=on:spl=backtracking:sp=reverse_arity_300");
    fallback.push("dis+11_2:1_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=2:nicw=on:ptb=off:ssec=off:sos=all:sac=on:sio=off:spo=on:spl=sat:sscc=on:sser=off:ssfq=2.0:sp=occurrence_100");
    fallback.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=2:ptb=off:ssec=off:spo=on:sp=occurrence_300");
    fallback.push("dis+3_2_bs=off:drc=off:ep=RST:fsr=off:nwc=1:ptb=off:ssec=off:sd=1:ss=axioms:st=1.2:sos=on:spl=sat:ssfq=2.0:ssnc=none:sfv=off_300");
    fallback.push("ott+11_64_bd=off:bs=off:br=off:cond=fast:drc=off:ep=on:gsp=input_only:gs=on:nwc=2:ptb=off:ssec=off:sos=all:sio=off:spl=backtracking:urr=on_100");
    fallback.push("ott+1011_1_bs=off:cond=on:drc=off:ecs=on:nwc=2:nicw=on:ptb=off:ssec=off:spl=sat:ssfp=1000:ssfq=1.2:ssnc=none_300");
    fallback.push("ott-1_2:3_bs=off:bms=on:cond=on:drc=off:fde=none:lcm=predicate:nwc=1.7:nicw=on:sd=5:ss=axioms:st=1.2:sio=off:urr=on_300");
    fallback.push("dis-1010_5:4_bs=off:bms=on:drc=off:nwc=10:sd=2:ss=axioms:st=1.5:sac=on:sio=off:updr=off_300");
    fallback.push("ott+2_8_bd=off:bs=off:bsr=unit_only:cond=on:flr=on:nwc=2:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=backtracking:updr=off_300");
    fallback.push("lrs-1004_50_bs=off:br=off:cond=fast:drc=off:nwc=1.5:nicw=on:sio=off:spl=off:urr=on:updr=off_600");
    fallback.push("ott+1_50_bd=off:bs=unit_only:drc=off:nwc=1.1:nicw=on:ptb=off:sio=off:spl=sat:ssfp=10000:ssfq=1.0:ssnc=all_dependent:sp=reverse_arity_300");
    fallback.push("ott+1_10_bs=off:cond=fast:drc=off:flr=on:lcm=predicate:nwc=10:ptb=off:ssec=off:sac=on:sio=off:spo=on:sp=occurrence:urr=on_300");
    fallback.push("ott-1003_2:1_bs=unit_only:bsr=unit_only:drc=off:nwc=1.2:ptb=off:ssec=off:sio=off:updr=off_300");
    fallback.push("ott-1010_8:1_bs=off:flr=on:fsr=off:gs=on:nwc=4:ptb=off:ssec=off:sac=on:sio=off:sp=reverse_arity:urr=on_600");
    fallback.push("dis+11_2:1_bs=off:ep=on:nwc=1:ptb=off:ssec=off:sswn=on:sswsr=on:sd=2:sgt=15:ss=axioms:sos=on:spl=sat_300");
    fallback.push("ott+3_32_bd=off:bs=off:drc=off:flr=on:lcm=predicate:nwc=1.1:nicw=on:ptb=off:ssec=off:sd=2:sgt=10:ss=axioms:st=3.0:sos=on:spl=sat:updr=off_300");
    fallback.push("ott+11_8:1_bs=off:drc=off:ecs=on:fsr=off:gs=on:lcm=reverse:nwc=2:nicw=on:ptb=off:ssec=off:sd=3:ss=axioms:sos=all:spl=sat:ssfq=2.0:ssnc=none_300");
    fallback.push("dis+2_1_bs=off:bsr=unit_only:drc=off:fsr=off:gsp=input_only:gs=on:lcm=reverse:nwc=5:ptb=off:ssec=off:sswn=on:sd=2:sgt=7:ss=axioms:st=1.2:sos=on:sio=off:spl=sat_300");
    fallback.push("dis+2_5:1_bd=off:bs=off:ep=RSTC:gs=on:lcm=reverse:nwc=1.2:nicw=on:sswn=on:sswsr=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:sac=on:sio=off:sfv=off:sp=reverse_arity_300");
    fallback.push("dis+10_8:1_bs=off:br=off:cond=on:drc=off:ecs=on:ep=RST:fsr=off:nwc=1.7:ptb=off:ssec=off:sd=4:ss=axioms:st=3.0:spo=on:spl=sat:ssfq=2.0:ssnc=none:urr=on_300");
    fallback.push("dis+1_14_bs=off:br=off:drc=off:fde=none:gs=on:nwc=1.1:ssec=off:sd=2:ss=axioms:st=2.0:sac=on:sio=off:urr=on_300");
    fallback.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:ptb=off:ssec=off:spl=sat_300");
    fallback.push("lrs+1_20_bs=off:cond=fast:gs=on:nwc=4:ptb=off:ssec=off:sswsr=on:sgt=20:ss=axioms:st=1.2:sos=on:sagn=off:sio=off:spo=on:spl=backtracking:updr=off_300");
    fallback.push("dis+11_4:1_bs=off:drc=off:ep=RST:fde=none:lcm=reverse:nwc=5:nicw=on:ssec=off:sd=2:sgt=5:ss=axioms:st=1.2:sos=on:sgo=on:sio=off:sp=occurrence_300");
    fallback.push("dis+2_4_bs=off:drc=off:ep=RST:flr=on:lcm=reverse:nwc=1.7:ptb=off:ssec=off:sos=on:spl=sat:sp=reverse_arity_300");
    fallback.push("ott+2_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ecs=on:ep=RST:flr=on:fde=none:lcm=reverse:nwc=5:ssec=off:sac=on:sio=off:urr=on_300");
    fallback.push("dis+10_16_bs=off:cond=fast:drc=off:ecs=on:gsp=input_only:nwc=10:nicw=on:ssec=off:sos=on:sac=on:urr=on_100");
    fallback.push("dis+1011_2_bs=off:cond=on:drc=off:gs=on:nwc=4:ptb=off:ssec=off:sd=1:ss=axioms:st=1.2:spl=sat:updr=off_300");
    fallback.push("ott-1004_3:2_br=off:drc=off:fsr=off:nwc=1.5:nicw=on:ptb=off:ssec=off:sos=all:spl=sat:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_200");
    fallback.push("ott+3_16_bd=off:bs=off:bsr=unit_only:nwc=4:nicw=on:ptb=off:ssec=off:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2_300");
    fallback.push("ott+1_4_bs=off:cond=on:drc=off:ep=on:flr=on:nwc=4:nicw=on:ptb=off:ssec=off:sswsr=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat_300");
    fallback.push("dis-1002_1024_bs=off:bms=on:cond=fast:drc=off:ecs=on:ep=on:lcm=predicate:nwc=3:nicw=on:ssec=off:sswn=on:sswsr=on:sac=on_300");
    fallback.push("dis+1_4_bs=off:drc=off:ep=on:fde=none:lcm=reverse:nwc=5:ptb=off:ssec=off:sos=on:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sp=reverse_arity_300");
    fallback.push("dis+1_2:1_bd=off:bs=off:bsr=on:drc=off:ep=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sswn=on:sd=1:ss=included:sos=on:sagn=off:sio=off:spo=on:spl=backtracking_300");
    fallback.push("ott+10_64_bd=off:bs=off:cond=on:drc=off:ecs=on:gs=on:nwc=1:ssec=off:sac=on:sio=off_300");
    fallback.push("ott+3_28_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sac=on:sio=off_300");
    fallback.push("dis+1002_5:1_bs=off:bms=on:drc=off:ep=on:flr=on:fde=none:gsp=input_only:nwc=1:nicw=on:ssec=off:sac=on:sio=off:updr=off_600");
    fallback.push("ott+10_1024_bs=off:drc=off:nwc=1.7:ssec=off:sac=on:sio=off:sp=occurrence:updr=off_600");
    fallback.push("dis-1010_40_bs=off:cond=on:drc=off:gs=on:nwc=3:nicw=on:ptb=off:ssec=off:sd=4:ss=axioms:st=1.5:sos=on:sio=off:spl=sat_300");
    fallback.push("ott+11_50_bd=off:bs=off:cond=fast:drc=off:ep=on:fsr=off:lcm=reverse:nwc=1:nicw=on:ptb=off:ssec=off:sos=on:spl=backtracking:sp=occurrence:updr=off_300");
    fallback.push("ott-11_8:1_bd=off:bs=off:drc=off:lcm=reverse:nwc=3:sio=off:spl=off:urr=on_300");
    fallback.push("ott+2_8:1_bs=off:bsr=on:cond=fast:drc=off:fde=none:lcm=reverse:nwc=4:nicw=on:ptb=off:ssec=off:sos=on:spl=sat:sser=off:ssfp=100000:ssnc=none:urr=on_300");
    fallback.push("dis+1011_2_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=4:ptb=off:ssec=off:sio=off:spl=backtracking:sp=occurrence_300");
    fallback.push("dis-1010_1024_bs=off:cond=fast:drc=off:fde=none:lcm=reverse:nwc=4:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=sat_300");
    fallback.push("ott+10_3:1_bsr=unit_only:cond=on:fsr=off:lcm=reverse:nwc=1.5:nicw=on:ptb=off:ssec=off:sio=off:spo=on:spl=sat:urr=on:updr=off_300");
    fallback.push("lrs+1_4_bs=off:drc=off:ecs=on:ep=on:flr=on:fsr=off:lcm=predicate:nwc=1.3:nicw=on:ptb=off:ssec=off:spl=sat:sser=off:ssfp=40000:ssfq=1.1:ssnc=none:sp=occurrence_300");
    fallback.push("dis-1010_128_bs=off:ep=RST:nwc=5:ptb=off:sd=1:ss=included:st=2.0:sos=on:sac=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:sfv=off_300");
    fallback.push("ott+1_5:1_bs=off:br=off:cond=fast:drc=off:nwc=1:nicw=on:ptb=off:ssec=off:sd=2:ss=axioms:st=5.0:sos=all:sio=off:spl=sat:urr=on_300");
    fallback.push("dis+1_2:1_bs=off:bms=on:br=off:drc=off:gsp=input_only:gs=on:lcm=predicate:nwc=1:nicw=on:ssec=off:sd=3:sgt=7:ss=axioms:st=2.0:sos=on:spo=on:urr=on_300");
    fallback.push("dis+11_1_bs=off:bms=on:drc=off:ep=RS:flr=on:fde=none:nwc=5:sos=on:sgo=on:sio=off_300");
    fallback.push("dis+1010_7_bs=off:bsr=on:bms=on:drc=off:fsr=off:fde=none:gs=on:nwc=1.7:sac=on:sgo=on:sio=off:sp=occurrence_300");
    fallback.push("dis+1011_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=5:nicw=on:ssec=off:sswn=on:sp=reverse_arity_300");
    fallback.push("dis+10_128_bd=off:bs=off:drc=off:nwc=3:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking_300");
    fallback.push("ott+11_8:1_bd=off:bs=unit_only:bsr=unit_only:drc=off:fsr=off:lcm=reverse:nwc=1.1:nicw=on:ptb=off:ssec=off:spl=sat:ssfq=2.0:ssnc=none_300");
    fallback.push("ott+11_2:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:fde=none:gsp=input_only:lcm=reverse:nwc=2.5:ptb=off:ssec=off:spl=sat:sp=occurrence_300");
    fallback.push("dis+1_3:1_bs=off:drc=off:fde=none:gs=on:nwc=1.7:ptb=off:ssec=off:sos=all:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence_100");
    fallback.push("dis-4_3:2_cond=on:drc=off:gs=on:nwc=4:nicw=on:ptb=off:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence_300");
    fallback.push("ott-4_7_bs=off:drc=off:fde=none:gs=on:lcm=reverse:nwc=1.7:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=backtracking:urr=on_300");
    fallback.push("dis+10_64_bd=off:bs=off:cond=fast:drc=off:ep=RSTC:flr=on:gsp=input_only:nwc=4:nicw=on:ptb=off:ssec=off:sswn=on:sd=10:ss=included:st=2.0:sos=all:spl=backtracking_300");
    fallback.push("dis+1011_5:1_bs=off:cond=fast:drc=off:ep=RS:nwc=5:nicw=on:ptb=off:ssec=off:sio=off:spo=on_200");
    fallback.push("ott+1_8:1_bs=off:drc=off:ep=on:flr=on:gsp=input_only:nwc=1.5:nicw=on:ptb=off:ssec=off:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:sac=on:sio=off:spl=backtracking_300");
    fallback.push("ott+11_2:3_bs=off:cond=on:drc=off:flr=on:lcm=predicate:nwc=1.5:nicw=on:ptb=off:ssec=off:spo=on:spl=sat:sp=reverse_arity:urr=on_300");
    fallback.push("ott+1011_5:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=backtracking:sp=occurrence_300");
    fallback.push("lrs-3_10_drc=off:nwc=5:ptb=off:ssec=off:spl=sat:sser=off:ssfp=4000:ssnc=none:urr=on_300");
    fallback.push("ott-2_5:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:fde=none:gs=on:lcm=reverse:nwc=3:ptb=off:ssec=off:sos=on:sio=off:spl=backtracking_300");
    fallback.push("dis+2_12_drc=off:ep=RST:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sgo=on:spl=sat:sp=occurrence_300");
    fallback.push("dis+10_64_bs=off:cond=on:drc=off:ep=RS:nwc=1:ptb=off:ssec=off:sio=off:spo=on_300");
    fallback.push("ott-1010_4_bs=off:cond=fast:drc=off:fsr=off:gs=on:nwc=10:nicw=on:ptb=off:ssec=off:sd=4:ss=axioms:st=1.2:sos=on:sac=on:sio=off:spl=backtracking:sfv=off_300");
    fallback.push("dis+2_128_bs=off:bms=on:drc=off:fde=none:lcm=reverse:nwc=1.3:nicw=on:ssec=off:sos=on:sio=off:spl=off_300");
    fallback.push("dis+3_4_bd=off:bs=off:cond=fast:drc=off:fsr=off:fde=none:gs=on:lcm=reverse:nwc=1.7:ptb=off:ssec=off:sos=on:spl=sat_300");
    fallback.push("lrs+11_50_bs=off:bms=on:cond=fast:drc=off:ecs=on:nwc=1.1:ssec=off:urr=on_900");
    fallback.push("dis+2_8:1_bs=off:drc=off:flr=on:lcm=reverse:nwc=1.5:ptb=off:ssec=off:sswn=on:sd=2:sgt=7:ss=axioms:sos=on:sio=off:spl=sat_300");
    fallback.push("ins+10_4:1_bs=off:bsr=unit_only:cond=on:ep=RSTC:gs=on:igbrr=0.4:igrr=1/128:igrp=100:igrpq=1.3:igwr=on:nwc=1.1:ptb=off:ssec=off:sos=on:sio=off:spl=off:urr=on_300");
    fallback.push("dis+11_16_bs=off:drc=off:gs=on:nwc=5:nicw=on:ptb=off:ssec=off:sos=on:sio=off:spo=on:spl=sat:sser=off:ssfp=1000:ssnc=none:sp=occurrence:urr=on_300");
    fallback.push("dis-1_6_bs=off:bsr=unit_only:cond=on:drc=off:fsr=off:gs=on:lcm=predicate:nwc=1.1:ptb=off:ssec=off:sos=on:sio=off:spo=on:urr=on_300");
    fallback.push("ott+4_5:1_bd=off:bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=3:ssec=off:sd=3:ss=axioms:sos=on:sio=off:spl=off:urr=on_300");
    fallback.push("ins-1003_3_bs=off:ep=RSTC:flr=on:gsp=input_only:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=2.0:igwr=on:nwc=2:ptb=off:ssec=off:sos=on:sio=off:spl=off_300");
    fallback.push("ott+11_10_bd=off:bs=off:cond=fast:drc=off:lcm=predicate:nwc=1:nicw=on:sac=on:sgo=on:urr=on_600");
    fallback.push("ott+3_3_bs=off:bsr=unit_only:cond=fast:drc=off:fsr=off:fde=none:gs=on:nwc=2.5:nicw=on:ptb=off:ssec=off:sgo=on:spo=on:spl=backtracking:sp=occurrence_600");
    fallback.push("lrs-2_2:1_bs=off:br=off:cond=on:ep=on:fde=none:lcm=reverse:nwc=10:ptb=off:ssec=off:sos=on:sio=off:spo=on:urr=on_600");
    fallback.push("ott+11_12_cond=fast:drc=off:ecs=on:gs=on:nwc=1.2:nicw=on:ssec=off:sp=occurrence_600");
    fallback.push("ott+3_24_bs=unit_only:drc=off:ecs=on:fde=none:nwc=1.1:ssec=off:sos=all:sac=on_600");
    fallback.push("ott+1_5:1_bs=off:cond=fast:drc=off:fde=none:lcm=reverse:nwc=10:nicw=on:ptb=off:ssec=off:sos=on:spl=sat:sp=occurrence_300");
    break;

  case Property::FNE:
    fallback.push("dis-1002_2:3_bd=off:bs=off:cond=fast:drc=off:fde=none:nwc=3:nicw=on:ptb=off:ssec=off:sac=on:sgo=on:sio=off:spl=backtracking_300");
    fallback.push("dis-1002_4_bs=off:bsr=unit_only:gsp=input_only:gs=on:nwc=3:nicw=on:ptb=off:ssec=off:spl=sat:urr=on:updr=off_300");
    fallback.push("dis+11_64_bs=off:cond=fast:gs=on:nwc=3:nicw=on:ptb=off:ss=axioms:st=1.2:sac=on:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none_100");
    fallback.push("dis+11_40_bs=off:bsr=on:cond=on:fsr=off:lcm=reverse:nwc=2.5:ptb=off:ssec=off:sio=off:spl=sat:updr=off_300");
    fallback.push("ins+1011_128_bs=off:cond=fast:gs=on:igbrr=0.3:igrr=1/64:igrpq=1.3:igwr=on:nwc=5:ptb=off:sos=all:sio=off:spl=off_300");
    fallback.push("dis+10_7_bd=off:bs=off:cond=fast:drc=off:ep=R:flr=on:fde=none:nwc=10:nicw=on:ptb=off:ssec=off:sswn=on:sd=10:ss=included:st=1.5:spo=on:spl=backtracking_300");
    fallback.push("dis+1011_64_bs=off:gs=on:nwc=3:nicw=on:ptb=off:ssec=off:sos=on:sac=on:sio=off:spl=sat_300");
    fallback.push("dis+3_7_bs=off:bms=on:cond=fast:ecs=on:fsr=off:lcm=reverse:nwc=1.1:ssec=off_300");
    fallback.push("ott-1010_1024_bs=off:drc=off:fde=none:nwc=2:sos=on_300");
    fallback.push("ott+10_2:1_bs=off:bsr=on:cond=fast:lcm=predicate:nwc=1:nicw=on:ptb=off:ssec=off:sac=on:sio=off:spl=sat:sp=occurrence:urr=on_300");
    fallback.push("dis+1004_6_bs=off:nwc=4:nicw=on:ptb=off:ssec=off:sagn=off:spl=backtracking_300");
    fallback.push("ott+1_10_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=2.5:ptb=off:ssec=off_600");
    fallback.push("tab+10_1_bms=on:sd=3:ss=axioms:st=1.2:sio=off:tbsr=off:tgawr=1/32:tglr=1/10:tipr=off:tlawr=3/1_100");
    fallback.push("ott+1_40_bsr=on:cond=fast:gs=on:nwc=1.5:ptb=off:ssec=off:sio=off:spo=on:spl=sat:urr=on_300");
    fallback.push("ott+10_4_bs=off:gsp=input_only:gs=on:lcm=reverse:nwc=10:nicw=on:ptb=off:ssec=off:sp=occurrence:urr=on_300");
    fallback.push("ott+11_14_bs=off:drc=off:ep=RS:flr=on:fde=none:lcm=reverse:nwc=1.2:ptb=off:ssec=off:sio=off:spo=on_300");
    fallback.push("dis+1010_10_bs=off:nwc=1.1:ptb=off:ssec=off:sio=off_300");
    fallback.push("dis+1011_7_bs=off:nwc=4:nicw=on:ptb=off:ssec=off:sos=all:sio=off:spo=on:spl=sat:ssfq=1.4:ssnc=none:updr=off_300");
    fallback.push("tab+10_1_spl=off:tfsr=off:tgawr=4/1:tglr=1/20:tipr=off:tlawr=8/1_300");
    fallback.push("dis+1011_7_bs=off:cond=on:fsr=off:gs=on:nwc=1:ptb=off:ssec=off:sos=all:spl=sat_300");
    fallback.push("ott-1010_128_bs=off:cond=on:nwc=1.3:ssec=off:urr=on_600");
    break;

  case Property::UEQ:
    fallback.push("lrs+11_20_bs=off:drc=off:ep=on:flr=on:gs=on:nwc=4:sp=occurrence_1200");
    fallback.push("ott+10_6_bs=off:drc=off:fsr=off:fde=none:nwc=1.2_300");
    fallback.push("lrs+10_128_bs=unit_only:drc=off:fsr=off:nwc=1.2:sfv=off:sp=occurrence_1200");
    fallback.push("dis+4_5:1_bs=off:br=off:ep=RSTC:fsr=off:nwc=3:ptb=off:sos=all:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_300");
    fallback.push("ott+10_8:1_bs=off:drc=off:fsr=off:fde=none:nwc=5_600");
    fallback.push("ott+10_7_bs=off:drc=off:fsr=off:gsp=input_only:nwc=3:sp=occurrence_600");
    fallback.push("lrs+10_20_bd=off:bs=unit_only:drc=off:fsr=off:nwc=1.2_900");
    fallback.push("ott+10_3_bs=off:drc=off:fsr=off:fde=none:gsp=input_only:nwc=1.3:sp=occurrence_300");
    fallback.push("ott+10_32_bs=off:bsr=on:drc=off:fsr=off:fde=none:nwc=1.3_200");
    fallback.push("ott+10_50_bd=off:drc=off:fsr=off:nwc=1.1:sp=occurrence_300");
    fallback.push("ott+10_7_bs=off:drc=off:fsr=off:nwc=1.1:sp=occurrence_300");
    fallback.push("dis+10_128_bs=unit_only:drc=off:fsr=off:gsp=input_only:nwc=10_300");
    fallback.push("lrs+10_8:1_nwc=1:sfv=off:sp=reverse_arity_900");
    fallback.push("ott+10_8_bd=off:bs=off:drc=off:fsr=off:fde=none:nwc=2:sp=occurrence_300");
    fallback.push("ott+10_8:1_bs=off:bsr=on:nwc=1.3_300");
    fallback.push("lrs+10_2:1_bd=off:bs=off:drc=off:fsr=off:nwc=2.5_300");
    fallback.push("dis+10_5_bs=off:drc=off:fsr=off:nwc=1.5:sp=occurrence_600");
    fallback.push("ott+10_1_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.3_600");
    break;
  }
}

void Schedules::getLtb2014Schedule(const Property& property, Schedule& quick)
{
  Property::Category cat = property.category();
  unsigned atoms = property.atoms();
  unsigned prop = property.props();

  switch (cat) {
  case Property::NEQ:
    if (prop == 131079) {
      quick.push("dis+11_4:1_bs=off:cond=fast:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssnc=none_8");
      quick.push("ott+3_8:1_bd=off:bs=off:bsr=unit_only:drc=off:fsr=off:nwc=2.5:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.0:ssnc=none:updr=off_12");
      quick.push("dis+11_2_bs=off:bsr=unit_only:drc=off:fde=none:lcm=reverse:nwc=1.5:sio=off:spl=sat:sser=off:ssnc=none_23");
      quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_5");
      quick.push("dis+11_4_bs=off:cond=fast:drc=off:ep=on:fde=none:nwc=1.7:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_68");
      quick.push("ott+11_2_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:ep=on:fde=none:lcm=reverse:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_36");
      quick.push("dis+1011_10_bs=off:drc=off:fsr=off:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_29");
      quick.push("dis+1011_24_cond=fast:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_120");
      quick.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:nwc=2:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_117");
      quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_18");
      quick.push("ott+1011_3_bs=off:bsr=unit_only:fde=none:nwc=2:sio=off:spl=sat:ssfq=1.0:ssnc=none_55");
      quick.push("ott+11_2_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=reverse_arity_228");
      quick.push("dis+11_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_283");
      quick.push("ott+11_8:1_drc=off:ep=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none:sp=occurrence:updr=off_480");
      quick.push("dis+1011_64_bs=off:drc=off:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_44");
      quick.push("ott+11_8:1_bs=off:drc=off:flr=on:lcm=predicate:nwc=2:sos=all:sio=off:spo=on:sfv=off_110");
    }
    else if (prop == 1) {
      if (atoms > 175) {
        quick.push("ott+2_1_bs=off:cond=on:drc=off:ep=on:nwc=1.7:nicw=on:ss=axioms:sos=all:sio=off:urr=on_1");
        quick.push("dis+1011_10_bs=off:drc=off:fsr=off:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_2");
        quick.push("dis-1010_3:1_bs=off:drc=off:ep=RS:flr=on:nwc=5:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_2");
        quick.push("dis+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none_38");
        quick.push("dis+10_8:1_bs=off:br=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=1:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_29");
        quick.push("ott+11_2:3_bs=off:drc=off:flr=on:lcm=predicate:nwc=2.5:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none:sp=occurrence_57");
        quick.push("ott-11_2:3_bs=off:drc=off:fsr=off:lcm=predicate:nwc=5:nicw=on:sac=on:sio=off:spo=on:sp=reverse_arity_26");
        quick.push("dis-1002_2:3_bs=off:drc=off:gsp=input_only:nwc=1.7:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=occurrence_3");
        quick.push("dis+1011_64_bs=off:drc=off:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_1");
        quick.push("dis-11_28_bs=off:drc=off:flr=on:lcm=predicate:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=1.0:ssnc=none_48");
        quick.push("ott-11_40_bd=off:bs=off:drc=off:flr=on:fsr=off:lcm=predicate:nwc=10:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:updr=off_42");
        quick.push("ott+1_3:1_bs=off:br=off:drc=off:flr=on:lcm=predicate:nwc=1.7:sio=off:spl=sat:ssfp=10000:ssfq=1.4:ssnc=none:sp=occurrence:urr=on_11");
        quick.push("ott+11_50_bd=off:bs=off:drc=off:ep=on:lcm=reverse:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_56");
        quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_33");
        quick.push("dis+10_1024_bs=off:drc=off:flr=on:fsr=off:nwc=1.1:sio=off:spl=sat:ssfq=1.4:ssnc=none_76");
        quick.push("ott+1_1024_bs=off:br=off:cond=on:drc=off:flr=on:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=1.2:sac=on:sio=off:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=all_dependent:urr=on:updr=off_65");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_72");
        quick.push("lrs+2_64_bd=off:bs=off:br=off:cond=on:drc=off:flr=on:fde=none:nwc=1.7:stl=30:sio=off:spl=sat:ssfp=1000:ssfq=1.4:ssnc=none:urr=on_54");
        quick.push("lrs+1011_14_bd=off:bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=1.7:stl=60:sio=off:spo=on:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:urr=on_211");
        quick.push("ott+11_8:1_bs=off:drc=off:flr=on:lcm=predicate:nwc=2:sos=all:sio=off:spo=on:sfv=off_171");
        quick.push("dis+10_1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:sos=on:sio=off:spl=sat:sser=off:ssnc=none_1");
        quick.push("ott+11_8:1_drc=off:ep=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none:sp=occurrence:updr=off_16");
        quick.push("ott+11_2_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:ep=on:fde=none:lcm=reverse:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_101");
      }
      else {
        quick.push("dis+3_4_bs=off:br=off:cond=on:drc=off:ep=RSTC:nwc=4:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:urr=on_2");
        quick.push("dis+11_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_15");
        quick.push("ins+1_3:1_bs=off:cond=on:ep=RSTC:flr=on:gs=on:igbrr=0.8:igrr=1/4:igrp=400:igrpq=1.1:igwr=on:nwc=1.5:sio=off:urr=on_4");
        quick.push("dis+1011_1024_bs=unit_only:drc=off:gsp=input_only:nwc=1.1:sio=off:spo=on:urr=on_10");
        quick.push("ott+11_2_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=reverse_arity_4");
        quick.push("dis+11_3:1_bs=off:cond=fast:drc=off:nwc=5:sio=off:spl=sat:sser=off:ssnc=none_17");
        quick.push("ott+10_28_bd=off:bs=off:drc=off:nwc=1.5:sio=off:spl=sat:ssnc=none_21");
        quick.push("lrs+2_14_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:nwc=1.1:stl=30:sio=off:spl=sat:ssac=none:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence:updr=off_227");
        quick.push("lrs+4_24_bd=off:bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:stl=60:sio=off:spl=sat:ssfp=4000:ssfq=1.2:ssnc=none_540");
        quick.push("ott+10_64_bd=off:bs=off:drc=off:flr=on:gsp=input_only:lcm=predicate:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_212");
        quick.push("ott+11_3_bd=off:bs=off:drc=off:ep=on:flr=on:nwc=1:sio=off:spl=sat:ssfq=1.1:ssnc=none:sfv=off_46");
      }
    }
    else if (prop == 3) {
      if (atoms <= 2000) {
        quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:ep=on:nwc=10:nicw=on:sio=off:spl=sat:ssnc=none:urr=on_8");
        quick.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:nwc=2:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_95");
        quick.push("ott+2_1_bs=off:cond=on:drc=off:ep=on:nwc=1.7:nicw=on:ss=axioms:sos=all:sio=off:urr=on_23");
        quick.push("ott+1_1024_bs=off:br=off:cond=on:drc=off:flr=on:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=1.2:sac=on:sio=off:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=all_dependent:urr=on:updr=off_2");
        quick.push("ott+1011_3_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.2:sio=off:spl=sat:ssfq=1.4:ssnc=none_69");
        quick.push("dis+4_8:1_bs=off:drc=off:ep=RS:nwc=2:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=sat:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_20");
        quick.push("ins+2_5_bs=off:cond=fast:ep=RSTC:flr=on:gs=on:igbrr=0.7:igrr=1/4:igrp=700:igrpq=1.2:igwr=on:lcm=reverse:nwc=2:sio=off:urr=on_5");
        quick.push("dis+2_24_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none_1");
        quick.push("ott+11_3_bd=off:bs=off:drc=off:ep=on:flr=on:nwc=1:sio=off:spl=sat:ssfq=1.1:ssnc=none:sfv=off_46");
        quick.push("dis+1002_4:1_bs=off:cond=fast:drc=off:ep=RST:nwc=3:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_25");
        quick.push("ott+10_2_bd=off:bs=off:drc=off:flr=on:fsr=off:nwc=1.7:sio=off:spl=sat:ssfq=2.0:ssnc=none_91");
        quick.push("lrs+10_1_bs=off:cond=fast:nwc=5:stl=20:sio=off:spl=sat:ssfq=1.1:ssnc=none_4");
        quick.push("dis+1011_64_bs=off:drc=off:ep=on:flr=on:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_50");
        quick.push("dis+1011_1024_bs=unit_only:cond=fast:lcm=reverse:nwc=1.1:sos=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_48");
        quick.push("dis+1_8:1_bs=off:drc=off:lcm=reverse:nwc=1.5:sos=all:sio=off:spl=sat:ssfq=2.0:ssnc=none:sfv=off_96");
        quick.push("dis+1011_2_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_202");
        quick.push("lrs+2_3_bd=off:cond=on:drc=off:flr=on:nwc=1.3:stl=30:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off:sp=reverse_arity_96");
        quick.push("ott+11_2:3_bs=off:drc=off:flr=on:lcm=predicate:nwc=2.5:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none:sp=occurrence_256");
        quick.push("dis+10_8:1_bs=off:br=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=1:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_140");
        quick.push("dis+2_4:1_bs=unit_only:cond=on:flr=on:lcm=predicate:nwc=1:ssac=none:ssfp=100000:ssfq=2.0:ssnc=all:sfv=off:urr=on_257");
        quick.push("dis+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none_106");
      }
      else {
        quick.push("dis-1010_3:1_bs=off:drc=off:ep=RS:flr=on:nwc=5:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_3");
        quick.push("ott+1011_3_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.2:sio=off:spl=sat:ssfq=1.4:ssnc=none_27");
        quick.push("dis+10_10_bs=off:cond=fast:drc=off:gs=on:nwc=1.7:nicw=on:sd=1:ss=axioms:st=3.0_2");
        quick.push("dis-1002_5:1_bs=off:cond=fast:drc=off:ep=on:nwc=1.1:sd=4:ss=axioms:sos=on:sio=off:spl=sat:urr=on_17");
        quick.push("ott-11_2:3_bs=off:drc=off:fsr=off:lcm=predicate:nwc=5:nicw=on:sac=on:sio=off:spo=on:sp=reverse_arity_14");
        quick.push("dis+11_4:1_bs=off:drc=off:gsp=input_only:nwc=3:sos=on_23");
        quick.push("dis+4_8:1_bs=off:drc=off:ep=RS:nwc=2:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=sat:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_2");
        quick.push("dis+2_24_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none_2");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_31");
        quick.push("ott+2_1_bs=off:cond=on:drc=off:ep=on:nwc=1.7:nicw=on:ss=axioms:sos=all:sio=off:urr=on_2");
        quick.push("dis+3_8:1_bs=off:drc=off:fsr=off:fde=none:nwc=10:sio=off:spl=sat:ssfp=10000:ssfq=1.4:ssnc=none:sp=reverse_arity_17");
        quick.push("ins-1010_8:1_bs=off:ep=RSTC:fsr=off:igbrr=1.0:igrr=1/128:igrp=700:igrpq=1.5:igwr=on:nwc=3:sos=on:sgo=on:sio=off_70");
        quick.push("ott-1010_2:1_bs=off:cond=fast:drc=off:nwc=3:sd=1:ss=axioms:st=2.0:sio=off:spl=sat:sser=off:ssnc=none_36");
        quick.push("dis+1_8:1_bs=off:drc=off:lcm=reverse:nwc=1.5:sos=all:sio=off:spl=sat:ssfq=2.0:ssnc=none:sfv=off_90");
        quick.push("ott+1_10_bd=off:bs=off:drc=off:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_111");
        quick.push("ott+1_1024_bs=off:br=off:cond=on:drc=off:flr=on:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=1.2:sac=on:sio=off:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=all_dependent:urr=on:updr=off_246");
        quick.push("lrs+2_3:1_bs=off:br=off:cond=fast:drc=off:flr=on:nwc=4:stl=60:sio=off:spo=on:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on_250");
        quick.push("lrs+1011_14_bd=off:bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=1.7:stl=60:sio=off:spo=on:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:urr=on_549");
      }
    }
    else {
      quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_8");
      quick.push("dis+1011_1024_bs=unit_only:cond=fast:lcm=reverse:nwc=1.1:sos=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_6");
      quick.push("dis+11_8:1_bs=off:drc=off:ep=RS:nwc=10:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_2");
      quick.push("dis+2_2:3_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=1:nicw=on:sos=all:sio=off:spl=sat:ssfq=1.1:ssnc=none:urr=on_7");
      quick.push("ott+1011_3_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.2:sio=off:spl=sat:ssfq=1.4:ssnc=none_12");
      quick.push("dis+2_5_cond=fast:drc=off:fsr=off:lcm=reverse:nwc=3:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none_9");
      quick.push("dis-1010_6_bs=off:drc=off:ep=on:gsp=input_only:nwc=10:nicw=on:sos=all:sp=occurrence_13");
      quick.push("dis+11_4:1_bs=off:drc=off:gsp=input_only:nwc=3:sos=on_1");
      quick.push("dis+2_24_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none_4");
      quick.push("dis+11_24_bs=off:cond=fast:drc=off:nwc=2.5:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_1");
      quick.push("dis+10_1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:sos=on:sio=off:spl=sat:sser=off:ssnc=none_19");
      quick.push("lrs-1010_3_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1.1:nicw=on:stl=30:sos=on:sac=on:sio=off_3");
      quick.push("dis+11_20_bs=off:cond=fast:fde=none:lcm=reverse:nwc=3:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sp=reverse_arity_9");
      quick.push("lrs-1002_6_bs=off:bsr=unit_only:cond=fast:fde=none:gsp=input_only:lcm=reverse:nwc=1:stl=30:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=occurrence_11");
      quick.push("dis-1002_3:2_bs=off:cond=fast:drc=off:ep=RST:flr=on:fde=none:lcm=predicate:nwc=10:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:updr=off_8");
      quick.push("lrs+1010_24_bd=off:bs=off:cond=on:drc=off:gsp=input_only:nwc=1.3:nicw=on:stl=10:sio=off:spo=on:spl=sat:sser=off:ssfq=1.0:ssnc=none:updr=off_18");
      quick.push("dis+1002_4:1_bs=off:cond=fast:drc=off:ep=RST:nwc=3:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_10");
      quick.push("dis-1002_12_bs=off:cond=on:drc=off:ep=on:gsp=input_only:nwc=10:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_1");
      quick.push("dis+11_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_26");
      quick.push("dis+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none_30");
      quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_252");
      quick.push("dis+11_4_ep=on:fde=none:lcm=reverse:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_240");
      quick.push("ins+2_5_bs=off:cond=fast:ep=RSTC:flr=on:gs=on:igbrr=0.7:igrr=1/4:igrp=700:igrpq=1.2:igwr=on:lcm=reverse:nwc=2:sio=off:urr=on_1");
      quick.push("dis+1011_64_bs=off:drc=off:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_26");
      quick.push("dis-1002_2:3_bs=off:drc=off:gsp=input_only:nwc=1.7:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=occurrence_60");
    }
    break;

  case Property::HEQ:
    if (prop == 2) {
      quick.push("dis+11_10_bs=off:drc=off:nwc=10:sos=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_3");
      quick.push("ott+11_128_bs=off:drc=off:gsp=input_only:gs=on:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.4:ssnc=none:sp=occurrence_626");
      quick.push("dis+11_5:4_bs=off:cond=fast:drc=off:fde=none:nwc=1:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.2:ssnc=none:urr=on_164");
      quick.push("ins+2_64_bs=off:cond=on:drc=off:flr=on:fde=none:igbrr=0.1:igrr=1/16:igrpq=1.3:igwr=on:nwc=1.7:sp=reverse_arity_1061");
    }
    else if (prop == 8194) {
      quick.push("lrs+11_2:1_bs=off:cond=on:drc=off:nwc=10:stl=60:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_487");
      quick.push("lrs+11_3:1_bs=unit_only:drc=off:fde=none:nwc=10:stl=60:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_128");
      quick.push("lrs+2_4_bs=off:cond=fast:drc=off:gsp=input_only:nwc=4:stl=90:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none_335");
      quick.push("ott+11_3:2_bs=off:cond=on:drc=off:ep=RSTC:flr=on:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssnc=none_205");
    }
    else {
      quick.push("ott+11_24_bd=off:bs=off:drc=off:nwc=3:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_2");
      quick.push("ins+11_64_bs=off:cond=on:drc=off:fde=none:igbrr=0.2:igrr=1/64:igrp=400:igrpq=1.05:igwr=on:nwc=1.2:sio=off_29");
      quick.push("dis+2_8_bd=off:bs=off:ep=RST:fsr=off:lcm=reverse:nwc=10:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_2");
      quick.push("dis-1002_3_bs=off:cond=fast:drc=off:nwc=10:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_2");
      quick.push("dis+10_1024_bd=off:bs=off:cond=on:drc=off:fde=none:nwc=2.5:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_162");
      quick.push("lrs-1_2_bs=off:drc=off:nwc=4:nicw=on:stl=30:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_236");
      quick.push("ins+4_5:4_bs=off:br=off:cond=on:ep=RSTC:flr=on:fsr=off:igbrr=0.1:igrr=1/16:igrpq=1.0:igwr=on:nwc=2:urr=on_323");
      quick.push("ott+11_24_bs=unit_only:cond=fast:drc=off:nwc=3:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_473");
    }
    break;

  case Property::PEQ:
    if (prop == 0) {
      if (atoms <= 15) {
        quick.push("dis+10_1024_bs=unit_only:cond=on:drc=off:nwc=1:sio=off:spl=sat:ssfq=1.0:ssnc=none_35");
        quick.push("lrs+3_4_bsr=unit_only:drc=off:fsr=off:nwc=1:stl=150:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_490");
        quick.push("lrs+4_14_cond=on:drc=off:flr=on:nwc=3:stl=180:spl=sat:sser=off:ssnc=none_947");
      }
      else {
        quick.push("dis-1010_14_bs=off:drc=off:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none_12");
        quick.push("dis+10_64_bd=off:cond=fast:drc=off:nwc=3:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.1:ssnc=none_241");
        quick.push("ott+11_5_bs=off:cond=fast:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_753");
        quick.push("ott+11_2:3_bd=off:drc=off:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.0:ssnc=none:updr=off_243");
        quick.push("dis+2_7_bd=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_441");
      }
    }
    else if (prop == 1) {
      quick.push("lrs-1_128_bs=off:cond=fast:ep=on:nwc=1.2:nicw=on:stl=30:sac=on:sio=off_14");
      quick.push("dis-1003_128_drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_40");
      quick.push("lrs+3_40_bs=unit_only:bsr=on:cond=on:drc=off:fsr=off:nwc=1.1:nicw=on:stl=100:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on:updr=off_645");
      quick.push("dis+10_64_bd=off:cond=fast:drc=off:nwc=3:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.1:ssnc=none_115");
      quick.push("dis+1_2_bd=off:bs=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_278");
      quick.push("dis+2_7_bd=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_544");
      quick.push("lrs+3_4_bsr=unit_only:drc=off:fsr=off:nwc=1:stl=150:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_1006");
    }
    else {
      quick.push("dis-1010_14_bs=off:drc=off:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none_18");
      quick.push("ott+3_4:1_bsr=unit_only:cond=on:drc=off:fsr=off:fde=none:gs=on:nwc=1:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence:urr=on_13");
      quick.push("lrs+3_16_bsr=unit_only:cond=on:drc=off:fsr=off:nwc=4:stl=150:sio=off:spl=sat:ssnc=none:urr=on_141");
      quick.push("lrs+10_2:3_bd=off:cond=on:drc=off:flr=on:nwc=5:nicw=on:stl=90:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_85");
      quick.push("dis-1004_1024_bs=off:cond=fast:drc=off:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none_72");
      quick.push("ott+11_3_bd=off:bs=unit_only:bsr=unit_only:drc=off:fsr=off:nwc=1.5:sio=off:spo=on:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none_39");
      quick.push("ott+2_20_bs=off:cond=fast:drc=off:fde=none:nwc=2:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:sp=occurrence_116");
      quick.push("ott-1003_8_bs=off:bsr=on:cond=on:drc=off:fsr=off:gs=on:nwc=5:sac=on:sio=off:sp=occurrence_156");
      quick.push("ins+1_20_bsr=on:br=off:cond=fast:drc=off:fsr=off:fde=none:igbrr=0.3:igrr=1/32:igrp=4000:igrpq=2.0:igwr=on:lcm=reverse:nwc=1.2:sgo=on:sio=off:sp=occurrence:urr=on_159");
      quick.push("ott+3_128_bs=off:drc=off:nwc=1.1:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_321");
    }
    break;

  case Property::HNE:
    if (prop == 8192) {
      if (atoms > 6) {
        quick.push("ott+4_6_bs=off:bsr=unit_only:cond=fast:nwc=3:nicw=on:sio=off:spl=sat:ssnc=none_76");
        quick.push("dis-1004_5:1_bs=off:cond=on:nwc=2:sio=off_17");
        quick.push("dis+11_2:3_bs=off:cond=fast:fsr=off:nwc=10:sio=off:spl=sat:sser=off:ssnc=none_125");
        quick.push("ott+1011_10_bs=off:cond=on:nwc=3:sio=off:spl=sat:ssnc=none_61");
        quick.push("lrs+10_3:2_bs=off:cond=fast:nwc=10:stl=90:sio=off:spo=on_127");
        quick.push("ott+1003_28_bs=off:cond=on:flr=on:fsr=off:lcm=predicate:nwc=1:nicw=on:sio=off:spl=off_275");
        quick.push("dis+2_5:4_cond=fast:flr=on:lcm=predicate:nwc=1.5:sio=off:spl=off_562");
        quick.push("dis+1002_64_bs=off:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:nicw=on:sio=off:spo=on_391");
        quick.push("lrs+10_8:1_bs=off:cond=fast:flr=on:fsr=off:nwc=2.5:stl=90:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_601");
      }
      else {
        quick.push("dis+1011_20_bs=off:fsr=off:nwc=2:sio=off:spl=off_103");
        quick.push("lrs+3_4_bs=off:cond=fast:flr=on:nwc=1:nicw=on:stl=30:sio=off:spl=sat:ssnc=none_172");
        quick.push("lrs+11_128_cond=fast:lcm=reverse:nwc=1.5:nicw=on:stl=60:sio=off:spl=sat:ssfp=100000:ssfq=2.0:ssnc=none_525");
        quick.push("lrs+1011_64_bs=unit_only:bsr=unit_only:cond=fast:flr=on:nwc=1:stl=180:sio=off:spl=sat:ssfq=2.0_611");
      }
    }
    else {
      quick.push("dis+1011_14_bs=off:nwc=4:sio=off:spl=off_2");
      quick.push("dis+10_3_bs=off:br=off:cond=fast:gs=on:nwc=1:sos=all:sio=off:urr=on_22");
      quick.push("ott+1011_10_bs=off:cond=on:nwc=3:sio=off:spl=sat:ssnc=none_14");
      quick.push("ott+4_6_bs=off:bsr=unit_only:cond=fast:nwc=3:nicw=on:sio=off:spl=sat:ssnc=none_1");
      quick.push("ott+1011_20_bs=off:cond=fast:nwc=5:sio=off:spl=off_7");
      quick.push("lrs+10_3:1_bs=off:cond=on:flr=on:nwc=10:stl=50:sio=off:spl=off:sp=reverse_arity_62");
      quick.push("ott+1_8:1_bs=off:cond=on:gs=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssnc=none_178");
      quick.push("lrs+10_3:2_bs=off:cond=fast:nwc=10:stl=90:sio=off:spo=on_16");
      quick.push("ott+1011_10_bs=off:cond=fast:flr=on:fsr=off:nwc=1.7:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_21");
    }
    break;

  case Property::NNE:
    quick.push("dis+1011_50_bs=off:cond=fast:flr=on:gsp=input_only:nwc=1.3:sos=all_102");
    quick.push("dis-1010_14_bsr=on:cond=on:nwc=1.5:sio=off:spl=sat:ssfp=100000:ssfq=1.1:ssnc=none_69");
    quick.push("dis+11_20_bs=off:fsr=off:gsp=input_only:lcm=reverse:nwc=1.3:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_13");
    quick.push("dis-2_16_bs=off:cond=fast:flr=on:lcm=predicate:nwc=1:nicw=on:sagn=off:sio=off:spl=sat:ssnc=none_50");
    quick.push("ott+1011_5_bs=off:gs=on:nwc=2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssnc=none_33");
    quick.push("dis+1011_16_bs=unit_only:cond=on:fsr=off:gsp=input_only:nwc=1.7:sos=all:sgo=on:sio=off:spl=sat:ssfq=2.0:ssnc=all_60");
    quick.push("ott-1002_1024_cond=on:flr=on:gs=on:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:urr=on_42");
    quick.push("dis+1011_12_bs=off:cond=fast:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_778");
    quick.push("dis+4_12_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_216");
    quick.push("dis+1011_40_bs=off:gsp=input_only:nwc=4:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_146");
    quick.push("dis+11_128_bs=off:cond=fast:flr=on:lcm=reverse:nwc=2:sio=off:spl=sat:ssnc=none_176");
    quick.push("dis+1011_128_bs=off:cond=fast:flr=on:fsr=off:lcm=reverse:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none_217");
    break;

  case Property::FEQ:
    if (prop == 131075) {
      if (atoms > 3000) {
        quick.push("dis-1002_3_bs=off:cond=on:drc=off:ep=RS:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none_19");
        quick.push("ins+1011_6_bs=off:bsr=unit_only:cond=on:ep=RSTC:fde=none:igbrr=0.2:igrr=8/1:igrp=1400:igrpq=1.5:igwr=on:nwc=1.1:sos=on_15");
        quick.push("dis-1002_8_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.7:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:updr=off_15");
        quick.push("ott+1011_5:4_bs=off:cond=fast:drc=off:flr=on:fsr=off:nwc=2:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_63");
        quick.push("ott+10_8:1_bd=off:bs=off:cond=fast:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_15");
        quick.push("dis-1010_3:2_bs=off:drc=off:ep=on:nwc=3:sac=on:sgo=on:sio=off:spo=on:sfv=off_41");
        quick.push("dis+10_8:1_bs=off:br=off:cond=on:drc=off:ep=RST:fsr=off:nwc=1.3:sd=4:ss=axioms:st=3.0:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_89");
        quick.push("ins-1003_7_bs=off:ep=RSTC:flr=on:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=1.1:igwr=on:nwc=1.3:sos=on:sio=off:spl=off_223");
        quick.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:stl=240:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_737");
        quick.push("ott+1011_2:3_bs=off:br=off:cond=on:drc=off:gs=on:nwc=10:sd=1:ss=axioms:st=1.2:spl=sat:sser=off:urr=on_11");
      }
      else {
        quick.push("dis-1002_8_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.7:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:updr=off_1");
        quick.push("dis+11_28_bd=off:bs=off:cond=fast:drc=off:ep=on:fsr=off:lcm=reverse:nwc=5:sos=on:sio=off:spl=sat:ssfq=1.0:ssnc=none:sp=occurrence_10");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_1");
        quick.push("ins-1010_3:2_bs=unit_only:drc=off:ep=on:fde=none:igbrr=0.5:igrr=1/128:igrpq=1.3:igwr=on:nwc=1.7_35");
        quick.push("ott+2_8:1_bs=off:bsr=on:drc=off:fde=none:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_75");
        quick.push("dis+10_8:1_bs=off:br=off:cond=on:drc=off:ep=RST:fsr=off:nwc=1.3:sd=4:ss=axioms:st=3.0:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_21");
        quick.push("lrs+11_4_cond=fast:drc=off:flr=on:lcm=reverse:nwc=10:stl=50:sos=on:sio=off:spl=sat:ssfp=100000:ssfq=1.4:ssnc=none:sp=reverse_arity_4");
        quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_30");
        quick.push("lrs-1_5:1_bs=off:cond=fast:drc=off:nwc=4:stl=120:sio=off:spl=sat:ssfp=4000:ssfq=2.0:ssnc=none_8");
        quick.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_179");
        quick.push("dis+1011_10_bd=off:bs=off:drc=off:ep=RS:fsr=off:nwc=1:nicw=on_8");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none_182");
        quick.push("lrs+1_6_bs=off:drc=off:nwc=3:stl=360:sos=all:sio=off:spl=sat:ssac=none:sscc=on:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_167");
        quick.push("ins+10_2:3_bs=unit_only:flr=on:fde=none:igbrr=0.7:igrr=1/32:igrp=200:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.7:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=off_92");
        quick.push("lrs-1004_32_bs=off:br=off:cond=fast:drc=off:flr=on:gs=on:lcm=reverse:nwc=1:nicw=on:stl=30:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:urr=on_132");
        quick.push("dis-1_3_bsr=unit_only:drc=off:lcm=reverse:nwc=4:sos=all:sac=on:sgo=on:sio=off:spo=on:sp=reverse_arity_144");
        quick.push("lrs-1010_7_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:stl=30:sgo=on:spo=on:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_291");
        quick.push("ott+1011_5:4_bs=off:cond=fast:drc=off:flr=on:fsr=off:nwc=2:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_298");
      }
    }
    else if (prop == 1) {
      if (atoms > 125) {
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_8");
        quick.push("ott+11_50_bs=off:cond=on:drc=off:fde=none:lcm=reverse:nwc=3:sos=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=reverse_arity:urr=on_7");
        quick.push("ins+11_3:1_bd=off:bs=off:cond=fast:drc=off:igbrr=0.9:igrr=1/128:igrp=200:igrpq=1.2:igwr=on:lcm=predicate:nwc=1:sos=all:sp=occurrence_1");
        quick.push("ott-1003_24_drc=off:nwc=2:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none:urr=on_2");
        quick.push("ott+1010_5_bd=off:bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_1");
        quick.push("dis-1002_3_bs=off:cond=on:drc=off:ep=RS:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none_190");
        quick.push("ott+11_10_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_10");
        quick.push("dis+1011_14_bd=off:bs=off:cond=fast:drc=off:nwc=4:sio=off:spl=sat:ssfq=1.0:ssnc=none_54");
        quick.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_168");
        quick.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_80");
        quick.push("dis+11_32_bs=off:nwc=1.1:sio=off:spl=off:updr=off_198");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_12");
      }
      else {
        quick.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_13");
        quick.push("ott-1002_5:1_bs=off:bsr=on:cond=fast:drc=off:gsp=input_only:nwc=2:sos=all:sio=off:spo=on:urr=on_3");
        quick.push("dis+1011_2_bs=off:drc=off:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:sfv=off:urr=on_3");
        quick.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_3");
        quick.push("dis+1011_1_bs=off:drc=off:nwc=1.2:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none_6");
        quick.push("dis-1002_3_bs=off:cond=on:drc=off:ep=RS:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none_1");
        quick.push("dis+2_2:3_bs=off:drc=off:lcm=reverse:nwc=2.5:sos=all:sser=off:ssfp=10000:ssfq=2.0:ssnc=all_8");
        quick.push("ott+10_1024_bs=off:cond=on:drc=off:gsp=input_only:nwc=1.2:sio=off:spl=sat:ssfp=10000:ssnc=none:updr=off_23");
        quick.push("ott-1010_3_bd=off:bs=off:bsr=unit_only:drc=off:fde=none:nwc=1:nicw=on:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_114");
        quick.push("ott+11_1_bs=off:drc=off:ep=RS:flr=on:fde=none:nwc=4:sos=all:sgo=on:sio=off_121");
        quick.push("ott+1_2:3_bs=off:cond=fast:drc=off:flr=on:lcm=predicate:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on_197");
        quick.push("ott+4_64_bd=off:bs=off:br=off:cond=on:drc=off:fsr=off:gs=on:nwc=1.7:sos=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.0:ssnc=none:urr=on_795");
      }
    }
    else if (prop == 0) {
      if (atoms < 8) {
        quick.push("ott+10_4_bs=off:drc=off:nwc=3:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:sp=occurrence_1");
        quick.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_424");
        quick.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:stl=270:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_1841");
      }
      else if (atoms < 12) {
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_774");
      }
      else if (atoms < 15) {
        quick.push("lrs-1004_12_bs=off:bsr=unit_only:drc=off:nwc=1.7:nicw=on:stl=60:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.4:ssnc=none_473");
        quick.push("lrs+1_6_bs=off:drc=off:nwc=3:stl=360:sos=all:sio=off:spl=sat:ssac=none:sscc=on:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_1806");
      }
      else if (atoms < 50) {
        quick.push("dis+4_5_bs=off:drc=off:lcm=reverse:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=40000:ssfq=1.4:ssnc=none:sp=occurrence_2");
        quick.push("ott+3_7_bs=off:cond=fast:nwc=3:sgt=20:ss=axioms:st=1.2:sos=on:sagn=off:sio=off:spl=sat_4");
        quick.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_33");
        quick.push("lrs-1_10_bs=off:cond=fast:drc=off:nwc=1.5:nicw=on:stl=20:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.4:ssnc=none_127");
        quick.push("ins+4_2_bd=off:bs=off:gs=on:igbrr=0.8:igrr=1/32:igrp=2000:igrpq=1.2:igwr=on:lcm=reverse:nwc=10:urr=on_3");
        quick.push("ott+11_10_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_126");
        quick.push("ott-3_10_bs=off:br=off:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_66");
        quick.push("ins-11_3:1_bs=off:cond=on:drc=off:fsr=off:igbrr=0.5:igrp=100:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.1:sos=all:sio=off:spl=off:sp=reverse_arity_12");
        quick.push("dis+4_2:1_bs=off:br=off:drc=off:fsr=off:nwc=1:sos=all:sio=off:spl=sat:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_148");
        quick.push("lrs-11_6_bs=off:bsr=unit_only:drc=off:gsp=input_only:lcm=predicate:nwc=10:stl=30:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on:updr=off_132");
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_354");
        quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_220");
        quick.push("lrs-1_5:1_bs=off:cond=fast:drc=off:nwc=4:stl=120:sio=off:spl=sat:ssfp=4000:ssfq=2.0:ssnc=none_569");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_1");
        quick.push("ott-1003_24_drc=off:nwc=2:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none:urr=on_46");
      }
      else if (atoms < 150) {
        quick.push("ott+10_3:1_bsr=unit_only:cond=fast:fsr=off:lcm=reverse:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:urr=on:updr=off_23");
        quick.push("ott+4_32_bs=off:flr=on:nwc=4:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_272");
        quick.push("lrs-2_6_bs=off:cond=fast:drc=off:nwc=4:nicw=on:stl=30:spo=on:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_4");
        quick.push("ott+10_24_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:sser=off:ssnc=none_288");
        quick.push("dis+2_20_drc=off:ep=RST:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none:sp=occurrence_62");
        quick.push("dis+4_40_cond=on:drc=off:flr=on:lcm=predicate:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_149");
        quick.push("ott+10_8:1_bd=off:bs=off:cond=fast:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_77");
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_551");
      }
      else if (atoms < 900) {
        quick.push("ins-1010_3:2_bs=unit_only:drc=off:ep=on:fde=none:igbrr=0.5:igrr=1/128:igrpq=1.3:igwr=on:nwc=1.7_30");
        quick.push("dis+1011_3_bs=off:cond=fast:drc=off:fsr=off:fde=none:gs=on:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_71");
        quick.push("dis-1010_4:1_bs=off:drc=off:lcm=predicate:nwc=2:nicw=on:sio=off_165");
        quick.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_498");
        quick.push("ott+11_10_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_929");
      }
      else {
        quick.push("dis+1011_1_bs=off:drc=off:nwc=1.2:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none_20");
        quick.push("ins-1010_3:2_bs=unit_only:drc=off:ep=on:fde=none:igbrr=0.5:igrr=1/128:igrpq=1.3:igwr=on:nwc=1.7_55");
        quick.push("ott+11_1024_bd=off:bs=off:br=off:cond=fast:drc=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on_127");
        quick.push("dis+4_10_bs=off:drc=off:flr=on:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=10000:ssfq=1.4:ssnc=none_274");
        quick.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_560");
        quick.push("dis-1010_3:1_bd=off:bs=off:cond=fast:gs=on:lcm=reverse:nwc=1.1:sac=on_505");
      }
    }
    else if (prop == 131087) {
      if (atoms > 140000) {
        quick.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_23");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_98");
        quick.push("dis+10_8:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.1:nicw=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_39");
        quick.push("ott+1_2_bs=off:drc=off:ep=on:nwc=3:nicw=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat:sser=off_43");
        quick.push("ott+2_3:1_bs=off:br=off:drc=off:nwc=1.1:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:urr=on_22");
        quick.push("dis-1010_2:3_bs=off:drc=off:nwc=3:sd=2:ss=axioms:st=1.5:sac=on:sio=off_20");
        quick.push("dis+1_3_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=1:ss=included:st=2.0:sagn=off:sio=off:spl=sat:sser=off:ssnc=none_134");
        quick.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_95");
        quick.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_24");
        quick.push("dis-4_4:1_bs=off:ep=RST:gsp=input_only:gs=on:nwc=5:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off:sp=occurrence_16");
        quick.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_136");
        quick.push("ott+4_40_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sd=2:sgt=10:ss=axioms:st=3.0:sos=on:spl=sat:sser=off:updr=off_33");
        quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_51");
        quick.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_42");
        quick.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_47");
        quick.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=1.5:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_168");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=2.5:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_129");
        quick.push("ott+10_8:1_bd=off:bs=off:drc=off:fde=none:gsp=input_only:nwc=1:sd=3:ss=axioms:sos=on:sio=off:spl=off:urr=on_213");
        quick.push("ott+10_2:3_bs=off:drc=off:gs=on:nwc=1.5:sd=2:ss=axioms:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:urr=on_111");
        quick.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_121");
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_149");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=10:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_305");
      }
      else if (atoms > 70000) {
        quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_13");
        quick.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_16");
        quick.push("ott+1_2_bs=off:drc=off:ep=on:nwc=3:nicw=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat:sser=off_17");
        quick.push("dis+2_1_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off_19");
        quick.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_9");
        quick.push("dis-1002_40_bs=off:ep=RST:flr=on:gs=on:lcm=predicate:nwc=2.5:nicw=on:sd=5:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:ssnc=none:sp=reverse_arity_8");
        quick.push("dis+10_8:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.1:nicw=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_40");
        quick.push("dis-1010_2_bd=off:bs=off:cond=fast:drc=off:nwc=5:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:sp=occurrence_37");
        quick.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_64");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_86");
        quick.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_9");
        quick.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=1.5:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_87");
        quick.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_19");
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_63");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=2.5:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_103");
        quick.push("dis+1_3_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=1:ss=included:st=2.0:sagn=off:sio=off:spl=sat:sser=off:ssnc=none_135");
        quick.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_276");
        quick.push("dis-1010_1024_bs=off:cond=fast:drc=off:fde=none:nwc=1:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_76");
        quick.push("dis+2_8:1_bd=off:bs=off:bsr=on:drc=off:lcm=reverse:nwc=4:nicw=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sser=off:sp=reverse_arity_104");
        quick.push("ott+10_8:1_bd=off:bs=off:drc=off:fde=none:gsp=input_only:nwc=1:sd=3:ss=axioms:sos=on:sio=off:spl=off:urr=on_122");
        quick.push("ott+10_2:3_bs=off:drc=off:gs=on:nwc=1.5:sd=2:ss=axioms:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:urr=on_275");
        quick.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_142");
        quick.push("dis-1002_8_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.7:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:updr=off_11");
      }
      else if (atoms > 3200) {
        quick.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_8");
        quick.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_3");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_21");
        quick.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_10");
        quick.push("dis+10_8:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.1:nicw=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_40");
        quick.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_47");
        quick.push("dis-4_4:1_bs=off:ep=RST:gsp=input_only:gs=on:nwc=5:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off:sp=occurrence_7");
        quick.push("dis+2_4_bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=4:sd=1:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_3");
        quick.push("ott+1_2_bs=off:drc=off:ep=on:nwc=3:nicw=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat:sser=off_55");
        quick.push("dis-1010_1024_bs=off:cond=fast:drc=off:fde=none:nwc=1:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_5");
        quick.push("dis+1011_1_bs=off:drc=off:nwc=1.2:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none_10");
        quick.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=1.5:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_36");
        quick.push("ott+2_3:1_bs=off:br=off:drc=off:nwc=1.1:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:urr=on_13");
        quick.push("dis+2_8:1_bd=off:bs=off:bsr=on:drc=off:lcm=reverse:nwc=4:nicw=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sser=off:sp=reverse_arity_28");
        quick.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_141");
        quick.push("dis+2_4_bs=off:drc=off:ep=RST:lcm=predicate:nwc=10:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_24");
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_21");
        quick.push("ott+2_4:1_bd=off:bs=off:drc=off:gsp=input_only:nwc=1.1:nicw=on:sd=3:ss=axioms:sos=on:spl=sat:urr=on_23");
        quick.push("ott+10_2:3_bs=off:drc=off:gs=on:nwc=1.5:sd=2:ss=axioms:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:urr=on_53");
        quick.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_259");
        quick.push("dis+1_14_bd=off:bs=off:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=4:sos=on:sio=off:spo=on:sp=reverse_arity_114");
        quick.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_175");
        quick.push("dis+2_4_bs=off:drc=off:ep=RST:flr=on:lcm=reverse:nwc=1.5:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sp=reverse_arity_179");
        quick.push("dis+2_1_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off_8");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=10:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_102");
      }
      else if (atoms > 2200) {
        quick.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_22");
        quick.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_5");
        quick.push("ott+1011_2:1_bs=off:cond=fast:drc=off:nwc=1:nicw=on:sos=all:sio=off:spo=on_29");
        quick.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_104");
        quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_205");
        quick.push("lrs-11_12_bs=off:drc=off:fde=none:gsp=input_only:gs=on:lcm=predicate:nwc=4:nicw=on:stl=300:sos=all:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sfv=off:sp=occurrence:urr=on:updr=off_1612");
      }
      else if (atoms > 900) {
        quick.push("lrs+1_5_bs=off:cond=fast:drc=off:flr=on:nwc=10:stl=30:sac=on:sio=off:urr=on_6");
        quick.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_40");
        quick.push("ott-1002_5:1_bs=off:bsr=on:cond=fast:drc=off:gsp=input_only:nwc=2:sos=all:sio=off:spo=on:urr=on_2");
        quick.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_47");
        quick.push("dis+1011_4_bs=off:drc=off:nwc=4:sgo=on_58");
        quick.push("dis+1010_2:3_bs=off:bsr=unit_only:drc=off:ep=on:fsr=off:fde=none:lcm=predicate:nwc=1.5:sos=on:sac=on:sio=off:spo=on:sp=occurrence_424");
        quick.push("ott+2_20_cond=fast:drc=off:flr=on:lcm=reverse:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity:updr=off_249");
        quick.push("ott-1010_4:1_bs=off:bsr=on:cond=on:drc=off:fde=none:gsp=input_only:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:ssfq=1.0:ssnc=none_287");
        quick.push("ott-1010_3_bd=off:bs=off:bsr=unit_only:drc=off:fde=none:nwc=1:nicw=on:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_325");
        quick.push("dis+2_4_bs=off:drc=off:ep=RST:lcm=predicate:nwc=10:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_7");
        quick.push("ott+2_8:1_bs=off:bsr=on:drc=off:fde=none:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_36");
        quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_86");
      }
      else {
        quick.push("ott-1010_4:1_bs=off:bsr=on:cond=on:drc=off:fde=none:gsp=input_only:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:ssfq=1.0:ssnc=none_7");
        quick.push("dis+2_2:3_bs=off:bsr=unit_only:cond=fast:drc=off:ep=RS:lcm=reverse:nwc=1.2:sos=all:sio=off:spl=sat:ssfp=4000:ssfq=1.4:ssnc=none:sfv=off:sp=occurrence_24");
        quick.push("dis+1010_40_bs=off:drc=off:ep=RS:nwc=1:sio=off:spo=on:spl=sat:ssnc=none_21");
        quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1.1:sos=all:sio=off:spo=on:sp=occurrence_17");
        quick.push("ott+11_2:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:fde=none:gsp=input_only:lcm=reverse:nwc=2.5:sio=off:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_23");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=2.5:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_9");
        quick.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_3");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=10:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_272");
        quick.push("dis-1010_5:4_bs=off:bsr=on:cond=fast:drc=off:fde=none:gsp=input_only:nwc=3:sgo=on:sio=off:sp=occurrence:urr=on_150");
        quick.push("ott+11_1_bs=off:drc=off:ep=RS:flr=on:fde=none:nwc=4:sos=all:sgo=on:sio=off_220");
        quick.push("lrs+11_4_cond=fast:drc=off:flr=on:lcm=reverse:nwc=10:stl=50:sos=on:sio=off:spl=sat:ssfp=100000:ssfq=1.4:ssnc=none:sp=reverse_arity_351");
        quick.push("dis-1002_3_bs=off:cond=on:drc=off:ep=RS:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none_18");
        quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ep=RST:flr=on:fde=none:lcm=reverse:nwc=10:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:urr=on_85");
      }
    }
    else if (prop == 131073) {
      if (atoms > 400) {
        quick.push("ott+11_1_bs=off:drc=off:ep=RS:flr=on:fde=none:nwc=4:sos=all:sgo=on:sio=off_2");
        quick.push("dis+1011_4_bs=off:drc=off:nwc=4:sgo=on_11");
        quick.push("dis+2_8:1_bd=off:bs=off:bsr=on:drc=off:lcm=reverse:nwc=4:nicw=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sser=off:sp=reverse_arity_12");
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_2");
        quick.push("dis-1010_2_bs=off:bsr=on:drc=off:nwc=4:ssac=none:sscc=on:ssfp=100000:ssfq=1.4:ssnc=all_4");
        quick.push("dis+1_128_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=10:sd=2:ss=included:st=2.0:sagn=off:sio=off:spl=sat:ssnc=none_3");
        quick.push("ins+10_2:3_bs=unit_only:flr=on:fde=none:igbrr=0.7:igrr=1/32:igrp=200:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.7:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=off_26");
        quick.push("dis+1_3_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=1:ss=included:st=2.0:sagn=off:sio=off:spl=sat:sser=off:ssnc=none_2");
        quick.push("dis+4_5_bs=off:drc=off:lcm=reverse:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=40000:ssfq=1.4:ssnc=none:sp=occurrence_7");
        quick.push("ins-1003_7_bs=off:ep=RSTC:flr=on:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=1.1:igwr=on:nwc=1.3:sos=on:sio=off:spl=off_2");
        quick.push("ott+11_2:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:fde=none:gsp=input_only:lcm=reverse:nwc=2.5:sio=off:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_12");
        quick.push("dis+1011_12_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_14");
        quick.push("ott+11_3:2_bs=off:cond=on:drc=off:ep=RSTC:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sagn=off:sio=off:spl=sat:urr=on_3");
        quick.push("dis+2_4_bs=off:br=off:drc=off:nwc=1.2:sd=2:ss=axioms:st=2.0:sio=off:urr=on_24");
        quick.push("ott-1010_2:3_bs=off:cond=fast:drc=off:nwc=1.7:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_9");
        quick.push("dis+1010_32_cond=fast:drc=off:ep=RS:fsr=off:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_66");
        quick.push("lrs-1002_3_bd=off:bs=off:drc=off:ep=on:nwc=1.7:stl=150:sos=on:sac=on:sio=off_21");
        quick.push("tab+10_1_ep=RST:gsp=input_only:sd=4:ss=axioms:st=2.0:sio=off:tbsr=off:tgawr=1/32:tglr=1/5:tipr=off:tlawr=8/1_73");
        quick.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_183");
        quick.push("ott-1002_5:1_bs=off:bsr=on:cond=fast:drc=off:gsp=input_only:nwc=2:sos=all:sio=off:spo=on:urr=on_122");
        quick.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_28");
        quick.push("ott+1011_5:4_bs=off:cond=fast:drc=off:flr=on:fsr=off:nwc=2:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_32");
        quick.push("dis+10_8:1_bs=off:br=off:cond=on:drc=off:ep=RST:fsr=off:nwc=1.3:sd=4:ss=axioms:st=3.0:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_92");
        quick.push("dis+2_1_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off_148");
        quick.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:stl=270:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_1");
        quick.push("dis+11_4_bs=off:drc=off:ep=on:gsp=input_only:nwc=5:sgt=15:ss=axioms:sos=on:spl=sat_1");
        quick.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat_2");
        quick.push("ott+10_8:1_bd=off:bs=off:cond=fast:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_4");
        quick.push("ott+1011_64_bs=off:cond=fast:drc=off:gsp=input_only:nwc=1:ss=axioms:st=1.2:spl=sat:sser=off:urr=on_5");
        quick.push("dis+2_128_bs=off:drc=off:lcm=reverse:nwc=1.3:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_8");
        quick.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_13");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none_13");
        quick.push("dis-1010_64_bs=off:drc=off:nwc=2.5:nicw=on:sgo=on:sio=off:spo=on:sser=off:ssfp=40000:ssfq=1.0:ssnc=none_20");
      }
      else if (atoms > 150) {
        quick.push("ott-1010_2:3_bs=off:cond=fast:drc=off:nwc=1.7:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_5");
        quick.push("dis+10_24_bs=off:br=off:drc=off:nwc=4:sd=5:ss=axioms:st=1.5:sgo=on:sio=off:spl=sat:ssac=none:ssfp=1000:ssfq=1.0:ssnc=all:urr=on_1");
        quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_20");
        quick.push("dis-1010_64_bs=off:drc=off:nwc=2.5:nicw=on:sgo=on:sio=off:spo=on:sser=off:ssfp=40000:ssfq=1.0:ssnc=none_2");
        quick.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_25");
        quick.push("dis-1002_1024_bs=off:cond=on:drc=off:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_1");
        quick.push("dis-1010_2_bd=off:bs=off:cond=fast:drc=off:nwc=5:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:sp=occurrence_27");
        quick.push("lrs-11_6_bs=off:bsr=unit_only:drc=off:gsp=input_only:lcm=predicate:nwc=10:stl=30:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on:updr=off_11");
        quick.push("dis+1011_12_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_68");
        quick.push("dis+2_4_bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=4:sd=1:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_19");
        quick.push("dis-1_3_bsr=unit_only:drc=off:lcm=reverse:nwc=4:sos=all:sac=on:sgo=on:sio=off:spo=on:sp=reverse_arity_146");
        quick.push("dis+1_24_bs=off:drc=off:ep=on:lcm=predicate:nwc=3:nicw=on:ss=included:st=5.0:sos=on:sagn=off:spl=sat:sser=off:ssnc=none_33");
        quick.push("ott+1_8:1_bs=off:cond=fast:drc=off:gsp=input_only:nwc=1.3:nicw=on:sd=2:sgt=10:ss=axioms:sagn=off:spl=sat:sser=off_72");
        quick.push("ott+1_2:3_bs=off:cond=fast:drc=off:flr=on:lcm=predicate:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on_20");
        quick.push("dis-1010_3:2_bs=off:drc=off:ep=on:nwc=3:sac=on:sgo=on:sio=off:spo=on:sfv=off_242");
        quick.push("ott+1010_5_bd=off:bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_189");
        quick.push("dis+2_16_bs=off:drc=off:ep=on:gsp=input_only:lcm=reverse:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none_420");
        quick.push("ott+1011_2_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:updr=off_227");
        quick.push("dis+2_2:3_bs=off:drc=off:lcm=reverse:nwc=2.5:sos=all:sser=off:ssfp=10000:ssfq=2.0:ssnc=all_231");
        quick.push("dis+2_4_bs=off:br=off:drc=off:nwc=1.2:sd=2:ss=axioms:st=2.0:sio=off:urr=on_1");
        quick.push("ott+1011_2:1_bs=off:cond=fast:drc=off:nwc=1:nicw=on:sos=all:sio=off:spo=on_23");
      }
      else {
        quick.push("lrs-1010_7_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:stl=30:sgo=on:spo=on:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_1");
        quick.push("dis-1002_6_bs=off:drc=off:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat:sser=off_1");
        quick.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_4");
        quick.push("ott+2_20_cond=fast:drc=off:flr=on:lcm=reverse:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity:updr=off_25");
        quick.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:stl=240:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_1");
        quick.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_97");
        quick.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:stl=270:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_8");
        quick.push("dis+2_4_bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=4:sd=1:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_18");
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_860");
        quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1.1:sos=all:sio=off:spo=on:sp=occurrence_26");
        quick.push("dis+1011_14_bd=off:bs=off:cond=fast:drc=off:nwc=4:sio=off:spl=sat:ssfq=1.0:ssnc=none_262");
      }
    }
    else if (prop == 2) {
      if (atoms > 25) {
        quick.push("ins-1010_1_bs=unit_only:drc=off:igbrr=0.0:igrr=1/4:igrp=4000:igrpq=1.0:igwr=on:nwc=1.1_16");
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_874");
        quick.push("lrs+1_64_bs=off:drc=off:gsp=input_only:nwc=1.7:nicw=on:stl=60:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:updr=off_282");
        quick.push("ott+10_1024_bs=off:cond=on:drc=off:gsp=input_only:nwc=1.2:sio=off:spl=sat:ssfp=10000:ssnc=none:updr=off_368");
      }
      else {
        quick.push("ins+3_10_bs=off:drc=off:fde=none:igbrr=0.8:igrr=1/128:igrp=100:igrpq=1.0:igwr=on:nwc=2.5:sio=off:spl=off_152");
        quick.push("lrs+1_6_bs=off:drc=off:nwc=3:stl=360:sos=all:sio=off:spl=sat:ssac=none:sscc=on:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_48");
        quick.push("ott+1_5_bs=off:drc=off:nwc=4:sio=off:sp=occurrence_245");
        quick.push("ott+11_24_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity_178");
        quick.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_577");
        quick.push("ott+4_14_bs=unit_only:cond=fast:drc=off:nwc=1.2:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_383");
        quick.push("ins-1010_1_bs=unit_only:drc=off:igbrr=0.0:igrr=1/4:igrp=4000:igrpq=1.0:igwr=on:nwc=1.1_73");
        quick.push("lrs+10_20_bs=off:cond=on:drc=off:gs=on:nwc=1.1:stl=10:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_80");
      }
    }
    else if (prop == 131072) {
      quick.push("dis-1010_2_bs=off:bsr=on:drc=off:nwc=4:ssac=none:sscc=on:ssfp=100000:ssfq=1.4:ssnc=all_16");
      quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_90");
      quick.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_1");
      quick.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_7");
      quick.push("dis-1002_1024_bs=off:cond=on:drc=off:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_16");
      quick.push("dis-1010_2:3_bs=off:drc=off:nwc=3:sd=2:ss=axioms:st=1.5:sac=on:sio=off_105");
      quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1.1:sos=all:sio=off:spo=on:sp=occurrence_110");
      quick.push("dis+1003_1024_bs=off:drc=off:fsr=off:nwc=1.7:nicw=on:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none_38");
      quick.push("lrs+10_20_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=2.5:stl=80:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_59");
      quick.push("ott+10_3:1_bd=off:drc=off:lcm=reverse:nwc=10:nicw=on:sio=off:spo=on:ssac=none:sscc=on:sser=off:ssfp=1000:ssfq=1.2:ssnc=all_667");
      quick.push("dis-1010_50_bs=unit_only:bsr=on:drc=off:lcm=predicate:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_580");
      quick.push("ins-1010_1_bs=unit_only:drc=off:igbrr=0.0:igrr=1/4:igrp=4000:igrpq=1.0:igwr=on:nwc=1.1_12");
    }
    else if (atoms > 10000) {
      quick.push("ins+10_2:3_bs=unit_only:flr=on:fde=none:igbrr=0.7:igrr=1/32:igrp=200:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.7:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=off_24");
      quick.push("dis+2_10_bs=off:cond=fast:ep=RST:lcm=reverse:nwc=1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_44");
      quick.push("dis-1002_1024_bs=off:cond=on:drc=off:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_33");
      quick.push("dis+1_128_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=10:sd=2:ss=included:st=2.0:sagn=off:sio=off:spl=sat:ssnc=none_35");
      quick.push("ins-1003_7_bs=off:ep=RSTC:flr=on:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=1.1:igwr=on:nwc=1.3:sos=on:sio=off:spl=off_6");
      quick.push("dis+1_10_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=7:ss=axioms:st=1.5:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_153");
      quick.push("dis+11_10_bs=off:lcm=predicate:nwc=1.3:sio=off_100");
      quick.push("dis+11_1024_bsr=unit_only:cond=fast:nwc=1.3:sio=off:spl=off_590");
      quick.push("ott+1_5_bs=off:drc=off:nwc=4:sio=off:sp=occurrence_217");
      quick.push("dis+11_32_bs=off:nwc=1.1:sio=off:spl=off:updr=off_244");
    }
    else if (prop == 131083) {
      quick.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat_1");
      quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_6");
      quick.push("dis+2_128_bs=off:drc=off:lcm=reverse:nwc=1.3:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_2");
      quick.push("ott-1010_4:1_bs=off:bsr=on:cond=on:drc=off:fde=none:gsp=input_only:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:ssfq=1.0:ssnc=none_187");
      quick.push("dis+1010_1_bs=off:cond=fast:ep=on:flr=on:fde=none:gsp=input_only:nwc=1.5:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_441");
      quick.push("lrs+10_20_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=2.5:stl=80:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_474");
    }
    else if (prop == 131074) {
      quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_2");
      quick.push("dis-1010_2_bs=off:bsr=on:drc=off:nwc=4:ssac=none:sscc=on:ssfp=100000:ssfq=1.4:ssnc=all_23");
      quick.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_4");
      quick.push("dis+10_2_bs=off:cond=on:drc=off:fde=none:lcm=predicate:nwc=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sp=reverse_arity_31");
      quick.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:stl=240:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_75");
      quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_236");
      quick.push("dis-1010_64_bs=off:drc=off:nwc=2.5:nicw=on:sgo=on:sio=off:spo=on:sser=off:ssfp=40000:ssfq=1.0:ssnc=none_27");
      quick.push("dis+1011_4_bs=off:drc=off:nwc=4:sgo=on_56");
      quick.push("ott+4_40_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sd=2:sgt=10:ss=axioms:st=3.0:sos=on:spl=sat:sser=off:updr=off_157");
    }
    else {
      quick.push("ott+1011_10_bs=off:cond=on:drc=off:nwc=1.5:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_4");
      quick.push("ins+10_2:3_bs=off:cond=fast:ep=RSTC:igbrr=0.8:igrr=1/2:igrp=100:igrpq=1.0:igwr=on:nwc=2:sos=all:sio=off:urr=on_1");
      quick.push("dis+2_2:3_bs=off:drc=off:lcm=reverse:nwc=2.5:sos=all:sser=off:ssfp=10000:ssfq=2.0:ssnc=all_9");
      quick.push("dis+4_5:1_bs=off:drc=off:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:ss=axioms:st=2.0:sac=on:sio=off:sp=occurrence:urr=on_3");
      quick.push("dis+11_2:3_bs=off:bsr=unit_only:drc=off:ep=R:lcm=reverse:nwc=2:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_1");
      quick.push("dis+1011_28_cond=on:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:updr=off_12");
      quick.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_1");
      quick.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_37");
      quick.push("ins+1011_8:1_bs=off:cond=fast:drc=off:ep=on:igbrr=0.1:igrr=1/4:igrpq=1.0:igwr=on:nwc=1.3:sos=all:sio=off:sfv=off:urr=on:updr=off_8");
      quick.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_1");
      quick.push("lrs-1010_7_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:stl=30:sgo=on:spo=on:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_17");
      quick.push("dis+1011_14_bd=off:bs=off:cond=fast:drc=off:nwc=4:sio=off:spl=sat:ssfq=1.0:ssnc=none_7");
      quick.push("ott+11_1024_bd=off:bs=off:br=off:cond=fast:drc=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on_128");
      quick.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_40");
      quick.push("dis+11_1_bs=off:drc=off:ep=on:flr=on:lcm=predicate:nwc=5:nicw=on:ss=included:st=5.0:sos=all:sagn=off:spl=sat:sser=off:ssnc=none:updr=off_24");
      quick.push("dis+1010_2:3_bs=off:bsr=unit_only:drc=off:ep=on:fsr=off:fde=none:lcm=predicate:nwc=1.5:sos=on:sac=on:sio=off:spo=on:sp=occurrence_9");
      quick.push("dis-1010_3:2_bs=off:drc=off:ep=on:nwc=3:sac=on:sgo=on:sio=off:spo=on:sfv=off_8");
      quick.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:stl=270:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_10");
      quick.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_48");
      quick.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:stl=240:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_12");
      quick.push("dis+10_24_bs=off:br=off:drc=off:nwc=4:sd=5:ss=axioms:st=1.5:sgo=on:sio=off:spl=sat:ssac=none:ssfp=1000:ssfq=1.0:ssnc=all:urr=on_7");
      quick.push("dis+1011_2_bs=off:drc=off:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:sfv=off:urr=on_53");
      quick.push("dis-1002_8:1_bs=off:cond=on:drc=off:flr=on:gs=on:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_2");
      quick.push("ins+2_4:1_bs=off:cond=on:ep=RSTC:gs=on:igbrr=0.4:igrr=1/128:igrpq=1.05:igwr=on:nwc=5:sos=on:sio=off:urr=on_1");
      quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_108");
      quick.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat_165");
      quick.push("dis+1_24_bs=off:drc=off:ep=on:lcm=predicate:nwc=3:nicw=on:ss=included:st=5.0:sos=on:sagn=off:spl=sat:sser=off:ssnc=none_31");
      quick.push("ott+2_8:1_bs=off:bsr=on:drc=off:fde=none:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_23");
      quick.push("dis+1010_1_bs=off:cond=fast:ep=on:flr=on:fde=none:gsp=input_only:nwc=1.5:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_118");
      quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_296");
      quick.push("ott+1011_64_bs=off:cond=fast:drc=off:gsp=input_only:nwc=1:ss=axioms:st=1.2:spl=sat:sser=off:urr=on_212");
      quick.push("dis+11_4_bs=off:drc=off:ep=on:gsp=input_only:nwc=5:sgt=15:ss=axioms:sos=on:spl=sat_164");
      quick.push("lrs+10_20_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=2.5:stl=80:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_14");
      quick.push("lrs+11_4_cond=fast:drc=off:flr=on:lcm=reverse:nwc=10:stl=50:sos=on:sio=off:spl=sat:ssfp=100000:ssfq=1.4:ssnc=none:sp=reverse_arity_24");
    }
    break;

  case Property::FNE:
    if (atoms > 200) {
      quick.push("ins+10_1_gsp=input_only:igbrr=0.9:igrp=100:igrpq=1.1:nwc=2_5");
      quick.push("dis+11_50_bs=off:bsr=on:cond=on:lcm=reverse:nwc=1:sio=off:spl=sat:sser=off:ssnc=none:updr=off_267");
      quick.push("dis+4_128_bs=off:cond=on:flr=on:nwc=1.2:sos=on:sac=on_20");
      quick.push("dis+1011_3_bs=off:cond=fast:flr=on:gsp=input_only:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_59");
      quick.push("tab+10_1_spl=off:tbsr=off:tfsr=off:tgawr=3/1:tglr=1/20:tipr=off:tlawr=8/1_114");
      quick.push("dis+3_6_flr=on:fsr=off:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:sp=occurrence_34");
      quick.push("ott+1011_50_bs=off:gs=on:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none:urr=on:updr=off_190");
      quick.push("ott+1_7_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=1.5:spo=on_354");
      quick.push("dis+1010_8_bs=off:nwc=1.1:sio=off_134");
      quick.push("dis+1004_10_bs=off:nwc=1.5:sagn=off:sio=off:spl=sat:ssnc=none_617");
    }
    else if (atoms > 120) {
      quick.push("ott-1002_3_bs=unit_only:bsr=unit_only:cond=on:gsp=input_only:nwc=1.2:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:urr=on_185");
      quick.push("dis+4_10_bs=off:cond=fast:flr=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence:updr=off_204");
      quick.push("lrs+1002_128_bs=off:cond=on:flr=on:lcm=reverse:nwc=1:stl=120:spo=on_990");
    }
    else {
      quick.push("dis+1011_3_bs=off:cond=fast:flr=on:gsp=input_only:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_1");
      quick.push("dis+4_10_bs=off:cond=fast:flr=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence:updr=off_1");
      quick.push("lrs+1002_128_bs=off:cond=on:flr=on:lcm=reverse:nwc=1:stl=120:spo=on_28");
      quick.push("ott+1_40_bsr=unit_only:cond=fast:gs=on:nwc=1.5:sio=off:spl=sat:ssnc=none:urr=on_102");
      quick.push("ins+10_1_igbrr=0.5:igrpq=1.2:nwc=1:sio=off_32");
      quick.push("ott+1_16_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=2.5_94");
      quick.push("ott-1010_128_bs=off:cond=on:lcm=predicate:nwc=1.3:urr=on_394");
      quick.push("ott+10_1024_bs=off:fsr=off:nwc=1.5:nicw=on:sio=off:spl=off_963");
    }
    break;

  case Property::EPR:
    if (prop == 131072) {
      quick.push("ott+10_1024_bs=off:br=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none:urr=on_158");
      quick.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_162");
    }
    else if (atoms > 2000) {
      quick.push("ott+2_20_bs=off:bsr=unit_only:nwc=5:nicw=on:sio=off:spl=sat:ssnc=none_119");
      quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_1935");
    }
    else if (atoms > 1300) {
      quick.push("dis-11_24_bs=off:cond=fast:drc=off:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_1");
      quick.push("ott+10_1024_bs=off:br=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none:urr=on_148");
      quick.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_171");
      quick.push("ins+3_128_bs=off:br=off:drc=off:igbrr=0.1:igrr=32/1:igrp=2000:igrpq=2.0:igwr=on:nwc=3:sos=all:sio=off:urr=on_262");
      quick.push("ins+2_128_bs=unit_only:drc=off:fsr=off:igbrr=1.0:igrr=128/1:igrp=200:igrpq=2.0:igwr=on:lcm=predicate:nwc=2:sos=on:sio=off:sp=occurrence_277");
      quick.push("dis+1_40_bs=unit_only:bsr=unit_only:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none_560");
    }
    else if (atoms > 450) {
      quick.push("dis-10_5:4_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=1.1:sac=on:sio=off:spo=on_5");
      quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_22");
      quick.push("ins+11_50_bs=off:cond=fast:drc=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=2000:igrpq=1.5:igwr=on:nwc=1.2:urr=on_56");
      quick.push("ins+3_50_bs=off:br=off:drc=off:igbrr=0.7:igrr=1/128:igrp=100:igrpq=1.05:igwr=on:lcm=predicate:nwc=2:sp=occurrence:urr=on_501");
      quick.push("ins+10_1_igbrr=0.4:igrp=200:igrpq=1.5:nwc=1.1:sio=off_561");
    }
    else {
      quick.push("ins+11_50_bs=off:cond=fast:drc=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=2000:igrpq=1.5:igwr=on:nwc=1.2:urr=on_12");
      quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_13");
      quick.push("ins+3_14_bs=off:cond=on:drc=off:flr=on:igbrr=0.2:igrr=1/128:igrp=200:igrpq=1.2:igwr=on:nwc=1:urr=on_43");
      quick.push("ott+1_128_bs=off:cond=fast:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none:urr=on_1");
      quick.push("dis+4_10_bs=off:drc=off:fsr=off:nwc=5:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.0:ssnc=none_482");
      quick.push("ins-10_14_bs=off:drc=off:fde=none:igbrr=0.9:igrr=1/4:igrp=2000:igrpq=2.0:igwr=on:nwc=1.5:sos=on:sgo=on:sio=off:updr=off_1506");
    }
    break;

  case Property::UEQ:
    if (prop == 2048) {
      quick.push("dis+2_14_bs=off:br=off:ep=RSTC:flr=on:fsr=off:nwc=2.5:nicw=on:sos=all:sio=off:spl=sat:ssnc=none:urr=on_2");
      quick.push("dis+10_32_bs=off:drc=off:fsr=off:nwc=5_2");
      quick.push("lrs+10_14_drc=off:fde=none:nwc=2.5:stl=180_1097");
    }
    else if (prop == 2) {
      if (atoms <= 17) {
        quick.push("dis+11_16_bs=off:bfnt=on:fsr=off:fde=none:gs=on:nwc=1.3:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none_1");
        quick.push("ott+10_20_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.3:sp=occurrence_290");
        quick.push("ott+10_50_bs=off:drc=off:fsr=off:fde=none:nwc=1_213");
        quick.push("ott+10_8:1_bs=off:bsr=on:nwc=4_77");
        quick.push("lrs+10_20_bd=off:bs=unit_only:drc=off:fsr=off:nwc=1.2:stl=90_442");
        quick.push("ott+10_64_drc=off:nwc=1.1_298");
      }
      else if (atoms <= 20) {
        quick.push("ott+2_10_bs=off:drc=off:ep=on:fsr=off:fde=none:gsp=input_only:nwc=1.7:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfq=1.1:ssnc=none_111");
        quick.push("lrs+10_2:1_bs=off:drc=off:fsr=off:nwc=2.5:stl=60:sp=occurrence_14");
        quick.push("lrs+10_14_drc=off:fde=none:nwc=2.5:stl=180_149");
        quick.push("ott+10_2_bs=off:drc=off:fsr=off:fde=none:gsp=input_only:nwc=1.2:sp=reverse_arity_158");
        quick.push("ott+10_50_bs=off:drc=off:fsr=off:fde=none:nwc=1_275");
        quick.push("ins+3_2_bs=off:bfnt=on:br=off:cond=on:fsr=off:fde=none:igbrr=0.9:igrr=1/64:igrpq=2.0:igwr=on:nwc=10:sio=off:urr=on_900");
      }
      else {
        quick.push("lrs+10_2:3_bs=unit_only:cond=on:drc=off:ep=on:fsr=off:gs=on:nwc=1:stl=20:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=occurrence_61");
        quick.push("lrs+10_8_bs=off:cond=fast:drc=off:fsr=off:gsp=input_only:gs=on:nwc=2:stl=120:urr=on_1148");
      }
    }
    else if (atoms <= 2) {
      quick.push("ott+2_10_bs=off:drc=off:ep=on:fsr=off:fde=none:gsp=input_only:nwc=1.7:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfq=1.1:ssnc=none_17");
      quick.push("dis+10_12_bs=off:bsr=unit_only:fsr=off:nwc=2_1");
      quick.push("ins+4_8_bs=off:bfnt=on:br=off:cond=fast:flr=on:fsr=off:igbrr=0.1:igrr=1/128:igrp=100:igrpq=2.0:igwr=on:nwc=1.3:sos=all:urr=on_1");
      quick.push("ott+10_7_bd=off:bs=off:drc=off:fsr=off:fde=none:nwc=1.1:sp=occurrence_87");
      quick.push("lrs+10_1024_nwc=4:stl=150_1463");
    }
    else if (atoms <= 9) {
      quick.push("ott+11_20_bs=off:bfnt=on:cond=fast:fsr=off:lcm=predicate:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.1:ssnc=none:sp=reverse_arity_1");
      quick.push("dis+4_5:1_bs=off:br=off:ep=RSTC:fsr=off:nwc=3:sos=all:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_3");
      quick.push("dis+10_12_bs=off:bsr=unit_only:fsr=off:nwc=2_1");
      quick.push("lrs+10_14_drc=off:fde=none:nwc=2.5:stl=180_852");
      quick.push("ins+3_2_bs=off:bfnt=on:br=off:cond=on:fsr=off:fde=none:igbrr=0.9:igrr=1/64:igrpq=2.0:igwr=on:nwc=10:sio=off:urr=on_33");
      quick.push("ott+10_5_bd=off:drc=off:nwc=2.5_761");
      quick.push("dis+10_12_bs=off:br=off:ep=RSTC:fsr=off:nwc=4:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none:urr=on_59");
    }
    else if (atoms <= 10) {
      quick.push("dis+11_16_bs=off:bfnt=on:fsr=off:fde=none:gs=on:nwc=1.3:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none_1");
      quick.push("ott+10_7_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.5_480");
      quick.push("ott+10_8:1_bs=off:drc=off:fsr=off:nwc=1.7_328");
      quick.push("ott+10_2:1_drc=off:nwc=5_1146");
      quick.push("ott+10_2:3_drc=off:nwc=2_504");
      quick.push("ott+10_8:1_nwc=3:sfv=off_860");
    }
    else if (atoms <= 15) {
      quick.push("ott+11_20_bs=off:bfnt=on:cond=fast:fsr=off:lcm=predicate:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.1:ssnc=none:sp=reverse_arity_1");
      quick.push("lrs+10_14_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.2:stl=20:sio=off:sp=occurrence_25");
      quick.push("lrs+3_20_bs=off:drc=off:fsr=off:fde=none:gsp=input_only:gs=on:lcm=reverse:nwc=3:stl=120_313");
      quick.push("ins+3_2_bs=off:bfnt=on:br=off:cond=on:fsr=off:fde=none:igbrr=0.9:igrr=1/64:igrpq=2.0:igwr=on:nwc=10:sio=off:urr=on_619");
      quick.push("ott+10_20_bd=off:drc=off:nwc=2:sp=occurrence_451");
    }
    else if (atoms <= 18) {
      quick.push("lrs+10_16_drc=off:nwc=1.2:stl=40_351");
      quick.push("dis+10_128_bs=unit_only:bsr=on:drc=off:fsr=off:nwc=2.5_295");
      quick.push("ott+10_8:1_bs=off:drc=off:fsr=off:nwc=1.7_275");
      quick.push("ott+10_10_drc=off:fde=none:nwc=1.1:sp=occurrence_363");
    }
    else {
      quick.push("lrs+10_16_drc=off:nwc=1.2:stl=40_391");
      quick.push("lrs+10_5:1_nwc=1:stl=90_214");
      quick.push("ott+10_2_fde=none:nwc=2.5:sp=reverse_arity_728");
    }
    break;
  }

  switch (cat) {
  case Property::NEQ:
    quick.push("dis-1010_6_bs=off:drc=off:ep=on:gsp=input_only:nwc=10:nicw=on:sos=all:sp=occurrence_300");
    quick.push("ott+11_2:3_bs=off:drc=off:flr=on:lcm=predicate:nwc=2.5:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none:sp=occurrence_300");
    quick.push("dis+10_5_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sgo=on:sio=off:sp=occurrence_100");
    quick.push("dis+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none_300");
    quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_300");
    quick.push("dis+11_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_300");
    quick.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:nwc=2:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_300");
    quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_300");
    quick.push("ott-1010_2:1_bs=off:cond=fast:drc=off:nwc=3:sd=1:ss=axioms:st=2.0:sio=off:spl=sat:sser=off:ssnc=none_300");
    quick.push("lrs+10_1_bs=off:cond=fast:nwc=5:sio=off:spl=sat:ssfq=1.1:ssnc=none_200");
    quick.push("ott+11_3_bd=off:bs=off:drc=off:ep=on:flr=on:nwc=1:sio=off:spl=sat:ssfq=1.1:ssnc=none:sfv=off_300");
    quick.push("dis-11_10_bfnt=on:cond=fast:lcm=predicate:nwc=1.2:sio=off:spl=off:sp=occurrence_600");
    quick.push("dis+11_4:1_bs=off:drc=off:gsp=input_only:nwc=3:sos=on_300");
    quick.push("dis+10_8:1_bs=off:br=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=1:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_300");
    quick.push("dis+2_24_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none_300");
    quick.push("dis+11_4:1_bs=off:cond=fast:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssnc=none_300");
    quick.push("ott+1_1024_bs=off:br=off:cond=on:drc=off:flr=on:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=1.2:sac=on:sio=off:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=all_dependent:urr=on:updr=off_300");
    quick.push("dis+1_40_bs=off:ep=RSTC:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none:urr=on_300");
    quick.push("ott+1011_3_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.2:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    quick.push("dis+1011_24_cond=fast:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_300");
    quick.push("ott+11_3:1_bs=off:drc=off:ep=on:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    quick.push("lrs+2_3:1_bs=off:br=off:cond=fast:drc=off:flr=on:nwc=4:sio=off:spo=on:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on_600");
    quick.push("dis-11_28_bs=off:drc=off:flr=on:lcm=predicate:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=1.0:ssnc=none_300");
    quick.push("ins-1010_8:1_bs=off:ep=RSTC:fsr=off:igbrr=1.0:igrr=1/128:igrp=700:igrpq=1.5:igwr=on:nwc=3:sos=on:sgo=on:sio=off_300");
    quick.push("ott+11_2_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:ep=on:fde=none:lcm=reverse:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_300");
    quick.push("dis+4_8:1_bs=off:drc=off:ep=RS:nwc=2:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=sat:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_200");
    quick.push("lrs+1011_14_bd=off:bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=1.7:sio=off:spo=on:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:urr=on_600");
    quick.push("ott+11_8:1_drc=off:ep=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none:sp=occurrence:updr=off_600");
    quick.push("dis+1011_2_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_300");
    quick.push("ott+11_50_bd=off:bs=off:drc=off:ep=on:lcm=reverse:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_300");
    quick.push("lrs+2_64_bd=off:bs=off:br=off:cond=on:drc=off:flr=on:fde=none:nwc=1.7:sio=off:spl=sat:ssfp=1000:ssfq=1.4:ssnc=none:urr=on_300");
    quick.push("dis+10_3:1_bs=off:bfnt=on:cond=on:lcm=predicate:nwc=1:nicw=on:sac=on:sio=off:spl=sat:sp=reverse_arity:urr=on:updr=off_300");
    quick.push("dis-1002_5:1_bs=off:cond=fast:drc=off:ep=on:nwc=1.1:sd=4:ss=axioms:sos=on:sio=off:spl=sat:urr=on_300");
    quick.push("ott+3_8:1_bd=off:bs=off:bsr=unit_only:drc=off:fsr=off:nwc=2.5:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.0:ssnc=none:updr=off_300");
    quick.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=1.0:igrp=400:igrpq=1.5:nwc=5:sio=off_300");
    quick.push("dis+10_1024_bs=off:drc=off:flr=on:fsr=off:nwc=1.1:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    quick.push("dis+1_8:1_bs=off:drc=off:lcm=reverse:nwc=1.5:sos=all:sio=off:spl=sat:ssfq=2.0:ssnc=none:sfv=off_300");
    quick.push("dis+2_4:1_bs=unit_only:cond=on:flr=on:lcm=predicate:nwc=1:ssac=none:ssfp=100000:ssfq=2.0:ssnc=all:sfv=off:urr=on_300");
    quick.push("dis-11_1024_bs=off:bfnt=on:lcm=predicate:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:sp=occurrence:urr=on_300");
    quick.push("dis+1011_10_bs=off:drc=off:fsr=off:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_300");
    quick.push("dis+10_10_bs=off:cond=fast:drc=off:gs=on:nwc=1.7:nicw=on:sd=1:ss=axioms:st=3.0_300");
    quick.push("dis+11_4_bs=off:cond=fast:drc=off:ep=on:fde=none:nwc=1.7:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    quick.push("ins+2_5_bs=off:cond=fast:ep=RSTC:flr=on:gs=on:igbrr=0.7:igrr=1/4:igrp=700:igrpq=1.2:igwr=on:lcm=reverse:nwc=2:sio=off:urr=on_300");
    quick.push("ott+1011_3_bs=off:bsr=unit_only:fde=none:nwc=2:sio=off:spl=sat:ssfq=1.0:ssnc=none_300");
    quick.push("lrs+1010_24_bd=off:bs=off:cond=on:drc=off:gsp=input_only:nwc=1.3:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfq=1.0:ssnc=none:updr=off_100");
    quick.push("dis+3_8:1_bs=off:drc=off:fsr=off:fde=none:nwc=10:sio=off:spl=sat:ssfp=10000:ssfq=1.4:ssnc=none:sp=reverse_arity_300");
    quick.push("dis-1_128_bfnt=on:fsr=off:nwc=1.5:nicw=on:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=1.2:ssnc=all_500");
    quick.push("lrs+2_14_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:nwc=1.1:sio=off:spl=sat:ssac=none:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence:updr=off_300");
    quick.push("dis-1002_3:2_bs=off:cond=fast:drc=off:ep=RST:flr=on:fde=none:lcm=predicate:nwc=10:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:updr=off_300");
    quick.push("dis+11_8:1_bs=off:drc=off:ep=RS:nwc=10:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_300");
    quick.push("ott+10_2_bd=off:bs=off:drc=off:flr=on:fsr=off:nwc=1.7:sio=off:spl=sat:ssfq=2.0:ssnc=none_300");
    quick.push("ott+10_28_bd=off:bs=off:drc=off:nwc=1.5:sio=off:spl=sat:ssnc=none_300");
    quick.push("ott-11_2:3_bs=off:drc=off:fsr=off:lcm=predicate:nwc=5:nicw=on:sac=on:sio=off:spo=on:sp=reverse_arity_300");
    quick.push("ott-11_40_bd=off:bs=off:drc=off:flr=on:fsr=off:lcm=predicate:nwc=10:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:updr=off_300");
    quick.push("dis+3_4_bs=off:br=off:cond=on:drc=off:ep=RSTC:nwc=4:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:urr=on_100");
    quick.push("ins+1_3:1_bs=off:cond=on:ep=RSTC:flr=on:gs=on:igbrr=0.8:igrr=1/4:igrp=400:igrpq=1.1:igwr=on:nwc=1.5:sio=off:urr=on_300");
    quick.push("lrs+2_3_bd=off:cond=on:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off:sp=reverse_arity_300");
    quick.push("ott+11_2_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=reverse_arity_300");
    quick.push("dis+11_20_bs=off:cond=fast:fde=none:lcm=reverse:nwc=3:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sp=reverse_arity_300");
    quick.push("dis+11_4_ep=on:fde=none:lcm=reverse:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_300");
    quick.push("ott+1_10_bd=off:bs=off:drc=off:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    quick.push("dis+1011_1024_bs=unit_only:cond=fast:lcm=reverse:nwc=1.1:sos=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_300");
    quick.push("dis+10_1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:sos=on:sio=off:spl=sat:sser=off:ssnc=none_300");
    quick.push("dis+1002_4:1_bs=off:cond=fast:drc=off:ep=RST:nwc=3:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_300");
    quick.push("ott+10_64_bd=off:bs=off:drc=off:flr=on:gsp=input_only:lcm=predicate:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_300");
    quick.push("dis+1011_64_bs=off:drc=off:ep=on:flr=on:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_300");
    quick.push("ott+11_8:1_bs=off:drc=off:flr=on:lcm=predicate:nwc=2:sos=all:sio=off:spo=on:sfv=off_200");
    quick.push("dis-1002_12_bs=off:cond=on:drc=off:ep=on:gsp=input_only:nwc=10:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_300");
    quick.push("dis+1011_1024_bs=unit_only:drc=off:gsp=input_only:nwc=1.1:sio=off:spo=on:urr=on_300");
    quick.push("dis+1_2:1_bs=off:bsr=on:cond=fast:ep=RSTC:gs=on:lcm=predicate:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on_900");
    quick.push("lrs+4_24_bd=off:bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sio=off:spl=sat:ssfp=4000:ssfq=1.2:ssnc=none_600");
    break;

  case Property::HEQ:
    quick.push("lrs+11_2:1_bs=off:cond=on:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_600");
    quick.push("ins+4_12_bs=off:bfnt=on:cond=fast:gsp=input_only:igbrr=0.8:igrr=128/1:igrpq=2.0:nwc=1_300");
    quick.push("ott+11_128_bs=off:drc=off:gsp=input_only:gs=on:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.4:ssnc=none:sp=occurrence_900");
    quick.push("ott+11_16_bfnt=on:cond=fast:nwc=1.1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=1000:ssfq=2.0:sp=reverse_arity_1200");
    quick.push("lrs+2_4_bs=off:cond=fast:drc=off:gsp=input_only:nwc=4:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none_900");
    quick.push("dis+2_8_bd=off:bs=off:ep=RST:fsr=off:lcm=reverse:nwc=10:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_300");
    quick.push("dis+11_5:4_bs=off:cond=fast:drc=off:fde=none:nwc=1:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.2:ssnc=none:urr=on_300");
    quick.push("ins+11_64_bs=off:cond=on:drc=off:fde=none:igbrr=0.2:igrr=1/64:igrp=400:igrpq=1.05:igwr=on:nwc=1.2:sio=off_900");
    quick.push("ott+2_16_bsr=unit_only:br=off:cond=on:flr=on:fsr=off:nwc=5:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.1:ssnc=none:sfv=off:urr=on:updr=off_900");
    quick.push("dis+2_1024_bd=off:bs=off:cond=fast:fsr=off:nwc=1:sio=off:spl=off_900");
    quick.push("lrs+11_3:1_bs=unit_only:drc=off:fde=none:nwc=10:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_600");
    quick.push("dis+10_1024_bd=off:bs=off:cond=on:drc=off:fde=none:nwc=2.5:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_300");
    quick.push("dis+10_2:3_bfnt=on:cond=on:fsr=off:lcm=predicate:nwc=1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.4:sp=reverse_arity_100");
    quick.push("lrs-1_2_bs=off:drc=off:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_300");
    quick.push("dis-1002_3_bs=off:cond=fast:drc=off:nwc=10:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_300");
    quick.push("dis+3_1024_bfnt=on:cond=fast:fsr=off:nwc=5:sio=off:spl=off_100");
    quick.push("ott+11_24_bd=off:bs=off:drc=off:nwc=3:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_300");
    quick.push("ins+4_5:4_bs=off:br=off:cond=on:ep=RSTC:flr=on:fsr=off:igbrr=0.1:igrr=1/16:igrpq=1.0:igwr=on:nwc=2:urr=on_600");
    quick.push("ins+2_64_bs=off:cond=on:drc=off:flr=on:fde=none:igbrr=0.1:igrr=1/16:igrpq=1.3:igwr=on:nwc=1.7:sp=reverse_arity_1200");
    break;

  case Property::PEQ:
    quick.push("lrs+3_40_bs=unit_only:bsr=on:cond=on:drc=off:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on:updr=off_1000");
    quick.push("dis-3_28_bd=off:cond=on:nwc=1.5:sio=off_600");
    quick.push("ott+11_5_bs=off:cond=fast:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_900");
    quick.push("dis+10_64_bd=off:cond=fast:drc=off:nwc=3:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.1:ssnc=none_300");
    quick.push("dis+1_1024_nwc=1.1:sio=off:spl=off:updr=off_400");
    quick.push("ins+1_8:1_bs=off:bfnt=on:igbrr=0.8:igrr=2/1:igrpq=1.2:igwr=on:nwc=3:sos=all:sgo=on_300");
    quick.push("dis-1003_128_drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_300");
    quick.push("lrs+10_2:3_bd=off:cond=on:drc=off:flr=on:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_900");
    quick.push("lrs+3_4_bsr=unit_only:drc=off:fsr=off:nwc=1:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_1500");
    quick.push("dis+2_7_bd=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_600");
    quick.push("ins+1_20_bsr=on:br=off:cond=fast:drc=off:fsr=off:fde=none:igbrr=0.3:igrr=1/32:igrp=4000:igrpq=2.0:igwr=on:lcm=reverse:nwc=1.2:sgo=on:sio=off:sp=occurrence:urr=on_400");
    quick.push("dis+1_2_bd=off:bs=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    quick.push("ott+2_20_bs=off:cond=fast:drc=off:fde=none:nwc=2:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:sp=occurrence_300");
    quick.push("ott+3_4:1_bsr=unit_only:cond=on:drc=off:fsr=off:fde=none:gs=on:nwc=1:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence:urr=on_300");
    quick.push("lrs-1_128_bs=off:cond=fast:ep=on:nwc=1.2:nicw=on:sac=on:sio=off_300");
    quick.push("dis-1004_1024_bs=off:cond=fast:drc=off:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    quick.push("ott+11_2:3_bd=off:drc=off:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.0:ssnc=none:updr=off_300");
    quick.push("ott+3_128_bs=off:drc=off:nwc=1.1:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_600");
    quick.push("dis+11_32_bd=off:bsr=on:fsr=off:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.4:ssnc=none_600");
    quick.push("ins+10_1_bfnt=on:igbrr=0.8:igrp=400:igrpq=2.0:nwc=1.7:sio=off_1500");
    quick.push("lrs+4_14_cond=on:drc=off:flr=on:nwc=3:spl=sat:sser=off:ssnc=none_1800");
    break;

  case Property::EPR:
    quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_2100");
    quick.push("dis-10_5:4_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=1.1:sac=on:sio=off:spo=on_300");
    quick.push("ott+10_1024_bs=off:br=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none:urr=on_300");
    quick.push("ins+3_50_bs=off:br=off:drc=off:igbrr=0.7:igrr=1/128:igrp=100:igrpq=1.05:igwr=on:lcm=predicate:nwc=2:sp=occurrence:urr=on_600");
    quick.push("ins+11_50_bs=off:cond=fast:drc=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=2000:igrpq=1.5:igwr=on:nwc=1.2:urr=on_300");
    quick.push("dis+1_40_bs=unit_only:bsr=unit_only:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none_900");
    quick.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_300");
    quick.push("dis-11_24_bs=off:cond=fast:drc=off:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_300");
    quick.push("ins+2_128_bs=unit_only:drc=off:fsr=off:igbrr=1.0:igrr=128/1:igrp=200:igrpq=2.0:igwr=on:lcm=predicate:nwc=2:sos=on:sio=off:sp=occurrence_300");
    quick.push("ins+3_128_bs=off:br=off:drc=off:igbrr=0.1:igrr=32/1:igrp=2000:igrpq=2.0:igwr=on:nwc=3:sos=all:sio=off:urr=on_300");
    quick.push("ins+3_14_bs=off:cond=on:drc=off:flr=on:igbrr=0.2:igrr=1/128:igrp=200:igrpq=1.2:igwr=on:nwc=1:urr=on_300");
    quick.push("ott+2_20_bs=off:bsr=unit_only:nwc=5:nicw=on:sio=off:spl=sat:ssnc=none_300");
    quick.push("ins-10_14_bs=off:drc=off:fde=none:igbrr=0.9:igrr=1/4:igrp=2000:igrpq=2.0:igwr=on:nwc=1.5:sos=on:sgo=on:sio=off:updr=off_1800");
    break;

  case Property::HNE:
    quick.push("ott+4_6_bs=off:bsr=unit_only:cond=fast:nwc=3:nicw=on:sio=off:spl=sat:ssnc=none_300");
    quick.push("dis+1011_20_bs=off:fsr=off:nwc=2:sio=off:spl=off_300");
    quick.push("ott+11_8_bs=off:bfnt=on:cond=fast:nwc=1:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none_300");
    quick.push("lrs+10_3:2_bs=off:cond=fast:nwc=10:sio=off:spo=on_900");
    quick.push("lrs+1_128_bs=off:cond=on:gs=on:nwc=4:nicw=on:sio=off_900");
    quick.push("dis+11_24_bs=off:cond=fast:nwc=1.3:nicw=on:sac=on:sio=off_300");
    quick.push("dis-11_8:1_bs=off:nwc=2.5:sio=off:spl=off_300");
    quick.push("ott+1003_28_bs=off:cond=on:flr=on:fsr=off:lcm=predicate:nwc=1:nicw=on:sio=off:spl=off_300");
    quick.push("lrs+3_4_bs=off:cond=fast:flr=on:nwc=1:nicw=on:sio=off:spl=sat:ssnc=none_300");
    quick.push("dis+11_2:3_bs=off:cond=fast:fsr=off:nwc=10:sio=off:spl=sat:sser=off:ssnc=none_300");
    quick.push("dis+2_5:4_cond=fast:flr=on:lcm=predicate:nwc=1.5:sio=off:spl=off_600");
    quick.push("lrs+1011_64_bs=unit_only:bsr=unit_only:cond=fast:flr=on:nwc=1:sio=off:spl=sat:ssfq=2.0_1800");
    quick.push("dis+10_4_bs=off:nwc=5:sac=on:sio=off_200");
    quick.push("lrs+11_128_cond=fast:lcm=reverse:nwc=1.5:nicw=on:sio=off:spl=sat:ssfp=100000:ssfq=2.0:ssnc=none_600");
    quick.push("ott+1_8:1_bs=off:cond=on:gs=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssnc=none_300");
    quick.push("lrs+10_8:1_bs=off:cond=fast:flr=on:fsr=off:nwc=2.5:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_900");
    quick.push("dis+1002_64_bs=off:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:nicw=on:sio=off:spo=on_600");
    break;

  case Property::NNE:
    quick.push("dis+1011_12_bs=off:cond=fast:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_900");
    quick.push("ins+11_12_bs=off:bfnt=on:gsp=input_only:igbrr=0.9:igrr=4/1:igrp=400:igrpq=2.0:nwc=4_300");
    quick.push("dis-1010_14_bsr=on:cond=on:nwc=1.5:sio=off:spl=sat:ssfp=100000:ssfq=1.1:ssnc=none_300");
    quick.push("ott+1_20_bfnt=on:cond=on:nwc=3:sac=on:sgo=on:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.4_300");
    quick.push("dis+1011_16_bs=unit_only:cond=on:fsr=off:gsp=input_only:nwc=1.7:sos=all:sgo=on:sio=off:spl=sat:ssfq=2.0:ssnc=all_200");
    quick.push("dis-11_40_bs=off:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssac=none:ssfp=40000:ssfq=2.0:ssnc=none_300");
    quick.push("ott+4_2:1_bs=off:cond=on:lcm=predicate:nwc=10:nicw=on:sac=on:sio=off_300");
    quick.push("dis-1_16_bs=off:bfnt=on:cond=on:lcm=predicate:nwc=1.2:nicw=on:ssfq=1.1:ssnc=none:sp=occurrence_100");
    quick.push("dis+1_28_bs=off:bfnt=on:cond=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.7:sio=off:spl=off:sp=occurrence_300");
    quick.push("ott-3_16_bs=off:gsp=input_only:nwc=2:nicw=on:sos=all:sio=off:spl=sat:ssnc=none_300");
    quick.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.7:igrpq=2.0:nwc=4_500");
    quick.push("dis+11_128_bs=off:cond=fast:flr=on:lcm=reverse:nwc=2:sio=off:spl=sat:ssnc=none_300");
    quick.push("dis+1011_128_bs=off:cond=fast:flr=on:fsr=off:lcm=reverse:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none_300");
    quick.push("ott-1002_1024_cond=on:flr=on:gs=on:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:urr=on_300");
    quick.push("dis+1011_50_bs=off:cond=fast:flr=on:gsp=input_only:nwc=1.3:sos=all_300");
    quick.push("dis+4_12_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_300");
    quick.push("dis+1011_40_bs=off:gsp=input_only:nwc=4:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_300");
    quick.push("ott+1011_5_bs=off:gs=on:nwc=2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssnc=none_100");
    quick.push("dis-2_16_bs=off:cond=fast:flr=on:lcm=predicate:nwc=1:nicw=on:sagn=off:sio=off:spl=sat:ssnc=none_300");
    break;

  case Property::FEQ:
    quick.push("dis-1010_2_bs=off:bsr=on:drc=off:nwc=4:ssac=none:sscc=on:ssfp=100000:ssfq=1.4:ssnc=all_300");
    quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_300");
    quick.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_2700");
    quick.push("ins+10_2:3_bs=unit_only:flr=on:fde=none:igbrr=0.7:igrr=1/32:igrp=200:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.7:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=off_300");
    quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_900");
    quick.push("ott+11_1024_bd=off:bs=off:br=off:cond=fast:drc=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on_300");
    quick.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_2400");
    quick.push("dis+1_3_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=1:ss=included:st=2.0:sagn=off:sio=off:spl=sat:sser=off:ssnc=none_300");
    quick.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_300");
    quick.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_300");
    quick.push("dis+1011_12_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_300");
    quick.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_300");
    quick.push("ott+1_2_bs=off:drc=off:ep=on:nwc=3:nicw=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat:sser=off_300");
    quick.push("dis+2_2:3_bs=off:drc=off:lcm=reverse:nwc=2.5:sos=all:sser=off:ssfp=10000:ssfq=2.0:ssnc=all_300");
    quick.push("ins-1010_3:2_bs=unit_only:drc=off:ep=on:fde=none:igbrr=0.5:igrr=1/128:igrpq=1.3:igwr=on:nwc=1.7_300");
    quick.push("tab+10_1_ep=RST:gsp=input_only:sd=4:ss=axioms:st=2.0:sio=off:tbsr=off:tgawr=1/32:tglr=1/5:tipr=off:tlawr=8/1_100");
    quick.push("ins+3_10_bs=off:drc=off:fde=none:igbrr=0.8:igrr=1/128:igrp=100:igrpq=1.0:igwr=on:nwc=2.5:sio=off:spl=off_300");
    quick.push("ott+2_12_bd=off:drc=off:lcm=reverse:nwc=2:sio=off:spo=on_600");
    quick.push("ins+10_2:3_bs=off:cond=fast:ep=RSTC:igbrr=0.8:igrr=1/2:igrp=100:igrpq=1.0:igwr=on:nwc=2:sos=all:sio=off:urr=on_300");
    quick.push("dis-1010_2:3_bs=off:drc=off:nwc=3:sd=2:ss=axioms:st=1.5:sac=on:sio=off_300");
    quick.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_300");
    quick.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=1.5:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_300");
    quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_300");
    quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_300");
    quick.push("dis-1002_1024_bs=off:cond=on:drc=off:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    quick.push("dis+1_10_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=7:ss=axioms:st=1.5:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_400");
    quick.push("ott+1011_2_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:updr=off_300");
    quick.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_300");
    quick.push("dis-4_4:1_bs=off:ep=RST:gsp=input_only:gs=on:nwc=5:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off:sp=occurrence_300");
    quick.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_600");
    quick.push("ott+10_2:3_bs=off:drc=off:gs=on:nwc=1.5:sd=2:ss=axioms:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:urr=on_300");
    quick.push("lrs-1_10_bs=off:cond=fast:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.4:ssnc=none_200");
    quick.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_300");
    quick.push("dis+1010_1_bs=off:cond=fast:ep=on:flr=on:fde=none:gsp=input_only:nwc=1.5:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_500");
    quick.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_300");
    quick.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat_300");
    quick.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_300");
    quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=2.5:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_300");
    quick.push("ott+4_32_bs=off:flr=on:nwc=4:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_300");
    quick.push("ott+1_5_bs=off:drc=off:nwc=4:sio=off:sp=occurrence_300");
    quick.push("dis+10_8:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.1:nicw=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_300");
    quick.push("lrs+1_6_bs=off:drc=off:nwc=3:sos=all:sio=off:spl=sat:ssac=none:sscc=on:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_3600");
    quick.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_300");
    quick.push("dis+11_1024_bsr=unit_only:cond=fast:nwc=1.3:sio=off:spl=off_600");
    quick.push("ott+2_3:1_bs=off:br=off:drc=off:nwc=1.1:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:urr=on_300");
    quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1.1:sos=all:sio=off:spo=on:sp=occurrence_300");
    quick.push("dis+10_8:1_bs=off:br=off:cond=on:drc=off:ep=RST:fsr=off:nwc=1.3:sd=4:ss=axioms:st=3.0:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_300");
    quick.push("ott+1011_64_bs=off:cond=fast:drc=off:gsp=input_only:nwc=1:ss=axioms:st=1.2:spl=sat:sser=off:urr=on_300");
    quick.push("dis+2_1_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off_300");
    quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_300");
    quick.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    quick.push("ott-1002_5:1_bs=off:bsr=on:cond=fast:drc=off:gsp=input_only:nwc=2:sos=all:sio=off:spo=on:urr=on_300");
    quick.push("dis+4_40_cond=on:drc=off:flr=on:lcm=predicate:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_300");
    quick.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_300");
    quick.push("lrs-1010_7_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:sgo=on:spo=on:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_300");
    quick.push("ott+10_8:1_bd=off:bs=off:drc=off:fde=none:gsp=input_only:nwc=1:sd=3:ss=axioms:sos=on:sio=off:spl=off:urr=on_300");
    quick.push("ott+10_3:1_bsr=unit_only:cond=fast:fsr=off:lcm=reverse:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:urr=on:updr=off_300");
    quick.push("ins-1003_7_bs=off:ep=RSTC:flr=on:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=1.1:igwr=on:nwc=1.3:sos=on:sio=off:spl=off_300");
    quick.push("ott+11_24_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity_400");
    quick.push("dis+2_4_bs=off:br=off:drc=off:nwc=1.2:sd=2:ss=axioms:st=2.0:sio=off:urr=on_300");
    quick.push("dis+11_4_bs=off:drc=off:ep=on:gsp=input_only:nwc=5:sgt=15:ss=axioms:sos=on:spl=sat_300");
    quick.push("ott-1010_4:1_bs=off:bsr=on:cond=on:drc=off:fde=none:gsp=input_only:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:ssfq=1.0:ssnc=none_300");
    quick.push("dis+11_10_bs=off:lcm=predicate:nwc=1.3:sio=off_300");
    quick.push("dis+10_2_bs=off:cond=on:drc=off:fde=none:lcm=predicate:nwc=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sp=reverse_arity_300");
    quick.push("dis+2_8:1_bd=off:bs=off:bsr=on:drc=off:lcm=reverse:nwc=4:nicw=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sser=off:sp=reverse_arity_300");
    quick.push("ott+11_3:2_bs=off:cond=on:drc=off:ep=RSTC:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sagn=off:sio=off:spl=sat:urr=on_300");
    quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=10:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_300");
    quick.push("ott+10_1024_bs=off:cond=on:drc=off:gsp=input_only:nwc=1.2:sio=off:spl=sat:ssfp=10000:ssnc=none:updr=off_600");
    quick.push("lrs+10_20_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=2.5:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_800");
    quick.push("ott-1010_3_bd=off:bs=off:bsr=unit_only:drc=off:fde=none:nwc=1:nicw=on:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_400");
    quick.push("dis+1010_2:3_bs=off:bsr=unit_only:drc=off:ep=on:fsr=off:fde=none:lcm=predicate:nwc=1.5:sos=on:sac=on:sio=off:spo=on:sp=occurrence_600");
    quick.push("dis+1_24_bs=off:drc=off:ep=on:lcm=predicate:nwc=3:nicw=on:ss=included:st=5.0:sos=on:sagn=off:spl=sat:sser=off:ssnc=none_300");
    quick.push("dis-1_3_bsr=unit_only:drc=off:lcm=reverse:nwc=4:sos=all:sac=on:sgo=on:sio=off:spo=on:sp=reverse_arity_300");
    quick.push("dis-1010_50_bs=unit_only:bsr=on:drc=off:lcm=predicate:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_900");
    quick.push("ott+10_8:1_bd=off:bs=off:cond=fast:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_300");
    quick.push("dis+4_2:1_bs=off:br=off:drc=off:fsr=off:nwc=1:sos=all:sio=off:spl=sat:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_200");
    quick.push("ott+1011_5:4_bs=off:cond=fast:drc=off:flr=on:fsr=off:nwc=2:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_300");
    quick.push("ott+11_1_bs=off:drc=off:ep=RS:flr=on:fde=none:nwc=4:sos=all:sgo=on:sio=off_300");
    quick.push("dis+1010_32_cond=fast:drc=off:ep=RS:fsr=off:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    quick.push("ott+1011_10_bs=off:cond=on:drc=off:nwc=1.5:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_200");
    quick.push("lrs-1004_32_bs=off:br=off:cond=fast:drc=off:flr=on:gs=on:lcm=reverse:nwc=1:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:urr=on_300");
    quick.push("dis-1010_2_bd=off:bs=off:cond=fast:drc=off:nwc=5:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:sp=occurrence_300");
    quick.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_100");
    quick.push("lrs-11_6_bs=off:bsr=unit_only:drc=off:gsp=input_only:lcm=predicate:nwc=10:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on:updr=off_300");
    quick.push("dis+1011_28_cond=on:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:updr=off_100");
    quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none_300");
    quick.push("ott+2_8:1_bs=off:bsr=on:drc=off:fde=none:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_300");
    quick.push("ott-3_10_bs=off:br=off:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_300");
    quick.push("dis+1003_1024_bs=off:drc=off:fsr=off:nwc=1.7:nicw=on:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none_300");
    quick.push("ott+4_40_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sd=2:sgt=10:ss=axioms:st=3.0:sos=on:spl=sat:sser=off:updr=off_300");
    quick.push("dis+1011_4_bs=off:drc=off:nwc=4:sgo=on_100");
    quick.push("dis+2_4_bs=off:drc=off:ep=RST:flr=on:lcm=reverse:nwc=1.5:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sp=reverse_arity_300");
    quick.push("ott+10_24_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:sser=off:ssnc=none_300");
    quick.push("dis+1_128_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=10:sd=2:ss=included:st=2.0:sagn=off:sio=off:spl=sat:ssnc=none_300");
    quick.push("ins+2_4:1_bs=off:cond=on:ep=RSTC:gs=on:igbrr=0.4:igrr=1/128:igrpq=1.05:igwr=on:nwc=5:sos=on:sio=off:urr=on_300");
    quick.push("dis-1002_8:1_bs=off:cond=on:drc=off:flr=on:gs=on:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_300");
    quick.push("dis+11_28_bd=off:bs=off:cond=fast:drc=off:ep=on:fsr=off:lcm=reverse:nwc=5:sos=on:sio=off:spl=sat:ssfq=1.0:ssnc=none:sp=occurrence_300");
    quick.push("dis+1011_1_bs=off:drc=off:nwc=1.2:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none_300");
    quick.push("ott+11_2:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:fde=none:gsp=input_only:lcm=reverse:nwc=2.5:sio=off:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_300");
    quick.push("ott+2_20_cond=fast:drc=off:flr=on:lcm=reverse:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity:updr=off_300");
    quick.push("lrs-1004_12_bs=off:bsr=unit_only:drc=off:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.4:ssnc=none_600");
    quick.push("lrs-1002_3_bd=off:bs=off:drc=off:ep=on:nwc=1.7:sos=on:sac=on:sio=off_1500");
    quick.push("dis+1010_40_bs=off:drc=off:ep=RS:nwc=1:sio=off:spo=on:spl=sat:ssnc=none_300");
    quick.push("dis+1011_3_bs=off:cond=fast:drc=off:fsr=off:fde=none:gs=on:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_300");
    quick.push("lrs+1_5_bs=off:cond=fast:drc=off:flr=on:nwc=10:sac=on:sio=off:urr=on_300");
    quick.push("dis+2_128_bs=off:drc=off:lcm=reverse:nwc=1.3:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_300");
    quick.push("dis+1_14_bd=off:bs=off:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=4:sos=on:sio=off:spo=on:sp=reverse_arity_200");
    quick.push("dis+11_32_bs=off:nwc=1.1:sio=off:spl=off:updr=off_300");
    quick.push("dis+2_20_drc=off:ep=RST:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none:sp=occurrence_300");
    quick.push("dis+1011_2_bs=off:drc=off:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:sfv=off:urr=on_300");
    quick.push("ott+1010_5_bd=off:bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_300");
    quick.push("lrs+1_64_bs=off:drc=off:gsp=input_only:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:updr=off_600");
    quick.push("dis-1010_64_bs=off:drc=off:nwc=2.5:nicw=on:sgo=on:sio=off:spo=on:sser=off:ssfp=40000:ssfq=1.0:ssnc=none_300");
    quick.push("lrs-1_5:1_bs=off:cond=fast:drc=off:nwc=4:sio=off:spl=sat:ssfp=4000:ssfq=2.0:ssnc=none_1200");
    quick.push("dis-1010_3:1_bd=off:bs=off:cond=fast:gs=on:lcm=reverse:nwc=1.1:sac=on_900");
    quick.push("ott+4_14_bs=unit_only:cond=fast:drc=off:nwc=1.2:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_600");
    quick.push("dis+2_16_bs=off:drc=off:ep=on:gsp=input_only:lcm=reverse:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none_600");
    quick.push("ott+11_10_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_1200");
    quick.push("lrs-11_12_bs=off:drc=off:fde=none:gsp=input_only:gs=on:lcm=predicate:nwc=4:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sfv=off:sp=occurrence:urr=on:updr=off_3000");
    quick.push("ott+10_3:1_bd=off:drc=off:lcm=reverse:nwc=10:nicw=on:sio=off:spo=on:ssac=none:sscc=on:sser=off:ssfp=1000:ssfq=1.2:ssnc=all_900");
    quick.push("ott+4_64_bd=off:bs=off:br=off:cond=on:drc=off:fsr=off:gs=on:nwc=1.7:sos=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.0:ssnc=none:urr=on_900");
    break;

  case Property::FNE:
    quick.push("dis-1010_12_bs=off:bsr=unit_only:cond=on:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence:urr=on_300");
    quick.push("dis+11_50_bs=off:cond=on:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssnc=none:urr=on_300");
    quick.push("dis+1011_3_bs=off:cond=fast:flr=on:gsp=input_only:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    quick.push("ott+1_7_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=1.5:spo=on_600");
    quick.push("dis+11_50_bs=off:bsr=on:cond=on:lcm=reverse:nwc=1:sio=off:spl=sat:sser=off:ssnc=none:updr=off_300");
    quick.push("ott+1011_50_bs=off:gs=on:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none:urr=on:updr=off_300");
    quick.push("dis+4_128_bs=off:cond=on:flr=on:nwc=1.2:sos=on:sac=on_300");
    quick.push("dis+1004_10_bs=off:nwc=1.5:sagn=off:sio=off:spl=sat:ssnc=none_900");
    quick.push("dis+4_10_bs=off:cond=fast:flr=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence:updr=off_300");
    quick.push("tab+10_1_spl=off:tbsr=off:tfsr=off:tgawr=3/1:tglr=1/20:tipr=off:tlawr=8/1_300");
    quick.push("ins+10_1_gsp=input_only:igbrr=0.9:igrp=100:igrpq=1.1:nwc=2_300");
    quick.push("ott-1002_3_bs=unit_only:bsr=unit_only:cond=on:gsp=input_only:nwc=1.2:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:urr=on_300");
    quick.push("ott+1_40_bsr=unit_only:cond=fast:gs=on:nwc=1.5:sio=off:spl=sat:ssnc=none:urr=on_300");
    quick.push("dis+3_6_flr=on:fsr=off:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:sp=occurrence_300");
    quick.push("dis+1010_8_bs=off:nwc=1.1:sio=off_300");
    quick.push("ott-1010_128_bs=off:cond=on:lcm=predicate:nwc=1.3:urr=on_600");
    quick.push("ott+10_1024_bs=off:fsr=off:nwc=1.5:nicw=on:sio=off:spl=off_1200");
    quick.push("lrs+1002_128_bs=off:cond=on:flr=on:lcm=reverse:nwc=1:spo=on_1200");
    break;

  case Property::UEQ:
    quick.push("lrs+10_8_bs=off:cond=fast:drc=off:fsr=off:gsp=input_only:gs=on:nwc=2:urr=on_1200");
    quick.push("lrs+10_14_drc=off:fde=none:nwc=2.5_1800");
    quick.push("ott+10_50_bs=off:drc=off:fsr=off:fde=none:nwc=1_300");
    quick.push("lrs+10_1024_nwc=4_1500");
    quick.push("ott+10_8:1_bs=off:drc=off:fsr=off:nwc=1.7_600");
    quick.push("ott+10_7_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.5_600");
    quick.push("dis+4_5:1_bs=off:br=off:ep=RSTC:fsr=off:nwc=3:sos=all:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_300");
    quick.push("ott+2_10_bs=off:drc=off:ep=on:fsr=off:fde=none:gsp=input_only:nwc=1.7:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfq=1.1:ssnc=none_200");
    quick.push("lrs+10_20_bd=off:bs=unit_only:drc=off:fsr=off:nwc=1.2_900");
    quick.push("dis+2_14_bs=off:br=off:ep=RSTC:flr=on:fsr=off:nwc=2.5:nicw=on:sos=all:sio=off:spl=sat:ssnc=none:urr=on_100");
    quick.push("lrs+10_5:1_nwc=1_900");
    quick.push("lrs+10_2:3_bs=unit_only:cond=on:drc=off:ep=on:fsr=off:gs=on:nwc=1:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=occurrence_200");
    quick.push("ott+10_20_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.3:sp=occurrence_600");
    quick.push("ott+10_8:1_bs=off:bsr=on:nwc=4_300");
    quick.push("ott+10_64_drc=off:nwc=1.1_400");
    quick.push("ott+10_2:1_drc=off:nwc=5_1200");
    quick.push("ott+10_20_bd=off:drc=off:nwc=2:sp=occurrence_600");
    quick.push("ott+10_2:3_drc=off:nwc=2_600");
    quick.push("ott+10_5_bd=off:drc=off:nwc=2.5_900");
    quick.push("ott+10_2_fde=none:nwc=2.5:sp=reverse_arity_900");
    quick.push("ott+10_8:1_nwc=3:sfv=off_900");
    break;
  }
}

void Schedules::getCasc2014EprSchedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  unsigned prop = property.props();
  unsigned atoms = property.atoms();

  if (prop == 131072) {
    quick.push("ott+10_1024_bs=off:br=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none:urr=on_158");
    quick.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_162");
  }
  else if (atoms > 2000) {
    quick.push("ott+2_20_bs=off:bsr=unit_only:nwc=5:nicw=on:sio=off:spl=sat:ssnc=none_119");
    quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_1935");
  }
  else if (atoms > 1300) {
    quick.push("dis-11_24_bs=off:cond=fast:drc=off:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_1");
    quick.push("ott+10_1024_bs=off:br=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none:urr=on_148");
    quick.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_171");
    quick.push("ins+3_128_bs=off:br=off:drc=off:igbrr=0.1:igrr=32/1:igrp=2000:igrpq=2.0:igwr=on:nwc=3:sos=all:sio=off:urr=on_262");
    quick.push("ins+2_128_bs=unit_only:drc=off:fsr=off:igbrr=1.0:igrr=128/1:igrp=200:igrpq=2.0:igwr=on:lcm=predicate:nwc=2:sos=on:sio=off:sp=occurrence_277");
    quick.push("dis+1_40_bs=unit_only:bsr=unit_only:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none_560");
  }
  else if (atoms > 450) {
    quick.push("dis-10_5:4_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=1.1:sac=on:sio=off:spo=on_5");
    quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_22");
    quick.push("ins+11_50_bs=off:cond=fast:drc=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=2000:igrpq=1.5:igwr=on:nwc=1.2:urr=on_56");
    quick.push("ins+3_50_bs=off:br=off:drc=off:igbrr=0.7:igrr=1/128:igrp=100:igrpq=1.05:igwr=on:lcm=predicate:nwc=2:sp=occurrence:urr=on_501");
    quick.push("ins+10_1_igbrr=0.4:igrp=200:igrpq=1.5:nwc=1.1:sio=off_561");
  }
  else {
    quick.push("ins+11_50_bs=off:cond=fast:drc=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=2000:igrpq=1.5:igwr=on:nwc=1.2:urr=on_12");
    quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_13");
    quick.push("ins+3_14_bs=off:cond=on:drc=off:flr=on:igbrr=0.2:igrr=1/128:igrp=200:igrpq=1.2:igwr=on:nwc=1:urr=on_43");
    quick.push("ott+1_128_bs=off:cond=fast:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none:urr=on_1");
    quick.push("dis+4_10_bs=off:drc=off:fsr=off:nwc=5:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.0:ssnc=none_482");
    quick.push("ins-10_14_bs=off:drc=off:fde=none:igbrr=0.9:igrr=1/4:igrp=2000:igrpq=2.0:igwr=on:nwc=1.5:sos=on:sgo=on:sio=off:updr=off_1506");
  }
}

void Schedules::getCascSat2014Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category();
  unsigned prop = property.props();
  unsigned atoms = property.atoms();

  switch (cat) {
  case Property::NEQ:
    quick.push("dis+10_5_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sgo=on:sio=off:sp=occurrence_1");
    quick.push("dis+1_40_bs=off:ep=RSTC:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none:urr=on_39");
    quick.push("dis-1_24_bs=off:bfnt=on:lcm=predicate:nwc=10:sgo=on:sp=occurrence_21");
    quick.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=1.0:igrp=400:igrpq=1.5:nwc=5:sio=off_124");
    quick.push("dis+10_3:1_bs=off:bfnt=on:cond=on:lcm=predicate:nwc=1:nicw=on:sac=on:sio=off:spl=sat:sp=reverse_arity:urr=on:updr=off_196");
    quick.push("dis-1_128_bfnt=on:fsr=off:nwc=1.5:nicw=on:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=1.2:ssnc=all_207");
    quick.push("dis-11_10_bfnt=on:cond=fast:lcm=predicate:nwc=1.2:sio=off:spl=off:sp=occurrence_464");
    quick.push("dis+1_2:1_bs=off:bsr=on:cond=fast:ep=RSTC:gs=on:lcm=predicate:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on_554");
    quick.push("ins-1004_5:1_bs=off:bfnt=on:cond=on:flr=on:igbrr=0.3:igrr=64/1:igrp=100:igrpq=1.2:igwr=on:nwc=1.3:sgo=on:sio=off_22");
    break;

  case Property::HEQ:
    quick.push("ins+4_12_bs=off:bfnt=on:cond=fast:gsp=input_only:igbrr=0.8:igrr=128/1:igrpq=2.0:nwc=1_23");
    quick.push("dis+3_1024_bfnt=on:cond=fast:fsr=off:nwc=5:sio=off:spl=off_55");
    quick.push("dis+10_2:3_bfnt=on:cond=on:fsr=off:lcm=predicate:nwc=1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.4:sp=reverse_arity_40");
    quick.push("ott+11_16_bfnt=on:cond=fast:nwc=1.1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=1000:ssfq=2.0:sp=reverse_arity_850");
    quick.push("dis+2_1024_bd=off:bs=off:cond=fast:fsr=off:nwc=1:sio=off:spl=off_786");
    break;

  case Property::PEQ:
    if (prop == 0) {
      quick.push("dis+10_1024_bs=unit_only:cond=on:drc=off:nwc=1:sio=off:spl=sat:ssfq=1.0:ssnc=none_1");
      quick.push("ins+1_8:1_bs=off:bfnt=on:igbrr=0.8:igrr=2/1:igrpq=1.2:igwr=on:nwc=3:sos=all:sgo=on_3");
      quick.push("ins+10_1_bfnt=on:igbrr=0.8:igrp=400:igrpq=2.0:nwc=1.7:sio=off_1284");
    }
    else {
      quick.push("dis+4_12_bs=off:cond=fast:nwc=1.1:nicw=on:sio=off:spl=sat:ssnc=none_2");
      quick.push("lrs-1_128_bs=off:cond=fast:ep=on:nwc=1.2:nicw=on:stl=30:sac=on:sio=off_6");
      quick.push("dis-3_28_bd=off:cond=on:nwc=1.5:sio=off_340");
      quick.push("ins+1_8:1_bs=off:bfnt=on:igbrr=0.8:igrr=2/1:igrpq=1.2:igwr=on:nwc=3:sos=all:sgo=on_92");
      quick.push("dis+11_32_bd=off:bsr=on:fsr=off:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.4:ssnc=none_467");
      quick.push("dis+10_1024_bs=unit_only:cond=on:drc=off:nwc=1:sio=off:spl=sat:ssfq=1.0:ssnc=none_7");
    }
    break;

  case Property::HNE:
    quick.push("dis+10_4_bs=off:nwc=5:sac=on:sio=off_1");
    quick.push("ott+11_8_bs=off:bfnt=on:cond=fast:nwc=1:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none_1");
    quick.push("dis+11_24_bs=off:cond=fast:nwc=1.3:nicw=on:sac=on:sio=off_4");
    quick.push("dis-11_8:1_bs=off:nwc=2.5:sio=off:spl=off_23");
    break;

  case Property::NNE:
    quick.push("dis-11_40_bs=off:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssac=none:ssfp=40000:ssfq=2.0:ssnc=none_1");
    quick.push("ott+11_1024_bs=off:cond=on:gsp=input_only:lcm=predicate:nwc=1.3:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=occurrence_1");
    quick.push("ins+11_12_bs=off:bfnt=on:gsp=input_only:igbrr=0.9:igrr=4/1:igrp=400:igrpq=2.0:nwc=4_144");
    quick.push("ott+1_20_bfnt=on:cond=on:nwc=3:sac=on:sgo=on:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.4_87");
    quick.push("ott+4_2:1_bs=off:cond=on:lcm=predicate:nwc=10:nicw=on:sac=on:sio=off_70");
    quick.push("dis-1_16_bs=off:bfnt=on:cond=on:lcm=predicate:nwc=1.2:nicw=on:ssfq=1.1:ssnc=none:sp=occurrence_47");
    quick.push("dis+1_28_bs=off:bfnt=on:cond=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.7:sio=off:spl=off:sp=occurrence_152");
    quick.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.7:igrpq=2.0:nwc=4_238");
    break;

  case Property::FEQ:
    if (atoms >= 180) {
      quick.push("ott+11_8:1_bfnt=on:cond=on:fde=none:lcm=predicate:nwc=1.2:nicw=on:sio=off:spl=sat:sp=occurrence:urr=on:updr=off_1");
      quick.push("dis+11_32_bs=off:nwc=1.1:sio=off:spl=off:updr=off_2");
      quick.push("dis-2_2:1_bs=unit_only:bsr=unit_only:bfnt=on:lcm=predicate:nwc=5:sp=reverse_arity:updr=off_217");
      quick.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.9:igrp=1400:igrpq=1.5:nwc=1.2_547");
      quick.push("dis+1_40_bfnt=on:fsr=off:nwc=1.7:sio=off:spl=off:urr=on:updr=off_12");
      quick.push("dis+10_8:1_bs=off:nwc=4:sio=off:spl=off_48");
    }
    else if (atoms >= 160) {
      quick.push("dis+1_40_bfnt=on:fsr=off:nwc=1.7:sio=off:spl=off:urr=on:updr=off_357");
      quick.push("ott-3_2_bfnt=on:cond=fast:fde=none:nwc=5:sgo=on:updr=off_1581");
    }
    else if (atoms >= 130) {
      quick.push("ins+2_32_bs=off:bfnt=on:cond=fast:flr=on:gsp=input_only:igbrr=0.4:igrr=16/1:igrp=400:igrpq=1.3:igwr=on:nwc=1.2:sos=all_63");
      quick.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.5:igrp=1400:igrpq=1.5:nwc=1:updr=off_546");
      quick.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.8:igrp=2000:igrpq=2.0:nwc=1:sio=off_330");
    }
    else {
      quick.push("lrs-1002_3_bd=off:bs=off:drc=off:ep=on:nwc=1.7:stl=150:sos=on:sac=on:sio=off_1");
      quick.push("dis+11_10_bs=off:lcm=predicate:nwc=1.3:sio=off_3");
      quick.push("dis+10_8:1_bs=off:nwc=4:sio=off:spl=off_1");
      quick.push("ott+11_8:1_bfnt=on:cond=on:fde=none:lcm=predicate:nwc=1.2:nicw=on:sio=off:spl=sat:sp=occurrence:urr=on:updr=off_13");
      quick.push("dis-11_1024_bs=off:bfnt=on:cond=on:nwc=1:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0_7");
      quick.push("ins+10_1_bfnt=on:igbrr=0.4:igrpq=1.3:nwc=10:sio=off_39");
      quick.push("ins+1_5:4_bs=off:bfnt=on:cond=on:ep=RSTC:igbrr=0.5:igrr=128/1:igrp=200:igrpq=2.0:igwr=on:nwc=1.3:sos=on:sio=off_39");
      quick.push("ins+10_1_bfnt=on:igbrr=0.8:igrpq=2.0:nwc=2:sio=off_370");
      quick.push("ott+1_1024_bfnt=on:nwc=1.2:nicw=on:sgo=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.0_1173");
    }
    break;

  case Property::FNE:
    if (atoms > 10000) {
      quick.push("ins+11_3:1_bs=unit_only:br=off:gsp=input_only:igbrr=0.6:igrr=1/2:igrp=100:igrpq=2.0:igwr=on:nwc=1.5:sio=off:spl=off:urr=on_68");
      quick.push("ott+1_2:1_bs=off:fsr=off:nwc=4:nicw=on:spl=sat:ssfp=4000:ssfq=1.0:ssnc=all:updr=off_1349");
    }
    else if (atoms > 3000) {
      quick.push("dis-11_1024_bs=off:cond=fast:gsp=input_only:nwc=3:sio=off:spl=off_3");
      quick.push("ott+1_128_bs=off:bfnt=on:fsr=off:nwc=1:nicw=on:sac=on:sgo=on:sio=off:updr=off_10");
      quick.push("ins+11_3:1_bs=unit_only:br=off:gsp=input_only:igbrr=0.6:igrr=1/2:igrp=100:igrpq=2.0:igwr=on:nwc=1.5:sio=off:spl=off:urr=on_14");
      quick.push("dis+11_3:2_cond=fast:nwc=5:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none_9");
      quick.push("dis+2_4:1_bs=off:fsr=off:gsp=input_only:nwc=2:sio=off:spl=off_1779");
    }
    else if (atoms > 150) {
      quick.push("dis+11_3:2_cond=fast:nwc=5:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none_2");
      quick.push("ott+1_128_bs=unit_only:bfnt=on:cond=fast:fsr=off:lcm=predicate:nwc=1.2:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_55");
      quick.push("ott+4_8:1_bs=off:cond=on:lcm=predicate:nwc=1.7:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:sp=occurrence_3");
      quick.push("dis-11_1024_bs=off:cond=fast:gsp=input_only:nwc=3:sio=off:spl=off_11");
      quick.push("dis-11_2:3_bs=off:bfnt=on:cond=fast:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.1:ssnc=none:sp=occurrence_1");
      quick.push("ott-11_32_bs=off:cond=on:lcm=predicate:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity_1");
      quick.push("dis+2_1024_bs=off:cond=on:fsr=off:gsp=input_only:nwc=2:sio=off:spl=sat_73");
      quick.push("dis+11_28_bs=off:cond=fast:lcm=predicate:nwc=1.5:sac=on:sp=occurrence:urr=on_63");
      quick.push("ins+11_3:1_bs=unit_only:br=off:gsp=input_only:igbrr=0.6:igrr=1/2:igrp=100:igrpq=2.0:igwr=on:nwc=1.5:sio=off:spl=off:urr=on_1030");
      quick.push("ott+1_1024_bs=off:cond=fast:lcm=predicate:nwc=2.5:sac=on:sio=off:spo=on_251");
      quick.push("dis+2_4:1_bs=off:bfnt=on:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=10000:ssfq=1.0:ssnc=none:sp=occurrence_104");
    }
    else {
      quick.push("dis+11_28_bs=off:cond=fast:lcm=predicate:nwc=1.5:sac=on:sp=occurrence:urr=on_1");
      quick.push("ott+3_8:1_bs=off:cond=on:nwc=1.3:sio=off_1");
      quick.push("dis+1_8_bs=off:bfnt=on:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:sp=occurrence_1");
      quick.push("ott+1_7_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=1.5:spo=on_1");
      quick.push("ins+4_64_bs=off:bfnt=on:br=off:igbrr=0.6:igrr=1/128:igrp=200:igrpq=2.0:igwr=on:nwc=2:sos=all:urr=on:updr=off_182");
      quick.push("ins-3_64_bsr=on:br=off:cond=fast:flr=on:igbrr=0.6:igrr=4/1:igrp=4000:igrpq=2.0:igwr=on:nwc=2:sos=all:sgo=on:urr=on:updr=off_319");
      quick.push("ott+10_40_bs=off:fsr=off:nwc=10:sio=off:spo=on_86");
      quick.push("ins+1_5_bs=off:br=off:cond=on:igbrr=0.7:igrr=1/4:igrp=1400:igrpq=2.0:igwr=on:nwc=5:sos=all:urr=on_623");
      quick.push("ott+1_1024_bs=off:cond=fast:lcm=predicate:nwc=2.5:sac=on:sio=off:spo=on_1");
    }
    break;

  case Property::EPR:
    if (atoms > 2200) {
      quick.push("ott+2_20_bs=off:bsr=unit_only:nwc=5:nicw=on:sio=off:spl=sat:ssnc=none_119");
      quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_1935");
    }
    else {
      quick.push("dis-11_24_bs=off:cond=fast:drc=off:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_1");
      quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_22");
      quick.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_171");
      quick.push("dis+1_40_bs=unit_only:bsr=unit_only:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none_560");
    }
    break;

  case Property::UEQ:
    quick.push("ott+11_20_bs=off:bfnt=on:cond=fast:fsr=off:lcm=predicate:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.1:ssnc=none:sp=reverse_arity_1");
    quick.push("dis+10_12_bs=off:bsr=unit_only:fsr=off:nwc=2_1");
    quick.push("ins+4_8_bs=off:bfnt=on:br=off:cond=fast:flr=on:fsr=off:igbrr=0.1:igrr=1/128:igrp=100:igrpq=2.0:igwr=on:nwc=1.3:sos=all:urr=on_1");
    quick.push("ins+3_2_bs=off:bfnt=on:br=off:cond=on:fsr=off:fde=none:igbrr=0.9:igrr=1/64:igrpq=2.0:igwr=on:nwc=10:sio=off:urr=on_900");
    break;
  }

  switch (cat) {
  case Property::NEQ:
  case Property::HEQ:
  case Property::PEQ:
  case Property::EPR:
  case Property::HNE:
  case Property::NNE:
  case Property::UEQ:
    fallback.push("ins+11_12_bs=off:bfnt=on:gsp=input_only:igbrr=0.9:igrr=4/1:igrp=400:igrpq=2.0:nwc=4_300");
    fallback.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_2100");
    fallback.push("dis+10_5_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sgo=on:sio=off:sp=occurrence_100");
    fallback.push("ins+4_12_bs=off:bfnt=on:cond=fast:gsp=input_only:igbrr=0.8:igrr=128/1:igrpq=2.0:nwc=1_300");
    fallback.push("dis-3_28_bd=off:cond=on:nwc=1.5:sio=off_600");
    fallback.push("dis+1_40_bs=unit_only:bsr=unit_only:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none_900");
    fallback.push("ins+3_2_bs=off:bfnt=on:br=off:cond=on:fsr=off:fde=none:igbrr=0.9:igrr=1/64:igrpq=2.0:igwr=on:nwc=10:sio=off:urr=on_900");
    fallback.push("ott+11_8_bs=off:bfnt=on:cond=fast:nwc=1:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none_300");
    fallback.push("lrs-1_128_bs=off:cond=fast:ep=on:nwc=1.2:nicw=on:sac=on:sio=off_300");
    fallback.push("dis-11_10_bfnt=on:cond=fast:lcm=predicate:nwc=1.2:sio=off:spl=off:sp=occurrence_600");
    fallback.push("dis+1_1024_nwc=1.1:sio=off:spl=off:updr=off_400");
    fallback.push("ins+1_8:1_bs=off:bfnt=on:igbrr=0.8:igrr=2/1:igrpq=1.2:igwr=on:nwc=3:sos=all:sgo=on_300");
    fallback.push("ott+11_16_bfnt=on:cond=fast:nwc=1.1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=1000:ssfq=2.0:sp=reverse_arity_1200");
    fallback.push("ott+1_20_bfnt=on:cond=on:nwc=3:sac=on:sgo=on:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.4_300");
    fallback.push("dis+11_24_bs=off:cond=fast:nwc=1.3:nicw=on:sac=on:sio=off_300");
    fallback.push("dis+1_40_bs=off:ep=RSTC:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none:urr=on_300");
    fallback.push("dis-11_40_bs=off:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssac=none:ssfp=40000:ssfq=2.0:ssnc=none_300");
    fallback.push("dis+10_12_bs=off:bsr=unit_only:fsr=off:nwc=2_300");
    fallback.push("ott+4_2:1_bs=off:cond=on:lcm=predicate:nwc=10:nicw=on:sac=on:sio=off_300");
    fallback.push("dis-11_8:1_bs=off:nwc=2.5:sio=off:spl=off_300");
    fallback.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.7:igrpq=2.0:nwc=4_500");
    fallback.push("dis+1_28_bs=off:bfnt=on:cond=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.7:sio=off:spl=off:sp=occurrence_300");
    fallback.push("dis+10_3:1_bs=off:bfnt=on:cond=on:lcm=predicate:nwc=1:nicw=on:sac=on:sio=off:spl=sat:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=1.0:igrp=400:igrpq=1.5:nwc=5:sio=off_300");
    fallback.push("dis-1_16_bs=off:bfnt=on:cond=on:lcm=predicate:nwc=1.2:nicw=on:ssfq=1.1:ssnc=none:sp=occurrence_100");
    fallback.push("dis+2_1024_bd=off:bs=off:cond=fast:fsr=off:nwc=1:sio=off:spl=off_900");
    fallback.push("dis-11_1024_bs=off:bfnt=on:lcm=predicate:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:sp=occurrence:urr=on_300");
    fallback.push("dis+10_2:3_bfnt=on:cond=on:fsr=off:lcm=predicate:nwc=1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.4:sp=reverse_arity_100");
    fallback.push("ins+4_8_bs=off:bfnt=on:br=off:cond=fast:flr=on:fsr=off:igbrr=0.1:igrr=1/128:igrp=100:igrpq=2.0:igwr=on:nwc=1.3:sos=all:urr=on_300");
    fallback.push("ott+11_20_bs=off:bfnt=on:cond=fast:fsr=off:lcm=predicate:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.1:ssnc=none:sp=reverse_arity_300");
    fallback.push("dis-1_128_bfnt=on:fsr=off:nwc=1.5:nicw=on:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=1.2:ssnc=all_500");
    fallback.push("dis+10_4_bs=off:nwc=5:sac=on:sio=off_200");
    fallback.push("ott+2_20_bs=off:bsr=unit_only:nwc=5:nicw=on:sio=off:spl=sat:ssnc=none_300");
    fallback.push("dis+3_1024_bfnt=on:cond=fast:fsr=off:nwc=5:sio=off:spl=off_100");
    fallback.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_300");
    fallback.push("dis+1_2:1_bs=off:bsr=on:cond=fast:ep=RSTC:gs=on:lcm=predicate:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on_900");
    fallback.push("dis+11_32_bd=off:bsr=on:fsr=off:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.4:ssnc=none_600");
    fallback.push("ins+10_1_bfnt=on:igbrr=0.8:igrp=400:igrpq=2.0:nwc=1.7:sio=off_1500");
    break;

  case Property::FEQ:
    fallback.push("dis+11_1024_bsr=unit_only:cond=fast:nwc=1.3:sio=off:spl=off_600");
    fallback.push("dis+1_40_bfnt=on:fsr=off:nwc=1.7:sio=off:spl=off:urr=on:updr=off_600");
    fallback.push("dis-2_2:1_bs=unit_only:bsr=unit_only:bfnt=on:lcm=predicate:nwc=5:sp=reverse_arity:updr=off_300");
    fallback.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.9:igrp=1400:igrpq=1.5:nwc=1.2_600");
    fallback.push("dis+10_8:1_bs=off:nwc=4:sio=off:spl=off_300");
    fallback.push("ott+11_10_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_1200");
    fallback.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.5:igrp=1400:igrpq=1.5:nwc=1:updr=off_600");
    fallback.push("ins+2_32_bs=off:bfnt=on:cond=fast:flr=on:gsp=input_only:igbrr=0.4:igrr=16/1:igrp=400:igrpq=1.3:igwr=on:nwc=1.2:sos=all_300");
    fallback.push("ott+11_8:1_bfnt=on:cond=on:fde=none:lcm=predicate:nwc=1.2:nicw=on:sio=off:spl=sat:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+1_1024_bfnt=on:nwc=1.2:nicw=on:sgo=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.0_1200");
    fallback.push("ins+10_1_bfnt=on:igbrr=0.4:igrpq=1.3:nwc=10:sio=off_300");
    fallback.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.8:igrp=2000:igrpq=2.0:nwc=1:sio=off_600");
    fallback.push("ott-3_2_bfnt=on:cond=fast:fde=none:nwc=5:sgo=on:updr=off_1800");
    break;

  case Property::FNE:
    fallback.push("dis+11_3:2_cond=fast:nwc=5:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none_300");
    fallback.push("dis+1_50_bs=off:bfnt=on:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=100000:ssfq=2.0:ssnc=none:sp=occurrence_300");
    fallback.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.3:igrp=700:igrpq=1.2:nwc=1.7_200");
    fallback.push("ins+11_3:1_bs=unit_only:br=off:gsp=input_only:igbrr=0.6:igrr=1/2:igrp=100:igrpq=2.0:igwr=on:nwc=1.5:sio=off:spl=off:urr=on_1200");
    fallback.push("dis+2_4:1_bs=off:fsr=off:gsp=input_only:nwc=2:sio=off:spl=off_2100");
    fallback.push("dis+11_28_bs=off:cond=fast:lcm=predicate:nwc=1.5:sac=on:sp=occurrence:urr=on_200");
    fallback.push("dis-11_2:3_bs=off:bfnt=on:cond=fast:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.1:ssnc=none:sp=occurrence_300");
    fallback.push("ins+4_64_bs=off:bfnt=on:br=off:igbrr=0.6:igrr=1/128:igrp=200:igrpq=2.0:igwr=on:nwc=2:sos=all:urr=on:updr=off_300");
    fallback.push("ott+1_128_bs=unit_only:bfnt=on:cond=fast:fsr=off:lcm=predicate:nwc=1.2:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_300");
    fallback.push("ott+1_2:1_bs=off:fsr=off:nwc=4:nicw=on:spl=sat:ssfp=4000:ssfq=1.0:ssnc=all:updr=off_1500");
    fallback.push("ott+10_40_bs=off:fsr=off:nwc=10:sio=off:spo=on_300");
    fallback.push("ott+1_7_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=1.5:spo=on_600");
    fallback.push("ott+4_8:1_bs=off:cond=on:lcm=predicate:nwc=1.7:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:sp=occurrence_300");
    fallback.push("ott+3_8:1_bs=off:cond=on:nwc=1.3:sio=off_300");
    fallback.push("ott+1_128_bs=off:bfnt=on:fsr=off:nwc=1:nicw=on:sac=on:sgo=on:sio=off:updr=off_300");
    fallback.push("ins+1_5_bs=off:br=off:cond=on:igbrr=0.7:igrr=1/4:igrp=1400:igrpq=2.0:igwr=on:nwc=5:sos=all:urr=on_900");
    break;
  }
}

void Schedules::getCasc2014Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category();
  unsigned prop = property.props();
  unsigned atoms = property.atoms();

  switch (cat) {
  case Property::NEQ:
    if (prop == 131079) {
      quick.push("dis+11_4:1_bs=off:cond=fast:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssnc=none_8");
      quick.push("ott+3_8:1_bd=off:bs=off:bsr=unit_only:drc=off:fsr=off:nwc=2.5:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.0:ssnc=none:updr=off_12");
      quick.push("dis+11_2_bs=off:bsr=unit_only:drc=off:fde=none:lcm=reverse:nwc=1.5:sio=off:spl=sat:sser=off:ssnc=none_23");
      quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_5");
      quick.push("dis+11_4_bs=off:cond=fast:drc=off:ep=on:fde=none:nwc=1.7:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_68");
      quick.push("ott+11_2_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:ep=on:fde=none:lcm=reverse:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_36");
      quick.push("dis+1011_10_bs=off:drc=off:fsr=off:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_29");
      quick.push("dis+1011_24_cond=fast:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_120");
      quick.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:nwc=2:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_117");
      quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_18");
      quick.push("ott+1011_3_bs=off:bsr=unit_only:fde=none:nwc=2:sio=off:spl=sat:ssfq=1.0:ssnc=none_55");
      quick.push("ott+11_2_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=reverse_arity_228");
      quick.push("dis+11_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_283");
      quick.push("ott+11_8:1_drc=off:ep=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none:sp=occurrence:updr=off_480");
      quick.push("dis+1011_64_bs=off:drc=off:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_44");
      quick.push("ott+11_8:1_bs=off:drc=off:flr=on:lcm=predicate:nwc=2:sos=all:sio=off:spo=on:sfv=off_110");
    }
    else if (prop == 1) {
      if (atoms > 175) {
        quick.push("ott+2_1_bs=off:cond=on:drc=off:ep=on:nwc=1.7:nicw=on:ss=axioms:sos=all:sio=off:urr=on_1");
        quick.push("dis+1011_10_bs=off:drc=off:fsr=off:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_2");
        quick.push("dis-1010_3:1_bs=off:drc=off:ep=RS:flr=on:nwc=5:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_2");
        quick.push("dis+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none_38");
        quick.push("dis+10_8:1_bs=off:br=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=1:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_29");
        quick.push("ott+11_2:3_bs=off:drc=off:flr=on:lcm=predicate:nwc=2.5:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none:sp=occurrence_57");
        quick.push("ott-11_2:3_bs=off:drc=off:fsr=off:lcm=predicate:nwc=5:nicw=on:sac=on:sio=off:spo=on:sp=reverse_arity_26");
        quick.push("dis-1002_2:3_bs=off:drc=off:gsp=input_only:nwc=1.7:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=occurrence_3");
        quick.push("dis+1011_64_bs=off:drc=off:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_1");
        quick.push("dis-11_28_bs=off:drc=off:flr=on:lcm=predicate:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=1.0:ssnc=none_48");
        quick.push("ott-11_40_bd=off:bs=off:drc=off:flr=on:fsr=off:lcm=predicate:nwc=10:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:updr=off_42");
        quick.push("ott+1_3:1_bs=off:br=off:drc=off:flr=on:lcm=predicate:nwc=1.7:sio=off:spl=sat:ssfp=10000:ssfq=1.4:ssnc=none:sp=occurrence:urr=on_11");
        quick.push("ott+11_50_bd=off:bs=off:drc=off:ep=on:lcm=reverse:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_56");
        quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_33");
        quick.push("dis+10_1024_bs=off:drc=off:flr=on:fsr=off:nwc=1.1:sio=off:spl=sat:ssfq=1.4:ssnc=none_76");
        quick.push("ott+1_1024_bs=off:br=off:cond=on:drc=off:flr=on:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=1.2:sac=on:sio=off:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=all_dependent:urr=on:updr=off_65");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_72");
        quick.push("lrs+2_64_bd=off:bs=off:br=off:cond=on:drc=off:flr=on:fde=none:nwc=1.7:stl=30:sio=off:spl=sat:ssfp=1000:ssfq=1.4:ssnc=none:urr=on_54");
        quick.push("lrs+1011_14_bd=off:bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=1.7:stl=60:sio=off:spo=on:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:urr=on_211");
        quick.push("ott+11_8:1_bs=off:drc=off:flr=on:lcm=predicate:nwc=2:sos=all:sio=off:spo=on:sfv=off_171");
        quick.push("dis+10_1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:sos=on:sio=off:spl=sat:sser=off:ssnc=none_1");
        quick.push("ott+11_8:1_drc=off:ep=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none:sp=occurrence:updr=off_16");
        quick.push("ott+11_2_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:ep=on:fde=none:lcm=reverse:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_101");
      }
      else {
        quick.push("dis+3_4_bs=off:br=off:cond=on:drc=off:ep=RSTC:nwc=4:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:urr=on_2");
        quick.push("dis+11_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_15");
        quick.push("ins+1_3:1_bs=off:cond=on:ep=RSTC:flr=on:gs=on:igbrr=0.8:igrr=1/4:igrp=400:igrpq=1.1:igwr=on:nwc=1.5:sio=off:urr=on_4");
        quick.push("dis+1011_1024_bs=unit_only:drc=off:gsp=input_only:nwc=1.1:sio=off:spo=on:urr=on_10");
        quick.push("ott+11_2_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=reverse_arity_4");
        quick.push("dis+11_3:1_bs=off:cond=fast:drc=off:nwc=5:sio=off:spl=sat:sser=off:ssnc=none_17");
        quick.push("ott+10_28_bd=off:bs=off:drc=off:nwc=1.5:sio=off:spl=sat:ssnc=none_21");
        quick.push("lrs+2_14_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:nwc=1.1:stl=30:sio=off:spl=sat:ssac=none:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence:updr=off_227");
        quick.push("lrs+4_24_bd=off:bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:stl=60:sio=off:spl=sat:ssfp=4000:ssfq=1.2:ssnc=none_540");
        quick.push("ott+10_64_bd=off:bs=off:drc=off:flr=on:gsp=input_only:lcm=predicate:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_212");
        quick.push("ott+11_3_bd=off:bs=off:drc=off:ep=on:flr=on:nwc=1:sio=off:spl=sat:ssfq=1.1:ssnc=none:sfv=off_46");
      }
    }
    else if (prop == 3) {
      if (atoms <= 2000) {
        quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:ep=on:nwc=10:nicw=on:sio=off:spl=sat:ssnc=none:urr=on_8");
        quick.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:nwc=2:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_95");
        quick.push("ott+2_1_bs=off:cond=on:drc=off:ep=on:nwc=1.7:nicw=on:ss=axioms:sos=all:sio=off:urr=on_23");
        quick.push("ott+1_1024_bs=off:br=off:cond=on:drc=off:flr=on:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=1.2:sac=on:sio=off:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=all_dependent:urr=on:updr=off_2");
        quick.push("ott+1011_3_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.2:sio=off:spl=sat:ssfq=1.4:ssnc=none_69");
        quick.push("dis+4_8:1_bs=off:drc=off:ep=RS:nwc=2:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=sat:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_20");
        quick.push("ins+2_5_bs=off:cond=fast:ep=RSTC:flr=on:gs=on:igbrr=0.7:igrr=1/4:igrp=700:igrpq=1.2:igwr=on:lcm=reverse:nwc=2:sio=off:urr=on_5");
        quick.push("dis+2_24_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none_1");
        quick.push("ott+11_3_bd=off:bs=off:drc=off:ep=on:flr=on:nwc=1:sio=off:spl=sat:ssfq=1.1:ssnc=none:sfv=off_46");
        quick.push("dis+1002_4:1_bs=off:cond=fast:drc=off:ep=RST:nwc=3:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_25");
        quick.push("ott+10_2_bd=off:bs=off:drc=off:flr=on:fsr=off:nwc=1.7:sio=off:spl=sat:ssfq=2.0:ssnc=none_91");
        quick.push("lrs+10_1_bs=off:cond=fast:nwc=5:stl=20:sio=off:spl=sat:ssfq=1.1:ssnc=none_4");
        quick.push("dis+1011_64_bs=off:drc=off:ep=on:flr=on:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_50");
        quick.push("dis+1011_1024_bs=unit_only:cond=fast:lcm=reverse:nwc=1.1:sos=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_48");
        quick.push("dis+1_8:1_bs=off:drc=off:lcm=reverse:nwc=1.5:sos=all:sio=off:spl=sat:ssfq=2.0:ssnc=none:sfv=off_96");
        quick.push("dis+1011_2_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_202");
        quick.push("lrs+2_3_bd=off:cond=on:drc=off:flr=on:nwc=1.3:stl=30:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off:sp=reverse_arity_96");
        quick.push("ott+11_2:3_bs=off:drc=off:flr=on:lcm=predicate:nwc=2.5:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none:sp=occurrence_256");
        quick.push("dis+10_8:1_bs=off:br=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=1:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_140");
        quick.push("dis+2_4:1_bs=unit_only:cond=on:flr=on:lcm=predicate:nwc=1:ssac=none:ssfp=100000:ssfq=2.0:ssnc=all:sfv=off:urr=on_257");
        quick.push("dis+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none_106");
      }
      else {
        quick.push("dis-1010_3:1_bs=off:drc=off:ep=RS:flr=on:nwc=5:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_3");
        quick.push("ott+1011_3_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.2:sio=off:spl=sat:ssfq=1.4:ssnc=none_27");
        quick.push("dis+10_10_bs=off:cond=fast:drc=off:gs=on:nwc=1.7:nicw=on:sd=1:ss=axioms:st=3.0_2");
        quick.push("dis-1002_5:1_bs=off:cond=fast:drc=off:ep=on:nwc=1.1:sd=4:ss=axioms:sos=on:sio=off:spl=sat:urr=on_17");
        quick.push("ott-11_2:3_bs=off:drc=off:fsr=off:lcm=predicate:nwc=5:nicw=on:sac=on:sio=off:spo=on:sp=reverse_arity_14");
        quick.push("dis+11_4:1_bs=off:drc=off:gsp=input_only:nwc=3:sos=on_23");
        quick.push("dis+4_8:1_bs=off:drc=off:ep=RS:nwc=2:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=sat:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_2");
        quick.push("dis+2_24_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none_2");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_31");
        quick.push("ott+2_1_bs=off:cond=on:drc=off:ep=on:nwc=1.7:nicw=on:ss=axioms:sos=all:sio=off:urr=on_2");
        quick.push("dis+3_8:1_bs=off:drc=off:fsr=off:fde=none:nwc=10:sio=off:spl=sat:ssfp=10000:ssfq=1.4:ssnc=none:sp=reverse_arity_17");
        quick.push("ins-1010_8:1_bs=off:ep=RSTC:fsr=off:igbrr=1.0:igrr=1/128:igrp=700:igrpq=1.5:igwr=on:nwc=3:sos=on:sgo=on:sio=off_70");
        quick.push("ott-1010_2:1_bs=off:cond=fast:drc=off:nwc=3:sd=1:ss=axioms:st=2.0:sio=off:spl=sat:sser=off:ssnc=none_36");
        quick.push("dis+1_8:1_bs=off:drc=off:lcm=reverse:nwc=1.5:sos=all:sio=off:spl=sat:ssfq=2.0:ssnc=none:sfv=off_90");
        quick.push("ott+1_10_bd=off:bs=off:drc=off:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_111");
        quick.push("ott+1_1024_bs=off:br=off:cond=on:drc=off:flr=on:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=1.2:sac=on:sio=off:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=all_dependent:urr=on:updr=off_246");
        quick.push("lrs+2_3:1_bs=off:br=off:cond=fast:drc=off:flr=on:nwc=4:stl=60:sio=off:spo=on:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on_250");
        quick.push("lrs+1011_14_bd=off:bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=1.7:stl=60:sio=off:spo=on:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:urr=on_549");
      }
    }
    else {
      quick.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_8");
      quick.push("dis+1011_1024_bs=unit_only:cond=fast:lcm=reverse:nwc=1.1:sos=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_6");
      quick.push("dis+11_8:1_bs=off:drc=off:ep=RS:nwc=10:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_2");
      quick.push("dis+2_2:3_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=1:nicw=on:sos=all:sio=off:spl=sat:ssfq=1.1:ssnc=none:urr=on_7");
      quick.push("ott+1011_3_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.2:sio=off:spl=sat:ssfq=1.4:ssnc=none_12");
      quick.push("dis+2_5_cond=fast:drc=off:fsr=off:lcm=reverse:nwc=3:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none_9");
      quick.push("dis-1010_6_bs=off:drc=off:ep=on:gsp=input_only:nwc=10:nicw=on:sos=all:sp=occurrence_13");
      quick.push("dis+11_4:1_bs=off:drc=off:gsp=input_only:nwc=3:sos=on_1");
      quick.push("dis+2_24_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none_4");
      quick.push("dis+11_24_bs=off:cond=fast:drc=off:nwc=2.5:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_1");
      quick.push("dis+10_1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:sos=on:sio=off:spl=sat:sser=off:ssnc=none_19");
      quick.push("lrs-1010_3_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1.1:nicw=on:stl=30:sos=on:sac=on:sio=off_3");
      quick.push("dis+11_20_bs=off:cond=fast:fde=none:lcm=reverse:nwc=3:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sp=reverse_arity_9");
      quick.push("lrs-1002_6_bs=off:bsr=unit_only:cond=fast:fde=none:gsp=input_only:lcm=reverse:nwc=1:stl=30:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=occurrence_11");
      quick.push("dis-1002_3:2_bs=off:cond=fast:drc=off:ep=RST:flr=on:fde=none:lcm=predicate:nwc=10:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:updr=off_8");
      quick.push("lrs+1010_24_bd=off:bs=off:cond=on:drc=off:gsp=input_only:nwc=1.3:nicw=on:stl=10:sio=off:spo=on:spl=sat:sser=off:ssfq=1.0:ssnc=none:updr=off_18");
      quick.push("dis+1002_4:1_bs=off:cond=fast:drc=off:ep=RST:nwc=3:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_10");
      quick.push("dis-1002_12_bs=off:cond=on:drc=off:ep=on:gsp=input_only:nwc=10:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_1");
      quick.push("dis+11_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_26");
      quick.push("dis+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none_30");
      quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_252");
      quick.push("dis+11_4_ep=on:fde=none:lcm=reverse:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_240");
      quick.push("ins+2_5_bs=off:cond=fast:ep=RSTC:flr=on:gs=on:igbrr=0.7:igrr=1/4:igrp=700:igrpq=1.2:igwr=on:lcm=reverse:nwc=2:sio=off:urr=on_1");
      quick.push("dis+1011_64_bs=off:drc=off:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_26");
      quick.push("dis-1002_2:3_bs=off:drc=off:gsp=input_only:nwc=1.7:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=occurrence_60");
    }
    break;

  case Property::HEQ:
    if (prop == 2) {
      quick.push("dis+11_10_bs=off:drc=off:nwc=10:sos=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_3");
      quick.push("ott+11_128_bs=off:drc=off:gsp=input_only:gs=on:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.4:ssnc=none:sp=occurrence_626");
      quick.push("dis+11_5:4_bs=off:cond=fast:drc=off:fde=none:nwc=1:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.2:ssnc=none:urr=on_164");
      quick.push("ins+2_64_bs=off:cond=on:drc=off:flr=on:fde=none:igbrr=0.1:igrr=1/16:igrpq=1.3:igwr=on:nwc=1.7:sp=reverse_arity_1061");
    }
    else if (prop == 8194) {
      quick.push("lrs+11_2:1_bs=off:cond=on:drc=off:nwc=10:stl=60:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_487");
      quick.push("lrs+11_3:1_bs=unit_only:drc=off:fde=none:nwc=10:stl=60:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_128");
      quick.push("lrs+2_4_bs=off:cond=fast:drc=off:gsp=input_only:nwc=4:stl=90:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none_335");
      quick.push("ott+11_3:2_bs=off:cond=on:drc=off:ep=RSTC:flr=on:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssnc=none_205");
    }
    else {
      quick.push("ott+11_24_bd=off:bs=off:drc=off:nwc=3:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_2");
      quick.push("ins+11_64_bs=off:cond=on:drc=off:fde=none:igbrr=0.2:igrr=1/64:igrp=400:igrpq=1.05:igwr=on:nwc=1.2:sio=off_29");
      quick.push("dis+2_8_bd=off:bs=off:ep=RST:fsr=off:lcm=reverse:nwc=10:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_2");
      quick.push("dis-1002_3_bs=off:cond=fast:drc=off:nwc=10:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_2");
      quick.push("dis+10_1024_bd=off:bs=off:cond=on:drc=off:fde=none:nwc=2.5:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_162");
      quick.push("lrs-1_2_bs=off:drc=off:nwc=4:nicw=on:stl=30:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_236");
      quick.push("ins+4_5:4_bs=off:br=off:cond=on:ep=RSTC:flr=on:fsr=off:igbrr=0.1:igrr=1/16:igrpq=1.0:igwr=on:nwc=2:urr=on_323");
      quick.push("ott+11_24_bs=unit_only:cond=fast:drc=off:nwc=3:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_473");
    }
    break;

  case Property::PEQ:
    if (prop == 0) {
      if (atoms <= 15) {
        quick.push("dis+10_1024_bs=unit_only:cond=on:drc=off:nwc=1:sio=off:spl=sat:ssfq=1.0:ssnc=none_35");
        quick.push("lrs+3_4_bsr=unit_only:drc=off:fsr=off:nwc=1:stl=150:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_490");
        quick.push("lrs+4_14_cond=on:drc=off:flr=on:nwc=3:stl=180:spl=sat:sser=off:ssnc=none_947");
      }
      else {
        quick.push("dis-1010_14_bs=off:drc=off:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none_12");
        quick.push("dis+10_64_bd=off:cond=fast:drc=off:nwc=3:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.1:ssnc=none_241");
        quick.push("ott+11_5_bs=off:cond=fast:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_753");
        quick.push("ott+11_2:3_bd=off:drc=off:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.0:ssnc=none:updr=off_243");
        quick.push("dis+2_7_bd=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_441");
      }
    }
    else if (prop == 1) {
      quick.push("lrs-1_128_bs=off:cond=fast:ep=on:nwc=1.2:nicw=on:stl=30:sac=on:sio=off_14");
      quick.push("dis-1003_128_drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_40");
      quick.push("lrs+3_40_bs=unit_only:bsr=on:cond=on:drc=off:fsr=off:nwc=1.1:nicw=on:stl=100:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on:updr=off_645");
      quick.push("dis+10_64_bd=off:cond=fast:drc=off:nwc=3:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.1:ssnc=none_115");
      quick.push("dis+1_2_bd=off:bs=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_278");
      quick.push("dis+2_7_bd=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_544");
      quick.push("lrs+3_4_bsr=unit_only:drc=off:fsr=off:nwc=1:stl=150:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_1006");
    }
    else {
      quick.push("dis-1010_14_bs=off:drc=off:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none_18");
      quick.push("ott+3_4:1_bsr=unit_only:cond=on:drc=off:fsr=off:fde=none:gs=on:nwc=1:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence:urr=on_13");
      quick.push("lrs+3_16_bsr=unit_only:cond=on:drc=off:fsr=off:nwc=4:stl=150:sio=off:spl=sat:ssnc=none:urr=on_141");
      quick.push("lrs+10_2:3_bd=off:cond=on:drc=off:flr=on:nwc=5:nicw=on:stl=90:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_85");
      quick.push("dis-1004_1024_bs=off:cond=fast:drc=off:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none_72");
      quick.push("ott+11_3_bd=off:bs=unit_only:bsr=unit_only:drc=off:fsr=off:nwc=1.5:sio=off:spo=on:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none_39");
      quick.push("ott+2_20_bs=off:cond=fast:drc=off:fde=none:nwc=2:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:sp=occurrence_116");
      quick.push("ott-1003_8_bs=off:bsr=on:cond=on:drc=off:fsr=off:gs=on:nwc=5:sac=on:sio=off:sp=occurrence_156");
      quick.push("ins+1_20_bsr=on:br=off:cond=fast:drc=off:fsr=off:fde=none:igbrr=0.3:igrr=1/32:igrp=4000:igrpq=2.0:igwr=on:lcm=reverse:nwc=1.2:sgo=on:sio=off:sp=occurrence:urr=on_159");
      quick.push("ott+3_128_bs=off:drc=off:nwc=1.1:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_321");
    }
    break;

  case Property::HNE:
    if (prop == 8192) {
      if (atoms > 6) {
        quick.push("ott+4_6_bs=off:bsr=unit_only:cond=fast:nwc=3:nicw=on:sio=off:spl=sat:ssnc=none_76");
        quick.push("dis-1004_5:1_bs=off:cond=on:nwc=2:sio=off_17");
        quick.push("dis+11_2:3_bs=off:cond=fast:fsr=off:nwc=10:sio=off:spl=sat:sser=off:ssnc=none_125");
        quick.push("ott+1011_10_bs=off:cond=on:nwc=3:sio=off:spl=sat:ssnc=none_61");
        quick.push("lrs+10_3:2_bs=off:cond=fast:nwc=10:stl=90:sio=off:spo=on_127");
        quick.push("ott+1003_28_bs=off:cond=on:flr=on:fsr=off:lcm=predicate:nwc=1:nicw=on:sio=off:spl=off_275");
        quick.push("dis+2_5:4_cond=fast:flr=on:lcm=predicate:nwc=1.5:sio=off:spl=off_562");
        quick.push("dis+1002_64_bs=off:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:nicw=on:sio=off:spo=on_391");
        quick.push("lrs+10_8:1_bs=off:cond=fast:flr=on:fsr=off:nwc=2.5:stl=90:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_601");
      }
      else {
        quick.push("dis+1011_20_bs=off:fsr=off:nwc=2:sio=off:spl=off_103");
        quick.push("lrs+3_4_bs=off:cond=fast:flr=on:nwc=1:nicw=on:stl=30:sio=off:spl=sat:ssnc=none_172");
        quick.push("lrs+11_128_cond=fast:lcm=reverse:nwc=1.5:nicw=on:stl=60:sio=off:spl=sat:ssfp=100000:ssfq=2.0:ssnc=none_525");
        quick.push("lrs+1011_64_bs=unit_only:bsr=unit_only:cond=fast:flr=on:nwc=1:stl=180:sio=off:spl=sat:ssfq=2.0_611");
      }
    }
    else {
      quick.push("dis+1011_14_bs=off:nwc=4:sio=off:spl=off_2");
      quick.push("dis+10_3_bs=off:br=off:cond=fast:gs=on:nwc=1:sos=all:sio=off:urr=on_22");
      quick.push("ott+1011_10_bs=off:cond=on:nwc=3:sio=off:spl=sat:ssnc=none_14");
      quick.push("ott+4_6_bs=off:bsr=unit_only:cond=fast:nwc=3:nicw=on:sio=off:spl=sat:ssnc=none_1");
      quick.push("ott+1011_20_bs=off:cond=fast:nwc=5:sio=off:spl=off_7");
      quick.push("lrs+10_3:1_bs=off:cond=on:flr=on:nwc=10:stl=50:sio=off:spl=off:sp=reverse_arity_62");
      quick.push("ott+1_8:1_bs=off:cond=on:gs=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssnc=none_178");
      quick.push("lrs+10_3:2_bs=off:cond=fast:nwc=10:stl=90:sio=off:spo=on_16");
      quick.push("ott+1011_10_bs=off:cond=fast:flr=on:fsr=off:nwc=1.7:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_21");
    }
    break;

  case Property::NNE:
    quick.push("dis+1011_50_bs=off:cond=fast:flr=on:gsp=input_only:nwc=1.3:sos=all_102");
    quick.push("dis-1010_14_bsr=on:cond=on:nwc=1.5:sio=off:spl=sat:ssfp=100000:ssfq=1.1:ssnc=none_69");
    quick.push("dis+11_20_bs=off:fsr=off:gsp=input_only:lcm=reverse:nwc=1.3:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_13");
    quick.push("dis-2_16_bs=off:cond=fast:flr=on:lcm=predicate:nwc=1:nicw=on:sagn=off:sio=off:spl=sat:ssnc=none_50");
    quick.push("ott+1011_5_bs=off:gs=on:nwc=2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssnc=none_33");
    quick.push("dis+1011_16_bs=unit_only:cond=on:fsr=off:gsp=input_only:nwc=1.7:sos=all:sgo=on:sio=off:spl=sat:ssfq=2.0:ssnc=all_60");
    quick.push("ott-1002_1024_cond=on:flr=on:gs=on:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:urr=on_42");
    quick.push("dis+1011_12_bs=off:cond=fast:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_778");
    quick.push("dis+4_12_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_216");
    quick.push("dis+1011_40_bs=off:gsp=input_only:nwc=4:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_146");
    quick.push("dis+11_128_bs=off:cond=fast:flr=on:lcm=reverse:nwc=2:sio=off:spl=sat:ssnc=none_176");
    quick.push("dis+1011_128_bs=off:cond=fast:flr=on:fsr=off:lcm=reverse:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none_217");
    break;

  case Property::FEQ:
    if (prop == 131075) {
      if (atoms > 3000) {
        quick.push("dis-1002_3_bs=off:cond=on:drc=off:ep=RS:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none_19");
        quick.push("ins+1011_6_bs=off:bsr=unit_only:cond=on:ep=RSTC:fde=none:igbrr=0.2:igrr=8/1:igrp=1400:igrpq=1.5:igwr=on:nwc=1.1:sos=on_15");
        quick.push("dis-1002_8_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.7:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:updr=off_15");
        quick.push("ott+1011_5:4_bs=off:cond=fast:drc=off:flr=on:fsr=off:nwc=2:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_63");
        quick.push("ott+10_8:1_bd=off:bs=off:cond=fast:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_15");
        quick.push("dis-1010_3:2_bs=off:drc=off:ep=on:nwc=3:sac=on:sgo=on:sio=off:spo=on:sfv=off_41");
        quick.push("dis+10_8:1_bs=off:br=off:cond=on:drc=off:ep=RST:fsr=off:nwc=1.3:sd=4:ss=axioms:st=3.0:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_89");
        quick.push("ins-1003_7_bs=off:ep=RSTC:flr=on:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=1.1:igwr=on:nwc=1.3:sos=on:sio=off:spl=off_223");
        quick.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:stl=240:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_737");
        quick.push("ott+1011_2:3_bs=off:br=off:cond=on:drc=off:gs=on:nwc=10:sd=1:ss=axioms:st=1.2:spl=sat:sser=off:urr=on_11");
      }
      else {
        quick.push("dis-1002_8_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.7:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:updr=off_1");
        quick.push("dis+11_28_bd=off:bs=off:cond=fast:drc=off:ep=on:fsr=off:lcm=reverse:nwc=5:sos=on:sio=off:spl=sat:ssfq=1.0:ssnc=none:sp=occurrence_10");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_1");
        quick.push("ins-1010_3:2_bs=unit_only:drc=off:ep=on:fde=none:igbrr=0.5:igrr=1/128:igrpq=1.3:igwr=on:nwc=1.7_35");
        quick.push("ott+2_8:1_bs=off:bsr=on:drc=off:fde=none:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_75");
        quick.push("dis+10_8:1_bs=off:br=off:cond=on:drc=off:ep=RST:fsr=off:nwc=1.3:sd=4:ss=axioms:st=3.0:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_21");
        quick.push("lrs+11_4_cond=fast:drc=off:flr=on:lcm=reverse:nwc=10:stl=50:sos=on:sio=off:spl=sat:ssfp=100000:ssfq=1.4:ssnc=none:sp=reverse_arity_4");
        quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_30");
        quick.push("lrs-1_5:1_bs=off:cond=fast:drc=off:nwc=4:stl=120:sio=off:spl=sat:ssfp=4000:ssfq=2.0:ssnc=none_8");
        quick.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_179");
        quick.push("dis+1011_10_bd=off:bs=off:drc=off:ep=RS:fsr=off:nwc=1:nicw=on_8");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none_182");
        quick.push("lrs+1_6_bs=off:drc=off:nwc=3:stl=360:sos=all:sio=off:spl=sat:ssac=none:sscc=on:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_167");
        quick.push("ins+10_2:3_bs=unit_only:flr=on:fde=none:igbrr=0.7:igrr=1/32:igrp=200:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.7:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=off_92");
        quick.push("lrs-1004_32_bs=off:br=off:cond=fast:drc=off:flr=on:gs=on:lcm=reverse:nwc=1:nicw=on:stl=30:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:urr=on_132");
        quick.push("dis-1_3_bsr=unit_only:drc=off:lcm=reverse:nwc=4:sos=all:sac=on:sgo=on:sio=off:spo=on:sp=reverse_arity_144");
        quick.push("lrs-1010_7_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:stl=30:sgo=on:spo=on:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_291");
        quick.push("ott+1011_5:4_bs=off:cond=fast:drc=off:flr=on:fsr=off:nwc=2:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_298");
      }
    }
    else if (prop == 1) {
      if (atoms > 125) {
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_8");
        quick.push("ott+11_50_bs=off:cond=on:drc=off:fde=none:lcm=reverse:nwc=3:sos=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=reverse_arity:urr=on_7");
        quick.push("ins+11_3:1_bd=off:bs=off:cond=fast:drc=off:igbrr=0.9:igrr=1/128:igrp=200:igrpq=1.2:igwr=on:lcm=predicate:nwc=1:sos=all:sp=occurrence_1");
        quick.push("ott-1003_24_drc=off:nwc=2:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none:urr=on_2");
        quick.push("ott+1010_5_bd=off:bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_1");
        quick.push("dis-1002_3_bs=off:cond=on:drc=off:ep=RS:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none_190");
        quick.push("ott+11_10_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_10");
        quick.push("dis+1011_14_bd=off:bs=off:cond=fast:drc=off:nwc=4:sio=off:spl=sat:ssfq=1.0:ssnc=none_54");
        quick.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_168");
        quick.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_80");
        quick.push("dis+11_32_bs=off:nwc=1.1:sio=off:spl=off:updr=off_198");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_12");
      }
      else {
        quick.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_13");
        quick.push("ott-1002_5:1_bs=off:bsr=on:cond=fast:drc=off:gsp=input_only:nwc=2:sos=all:sio=off:spo=on:urr=on_3");
        quick.push("dis+1011_2_bs=off:drc=off:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:sfv=off:urr=on_3");
        quick.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_3");
        quick.push("dis+1011_1_bs=off:drc=off:nwc=1.2:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none_6");
        quick.push("dis-1002_3_bs=off:cond=on:drc=off:ep=RS:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none_1");
        quick.push("dis+2_2:3_bs=off:drc=off:lcm=reverse:nwc=2.5:sos=all:sser=off:ssfp=10000:ssfq=2.0:ssnc=all_8");
        quick.push("ott+10_1024_bs=off:cond=on:drc=off:gsp=input_only:nwc=1.2:sio=off:spl=sat:ssfp=10000:ssnc=none:updr=off_23");
        quick.push("ott-1010_3_bd=off:bs=off:bsr=unit_only:drc=off:fde=none:nwc=1:nicw=on:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_114");
        quick.push("ott+11_1_bs=off:drc=off:ep=RS:flr=on:fde=none:nwc=4:sos=all:sgo=on:sio=off_121");
        quick.push("ott+1_2:3_bs=off:cond=fast:drc=off:flr=on:lcm=predicate:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on_197");
        quick.push("ott+4_64_bd=off:bs=off:br=off:cond=on:drc=off:fsr=off:gs=on:nwc=1.7:sos=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.0:ssnc=none:urr=on_795");
      }
    }
    else if (prop == 0) {
      if (atoms < 8) {
        quick.push("ott+10_4_bs=off:drc=off:nwc=3:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:sp=occurrence_1");
        quick.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_424");
        quick.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:stl=270:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_1841");
      }
      else if (atoms < 12) {
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_774");
      }
      else if (atoms < 15) {
        quick.push("lrs-1004_12_bs=off:bsr=unit_only:drc=off:nwc=1.7:nicw=on:stl=60:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.4:ssnc=none_473");
        quick.push("lrs+1_6_bs=off:drc=off:nwc=3:stl=360:sos=all:sio=off:spl=sat:ssac=none:sscc=on:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_1806");
      }
      else if (atoms < 50) {
        quick.push("dis+4_5_bs=off:drc=off:lcm=reverse:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=40000:ssfq=1.4:ssnc=none:sp=occurrence_2");
        quick.push("ott+3_7_bs=off:cond=fast:nwc=3:sgt=20:ss=axioms:st=1.2:sos=on:sagn=off:sio=off:spl=sat_4");
        quick.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_33");
        quick.push("lrs-1_10_bs=off:cond=fast:drc=off:nwc=1.5:nicw=on:stl=20:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.4:ssnc=none_127");
        quick.push("ins+4_2_bd=off:bs=off:gs=on:igbrr=0.8:igrr=1/32:igrp=2000:igrpq=1.2:igwr=on:lcm=reverse:nwc=10:urr=on_3");
        quick.push("ott+11_10_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_126");
        quick.push("ott-3_10_bs=off:br=off:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_66");
        quick.push("ins-11_3:1_bs=off:cond=on:drc=off:fsr=off:igbrr=0.5:igrp=100:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.1:sos=all:sio=off:spl=off:sp=reverse_arity_12");
        quick.push("dis+4_2:1_bs=off:br=off:drc=off:fsr=off:nwc=1:sos=all:sio=off:spl=sat:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_148");
        quick.push("lrs-11_6_bs=off:bsr=unit_only:drc=off:gsp=input_only:lcm=predicate:nwc=10:stl=30:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on:updr=off_132");
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_354");
        quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_220");
        quick.push("lrs-1_5:1_bs=off:cond=fast:drc=off:nwc=4:stl=120:sio=off:spl=sat:ssfp=4000:ssfq=2.0:ssnc=none_569");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_1");
        quick.push("ott-1003_24_drc=off:nwc=2:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none:urr=on_46");
      }
      else if (atoms < 150) {
        quick.push("ott+10_3:1_bsr=unit_only:cond=fast:fsr=off:lcm=reverse:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:urr=on:updr=off_23");
        quick.push("ott+4_32_bs=off:flr=on:nwc=4:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_272");
        quick.push("lrs-2_6_bs=off:cond=fast:drc=off:nwc=4:nicw=on:stl=30:spo=on:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_4");
        quick.push("ott+10_24_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:sser=off:ssnc=none_288");
        quick.push("dis+2_20_drc=off:ep=RST:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none:sp=occurrence_62");
        quick.push("dis+4_40_cond=on:drc=off:flr=on:lcm=predicate:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_149");
        quick.push("ott+10_8:1_bd=off:bs=off:cond=fast:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_77");
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_551");
      }
      else if (atoms < 900) {
        quick.push("ins-1010_3:2_bs=unit_only:drc=off:ep=on:fde=none:igbrr=0.5:igrr=1/128:igrpq=1.3:igwr=on:nwc=1.7_30");
        quick.push("dis+1011_3_bs=off:cond=fast:drc=off:fsr=off:fde=none:gs=on:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_71");
        quick.push("dis-1010_4:1_bs=off:drc=off:lcm=predicate:nwc=2:nicw=on:sio=off_165");
        quick.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_498");
        quick.push("ott+11_10_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_929");
      }
      else {
        quick.push("dis+1011_1_bs=off:drc=off:nwc=1.2:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none_20");
        quick.push("ins-1010_3:2_bs=unit_only:drc=off:ep=on:fde=none:igbrr=0.5:igrr=1/128:igrpq=1.3:igwr=on:nwc=1.7_55");
        quick.push("ott+11_1024_bd=off:bs=off:br=off:cond=fast:drc=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on_127");
        quick.push("dis+4_10_bs=off:drc=off:flr=on:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=10000:ssfq=1.4:ssnc=none_274");
        quick.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_560");
        quick.push("dis-1010_3:1_bd=off:bs=off:cond=fast:gs=on:lcm=reverse:nwc=1.1:sac=on_505");
      }
    }
    else if (prop == 131087) {
      if (atoms > 140000) {
        quick.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_23");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_98");
        quick.push("dis+10_8:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.1:nicw=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_39");
        quick.push("ott+1_2_bs=off:drc=off:ep=on:nwc=3:nicw=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat:sser=off_43");
        quick.push("ott+2_3:1_bs=off:br=off:drc=off:nwc=1.1:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:urr=on_22");
        quick.push("dis-1010_2:3_bs=off:drc=off:nwc=3:sd=2:ss=axioms:st=1.5:sac=on:sio=off_20");
        quick.push("dis+1_3_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=1:ss=included:st=2.0:sagn=off:sio=off:spl=sat:sser=off:ssnc=none_134");
        quick.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_95");
        quick.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_24");
        quick.push("dis-4_4:1_bs=off:ep=RST:gsp=input_only:gs=on:nwc=5:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off:sp=occurrence_16");
        quick.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_136");
        quick.push("ott+4_40_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sd=2:sgt=10:ss=axioms:st=3.0:sos=on:spl=sat:sser=off:updr=off_33");
        quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_51");
        quick.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_42");
        quick.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_47");
        quick.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=1.5:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_168");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=2.5:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_129");
        quick.push("ott+10_8:1_bd=off:bs=off:drc=off:fde=none:gsp=input_only:nwc=1:sd=3:ss=axioms:sos=on:sio=off:spl=off:urr=on_213");
        quick.push("ott+10_2:3_bs=off:drc=off:gs=on:nwc=1.5:sd=2:ss=axioms:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:urr=on_111");
        quick.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_121");
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_149");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=10:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_305");
      }
      else if (atoms > 70000) {
        quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_13");
        quick.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_16");
        quick.push("ott+1_2_bs=off:drc=off:ep=on:nwc=3:nicw=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat:sser=off_17");
        quick.push("dis+2_1_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off_19");
        quick.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_9");
        quick.push("dis-1002_40_bs=off:ep=RST:flr=on:gs=on:lcm=predicate:nwc=2.5:nicw=on:sd=5:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:ssnc=none:sp=reverse_arity_8");
        quick.push("dis+10_8:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.1:nicw=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_40");
        quick.push("dis-1010_2_bd=off:bs=off:cond=fast:drc=off:nwc=5:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:sp=occurrence_37");
        quick.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_64");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_86");
        quick.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_9");
        quick.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=1.5:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_87");
        quick.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_19");
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_63");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=2.5:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_103");
        quick.push("dis+1_3_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=1:ss=included:st=2.0:sagn=off:sio=off:spl=sat:sser=off:ssnc=none_135");
        quick.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_276");
        quick.push("dis-1010_1024_bs=off:cond=fast:drc=off:fde=none:nwc=1:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_76");
        quick.push("dis+2_8:1_bd=off:bs=off:bsr=on:drc=off:lcm=reverse:nwc=4:nicw=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sser=off:sp=reverse_arity_104");
        quick.push("ott+10_8:1_bd=off:bs=off:drc=off:fde=none:gsp=input_only:nwc=1:sd=3:ss=axioms:sos=on:sio=off:spl=off:urr=on_122");
        quick.push("ott+10_2:3_bs=off:drc=off:gs=on:nwc=1.5:sd=2:ss=axioms:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:urr=on_275");
        quick.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_142");
        quick.push("dis-1002_8_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.7:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:updr=off_11");
      }
      else if (atoms > 3200) {
        quick.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_8");
        quick.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_3");
        quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_21");
        quick.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_10");
        quick.push("dis+10_8:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.1:nicw=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_40");
        quick.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_47");
        quick.push("dis-4_4:1_bs=off:ep=RST:gsp=input_only:gs=on:nwc=5:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off:sp=occurrence_7");
        quick.push("dis+2_4_bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=4:sd=1:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_3");
        quick.push("ott+1_2_bs=off:drc=off:ep=on:nwc=3:nicw=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat:sser=off_55");
        quick.push("dis-1010_1024_bs=off:cond=fast:drc=off:fde=none:nwc=1:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_5");
        quick.push("dis+1011_1_bs=off:drc=off:nwc=1.2:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none_10");
        quick.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=1.5:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_36");
        quick.push("ott+2_3:1_bs=off:br=off:drc=off:nwc=1.1:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:urr=on_13");
        quick.push("dis+2_8:1_bd=off:bs=off:bsr=on:drc=off:lcm=reverse:nwc=4:nicw=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sser=off:sp=reverse_arity_28");
        quick.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_141");
        quick.push("dis+2_4_bs=off:drc=off:ep=RST:lcm=predicate:nwc=10:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_24");
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_21");
        quick.push("ott+2_4:1_bd=off:bs=off:drc=off:gsp=input_only:nwc=1.1:nicw=on:sd=3:ss=axioms:sos=on:spl=sat:urr=on_23");
        quick.push("ott+10_2:3_bs=off:drc=off:gs=on:nwc=1.5:sd=2:ss=axioms:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:urr=on_53");
        quick.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_259");
        quick.push("dis+1_14_bd=off:bs=off:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=4:sos=on:sio=off:spo=on:sp=reverse_arity_114");
        quick.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_175");
        quick.push("dis+2_4_bs=off:drc=off:ep=RST:flr=on:lcm=reverse:nwc=1.5:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sp=reverse_arity_179");
        quick.push("dis+2_1_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off_8");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=10:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_102");
      }
      else if (atoms > 2200) {
        quick.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_22");
        quick.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_5");
        quick.push("ott+1011_2:1_bs=off:cond=fast:drc=off:nwc=1:nicw=on:sos=all:sio=off:spo=on_29");
        quick.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_104");
        quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_205");
        quick.push("lrs-11_12_bs=off:drc=off:fde=none:gsp=input_only:gs=on:lcm=predicate:nwc=4:nicw=on:stl=300:sos=all:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sfv=off:sp=occurrence:urr=on:updr=off_1612");
      }
      else if (atoms > 900) {
        quick.push("lrs+1_5_bs=off:cond=fast:drc=off:flr=on:nwc=10:stl=30:sac=on:sio=off:urr=on_6");
        quick.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_40");
        quick.push("ott-1002_5:1_bs=off:bsr=on:cond=fast:drc=off:gsp=input_only:nwc=2:sos=all:sio=off:spo=on:urr=on_2");
        quick.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_47");
        quick.push("dis+1011_4_bs=off:drc=off:nwc=4:sgo=on_58");
        quick.push("dis+1010_2:3_bs=off:bsr=unit_only:drc=off:ep=on:fsr=off:fde=none:lcm=predicate:nwc=1.5:sos=on:sac=on:sio=off:spo=on:sp=occurrence_424");
        quick.push("ott+2_20_cond=fast:drc=off:flr=on:lcm=reverse:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity:updr=off_249");
        quick.push("ott-1010_4:1_bs=off:bsr=on:cond=on:drc=off:fde=none:gsp=input_only:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:ssfq=1.0:ssnc=none_287");
        quick.push("ott-1010_3_bd=off:bs=off:bsr=unit_only:drc=off:fde=none:nwc=1:nicw=on:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_325");
        quick.push("dis+2_4_bs=off:drc=off:ep=RST:lcm=predicate:nwc=10:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_7");
        quick.push("ott+2_8:1_bs=off:bsr=on:drc=off:fde=none:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_36");
        quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_86");
      }
      else {
        quick.push("ott-1010_4:1_bs=off:bsr=on:cond=on:drc=off:fde=none:gsp=input_only:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:ssfq=1.0:ssnc=none_7");
        quick.push("dis+2_2:3_bs=off:bsr=unit_only:cond=fast:drc=off:ep=RS:lcm=reverse:nwc=1.2:sos=all:sio=off:spl=sat:ssfp=4000:ssfq=1.4:ssnc=none:sfv=off:sp=occurrence_24");
        quick.push("dis+1010_40_bs=off:drc=off:ep=RS:nwc=1:sio=off:spo=on:spl=sat:ssnc=none_21");
        quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1.1:sos=all:sio=off:spo=on:sp=occurrence_17");
        quick.push("ott+11_2:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:fde=none:gsp=input_only:lcm=reverse:nwc=2.5:sio=off:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_23");
        quick.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=2.5:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_9");
        quick.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_3");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=10:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_272");
        quick.push("dis-1010_5:4_bs=off:bsr=on:cond=fast:drc=off:fde=none:gsp=input_only:nwc=3:sgo=on:sio=off:sp=occurrence:urr=on_150");
        quick.push("ott+11_1_bs=off:drc=off:ep=RS:flr=on:fde=none:nwc=4:sos=all:sgo=on:sio=off_220");
        quick.push("lrs+11_4_cond=fast:drc=off:flr=on:lcm=reverse:nwc=10:stl=50:sos=on:sio=off:spl=sat:ssfp=100000:ssfq=1.4:ssnc=none:sp=reverse_arity_351");
        quick.push("dis-1002_3_bs=off:cond=on:drc=off:ep=RS:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none_18");
        quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:ep=RST:flr=on:fde=none:lcm=reverse:nwc=10:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:urr=on_85");
      }
    }
    else if (prop == 131073) {
      if (atoms > 400) {
        quick.push("ott+11_1_bs=off:drc=off:ep=RS:flr=on:fde=none:nwc=4:sos=all:sgo=on:sio=off_2");
        quick.push("dis+1011_4_bs=off:drc=off:nwc=4:sgo=on_11");
        quick.push("dis+2_8:1_bd=off:bs=off:bsr=on:drc=off:lcm=reverse:nwc=4:nicw=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sser=off:sp=reverse_arity_12");
        quick.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_2");
        quick.push("dis-1010_2_bs=off:bsr=on:drc=off:nwc=4:ssac=none:sscc=on:ssfp=100000:ssfq=1.4:ssnc=all_4");
        quick.push("dis+1_128_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=10:sd=2:ss=included:st=2.0:sagn=off:sio=off:spl=sat:ssnc=none_3");
        quick.push("ins+10_2:3_bs=unit_only:flr=on:fde=none:igbrr=0.7:igrr=1/32:igrp=200:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.7:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=off_26");
        quick.push("dis+1_3_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=1:ss=included:st=2.0:sagn=off:sio=off:spl=sat:sser=off:ssnc=none_2");
        quick.push("dis+4_5_bs=off:drc=off:lcm=reverse:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=40000:ssfq=1.4:ssnc=none:sp=occurrence_7");
        quick.push("ins-1003_7_bs=off:ep=RSTC:flr=on:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=1.1:igwr=on:nwc=1.3:sos=on:sio=off:spl=off_2");
        quick.push("ott+11_2:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:fde=none:gsp=input_only:lcm=reverse:nwc=2.5:sio=off:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_12");
        quick.push("dis+1011_12_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_14");
        quick.push("ott+11_3:2_bs=off:cond=on:drc=off:ep=RSTC:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sagn=off:sio=off:spl=sat:urr=on_3");
        quick.push("dis+2_4_bs=off:br=off:drc=off:nwc=1.2:sd=2:ss=axioms:st=2.0:sio=off:urr=on_24");
        quick.push("ott-1010_2:3_bs=off:cond=fast:drc=off:nwc=1.7:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_9");
        quick.push("dis+1010_32_cond=fast:drc=off:ep=RS:fsr=off:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_66");
        quick.push("lrs-1002_3_bd=off:bs=off:drc=off:ep=on:nwc=1.7:stl=150:sos=on:sac=on:sio=off_21");
        quick.push("tab+10_1_ep=RST:gsp=input_only:sd=4:ss=axioms:st=2.0:sio=off:tbsr=off:tgawr=1/32:tglr=1/5:tipr=off:tlawr=8/1_73");
        quick.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_183");
        quick.push("ott-1002_5:1_bs=off:bsr=on:cond=fast:drc=off:gsp=input_only:nwc=2:sos=all:sio=off:spo=on:urr=on_122");
        quick.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_28");
        quick.push("ott+1011_5:4_bs=off:cond=fast:drc=off:flr=on:fsr=off:nwc=2:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_32");
        quick.push("dis+10_8:1_bs=off:br=off:cond=on:drc=off:ep=RST:fsr=off:nwc=1.3:sd=4:ss=axioms:st=3.0:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_92");
        quick.push("dis+2_1_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off_148");
        quick.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:stl=270:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_1");
        quick.push("dis+11_4_bs=off:drc=off:ep=on:gsp=input_only:nwc=5:sgt=15:ss=axioms:sos=on:spl=sat_1");
        quick.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat_2");
        quick.push("ott+10_8:1_bd=off:bs=off:cond=fast:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_4");
        quick.push("ott+1011_64_bs=off:cond=fast:drc=off:gsp=input_only:nwc=1:ss=axioms:st=1.2:spl=sat:sser=off:urr=on_5");
        quick.push("dis+2_128_bs=off:drc=off:lcm=reverse:nwc=1.3:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_8");
        quick.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_13");
        quick.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none_13");
        quick.push("dis-1010_64_bs=off:drc=off:nwc=2.5:nicw=on:sgo=on:sio=off:spo=on:sser=off:ssfp=40000:ssfq=1.0:ssnc=none_20");
      }
      else if (atoms > 150) {
        quick.push("ott-1010_2:3_bs=off:cond=fast:drc=off:nwc=1.7:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_5");
        quick.push("dis+10_24_bs=off:br=off:drc=off:nwc=4:sd=5:ss=axioms:st=1.5:sgo=on:sio=off:spl=sat:ssac=none:ssfp=1000:ssfq=1.0:ssnc=all:urr=on_1");
        quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_20");
        quick.push("dis-1010_64_bs=off:drc=off:nwc=2.5:nicw=on:sgo=on:sio=off:spo=on:sser=off:ssfp=40000:ssfq=1.0:ssnc=none_2");
        quick.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_25");
        quick.push("dis-1002_1024_bs=off:cond=on:drc=off:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_1");
        quick.push("dis-1010_2_bd=off:bs=off:cond=fast:drc=off:nwc=5:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:sp=occurrence_27");
        quick.push("lrs-11_6_bs=off:bsr=unit_only:drc=off:gsp=input_only:lcm=predicate:nwc=10:stl=30:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on:updr=off_11");
        quick.push("dis+1011_12_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_68");
        quick.push("dis+2_4_bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=4:sd=1:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_19");
        quick.push("dis-1_3_bsr=unit_only:drc=off:lcm=reverse:nwc=4:sos=all:sac=on:sgo=on:sio=off:spo=on:sp=reverse_arity_146");
        quick.push("dis+1_24_bs=off:drc=off:ep=on:lcm=predicate:nwc=3:nicw=on:ss=included:st=5.0:sos=on:sagn=off:spl=sat:sser=off:ssnc=none_33");
        quick.push("ott+1_8:1_bs=off:cond=fast:drc=off:gsp=input_only:nwc=1.3:nicw=on:sd=2:sgt=10:ss=axioms:sagn=off:spl=sat:sser=off_72");
        quick.push("ott+1_2:3_bs=off:cond=fast:drc=off:flr=on:lcm=predicate:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on_20");
        quick.push("dis-1010_3:2_bs=off:drc=off:ep=on:nwc=3:sac=on:sgo=on:sio=off:spo=on:sfv=off_242");
        quick.push("ott+1010_5_bd=off:bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_189");
        quick.push("dis+2_16_bs=off:drc=off:ep=on:gsp=input_only:lcm=reverse:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none_420");
        quick.push("ott+1011_2_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:updr=off_227");
        quick.push("dis+2_2:3_bs=off:drc=off:lcm=reverse:nwc=2.5:sos=all:sser=off:ssfp=10000:ssfq=2.0:ssnc=all_231");
        quick.push("dis+2_4_bs=off:br=off:drc=off:nwc=1.2:sd=2:ss=axioms:st=2.0:sio=off:urr=on_1");
        quick.push("ott+1011_2:1_bs=off:cond=fast:drc=off:nwc=1:nicw=on:sos=all:sio=off:spo=on_23");
      }
      else {
        quick.push("lrs-1010_7_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:stl=30:sgo=on:spo=on:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_1");
        quick.push("dis-1002_6_bs=off:drc=off:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat:sser=off_1");
        quick.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_4");
        quick.push("ott+2_20_cond=fast:drc=off:flr=on:lcm=reverse:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity:updr=off_25");
        quick.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:stl=240:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_1");
        quick.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_97");
        quick.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:stl=270:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_8");
        quick.push("dis+2_4_bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=4:sd=1:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off:sp=occurrence_18");
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_860");
        quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1.1:sos=all:sio=off:spo=on:sp=occurrence_26");
        quick.push("dis+1011_14_bd=off:bs=off:cond=fast:drc=off:nwc=4:sio=off:spl=sat:ssfq=1.0:ssnc=none_262");
      }
    }
    else if (prop == 2) {
      if (atoms > 25) {
        quick.push("ins-1010_1_bs=unit_only:drc=off:igbrr=0.0:igrr=1/4:igrp=4000:igrpq=1.0:igwr=on:nwc=1.1_16");
        quick.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:stl=90:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_874");
        quick.push("lrs+1_64_bs=off:drc=off:gsp=input_only:nwc=1.7:nicw=on:stl=60:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:updr=off_282");
        quick.push("ott+10_1024_bs=off:cond=on:drc=off:gsp=input_only:nwc=1.2:sio=off:spl=sat:ssfp=10000:ssnc=none:updr=off_368");
      }
      else {
        quick.push("ins+3_10_bs=off:drc=off:fde=none:igbrr=0.8:igrr=1/128:igrp=100:igrpq=1.0:igwr=on:nwc=2.5:sio=off:spl=off_152");
        quick.push("lrs+1_6_bs=off:drc=off:nwc=3:stl=360:sos=all:sio=off:spl=sat:ssac=none:sscc=on:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_48");
        quick.push("ott+1_5_bs=off:drc=off:nwc=4:sio=off:sp=occurrence_245");
        quick.push("ott+11_24_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity_178");
        quick.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_577");
        quick.push("ott+4_14_bs=unit_only:cond=fast:drc=off:nwc=1.2:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_383");
        quick.push("ins-1010_1_bs=unit_only:drc=off:igbrr=0.0:igrr=1/4:igrp=4000:igrpq=1.0:igwr=on:nwc=1.1_73");
        quick.push("lrs+10_20_bs=off:cond=on:drc=off:gs=on:nwc=1.1:stl=10:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_80");
      }
    }
    else if (prop == 131072) {
      quick.push("dis-1010_2_bs=off:bsr=on:drc=off:nwc=4:ssac=none:sscc=on:ssfp=100000:ssfq=1.4:ssnc=all_16");
      quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_90");
      quick.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_1");
      quick.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_7");
      quick.push("dis-1002_1024_bs=off:cond=on:drc=off:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_16");
      quick.push("dis-1010_2:3_bs=off:drc=off:nwc=3:sd=2:ss=axioms:st=1.5:sac=on:sio=off_105");
      quick.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1.1:sos=all:sio=off:spo=on:sp=occurrence_110");
      quick.push("dis+1003_1024_bs=off:drc=off:fsr=off:nwc=1.7:nicw=on:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none_38");
      quick.push("lrs+10_20_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=2.5:stl=80:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_59");
      quick.push("ott+10_3:1_bd=off:drc=off:lcm=reverse:nwc=10:nicw=on:sio=off:spo=on:ssac=none:sscc=on:sser=off:ssfp=1000:ssfq=1.2:ssnc=all_667");
      quick.push("dis-1010_50_bs=unit_only:bsr=on:drc=off:lcm=predicate:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_580");
      quick.push("ins-1010_1_bs=unit_only:drc=off:igbrr=0.0:igrr=1/4:igrp=4000:igrpq=1.0:igwr=on:nwc=1.1_12");
    }
    else if (atoms > 10000) {
      quick.push("ins+10_2:3_bs=unit_only:flr=on:fde=none:igbrr=0.7:igrr=1/32:igrp=200:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.7:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=off_24");
      quick.push("dis+2_10_bs=off:cond=fast:ep=RST:lcm=reverse:nwc=1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_44");
      quick.push("dis-1002_1024_bs=off:cond=on:drc=off:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_33");
      quick.push("dis+1_128_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=10:sd=2:ss=included:st=2.0:sagn=off:sio=off:spl=sat:ssnc=none_35");
      quick.push("ins-1003_7_bs=off:ep=RSTC:flr=on:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=1.1:igwr=on:nwc=1.3:sos=on:sio=off:spl=off_6");
      quick.push("dis+1_10_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=7:ss=axioms:st=1.5:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_153");
      quick.push("dis+11_10_bs=off:lcm=predicate:nwc=1.3:sio=off_100");
      quick.push("dis+11_1024_bsr=unit_only:cond=fast:nwc=1.3:sio=off:spl=off_590");
      quick.push("ott+1_5_bs=off:drc=off:nwc=4:sio=off:sp=occurrence_217");
      quick.push("dis+11_32_bs=off:nwc=1.1:sio=off:spl=off:updr=off_244");
    }
    else if (prop == 131083) {
      quick.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat_1");
      quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_6");
      quick.push("dis+2_128_bs=off:drc=off:lcm=reverse:nwc=1.3:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_2");
      quick.push("ott-1010_4:1_bs=off:bsr=on:cond=on:drc=off:fde=none:gsp=input_only:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:ssfq=1.0:ssnc=none_187");
      quick.push("dis+1010_1_bs=off:cond=fast:ep=on:flr=on:fde=none:gsp=input_only:nwc=1.5:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_441");
      quick.push("lrs+10_20_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=2.5:stl=80:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_474");
    }
    else if (prop == 131074) {
      quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_2");
      quick.push("dis-1010_2_bs=off:bsr=on:drc=off:nwc=4:ssac=none:sscc=on:ssfp=100000:ssfq=1.4:ssnc=all_23");
      quick.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_4");
      quick.push("dis+10_2_bs=off:cond=on:drc=off:fde=none:lcm=predicate:nwc=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sp=reverse_arity_31");
      quick.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:stl=240:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_75");
      quick.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_236");
      quick.push("dis-1010_64_bs=off:drc=off:nwc=2.5:nicw=on:sgo=on:sio=off:spo=on:sser=off:ssfp=40000:ssfq=1.0:ssnc=none_27");
      quick.push("dis+1011_4_bs=off:drc=off:nwc=4:sgo=on_56");
      quick.push("ott+4_40_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sd=2:sgt=10:ss=axioms:st=3.0:sos=on:spl=sat:sser=off:updr=off_157");
    }
    else {
      quick.push("ott+1011_10_bs=off:cond=on:drc=off:nwc=1.5:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_4");
      quick.push("ins+10_2:3_bs=off:cond=fast:ep=RSTC:igbrr=0.8:igrr=1/2:igrp=100:igrpq=1.0:igwr=on:nwc=2:sos=all:sio=off:urr=on_1");
      quick.push("dis+2_2:3_bs=off:drc=off:lcm=reverse:nwc=2.5:sos=all:sser=off:ssfp=10000:ssfq=2.0:ssnc=all_9");
      quick.push("dis+4_5:1_bs=off:drc=off:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:ss=axioms:st=2.0:sac=on:sio=off:sp=occurrence:urr=on_3");
      quick.push("dis+11_2:3_bs=off:bsr=unit_only:drc=off:ep=R:lcm=reverse:nwc=2:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_1");
      quick.push("dis+1011_28_cond=on:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:updr=off_12");
      quick.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_1");
      quick.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_37");
      quick.push("ins+1011_8:1_bs=off:cond=fast:drc=off:ep=on:igbrr=0.1:igrr=1/4:igrpq=1.0:igwr=on:nwc=1.3:sos=all:sio=off:sfv=off:urr=on:updr=off_8");
      quick.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_1");
      quick.push("lrs-1010_7_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:stl=30:sgo=on:spo=on:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_17");
      quick.push("dis+1011_14_bd=off:bs=off:cond=fast:drc=off:nwc=4:sio=off:spl=sat:ssfq=1.0:ssnc=none_7");
      quick.push("ott+11_1024_bd=off:bs=off:br=off:cond=fast:drc=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on_128");
      quick.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_40");
      quick.push("dis+11_1_bs=off:drc=off:ep=on:flr=on:lcm=predicate:nwc=5:nicw=on:ss=included:st=5.0:sos=all:sagn=off:spl=sat:sser=off:ssnc=none:updr=off_24");
      quick.push("dis+1010_2:3_bs=off:bsr=unit_only:drc=off:ep=on:fsr=off:fde=none:lcm=predicate:nwc=1.5:sos=on:sac=on:sio=off:spo=on:sp=occurrence_9");
      quick.push("dis-1010_3:2_bs=off:drc=off:ep=on:nwc=3:sac=on:sgo=on:sio=off:spo=on:sfv=off_8");
      quick.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:stl=270:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_10");
      quick.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_48");
      quick.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:stl=240:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_12");
      quick.push("dis+10_24_bs=off:br=off:drc=off:nwc=4:sd=5:ss=axioms:st=1.5:sgo=on:sio=off:spl=sat:ssac=none:ssfp=1000:ssfq=1.0:ssnc=all:urr=on_7");
      quick.push("dis+1011_2_bs=off:drc=off:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:sfv=off:urr=on_53");
      quick.push("dis-1002_8:1_bs=off:cond=on:drc=off:flr=on:gs=on:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_2");
      quick.push("ins+2_4:1_bs=off:cond=on:ep=RSTC:gs=on:igbrr=0.4:igrr=1/128:igrpq=1.05:igwr=on:nwc=5:sos=on:sio=off:urr=on_1");
      quick.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_108");
      quick.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat_165");
      quick.push("dis+1_24_bs=off:drc=off:ep=on:lcm=predicate:nwc=3:nicw=on:ss=included:st=5.0:sos=on:sagn=off:spl=sat:sser=off:ssnc=none_31");
      quick.push("ott+2_8:1_bs=off:bsr=on:drc=off:fde=none:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_23");
      quick.push("dis+1010_1_bs=off:cond=fast:ep=on:flr=on:fde=none:gsp=input_only:nwc=1.5:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_118");
      quick.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_296");
      quick.push("ott+1011_64_bs=off:cond=fast:drc=off:gsp=input_only:nwc=1:ss=axioms:st=1.2:spl=sat:sser=off:urr=on_212");
      quick.push("dis+11_4_bs=off:drc=off:ep=on:gsp=input_only:nwc=5:sgt=15:ss=axioms:sos=on:spl=sat_164");
      quick.push("lrs+10_20_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=2.5:stl=80:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_14");
      quick.push("lrs+11_4_cond=fast:drc=off:flr=on:lcm=reverse:nwc=10:stl=50:sos=on:sio=off:spl=sat:ssfp=100000:ssfq=1.4:ssnc=none:sp=reverse_arity_24");
    }
    break;

  case Property::FNE:
    if (atoms > 200) {
      quick.push("ins+10_1_gsp=input_only:igbrr=0.9:igrp=100:igrpq=1.1:nwc=2_5");
      quick.push("dis+11_50_bs=off:bsr=on:cond=on:lcm=reverse:nwc=1:sio=off:spl=sat:sser=off:ssnc=none:updr=off_267");
      quick.push("dis+4_128_bs=off:cond=on:flr=on:nwc=1.2:sos=on:sac=on_20");
      quick.push("dis+1011_3_bs=off:cond=fast:flr=on:gsp=input_only:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_59");
      quick.push("tab+10_1_spl=off:tbsr=off:tfsr=off:tgawr=3/1:tglr=1/20:tipr=off:tlawr=8/1_114");
      quick.push("dis+3_6_flr=on:fsr=off:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:sp=occurrence_34");
      quick.push("ott+1011_50_bs=off:gs=on:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none:urr=on:updr=off_190");
      quick.push("ott+1_7_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=1.5:spo=on_354");
      quick.push("dis+1010_8_bs=off:nwc=1.1:sio=off_134");
      quick.push("dis+1004_10_bs=off:nwc=1.5:sagn=off:sio=off:spl=sat:ssnc=none_617");
    }
    else if (atoms > 120) {
      quick.push("ott-1002_3_bs=unit_only:bsr=unit_only:cond=on:gsp=input_only:nwc=1.2:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:urr=on_185");
      quick.push("dis+4_10_bs=off:cond=fast:flr=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence:updr=off_204");
      quick.push("lrs+1002_128_bs=off:cond=on:flr=on:lcm=reverse:nwc=1:stl=120:spo=on_990");
    }
    else {
      quick.push("dis+1011_3_bs=off:cond=fast:flr=on:gsp=input_only:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_1");
      quick.push("dis+4_10_bs=off:cond=fast:flr=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence:updr=off_1");
      quick.push("lrs+1002_128_bs=off:cond=on:flr=on:lcm=reverse:nwc=1:stl=120:spo=on_28");
      quick.push("ott+1_40_bsr=unit_only:cond=fast:gs=on:nwc=1.5:sio=off:spl=sat:ssnc=none:urr=on_102");
      quick.push("ins+10_1_igbrr=0.5:igrpq=1.2:nwc=1:sio=off_32");
      quick.push("ott+1_16_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=2.5_94");
      quick.push("ott-1010_128_bs=off:cond=on:lcm=predicate:nwc=1.3:urr=on_394");
      quick.push("ott+10_1024_bs=off:fsr=off:nwc=1.5:nicw=on:sio=off:spl=off_963");
    }
    break;

  case Property::EPR:
    if (prop == 131072) {
      quick.push("ott+10_1024_bs=off:br=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none:urr=on_158");
      quick.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_162");
    }
    else if (atoms > 2000) {
      quick.push("ott+2_20_bs=off:bsr=unit_only:nwc=5:nicw=on:sio=off:spl=sat:ssnc=none_119");
      quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_1935");
    }
    else if (atoms > 1300) {
      quick.push("dis-11_24_bs=off:cond=fast:drc=off:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_1");
      quick.push("ott+10_1024_bs=off:br=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none:urr=on_148");
      quick.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_171");
      quick.push("ins+3_128_bs=off:br=off:drc=off:igbrr=0.1:igrr=32/1:igrp=2000:igrpq=2.0:igwr=on:nwc=3:sos=all:sio=off:urr=on_262");
      quick.push("ins+2_128_bs=unit_only:drc=off:fsr=off:igbrr=1.0:igrr=128/1:igrp=200:igrpq=2.0:igwr=on:lcm=predicate:nwc=2:sos=on:sio=off:sp=occurrence_277");
      quick.push("dis+1_40_bs=unit_only:bsr=unit_only:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none_560");
    }
    else if (atoms > 450) {
      quick.push("dis-10_5:4_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=1.1:sac=on:sio=off:spo=on_5");
      quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_22");
      quick.push("ins+11_50_bs=off:cond=fast:drc=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=2000:igrpq=1.5:igwr=on:nwc=1.2:urr=on_56");
      quick.push("ins+3_50_bs=off:br=off:drc=off:igbrr=0.7:igrr=1/128:igrp=100:igrpq=1.05:igwr=on:lcm=predicate:nwc=2:sp=occurrence:urr=on_501");
      quick.push("ins+10_1_igbrr=0.4:igrp=200:igrpq=1.5:nwc=1.1:sio=off_561");
    }
    else {
      quick.push("ins+11_50_bs=off:cond=fast:drc=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=2000:igrpq=1.5:igwr=on:nwc=1.2:urr=on_12");
      quick.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_13");
      quick.push("ins+3_14_bs=off:cond=on:drc=off:flr=on:igbrr=0.2:igrr=1/128:igrp=200:igrpq=1.2:igwr=on:nwc=1:urr=on_43");
      quick.push("ott+1_128_bs=off:cond=fast:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none:urr=on_1");
      quick.push("dis+4_10_bs=off:drc=off:fsr=off:nwc=5:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.0:ssnc=none_482");
      quick.push("ins-10_14_bs=off:drc=off:fde=none:igbrr=0.9:igrr=1/4:igrp=2000:igrpq=2.0:igwr=on:nwc=1.5:sos=on:sgo=on:sio=off:updr=off_1506");
    }
    break;

  case Property::UEQ:
    if (prop == 2048) {
      quick.push("dis+2_14_bs=off:br=off:ep=RSTC:flr=on:fsr=off:nwc=2.5:nicw=on:sos=all:sio=off:spl=sat:ssnc=none:urr=on_2");
      quick.push("dis+10_32_bs=off:drc=off:fsr=off:nwc=5_2");
      quick.push("lrs+10_14_drc=off:fde=none:nwc=2.5:stl=180_1097");
    }
    else if (prop == 2) {
      if (atoms <= 17) {
        quick.push("dis+11_16_bs=off:bfnt=on:fsr=off:fde=none:gs=on:nwc=1.3:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none_1");
        quick.push("ott+10_20_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.3:sp=occurrence_290");
        quick.push("ott+10_50_bs=off:drc=off:fsr=off:fde=none:nwc=1_213");
        quick.push("ott+10_8:1_bs=off:bsr=on:nwc=4_77");
        quick.push("lrs+10_20_bd=off:bs=unit_only:drc=off:fsr=off:nwc=1.2:stl=90_442");
        quick.push("ott+10_64_drc=off:nwc=1.1_298");
      }
      else if (atoms <= 20) {
        quick.push("ott+2_10_bs=off:drc=off:ep=on:fsr=off:fde=none:gsp=input_only:nwc=1.7:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfq=1.1:ssnc=none_111");
        quick.push("lrs+10_2:1_bs=off:drc=off:fsr=off:nwc=2.5:stl=60:sp=occurrence_14");
        quick.push("lrs+10_14_drc=off:fde=none:nwc=2.5:stl=180_149");
        quick.push("ott+10_2_bs=off:drc=off:fsr=off:fde=none:gsp=input_only:nwc=1.2:sp=reverse_arity_158");
        quick.push("ott+10_50_bs=off:drc=off:fsr=off:fde=none:nwc=1_275");
        quick.push("ins+3_2_bs=off:bfnt=on:br=off:cond=on:fsr=off:fde=none:igbrr=0.9:igrr=1/64:igrpq=2.0:igwr=on:nwc=10:sio=off:urr=on_900");
      }
      else {
        quick.push("lrs+10_2:3_bs=unit_only:cond=on:drc=off:ep=on:fsr=off:gs=on:nwc=1:stl=20:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=occurrence_61");
        quick.push("lrs+10_8_bs=off:cond=fast:drc=off:fsr=off:gsp=input_only:gs=on:nwc=2:stl=120:urr=on_1148");
      }
    }
    else if (atoms <= 2) {
      quick.push("ott+2_10_bs=off:drc=off:ep=on:fsr=off:fde=none:gsp=input_only:nwc=1.7:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfq=1.1:ssnc=none_17");
      quick.push("dis+10_12_bs=off:bsr=unit_only:fsr=off:nwc=2_1");
      quick.push("ins+4_8_bs=off:bfnt=on:br=off:cond=fast:flr=on:fsr=off:igbrr=0.1:igrr=1/128:igrp=100:igrpq=2.0:igwr=on:nwc=1.3:sos=all:urr=on_1");
      quick.push("ott+10_7_bd=off:bs=off:drc=off:fsr=off:fde=none:nwc=1.1:sp=occurrence_87");
      quick.push("lrs+10_1024_nwc=4:stl=150_1463");
    }
    else if (atoms <= 9) {
      quick.push("ott+11_20_bs=off:bfnt=on:cond=fast:fsr=off:lcm=predicate:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.1:ssnc=none:sp=reverse_arity_1");
      quick.push("dis+4_5:1_bs=off:br=off:ep=RSTC:fsr=off:nwc=3:sos=all:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_3");
      quick.push("dis+10_12_bs=off:bsr=unit_only:fsr=off:nwc=2_1");
      quick.push("lrs+10_14_drc=off:fde=none:nwc=2.5:stl=180_852");
      quick.push("ins+3_2_bs=off:bfnt=on:br=off:cond=on:fsr=off:fde=none:igbrr=0.9:igrr=1/64:igrpq=2.0:igwr=on:nwc=10:sio=off:urr=on_33");
      quick.push("ott+10_5_bd=off:drc=off:nwc=2.5_761");
      quick.push("dis+10_12_bs=off:br=off:ep=RSTC:fsr=off:nwc=4:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none:urr=on_59");
    }
    else if (atoms <= 10) {
      quick.push("dis+11_16_bs=off:bfnt=on:fsr=off:fde=none:gs=on:nwc=1.3:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none_1");
      quick.push("ott+10_7_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.5_480");
      quick.push("ott+10_8:1_bs=off:drc=off:fsr=off:nwc=1.7_328");
      quick.push("ott+10_2:1_drc=off:nwc=5_1146");
      quick.push("ott+10_2:3_drc=off:nwc=2_504");
      quick.push("ott+10_8:1_nwc=3:sfv=off_860");
    }
    else if (atoms <= 15) {
      quick.push("ott+11_20_bs=off:bfnt=on:cond=fast:fsr=off:lcm=predicate:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.1:ssnc=none:sp=reverse_arity_1");
      quick.push("lrs+10_14_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.2:stl=20:sio=off:sp=occurrence_25");
      quick.push("lrs+3_20_bs=off:drc=off:fsr=off:fde=none:gsp=input_only:gs=on:lcm=reverse:nwc=3:stl=120_313");
      quick.push("ins+3_2_bs=off:bfnt=on:br=off:cond=on:fsr=off:fde=none:igbrr=0.9:igrr=1/64:igrpq=2.0:igwr=on:nwc=10:sio=off:urr=on_619");
      quick.push("ott+10_20_bd=off:drc=off:nwc=2:sp=occurrence_451");
    }
    else if (atoms <= 18) {
      quick.push("lrs+10_16_drc=off:nwc=1.2:stl=40_351");
      quick.push("dis+10_128_bs=unit_only:bsr=on:drc=off:fsr=off:nwc=2.5_295");
      quick.push("ott+10_8:1_bs=off:drc=off:fsr=off:nwc=1.7_275");
      quick.push("ott+10_10_drc=off:fde=none:nwc=1.1:sp=occurrence_363");
    }
    else {
      quick.push("lrs+10_16_drc=off:nwc=1.2:stl=40_391");
      quick.push("lrs+10_5:1_nwc=1:stl=90_214");
      quick.push("ott+10_2_fde=none:nwc=2.5:sp=reverse_arity_728");
    }
    break;
  }

  switch (cat) {
  case Property::NEQ:
    fallback.push("dis-1010_6_bs=off:drc=off:ep=on:gsp=input_only:nwc=10:nicw=on:sos=all:sp=occurrence_300");
    fallback.push("ott+11_2:3_bs=off:drc=off:flr=on:lcm=predicate:nwc=2.5:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none:sp=occurrence_300");
    fallback.push("dis+10_5_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sgo=on:sio=off:sp=occurrence_100");
    fallback.push("dis+1011_6_bs=off:drc=off:ep=on:fde=none:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none_300");
    fallback.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_300");
    fallback.push("dis+11_64_bs=off:drc=off:fde=none:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_300");
    fallback.push("ott+10_4:1_bd=off:bs=off:bsr=unit_only:drc=off:nwc=2:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_300");
    fallback.push("dis+11_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_300");
    fallback.push("ott-1010_2:1_bs=off:cond=fast:drc=off:nwc=3:sd=1:ss=axioms:st=2.0:sio=off:spl=sat:sser=off:ssnc=none_300");
    fallback.push("lrs+10_1_bs=off:cond=fast:nwc=5:sio=off:spl=sat:ssfq=1.1:ssnc=none_200");
    fallback.push("ott+11_3_bd=off:bs=off:drc=off:ep=on:flr=on:nwc=1:sio=off:spl=sat:ssfq=1.1:ssnc=none:sfv=off_300");
    fallback.push("dis-11_10_bfnt=on:cond=fast:lcm=predicate:nwc=1.2:sio=off:spl=off:sp=occurrence_600");
    fallback.push("dis+11_4:1_bs=off:drc=off:gsp=input_only:nwc=3:sos=on_300");
    fallback.push("dis+10_8:1_bs=off:br=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=1:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_300");
    fallback.push("dis+2_24_bs=off:cond=fast:drc=off:fsr=off:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none_300");
    fallback.push("dis+11_4:1_bs=off:cond=fast:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssnc=none_300");
    fallback.push("ott+1_1024_bs=off:br=off:cond=on:drc=off:flr=on:gs=on:nwc=2.5:nicw=on:sd=1:ss=axioms:st=1.2:sac=on:sio=off:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=all_dependent:urr=on:updr=off_300");
    fallback.push("dis+1_40_bs=off:ep=RSTC:nwc=1.1:sio=off:spl=sat:sser=off:ssnc=none:urr=on_300");
    fallback.push("ott+1011_3_bs=off:cond=on:drc=off:ep=on:fde=none:nwc=1.2:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    fallback.push("dis+1011_24_cond=fast:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_300");
    fallback.push("ott+11_3:1_bs=off:drc=off:ep=on:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    fallback.push("lrs+2_3:1_bs=off:br=off:cond=fast:drc=off:flr=on:nwc=4:sio=off:spo=on:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on_600");
    fallback.push("dis-11_28_bs=off:drc=off:flr=on:lcm=predicate:nwc=1.7:sos=on:sio=off:spl=sat:ssfq=1.0:ssnc=none_300");
    fallback.push("ins-1010_8:1_bs=off:ep=RSTC:fsr=off:igbrr=1.0:igrr=1/128:igrp=700:igrpq=1.5:igwr=on:nwc=3:sos=on:sgo=on:sio=off_300");
    fallback.push("ott+11_2_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:ep=on:fde=none:lcm=reverse:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_300");
    fallback.push("dis+4_8:1_bs=off:drc=off:ep=RS:nwc=2:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=sat:sscc=on:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_200");
    fallback.push("lrs+1011_14_bd=off:bs=off:cond=on:drc=off:ep=on:lcm=predicate:nwc=1.7:sio=off:spo=on:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:urr=on_600");
    fallback.push("ott+11_8:1_drc=off:ep=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssnc=none:sp=occurrence:updr=off_600");
    fallback.push("dis+1011_2_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on:updr=off_300");
    fallback.push("ott+11_50_bd=off:bs=off:drc=off:ep=on:lcm=reverse:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_300");
    fallback.push("lrs+2_64_bd=off:bs=off:br=off:cond=on:drc=off:flr=on:fde=none:nwc=1.7:sio=off:spl=sat:ssfp=1000:ssfq=1.4:ssnc=none:urr=on_300");
    fallback.push("dis+10_3:1_bs=off:bfnt=on:cond=on:lcm=predicate:nwc=1:nicw=on:sac=on:sio=off:spl=sat:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis-1002_5:1_bs=off:cond=fast:drc=off:ep=on:nwc=1.1:sd=4:ss=axioms:sos=on:sio=off:spl=sat:urr=on_300");
    fallback.push("ott+3_8:1_bd=off:bs=off:bsr=unit_only:drc=off:fsr=off:nwc=2.5:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.0:ssnc=none:updr=off_300");
    fallback.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=1.0:igrp=400:igrpq=1.5:nwc=5:sio=off_300");
    fallback.push("dis+10_1024_bs=off:drc=off:flr=on:fsr=off:nwc=1.1:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    fallback.push("dis+1_8:1_bs=off:drc=off:lcm=reverse:nwc=1.5:sos=all:sio=off:spl=sat:ssfq=2.0:ssnc=none:sfv=off_300");
    fallback.push("dis+2_4:1_bs=unit_only:cond=on:flr=on:lcm=predicate:nwc=1:ssac=none:ssfp=100000:ssfq=2.0:ssnc=all:sfv=off:urr=on_300");
    fallback.push("dis-11_1024_bs=off:bfnt=on:lcm=predicate:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:sp=occurrence:urr=on_300");
    fallback.push("dis+1011_10_bs=off:drc=off:fsr=off:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_300");
    fallback.push("dis+10_10_bs=off:cond=fast:drc=off:gs=on:nwc=1.7:nicw=on:sd=1:ss=axioms:st=3.0_300");
    fallback.push("dis+11_4_bs=off:cond=fast:drc=off:ep=on:fde=none:nwc=1.7:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    fallback.push("ins+2_5_bs=off:cond=fast:ep=RSTC:flr=on:gs=on:igbrr=0.7:igrr=1/4:igrp=700:igrpq=1.2:igwr=on:lcm=reverse:nwc=2:sio=off:urr=on_300");
    fallback.push("ott+1011_3_bs=off:bsr=unit_only:fde=none:nwc=2:sio=off:spl=sat:ssfq=1.0:ssnc=none_300");
    fallback.push("lrs+1010_24_bd=off:bs=off:cond=on:drc=off:gsp=input_only:nwc=1.3:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfq=1.0:ssnc=none:updr=off_100");
    fallback.push("dis+3_8:1_bs=off:drc=off:fsr=off:fde=none:nwc=10:sio=off:spl=sat:ssfp=10000:ssfq=1.4:ssnc=none:sp=reverse_arity_300");
    fallback.push("dis-1_128_bfnt=on:fsr=off:nwc=1.5:nicw=on:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=1.2:ssnc=all_500");
    fallback.push("lrs+2_14_bd=off:bs=off:bsr=unit_only:cond=on:drc=off:nwc=1.1:sio=off:spl=sat:ssac=none:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence:updr=off_300");
    fallback.push("dis-1002_3:2_bs=off:cond=fast:drc=off:ep=RST:flr=on:fde=none:lcm=predicate:nwc=10:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:updr=off_300");
    fallback.push("dis+11_8:1_bs=off:drc=off:ep=RS:nwc=10:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_300");
    fallback.push("ott+10_2_bd=off:bs=off:drc=off:flr=on:fsr=off:nwc=1.7:sio=off:spl=sat:ssfq=2.0:ssnc=none_300");
    fallback.push("ott+10_28_bd=off:bs=off:drc=off:nwc=1.5:sio=off:spl=sat:ssnc=none_300");
    fallback.push("ott-11_2:3_bs=off:drc=off:fsr=off:lcm=predicate:nwc=5:nicw=on:sac=on:sio=off:spo=on:sp=reverse_arity_300");
    fallback.push("ott-11_40_bd=off:bs=off:drc=off:flr=on:fsr=off:lcm=predicate:nwc=10:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:updr=off_300");
    fallback.push("dis+3_4_bs=off:br=off:cond=on:drc=off:ep=RSTC:nwc=4:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:urr=on_100");
    fallback.push("ins+1_3:1_bs=off:cond=on:ep=RSTC:flr=on:gs=on:igbrr=0.8:igrr=1/4:igrp=400:igrpq=1.1:igwr=on:nwc=1.5:sio=off:urr=on_300");
    fallback.push("lrs+2_3_bd=off:cond=on:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off:sp=reverse_arity_300");
    fallback.push("ott+11_2_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sp=reverse_arity_300");
    fallback.push("dis+11_20_bs=off:cond=fast:fde=none:lcm=reverse:nwc=3:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sp=reverse_arity_300");
    fallback.push("dis+11_4_ep=on:fde=none:lcm=reverse:nwc=10:sos=on:sio=off:spl=sat:ssnc=none_300");
    fallback.push("ott+1_10_bd=off:bs=off:drc=off:gsp=input_only:lcm=predicate:nwc=1.1:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    fallback.push("dis+1011_1024_bs=unit_only:cond=fast:lcm=reverse:nwc=1.1:sos=on:sio=off:spl=sat:sser=off:ssnc=none:sfv=off_300");
    fallback.push("dis+10_1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.5:sos=on:sio=off:spl=sat:sser=off:ssnc=none_300");
    fallback.push("dis+1002_4:1_bs=off:cond=fast:drc=off:ep=RST:nwc=3:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_300");
    fallback.push("ott+10_64_bd=off:bs=off:drc=off:flr=on:gsp=input_only:lcm=predicate:nwc=2:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_300");
    fallback.push("dis+1011_64_bs=off:drc=off:ep=on:flr=on:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_300");
    fallback.push("ott+11_8:1_bs=off:drc=off:flr=on:lcm=predicate:nwc=2:sos=all:sio=off:spo=on:sfv=off_200");
    fallback.push("dis-1002_12_bs=off:cond=on:drc=off:ep=on:gsp=input_only:nwc=10:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_300");
    fallback.push("dis+1011_1024_bs=unit_only:drc=off:gsp=input_only:nwc=1.1:sio=off:spo=on:urr=on_300");
    fallback.push("dis+1_2:1_bs=off:bsr=on:cond=fast:ep=RSTC:gs=on:lcm=predicate:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on_900");
    fallback.push("lrs+4_24_bd=off:bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sio=off:spl=sat:ssfp=4000:ssfq=1.2:ssnc=none_600");
    break;

  case Property::HEQ:
    fallback.push("lrs+11_2:1_bs=off:cond=on:drc=off:nwc=10:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_600");
    fallback.push("ins+4_12_bs=off:bfnt=on:cond=fast:gsp=input_only:igbrr=0.8:igrr=128/1:igrpq=2.0:nwc=1_300");
    fallback.push("ott+11_128_bs=off:drc=off:gsp=input_only:gs=on:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.4:ssnc=none:sp=occurrence_900");
    fallback.push("ott+11_16_bfnt=on:cond=fast:nwc=1.1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=1000:ssfq=2.0:sp=reverse_arity_1200");
    fallback.push("lrs+2_4_bs=off:cond=fast:drc=off:gsp=input_only:nwc=4:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none_900");
    fallback.push("dis+2_8_bd=off:bs=off:ep=RST:fsr=off:lcm=reverse:nwc=10:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_300");
    fallback.push("dis+11_5:4_bs=off:cond=fast:drc=off:fde=none:nwc=1:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.2:ssnc=none:urr=on_300");
    fallback.push("ins+11_64_bs=off:cond=on:drc=off:fde=none:igbrr=0.2:igrr=1/64:igrp=400:igrpq=1.05:igwr=on:nwc=1.2:sio=off_900");
    fallback.push("ott+2_16_bsr=unit_only:br=off:cond=on:flr=on:fsr=off:nwc=5:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.1:ssnc=none:sfv=off:urr=on:updr=off_900");
    fallback.push("dis+2_1024_bd=off:bs=off:cond=fast:fsr=off:nwc=1:sio=off:spl=off_900");
    fallback.push("lrs+11_3:1_bs=unit_only:drc=off:fde=none:nwc=10:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_600");
    fallback.push("dis+10_1024_bd=off:bs=off:cond=on:drc=off:fde=none:nwc=2.5:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_300");
    fallback.push("dis+10_2:3_bfnt=on:cond=on:fsr=off:lcm=predicate:nwc=1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.4:sp=reverse_arity_100");
    fallback.push("lrs-1_2_bs=off:drc=off:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_300");
    fallback.push("dis-1002_3_bs=off:cond=fast:drc=off:nwc=10:nicw=on:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_300");
    fallback.push("dis+3_1024_bfnt=on:cond=fast:fsr=off:nwc=5:sio=off:spl=off_100");
    fallback.push("ott+11_24_bd=off:bs=off:drc=off:nwc=3:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none_300");
    fallback.push("ins+4_5:4_bs=off:br=off:cond=on:ep=RSTC:flr=on:fsr=off:igbrr=0.1:igrr=1/16:igrpq=1.0:igwr=on:nwc=2:urr=on_600");
    fallback.push("ins+2_64_bs=off:cond=on:drc=off:flr=on:fde=none:igbrr=0.1:igrr=1/16:igrpq=1.3:igwr=on:nwc=1.7:sp=reverse_arity_1200");
    break;

  case Property::PEQ:
    fallback.push("lrs+3_40_bs=unit_only:bsr=on:cond=on:drc=off:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on:updr=off_1000");
    fallback.push("dis-3_28_bd=off:cond=on:nwc=1.5:sio=off_600");
    fallback.push("ott+11_5_bs=off:cond=fast:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_900");
    fallback.push("dis+10_64_bd=off:cond=fast:drc=off:nwc=3:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.1:ssnc=none_300");
    fallback.push("dis+1_1024_nwc=1.1:sio=off:spl=off:updr=off_400");
    fallback.push("ins+1_8:1_bs=off:bfnt=on:igbrr=0.8:igrr=2/1:igrpq=1.2:igwr=on:nwc=3:sos=all:sgo=on_300");
    fallback.push("dis-1003_128_drc=off:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none_300");
    fallback.push("lrs+10_2:3_bd=off:cond=on:drc=off:flr=on:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_900");
    fallback.push("lrs+3_4_bsr=unit_only:drc=off:fsr=off:nwc=1:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_1500");
    fallback.push("dis+2_7_bd=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sfv=off_600");
    fallback.push("ins+1_20_bsr=on:br=off:cond=fast:drc=off:fsr=off:fde=none:igbrr=0.3:igrr=1/32:igrp=4000:igrpq=2.0:igwr=on:lcm=reverse:nwc=1.2:sgo=on:sio=off:sp=occurrence:urr=on_400");
    fallback.push("dis+1_2_bd=off:bs=off:drc=off:flr=on:fsr=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    fallback.push("ott+2_20_bs=off:cond=fast:drc=off:fde=none:nwc=2:sio=off:spl=sat:sser=off:ssfq=2.0:ssnc=none:sp=occurrence_300");
    fallback.push("ott+3_4:1_bsr=unit_only:cond=on:drc=off:fsr=off:fde=none:gs=on:nwc=1:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence:urr=on_300");
    fallback.push("lrs-1_128_bs=off:cond=fast:ep=on:nwc=1.2:nicw=on:sac=on:sio=off_300");
    fallback.push("dis-1004_1024_bs=off:cond=fast:drc=off:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    fallback.push("ott+11_2:3_bd=off:drc=off:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.0:ssnc=none:updr=off_300");
    fallback.push("ott+3_128_bs=off:drc=off:nwc=1.1:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:sp=occurrence_600");
    fallback.push("dis+11_32_bd=off:bsr=on:fsr=off:nwc=1.2:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.4:ssnc=none_600");
    fallback.push("ins+10_1_bfnt=on:igbrr=0.8:igrp=400:igrpq=2.0:nwc=1.7:sio=off_1500");
    fallback.push("lrs+4_14_cond=on:drc=off:flr=on:nwc=3:spl=sat:sser=off:ssnc=none_1800");
    break;

  case Property::EPR:
    fallback.push("ins+10_1_bs=unit_only:br=off:cond=on:drc=off:gsp=input_only:igbrr=0.7:igrr=1/4:igrp=400:igrpq=1.5:igwr=on:nwc=1.2:sio=off:urr=on:updr=off_2100");
    fallback.push("dis-10_5:4_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=1.1:sac=on:sio=off:spo=on_300");
    fallback.push("ott+10_1024_bs=off:br=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.2:ssnc=none:urr=on_300");
    fallback.push("ins+3_50_bs=off:br=off:drc=off:igbrr=0.7:igrr=1/128:igrp=100:igrpq=1.05:igwr=on:lcm=predicate:nwc=2:sp=occurrence:urr=on_600");
    fallback.push("ins+11_50_bs=off:cond=fast:drc=off:flr=on:gsp=input_only:igbrr=0.7:igrr=1/8:igrp=2000:igrpq=1.5:igwr=on:nwc=1.2:urr=on_300");
    fallback.push("dis+1_40_bs=unit_only:bsr=unit_only:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none_900");
    fallback.push("dis+10_1024_bs=off:cond=fast:fsr=off:lcm=predicate:nwc=3:sio=off:spl=off:sp=occurrence:urr=on_300");
    fallback.push("dis-11_24_bs=off:cond=fast:drc=off:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:urr=on_300");
    fallback.push("ins+2_128_bs=unit_only:drc=off:fsr=off:igbrr=1.0:igrr=128/1:igrp=200:igrpq=2.0:igwr=on:lcm=predicate:nwc=2:sos=on:sio=off:sp=occurrence_300");
    fallback.push("ins+3_128_bs=off:br=off:drc=off:igbrr=0.1:igrr=32/1:igrp=2000:igrpq=2.0:igwr=on:nwc=3:sos=all:sio=off:urr=on_300");
    fallback.push("ins+3_14_bs=off:cond=on:drc=off:flr=on:igbrr=0.2:igrr=1/128:igrp=200:igrpq=1.2:igwr=on:nwc=1:urr=on_300");
    fallback.push("ott+2_20_bs=off:bsr=unit_only:nwc=5:nicw=on:sio=off:spl=sat:ssnc=none_300");
    fallback.push("ins-10_14_bs=off:drc=off:fde=none:igbrr=0.9:igrr=1/4:igrp=2000:igrpq=2.0:igwr=on:nwc=1.5:sos=on:sgo=on:sio=off:updr=off_1800");
    break;

  case Property::HNE:
    fallback.push("ott+4_6_bs=off:bsr=unit_only:cond=fast:nwc=3:nicw=on:sio=off:spl=sat:ssnc=none_300");
    fallback.push("dis+1011_20_bs=off:fsr=off:nwc=2:sio=off:spl=off_300");
    fallback.push("ott+11_8_bs=off:bfnt=on:cond=fast:nwc=1:sio=off:spl=sat:sser=off:ssfp=100000:ssnc=none_300");
    fallback.push("lrs+10_3:2_bs=off:cond=fast:nwc=10:sio=off:spo=on_900");
    fallback.push("lrs+1_128_bs=off:cond=on:gs=on:nwc=4:nicw=on:sio=off_900");
    fallback.push("dis+11_24_bs=off:cond=fast:nwc=1.3:nicw=on:sac=on:sio=off_300");
    fallback.push("dis-11_8:1_bs=off:nwc=2.5:sio=off:spl=off_300");
    fallback.push("ott+1003_28_bs=off:cond=on:flr=on:fsr=off:lcm=predicate:nwc=1:nicw=on:sio=off:spl=off_300");
    fallback.push("lrs+3_4_bs=off:cond=fast:flr=on:nwc=1:nicw=on:sio=off:spl=sat:ssnc=none_300");
    fallback.push("dis+11_2:3_bs=off:cond=fast:fsr=off:nwc=10:sio=off:spl=sat:sser=off:ssnc=none_300");
    fallback.push("dis+2_5:4_cond=fast:flr=on:lcm=predicate:nwc=1.5:sio=off:spl=off_600");
    fallback.push("lrs+1011_64_bs=unit_only:bsr=unit_only:cond=fast:flr=on:nwc=1:sio=off:spl=sat:ssfq=2.0_1800");
    fallback.push("dis+10_4_bs=off:nwc=5:sac=on:sio=off_200");
    fallback.push("lrs+11_128_cond=fast:lcm=reverse:nwc=1.5:nicw=on:sio=off:spl=sat:ssfp=100000:ssfq=2.0:ssnc=none_600");
    fallback.push("ott+1_8:1_bs=off:cond=on:gs=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssnc=none_300");
    fallback.push("lrs+10_8:1_bs=off:cond=fast:flr=on:fsr=off:nwc=2.5:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_900");
    fallback.push("dis+1002_64_bs=off:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:nicw=on:sio=off:spo=on_600");
    break;

  case Property::NNE:
    fallback.push("dis+1011_12_bs=off:cond=fast:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.0:ssnc=none_900");
    fallback.push("ins+11_12_bs=off:bfnt=on:gsp=input_only:igbrr=0.9:igrr=4/1:igrp=400:igrpq=2.0:nwc=4_300");
    fallback.push("dis-1010_14_bsr=on:cond=on:nwc=1.5:sio=off:spl=sat:ssfp=100000:ssfq=1.1:ssnc=none_300");
    fallback.push("ott+1_20_bfnt=on:cond=on:nwc=3:sac=on:sgo=on:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.4_300");
    fallback.push("dis+1011_16_bs=unit_only:cond=on:fsr=off:gsp=input_only:nwc=1.7:sos=all:sgo=on:sio=off:spl=sat:ssfq=2.0:ssnc=all_200");
    fallback.push("dis-11_40_bs=off:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssac=none:ssfp=40000:ssfq=2.0:ssnc=none_300");
    fallback.push("ott+4_2:1_bs=off:cond=on:lcm=predicate:nwc=10:nicw=on:sac=on:sio=off_300");
    fallback.push("dis-1_16_bs=off:bfnt=on:cond=on:lcm=predicate:nwc=1.2:nicw=on:ssfq=1.1:ssnc=none:sp=occurrence_100");
    fallback.push("dis+1_28_bs=off:bfnt=on:cond=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.7:sio=off:spl=off:sp=occurrence_300");
    fallback.push("ott-3_16_bs=off:gsp=input_only:nwc=2:nicw=on:sos=all:sio=off:spl=sat:ssnc=none_300");
    fallback.push("ins+10_1_bfnt=on:gsp=input_only:igbrr=0.7:igrpq=2.0:nwc=4_500");
    fallback.push("dis+11_128_bs=off:cond=fast:flr=on:lcm=reverse:nwc=2:sio=off:spl=sat:ssnc=none_300");
    fallback.push("dis+1011_128_bs=off:cond=fast:flr=on:fsr=off:lcm=reverse:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none_300");
    fallback.push("ott-1002_1024_cond=on:flr=on:gs=on:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:urr=on_300");
    fallback.push("dis+1011_50_bs=off:cond=fast:flr=on:gsp=input_only:nwc=1.3:sos=all_300");
    fallback.push("dis+4_12_bs=off:cond=fast:gsp=input_only:lcm=predicate:nwc=4:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_300");
    fallback.push("dis+1011_40_bs=off:gsp=input_only:nwc=4:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_300");
    fallback.push("ott+1011_5_bs=off:gs=on:nwc=2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssnc=none_100");
    fallback.push("dis-2_16_bs=off:cond=fast:flr=on:lcm=predicate:nwc=1:nicw=on:sagn=off:sio=off:spl=sat:ssnc=none_300");
    break;

  case Property::FEQ:
    fallback.push("dis-1010_2_bs=off:bsr=on:drc=off:nwc=4:ssac=none:sscc=on:ssfp=100000:ssfq=1.4:ssnc=all_300");
    fallback.push("ott+1011_5:1_bs=off:bsr=unit_only:cond=on:drc=off:nwc=2:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_300");
    fallback.push("lrs+1011_8_bd=off:bs=unit_only:bsr=on:drc=off:lcm=reverse:nwc=1:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_2700");
    fallback.push("ins+10_2:3_bs=unit_only:flr=on:fde=none:igbrr=0.7:igrr=1/32:igrp=200:igrpq=1.0:igwr=on:lcm=predicate:nwc=1.7:sd=2:ss=axioms:st=3.0:sos=all:sio=off:spl=off_300");
    fallback.push("lrs+10_20_bs=off:drc=off:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=2.0:ssnc=none_900");
    fallback.push("ott+11_1024_bd=off:bs=off:br=off:cond=fast:drc=off:ep=RSTC:nwc=1:sio=off:spl=sat:ssfq=1.2:ssnc=none:urr=on_300");
    fallback.push("lrs+1011_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1:nicw=on:sos=all:sio=off:spo=on:ssac=none:ssfp=1000:ssfq=1.1:ssnc=all_dependent_2400");
    fallback.push("dis+1_3_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=1:ss=included:st=2.0:sagn=off:sio=off:spl=sat:sser=off:ssnc=none_300");
    fallback.push("dis+1010_12_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:ssnc=none:sfv=off_300");
    fallback.push("dis-1002_6_bs=off:cond=on:drc=off:fde=none:nwc=1.5:sd=1:ss=included:sos=on:sagn=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_300");
    fallback.push("dis+1011_12_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=1.2:ssnc=none_300");
    fallback.push("ott+1011_3:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=1.5:sd=3:ss=axioms:sos=all:sio=off:spl=off_300");
    fallback.push("ott+1_2_bs=off:drc=off:ep=on:nwc=3:nicw=on:sd=2:sgt=20:ss=axioms:sos=on:sagn=off:spl=sat:sser=off_300");
    fallback.push("dis+2_2:3_bs=off:drc=off:lcm=reverse:nwc=2.5:sos=all:sser=off:ssfp=10000:ssfq=2.0:ssnc=all_300");
    fallback.push("ins-1010_3:2_bs=unit_only:drc=off:ep=on:fde=none:igbrr=0.5:igrr=1/128:igrpq=1.3:igwr=on:nwc=1.7_300");
    fallback.push("tab+10_1_ep=RST:gsp=input_only:sd=4:ss=axioms:st=2.0:sio=off:tbsr=off:tgawr=1/32:tglr=1/5:tipr=off:tlawr=8/1_100");
    fallback.push("ins+3_10_bs=off:drc=off:fde=none:igbrr=0.8:igrr=1/128:igrp=100:igrpq=1.0:igwr=on:nwc=2.5:sio=off:spl=off_300");
    fallback.push("ott+2_12_bd=off:drc=off:lcm=reverse:nwc=2:sio=off:spo=on_600");
    fallback.push("ins+10_2:3_bs=off:cond=fast:ep=RSTC:igbrr=0.8:igrr=1/2:igrp=100:igrpq=1.0:igwr=on:nwc=2:sos=all:sio=off:urr=on_300");
    fallback.push("dis-1010_2:3_bs=off:drc=off:nwc=3:sd=2:ss=axioms:st=1.5:sac=on:sio=off_300");
    fallback.push("ott+1_2:3_bs=off:bsr=unit_only:drc=off:lcm=predicate:nwc=1.3:sd=3:sgt=7:ss=axioms:st=3.0:sos=on:spl=sat_300");
    fallback.push("dis-2_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=1.5:sd=3:sgt=10:ss=axioms:sos=on:spl=sat_300");
    fallback.push("ott+1011_8:1_bs=off:drc=off:fde=none:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off:sp=occurrence_300");
    fallback.push("ott+11_4:1_bs=off:cond=on:drc=off:flr=on:nwc=3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:sfv=off_300");
    fallback.push("dis-1002_1024_bs=off:cond=on:drc=off:nwc=3:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    fallback.push("dis+1_10_bs=off:cond=on:drc=off:lcm=predicate:nwc=2.5:sd=7:ss=axioms:st=1.5:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_400");
    fallback.push("ott+1011_2_bs=off:drc=off:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:updr=off_300");
    fallback.push("dis+1_2:3_bs=off:drc=off:lcm=predicate:nwc=3:nicw=on:ss=included:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off:sp=occurrence_300");
    fallback.push("dis-4_4:1_bs=off:ep=RST:gsp=input_only:gs=on:nwc=5:sd=1:ss=included:st=5.0:sos=on:sio=off:sfv=off:sp=occurrence_300");
    fallback.push("ott-1010_64_bs=off:br=off:drc=off:flr=on:gs=on:nwc=1:sac=on:sio=off:urr=on_600");
    fallback.push("ott+10_2:3_bs=off:drc=off:gs=on:nwc=1.5:sd=2:ss=axioms:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:urr=on_300");
    fallback.push("lrs-1_10_bs=off:cond=fast:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.4:ssnc=none_200");
    fallback.push("ott+1_3:1_bd=off:bs=off:bsr=unit_only:ep=on:nwc=10:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:spl=sat_300");
    fallback.push("dis+1010_1_bs=off:cond=fast:ep=on:flr=on:fde=none:gsp=input_only:nwc=1.5:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=2.0:ssnc=none_500");
    fallback.push("dis+1011_10_bs=off:drc=off:nwc=1.3:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off_300");
    fallback.push("dis-1010_7_bs=off:cond=fast:drc=off:fde=none:nwc=1.1:nicw=on:sd=1:ss=axioms:st=3.0:sio=off:spl=sat_300");
    fallback.push("ott+11_2:1_bs=off:br=off:cond=fast:drc=off:fde=none:gsp=input_only:lcm=predicate:nwc=1.5:sd=2:ss=axioms:st=1.5:sos=all:sio=off:spl=sat:sser=off:ssfp=40000:ssfq=1.0:ssnc=none:urr=on_300");
    fallback.push("dis-1004_3:2_bs=off:cond=fast:drc=off:ep=RST:gsp=input_only:nwc=2.5:sd=2:sgt=3:ss=axioms:st=1.2:sos=on:spl=sat_300");
    fallback.push("ott+4_32_bs=off:flr=on:nwc=4:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence_300");
    fallback.push("ott+1_5_bs=off:drc=off:nwc=4:sio=off:sp=occurrence_300");
    fallback.push("dis+10_8:1_bs=off:cond=fast:drc=off:lcm=predicate:nwc=1.1:nicw=on:sd=2:sgt=5:ss=axioms:st=5.0:sos=on:spl=sat:sp=reverse_arity_300");
    fallback.push("lrs+1_6_bs=off:drc=off:nwc=3:sos=all:sio=off:spl=sat:ssac=none:sscc=on:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_3600");
    fallback.push("dis-1002_16_bs=off:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sd=3:ss=axioms:st=1.5:sos=on:sagn=off:sio=off:spl=sat:sser=off_300");
    fallback.push("dis+11_1024_bsr=unit_only:cond=fast:nwc=1.3:sio=off:spl=off_600");
    fallback.push("ott+2_3:1_bs=off:br=off:drc=off:nwc=1.1:nicw=on:sd=3:ss=axioms:st=3.0:sos=all:sio=off:spl=off:urr=on_300");
    fallback.push("ott-1002_8:1_bd=off:bs=off:bsr=unit_only:cond=fast:drc=off:fde=none:nwc=1.1:sos=all:sio=off:spo=on:sp=occurrence_300");
    fallback.push("dis+10_8:1_bs=off:br=off:cond=on:drc=off:ep=RST:fsr=off:nwc=1.3:sd=4:ss=axioms:st=3.0:spl=sat:sser=off:ssfq=2.0:ssnc=none:urr=on_300");
    fallback.push("ott+1011_64_bs=off:cond=fast:drc=off:gsp=input_only:nwc=1:ss=axioms:st=1.2:spl=sat:sser=off:urr=on_300");
    fallback.push("dis+2_1_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:nwc=1:sd=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sser=off_300");
    fallback.push("dis-10_4:1_bs=off:drc=off:ep=on:lcm=predicate:nwc=4:sgt=10:ss=axioms:sos=on:spl=sat:sp=occurrence_300");
    fallback.push("dis+1011_12_bs=off:cond=fast:drc=off:ep=RS:flr=on:nwc=1.5:nicw=on:sio=off:spl=sat:ssfq=1.4:ssnc=none_300");
    fallback.push("ott-1002_5:1_bs=off:bsr=on:cond=fast:drc=off:gsp=input_only:nwc=2:sos=all:sio=off:spo=on:urr=on_300");
    fallback.push("dis+4_40_cond=on:drc=off:flr=on:lcm=predicate:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence_300");
    fallback.push("dis+11_3:1_bs=off:br=off:cond=on:drc=off:ep=on:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sos=on:sagn=off:sio=off:spl=sat:sser=off:urr=on_300");
    fallback.push("lrs-1010_7_bs=unit_only:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:lcm=predicate:nwc=1.7:sgo=on:spo=on:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_300");
    fallback.push("ott+10_8:1_bd=off:bs=off:drc=off:fde=none:gsp=input_only:nwc=1:sd=3:ss=axioms:sos=on:sio=off:spl=off:urr=on_300");
    fallback.push("ott+10_3:1_bsr=unit_only:cond=fast:fsr=off:lcm=reverse:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:urr=on:updr=off_300");
    fallback.push("ins-1003_7_bs=off:ep=RSTC:flr=on:igbrr=0.0:igrr=1/128:igrp=1400:igrpq=1.1:igwr=on:nwc=1.3:sos=on:sio=off:spl=off_300");
    fallback.push("ott+11_24_bs=off:cond=fast:drc=off:fde=none:gs=on:nwc=1.1:nicw=on:sio=off:spl=sat:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity_400");
    fallback.push("dis+2_4_bs=off:br=off:drc=off:nwc=1.2:sd=2:ss=axioms:st=2.0:sio=off:urr=on_300");
    fallback.push("dis+11_4_bs=off:drc=off:ep=on:gsp=input_only:nwc=5:sgt=15:ss=axioms:sos=on:spl=sat_300");
    fallback.push("ott-1010_4:1_bs=off:bsr=on:cond=on:drc=off:fde=none:gsp=input_only:nwc=2.5:sd=1:ss=axioms:sos=all:spl=sat:ssfq=1.0:ssnc=none_300");
    fallback.push("dis+11_10_bs=off:lcm=predicate:nwc=1.3:sio=off_300");
    fallback.push("dis+10_2_bs=off:cond=on:drc=off:fde=none:lcm=predicate:nwc=2:sgt=2:ss=axioms:st=1.2:sos=on:spl=sat:sp=reverse_arity_300");
    fallback.push("dis+2_8:1_bd=off:bs=off:bsr=on:drc=off:lcm=reverse:nwc=4:nicw=on:sd=3:sgt=5:ss=axioms:st=1.5:sos=on:spl=sat:sser=off:sp=reverse_arity_300");
    fallback.push("ott+11_3:2_bs=off:cond=on:drc=off:ep=RSTC:gs=on:nwc=4:nicw=on:sd=2:sgt=10:ss=axioms:sagn=off:sio=off:spl=sat:urr=on_300");
    fallback.push("ott+1011_8:1_bs=off:cond=fast:drc=off:fde=none:gsp=input_only:nwc=10:sd=2:ss=axioms:sos=all:spl=sat:sser=off:ssfq=1.0:ssnc=none:sfv=off_300");
    fallback.push("ott+10_1024_bs=off:cond=on:drc=off:gsp=input_only:nwc=1.2:sio=off:spl=sat:ssfp=10000:ssnc=none:updr=off_600");
    fallback.push("lrs+10_20_bs=off:cond=fast:drc=off:ep=on:gsp=input_only:lcm=predicate:nwc=2.5:sos=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none_800");
    fallback.push("ott-1010_3_bd=off:bs=off:bsr=unit_only:drc=off:fde=none:nwc=1:nicw=on:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity:updr=off_400");
    fallback.push("dis+1010_2:3_bs=off:bsr=unit_only:drc=off:ep=on:fsr=off:fde=none:lcm=predicate:nwc=1.5:sos=on:sac=on:sio=off:spo=on:sp=occurrence_600");
    fallback.push("dis+1_24_bs=off:drc=off:ep=on:lcm=predicate:nwc=3:nicw=on:ss=included:st=5.0:sos=on:sagn=off:spl=sat:sser=off:ssnc=none_300");
    fallback.push("dis-1_3_bsr=unit_only:drc=off:lcm=reverse:nwc=4:sos=all:sac=on:sgo=on:sio=off:spo=on:sp=reverse_arity_300");
    fallback.push("dis-1010_50_bs=unit_only:bsr=on:drc=off:lcm=predicate:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none_900");
    fallback.push("ott+10_8:1_bd=off:bs=off:cond=fast:drc=off:flr=on:nwc=1.3:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_300");
    fallback.push("dis+4_2:1_bs=off:br=off:drc=off:fsr=off:nwc=1:sos=all:sio=off:spl=sat:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_200");
    fallback.push("ott+1011_5:4_bs=off:cond=fast:drc=off:flr=on:fsr=off:nwc=2:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_300");
    fallback.push("ott+11_1_bs=off:drc=off:ep=RS:flr=on:fde=none:nwc=4:sos=all:sgo=on:sio=off_300");
    fallback.push("dis+1010_32_cond=fast:drc=off:ep=RS:fsr=off:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    fallback.push("ott+1011_10_bs=off:cond=on:drc=off:nwc=1.5:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_200");
    fallback.push("lrs-1004_32_bs=off:br=off:cond=fast:drc=off:flr=on:gs=on:lcm=reverse:nwc=1:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none:urr=on_300");
    fallback.push("dis-1010_2_bd=off:bs=off:cond=fast:drc=off:nwc=5:nicw=on:sd=2:ss=axioms:st=1.5:sos=on:sio=off:spl=sat:sser=off:sp=occurrence_300");
    fallback.push("ott+1_5:1_bs=off:bsr=unit_only:cond=fast:drc=off:nwc=5:ss=axioms:sos=all_100");
    fallback.push("lrs-11_6_bs=off:bsr=unit_only:drc=off:gsp=input_only:lcm=predicate:nwc=10:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on:updr=off_300");
    fallback.push("dis+1011_28_cond=on:drc=off:nwc=5:nicw=on:sio=off:spl=sat:ssfq=2.0:ssnc=none:updr=off_100");
    fallback.push("ott+1011_8:1_bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none_300");
    fallback.push("ott+2_8:1_bs=off:bsr=on:drc=off:fde=none:lcm=reverse:nwc=1.2:nicw=on:sos=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_300");
    fallback.push("ott-3_10_bs=off:br=off:drc=off:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.4:ssnc=none:urr=on_300");
    fallback.push("dis+1003_1024_bs=off:drc=off:fsr=off:nwc=1.7:nicw=on:sos=on:sio=off:spl=sat:ssfq=2.0:ssnc=none_300");
    fallback.push("ott+4_40_bs=off:bsr=unit_only:cond=fast:drc=off:ep=on:flr=on:gsp=input_only:lcm=predicate:nwc=5:nicw=on:sd=2:sgt=10:ss=axioms:st=3.0:sos=on:spl=sat:sser=off:updr=off_300");
    fallback.push("dis+1011_4_bs=off:drc=off:nwc=4:sgo=on_100");
    fallback.push("dis+2_4_bs=off:drc=off:ep=RST:flr=on:lcm=reverse:nwc=1.5:sos=on:sio=off:spl=sat:ssfq=1.4:ssnc=none:sp=reverse_arity_300");
    fallback.push("ott+10_24_bs=off:drc=off:fde=none:nwc=1.3:nicw=on:sio=off:spl=sat:sser=off:ssnc=none_300");
    fallback.push("dis+1_128_bs=off:cond=fast:drc=off:gsp=input_only:lcm=predicate:nwc=10:sd=2:ss=included:st=2.0:sagn=off:sio=off:spl=sat:ssnc=none_300");
    fallback.push("ins+2_4:1_bs=off:cond=on:ep=RSTC:gs=on:igbrr=0.4:igrr=1/128:igrpq=1.05:igwr=on:nwc=5:sos=on:sio=off:urr=on_300");
    fallback.push("dis-1002_8:1_bs=off:cond=on:drc=off:flr=on:gs=on:nwc=4:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:urr=on_300");
    fallback.push("dis+11_28_bd=off:bs=off:cond=fast:drc=off:ep=on:fsr=off:lcm=reverse:nwc=5:sos=on:sio=off:spl=sat:ssfq=1.0:ssnc=none:sp=occurrence_300");
    fallback.push("dis+1011_1_bs=off:drc=off:nwc=1.2:nicw=on:sio=off:spl=sat:ssfp=1000:ssfq=2.0:ssnc=none_300");
    fallback.push("ott+11_2:1_bs=off:cond=fast:drc=off:ep=RS:flr=on:fde=none:gsp=input_only:lcm=reverse:nwc=2.5:sio=off:spl=sat:ssfq=1.1:ssnc=none:sp=occurrence_300");
    fallback.push("ott+2_20_cond=fast:drc=off:flr=on:lcm=reverse:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity:updr=off_300");
    fallback.push("lrs-1004_12_bs=off:bsr=unit_only:drc=off:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.4:ssnc=none_600");
    fallback.push("lrs-1002_3_bd=off:bs=off:drc=off:ep=on:nwc=1.7:sos=on:sac=on:sio=off_1500");
    fallback.push("dis+1010_40_bs=off:drc=off:ep=RS:nwc=1:sio=off:spo=on:spl=sat:ssnc=none_300");
    fallback.push("dis+1011_3_bs=off:cond=fast:drc=off:fsr=off:fde=none:gs=on:nwc=1.1:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.0:ssnc=none_300");
    fallback.push("lrs+1_5_bs=off:cond=fast:drc=off:flr=on:nwc=10:sac=on:sio=off:urr=on_300");
    fallback.push("dis+2_128_bs=off:drc=off:lcm=reverse:nwc=1.3:nicw=on:sos=on:sio=off:spl=sat:ssnc=none_300");
    fallback.push("dis+1_14_bd=off:bs=off:drc=off:fde=none:gsp=input_only:lcm=reverse:nwc=4:sos=on:sio=off:spo=on:sp=reverse_arity_200");
    fallback.push("dis+11_32_bs=off:nwc=1.1:sio=off:spl=off:updr=off_300");
    fallback.push("dis+2_20_drc=off:ep=RST:nwc=1.3:sio=off:spl=sat:ssfq=1.4:ssnc=none:sp=occurrence_300");
    fallback.push("dis+1011_2_bs=off:drc=off:nwc=1.1:sos=all:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none:sfv=off:urr=on_300");
    fallback.push("ott+1010_5_bd=off:bs=off:cond=fast:drc=off:nwc=2.5:nicw=on:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.1:ssnc=none_300");
    fallback.push("lrs+1_64_bs=off:drc=off:gsp=input_only:nwc=1.7:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none:updr=off_600");
    fallback.push("dis-1010_64_bs=off:drc=off:nwc=2.5:nicw=on:sgo=on:sio=off:spo=on:sser=off:ssfp=40000:ssfq=1.0:ssnc=none_300");
    fallback.push("lrs-1_5:1_bs=off:cond=fast:drc=off:nwc=4:sio=off:spl=sat:ssfp=4000:ssfq=2.0:ssnc=none_1200");
    fallback.push("dis-1010_3:1_bd=off:bs=off:cond=fast:gs=on:lcm=reverse:nwc=1.1:sac=on_900");
    fallback.push("ott+4_14_bs=unit_only:cond=fast:drc=off:nwc=1.2:sio=off:spl=sat:ssfq=2.0:ssnc=none:sp=occurrence_600");
    fallback.push("dis+2_16_bs=off:drc=off:ep=on:gsp=input_only:lcm=reverse:nwc=1:sio=off:spl=sat:ssfp=10000:ssfq=1.1:ssnc=none_600");
    fallback.push("ott+11_10_bs=off:bsr=unit_only:cond=fast:drc=off:flr=on:nwc=1:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfp=4000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=on_1200");
    fallback.push("lrs-11_12_bs=off:drc=off:fde=none:gsp=input_only:gs=on:lcm=predicate:nwc=4:nicw=on:sos=all:sio=off:spl=sat:sser=off:ssfp=1000:ssfq=1.2:ssnc=none:sfv=off:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("ott+10_3:1_bd=off:drc=off:lcm=reverse:nwc=10:nicw=on:sio=off:spo=on:ssac=none:sscc=on:sser=off:ssfp=1000:ssfq=1.2:ssnc=all_900");
    fallback.push("ott+4_64_bd=off:bs=off:br=off:cond=on:drc=off:fsr=off:gs=on:nwc=1.7:sos=on:sio=off:spl=sat:sser=off:ssfp=4000:ssfq=1.0:ssnc=none:urr=on_900");
    break;

  case Property::FNE:
    fallback.push("dis-1010_12_bs=off:bsr=unit_only:cond=on:gsp=input_only:nwc=1.5:nicw=on:sio=off:spl=sat:sser=off:ssnc=none:sp=occurrence:urr=on_300");
    fallback.push("dis+11_50_bs=off:cond=on:nwc=1.3:sos=all:sio=off:spl=sat:sser=off:ssnc=none:urr=on_300");
    fallback.push("dis+1011_3_bs=off:cond=fast:flr=on:gsp=input_only:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfq=1.1:ssnc=none_300");
    fallback.push("ott+1_7_bs=off:bsr=unit_only:fsr=off:gsp=input_only:nwc=1.5:spo=on_600");
    fallback.push("dis+11_50_bs=off:bsr=on:cond=on:lcm=reverse:nwc=1:sio=off:spl=sat:sser=off:ssnc=none:updr=off_300");
    fallback.push("ott+1011_50_bs=off:gs=on:nwc=2.5:nicw=on:sio=off:spl=sat:ssnc=none:urr=on:updr=off_300");
    fallback.push("dis+4_128_bs=off:cond=on:flr=on:nwc=1.2:sos=on:sac=on_300");
    fallback.push("dis+1004_10_bs=off:nwc=1.5:sagn=off:sio=off:spl=sat:ssnc=none_900");
    fallback.push("dis+4_10_bs=off:cond=fast:flr=on:fsr=off:gsp=input_only:lcm=predicate:nwc=1.2:sio=off:spl=sat:ssfp=1000:ssfq=1.0:ssnc=none:sp=occurrence:updr=off_300");
    fallback.push("tab+10_1_spl=off:tbsr=off:tfsr=off:tgawr=3/1:tglr=1/20:tipr=off:tlawr=8/1_300");
    fallback.push("ins+10_1_gsp=input_only:igbrr=0.9:igrp=100:igrpq=1.1:nwc=2_300");
    fallback.push("ott-1002_3_bs=unit_only:bsr=unit_only:cond=on:gsp=input_only:nwc=1.2:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=2.0:ssnc=none:urr=on_300");
    fallback.push("ott+1_40_bsr=unit_only:cond=fast:gs=on:nwc=1.5:sio=off:spl=sat:ssnc=none:urr=on_300");
    fallback.push("dis+3_6_flr=on:fsr=off:nwc=5:nicw=on:sio=off:spl=sat:sser=off:ssfp=100000:ssfq=1.2:ssnc=none:sp=occurrence_300");
    fallback.push("dis+1010_8_bs=off:nwc=1.1:sio=off_300");
    fallback.push("ott-1010_128_bs=off:cond=on:lcm=predicate:nwc=1.3:urr=on_600");
    fallback.push("ott+10_1024_bs=off:fsr=off:nwc=1.5:nicw=on:sio=off:spl=off_1200");
    fallback.push("lrs+1002_128_bs=off:cond=on:flr=on:lcm=reverse:nwc=1:spo=on_1200");
    break;

  case Property::UEQ:
    fallback.push("lrs+10_8_bs=off:cond=fast:drc=off:fsr=off:gsp=input_only:gs=on:nwc=2:urr=on_1200");
    fallback.push("lrs+10_14_drc=off:fde=none:nwc=2.5_1800");
    fallback.push("ott+10_50_bs=off:drc=off:fsr=off:fde=none:nwc=1_300");
    fallback.push("lrs+10_1024_nwc=4_1500");
    fallback.push("ott+10_8:1_bs=off:drc=off:fsr=off:nwc=1.7_600");
    fallback.push("ott+10_7_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.5_600");
    fallback.push("dis+4_5:1_bs=off:br=off:ep=RSTC:fsr=off:nwc=3:sos=all:sio=off:spl=sat:ssfq=1.4:ssnc=none:urr=on_300");
    fallback.push("ott+2_10_bs=off:drc=off:ep=on:fsr=off:fde=none:gsp=input_only:nwc=1.7:nicw=on:sio=off:spo=on:spl=sat:sser=off:ssfq=1.1:ssnc=none_200");
    fallback.push("lrs+10_20_bd=off:bs=unit_only:drc=off:fsr=off:nwc=1.2_900");
    fallback.push("dis+2_14_bs=off:br=off:ep=RSTC:flr=on:fsr=off:nwc=2.5:nicw=on:sos=all:sio=off:spl=sat:ssnc=none:urr=on_100");
    fallback.push("lrs+10_5:1_nwc=1_900");
    fallback.push("lrs+10_2:3_bs=unit_only:cond=on:drc=off:ep=on:fsr=off:gs=on:nwc=1:sio=off:spl=sat:sser=off:ssfq=1.2:ssnc=none:sp=occurrence_200");
    fallback.push("ott+10_20_bs=off:drc=off:fsr=off:gsp=input_only:nwc=1.3:sp=occurrence_600");
    fallback.push("ott+10_8:1_bs=off:bsr=on:nwc=4_300");
    fallback.push("ott+10_64_drc=off:nwc=1.1_400");
    fallback.push("ott+10_2:1_drc=off:nwc=5_1200");
    fallback.push("ott+10_20_bd=off:drc=off:nwc=2:sp=occurrence_600");
    fallback.push("ott+10_2:3_drc=off:nwc=2_600");
    fallback.push("ott+10_5_bd=off:drc=off:nwc=2.5_900");
    fallback.push("ott+10_2_fde=none:nwc=2.5:sp=reverse_arity_900");
    fallback.push("ott+10_8:1_nwc=3:sfv=off_900");
    break;
  }
}

void Schedules::getLtb2015Hh4SlowSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 20000) {
    sched.push("dis+11_4:1_bsr=unit_only:ccuc=small_ones:cond=on:nm=64:nwc=3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sscc=on:sfr=on:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=occurrence:updr=off_1");
    sched.push("dis+10_24_cond=fast:ep=RST:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:gsssp=full:nm=0:nwc=1:sas=minisat:sd=3:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:ssfp=1000:ssfq=1.4:smm=sco:ssnc=none:sp=occurrence_2");
    sched.push("lrs+4_3:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:lwlo=on:nm=0:nwc=1.7:stl=34:spl=off:sp=reverse_arity:updr=off_2");
    sched.push("dis+11_3_bd=off:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:sos=all:spl=off:updr=off_8");
    sched.push("dis+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on:uhcvi=on_8");
    sched.push("ott+11_3_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:spl=off:sp=occurrence:urr=on:uhcvi=on_8");
    sched.push("dis+10_7_bs=unit_only:cond=on:fde=none:gs=on:gsem=on:gsssp=full:lwlo=on:nm=0:nwc=2.5:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=1000:ssfq=1.0:smm=sco:ssnc=all_dependent:sp=occurrence:updr=off_9");
    sched.push("ins+11_4_cond=on:fde=none:gs=on:gsem=off:gsssp=full:igbrr=0.5:igpr=on:igrr=1/4:igrp=4000:igrpq=1.05:igs=1:igwr=on:nm=0:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.2:sos=all:spl=off:sp=reverse_arity:updr=off:uhcvi=on_11");
    sched.push("dis+11_1_cond=fast:ep=RST:fde=none:nm=1024:nwc=2:sd=2:ss=priority:st=1.5:sdd=off:ssfp=100000:ssfq=1.1:smm=off:sp=occurrence:urr=on:updr=off:uhcvi=on_11");
    sched.push("dis+11_3_cond=fast:ep=RST:gs=on:gsaa=full_model:gsem=off:lwlo=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:sac=on:sdd=large:sfr=on:ssfp=10000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:updr=off_16");
    sched.push("lrs+11_5:4_fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=occurrence:uhcvi=on_18");
    sched.push("dis-2_3_bd=off:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:urr=ec_only:uhcvi=on_19");
    sched.push("dis-2_1_cond=fast:fsr=off:fde=unused:nwc=10:sd=2:ss=priority:st=2.0:spl=off:sp=occurrence:updr=off_20");
    sched.push("dis+11_1_cond=fast:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:spl=off:updr=off:uhcvi=on_20");
    sched.push("dis+11_3_ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:gsssp=full:nm=0:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sac=on:smm=sco:ssnc=none:urr=ec_only:updr=off_21");
    sched.push("lrs+11_2_bsr=unit_only:fsr=off:fde=none:gsp=input_only:gs=on:nwc=1:stl=34:sd=1:ss=axioms:st=3.0:sos=on:sdd=off:ssfp=10000:ssfq=2.0:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_21");
    sched.push("dis+4_4_bsr=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:nm=64:nwc=1:sd=2:ss=priority:st=5.0:sac=on:sdd=off:sfr=on:ssfp=4000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity_21");
    sched.push("lrs+1010_2_bs=unit_only:bsr=unit_only:ccuc=first:cond=on:fsr=off:fde=unused:gs=on:gsssp=full:nm=0:nwc=1.5:nicw=on:sas=minisat:stl=34:sd=2:ss=axioms:st=5.0:sos=on:sac=on:sscc=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_23");
    sched.push("dis+10_4_bd=off:fsr=off:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=64:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:sdd=large:sser=off:ssfp=4000:ssfq=1.1:ssnc=none:sp=occurrence:urr=ec_only:updr=off_24");
    sched.push("lrs+11_5_bd=off:cond=fast:fsr=off:fde=none:nm=0:nwc=1:stl=34:sd=2:ss=axioms:st=3.0:sos=all:spl=off:updr=off:uhcvi=on_29");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:updr=off_31");
    sched.push("dis+11_3_br=off:cond=on:ep=RST:fsr=off:fde=none:gsp=input_only:gs=on:nwc=1:sd=2:ss=axioms:sos=all:sac=on:sdd=off:ssfp=40000:ssfq=1.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off_31");
    sched.push("lrs+11_3:1_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:sas=minisat:stl=34:sd=2:ss=priority:sac=on:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_34");
    sched.push("lrs+1003_4_bd=off:bsr=unit_only:cond=on:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=occurrence:urr=on:updr=off_35");
    sched.push("lrs+11_3_bd=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:st=1.5:sos=all:spl=off:sp=reverse_arity:uhcvi=on_36");
    sched.push("lrs+11_2_cond=on:fde=unused:gs=on:gsssp=full:nm=64:nwc=1.5:sas=minisat:stl=34:sd=3:ss=axioms:spl=off:sp=reverse_arity:updr=off_39");
    sched.push("ott+1011_5:4_fde=unused:gs=on:gsem=off:nwc=1.3:sas=minisat:sd=4:ss=priority:st=5.0:sdd=off:ssfp=1000:ssfq=1.2:smm=sco:sp=reverse_arity:urr=on_41");
    sched.push("dis+2_1_fsr=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_44");
    sched.push("lrs+11_5:4_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=occurrence:uhcvi=on_46");
    sched.push("dis+11_4:1_bsr=unit_only:ccuc=small_ones:fsr=off:nm=64:nwc=3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sscc=on:sfr=on:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=occurrence:updr=off_51");
    sched.push("lrs+11_5_bd=off:cond=fast:gs=on:gsssp=full:nwc=1.1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=all:spl=off:uhcvi=on_55");
    sched.push("dis+11_6_fsr=off:fde=none:gs=on:gsem=on:gsssp=full:lwlo=on:nm=0:nwc=1:sd=3:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=4000:ssfq=1.0:smm=sco:ssnc=all:sp=occurrence:urr=ec_only:uhcvi=on_57");
    sched.push("lrs+10_5:4_fde=unused:gs=on:gsem=on:gsssp=full:nm=0:nwc=5:sas=minisat:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_62");
    sched.push("ott+2_2:1_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:sd=3:ss=priority:st=1.5:sos=on:spl=off:sp=occurrence:updr=off_62");
    sched.push("dis+4_3_bd=off:cond=on:fde=unused:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=3.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:urr=ec_only:updr=off:uhcvi=on_63");
    sched.push("ins+11_3_cond=fast:fsr=off:fde=none:igbrr=0.6:igrr=1/16:igrp=1400:igrpq=1.1:igs=1002:igwr=on:nm=0:nwc=1:sd=2:ss=priority:spl=off:urr=ec_only:dm=on:uhcvi=on_64");
    sched.push("ins+11_5_cond=fast:ep=RST:gs=on:gsem=on:igbrr=0.4:igpr=on:igrr=1/64:igrp=4000:igrpq=1.3:igwr=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:spl=off:sp=occurrence:dm=on:uhcvi=on_64");
    sched.push("dis+1011_3_cond=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sfr=on:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_83");
    sched.push("dis+11_4_cond=on:gsp=input_only:gs=on:nm=0:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on:updr=off:uhcvi=on_93");
    sched.push("lrs-3_8:1_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=1:sas=minisat:stl=34:sd=4:ss=priority:st=3.0:spl=off:sp=occurrence_98");
    sched.push("lrs+1_2_ep=RS:gsp=input_only:gs=on:gsaa=from_current:nwc=1:stl=34:sd=1:ss=axioms:sos=all:sac=on:sfr=on:ssfp=40000:ssfq=2.0:ssnc=none_101");
    sched.push("lrs+11_2:3_br=off:cond=on:fde=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sos=all:spl=off:sp=occurrence:urr=on:updr=off_111");
    sched.push("dis+11_3:1_br=off:cond=fast:fde=unused:gs=on:gsem=off:nm=0:nwc=1.7:sd=1:ss=axioms:st=1.5:sac=on:sdd=off:sser=off:sfr=on:ssfp=10000:ssfq=1.4:ssnc=none:urr=on:uhcvi=on_113");
    sched.push("lrs+11_2:3_cond=on:fde=unused:gs=on:gsaa=full_model:nwc=4:sas=minisat:stl=34:sd=2:ss=priority:st=5.0:sac=on:sdd=off:sfr=on:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_129");
    sched.push("dis+10_3:1_bsr=unit_only:cond=fast:fde=none:nm=64:nwc=1:sd=2:ss=axioms:sos=all:spl=off:sp=reverse_arity:updr=off_138");
    sched.push("lrs+11_4:1_cond=fast:fde=none:nm=0:nwc=1:sas=minisat:stl=34:sd=3:ss=priority:st=2.0:spl=off:sp=occurrence:urr=ec_only_164");
    sched.push("lrs+11_4_fde=unused:gsp=input_only:lcm=predicate:nm=0:nwc=1.3:stl=34:sd=1:ss=axioms:st=2.0:spl=off:sp=occurrence:urr=on:uhcvi=on_166");
    sched.push("dis+11_8_bd=preordered:br=off:cond=fast:fsr=off:fde=none:gsp=input_only:nm=0:nwc=5:sd=2:ss=axioms:st=1.2:spl=off:sp=occurrence:urr=on_193");
    sched.push("lrs+1003_8:1_br=off:cond=on:fde=none:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on_226");
    sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_251");
    sched.push("dis+1002_4_cond=fast:ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:sac=on:sdd=large:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:updr=off:uhcvi=on_274");
    sched.push("lrs+11_5_cond=on:ep=RST:fde=none:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sdd=large:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:urr=ec_only:uhcvi=on_328");
  }
  else {
    sched.push("ins+11_4_ep=RST:fsr=off:igrr=1/16:igrp=400:igrpq=2.0:igs=1:igwr=on:nm=0:nwc=1.3:sd=1:ss=axioms:st=5.0:spl=off:dm=on_19");
    sched.push("dis+11_4_cond=fast:fde=unused:gs=on:gsssp=full:nwc=1:sd=1:ss=axioms:st=1.2:sos=all:spl=off:sp=reverse_arity:urr=on:updr=off:uhcvi=on_19");
    sched.push("dis+11_6_fsr=off:fde=none:gs=on:gsem=on:gsssp=full:lwlo=on:nm=0:nwc=1:sd=3:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=4000:ssfq=1.0:smm=sco:ssnc=all:sp=occurrence:urr=ec_only:uhcvi=on_19");
    sched.push("dis+11_3_ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:gsssp=full:nm=0:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sac=on:smm=sco:ssnc=none:urr=ec_only:updr=off_20");
    sched.push("dis-2_3_bd=off:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:urr=ec_only:uhcvi=on_27");
    sched.push("dis+4_3_bd=off:cond=fast:fsr=off:fde=unused:gs=on:gsssp=full:lwlo=on:nm=64:nwc=1:sas=minisat:ss=axioms:st=2.0:sos=on:spl=off:sp=occurrence:updr=off_32");
    sched.push("dis+11_1_cond=fast:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:spl=off:updr=off:uhcvi=on_39");
    sched.push("lrs+10_5:4_fde=unused:gs=on:gsem=on:gsssp=full:nm=0:nwc=5:sas=minisat:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_41");
    sched.push("lrs+11_2_cond=on:fde=unused:gs=on:gsssp=full:nm=64:nwc=1.5:sas=minisat:stl=34:sd=3:ss=axioms:spl=off:sp=reverse_arity:updr=off_41");
    sched.push("dis-2_1_cond=fast:fsr=off:fde=unused:nwc=10:sd=2:ss=priority:st=2.0:spl=off:sp=occurrence:updr=off_44");
    sched.push("dis+10_3:1_bsr=unit_only:cond=fast:fde=none:nm=64:nwc=1:sd=2:ss=axioms:sos=all:spl=off:sp=reverse_arity:updr=off_48");
    sched.push("dis+11_3:1_br=off:cond=fast:fde=unused:gs=on:gsem=off:nm=0:nwc=1.7:sd=1:ss=axioms:st=1.5:sac=on:sdd=off:sser=off:sfr=on:ssfp=10000:ssfq=1.4:ssnc=none:urr=on:uhcvi=on_50");
    sched.push("lrs+11_4:1_cond=fast:fde=none:nm=0:nwc=1:sas=minisat:stl=34:sd=3:ss=priority:st=2.0:spl=off:sp=occurrence:urr=ec_only_51");
    sched.push("dis+1011_3_cond=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sfr=on:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_63");
    sched.push("ins+11_5_cond=fast:ep=RST:gs=on:gsem=on:igbrr=0.4:igpr=on:igrr=1/64:igrp=4000:igrpq=1.3:igwr=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:spl=off:sp=occurrence:dm=on:uhcvi=on_65");
    sched.push("ins+11_5_cond=fast:igbrr=0.1:igrr=1/32:igrp=1400:igs=1004:igwr=on:nwc=1:sd=2:ss=axioms:sos=on:spl=off:urr=on:updr=off_70");
    sched.push("dis+4_3_bd=off:cond=on:fde=unused:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=3.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:urr=ec_only:updr=off:uhcvi=on_70");
    sched.push("ins+11_5_cond=on:fsr=off:fde=none:gs=on:gsem=on:gsssp=full:igbrr=0.4:igrr=1/8:igrp=1400:igrpq=1.5:igwr=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_71");
    sched.push("lrs+11_4_fde=unused:gsp=input_only:lcm=predicate:nm=0:nwc=1.3:stl=34:sd=1:ss=axioms:st=2.0:spl=off:sp=occurrence:urr=on:uhcvi=on_79");
    sched.push("dis+11_4_cond=on:gsp=input_only:gs=on:nm=0:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on:updr=off:uhcvi=on_80");
    sched.push("ott+11_3_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:spl=off:sp=occurrence:urr=on:uhcvi=on_83");
    sched.push("dis+2_1_fsr=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_88");
    sched.push("lrs+1003_4_bd=off:bsr=unit_only:cond=on:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=occurrence:urr=on:updr=off_90");
    sched.push("dis+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on:uhcvi=on_98");
    sched.push("lrs+11_5_bd=off:cond=fast:gs=on:gsssp=full:nwc=1.1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=all:spl=off:uhcvi=on_98");
    sched.push("lrs+11_3_bd=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:st=1.5:sos=all:spl=off:sp=reverse_arity:uhcvi=on_105");
    sched.push("lrs+11_2:3_br=off:cond=on:fde=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sos=all:spl=off:sp=occurrence:urr=on:updr=off_108");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:updr=off_111");
    sched.push("lrs+1003_8:1_br=off:cond=on:fde=none:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on_116");
    sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_117");
    sched.push("dis+11_4:1_bsr=unit_only:ccuc=small_ones:fsr=off:nm=64:nwc=3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sscc=on:sfr=on:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=occurrence:updr=off_120");
    sched.push("lrs+11_2:3_cond=on:fde=unused:gs=on:gsaa=full_model:nwc=4:sas=minisat:stl=34:sd=2:ss=priority:st=5.0:sac=on:sdd=off:sfr=on:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_151");
    sched.push("lrs+1010_2_bs=unit_only:bsr=unit_only:ccuc=first:cond=on:fsr=off:fde=unused:gs=on:gsssp=full:nm=0:nwc=1.5:nicw=on:sas=minisat:stl=34:sd=2:ss=axioms:st=5.0:sos=on:sac=on:sscc=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_152");
    sched.push("dis+1002_4_cond=fast:ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:sac=on:sdd=large:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:updr=off:uhcvi=on_248");
    sched.push("lrs+11_3:1_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:sas=minisat:stl=34:sd=2:ss=priority:sac=on:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_285");
    sched.push("ott+2_2:1_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:sd=3:ss=priority:st=1.5:sos=on:spl=off:sp=occurrence:updr=off_304");
    sched.push("lrs+11_5_cond=on:ep=RST:fde=none:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sdd=large:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:urr=ec_only:uhcvi=on_346");
  }
}

void Schedules::getLtb2015Hh4MiddSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 20000) {
    sched.push("lrs+4_3:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:lwlo=on:nm=0:nwc=1.7:stl=34:spl=off:sp=reverse_arity:updr=off_2");
    sched.push("dis+10_24_cond=fast:ep=RST:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:gsssp=full:nm=0:nwc=1:sas=minisat:sd=3:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:ssfp=1000:ssfq=1.4:smm=sco:ssnc=none:sp=occurrence_2");
    sched.push("dis+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on:uhcvi=on_8");
    sched.push("dis+11_3_bd=off:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:sos=all:spl=off:updr=off_8");
    sched.push("dis+10_7_bs=unit_only:cond=on:fde=none:gs=on:gsem=on:gsssp=full:lwlo=on:nm=0:nwc=2.5:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=1000:ssfq=1.0:smm=sco:ssnc=all_dependent:sp=occurrence:updr=off_9");
    sched.push("dis+11_1_cond=fast:ep=RST:fde=none:nm=1024:nwc=2:sd=2:ss=priority:st=1.5:sdd=off:ssfp=100000:ssfq=1.1:smm=off:sp=occurrence:urr=on:updr=off:uhcvi=on_11");
    sched.push("ins+11_4_cond=on:fde=none:gs=on:gsem=off:gsssp=full:igbrr=0.5:igpr=on:igrr=1/4:igrp=4000:igrpq=1.05:igs=1:igwr=on:nm=0:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.2:sos=all:spl=off:sp=reverse_arity:updr=off:uhcvi=on_11");
    sched.push("lrs+11_4:1_cond=fast:fde=none:nm=0:nwc=1:sas=minisat:stl=34:sd=3:ss=priority:st=2.0:spl=off:sp=occurrence:urr=ec_only_16");
    sched.push("lrs+1_2_ep=RS:gsp=input_only:gs=on:gsaa=from_current:nwc=1:stl=34:sd=1:ss=axioms:sos=all:sac=on:sfr=on:ssfp=40000:ssfq=2.0:ssnc=none_16");
    sched.push("dis+11_3_cond=fast:ep=RST:gs=on:gsaa=full_model:gsem=off:lwlo=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:sac=on:sdd=large:sfr=on:ssfp=10000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:updr=off_17");
    sched.push("lrs+11_5:4_fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=occurrence:uhcvi=on_18");
    sched.push("lrs+1003_4_bd=off:bsr=unit_only:cond=on:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=occurrence:urr=on:updr=off_19");
    sched.push("dis-2_3_bd=off:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:urr=ec_only:uhcvi=on_19");
    sched.push("dis-2_1_cond=fast:fsr=off:fde=unused:nwc=10:sd=2:ss=priority:st=2.0:spl=off:sp=occurrence:updr=off_20");
    sched.push("dis+11_1_cond=fast:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:spl=off:updr=off:uhcvi=on_20");
    sched.push("lrs+11_2_bsr=unit_only:fsr=off:fde=none:gsp=input_only:gs=on:nwc=1:stl=34:sd=1:ss=axioms:st=3.0:sos=on:sdd=off:ssfp=10000:ssfq=2.0:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_21");
    sched.push("dis+4_4_bsr=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:nm=64:nwc=1:sd=2:ss=priority:st=5.0:sac=on:sdd=off:sfr=on:ssfp=4000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity_21");
    sched.push("dis+11_3_ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:gsssp=full:nm=0:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sac=on:smm=sco:ssnc=none:urr=ec_only:updr=off_21");
    sched.push("lrs+1010_2_bs=unit_only:bsr=unit_only:ccuc=first:cond=on:fsr=off:fde=unused:gs=on:gsssp=full:nm=0:nwc=1.5:nicw=on:sas=minisat:stl=34:sd=2:ss=axioms:st=5.0:sos=on:sac=on:sscc=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_23");
    sched.push("dis+10_4_bd=off:fsr=off:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=64:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:sdd=large:sser=off:ssfp=4000:ssfq=1.1:ssnc=none:sp=occurrence:urr=ec_only:updr=off_24");
    sched.push("dis+11_3_br=off:cond=on:ep=RST:fsr=off:fde=none:gsp=input_only:gs=on:nwc=1:sd=2:ss=axioms:sos=all:sac=on:sdd=off:ssfp=40000:ssfq=1.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off_31");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:updr=off_31");
    sched.push("lrs+11_5_bd=off:cond=fast:gs=on:gsssp=full:nwc=1.1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=all:spl=off:uhcvi=on_32");
    sched.push("dis+1011_3_cond=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sfr=on:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_33");
    sched.push("ins+11_5_cond=fast:ep=RST:gs=on:gsem=on:igbrr=0.4:igpr=on:igrr=1/64:igrp=4000:igrpq=1.3:igwr=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:spl=off:sp=occurrence:dm=on:uhcvi=on_34");
    sched.push("lrs+11_3:1_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:sas=minisat:stl=34:sd=2:ss=priority:sac=on:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_34");
    sched.push("lrs+11_2_cond=on:fde=unused:gs=on:gsssp=full:nm=64:nwc=1.5:sas=minisat:stl=34:sd=3:ss=axioms:spl=off:sp=reverse_arity:updr=off_39");
    sched.push("ott+1011_5:4_fde=unused:gs=on:gsem=off:nwc=1.3:sas=minisat:sd=4:ss=priority:st=5.0:sdd=off:ssfp=1000:ssfq=1.2:smm=sco:sp=reverse_arity:urr=on_41");
    sched.push("lrs+11_5:4_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=occurrence:uhcvi=on_46");
    sched.push("dis+11_6_fsr=off:fde=none:gs=on:gsem=on:gsssp=full:lwlo=on:nm=0:nwc=1:sd=3:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=4000:ssfq=1.0:smm=sco:ssnc=all:sp=occurrence:urr=ec_only:uhcvi=on_57");
    sched.push("ott+2_2:1_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:sd=3:ss=priority:st=1.5:sos=on:spl=off:sp=occurrence:updr=off_62");
    sched.push("lrs+10_5:4_fde=unused:gs=on:gsem=on:gsssp=full:nm=0:nwc=5:sas=minisat:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_62");
    sched.push("dis+4_3_bd=off:cond=on:fde=unused:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=3.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:urr=ec_only:updr=off:uhcvi=on_63");
    sched.push("ins+11_3_cond=fast:fsr=off:fde=none:igbrr=0.6:igrr=1/16:igrp=1400:igrpq=1.1:igs=1002:igwr=on:nm=0:nwc=1:sd=2:ss=priority:spl=off:urr=ec_only:dm=on:uhcvi=on_64");
    sched.push("lrs+1003_8:1_br=off:cond=on:fde=none:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on_66");
    sched.push("lrs+11_2:3_cond=on:fde=unused:gs=on:gsaa=full_model:nwc=4:sas=minisat:stl=34:sd=2:ss=priority:st=5.0:sac=on:sdd=off:sfr=on:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_87");
    sched.push("lrs-3_8:1_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=1:sas=minisat:stl=34:sd=4:ss=priority:st=3.0:spl=off:sp=occurrence_89");
    sched.push("dis+11_4_cond=on:gsp=input_only:gs=on:nm=0:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on:updr=off:uhcvi=on_93");
    sched.push("lrs+11_2:3_br=off:cond=on:fde=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sos=all:spl=off:sp=occurrence:urr=on:updr=off_111");
    sched.push("dis+11_3:1_br=off:cond=fast:fde=unused:gs=on:gsem=off:nm=0:nwc=1.7:sd=1:ss=axioms:st=1.5:sac=on:sdd=off:sser=off:sfr=on:ssfp=10000:ssfq=1.4:ssnc=none:urr=on:uhcvi=on_113");
    sched.push("dis+10_3:1_bsr=unit_only:cond=fast:fde=none:nm=64:nwc=1:sd=2:ss=axioms:sos=all:spl=off:sp=reverse_arity:updr=off_138");
    sched.push("dis+1002_4_cond=fast:ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:sac=on:sdd=large:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:updr=off:uhcvi=on_202");
    sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_234");
    sched.push("lrs+11_5_cond=on:ep=RST:fde=none:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sdd=large:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:urr=ec_only:uhcvi=on_290");
  }
  else {
    sched.push("dis+11_6_fsr=off:fde=none:gs=on:gsem=on:gsssp=full:lwlo=on:nm=0:nwc=1:sd=3:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=4000:ssfq=1.0:smm=sco:ssnc=all:sp=occurrence:urr=ec_only:uhcvi=on_19");
    sched.push("dis-2_3_bd=off:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:urr=ec_only:uhcvi=on_27");
    sched.push("lrs+10_5:4_fde=unused:gs=on:gsem=on:gsssp=full:nm=0:nwc=5:sas=minisat:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_41");
    sched.push("lrs+11_2_cond=on:fde=unused:gs=on:gsssp=full:nm=64:nwc=1.5:sas=minisat:stl=34:sd=3:ss=axioms:spl=off:sp=reverse_arity:updr=off_41");
    sched.push("dis-2_1_cond=fast:fsr=off:fde=unused:nwc=10:sd=2:ss=priority:st=2.0:spl=off:sp=occurrence:updr=off_44");
    sched.push("dis+10_3:1_bsr=unit_only:cond=fast:fde=none:nm=64:nwc=1:sd=2:ss=axioms:sos=all:spl=off:sp=reverse_arity:updr=off_48");
    sched.push("lrs+11_5_bd=off:cond=fast:gs=on:gsssp=full:nwc=1.1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=all:spl=off:uhcvi=on_56");
    sched.push("dis+1011_3_cond=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sfr=on:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_63");
    sched.push("ins+11_5_cond=fast:ep=RST:gs=on:gsem=on:igbrr=0.4:igpr=on:igrr=1/64:igrp=4000:igrpq=1.3:igwr=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:spl=off:sp=occurrence:dm=on:uhcvi=on_65");
    sched.push("dis+4_3_bd=off:cond=on:fde=unused:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=3.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:urr=ec_only:updr=off:uhcvi=on_70");
    sched.push("ins+11_5_cond=on:fsr=off:fde=none:gs=on:gsem=on:gsssp=full:igbrr=0.4:igrr=1/8:igrp=1400:igrpq=1.5:igwr=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_71");
    sched.push("lrs+11_4_fde=unused:gsp=input_only:lcm=predicate:nm=0:nwc=1.3:stl=34:sd=1:ss=axioms:st=2.0:spl=off:sp=occurrence:urr=on:uhcvi=on_79");
    sched.push("dis+11_4_cond=on:gsp=input_only:gs=on:nm=0:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on:updr=off:uhcvi=on_80");
    sched.push("lrs+1003_8:1_br=off:cond=on:fde=none:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on_82");
    sched.push("lrs+11_3_bd=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:st=1.5:sos=all:spl=off:sp=reverse_arity:uhcvi=on_87");
    sched.push("dis+1002_4_cond=fast:ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:sac=on:sdd=large:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:updr=off:uhcvi=on_88");
    sched.push("dis+2_1_fsr=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_88");
    sched.push("lrs+1003_4_bd=off:bsr=unit_only:cond=on:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=occurrence:urr=on:updr=off_90");
    sched.push("lrs+1010_2_bs=unit_only:bsr=unit_only:ccuc=first:cond=on:fsr=off:fde=unused:gs=on:gsssp=full:nm=0:nwc=1.5:nicw=on:sas=minisat:stl=34:sd=2:ss=axioms:st=5.0:sos=on:sac=on:sscc=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_92");
    sched.push("dis+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on:uhcvi=on_98");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:updr=off_98");
    sched.push("lrs+11_2:3_br=off:cond=on:fde=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sos=all:spl=off:sp=occurrence:urr=on:updr=off_108");
    sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_117");
    sched.push("lrs+11_3:1_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:sas=minisat:stl=34:sd=2:ss=priority:sac=on:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_119");
    sched.push("lrs+11_2:3_cond=on:fde=unused:gs=on:gsaa=full_model:nwc=4:sas=minisat:stl=34:sd=2:ss=priority:st=5.0:sac=on:sdd=off:sfr=on:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_151");
    sched.push("ott+2_2:1_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:sd=3:ss=priority:st=1.5:sos=on:spl=off:sp=occurrence:updr=off_162");
    sched.push("lrs+11_5_cond=on:ep=RST:fde=none:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sdd=large:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:urr=ec_only:uhcvi=on_292");
  }
}

void Schedules::getLtb2015Hh4FastSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 20000) {
    sched.push("dis+11_4:1_bsr=unit_only:ccuc=small_ones:cond=on:nm=64:nwc=3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sscc=on:sfr=on:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=occurrence:updr=off_1");
    sched.push("dis+10_24_cond=fast:ep=RST:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:gsssp=full:nm=0:nwc=1:sas=minisat:sd=3:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:ssfp=1000:ssfq=1.4:smm=sco:ssnc=none:sp=occurrence_2");
    sched.push("lrs+11_5:4_cond=fast:fde=none:gs=on:gsaa=from_current:gsem=on:nwc=1:sas=minisat:stl=34:sd=7:ss=axioms:st=3.0:sos=all:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_2");
    sched.push("lrs+4_3:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:lwlo=on:nm=0:nwc=1.7:stl=34:spl=off:sp=reverse_arity:updr=off_2");
    sched.push("dis+11_3_bd=off:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:sos=all:spl=off:updr=off_5");
    sched.push("ott+1011_5:4_fde=unused:gs=on:gsem=off:nwc=1.3:sas=minisat:sd=4:ss=priority:st=5.0:sdd=off:ssfp=1000:ssfq=1.2:smm=sco:sp=reverse_arity:urr=on_5");
    sched.push("dis+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on:uhcvi=on_8");
    sched.push("dis+10_7_bs=unit_only:cond=on:fde=none:gs=on:gsem=on:gsssp=full:lwlo=on:nm=0:nwc=2.5:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=1000:ssfq=1.0:smm=sco:ssnc=all_dependent:sp=occurrence:updr=off_9");
    sched.push("lrs-3_8:1_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=1:sas=minisat:stl=34:sd=4:ss=priority:st=3.0:spl=off:sp=occurrence_10");
    sched.push("ins+11_4_cond=on:fde=none:gs=on:gsem=off:gsssp=full:igbrr=0.5:igpr=on:igrr=1/4:igrp=4000:igrpq=1.05:igs=1:igwr=on:nm=0:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.2:sos=all:spl=off:sp=reverse_arity:updr=off:uhcvi=on_11");
    sched.push("lrs+11_3:1_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:sas=minisat:stl=34:sd=2:ss=priority:sac=on:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_14");
    sched.push("dis+1011_3_cond=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sfr=on:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_16");
    sched.push("lrs+1_2_ep=RS:gsp=input_only:gs=on:gsaa=from_current:nwc=1:stl=34:sd=1:ss=axioms:sos=all:sac=on:sfr=on:ssfp=40000:ssfq=2.0:ssnc=none_16");
    sched.push("dis+11_3_cond=fast:ep=RST:gs=on:gsaa=full_model:gsem=off:lwlo=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:sac=on:sdd=large:sfr=on:ssfp=10000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:updr=off_17");
    sched.push("dis-2_3_bd=off:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:urr=ec_only:uhcvi=on_19");
    sched.push("lrs+1003_4_bd=off:bsr=unit_only:cond=on:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=occurrence:urr=on:updr=off_19");
    sched.push("dis+11_3_br=off:cond=on:ep=RST:fsr=off:fde=none:gsp=input_only:gs=on:nwc=1:sd=2:ss=axioms:sos=all:sac=on:sdd=off:ssfp=40000:ssfq=1.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off_19");
    sched.push("dis+11_1_cond=fast:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:spl=off:updr=off:uhcvi=on_20");
    sched.push("lrs+1010_2_bs=unit_only:bsr=unit_only:ccuc=first:cond=on:fsr=off:fde=unused:gs=on:gsssp=full:nm=0:nwc=1.5:nicw=on:sas=minisat:stl=34:sd=2:ss=axioms:st=5.0:sos=on:sac=on:sscc=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_23");
    sched.push("lrs+11_5_bd=off:cond=fast:gs=on:gsssp=full:nwc=1.1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=all:spl=off:uhcvi=on_24");
    sched.push("dis+1002_4_cond=fast:ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:sac=on:sdd=large:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:updr=off:uhcvi=on_26");
    sched.push("lrs+11_5:4_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=occurrence:uhcvi=on_27");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:updr=off_31");
    sched.push("ins+11_5_cond=fast:ep=RST:gs=on:gsem=on:igbrr=0.4:igpr=on:igrr=1/64:igrp=4000:igrpq=1.3:igwr=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:spl=off:sp=occurrence:dm=on:uhcvi=on_34");
    sched.push("lrs+11_2_cond=on:fde=unused:gs=on:gsssp=full:nm=64:nwc=1.5:sas=minisat:stl=34:sd=3:ss=axioms:spl=off:sp=reverse_arity:updr=off_39");
    sched.push("ott+2_2:1_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:sd=3:ss=priority:st=1.5:sos=on:spl=off:sp=occurrence:updr=off_46");
    sched.push("lrs+10_5:4_fde=unused:gs=on:gsem=on:gsssp=full:nm=0:nwc=5:sas=minisat:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_62");
    sched.push("dis+4_3_bd=off:cond=on:fde=unused:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=3.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:urr=ec_only:updr=off:uhcvi=on_63");
    sched.push("lrs+1003_8:1_br=off:cond=on:fde=none:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on_66");
    sched.push("dis+11_3:1_br=off:cond=fast:fde=unused:gs=on:gsem=off:nm=0:nwc=1.7:sd=1:ss=axioms:st=1.5:sac=on:sdd=off:sser=off:sfr=on:ssfp=10000:ssfq=1.4:ssnc=none:urr=on:uhcvi=on_78");
    sched.push("dis+11_4_cond=on:gsp=input_only:gs=on:nm=0:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on:updr=off:uhcvi=on_82");
    sched.push("lrs+11_2:3_cond=on:fde=unused:gs=on:gsaa=full_model:nwc=4:sas=minisat:stl=34:sd=2:ss=priority:st=5.0:sac=on:sdd=off:sfr=on:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_87");
    sched.push("lrs+11_2:3_br=off:cond=on:fde=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sos=all:spl=off:sp=occurrence:urr=on:updr=off_93");
    sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_96");
    sched.push("lrs+11_5_cond=on:ep=RST:fde=none:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sdd=large:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:urr=ec_only:uhcvi=on_110");
  }
  else {
    sched.push("dis-2_3_bd=off:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:urr=ec_only:uhcvi=on_27");
    sched.push("ins+11_5_cond=on:fsr=off:fde=none:gs=on:gsem=on:gsssp=full:igbrr=0.4:igrr=1/8:igrp=1400:igrpq=1.5:igwr=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_27");
    sched.push("dis-2_1_cond=fast:fsr=off:fde=unused:nwc=10:sd=2:ss=priority:st=2.0:spl=off:sp=occurrence:updr=off_29");
    sched.push("dis+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on:uhcvi=on_36");
    sched.push("lrs+10_5:4_fde=unused:gs=on:gsem=on:gsssp=full:nm=0:nwc=5:sas=minisat:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_41");
    sched.push("ins+11_5_cond=fast:ep=RST:gs=on:gsem=on:igbrr=0.4:igpr=on:igrr=1/64:igrp=4000:igrpq=1.3:igwr=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:spl=off:sp=occurrence:dm=on:uhcvi=on_43");
    sched.push("dis+2_1_fsr=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_48");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:updr=off_51");
    sched.push("lrs+1003_4_bd=off:bsr=unit_only:cond=on:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=occurrence:urr=on:updr=off_55");
    sched.push("dis+11_4_cond=on:gsp=input_only:gs=on:nm=0:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on:updr=off:uhcvi=on_58");
    sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_64");
    sched.push("ott+2_2:1_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:sd=3:ss=priority:st=1.5:sos=on:spl=off:sp=occurrence:updr=off_70");
    sched.push("lrs+11_5_bd=off:cond=fast:gs=on:gsssp=full:nwc=1.1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=all:spl=off:uhcvi=on_73");
    sched.push("lrs+11_2:3_cond=on:fde=unused:gs=on:gsaa=full_model:nwc=4:sas=minisat:stl=34:sd=2:ss=priority:st=5.0:sac=on:sdd=off:sfr=on:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_79");
    sched.push("lrs+1003_8:1_br=off:cond=on:fde=none:gs=on:gsem=off:nm=0:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=occurrence:urr=on_82");
    sched.push("dis+1002_4_cond=fast:ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:sac=on:sdd=large:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:updr=off:uhcvi=on_88");
    sched.push("lrs+11_5_cond=on:ep=RST:fde=none:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sdd=large:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:urr=ec_only:uhcvi=on_96");
    sched.push("lrs+11_2:3_br=off:cond=on:fde=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sos=all:spl=off:sp=occurrence:urr=on:updr=off_100");
    sched.push("lrs+11_3:1_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:sas=minisat:stl=34:sd=2:ss=priority:sac=on:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_119");
  }
}

void Schedules::getLtb2015HllSlowSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 12000) {
    sched.push("lrs+1003_4_bsr=unit_only:cond=fast:fsr=off:gsp=input_only:gs=on:gsaa=from_current:nm=0:nwc=1:stl=34:sos=on:sac=on:sdd=large:ssfp=10000:ssfq=1.1:ssnc=none:urr=ec_only:uhcvi=on_6");
    sched.push("dis+1011_4:1_fsr=off:nwc=1:sd=3:ss=priority:sos=on:spl=off:sp=reverse_arity_6");
    sched.push("ins+11_4:1_cond=on:ep=RSTC:fde=none:gs=on:gsem=on:igbrr=0.2:igpr=on:igrr=32/1:igrp=2000:igrpq=1.3:igs=1002:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:sos=all:spl=off:sp=reverse_arity:uhcvi=on_8");
    sched.push("lrs+11_3_bsr=unit_only:cond=on:ep=RST:fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=10:ss=axioms:st=1.5:sos=all:sdd=off:ssfp=40000:ssfq=1.0:ssnc=none:sp=occurrence:urr=on_10");
    sched.push("lrs+10_5:4_bd=off:gs=on:gsssp=full:lcm=reverse:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("dis+11_1_cond=fast:fde=none:gs=on:gsssp=full:nwc=1.1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_11");
    sched.push("lrs+11_4_bd=off:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:sos=all:ssfp=1000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_11");
    sched.push("lrs+10_4_bsr=unit_only:cond=fast:fsr=off:fde=unused:gs=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=2.0:sos=on:ssfp=10000:ssfq=1.0:smm=sco:ssnc=all_dependent:sp=occurrence:uhcvi=on_12");
    sched.push("lrs+11_14_cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:nwc=1:stl=34:sos=on:spl=off:sp=occurrence_12");
    sched.push("dis+2_5_bd=off:cond=fast:gs=on:lcm=reverse:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_13");
    sched.push("lrs+11_2:3_bd=off:cond=on:fde=none:nwc=1:stl=34:sd=10:ss=axioms:st=1.2:sos=all:spl=off:sp=occurrence:updr=off:uhcvi=on_18");
    sched.push("lrs+1_7_bd=off:bsr=unit_only:nwc=1:sas=minisat:stl=34:sd=5:ss=axioms:st=1.2:sos=on:sser=off:ssfp=1000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_18");
    sched.push("dis+11_4_br=off:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=1.5:sos=all:sac=on:sdd=large:sfr=on:ssfp=10000:ssfq=1.2:ssnc=none:sp=occurrence:urr=on_19");
    sched.push("dis+10_1_bd=preordered:bs=unit_only:cond=on:fde=none:lcm=predicate:nwc=1:sas=minisat:sd=2:ss=axioms:sos=all:sac=on:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_19");
    sched.push("dis+1011_1_cond=on:fde=unused:gsp=input_only:nwc=1:sd=1:ss=axioms:sos=all:spl=off:sp=reverse_arity_20");
    sched.push("dis+10_4_bd=off:cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:spl=off:urr=on_21");
    sched.push("dis+1011_5:1_ep=RSTC:fde=unused:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_21");
    sched.push("lrs+10_4_bd=off:bsr=unit_only:fde=none:gs=on:lcm=reverse:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=3.0:sos=on:spl=off:uhcvi=on_32");
    sched.push("dis+1010_5:4_bd=off:fsr=off:fde=unused:gs=on:nm=64:nwc=1:sd=4:ss=axioms:st=1.2:sos=on:spl=off:sp=reverse_arity:uhcvi=on_32");
    sched.push("dis+11_3_ep=RSTC:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:sd=1:ss=axioms:st=2.0:sos=on:sac=on:sdd=large:sfr=on:ssfp=40000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:urr=on:uhcvi=on_33");
    sched.push("lrs+10_4_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:stl=34:sd=4:ss=axioms:st=5.0:sos=all:spl=off:sp=reverse_arity:updr=off:uhcvi=on_46");
    sched.push("lrs+10_4_bd=off:cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=1.5:sos=on:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_47");
    sched.push("dis+1002_7_bsr=unit_only:cond=fast:nm=64:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sac=on:sfr=on:ssfp=100000:ssfq=1.4:ssnc=none:uhcvi=on_48");
    sched.push("dis+10_3:1_ep=RST:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1.1:sd=2:ss=priority:st=2.0:sos=on:sac=on:sdd=large:sser=off:ssfp=10000:ssfq=1.1:ssnc=none:sp=reverse_arity_48");
    sched.push("dis+1002_3_cond=on:ep=RS:fsr=off:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sas=minisat:sd=5:ss=axioms:st=2.0:sos=on:ssfp=4000:ssfq=1.4:smm=off:ssnc=none:updr=off_49");
    sched.push("dis+11_1_fsr=off:fde=none:gs=on:gsem=on:nwc=1:sas=minisat:sd=3:ss=priority:sos=on:spl=off:urr=on:updr=off_51");
    sched.push("dis+10_3_bd=off:cond=on:gs=on:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=on:sac=on:ssfp=4000:ssfq=1.4:smm=off:ssnc=none:urr=on:updr=off:uhcvi=on_57");
    sched.push("dis+11_4_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsaa=from_current:nwc=1:sd=2:ss=axioms:sos=all:sdd=off:sser=off:ssnc=none:sp=occurrence:uhcvi=on_58");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sdd=off:ssfp=4000:ssfq=2.0:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_59");
    sched.push("dis+4_5:4_bd=off:fsr=off:fde=unused:gs=on:nwc=1:sd=5:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence:uhcvi=on_66");
    sched.push("lrs+4_3_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_66");
    sched.push("ins+11_5_ep=RS:fsr=off:gs=on:igbrr=0.4:igpr=on:igrr=1/2:igrp=4000:igrpq=1.2:igs=1010:igwr=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=reverse_arity:urr=on:updr=off_67");
    sched.push("dis+1011_2:1_cond=fast:ep=RST:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=1.1:ssnc=none:sp=reverse_arity_92");
    sched.push("lrs+1004_3:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:sos=on:spl=off:updr=off_133");
    sched.push("dis+1011_2:3_cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:sas=minisat:sd=2:ss=axioms:sos=on:sac=on:sdd=large:sfr=on:ssfp=1000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=ec_only:uhcvi=on_139");
    sched.push("lrs+10_3_bd=off:cond=fast:fsr=off:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=100000:ssfq=1.4:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_141");
    sched.push("dis+1002_1_ep=RST:gs=on:gsaa=full_model:gsem=on:nm=64:nwc=1:sas=minisat:sd=7:ss=axioms:st=1.2:sos=on:sser=off:ssfp=40000:ssfq=1.2:ssnc=none:updr=off:uhcvi=on_190");
    sched.push("lrs+11_5_br=off:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:stl=34:sd=3:ss=axioms:sos=all:sac=on:sdd=large:sfr=on:ssfp=4000:ssfq=1.2:smm=sco:ssnc=none:urr=on:uhcvi=on_202");
    sched.push("lrs+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:urr=on:updr=off:uhcvi=on_206");
    sched.push("lrs+11_8:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:spl=off:sp=occurrence:urr=on:updr=off_220");
    sched.push("lrs+4_5:4_bd=off:bs=on:bsr=unit_only:cond=fast:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=reverse_arity:uhcvi=on_251");
    sched.push("ins+11_3_ep=RST:fde=unused:gsp=input_only:igbrr=0.4:igrr=1/8:igrpq=1.5:igs=1:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:spl=off:updr=off:dm=on:uhcvi=on_272");
    sched.push("lrs+11_3:2_bsr=unit_only:cond=fast:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:stl=34:sd=2:ss=axioms:sdd=off:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_336");
    sched.push("lrs+11_3_bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=all:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on:updr=off_338");
  }
  else {
    sched.push("lrs+11_3_bd=off:bsr=unit_only:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=on:uhcvi=on_18");
    sched.push("lrs+11_3:2_bsr=unit_only:cond=fast:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:stl=34:sd=2:ss=axioms:sdd=off:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_19");
    sched.push("dis+11_4_br=off:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=1.5:sos=all:sac=on:sdd=large:sfr=on:ssfp=10000:ssfq=1.2:ssnc=none:sp=occurrence:urr=on_34");
    sched.push("dis+10_4_bd=off:cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:spl=off:urr=on_35");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sdd=off:ssfp=4000:ssfq=2.0:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_40");
    sched.push("lrs+10_5:4_bd=off:gs=on:gsssp=full:lcm=reverse:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_41");
    sched.push("lrs+10_4_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:stl=34:sd=4:ss=axioms:st=5.0:sos=all:spl=off:sp=reverse_arity:updr=off:uhcvi=on_52");
    sched.push("lrs+10_4_bd=off:bsr=unit_only:fde=none:gs=on:lcm=reverse:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=3.0:sos=on:spl=off:uhcvi=on_59");
    sched.push("dis+1002_7_bsr=unit_only:cond=fast:nm=64:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sac=on:sfr=on:ssfp=100000:ssfq=1.4:ssnc=none:uhcvi=on_60");
    sched.push("dis+10_3:1_ep=RST:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1.1:sd=2:ss=priority:st=2.0:sos=on:sac=on:sdd=large:sser=off:ssfp=10000:ssfq=1.1:ssnc=none:sp=reverse_arity_62");
    sched.push("dis+10_1_bd=preordered:bs=unit_only:cond=on:fde=none:lcm=predicate:nwc=1:sas=minisat:sd=2:ss=axioms:sos=all:sac=on:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_66");
    sched.push("lrs+11_2:3_bd=off:cond=on:fde=none:nwc=1:stl=34:sd=10:ss=axioms:st=1.2:sos=all:spl=off:sp=occurrence:updr=off:uhcvi=on_72");
    sched.push("dis+1010_5:4_bd=off:fsr=off:fde=unused:gs=on:nm=64:nwc=1:sd=4:ss=axioms:st=1.2:sos=on:spl=off:sp=reverse_arity:uhcvi=on_73");
    sched.push("dis+1011_4:1_fsr=off:nwc=1:sd=3:ss=priority:sos=on:spl=off:sp=reverse_arity_77");
    sched.push("dis+1011_5:1_ep=RSTC:fde=unused:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_77");
    sched.push("lrs+4_5_bd=off:fde=none:nwc=1.1:sas=minisat:stl=34:sd=2:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_80");
    sched.push("lrs+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:urr=on:updr=off:uhcvi=on_96");
    sched.push("lrs+4_3_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_100");
    sched.push("dis+11_4_cond=on:ep=RST:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=10:ss=axioms:st=1.2:sos=all:sac=on:sdd=large:ssfp=10000:ssfq=1.0:smm=sco:ssnc=none:sp=occurrence:urr=ec_only:updr=off_103");
    sched.push("dis+1011_2:1_cond=fast:ep=RST:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=1.1:ssnc=none:sp=reverse_arity_108");
    sched.push("dis+11_1_cond=fast:fde=none:gs=on:gsssp=full:nwc=1.1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_110");
    sched.push("dis+1011_2:3_cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:sas=minisat:sd=2:ss=axioms:sos=on:sac=on:sdd=large:sfr=on:ssfp=1000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=ec_only:uhcvi=on_113");
    sched.push("dis+1011_1_cond=on:fde=unused:gsp=input_only:nwc=1:sd=1:ss=axioms:sos=all:spl=off:sp=reverse_arity_131");
    sched.push("lrs+4_5:4_bd=off:bs=on:bsr=unit_only:cond=fast:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=reverse_arity:uhcvi=on_141");
    sched.push("dis+11_4_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsaa=from_current:nwc=1:sd=2:ss=axioms:sos=all:sdd=off:sser=off:ssnc=none:sp=occurrence:uhcvi=on_146");
    sched.push("lrs+11_5_br=off:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:stl=34:sd=3:ss=axioms:sos=all:sac=on:sdd=large:sfr=on:ssfp=4000:ssfq=1.2:smm=sco:ssnc=none:urr=on:uhcvi=on_148");
    sched.push("dis+1002_1_ep=RST:gs=on:gsaa=full_model:gsem=on:nm=64:nwc=1:sas=minisat:sd=7:ss=axioms:st=1.2:sos=on:sser=off:ssfp=40000:ssfq=1.2:ssnc=none:updr=off:uhcvi=on_155");
    sched.push("dis+2_5_bd=off:cond=fast:gs=on:lcm=reverse:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_158");
    sched.push("lrs+11_8:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:spl=off:sp=occurrence:urr=on:updr=off_184");
    sched.push("lrs+1004_3:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:sos=on:spl=off:updr=off_288");
    sched.push("ins+11_3_ep=RST:fde=unused:gsp=input_only:igbrr=0.4:igrr=1/8:igrpq=1.5:igs=1:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:spl=off:updr=off:dm=on:uhcvi=on_331");
    sched.push("lrs+11_3_bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=all:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on:updr=off_345");
  }
}

void Schedules::getLtb2015HllMiddSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 12000) {
    sched.push("lrs+1003_4_bsr=unit_only:cond=fast:fsr=off:gsp=input_only:gs=on:gsaa=from_current:nm=0:nwc=1:stl=34:sos=on:sac=on:sdd=large:ssfp=10000:ssfq=1.1:ssnc=none:urr=ec_only:uhcvi=on_6");
    sched.push("dis+1011_4:1_fsr=off:nwc=1:sd=3:ss=priority:sos=on:spl=off:sp=reverse_arity_6");
    sched.push("ins+11_4:1_cond=on:ep=RSTC:fde=none:gs=on:gsem=on:igbrr=0.2:igpr=on:igrr=32/1:igrp=2000:igrpq=1.3:igs=1002:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:sos=all:spl=off:sp=reverse_arity:uhcvi=on_8");
    sched.push("lrs+11_3_bsr=unit_only:cond=on:ep=RST:fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=10:ss=axioms:st=1.5:sos=all:sdd=off:ssfp=40000:ssfq=1.0:ssnc=none:sp=occurrence:urr=on_10");
    sched.push("dis+11_4_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsaa=from_current:nwc=1:sd=2:ss=axioms:sos=all:sdd=off:sser=off:ssnc=none:sp=occurrence:uhcvi=on_10");
    sched.push("lrs+10_5:4_bd=off:gs=on:gsssp=full:lcm=reverse:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("dis+11_1_cond=fast:fde=none:gs=on:gsssp=full:nwc=1.1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_11");
    sched.push("lrs+11_4_bd=off:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:sos=all:ssfp=1000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_11");
    sched.push("lrs+10_4_bsr=unit_only:cond=fast:fsr=off:fde=unused:gs=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=2.0:sos=on:ssfp=10000:ssfq=1.0:smm=sco:ssnc=all_dependent:sp=occurrence:uhcvi=on_12");
    sched.push("ins+11_5_ep=RS:fsr=off:gs=on:igbrr=0.4:igpr=on:igrr=1/2:igrp=4000:igrpq=1.2:igs=1010:igwr=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=reverse_arity:urr=on:updr=off_12");
    sched.push("lrs+11_14_cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:nwc=1:stl=34:sos=on:spl=off:sp=occurrence_12");
    sched.push("dis+2_5_bd=off:cond=fast:gs=on:lcm=reverse:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_13");
    sched.push("lrs+11_3:2_bsr=unit_only:cond=fast:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:stl=34:sd=2:ss=axioms:sdd=off:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_13");
    sched.push("lrs+11_2:3_bd=off:cond=on:fde=none:nwc=1:stl=34:sd=10:ss=axioms:st=1.2:sos=all:spl=off:sp=occurrence:updr=off:uhcvi=on_18");
    sched.push("lrs+1_7_bd=off:bsr=unit_only:nwc=1:sas=minisat:stl=34:sd=5:ss=axioms:st=1.2:sos=on:sser=off:ssfp=1000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_18");
    sched.push("lrs+4_5:4_bd=off:bs=on:bsr=unit_only:cond=fast:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=reverse_arity:uhcvi=on_18");
    sched.push("dis+10_1_bd=preordered:bs=unit_only:cond=on:fde=none:lcm=predicate:nwc=1:sas=minisat:sd=2:ss=axioms:sos=all:sac=on:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_19");
    sched.push("dis+11_4_br=off:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=1.5:sos=all:sac=on:sdd=large:sfr=on:ssfp=10000:ssfq=1.2:ssnc=none:sp=occurrence:urr=on_19");
    sched.push("dis+1011_1_cond=on:fde=unused:gsp=input_only:nwc=1:sd=1:ss=axioms:sos=all:spl=off:sp=reverse_arity_20");
    sched.push("dis+10_4_bd=off:cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:spl=off:urr=on_21");
    sched.push("dis+1011_5:1_ep=RSTC:fde=unused:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_21");
    sched.push("dis+1010_5:4_bd=off:fsr=off:fde=unused:gs=on:nm=64:nwc=1:sd=4:ss=axioms:st=1.2:sos=on:spl=off:sp=reverse_arity:uhcvi=on_32");
    sched.push("lrs+10_4_bd=off:bsr=unit_only:fde=none:gs=on:lcm=reverse:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=3.0:sos=on:spl=off:uhcvi=on_32");
    sched.push("dis+11_3_ep=RSTC:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:sd=1:ss=axioms:st=2.0:sos=on:sac=on:sdd=large:sfr=on:ssfp=40000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:urr=on:uhcvi=on_33");
    sched.push("lrs+10_4_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:stl=34:sd=4:ss=axioms:st=5.0:sos=all:spl=off:sp=reverse_arity:updr=off:uhcvi=on_46");
    sched.push("lrs+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:urr=on:updr=off:uhcvi=on_46");
    sched.push("lrs+10_4_bd=off:cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=1.5:sos=on:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_47");
    sched.push("dis+1002_7_bsr=unit_only:cond=fast:nm=64:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sac=on:sfr=on:ssfp=100000:ssfq=1.4:ssnc=none:uhcvi=on_48");
    sched.push("dis+10_3:1_ep=RST:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1.1:sd=2:ss=priority:st=2.0:sos=on:sac=on:sdd=large:sser=off:ssfp=10000:ssfq=1.1:ssnc=none:sp=reverse_arity_48");
    sched.push("dis+1002_3_cond=on:ep=RS:fsr=off:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sas=minisat:sd=5:ss=axioms:st=2.0:sos=on:ssfp=4000:ssfq=1.4:smm=off:ssnc=none:updr=off_49");
    sched.push("dis+11_1_fsr=off:fde=none:gs=on:gsem=on:nwc=1:sas=minisat:sd=3:ss=priority:sos=on:spl=off:urr=on:updr=off_51");
    sched.push("dis+10_3_bd=off:cond=on:gs=on:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=on:sac=on:ssfp=4000:ssfq=1.4:smm=off:ssnc=none:urr=on:updr=off:uhcvi=on_57");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sdd=off:ssfp=4000:ssfq=2.0:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_59");
    sched.push("dis+4_5:4_bd=off:fsr=off:fde=unused:gs=on:nwc=1:sd=5:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence:uhcvi=on_66");
    sched.push("lrs+4_3_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_66");
    sched.push("dis+1011_2:3_cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:sas=minisat:sd=2:ss=axioms:sos=on:sac=on:sdd=large:sfr=on:ssfp=1000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=ec_only:uhcvi=on_81");
    sched.push("dis+1011_2:1_cond=fast:ep=RST:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=1.1:ssnc=none:sp=reverse_arity_92");
    sched.push("lrs+11_5_br=off:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:stl=34:sd=3:ss=axioms:sos=all:sac=on:sdd=large:sfr=on:ssfp=4000:ssfq=1.2:smm=sco:ssnc=none:urr=on:uhcvi=on_92");
    sched.push("lrs+1004_3:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:sos=on:spl=off:updr=off_133");
    sched.push("dis+1002_1_ep=RST:gs=on:gsaa=full_model:gsem=on:nm=64:nwc=1:sas=minisat:sd=7:ss=axioms:st=1.2:sos=on:sser=off:ssfp=40000:ssfq=1.2:ssnc=none:updr=off:uhcvi=on_190");
    sched.push("lrs+11_8:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:spl=off:sp=occurrence:urr=on:updr=off_195");
    sched.push("ins+11_3_ep=RST:fde=unused:gsp=input_only:igbrr=0.4:igrr=1/8:igrpq=1.5:igs=1:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:spl=off:updr=off:dm=on:uhcvi=on_250");
    sched.push("lrs+11_3_bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=all:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on:updr=off_338");
  }
  else {
    sched.push("lrs+11_3_bd=off:bsr=unit_only:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=on:uhcvi=on_18");
    sched.push("lrs+11_3:2_bsr=unit_only:cond=fast:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:stl=34:sd=2:ss=axioms:sdd=off:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_19");
    sched.push("lrs+4_5:4_bd=off:bs=on:bsr=unit_only:cond=fast:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=reverse_arity:uhcvi=on_24");
    sched.push("dis+11_4_br=off:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=1.5:sos=all:sac=on:sdd=large:sfr=on:ssfp=10000:ssfq=1.2:ssnc=none:sp=occurrence:urr=on_34");
    sched.push("dis+10_4_bd=off:cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:spl=off:urr=on_35");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sdd=off:ssfp=4000:ssfq=2.0:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_40");
    sched.push("lrs+10_5:4_bd=off:gs=on:gsssp=full:lcm=reverse:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_41");
    sched.push("dis+11_1_cond=fast:fde=none:gs=on:gsssp=full:nwc=1.1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_46");
    sched.push("lrs+10_4_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:stl=34:sd=4:ss=axioms:st=5.0:sos=all:spl=off:sp=reverse_arity:updr=off:uhcvi=on_52");
    sched.push("dis+1002_7_bsr=unit_only:cond=fast:nm=64:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sac=on:sfr=on:ssfp=100000:ssfq=1.4:ssnc=none:uhcvi=on_60");
    sched.push("dis+10_3:1_ep=RST:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1.1:sd=2:ss=priority:st=2.0:sos=on:sac=on:sdd=large:sser=off:ssfp=10000:ssfq=1.1:ssnc=none:sp=reverse_arity_62");
    sched.push("dis+10_1_bd=preordered:bs=unit_only:cond=on:fde=none:lcm=predicate:nwc=1:sas=minisat:sd=2:ss=axioms:sos=all:sac=on:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_66");
    sched.push("lrs+11_2:3_bd=off:cond=on:fde=none:nwc=1:stl=34:sd=10:ss=axioms:st=1.2:sos=all:spl=off:sp=occurrence:updr=off:uhcvi=on_72");
    sched.push("dis+1010_5:4_bd=off:fsr=off:fde=unused:gs=on:nm=64:nwc=1:sd=4:ss=axioms:st=1.2:sos=on:spl=off:sp=reverse_arity:uhcvi=on_73");
    sched.push("dis+1011_5:1_ep=RSTC:fde=unused:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_77");
    sched.push("lrs+4_5_bd=off:fde=none:nwc=1.1:sas=minisat:stl=34:sd=2:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_80");
    sched.push("lrs+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:urr=on:updr=off:uhcvi=on_96");
    sched.push("lrs+11_8:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:spl=off:sp=occurrence:urr=on:updr=off_97");
    sched.push("lrs+4_3_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_100");
    sched.push("dis+1011_2:1_cond=fast:ep=RST:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=1.1:ssnc=none:sp=reverse_arity_108");
    sched.push("dis+1011_2:3_cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:sas=minisat:sd=2:ss=axioms:sos=on:sac=on:sdd=large:sfr=on:ssfp=1000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=ec_only:uhcvi=on_113");
    sched.push("dis+1002_1_ep=RST:gs=on:gsaa=full_model:gsem=on:nm=64:nwc=1:sas=minisat:sd=7:ss=axioms:st=1.2:sos=on:sser=off:ssfp=40000:ssfq=1.2:ssnc=none:updr=off:uhcvi=on_155");
    sched.push("ins+11_3_ep=RST:fde=unused:gsp=input_only:igbrr=0.4:igrr=1/8:igrpq=1.5:igs=1:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:spl=off:updr=off:dm=on:uhcvi=on_240");
    sched.push("lrs+1004_3:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:sos=on:spl=off:updr=off_288");
    sched.push("lrs+11_3_bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=all:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on:updr=off_345");
  }
}

void Schedules::getLtb2015HllFastSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 12000) {
    sched.push("lrs+1003_4_bsr=unit_only:cond=fast:fsr=off:gsp=input_only:gs=on:gsaa=from_current:nm=0:nwc=1:stl=34:sos=on:sac=on:sdd=large:ssfp=10000:ssfq=1.1:ssnc=none:urr=ec_only:uhcvi=on_6");
    sched.push("dis+1011_4:1_fsr=off:nwc=1:sd=3:ss=priority:sos=on:spl=off:sp=reverse_arity_6");
    sched.push("ins+11_4:1_cond=on:ep=RSTC:fde=none:gs=on:gsem=on:igbrr=0.2:igpr=on:igrr=32/1:igrp=2000:igrpq=1.3:igs=1002:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:sos=all:spl=off:sp=reverse_arity:uhcvi=on_8");
    sched.push("dis+4_5:4_bd=off:fsr=off:fde=unused:gs=on:nwc=1:sd=5:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence:uhcvi=on_9");
    sched.push("lrs+11_3_bsr=unit_only:cond=on:ep=RST:fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=10:ss=axioms:st=1.5:sos=all:sdd=off:ssfp=40000:ssfq=1.0:ssnc=none:sp=occurrence:urr=on_10");
    sched.push("lrs+10_4_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:stl=34:sd=4:ss=axioms:st=5.0:sos=all:spl=off:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("dis+11_4_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsaa=from_current:nwc=1:sd=2:ss=axioms:sos=all:sdd=off:sser=off:ssnc=none:sp=occurrence:uhcvi=on_10");
    sched.push("lrs+10_5:4_bd=off:gs=on:gsssp=full:lcm=reverse:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("dis+11_1_cond=fast:fde=none:gs=on:gsssp=full:nwc=1.1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_11");
    sched.push("lrs+11_4_bd=off:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:sos=all:ssfp=1000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_11");
    sched.push("lrs+10_4_bsr=unit_only:cond=fast:fsr=off:fde=unused:gs=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=2.0:sos=on:ssfp=10000:ssfq=1.0:smm=sco:ssnc=all_dependent:sp=occurrence:uhcvi=on_12");
    sched.push("ins+11_5_ep=RS:fsr=off:gs=on:igbrr=0.4:igpr=on:igrr=1/2:igrp=4000:igrpq=1.2:igs=1010:igwr=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=reverse_arity:urr=on:updr=off_12");
    sched.push("lrs+11_14_cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:nwc=1:stl=34:sos=on:spl=off:sp=occurrence_12");
    sched.push("dis+2_5_bd=off:cond=fast:gs=on:lcm=reverse:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_13");
    sched.push("lrs+4_3_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_13");
    sched.push("lrs+11_3:2_bsr=unit_only:cond=fast:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:stl=34:sd=2:ss=axioms:sdd=off:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_13");
    sched.push("dis+11_3_ep=RSTC:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:sd=1:ss=axioms:st=2.0:sos=on:sac=on:sdd=large:sfr=on:ssfp=40000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:urr=on:uhcvi=on_17");
    sched.push("lrs+11_2:3_bd=off:cond=on:fde=none:nwc=1:stl=34:sd=10:ss=axioms:st=1.2:sos=all:spl=off:sp=occurrence:updr=off:uhcvi=on_18");
    sched.push("lrs+1_7_bd=off:bsr=unit_only:nwc=1:sas=minisat:stl=34:sd=5:ss=axioms:st=1.2:sos=on:sser=off:ssfp=1000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_18");
    sched.push("lrs+4_5:4_bd=off:bs=on:bsr=unit_only:cond=fast:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=reverse_arity:uhcvi=on_18");
    sched.push("dis+11_4_br=off:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=1.5:sos=all:sac=on:sdd=large:sfr=on:ssfp=10000:ssfq=1.2:ssnc=none:sp=occurrence:urr=on_19");
    sched.push("dis+1011_1_cond=on:fde=unused:gsp=input_only:nwc=1:sd=1:ss=axioms:sos=all:spl=off:sp=reverse_arity_20");
    sched.push("dis+10_4_bd=off:cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:spl=off:urr=on_21");
    sched.push("dis+1011_5:1_ep=RSTC:fde=unused:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_21");
    sched.push("lrs+11_8:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:spl=off:sp=occurrence:urr=on:updr=off_23");
    sched.push("lrs+11_5_br=off:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:stl=34:sd=3:ss=axioms:sos=all:sac=on:sdd=large:sfr=on:ssfp=4000:ssfq=1.2:smm=sco:ssnc=none:urr=on:uhcvi=on_29");
    sched.push("dis+1010_5:4_bd=off:fsr=off:fde=unused:gs=on:nm=64:nwc=1:sd=4:ss=axioms:st=1.2:sos=on:spl=off:sp=reverse_arity:uhcvi=on_32");
    sched.push("lrs+10_4_bd=off:bsr=unit_only:fde=none:gs=on:lcm=reverse:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=3.0:sos=on:spl=off:uhcvi=on_32");
    sched.push("lrs+1004_3:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:sos=on:spl=off:updr=off_33");
    sched.push("dis+1011_2:1_cond=fast:ep=RST:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=1.1:ssnc=none:sp=reverse_arity_44");
    sched.push("lrs+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:urr=on:updr=off:uhcvi=on_46");
    sched.push("lrs+10_4_bd=off:cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=1.5:sos=on:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_47");
    sched.push("dis+1002_7_bsr=unit_only:cond=fast:nm=64:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sac=on:sfr=on:ssfp=100000:ssfq=1.4:ssnc=none:uhcvi=on_48");
    sched.push("dis+10_3:1_ep=RST:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1.1:sd=2:ss=priority:st=2.0:sos=on:sac=on:sdd=large:sser=off:ssfp=10000:ssfq=1.1:ssnc=none:sp=reverse_arity_48");
    sched.push("dis+1002_3_cond=on:ep=RS:fsr=off:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sas=minisat:sd=5:ss=axioms:st=2.0:sos=on:ssfp=4000:ssfq=1.4:smm=off:ssnc=none:updr=off_49");
    sched.push("dis+10_3_bd=off:cond=on:gs=on:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=on:sac=on:ssfp=4000:ssfq=1.4:smm=off:ssnc=none:urr=on:updr=off:uhcvi=on_57");
    sched.push("dis+1002_1_ep=RST:gs=on:gsaa=full_model:gsem=on:nm=64:nwc=1:sas=minisat:sd=7:ss=axioms:st=1.2:sos=on:sser=off:ssfp=40000:ssfq=1.2:ssnc=none:updr=off:uhcvi=on_62");
    sched.push("dis+1011_2:3_cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:sas=minisat:sd=2:ss=axioms:sos=on:sac=on:sdd=large:sfr=on:ssfp=1000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=ec_only:uhcvi=on_81");
    sched.push("ins+11_3_ep=RST:fde=unused:gsp=input_only:igbrr=0.4:igrr=1/8:igrpq=1.5:igs=1:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:spl=off:updr=off:dm=on:uhcvi=on_98");
    sched.push("lrs+11_3_bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=all:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on:updr=off_120");
  }
  else {
    sched.push("dis+1010_5:4_bd=off:fsr=off:fde=unused:gs=on:nm=64:nwc=1:sd=4:ss=axioms:st=1.2:sos=on:spl=off:sp=reverse_arity:uhcvi=on_13");
    sched.push("lrs+10_4_bd=off:cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=1.5:sos=on:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_17");
    sched.push("dis+10_5_bd=off:cond=fast:fde=none:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sdd=large:ssfp=100000:ssfq=1.2:ssnc=none:urr=on_18");
    sched.push("lrs+11_3_bd=off:bsr=unit_only:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=on:uhcvi=on_18");
    sched.push("lrs+11_3:2_bsr=unit_only:cond=fast:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:stl=34:sd=2:ss=axioms:sdd=off:sser=off:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_19");
    sched.push("lrs+4_5:4_bd=off:bs=on:bsr=unit_only:cond=fast:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=2.0:sos=on:spl=off:sp=reverse_arity:uhcvi=on_24");
    sched.push("lrs+4_5_bd=off:fde=none:nwc=1.1:sas=minisat:stl=34:sd=2:ss=axioms:st=5.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_28");
    sched.push("dis+11_4_br=off:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:st=1.5:sos=all:sac=on:sdd=large:sfr=on:ssfp=10000:ssfq=1.2:ssnc=none:sp=occurrence:urr=on_34");
    sched.push("lrs+11_5_bd=off:bsr=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sdd=off:ssfp=4000:ssfq=2.0:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_40");
    sched.push("lrs+10_5:4_bd=off:gs=on:gsssp=full:lcm=reverse:nwc=1:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_41");
    sched.push("dis+1011_2:1_cond=fast:ep=RST:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=1.1:ssnc=none:sp=reverse_arity_42");
    sched.push("dis+11_1_cond=fast:fde=none:gs=on:gsssp=full:nwc=1.1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_46");
    sched.push("lrs+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:urr=on:updr=off:uhcvi=on_46");
    sched.push("dis+1011_5:1_ep=RSTC:fde=unused:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sas=minisat:sd=1:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:updr=off:uhcvi=on_49");
    sched.push("dis+1002_7_bsr=unit_only:cond=fast:nm=64:nwc=1:sas=minisat:sd=1:ss=axioms:sos=on:sac=on:sfr=on:ssfp=100000:ssfq=1.4:ssnc=none:uhcvi=on_60");
    sched.push("dis+10_3:1_ep=RST:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1.1:sd=2:ss=priority:st=2.0:sos=on:sac=on:sdd=large:sser=off:ssfp=10000:ssfq=1.1:ssnc=none:sp=reverse_arity_62");
    sched.push("lrs+4_3_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.0:smm=sco:ssnc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_69");
    sched.push("dis+1011_2:3_cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:sas=minisat:sd=2:ss=axioms:sos=on:sac=on:sdd=large:sfr=on:ssfp=1000:ssfq=2.0:ssnc=none:sp=reverse_arity:urr=ec_only:uhcvi=on_94");
    sched.push("dis+1002_1_ep=RST:gs=on:gsaa=full_model:gsem=on:nm=64:nwc=1:sas=minisat:sd=7:ss=axioms:st=1.2:sos=on:sser=off:ssfp=40000:ssfq=1.2:ssnc=none:updr=off:uhcvi=on_125");
    sched.push("lrs+11_3_bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=all:sdd=large:sser=off:sfr=on:ssfp=4000:ssfq=2.0:ssnc=none:sp=occurrence:urr=on:updr=off_148");
    sched.push("ins+11_3_ep=RST:fde=unused:gsp=input_only:igbrr=0.4:igrr=1/8:igrpq=1.5:igs=1:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:spl=off:updr=off:dm=on:uhcvi=on_197");
  }
}

void Schedules::getLtb2015IsaSlowSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 90000) {
    sched.push("dis+11_5_ep=RST:fsr=off:fde=unused:gs=on:nwc=1:sas=minisat:sd=1:ss=priority:ssfp=1000:ssfq=1.0:smm=off:ssnc=none:sp=occurrence:uhcvi=on_16");
    sched.push("lrs+11_3_ep=RST:fsr=off:nm=64:nwc=1:sas=minisat:stl=34:sd=7:ss=axioms:st=1.2:sos=all:spl=off:uhcvi=on_17");
    sched.push("dis+11_4_cond=fast:ep=RST:fde=unused:gs=on:gsssp=full:lwlo=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=3.0:sos=all:sdd=large:ssfp=10000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:updr=off_17");
    sched.push("dis+1002_5_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sdd=off:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_25");
    sched.push("dis+11_5_fsr=off:fde=none:gs=on:gsaa=full_model:gsssp=full:nm=64:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=1000:ssfq=1.4:smm=off:ssnc=none:uhcvi=on_25");
    sched.push("lrs+11_3_bsr=unit_only:br=off:cond=on:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=all:sdd=large:sfr=on:ssfp=1000:ssfq=1.1:smm=off:ssnc=none:urr=on:uhcvi=on_32");
    sched.push("dis+10_4_cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:nwc=1.5:sd=1:ss=axioms:st=3.0:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_35");
    sched.push("dis-11_3:1_bd=off:fsr=off:fde=unused:lcm=reverse:nm=64:nwc=2.5:sd=5:ss=priority:st=3.0:spl=off_39");
    sched.push("dis-11_3_cond=on:fsr=off:gs=on:gsem=on:lcm=reverse:lwlo=on:nwc=1:sd=2:ss=axioms:sos=on:spl=off_39");
    sched.push("dis-11_4_bd=off:fde=none:gs=on:gsem=on:lwlo=on:nwc=1:sas=minisat:sd=3:ss=axioms:st=5.0:sos=all:spl=off:sp=reverse_arity:urr=on:updr=off_40");
    sched.push("lrs+11_3_cond=on:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on_40");
    sched.push("dis+1002_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=large:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=reverse_arity:updr=off_43");
    sched.push("dis+10_1_cond=fast:ep=RST:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:sd=4:ss=axioms:st=3.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=40000:ssfq=2.0:ssnc=none:sp=occurrence:urr=ec_only_44");
    sched.push("dis-1_2_cond=fast:gs=on:gsem=on:nm=64:nwc=1.7:sd=3:ss=priority:st=1.2:spl=off:updr=off_49");
    sched.push("ott+11_8:1_bsr=unit_only:fsr=off:fde=unused:nwc=1:nicw=on:sas=minisat:sd=2:ss=priority:st=2.0:sos=all:sac=on:ssfp=100000:ssfq=1.0:smm=sco:ssnc=all_dependent:sp=occurrence:uhcvi=on_52");
    sched.push("lrs+11_12_bd=off:bs=unit_only:ccuc=small_ones:cond=fast:fde=none:nwc=2.5:stl=34:sd=5:ss=priority:st=1.2:sos=all:sscc=model:sser=off:ssfp=100000:ssfq=1.4:ssnc=none_52");
    sched.push("lrs+11_40_bs=unit_only:cond=fast:gs=on:gsem=on:gsssp=full:lcm=reverse:nm=64:nwc=1.3:sas=minisat:stl=34:sd=3:ss=priority:spl=off:sp=reverse_arity:updr=off_54");
    sched.push("lrs+11_4_bd=off:br=off:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:st=5.0:sos=all:sac=on:sdd=large:sser=off:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:urr=on:uhcvi=on_55");
    sched.push("ott+1003_2_bd=off:bsr=unit_only:cond=fast:gs=on:nwc=1:sas=minisat:sd=1:ss=axioms:sos=all:spl=off:sp=occurrence:urr=on:updr=off_62");
    sched.push("dis+11_3_br=off:ccuc=small_ones:cond=fast:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:sscc=on:sfr=on:ssfp=1000:ssfq=2.0:smm=off:ssnc=none:sp=reverse_arity:urr=on:updr=off_63");
    sched.push("dis+11_4_cond=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:spl=off:updr=off:uhcvi=on_63");
    sched.push("dis+11_2:3_bd=off:ccuc=small_ones:cond=fast:fsr=off:fde=unused:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=all:sscc=model:sdd=large:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=ec_only:updr=off_70");
    sched.push("dis+1003_3_cond=on:ep=RST:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sser=off:sfr=on:ssfp=1000:ssfq=1.0:ssnc=none:updr=off_72");
    sched.push("ins+11_4_cond=on:er=filter:fde=none:igbrr=0.1:igrr=1/32:igrp=1400:igrpq=1.1:igs=1002:igwr=on:lcm=predicate:nwc=3:sas=minisat:sd=2:ss=axioms:st=2.0:spl=off:sp=occurrence:uhcvi=on_88");
    sched.push("dis+10_2_bd=off:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:nm=64:nwc=1:sd=3:ss=axioms:st=5.0:sos=on:sac=on:ssfp=10000:ssfq=1.4:ssnc=none:updr=off:uhcvi=on_93");
    sched.push("dis+1002_4_ep=RST:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=on:sser=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none_96");
    sched.push("dis+1002_7_gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=40000:ssfq=1.0:smm=off:ssnc=none_119");
    sched.push("dis+1011_3:2_bsr=unit_only:cond=fast:nwc=3:nicw=on:sd=3:ss=priority:sdd=off:sfr=on:ssfp=10000:ssfq=1.2:uhcvi=on_121");
    sched.push("lrs+1010_1_cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=on:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:urr=on:updr=off_148");
    sched.push("dis+10_2_bd=off:fde=unused:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:uhcvi=on_151");
    sched.push("lrs+11_5_fde=none:gs=on:lcm=reverse:nwc=1.2:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_158");
    sched.push("dis+11_1_cond=fast:gsp=input_only:lcm=predicate:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:spl=off:sp=occurrence:uhcvi=on_173");
    sched.push("lrs+10_3:1_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1.2:sas=minisat:stl=34:sd=1:ss=priority:sos=on:sac=on:sdd=off:ssfp=1000:ssfq=1.4:smm=sco:ssnc=all:sp=reverse_arity:urr=on:updr=off:uhcvi=on_201");
    sched.push("dis+1002_3_bd=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:sfr=on:smm=sco:ssnc=none:sp=occurrence_219");
    sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_233");
    sched.push("lrs+11_4_bd=off:bsr=on:cond=fast:fsr=off:lcm=reverse:nm=64:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=1.2:sac=on:sdd=off:ssfp=4000:ssfq=2.0:ssnc=none:uhcvi=on_239");
    sched.push("lrs+10_4:1_bd=off:ccuc=small_ones:gs=on:nwc=1:stl=34:sd=2:ss=priority:sos=all:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none_241");
    sched.push("dis+1011_5_fsr=off:fde=unused:nm=64:nwc=3:sd=2:ss=priority:spl=off:sp=occurrence:uhcvi=on_259");
  }
  else {
    sched.push("lrs+1_5:1_bd=off:cond=fast:fde=none:gs=on:gsem=on:lcm=reverse:nm=0:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:spl=off:sp=occurrence:uhcvi=on_18");
    sched.push("dis+11_5_cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lwlo=on:nwc=1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=off:sser=off:sfr=on:ssfp=1000:ssfq=1.2:ssnc=none:sp=occurrence:uhcvi=on_18");
    sched.push("ott+1003_2_bd=off:bsr=unit_only:cond=fast:gs=on:nwc=1:sas=minisat:sd=1:ss=axioms:sos=all:spl=off:sp=occurrence:urr=on:updr=off_27");
    sched.push("dis+10_3_ep=RST:fde=unused:gs=on:gsem=off:nwc=1:sas=minisat:sos=on:ssfp=100000:ssfq=1.1:smm=sco:ssnc=none:sp=occurrence:updr=off_29");
    sched.push("dis+11_4_cond=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:spl=off:updr=off:uhcvi=on_31");
    sched.push("dis+10_2_bd=off:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:nm=64:nwc=1:sd=3:ss=axioms:st=5.0:sos=on:sac=on:ssfp=10000:ssfq=1.4:ssnc=none:updr=off:uhcvi=on_32");
    sched.push("ins+11_4_cond=on:er=filter:fde=none:igbrr=0.1:igrr=1/32:igrp=1400:igrpq=1.1:igs=1002:igwr=on:lcm=predicate:nwc=3:sas=minisat:sd=2:ss=axioms:st=2.0:spl=off:sp=occurrence:uhcvi=on_33");
    sched.push("dis-1_2_cond=fast:gs=on:gsem=on:nm=64:nwc=1.7:sd=3:ss=priority:st=1.2:spl=off:updr=off_39");
    sched.push("dis+11_1_cond=fast:gsp=input_only:lcm=predicate:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:spl=off:sp=occurrence:uhcvi=on_40");
    sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=1000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_42");
    sched.push("dis+10_2_bd=off:fde=unused:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:uhcvi=on_43");
    sched.push("lrs+10_4:1_bd=off:ccuc=small_ones:gs=on:nwc=1:stl=34:sd=2:ss=priority:sos=all:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none_49");
    sched.push("dis+11_5_br=off:ccuc=small_ones:cond=fast:fsr=off:gs=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sscc=on:sdd=large:ssfp=100000:ssfq=1.2:smm=off:ssnc=none:sp=reverse_arity:urr=on:updr=off_55");
    sched.push("lrs+11_4_bd=off:br=off:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:st=5.0:sos=all:sac=on:sdd=large:sser=off:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:urr=on:uhcvi=on_65");
    sched.push("lrs+11_24_bd=off:bsr=unit_only:cond=on:gs=on:gsssp=full:nm=0:nwc=1.1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=occurrence:urr=ec_only_69");
    sched.push("dis-1_5_cond=on:ep=R:fsr=off:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:sas=minisat:sd=3:ss=priority:sos=on:spl=off:urr=ec_only:uhcvi=on_72");
    sched.push("lrs+1002_5_bsr=unit_only:ep=RS:fsr=off:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=64:nwc=1:stl=34:sd=2:ss=axioms:st=2.0:sos=on:sac=on:sser=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none:sp=occurrence:uhcvi=on_78");
    sched.push("dis+1002_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=large:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=reverse_arity:updr=off_97");
    sched.push("dis+1002_7_gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=40000:ssfq=1.0:smm=off:ssnc=none_108");
    sched.push("lrs+11_5_fde=none:gs=on:lcm=reverse:nwc=1.2:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_109");
    sched.push("lrs+10_3:1_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1.2:sas=minisat:stl=34:sd=1:ss=priority:sos=on:sac=on:sdd=off:ssfp=1000:ssfq=1.4:smm=sco:ssnc=all:sp=reverse_arity:urr=on:updr=off:uhcvi=on_119");
    sched.push("lrs+11_3_cond=on:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on_132");
    sched.push("dis+1002_5_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sdd=off:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_132");
    sched.push("lrs+1010_1_cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=on:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:urr=on:updr=off_142");
    sched.push("dis+1003_3_cond=on:ep=RST:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sser=off:sfr=on:ssfp=1000:ssfq=1.0:ssnc=none:updr=off_158");
    sched.push("dis+1002_3_bd=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:sfr=on:smm=sco:ssnc=none:sp=occurrence_175");
    sched.push("dis+1011_3:2_bsr=unit_only:cond=fast:nwc=3:nicw=on:sd=3:ss=priority:sdd=off:sfr=on:ssfp=10000:ssfq=1.2:uhcvi=on_208");
    sched.push("dis+1011_5_fsr=off:fde=unused:nm=64:nwc=3:sd=2:ss=priority:spl=off:sp=occurrence:uhcvi=on_224");
    sched.push("dis+1002_4_ep=RST:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=on:sser=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none_267");
    sched.push("dis-11_3_cond=on:fsr=off:gs=on:gsem=on:lcm=reverse:lwlo=on:nwc=1:sd=2:ss=axioms:sos=on:spl=off_277");
    sched.push("dis+11_2:3_bd=off:ccuc=small_ones:cond=fast:fsr=off:fde=unused:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=all:sscc=model:sdd=large:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=ec_only:updr=off_280");
    sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_286");
  }
}

void Schedules::getLtb2015IsaMiddSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 90000) {
    sched.push("dis+11_1_cond=fast:gsp=input_only:lcm=predicate:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:spl=off:sp=occurrence:uhcvi=on_12");
    sched.push("dis+11_5_ep=RST:fsr=off:fde=unused:gs=on:nwc=1:sas=minisat:sd=1:ss=priority:ssfp=1000:ssfq=1.0:smm=off:ssnc=none:sp=occurrence:uhcvi=on_16");
    sched.push("lrs+11_3_ep=RST:fsr=off:nm=64:nwc=1:sas=minisat:stl=34:sd=7:ss=axioms:st=1.2:sos=all:spl=off:uhcvi=on_17");
    sched.push("dis+11_4_cond=fast:ep=RST:fde=unused:gs=on:gsssp=full:lwlo=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=3.0:sos=all:sdd=large:ssfp=10000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:updr=off_17");
    sched.push("dis+1002_5_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sdd=off:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_25");
    sched.push("dis+11_5_fsr=off:fde=none:gs=on:gsaa=full_model:gsssp=full:nm=64:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=1000:ssfq=1.4:smm=off:ssnc=none:uhcvi=on_25");
    sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_31");
    sched.push("lrs+11_3_bsr=unit_only:br=off:cond=on:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=all:sdd=large:sfr=on:ssfp=1000:ssfq=1.1:smm=off:ssnc=none:urr=on:uhcvi=on_32");
    sched.push("dis+10_4_cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:nwc=1.5:sd=1:ss=axioms:st=3.0:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_35");
    sched.push("dis-11_3_cond=on:fsr=off:gs=on:gsem=on:lcm=reverse:lwlo=on:nwc=1:sd=2:ss=axioms:sos=on:spl=off_39");
    sched.push("dis-11_3:1_bd=off:fsr=off:fde=unused:lcm=reverse:nm=64:nwc=2.5:sd=5:ss=priority:st=3.0:spl=off_39");
    sched.push("lrs+11_3_cond=on:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on_40");
    sched.push("dis-11_4_bd=off:fde=none:gs=on:gsem=on:lwlo=on:nwc=1:sas=minisat:sd=3:ss=axioms:st=5.0:sos=all:spl=off:sp=reverse_arity:urr=on:updr=off_40");
    sched.push("dis+1002_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=large:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=reverse_arity:updr=off_43");
    sched.push("dis+10_1_cond=fast:ep=RST:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:sd=4:ss=axioms:st=3.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=40000:ssfq=2.0:ssnc=none:sp=occurrence:urr=ec_only_44");
    sched.push("dis-1_2_cond=fast:gs=on:gsem=on:nm=64:nwc=1.7:sd=3:ss=priority:st=1.2:spl=off:updr=off_49");
    sched.push("ott+11_8:1_bsr=unit_only:fsr=off:fde=unused:nwc=1:nicw=on:sas=minisat:sd=2:ss=priority:st=2.0:sos=all:sac=on:ssfp=100000:ssfq=1.0:smm=sco:ssnc=all_dependent:sp=occurrence:uhcvi=on_52");
    sched.push("lrs+11_12_bd=off:bs=unit_only:ccuc=small_ones:cond=fast:fde=none:nwc=2.5:stl=34:sd=5:ss=priority:st=1.2:sos=all:sscc=model:sser=off:ssfp=100000:ssfq=1.4:ssnc=none_52");
    sched.push("lrs+11_40_bs=unit_only:cond=fast:gs=on:gsem=on:gsssp=full:lcm=reverse:nm=64:nwc=1.3:sas=minisat:stl=34:sd=3:ss=priority:spl=off:sp=reverse_arity:updr=off_54");
    sched.push("lrs+11_4_bd=off:br=off:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:st=5.0:sos=all:sac=on:sdd=large:sser=off:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:urr=on:uhcvi=on_55");
    sched.push("lrs+10_3:1_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1.2:sas=minisat:stl=34:sd=1:ss=priority:sos=on:sac=on:sdd=off:ssfp=1000:ssfq=1.4:smm=sco:ssnc=all:sp=reverse_arity:urr=on:updr=off:uhcvi=on_59");
    sched.push("ott+1003_2_bd=off:bsr=unit_only:cond=fast:gs=on:nwc=1:sas=minisat:sd=1:ss=axioms:sos=all:spl=off:sp=occurrence:urr=on:updr=off_62");
    sched.push("dis+11_3_br=off:ccuc=small_ones:cond=fast:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:sscc=on:sfr=on:ssfp=1000:ssfq=2.0:smm=off:ssnc=none:sp=reverse_arity:urr=on:updr=off_63");
    sched.push("dis+11_4_cond=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:spl=off:updr=off:uhcvi=on_63");
    sched.push("dis+11_2:3_bd=off:ccuc=small_ones:cond=fast:fsr=off:fde=unused:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=all:sscc=model:sdd=large:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=ec_only:updr=off_70");
    sched.push("dis+1003_3_cond=on:ep=RST:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sser=off:sfr=on:ssfp=1000:ssfq=1.0:ssnc=none:updr=off_72");
    sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=1000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_74");
    sched.push("lrs+11_5_fde=none:gs=on:lcm=reverse:nwc=1.2:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_86");
    sched.push("dis+10_2_bd=off:fde=unused:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:uhcvi=on_87");
    sched.push("lrs+10_4:1_bd=off:ccuc=small_ones:gs=on:nwc=1:stl=34:sd=2:ss=priority:sos=all:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none_89");
    sched.push("dis+10_2_bd=off:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:nm=64:nwc=1:sd=3:ss=axioms:st=5.0:sos=on:sac=on:ssfp=10000:ssfq=1.4:ssnc=none:updr=off:uhcvi=on_93");
    sched.push("dis+1002_4_ep=RST:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=on:sser=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none_96");
    sched.push("dis+1002_7_gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=40000:ssfq=1.0:smm=off:ssnc=none_119");
    sched.push("dis+1011_3:2_bsr=unit_only:cond=fast:nwc=3:nicw=on:sd=3:ss=priority:sdd=off:sfr=on:ssfp=10000:ssfq=1.2:uhcvi=on_121");
    sched.push("dis+1011_5_fsr=off:fde=unused:nm=64:nwc=3:sd=2:ss=priority:spl=off:sp=occurrence:uhcvi=on_136");
    sched.push("lrs+1010_1_cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=on:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:urr=on:updr=off_148");
    sched.push("dis+1002_3_bd=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:sfr=on:smm=sco:ssnc=none:sp=occurrence_157");
  }
  else {
    sched.push("dis+11_5_cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lwlo=on:nwc=1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=off:sser=off:sfr=on:ssfp=1000:ssfq=1.2:ssnc=none:sp=occurrence:uhcvi=on_18");
    sched.push("lrs+1_5:1_bd=off:cond=fast:fde=none:gs=on:gsem=on:lcm=reverse:nm=0:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:spl=off:sp=occurrence:uhcvi=on_18");
    sched.push("ott+1003_2_bd=off:bsr=unit_only:cond=fast:gs=on:nwc=1:sas=minisat:sd=1:ss=axioms:sos=all:spl=off:sp=occurrence:urr=on:updr=off_27");
    sched.push("dis+10_3_ep=RST:fde=unused:gs=on:gsem=off:nwc=1:sas=minisat:sos=on:ssfp=100000:ssfq=1.1:smm=sco:ssnc=none:sp=occurrence:updr=off_29");
    sched.push("dis+11_4_cond=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:spl=off:updr=off:uhcvi=on_31");
    sched.push("dis+10_2_bd=off:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:nm=64:nwc=1:sd=3:ss=axioms:st=5.0:sos=on:sac=on:ssfp=10000:ssfq=1.4:ssnc=none:updr=off:uhcvi=on_32");
    sched.push("ins+11_4_cond=on:er=filter:fde=none:igbrr=0.1:igrr=1/32:igrp=1400:igrpq=1.1:igs=1002:igwr=on:lcm=predicate:nwc=3:sas=minisat:sd=2:ss=axioms:st=2.0:spl=off:sp=occurrence:uhcvi=on_33");
    sched.push("lrs+11_5_fde=none:gs=on:lcm=reverse:nwc=1.2:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_35");
    sched.push("dis-1_2_cond=fast:gs=on:gsem=on:nm=64:nwc=1.7:sd=3:ss=priority:st=1.2:spl=off:updr=off_39");
    sched.push("dis+11_1_cond=fast:gsp=input_only:lcm=predicate:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:spl=off:sp=occurrence:uhcvi=on_40");
    sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=1000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_42");
    sched.push("dis+10_2_bd=off:fde=unused:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:uhcvi=on_43");
    sched.push("lrs+10_4:1_bd=off:ccuc=small_ones:gs=on:nwc=1:stl=34:sd=2:ss=priority:sos=all:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none_49");
    sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_51");
    sched.push("dis+1011_3:2_bsr=unit_only:cond=fast:nwc=3:nicw=on:sd=3:ss=priority:sdd=off:sfr=on:ssfp=10000:ssfq=1.2:uhcvi=on_51");
    sched.push("dis+11_5_br=off:ccuc=small_ones:cond=fast:fsr=off:gs=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sscc=on:sdd=large:ssfp=100000:ssfq=1.2:smm=off:ssnc=none:sp=reverse_arity:urr=on:updr=off_55");
    sched.push("dis+1011_5_fsr=off:fde=unused:nm=64:nwc=3:sd=2:ss=priority:spl=off:sp=occurrence:uhcvi=on_59");
    sched.push("lrs+11_4_bd=off:br=off:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:st=5.0:sos=all:sac=on:sdd=large:sser=off:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:urr=on:uhcvi=on_65");
    sched.push("lrs+11_24_bd=off:bsr=unit_only:cond=on:gs=on:gsssp=full:nm=0:nwc=1.1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=occurrence:urr=ec_only_69");
    sched.push("dis-1_5_cond=on:ep=R:fsr=off:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:sas=minisat:sd=3:ss=priority:sos=on:spl=off:urr=ec_only:uhcvi=on_72");
    sched.push("lrs+11_3_cond=on:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on_74");
    sched.push("lrs+1002_5_bsr=unit_only:ep=RS:fsr=off:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=64:nwc=1:stl=34:sd=2:ss=axioms:st=2.0:sos=on:sac=on:sser=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none:sp=occurrence:uhcvi=on_78");
    sched.push("dis+1002_5_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sdd=off:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_80");
    sched.push("dis+1002_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=large:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=reverse_arity:updr=off_97");
    sched.push("dis+1002_3_bd=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:sfr=on:smm=sco:ssnc=none:sp=occurrence_100");
    sched.push("dis+1002_7_gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=40000:ssfq=1.0:smm=off:ssnc=none_108");
    sched.push("lrs+10_3:1_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1.2:sas=minisat:stl=34:sd=1:ss=priority:sos=on:sac=on:sdd=off:ssfp=1000:ssfq=1.4:smm=sco:ssnc=all:sp=reverse_arity:urr=on:updr=off:uhcvi=on_119");
    sched.push("lrs+1010_1_cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=on:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:urr=on:updr=off_142");
    sched.push("dis+1003_3_cond=on:ep=RST:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sser=off:sfr=on:ssfp=1000:ssfq=1.0:ssnc=none:updr=off_158");
    sched.push("dis+1002_4_ep=RST:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=on:sser=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none_267");
    sched.push("dis-11_3_cond=on:fsr=off:gs=on:gsem=on:lcm=reverse:lwlo=on:nwc=1:sd=2:ss=axioms:sos=on:spl=off_277");
  }
}

void Schedules::getLtb2015IsaFastSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 90000) {
    sched.push("dis+11_1_cond=fast:gsp=input_only:lcm=predicate:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:spl=off:sp=occurrence:uhcvi=on_12");
    sched.push("dis+11_5_cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lwlo=on:nwc=1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=off:sser=off:sfr=on:ssfp=1000:ssfq=1.2:ssnc=none:sp=occurrence:uhcvi=on_14");
    sched.push("dis+11_5_ep=RST:fsr=off:fde=unused:gs=on:nwc=1:sas=minisat:sd=1:ss=priority:ssfp=1000:ssfq=1.0:smm=off:ssnc=none:sp=occurrence:uhcvi=on_16");
    sched.push("dis+11_4_cond=fast:ep=RST:fde=unused:gs=on:gsssp=full:lwlo=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=3.0:sos=all:sdd=large:ssfp=10000:ssfq=1.2:smm=sco:ssnc=none:sp=reverse_arity:updr=off_17");
    sched.push("dis-11_3_cond=on:fsr=off:gs=on:gsem=on:lcm=reverse:lwlo=on:nwc=1:sd=2:ss=axioms:sos=on:spl=off_17");
    sched.push("lrs+11_3_ep=RST:fsr=off:nm=64:nwc=1:sas=minisat:stl=34:sd=7:ss=axioms:st=1.2:sos=all:spl=off:uhcvi=on_17");
    sched.push("lrs+11_40_bs=unit_only:cond=fast:gs=on:gsem=on:gsssp=full:lcm=reverse:nm=64:nwc=1.3:sas=minisat:stl=34:sd=3:ss=priority:spl=off:sp=reverse_arity:updr=off_20");
    sched.push("dis-1_2_cond=fast:gs=on:gsem=on:nm=64:nwc=1.7:sd=3:ss=priority:st=1.2:spl=off:updr=off_20");
    sched.push("dis+11_4_cond=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:spl=off:updr=off:uhcvi=on_21");
    sched.push("lrs+11_12_bd=off:bs=unit_only:ccuc=small_ones:cond=fast:fde=none:nwc=2.5:stl=34:sd=5:ss=priority:st=1.2:sos=all:sscc=model:sser=off:ssfp=100000:ssfq=1.4:ssnc=none_23");
    sched.push("dis+1002_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=large:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=reverse_arity:updr=off_24");
    sched.push("dis+1002_5_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sdd=off:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_25");
    sched.push("lrs+1010_1_cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=on:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:urr=on:updr=off_26");
    sched.push("dis+1003_3_cond=on:ep=RST:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sser=off:sfr=on:ssfp=1000:ssfq=1.0:ssnc=none:updr=off_28");
    sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_31");
    sched.push("dis+1002_3_bd=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:sfr=on:smm=sco:ssnc=none:sp=occurrence_31");
    sched.push("dis+10_4_cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:nwc=1.5:sd=1:ss=axioms:st=3.0:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_35");
    sched.push("dis-11_3:1_bd=off:fsr=off:fde=unused:lcm=reverse:nm=64:nwc=2.5:sd=5:ss=priority:st=3.0:spl=off_39");
    sched.push("lrs+11_3_cond=on:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on_40");
    sched.push("dis+11_2:3_bd=off:ccuc=small_ones:cond=fast:fsr=off:fde=unused:gs=on:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=all:sscc=model:sdd=large:ssfp=100000:ssfq=1.1:ssnc=none:sp=reverse_arity:urr=ec_only:updr=off_50");
    sched.push("dis+1011_5_fsr=off:fde=unused:nm=64:nwc=3:sd=2:ss=priority:spl=off:sp=occurrence:uhcvi=on_51");
    sched.push("lrs+11_4_bd=off:br=off:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=34:sd=2:ss=axioms:st=5.0:sos=all:sac=on:sdd=large:sser=off:ssfp=1000:ssfq=1.4:ssnc=none:sp=occurrence:urr=on:uhcvi=on_55");
    sched.push("dis+1011_3:2_bsr=unit_only:cond=fast:nwc=3:nicw=on:sd=3:ss=priority:sdd=off:sfr=on:ssfp=10000:ssfq=1.2:uhcvi=on_63");
    sched.push("dis+11_3_br=off:ccuc=small_ones:cond=fast:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:sscc=on:sfr=on:ssfp=1000:ssfq=2.0:smm=off:ssnc=none:sp=reverse_arity:urr=on:updr=off_63");
    sched.push("lrs+10_3:1_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1.2:sas=minisat:stl=34:sd=1:ss=priority:sos=on:sac=on:sdd=off:ssfp=1000:ssfq=1.4:smm=sco:ssnc=all:sp=reverse_arity:urr=on:updr=off:uhcvi=on_65");
    sched.push("dis+1002_7_gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=40000:ssfq=1.0:smm=off:ssnc=none_89");
    sched.push("lrs+10_4:1_bd=off:ccuc=small_ones:gs=on:nwc=1:stl=34:sd=2:ss=priority:sos=all:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none_89");
    sched.push("dis+10_2_bd=off:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:nm=64:nwc=1:sd=3:ss=axioms:st=5.0:sos=on:sac=on:ssfp=10000:ssfq=1.4:ssnc=none:updr=off:uhcvi=on_93");
    sched.push("dis+1002_4_ep=RST:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=on:sser=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none_96");
  }
  else {
    sched.push("dis+1003_3_cond=on:ep=RST:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sser=off:sfr=on:ssfp=1000:ssfq=1.0:ssnc=none:updr=off_14");
    sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:sos=on:sac=on:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_18");
    sched.push("dis+11_5_cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lwlo=on:nwc=1:sas=minisat:sd=1:ss=axioms:sac=on:sdd=off:sser=off:sfr=on:ssfp=1000:ssfq=1.2:ssnc=none:sp=occurrence:uhcvi=on_18");
    sched.push("lrs+1_5:1_bd=off:cond=fast:fde=none:gs=on:gsem=on:lcm=reverse:nm=0:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:spl=off:sp=occurrence:uhcvi=on_18");
    sched.push("dis+10_3_ep=RST:fde=unused:gs=on:gsem=off:nwc=1:sas=minisat:sos=on:ssfp=100000:ssfq=1.1:smm=sco:ssnc=none:sp=occurrence:updr=off_29");
    sched.push("dis+11_4_cond=on:lcm=reverse:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:spl=off:updr=off:uhcvi=on_31");
    sched.push("lrs+11_5_fde=none:gs=on:lcm=reverse:nwc=1.2:stl=34:sd=1:ss=axioms:spl=off:updr=off:uhcvi=on_35");
    sched.push("dis-1_5_cond=on:ep=R:fsr=off:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:sas=minisat:sd=3:ss=priority:sos=on:spl=off:urr=ec_only:uhcvi=on_37");
    sched.push("dis-1_2_cond=fast:gs=on:gsem=on:nm=64:nwc=1.7:sd=3:ss=priority:st=1.2:spl=off:updr=off_39");
    sched.push("dis+11_1_cond=fast:gsp=input_only:lcm=predicate:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:spl=off:sp=occurrence:uhcvi=on_40");
    sched.push("dis+10_2_bd=off:fde=unused:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:uhcvi=on_43");
    sched.push("lrs+10_3:1_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1.2:sas=minisat:stl=34:sd=1:ss=priority:sos=on:sac=on:sdd=off:ssfp=1000:ssfq=1.4:smm=sco:ssnc=all:sp=reverse_arity:urr=on:updr=off:uhcvi=on_44");
    sched.push("lrs+10_4:1_bd=off:ccuc=small_ones:gs=on:nwc=1:stl=34:sd=2:ss=priority:sos=all:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.1:smm=off:ssnc=none_49");
    sched.push("dis+1011_3:2_bsr=unit_only:cond=fast:nwc=3:nicw=on:sd=3:ss=priority:sdd=off:sfr=on:ssfp=10000:ssfq=1.2:uhcvi=on_51");
    sched.push("dis+1002_3_bd=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.2:sos=on:sfr=on:smm=sco:ssnc=none:sp=occurrence_52");
    sched.push("dis+11_5_br=off:ccuc=small_ones:cond=fast:fsr=off:gs=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sscc=on:sdd=large:ssfp=100000:ssfq=1.2:smm=off:ssnc=none:sp=reverse_arity:urr=on:updr=off_55");
    sched.push("dis+1011_5_fsr=off:fde=unused:nm=64:nwc=3:sd=2:ss=priority:spl=off:sp=occurrence:uhcvi=on_59");
    sched.push("dis+1002_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=large:ssfp=40000:ssfq=1.1:smm=off:ssnc=none:sp=reverse_arity:updr=off_62");
    sched.push("dis-11_3_cond=on:fsr=off:gs=on:gsem=on:lcm=reverse:lwlo=on:nwc=1:sd=2:ss=axioms:sos=on:spl=off_70");
    sched.push("lrs+11_3_cond=on:fde=unused:gs=on:gsem=off:nwc=1:stl=34:sd=2:ss=axioms:st=1.5:sos=on:spl=off:urr=on_74");
    sched.push("dis+1002_7_gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:sdd=large:sfr=on:ssfp=40000:ssfq=1.0:smm=off:ssnc=none_80");
    sched.push("dis+1002_5_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:sdd=off:sfr=on:smm=sco:ssnc=none:updr=off:uhcvi=on_80");
    sched.push("dis+1002_4_ep=RST:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=on:sser=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none_82");
    sched.push("lrs+1010_1_cond=on:fde=none:gs=on:gsem=off:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=3.0:sos=on:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:urr=on:updr=off_83");
  }
}

void Schedules::getLtb2015MzrSlowSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 6000) {
    sched.push("ins+11_3_cond=fast:ep=RST:igbrr=0.3:igrr=1/64:igrp=700:igrpq=2.0:igs=1002:igwr=on:nwc=1:sd=3:ss=axioms:spl=off:sp=reverse_arity:updr=off:uhcvi=on_2");
    sched.push("dis+1010_2_bs=on:cond=fast:ep=RSTC:fde=unused:lwlo=on:nwc=1:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:sp=reverse_arity:uhcvi=on_3");
    sched.push("ott+11_4_cond=fast:fde=none:gs=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence_3");
    sched.push("ott+1003_4:1_er=filter:nwc=10:spl=off:sp=occurrence:updr=off:uhcvi=on_4");
    sched.push("ins+11_4_ep=RS:fsr=off:fde=none:gsp=input_only:gs=on:igbrr=1.0:igpr=on:igrr=1/2:igrp=200:igrpq=1.5:igs=1:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:spl=off:urr=on:updr=off_4");
    sched.push("lrs+11_3_gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sos=all:sac=on:sdd=large:sfr=on:ssfp=100000:ssfq=1.1:smm=sco:ssnc=none:sp=occurrence:urr=on:updr=off_5");
    sched.push("ins+11_3_cond=fast:igbrr=0.7:igpr=on:igrr=1/32:igrp=700:igrpq=1.5:igs=1003:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=occurrence:uhcvi=on_8");
    sched.push("dis+10_4_cond=fast:gs=on:gsaa=from_current:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("lrs+10_2:3_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity_10");
    sched.push("dis+1004_2_bs=unit_only:bsr=unit_only:fde=unused:gs=on:nwc=1:sos=on:sdd=large:sfr=on_12");
    sched.push("lrs+10_4:1_fde=unused:lcm=predicate:nwc=1:stl=34:sd=3:ss=priority:sos=on:spl=off_14");
    sched.push("dis+11_4_bd=off:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity_16");
    sched.push("dis-1_1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1.3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sdd=off:ssfp=1000:ssfq=2.0:smm=sco:sp=occurrence:updr=off_19");
    sched.push("dis+11_5_20");
    sched.push("dis+10_2:1_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.4:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_20");
    sched.push("dis+1002_2_cond=on:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_27");
    sched.push("dis+1_2:1_cond=on:fsr=off:fde=none:gs=on:gsem=on:lwlo=on:nwc=1.3:sd=2:ss=axioms:spl=off:sp=reverse_arity:urr=on_50");
    sched.push("dis+10_3_cond=fast:fde=unused:gs=on:gsem=off:lwlo=on:nwc=1:sd=3:ss=axioms:sos=on:sdd=large:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity_57");
    sched.push("lrs+1_1_bs=on:bsr=on:br=off:cond=fast:fsr=off:gs=on:gsem=off:lwlo=on:nwc=3:stl=34:sd=3:ss=priority:sdd=large:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off_57");
    sched.push("lrs-2_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsssp=full:lcm=predicate:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:st=3.0:sos=on:sac=on:sfr=on:ssfp=10000:ssfq=1.1:ssnc=none:updr=off_66");
    sched.push("lrs+1010_2:3_bd=off:ccuc=first:cond=fast:fde=none:nwc=1:nicw=on:sas=minisat:stl=34:sos=on:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=ec_only:updr=off_77");
    sched.push("lrs+11_5_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:urr=on_77");
    sched.push("lrs-10_4:1_cond=on:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:sos=on:spl=off:urr=on_90");
    sched.push("dis+11_4_ep=RS:fde=none:gs=on:gsaa=full_model:gsem=off:nwc=1:sas=minisat:sd=1:ss=priority:st=1.2:sos=all:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_116");
    sched.push("ins+11_4_fde=unused:gs=on:igbrr=0.7:igpr=on:igrr=1/4:igrp=100:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence:urr=on:uhcvi=on_151");
    sched.push("lrs+11_4_cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sac=on:sdd=off:sser=off:ssfp=100000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_151");
    sched.push("lrs+1002_1_bsr=unit_only:nwc=1:stl=34:sd=1:ss=axioms:st=1.5:sos=all:spl=off:updr=off:uhcvi=on_158");
    sched.push("ins+11_4_cond=fast:fde=unused:igbrr=0.4:igpr=on:igrpq=1.3:igs=1004:igwr=on:lcm=predicate:nwc=1:sos=on:spl=off:updr=off:uhcvi=on_158");
    sched.push("dis-10_2:3_cond=on:fde=none:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:updr=off:uhcvi=on_181");
    sched.push("lrs+11_5_gs=on:gsem=off:nwc=1:stl=34:sd=5:ss=priority:st=1.2:sos=all:sdd=large:sfr=on:ssfp=100000:ssfq=1.4:sp=occurrence:updr=off_239");
    sched.push("lrs+10_3_ep=RST:fsr=off:fde=none:gs=on:nwc=1:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.0:smm=off:ssnc=none_247");
    sched.push("dis+11_3_cond=on:gs=on:gsem=on:gsssp=full:nwc=1:sd=2:ss=axioms:sos=all:sac=on:ssfp=10000:ssfq=2.0:ssnc=none:urr=on:updr=off:uhcvi=on_267");
    sched.push("dis+10_14_br=off:fsr=off:fde=unused:nwc=1.1:sas=minisat:sd=2:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on:updr=off_279");
    sched.push("lrs+11_5_fde=unused:gs=on:gsssp=full:nwc=1:sas=minisat:stl=34:sos=all:spl=off:updr=off:uhcvi=on_323");
  }
  else {
    sched.push("dis+11_3_cond=on:gs=on:gsem=on:gsssp=full:nwc=1:sd=2:ss=axioms:sos=all:sac=on:ssfp=10000:ssfq=2.0:ssnc=none:urr=on:updr=off:uhcvi=on_2");
    sched.push("dis+10_3_cond=fast:fde=unused:gs=on:gsem=off:lwlo=on:nwc=1:sd=3:ss=axioms:sos=on:sdd=large:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity_2");
    sched.push("lrs+10_3_ep=RST:fsr=off:fde=none:gs=on:nwc=1:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.0:smm=off:ssnc=none_2");
    sched.push("lrs+11_4_cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sac=on:sdd=off:sser=off:ssfp=100000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_3");
    sched.push("dis+11_4_bd=off:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity_3");
    sched.push("dis+11_5_4");
    sched.push("ott+11_4_cond=fast:fde=none:gs=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence_5");
    sched.push("lrs+1003_4_cond=on:fsr=off:fde=none:gs=on:gsem=off:nwc=1:stl=34:sd=3:ss=priority:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_6");
    sched.push("dis+1010_2_bs=on:cond=fast:ep=RSTC:fde=unused:lwlo=on:nwc=1:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:sp=reverse_arity:uhcvi=on_6");
    sched.push("ins+11_4_ep=RS:fsr=off:fde=none:gsp=input_only:gs=on:igbrr=1.0:igpr=on:igrr=1/2:igrp=200:igrpq=1.5:igs=1:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:spl=off:urr=on:updr=off_8");
    sched.push("dis-10_2:3_cond=on:fde=none:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:updr=off:uhcvi=on_10");
    sched.push("dis+10_4_cond=fast:gs=on:gsaa=from_current:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("dis+1004_2_bs=unit_only:bsr=unit_only:fde=unused:gs=on:nwc=1:sos=on:sdd=large:sfr=on_12");
    sched.push("dis-1_1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1.3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sdd=off:ssfp=1000:ssfq=2.0:smm=sco:sp=occurrence:updr=off_16");
    sched.push("dis+11_4_ep=RS:fde=none:gs=on:gsaa=full_model:gsem=off:nwc=1:sas=minisat:sd=1:ss=priority:st=1.2:sos=all:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_19");
    sched.push("lrs+1011_1_cond=on:fsr=off:gs=on:nwc=1:stl=34:sd=4:ss=priority:st=1.2:sos=on:spl=off:sp=reverse_arity:urr=on_23");
    sched.push("lrs+10_2:3_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity_24");
    sched.push("lrs+1_1_bs=on:bsr=on:br=off:cond=fast:fsr=off:gs=on:gsem=off:lwlo=on:nwc=3:stl=34:sd=3:ss=priority:sdd=large:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off_34");
    sched.push("lrs+11_3_cond=on:fsr=off:fde=none:gs=on:gsssp=full:lcm=reverse:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=on:spl=off:urr=on:updr=off_54");
    sched.push("dis+10_8_fsr=off:fde=unused:gsp=input_only:gs=on:nm=0:nwc=1:nicw=on:sd=2:ss=axioms:st=1.5:sos=all:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity_73");
    sched.push("ins+11_4_fde=unused:gs=on:igbrr=0.7:igpr=on:igrr=1/4:igrp=100:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence:urr=on:uhcvi=on_77");
    sched.push("dis+10_2:1_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.4:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_80");
    sched.push("lrs-10_4:1_cond=on:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:sos=on:spl=off:urr=on_100");
    sched.push("lrs+11_8_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:nicw=on:sas=minisat:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sdd=off:ssfp=100000:ssfq=1.4:smm=off:ssnc=all:sp=reverse_arity:urr=on:uhcvi=on_101");
    sched.push("lrs+1010_2:3_bd=off:ccuc=first:cond=fast:fde=none:nwc=1:nicw=on:sas=minisat:stl=34:sos=on:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=ec_only:updr=off_109");
    sched.push("dis+1002_2_cond=on:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_113");
    sched.push("lrs+11_5_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:urr=on_133");
    sched.push("lrs+10_4:1_fde=unused:lcm=predicate:nwc=1:stl=34:sd=3:ss=priority:sos=on:spl=off_151");
    sched.push("ins+11_3_cond=fast:igbrr=0.7:igpr=on:igrr=1/32:igrp=700:igrpq=1.5:igs=1003:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=occurrence:uhcvi=on_157");
    sched.push("dis+10_5_cond=on:fsr=off:fde=none:gs=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:spl=off_232");
    sched.push("lrs-2_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsssp=full:lcm=predicate:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:st=3.0:sos=on:sac=on:sfr=on:ssfp=10000:ssfq=1.1:ssnc=none:updr=off_270");
    sched.push("lrs+1002_1_bsr=unit_only:nwc=1:stl=34:sd=1:ss=axioms:st=1.5:sos=all:spl=off:updr=off:uhcvi=on_310");
    sched.push("ins+11_4_cond=fast:fde=unused:igbrr=0.4:igpr=on:igrpq=1.3:igs=1004:igwr=on:lcm=predicate:nwc=1:sos=on:spl=off:updr=off:uhcvi=on_323");
    sched.push("lrs+11_5_gs=on:gsem=off:nwc=1:stl=34:sd=5:ss=priority:st=1.2:sos=all:sdd=large:sfr=on:ssfp=100000:ssfq=1.4:sp=occurrence:updr=off_331");
    sched.push("dis+10_14_br=off:fsr=off:fde=unused:nwc=1.1:sas=minisat:sd=2:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on:updr=off_338");
    sched.push("dis+1_2:1_cond=on:fsr=off:fde=none:gs=on:gsem=on:lwlo=on:nwc=1.3:sd=2:ss=axioms:spl=off:sp=reverse_arity:urr=on_343");
  }
}

void Schedules::getLtb2015MzrMiddSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 6000) {
    sched.push("dis+11_3_cond=on:gs=on:gsem=on:gsssp=full:nwc=1:sd=2:ss=axioms:sos=all:sac=on:ssfp=10000:ssfq=2.0:ssnc=none:urr=on:updr=off:uhcvi=on_1");
    sched.push("ins+11_3_cond=fast:ep=RST:igbrr=0.3:igrr=1/64:igrp=700:igrpq=2.0:igs=1002:igwr=on:nwc=1:sd=3:ss=axioms:spl=off:sp=reverse_arity:updr=off:uhcvi=on_2");
    sched.push("dis+1010_2_bs=on:cond=fast:ep=RSTC:fde=unused:lwlo=on:nwc=1:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:sp=reverse_arity:uhcvi=on_3");
    sched.push("ott+11_4_cond=fast:fde=none:gs=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence_3");
    sched.push("ott+1003_4:1_er=filter:nwc=10:spl=off:sp=occurrence:updr=off:uhcvi=on_4");
    sched.push("ins+11_4_ep=RS:fsr=off:fde=none:gsp=input_only:gs=on:igbrr=1.0:igpr=on:igrr=1/2:igrp=200:igrpq=1.5:igs=1:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:spl=off:urr=on:updr=off_4");
    sched.push("lrs+11_3_gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:stl=34:sos=all:sac=on:sdd=large:sfr=on:ssfp=100000:ssfq=1.1:smm=sco:ssnc=none:sp=occurrence:urr=on:updr=off_5");
    sched.push("ins+11_4_cond=fast:fde=unused:igbrr=0.4:igpr=on:igrpq=1.3:igs=1004:igwr=on:lcm=predicate:nwc=1:sos=on:spl=off:updr=off:uhcvi=on_6");
    sched.push("ins+11_3_cond=fast:igbrr=0.7:igpr=on:igrr=1/32:igrp=700:igrpq=1.5:igs=1003:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=occurrence:uhcvi=on_8");
    sched.push("dis+10_4_cond=fast:gs=on:gsaa=from_current:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("lrs+10_2:3_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity_10");
    sched.push("dis+1004_2_bs=unit_only:bsr=unit_only:fde=unused:gs=on:nwc=1:sos=on:sdd=large:sfr=on_12");
    sched.push("lrs+10_4:1_fde=unused:lcm=predicate:nwc=1:stl=34:sd=3:ss=priority:sos=on:spl=off_14");
    sched.push("dis+11_4_bd=off:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity_16");
    sched.push("dis-1_1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1.3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sdd=off:ssfp=1000:ssfq=2.0:smm=sco:sp=occurrence:updr=off_19");
    sched.push("dis+11_5_20");
    sched.push("dis+10_2:1_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.4:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_20");
    sched.push("dis+1002_2_cond=on:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_27");
    sched.push("dis+1_2:1_cond=on:fsr=off:fde=none:gs=on:gsem=on:lwlo=on:nwc=1.3:sd=2:ss=axioms:spl=off:sp=reverse_arity:urr=on_50");
    sched.push("lrs+1_1_bs=on:bsr=on:br=off:cond=fast:fsr=off:gs=on:gsem=off:lwlo=on:nwc=3:stl=34:sd=3:ss=priority:sdd=large:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off_57");
    sched.push("dis+10_3_cond=fast:fde=unused:gs=on:gsem=off:lwlo=on:nwc=1:sd=3:ss=axioms:sos=on:sdd=large:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity_57");
    sched.push("lrs-2_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsssp=full:lcm=predicate:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:st=3.0:sos=on:sac=on:sfr=on:ssfp=10000:ssfq=1.1:ssnc=none:updr=off_66");
    sched.push("lrs+1010_2:3_bd=off:ccuc=first:cond=fast:fde=none:nwc=1:nicw=on:sas=minisat:stl=34:sos=on:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=ec_only:updr=off_77");
    sched.push("lrs+11_5_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:urr=on_77");
    sched.push("lrs-10_4:1_cond=on:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:sos=on:spl=off:urr=on_90");
    sched.push("dis+11_4_ep=RS:fde=none:gs=on:gsaa=full_model:gsem=off:nwc=1:sas=minisat:sd=1:ss=priority:st=1.2:sos=all:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_116");
    sched.push("lrs+11_4_cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sac=on:sdd=off:sser=off:ssfp=100000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_151");
    sched.push("ins+11_4_fde=unused:gs=on:igbrr=0.7:igpr=on:igrr=1/4:igrp=100:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence:urr=on:uhcvi=on_151");
    sched.push("lrs+11_3_cond=on:fsr=off:fde=none:gs=on:gsssp=full:lcm=reverse:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=on:spl=off:urr=on:updr=off_158");
    sched.push("lrs+1002_1_bsr=unit_only:nwc=1:stl=34:sd=1:ss=axioms:st=1.5:sos=all:spl=off:updr=off:uhcvi=on_158");
    sched.push("dis-10_2:3_cond=on:fde=none:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:updr=off:uhcvi=on_181");
    sched.push("lrs+11_5_gs=on:gsem=off:nwc=1:stl=34:sd=5:ss=priority:st=1.2:sos=all:sdd=large:sfr=on:ssfp=100000:ssfq=1.4:sp=occurrence:updr=off_239");
    sched.push("lrs+10_3_ep=RST:fsr=off:fde=none:gs=on:nwc=1:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.0:smm=off:ssnc=none_247");
    sched.push("lrs+11_5_fde=unused:gs=on:gsssp=full:nwc=1:sas=minisat:stl=34:sos=all:spl=off:updr=off:uhcvi=on_323");
  }
  else {
    sched.push("lrs+10_3_ep=RST:fsr=off:fde=none:gs=on:nwc=1:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.0:smm=off:ssnc=none_2");
    sched.push("dis+10_3_cond=fast:fde=unused:gs=on:gsem=off:lwlo=on:nwc=1:sd=3:ss=axioms:sos=on:sdd=large:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity_2");
    sched.push("dis+11_3_cond=on:gs=on:gsem=on:gsssp=full:nwc=1:sd=2:ss=axioms:sos=all:sac=on:ssfp=10000:ssfq=2.0:ssnc=none:urr=on:updr=off:uhcvi=on_2");
    sched.push("lrs+11_4_cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sac=on:sdd=off:sser=off:ssfp=100000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_3");
    sched.push("dis+11_4_bd=off:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity_3");
    sched.push("ott+11_4_cond=fast:fde=none:gs=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence_3");
    sched.push("dis+11_5_4");
    sched.push("dis+1010_2_bs=on:cond=fast:ep=RSTC:fde=unused:lwlo=on:nwc=1:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:sp=reverse_arity:uhcvi=on_6");
    sched.push("lrs+1003_4_cond=on:fsr=off:fde=none:gs=on:gsem=off:nwc=1:stl=34:sd=3:ss=priority:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_6");
    sched.push("ins+11_4_ep=RS:fsr=off:fde=none:gsp=input_only:gs=on:igbrr=1.0:igpr=on:igrr=1/2:igrp=200:igrpq=1.5:igs=1:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:spl=off:urr=on:updr=off_8");
    sched.push("ins+11_4_cond=fast:fde=unused:igbrr=0.4:igpr=on:igrpq=1.3:igs=1004:igwr=on:lcm=predicate:nwc=1:sos=on:spl=off:updr=off:uhcvi=on_8");
    sched.push("dis+10_4_cond=fast:gs=on:gsaa=from_current:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("dis-10_2:3_cond=on:fde=none:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:updr=off:uhcvi=on_10");
    sched.push("dis+1004_2_bs=unit_only:bsr=unit_only:fde=unused:gs=on:nwc=1:sos=on:sdd=large:sfr=on_12");
    sched.push("dis-1_1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1.3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sdd=off:ssfp=1000:ssfq=2.0:smm=sco:sp=occurrence:updr=off_16");
    sched.push("dis+11_4_ep=RS:fde=none:gs=on:gsaa=full_model:gsem=off:nwc=1:sas=minisat:sd=1:ss=priority:st=1.2:sos=all:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_19");
    sched.push("lrs+1011_1_cond=on:fsr=off:gs=on:nwc=1:stl=34:sd=4:ss=priority:st=1.2:sos=on:spl=off:sp=reverse_arity:urr=on_23");
    sched.push("lrs+10_2:3_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity_24");
    sched.push("lrs-2_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsssp=full:lcm=predicate:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:st=3.0:sos=on:sac=on:sfr=on:ssfp=10000:ssfq=1.1:ssnc=none:updr=off_28");
    sched.push("lrs+1_1_bs=on:bsr=on:br=off:cond=fast:fsr=off:gs=on:gsem=off:lwlo=on:nwc=3:stl=34:sd=3:ss=priority:sdd=large:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off_34");
    sched.push("lrs+11_3_cond=on:fsr=off:fde=none:gs=on:gsssp=full:lcm=reverse:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=on:spl=off:urr=on:updr=off_54");
    sched.push("dis+10_8_fsr=off:fde=unused:gsp=input_only:gs=on:nm=0:nwc=1:nicw=on:sd=2:ss=axioms:st=1.5:sos=all:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity_73");
    sched.push("ins+11_4_fde=unused:gs=on:igbrr=0.7:igpr=on:igrr=1/4:igrp=100:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence:urr=on:uhcvi=on_77");
    sched.push("dis+10_2:1_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.4:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_80");
    sched.push("dis+10_14_br=off:fsr=off:fde=unused:nwc=1.1:sas=minisat:sd=2:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on:updr=off_81");
    sched.push("lrs-10_4:1_cond=on:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:sos=on:spl=off:urr=on_100");
    sched.push("lrs+11_8_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:nicw=on:sas=minisat:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sdd=off:ssfp=100000:ssfq=1.4:smm=off:ssnc=all:sp=reverse_arity:urr=on:uhcvi=on_101");
    sched.push("lrs+1010_2:3_bd=off:ccuc=first:cond=fast:fde=none:nwc=1:nicw=on:sas=minisat:stl=34:sos=on:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=ec_only:updr=off_109");
    sched.push("dis+1002_2_cond=on:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_113");
    sched.push("lrs+11_5_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:urr=on_133");
    sched.push("lrs+10_4:1_fde=unused:lcm=predicate:nwc=1:stl=34:sd=3:ss=priority:sos=on:spl=off_151");
    sched.push("ins+11_3_cond=fast:igbrr=0.7:igpr=on:igrr=1/32:igrp=700:igrpq=1.5:igs=1003:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=occurrence:uhcvi=on_157");
    sched.push("dis+10_5_cond=on:fsr=off:fde=none:gs=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:spl=off_232");
    sched.push("lrs+1002_1_bsr=unit_only:nwc=1:stl=34:sd=1:ss=axioms:st=1.5:sos=all:spl=off:updr=off:uhcvi=on_310");
    sched.push("lrs+11_5_gs=on:gsem=off:nwc=1:stl=34:sd=5:ss=priority:st=1.2:sos=all:sdd=large:sfr=on:ssfp=100000:ssfq=1.4:sp=occurrence:updr=off_331");
  }
}

void Schedules::getLtb2015MzrFastSchedule(const Property& property, Schedule& sched)
{
  unsigned atoms = property.atoms();
  if (atoms < 6000) {
    sched.push("lrs+1011_1_cond=on:fsr=off:gs=on:nwc=1:stl=34:sd=4:ss=priority:st=1.2:sos=on:spl=off:sp=reverse_arity:urr=on_1");
    sched.push("dis+11_3_cond=on:gs=on:gsem=on:gsssp=full:nwc=1:sd=2:ss=axioms:sos=all:sac=on:ssfp=10000:ssfq=2.0:ssnc=none:urr=on:updr=off:uhcvi=on_1");
    sched.push("ins+11_3_cond=fast:igbrr=0.7:igpr=on:igrr=1/32:igrp=700:igrpq=1.5:igs=1003:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=occurrence:uhcvi=on_1");
    sched.push("ins+11_3_cond=fast:ep=RST:igbrr=0.3:igrr=1/64:igrp=700:igrpq=2.0:igs=1002:igwr=on:nwc=1:sd=3:ss=axioms:spl=off:sp=reverse_arity:updr=off:uhcvi=on_2");
    sched.push("ott+11_4_cond=fast:fde=none:gs=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence_2");
    sched.push("dis+1010_2_bs=on:cond=fast:ep=RSTC:fde=unused:lwlo=on:nwc=1:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:sp=reverse_arity:uhcvi=on_3");
    sched.push("lrs+11_8_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:nicw=on:sas=minisat:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sdd=off:ssfp=100000:ssfq=1.4:smm=off:ssnc=all:sp=reverse_arity:urr=on:uhcvi=on_4");
    sched.push("ins+11_4_ep=RS:fsr=off:fde=none:gsp=input_only:gs=on:igbrr=1.0:igpr=on:igrr=1/2:igrp=200:igrpq=1.5:igs=1:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:spl=off:urr=on:updr=off_4");
    sched.push("ott+1003_4:1_er=filter:nwc=10:spl=off:sp=occurrence:updr=off:uhcvi=on_4");
    sched.push("dis+10_3_cond=fast:fde=unused:gs=on:gsem=off:lwlo=on:nwc=1:sd=3:ss=axioms:sos=on:sdd=large:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity_10");
    sched.push("lrs+10_2:3_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity_10");
    sched.push("lrs-10_4:1_cond=on:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:sos=on:spl=off:urr=on_10");
    sched.push("dis+10_4_cond=fast:gs=on:gsaa=from_current:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("dis+1004_2_bs=unit_only:bsr=unit_only:fde=unused:gs=on:nwc=1:sos=on:sdd=large:sfr=on_12");
    sched.push("lrs+10_4:1_fde=unused:lcm=predicate:nwc=1:stl=34:sd=3:ss=priority:sos=on:spl=off_14");
    sched.push("dis+11_4_bd=off:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity_16");
    sched.push("dis-10_2:3_cond=on:fde=none:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:updr=off:uhcvi=on_16");
    sched.push("lrs+11_4_cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sac=on:sdd=off:sser=off:ssfp=100000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_17");
    sched.push("dis-1_1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1.3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sdd=off:ssfp=1000:ssfq=2.0:smm=sco:sp=occurrence:updr=off_19");
    sched.push("dis+11_5_20");
    sched.push("dis+10_2:1_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.4:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_20");
    sched.push("lrs+11_5_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:st=3.0:sos=on:spl=off:sp=occurrence:urr=on_24");
    sched.push("lrs+11_5_fde=unused:gs=on:gsssp=full:nwc=1:sas=minisat:stl=34:sos=all:spl=off:updr=off:uhcvi=on_24");
    sched.push("dis+1002_2_cond=on:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_27");
    sched.push("dis+11_4_ep=RS:fde=none:gs=on:gsaa=full_model:gsem=off:nwc=1:sas=minisat:sd=1:ss=priority:st=1.2:sos=all:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_44");
    sched.push("dis+1_2:1_cond=on:fsr=off:fde=none:gs=on:gsem=on:lwlo=on:nwc=1.3:sd=2:ss=axioms:spl=off:sp=reverse_arity:urr=on_50");
    sched.push("lrs+1_1_bs=on:bsr=on:br=off:cond=fast:fsr=off:gs=on:gsem=off:lwlo=on:nwc=3:stl=34:sd=3:ss=priority:sdd=large:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off_57");
    sched.push("lrs-2_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsssp=full:lcm=predicate:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:st=3.0:sos=on:sac=on:sfr=on:ssfp=10000:ssfq=1.1:ssnc=none:updr=off_66");
    sched.push("lrs+1010_2:3_bd=off:ccuc=first:cond=fast:fde=none:nwc=1:nicw=on:sas=minisat:stl=34:sos=on:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=ec_only:updr=off_77");
    sched.push("lrs+11_5_gs=on:gsem=off:nwc=1:stl=34:sd=5:ss=priority:st=1.2:sos=all:sdd=large:sfr=on:ssfp=100000:ssfq=1.4:sp=occurrence:updr=off_101");
    sched.push("lrs+1002_1_bsr=unit_only:nwc=1:stl=34:sd=1:ss=axioms:st=1.5:sos=all:spl=off:updr=off:uhcvi=on_126");
    sched.push("ins+11_4_fde=unused:gs=on:igbrr=0.7:igpr=on:igrr=1/4:igrp=100:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence:urr=on:uhcvi=on_151");
    sched.push("lrs+10_3_ep=RST:fsr=off:fde=none:gs=on:nwc=1:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.0:smm=off:ssnc=none_189");
  }
  else {
    sched.push("lrs+10_3_ep=RST:fsr=off:fde=none:gs=on:nwc=1:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sfr=on:ssfp=100000:ssfq=1.0:smm=off:ssnc=none_2");
    sched.push("dis+11_3_cond=on:gs=on:gsem=on:gsssp=full:nwc=1:sd=2:ss=axioms:sos=all:sac=on:ssfp=10000:ssfq=2.0:ssnc=none:urr=on:updr=off:uhcvi=on_2");
    sched.push("dis+10_3_cond=fast:fde=unused:gs=on:gsem=off:lwlo=on:nwc=1:sd=3:ss=axioms:sos=on:sdd=large:ssfp=10000:ssfq=2.0:ssnc=none:sp=reverse_arity_2");
    sched.push("dis+11_4_bd=off:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:sd=1:ss=axioms:sac=on:sdd=large:ssfp=1000:ssfq=2.0:smm=sco:ssnc=none:sp=reverse_arity_3");
    sched.push("lrs+11_4_cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=34:sd=1:ss=axioms:st=2.0:sac=on:sdd=off:sser=off:ssfp=100000:ssfq=1.4:ssnc=none:sp=occurrence:updr=off:uhcvi=on_3");
    sched.push("dis+11_5_4");
    sched.push("ott+11_4_cond=fast:fde=none:gs=on:nwc=1:sas=minisat:sd=4:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence_5");
    sched.push("dis+1010_2_bs=on:cond=fast:ep=RSTC:fde=unused:lwlo=on:nwc=1:sos=on:sac=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.4:sp=reverse_arity:uhcvi=on_6");
    sched.push("lrs+1003_4_cond=on:fsr=off:fde=none:gs=on:gsem=off:nwc=1:stl=34:sd=3:ss=priority:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_6");
    sched.push("dis+10_8_fsr=off:fde=unused:gsp=input_only:gs=on:nm=0:nwc=1:nicw=on:sd=2:ss=axioms:st=1.5:sos=all:sser=off:ssfp=10000:ssfq=1.2:ssnc=none:sp=reverse_arity_8");
    sched.push("ins+11_4_cond=fast:fde=unused:igbrr=0.4:igpr=on:igrpq=1.3:igs=1004:igwr=on:lcm=predicate:nwc=1:sos=on:spl=off:updr=off:uhcvi=on_8");
    sched.push("ins+11_4_ep=RS:fsr=off:fde=none:gsp=input_only:gs=on:igbrr=1.0:igpr=on:igrr=1/2:igrp=200:igrpq=1.5:igs=1:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:spl=off:urr=on:updr=off_8");
    sched.push("dis+10_4_cond=fast:gs=on:gsaa=from_current:nwc=1:sas=minisat:sd=2:ss=axioms:st=2.0:sos=on:sdd=off:sfr=on:ssfp=10000:ssfq=1.0:smm=off:ssnc=none:sp=reverse_arity:updr=off:uhcvi=on_10");
    sched.push("dis-10_2:3_cond=on:fde=none:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:spl=off:updr=off:uhcvi=on_10");
    sched.push("dis+10_5_cond=on:fsr=off:fde=none:gs=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:spl=off_11");
    sched.push("dis+1004_2_bs=unit_only:bsr=unit_only:fde=unused:gs=on:nwc=1:sos=on:sdd=large:sfr=on_12");
    sched.push("dis-1_1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1.3:sas=minisat:sd=1:ss=axioms:st=1.2:sos=on:sdd=off:ssfp=1000:ssfq=2.0:smm=sco:sp=occurrence:updr=off_16");
    sched.push("dis+11_4_ep=RS:fde=none:gs=on:gsaa=full_model:gsem=off:nwc=1:sas=minisat:sd=1:ss=priority:st=1.2:sos=all:sac=on:ssfp=10000:ssfq=1.1:smm=sco:ssnc=none:sp=reverse_arity:uhcvi=on_19");
    sched.push("lrs+1010_2:3_bd=off:ccuc=first:cond=fast:fde=none:nwc=1:nicw=on:sas=minisat:stl=34:sos=on:sac=on:sscc=model:sdd=off:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=ec_only:updr=off_20");
    sched.push("lrs+10_4:1_fde=unused:lcm=predicate:nwc=1:stl=34:sd=3:ss=priority:sos=on:spl=off_23");
    sched.push("lrs+1011_1_cond=on:fsr=off:gs=on:nwc=1:stl=34:sd=4:ss=priority:st=1.2:sos=on:spl=off:sp=reverse_arity:urr=on_23");
    sched.push("lrs+10_2:3_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=34:sd=2:ss=axioms:sos=on:spl=off:sp=reverse_arity_24");
    sched.push("lrs-2_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsssp=full:lcm=predicate:nwc=1:sas=minisat:stl=34:sd=4:ss=axioms:st=3.0:sos=on:sac=on:sfr=on:ssfp=10000:ssfq=1.1:ssnc=none:updr=off_28");
    sched.push("ins+11_3_cond=fast:igbrr=0.7:igpr=on:igrr=1/32:igrp=700:igrpq=1.5:igs=1003:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:spl=off:sp=occurrence:uhcvi=on_34");
    sched.push("lrs+1_1_bs=on:bsr=on:br=off:cond=fast:fsr=off:gs=on:gsem=off:lwlo=on:nwc=3:stl=34:sd=3:ss=priority:sdd=large:sfr=on:ssfp=40000:ssfq=1.4:smm=off:ssnc=none:sp=occurrence:urr=on:updr=off_34");
    sched.push("lrs+11_8_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:nicw=on:sas=minisat:stl=34:sd=1:ss=axioms:st=5.0:sos=all:sac=on:sdd=off:ssfp=100000:ssfq=1.4:smm=off:ssnc=all:sp=reverse_arity:urr=on:uhcvi=on_36");
    sched.push("lrs+11_3_cond=on:fsr=off:fde=none:gs=on:gsssp=full:lcm=reverse:nwc=1:sas=minisat:stl=34:sd=2:ss=axioms:st=1.2:sos=on:spl=off:urr=on:updr=off_54");
    sched.push("ins+11_4_fde=unused:gs=on:igbrr=0.7:igpr=on:igrr=1/4:igrp=100:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:spl=off:sp=occurrence:urr=on:uhcvi=on_77");
    sched.push("dis+10_2:1_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:sdd=off:sfr=on:ssfp=100000:ssfq=1.4:smm=sco:ssnc=none:urr=on:updr=off:uhcvi=on_80");
    sched.push("dis+10_14_br=off:fsr=off:fde=unused:nwc=1.1:sas=minisat:sd=2:ss=axioms:sos=all:sdd=off:sfr=on:ssfp=40000:ssfq=1.2:ssnc=none:sp=reverse_arity:urr=on:updr=off_81");
    sched.push("lrs-10_4:1_cond=on:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sas=minisat:stl=34:sd=3:ss=axioms:sos=on:spl=off:urr=on_100");
    sched.push("lrs+1002_1_bsr=unit_only:nwc=1:stl=34:sd=1:ss=axioms:st=1.5:sos=all:spl=off:updr=off:uhcvi=on_108");
    sched.push("dis+1002_2_cond=on:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:sos=on:spl=off:sp=reverse_arity:updr=off:uhcvi=on_113");
    sched.push("lrs+11_5_gs=on:gsem=off:nwc=1:stl=34:sd=5:ss=priority:st=1.2:sos=all:sdd=large:sfr=on:ssfp=100000:ssfq=1.4:sp=occurrence:updr=off_162");
  }
}

void Schedules::getSmtcomp2016Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  switch (property.getSMTLIBLogic()) {
  case SMT_ALIA:
    quick.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:stl=30:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_15");
    quick.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:stl=30:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_47");
    quick.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:stl=30:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_36");
    quick.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_37");
    quick.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:stl=30:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_141");
    quick.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:stl=120:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_81");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    return;
  case SMT_AUFLIA:
    quick.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_2");
    quick.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:stl=30:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_36");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    return;
  case SMT_AUFLIRA:
    quick.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_44");
    quick.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_38");
    quick.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_5");
    quick.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_2");
    quick.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_4");
    quick.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:stl=120:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_1198");
    quick.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:stl=30:sac=on:aac=none:anc=none:sp=occurrence_75");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    return;
  case SMT_AUFNIRA:
    quick.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_1");
    quick.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:stl=30:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_2");
    quick.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_1");
    quick.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:stl=30:sac=on:aac=none:anc=none:sp=occurrence_15");
    quick.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_2");
    quick.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_29");
    quick.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_74");
    quick.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_300");
    quick.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_140");
    quick.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:stl=30:av=off:sp=reverse_arity:tha=off:updr=off_91");
    quick.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_111");
    quick.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:stl=30:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_142");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    return;
  case SMT_LIA:
    quick.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_10");
    quick.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_2");
    quick.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_32");
    quick.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_90");
    quick.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_75");
    quick.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:stl=30:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_153");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    return;

  case SMT_LRA:
    quick.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_23");
    quick.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_24");
    quick.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_12");
    quick.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:stl=30:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_27");
    quick.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_30");
    quick.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:stl=30:av=off:sp=reverse_arity:updr=off_42");
    quick.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_122");
    quick.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:stl=30:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_256");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    return;
  case SMT_NIA:
    quick.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:stl=30:sac=on:aac=none:anc=none:sp=occurrence_295");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    return;
  case SMT_NRA:
    quick.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3");
    quick.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:stl=30:sac=on:aac=none:anc=none:sp=occurrence_35");
    quick.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:stl=30:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_12");
    quick.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:stl=30:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_75");
    quick.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_243");
    quick.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_7");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    return;
  case SMT_UF:
    quick.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:stl=30:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_7");
    quick.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:stl=30:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_63");
    quick.push("fmb+10_1_fmbsr=1.8:ile=on:nm=64:newcnf=on:av=off_65");
    quick.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_4");
    quick.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_6");
    quick.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:stl=30:av=off:urr=on:updr=off_43");
    quick.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_16");
    quick.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_38");
    quick.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_85");
    quick.push("ott+11_128_gsp=input_only:gs=on:gsem=on:ile=on:irw=on:lcm=predicate:nm=2:newcnf=on:nwc=1:aac=none:acc=on:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:updr=off_40");
    quick.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_128");
    quick.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_5");
    quick.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_73");
    quick.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_146");
    quick.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:stl=30:av=off:sp=occurrence:updr=off_22");
    quick.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_21");
    quick.push("fmb+10_1_fmbes=contour:fmbsr=1.5:ile=on:nm=64:newcnf=on:av=off:updr=off_257");
    quick.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_113");
    quick.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_299");
    quick.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_24");
    quick.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:stl=30:sac=on:aac=none:anc=none:sp=occurrence_138");
    quick.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:stl=30:av=off:updr=off_81");
    quick.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_136");
    quick.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:stl=30:av=off:sp=reverse_arity:urr=on_169");
    quick.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_295");
    quick.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_255");
    quick.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_87");
    quick.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:stl=30:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_23");
    quick.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_43");
    quick.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_129");
    quick.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:stl=30:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_272");
    quick.push("fmb+10_1_fmbes=contour:fmbsr=1.6_1530");
    quick.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:stl=30:acc=on:add=off:afp=1000:afq=1.1:anc=none_106");
    quick.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_171");
    quick.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_276");
    quick.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:stl=30:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_134");
    quick.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_261");
    quick.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_254");
    quick.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_148");
    quick.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_222");
    quick.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_434");
    quick.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:stl=30:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_90");
    quick.push("fmb+10_1_fmbas=expand:fmbsr=1.8:ile=on:nm=4:av=off_190");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    return;
  case SMT_UFIDL:
    quick.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3");
    quick.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3");
    quick.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_15");
    quick.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_21");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    return;

  case SMT_UFLIA:
    quick.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_16");
    quick.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:stl=30:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_41");
    quick.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:stl=30:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_91");
    quick.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_141");
    quick.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:stl=30:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_259");
    quick.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_43");
    quick.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:stl=30:av=off:sp=reverse_arity_8");
    quick.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:stl=30:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_53");
    quick.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:stl=30:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_226");
    quick.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:stl=30:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_60");
    quick.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_150");
    quick.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_4");
    quick.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:stl=30:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_13");
    quick.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_199");
    quick.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:stl=60:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_83");
    quick.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_270");
    quick.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_45");
    quick.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_546");
    quick.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:stl=30:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_213");
    quick.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:stl=30:sos=theory:av=off:urr=on_285");
    quick.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_131");
    quick.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_61");
    quick.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_152");
    quick.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:stl=30:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_280");
    quick.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_148");
    quick.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_235");
    quick.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:stl=30:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_6");
    quick.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_382");
    quick.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_281");
    quick.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:stl=30:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_17");
    quick.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:stl=30:add=large:afp=4000:afq=1.4:anc=none_27");
    quick.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_272");
    quick.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:av=off:urr=on:updr=off_62");
    quick.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_5");
    quick.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_175");
    quick.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_17");
    quick.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:stl=240:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_21");
    quick.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_346");
    quick.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:stl=30:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_60");
    quick.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_9");
    quick.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:stl=30:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_116");
    quick.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_11");
    quick.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_6");
    quick.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_60");
    quick.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_37");
    quick.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:stl=30:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_189");
    quick.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_21");
    quick.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_183");
    quick.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:stl=30:sos=on:add=off:afp=4000:afq=1.0:tha=off_38");
    quick.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:stl=30:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_2");
    quick.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_227");
    quick.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:stl=30:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_293");
    quick.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_478");
    quick.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_160");
    quick.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_83");
    quick.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_259");
    quick.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_88");
    quick.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_137");
    quick.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_17");
    quick.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:av=off:tha=off:thf=on:urr=on:updr=off_11");
    quick.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_61");
    quick.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:stl=30:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_201");
    quick.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:stl=30:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_132");
    quick.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:stl=30:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_16");
    quick.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_199");
    quick.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_84");
    quick.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:stl=150:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_63");
    quick.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:stl=30:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_270");
    quick.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:stl=60:av=off:sp=reverse_arity:tha=off:uhcvi=on_391");
    quick.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_185");
    quick.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_255");
    quick.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:stl=90:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_732");
    quick.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_370");
    quick.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_169");
    quick.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_172");
    quick.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:stl=60:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_582");
    quick.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:stl=60:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_235");
    quick.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:stl=30:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_260");
    quick.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:stl=240:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_2035");
    quick.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_4");
    quick.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:stl=30:av=off:tha=off:updr=off:uhcvi=on_27");
    quick.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_120");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    return;

  case SMT_UFLRA:
    quick.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:stl=30:sac=on:aac=none:anc=none:sp=occurrence_2");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sos=all:av=off:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    fallback.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_3000");
    fallback.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:sp=occurrence_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs+1010_5_fsr=off:nm=64:nwc=1:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:av=off:urr=on_3000");
    fallback.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:sos=on:av=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    return;

  case SMT_UFNIA:
    quick.push("lrs-1_5:4_bsr=on:cond=fast:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:stl=30:sos=all:av=off:sp=occurrence:updr=off_9");
    quick.push("dis+1011_8_cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1.1:sos=on:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off_26");
    quick.push("dis+1010_2:1_gs=on:gsem=off:nm=64:nwc=1.5:sas=z3:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_90");
    quick.push("ott+1_3_fsr=off:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off_13");
    quick.push("lrs+1011_3_bs=unit_only:cond=on:er=filter:gs=on:nm=64:newcnf=on:nwc=1:stl=30:afp=4000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_25");
    quick.push("dis+11_4_cond=fast:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:add=large:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:uhcvi=on_6");
    quick.push("dis-11_4:1_inw=on:irw=on:inst=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:av=off:sp=reverse_arity_114");
    quick.push("lrs+11_5:1_inw=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:afp=4000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_41");
    quick.push("lrs+10_3_fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1.3:stl=30:av=off:sp=reverse_arity:updr=off_5");
    quick.push("lrs-1_1_br=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:stl=30:sos=all:av=off:tha=off:thf=on:urr=on_19");
    quick.push("lrs+1011_4_gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sd=5:ss=axioms:st=1.2:afp=1000:afq=1.4:anc=none:sp=occurrence_7");
    quick.push("dis+1011_5_gs=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sos=all:sac=on:add=off:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_4");
    quick.push("lrs+1010_3:1_bd=off:br=off:cond=on:fsr=off:inw=on:irw=on:lma=on:nm=64:nwc=1:stl=30:av=off:urr=on_21");
    quick.push("lrs+1011_16_bd=off:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=32:nwc=1:stl=240:sd=3:ss=axioms:st=5.0:sac=on:aac=none:add=off:afp=100000:afq=1.4:amm=sco:sp=occurrence:updr=off_2006");
    quick.push("ott+1002_2:1_cond=on:gs=on:gsem=off:nm=2:newcnf=on:nwc=1.5:av=off:updr=off:uhcvi=on_50");
    quick.push("dis+2_5:1_irw=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:thf=on:updr=off_24");
    quick.push("lrs+10_5:1_bd=off:cond=on:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:stl=30:sos=all:aac=none:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_213");
    quick.push("dis+11_64_cond=on:gs=on:gsem=on:ile=on:irw=on:inst=on:lma=on:nm=64:nwc=3:sas=z3:sac=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:updr=off:uhcvi=on_2");
    quick.push("dis+1_5_cond=on:gs=on:gsem=off:irw=on:inst=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:afp=1000:afq=1.4:amm=off:sp=reverse_arity:tha=off:thf=on:updr=off_3");
    quick.push("lrs+10_3:1_bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:inw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:stl=30:add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_66");
    quick.push("lrs+1011_2_bd=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:inw=on:inst=on:nm=4:nwc=1:stl=30:sos=on:av=off:sp=occurrence:tha=off:updr=off_31");
    quick.push("lrs+1002_1_bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1:stl=30:sos=all:add=off:afp=4000:afq=2.0:anc=none:uhcvi=on_100");
    quick.push("ott+1011_3:1_bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=on:ile=on:inst=on:nm=0:newcnf=on:nwc=2.5:acc=on:add=large:afr=on:afp=1000:afq=1.0:anc=none:sp=reverse_arity:thf=on_13");
    quick.push("lrs-1_3:1_er=filter:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:av=off:sp=occurrence_73");
    quick.push("lrs+1011_2:3_bd=off:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:stl=30:av=off:sp=occurrence:tha=off:updr=off_155");
    quick.push("lrs-10_12_gsp=input_only:inw=on:inst=on:lcm=predicate:lma=on:nwc=1:stl=30:sos=all:av=off:sp=reverse_arity:tha=off:thf=on:updr=off_8");
    quick.push("ott+4_20_br=off:cond=on:fde=unused:gs=on:gsem=on:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:av=off:tha=off:thf=on:urr=on:uhcvi=on_46");
    quick.push("lrs+1010_2:1_gs=on:inw=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:stl=30:sos=all:av=off:sp=reverse_arity:urr=on_120");
    quick.push("dis+11_5:4_bsr=on:fsr=off:inw=on:nm=2:nwc=1:av=off:sp=occurrence:tha=off:thf=on_92");
    quick.push("lrs-11_5:4_bsr=on:ccuc=first:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=1.3:stl=30:sd=10:ss=axioms:st=3.0:sos=on:aac=none:acc=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_288");
    quick.push("lrs+1010_5_fsr=off:nm=64:nwc=1:stl=30:aac=none:add=large:afr=on:afp=40000:afq=1.0:sp=reverse_arity:tha=off_149");
    quick.push("lrs+1010_3_gsp=input_only:gs=on:gsem=off:nm=4:nwc=1:nicw=on:sas=z3:stl=30:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_244");
    quick.push("lrs+1_2:3_bd=off:cond=on:fsr=off:gs=on:inw=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:add=off:afp=1000:afq=1.2:amm=off:anc=none:tha=off:urr=ec_only:updr=off_291");
    quick.push("dis+11_5_cond=on:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sos=all:add=off:afp=10000:afq=1.2:anc=none:updr=off_36");
    fallback.push("dis+10_14_fsr=off:inw=on:ile=on:nm=6:nwc=1:sos=on:av=off:sp=occurrence:updr=off_3000");
    fallback.push("dis-10_8:1_cond=on:ile=on:irw=on:nm=64:nwc=4:sas=z3:sos=theory:anc=none:updr=off_3000");
    fallback.push("lrs+4_8:1_bs=unit_only:bsr=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sos=theory:aac=none:afp=1000:afq=2.0:amm=off:urr=on_3000");
    fallback.push("dis+1011_5:1_cond=on:ile=on:irw=on:nm=6:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("lrs+4_8:1_er=filter:gs=on:gsem=on:lma=on:nm=16:nwc=3:nicw=on:sas=z3:aac=none:afr=on:afp=10000:afq=1.0:anc=none:urr=on:updr=off_3000");
    fallback.push("ins+10_1_br=off:fsr=off:ile=on:irw=on:igbrr=1.0:igrr=1/32:igrp=1400:igrpq=1.2:igs=1010:igwr=on:lma=on:nm=4:nwc=1.1:sos=all:av=off:sp=reverse_arity:urr=on:dm=on_3000");
    fallback.push("ott+1002_3:1_cond=fast:ep=RS:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+4_5_irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=1.2:sac=on:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+11_1_bd=off:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+11_4:1_bsr=on:cond=on:fsr=off:ile=on:nm=64:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2:1_bsr=on:cond=on:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=32:nwc=2:sas=z3:add=large:afp=4000:afq=1.4:anc=none_3000");
    fallback.push("dis+11_2_bd=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+1011_3:2_bsr=on:cond=fast:gs=on:gsem=off:inw=on:inst=on:nm=64:nwc=2:sac=on:aac=none:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1002_32_bd=off:bs=unit_only:cond=on:gsp=input_only:inw=on:ile=on:nm=64:nwc=1:sos=on:add=off:afp=4000:afq=1.0:tha=off_3000");
    fallback.push("dis+10_14_fsr=off:nm=64:newcnf=on:nwc=1.1:av=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+2_3:2_cond=on:lma=on:nm=2:nwc=1:sos=on:sac=on:afr=on:afp=4000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+2_5:4_fsr=off:gs=on:gsem=off:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:aac=none:add=off:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs-1_8:1_cond=on:gs=on:gsem=off:ile=on:lwlo=on:nm=0:nwc=3:sas=z3:add=large:afp=4000:afq=1.1:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+1011_2:1_bd=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("dis+4_1_ccuc=first:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=2.5:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("lrs-10_5:4_bs=on:bsr=on:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("ott+1_6_bd=off:fsr=off:gs=on:gsem=off:inw=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:anc=none:sp=occurrence:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("lrs-11_2_bs=unit_only:bsr=on:gs=on:gsem=off:irw=on:nm=32:nwc=1:sas=z3:sos=theory:sac=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("ott+1_14_bd=off:cond=on:er=filter:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+4_8_gs=on:ile=on:irw=on:nm=32:nwc=2.5:av=off:updr=off_3000");
    fallback.push("lrs+1002_4_bd=off:fsr=off:gs=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:add=large:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis-1_4_cond=on:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sas=z3:sos=all:afp=10000:afq=1.4:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+11_5:1_cond=on:fsr=off:nm=4:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs-11_32_bd=off:fsr=off:gsp=input_only:gs=on:irw=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afp=10000:afq=1.4:amm=off:anc=none:tha=off:urr=on_3000");
    fallback.push("lrs+1010_2:1_cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=16:nwc=1.1:sas=z3:aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1011_20_cond=on:gsp=input_only:gs=on:inw=on:ile=on:lcm=reverse:lma=on:nwc=1.2:nicw=on:sas=z3:aac=none:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("ott+1011_8:1_gs=on:gsem=on:ile=on:irw=on:lma=on:nwc=1.1:sas=z3:add=off:afp=100000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("dis+10_5_cond=on:gs=on:gsem=off:irw=on:lwlo=on:nm=64:nwc=1:sos=all:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1010_5_bd=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:inst=on:lcm=reverse:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+1011_3_inw=on:lma=on:nm=64:newcnf=on:nwc=1:add=off:afp=100000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1002_3:2_fsr=off:gs=on:gsem=on:ile=on:lwlo=on:nm=64:nwc=3:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("dis+1011_32_bs=on:inw=on:nm=64:nwc=1.5:sd=10:ss=axioms:st=3.0:av=off:tha=off_3000");
    fallback.push("dis+11_5:4_bd=off:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("ott+10_8:1_cond=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("lrs+1010_5:4_fsr=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:av=off:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis-11_12_cond=on:gsp=input_only:irw=on:lwlo=on:nm=32:nwc=1:sas=z3:sos=all:sac=on:afr=on:afp=4000:afq=1.0:anc=none:tha=off_3000");
    fallback.push("lrs+1002_5:1_bd=off:lwlo=on:nm=64:nwc=1.7:nicw=on:sas=z3:sos=all:aac=none:add=large:afr=on:afp=1000:afq=1.1:amm=off_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=1:sos=theory:add=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity:urr=on_3000");
    fallback.push("ott+1011_5:1_bd=off:bsr=on:fsr=off:gsp=input_only:irw=on:nm=2:nwc=1:sos=theory:av=off:sp=occurrence:urr=on_3000");
    fallback.push("lrs-11_3_cond=on:inst=on:lma=on:nm=32:nwc=1:acc=model:add=off:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+1002_2:1_bd=off:cond=on:fsr=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1:sas=z3:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1010_3_er=known:gsp=input_only:lma=on:nm=64:newcnf=on:nwc=1:sos=all:av=off:sp=reverse_arity_3000");
    fallback.push("dis+1010_3_cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:add=off:afr=on:afp=40000:afq=2.0:amm=sco:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+10_128_er=known:fsr=off:gsp=input_only:nm=0:newcnf=on:nwc=4:aac=none:add=large:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("lrs+1010_6_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:inw=on:ile=on:nm=64:nwc=1.7:sas=z3:sos=all:aac=none:add=large:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+11_2:3_gs=on:lma=on:nm=6:nwc=3:sas=z3:add=large:afp=100000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+11_5_bd=off:gs=on:gsem=on:ile=on:irw=on:nm=64:nwc=1:sos=on:acc=on:afr=on:afp=100000:afq=1.1:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+11_4_bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=1:sos=all:av=off:thf=on_3000");
    fallback.push("lrs+10_4:1_bsr=on:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:lcm=reverse:nm=64:nwc=1.1:sos=all:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:uhcvi=on_3000");
    fallback.push("lrs+1011_1_cond=on:ile=on:nwc=1.2:add=off:afr=on:afp=4000:afq=1.0:urr=ec_only:updr=off_3000");
    fallback.push("lrs+11_2:3_bd=off:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("ott-10_2:1_bsr=on:ccuc=small_ones:cond=on:ile=on:irw=on:lcm=reverse:nwc=1:nicw=on:sos=on:acc=model:afp=1000:afq=1.4:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+4_2:1_cond=on:nm=64:newcnf=on:nwc=1:sos=on:afr=on:afp=10000:afq=1.4:sp=occurrence_3000");
    fallback.push("ott+11_2:3_fsr=off:gs=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off:urr=on_3000");
    fallback.push("dis+1011_3_cond=on:gs=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:thf=on_3000");
    fallback.push("dis+11_8_cond=fast:fde=none:gsp=input_only:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:av=off:sp=occurrence:tha=off:urr=ec_only:updr=off_3000");
    fallback.push("lrs+2_3:1_bd=off:cond=on:gs=on:gsem=on:inw=on:irw=on:lma=on:newcnf=on:nwc=1:av=off:sp=reverse_arity_3000");
    fallback.push("lrs+1011_2:3_bs=on:fsr=off:fde=none:ile=on:lcm=reverse:nm=32:nwc=1.5:sas=z3:sos=all:aac=none:add=large:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("lrs-11_1_cond=on:gsp=input_only:gs=on:gsem=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+10_5_gs=on:gsem=on:inw=on:ile=on:inst=on:lwlo=on:nm=64:nwc=1:nicw=on:sas=z3:sac=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("lrs+1010_5:1_bs=on:cond=on:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off_3000");
    fallback.push("lrs+11_3:2_bd=off:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("ott+1011_3_bs=unit_only:bsr=on:cond=on:inw=on:ile=on:nm=4:nwc=1:sas=z3:sos=all:add=off:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("lrs+10_4_cond=on:fsr=off:lwlo=on:nm=64:nwc=1:av=off:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("lrs+1010_5:4_bsr=on:cond=fast:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=2:newcnf=on:nwc=2.5:sas=z3:sac=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:updr=off:uhcvi=on_3000");
    fallback.push("ott+2_8:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:inw=on:inst=on:lwlo=on:nm=64:newcnf=on:nwc=1:acc=model:add=off:afp=100000:afq=1.1:tha=off_3000");
    fallback.push("ott+1011_5_bd=off:bs=on:bsr=on:cond=on:fsr=off:gs=on:gsem=off:inst=on:lma=on:nm=16:newcnf=on:nwc=5:nicw=on:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:tha=off:urr=ec_only_3000");
    fallback.push("dis+11_2_bs=unit_only:cond=on:fsr=off:ile=on:inst=on:nm=64:nwc=1.2:nicw=on:sas=z3:sos=on:sac=on:aac=none:add=large:afr=on:afp=1000:afq=1.1:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+1011_3_bd=off:bs=unit_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:aac=none:acc=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+11_2_bs=unit_only:fsr=off:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1:afr=on:afp=1000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+1011_5:4_bd=off:bs=unit_only:cond=fast:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:av=off:sp=occurrence:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("dis-1_128_bs=unit_only:ccuc=first:cond=on:gsp=input_only:ile=on:irw=on:inst=on:lma=on:newcnf=on:nwc=1:acc=model:afr=on:afp=40000:afq=2.0:amm=off:sp=reverse_arity:tha=off:urr=ec_only_3000");
    fallback.push("dis+1010_7_fsr=off:inst=on:nm=4:nwc=2:sas=z3:sac=on:aac=none:add=large:afp=10000:afq=1.4:amm=off:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+11_2:1_bd=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:av=off_3000");
    fallback.push("lrs+10_4:1_irw=on:inst=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("ott+1010_2:1_bs=unit_only:bsr=on:br=off:ccuc=first:cond=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:sac=on:acc=model:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+4_5_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=32:nwc=1:nicw=on:sos=all:sac=on:acc=on:add=off:afr=on:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+1_4:1_bs=unit_only:bsr=on:ccuc=first:fsr=off:inw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:acc=model:add=large:afp=100000:afq=1.0:amm=off:sp=occurrence:tha=off:thf=on:updr=off_3000");
    fallback.push("dis+1011_128_ile=on:nwc=1:sos=on:av=off_3000");
    fallback.push("lrs+11_4_fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:tha=off_3000");
    fallback.push("ott+2_8:1_bsr=on:br=off:ile=on:irw=on:nm=2:nwc=2.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("lrs+10_3:1_bd=off:fsr=off:gs=on:gsem=off:inst=on:nm=32:nwc=1.1:sas=z3:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity:updr=off_3000");
    fallback.push("ott-10_4:1_bd=off:bs=unit_only:fsr=off:gs=on:inw=on:inst=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:tha=off:urr=on:updr=off_3000");
    fallback.push("dis+11_4_fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=40000:afq=1.2:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+1011_3_bsr=on:fsr=off:gs=on:gsem=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:sos=on:aac=none:add=off:afr=on:afp=1000:afq=1.2:sp=occurrence:tha=off:thf=on_3000");
    fallback.push("lrs+10_7_bd=off:bs=unit_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sac=on:aac=none:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off:uhcvi=on_3000");
    fallback.push("dis+2_3_fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1011_5_cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.7:sas=z3:add=off:afp=1000:afq=1.2:amm=off:anc=none_3000");
    fallback.push("dis+1010_3:1_bs=on:gs=on:gsem=off:inw=on:ile=on:lma=on:nm=6:nwc=1.3:sas=z3:sos=on:sac=on:aac=none:afp=10000:afq=2.0:amm=off:anc=none:tha=off:urr=ec_only_3000");
    fallback.push("lrs+11_1_cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:sas=z3:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1011_8:1_gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:urr=on_3000");
    fallback.push("lrs+11_3:2_cond=on:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("ott+1_3_bd=off:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:nwc=1:sos=theory:av=off:sp=reverse_arity:updr=off_3000");
    fallback.push("ott+1011_3:1_bs=on:bsr=on:er=filter:fsr=off:inw=on:ile=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_3000");
    fallback.push("dis+4_4_cond=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+10_4_bd=off:bsr=on:fsr=off:gsp=input_only:gs=on:lcm=reverse:lwlo=on:nm=2:nwc=1.2:nicw=on:sd=7:ss=axioms:st=2.0:sos=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("ott+2_5:1_gsp=input_only:gs=on:ile=on:lma=on:nm=16:nwc=10:add=off:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+1010_3:2_er=known:inw=on:ile=on:lma=on:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_3000");
    fallback.push("ott+1011_3:1_bd=off:br=off:fsr=off:gs=on:gsem=on:irw=on:inst=on:lma=on:nm=0:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_1_cond=on:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:updr=off_3000");
    fallback.push("lrs+10_2_cond=on:gs=on:gsem=on:inw=on:irw=on:lcm=reverse:lma=on:nm=0:nwc=1:sos=all:sac=on:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:thf=on:urr=on_3000");
    fallback.push("ott+1_4:1_bd=off:ccuc=first:cond=on:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:acc=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:urr=on:updr=off_3000");
    fallback.push("ott+1010_1_bd=off:bsr=on:fsr=off:gsp=input_only:ile=on:irw=on:inst=on:nm=64:newcnf=on:nwc=1:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:urr=on:updr=off_3000");
    fallback.push("dis+11_3_cond=on:gs=on:inst=on:lcm=reverse:nm=16:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:updr=off_3000");
    fallback.push("dis+1011_28_bs=unit_only:bsr=on:er=filter:gs=on:gsem=on:inw=on:nm=64:nwc=1:nicw=on:afp=10000:afq=2.0:tha=off:updr=off_3000");
    fallback.push("lrs+1011_14_bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:inw=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:afp=100000:afq=2.0:amm=sco:anc=none:tha=off:updr=off_3000");
    fallback.push("dis+1010_4_cond=on:lma=on:nm=64:nwc=1.2:sac=on:add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fde=unused:gs=on:gsaa=from_current:inw=on:ile=on:nm=6:nwc=1.1:sas=z3:sos=on:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("ott+4_8:1_bd=off:gsp=input_only:gs=on:inw=on:ile=on:nm=0:newcnf=on:nwc=1:sac=on:add=large:afp=100000:afq=1.4:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("dis+1011_5_fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off_3000");
    fallback.push("dis+1011_1_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:lma=on:nm=2:nwc=10:nicw=on:sd=2:ss=axioms:st=3.0:sos=on:aac=none:add=large:afp=40000:afq=1.2:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("dis+2_4:1_fde=none:gsp=input_only:ile=on:irw=on:nm=64:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off:thf=on_3000");
    fallback.push("ott+1_1_bd=off:cond=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sos=all:sac=on:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:thf=on:urr=on_3000");
    fallback.push("lrs-1_64_bs=unit_only:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:inst=on:lcm=predicate:nm=6:newcnf=on:nwc=1:nicw=on:sac=on:aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:tha=off:uhcvi=on_3000");
    fallback.push("lrs+11_28_bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:sp=reverse_arity:tha=off_3000");
    fallback.push("lrs+4_3:1_bs=on:fsr=off:gs=on:gsem=off:ile=on:inst=on:lma=on:nm=64:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afp=10000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=ec_only:updr=off_3000");
    fallback.push("dis+1011_4_cond=on:irw=on:lma=on:nm=64:nwc=1:sos=all:av=off:sp=occurrence_3000");
    fallback.push("lrs+1011_5:4_bd=preordered:gs=on:gsem=off:inw=on:irw=on:nm=64:newcnf=on:nwc=1:av=off:tha=off:updr=off:uhcvi=on_3000");
    fallback.push("lrs+1011_3:1_bsr=on:fsr=off:ile=on:nm=64:nwc=1:av=off:sp=occurrence:updr=off_3000");
    fallback.push("lrs+1010_28_cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:sos=on:aac=none:afp=10000:afq=1.4:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("dis+10_24_fsr=off:ile=on:lma=on:nm=64:nwc=2:sas=z3:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:thf=on_3000");
    fallback.push("lrs+1002_1_fsr=off:gs=on:gsem=on:inw=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:av=off:urr=on:updr=off_3000");
    fallback.push("lrs+1011_5:1_bsr=on:er=known:fsr=off:gsp=input_only:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("ott+1_4_cond=on:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis+11_3:2_bd=off:gsp=input_only:gs=on:gsem=on:ile=on:lcm=reverse:nm=2:newcnf=on:nwc=3:nicw=on:sas=z3:afp=4000:afq=2.0:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1011_5_bd=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+4_3_cond=on:fsr=off:gs=on:gsem=on:inst=on:lma=on:nm=64:nwc=1:nicw=on:sas=z3:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3000");
    fallback.push("dis+11_4:1_bd=off:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("lrs+10_2:3_gs=on:gsem=off:inw=on:ile=on:inst=on:nm=64:nwc=5:nicw=on:sas=z3:sos=theory:aac=none:afr=on:afp=40000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+1_3_fsr=off:gs=on:nm=64:nwc=1:sas=z3:sac=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_3000");
    fallback.push("dis+2_2_bd=off:cond=on:fsr=off:gs=on:nwc=1.1:nicw=on:sos=on:add=large:afr=on:afp=40000:afq=2.0:amm=off:updr=off_3000");
    fallback.push("dis-2_8:1_bd=off:ccuc=small_ones:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:acc=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=ec_only_3000");
    fallback.push("ins+11_8:1_cond=on:ile=on:igbrr=0.6:igrr=8/1:igrp=4000:igs=1004:igwr=on:lma=on:nm=2:newcnf=on:nwc=4:av=off:updr=off:dm=on_3000");
    fallback.push("dis+10_5:4_bd=off:cond=on:gsp=input_only:ile=on:nm=2:nwc=1:add=large:afr=on:afp=100000:afq=1.2:amm=sco:anc=none_3000");
    fallback.push("lrs+1011_4_bd=off:bsr=on:gs=on:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none_3000");
    fallback.push("ott+4_3:1_ccuc=first:cond=on:gs=on:ile=on:irw=on:nm=16:newcnf=on:nwc=1:sos=all:acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("lrs+1011_2:3_fsr=off:lma=on:nm=64:nwc=3:sas=z3:sac=on:add=large:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence_3000");
    fallback.push("lrs+1010_10_cond=on:fsr=off:gs=on:gsem=off:inst=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:add=off:afr=on:afp=40000:afq=1.4:amm=off:anc=none:tha=off:thf=on:urr=on_3000");
    fallback.push("lrs+2_5:1_bd=off:bsr=on:br=off:fsr=off:ile=on:nm=4:nwc=1:av=off:sp=reverse_arity:urr=on_3000");
    fallback.push("dis-11_10_bs=on:bsr=on:fsr=off:gs=on:irw=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on:updr=off_3000");
    fallback.push("ott+2_8:1_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=0:newcnf=on:nwc=1.1:av=off:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("ott+1003_3:2_fde=unused:gs=on:gsem=on:inw=on:nm=64:newcnf=on:nwc=1:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:tha=off:uhcvi=on_3000");
    fallback.push("dis-1_4:1_er=filter:ile=on:nm=64:newcnf=on:nwc=1:sac=on:add=off:afp=1000:afq=1.0:anc=none:sp=occurrence:urr=on:updr=off_3000");
    fallback.push("dis+1_24_bs=on:fsr=off:gsp=input_only:gs=on:gsem=on:nm=4:newcnf=on:nwc=1:sas=z3:amm=off:anc=none_3000");
    fallback.push("dis+2_3:1_ccuc=first:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_3000");
    fallback.push("dis+4_3:1_cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3000");
    fallback.push("dis+10_2:1_bd=off:ccuc=small_ones:gsp=input_only:gs=on:ile=on:nm=0:newcnf=on:nwc=1:aac=none:acc=on:afp=4000:afq=1.2:amm=off:anc=none:urr=ec_only_3000");
    fallback.push("dis+10_5_cond=on:inw=on:ile=on:inst=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=occurrence:tha=off:updr=off_3000");
    fallback.push("lrs+4_7_bs=on:fsr=off:gsp=input_only:ile=on:nm=4:newcnf=on:nwc=1:sos=all:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:tha=off:updr=off_3000");
    fallback.push("lrs+1002_24_bd=off:bs=unit_only:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=2:nwc=2:acc=on:add=off:afp=1000:afq=1.1:anc=none_3000");
    fallback.push("dis+2_8:1_gsp=input_only:inst=on:lma=on:nm=4:nwc=4:add=large:afp=1000:afq=1.1:amm=sco:sp=occurrence:tha=off_3000");
    fallback.push("lrs+1011_8:1_bd=off:bsr=on:gs=on:inst=on:nwc=2.5:nicw=on:sas=z3:sos=theory:sac=on:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none_3000");
    fallback.push("lrs+1011_5:4_bs=on:cond=on:fsr=off:gsp=input_only:ile=on:lma=on:nm=4:nwc=1.1:sos=theory:av=off:urr=on_3000");
    fallback.push("lrs+1_3:2_bd=off:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:tha=off:updr=off_3000");
    fallback.push("lrs+1011_2:1_gsp=input_only:ile=on:irw=on:nm=2:nwc=1.7:sas=z3:afr=on:afp=1000:afq=1.0:anc=none:sp=occurrence:tha=off:urr=on_3000");
    fallback.push("lrs+1011_8:1_bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:inst=on:lma=on:nm=4:nwc=1:nicw=on:sas=z3:sos=on:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=on_3000");
    fallback.push("dis+1002_5:4_bd=off:bs=unit_only:bsr=on:gs=on:gsem=on:ile=on:inst=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:sos=all:afr=on:afp=1000:afq=1.4:sp=occurrence:tha=off:urr=ec_only_3000");
    fallback.push("dis+4_4_fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:tha=off:thf=on_3000");
    fallback.push("lrs+11_10_bd=off:bs=unit_only:bsr=on:fde=unused:ile=on:lma=on:lwlo=on:nm=6:nwc=2:nicw=on:sos=all:acc=on:afp=4000:afq=1.0:amm=off:anc=all_dependent:sp=reverse_arity:tha=off:thf=on:urr=ec_only:uhcvi=on_3000");
    fallback.push("dis+11_5_gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:afp=1000:afq=1.0:amm=off:anc=none:updr=off_3000");
    fallback.push("dis+11_3_cond=fast:fsr=off:lma=on:nm=64:newcnf=on:nwc=1:av=off:sp=reverse_arity:tha=off:urr=ec_only:updr=off:uhcvi=on_3000");
    fallback.push("dis+1_3:2_bd=off:lma=on:nm=32:nwc=2.5:sas=z3:sos=theory:add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:sp=reverse_arity_3000");
    fallback.push("dis+1011_10_gs=on:gsem=off:nm=64:nwc=1:sos=all:av=off:tha=off_3000");
    return;

  default:
    throw UserErrorException("This version cannot be used with this logic!");
  }
}

void Schedules::getCascSat2016Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category();
  unsigned long prop = property.props();
  unsigned atoms = property.atoms();

  switch (cat) {
  case Property::FNE:
    quick.push("dis+11_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:add=large:afr=on:afp=100000:afq=2.0:updr=off_12");
    quick.push("dis+2_64_cond=fast:fsr=off:gsp=input_only:nwc=1:sas=minisat:afp=100000:afq=1.4:anc=none:sp=occurrence_52");
    quick.push("fmb+10_1_sas=minisat_2046");
    quick.push("lrs+1_5_cond=fast:er=known:fde=unused:gs=on:gsem=on:gsssp=full:nwc=3:sas=minisat:stl=90:aer=off:afp=1000:afq=1.1:anc=none:sp=reverse_arity:updr=off_839");
    break;
  case Property::FEQ:
    if (atoms > 2000) {
      quick.push("dis-2_4_cond=on:nwc=1:sac=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence_7");
      quick.push("dis+11_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sas=minisat:add=large:afr=on:afp=100000:afq=2.0:updr=off_8");
      quick.push("dis+11_64_bs=on:bsr=unit_only:nm=0:nwc=1:sas=minisat:aac=none:aer=off:afp=1000:afq=2.0:urr=ec_only:updr=off_246");
      quick.push("ott+4_12_bd=off:ccuc=first:fde=none:gs=on:gsaa=from_current:gsem=on:nwc=1:acc=model:add=off:aer=off:afp=40000:afq=1.0:updr=off_127");
      quick.push("ins+11_4_cond=fast:gs=on:igrr=1/128:igrpq=1.2:igs=1010:igwr=on:nwc=1:sas=minisat:av=off_301");
      quick.push("fmb+10_1_sas=minisat_6000");
    }
    else {
      quick.push("dis+11_5_cond=fast:er=known:fsr=off:fde=unused:gs=on:gsssp=full:nm=0:nwc=1:sas=minisat:sac=on:add=large:aer=off:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence_1");
      quick.push("fmb+10_1_sas=minisat_77");
      quick.push("dis+11_5_cond=on:fsr=off:fde=none:gsp=input_only:lcm=reverse:nm=0:nwc=4:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:updr=off_1");
      quick.push("ins+11_5:4_bd=off:br=off:fsr=off:fde=unused:igbrr=1.0:igrr=8/1:igrp=400:igrpq=2.0:igs=1:igwr=on:lcm=predicate:nm=0:nwc=1:sos=on:av=off:sp=reverse_arity:urr=on:updr=off:dm=on_446");
      quick.push("ins+10_1_fde=unused:gsp=input_only:igbrr=0.7:igpr=on:igrp=200:igrpq=1.2:igs=1002:nwc=1:av=off:dm=on_556");
      quick.push("ott+11_5:1_bsr=unit_only:cond=fast:fde=none:nwc=1:sas=minisat:av=off:updr=off_567");
      quick.push("ins+10_1_fde=unused:igbrr=0.6:igrp=1400:igrpq=1.2:igs=1010:nwc=1:sos=all:av=off:sp=reverse_arity:updr=off:dm=on_707");
    }
    break;
  case Property::NEQ:
    quick.push("dis+11_7_1");
    quick.push("fmb+10_1_sas=minisat_6000");
    break;
  case Property::UEQ:
    quick.push("dis+11_7_1");
    quick.push("fmb+10_1_fmbsr=1.3:nm=0:sas=minisat_2313");
    quick.push("fmb+10_1_sas=minisat_6000");
    break;
  case Property::HNE:
  case Property::HEQ:
  case Property::PEQ:
  case Property::NNE:
    quick.push("dis+10_3_cond=fast:fsr=off:gs=on:gsaa=full_model:gsssp=full:nwc=1:sac=on:add=large:aer=off:afr=on:afp=10000:afq=1.2:anc=none:sp=occurrence_1");
    quick.push("ott+11_5_cond=on:er=filter:fsr=off:gs=on:gsaa=from_current:gsem=off:nwc=1:sac=on:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:updr=off_9");
    quick.push("dis+11_7_184");
    quick.push("ins+10_1_cond=on:fde=none:gs=on:gsem=on:igbrr=0.8:igpr=on:igrr=2/1:igs=1004:igwr=on:nwc=1:sas=minisat:sos=on:av=off:updr=off:dm=on_54");
    quick.push("fmb+10_1_sas=minisat_6000");
    break;
  case Property::EPR:
    if (prop == 131072) {
      quick.push("fmb+10_1_sas=minisat_17");
      quick.push("ins+1_1024_bd=preordered:br=off:cond=on:igbrr=0.9:igrr=1/16:igrp=2000:igrpq=2.0:igs=1010:igwr=on:nwc=1:av=off:sp=occurrence:urr=on:dm=on_154");
      quick.push("ott+2_5_cond=fast:er=filter:fde=none:nwc=1.5:nicw=on:sas=minisat:aac=none:add=large:afr=on:afp=100000:afq=2.0:amm=off:sp=reverse_arity:updr=off_215");
      quick.push("ott-11_3:2_bd=off:bs=unit_only:cond=fast:lcm=predicate:nwc=3:av=off:sp=occurrence_312");
      quick.push("ins+11_1024_bd=off:br=off:cond=fast:fsr=off:fde=none:igbrr=0.9:igpr=on:igrr=1/16:igrp=4000:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:dm=on_141");
      quick.push("dis+1003_3_cond=on:fsr=off:nwc=1.7:av=off:sp=occurrence:updr=off_506");
      quick.push("ins+10_1_fde=none:igbrr=0.7:igrp=4000:igrpq=1.3:igs=1:lcm=reverse:nwc=1.2:av=off:sp=reverse_arity:dm=on_488");
    }
    else if (prop == 131073) {
      quick.push("ott+2_5_cond=fast:er=filter:fde=none:nwc=1.5:nicw=on:sas=minisat:aac=none:add=large:afr=on:afp=100000:afq=2.0:amm=off:sp=reverse_arity:updr=off_225");
      quick.push("ins+11_1024_bd=off:br=off:cond=fast:fsr=off:fde=none:igbrr=0.9:igpr=on:igrr=1/16:igrp=4000:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:dm=on_1");
      quick.push("ott+3_5:1_ccuc=small_ones:fsr=off:lcm=predicate:nwc=1.1:sas=minisat:acc=on:add=off:aer=off:afp=1000:afq=1.2:anc=none:sp=reverse_arity:urr=ec_only:updr=off_200");
    }
    else if (atoms > 1300) {
      quick.push("dis-1_4_bd=preordered:cond=fast:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:sac=on:add=large:aer=off:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_46");
      quick.push("fmb+10_1_fmbsr=1.3:nm=0:sas=minisat_81");
      quick.push("dis+1011_128_bd=preordered:br=off:cond=on:nwc=1:sas=minisat:afr=on:afp=40000:afq=1.2:anc=none:urr=on:updr=off_18");
      quick.push("dis-11_8:1_bd=off:bs=unit_only:cond=fast:gs=on:gsem=off:nwc=1:aac=none:acc=on:add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=all_dependent_47");
      quick.push("ins+11_1024_bd=off:br=off:cond=fast:fsr=off:fde=none:igbrr=0.9:igpr=on:igrr=1/16:igrp=4000:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:dm=on_250");
      quick.push("ott-11_24_cond=fast:fde=none:gs=on:lcm=predicate:nwc=1:sas=minisat:av=off:sp=occurrence_19");
      quick.push("dis+10_3:1_bd=preordered:bsr=unit_only:fsr=off:fde=unused:gs=on:nwc=1:add=off:afp=100000:afq=1.2:amm=off:anc=none_272");
      quick.push("dis+1_28_bd=preordered:bs=unit_only:br=off:ccuc=small_ones:fsr=off:fde=none:gs=on:gsem=on:nwc=1:sas=minisat:sac=on:acc=model:aer=off:afr=on:afp=100000:afq=1.0:anc=all_dependent:sp=reverse_arity:urr=on_76");
      quick.push("dis+10_4:1_bd=off:ccuc=first:fde=none:gs=on:gsem=off:nwc=1:nicw=on:aac=none:acc=model:add=large:aer=off:afr=on:afp=4000:afq=2.0:urr=on:updr=off_97");
      quick.push("dis-4_8_bd=off:fde=unused:gs=on:nwc=1.2:sac=on:add=off:afr=on:afp=100000:afq=2.0:amm=sco:anc=all:urr=ec_only:updr=off_197");
      quick.push("ins+10_40_bd=off:br=off:fde=none:gsp=input_only:igbrr=1.0:igpr=on:igrr=1/32:igrp=1400:igrpq=1.05:igs=1:igwr=on:lcm=reverse:nwc=2.5:av=off:sp=occurrence:urr=on_331");
      quick.push("dis+2_1_bs=on:bsr=unit_only:ccuc=small_ones:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sac=on:acc=model:add=large:anc=none:urr=ec_only_352");
      quick.push("ott+11_5_bd=preordered:bsr=unit_only:cond=fast:fde=none:nwc=1:sas=minisat:afp=10000:afq=1.2:amm=sco:anc=all_dependent:sp=occurrence:updr=off_160");
    }
    else if (atoms > 430) {
      quick.push("dis+11_7_63");
      quick.push("ott+10_3_bd=preordered:bs=on:bsr=unit_only:cond=fast:fde=none:gs=on:lcm=predicate:nwc=2:sas=minisat:av=off:sp=reverse_arity:urr=on:updr=off_129");
      quick.push("ins+11_14_br=off:cond=on:fsr=off:igbrr=0.9:igrr=1/128:igrp=100:igrpq=1.05:igwr=on:lcm=predicate:nwc=1.7:av=off:urr=on_536");
      quick.push("ins+10_1_igbrr=0.4:igrp=400:igrpq=2.0:igs=1003:nwc=1.5:av=off:updr=off:dm=on_854");
      quick.push("ins+10_1_fde=unused:gs=on:igbrr=1.0:igrp=200:igrpq=2.0:igs=1002:lcm=predicate:nwc=3:av=off:sp=reverse_arity:dm=on_632");
      quick.push("ott+2_5_cond=fast:er=filter:fde=none:nwc=1.5:nicw=on:sas=minisat:aac=none:add=large:afr=on:afp=100000:afq=2.0:amm=off:sp=reverse_arity:updr=off_34");
    }
    else {
      quick.push("ott+11_4_bd=preordered:cond=on:fsr=off:nwc=1:sas=minisat:anc=none:sp=occurrence:updr=off_1323");
      quick.push("ins+11_50_bd=preordered:br=off:fsr=off:fde=none:gs=on:gsem=off:igbrr=0.5:igpr=on:igrr=1/128:igrp=200:igs=1:igwr=on:nwc=1:av=off:urr=on:dm=on_444");
      quick.push("ins+10_1_igbrr=0.4:igrp=400:igrpq=2.0:igs=1003:nwc=1.5:av=off:updr=off:dm=on_549");
    }
    break;
  }
  fallback.push("fmb+10_1_sas=minisat_3000");
  fallback.push("dis+11_7_300");
  fallback.push("dis+11_64_bs=on:bsr=unit_only:igrpq=1.0:nm=0:nwc=1:sas=minisat:aac=none:aer=off:afp=1000:afq=2.0:urr=ec_only:updr=off_300");
  fallback.push("dis+2_64_cond=fast:fsr=off:gsp=input_only:igrpq=1.0:nwc=1:sas=minisat:afp=100000:afq=1.4:anc=none:sp=occurrence_300");
  fallback.push("dis+2_1_bs=on:bsr=unit_only:ccuc=small_ones:fsr=off:gs=on:gsem=off:gsssp=full:igrpq=1.0:nwc=1:sas=minisat:sac=on:acc=model:add=large:anc=none:urr=ec_only_600");
  fallback.push("ott+11_5_cond=on:er=filter:fsr=off:gs=on:gsaa=from_current:gsem=off:igrpq=1.0:nwc=1:sac=on:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:updr=off_300");
  fallback.push("dis-11_8:1_bd=off:bs=unit_only:cond=fast:gs=on:gsem=off:igrpq=1.0:nwc=1:aac=none:acc=on:add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=all_dependent_300");
  fallback.push("ins+11_5:4_bd=off:br=off:fsr=off:fde=unused:igbrr=1.0:igrr=8/1:igrp=400:igrpq=2.0:igs=1:igwr=on:lcm=predicate:nm=0:nwc=1:sos=on:av=off:sp=reverse_arity:urr=on:updr=off:dm=on_600");
  fallback.push("ins+10_1_fde=unused:gsp=input_only:igbrr=0.7:igpr=on:igrp=200:igrpq=1.2:igs=1002:nwc=1:av=off:dm=on_600");
  fallback.push("dis+11_5_cond=fast:er=known:fsr=off:fde=unused:gs=on:gsssp=full:igrpq=1.0:nm=0:nwc=1:sas=minisat:sac=on:add=large:aer=off:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence_300");
  fallback.push("dis+10_3:1_bd=preordered:bsr=unit_only:fsr=off:fde=unused:gs=on:igrpq=1.0:nwc=1:add=off:afp=100000:afq=1.2:amm=off:anc=none_300");
  fallback.push("ins+10_1_cond=on:fde=none:gs=on:gsem=on:igbrr=0.8:igpr=on:igrr=2/1:igs=1004:igwr=on:nwc=1:sas=minisat:sos=on:av=off:updr=off:dm=on_300");
  fallback.push("dis-1_4_bd=preordered:cond=fast:fde=none:gs=on:gsssp=full:igrpq=1.0:nwc=1:sas=minisat:sac=on:add=large:aer=off:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_300");
  fallback.push("dis+10_3_cond=fast:fsr=off:gs=on:gsaa=full_model:gsssp=full:igrpq=1.0:nwc=1:sac=on:add=large:aer=off:afr=on:afp=10000:afq=1.2:anc=none:sp=occurrence_300");
  fallback.push("ott+4_12_bd=off:ccuc=first:fde=none:gs=on:gsaa=from_current:gsem=on:igrpq=1.0:nwc=1:acc=model:add=off:aer=off:afp=40000:afq=1.0:updr=off_300");
  fallback.push("ins+11_4_cond=fast:gs=on:igbrr=0.0:igrr=1/128:igrpq=1.2:igs=1010:igwr=on:nwc=1:sas=minisat:av=off_600");
  fallback.push("ott+11_5:1_bsr=unit_only:cond=fast:fde=none:igrpq=1.0:nwc=1:sas=minisat:av=off:updr=off_600");
  fallback.push("fmb+10_1_sas=minisat:nm=0:fmbsr=1.3_3000");
  fallback.push("ins+10_1_fde=unused:igbrr=0.6:igrp=1400:igrpq=1.2:igs=1010:nwc=1:sos=all:av=off:sp=reverse_arity:updr=off:dm=on_900");
  fallback.push("lrs+1_5_cond=fast:er=known:fde=unused:gs=on:gsem=on:gsssp=full:igrpq=1.0:nwc=3:sas=minisat:aer=off:afp=1000:afq=1.1:anc=none:sp=reverse_arity:updr=off_900");
}

void Schedules::getCasc2016Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category();
  unsigned long prop = property.props();
  unsigned atoms = property.atoms();
  // for theory problems, we make the schedule before the main choice
  if (prop & (524288ul | 1048576ul | 2097152ul)) { // theories
    if (prop == 13107200ul) {
      quick.push("dis+11_3_cond=on:gs=on:gsem=on:inw=on:irw=on:inst=on:nm=64:nwc=1:sas=z3:sd=10:ss=axioms:st=5.0:afp=4000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:tha=off_69");
      quick.push("lrs+1_24_bs=on:lma=on:nm=64:newcnf=on:nwc=2:stl=60:av=off:tha=off:urr=on_552");
      quick.push("lrs+2_24_bd=off:cond=on:gs=on:gsem=off:inw=on:inst=on:lma=on:nm=0:nwc=1.1:nicw=on:stl=90:add=large:afr=on:afp=10000:afq=1.1:amm=off:sp=occurrence:tha=off:updr=off_638");
    }
    else {
      quick.push("dis+1002_4_gs=on:inst=on:nm=64:nwc=1.3:sas=z3:sac=on:afr=on:amm=off:anc=none:sp=occurrence:tha=off:thf=on:updr=off_1");
      quick.push("lrs+1_24_bs=unit_only:fsr=off:gs=on:gsem=on:inst=on:nm=64:nwc=3:sas=z3:stl=30:sac=on:add=large:afr=on:afp=4000:afq=2.0:anc=none:sp=reverse_arity:thf=on:updr=off_1");
      quick.push("lrs+11_4:1_cond=on:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.7:sas=z3:stl=30:add=large:afp=100000:afq=1.2:amm=off:anc=none:tha=off_3");
      quick.push("dis+10_2:1_gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nm=2:nwc=1.3:nicw=on:sas=z3:sac=on:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:sp=occurrence:tha=off:updr=off_7");
      quick.push("dis+11_2:1_bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sos=all:aac=none:add=large:afr=on:afp=40000:afq=1.0:anc=none:sp=reverse_arity:tha=off:updr=off_9");
      quick.push("lrs+10_1_cond=on:fsr=off:gs=on:gsem=on:inst=on:lwlo=on:nm=64:nwc=2:stl=30:add=large:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:updr=off_38");
      quick.push("lrs+1010_2:1_bd=off:cond=on:er=filter:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=0:nwc=1.3:sas=z3:stl=30:add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=reverse_arity:updr=off_8");
      quick.push("dis+1_24_bs=on:fsr=off:irw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:av=off:sp=occurrence:urr=on_154");
      quick.push("lrs-11_3_bd=off:cond=on:gs=on:gsem=off:irw=on:inst=on:nm=4:nwc=3:sas=z3:stl=30:add=off:afr=on:afp=100000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_4");
      quick.push("dis+10_4_cond=on:inw=on:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:sas=z3:sos=all:add=large:afr=on:afp=10000:afq=1.0:anc=none:sp=occurrence:tha=off:updr=off_4");
      quick.push("ott+1010_3:1_bs=unit_only:ccuc=small_ones:fsr=off:irw=on:nm=0:newcnf=on:nwc=1.1:nicw=on:sos=on:aac=none:acc=model:add=off:afp=10000:afq=1.4:anc=none:sp=reverse_arity:tha=off_10");
      quick.push("lrs+11_8:1_gs=on:inst=on:lcm=reverse:lma=on:nm=64:nwc=2:sas=z3:stl=30:add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:updr=off_4");
      quick.push("dis+11_3:1_inw=on:inst=on:nm=64:nwc=1:sas=z3:sos=all:afp=40000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:updr=off_4");
      quick.push("dis+10_24_lma=on:nm=64:nwc=1:sas=z3:sac=on:add=large:amm=off:anc=none:sp=occurrence_1");
      quick.push("lrs+1002_5_gs=on:gsem=on:irw=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:afr=on:afp=10000:afq=1.1:amm=off:anc=none:tha=off:updr=off_10");
      quick.push("dis+11_3_cond=on:fsr=off:gs=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:add=off:afp=40000:afq=1.4:amm=sco:anc=none:sp=occurrence:tha=off:updr=off_1");
      quick.push("dis+11_4_fsr=off:gs=on:inst=on:lma=on:nm=64:nwc=1:sac=on:afp=4000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:thf=on:urr=on:updr=off_20");
      quick.push("dis-10_3:1_cond=on:inw=on:nm=2:newcnf=on:nwc=1:sos=on:av=off:tha=off:updr=off_10");
      quick.push("lrs+4_32_bd=off:fsr=off:gs=on:gsem=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sac=on:add=large:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=occurrence:tha=off:thf=on:updr=off_9");
      quick.push("dis-11_3_gs=on:gsem=on:inw=on:inst=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=1:av=off:sp=occurrence:thf=on_8");
      quick.push("lrs+4_10_cond=on:irw=on:nwc=1:stl=30:sd=10:ss=axioms:st=3.0:sos=all:av=off:sp=occurrence:updr=off_18");
      quick.push("dis+1010_3_gs=on:gsem=on:inw=on:irw=on:nm=64:nwc=3:sac=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:tha=off_4");
      quick.push("lrs-11_1_br=off:cond=on:gs=on:lwlo=on:nm=64:nwc=4:sas=z3:stl=30:sac=on:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_30");
      quick.push("dis+11_4_cond=on:inw=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:afr=on:afp=100000:afq=1.0:amm=off:anc=none:tha=off_14");
      quick.push("lrs+1011_3:1_cond=on:irw=on:nm=32:nwc=1:stl=30:add=large:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_2");
      quick.push("dis+1011_1_ccuc=small_ones:cond=on:fsr=off:gs=on:gsem=off:lma=on:nm=64:nwc=1:nicw=on:sos=all:sac=on:aac=none:acc=on:add=large:afr=on:afp=1000:afq=1.2:amm=sco:tha=off:thf=on_11");
      quick.push("lrs+1010_5:4_bd=off:bsr=on:gs=on:gsem=off:inw=on:nm=2:nwc=4:sas=z3:stl=30:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_3");
      quick.push("dis+11_5_gs=on:gsem=off:inw=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=on:add=off:afr=on:afp=4000:afq=1.4:anc=none:sp=occurrence:tha=off:updr=off_25");
      quick.push("ott+1010_5:4_bsr=on:ccuc=small_ones:fsr=off:inw=on:inst=on:lma=on:nm=16:nwc=1.1:aac=none:acc=on:add=off:afp=100000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:updr=off_41");
      quick.push("ott+1011_4_gs=on:irw=on:lma=on:nm=2:nwc=10:nicw=on:add=off:afr=on:afp=100000:afq=1.1:anc=none:tha=off:urr=on_14");
      quick.push("lrs+1002_2:3_fsr=off:gs=on:gsem=on:lwlo=on:nm=64:nwc=2:stl=30:sac=on:add=large:afp=10000:afq=1.2:amm=sco:anc=none:updr=off_57");
      quick.push("lrs+1011_2_bd=off:bs=unit_only:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1.2:sas=z3:stl=30:afp=10000:afq=2.0:anc=none:sp=occurrence:tha=off_12");
      quick.push("dis+1010_5:1_cond=on:gs=on:gsem=on:irw=on:nm=64:nwc=1:sas=z3:sos=on:afp=4000:afq=1.1:amm=off:anc=none:sp=occurrence:thf=on:urr=on_48");
      quick.push("dis+1010_8_bs=unit_only:fsr=off:gs=on:gsem=off:inw=on:lwlo=on:nm=16:nwc=1:sas=z3:sos=on:afr=on:afp=1000:afq=1.4:amm=off:anc=none:sp=reverse_arity:tha=off:urr=ec_only_71");
      quick.push("lrs-1_5:1_cond=on:er=filter:fsr=off:gs=on:gsem=on:inw=on:irw=on:inst=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1.1:stl=30:sos=all:acc=model:add=large:afp=4000:afq=2.0:sp=occurrence_53");
      quick.push("lrs-11_64_cond=on:gs=on:gsem=on:irw=on:nm=32:nwc=1:stl=30:sd=10:ss=axioms:st=5.0:sos=on:add=off:afp=4000:afq=1.4:sp=occurrence:urr=on_78");
      quick.push("dis+1002_5:4_bsr=on:fsr=off:lma=on:nm=4:nwc=1:sas=z3:aac=none:add=off:afr=on:afp=4000:afq=1.1:amm=off:tha=off:updr=off_120");
      quick.push("dis+11_2_bd=off:bs=on:bsr=on:ccuc=small_ones:inw=on:irw=on:lma=on:nm=16:nwc=1.3:nicw=on:acc=model:add=off:afp=100000:afq=1.1:sp=reverse_arity:tha=off:thf=on:urr=ec_only_197");
      quick.push("lrs+1011_4:1_bd=off:gs=on:gsem=off:lma=on:lwlo=on:nm=4:nwc=4:sas=z3:stl=30:aac=none:add=large:afp=40000:afq=2.0:anc=none:sp=occurrence:tha=off:updr=off_63");
      quick.push("lrs-11_4_lwlo=on:nm=64:nwc=1:stl=30:sos=all:av=off:sp=reverse_arity:thf=on:updr=off_158");
    }

    // add very long final fallback strategy
    fallback.push("lrs+1010_2:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:igrpq=1.0:nm=0:nwc=2.5:sac=on:acc=model:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+1003_2:1_bd=off:bs=unit_only:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=off:inw=on:inst=on:nm=0:nwc=3:sas=z3:sos=all:acc=model:add=off:aer=off:afr=on:afp=4000:afq=1.1:sp=occurrence:tha=off_300");
    fallback.push("lrs+11_5:1_bd=off:gs=on:gsem=off:gsssp=full:nwc=1.3:sas=minisat:sos=all:sac=on:aac=none:acc=model:add=large:afp=4000:afq=1.4:anc=none:sp=occurrence_300");
    fallback.push("ins+11_32_br=off:ep=RSTC:inw=on:igbrr=0.9:igrr=1/32:igrp=100:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:updr=off:dm=on_300");
    fallback.push("dis+11_4_cond=fast:fsr=off:gs=on:gsaa=from_current:gsem=on:nwc=1:sas=minisat:sd=10:ss=axioms:st=5.0:sos=all:aer=off:afp=4000:afq=1.0:anc=none:sp=occurrence_300");
    fallback.push("lrs+1_2_bs=on:bsr=on:fsr=off:fde=none:gs=on:nm=64:nwc=5:sas=minisat:aac=none:afr=on:amm=off:anc=all:sp=reverse_arity:tha=off_1500");
    fallback.push("lrs+11_8:1_br=off:cond=on:fsr=off:fde=none:gs=on:inw=on:igrpq=1.0:lwlo=on:nwc=1.5:aer=off:afp=10000:afq=1.2:anc=none:sp=reverse_arity:urr=on_300");
    fallback.push("lrs+1011_2:3_fsr=off:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=5:sas=z3:acc=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1010_12_bd=off:gs=on:gsaa=from_current:gsem=on:nm=0:nwc=4:sas=z3:aer=off:afp=4000:afq=1.2:anc=all:sp=occurrence:tha=off_300");
    fallback.push("lrs+1004_5:1_gs=on:gsaa=from_current:gsssp=full:nm=0:nwc=5:sas=z3:aac=none:add=off:aer=off:afp=40000:afq=1.1:anc=all_dependent_600");
    fallback.push("lrs+11_2:1_bd=off:bsr=on:br=off:fsr=off:fde=none:gs=on:gsem=off:nwc=1:sas=z3:sos=all:add=off:afp=40000:afq=1.1:amm=sco:anc=all:thf=on:urr=on:updr=off_300");
    fallback.push("dis+4_128_cond=fast:fsr=off:fde=unused:gs=on:gsem=on:inst=on:lcm=predicate:nwc=1:sas=minisat:add=large:aer=off:afr=on:afp=40000:afq=2.0:anc=none:tha=off:updr=off:uhcvi=on_300");
    fallback.push("lrs+4_3:1_bd=off:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:nwc=2.5:nicw=on:sas=z3:acc=model:aer=off:afr=on:afp=1000:afq=1.1:anc=none:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1010_5:4_bd=off:bs=unit_only:fde=none:gs=on:gsaa=full_model:gsem=on:inw=on:nwc=1.3:sas=z3:sos=all:afr=on:afp=4000:afq=1.1:amm=sco:sp=occurrence_300");
    fallback.push("dis+1004_8:1_bd=off:bsr=unit_only:ccuc=first:cond=fast:fde=unused:nm=0:nwc=1.2:sas=minisat:acc=on:afr=on:afp=1000:afq=2.0:anc=none:urr=on_300");
    fallback.push("dis+10_1024_cond=fast:fde=none:gs=on:gsem=off:nwc=1:sd=7:ss=axioms:st=5.0:sos=all:av=off:sp=reverse_arity_300");
    fallback.push("dis+1_1024_bs=on:cond=fast:fde=none:inst=on:nwc=1:av=off:sp=reverse_arity:tha=off:thf=on:urr=on:uhcvi=on_900");
    fallback.push("lrs-10_2:1_bsr=unit_only:gs=on:gsaa=from_current:lcm=reverse:nwc=1.1:nicw=on:sas=z3:aac=none:acc=on:add=large:afp=10000:afq=1.1:amm=sco:sp=occurrence:tha=off:urr=ec_only_900");
    fallback.push("lrs+1010_1_bs=on:bsr=unit_only:ccuc=first:cond=fast:gs=on:gsaa=from_current:inw=on:nwc=1:sas=z3:sos=on:sac=on:acc=on:add=large:afp=100000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("lrs+4_32_bsr=unit_only:cond=on:er=filter:fde=unused:gs=on:gsssp=full:inst=on:lcm=predicate:nm=64:nwc=1.1:ss=priority:st=2.0:sos=on:add=off:afp=10000:afq=1.2:amm=sco:sp=occurrence:urr=on:updr=off:uhcvi=on_300");
    fallback.push("ins+11_5_cond=on:gs=on:gsem=off:igbrr=0.3:igpr=on:igrr=1/32:igrp=200:igrpq=2.0:igs=1004:igwr=on:nwc=1:sos=all:av=off:sp=occurrence:dm=on_300");
    fallback.push("dis+1011_5_fsr=off:gs=on:gsaa=full_model:gsssp=full:nwc=1:sas=z3:sos=on:afp=40000:afq=2.0:amm=sco:anc=all:tha=off:updr=off_300");
    fallback.push("ott-1_5:1_ccuc=first:fsr=off:gs=on:lcm=reverse:nm=64:nwc=1.1:nicw=on:aac=none:acc=on:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on_300");
    fallback.push("dis+1010_5:1_fde=none:igrpq=1.0:lwlo=on:nm=0:nwc=1:sas=minisat:sos=on:add=off:afr=on:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off_300");
    fallback.push("ott-11_4_br=off:cond=on:gs=on:gsem=off:gsssp=full:igrpq=1.0:nwc=5:sas=minisat:sac=on:add=large:afr=on:afp=4000:afq=2.0:anc=all:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+1010_128_cond=fast:fde=unused:gs=on:gsssp=full:nm=1024:nwc=1.1:sas=z3:sos=on:aac=none:acc=model:add=large:aer=off:afp=4000:afq=1.1:anc=all_dependent:sp=occurrence:tha=off_300");
    fallback.push("ott+11_2:1_cond=on:gs=on:gsssp=full:lwlo=on:nwc=1:sas=minisat:sos=all:av=off:sp=occurrence:tha=off_300");
    fallback.push("dis+1003_3:2_bd=off:bsr=unit_only:nwc=1.3:sas=minisat:sac=on:add=large:aer=off:afr=on:afp=1000:afq=1.2:anc=none:updr=off_300");
    fallback.push("lrs+10_40_bd=off:bs=unit_only:bsr=unit_only:ccuc=first:fsr=off:gs=on:gsem=on:lcm=reverse:nwc=3:nicw=on:sas=z3:sac=on:aac=none:acc=model:add=off:afr=on:amm=off:anc=none:sp=occurrence:tha=off_300");
    fallback.push("lrs+2_8:1_cond=fast:er=filter:fde=unused:igrpq=1.0:lcm=predicate:nwc=2.5:sas=minisat:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=occurrence:updr=off_600");
    fallback.push("lrs+1002_3:1_cond=on:fde=unused:inst=on:nm=1024:nwc=2:sas=minisat:aer=off:afp=100000:afq=1.2:anc=none:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+1004_1_cond=fast:fde=unused:gs=on:gsem=off:gsssp=full:lwlo=on:nwc=2.5:sas=minisat:av=off:tha=off:thf=on:urr=ec_only:uhcvi=on_1500");
    fallback.push("dis+11_1_3600");
    return;
  }

  switch (cat) {
  case Property::NEQ:
    if (prop == 131079) {
      quick.push("dis+10_2:3_fde=unused:lcm=predicate:nwc=1:sas=minisat:sos=all:sac=on:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity_12");
      quick.push("dis+11_2:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:nwc=1:sas=minisat:sos=on:av=off:sp=reverse_arity:urr=on_9");
      quick.push("dis+11_5:4_cond=fast:fsr=off:nwc=10:av=off_5");
      quick.push("dis+11_5_bd=off:cond=fast:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=4:add=off:afp=4000:afq=1.4:amm=sco:sp=occurrence_2");
      quick.push("lrs+11_5:4_bd=off:gsp=input_only:gs=on:gsem=on:lcm=predicate:nwc=1:sas=minisat:stl=30:sos=all:av=off:sp=occurrence:urr=on_5");
      quick.push("ins+11_3_cond=on:igbrr=0.5:igrr=1/16:igrp=4000:igrpq=1.1:igs=1:igwr=on:nwc=4:av=off:sp=reverse_arity:dm=on_6");
      quick.push("lrs+11_5:4_bd=off:cond=on:fde=unused:nwc=3:stl=30:av=off:sp=occurrence:updr=off_68");
      quick.push("dis+11_2:3_cond=on:er=known:gs=on:nwc=1.5:add=off:afr=on:afp=4000:afq=2.0:anc=none_272");
      quick.push("lrs+10_8:1_bsr=unit_only:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=off:nwc=1:stl=30:sos=on:av=off:sp=occurrence:urr=on_1");
      quick.push("ott+1004_5:1_bd=off:bsr=unit_only:cond=fast:fsr=off:nwc=1:sos=all:av=off_13");
      quick.push("lrs+1011_10_cond=on:gsp=input_only:nwc=1.5:stl=30:av=off:sp=occurrence:updr=off_195");
      quick.push("dis+11_4_cond=fast:ep=R:gs=on:gsaa=from_current:gsem=on:nwc=1:sas=minisat:afp=1000:afq=1.4:amm=sco:anc=none:sp=occurrence:updr=off_277");
      quick.push("ins+11_5_cond=fast:gsp=input_only:igbrr=0.7:igrr=1/4:igs=1003:igwr=on:lcm=reverse:nwc=1:sos=all:av=off:urr=ec_only_58");
      quick.push("dis+11_3:2_bs=unit_only:cond=on:fde=unused:lcm=reverse:nwc=1:sos=all:av=off_178");
      quick.push("dis+11_7_268");
      quick.push("lrs+2_4:1_fsr=off:fde=none:gsp=input_only:lcm=predicate:lwlo=on:nwc=1.2:stl=90:av=off:sp=occurrence:urr=ec_only:updr=off_444");
      quick.push("dis+11_5_cond=on:gs=on:gsem=on:nwc=1:sos=on:sac=on:add=large:afp=4000:afq=1.1:amm=sco:anc=none:sp=occurrence:updr=off_13");
    }
    else if (prop == 3) {
      quick.push("dis+11_5:4_bd=off:cond=on:gs=on:gsssp=full:nwc=1:sas=minisat:sos=on:av=off:sp=occurrence_3");
      quick.push("ott+1003_3:1_bsr=unit_only:fsr=off:fde=unused:gs=on:gsem=on:nwc=10:sac=on:add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:urr=on_6");
      quick.push("lrs-11_2_bs=unit_only:cond=fast:lcm=predicate:nwc=1:sas=minisat:stl=30:sos=on:av=off:sp=occurrence:updr=off_1");
      quick.push("dis+1011_2_cond=on:ep=RST:gs=on:gsem=on:nwc=1:sac=on:afp=100000:afq=1.1:amm=off:anc=none:urr=ec_only_7");
      quick.push("dis+11_4_fde=unused:gs=on:gsem=on:gsssp=full:lcm=reverse:lwlo=on:nwc=4:sas=minisat:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_13");
      quick.push("ins+4_3_bsr=unit_only:fde=unused:igrr=1/2:igrp=2000:igrpq=2.0:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:dm=on_53");
      quick.push("dis+1011_2:1_cond=fast:gsp=input_only:gs=on:nwc=1:sas=minisat:sos=all:av=off_46");
      quick.push("lrs+11_2_bd=off:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:lcm=reverse:nwc=1:sas=minisat:stl=30:sos=on:add=large:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:updr=off_5");
      quick.push("lrs+11_5:4_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:av=off:sp=reverse_arity_88");
      quick.push("lrs+11_5_cond=fast:er=filter:nwc=1:stl=30:sos=all:av=off:urr=ec_only_67");
      quick.push("dis+11_5_cond=on:gs=on:gsem=on:nwc=1:sos=on:sac=on:add=large:afp=4000:afq=1.1:amm=sco:anc=none:sp=occurrence:updr=off_1");
      quick.push("lrs+1002_3:1_bd=off:bsr=unit_only:fde=none:gs=on:gsem=on:nwc=1:stl=30:sd=1:ss=axioms:av=off:sp=occurrence_2");
      quick.push("dis+1010_4_bs=unit_only:cond=fast:fsr=off:fde=unused:nwc=1:sos=on:add=off:afr=on:afp=10000:afq=2.0:sp=reverse_arity:updr=off_40");
      quick.push("dis+11_4_fsr=off:fde=none:gs=on:gsaa=from_current:nwc=1:afr=on:afp=1000:afq=2.0:anc=none:urr=on:updr=off_2");
      quick.push("dis+1_20_bs=unit_only:fsr=off:gs=on:gsem=on:gsssp=full:nwc=1.7:sas=minisat:av=off:sp=occurrence_28");
      quick.push("dis+10_3_gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:sos=all:add=off:afr=on:afp=4000:afq=1.0:amm=sco:anc=none:updr=off_2");
      quick.push("dis+11_5_gs=on:gsem=on:nwc=1:sd=1:ss=axioms:st=3.0:sac=on:add=large:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence_79");
      quick.push("ins+11_5_ep=RST:fsr=off:fde=none:gs=on:gsem=on:igbrr=0.8:igpr=on:igrr=1/32:igrp=200:igrpq=1.5:igs=1010:igwr=on:nwc=1:sas=minisat:sos=on:av=off:dm=on_32");
      quick.push("lrs+1003_5_bd=off:bsr=on:cond=on:fsr=off:fde=none:gsp=input_only:lwlo=on:nwc=1:stl=30:sos=all:av=off:sp=reverse_arity_47");
      quick.push("dis+10_2:3_fde=unused:lcm=predicate:nwc=1:sas=minisat:sos=all:sac=on:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity_63");
      quick.push("lrs+2_4:1_fsr=off:fde=none:gsp=input_only:lcm=predicate:lwlo=on:nwc=1.2:stl=90:av=off:sp=occurrence:urr=ec_only:updr=off_211");
      quick.push("lrs+1011_10_cond=on:gsp=input_only:nwc=1.5:stl=30:av=off:sp=occurrence:updr=off_163");
      quick.push("dis+1004_3_fsr=off:fde=none:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.5:sos=all:av=off:sp=reverse_arity_83");
      quick.push("ins+11_3_cond=on:igbrr=0.5:igrr=1/16:igrp=4000:igrpq=1.1:igs=1:igwr=on:nwc=4:av=off:sp=reverse_arity:dm=on_96");
      quick.push("dis+10_24_gs=on:nwc=1:sas=minisat:sd=3:ss=axioms:sos=on:av=off:sp=reverse_arity:updr=off_163");
      quick.push("lrs-11_3:1_bd=off:ccuc=small_ones:cond=fast:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=30:sos=all:acc=on:add=large:aer=off:afp=40000:afq=1.0:anc=none:sp=reverse_arity:updr=off_17");
    }
    else {
      quick.push("dis+1010_4_bs=unit_only:cond=fast:fsr=off:fde=unused:nwc=1:sos=on:add=off:afr=on:afp=10000:afq=2.0:sp=reverse_arity:updr=off_11");
      quick.push("dis+11_4_fde=unused:gs=on:gsem=on:gsssp=full:lcm=reverse:lwlo=on:nwc=4:sas=minisat:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_30");
      quick.push("dis+1011_2_bs=unit_only:cond=fast:er=filter:fsr=off:fde=unused:nwc=2.5:aac=none:afp=4000:afq=1.0:amm=off:sp=occurrence:updr=off_18");
      quick.push("lrs+1011_8_bd=preordered:cond=on:fsr=off:fde=none:gs=on:gsssp=full:lcm=reverse:nwc=1.7:sas=minisat:stl=30:av=off:sp=reverse_arity:urr=ec_only_8");
      quick.push("ins+4_3_bsr=unit_only:fde=unused:igrr=1/2:igrp=2000:igrpq=2.0:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:dm=on_61");
      quick.push("dis+11_5_gs=on:gsem=on:nwc=1:sd=1:ss=axioms:st=3.0:sac=on:add=large:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence_2");
      quick.push("lrs+11_5:4_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:av=off:sp=reverse_arity_1");
      quick.push("ins+11_5_br=off:cond=fast:ep=RS:igbrr=0.9:igrr=1/128:igrp=400:igs=1003:igwr=on:nwc=1:sas=minisat:av=off:urr=on:dm=on_7");
      quick.push("ott+1011_2_er=known:fde=none:nwc=1:av=off:sp=occurrence_141");
      quick.push("lrs+11_2_bd=off:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:lcm=reverse:nwc=1:sas=minisat:stl=30:sos=on:add=large:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:updr=off_26");
      quick.push("dis+1011_2_cond=on:ep=RST:gs=on:gsem=on:nwc=1:sac=on:afp=100000:afq=1.1:amm=off:anc=none:urr=ec_only_5");
      quick.push("dis+11_5_cond=on:gs=on:gsem=on:nwc=1:sos=on:sac=on:add=large:afp=4000:afq=1.1:amm=sco:anc=none:sp=occurrence:updr=off_1");
      quick.push("lrs+2_4:1_fsr=off:fde=none:gsp=input_only:lcm=predicate:lwlo=on:nwc=1.2:stl=90:av=off:sp=occurrence:urr=ec_only:updr=off_233");
      quick.push("dis+11_3:2_bs=unit_only:cond=on:fde=unused:lcm=reverse:nwc=1:sos=all:av=off_268");
      quick.push("lrs+1010_10_bd=off:fsr=off:fde=none:nwc=4:sas=minisat:stl=30:aac=none:add=off:aer=off:afp=4000:afq=1.4:sp=occurrence:urr=ec_only:updr=off_294");
      quick.push("lrs+1011_128_bsr=unit_only:cond=fast:lwlo=on:nwc=2:stl=30:sos=all:av=off:urr=on:updr=off_50");
      quick.push("lrs+11_2:1_bs=unit_only:bsr=unit_only:fsr=off:fde=none:gsp=input_only:lcm=reverse:lwlo=on:nwc=1:stl=60:sos=on:av=off:urr=ec_only_208");
      quick.push("lrs+11_5:4_bd=off:cond=on:fde=unused:nwc=3:stl=30:av=off:sp=occurrence:updr=off_188");
      quick.push("lrs+1011_10_cond=on:gsp=input_only:nwc=1.5:stl=30:av=off:sp=occurrence:updr=off_213");
      quick.push("dis+1010_6_bd=off:bsr=unit_only:ccuc=first:cond=fast:fsr=off:fde=unused:lwlo=on:nwc=1:sas=minisat:sos=on:sac=on:acc=model:add=large:aer=off:afp=1000:afq=1.1:anc=all_dependent_130");
      quick.push("dis+11_7_182");
      quick.push("dis+10_2:3_fde=unused:lcm=predicate:nwc=1:sas=minisat:sos=all:sac=on:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity_221");
    }
    break;

  case Property::HEQ:
    quick.push("lrs+11_1_bd=off:bsr=unit_only:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:gsssp=full:lwlo=on:nwc=1:sas=minisat:stl=30:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_20");
    quick.push("lrs+11_2:3_cond=on:gs=on:gsem=on:lwlo=on:nwc=1.7:sas=minisat:stl=30:av=off:updr=off_123");
    quick.push("ott+11_24_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=3:add=off:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence_2");
    quick.push("dis+1011_5_cond=fast:fsr=off:gs=on:gsaa=full_model:nwc=1:sas=minisat:sos=all:add=off:afp=4000:afq=1.2:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_1");
    quick.push("dis+1010_24_cond=fast:ep=RS:fde=unused:lwlo=on:nwc=1.5:sas=minisat:aer=off:afr=on:afp=1000:afq=1.1:anc=none_2");
    quick.push("ins+11_4_cond=fast:fsr=off:igbrr=0.8:igpr=on:igrr=1/8:igrp=200:igrpq=1.5:igs=1003:igwr=on:nwc=10:av=off:sp=occurrence:updr=off_304");
    quick.push("lrs+11_3:1_bd=off:fde=none:gs=on:lwlo=on:nwc=2:sas=minisat:stl=90:sac=on:add=off:afr=on:afp=4000:afq=1.4:amm=sco:anc=none:sp=occurrence_64");
    quick.push("lrs+10_5_bd=preordered:cond=on:fde=none:lcm=reverse:lwlo=on:nwc=1.7:stl=30:av=off:sp=occurrence_9");
    quick.push("ins+10_4_cond=on:fsr=off:fde=none:igbrr=0.6:igrr=1/32:igrp=2000:igrpq=1.05:igs=1002:igwr=on:nwc=5:av=off:sp=occurrence:updr=off:dm=on_175");
    quick.push("lrs+11_10_gs=on:gsem=on:lcm=reverse:nwc=1:stl=30:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_132");
    quick.push("dis+11_5:4_fsr=off:fde=none:gs=on:gsem=off:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:urr=on_169");
    quick.push("lrs+11_1024_bd=off:fsr=off:gs=on:gsem=on:nwc=1:stl=30:av=off:sp=occurrence:urr=on_223");
    quick.push("ott+11_5:4_cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:nwc=1:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=occurrence:updr=off_195");
    quick.push("dis+11_7_259");
    break;

  case Property::PEQ:
    if (prop == 1) {
      quick.push("lrs+11_3_bsr=unit_only:er=known:fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=1.3:sas=minisat:stl=30:add=large:amm=sco:anc=none_1");
      quick.push("dis+11_7_9");
      quick.push("lrs+10_10_er=known:fde=none:gs=on:gsem=on:nwc=1.7:stl=30:av=off:updr=off_5");
      quick.push("lrs+11_5:4_bsr=unit_only:ccuc=small_ones:cond=on:fsr=off:nwc=1:sas=minisat:stl=30:sac=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=reverse_arity_3");
      quick.push("dis+1002_3_bsr=unit_only:cond=on:nwc=1.2:nicw=on:sos=all:add=large:aer=off:sp=occurrence:updr=off_11");
      quick.push("dis+10_4_bsr=unit_only:gs=on:gsssp=full:nwc=1.5:nicw=on:sas=minisat:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:updr=off_25");
      quick.push("lrs+1011_3_cond=on:fsr=off:fde=none:gs=on:gsssp=full:nwc=1:stl=30:sos=all:av=off:sp=reverse_arity:updr=off_53");
      quick.push("lrs+10_3:1_bs=on:bsr=unit_only:gs=on:gsem=off:gsssp=full:lwlo=on:nwc=1:sas=minisat:stl=240:av=off:sp=reverse_arity:updr=off_354");
      quick.push("lrs+11_128_bs=unit_only:fde=unused:gs=on:gsem=off:gsssp=full:nwc=1:nicw=on:sas=minisat:stl=120:sos=on:sac=on:aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=sco:anc=all_799");
    }
    else if (prop == 131073) {
      quick.push("lrs+10_3:1_bs=on:bsr=unit_only:gs=on:gsem=off:gsssp=full:lwlo=on:nwc=1:sas=minisat:stl=240:av=off:sp=reverse_arity:updr=off_2277");
    }
    else {
      quick.push("lrs+11_14_bs=unit_only:bsr=unit_only:cond=on:nwc=1:sas=minisat:stl=30:add=off:aer=off:afp=1000:afq=1.1:anc=none:sp=occurrence_18");
      quick.push("lrs+11_2_ccuc=small_ones:cond=fast:fsr=off:gs=on:gsem=off:gsssp=full:lwlo=on:nwc=1:nicw=on:sas=minisat:stl=60:sac=on:acc=model:add=off:afp=100000:afq=1.2:amm=off:anc=all_dependent:sp=reverse_arity:updr=off_96");
      quick.push("lrs+2_7_bs=unit_only:bsr=unit_only:cond=on:fsr=off:gs=on:nwc=1.7:sas=minisat:stl=30:sos=on:sac=on:afr=on:afp=100000:afq=1.0:amm=off:anc=all_dependent_49");
      quick.push("dis+11_5_fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sos=all:av=off:sp=reverse_arity_24");
      quick.push("dis+1002_3_bsr=unit_only:cond=on:nwc=1.2:nicw=on:sos=all:add=large:aer=off:sp=occurrence:updr=off_49");
      quick.push("dis+11_64_bs=unit_only:cond=on:fde=none:nwc=2:av=off:updr=off_162");
      quick.push("lrs-1_3_fsr=off:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1.2:sas=minisat:stl=30:sac=on:add=off:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_131");
      quick.push("dis+10_4_bsr=unit_only:gs=on:gsssp=full:nwc=1.5:nicw=on:sas=minisat:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:updr=off_114");
      quick.push("dis+1_64_bs=unit_only:cond=fast:fde=none:gs=on:gsaa=from_current:gsem=off:nwc=3:nicw=on:sas=minisat:sos=on:sac=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=all_dependent:sp=reverse_arity:updr=off_441");
      quick.push("lrs+11_4_bs=unit_only:cond=fast:fde=none:gs=on:lwlo=on:nwc=1:stl=30:afp=1000:afq=1.2:anc=none:sp=occurrence_262");
      quick.push("ott+10_8_bsr=unit_only:gs=on:gsaa=from_current:gsem=on:nwc=4:sas=minisat:afr=on:afp=4000:afq=1.4:amm=off:anc=all_536");
      quick.push("lrs+11_128_bs=unit_only:fde=unused:gs=on:gsem=off:gsssp=full:nwc=1:nicw=on:sas=minisat:stl=120:sos=on:sac=on:aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=sco:anc=all_691");
    }
    break;

  case Property::HNE:
    quick.push("dis+1002_5_gs=on:gsem=off:gsssp=full:lwlo=on:nwc=1:sas=minisat:sos=on:sac=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:sp=reverse_arity_1");
    quick.push("lrs+2_50_bs=unit_only:bsr=unit_only:cond=fast:fsr=off:nwc=1:stl=30:av=off:sp=occurrence_84");
    quick.push("dis+1_64_gsp=input_only:nwc=1.2:sos=all:av=off:sp=occurrence:updr=off_34");
    quick.push("dis+1_40_bs=unit_only:fsr=off:nwc=1:sas=minisat:add=large:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:updr=off_110");
    quick.push("lrs+11_3_fsr=off:gs=on:gsssp=full:nwc=2:stl=60:av=off:sp=occurrence:urr=on:updr=off_35");
    quick.push("lrs+11_1_bs=on:nwc=1.1:stl=30:av=off:sp=occurrence:urr=ec_only:updr=off_128");
    quick.push("dis+11_2_cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:nwc=5:sac=on:aac=none:afr=on:afp=4000:afq=1.4:amm=off:anc=none:sp=reverse_arity_68");
    quick.push("lrs+11_1_bsr=unit_only:cond=on:fsr=off:gs=on:gsem=on:lwlo=on:nwc=2:sas=minisat:stl=90:av=off_888");
    quick.push("dis+11_8:1_br=off:cond=fast:fsr=off:nwc=1:sos=all:add=off:aer=off:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:urr=on:updr=off_223");
    quick.push("dis+1_3_cond=on:fsr=off:nwc=1.1:sas=minisat:av=off:sp=reverse_arity:urr=ec_only:updr=off_155");
    break;

  case Property::NNE:
    quick.push("lrs+11_3_cond=on:fsr=off:lcm=reverse:nwc=1:sas=minisat:stl=30:sos=all:add=off:aer=off:afr=on:afp=1000:afq=1.4:anc=none:sp=occurrence_2");
    quick.push("lrs+10_4_cond=fast:nwc=1:nicw=on:sas=minisat:stl=60:afr=on:afp=10000:afq=1.2:amm=off_47");
    quick.push("dis+1003_50_cond=fast:nwc=1:sos=on:av=off:sp=occurrence_89");
    quick.push("dis+11_64_fsr=off:gsp=input_only:nwc=1.1:sos=all:av=off:sp=occurrence:updr=off_259");
    quick.push("dis+3_64_cond=fast:lcm=reverse:nwc=1.1:sos=on:av=off:updr=off_62");
    quick.push("dis+1011_2_nwc=1:sas=minisat:sos=on:av=off:sp=occurrence_204");
    quick.push("dis+1002_10_bsr=unit_only:cond=fast:nwc=1:sos=all:av=off:sp=occurrence_136");
    quick.push("dis+11_7_198");
    quick.push("dis-2_5_cond=on:nwc=1:sas=minisat:av=off:sp=occurrence_228");
    quick.push("lrs+1004_28_nwc=1.1:stl=60:sos=all:av=off_495");
    break;

  case Property::FEQ:
    if (atoms > 1000000) {
      quick.push("dis+11_4_bsr=unit_only:cond=fast:fde=none:lwlo=on:nm=0:nwc=1.2:av=off:sp=occurrence_1218");
      quick.push("lrs+1011_4_cond=fast:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:stl=90:sd=5:ss=axioms:st=3.0:sac=on:add=off:aer=off:afr=on:afp=1000:afq=1.0:anc=none_693");
    }
    else if (atoms > 200000) {
      quick.push("lrs+1002_3_ep=RST:er=known:fsr=off:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=30:sd=2:ss=axioms:st=5.0:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:sp=occurrence_51");
      quick.push("lrs+1002_2:3_bsr=unit_only:er=known:fde=none:gs=on:gsem=on:nwc=1:sas=minisat:stl=30:sd=1:ss=axioms:st=1.5:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_45");
      quick.push("dis-2_4_cond=fast:ep=RST:er=filter:fde=unused:gs=on:gsem=on:lcm=reverse:nwc=1:sd=3:ss=axioms:sos=on:av=off:updr=off_35");
      quick.push("dis-3_10_bsr=unit_only:er=filter:fde=unused:lcm=predicate:nm=64:nwc=1:av=off:sp=occurrence:updr=off_187");
      quick.push("ins+11_3_cond=fast:fde=none:igbrr=0.8:igrr=1/16:igrp=200:igrpq=1.05:igs=1003:igwr=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:av=off:updr=off_46");
      quick.push("dis+2_5:4_fde=none:nwc=1:sd=2:ss=axioms:sos=on:afp=40000:afq=2.0_27");
      quick.push("dis+11_4_cond=fast:fsr=off:gs=on:gsaa=from_current:gsem=on:nwc=1:sas=minisat:sd=10:ss=axioms:st=5.0:sos=all:aer=off:afp=4000:afq=1.0:anc=none:sp=occurrence_55");
      quick.push("lrs+1010_4:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=on:nm=0:nwc=1:sas=minisat:stl=30:sd=2:ss=axioms:st=1.5:sos=on:sac=on:add=off:aer=off:afr=on:afp=100000:afq=1.4:anc=none:sp=occurrence_21");
      quick.push("dis+11_5_fde=none:gs=on:gsaa=from_current:gsssp=full:lcm=reverse:nwc=1:sas=minisat:ss=axioms:st=1.5:sos=on:afp=4000:afq=1.2:amm=sco:anc=none_124");
      quick.push("lrs-4_4_cond=fast:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:gsssp=full:lcm=reverse:nwc=1:sas=minisat:stl=30:sd=4:ss=axioms:sos=on:sac=on:aac=none:add=large:aer=off:afp=1000:afq=1.2:anc=none:sp=reverse_arity_43");
      quick.push("ott+11_8:1_cond=fast:gs=on:gsem=off:nwc=1:nicw=on:sd=2:ss=axioms:st=1.2:sos=all:acc=on:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:urr=on_47");
      quick.push("ott+1011_10_cond=fast:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:sos=all:av=off:sp=occurrence:updr=off_124");
      quick.push("lrs+1004_2:1_cond=fast:fde=none:gs=on:gsem=off:nwc=1:stl=30:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity:updr=off_45");
      quick.push("ins+11_4_cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:igbrr=0.3:igpr=on:igrr=1/8:igrp=100:igrpq=1.5:igwr=on:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:av=off:dm=on_66");
      quick.push("lrs+4_2:1_ep=R:fde=unused:gs=on:nwc=1:stl=30:sos=all:sac=on:aac=none:aer=off:afr=on:afp=10000:afq=1.2:anc=none:sp=occurrence:updr=off_281");
      quick.push("lrs-2_5_cond=on:fde=none:lcm=predicate:nwc=1:stl=30:sd=2:ss=axioms:st=3.0:sos=on:add=off:afp=100000:afq=1.4:anc=none_98");
      quick.push("dis+4_3:1_gs=on:nwc=1:sd=1:ss=axioms:sos=all:av=off:updr=off_51");
      quick.push("dis+11_2_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_197");
      quick.push("lrs-1_2:1_bd=preordered:bsr=on:br=off:cond=on:gs=on:lcm=reverse:nwc=1:stl=30:sd=2:ss=axioms:sos=on:add=large:aer=off:afp=100000:afq=2.0:anc=none:sp=occurrence:urr=on:updr=off_135");
      quick.push("ott+11_24_cond=fast:ep=RST:fsr=off:fde=none:gs=on:lcm=predicate:nm=64:nwc=1:sas=minisat:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence_282");
      quick.push("lrs+1_3:1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=30:sd=2:ss=axioms:st=1.2:sos=on:sac=on:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=reverse_arity_190");
      quick.push("dis+1003_8:1_bd=off:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:lcm=reverse:nwc=1:sas=minisat:sos=all:aac=none:acc=on:add=off:afr=on:amm=sco:anc=none:sp=reverse_arity_150");
      quick.push("lrs+11_3:2_cond=fast:fsr=off:fde=none:gs=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:av=off:urr=on_227");
    }
    else if (prop == 0) {
      if (atoms >= 1000) {
        quick.push("dis+11_7_41");
        quick.push("dis+11_1024_br=off:ep=RSTC:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sac=on:afp=40000:afq=1.0:anc=none:sp=reverse_arity:urr=on_172");
        quick.push("ott+2_8_bsr=unit_only:cond=fast:fde=none:gs=on:lwlo=on:nwc=1:sas=minisat:av=off_293");
        quick.push("dis+4_64_bs=unit_only:cond=on:er=known:fde=unused:gs=on:gsem=off:nwc=1.2:sas=minisat:aac=none:aer=off:afr=on:afp=10000:afq=2.0:anc=all:sp=reverse_arity:updr=off_281");
        quick.push("lrs+3_5_bd=preordered:bsr=unit_only:gsp=input_only:gs=on:gsem=on:gsssp=full:lwlo=on:nm=64:nwc=1:sas=minisat:stl=90:av=off:sp=occurrence:urr=ec_only:updr=off_778");
        quick.push("dis+11_4_bsr=unit_only:cond=fast:fde=none:lwlo=on:nm=0:nwc=1.2:av=off:sp=occurrence_926");
      }
      else {
        quick.push("dis+11_7_3");
        quick.push("lrs-11_2_cond=on:fde=unused:gs=on:nwc=3:stl=30:add=off:afr=on:afp=100000:afq=1.4:amm=sco:anc=all_4");
        quick.push("lrs+11_2_br=off:cond=on:fde=none:gs=on:gsaa=full_model:lwlo=on:nwc=2:sas=minisat:stl=30:afp=100000:afq=1.4:amm=sco:anc=none:sp=occurrence:urr=on_98");
        quick.push("dis+11_5_fde=none:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=all:add=large:aer=off:afr=on:afp=100000:afq=2.0:anc=none_37");
        quick.push("lrs+1002_2:3_br=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=30:sac=on:aer=off:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:urr=on_4");
        quick.push("lrs+1011_12_bs=on:bsr=unit_only:cond=on:gs=on:gsaa=from_current:gsssp=full:nwc=1.1:sas=minisat:stl=60:sos=all:sac=on:add=off:aer=off:afr=on:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_14");
        quick.push("lrs+10_8:1_bd=preordered:bs=on:ccuc=first:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:nicw=on:sas=minisat:stl=120:sos=on:acc=on:aer=off:afr=on:afp=4000:afq=1.0:anc=none:sp=reverse_arity:urr=on_7");
        quick.push("lrs+1_5:4_cond=on:fsr=off:fde=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:stl=60:sos=all:av=off_80");
        quick.push("lrs+11_28_cond=on:gs=on:gsaa=from_current:gsem=on:gsssp=full:lwlo=on:nm=64:nwc=1.7:stl=30:sac=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:urr=on:updr=off_109");
        quick.push("ott+11_3:1_br=off:gs=on:gsem=on:nwc=1:sas=minisat:sos=all:av=off:urr=on:updr=off_135");
        quick.push("lrs+3_5_bd=preordered:bsr=unit_only:gsp=input_only:gs=on:gsem=on:gsssp=full:lwlo=on:nm=64:nwc=1:sas=minisat:stl=90:av=off:sp=occurrence:urr=ec_only:updr=off_815");
        quick.push("lrs+1011_4_cond=fast:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:stl=90:sd=5:ss=axioms:st=3.0:sac=on:add=off:aer=off:afr=on:afp=1000:afq=1.0:anc=none_402");
      }
    }
    else if (prop == 1) {
      if (atoms > 80) {
        quick.push("lrs+11_1_br=off:cond=on:er=known:fsr=off:fde=unused:nwc=1:stl=30:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=occurrence:urr=on_11");
        quick.push("ott+1002_5_bsr=on:fsr=off:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:acc=on:aer=off:afp=100000:afq=1.1:sp=occurrence_3");
        quick.push("lrs+1011_3:1_bsr=unit_only:cond=fast:er=known:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sas=minisat:stl=30:add=large:aer=off:afr=on:afp=100000:afq=2.0:updr=off_5");
        quick.push("ins+11_3_cond=on:fde=none:gs=on:gsem=off:gsssp=full:igbrr=1.0:igrr=1/16:igrp=100:igrpq=1.05:igs=1:igwr=on:nwc=1:sas=minisat:sos=on:av=off:sp=occurrence:urr=on_4");
        quick.push("ott-1_2:3_bs=unit_only:bsr=unit_only:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:sos=on:add=large:afr=on:afp=1000:afq=1.4:amm=off:anc=all:sp=occurrence_58");
        quick.push("dis+1011_4_fde=none:gs=on:nwc=1:sos=on:add=off:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:updr=off_67");
        quick.push("dis+11_4_cond=fast:fsr=off:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:ss=axioms:st=2.0:sos=all:add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:updr=off_86");
        quick.push("dis+1002_4_cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:nm=64:nwc=1:sas=minisat:sos=on:acc=model:aer=off:afr=on:afp=4000:afq=1.4:anc=none:updr=off_6");
        quick.push("lrs+1010_4_fde=unused:gs=on:gsem=off:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on:sac=on:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_3");
        quick.push("ott+2_8_bsr=unit_only:cond=fast:fde=none:gs=on:lwlo=on:nwc=1:sas=minisat:av=off_278");
        quick.push("lrs+1002_3_bd=off:bs=on:bsr=unit_only:cond=fast:fsr=off:lcm=predicate:nwc=1.5:stl=30:sos=on:add=off:aer=off:afr=on:sp=reverse_arity_32");
        quick.push("ott+11_5_nwc=1:sd=7:ss=axioms:st=2.0:sac=on:afr=on:afp=40000:afq=1.2:anc=none_380");
        quick.push("lrs+10_8_br=off:fsr=off:gsp=input_only:gs=on:gsem=off:nwc=1:sas=minisat:stl=30:av=off:sp=reverse_arity:urr=on:updr=off_118");
        quick.push("lrs+3_5_bd=preordered:bsr=unit_only:gsp=input_only:gs=on:gsem=on:gsssp=full:lwlo=on:nm=64:nwc=1:sas=minisat:stl=90:av=off:sp=occurrence:urr=ec_only:updr=off_394");
        quick.push("lrs+3_4:1_bs=unit_only:bsr=on:cond=on:er=known:fde=none:lcm=predicate:lwlo=on:nwc=1.5:sas=minisat:stl=210:sos=all:afr=on:afp=100000:afq=1.1:amm=sco:anc=all_dependent:sp=occurrence:updr=off_1497");
      }
      else {
        quick.push("lrs+10_50_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:nwc=1.3:nicw=on:stl=30:sos=all:add=off:aer=off:afr=on:afp=10000:afq=2.0:anc=none:sp=occurrence_4");
        quick.push("dis+1003_8:1_bd=off:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:lcm=reverse:nwc=1:sas=minisat:sos=all:aac=none:acc=on:add=off:afr=on:amm=sco:anc=none:sp=reverse_arity_2");
        quick.push("dis+11_7_297");
        quick.push("ott+1004_28_cond=fast:nwc=1:sos=all:av=off:updr=off_109");
        quick.push("lrs+10_14_cond=on:gs=on:gsem=off:nwc=1.5:nicw=on:sas=minisat:stl=30:sac=on:afr=on:afp=4000:afq=1.2:anc=all:sp=reverse_arity:updr=off_159");
        quick.push("lrs+11_1_br=off:cond=on:er=known:fsr=off:fde=unused:nwc=1:stl=30:sac=on:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=occurrence:urr=on_78");
        quick.push("ott+1011_1024_bd=preordered:bs=on:cond=on:nwc=1:av=off:updr=off_173");
        quick.push("dis+4_64_bs=unit_only:cond=on:er=known:fde=unused:gs=on:gsem=off:nwc=1.2:sas=minisat:aac=none:aer=off:afr=on:afp=10000:afq=2.0:anc=all:sp=reverse_arity:updr=off_174");
        quick.push("lrs-1_4_cond=fast:ep=RST:fde=unused:gs=on:gsem=on:gsssp=full:lwlo=on:nwc=1:sas=minisat:stl=30:av=off:urr=ec_only_259");
        quick.push("lrs+3_4:1_bs=unit_only:bsr=on:cond=on:er=known:fde=none:lcm=predicate:lwlo=on:nwc=1.5:sas=minisat:stl=210:sos=all:afr=on:afp=100000:afq=1.1:amm=sco:anc=all_dependent:sp=occurrence:updr=off_1609");
      }
    }
    else if (prop == 2) {
      if (atoms <= 17) {
        quick.push("ott+10_8_bs=on:bsr=unit_only:cond=fast:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:lcm=reverse:nwc=1.3:sas=minisat:sac=on:add=off:aer=off:afp=4000:afq=1.0:anc=all_dependent:sp=reverse_arity_3");
        quick.push("ott+1010_40_bs=unit_only:cond=fast:nwc=1:sas=minisat:add=off:afp=10000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_185");
        quick.push("lrs+1002_6_ccuc=first:cond=on:fsr=off:nwc=4:nicw=on:sas=minisat:stl=30:acc=on:afr=on:afp=40000:afq=1.0:amm=sco:anc=all_dependent:sp=reverse_arity:urr=ec_only_122");
        quick.push("lrs+1011_4_cond=fast:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:stl=90:sd=5:ss=axioms:st=3.0:sac=on:add=off:aer=off:afr=on:afp=1000:afq=1.0:anc=none_864");
        quick.push("dis+1011_24_cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsem=off:lwlo=on:nm=64:nwc=1:sos=on:av=off:sp=occurrence_129");
        quick.push("dis+11_8_bs=unit_only:nwc=1:sd=10:ss=axioms:st=1.5:av=off:sp=occurrence:updr=off_238");
        quick.push("lrs+1011_12_bs=on:bsr=unit_only:cond=on:gs=on:gsaa=from_current:gsssp=full:nwc=1.1:sas=minisat:stl=60:sos=all:sac=on:add=off:aer=off:afr=on:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_548");
      }
      else {
        quick.push("dis+1002_3_cond=fast:fsr=off:nwc=2.5:sd=3:ss=axioms:st=1.5:av=off:updr=off_5");
        quick.push("lrs+4_64_fsr=off:nwc=1.5:sas=minisat:stl=30:av=off:sp=occurrence_15");
        quick.push("lrs+10_14_cond=on:gs=on:gsem=off:nwc=1.5:nicw=on:sas=minisat:stl=30:sac=on:afr=on:afp=4000:afq=1.2:anc=all:sp=reverse_arity:updr=off_7");
        quick.push("ott+11_5_nwc=1:sd=7:ss=axioms:st=2.0:sac=on:afr=on:afp=40000:afq=1.2:anc=none_117");
        quick.push("ott+10_8_bs=on:bsr=unit_only:cond=fast:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:lcm=reverse:nwc=1.3:sas=minisat:sac=on:add=off:aer=off:afp=4000:afq=1.0:anc=all_dependent:sp=reverse_arity_14");
        quick.push("lrs+11_28_cond=on:gs=on:gsaa=from_current:gsem=on:gsssp=full:lwlo=on:nm=64:nwc=1.7:stl=30:sac=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:urr=on:updr=off_106");
        quick.push("lrs+10_5:4_bsr=unit_only:fde=none:lcm=reverse:nm=64:nwc=10:stl=30:av=off:sp=occurrence:updr=off_26");
        quick.push("dis+11_20_fsr=off:fde=unused:gs=on:gsssp=full:nm=0:nwc=1.3:nicw=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=all:sp=reverse_arity_162");
        quick.push("ott+11_14_bd=preordered:fsr=off:gs=on:gsem=off:nwc=2:afp=4000:afq=1.2:amm=off:anc=none:sp=reverse_arity:updr=off_214");
        quick.push("ott+4_8_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:nwc=1:sos=all:sac=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity_161");
        quick.push("lrs+4_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:nwc=1:stl=90:sd=5:ss=axioms:st=1.2:sac=on:add=off:aer=off:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_341");
        quick.push("ott+1010_40_bs=unit_only:cond=fast:nwc=1:sas=minisat:add=off:afp=10000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_383");
        quick.push("ott+11_20_bs=unit_only:fsr=off:gsp=input_only:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sac=on:afp=1000:afq=2.0:anc=none:sp=occurrence_538");
        quick.push("lrs+1002_7_ccuc=first:cond=on:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:stl=90:sd=3:ss=axioms:acc=model:aer=off:afp=1000:afq=2.0:anc=none:sp=reverse_arity_542");
      }
    }
    else if (prop == 131073) {
      if (atoms > 300) {
        quick.push("dis+11_5_bd=off:cond=fast:fde=unused:gs=on:gsem=on:lwlo=on:nwc=1:sos=on:sac=on:add=off:aer=off:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:urr=on_1");
        quick.push("lrs+1002_3_bd=off:bs=on:bsr=unit_only:cond=fast:fsr=off:lcm=predicate:nwc=1.5:stl=30:sos=on:add=off:aer=off:afr=on:sp=reverse_arity_7");
        quick.push("dis+1011_2_fde=unused:gs=on:nwc=1:sac=on:afp=40000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=ec_only_20");
        quick.push("dis+11_7_fde=none:nm=0:nwc=1:sd=3:ss=axioms:st=2.0:av=off:sp=occurrence:urr=on:updr=off_8");
        quick.push("dis+1010_3:1_cond=fast:fde=unused:gs=on:nwc=1:sd=2:ss=axioms:sos=on:add=large:aer=off:afr=on:afp=100000:afq=1.2:updr=off_18");
        quick.push("lrs+1010_8:1_fsr=off:fde=none:gs=on:gsem=on:gsssp=full:nwc=1.1:sas=minisat:stl=30:sos=all:aac=none:afr=on:afp=100000:afq=1.0:amm=off:anc=all:sp=occurrence:updr=off_9");
        quick.push("lrs+4_2:1_ep=R:fde=unused:gs=on:nwc=1:stl=30:sos=all:sac=on:aac=none:aer=off:afr=on:afp=10000:afq=1.2:anc=none:sp=occurrence:updr=off_1");
        quick.push("dis-3_10_bsr=unit_only:er=filter:fde=unused:lcm=predicate:nm=64:nwc=1:av=off:sp=occurrence:updr=off_109");
        quick.push("lrs-1_4_cond=fast:ep=RST:fde=unused:gs=on:gsem=on:gsssp=full:lwlo=on:nwc=1:sas=minisat:stl=30:av=off:urr=ec_only_32");
        quick.push("dis+1010_5:1_bs=unit_only:ccuc=small_ones:cond=on:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=3:sos=on:sac=on:acc=on:afp=40000:afq=1.2:amm=sco:anc=all_dependent:sp=occurrence:urr=ec_only_2");
        quick.push("lrs-2_5_cond=on:fde=none:lcm=predicate:nwc=1:stl=30:sd=2:ss=axioms:st=3.0:sos=on:add=off:afp=100000:afq=1.4:anc=none_3");
        quick.push("lrs+11_5:4_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=30:sd=3:ss=axioms:st=3.0:sos=all:sac=on:aer=off:afr=on:afp=1000:afq=1.4:anc=all:sp=reverse_arity:urr=on:updr=off_63");
        quick.push("lrs+1011_3_bsr=unit_only:cond=on:fsr=off:lwlo=on:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:av=off_7");
        quick.push("ins+11_3_cond=fast:fde=none:igbrr=0.8:igrr=1/16:igrp=200:igrpq=1.05:igs=1003:igwr=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:av=off:updr=off_18");
        quick.push("dis+1010_1_fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:afr=on:amm=off:anc=all:sp=reverse_arity:updr=off_6");
        quick.push("lrs+11_3:2_cond=fast:fsr=off:fde=none:gs=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:av=off:urr=on_3");
        quick.push("ott+1010_40_bs=unit_only:cond=fast:nwc=1:sas=minisat:add=off:afp=10000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_46");
        quick.push("dis+1002_4_cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:nm=64:nwc=1:sas=minisat:sos=on:acc=model:aer=off:afr=on:afp=4000:afq=1.4:anc=none:updr=off_25");
        quick.push("dis+1010_50_gs=on:gsaa=full_model:gsem=on:nwc=4:nicw=on:sas=minisat:aac=none:acc=model:afp=100000:afq=2.0:amm=off:anc=none:sp=reverse_arity:updr=off_66");
        quick.push("dis+11_5_cond=on:fsr=off:fde=none:gsp=input_only:lcm=reverse:nm=0:nwc=4:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:updr=off_47");
        quick.push("lrs+11_2_br=off:cond=on:fde=none:gs=on:gsaa=full_model:lwlo=on:nwc=2:sas=minisat:stl=30:afp=100000:afq=1.4:amm=sco:anc=none:sp=occurrence:urr=on_87");
        quick.push("dis+11_2_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_76");
        quick.push("lrs+10_1_bd=off:fde=none:gsp=input_only:lcm=predicate:nm=0:nwc=1:stl=30:sos=on:av=off:updr=off_52");
        quick.push("dis-4_2_cond=on:fde=unused:lcm=reverse:nwc=1:sos=on:av=off:sp=reverse_arity:updr=off_58");
        quick.push("lrs+11_4:1_fsr=off:fde=unused:gs=on:gsem=off:nwc=1:sas=minisat:stl=30:av=off:sp=reverse_arity:urr=ec_only_73");
        quick.push("lrs+1002_7_ccuc=first:cond=on:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:stl=90:sd=3:ss=axioms:acc=model:aer=off:afp=1000:afq=2.0:anc=none:sp=reverse_arity_874");
        quick.push("ott+11_24_cond=fast:ep=RST:fsr=off:fde=none:gs=on:lcm=predicate:nm=64:nwc=1:sas=minisat:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence_1");
        quick.push("dis+1011_24_cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsem=off:lwlo=on:nm=64:nwc=1:sos=on:av=off:sp=occurrence_10");
        quick.push("dis+11_8_bs=unit_only:nwc=1:sd=10:ss=axioms:st=1.5:av=off:sp=occurrence:updr=off_83");
      }
      else {
        quick.push("dis+1002_4_cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:nm=64:nwc=1:sas=minisat:sos=on:acc=model:aer=off:afr=on:afp=4000:afq=1.4:anc=none:updr=off_3");
        quick.push("lrs+1011_2:1_br=off:cond=fast:fde=none:gs=on:gsssp=full:nwc=1:stl=30:sos=all:sac=on:add=off:afp=1000:afq=2.0:amm=sco:anc=none:urr=on_16");
        quick.push("ins+11_3_cond=fast:fde=none:igbrr=0.8:igrr=1/16:igrp=200:igrpq=1.05:igs=1003:igwr=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:av=off:updr=off_6");
        quick.push("lrs+10_2_bsr=unit_only:cond=fast:gsp=input_only:gs=on:gsssp=full:lcm=reverse:lwlo=on:nwc=1:sas=minisat:stl=30:sos=on:av=off:sp=reverse_arity_2");
        quick.push("dis+11_7_3");
        quick.push("dis+1010_5:1_cond=fast:ep=RSTC:er=known:fde=unused:nm=0:nwc=2:av=off_10");
        quick.push("dis+11_5_fde=none:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=all:add=large:aer=off:afr=on:afp=100000:afq=2.0:anc=none_9");
        quick.push("dis+1003_8:1_bd=off:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:lcm=reverse:nwc=1:sas=minisat:sos=all:aac=none:acc=on:add=off:afr=on:amm=sco:anc=none:sp=reverse_arity_7");
        quick.push("dis+11_8_bs=unit_only:nwc=1:sd=10:ss=axioms:st=1.5:av=off:sp=occurrence:updr=off_5");
        quick.push("dis+1002_3_cond=fast:er=known:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:afp=1000:afq=1.4:anc=all_dependent:sp=occurrence_1");
        quick.push("lrs+1002_2:3_bsr=unit_only:er=known:fde=none:gs=on:gsem=on:nwc=1:sas=minisat:stl=30:sd=1:ss=axioms:st=1.5:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_9");
        quick.push("lrs+1002_4_cond=on:er=filter:fde=unused:gsp=input_only:gs=on:gsssp=full:nwc=10:sas=minisat:stl=30:av=off:sp=occurrence_5");
        quick.push("dis+1010_2_bd=off:bsr=unit_only:ccuc=small_ones:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:sas=minisat:sos=on:sac=on:acc=on:afr=on:afp=4000:afq=1.2:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_12");
        quick.push("lrs+1002_7_ccuc=first:cond=on:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:stl=90:sd=3:ss=axioms:acc=model:aer=off:afp=1000:afq=2.0:anc=none:sp=reverse_arity_56");
        quick.push("ins+11_3_bsr=unit_only:fde=none:gs=on:gsem=off:igrr=1/16:igrpq=1.5:igs=1004:igwr=on:lcm=reverse:nm=0:nwc=1:sos=all:av=off:updr=off:dm=on_27");
        quick.push("dis+2_2:1_cond=on:er=filter:fde=none:gs=on:gsem=on:nwc=1:sac=on:add=large:afp=10000:afq=1.2:amm=off:sp=occurrence_24");
        quick.push("lrs+10_4_ccuc=first:cond=on:gs=on:gsssp=full:nwc=1:stl=90:sd=2:ss=axioms:st=1.5:sos=on:acc=on:aer=off:afp=40000:afq=1.2:anc=none:sp=reverse_arity:updr=off_34");
        quick.push("ott+1011_3_bd=off:fde=unused:nwc=1.5:av=off:sp=occurrence:updr=off_80");
        quick.push("ott+1002_5_bsr=on:fsr=off:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:acc=on:aer=off:afp=100000:afq=1.1:sp=occurrence_291");
        quick.push("lrs+1010_1_bs=unit_only:cond=fast:gs=on:gsem=off:nwc=1:stl=30:sos=all:av=off_143");
        quick.push("lrs+1002_3_bd=off:bs=on:bsr=unit_only:cond=fast:fsr=off:lcm=predicate:nwc=1.5:stl=30:sos=on:add=off:aer=off:afr=on:sp=reverse_arity_141");
        quick.push("lrs+4_3:1_bs=on:bsr=unit_only:ccuc=small_ones:cond=fast:er=filter:fsr=off:lcm=predicate:nm=1024:nwc=1:sas=minisat:stl=30:sos=on:sac=on:acc=on:afp=1000:afq=1.2:amm=sco:anc=all_dependent:sp=reverse_arity:updr=off_271");
        quick.push("lrs+1011_8_br=off:cond=fast:fsr=off:fde=none:nwc=1:sas=minisat:stl=30:sd=2:ss=axioms:sos=all:av=off:urr=on_148");
        quick.push("dis+1010_4:1_cond=fast:fsr=off:nm=0:nwc=1:sas=minisat:sac=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only_298");
        quick.push("dis+1010_3:1_cond=fast:fde=unused:gs=on:nwc=1:sd=2:ss=axioms:sos=on:add=large:aer=off:afr=on:afp=100000:afq=1.2:updr=off_18");
        quick.push("ott+11_8:1_cond=fast:gs=on:gsem=off:nwc=1:nicw=on:sd=2:ss=axioms:st=1.2:sos=all:acc=on:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:urr=on_19");
        quick.push("dis+1011_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sas=minisat:sd=3:ss=axioms:sos=on:aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=occurrence_27");
        quick.push("lrs+10_1_bd=off:fde=none:gsp=input_only:lcm=predicate:nm=0:nwc=1:stl=30:sos=on:av=off:updr=off_30");
      }
    }
    else if (prop == 131075) {
      quick.push("ott+2_2_cond=fast:fsr=off:gs=on:nwc=1:sd=1:ss=axioms:st=2.0:sos=on:sac=on:add=large:aer=off:afr=on:afp=4000:afq=1.4:anc=none:sp=reverse_arity:urr=on:updr=off_3");
      quick.push("lrs+1011_3_bsr=unit_only:cond=on:fsr=off:lwlo=on:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:av=off_20");
      quick.push("lrs+4_2:1_ep=R:fde=unused:gs=on:nwc=1:stl=30:sos=all:sac=on:aac=none:aer=off:afr=on:afp=10000:afq=1.2:anc=none:sp=occurrence:updr=off_2");
      quick.push("dis+1010_4:1_cond=fast:fsr=off:nm=0:nwc=1:sas=minisat:sac=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only_15");
      quick.push("lrs+4_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:nwc=1:stl=90:sd=5:ss=axioms:st=1.2:sac=on:add=off:aer=off:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_2");
      quick.push("lrs+10_2:3_bsr=on:fsr=off:fde=unused:nwc=1:sas=minisat:stl=30:sos=all:sac=on:add=off:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:urr=on:updr=off_3");
      quick.push("dis+1010_2_bd=off:bsr=unit_only:ccuc=small_ones:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:sas=minisat:sos=on:sac=on:acc=on:afr=on:afp=4000:afq=1.2:amm=sco:anc=none:sp=occurrence:urr=ec_only:updr=off_3");
      quick.push("ott+1011_3_bd=off:fde=unused:nwc=1.5:av=off:sp=occurrence:updr=off_43");
      quick.push("lrs+10_2_bsr=unit_only:cond=fast:gsp=input_only:gs=on:gsssp=full:lcm=reverse:lwlo=on:nwc=1:sas=minisat:stl=30:sos=on:av=off:sp=reverse_arity_37");
      quick.push("lrs+3_5:1_bd=off:bs=unit_only:bsr=unit_only:br=off:ccuc=small_ones:er=known:fsr=off:fde=unused:gs=on:nm=0:nwc=1.1:stl=30:sd=3:ss=axioms:sos=on:acc=model:add=off:aer=off:afp=4000:afq=1.4:sp=occurrence:urr=on:updr=off_1");
      quick.push("lrs-3_2:1_bsr=unit_only:fde=none:gs=on:gsssp=full:nm=0:nwc=1.5:sas=minisat:stl=30:sos=all:sac=on:amm=sco:anc=none:sp=occurrence_22");
      quick.push("dis+4_3:1_gs=on:nwc=1:sd=1:ss=axioms:sos=all:av=off:updr=off_51");
      quick.push("ott+11_8:1_cond=fast:gs=on:gsem=off:nwc=1:nicw=on:sd=2:ss=axioms:st=1.2:sos=all:acc=on:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:urr=on_43");
      quick.push("ott+1_3:1_cond=fast:gs=on:gsem=off:nwc=1:sas=minisat:sos=all:afp=10000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=on:updr=off_17");
      quick.push("dis-4_3:2_cond=fast:fsr=off:fde=none:gsp=input_only:lcm=predicate:nwc=1:sos=on:av=off:urr=ec_only_13");
      quick.push("dis+1002_6_ccuc=first:cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=64:nwc=2.5:nicw=on:sas=minisat:sos=on:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.2:amm=off:sp=occurrence:urr=ec_only:updr=off_222");
      quick.push("lrs+10_5:4_bsr=unit_only:fde=none:lcm=reverse:nm=64:nwc=10:stl=30:av=off:sp=occurrence:updr=off_24");
      quick.push("lrs-1_2:1_bd=preordered:bsr=on:br=off:cond=on:gs=on:lcm=reverse:nwc=1:stl=30:sd=2:ss=axioms:sos=on:add=large:aer=off:afp=100000:afq=2.0:anc=none:sp=occurrence:urr=on:updr=off_2");
      quick.push("lrs+11_2:3_bd=off:fsr=off:gsp=input_only:lcm=predicate:lwlo=on:nwc=1:stl=30:av=off:sp=occurrence:urr=ec_only_59");
      quick.push("lrs-1_4_cond=fast:ep=RST:fde=unused:gs=on:gsem=on:gsssp=full:lwlo=on:nwc=1:sas=minisat:stl=30:av=off:urr=ec_only_26");
      quick.push("lrs+10_2_bd=off:bsr=unit_only:ccuc=first:cond=fast:fsr=off:fde=none:gs=on:gsem=on:nwc=1.5:nicw=on:stl=240:sos=all:sac=on:aac=none:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:updr=off_22");
      quick.push("dis+11_4:1_br=off:ccuc=first:fsr=off:fde=none:nm=0:nwc=1:sos=all:acc=model:add=off:aer=off:afp=10000:afq=1.1:anc=all_dependent:sp=occurrence:urr=on:updr=off_29");
      quick.push("lrs+1002_6_ccuc=first:cond=on:fsr=off:nwc=4:nicw=on:sas=minisat:stl=30:acc=on:afr=on:afp=40000:afq=1.0:amm=sco:anc=all_dependent:sp=reverse_arity:urr=ec_only_246");
      quick.push("dis+1010_5:1_bs=unit_only:ccuc=small_ones:cond=on:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=3:sos=on:sac=on:acc=on:afp=40000:afq=1.2:amm=sco:anc=all_dependent:sp=occurrence:urr=ec_only_66");
      quick.push("lrs+1002_2:3_br=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=30:sac=on:aer=off:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:urr=on_79");
      quick.push("dis+1010_8_bd=off:bsr=on:ccuc=first:cond=fast:er=known:fsr=off:gs=on:gsssp=full:lcm=reverse:nm=0:nwc=1:sas=minisat:sd=2:ss=axioms:st=5.0:acc=on:add=off:afp=100000:afq=1.1:urr=ec_only:updr=off_276");
      quick.push("lrs+3_5_bd=preordered:bsr=unit_only:gsp=input_only:gs=on:gsem=on:gsssp=full:lwlo=on:nm=64:nwc=1:sas=minisat:stl=90:av=off:sp=occurrence:urr=ec_only:updr=off_646");
      quick.push("ins+11_5_fde=unused:igpr=on:igrr=1/16:igrp=200:igrpq=1.1:igs=1010:igwr=on:nwc=1:sos=all:av=off_767");
      quick.push("dis+1003_3_bs=unit_only:cond=fast:gs=on:gsaa=full_model:lwlo=on:nwc=1.5:sas=minisat:sd=1:ss=axioms:st=2.0:sac=on:add=large:afr=on:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_4");
      quick.push("dis+1002_5_cond=fast:ep=RST:fsr=off:fde=none:gsp=input_only:nwc=1:sd=2:ss=axioms:av=off:urr=ec_only:updr=off_8");
      quick.push("lrs+11_3:2_cond=fast:fsr=off:fde=none:gs=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:av=off:urr=on_28");
    }
    else if (prop == 131087) {
      if (atoms > 50000) {
        quick.push("ott+1011_10_cond=fast:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:sos=all:av=off:sp=occurrence:updr=off_10");
        quick.push("lrs-2_5_cond=on:fde=none:lcm=predicate:nwc=1:stl=30:sd=2:ss=axioms:st=3.0:sos=on:add=off:afp=100000:afq=1.4:anc=none_19");
        quick.push("lrs+1011_5_bd=off:br=off:ccuc=small_ones:fde=none:gs=on:gsem=off:nwc=1:stl=30:sd=1:ss=axioms:sos=on:acc=model:afp=100000:afq=1.4:amm=off:anc=none:sp=occurrence:urr=on_6");
        quick.push("dis+1010_1_fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:afr=on:amm=off:anc=all:sp=reverse_arity:updr=off_6");
        quick.push("dis-2_4_cond=fast:ep=RST:er=filter:fde=unused:gs=on:gsem=on:lcm=reverse:nwc=1:sd=3:ss=axioms:sos=on:av=off:updr=off_14");
        quick.push("ins+11_4_cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:igbrr=0.3:igpr=on:igrr=1/8:igrp=100:igrpq=1.5:igwr=on:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:av=off:dm=on_49");
        quick.push("lrs+10_4_ccuc=first:cond=on:gs=on:gsssp=full:nwc=1:stl=90:sd=2:ss=axioms:st=1.5:sos=on:acc=on:aer=off:afp=40000:afq=1.2:anc=none:sp=reverse_arity:updr=off_15");
        quick.push("lrs-1_2:1_bd=preordered:bsr=on:br=off:cond=on:gs=on:lcm=reverse:nwc=1:stl=30:sd=2:ss=axioms:sos=on:add=large:aer=off:afp=100000:afq=2.0:anc=none:sp=occurrence:urr=on:updr=off_65");
        quick.push("lrs+1002_3_ep=RST:er=known:fsr=off:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=30:sd=2:ss=axioms:st=5.0:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:sp=occurrence_75");
        quick.push("dis+1011_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sas=minisat:sd=3:ss=axioms:sos=on:aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=occurrence_17");
        quick.push("dis+4_3:1_gs=on:nwc=1:sd=1:ss=axioms:sos=all:av=off:updr=off_12");
        quick.push("lrs+1010_4:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=on:nm=0:nwc=1:sas=minisat:stl=30:sd=2:ss=axioms:st=1.5:sos=on:sac=on:add=off:aer=off:afr=on:afp=100000:afq=1.4:anc=none:sp=occurrence_74");
        quick.push("lrs+11_5:4_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=30:sd=3:ss=axioms:st=3.0:sos=all:sac=on:aer=off:afr=on:afp=1000:afq=1.4:anc=all:sp=reverse_arity:urr=on:updr=off_18");
        quick.push("dis+1002_4_cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:nm=64:nwc=1:sas=minisat:sos=on:acc=model:aer=off:afr=on:afp=4000:afq=1.4:anc=none:updr=off_46");
        quick.push("ott+1_2_cond=fast:er=filter:fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.2:sac=on:add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none_13");
        quick.push("dis-4_2_cond=on:fde=unused:lcm=reverse:nwc=1:sos=on:av=off:sp=reverse_arity:updr=off_40");
        quick.push("dis+1003_3_bs=unit_only:cond=fast:gs=on:gsaa=full_model:lwlo=on:nwc=1.5:sas=minisat:sd=1:ss=axioms:st=2.0:sac=on:add=large:afr=on:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_8");
        quick.push("dis+2_5:4_fde=none:nwc=1:sd=2:ss=axioms:sos=on:afp=40000:afq=2.0_131");
        quick.push("ott+11_8:1_cond=fast:gs=on:gsem=off:nwc=1:nicw=on:sd=2:ss=axioms:st=1.2:sos=all:acc=on:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:urr=on_95");
        quick.push("dis+11_2_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_241");
        quick.push("ins+11_3_cond=fast:fde=none:igbrr=0.8:igrr=1/16:igrp=200:igrpq=1.05:igs=1003:igwr=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:av=off:updr=off_285");
        quick.push("lrs+11_3:2_cond=fast:fsr=off:fde=none:gs=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:av=off:urr=on_234");
        quick.push("lrs+1011_8_br=off:cond=fast:fsr=off:fde=none:nwc=1:sas=minisat:stl=30:sd=2:ss=axioms:sos=all:av=off:urr=on_13");
        quick.push("lrs+10_1_bd=off:fde=none:gsp=input_only:lcm=predicate:nm=0:nwc=1:stl=30:sos=on:av=off:updr=off_75");
      }
      else {
        quick.push("lrs+1010_4_fde=unused:gs=on:gsem=off:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on:sac=on:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_12");
        quick.push("lrs+1011_2:1_br=off:cond=fast:fde=none:gs=on:gsssp=full:nwc=1:stl=30:sos=all:sac=on:add=off:afp=1000:afq=2.0:amm=sco:anc=none:urr=on_5");
        quick.push("lrs+10_4_ccuc=first:cond=on:gs=on:gsssp=full:nwc=1:stl=90:sd=2:ss=axioms:st=1.5:sos=on:acc=on:aer=off:afp=40000:afq=1.2:anc=none:sp=reverse_arity:updr=off_4");
        quick.push("lrs+1011_4_cond=fast:gs=on:gsaa=from_current:gsem=off:gsssp=full:nm=0:nwc=1:sas=minisat:stl=30:sd=3:ss=axioms:st=2.0:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:sp=reverse_arity_58");
        quick.push("ott+11_8:1_cond=fast:gs=on:gsem=off:nwc=1:nicw=on:sd=2:ss=axioms:st=1.2:sos=all:acc=on:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:urr=on_27");
        quick.push("dis-4_2_cond=on:fde=unused:lcm=reverse:nwc=1:sos=on:av=off:sp=reverse_arity:updr=off_4");
        quick.push("ott+11_5_nwc=1:sd=7:ss=axioms:st=2.0:sac=on:afr=on:afp=40000:afq=1.2:anc=none_19");
        quick.push("lrs+1011_5_bd=off:br=off:ccuc=small_ones:fde=none:gs=on:gsem=off:nwc=1:stl=30:sd=1:ss=axioms:sos=on:acc=model:afp=100000:afq=1.4:amm=off:anc=none:sp=occurrence:urr=on_4");
        quick.push("dis+1002_4_cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:nm=64:nwc=1:sas=minisat:sos=on:acc=model:aer=off:afr=on:afp=4000:afq=1.4:anc=none:updr=off_19");
        quick.push("ins+11_4_cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:igbrr=0.3:igpr=on:igrr=1/8:igrp=100:igrpq=1.5:igwr=on:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:av=off:dm=on_9");
        quick.push("lrs+11_5:4_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=30:sd=3:ss=axioms:st=3.0:sos=all:sac=on:aer=off:afr=on:afp=1000:afq=1.4:anc=all:sp=reverse_arity:urr=on:updr=off_122");
        quick.push("ott+1_2_cond=fast:er=filter:fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.2:sac=on:add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none_5");
        quick.push("dis+4_3:1_gs=on:nwc=1:sd=1:ss=axioms:sos=all:av=off:updr=off_6");
        quick.push("dis+1010_5:1_cond=fast:ep=RSTC:er=known:fde=unused:nm=0:nwc=2:av=off_18");
        quick.push("dis+1011_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sas=minisat:sd=3:ss=axioms:sos=on:aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=occurrence_57");
        quick.push("lrs-1_2:1_bd=preordered:bsr=on:br=off:cond=on:gs=on:lcm=reverse:nwc=1:stl=30:sd=2:ss=axioms:sos=on:add=large:aer=off:afp=100000:afq=2.0:anc=none:sp=occurrence:urr=on:updr=off_67");
        quick.push("dis+1010_4:1_cond=fast:fsr=off:nm=0:nwc=1:sas=minisat:sac=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only_17");
        quick.push("lrs+4_2:1_ep=R:fde=unused:gs=on:nwc=1:stl=30:sos=all:sac=on:aac=none:aer=off:afr=on:afp=10000:afq=1.2:anc=none:sp=occurrence:updr=off_294");
        quick.push("lrs+1003_7_cond=fast:nwc=1:stl=30:sd=4:ss=axioms:sos=all:av=off:updr=off_129");
        quick.push("ott+1011_3_bd=off:fde=unused:nwc=1.5:av=off:sp=occurrence:updr=off_66");
        quick.push("lrs+1011_8_br=off:cond=fast:fsr=off:fde=none:nwc=1:sas=minisat:stl=30:sd=2:ss=axioms:sos=all:av=off:urr=on_34");
        quick.push("lrs+1_3:1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=30:sd=2:ss=axioms:st=1.2:sos=on:sac=on:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=reverse_arity_41");
        quick.push("lrs+1002_2:3_bsr=unit_only:er=known:fde=none:gs=on:gsem=on:nwc=1:sas=minisat:stl=30:sd=1:ss=axioms:st=1.5:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_226");
        quick.push("dis+11_2_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_275");
        quick.push("ins+11_3_cond=fast:fde=none:igbrr=0.8:igrr=1/16:igrp=200:igrpq=1.05:igs=1003:igwr=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:av=off:updr=off_89");
        quick.push("dis+1011_2_fde=unused:gs=on:nwc=1:sac=on:afp=40000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=ec_only_116");
        quick.push("dis+1003_8:1_bd=off:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:lcm=reverse:nwc=1:sas=minisat:sos=all:aac=none:acc=on:add=off:afr=on:amm=sco:anc=none:sp=reverse_arity_280");
        quick.push("lrs+3_5:1_bd=off:bs=unit_only:bsr=unit_only:br=off:ccuc=small_ones:er=known:fsr=off:fde=unused:gs=on:nm=0:nwc=1.1:stl=30:sd=3:ss=axioms:sos=on:acc=model:add=off:aer=off:afp=4000:afq=1.4:sp=occurrence:urr=on:updr=off_148");
        quick.push("lrs+1002_3_ep=RST:er=known:fsr=off:gs=on:gsaa=from_current:nwc=1:sas=minisat:stl=30:sd=2:ss=axioms:st=5.0:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:sp=occurrence_231");
        quick.push("ott+11_3:1_br=off:gs=on:gsem=on:nwc=1:sas=minisat:sos=all:av=off:urr=on:updr=off_81");
      }
    }
    else if (prop & 67108864ul) {
      quick.push("dis+10_3_br=off:cond=fast:fde=none:gs=on:gsem=on:gsssp=full:inw=on:nwc=1:sas=minisat:sos=all:sac=on:add=large:afp=100000:afq=1.1:anc=none:sp=reverse_arity:urr=on_1");
      quick.push("dis+1010_5:1_fde=none:lwlo=on:nm=0:nwc=1:sas=minisat:sos=on:add=off:afr=on:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off_12");
      quick.push("lrs+11_1_br=off:fde=unused:gs=on:gsem=off:inw=on:nwc=1:sas=minisat:stl=30:sac=on:acc=model:aer=off:afp=100000:afq=1.2:anc=none:sp=reverse_arity:urr=on:updr=off_3");
      quick.push("lrs+11_2_bd=off:ccuc=first:cond=on:fde=unused:gs=on:gsem=off:nwc=1:stl=30:sos=all:acc=model:add=large:aer=off:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_3");
      quick.push("dis+10_5_bd=off:cond=fast:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:nwc=1:sas=minisat:sos=on:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:urr=ec_only:updr=off_2");
      quick.push("ott+11_6_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:gsssp=full:inw=on:nwc=1.5:sas=minisat:av=off:sp=occurrence:urr=on_27");
      quick.push("dis+1002_2_cond=on:gs=on:inw=on:nwc=1:sas=minisat:sos=on:sac=on:add=large:aer=off:afr=on:afp=4000:afq=1.2:anc=none:sp=reverse_arity_1");
      quick.push("lrs+1010_2:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:nm=0:nwc=2.5:stl=30:sac=on:acc=model:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_28");
      quick.push("ins+11_32_br=off:ep=RSTC:inw=on:igbrr=0.9:igrr=1/32:igrp=100:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:updr=off:dm=on_1");
      quick.push("dis+1003_3:2_bd=off:bsr=unit_only:nwc=1.3:sas=minisat:sac=on:add=large:aer=off:afr=on:afp=1000:afq=1.2:anc=none:updr=off_23");
      quick.push("ott+11_2:1_cond=on:gs=on:gsssp=full:lwlo=on:nwc=1:sas=minisat:sos=all:av=off:sp=occurrence:tha=off_12");
      quick.push("lrs+1011_8:1_gs=on:gsssp=full:inw=on:nwc=1:stl=30:add=off:afr=on:afp=4000:afq=1.4:amm=off:anc=none_26");
      quick.push("lrs+2_8:1_cond=fast:er=filter:fde=unused:lcm=predicate:nwc=2.5:sas=minisat:stl=60:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=occurrence:updr=off_9");
      quick.push("lrs+11_5:1_bd=off:gs=on:gsem=off:gsssp=full:nwc=1.3:sas=minisat:stl=30:sos=all:sac=on:aac=none:acc=model:add=large:afp=4000:afq=1.4:anc=none:sp=occurrence_203");
      quick.push("ott-11_4_br=off:cond=on:gs=on:gsem=off:gsssp=full:nwc=5:sas=minisat:sac=on:add=large:afr=on:afp=4000:afq=2.0:anc=all:sp=occurrence:urr=on:updr=off_39");
      quick.push("lrs+11_8:1_br=off:cond=on:fsr=off:fde=none:gs=on:inw=on:lwlo=on:nwc=1.5:stl=30:aer=off:afp=10000:afq=1.2:anc=none:sp=reverse_arity:urr=on_99");
      quick.push("ott+1010_3:1_cond=fast:fde=none:nwc=1:sos=all:av=off_170");
      quick.push("ott+1011_4:1_bd=off:bsr=unit_only:cond=fast:fsr=off:fde=none:inw=on:nm=0:nwc=1.1:sas=minisat:sos=on:afp=10000:afq=1.2:anc=none:sp=occurrence_172");
      quick.push("dis+10_3_cond=fast:fsr=off:gs=on:gsaa=from_current:nwc=1:sas=minisat:sac=on:afp=10000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_206");
      quick.push("lrs+1010_2:1_bd=preordered:bs=on:cond=fast:fde=none:gs=on:inw=on:lwlo=on:nwc=1:sas=minisat:stl=60:sos=all:av=off_216");
    }
    else {
      quick.push("lrs+11_5:1_bd=off:gs=on:gsem=off:gsssp=full:nwc=1.3:sas=minisat:stl=30:sos=all:sac=on:aac=none:acc=model:add=large:afp=4000:afq=1.4:anc=none:sp=occurrence_3");
      quick.push("lrs+1011_3:1_bsr=unit_only:cond=fast:er=known:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sas=minisat:stl=30:add=large:aer=off:afr=on:afp=100000:afq=2.0:updr=off_18");
      quick.push("dis+3_3:2_cond=on:fde=none:gs=on:gsem=on:nm=64:nwc=1:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.2:amm=off:anc=none:sp=occurrence_8");
      quick.push("dis+10_1024_cond=fast:fde=none:gs=on:gsem=off:nwc=1:sd=7:ss=axioms:st=5.0:sos=all:av=off:sp=reverse_arity_12");
      quick.push("lrs+4_2:1_ep=R:fde=unused:gs=on:nwc=1:stl=30:sos=all:sac=on:aac=none:aer=off:afr=on:afp=10000:afq=1.2:anc=none:sp=occurrence:updr=off_14");
      quick.push("dis+1002_6_ccuc=first:cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=64:nwc=2.5:nicw=on:sas=minisat:sos=on:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.2:amm=off:sp=occurrence:urr=ec_only:updr=off_3");
      quick.push("lrs+1002_4_cond=on:er=filter:fde=unused:gsp=input_only:gs=on:gsssp=full:nwc=10:sas=minisat:stl=30:av=off:sp=occurrence_6");
      quick.push("lrs+1011_4_cond=fast:gs=on:gsaa=from_current:gsem=off:gsssp=full:nm=0:nwc=1:sas=minisat:stl=30:sd=3:ss=axioms:st=2.0:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:sp=reverse_arity_8");
      quick.push("ins+10_1_av=off_1");
      quick.push("lrs+1_3:1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=30:sd=2:ss=axioms:st=1.2:sos=on:sac=on:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=reverse_arity_1");
      quick.push("lrs+11_5:4_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=30:sd=3:ss=axioms:st=3.0:sos=all:sac=on:aer=off:afr=on:afp=1000:afq=1.4:anc=all:sp=reverse_arity:urr=on:updr=off_3");
      quick.push("dis+1011_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sas=minisat:sd=3:ss=axioms:sos=on:aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=occurrence_2");
      quick.push("lrs+1010_1_bs=unit_only:cond=fast:gs=on:gsem=off:nwc=1:stl=30:sos=all:av=off_34");
      quick.push("lrs-3_2:1_bsr=unit_only:fde=none:gs=on:gsssp=full:nm=0:nwc=1.5:sas=minisat:stl=30:sos=all:sac=on:amm=sco:anc=none:sp=occurrence_12");
      quick.push("lrs+1_5_cond=fast:er=known:fde=unused:gs=on:gsem=on:gsssp=full:nwc=3:sas=minisat:stl=90:aer=off:afp=1000:afq=1.1:anc=none:sp=reverse_arity:updr=off_6");
      quick.push("lrs+11_8:1_br=off:cond=on:fsr=off:fde=none:gs=on:inw=on:lwlo=on:nwc=1.5:stl=30:aer=off:afp=10000:afq=1.2:anc=none:sp=reverse_arity:urr=on_54");
      quick.push("lrs+1002_2:3_br=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:stl=30:sac=on:aer=off:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:urr=on_6");
      quick.push("dis+1010_3:1_cond=fast:fde=unused:gs=on:nwc=1:sd=2:ss=axioms:sos=on:add=large:aer=off:afr=on:afp=100000:afq=1.2:updr=off_4");
      quick.push("dis+11_5_fsr=off:fde=none:gs=on:gsem=off:gsssp=full:inw=on:inst=on:nwc=1:aer=off:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off_1");
      quick.push("lrs+10_5:4_bsr=unit_only:fde=none:lcm=reverse:nm=64:nwc=10:stl=30:av=off:sp=occurrence:updr=off_5");
      quick.push("dis+11_7_fde=none:nm=0:nwc=1:sd=3:ss=axioms:st=2.0:av=off:sp=occurrence:urr=on:updr=off_41");
      quick.push("lrs+1002_2:3_bsr=unit_only:er=known:fde=none:gs=on:gsem=on:nwc=1:sas=minisat:stl=30:sd=1:ss=axioms:st=1.5:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_1");
      quick.push("dis+1010_4:1_cond=fast:fsr=off:nm=0:nwc=1:sas=minisat:sac=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only_11");
      quick.push("dis+11_1_cond=on:fsr=off:lcm=reverse:nwc=2.5:av=off:sp=reverse_arity:updr=off_4");
      quick.push("ins+11_32_br=off:ep=RSTC:inw=on:igbrr=0.9:igrr=1/32:igrp=100:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:updr=off:dm=on_163");
      quick.push("dis+11_3_cond=fast:fsr=off:gs=on:gsem=off:gsssp=full:inw=on:nwc=1.7:sas=minisat:add=off:aer=off:afp=1000:afq=1.2:anc=none:sp=occurrence:urr=on:updr=off_7");
      quick.push("lrs+1002_7_ccuc=first:cond=on:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:stl=90:sd=3:ss=axioms:acc=model:aer=off:afp=1000:afq=2.0:anc=none:sp=reverse_arity_41");
      quick.push("ott+2_2_cond=fast:fsr=off:gs=on:nwc=1:sd=1:ss=axioms:st=2.0:sos=on:sac=on:add=large:aer=off:afr=on:afp=4000:afq=1.4:anc=none:sp=reverse_arity:urr=on:updr=off_29");
      quick.push("dis+11_4_fde=unused:gs=on:gsaa=from_current:nwc=2.5:sac=on:add=large:aer=off:afp=100000:afq=1.4:anc=none_1");
      quick.push("ins+11_5_fde=unused:igpr=on:igrr=1/16:igrp=200:igrpq=1.1:igs=1010:igwr=on:nwc=1:sos=all:av=off_25");
      quick.push("dis+11_7_278");
      quick.push("ott+11_14_bd=preordered:fsr=off:gs=on:gsem=off:nwc=2:afp=4000:afq=1.2:amm=off:anc=none:sp=reverse_arity:updr=off_8");
      quick.push("dis+11_3_cond=on:ep=RS:er=filter:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sas=minisat:sd=1:ss=axioms:sac=on:afp=4000:afq=1.4:amm=off:anc=none:sp=occurrence:updr=off_1");
      quick.push("dis+11_5_bd=off:cond=fast:fde=unused:gs=on:gsem=on:lwlo=on:nwc=1:sos=on:sac=on:add=off:aer=off:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:urr=on_6");
      quick.push("lrs+10_1_bd=off:fde=none:gsp=input_only:lcm=predicate:nm=0:nwc=1:stl=30:sos=on:av=off:updr=off_3");
      quick.push("lrs+1010_2:1_bd=preordered:bs=on:cond=fast:fde=none:gs=on:inw=on:lwlo=on:nwc=1:sas=minisat:stl=60:sos=all:av=off_1");
      quick.push("dis+11_4_bsr=unit_only:cond=fast:fde=none:lwlo=on:nm=0:nwc=1.2:av=off:sp=occurrence_1");
      quick.push("dis-3_10_bsr=unit_only:er=filter:fde=unused:lcm=predicate:nm=64:nwc=1:av=off:sp=occurrence:updr=off_2");
      quick.push("dis+1011_2_fde=unused:gs=on:nwc=1:sac=on:afp=40000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=ec_only_16");
      quick.push("lrs+1010_8:1_fsr=off:fde=none:gs=on:gsem=on:gsssp=full:nwc=1.1:sas=minisat:stl=30:sos=all:aac=none:afr=on:afp=100000:afq=1.0:amm=off:anc=all:sp=occurrence:updr=off_137");
      quick.push("ott+11_4:1_bd=off:bsr=unit_only:cond=on:fsr=off:lcm=reverse:nwc=1:sas=minisat:sos=on:av=off:urr=on:updr=off_23");
      quick.push("dis+1011_4_fde=none:gs=on:nwc=1:sos=on:add=off:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:updr=off_115");
      quick.push("lrs+10_2_bsr=unit_only:cond=fast:gsp=input_only:gs=on:gsssp=full:lcm=reverse:lwlo=on:nwc=1:sas=minisat:stl=30:sos=on:av=off:sp=reverse_arity_10");
      quick.push("ott+10_5_cond=fast:fde=none:nwc=1:sas=minisat:sos=all:av=off:sp=occurrence:updr=off_84");
      quick.push("lrs+4_40_bsr=unit_only:cond=fast:fde=none:gs=on:gsem=on:lwlo=on:nwc=1.2:stl=60:sd=7:ss=axioms:st=5.0:aac=none:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=all_dependent:sp=reverse_arity:tha=off_263");
      quick.push("ins+11_3_bsr=unit_only:fde=none:gs=on:gsem=off:igrr=1/16:igrpq=1.5:igs=1004:igwr=on:lcm=reverse:nm=0:nwc=1:sos=all:av=off:updr=off:dm=on_126");
      quick.push("ott+1_3:1_cond=fast:gs=on:gsem=off:nwc=1:sas=minisat:sos=all:afp=10000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=on:updr=off_132");
      quick.push("ins+10_1_igbrr=0.6:igrpq=1.5:igs=1002:nwc=1:av=off:sp=reverse_arity:tha=off:dm=on_562");
      quick.push("dis+1002_3_cond=fast:er=known:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sas=minisat:afp=1000:afq=1.4:anc=all_dependent:sp=occurrence_1");
      quick.push("lrs-11_2_cond=on:fde=unused:gs=on:nwc=3:stl=30:add=off:afr=on:afp=100000:afq=1.4:amm=sco:anc=all_33");
      quick.push("ott+1002_5_bsr=on:fsr=off:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:acc=on:aer=off:afp=100000:afq=1.1:sp=occurrence_56");
      quick.push("dis+11_2_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_110");

    }
    break;

  case Property::FNE:
    if (atoms > 2000) {
      quick.push("dis+1011_40_bs=on:cond=on:gs=on:gsaa=from_current:nwc=1:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:updr=off_282");
      quick.push("lrs+1011_3_nwc=1:stl=90:sos=on:av=off:sp=reverse_arity_133");
      quick.push("dis-10_5_cond=fast:gsp=input_only:gs=on:gsem=off:nwc=1:sas=minisat:sos=all:av=off:sp=occurrence_190");
      quick.push("lrs+1011_5_cond=fast:gs=on:nwc=2.5:stl=30:sd=3:ss=axioms:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=occurrence_278");
      quick.push("lrs-3_5:4_bs=on:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:gsem=on:lcm=predicate:nwc=1.1:nicw=on:sas=minisat:stl=60:sd=3:ss=axioms:sac=on:aac=none:afr=on:afp=1000:afq=1.0:anc=all:sp=reverse_arity:urr=ec_only:updr=off_480");
    }
    else if (atoms > 1200) {
      quick.push("lrs+1011_5_cond=fast:gs=on:nwc=2.5:stl=30:sd=3:ss=axioms:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=occurrence_2");
      quick.push("dis+1011_8_bsr=unit_only:cond=fast:fsr=off:gs=on:gsaa=full_model:nm=0:nwc=1:sas=minisat:sos=all:afr=on:afp=4000:afq=1.1:amm=off:sp=reverse_arity_859");
      quick.push("dis+11_7_gs=on:gsaa=full_model:lcm=predicate:nwc=1.1:sas=minisat:aac=none:afp=1000:afq=1.0:amm=sco:sp=reverse_arity:urr=ec_only_878");
      quick.push("ins+11_5_br=off:gs=on:gsem=off:igbrr=0.9:igrr=1/64:igrp=1400:igrpq=1.1:igs=1003:igwr=on:lcm=reverse:nwc=1:av=off:urr=on:updr=off_1192");
    }
    else {
      quick.push("dis+11_7_16");
      quick.push("dis+1011_5:4_gs=on:gsssp=full:nwc=1.5:sas=minisat:aac=none:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=all:sp=reverse_arity:updr=off_2");
      quick.push("dis+1011_40_bs=on:cond=on:gs=on:gsaa=from_current:nwc=1:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:updr=off_14");
      quick.push("dis+11_3_cond=fast:fsr=off:gs=on:gsssp=full:nm=0:nwc=1:sas=minisat:sac=on:add=large:afp=10000:afq=1.2:amm=sco:anc=none:sp=occurrence:urr=ec_only_1");
      quick.push("ott-11_4_br=off:cond=on:gs=on:gsem=off:gsssp=full:nwc=5:sas=minisat:sac=on:add=large:afr=on:afp=4000:afq=2.0:anc=all:sp=occurrence:urr=on:updr=off_48");
      quick.push("ott+11_5_bs=on:bsr=on:gs=on:gsem=on:gsssp=full:nwc=1:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=all:sp=reverse_arity:urr=on:updr=off_19");
      quick.push("lrs+11_8:1_br=off:cond=on:fsr=off:fde=none:gs=on:inw=on:lwlo=on:nwc=1.5:stl=30:aer=off:afp=10000:afq=1.2:anc=none:sp=reverse_arity:urr=on_47");
      quick.push("dis+11_5_cond=fast:fsr=off:gs=on:gsem=on:gsssp=full:nwc=1:sac=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=occurrence:thf=on_1");
      quick.push("lrs+11_2_bd=off:ccuc=first:cond=on:fde=unused:gs=on:gsem=off:nwc=1:stl=30:sos=all:acc=model:add=large:aer=off:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_4");
      quick.push("lrs+11_5:1_bd=off:gs=on:gsem=off:gsssp=full:nwc=1.3:sas=minisat:stl=30:sos=all:sac=on:aac=none:acc=model:add=large:afp=4000:afq=1.4:anc=none:sp=occurrence_12");
      quick.push("dis-10_5_cond=fast:gsp=input_only:gs=on:gsem=off:nwc=1:sas=minisat:sos=all:av=off:sp=occurrence_7");
      quick.push("dis+11_1_cond=on:fsr=off:lcm=reverse:nwc=2.5:av=off:sp=reverse_arity:updr=off_74");
      quick.push("dis+1002_128_bs=on:cond=on:gs=on:gsem=on:nm=0:nwc=1:sos=all:av=off:updr=off_151");
      quick.push("ins+11_5_br=off:gs=on:gsem=off:igbrr=0.9:igrr=1/64:igrp=1400:igrpq=1.1:igs=1003:igwr=on:lcm=reverse:nwc=1:av=off:urr=on:updr=off_283");
      quick.push("ott+2_8_lcm=reverse:nm=0:nwc=2.5:aer=off:afp=4000:afq=1.1:anc=none:sp=occurrence_775");
      quick.push("dis+1010_28_nwc=1.3:sos=on:av=off:sp=reverse_arity:updr=off_456");
      quick.push("dis+10_4_cond=fast:fsr=off:nwc=1:sas=minisat:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=occurrence_829");
      quick.push("lrs-10_40_bsr=unit_only:br=off:cond=on:fsr=off:gs=on:gsaa=full_model:inw=on:lcm=reverse:lwlo=on:nwc=2.5:stl=30:sac=on:add=large:afp=4000:afq=1.0:amm=off:anc=none:sp=occurrence:urr=on:updr=off_19");
      quick.push("dis+1011_3_cond=on:gs=on:gsssp=full:nwc=1:sas=minisat:av=off:sp=occurrence_36");
      quick.push("dis+1011_5:1_cond=on:fsr=off:gs=on:gsssp=full:nwc=1:sas=minisat:sac=on:add=large:afr=on:afp=40000:afq=1.1:anc=none:updr=off_177");
    }
    break;

  case Property::EPR:
    if (prop == 131072) {
      quick.push("fmb+10_1_sas=minisat_17");
      quick.push("ins+1_1024_bd=preordered:br=off:cond=on:igbrr=0.9:igrr=1/16:igrp=2000:igrpq=2.0:igs=1010:igwr=on:nwc=1:av=off:sp=occurrence:urr=on:dm=on_154");
      quick.push("ott+2_5_cond=fast:er=filter:fde=none:nwc=1.5:nicw=on:sas=minisat:aac=none:add=large:afr=on:afp=100000:afq=2.0:amm=off:sp=reverse_arity:updr=off_215");
      quick.push("ott-11_3:2_bd=off:bs=unit_only:cond=fast:lcm=predicate:nwc=3:av=off:sp=occurrence_312");
      quick.push("ins+11_1024_bd=off:br=off:cond=fast:fsr=off:fde=none:igbrr=0.9:igpr=on:igrr=1/16:igrp=4000:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:dm=on_141");
      quick.push("dis+1003_3_cond=on:fsr=off:nwc=1.7:av=off:sp=occurrence:updr=off_506");
      quick.push("ins+10_1_fde=none:igbrr=0.7:igrp=4000:igrpq=1.3:igs=1:lcm=reverse:nwc=1.2:av=off:sp=reverse_arity:dm=on_488");
    }
    else if (prop == 131073) {
      quick.push("ott+2_5_cond=fast:er=filter:fde=none:nwc=1.5:nicw=on:sas=minisat:aac=none:add=large:afr=on:afp=100000:afq=2.0:amm=off:sp=reverse_arity:updr=off_225");
      quick.push("ins+11_1024_bd=off:br=off:cond=fast:fsr=off:fde=none:igbrr=0.9:igpr=on:igrr=1/16:igrp=4000:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:dm=on_1");
      quick.push("ott+3_5:1_ccuc=small_ones:fsr=off:lcm=predicate:nwc=1.1:sas=minisat:acc=on:add=off:aer=off:afp=1000:afq=1.2:anc=none:sp=reverse_arity:urr=ec_only:updr=off_200");
    }
    else if (atoms > 1300) {
      quick.push("dis-1_4_bd=preordered:cond=fast:fde=none:gs=on:gsssp=full:nwc=1:sas=minisat:sac=on:add=large:aer=off:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_46");
      quick.push("fmb+10_1_fmbsr=1.3:nm=0:sas=minisat_81");
      quick.push("dis+1011_128_bd=preordered:br=off:cond=on:nwc=1:sas=minisat:afr=on:afp=40000:afq=1.2:anc=none:urr=on:updr=off_18");
      quick.push("dis-11_8:1_bd=off:bs=unit_only:cond=fast:gs=on:gsem=off:nwc=1:aac=none:acc=on:add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=all_dependent_47");
      quick.push("ins+11_1024_bd=off:br=off:cond=fast:fsr=off:fde=none:igbrr=0.9:igpr=on:igrr=1/16:igrp=4000:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:dm=on_250");
      quick.push("ott-11_24_cond=fast:fde=none:gs=on:lcm=predicate:nwc=1:sas=minisat:av=off:sp=occurrence_19");
      quick.push("dis+10_3:1_bd=preordered:bsr=unit_only:fsr=off:fde=unused:gs=on:nwc=1:add=off:afp=100000:afq=1.2:amm=off:anc=none_272");
      quick.push("dis+1_28_bd=preordered:bs=unit_only:br=off:ccuc=small_ones:fsr=off:fde=none:gs=on:gsem=on:nwc=1:sas=minisat:sac=on:acc=model:aer=off:afr=on:afp=100000:afq=1.0:anc=all_dependent:sp=reverse_arity:urr=on_76");
      quick.push("dis+10_4:1_bd=off:ccuc=first:fde=none:gs=on:gsem=off:nwc=1:nicw=on:aac=none:acc=model:add=large:aer=off:afr=on:afp=4000:afq=2.0:urr=on:updr=off_97");
      quick.push("dis-4_8_bd=off:fde=unused:gs=on:nwc=1.2:sac=on:add=off:afr=on:afp=100000:afq=2.0:amm=sco:anc=all:urr=ec_only:updr=off_197");
      quick.push("ins+10_40_bd=off:br=off:fde=none:gsp=input_only:igbrr=1.0:igpr=on:igrr=1/32:igrp=1400:igrpq=1.05:igs=1:igwr=on:lcm=reverse:nwc=2.5:av=off:sp=occurrence:urr=on_331");
      quick.push("dis+2_1_bs=on:bsr=unit_only:ccuc=small_ones:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sas=minisat:sac=on:acc=model:add=large:anc=none:urr=ec_only_352");
      quick.push("ott+11_5_bd=preordered:bsr=unit_only:cond=fast:fde=none:nwc=1:sas=minisat:afp=10000:afq=1.2:amm=sco:anc=all_dependent:sp=occurrence:updr=off_160");
    }
    else if (atoms > 430) {
      quick.push("dis+11_7_63");
      quick.push("ott+10_3_bd=preordered:bs=on:bsr=unit_only:cond=fast:fde=none:gs=on:lcm=predicate:nwc=2:sas=minisat:av=off:sp=reverse_arity:urr=on:updr=off_129");
      quick.push("ins+11_14_br=off:cond=on:fsr=off:igbrr=0.9:igrr=1/128:igrp=100:igrpq=1.05:igwr=on:lcm=predicate:nwc=1.7:av=off:urr=on_536");
      quick.push("ins+10_1_igbrr=0.4:igrp=400:igrpq=2.0:igs=1003:nwc=1.5:av=off:updr=off:dm=on_854");
      quick.push("ins+10_1_fde=unused:gs=on:igbrr=1.0:igrp=200:igrpq=2.0:igs=1002:lcm=predicate:nwc=3:av=off:sp=reverse_arity:dm=on_632");
      quick.push("ott+2_5_cond=fast:er=filter:fde=none:nwc=1.5:nicw=on:sas=minisat:aac=none:add=large:afr=on:afp=100000:afq=2.0:amm=off:sp=reverse_arity:updr=off_34");
    }
    else {
      quick.push("ott+11_4_bd=preordered:cond=on:fsr=off:nwc=1:sas=minisat:anc=none:sp=occurrence:updr=off_1323");
      quick.push("ins+11_50_bd=preordered:br=off:fsr=off:fde=none:gs=on:gsem=off:igbrr=0.5:igpr=on:igrr=1/128:igrp=200:igs=1:igwr=on:nwc=1:av=off:urr=on:dm=on_444");
      quick.push("ins+10_1_igbrr=0.4:igrp=400:igrpq=2.0:igs=1003:nwc=1.5:av=off:updr=off:dm=on_549");
    }
    break;

  case Property::UEQ:
    if (prop == 2) {
      quick.push("ott+10_40_fde=none:ins=3:nwc=1:av=off:sp=reverse_arity_147");
      quick.push("lrs+10_4:1_bd=preordered:ins=3:nwc=1:stl=60:av=off_61");
      quick.push("ott+10_14_ins=3:nwc=2:av=off:sp=occurrence_475");
      quick.push("lrs+10_24_ins=3:nwc=1:stl=120:av=off:sp=reverse_arity_773");
      quick.push("ott+10_2_bd=preordered:fde=none:ins=3:nwc=2:av=off:sp=reverse_arity_219");
      quick.push("ott+10_3:1_fde=none:ins=3:nwc=3:av=off_243");
      quick.push("ott+10_12_bd=off:ins=3:nwc=1:av=off:sp=reverse_arity_421");
      quick.push("lrs+10_5:4_ins=3:lwlo=on:nwc=1:stl=90:av=off:sp=occurrence_678");
      quick.push("dis+10_28_fde=none:ins=3:nwc=4:av=off_259");
    }
    else if (prop != 0) {
      quick.push("dis+10_128_fde=unused:ins=3:nwc=2.5:av=off:sp=occurrence_15");
      quick.push("lrs+10_8:1_bd=off:fde=unused:ins=3:nwc=2.5:stl=120:av=off_146");
      quick.push("lrs+10_10_bd=preordered:fde=unused:ins=3:lwlo=on:nwc=5:stl=60:av=off:sp=occurrence_128");
      quick.push("ott+10_128_bd=off:ins=3:nwc=1:av=off:sp=reverse_arity_42");
      quick.push("ott+10_3_bd=off:ins=3:nwc=1:av=off:sp=reverse_arity_310");
    }
    else if (atoms > 10) {
      quick.push("ott+10_2_bd=preordered:fde=none:ins=3:nwc=2:av=off:sp=reverse_arity_521");
      quick.push("ott+10_64_bd=off:fde=none:ins=3:nwc=1:av=off:sp=reverse_arity_889");
      quick.push("lrs+10_4:1_bd=preordered:ins=3:nwc=1:stl=60:av=off_238");
      quick.push("ott+10_40_fde=none:ins=3:nwc=1:av=off:sp=reverse_arity_554");
      quick.push("ott+10_8_bd=preordered:ins=3:nwc=1.5:av=off:sp=occurrence_520");
    }
    else {
      quick.push("lrs+10_5:4_ins=3:lwlo=on:nwc=1:stl=90:av=off:sp=occurrence_162");
      quick.push("lrs+10_64_fde=none:ins=3:lwlo=on:nwc=1:stl=120:av=off:sp=occurrence_657");
      quick.push("ott+10_3:1_bd=preordered:ins=3:nwc=5:av=off_159");
      quick.push("ott+10_5:4_fde=none:ins=3:nwc=1.7:av=off:sp=occurrence_188");
      quick.push("lrs+10_5:1_fde=unused:ins=3:nwc=1.3:stl=120:av=off:sp=occurrence_462");
      quick.push("lrs+10_24_ins=3:nwc=1:stl=120:av=off:sp=reverse_arity_960");
    }
    break;
  }

  switch (cat) {
  case Property::HEQ:
  case Property::PEQ:
  case Property::NEQ:
  case Property::HNE:
  case Property::NNE:
    fallback.push("dis+11_7_300");
    fallback.push("lrs-11_3:1_bd=off:ccuc=small_ones:cond=fast:gs=on:gsaa=from_current:nwc=1:sas=minisat:sos=all:acc=on:add=large:aer=off:afp=40000:afq=1.0:anc=none:sp=reverse_arity:updr=off_300");
    fallback.push("dis+11_4_fde=unused:gs=on:gsem=on:gsssp=full:igrpq=1.0:lcm=reverse:lwlo=on:nwc=4:sas=minisat:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_300");
    fallback.push("dis+2_5_cond=fast:igrpq=1.0:nwc=1:sos=all:av=off:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_1_bsr=unit_only:cond=on:fsr=off:gs=on:gsem=on:lwlo=on:nwc=2:sas=minisat:av=off_900");
    fallback.push("lrs+2_4:1_fsr=off:fde=none:gsp=input_only:lcm=predicate:lwlo=on:nwc=1.2:av=off:sp=occurrence:urr=ec_only:updr=off_900");
    fallback.push("ins+11_4_cond=fast:fsr=off:igbrr=0.8:igpr=on:igrr=1/8:igrp=200:igrpq=1.5:igs=1003:igwr=on:nwc=10:av=off:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_5:4_bd=off:cond=on:fde=unused:nwc=3:av=off:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_3:1_bs=on:bsr=unit_only:gs=on:gsem=off:gsssp=full:lwlo=on:nwc=1:sas=minisat:av=off:sp=reverse_arity:updr=off_2400");
    fallback.push("lrs+1010_10_bd=off:fsr=off:fde=none:nwc=4:sas=minisat:aac=none:add=off:aer=off:afp=4000:afq=1.4:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_3:1_bd=off:fde=none:gs=on:lwlo=on:nwc=2:sas=minisat:sac=on:add=off:afr=on:afp=4000:afq=1.4:amm=sco:anc=none:sp=occurrence_900");
    fallback.push("dis+11_5_gs=on:gsem=on:nwc=1:sd=1:ss=axioms:st=3.0:sac=on:add=large:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence_300");
    fallback.push("dis+11_2:3_cond=on:er=known:gs=on:igrpq=1.0:nwc=1.5:add=off:afr=on:afp=4000:afq=2.0:anc=none_300");
    fallback.push("lrs+1002_3:1_bs=unit_only:cond=on:gsp=input_only:igrpq=1.0:nwc=10:sas=minisat:sac=on:aac=none:afr=on:afp=4000:afq=1.0:amm=off:anc=none:sp=occurrence_300");
    fallback.push("lrs+11_2:1_bs=unit_only:bsr=unit_only:fsr=off:fde=none:gsp=input_only:lcm=reverse:lwlo=on:nwc=1:sos=on:av=off:urr=ec_only_600");
    fallback.push("lrs+1011_10_cond=on:gsp=input_only:igrpq=1.0:nwc=1.5:av=off:sp=occurrence:updr=off_300");
    fallback.push("dis+11_5_cond=on:gs=on:gsem=on:igrpq=1.0:nwc=1:sos=on:sac=on:add=large:afp=4000:afq=1.1:amm=sco:anc=none:sp=occurrence:updr=off_300");
    fallback.push("dis+11_3:2_bs=unit_only:cond=on:fde=unused:lcm=reverse:nwc=1:sos=all:av=off_300");
    fallback.push("ins+11_3_cond=on:igbrr=0.5:igrr=1/16:igrp=4000:igrpq=1.1:igs=1:igwr=on:nwc=4:av=off:sp=reverse_arity:dm=on_300");
    fallback.push("dis+1003_50_cond=fast:igrpq=1.0:nwc=1:sos=on:av=off:sp=occurrence_300");
    fallback.push("dis+1002_3_bsr=unit_only:cond=on:nwc=1.2:nicw=on:sos=all:add=large:aer=off:sp=occurrence:updr=off_300");
    fallback.push("ins+4_3_bsr=unit_only:fde=unused:igrr=1/2:igrp=2000:igrpq=2.0:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:dm=on_300");
    fallback.push("dis+11_64_fsr=off:gsp=input_only:nwc=1.1:sos=all:av=off:sp=occurrence:updr=off_300");
    fallback.push("ott+1011_2_er=known:fde=none:nwc=1:av=off:sp=occurrence_300");
    fallback.push("dis+1011_2_cond=on:ep=RST:gs=on:gsem=on:igrpq=1.0:nwc=1:sac=on:afp=100000:afq=1.1:amm=off:anc=none:urr=ec_only_300");
    fallback.push("ott+1004_5:1_bd=off:bsr=unit_only:cond=fast:fsr=off:nwc=1:sos=all:av=off_300");
    fallback.push("lrs+11_14_bs=unit_only:bsr=unit_only:cond=on:igrpq=1.0:nwc=1:sas=minisat:add=off:aer=off:afp=1000:afq=1.1:anc=none:sp=occurrence_300");
    fallback.push("dis+1_64_gsp=input_only:nwc=1.2:sos=all:av=off:sp=occurrence:updr=off_300");
    fallback.push("ott+11_24_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=3:add=off:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence_300");
    fallback.push("ins+10_4_cond=on:fsr=off:fde=none:igbrr=0.6:igrr=1/32:igrp=2000:igrpq=1.05:igs=1002:igwr=on:nwc=5:av=off:sp=occurrence:updr=off:dm=on_300");
    fallback.push("lrs+11_2:3_cond=on:gs=on:gsem=on:igrpq=1.0:lwlo=on:nwc=1.7:sas=minisat:av=off:updr=off_300");
    fallback.push("dis+1011_2_nwc=1:sas=minisat:sos=on:av=off:sp=occurrence_300");
    fallback.push("ott+1003_3:1_bsr=unit_only:fsr=off:fde=unused:gs=on:gsem=on:igrpq=1.0:nwc=10:sac=on:add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:urr=on_300");
    fallback.push("lrs+10_10_er=known:fde=none:gs=on:gsem=on:igrpq=1.0:nwc=1.7:av=off:updr=off_300");
    fallback.push("dis+1010_4_bs=unit_only:cond=fast:fsr=off:fde=unused:nwc=1:sos=on:add=off:afr=on:afp=10000:afq=2.0:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+2_7_bs=unit_only:bsr=unit_only:cond=on:fsr=off:gs=on:nwc=1.7:sas=minisat:sos=on:sac=on:afr=on:afp=100000:afq=1.0:amm=off:anc=all_dependent_300");
    fallback.push("lrs+1011_128_bsr=unit_only:cond=fast:lwlo=on:nwc=2:sos=all:av=off:urr=on:updr=off_300");
    fallback.push("lrs+1_20_bs=on:cond=fast:gs=on:gsem=on:nwc=1.1:add=off:aer=off:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:updr=off_600");
    fallback.push("ins+11_5_ep=RST:fsr=off:fde=none:gs=on:gsem=on:igbrr=0.8:igpr=on:igrr=1/32:igrp=200:igrpq=1.5:igs=1010:igwr=on:nwc=1:sas=minisat:sos=on:av=off:dm=on_300");
    fallback.push("dis+11_5:4_cond=fast:fsr=off:igrpq=1.0:nwc=10:av=off_300");
    fallback.push("ins+11_5_cond=fast:gsp=input_only:igbrr=0.7:igrr=1/4:igs=1003:igwr=on:lcm=reverse:nwc=1:sos=all:av=off:urr=ec_only_300");
    fallback.push("lrs+11_2_bd=off:fsr=off:gs=on:gsaa=full_model:gsem=off:gsssp=full:igrpq=1.0:lcm=reverse:nwc=1:sas=minisat:sos=on:add=large:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:updr=off_300");
    fallback.push("lrs+1011_8_bd=preordered:cond=on:fsr=off:fde=none:gs=on:gsssp=full:igrpq=1.0:lcm=reverse:nwc=1.7:sas=minisat:av=off:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+10_4_bsr=unit_only:gs=on:gsssp=full:nwc=1.5:nicw=on:sas=minisat:afr=on:afp=4000:afq=1.2:amm=off:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_5:4_bsr=unit_only:ccuc=small_ones:cond=on:fsr=off:igrpq=1.0:nwc=1:sas=minisat:sac=on:acc=on:add=off:afp=40000:afq=2.0:anc=none:sp=reverse_arity_300");
    fallback.push("dis+11_4_cond=fast:ep=R:gs=on:gsaa=from_current:gsem=on:igrpq=1.0:nwc=1:sas=minisat:afp=1000:afq=1.4:amm=sco:anc=none:sp=occurrence:updr=off_300");
    fallback.push("dis-2_5_cond=on:igrpq=1.0:nwc=1:sas=minisat:av=off:sp=occurrence_300");
    fallback.push("dis+11_5:4_fsr=off:fde=none:gs=on:gsem=off:nwc=1:sac=on:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:urr=on_300");
    fallback.push("dis+11_8:1_br=off:cond=fast:fsr=off:igrpq=1.0:nwc=1:sos=all:add=off:aer=off:afr=on:afp=40000:afq=1.1:anc=none:sp=occurrence:urr=on:updr=off_300");
    fallback.push("dis+1010_24_cond=fast:ep=RS:fde=unused:lwlo=on:nwc=1.5:sas=minisat:aer=off:afr=on:afp=1000:afq=1.1:anc=none_300");
    fallback.push("lrs+1011_3_cond=on:fsr=off:fde=none:gs=on:gsssp=full:igrpq=1.0:nwc=1:sos=all:av=off:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_2_bs=unit_only:cond=fast:er=filter:fsr=off:fde=unused:nwc=2.5:aac=none:afp=4000:afq=1.0:amm=off:sp=occurrence:updr=off_300");
    fallback.push("dis+3_64_cond=fast:igrpq=1.0:lcm=reverse:nwc=1.1:sos=on:av=off:updr=off_300");
    fallback.push("lrs+11_1024_bd=off:fsr=off:gs=on:gsem=on:nwc=1:av=off:sp=occurrence:urr=on_300");
    fallback.push("ott+10_8_bsr=unit_only:gs=on:gsaa=from_current:gsem=on:nwc=4:sas=minisat:afr=on:afp=4000:afq=1.4:amm=off:anc=all_600");
    fallback.push("lrs+1004_28_nwc=1.1:sos=all:av=off_600");
    fallback.push("lrs+11_10_gs=on:gsem=on:igrpq=1.0:lcm=reverse:nwc=1:sac=on:afr=on:afp=10000:afq=1.0:anc=none:updr=off_300");
    fallback.push("dis+1010_6_bd=off:bsr=unit_only:ccuc=first:cond=fast:fsr=off:fde=unused:igrpq=1.0:lwlo=on:nwc=1:sas=minisat:sos=on:sac=on:acc=model:add=large:aer=off:afp=1000:afq=1.1:anc=all_dependent_300");
    fallback.push("dis+10_2:3_fde=unused:igrpq=1.0:lcm=predicate:nwc=1:sas=minisat:sos=all:sac=on:add=off:afr=on:afp=100000:afq=1.0:amm=off:anc=none:sp=reverse_arity_300");
    fallback.push("dis+11_2_cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:igrpq=1.0:nwc=5:sac=on:aac=none:afr=on:afp=4000:afq=1.4:amm=off:anc=none:sp=reverse_arity_300");
    fallback.push("dis+11_4_fsr=off:fde=none:gs=on:gsaa=from_current:igrpq=1.0:nwc=1:afr=on:afp=1000:afq=2.0:anc=none:urr=on:updr=off_300");
    fallback.push("dis+1002_10_bsr=unit_only:cond=fast:nwc=1:sos=all:av=off:sp=occurrence_300");
    fallback.push("lrs+11_5:4_bd=off:gsp=input_only:gs=on:gsem=on:lcm=predicate:nwc=1:sas=minisat:sos=all:av=off:sp=occurrence:urr=on_300");
    fallback.push("lrs+11_1_bs=on:nwc=1.1:av=off:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("ins+4_4_fsr=off:fde=none:igbrr=0.8:igpr=on:igrr=1/8:igrpq=1.0:igs=1002:igwr=on:nwc=1:sos=all:av=off:sp=reverse_arity:urr=ec_only:dm=on_300");
    fallback.push("ott+11_5:4_cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:nwc=1:sos=all:add=large:afr=on:afp=10000:afq=2.0:anc=none:sp=occurrence:updr=off_300");
    fallback.push("dis+1011_2:1_cond=fast:gsp=input_only:gs=on:nwc=1:sas=minisat:sos=all:av=off_300");
    fallback.push("lrs+11_3_fsr=off:gs=on:gsssp=full:nwc=2:av=off:sp=occurrence:urr=on:updr=off_600");
    fallback.push("lrs-1_3_fsr=off:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1.2:sas=minisat:sac=on:add=off:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_300");
    fallback.push("dis+1004_3_fsr=off:fde=none:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.5:sos=all:av=off:sp=reverse_arity_300");
    fallback.push("dis+1_20_bs=unit_only:fsr=off:gs=on:gsem=on:gsssp=full:nwc=1.7:sas=minisat:av=off:sp=occurrence_300");
    fallback.push("dis+10_3_gs=on:gsem=on:gsssp=full:igrpq=1.0:nwc=1:sas=minisat:sd=3:ss=axioms:sos=all:add=off:afr=on:afp=4000:afq=1.0:amm=sco:anc=none:updr=off_300");
    fallback.push("lrs+11_4_bs=unit_only:cond=fast:fde=none:gs=on:igrpq=1.0:lwlo=on:nwc=1:afp=1000:afq=1.2:anc=none:sp=occurrence_300");
    fallback.push("dis+10_24_gs=on:igrpq=1.0:nwc=1:sas=minisat:sd=3:ss=axioms:sos=on:av=off:sp=reverse_arity:updr=off_300");
    fallback.push("dis+2_12_fsr=off:lcm=reverse:nwc=3:av=off:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1002_3:1_bd=off:bsr=unit_only:fde=none:gs=on:gsem=on:nwc=1:sd=1:ss=axioms:av=off:sp=occurrence_300");
    fallback.push("lrs+11_5_cond=fast:er=filter:igrpq=1.0:nwc=1:sos=all:av=off:urr=ec_only_300");
    fallback.push("dis+11_64_bs=unit_only:cond=on:fde=none:igrpq=1.0:nwc=2:av=off:updr=off_300");
    fallback.push("lrs+10_5_bd=preordered:cond=on:fde=none:igrpq=1.0:lcm=reverse:lwlo=on:nwc=1.7:av=off:sp=occurrence_300");
    fallback.push("dis+11_5_fsr=off:gs=on:gsem=off:igrpq=1.0:lwlo=on:nwc=1:sos=all:av=off:sp=reverse_arity_300");
    fallback.push("lrs+10_4_cond=fast:nwc=1:nicw=on:sas=minisat:afr=on:afp=10000:afq=1.2:amm=off_600");
    fallback.push("lrs+1003_5_bd=off:bsr=on:cond=on:fsr=off:fde=none:gsp=input_only:lwlo=on:nwc=1:sos=all:av=off:sp=reverse_arity_300");
    fallback.push("lrs+2_50_bs=unit_only:bsr=unit_only:cond=fast:fsr=off:igrpq=1.0:nwc=1:av=off:sp=occurrence_300");
    fallback.push("dis+1_3_cond=on:fsr=off:nwc=1.1:sas=minisat:av=off:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+1002_3_fsr=off:gs=on:gsaa=from_current:gsem=on:igrpq=1.0:nwc=1:sos=on:sac=on:afr=on:afp=1000:afq=1.2:amm=off:anc=none:updr=off_300");
    fallback.push("lrs+11_128_bs=unit_only:fde=unused:gs=on:gsem=off:gsssp=full:igrpq=1.0:nwc=1:nicw=on:sas=minisat:sos=on:sac=on:aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=sco:anc=all_1200");
    fallback.push("dis+1_64_bs=unit_only:cond=fast:fde=none:gs=on:gsaa=from_current:gsem=off:igrpq=1.0:nwc=3:nicw=on:sas=minisat:sos=on:sac=on:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=all_dependent:sp=reverse_arity:updr=off_600");
    break;

  case Property::EPR:
    fallback.push("ott+11_5_bd=preordered:bsr=unit_only:cond=fast:fde=none:igrpq=1.0:nwc=1:sas=minisat:afp=10000:afq=1.2:amm=sco:anc=all_dependent:sp=occurrence:updr=off_300");
    fallback.push("ins+1_1024_bd=preordered:br=off:cond=on:igbrr=0.9:igrr=1/16:igrp=2000:igrpq=2.0:igs=1010:igwr=on:nwc=1:av=off:sp=occurrence:urr=on:dm=on_300");
    fallback.push("fmb+10_1_sas=minisat_3000");
    fallback.push("dis+10_2:1_bd=preordered:fsr=off:gs=on:gsem=off:igrpq=1.0:nwc=1.1:sas=minisat:aac=none:add=large:afr=on:afp=40000:afq=2.0:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("ott+2_5_cond=fast:er=filter:fde=none:igrpq=1.0:nwc=1.5:nicw=on:sas=minisat:aac=none:add=large:afr=on:afp=100000:afq=2.0:amm=off:sp=reverse_arity:updr=off_300");
    fallback.push("ott+11_4_bd=preordered:cond=on:fsr=off:igrpq=1.0:nwc=1:sas=minisat:anc=none:sp=occurrence:updr=off_1500");
    fallback.push("ott+10_3_bd=preordered:bs=on:bsr=unit_only:cond=fast:fde=none:gs=on:igrpq=1.0:lcm=predicate:nwc=2:sas=minisat:av=off:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+1003_3_cond=on:fsr=off:igrpq=1.0:nwc=1.7:av=off:sp=occurrence:updr=off_600");
    fallback.push("ins+10_1_igbrr=0.4:igrp=400:igrpq=2.0:igs=1003:nwc=1.5:av=off:updr=off:dm=on_1200");
    fallback.push("ins+10_40_bd=off:br=off:fde=none:gsp=input_only:igbrr=1.0:igpr=on:igrr=1/32:igrp=1400:igrpq=1.05:igs=1:igwr=on:lcm=reverse:nwc=2.5:av=off:sp=occurrence:urr=on_600");
    fallback.push("ott-11_24_cond=fast:fde=none:gs=on:igrpq=1.0:lcm=predicate:nwc=1:sas=minisat:av=off:sp=occurrence_300");
    fallback.push("dis-11_8:1_bd=off:bs=unit_only:cond=fast:gs=on:gsem=off:igrpq=1.0:nwc=1:aac=none:acc=on:add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=all_dependent_300");
    fallback.push("dis-1_4_bd=preordered:cond=fast:fde=none:gs=on:gsssp=full:igrpq=1.0:nwc=1:sas=minisat:sac=on:add=large:aer=off:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_300");
    fallback.push("dis-4_8_bd=off:fde=unused:gs=on:igrpq=1.0:nwc=1.2:sac=on:add=off:afr=on:afp=100000:afq=2.0:amm=sco:anc=all:urr=ec_only:updr=off_300");
    fallback.push("ins+11_1024_bd=preordered:br=off:cond=fast:gs=on:igbrr=1.0:igpr=on:igrr=1/64:igrp=700:igrpq=2.0:igs=1:igwr=on:lcm=predicate:nwc=1.7:av=off:sp=occurrence:urr=on:updr=off_300");
    fallback.push("dis+1011_128_bd=preordered:br=off:cond=on:igrpq=1.0:nwc=1:sas=minisat:afr=on:afp=40000:afq=1.2:anc=none:urr=on:updr=off_300");
    fallback.push("dis+10_4:1_bd=off:ccuc=first:fde=none:gs=on:gsem=off:nwc=1:nicw=on:aac=none:acc=model:add=large:aer=off:afr=on:afp=4000:afq=2.0:urr=on:updr=off_300");
    fallback.push("dis+10_3:1_bd=preordered:bsr=unit_only:fsr=off:fde=unused:gs=on:igrpq=1.0:nwc=1:add=off:afp=100000:afq=1.2:amm=off:anc=none_300");
    fallback.push("ins+11_1024_bd=off:br=off:cond=fast:fsr=off:fde=none:igbrr=0.9:igpr=on:igrr=1/16:igrp=4000:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:dm=on_300");
    fallback.push("ins+11_14_br=off:cond=on:fsr=off:igbrr=0.9:igrr=1/128:igrp=100:igrpq=1.05:igwr=on:lcm=predicate:nwc=1.7:av=off:urr=on_600");
    fallback.push("ins+11_50_bd=preordered:br=off:fsr=off:fde=none:gs=on:gsem=off:igbrr=0.5:igpr=on:igrr=1/128:igrp=200:igrpq=1.0:igs=1:igwr=on:nwc=1:av=off:urr=on:dm=on_600");
    fallback.push("dis+2_1_bs=on:bsr=unit_only:ccuc=small_ones:fsr=off:gs=on:gsem=off:gsssp=full:igrpq=1.0:nwc=1:sas=minisat:sac=on:acc=model:add=large:anc=none:urr=ec_only_600");
    fallback.push("ott-11_3:2_bd=off:bs=unit_only:cond=fast:igrpq=1.0:lcm=predicate:nwc=3:av=off:sp=occurrence_600");
    fallback.push("ins+10_1_fde=unused:gs=on:igbrr=1.0:igrp=200:igrpq=2.0:igs=1002:lcm=predicate:nwc=3:av=off:sp=reverse_arity:dm=on_900");
    break;

  case Property::UEQ:
    fallback.push("lrs+10_5:4_ins=3:igrpq=1.0:lwlo=on:nwc=1:av=off:sp=occurrence_900");
    fallback.push("lrs+10_14_fde=none:ins=3:igrpq=1.0:nwc=3:av=off:sp=reverse_arity_900");
    fallback.push("ott+10_14_ins=3:nwc=2:av=off:sp=occurrence_600");
    fallback.push("ott+10_64_bd=off:fde=none:ins=3:nwc=1:av=off:sp=reverse_arity_900");
    fallback.push("lrs+10_10_bd=preordered:fde=unused:ins=3:igrpq=1.0:lwlo=on:nwc=5:av=off:sp=occurrence_600");
    fallback.push("lrs+10_4:1_bd=preordered:ins=3:igrpq=1.0:nwc=1:av=off_600");
    fallback.push("ott+10_2_bd=preordered:fde=none:ins=3:nwc=2:av=off:sp=reverse_arity_600");
    fallback.push("ott+10_40_fde=none:ins=3:nwc=1:av=off:sp=reverse_arity_600");
    fallback.push("ott+10_4_bd=preordered:ins=3:nwc=5:av=off:sp=occurrence_600");
    fallback.push("lrs+10_24_ins=3:igrpq=1.0:nwc=1:av=off:sp=reverse_arity_1200");
    fallback.push("ott+10_3:1_fde=none:ins=3:nwc=3:av=off_300");
    fallback.push("ott+10_5:4_fde=none:ins=3:nwc=1.7:av=off:sp=occurrence_300");
    fallback.push("lrs+10_8:1_bd=off:fde=unused:ins=3:nwc=2.5:av=off_1200");
    fallback.push("lrs+10_5:1_fde=unused:ins=3:nwc=1.3:av=off:sp=occurrence_1200");
    fallback.push("ott+10_12_bd=off:ins=3:igrpq=1.0:nwc=1:av=off:sp=reverse_arity_600");
    fallback.push("ott+10_8_bd=preordered:ins=3:nwc=1.5:av=off:sp=occurrence_600");
    fallback.push("lrs+10_64_fde=none:ins=3:lwlo=on:nwc=1:av=off:sp=occurrence_1200");
    break;

  case Property::FEQ:
    fallback.push("dis+11_7_300");
    fallback.push("ott+11_8:1_cond=fast:gs=on:gsem=off:igrpq=1.0:nwc=1:nicw=on:sd=2:ss=axioms:st=1.2:sos=all:acc=on:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:urr=on_300");
    fallback.push("lrs+1011_4_cond=fast:gs=on:gsaa=from_current:gsem=off:gsssp=full:igrpq=1.0:nm=0:nwc=1:sas=minisat:sd=3:ss=axioms:st=2.0:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:sp=reverse_arity_300");
    fallback.push("dis+11_4_bsr=unit_only:cond=fast:fde=none:lwlo=on:nm=0:nwc=1.2:av=off:sp=occurrence_1200");
    fallback.push("lrs+1002_6_ccuc=first:cond=on:fsr=off:igrpq=1.0:nwc=4:nicw=on:sas=minisat:acc=on:afr=on:afp=40000:afq=1.0:amm=sco:anc=all_dependent:sp=reverse_arity:urr=ec_only_300");
    fallback.push("ott+1011_4:1_bd=off:bsr=unit_only:cond=fast:fsr=off:fde=none:inw=on:igrpq=1.0:nm=0:nwc=1.1:sas=minisat:sos=on:afp=10000:afq=1.2:anc=none:sp=occurrence_300");
    fallback.push("lrs+4_2:1_ep=R:fde=unused:gs=on:igrpq=1.0:nwc=1:sos=all:sac=on:aac=none:aer=off:afr=on:afp=10000:afq=1.2:anc=none:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_3_bsr=unit_only:cond=on:fsr=off:lwlo=on:nwc=1:sd=3:ss=axioms:st=3.0:av=off_300");
    fallback.push("lrs+1010_1_bs=unit_only:cond=fast:gs=on:gsem=off:igrpq=1.0:nwc=1:sos=all:av=off_300");
    fallback.push("dis+1010_3:1_cond=fast:fde=unused:gs=on:igrpq=1.0:nwc=1:sd=2:ss=axioms:sos=on:add=large:aer=off:afr=on:afp=100000:afq=1.2:updr=off_300");
    fallback.push("lrs+11_5:4_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:igrpq=1.0:nwc=1:sas=minisat:sd=3:ss=axioms:st=3.0:sos=all:sac=on:aer=off:afr=on:afp=1000:afq=1.4:anc=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis-3_10_bsr=unit_only:er=filter:fde=unused:igrpq=1.0:lcm=predicate:nm=64:nwc=1:av=off:sp=occurrence:updr=off_300");
    fallback.push("ott+1010_40_bs=unit_only:cond=fast:nwc=1:sas=minisat:add=off:afp=10000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off_600");
    fallback.push("ins+11_3_cond=fast:fde=none:igbrr=0.8:igrr=1/16:igrp=200:igrpq=1.05:igs=1003:igwr=on:nm=64:nwc=1:sas=minisat:sd=2:ss=axioms:st=3.0:sos=on:av=off:updr=off_300");
    fallback.push("dis+11_1024_br=off:ep=RSTC:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sac=on:afp=40000:afq=1.0:anc=none:sp=reverse_arity:urr=on_300");
    fallback.push("ins+11_32_br=off:ep=RSTC:inw=on:igbrr=0.9:igrr=1/32:igrp=100:igrpq=2.0:igs=1004:igwr=on:nwc=1:av=off:urr=on:updr=off:dm=on_300");
    fallback.push("lrs+11_5:1_bd=off:gs=on:gsem=off:gsssp=full:nwc=1.3:sas=minisat:sos=all:sac=on:aac=none:acc=model:add=large:afp=4000:afq=1.4:anc=none:sp=occurrence_300");
    fallback.push("lrs+1002_2:3_bsr=unit_only:er=known:fde=none:gs=on:gsem=on:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.5:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=reverse_arity_300");
    fallback.push("lrs+3_5_bd=preordered:bsr=unit_only:gsp=input_only:gs=on:gsem=on:gsssp=full:lwlo=on:nm=64:nwc=1:sas=minisat:av=off:sp=occurrence:urr=ec_only:updr=off_900");
    fallback.push("ott+1011_10_cond=fast:fde=none:gsp=input_only:gs=on:gsssp=full:nwc=1:sas=minisat:sd=3:ss=axioms:sos=all:av=off:sp=occurrence:updr=off_300");
    fallback.push("dis+1002_6_ccuc=first:cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=64:nwc=2.5:nicw=on:sas=minisat:sos=on:aac=none:acc=on:add=large:afr=on:afp=100000:afq=1.2:amm=off:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("ott+10_5_cond=fast:fde=none:nwc=1:sas=minisat:sos=all:av=off:sp=occurrence:updr=off_300");
    fallback.push("ins+11_4_cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:igbrr=0.3:igpr=on:igrr=1/8:igrp=100:igrpq=1.5:igwr=on:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:av=off:dm=on_300");
    fallback.push("lrs+1002_2:3_br=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:igrpq=1.0:nwc=1:sas=minisat:sac=on:aer=off:afr=on:afp=100000:afq=2.0:anc=none:sp=reverse_arity:urr=on_300");
    fallback.push("dis+11_7_fde=none:igrpq=1.0:nm=0:nwc=1:sd=3:ss=axioms:st=2.0:av=off:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+11_5_nwc=1:sd=7:ss=axioms:st=2.0:sac=on:afr=on:afp=40000:afq=1.2:anc=none_900");
    fallback.push("dis+11_2_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1002_3_ep=RST:er=known:fsr=off:gs=on:gsaa=from_current:igrpq=1.0:nwc=1:sas=minisat:sd=2:ss=axioms:st=5.0:sos=on:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:sp=occurrence_300");
    fallback.push("lrs+11_8:1_br=off:cond=on:fsr=off:fde=none:gs=on:inw=on:igrpq=1.0:lwlo=on:nwc=1.5:aer=off:afp=10000:afq=1.2:anc=none:sp=reverse_arity:urr=on_300");
    fallback.push("lrs-2_5_cond=on:fde=none:igrpq=1.0:lcm=predicate:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:add=off:afp=100000:afq=1.4:anc=none_300");
    fallback.push("dis+11_4_cond=fast:fsr=off:gs=on:gsaa=from_current:gsem=on:nwc=1:sas=minisat:sd=10:ss=axioms:st=5.0:sos=all:aer=off:afp=4000:afq=1.0:anc=none:sp=occurrence_300");
    fallback.push("ins+11_5_fde=unused:igpr=on:igrr=1/16:igrp=200:igrpq=1.1:igs=1010:igwr=on:nwc=1:sos=all:av=off_900");
    fallback.push("lrs-1_2:1_bd=preordered:bsr=on:br=off:cond=on:gs=on:lcm=reverse:nwc=1:sd=2:ss=axioms:sos=on:add=large:aer=off:afp=100000:afq=2.0:anc=none:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+11_2_br=off:cond=on:fde=none:gs=on:gsaa=full_model:igrpq=1.0:lwlo=on:nwc=2:sas=minisat:afp=100000:afq=1.4:amm=sco:anc=none:sp=occurrence:urr=on_300");
    fallback.push("dis-4_2_cond=on:fde=unused:igrpq=1.0:lcm=reverse:nwc=1:sos=on:av=off:sp=reverse_arity:updr=off_300");
    fallback.push("dis+4_3:1_gs=on:nwc=1:sd=1:ss=axioms:sos=all:av=off:updr=off_300");
    fallback.push("dis+1002_4_cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:igrpq=1.0:nm=64:nwc=1:sas=minisat:sos=on:acc=model:aer=off:afr=on:afp=4000:afq=1.4:anc=none:updr=off_300");
    fallback.push("lrs+4_40_bsr=unit_only:cond=fast:fde=none:gs=on:gsem=on:igrpq=1.0:lwlo=on:nwc=1.2:sd=7:ss=axioms:st=5.0:aac=none:add=off:afr=on:afp=1000:afq=1.1:amm=sco:anc=all_dependent:sp=reverse_arity:tha=off_600");
    fallback.push("lrs+1010_4:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=on:igrpq=1.0:nm=0:nwc=1:sas=minisat:sd=2:ss=axioms:st=1.5:sos=on:sac=on:add=off:aer=off:afr=on:afp=100000:afq=1.4:anc=none:sp=occurrence_300");
    fallback.push("dis+1011_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:igrpq=1.0:nm=0:nwc=1:sas=minisat:sd=3:ss=axioms:sos=on:aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=occurrence_300");
    fallback.push("lrs+1010_8:1_fsr=off:fde=none:gs=on:gsem=on:gsssp=full:igrpq=1.0:nwc=1.1:sas=minisat:sos=all:aac=none:afr=on:afp=100000:afq=1.0:amm=off:anc=all:sp=occurrence:updr=off_300");
    fallback.push("dis+11_5_fde=none:gs=on:gsaa=from_current:gsssp=full:igrpq=1.0:lcm=reverse:nwc=1:sas=minisat:ss=axioms:st=1.5:sos=on:afp=4000:afq=1.2:amm=sco:anc=none_300");
    fallback.push("lrs+1002_4_cond=on:er=filter:fde=unused:gsp=input_only:gs=on:gsssp=full:igrpq=1.0:nwc=10:sas=minisat:av=off:sp=occurrence_300");
    fallback.push("ott+1011_3_bd=off:fde=unused:nwc=1.5:av=off:sp=occurrence:updr=off_300");
    fallback.push("ott+1002_5_bsr=on:fsr=off:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:acc=on:aer=off:afp=100000:afq=1.1:sp=occurrence_300");
    fallback.push("lrs+10_14_cond=on:gs=on:gsem=off:nwc=1.5:nicw=on:sas=minisat:sac=on:afr=on:afp=4000:afq=1.2:anc=all:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1010_50_gs=on:gsaa=full_model:gsem=on:nwc=4:nicw=on:sas=minisat:aac=none:acc=model:afp=100000:afq=2.0:amm=off:anc=none:sp=reverse_arity:updr=off_300");
    fallback.push("dis+11_5_fsr=off:fde=none:gs=on:gsem=off:gsssp=full:inw=on:inst=on:nwc=1:aer=off:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+10_4_ccuc=first:cond=on:gs=on:gsssp=full:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:acc=on:aer=off:afp=40000:afq=1.2:anc=none:sp=reverse_arity:updr=off_900");
    fallback.push("ott+1004_28_cond=fast:igrpq=1.0:nwc=1:sos=all:av=off:updr=off_300");
    fallback.push("lrs-1_4_cond=fast:ep=RST:fde=unused:gs=on:gsem=on:gsssp=full:igrpq=1.0:lwlo=on:nwc=1:sas=minisat:av=off:urr=ec_only_300");
    fallback.push("lrs+1011_3:1_bsr=unit_only:cond=fast:er=known:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:igrpq=1.0:nwc=1:sas=minisat:add=large:aer=off:afr=on:afp=100000:afq=2.0:updr=off_300");
    fallback.push("ott+1010_3:1_cond=fast:fde=none:igrpq=1.0:nwc=1:sos=all:av=off_300");
    fallback.push("lrs+1010_2:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:igrpq=1.0:nm=0:nwc=2.5:sac=on:acc=model:add=off:afp=100000:afq=1.4:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("ott+11_24_cond=fast:ep=RST:fsr=off:fde=none:gs=on:lcm=predicate:nm=64:nwc=1:sas=minisat:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence_300");
    fallback.push("lrs+2_8:1_cond=fast:er=filter:fde=unused:igrpq=1.0:lcm=predicate:nwc=2.5:sas=minisat:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:sp=occurrence:updr=off_600");
    fallback.push("lrs+1011_5_bd=off:br=off:ccuc=small_ones:fde=none:gs=on:gsem=off:igrpq=1.0:nwc=1:sd=1:ss=axioms:sos=on:acc=model:afp=100000:afq=1.4:amm=off:anc=none:sp=occurrence:urr=on_300");
    fallback.push("dis+1011_4_fde=none:gs=on:igrpq=1.0:nwc=1:sos=on:add=off:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:updr=off_300");
    fallback.push("ott+2_2_cond=fast:fsr=off:gs=on:nwc=1:sd=1:ss=axioms:st=2.0:sos=on:sac=on:add=large:aer=off:afr=on:afp=4000:afq=1.4:anc=none:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis-2_4_cond=fast:ep=RST:er=filter:fde=unused:gs=on:gsem=on:igrpq=1.0:lcm=reverse:nwc=1:sd=3:ss=axioms:sos=on:av=off:updr=off_300");
    fallback.push("ott+1_3:1_cond=fast:gs=on:gsem=off:igrpq=1.0:nwc=1:sas=minisat:sos=all:afp=10000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+10_2_bsr=unit_only:cond=fast:gsp=input_only:gs=on:gsssp=full:lcm=reverse:lwlo=on:nwc=1:sas=minisat:sos=on:av=off:sp=reverse_arity_300");
    fallback.push("dis+1010_8_bd=off:bsr=on:ccuc=first:cond=fast:er=known:fsr=off:gs=on:gsssp=full:lcm=reverse:nm=0:nwc=1:sas=minisat:sd=2:ss=axioms:st=5.0:acc=on:add=off:afp=100000:afq=1.1:urr=ec_only:updr=off_300");
    fallback.push("lrs+10_1_bd=off:fde=none:gsp=input_only:lcm=predicate:nm=0:nwc=1:sos=on:av=off:updr=off_300");
    fallback.push("dis+11_4_fde=unused:gs=on:gsaa=from_current:nwc=2.5:sac=on:add=large:aer=off:afp=100000:afq=1.4:anc=none_300");
    fallback.push("lrs+11_3:2_cond=fast:fsr=off:fde=none:gs=on:igrpq=1.0:nm=0:nwc=1.7:sd=2:ss=axioms:st=2.0:av=off:urr=on_300");
    fallback.push("lrs-3_2:1_bsr=unit_only:fde=none:gs=on:gsssp=full:nm=0:nwc=1.5:sas=minisat:sos=all:sac=on:amm=sco:anc=none:sp=occurrence_300");
    fallback.push("lrs+10_8_br=off:fsr=off:gsp=input_only:gs=on:gsem=off:igrpq=1.0:nwc=1:sas=minisat:av=off:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+2_5:4_fde=none:igrpq=1.0:nwc=1:sd=2:ss=axioms:sos=on:afp=40000:afq=2.0_300");
    fallback.push("lrs+1002_7_ccuc=first:cond=on:fde=none:gs=on:gsssp=full:igrpq=1.0:nwc=1:sas=minisat:sd=3:ss=axioms:acc=model:aer=off:afp=1000:afq=2.0:anc=none:sp=reverse_arity_900");
    fallback.push("ins+10_1_av=off_300");
    fallback.push("lrs+11_28_cond=on:gs=on:gsaa=from_current:gsem=on:gsssp=full:igrpq=1.0:lwlo=on:nm=64:nwc=1.7:sac=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:urr=on:updr=off_300");
    fallback.push("lrs+1_3:1_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=off:igrpq=1.0:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=reverse_arity_300");
    fallback.push("dis+3_3:2_cond=on:fde=none:gs=on:gsem=on:nm=64:nwc=1:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.2:amm=off:anc=none:sp=occurrence_300");
    fallback.push("dis+1010_5:1_cond=fast:ep=RSTC:er=known:fde=unused:igrpq=1.0:nm=0:nwc=2:av=off_300");
    fallback.push("lrs+1002_3_bd=off:bs=on:bsr=unit_only:cond=fast:fsr=off:igrpq=1.0:lcm=predicate:nwc=1.5:sos=on:add=off:aer=off:afr=on:sp=reverse_arity_300");
    fallback.push("dis+10_1024_cond=fast:fde=none:gs=on:gsem=off:nwc=1:sd=7:ss=axioms:st=5.0:sos=all:av=off:sp=reverse_arity_300");
    fallback.push("lrs+4_3:1_bs=on:bsr=unit_only:ccuc=small_ones:cond=fast:er=filter:fsr=off:lcm=predicate:nm=1024:nwc=1:sas=minisat:sos=on:sac=on:acc=on:afp=1000:afq=1.2:amm=sco:anc=all_dependent:sp=reverse_arity:updr=off_300");
    fallback.push("dis+11_5_fde=none:igrpq=1.0:nwc=1:sas=minisat:sd=1:ss=axioms:st=5.0:sos=all:add=large:aer=off:afr=on:afp=100000:afq=2.0:anc=none_300");
    fallback.push("ott+2_8_bsr=unit_only:cond=fast:fde=none:gs=on:igrpq=1.0:lwlo=on:nwc=1:sas=minisat:av=off_300");
    fallback.push("ins+11_3_cond=on:fde=none:gs=on:gsem=off:gsssp=full:igbrr=1.0:igrr=1/16:igrp=100:igrpq=1.05:igs=1:igwr=on:nwc=1:sas=minisat:sos=on:av=off:sp=occurrence:urr=on_300");
    fallback.push("dis+11_20_fsr=off:fde=unused:gs=on:gsssp=full:igrpq=1.0:nm=0:nwc=1.3:nicw=on:add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=all:sp=reverse_arity_300");
    fallback.push("lrs+3_4:1_bs=unit_only:bsr=on:cond=on:er=known:fde=none:lcm=predicate:lwlo=on:nwc=1.5:sas=minisat:sos=all:afr=on:afp=100000:afq=1.1:amm=sco:anc=all_dependent:sp=occurrence:updr=off_2100");
    fallback.push("ins+10_1_igbrr=0.6:igrpq=1.5:igs=1002:nwc=1:av=off:sp=reverse_arity:tha=off:dm=on_600");
    fallback.push("lrs+1010_2:1_bd=preordered:bs=on:cond=fast:fde=none:gs=on:inw=on:igrpq=1.0:lwlo=on:nwc=1:sas=minisat:sos=all:av=off_600");
    fallback.push("lrs+11_3_gs=on:nwc=1.3:av=off:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("ott+11_4:1_bd=off:bsr=unit_only:cond=on:fsr=off:lcm=reverse:nwc=1:sas=minisat:sos=on:av=off:urr=on:updr=off_300");
    fallback.push("ott+1_4_er=filter:fsr=off:nwc=1:add=large:aer=off:afr=on:afp=40000:afq=1.0:anc=none:sp=reverse_arity_300");
    fallback.push("dis+1010_5:1_fde=none:igrpq=1.0:lwlo=on:nm=0:nwc=1:sas=minisat:sos=on:add=off:afr=on:afp=1000:afq=1.1:anc=none:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+1003_7_cond=fast:igrpq=1.0:nwc=1:sd=4:ss=axioms:sos=all:av=off:updr=off_300");
    fallback.push("dis+1010_1_fde=none:gs=on:gsem=off:gsssp=full:igrpq=1.0:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:afr=on:amm=off:anc=all:sp=reverse_arity:updr=off_300");
    fallback.push("dis+2_2:1_cond=on:er=filter:fde=none:gs=on:gsem=on:igrpq=1.0:nwc=1:sac=on:add=large:afp=10000:afq=1.2:amm=off:sp=occurrence_300");
    fallback.push("dis+11_3_cond=fast:fsr=off:gs=on:gsem=off:gsssp=full:inw=on:nwc=1.7:sas=minisat:add=off:aer=off:afp=1000:afq=1.2:anc=none:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+11_14_bd=preordered:fsr=off:gs=on:gsem=off:igrpq=1.0:nwc=2:afp=4000:afq=1.2:amm=off:anc=none:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1003_8:1_bd=off:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:igrpq=1.0:lcm=reverse:nwc=1:sas=minisat:sos=all:aac=none:acc=on:add=off:afr=on:amm=sco:anc=none:sp=reverse_arity_300");
    fallback.push("dis+11_5_cond=on:fsr=off:fde=none:gsp=input_only:igrpq=1.0:lcm=reverse:nm=0:nwc=4:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:updr=off_300");
    fallback.push("ott+1010_3:1_bd=off:bs=unit_only:bsr=unit_only:fsr=off:gs=on:gsaa=from_current:nwc=1.7:aer=off:afp=10000:afq=1.2:anc=none:sp=occurrence:urr=ec_only_300");
    fallback.push("ott+1_2_cond=fast:er=filter:fsr=off:fde=none:gs=on:gsem=off:gsssp=full:igrpq=1.0:nwc=1:sas=minisat:sd=1:ss=axioms:st=1.2:sac=on:add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none_300");
    fallback.push("ott-11_4_br=off:cond=on:gs=on:gsem=off:gsssp=full:igrpq=1.0:nwc=5:sas=minisat:sac=on:add=large:afr=on:afp=4000:afq=2.0:anc=all:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+10_2_bd=off:bsr=unit_only:ccuc=first:cond=fast:fsr=off:fde=none:gs=on:gsem=on:nwc=1.5:nicw=on:sos=all:sac=on:aac=none:acc=model:afr=on:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:updr=off_2400");
    fallback.push("dis+1011_2_fde=unused:gs=on:igrpq=1.0:nwc=1:sac=on:afp=40000:afq=1.1:amm=off:anc=none:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+3_5:1_bd=off:bs=unit_only:bsr=unit_only:br=off:ccuc=small_ones:er=known:fsr=off:fde=unused:gs=on:nm=0:nwc=1.1:sd=3:ss=axioms:sos=on:acc=model:add=off:aer=off:afp=4000:afq=1.4:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+1004_2:1_cond=fast:fde=none:gs=on:gsem=off:igrpq=1.0:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1010_4:1_cond=fast:fsr=off:igrpq=1.0:nm=0:nwc=1:sas=minisat:sac=on:add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=ec_only_300");
    fallback.push("lrs-11_2_cond=on:fde=unused:gs=on:igrpq=1.0:nwc=3:add=off:afr=on:afp=100000:afq=1.4:amm=sco:anc=all_300");
    fallback.push("lrs+1011_8_br=off:cond=fast:fsr=off:fde=none:igrpq=1.0:nwc=1:sas=minisat:sd=2:ss=axioms:sos=all:av=off:urr=on_300");
    fallback.push("lrs+11_4:1_fsr=off:fde=unused:gs=on:gsem=off:igrpq=1.0:nwc=1:sas=minisat:av=off:sp=reverse_arity:urr=ec_only_300");
    fallback.push("ott+4_8_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:igrpq=1.0:nwc=1:sos=all:sac=on:afp=100000:afq=1.1:amm=off:anc=none:sp=reverse_arity_300");
    fallback.push("dis+11_8_bs=unit_only:nwc=1:sd=10:ss=axioms:st=1.5:av=off:sp=occurrence:updr=off_900");
    fallback.push("dis+1003_3:2_bd=off:bsr=unit_only:nwc=1.3:sas=minisat:sac=on:add=large:aer=off:afr=on:afp=1000:afq=1.2:anc=none:updr=off_300");
    fallback.push("dis+1003_3_bs=unit_only:cond=fast:gs=on:gsaa=full_model:igrpq=1.0:lwlo=on:nwc=1.5:sas=minisat:sd=1:ss=axioms:st=2.0:sac=on:add=large:afr=on:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_300");
    fallback.push("dis+11_4:1_br=off:ccuc=first:fsr=off:fde=none:igrpq=1.0:nm=0:nwc=1:sos=all:acc=model:add=off:aer=off:afp=10000:afq=1.1:anc=all_dependent:sp=occurrence:urr=on:updr=off_300");
    fallback.push("dis+4_64_bs=unit_only:cond=on:er=known:fde=unused:gs=on:gsem=off:igrpq=1.0:nwc=1.2:sas=minisat:aac=none:aer=off:afr=on:afp=10000:afq=2.0:anc=all:sp=reverse_arity:updr=off_300");
    fallback.push("ott+11_2:1_cond=on:gs=on:gsssp=full:lwlo=on:nwc=1:sas=minisat:sos=all:av=off:sp=occurrence:tha=off_300");
    fallback.push("ott+11_6_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:gsssp=full:inw=on:igrpq=1.0:nwc=1.5:sas=minisat:av=off:sp=occurrence:urr=on_300");
    fallback.push("lrs+1011_2:1_br=off:cond=fast:fde=none:gs=on:gsssp=full:nwc=1:sos=all:sac=on:add=off:afp=1000:afq=2.0:amm=sco:anc=none:urr=on_300");
    fallback.push("lrs+11_2_bd=off:ccuc=first:cond=on:fde=unused:gs=on:gsem=off:nwc=1:sos=all:acc=model:add=large:aer=off:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_300");
    fallback.push("dis+1002_2_cond=on:gs=on:inw=on:nwc=1:sas=minisat:sos=on:sac=on:add=large:aer=off:afr=on:afp=4000:afq=1.2:anc=none:sp=reverse_arity_300");
    fallback.push("ins+11_3_bsr=unit_only:fde=none:gs=on:gsem=off:igbrr=0.0:igrr=1/16:igrpq=1.5:igs=1004:igwr=on:lcm=reverse:nm=0:nwc=1:sos=all:av=off:updr=off:dm=on_300");
    fallback.push("ott+1011_1024_bd=preordered:bs=on:cond=on:nwc=1:av=off:updr=off_300");
    fallback.push("ott-1_2:3_bs=unit_only:bsr=unit_only:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:sas=minisat:sos=on:add=large:afr=on:afp=1000:afq=1.4:amm=off:anc=all:sp=occurrence_300");
    fallback.push("lrs+4_64_fsr=off:igrpq=1.0:nwc=1.5:sas=minisat:av=off:sp=occurrence_300");
    fallback.push("lrs+1_5:4_cond=on:fsr=off:fde=none:gs=on:gsem=on:igrpq=1.0:lwlo=on:nm=64:nwc=1:sos=all:av=off_600");
    fallback.push("lrs-4_4_cond=fast:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:gsssp=full:lcm=reverse:nwc=1:sas=minisat:sd=4:ss=axioms:sos=on:sac=on:aac=none:add=large:aer=off:afp=1000:afq=1.2:anc=none:sp=reverse_arity_300");
    fallback.push("dis+10_3_cond=fast:fsr=off:gs=on:gsaa=from_current:nwc=1:sas=minisat:sac=on:afp=10000:afq=1.0:amm=sco:anc=none:sp=occurrence:tha=off_300");
    fallback.push("lrs+11_1_br=off:fde=unused:gs=on:gsem=off:inw=on:nwc=1:sas=minisat:sac=on:acc=model:aer=off:afp=100000:afq=1.2:anc=none:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+1010_5:1_bs=unit_only:ccuc=small_ones:cond=on:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=3:sos=on:sac=on:acc=on:afp=40000:afq=1.2:amm=sco:anc=all_dependent:sp=occurrence:urr=ec_only_300");
    fallback.push("lrs+10_8:1_bd=preordered:bs=on:ccuc=first:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:nicw=on:sas=minisat:sos=on:acc=on:aer=off:afr=on:afp=4000:afq=1.0:anc=none:sp=reverse_arity:urr=on_1200");
    fallback.push("dis+11_5_bd=off:cond=fast:fde=unused:gs=on:gsem=on:igrpq=1.0:lwlo=on:nwc=1:sos=on:sac=on:add=off:aer=off:afr=on:afp=10000:afq=2.0:anc=none:sp=reverse_arity:urr=on_300");
    fallback.push("lrs+10_50_bd=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:nwc=1.3:nicw=on:sos=all:add=off:aer=off:afr=on:afp=10000:afq=2.0:anc=none:sp=occurrence_300");
    fallback.push("lrs+1011_12_bs=on:bsr=unit_only:cond=on:gs=on:gsaa=from_current:gsssp=full:nwc=1.1:sas=minisat:sos=all:sac=on:add=off:aer=off:afr=on:afp=100000:afq=1.2:anc=none:sp=reverse_arity:updr=off_600");
    fallback.push("ott+11_20_bs=unit_only:fsr=off:gsp=input_only:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sac=on:afp=1000:afq=2.0:anc=none:sp=occurrence_600");
    fallback.push("lrs+4_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:igrpq=1.0:nwc=1:sd=5:ss=axioms:st=1.2:sac=on:add=off:aer=off:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence_900");
    fallback.push("lrs+1011_4_cond=fast:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:sd=5:ss=axioms:st=3.0:sac=on:add=off:aer=off:afr=on:afp=1000:afq=1.0:anc=none_900");
    break;

  case Property::FNE:
    fallback.push("dis+11_7_300");
    fallback.push("dis+1011_40_bs=on:cond=on:gs=on:gsaa=from_current:igrpq=1.0:nwc=1:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:updr=off_600");
    fallback.push("lrs+11_8:1_br=off:cond=on:fsr=off:fde=none:gs=on:inw=on:igrpq=1.0:lwlo=on:nwc=1.5:aer=off:afp=10000:afq=1.2:anc=none:sp=reverse_arity:urr=on_300");
    fallback.push("dis+1011_5:4_gs=on:gsssp=full:igrpq=1.0:nwc=1.5:sas=minisat:aac=none:add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=all:sp=reverse_arity:updr=off_300");
    fallback.push("dis-10_5_cond=fast:gsp=input_only:gs=on:gsem=off:igrpq=1.0:nwc=1:sas=minisat:sos=all:av=off:sp=occurrence_300");
    fallback.push("lrs+11_5:1_bd=off:gs=on:gsem=off:gsssp=full:nwc=1.3:sas=minisat:sos=all:sac=on:aac=none:acc=model:add=large:afp=4000:afq=1.4:anc=none:sp=occurrence_300");
    fallback.push("ins+11_5_br=off:gs=on:gsem=off:igbrr=0.9:igrr=1/64:igrp=1400:igrpq=1.1:igs=1003:igwr=on:lcm=reverse:nwc=1:av=off:urr=on:updr=off_1200");
    fallback.push("ott+2_8_lcm=reverse:nm=0:nwc=2.5:aer=off:afp=4000:afq=1.1:anc=none:sp=occurrence_900");
    fallback.push("ott+11_5_bs=on:bsr=on:gs=on:gsem=on:gsssp=full:igrpq=1.0:nwc=1:add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+11_5_cond=fast:fsr=off:gs=on:gsem=on:gsssp=full:nwc=1:sac=on:afp=100000:afq=1.2:amm=sco:anc=none:sp=occurrence:thf=on_300");
    fallback.push("dis+11_1_cond=on:fsr=off:igrpq=1.0:lcm=reverse:nwc=2.5:av=off:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1011_3_igrpq=1.0:nwc=1:sos=on:av=off:sp=reverse_arity_900");
    fallback.push("dis+1002_128_bs=on:cond=on:gs=on:gsem=on:igrpq=1.0:nm=0:nwc=1:sos=all:av=off:updr=off_300");
    fallback.push("lrs+1011_5_cond=fast:gs=on:igrpq=1.0:nwc=2.5:sd=3:ss=axioms:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=occurrence_300");
    fallback.push("ott-11_4_br=off:cond=on:gs=on:gsem=off:gsssp=full:igrpq=1.0:nwc=5:sas=minisat:sac=on:add=large:afr=on:afp=4000:afq=2.0:anc=all:sp=occurrence:urr=on:updr=off_300");
    fallback.push("dis+1010_28_nwc=1.3:sos=on:av=off:sp=reverse_arity:updr=off_600");
    fallback.push("lrs-3_5:4_bs=on:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:gsem=on:lcm=predicate:nwc=1.1:nicw=on:sas=minisat:sd=3:ss=axioms:sac=on:aac=none:afr=on:afp=1000:afq=1.0:anc=all:sp=reverse_arity:urr=ec_only:updr=off_600");
    fallback.push("dis+1011_8_bsr=unit_only:cond=fast:fsr=off:gs=on:gsaa=full_model:nm=0:nwc=1:sas=minisat:sos=all:afr=on:afp=4000:afq=1.1:amm=off:sp=reverse_arity_1200");
    fallback.push("dis+10_4_cond=fast:fsr=off:igrpq=1.0:nwc=1:sas=minisat:sac=on:add=large:afp=10000:afq=1.1:anc=none:sp=occurrence_900");
    fallback.push("dis+11_7_gs=on:gsaa=full_model:lcm=predicate:nwc=1.1:sas=minisat:aac=none:afp=1000:afq=1.0:amm=sco:sp=reverse_arity:urr=ec_only_1200");
    break;
  }

  // add very long final fallback strategy
  fallback.push("dis+11_1_3600");
}

/**
 * Get schedules for a problem of given property.
 * The schedules are to be executed from the bottom of the stack, i.e. in the order in which they are mentioned in the file.
 */
void Schedules::getCasc2017Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category();
  unsigned long prop = property.props();
  unsigned atoms = property.atoms();

  // for theory problems, we make the schedule before the main choice
  if (prop & (524288ul | 1048576ul | 2097152ul)) { // contains integers, rationals and reals
    if (prop & 67108864ul) { // uses linear integer functions
      quick.push("dis+10_3_afp=1000:afq=2.0:amm=off:anc=none:cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sac=on:sp=reverse_arity:updr=off_2");
      quick.push("lrs-10_8_add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:bsr=on:gsp=input_only:gs=on:lma=on:nm=64:nwc=1:stl=30:sos=all:sac=on:updr=off_2");
      quick.push("dis+1011_5_aac=none:add=large:afp=40000:afq=1.2:amm=off:anc=none:bd=off:fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:updr=off_26");
      quick.push("dis+1002_4_add=large:afp=40000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsaa=full_model:lma=on:lwlo=on:nm=0:nwc=1.5:sas=z3:sp=reverse_arity:tha=off:thi=strong_17");
      quick.push("dis+1010_24_aac=none:afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:nm=6:nwc=1:sas=z3:sos=on:sp=reverse_arity:tha=off_9");
      quick.push("lrs+1010_4:1_aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=on:gs=on:gsem=on:irw=on:nm=0:nwc=2.5:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_11");
      quick.push("dis+11_6_add=large:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=fast:gs=on:gsaa=from_current:gsem=off:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:thi=strong:updr=off_2");
      quick.push("dis+1010_2:3_add=off:afr=on:afp=10000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:tha=off_9");
      quick.push("dis-3_4_add=off:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:inw=on:lma=on:nm=64:nwc=1.5:nicw=on:sas=z3:sp=reverse_arity:tha=off:thf=on:uhcvi=on_13");
      quick.push("lrs-11_4:1_afp=4000:afq=2.0:anc=none:br=off:gs=on:lwlo=on:nm=64:nwc=3:stl=30:urr=on_2");
      quick.push("lrs+10_2:3_aac=none:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:gs=on:gsem=on:inw=on:newcnf=on:nwc=1:sas=z3:stl=30:sos=all:sp=reverse_arity:tha=off_5");
      quick.push("dis+10_3:2_afr=on:afp=1000:afq=1.2:bd=off:irw=on:lcm=predicate:lwlo=on:nm=0:newcnf=on:nwc=2:sos=on:tha=off:thf=on:urr=ec_only_11");
      quick.push("dis+1002_4_add=off:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1:sos=on:sac=on:sp=occurrence:tha=off:updr=off_2");
      quick.push("dis+1_3:1_acc=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:fsr=off:gs=on:inw=on:lma=on:nm=32:nwc=1:urr=on_6");
      quick.push("dis-10_4:1_aac=none:add=off:afp=1000:afq=1.4:amm=off:anc=none:cond=fast:ep=RSTC:gs=on:gsaa=from_current:gsem=on:inw=on:lma=on:nm=64:nwc=4:sas=z3:tha=off:thi=strong:uwa=interpreted_only:updr=off:uhcvi=on_6");
      quick.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_73");
      quick.push("dis+1010_4_add=off:afp=100000:afq=1.0:anc=none:fsr=off:gs=on:gsem=off:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sac=on:tha=off:thf=on_179");
      quick.push("lrs+10_3_add=off:afp=40000:afq=1.4:amm=off:anc=none:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=full_model:inw=on:lma=on:nm=64:nwc=1:sas=z3:stl=30:sos=all:sp=reverse_arity:tha=off:updr=off_5");
      quick.push("dis+1011_5_add=off:afp=4000:afq=2.0:anc=none:fsr=off:inw=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:sp=reverse_arity_182");
      quick.push("ott+1010_4:1_av=off:br=off:gs=on:lma=on:nm=64:nwc=1:sp=occurrence:urr=on_81");
      quick.push("dis+1011_2:3_add=off:afr=on:afp=4000:afq=1.4:anc=none:bs=unit_only:fsr=off:gs=on:gsem=on:lwlo=on:nm=16:nwc=1.3:nicw=on:sas=z3:sac=on:tha=off_260");
      quick.push("lrs+1_5:4_aac=none:add=off:afr=on:afp=4000:afq=1.2:amm=sco:anc=none:gsp=input_only:gs=on:irw=on:nm=64:newcnf=on:nwc=1.3:nicw=on:sas=z3:stl=30:sp=occurrence:tha=off_123");
      quick.push("dis+11_3_afp=100000:afq=2.0:amm=sco:anc=none:gs=on:gsem=off:inw=on:lma=on:nm=64:nwc=1:sas=z3:sd=10:ss=axioms:st=5.0:sp=occurrence:tha=off:updr=off_202");
      quick.push("lrs+1011_2:1_add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:thf=on:urr=on:updr=off_81");
    }
    else {
      quick.push("dis+10_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:irw=on:nm=64:nwc=1:sas=z3:sac=on_2");
      quick.push("lrs+10_5:4_afr=on:afp=40000:afq=1.2:bd=off:gsp=input_only:gs=on:inw=on:nm=0:nwc=1:sas=z3:stl=30:sos=all:sp=reverse_arity:tha=off:thf=on:urr=on_2");
      quick.push("dis+11_4_afp=100000:afq=1.1:anc=none:cond=on:gs=on:gsaa=full_model:nm=64:nwc=1:sac=on:sp=reverse_arity:thi=all_2");
      quick.push("dis+11_3_afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=preordered:bce=on:fsr=off:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1:sd=10:ss=axioms:st=5.0:sac=on:sp=occurrence:tha=off:urr=ec_only_85");
      quick.push("lrs+10_2:3_aac=none:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:gs=on:gsem=on:inw=on:newcnf=on:nwc=1:sas=z3:stl=30:sos=all:sp=reverse_arity:tha=off_5");
      quick.push("dis+10_10_add=large:afp=4000:afq=1.1:amm=sco:anc=none:irw=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=on_30");
      quick.push("lrs+1010_4:1_aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=on:gs=on:gsem=on:irw=on:nm=0:nwc=2.5:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_2");
      quick.push("dis+10_5_add=off:afp=4000:afq=1.1:anc=none:cond=fast:ep=RSTC:fsr=off:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sp=reverse_arity:thi=all_3");
      quick.push("dis+11_3_add=off:afp=10000:afq=2.0:amm=sco:anc=none:ep=RST:gs=on:gsaa=from_current:gsem=on:inw=on:nm=64:nwc=1:sd=10:ss=axioms:st=5.0:sos=all:tha=off:updr=off:uhcvi=on_58");
      quick.push("lrs-11_4:1_afp=4000:afq=2.0:anc=none:br=off:gs=on:lwlo=on:nm=64:nwc=3:stl=30:urr=on_2");
      quick.push("lrs+1_1024_av=off:bs=on:fde=none:inw=on:irw=on:nm=64:nwc=1.2:stl=60:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_595");
      quick.push("ott-11_3_add=large:afp=100000:afq=1.2:anc=none:bs=on:cond=fast:fde=none:gs=on:gsem=off:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:sp=occurrence:tha=off:urr=on:uhcvi=on_268");
      quick.push("ott-2_28_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=16:newcnf=on:nwc=1:nicw=on:sp=occurrence:thf=on_167");
    }
    // add very long final fallback strategy from UFLIA problems
    fallback.push("lrs+1011_2:1_afr=on:afp=10000:afq=2.0:amm=off:gsp=input_only:gs=on:inw=on:ile=on:nm=2:nwc=1:sas=z3:tha=off_300");
    fallback.push("ott+1010_2:1_acc=on:add=large:afr=on:afp=40000:afq=1.1:anc=none:gs=on:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("dis+2_1_add=large:afr=on:afp=1000:afq=1.2:anc=none:cond=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_300");
    fallback.push("ott+10_4:1_aac=none:add=off:afp=40000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1011_2_acc=on:afp=10000:afq=1.1:amm=sco:anc=none:ccuc=small_ones:cond=fast:fde=unused:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off:updr=off:uhcvi=on_300");
    fallback.push("ott+11_5:4_aac=none:add=large:afp=4000:afq=1.4:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=on:inw=on:ile=on:nm=2:newcnf=on:nwc=1:sas=z3:sos=on:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("dis+10_14_add=large:afp=4000:afq=1.1:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=none:inw=on:irw=on:lcm=predicate:nm=4:nwc=1.1:sos=on:sac=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1002_5:4_add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:updr=off_300");
    fallback.push("lrs+11_5:1_add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:er=known:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=predicate:lwlo=on:nm=64:newcnf=on:nwc=1.1:sac=on:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("dis+1011_5:1_afp=4000:afq=1.4:amm=off:anc=none:cond=on:fde=unused:gsp=input_only:ile=on:lma=on:nm=16:nwc=1:sos=on:sac=on:tha=off:urr=ec_only:uhcvi=on_300");
    fallback.push("dis+1010_1_add=off:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=6:nwc=1.3:nicw=on:sas=z3:tha=off:urr=ec_only_300");
    fallback.push("dis+1002_2_aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=all:bsr=on:fde=unused:inw=on:ile=on:lcm=reverse:nm=4:nwc=4:nicw=on:sos=theory:sac=on:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+1011_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off_300");
    fallback.push("ott+1002_3:1_av=off:bsr=on:ep=RS:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_3_add=off:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:bce=on:cond=fast:fde=unused:ile=on:lma=on:nm=6:nwc=2:nicw=on:sas=z3:sd=3:ss=axioms:st=2.0:sp=reverse_arity:tha=off_300");
    fallback.push("ott+1011_3:2_av=off:bd=off:bs=on:bce=on:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:tha=off:updr=off_300");
    fallback.push("dis+11_3:1_av=off:br=off:ep=R:fsr=off:gsp=input_only:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("lrs-2_8:1_add=large:afp=100000:afq=1.4:amm=sco:anc=none:bs=on:bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=4:nicw=on:sas=z3:sp=reverse_arity:updr=off_600");
    fallback.push("dis+1010_2_acc=on:afr=on:afp=100000:afq=1.2:anc=none:bsr=on:fsr=off:ile=on:irw=on:nm=16:newcnf=on:nwc=4:sp=occurrence:tha=off:urr=ec_only_300");
    fallback.push("ott+1010_1_add=large:afp=1000:afq=1.2:anc=none:bd=off:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("lrs+1002_2_add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:inw=on:lwlo=on:nm=32:newcnf=on:nwc=1:sos=theory:sac=on:sp=occurrence:urr=on_300");
    fallback.push("lrs-10_3_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=unused:gs=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.2:sas=z3:tha=off:urr=ec_only_300");
    fallback.push("dis+10_3_add=off:afp=100000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sp=reverse_arity:tha=off:thf=on:updr=off_300");
    fallback.push("lrs+2_4_add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:cond=on:ep=R:gs=on:gsaa=from_current:gsem=off:ile=on:lcm=reverse:lma=on:nm=2:nwc=1.1:sos=on:sac=on:tha=off:updr=off_300");
    fallback.push("lrs+2_1024_av=off:bd=off:bsr=on:cond=fast:fsr=off:fde=none:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:tha=off:thi=overlap:uwa=one_side_constant:updr=off:uhcvi=on_300");
    fallback.push("ott+1011_8:1_afr=on:afp=1000:afq=1.4:amm=sco:anc=none:bd=off:fsr=off:fde=unused:inw=on:ile=on:nm=2:nwc=1:nicw=on:sas=z3:sos=theory:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_600");
    fallback.push("lrs+10_2:3_afr=on:afp=1000:afq=1.1:bd=off:bce=on:cond=on:gsp=input_only:gs=on:gsaa=from_current:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:uwa=interpreted_only:updr=off:uhcvi=on_300");
    fallback.push("dis+10_32_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:fde=none:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sp=occurrence:tha=off:thi=full:uhcvi=on_300");
    fallback.push("ott+1010_2:1_add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:cond=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=1.5:sac=on:tha=off:updr=off_300");
    fallback.push("lrs-11_2:1_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.5:sas=z3:sp=reverse_arity:urr=on_300");
    fallback.push("dis+11_7_add=large:afr=on:afp=10000:afq=1.2:bd=off:bsr=on:cond=on:fsr=off:fde=unused:gs=on:ile=on:lcm=predicate:lma=on:nm=2:newcnf=on:nwc=3:sos=on:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1011_1_afp=40000:afq=1.2:anc=none:cond=on:gsp=input_only:ile=on:irw=on:lma=on:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+10_3_av=off:fde=unused:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1.7:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_600");
    fallback.push("lrs+11_2:1_add=off:anc=none:bsr=on:br=off:cond=on:er=filter:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=2:nwc=1:sas=z3:sos=all:sac=on:uwa=ground:urr=on_300");
    fallback.push("lrs+1003_3:2_afp=1000:afq=2.0:amm=off:anc=none:cond=on:gs=on:ile=on:lma=on:nm=6:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=all:updr=off_300");
    fallback.push("lrs+4_8:1_av=off:cond=on:gs=on:gsem=on:irw=on:nm=64:newcnf=on:nwc=1.1:sp=occurrence:tha=off:urr=on:updr=off_300");
    fallback.push("ott-4_5:4_aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:cond=fast:ile=on:irw=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=3:thf=on:urr=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1011_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=4:nwc=1:sac=on:tha=off:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_2:1_av=off:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sp=occurrence:tha=off:urr=ec_only:uhcvi=on_300");
    fallback.push("dis+1011_4_afr=on:afp=10000:afq=1.1:amm=off:anc=none:ep=RS:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1010_1_afp=1000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:br=off:cond=on:ile=on:irw=on:nm=2:nwc=1:nicw=on:sas=z3:sos=all:sp=reverse_arity:tha=off:urr=on:updr=off_300");
    fallback.push("lrs+10_24_afp=4000:afq=2.0:bd=off:bsr=on:bce=on:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:nwc=1.3:sp=occurrence:tha=off:uwa=one_side_constant:urr=ec_only_300");
    fallback.push("lrs-2_5:1_acc=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bd=off:cond=fast:gs=on:ile=on:nm=0:newcnf=on:nwc=3:sac=on:thf=on:urr=ec_only_300");
    fallback.push("lrs+1_5:1_add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity_300");
    fallback.push("lrs+1010_1_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:fsr=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on_300");
    fallback.push("lrs+1011_2:1_aac=none:add=off:afp=1000:afq=1.0:amm=off:bs=on:gs=on:gsaa=from_current:gsem=on:ile=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off_300");
    fallback.push("dis+1010_2_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:fsr=off:fde=none:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:sp=reverse_arity_300");
    fallback.push("ott+1011_3:1_add=off:afp=100000:afq=2.0:amm=off:anc=none:bs=unit_only:gs=on:gsem=on:irw=on:newcnf=on:nwc=1:sas=z3:tha=off_300");
    fallback.push("dis-3_7_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sp=occurrence:tha=off:thi=overlap:uwa=interpreted_only:uhcvi=on_300");
    fallback.push("dis+11_32_add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:er=filter:ile=on:lcm=predicate:lma=on:newcnf=on:nwc=5:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_2:1_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=preordered:ccuc=first:cond=fast:fsr=off:fde=unused:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=2:nicw=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+2_8_av=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1002_1_add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:fsr=off:gsp=input_only:inw=on:ile=on:lcm=predicate:lwlo=on:nm=64:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("ott+10_4:1_afp=100000:afq=1.1:anc=none:bd=off:inw=on:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sac=on:sp=occurrence:tha=off:urr=on:updr=off_300");
    fallback.push("dis+1011_5:1_afr=on:afp=10000:afq=1.2:amm=sco:bd=preordered:bs=unit_only:cond=on:fsr=off:inw=on:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=1.1:sd=7:ss=axioms:st=1.2:tha=off:uhcvi=on_300");
    fallback.push("dis+1011_3_aac=none:afr=on:afp=40000:afq=1.4:amm=off:bs=on:fsr=off:fde=none:gsp=input_only:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+4_4_av=off:bd=off:bs=unit_only:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:updr=off_300");
    fallback.push("lrs+11_10_add=off:afp=40000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bce=on:cond=fast:gsp=input_only:inw=on:lma=on:nm=4:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:urr=ec_only:uhcvi=on_300");
    fallback.push("ott+1011_2:3_av=off:bs=unit_only:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thi=all:uwa=all:urr=on:uhcvi=on_300");
    fallback.push("lrs+11_6_aac=none:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:cond=fast:ile=on:nm=16:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:uhcvi=on_300");
    fallback.push("ott+1011_2:3_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:bce=on:cond=fast:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1.1:sp=reverse_arity:tha=off:urr=on:updr=off_300");
    fallback.push("lrs+4_8:1_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:tha=off_300");
    fallback.push("lrs-2_3:1_add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=on:er=filter:fde=unused:ile=on:irw=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on:thi=strong:uhcvi=on_600");
    fallback.push("dis+1011_10_av=off:bd=off:cond=fast:er=known:inw=on:ile=on:irw=on:lma=on:nwc=1.7:sp=occurrence:tha=off:uhcvi=on_300");
    fallback.push("dis+1010_3_afp=10000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:tha=off:urr=on_300");
    fallback.push("lrs+2_3:2_av=off:cond=fast:inw=on:ile=on:nm=2:nwc=1:sos=theory:urr=on_300");
    fallback.push("lrs+11_2_av=off:cond=on:fsr=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("dis+10_2:1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=on:inw=on:ile=on:irw=on:nm=2:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
    fallback.push("ott+1003_12_add=large:anc=all:bd=preordered:bce=on:fde=none:lcm=reverse:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:uwa=ground_600");
    fallback.push("dis+1011_1_aac=none:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bce=on:cond=on:er=known:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=off:ile=on:newcnf=on:nwc=1:sas=z3:ss=axioms:st=2.0:sp=occurrence:tha=off:urr=ec_only_300");
    fallback.push("dis+1002_1_add=large:afp=4000:afq=1.2:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only:uhcvi=on_300");
    fallback.push("dis+2_4_afp=10000:afq=1.1:bd=off:bs=on:cond=on:er=filter:ile=on:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("lrs+1003_8:1_av=off:fsr=off:fde=unused:gsp=input_only:gs=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on_300");
    fallback.push("lrs+11_8:1_afp=100000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:nm=64:nwc=2:nicw=on:sac=on:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs-11_8:1_afr=on:afp=1000:afq=1.4:amm=off:anc=none:bd=off:bs=on:gs=on:ile=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only_300");
    fallback.push("lrs+10_3:1_add=large:afp=10000:afq=1.1:amm=off:anc=none:cond=on:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sd=5:ss=axioms:st=1.5:tha=off:urr=on_300");
    fallback.push("lrs+1010_3:1_av=off:bd=off:bsr=on:irw=on:nm=64:newcnf=on:nwc=1.7:sos=all:updr=off_300");
    fallback.push("dis+11_5:1_afr=on:afp=40000:afq=2.0:amm=sco:anc=all_dependent:cond=fast:fde=unused:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=2:nwc=1:sos=all:urr=on:uhcvi=on_300");
    fallback.push("lrs+1004_1_aac=none:add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=all_dependent:bd=off:cond=fast:fsr=off:gs=on:gsaa=from_current:lcm=reverse:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thf=on:urr=on:updr=off_300");
    fallback.push("lrs-10_3:2_aac=none:add=off:afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=16:nwc=1:nicw=on:sas=z3:sd=2:ss=axioms:sos=on:sp=occurrence:updr=off_600");
    fallback.push("ott+1002_2:1_add=large:afr=on:afp=100000:afq=1.1:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off_300");
    fallback.push("dis-4_7_acc=on:afp=40000:afq=1.4:anc=all_dependent:bsr=on:br=off:bce=on:ccuc=first:er=filter:fsr=off:fde=unused:gsp=input_only:ile=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:thi=full:uwa=ground:urr=on:updr=off:uhcvi=on_300");
    fallback.push("ott-1_1_acc=model:add=off:afr=on:afp=4000:afq=1.2:anc=all:bd=preordered:bs=unit_only:bsr=on:ccuc=first:gs=on:gsaa=from_current:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=occurrence:tha=off:thi=strong:updr=off_300");
    fallback.push("lrs+1011_5:4_av=off:bd=off:bs=on:cond=on:er=known:gs=on:gsem=on:inw=on:ile=on:lcm=reverse:nm=6:newcnf=on:nwc=1:sp=occurrence:tha=off:uhcvi=on_300");
    fallback.push("dis+1002_14_av=off:cond=fast:fde=unused:inw=on:ile=on:lma=on:nm=0:nwc=1:sos=all:sp=reverse_arity:tha=off:uwa=one_side_interpreted:uhcvi=on_300");
    fallback.push("dis+1011_8:1_add=off:afp=10000:afq=1.1:anc=none:bce=on:er=filter:gs=on:gsaa=from_current:gsem=off:inw=on:ile=on:lma=on:nm=2:nwc=3:sac=on:urr=on:updr=off_300");
    fallback.push("dis+4_4:1_add=off:afp=4000:afq=1.2:amm=sco:anc=none:br=off:cond=fast:ep=RS:fsr=off:inw=on:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on:urr=on:uhcvi=on_300");
    fallback.push("lrs+1002_1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+1002_2:1_aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=none:cond=fast:gs=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_3:1_afp=1000:afq=1.4:amm=off:anc=none:bsr=on:inw=on:ile=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:tha=off:urr=on:updr=off_300");
    fallback.push("ott+10_2:1_av=off:bd=off:br=off:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+10_24_av=off:bs=unit_only:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sd=7:ss=axioms:st=1.2:sp=occurrence:tha=off:thf=on:uhcvi=on_600");
    fallback.push("lrs+4_28_afp=10000:afq=1.2:amm=sco:anc=none:bd=off:bce=on:cond=on:fsr=off:ile=on:irw=on:lcm=reverse:nm=16:newcnf=on:nwc=2:sas=z3:sp=occurrence:tha=off:updr=off:uhcvi=on_600");
    return;
  }

  switch (cat) {
  case Property::EPR:
    if (prop != 0) { // 1100
      quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_57");
      quick.push("ins+11_128_av=off:cond=on:fsr=off:irw=on:igbrr=0.5:igpr=on:igrr=1/2:igrp=2000:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:sos=all:sp=occurrence:updr=off_2");
      quick.push("lrs+1_16_add=off:afp=100000:afq=1.0:amm=off:cond=fast:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=2:nicw=on:stl=60:sd=7:ss=axioms:st=5.0:sos=theory:sp=reverse_arity:urr=ec_only_344");
      quick.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:stl=30:uhcvi=on_301");
      quick.push("dis+4_5_afp=1000:afq=1.1:amm=off:anc=none:bd=off:gs=on:irw=on:lcm=predicate:lma=on:nwc=1:sas=z3:sos=all:sp=occurrence_151");
      quick.push("dis+11_2:3_add=large:afp=10000:afq=1.2:anc=none:bd=off:bce=on:cond=fast:er=filter:fsr=off:fde=unused:gsp=input_only:nwc=5:sos=theory:sac=on:urr=on_245");
    }
    else if (atoms <= 3000) { // 1202
      quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_55");
      quick.push("fmb+10_1_av=off:fmbsr=1.1:updr=off_4");
      quick.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_283");
      quick.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:stl=30:uhcvi=on_25");
      quick.push("lrs+1011_14_afr=on:afp=100000:afq=1.1:anc=none:bd=off:bsr=on:irw=on:nwc=1:sas=z3:stl=30_90");
      quick.push("dis-11_32_av=off:bs=unit_only:gs=on:irw=on:lma=on:nwc=1:updr=off_34");
      quick.push("lrs+1003_10_afp=4000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:bce=on:fde=unused:lma=on:nwc=1:nicw=on:stl=120:sac=on:urr=on:updr=off:uhcvi=on_517");
      quick.push("ins+11_128_av=off:cond=on:fsr=off:irw=on:igbrr=0.5:igpr=on:igrr=1/2:igrp=2000:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:sos=all:sp=occurrence:updr=off_194");
    }
    else { // 2109
      quick.push("dis-1_64_acc=on:add=large:afp=100000:afq=1.1:anc=none:bd=preordered:ccuc=small_ones:gs=on:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_83");
      quick.push("fmb+10_1_av=off:fmbsr=1.1:updr=off_81");
      quick.push("dis+1011_128_afp=100000:afq=1.1:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:irw=on:lma=on:nwc=1:sos=theory:sac=on:urr=on:updr=off_19");
      quick.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_90");
      quick.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_12");
      quick.push("lrs+4_6_afp=10000:afq=1.2:amm=sco:anc=all_dependent:bs=on:bsr=on:fde=unused:irw=on:lma=on:nwc=1:stl=30:sos=theory:sp=occurrence:uhcvi=on_264");
      quick.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_233");
      quick.push("lrs+1_8:1_add=off:anc=none:bd=preordered:br=off:bce=on:fsr=off:fde=none:nwc=1:nicw=on:stl=90:sos=theory:sp=reverse_arity:urr=on_815");
      quick.push("ott-4_6_add=off:afr=on:afp=100000:afq=1.4:amm=sco:bs=on:fde=unused:gs=on:gsaa=full_model:gsem=on:irw=on:nwc=1:sas=z3:sac=on:updr=off:uhcvi=on_334");
      quick.push("lrs+1_5:1_add=off:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:gs=on:nwc=1.1:nicw=on:sas=z3:stl=30:sos=theory:urr=on:updr=off_178");
    }
    break;

  case Property::FEQ:
    if (atoms > 1000000) {
      quick.push("dis+11_32_av=off:ep=RST:fsr=off:lwlo=on:nm=6:nwc=1.1:sd=5:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_1001");
      quick.push("dis+11_40_aac=none:add=large:afp=10000:afq=1.2:amm=off:cond=fast:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:irw=on:nm=6:newcnf=on:nwc=1.2:sd=7:ss=axioms:st=1.5:sos=on:sp=reverse_arity:urr=on:uhcvi=on_915");
    }
    else if (atoms > 400000) { // 2040
      quick.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_188");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_137");
      quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_58");
      quick.push("dis-10_3:2_aac=none:afp=1000:afq=1.1:cond=on:fsr=off:lcm=reverse:lwlo=on:nm=16:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity:updr=off_108");
      quick.push("ott+11_3_afp=1000:afq=2.0:anc=none:fsr=off:irw=on:nwc=1.7:ss=axioms:st=1.5:sac=on:updr=off_137");
      quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_76");
      quick.push("dis+1011_3_afp=100000:afq=1.1:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:irw=on:nm=6:nwc=1:sas=z3:sd=2:ss=axioms:sos=on:sac=on:sp=reverse_arity:updr=off_45");
      quick.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_175");
      quick.push("dis+4_8:1_add=large:afp=100000:afq=1.4:ep=RST:fde=unused:gsp=input_only:lcm=predicate:nwc=1:sos=all:sp=occurrence:updr=off:uhcvi=on_169");
      quick.push("dis+10_4_add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=predicate:lma=on:nm=64:nwc=1:sd=3:ss=axioms:sos=on:sp=reverse_arity_42");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_83");
      quick.push("lrs+1011_1_av=off:cond=on:gs=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity:updr=off_78");
      quick.push("dis+11_8_add=off:afp=10000:afq=1.2:amm=off:anc=none:cond=fast:ep=R:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=4:sd=1:ss=axioms:sac=on:updr=off:uhcvi=on_55");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_294");
      quick.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:sos=all:sp=occurrence_66");
      quick.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_130");
      quick.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_199");
    }
    else if (atoms > 200000) { // 1928
      quick.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_38");
      quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_40");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_30");
      quick.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_172");
      quick.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_30");
      quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_27");
      quick.push("dis+11_8:1_afp=100000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=off:lcm=reverse:lma=on:nm=64:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on_20");
      quick.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_50");
      quick.push("ott+11_2:1_add=off:afp=10000:afq=1.1:anc=none:cond=on:gsp=input_only:lcm=reverse:lwlo=on:nm=32:nwc=5:sp=occurrence:updr=off_291");
      quick.push("lrs+10_4_add=off:afp=100000:afq=2.0:anc=none:bd=off:er=known:gs=on:gsem=off:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=2.5:sas=z3:stl=30:sac=on:urr=ec_only:updr=off_44");
      quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_37");
      quick.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_54");
      quick.push("dis+10_5_av=off:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_92");
      quick.push("lrs+1_7_av=off:cond=fast:fde=none:gs=on:gsem=off:lcm=predicate:nm=6:nwc=1:stl=30:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_23");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_60");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_165");
      quick.push("dis+11_2:1_add=large:afp=4000:afq=1.4:amm=sco:anc=none:fsr=off:nm=16:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_127");
      quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_251");
      quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_50");
      quick.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:stl=30:sd=2:ss=axioms:sos=all:urr=ec_only_42");
      quick.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_39");
      quick.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_247");
    }
    else if (atoms > 120000) { // 1928
      quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_24");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_61");
      quick.push("dis+1002_5_av=off:cond=on:gs=on:lma=on:nm=2:nwc=1:sd=3:ss=axioms:st=1.5:sos=on:updr=off_28");
      quick.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_25");
      quick.push("lrs+11_50_afp=100000:afq=1.1:amm=sco:anc=none:bs=unit_only:cond=on:irw=on:lma=on:nm=32:nwc=1.1:stl=30:sp=reverse_arity_252");
      quick.push("ott+11_8:1_acc=model:afp=1000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=16:nwc=1.5:sac=on:sp=occurrence:urr=on_50");
      quick.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_19");
      quick.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_14");
      quick.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_33");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_25");
      quick.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:stl=30:sp=occurrence_58");
      quick.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:stl=30:sd=2:ss=axioms:sos=all:urr=ec_only_23");
      quick.push("dis-10_3:2_aac=none:afp=1000:afq=1.1:cond=on:fsr=off:lcm=reverse:lwlo=on:nm=16:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity:updr=off_18");
      quick.push("ott+11_8_amm=off:anc=none:bsr=on:cond=on:irw=on:nm=2:nwc=1.1:ss=axioms:st=2.0:sac=on_39");
      quick.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_15");
      quick.push("dis+1011_8:1_aac=none:acc=on:afp=1000:afq=1.4:amm=off:anc=all:bs=unit_only:bce=on:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:lma=on:nm=16:nwc=2.5:sd=4:ss=axioms:st=1.5:sos=all:uhcvi=on_100");
      quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_32");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_290");
      quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_14");
      quick.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:sos=all_172");
      quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_272");
      quick.push("ott+11_2:1_av=off:bd=off:bsr=on:br=off:cond=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_41");
      quick.push("lrs+11_3_av=off:cond=on:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=5:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only_25");
      quick.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:sos=all:sp=occurrence_16");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_48");
    }
    else if (prop == 0) {
      if (atoms >= 70) { //
        quick.push("dis+1002_6_add=large:afp=40000:afq=2.0:bsr=on:cond=on:irw=on:lma=on:nm=2:nwc=2.5:nicw=on:sp=reverse_arity:updr=off_2");
        quick.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_7");
        quick.push("ott+2_2_afp=10000:afq=1.4:amm=off:anc=none:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:nm=32:nwc=1.5:sos=on:sp=reverse_arity_2");
        quick.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:stl=30:sp=occurrence:urr=on_118");
        quick.push("lrs+1010_32_afr=on:anc=none:bd=off:fsr=off:gs=on:gsem=on:nwc=1.3:nicw=on:sas=z3:stl=30:updr=off_63");
        quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_136");
        quick.push("lrs+10_128_av=off:bs=on:cond=on:gs=on:gsem=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:stl=30:updr=off_207");
        quick.push("lrs+1011_50_afr=on:afp=40000:afq=1.0:amm=off:anc=all_dependent:bs=on:bsr=on:bce=on:fde=unused:gs=on:lma=on:nm=16:nwc=1.1:stl=60:sp=occurrence:updr=off_474");
        quick.push("lrs+1011_1024_av=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=32:nwc=1:stl=90:sp=reverse_arity:uhcvi=on_426");
      }
      else { // 1809
        quick.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:stl=30:sac=on:sp=occurrence_9");
        quick.push("dis-10_1_aac=none:afr=on:afp=10000:afq=1.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=4:newcnf=on:nwc=2:sp=occurrence_2");
        quick.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:stl=30:sac=on:urr=on_79");
        quick.push("lrs+1010_32_afr=on:anc=none:bd=off:fsr=off:gs=on:gsem=on:nwc=1.3:nicw=on:sas=z3:stl=30:updr=off_16");
        quick.push("ott+11_5:4_add=large:afr=on:afp=100000:afq=1.0:anc=none:bsr=on:fsr=off:gs=on:lcm=reverse:nm=64:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity_2");
        quick.push("lrs+11_3:2_add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bs=unit_only:cond=on:gs=on:gsaa=full_model:gsem=off:irw=on:lwlo=on:nm=32:nwc=1.2:sas=z3:stl=90:sac=on_462");
        quick.push("lrs+11_5_av=off:cond=on:fsr=off:lma=on:lwlo=on:nwc=1.2:sas=z3:stl=30:sp=reverse_arity:updr=off_142");
        quick.push("ott+11_50_add=large:afr=on:afp=40000:afq=1.1:amm=sco:anc=all:bd=off:cond=fast:gs=on:gsem=on:irw=on:lma=on:nm=0:nwc=1:sp=occurrence:updr=off:uhcvi=on_102");
        quick.push("lrs+1004_5:4_aac=none:add=large:afp=100000:afq=1.4:anc=all_dependent:bd=off:cond=fast:gsp=input_only:gs=on:gsem=off:lma=on:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sp=occurrence:updr=off:uhcvi=on_99");
        quick.push("ott+1011_4:1_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=preordered:cond=fast:nm=32:newcnf=on:nwc=1.2:sas=z3:sac=on:sp=occurrence:urr=on:updr=off_122");
        quick.push("lrs-11_24_av=off:bd=off:bs=on:fsr=off:gs=on:gsem=off:lma=on:nm=4:nwc=1.7:stl=90:sp=occurrence:updr=off_61");
        quick.push("ott+10_8:1_av=off:bd=preordered:bsr=on:cond=fast:fsr=off:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1.2:sp=reverse_arity:urr=on:updr=off:uhcvi=on_713");
      }
    }
    else if (prop == 2) { // 2074
      quick.push("dis+11_4_afr=on:afp=1000:afq=1.4:amm=off:anc=none:lcm=reverse:lma=on:lwlo=on:nm=6:newcnf=on:nwc=1:sd=2:ss=axioms:st=2.0:sp=reverse_arity_20");
      quick.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:stl=30:sp=occurrence_77");
      quick.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_3");
      quick.push("dis+11_5:4_acc=on:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ccuc=small_ones:fsr=off:irw=on:nm=32:nwc=2.5:nicw=on:urr=on:updr=off_10");
      quick.push("dis+11_50_add=large:afp=1000:afq=1.0:amm=sco:anc=none:fsr=off:nm=16:nwc=1.5:sac=on_13");
      quick.push("ott+11_20_afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bsr=on:irw=on:lma=on:nm=4:nwc=1.2:sac=on:sp=occurrence_294");
      quick.push("ott+10_7_av=off:bd=preordered:bs=unit_only:bsr=on:cond=fast:fsr=off:fde=unused:irw=on:lma=on:nm=32:nwc=1:sos=all:sp=reverse_arity:updr=off_142");
      quick.push("dis+1_24_av=off:cond=on:irw=on:lma=on:nm=2:nwc=1.5:sp=occurrence:updr=off_16");
      quick.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_122");
      quick.push("lrs+11_14_add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:lma=on:nm=0:nwc=2:stl=30:sac=on:sp=reverse_arity_20");
      quick.push("lrs+11_24_add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=fast:fde=unused:gs=on:irw=on:lma=on:nm=4:nwc=1.3:nicw=on:sas=z3:stl=30:sac=on:sp=reverse_arity:uhcvi=on_136");
      quick.push("lrs+11_5_av=off:cond=on:fsr=off:lma=on:lwlo=on:nwc=1.2:sas=z3:stl=30:sp=reverse_arity:updr=off_123");
      quick.push("lrs+10_24_add=off:afr=on:afp=1000:afq=1.4:anc=none:bs=unit_only:gs=on:gsaa=from_current:gsem=on:lma=on:nm=2:nwc=1.1:stl=60:sac=on:uhcvi=on_461");
      quick.push("ott+11_20_afp=10000:afq=1.1:anc=none:bs=unit_only:bsr=on:bce=on:fsr=off:gs=on:gsem=on:nwc=2.5:sas=z3:sp=occurrence:updr=off:uhcvi=on_401");
      quick.push("lrs+11_128_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:nwc=1.3:stl=30:sac=on:uhcvi=on_236");
    }
    else if (prop == 131073) {
      if (atoms > 200) { // 1292
        quick.push("dis+1010_4_afp=10000:afq=1.2:anc=none:irw=on:lma=on:nm=64:nwc=10:sas=z3:sac=on:sp=reverse_arity:updr=off_2");
        quick.push("ott+11_8_amm=off:anc=none:bsr=on:cond=on:irw=on:nm=2:nwc=1.1:ss=axioms:st=2.0:sac=on_8");
        quick.push("dis+1011_10_aac=none:add=large:afp=10000:afq=1.1:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=6:newcnf=on:nwc=2.5:sp=reverse_arity:updr=off_2");
        quick.push("dis+11_28_av=off:fsr=off:irw=on:lcm=predicate:nm=2:newcnf=on:nwc=5:sp=occurrence:urr=on:updr=off_4");
        quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_4");
        quick.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_2");
        quick.push("lrs+1011_3:2_aac=none:afp=10000:afq=1.2:amm=off:anc=all:cond=on:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_21");
        quick.push("lrs+10_6_aac=none:acc=model:add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bd=off:ccuc=small_ones:irw=on:lcm=reverse:nm=0:nwc=1:nicw=on:stl=30:sos=on:sp=reverse_arity:updr=off_2");
        quick.push("dis+11_5_av=off:bd=off:bs=unit_only:bsr=on:cond=on:lcm=reverse:nm=0:nwc=1.2_5");
        quick.push("dis+11_32_av=off:ep=RST:fsr=off:lwlo=on:nm=6:nwc=1.1:sd=5:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_2");
        quick.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_5");
        quick.push("dis+11_4_afr=on:afp=1000:afq=1.4:amm=off:anc=none:lcm=reverse:lma=on:lwlo=on:nm=6:newcnf=on:nwc=1:sd=2:ss=axioms:st=2.0:sp=reverse_arity_3");
        quick.push("ott+1002_128_av=off:bd=off:bs=on:bsr=on:cond=on:fsr=off:nm=6:newcnf=on:nwc=1:sp=reverse_arity:updr=off_82");
        quick.push("lrs+1002_8_add=large:afp=40000:afq=1.0:amm=off:anc=none:cond=on:gs=on:irw=on:nm=16:newcnf=on:nwc=1:stl=30:sos=on:sp=reverse_arity:updr=off_6");
        quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_51");
        quick.push("lrs+1_3:2_aac=none:add=large:anc=all_dependent:bce=on:cond=fast:ep=RST:fsr=off:lma=on:nm=2:nwc=1:stl=30:sos=on:sp=occurrence:urr=on:updr=off:uhcvi=on_5");
        quick.push("ott+1_8_av=off:bd=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:lwlo=on:nwc=1:sos=on_10");
        quick.push("dis+11_5_afp=40000:afq=1.4:anc=none:br=off:cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1:sd=3:ss=axioms:st=2.0:urr=on:updr=off_5");
        quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_2");
        quick.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:stl=30:sos=all:sp=occurrence:urr=on_7");
        quick.push("lrs+1003_2_acc=on:afp=4000:afq=2.0:amm=sco:bd=off:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:nm=64:newcnf=on:nwc=2.5:nicw=on:stl=30:urr=ec_only_7");
        quick.push("ott+4_40_av=off:bce=on:cond=fast:fde=none:nm=0:nwc=1:sos=all:updr=off_2");
        quick.push("dis-3_5_av=off:cond=on:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1:sos=on:sp=reverse_arity_3");
        quick.push("lrs+1011_14_av=off:fsr=off:irw=on:nwc=1:stl=30:sos=on:sp=occurrence:urr=ec_only:updr=off_53");
        quick.push("dis+11_10_av=off:lma=on:nm=64:nwc=5:sp=reverse_arity_3");
        quick.push("lrs+1011_4:1_av=off:fsr=off:irw=on:nwc=1:stl=30:sd=4:ss=axioms:st=1.5:sp=reverse_arity_12");
        quick.push("lrs+10_2:3_afp=1000:afq=1.1:amm=sco:anc=none:er=known:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=2:stl=30:sd=5:ss=axioms:sos=theory:sac=on:sp=occurrence_233");
        quick.push("lrs+1011_3:2_av=off:er=known:lma=on:nm=2:newcnf=on:nwc=2:stl=30:sd=2:ss=axioms:st=3.0:urr=on:updr=off_5");
        quick.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:stl=30:sos=theory:sp=occurrence:urr=ec_only:updr=off_36");
        quick.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:stl=30:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_59");
        quick.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_26");
        quick.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:stl=30:sac=on:sp=occurrence_18");
        quick.push("dis+1010_1_acc=model:add=large:afr=on:amm=off:anc=none:bd=off:bsr=on:ccuc=small_ones:gs=on:gsem=on:nm=16:nwc=2:urr=ec_only:updr=off_198");
        quick.push("dis+11_2_add=large:afr=on:anc=none:gs=on:gsem=on:lwlo=on:nm=16:nwc=1:sd=1:ss=axioms:st=3.0:sos=on_3");
        quick.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_39");
        quick.push("ott+1002_2_av=off:bd=preordered:irw=on:lma=on:nm=64:nwc=10:sp=reverse_arity:updr=off_22");
        quick.push("lrs+10_128_acc=model:afp=100000:afq=2.0:anc=all_dependent:bs=on:bsr=on:cond=fast:er=filter:gs=on:gsem=off:lcm=reverse:lma=on:nm=32:nwc=3:stl=30:sac=on:sp=occurrence:urr=ec_only_70");
        quick.push("ott-3_3_av=off:cond=fast:fde=none:lcm=reverse:nm=6:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:sp=reverse_arity:updr=off:uhcvi=on_17");
        quick.push("lrs+4_5:4_av=off:bd=off:er=filter:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence:updr=off_148");
        quick.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_72");
        quick.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_38");
      }
      else if (atoms > 100) { // 1873
        quick.push("lrs+1011_2_acc=on:add=off:afr=on:afp=40000:afq=2.0:amm=off:bd=off:bs=on:bsr=on:ccuc=small_ones:cond=on:er=filter:fsr=off:gs=on:gsem=on:irw=on:lcm=reverse:nwc=2:nicw=on:stl=30:sac=on:sp=occurrence:urr=on_2");
        quick.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_2");
        quick.push("dis+11_5_av=off:bd=off:bs=unit_only:bsr=on:cond=on:lcm=reverse:nm=0:nwc=1.2_5");
        quick.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_2");
        quick.push("dis+10_1_add=off:afp=4000:afq=1.2:anc=none:br=off:cond=on:gs=on:irw=on:lcm=reverse:nwc=10:sd=10:ss=axioms:sos=theory:sac=on:sp=occurrence:urr=on_2");
        quick.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_6");
        quick.push("dis+1011_4_av=off:cond=on:irw=on:lma=on:nm=2:nwc=1:sos=all:sp=occurrence_5");
        quick.push("dis+1010_3:1_av=off:gsp=input_only:nm=6:nwc=1:sos=all:sp=occurrence_49");
        quick.push("dis+10_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1.3:sp=reverse_arity:updr=off_9");
        quick.push("ott+4_32_av=off:bsr=on:cond=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=4:nwc=1.2:sos=theory_124");
        quick.push("ott+11_4:1_add=large:afp=40000:afq=1.0:anc=none:br=off:cond=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sos=all:sp=reverse_arity:urr=on_2");
        quick.push("dis+1002_8_add=large:afp=100000:afq=1.2:amm=off:bs=on:irw=on:nm=2:newcnf=on:nwc=1.1:sos=on:sp=reverse_arity:urr=ec_only:updr=off_259");
        quick.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:stl=30:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_85");
        quick.push("lrs+1011_2:3_add=large:afr=on:afp=100000:afq=1.2:anc=none:gs=on:gsem=off:irw=on:nm=64:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity_48");
        quick.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_205");
        quick.push("ott-11_12_aac=none:afp=100000:afq=1.2:amm=sco:bs=on:bce=on:cond=fast:er=known:gs=on:gsaa=from_current:gsem=off:irw=on:nm=4:nwc=2:sas=z3:sos=all:sp=occurrence:urr=ec_only:updr=off_253");
        quick.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_145");
        quick.push("lrs+1_8:1_av=off:cond=fast:fde=unused:lcm=predicate:nm=16:nwc=10:stl=60:sp=occurrence:urr=ec_only_308");
        quick.push("dis+1010_3:1_av=off:irw=on:nm=32:nwc=1:sos=all:urr=ec_only:updr=off_158");
        quick.push("dis+1010_7_afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:urr=ec_only:updr=off_204");
      }
      else { // 1917
        quick.push("dis+1010_5_add=large:afp=10000:afq=1.2:amm=off:bs=unit_only:bsr=on:bce=on:cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1.3:nicw=on:sos=all:sac=on:updr=off_34");
        quick.push("dis+1002_2_add=off:afr=on:afp=10000:afq=2.0:amm=off:anc=none:cond=on:er=filter:fsr=off:nm=0:nwc=1.3:sp=occurrence_4");
        quick.push("dis+11_2:1_add=large:afp=4000:afq=1.4:amm=sco:anc=none:fsr=off:nm=16:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_6");
        quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_2");
        quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_8");
        quick.push("lrs+11_20_av=off:bs=unit_only:bsr=on:bce=on:cond=on:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=theory:sp=reverse_arity:uhcvi=on_92");
        quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_2");
        quick.push("dis+11_1_acc=on:afp=1000:afq=1.4:amm=sco:anc=all_dependent:bs=on:ccuc=small_ones:cond=fast:fde=none:gsp=input_only:nm=64:nwc=1:sac=on:urr=ec_only:updr=off:uhcvi=on_105");
        quick.push("lrs+4_3_av=off:br=off:nm=0:newcnf=on:nwc=1:stl=30:sp=occurrence:urr=on_32");
        quick.push("ott+4_32_av=off:bsr=on:cond=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=4:nwc=1.2:sos=theory_197");
        quick.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_98");
        quick.push("lrs+10_24_add=off:afr=on:afp=1000:afq=1.4:anc=none:bs=unit_only:gs=on:gsaa=from_current:gsem=on:lma=on:nm=2:nwc=1.1:stl=60:sac=on:uhcvi=on_364");
        quick.push("ott+11_8:1_acc=model:afp=1000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=16:nwc=1.5:sac=on:sp=occurrence:urr=on_104");
        quick.push("lrs+4_5:4_av=off:bd=off:er=filter:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence:updr=off_230");
        quick.push("dis+2_128_add=large:afp=100000:afq=1.4:amm=sco:anc=none:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sos=theory:sac=on:updr=off_288");
        quick.push("lrs+4_24_av=off:bd=preordered:bsr=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:stl=30:sos=theory:updr=off:uhcvi=on_220");
        quick.push("ott+4_40_av=off:bce=on:cond=fast:fde=none:nm=0:nwc=1:sos=all:updr=off_131");
      }
    }
    else if (prop == 131075) { // 2299
      quick.push("dis+1002_2_add=off:afr=on:afp=10000:afq=2.0:amm=off:anc=none:cond=on:er=filter:fsr=off:nm=0:nwc=1.3:sp=occurrence_2");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_2");
      quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_9");
      quick.push("dis+11_2_add=large:afr=on:anc=none:gs=on:gsem=on:lwlo=on:nm=16:nwc=1:sd=1:ss=axioms:st=3.0:sos=on_2");
      quick.push("dis+11_3_av=off:fsr=off:lcm=predicate:lma=on:nm=4:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:updr=off_4");
      quick.push("ott+11_8:1_acc=model:afp=1000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=16:nwc=1.5:sac=on:sp=occurrence:urr=on_33");
      quick.push("dis+1002_8:1_av=off:br=off:cond=on:irw=on:lma=on:nm=32:nwc=1:sp=occurrence:urr=on_89");
      quick.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:stl=30:sp=occurrence_66");
      quick.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:sos=all_2");
      quick.push("lrs-1_14_add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:br=off:cond=fast:fde=unused:gs=on:lcm=reverse:lma=on:nwc=1:stl=30:sos=on:urr=on:updr=off_14");
      quick.push("lrs+1011_3:2_av=off:er=known:lma=on:nm=2:newcnf=on:nwc=2:stl=30:sd=2:ss=axioms:st=3.0:urr=on:updr=off_65");
      quick.push("lrs+4_1_acc=on:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bs=on:bsr=on:ccuc=first:fsr=off:fde=unused:irw=on:lma=on:nm=0:nwc=1.3:stl=30:sd=10:ss=axioms:st=3.0:sos=all:sp=occurrence:uhcvi=on_3");
      quick.push("dis+1010_3:2_av=off:lma=on:nm=2:newcnf=on:nwc=1:sd=3:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_99");
      quick.push("dis+1002_1_av=off:bsr=on:cond=on:lma=on:nwc=2:sd=3:ss=axioms:st=3.0:updr=off_24");
      quick.push("dis+1010_2_afr=on:afp=1000:afq=1.1:amm=off:anc=none:bs=unit_only:bce=on:cond=fast:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=5:nicw=on:sas=z3:sac=on:urr=ec_only:updr=off:uhcvi=on_92");
      quick.push("dis+1011_5:1_acc=on:add=off:afp=1000:afq=2.0:anc=none:bsr=on:ccuc=small_ones:cond=on:gsp=input_only:nm=0:nwc=1:sos=all:sp=occurrence_10");
      quick.push("dis+1002_7_acc=on:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:ccuc=first:fsr=off:gsp=input_only:gs=on:gsem=on:nm=6:nwc=1.1:nicw=on:sos=on:sac=on:sp=occurrence:urr=ec_only_217");
      quick.push("ins+1010_3_av=off:bd=off:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.3:igwr=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=1:ss=axioms:sos=on:sp=occurrence:updr=off_8");
      quick.push("lrs+1010_2_add=large:afp=4000:afq=2.0:amm=off:bd=off:bs=unit_only:bsr=on:br=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=32:nwc=1.1:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_7");
      quick.push("dis-1_2:3_av=off:cond=on:fsr=off:irw=on:lma=on:nwc=3:sd=3:ss=axioms:st=3.0:sp=occurrence:updr=off_9");
      quick.push("lrs+1002_2:1_aac=none:afr=on:afp=1000:afq=1.2:anc=all:bd=preordered:bsr=on:cond=fast:gsp=input_only:gs=on:nm=0:nwc=2.5:nicw=on:sas=z3:stl=30:sd=4:ss=axioms:st=2.0:sos=on:sac=on:urr=on:updr=off:uhcvi=on_22");
      quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_3");
      quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_97");
      quick.push("lrs+11_3_afr=on:afp=10000:afq=1.0:cond=fast:fsr=off:fde=none:gs=on:gsem=off:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:stl=30:sd=10:ss=axioms:st=2.0:sos=all:sp=reverse_arity:urr=on:uhcvi=on_23");
      quick.push("dis+1010_1_acc=model:add=large:afr=on:amm=off:anc=none:bd=off:bsr=on:ccuc=small_ones:gs=on:gsem=on:nm=16:nwc=2:urr=ec_only:updr=off_183");
      quick.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_171");
      quick.push("ott+1011_14_add=large:afr=on:afp=10000:afq=1.4:anc=none:bs=unit_only:cond=on:fde=unused:irw=on:nm=2:nwc=1.5:uhcvi=on_2");
      quick.push("ott+1010_8:1_add=off:afp=4000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=4:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off:uhcvi=on_239");
      quick.push("lrs+1010_3:2_afr=on:afp=100000:afq=1.1:anc=none:gsp=input_only:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1.7:stl=30:sac=on:sp=occurrence_95");
      quick.push("ott-3_3_av=off:cond=fast:fde=none:lcm=reverse:nm=6:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:sp=reverse_arity:updr=off:uhcvi=on_88");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_11");
      quick.push("lrs+1011_2:3_add=large:afr=on:afp=100000:afq=1.2:anc=none:gs=on:gsem=off:irw=on:nm=64:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity_4");
      quick.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:stl=30:sos=theory:sp=occurrence:urr=ec_only:updr=off_223");
      quick.push("lrs+1010_4:1_av=off:fsr=off:gs=on:gsem=on:lma=on:nm=4:newcnf=on:nwc=1.5:stl=30:sd=3:ss=axioms:st=5.0:sp=occurrence:updr=off_83");
      quick.push("lrs+1010_5_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:bd=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=4:sas=z3:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_298");
    }
    else if (prop == 131087) {
      if (atoms > 19000) { // 1939
        quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_60");
        quick.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_11");
        quick.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_20");
        quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_10");
        quick.push("lrs+11_8:1_av=off:bd=off:bs=unit_only:gs=on:gsem=on:lma=on:lwlo=on:nm=0:nwc=1:stl=30:sd=1:ss=axioms:sos=all:urr=on:updr=off_3");
        quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_16");
        quick.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_15");
        quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_7");
        quick.push("dis-1_4_aac=none:acc=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sac=on:sp=occurrence:updr=off_26");
        quick.push("dis-10_3:2_aac=none:afp=1000:afq=1.1:cond=on:fsr=off:lcm=reverse:lwlo=on:nm=16:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity:updr=off_27");
        quick.push("lrs+1002_1_av=off:er=filter:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:st=1.5:sos=on_13");
        quick.push("ott+2_8:1_aac=none:acc=on:add=large:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:fsr=off:lcm=predicate:lma=on:nm=2:nwc=1.7:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_25");
        quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_26");
        quick.push("dis+10_5_av=off:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_6");
        quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_21");
        quick.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:stl=30:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_12");
        quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_15");
        quick.push("lrs+1011_1_av=off:cond=on:gs=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity:updr=off_118");
        quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_76");
        quick.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:stl=30:sd=2:ss=axioms:sos=all:urr=ec_only_13");
        quick.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_17");
        quick.push("dis+1011_3_afp=100000:afq=1.1:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:irw=on:nm=6:nwc=1:sas=z3:sd=2:ss=axioms:sos=on:sac=on:sp=reverse_arity:updr=off_7");
        quick.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:stl=30:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_50");
        quick.push("lrs+11_1_av=off:bsr=on:gsp=input_only:gs=on:lcm=predicate:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_17");
        quick.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=2.0:sac=on:urr=on_115");
        quick.push("lrs+11_4:1_av=off:bd=off:br=off:cond=fast:fde=unused:irw=on:lcm=reverse:lma=on:lwlo=on:nm=4:nwc=1:stl=60:sd=3:ss=axioms:sos=all:urr=on_61");
        quick.push("lrs+4_1_acc=on:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bs=on:bsr=on:ccuc=first:fsr=off:fde=unused:irw=on:lma=on:nm=0:nwc=1.3:stl=30:sd=10:ss=axioms:st=3.0:sos=all:sp=occurrence:uhcvi=on_26");
        quick.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_8");
        quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_11");
        quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_225");
        quick.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_122");
        quick.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:sos=all_229");
        quick.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_21");
        quick.push("ott+1010_5:4_av=off:bd=off:fde=none:irw=on:lma=on:nm=32:nwc=2.5:sd=2:ss=axioms:st=3.0:urr=on_190");
        quick.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_159");
        quick.push("lrs+4_4:1_acc=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:ccuc=first:cond=on:fsr=off:irw=on:lcm=predicate:lma=on:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all_161");
      }
      else { // 2242
        quick.push("lrs+11_2:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=16:nwc=1:stl=30:sd=1:ss=axioms:st=1.2:sp=reverse_arity_11");
        quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_4");
        quick.push("dis+1_3:2_acc=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:ccuc=small_ones:cond=on:lma=on:nm=64:nwc=1.3:sos=all:urr=on_3");
        quick.push("ott+1_5:1_acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:lcm=predicate:nm=16:nwc=1.1:sd=1:ss=axioms:st=3.0:sos=on:sac=on:updr=off_18");
        quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_6");
        quick.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_9");
        quick.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_15");
        quick.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:stl=30:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_8");
        quick.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:stl=30:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_6");
        quick.push("dis+1011_8:1_aac=none:acc=on:afp=1000:afq=1.4:amm=off:anc=all:bs=unit_only:bce=on:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:lma=on:nm=16:nwc=2.5:sd=4:ss=axioms:st=1.5:sos=all:uhcvi=on_65");
        quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_42");
        quick.push("dis+1_5:1_aac=none:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:br=off:gsp=input_only:gs=on:gsem=on:lcm=reverse:nm=6:nwc=1:nicw=on:sas=z3:sos=on:urr=on_9");
        quick.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:sos=all:sp=occurrence_37");
        quick.push("dis+4_8:1_add=large:afp=100000:afq=1.4:ep=RST:fde=unused:gsp=input_only:lcm=predicate:nwc=1:sos=all:sp=occurrence:updr=off:uhcvi=on_33");
        quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_41");
        quick.push("lrs-2_3:1_add=off:afr=on:afp=1000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bs=unit_only:bsr=on:cond=on:fde=unused:gs=on:gsem=on:irw=on:lcm=reverse:nm=32:nwc=1.7:sas=z3:stl=30:sos=all:sac=on_28");
        quick.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:stl=30:sos=all:sp=occurrence:urr=on_13");
        quick.push("dis+1010_2:3_afr=on:afp=40000:afq=1.4:amm=off:anc=none:lma=on:nm=16:nwc=1:sp=occurrence:updr=off:uhcvi=on_34");
        quick.push("ott+1011_3:1_aac=none:add=off:afp=1000:afq=1.2:amm=sco:bd=off:bs=on:bsr=on:cond=on:gsp=input_only:gs=on:lma=on:nm=6:newcnf=on:nwc=1.3:nicw=on:sd=3:ss=axioms:st=2.0:sp=reverse_arity:urr=ec_only:updr=off_58");
        quick.push("dis+11_2_add=large:afr=on:anc=none:gs=on:gsem=on:lwlo=on:nm=16:nwc=1:sd=1:ss=axioms:st=3.0:sos=on_4");
        quick.push("ott+2_8:1_aac=none:acc=on:add=large:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:fsr=off:lcm=predicate:lma=on:nm=2:nwc=1.7:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_8");
        quick.push("dis+10_1_add=off:afp=4000:afq=1.2:anc=none:br=off:cond=on:gs=on:irw=on:lcm=reverse:nwc=10:sd=10:ss=axioms:sos=theory:sac=on:sp=occurrence:urr=on_4");
        quick.push("lrs+1002_3_aac=none:acc=on:add=off:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1.1:nicw=on:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:updr=off_19");
        quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_8");
        quick.push("lrs+1002_8:1_add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:er=known:lwlo=on:nm=0:nwc=1.2:stl=30:sd=1:ss=axioms:sp=occurrence:updr=off_51");
        quick.push("ott+10_1_av=off:bd=off:bsr=on:cond=on:fsr=off:irw=on:nm=2:nwc=1:sd=3:ss=axioms:sos=on:sp=occurrence:urr=on_26");
        quick.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_5");
        quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_3");
        quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_6");
        quick.push("lrs+11_3_av=off:bce=on:cond=fast:ep=R:lcm=reverse:lma=on:newcnf=on:nwc=1.3:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence:uhcvi=on_90");
        quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_11");
        quick.push("lrs+11_1_av=off:bsr=on:gsp=input_only:gs=on:lcm=predicate:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_236");
        quick.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_53");
        quick.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_6");
        quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_254");
        quick.push("ins+1010_3_av=off:bd=off:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.3:igwr=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=1:ss=axioms:sos=on:sp=occurrence:updr=off_162");
        quick.push("dis+1004_1_aac=none:acc=on:afp=40000:afq=1.2:anc=none:cond=on:fde=unused:gs=on:gsem=off:irw=on:nm=32:nwc=2:sd=1:ss=axioms:sos=theory:sp=reverse_arity:urr=ec_only_17");
        quick.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_7");
        quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_49");
        quick.push("dis+4_2_av=off:bs=on:fsr=off:gsp=input_only:newcnf=on:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sp=reverse_arity:urr=ec_only:updr=off_127");
        quick.push("dis+11_1_acc=on:afp=1000:afq=1.4:amm=sco:anc=all_dependent:bs=on:ccuc=small_ones:cond=fast:fde=none:gsp=input_only:nm=64:nwc=1:sac=on:urr=ec_only:updr=off:uhcvi=on_138");
        quick.push("lrs+10_2:3_afp=1000:afq=1.1:amm=sco:anc=none:er=known:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=2:stl=30:sd=5:ss=axioms:sos=theory:sac=on:sp=occurrence_121");
        quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_41");
        quick.push("dis+10_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1.3:sp=reverse_arity:updr=off_1");
        quick.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_112");
        quick.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_243");
      }
    }
    else if (prop > 131076) { // 1460
      quick.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_11");
      quick.push("ott+1002_8:1_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:fsr=off:gs=on:gsem=off:nm=32:nwc=10:sas=z3:sp=occurrence:urr=on:updr=off_17");
      quick.push("lrs+1011_7_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:cond=on:er=known:fsr=off:lma=on:nm=4:nwc=2.5:stl=30:sp=reverse_arity:updr=off_2");
      quick.push("ott+11_16_av=off:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.3:sas=z3:sp=reverse_arity_16");
      quick.push("dis+10_4_av=off:bsr=on:cond=fast:er=filter:fde=none:gsp=input_only:lcm=reverse:lma=on:nwc=4:sp=occurrence:urr=on_8");
      quick.push("dis+1011_5_add=off:afr=on:afp=10000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:nm=32:nwc=1:sas=z3:sd=3:ss=axioms:st=2.0:sp=occurrence:updr=off_2");
      quick.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_38");
      quick.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_4");
      quick.push("lrs+10_4:1_add=large:afp=100000:afq=1.1:anc=none:ep=RST:fde=unused:gsp=input_only:nm=6:newcnf=on:nwc=1:stl=30:sos=all:sac=on:sp=reverse_arity:urr=ec_only_22");
      quick.push("dis+2_2:1_aac=none:afp=100000:afq=1.1:amm=sco:anc=none:bsr=on:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=on_6");
      quick.push("lrs+10_8:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lcm=predicate:lwlo=on:nm=64:nwc=1:stl=30:sos=all:updr=off_78");
      quick.push("lrs-11_12_av=off:nm=32:nwc=1.3:stl=30:sd=3:ss=axioms:sos=all_2");
      quick.push("lrs+11_1_add=large:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:gs=on:gsem=on:irw=on:nm=32:newcnf=on:nwc=2.5:nicw=on:stl=30:sd=3:ss=axioms:sos=all:urr=on_34");
      quick.push("lrs+10_3:1_av=off:bsr=on:cond=on:er=known:gs=on:lcm=reverse:nm=32:nwc=4:stl=30:sp=occurrence:urr=on:updr=off_73");
      quick.push("dis+11_40_aac=none:add=large:afp=10000:afq=1.2:amm=off:cond=fast:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:irw=on:nm=6:newcnf=on:nwc=1.2:sd=7:ss=axioms:st=1.5:sos=on:sp=reverse_arity:urr=on:uhcvi=on_4");
      quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_111");
      quick.push("lrs+10_8:1_av=off:bs=unit_only:cond=on:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:nwc=1.2:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_12");
      quick.push("lrs+11_4:1_aac=none:add=large:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:er=filter:fsr=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:newcnf=on:nwc=1.1:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_3");
      quick.push("dis+1010_3:2_av=off:gsp=input_only:nm=2:nwc=1:sp=reverse_arity:urr=ec_only_29");
      quick.push("dis+1002_6_add=large:afp=40000:afq=2.0:bsr=on:cond=on:irw=on:lma=on:nm=2:nwc=2.5:nicw=on:sp=reverse_arity:updr=off_10");
      quick.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:stl=30:sos=all:sp=occurrence:urr=on_8");
      quick.push("ott+1_5_afp=40000:afq=1.0:anc=all:fde=none:gs=on:irw=on:lma=on:nm=32:nwc=2:sos=all:sac=on:sp=occurrence:urr=ec_only:uhcvi=on_34");
      quick.push("ott+2_2_afp=10000:afq=1.4:amm=off:anc=none:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:nm=32:nwc=1.5:sos=on:sp=reverse_arity_18");
      quick.push("dis+1_2:3_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:nm=64:newcnf=on:nwc=1_3");
      quick.push("lrs+1010_3:2_afr=on:afp=100000:afq=1.1:anc=none:gsp=input_only:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1.7:stl=30:sac=on:sp=occurrence_92");
      quick.push("ott+11_3_afp=1000:afq=2.0:anc=none:fsr=off:irw=on:nwc=1.7:ss=axioms:st=1.5:sac=on:updr=off_3");
      quick.push("lrs+1010_3_av=off:fsr=off:gs=on:gsem=off:nm=2:newcnf=on:nwc=2:stl=30:sp=reverse_arity:urr=on:updr=off_9");
      quick.push("dis+11_4:1_afp=40000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:sos=on:sp=occurrence:urr=on:updr=off_83");
      quick.push("dis+1010_3:1_av=off:irw=on:nm=32:nwc=1:sos=all:urr=ec_only:updr=off_77");
      quick.push("lrs+11_1_av=off:bsr=on:gsp=input_only:gs=on:lcm=predicate:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_245");
      quick.push("dis+1011_10_av=off:cond=on:lma=on:nm=0:newcnf=on:nwc=1:sos=on:sp=occurrence:updr=off_4");
      quick.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=2.0:sac=on:urr=on_127");
      quick.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_275");
    }
    else { // 1794
      quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_11");
      quick.push("lrs+1002_8_add=large:afp=40000:afq=1.0:amm=off:anc=none:cond=on:gs=on:irw=on:nm=16:newcnf=on:nwc=1:stl=30:sos=on:sp=reverse_arity:updr=off_2");
      quick.push("lrs+10_50_add=large:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gsp=input_only:gs=on:irw=on:lma=on:nm=64:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on_8");
      quick.push("dis+1002_3:1_acc=model:afr=on:afp=40000:afq=1.1:anc=none:ccuc=first:fsr=off:gsp=input_only:irw=on:nm=16:nwc=1:sos=all_5");
      quick.push("ott+11_8_amm=off:anc=none:bsr=on:cond=on:irw=on:nm=2:nwc=1.1:ss=axioms:st=2.0:sac=on_36");
      quick.push("ott+1_128_add=large:afr=on:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.1:nicw=on:sas=z3:sac=on:sp=reverse_arity_44");
      quick.push("ott+11_128_afp=4000:afq=1.1:anc=none:bs=unit_only:cond=on:gsp=input_only:lma=on:nm=4:nwc=1.5:sos=theory:sp=occurrence:updr=off_1");
      quick.push("lrs+1010_2:1_aac=none:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:gs=on:gsem=off:irw=on:nm=16:nwc=3:stl=30_7");
      quick.push("lrs+1011_10_aac=none:acc=model:add=large:afp=40000:afq=2.0:anc=none:bd=off:bsr=on:fsr=off:gs=on:gsem=off:irw=on:lcm=reverse:lwlo=on:nm=64:nwc=3:nicw=on:stl=30_38");
      quick.push("lrs+4_4:1_acc=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:ccuc=first:cond=on:fsr=off:irw=on:lcm=predicate:lma=on:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all_2");
      quick.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_2");
      quick.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_9");
      quick.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_88");
      quick.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:stl=30:sp=occurrence:urr=on_129");
      quick.push("lrs+1_7_av=off:irw=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=1:stl=30:sos=all:sp=occurrence:updr=off_81");
      quick.push("lrs+1002_16_av=off:cond=on:nwc=3:stl=30_83");
      quick.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_180");
      quick.push("ott+1002_7_acc=on:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bs=unit_only:gsp=input_only:gs=on:nm=2:nwc=10:sos=theory:sp=reverse_arity:urr=on:updr=off_45");
      quick.push("lrs+1011_5_afr=on:afp=100000:afq=1.0:amm=off:anc=none:cond=on:lma=on:nm=6:nwc=1:sas=z3:stl=30:sac=on:urr=on_12");
      quick.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_169");
      quick.push("ott+1010_8:1_add=off:afp=4000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=4:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off:uhcvi=on_4");
      quick.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:stl=30:sac=on:urr=on_41");
      quick.push("lrs+1011_3:2_aac=none:afp=10000:afq=1.2:amm=off:anc=all:cond=on:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_5");
      quick.push("lrs+4_2_av=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:stl=30:sos=on:urr=on_159");
      quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_3");
      quick.push("ott+4_40_av=off:bce=on:cond=fast:fde=none:nm=0:nwc=1:sos=all:updr=off_197");
      quick.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_143");
      quick.push("dis+3_24_av=off:bd=off:bs=unit_only:fsr=off:fde=unused:gs=on:irw=on:lma=on:nm=0:nwc=1.1:sos=on:uhcvi=on_180");
      quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_10");
      quick.push("lrs+11_3:2_add=large:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:er=filter:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:sas=z3:stl=30:sp=occurrence:urr=on:updr=off_100");
    }
    break;

  case Property::FNE:
    if (atoms < 1700) {
      quick.push("lrs+1010_5:4_afp=100000:afq=1.2:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=64:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_2");
      quick.push("dis+1011_10_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=64:nwc=4:sac=on:sp=occurrence_2");
      quick.push("dis-10_28_aac=none:add=large:afr=on:afp=4000:afq=1.2:bd=preordered:bsr=on:bce=on:fde=none:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=6:newcnf=on:nwc=1:sas=z3:tha=off:thi=overlap:uwa=all:urr=on_2");
      quick.push("dis+11_2_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:uwa=all:updr=off_2");
      quick.push("dis+1011_4:1_anc=none:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence_2");
      quick.push("ott-11_24_afr=on:afp=4000:afq=1.0:amm=off:anc=none:bs=unit_only:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=16:nwc=1.3:nicw=on:sas=z3:sac=on:tha=off:thi=strong:uwa=one_side_constant:urr=on_2");
      quick.push("dis+1_5:1_add=off:afp=40000:afq=1.2:anc=none:bd=off:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=4:sas=z3:updr=off_2");
      quick.push("lrs+1010_4:1_aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=on:gs=on:gsem=on:irw=on:nm=0:nwc=2.5:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_2");
      quick.push("ott+11_1_add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:urr=ec_only_2");
      quick.push("dis+10_128_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=2:nwc=1:sp=reverse_arity_3");
      quick.push("dis+11_5_av=off:cond=on:fsr=off:ile=on:lwlo=on:nm=64:nwc=3:sp=reverse_arity:updr=off_2");
      quick.push("lrs-11_4:1_afp=4000:afq=2.0:anc=none:br=off:gs=on:lwlo=on:nm=64:nwc=3:stl=30:urr=on_2");
      quick.push("lrs+4_32_add=large:afp=10000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lcm=predicate:lma=on:nm=2:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_7");
      quick.push("dis+11_3_add=large:afp=100000:afq=1.4:amm=off:anc=none:fsr=off:gs=on:ile=on:irw=on:lma=on:nm=32:nwc=1:tha=off:updr=off_2");
      quick.push("ins+11_32_av=off:igbrr=0.4:igrr=1/64:igrpq=1.05:igwr=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:updr=off_55");
      quick.push("dis+11_24_afp=40000:afq=1.1:amm=sco:anc=none:bs=on:gs=on:gsem=off:lcm=predicate:lma=on:nm=2:nwc=1:sos=on:sac=on:updr=off_91");
      quick.push("dis+10_50_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:gs=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:thf=on:updr=off_2");
      quick.push("lrs+1011_1024_add=large:afp=4000:afq=1.1:anc=none:br=off:fsr=off:gsp=input_only:lma=on:nwc=1:stl=30:sos=on:urr=on_60");
      quick.push("dis+11_3_afr=on:afp=40000:afq=2.0:anc=none:fsr=off:gs=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:urr=on_11");
      quick.push("ott+1_40_av=off:bs=unit_only:bsr=on:br=off:fsr=off:lma=on:nm=64:newcnf=on:nwc=1.5:sp=occurrence:urr=on:updr=off_81");
      quick.push("lrs+1_1_av=off:bsr=on:br=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sp=occurrence:urr=on:updr=off_152");
      quick.push("dis-10_4:1_aac=none:add=off:afp=1000:afq=1.4:amm=off:anc=none:cond=fast:ep=RSTC:gs=on:gsaa=from_current:gsem=on:inw=on:lma=on:nm=64:nwc=4:sas=z3:tha=off:thi=strong:uwa=interpreted_only:updr=off:uhcvi=on_2");
      quick.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_73");
      quick.push("lrs+1011_2:1_add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:thf=on:urr=on:updr=off_81");
      quick.push("ott+11_5:1_afp=100000:afq=1.1:br=off:gs=on:nm=64:nwc=1:sos=all:urr=on:updr=off_287");
      quick.push("lrs+1003_2:3_afr=on:afp=100000:afq=1.1:amm=off:anc=none:bs=on:fde=unused:gs=on:inw=on:nm=0:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:sp=occurrence:tha=off:updr=off:uhcvi=on_2");
      quick.push("dis+1011_5_add=off:afp=4000:afq=2.0:anc=none:fsr=off:inw=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:sp=reverse_arity_182");
      quick.push("lrs+10_4_add=off:afp=100000:afq=2.0:amm=sco:anc=none:nm=64:nwc=1:stl=150:sp=occurrence:updr=off_733");
      quick.push("dis+1_8_afp=4000:afq=1.1:amm=sco:gsp=input_only:nm=64:newcnf=on:nwc=4:sac=on:sp=occurrence:updr=off_191");
    }
    else {
      quick.push("dis+11_3_afr=on:afp=4000:afq=1.4:anc=none:cond=on:fsr=off:gs=on:lcm=reverse:nm=64:nwc=1:sos=on:sp=reverse_arity_3");
      quick.push("dis+1011_10_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=64:nwc=4:sac=on:sp=occurrence_75");
      quick.push("lrs+1011_1024_add=large:afp=4000:afq=1.1:anc=none:br=off:fsr=off:gsp=input_only:lma=on:nwc=1:stl=30:sos=on:urr=on_200");
      quick.push("lrs+4_32_add=large:afp=10000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lcm=predicate:lma=on:nm=2:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_11");
      quick.push("dis+11_24_afp=40000:afq=1.1:amm=sco:anc=none:bs=on:gs=on:gsem=off:lcm=predicate:lma=on:nm=2:nwc=1:sos=on:sac=on:updr=off_52");
      quick.push("lrs+1011_40_add=off:afr=on:afp=4000:afq=1.2:amm=sco:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:stl=60:sos=all:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_388");
      quick.push("ott+11_3:2_afp=40000:afq=1.0:amm=sco:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=full_model:lcm=reverse:nm=32:newcnf=on:nwc=5:nicw=on:sd=3:ss=axioms:sac=on:urr=on:updr=off_1019");
    }
    break;

  case Property::NEQ:
    if (prop == 1) {
      quick.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_7");
      quick.push("lrs+11_2_afr=on:afp=100000:afq=1.4:amm=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:stl=30:sos=on:updr=off_3");
      quick.push("dis+10_10_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:gs=on:gsem=off:nwc=1:sas=z3:sd=3:ss=axioms:st=2.0:sp=occurrence:updr=off_2");
      quick.push("dis+1010_3:1_aac=none:add=large:afp=100000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:gs=on:lma=on:nwc=1:sos=all:sp=occurrence_18");
      quick.push("lrs+11_14_av=off:bd=off:bs=unit_only:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sos=on:sp=reverse_arity:urr=on:updr=off_46");
      quick.push("lrs+10_3_av=off:bd=off:cond=on:gs=on:gsem=off:irw=on:lwlo=on:nwc=1.2:stl=30:sp=reverse_arity:updr=off_72");
      quick.push("lrs+4_2_av=off:bs=on:er=known:gs=on:irw=on:lwlo=on:nwc=10:stl=30:sp=occurrence_243");
      quick.push("dis+4_5:1_av=off:nwc=1:sos=all:sp=reverse_arity:urr=on:updr=off_53");
      quick.push("dis-10_3:1_add=large:afr=on:afp=1000:afq=2.0:anc=none:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nwc=1:sas=z3:sd=2:ss=axioms:st=2.0:sos=all:sac=on:sp=reverse_arity:urr=on:uhcvi=on_84");
      quick.push("dis+1011_2:1_av=off:lma=on:nwc=1:sd=3:ss=axioms:st=5.0:sp=occurrence:urr=ec_only_30");
      quick.push("lrs+10_12_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=off:cond=on:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=1.3:sas=z3:stl=30:sac=on_71");
      quick.push("dis+1011_12_av=off:bd=off:bs=unit_only:fsr=off:lma=on:nwc=1:urr=ec_only:updr=off_18");
      quick.push("dis+11_5_afr=on:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:urr=ec_only:updr=off_6");
      quick.push("dis+11_4_aac=none:add=large:afp=100000:afq=1.0:amm=off:anc=none:bsr=on:cond=on:fsr=off:gs=on:lcm=reverse:lma=on:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=ec_only_3");
      quick.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bce=on:fde=unused:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_163");
      quick.push("dis-11_64_av=off:bd=off:bs=on:cond=on:fsr=off:nwc=1:sd=1:ss=axioms:urr=ec_only:updr=off_99");
      quick.push("lrs+1011_2:1_av=off:bce=on:cond=fast:fsr=off:fde=none:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:st=3.0:sp=occurrence:urr=ec_only:updr=off_144");
      quick.push("dis+1011_1_add=off:afp=100000:afq=1.4:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:nwc=1:nicw=on:sac=on:sp=occurrence:urr=on_77");
      quick.push("lrs+11_10_aac=none:acc=on:afp=4000:afq=1.2:amm=sco:anc=none:ccuc=first:fsr=off:irw=on:nwc=2:nicw=on:stl=30:sd=5:ss=axioms:st=1.2:sos=theory:urr=ec_only:updr=off_155");
      quick.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:stl=30:updr=off_2");
      quick.push("ott+10_4:1_av=off:bd=preordered:cond=fast:fde=unused:irw=on:lcm=reverse:nwc=3:sd=2:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_41");
      quick.push("lrs+1011_3:1_av=off:bs=unit_only:bsr=on:er=known:fsr=off:fde=unused:irw=on:lcm=reverse:lwlo=on:nwc=1.7:stl=30:sos=on:sp=occurrence:updr=off_229");
      quick.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_68");
      quick.push("lrs+1_20_add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=z3:stl=30:sd=3:ss=axioms:st=1.2:sp=occurrence:updr=off_136");
      quick.push("dis+10_3_acc=on:afr=on:afp=1000:afq=1.0:amm=sco:bs=unit_only:ccuc=first:fsr=off:irw=on:lcm=reverse:lma=on:nwc=1.5:updr=off_63");
      quick.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=off:anc=none:cond=on:irw=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0:sos=all:sp=occurrence_205");
      quick.push("lrs+11_5_av=off:cond=on:gs=on:gsem=on:irw=on:nwc=1:stl=30:sd=1:ss=axioms:sos=on:sp=occurrence:urr=on_4");
      quick.push("lrs-2_2:1_afr=on:afp=1000:afq=2.0:anc=none:bd=off:bce=on:cond=fast:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1.5:stl=30:sac=on:sp=reverse_arity:uhcvi=on_183");
    }
    else if (prop == 3) {
      quick.push("lrs+1_2:1_av=off:fsr=off:lma=on:nwc=1:stl=30:sd=7:ss=axioms:st=1.2:sos=on:urr=ec_only_12");
      quick.push("dis+11_5_afr=on:afp=1000:afq=1.0:amm=off:anc=none:irw=on:lcm=reverse:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sp=occurrence_2");
      quick.push("lrs+2_3:2_aac=none:acc=model:add=off:afr=on:afp=10000:afq=1.4:anc=none:bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=off:lcm=reverse:lma=on:nwc=3:stl=30:sd=3:ss=axioms:st=2.0:sac=on:urr=on_29");
      quick.push("dis-11_4:1_amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nwc=10:sac=on:sp=reverse_arity:urr=on_14");
      quick.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=off:anc=none:cond=on:irw=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0:sos=all:sp=occurrence_3");
      quick.push("dis+1011_1_add=off:afp=100000:afq=1.4:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:nwc=1:nicw=on:sac=on:sp=occurrence:urr=on_6");
      quick.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_5");
      quick.push("lrs+1011_2:1_av=off:bsr=on:cond=on:nwc=1:sas=z3:stl=30:sd=1:ss=axioms:st=1.2:sp=reverse_arity_8");
      quick.push("dis+1010_1024_afr=on:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:irw=on:lwlo=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0_3");
      quick.push("lrs+1010_3:1_av=off:cond=on:nwc=5:stl=30:sp=reverse_arity_18");
      quick.push("lrs+1011_4:1_av=off:cond=on:irw=on:lma=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_124");
      quick.push("lrs+1_3_av=off:fsr=off:lma=on:nwc=1:stl=30:sd=1:ss=axioms:sp=occurrence:urr=on_22");
      quick.push("lrs+1002_3:1_av=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity_4");
      quick.push("lrs+1011_2:1_av=off:cond=on:er=known:gs=on:gsem=on:lwlo=on:nwc=1.3:stl=30:updr=off:uhcvi=on_180");
      quick.push("dis+1011_3_av=off:nwc=1:sos=all:sp=reverse_arity_67");
      quick.push("dis-11_5:1_acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=first:cond=on:gs=on:gsem=off:nwc=1:sd=3:ss=axioms:st=1.2:sos=all_76");
      quick.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_168");
      quick.push("lrs+10_5:1_add=large:afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_5");
      quick.push("lrs+1_32_av=off:bd=off:br=off:gs=on:gsem=on:irw=on:nwc=1:stl=30:sd=1:ss=axioms:sp=occurrence:urr=on:updr=off_9");
      quick.push("lrs+1010_8:1_av=off:bs=unit_only:br=off:cond=on:fsr=off:irw=on:nwc=1.3:stl=30:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=on:updr=off_62");
      quick.push("lrs+1011_5:4_av=off:bd=off:fsr=off:gs=on:nwc=1.3:stl=30:urr=ec_only:updr=off_91");
      quick.push("dis+11_2_av=off:gs=on:gsem=on:irw=on:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=reverse_arity_6");
      quick.push("lrs+1011_40_add=off:afp=1000:afq=2.0:anc=none:bs=unit_only:fsr=off:irw=on:nwc=1:sas=z3:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_86");
      quick.push("lrs-11_3_aac=none:acc=on:afr=on:afp=10000:afq=1.1:bd=off:bs=unit_only:ccuc=first:irw=on:lcm=predicate:lma=on:nwc=1.5:stl=30:sos=all:sac=on:sp=occurrence:updr=off_111");
      quick.push("dis+1011_2:1_av=off:lma=on:nwc=1:sd=3:ss=axioms:st=5.0:sp=occurrence:urr=ec_only_154");
      quick.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:stl=30:updr=off_287");
      quick.push("lrs+1003_3_av=off:bd=off:fde=none:gs=on:lma=on:nwc=1:stl=30:sd=7:ss=axioms:st=1.2:sos=all:sp=reverse_arity:updr=off:uhcvi=on_103");
      quick.push("dis+2_5:4_add=large:afp=4000:afq=1.2:anc=all:bce=on:cond=fast:fde=none:lma=on:nwc=10:sd=1:ss=axioms:st=1.5:sac=on_9");
      quick.push("lrs+1004_20_av=off:cond=on:er=filter:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:sos=on:urr=ec_only_57");
      quick.push("dis+11_3_av=off:bd=off:bsr=on:bce=on:gsp=input_only:gs=on:gsem=on:lma=on:nwc=2.5:sp=reverse_arity:urr=ec_only:uhcvi=on_205");
      quick.push("dis+1002_5:1_av=off:cond=fast:fsr=off:fde=none:lma=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:urr=on:updr=off_19");
      quick.push("lrs-1_14_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=ec_only:updr=off_175");
      quick.push("lrs+1011_2:1_av=off:bce=on:cond=fast:fsr=off:fde=none:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:st=3.0:sp=occurrence:urr=ec_only:updr=off_102");
      quick.push("ins+11_3_av=off:bd=off:igbrr=0.6:igrr=1/8:igrp=700:igrpq=2.0:igs=1:igwr=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:updr=off_3");
    }
    else if (prop == 131079) {
      quick.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_13");
      quick.push("lrs+1011_2_acc=on:add=off:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:er=known:nwc=4:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off_13");
      quick.push("dis+11_12_av=off:bsr=on:cond=on:lcm=predicate:lma=on:nwc=5:sp=reverse_arity:updr=off_45");
      quick.push("dis+11_3_afp=100000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=10:sac=on_17");
      quick.push("lrs+4_2_av=off:bs=on:er=known:gs=on:irw=on:lwlo=on:nwc=10:stl=30:sp=occurrence_100");
      quick.push("lrs+4_1_av=off:bd=off:bsr=on:bce=on:fsr=off:irw=on:lcm=predicate:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on:uhcvi=on_39");
      quick.push("dis+10_3_av=off:cond=on:irw=on:lma=on:nwc=1.7:sp=occurrence:updr=off_12");
      quick.push("lrs+1011_2:1_av=off:cond=on:lwlo=on:nwc=1.5:stl=30_24");
      quick.push("ott+1011_2:3_av=off:cond=fast:er=filter:fde=unused:gsp=input_only:irw=on:nwc=3:sp=occurrence:updr=off:uhcvi=on_4");
      quick.push("lrs+1011_5_av=off:cond=on:er=filter:gs=on:nwc=1.7:stl=30:updr=off_17");
      quick.push("lrs+11_5_add=large:afr=on:afp=40000:afq=1.2:cond=on:er=known:gs=on:gsem=off:nwc=1.5:stl=30:sp=occurrence:updr=off_65");
      quick.push("dis+1011_3_av=off:nwc=1:sos=all:sp=reverse_arity_114");
      quick.push("lrs+11_1_av=off:fsr=off:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=reverse_arity:urr=on:updr=off_44");
      quick.push("lrs+10_2:1_av=off:bsr=on:cond=on:er=known:irw=on:lcm=predicate:nwc=4:stl=30:sp=occurrence_5");
      quick.push("lrs+4_5_av=off:bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:nwc=2.5:stl=30:sp=occurrence:updr=off_139");
      quick.push("lrs+1011_10_av=off:bs=unit_only:bsr=on:er=filter:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nwc=1.2:stl=30:sp=reverse_arity:updr=off_105");
      quick.push("lrs+1011_2_av=off:bd=preordered:bs=unit_only:cond=fast:fsr=off:fde=unused:irw=on:lma=on:lwlo=on:nwc=1.2:stl=30:sp=occurrence:uhcvi=on_90");
      quick.push("lrs+1011_3:1_av=off:bs=unit_only:bsr=on:er=known:fsr=off:fde=unused:irw=on:lcm=reverse:lwlo=on:nwc=1.7:stl=30:sos=on:sp=occurrence:updr=off_3");
      quick.push("ott+11_6_av=off:bs=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=10:sp=reverse_arity_100");
      quick.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_29");
      quick.push("dis+11_2:1_acc=model:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:er=known:gsp=input_only:irw=on:lma=on:nwc=5:sac=on:urr=ec_only_148");
      quick.push("lrs+1011_5:1_afp=100000:afq=1.0:anc=none:bd=off:gsp=input_only:gs=on:gsem=off:lwlo=on:nwc=5:nicw=on:sas=z3:stl=30:sac=on:updr=off_176");
      quick.push("ott+1011_8:1_av=off:bd=off:cond=on:nwc=1:sos=all:sp=reverse_arity_182");
    }
    else {
      quick.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:stl=30:updr=off_3");
      quick.push("dis+1011_32_aac=none:afp=10000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=3:sas=z3:sp=reverse_arity_3");
      quick.push("lrs+10_5:1_add=large:afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_5");
      quick.push("lrs+11_2_afr=on:afp=100000:afq=1.4:amm=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:stl=30:sos=on:updr=off_5");
      quick.push("dis-11_5:1_acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=first:cond=on:gs=on:gsem=off:nwc=1:sd=3:ss=axioms:st=1.2:sos=all_7");
      quick.push("dis+1011_8_av=off:bd=off:cond=fast:er=known:fde=unused:gsp=input_only:lcm=predicate:lma=on:nwc=1.2:sp=reverse_arity:updr=off:uhcvi=on_33");
      quick.push("dis+2_3_av=off:cond=on:fsr=off:lcm=reverse:lma=on:nwc=1:sos=on:sp=reverse_arity_6");
      quick.push("dis+1002_4_afr=on:afp=1000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=off:lma=on:nwc=1:sos=on:sac=on:sp=occurrence_45");
      quick.push("lrs+1011_2_acc=on:add=off:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:er=known:nwc=4:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off_3");
      quick.push("lrs+1011_5:1_afp=100000:afq=1.0:anc=none:bd=off:gsp=input_only:gs=on:gsem=off:lwlo=on:nwc=5:nicw=on:sas=z3:stl=30:sac=on:updr=off_67");
      quick.push("lrs+1011_2_av=off:bd=preordered:bs=unit_only:cond=fast:fsr=off:fde=unused:irw=on:lma=on:lwlo=on:nwc=1.2:stl=30:sp=occurrence:uhcvi=on_73");
      quick.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_8");
      quick.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_80");
      quick.push("dis+11_12_av=off:bsr=on:cond=on:lcm=predicate:lma=on:nwc=5:sp=reverse_arity:updr=off_1");
      quick.push("lrs+1010_3:2_av=off:fsr=off:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=ec_only_101");
      quick.push("dis+11_2:1_acc=model:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:er=known:gsp=input_only:irw=on:lma=on:nwc=5:sac=on:urr=ec_only_25");
      quick.push("ott+1011_4_acc=on:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:bs=unit_only:ccuc=small_ones:cond=on:fsr=off:irw=on:lwlo=on:nwc=1.3:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_23");
      quick.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_100");
      quick.push("dis+2_10_afp=100000:afq=1.2:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1_5");
      quick.push("dis+1003_3_add=off:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:bsr=on:bce=on:cond=fast:fde=none:gs=on:gsaa=from_current:gsem=off:nwc=1.2:sas=z3:sac=on:sp=reverse_arity_13");
      quick.push("lrs+4_4:1_add=large:afr=on:afp=100000:afq=1.0:anc=none:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nwc=1.5:sas=z3:stl=30:sos=all:sac=on:sp=reverse_arity_148");
      quick.push("lrs+1010_4_aac=none:acc=model:add=large:afp=10000:afq=1.4:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:cond=on:fsr=off:irw=on:nwc=2:stl=30:sac=on:sp=reverse_arity:urr=ec_only_66");
      quick.push("ott+10_28_acc=model:add=off:afp=40000:afq=2.0:amm=off:bs=unit_only:bsr=on:ccuc=small_ones:cond=on:fsr=off:gs=on:gsem=on:nwc=1.1:nicw=on:urr=on:updr=off_50");
      quick.push("lrs+1011_2:1_afp=40000:afq=1.1:amm=off:anc=none:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=on:nwc=1:sas=z3:stl=30:sos=all:sp=reverse_arity:updr=off:uhcvi=on_38");
      quick.push("ott+1011_2:3_av=off:cond=fast:er=filter:fde=unused:gsp=input_only:irw=on:nwc=3:sp=occurrence:updr=off:uhcvi=on_2");
      quick.push("lrs+1011_2:1_av=off:cond=on:lwlo=on:nwc=1.5:stl=30_187");
      quick.push("lrs+1011_2:1_av=off:cond=on:er=known:gs=on:gsem=on:lwlo=on:nwc=1.3:stl=30:updr=off:uhcvi=on_133");
      quick.push("lrs+1011_40_add=off:afp=1000:afq=2.0:anc=none:bs=unit_only:fsr=off:irw=on:nwc=1:sas=z3:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_68");
    }
    break;

  case Property::HEQ:
    quick.push("lrs+11_24_afp=100000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:irw=on:nwc=3:stl=30_3");
    quick.push("dis+10_4_aac=none:afp=10000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:irw=on:lcm=reverse:lwlo=on:nwc=2.5:sas=z3:sp=reverse_arity_4");
    quick.push("lrs+11_3:1_av=off:cond=on:fsr=off:irw=on:lma=on:nwc=10:sas=z3:stl=30:updr=off_174");
    quick.push("dis+10_6_av=off:cond=on:gs=on:lcm=reverse:lma=on:nwc=1.7:sp=reverse_arity:updr=off_7");
    quick.push("lrs+11_4_aac=none:acc=model:add=large:afp=100000:afq=1.2:amm=off:anc=none:ccuc=first:cond=fast:gs=on:lma=on:nwc=4:stl=30:sac=on:sp=occurrence_237");
    quick.push("lrs+10_40_aac=none:acc=on:add=off:afp=1000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:ccuc=first:cond=fast:fsr=off:fde=none:gs=on:gsem=on:lma=on:nwc=1.3:stl=30:sp=reverse_arity:updr=off:uhcvi=on_38");
    quick.push("ott+11_64_add=large:afp=1000:afq=2.0:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=on:nwc=2_246");
    quick.push("lrs+2_14_aac=none:add=off:afp=100000:afq=1.1:anc=none:nwc=3:sas=z3:stl=30:sac=on:sp=reverse_arity:updr=off_168");
    quick.push("lrs+11_4:1_av=off:bce=on:cond=fast:fde=none:gsp=input_only:lma=on:nwc=5:stl=30:sp=occurrence_118");
    quick.push("lrs+1011_20_acc=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:ccuc=small_ones:cond=on:gs=on:irw=on:lwlo=on:nwc=1:stl=30:sp=occurrence_38");
    quick.push("lrs+11_40_add=off:afp=10000:afq=1.2:amm=off:cond=on:fsr=off:lma=on:nwc=1.7:stl=30:sac=on:sp=occurrence_182");
    quick.push("dis+11_28_add=off:afr=on:afp=40000:afq=2.0:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off_54");
    quick.push("lrs+10_1024_av=off:bd=off:fsr=off:lma=on:nwc=1:stl=30:sp=occurrence:urr=on_205");
    quick.push("lrs+11_3:2_add=off:afr=on:afp=4000:afq=1.4:anc=none:cond=on:lma=on:lwlo=on:nwc=3:sas=z3:stl=30:sac=on:sp=reverse_arity_124");
    quick.push("ott+11_2:1_afp=1000:afq=1.0:bd=preordered:fsr=off:fde=none:lma=on:nwc=1:sos=all:sp=occurrence:uhcvi=on_171");
    quick.push("lrs+10_3:1_add=off:afp=40000:afq=1.4:anc=none:br=off:fsr=off:gs=on:gsem=on:irw=on:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:urr=on_222");
    break;

  case Property::PEQ:
    if (prop == 1) {
      quick.push("dis+11_5:4_add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:lwlo=on:nwc=1:sas=z3:sac=on:sp=reverse_arity_8");
      quick.push("lrs+11_128_aac=none:add=large:afp=4000:afq=2.0:amm=sco:bd=off:gs=on:gsem=on:nwc=1:nicw=on:stl=30:sos=all:sac=on:sp=reverse_arity:updr=off_15");
      quick.push("lrs+11_2:3_add=large:afp=4000:afq=2.0:amm=sco:anc=none:er=known:gs=on:nwc=1:sas=z3:stl=30:updr=off_4");
      quick.push("dis+10_5_add=off:afr=on:afp=10000:afq=1.4:anc=none:er=known:gs=on:gsem=off:lma=on:nwc=1:nicw=on:sas=z3:sac=on:sp=reverse_arity_5");
      quick.push("lrs+1011_7_av=off:irw=on:nwc=1:stl=30:sos=all_69");
      quick.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:stl=30:sp=occurrence_33");
      quick.push("lrs+10_5_av=off:bd=off:bs=unit_only:cond=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:stl=30:sp=occurrence_224");
      quick.push("dis+1011_2:3_av=off:irw=on:nwc=1.2:sp=reverse_arity:updr=off_157");
      quick.push("ott+11_1_afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sac=on:sp=occurrence:updr=off:uhcvi=on_624");
      quick.push("lrs+11_32_add=off:afp=10000:afq=1.1:anc=all:bs=unit_only:cond=fast:fde=none:gs=on:gsaa=from_current:irw=on:lma=on:nwc=1:nicw=on:stl=60:sos=all:sac=on:sp=occurrence:updr=off:uhcvi=on_501");
      quick.push("lrs+11_2:1_av=off:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=2:stl=60:sp=occurrence:updr=off:uhcvi=on_546");
    }
    else if (prop == 0) {
      quick.push("lrs+11_3:2_afp=10000:afq=1.0:anc=none:bsr=on:cond=on:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:stl=30:sac=on:updr=off_7");
      quick.push("lrs+11_5_acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:bs=unit_only:ccuc=first:cond=on:lma=on:lwlo=on:nwc=1:stl=30:sp=reverse_arity:updr=off_234");
      quick.push("dis+10_28_add=large:afp=4000:afq=1.0:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence_72");
      quick.push("lrs+2_2_acc=model:add=off:afp=10000:afq=1.2:anc=all:bd=off:bs=on:bsr=on:ccuc=small_ones:cond=on:fsr=off:fde=unused:gs=on:lma=on:nwc=1.2:stl=30:sos=on:updr=off:uhcvi=on_43");
      quick.push("lrs+11_3:1_add=off:afr=on:afp=100000:afq=1.4:anc=none:cond=on:gs=on:irw=on:lma=on:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sac=on:sp=occurrence:updr=off_89");
      quick.push("ott+11_14_av=off:cond=on:nwc=1.3:sp=reverse_arity:updr=off_1");
      quick.push("dis+1011_8_av=off:bsr=on:cond=on:fsr=off:fde=none:gs=on:lma=on:nwc=1.1:sos=all:sp=reverse_arity_207");
      quick.push("lrs+10_2_afr=on:afp=4000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:sac=on:sp=occurrence:updr=off_107");
      quick.push("lrs+10_5:4_add=large:afr=on:amm=sco:anc=all_dependent:bd=preordered:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1:sas=z3:stl=60:sos=all:sac=on:updr=off:uhcvi=on_142");
      quick.push("lrs+10_4:1_av=off:bd=off:bs=unit_only:bsr=on:fsr=off:gs=on:gsem=on:lwlo=on:nwc=1:stl=30:sos=all_135");
      quick.push("ott+1_3_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bsr=on:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence:uhcvi=on_138");
      quick.push("ott+1011_5_afr=on:afp=1000:afq=1.4:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=unused:gsp=input_only:gs=on:nwc=1:nicw=on:sas=z3:sos=all:updr=off:uhcvi=on_484");
      quick.push("lrs+11_3_aac=none:add=large:afp=1000:afq=1.4:anc=all_dependent:bd=off:bs=on:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=1:stl=60:sos=all:sac=on:sp=occurrence:updr=off_587");
    }
    else {
      quick.push("lrs+11_2_add=large:afp=4000:afq=1.1:anc=none:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence:updr=off_110");
      quick.push("lrs+11_1024_add=off:afp=10000:afq=1.0:anc=none:bd=off:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:stl=30:sp=occurrence:updr=off_9");
      quick.push("ott+1010_14_add=large:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:bsr=on:cond=fast:lma=on:nwc=1.2:sp=occurrence:updr=off_259");
      quick.push("ott+11_7_acc=model:afr=on:afp=40000:afq=2.0:amm=off:anc=all_dependent:bs=unit_only:ccuc=small_ones:fsr=off:gs=on:gsaa=from_current:lma=on:nwc=1.7:nicw=on:sp=occurrence:uhcvi=on_116");
      quick.push("lrs+11_5_acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:bs=unit_only:ccuc=first:cond=on:lma=on:lwlo=on:nwc=1:stl=30:sp=reverse_arity:updr=off_119");
      quick.push("ott+11_5:1_add=off:afp=10000:afq=1.4:anc=none:bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:lma=on:nwc=1_147");
      quick.push("lrs+11_3:1_add=off:afr=on:afp=100000:afq=1.4:anc=none:cond=on:gs=on:irw=on:lma=on:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sac=on:sp=occurrence:updr=off_298");
      quick.push("lrs+11_128_aac=none:add=large:afp=4000:afq=2.0:amm=sco:bd=off:gs=on:gsem=on:nwc=1:nicw=on:stl=30:sos=all:sac=on:sp=reverse_arity:updr=off_1");
    }
    break;

  case Property::HNE:
    quick.push("lrs+1011_8_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bs=on:gs=on:gsem=off:nwc=1.5:nicw=on:stl=30:sac=on:sp=reverse_arity:updr=off_32");
    quick.push("dis+11_1_add=off:afp=4000:afq=2.0:amm=sco:anc=none:br=off:cond=on:lma=on:nwc=1:sos=all:sac=on:urr=on_7");
    quick.push("lrs+10_3:2_afr=on:afp=100000:afq=2.0:anc=none:bs=unit_only:bsr=on:cond=fast:fsr=off:lwlo=on:nwc=2:nicw=on:stl=30:sp=reverse_arity_211");
    quick.push("lrs+10_2:3_av=off:bsr=on:cond=on:lwlo=on:nwc=1.7:stl=60:sp=occurrence:updr=off_171");
    quick.push("lrs+11_24_afp=4000:afq=2.0:amm=sco:anc=all:br=off:cond=fast:gsp=input_only:gs=on:gsem=on:lma=on:lwlo=on:nwc=1.7:nicw=on:stl=30:sos=theory:sac=on:sp=reverse_arity:urr=on_122");
    quick.push("lrs+1011_3_av=off:bs=unit_only:cond=on:fsr=off:lcm=predicate:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_215");
    quick.push("dis+11_3:1_aac=none:add=off:afp=4000:afq=1.4:fsr=off:nwc=3:nicw=on:sp=occurrence_101");
    quick.push("lrs+10_5:4_av=off:bsr=on:gs=on:gsem=off:lma=on:nwc=4:stl=30:uhcvi=on_146");
    quick.push("lrs+1_2:3_av=off:fsr=off:lma=on:nwc=1:sas=z3:stl=30:urr=ec_only:updr=off_72");
    quick.push("lrs+4_3_add=off:afr=on:afp=10000:afq=2.0:anc=none:lma=on:nwc=1.1:nicw=on:sas=z3:stl=30:sac=on_203");
    quick.push("dis+1_40_av=off:lwlo=on:nwc=4:sos=all:sp=occurrence:updr=off_117");
    quick.push("lrs+10_2:3_av=off:cond=on:fsr=off:lwlo=on:nwc=2.5:stl=30:sp=reverse_arity:updr=off_68");
    quick.push("lrs+1_128_afp=100000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:stl=60:sac=on_455");
    quick.push("dis+11_8:1_afr=on:afp=1000:afq=1.0:amm=off:anc=none:nwc=1:sos=all:sp=occurrence:updr=off_184");
    break;

  case Property::NNE:
    quick.push("dis+1011_5_av=off:gs=on:gsem=on:nwc=1:sos=on:updr=off_3");
    quick.push("lrs+11_14_aac=none:afp=1000:afq=2.0:fsr=off:lma=on:nwc=1:stl=30:sp=reverse_arity_26");
    quick.push("dis-1_3_av=off:cond=on:fsr=off:gs=on:gsem=on:nwc=1:updr=off_4");
    quick.push("ins+10_1_av=off:cond=on:fsr=off:gsp=input_only:igbrr=0.6:igpr=on:igrr=64/1:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:updr=off_9");
    quick.push("dis+1011_64_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:gsp=input_only:gs=on:gsem=on:lma=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off:uhcvi=on_26");
    quick.push("dis+1011_5_av=off:lma=on:nwc=1.7:sos=all:sp=reverse_arity:updr=off_429");
    quick.push("lrs+1011_50_av=off:bs=unit_only:cond=on:nwc=2:stl=30:sp=occurrence:updr=off_47");
    quick.push("dis+4_7_av=off:cond=fast:gsp=input_only:lma=on:nwc=1.3:sp=occurrence:urr=ec_only:uhcvi=on_162");
    quick.push("dis+10_5:4_afp=100000:afq=1.0:amm=sco:anc=all:cond=fast:fsr=off:gs=on:lma=on:nwc=1:sp=reverse_arity:updr=off:uhcvi=on_24");
    quick.push("lrs+1010_14_av=off:fsr=off:lma=on:lwlo=on:nwc=2.5:stl=30:updr=off:uhcvi=on_82");
    quick.push("dis+2_2_add=off:afr=on:afp=4000:afq=1.0:amm=sco:anc=none:fsr=off:lcm=predicate:lma=on:nwc=1.3:nicw=on:sos=theory:sp=reverse_arity:urr=ec_only:updr=off_26");
    quick.push("dis+1003_128_add=large:afr=on:amm=off:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:sas=z3:sos=on:sp=occurrence:updr=off:uhcvi=on_64");
    quick.push("lrs+4_20_av=off:gs=on:gsem=off:lcm=reverse:lma=on:nwc=1:stl=30:sos=on:updr=off_43");
    quick.push("lrs+1011_2_av=off:cond=on:gsp=input_only:gs=on:lwlo=on:nwc=1:stl=30:sos=all:uhcvi=on_300");
    quick.push("dis+3_64_av=off:cond=fast:lcm=reverse:lma=on:lwlo=on:nwc=1:sos=on:updr=off_68");
    quick.push("lrs+11_2_afp=1000:afq=1.4:amm=sco:anc=none:cond=on:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_155");
    quick.push("lrs+1011_3:2_av=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:lcm=predicate:lwlo=on:nwc=2.5:stl=30:sos=all:updr=off_158");
    break;

  case Property::UEQ:
    if (prop == 2) {
      quick.push("ott+10_20_av=off:bd=preordered:fde=none:ins=3:nwc=1:sp=occurrence_98");
      quick.push("lrs+10_4:1_av=off:bd=preordered:fde=none:ins=3:lwlo=on:nwc=3:stl=60:sp=reverse_arity_96");
      quick.push("ott+10_20_av=off:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_862");
      quick.push("lrs+10_1_av=off:ins=3:nwc=1.1:stl=60:sp=reverse_arity_390");
      quick.push("lrs+10_2:1_av=off:bd=off:fde=none:ins=3:lwlo=on:nwc=1:stl=90:uhcvi=on_296");
      quick.push("lrs+10_28_av=off:ins=3:nwc=1:stl=30_186");
      quick.push("ott+10_4_av=off:fde=none:ins=3:nwc=1:sos=on:sp=occurrence:uhcvi=on_122");
      quick.push("ott+10_28_av=off:bd=preordered:fde=unused:ins=3:nwc=2.5:sp=occurrence_275");
      quick.push("ott+10_5:1_av=off:bd=preordered:fde=unused:ins=3:nwc=1_268");
      quick.push("ott+10_8_av=off:bd=preordered:ins=3:nwc=1.2:sp=reverse_arity:uhcvi=on_456");
      quick.push("lrs+10_3:2_av=off:bd=preordered:ins=3:lwlo=on:nwc=1.1:stl=90:uhcvi=on_529");
      quick.push("ott+10_2_av=off:bd=off:ins=3:nwc=1.2_430");
    }
    else if (prop == 0) {
      quick.push("lrs+10_3_av=off:ins=3:lwlo=on:nwc=1:stl=90:sp=occurrence:uhcvi=on_793");
      quick.push("ott+10_2_av=off:bd=preordered:fde=none:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_580");
      quick.push("ott+10_28_av=off:bd=preordered:ins=3:nwc=3:sp=reverse_arity_966");
      quick.push("ott+10_4:1_av=off:bd=preordered:ins=3:nwc=1:sp=reverse_arity_744");
      quick.push("lrs+10_3:1_av=off:fde=none:ins=3:nwc=1:stl=120:sp=reverse_arity:uhcvi=on_350");
      quick.push("ott+10_6_av=off:bd=preordered:fde=none:ins=3:nwc=1.1:sos=on:sp=occurrence_786");
      quick.push("ott+10_4_av=off:bd=off:ins=3:nwc=1.5:sp=reverse_arity_401");
      quick.push("ott+10_2:1_av=off:ins=3:nwc=5:sp=occurrence_1");
      quick.push("ott+10_8:1_av=off:bd=preordered:fde=none:ins=3:nwc=1_442");
    }
    else {
      quick.push("lrs+10_1_av=off:bd=off:ins=3:lwlo=on:nwc=3:stl=30_64");
      quick.push("lrs+10_5:1_av=off:bd=off:ins=3:nwc=2.5:stl=60:sp=reverse_arity_138");
      quick.push("ott+10_28_av=off:bd=preordered:fde=unused:ins=3:nwc=2.5:sp=occurrence_36");
      quick.push("ott+10_128_av=off:bd=off:ins=3:nwc=1:sp=reverse_arity_45");
      quick.push("lrs+1011_8_av=off:bs=unit_only:ep=RSTC:gsp=input_only:gs=on:gsem=on:lwlo=on:nwc=1:stl=30_226");
      quick.push("ott+10_4_av=off:bd=off:ins=3:nwc=1.5:sp=reverse_arity_568");
      quick.push("lrs+10_6_av=off:fde=unused:ins=3:lwlo=on:nwc=1:stl=60:sp=reverse_arity:uhcvi=on_1");
      quick.push("lrs+10_3_av=off:ins=3:lwlo=on:nwc=1:stl=90:sp=occurrence:uhcvi=on_2");
      quick.push("lrs+10_4_av=off:ins=3:lwlo=on:nwc=4:stl=30:sp=occurrence_1");
    }
    break;
  }

  switch (cat) {
  case Property::HEQ:
  case Property::PEQ:
  case Property::NEQ:
  case Property::HNE:
  case Property::NNE:
    fallback.push("lrs+1011_5:1_afp=100000:afq=1.0:anc=none:bd=off:gsp=input_only:gs=on:gsem=off:lwlo=on:nwc=5:nicw=on:sas=z3:sac=on:updr=off_300");
    fallback.push("lrs+10_2:3_av=off:bsr=on:cond=on:lwlo=on:nwc=1.7:sp=occurrence:updr=off_600");
    fallback.push("dis+1011_5_av=off:lma=on:nwc=1.7:sos=all:sp=reverse_arity:updr=off_600");
    fallback.push("lrs+11_3:1_av=off:cond=on:fsr=off:irw=on:lma=on:nwc=10:sas=z3:updr=off_300");
    fallback.push("lrs+11_2_add=large:afp=4000:afq=1.1:anc=none:lma=on:lwlo=on:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("lrs+4_2_av=off:bs=on:er=known:gs=on:irw=on:lwlo=on:nwc=10:sp=occurrence_300");
    fallback.push("dis+1011_3_av=off:nwc=1:sos=all:sp=reverse_arity_300");
    fallback.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bce=on:fde=unused:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+11_2_afr=on:afp=100000:afq=1.4:amm=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on:updr=off_300");
    fallback.push("lrs+1011_3_av=off:bs=unit_only:cond=on:fsr=off:lcm=predicate:lma=on:lwlo=on:nwc=1:sp=occurrence_300");
    fallback.push("lrs+1011_2:1_av=off:cond=on:er=known:gs=on:gsem=on:lwlo=on:nwc=1.3:updr=off:uhcvi=on_300");
    fallback.push("ott+11_14_av=off:cond=on:nwc=1.3:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_4_aac=none:acc=model:add=large:afp=100000:afq=1.2:amm=off:anc=none:ccuc=first:cond=fast:gs=on:lma=on:nwc=4:sac=on:sp=occurrence_300");
    fallback.push("dis+2_2_add=off:afr=on:afp=4000:afq=1.0:amm=sco:anc=none:fsr=off:lcm=predicate:lma=on:nwc=1.3:nicw=on:sos=theory:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=off:anc=none:cond=on:irw=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0:sos=all:sp=occurrence_300");
    fallback.push("lrs+11_5_add=large:afr=on:afp=40000:afq=1.2:cond=on:er=known:gs=on:gsem=off:nwc=1.5:sp=occurrence:updr=off_300");
    fallback.push("lrs+1003_3_av=off:bd=off:fde=none:gs=on:lma=on:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs-1_14_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:sd=1:ss=axioms:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_6_afr=on:afp=100000:afq=1.1:anc=none:br=off:gs=on:lma=on:nwc=3:sac=on:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+4_4:1_add=large:afr=on:afp=100000:afq=1.0:anc=none:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nwc=1.5:sas=z3:sos=all:sac=on:sp=reverse_arity_300");
    fallback.push("lrs+11_3_aac=none:add=large:afp=1000:afq=1.4:anc=all_dependent:bd=off:bs=on:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=1:sos=all:sac=on:sp=occurrence:updr=off_600");
    fallback.push("ott+11_64_add=large:afp=1000:afq=2.0:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=on:nwc=2_300");
    fallback.push("lrs+1011_3:2_av=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:lcm=predicate:lwlo=on:nwc=2.5:sos=all:updr=off_300");
    fallback.push("lrs+10_3_av=off:bd=off:cond=on:gs=on:gsem=off:irw=on:lwlo=on:nwc=1.2:sp=reverse_arity:updr=off_300");
    fallback.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:sac=on:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1010_4_aac=none:acc=model:add=large:afp=10000:afq=1.4:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:cond=on:fsr=off:irw=on:nwc=2:sac=on:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+10_28_add=large:afp=4000:afq=1.0:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence_300");
    fallback.push("lrs+4_1_av=off:bd=off:bsr=on:bce=on:fsr=off:irw=on:lcm=predicate:nwc=1:sos=all:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("ins+10_1_av=off:cond=on:fsr=off:gsp=input_only:igbrr=0.6:igpr=on:igrr=64/1:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:updr=off_300");
    fallback.push("dis+10_4_aac=none:afp=10000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:irw=on:lcm=reverse:lwlo=on:nwc=2.5:sas=z3:sp=reverse_arity_300");
    fallback.push("lrs+11_5_acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:bs=unit_only:ccuc=first:cond=on:lma=on:lwlo=on:nwc=1:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1_40_av=off:lwlo=on:nwc=4:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+10_5_add=off:afr=on:afp=10000:afq=1.4:anc=none:er=known:gs=on:gsem=off:lma=on:nwc=1:nicw=on:sas=z3:sac=on:sp=reverse_arity_300");
    fallback.push("lrs+4_5_av=off:bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:nwc=2.5:sp=occurrence:updr=off_300");
    fallback.push("dis+2_3_av=off:cond=on:fsr=off:lcm=reverse:lma=on:nwc=1:sos=on:sp=reverse_arity_300");
    fallback.push("dis+1011_2:1_av=off:lma=on:nwc=1:sd=3:ss=axioms:st=5.0:sp=occurrence:urr=ec_only_300");
    fallback.push("dis+1011_8_av=off:bd=off:cond=fast:er=known:fde=unused:gsp=input_only:lcm=predicate:lma=on:nwc=1.2:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_4:1_av=off:cond=on:irw=on:lma=on:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("ott+1011_2:3_av=off:cond=fast:er=filter:fde=unused:gsp=input_only:irw=on:nwc=3:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+10_3:2_afr=on:afp=100000:afq=2.0:anc=none:bs=unit_only:bsr=on:cond=fast:fsr=off:lwlo=on:nwc=2:nicw=on:sp=reverse_arity_300");
    fallback.push("dis+11_2:1_acc=model:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:er=known:gsp=input_only:irw=on:lma=on:nwc=5:sac=on:urr=ec_only_300");
    fallback.push("lrs+11_40_add=off:afp=10000:afq=1.2:amm=off:cond=on:fsr=off:lma=on:nwc=1.7:sac=on:sp=occurrence_300");
    fallback.push("lrs+10_5:4_av=off:bsr=on:gs=on:gsem=off:lma=on:nwc=4:uhcvi=on_300");
    fallback.push("lrs+1010_3:1_av=off:cond=on:nwc=5:sp=reverse_arity_300");
    fallback.push("lrs+11_3:2_add=off:afr=on:afp=4000:afq=1.4:anc=none:cond=on:lma=on:lwlo=on:nwc=3:sas=z3:sac=on:sp=reverse_arity_300");
    fallback.push("lrs+4_3_add=off:afr=on:afp=10000:afq=2.0:anc=none:lma=on:nwc=1.1:nicw=on:sas=z3:sac=on_300");
    fallback.push("ott+11_6_av=off:bs=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=10:sp=reverse_arity_300");
    fallback.push("dis+11_12_av=off:bsr=on:cond=on:lcm=predicate:lma=on:nwc=5:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_3:1_add=off:afr=on:afp=100000:afq=1.4:anc=none:cond=on:gs=on:irw=on:lma=on:lwlo=on:nwc=1:nicw=on:sas=z3:sac=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:sp=occurrence_300");
    fallback.push("ott+1011_5_afr=on:afp=1000:afq=1.4:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=unused:gsp=input_only:gs=on:nwc=1:nicw=on:sas=z3:sos=all:updr=off:uhcvi=on_600");
    fallback.push("ott+1010_14_add=large:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:bsr=on:cond=fast:lma=on:nwc=1.2:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_2_afp=1000:afq=1.4:amm=sco:anc=none:cond=on:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+11_3:2_afp=10000:afq=1.0:anc=none:bsr=on:cond=on:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:sac=on:updr=off_300");
    fallback.push("lrs+1011_40_add=off:afp=1000:afq=2.0:anc=none:bs=unit_only:fsr=off:irw=on:nwc=1:sas=z3:sos=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_12_av=off:bd=off:bs=unit_only:fsr=off:lma=on:nwc=1:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_7_av=off:irw=on:nwc=1:sos=all_300");
    fallback.push("lrs+10_12_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=off:cond=on:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=1.3:sas=z3:sac=on_300");
    fallback.push("lrs+1_3_av=off:fsr=off:lma=on:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on_300");
    fallback.push("dis+11_5_afr=on:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_2:1_av=off:bce=on:cond=fast:fsr=off:fde=none:lwlo=on:nwc=1:sd=1:ss=axioms:st=3.0:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("dis+4_7_av=off:cond=fast:gsp=input_only:lma=on:nwc=1.3:sp=occurrence:urr=ec_only:uhcvi=on_300");
    fallback.push("lrs+1011_50_av=off:bs=unit_only:cond=on:nwc=2:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_3:1_av=off:bs=unit_only:bsr=on:er=known:fsr=off:fde=unused:irw=on:lcm=reverse:lwlo=on:nwc=1.7:sos=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_1024_av=off:bd=off:fsr=off:lma=on:nwc=1:sp=occurrence:urr=on_300");
    fallback.push("lrs+10_3:1_add=off:afp=40000:afq=1.4:anc=none:br=off:fsr=off:gs=on:gsem=on:irw=on:lwlo=on:nwc=1:nicw=on:sas=z3:sos=all:urr=on_300");
    fallback.push("lrs+1_128_afp=100000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:sac=on_600");
    fallback.push("lrs+1010_3:2_av=off:fsr=off:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:sos=all:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+1011_20_acc=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:ccuc=small_ones:cond=on:gs=on:irw=on:lwlo=on:nwc=1:sp=occurrence_300");
    fallback.push("lrs+1010_14_av=off:fsr=off:lma=on:lwlo=on:nwc=2.5:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_2:1_add=large:afr=on:afp=100000:afq=2.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:lma=on:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on_300");
    fallback.push("ott+10_4:1_av=off:bd=preordered:cond=fast:fde=unused:irw=on:lcm=reverse:nwc=3:sd=2:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+2_14_aac=none:add=off:afp=100000:afq=1.1:anc=none:nwc=3:sas=z3:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+10_5:4_add=large:afr=on:amm=sco:anc=all_dependent:bd=preordered:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1:sas=z3:sos=all:sac=on:updr=off:uhcvi=on_600");
    fallback.push("ott+1011_8:1_av=off:bd=off:cond=on:nwc=1:sos=all:sp=reverse_arity_300");
    fallback.push("dis+11_28_add=off:afr=on:afp=40000:afq=2.0:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1_20_add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=z3:sd=3:ss=axioms:st=1.2:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_14_av=off:bd=off:bs=unit_only:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lcm=reverse:lwlo=on:nwc=1:sos=on:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+3_64_av=off:cond=fast:lcm=reverse:lma=on:lwlo=on:nwc=1:sos=on:updr=off_300");
    fallback.push("dis+1011_5_av=off:gs=on:gsem=on:nwc=1:sos=on:updr=off_300");
    fallback.push("dis+1003_128_add=large:afr=on:amm=off:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:sas=z3:sos=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis-11_4:1_amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nwc=10:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("lrs+11_1_av=off:fsr=off:irw=on:lma=on:lwlo=on:nwc=1:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+2_10_afp=100000:afq=1.2:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1_300");
    fallback.push("lrs+10_2:3_av=off:cond=on:fsr=off:lwlo=on:nwc=2.5:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1010_1024_afr=on:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:irw=on:lwlo=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0_300");
    fallback.push("lrs+1002_3:1_av=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nwc=4:sas=z3:sd=1:ss=axioms:st=5.0:sp=reverse_arity_300");
    fallback.push("lrs+10_5:1_add=large:afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_5:4_av=off:bd=off:fsr=off:gs=on:nwc=1.3:urr=ec_only:updr=off_300");
    fallback.push("ins+11_3_av=off:bd=off:igbrr=0.6:igrr=1/8:igrp=700:igrpq=2.0:igs=1:igwr=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:updr=off_300");
    fallback.push("lrs+10_40_aac=none:acc=on:add=off:afp=1000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:ccuc=first:cond=fast:fsr=off:fde=none:gs=on:gsem=on:lma=on:nwc=1.3:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("dis+1011_32_aac=none:afp=10000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=3:sas=z3:sp=reverse_arity_300");
    fallback.push("dis+1011_64_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:gsp=input_only:gs=on:gsem=on:lma=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_32_av=off:bd=off:br=off:gs=on:gsem=on:irw=on:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on:updr=off_300");
    fallback.push("dis+11_3_av=off:bd=off:bsr=on:bce=on:gsp=input_only:gs=on:gsem=on:lma=on:nwc=2.5:sp=reverse_arity:urr=ec_only:uhcvi=on_300");
    fallback.push("lrs+11_24_afp=100000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:irw=on:nwc=3_300");
    fallback.push("lrs+1011_2:1_afp=40000:afq=1.1:amm=off:anc=none:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=on:nwc=1:sas=z3:sos=all:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1010_8:1_av=off:bs=unit_only:br=off:cond=on:fsr=off:irw=on:nwc=1.3:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:updr=off_300");
    fallback.push("lrs+10_2_afr=on:afp=4000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:updr=off_300");
    fallback.push("dis-11_5:1_acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=first:cond=on:gs=on:gsem=off:nwc=1:sd=3:ss=axioms:st=1.2:sos=all_300");
    fallback.push("lrs+10_4:1_av=off:bd=off:bs=unit_only:bsr=on:fsr=off:gs=on:gsem=on:lwlo=on:nwc=1:sos=all_300");
    fallback.push("dis+2_5:4_add=large:afp=4000:afq=1.2:anc=all:bce=on:cond=fast:fde=none:lma=on:nwc=10:sd=1:ss=axioms:st=1.5:sac=on_300");
    fallback.push("lrs+10_5_av=off:bd=off:bs=unit_only:cond=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sp=occurrence_300");
    fallback.push("dis+1011_2:3_av=off:irw=on:nwc=1.2:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1002_5:1_av=off:cond=fast:fsr=off:fde=none:lma=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:urr=on:updr=off_300");
    fallback.push("lrs+2_2_acc=model:add=off:afp=10000:afq=1.2:anc=all:bd=off:bs=on:bsr=on:ccuc=small_ones:cond=on:fsr=off:fde=unused:gs=on:lma=on:nwc=1.2:sos=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1003_3_add=off:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:bsr=on:bce=on:cond=fast:fde=none:gs=on:gsaa=from_current:gsem=off:nwc=1.2:sas=z3:sac=on:sp=reverse_arity_300");
    fallback.push("dis+11_3:1_aac=none:add=off:afp=4000:afq=1.4:fsr=off:nwc=3:nicw=on:sp=occurrence_300");
    fallback.push("lrs+1011_2:1_av=off:cond=on:lwlo=on:nwc=1.5_300");
    fallback.push("lrs+11_24_afp=4000:afq=2.0:amm=sco:anc=all:br=off:cond=fast:gsp=input_only:gs=on:gsem=on:lma=on:lwlo=on:nwc=1.7:nicw=on:sos=theory:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("dis+10_5:4_afp=100000:afq=1.0:amm=sco:anc=all:cond=fast:fsr=off:gs=on:lma=on:nwc=1:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs-2_2:1_afr=on:afp=1000:afq=2.0:anc=none:bd=off:bce=on:cond=fast:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1.5:sac=on:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+1_2:1_av=off:fsr=off:lma=on:nwc=1:sd=7:ss=axioms:st=1.2:sos=on:urr=ec_only_300");
    fallback.push("dis+11_4_aac=none:add=large:afp=100000:afq=1.0:amm=off:anc=none:bsr=on:cond=on:fsr=off:gs=on:lcm=reverse:lma=on:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+1011_10_av=off:bs=unit_only:bsr=on:er=filter:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nwc=1.2:sp=reverse_arity:updr=off_300");
    fallback.push("dis-1_3_av=off:cond=on:fsr=off:gs=on:gsem=on:nwc=1:updr=off_300");
    fallback.push("lrs+11_4:1_av=off:bce=on:cond=fast:fde=none:gsp=input_only:lma=on:nwc=5:sp=occurrence_300");
    fallback.push("ott+11_7_acc=model:afr=on:afp=40000:afq=2.0:amm=off:anc=all_dependent:bs=unit_only:ccuc=small_ones:fsr=off:gs=on:gsaa=from_current:lma=on:nwc=1.7:nicw=on:sp=occurrence:uhcvi=on_300");
    fallback.push("ott+10_28_acc=model:add=off:afp=40000:afq=2.0:amm=off:bs=unit_only:bsr=on:ccuc=small_ones:cond=on:fsr=off:gs=on:gsem=on:nwc=1.1:nicw=on:urr=on:updr=off_300");
    fallback.push("lrs+1011_2_av=off:bd=preordered:bs=unit_only:cond=fast:fsr=off:fde=unused:irw=on:lma=on:lwlo=on:nwc=1.2:sp=occurrence:uhcvi=on_300");
    fallback.push("dis+11_8:1_afr=on:afp=1000:afq=1.0:amm=off:anc=none:nwc=1:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+10_3_acc=on:afr=on:afp=1000:afq=1.0:amm=sco:bs=unit_only:ccuc=first:fsr=off:irw=on:lcm=reverse:lma=on:nwc=1.5:updr=off_300");
    fallback.push("ott+11_5:1_add=off:afp=10000:afq=1.4:anc=none:bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:lma=on:nwc=1_300");
    fallback.push("lrs+1004_20_av=off:cond=on:er=filter:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1:sd=3:ss=axioms:st=3.0:sos=on:urr=ec_only_300");
    fallback.push("dis+1011_8_av=off:bsr=on:cond=on:fsr=off:fde=none:gs=on:lma=on:nwc=1.1:sos=all:sp=reverse_arity_300");
    fallback.push("lrs+11_14_aac=none:afp=1000:afq=2.0:fsr=off:lma=on:nwc=1:sp=reverse_arity_300");
    fallback.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+1_2:3_av=off:fsr=off:lma=on:nwc=1:sas=z3:urr=ec_only:updr=off_300");
    fallback.push("ott+11_2:1_afp=1000:afq=1.0:bd=preordered:fsr=off:fde=none:lma=on:nwc=1:sos=all:sp=occurrence:uhcvi=on_300");
    fallback.push("ott+1_3_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bsr=on:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence:uhcvi=on_300");
    fallback.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sp=occurrence_300");
    fallback.push("dis+11_2_av=off:gs=on:gsem=on:irw=on:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=reverse_arity_300");
    fallback.push("lrs+1011_8_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bs=on:gs=on:gsem=off:nwc=1.5:nicw=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis-11_64_av=off:bd=off:bs=on:cond=on:fsr=off:nwc=1:sd=1:ss=axioms:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_1024_add=off:afp=10000:afq=1.0:anc=none:bd=off:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("lrs-11_3_aac=none:acc=on:afr=on:afp=10000:afq=1.1:bd=off:bs=unit_only:ccuc=first:irw=on:lcm=predicate:lma=on:nwc=1.5:sos=all:sac=on:sp=occurrence:updr=off_300");
    fallback.push("dis-10_3:1_add=large:afr=on:afp=1000:afq=2.0:anc=none:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nwc=1:sas=z3:sd=2:ss=axioms:st=2.0:sos=all:sac=on:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("lrs+11_2:1_av=off:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=2:sp=occurrence:updr=off:uhcvi=on_600");
    fallback.push("lrs+11_32_add=off:afp=10000:afq=1.1:anc=all:bs=unit_only:cond=fast:fde=none:gs=on:gsaa=from_current:irw=on:lma=on:nwc=1:nicw=on:sos=all:sac=on:sp=occurrence:updr=off:uhcvi=on_600");
    fallback.push("ott+11_1_afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sac=on:sp=occurrence:updr=off:uhcvi=on_900");
    break;

  case Property::EPR:
    fallback.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_300");
    fallback.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:uhcvi=on_300");
    fallback.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_3000");
    fallback.push("dis+4_5_afp=1000:afq=1.1:amm=off:anc=none:bd=off:gs=on:irw=on:lcm=predicate:lma=on:nwc=1:sas=z3:sos=all:sp=occurrence_300");
    fallback.push("lrs+1003_10_afp=4000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:bce=on:fde=unused:lma=on:nwc=1:nicw=on:sac=on:urr=on:updr=off:uhcvi=on_1200");
    fallback.push("dis+11_2:3_add=large:afp=10000:afq=1.2:anc=none:bd=off:bce=on:cond=fast:er=filter:fsr=off:fde=unused:gsp=input_only:nwc=5:sos=theory:sac=on:urr=on_300");
    fallback.push("ott-4_6_add=off:afr=on:afp=100000:afq=1.4:amm=sco:bs=on:fde=unused:gs=on:gsaa=full_model:gsem=on:irw=on:nwc=1:sas=z3:sac=on:updr=off:uhcvi=on_600");
    fallback.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_200");
    fallback.push("dis-11_32_av=off:bs=unit_only:gs=on:irw=on:lma=on:nwc=1:updr=off_300");
    fallback.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_5:1_add=off:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:gs=on:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
    fallback.push("lrs+1_16_add=off:afp=100000:afq=1.0:amm=off:cond=fast:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=2:nicw=on:sd=7:ss=axioms:st=5.0:sos=theory:sp=reverse_arity:urr=ec_only_600");
    fallback.push("dis+1011_5_add=large:anc=none:bd=preordered:cond=on:fsr=off:fde=unused:lma=on:nwc=1:sos=theory:sp=occurrence:updr=off_1800");
    fallback.push("lrs+1_8:1_add=off:anc=none:bd=preordered:br=off:bce=on:fsr=off:fde=none:nwc=1:nicw=on:sos=theory:sp=reverse_arity:urr=on_900");
    break;

  case Property::UEQ:
    fallback.push("lrs+10_3:2_av=off:bd=preordered:ins=3:lwlo=on:nwc=1.1:uhcvi=on_900");
    fallback.push("ott+10_28_av=off:bd=preordered:ins=3:nwc=3:sp=reverse_arity_1200");
    fallback.push("lrs+10_4:1_av=off:bd=preordered:fde=none:ins=3:lwlo=on:nwc=3:sp=reverse_arity_600");
    fallback.push("ott+10_4_av=off:bd=off:ins=3:nwc=1.5:sp=reverse_arity_600");
    fallback.push("ott+10_4:1_av=off:bd=preordered:ins=3:nwc=1:sp=reverse_arity_900");
    fallback.push("lrs+10_3_av=off:ins=3:lwlo=on:nwc=1:sp=occurrence:uhcvi=on_900");
    fallback.push("ott+10_128_av=off:bd=off:ins=3:nwc=1:sp=reverse_arity_300");
    fallback.push("ott+10_5:1_av=off:bd=preordered:fde=unused:ins=3:nwc=1_300");
    fallback.push("lrs+10_2:1_av=off:bd=off:fde=none:ins=3:lwlo=on:nwc=1:uhcvi=on_900");
    fallback.push("ott+10_20_av=off:bd=preordered:fde=none:ins=3:nwc=1:sp=occurrence_300");
    fallback.push("lrs+10_5:1_av=off:bd=off:ins=3:nwc=2.5:sp=reverse_arity_600");
    fallback.push("lrs+1011_8_av=off:bs=unit_only:ep=RSTC:gsp=input_only:gs=on:gsem=on:lwlo=on:nwc=1_300");
    fallback.push("ott+10_5:4_av=off:ins=3:nwc=1.7:sp=reverse_arity_300");
    fallback.push("ott+10_4_av=off:fde=none:ins=3:nwc=1:sos=on:sp=occurrence:uhcvi=on_300");
    fallback.push("ott+10_2_av=off:bd=preordered:fde=none:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_600");
    fallback.push("ott+10_8:1_av=off:bd=preordered:fde=none:ins=3:nwc=1_600");
    fallback.push("ott+10_2_av=off:bd=off:ins=3:nwc=1.2_600");
    fallback.push("ott+10_20_av=off:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_1200");
    fallback.push("lrs+10_3:1_av=off:fde=none:ins=3:nwc=1:sp=reverse_arity:uhcvi=on_1200");
    fallback.push("ott+10_8_av=off:bd=preordered:ins=3:nwc=1.2:sp=reverse_arity:uhcvi=on_600");
    fallback.push("lrs+10_1_av=off:ins=3:nwc=1.1:sp=reverse_arity_600");
    fallback.push("ott+10_6_av=off:bd=preordered:fde=none:ins=3:nwc=1.1:sos=on:sp=occurrence_900");
    break;

  case Property::FEQ:
    fallback.push("lrs+1010_3:2_afr=on:afp=100000:afq=1.1:anc=none:gsp=input_only:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1.7:sac=on:sp=occurrence_300");
    fallback.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:updr=off_300");
    fallback.push("dis+1002_4_add=large:afp=40000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsaa=full_model:lma=on:lwlo=on:nm=0:nwc=1.5:sas=z3:sp=reverse_arity:tha=off:thi=strong_300");
    fallback.push("lrs+1011_2:1_av=off:irw=on:lwlo=on:nm=16:newcnf=on:nwc=2:sd=4:ss=axioms:st=3.0:sp=occurrence_300");
    fallback.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:sp=occurrence_300");
    fallback.push("lrs+11_3:2_add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bs=unit_only:cond=on:gs=on:gsaa=full_model:gsem=off:irw=on:lwlo=on:nm=32:nwc=1.2:sas=z3:sac=on_900");
    fallback.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:sos=all:sp=occurrence:urr=on_300");
    fallback.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:sd=2:ss=axioms:st=2.0:sos=all_300");
    fallback.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+1_1024_av=off:bs=on:fde=none:inw=on:irw=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_600");
    fallback.push("lrs+11_1_av=off:bsr=on:gsp=input_only:gs=on:lcm=predicate:nm=64:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+1010_2:3_add=off:afr=on:afp=10000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:tha=off_300");
    fallback.push("dis+1010_7_afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:sp=occurrence:urr=on_300");
    fallback.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("ott+4_32_av=off:bsr=on:cond=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=4:nwc=1.2:sos=theory_300");
    fallback.push("dis+11_32_av=off:ep=RST:fsr=off:lwlo=on:nm=6:nwc=1.1:sd=5:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_1500");
    fallback.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_300");
    fallback.push("lrs+11_50_afp=100000:afq=1.1:amm=sco:anc=none:bs=unit_only:cond=on:irw=on:lma=on:nm=32:nwc=1.1:sp=reverse_arity_300");
    fallback.push("lrs+1011_7_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:cond=on:er=known:fsr=off:lma=on:nm=4:nwc=2.5:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_300");
    fallback.push("dis+1002_6_add=large:afp=40000:afq=2.0:bsr=on:cond=on:irw=on:lma=on:nm=2:nwc=2.5:nicw=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_2:3_add=off:afr=on:afp=4000:afq=1.4:anc=none:bs=unit_only:fsr=off:gs=on:gsem=on:lwlo=on:nm=16:nwc=1.3:nicw=on:sas=z3:sac=on:tha=off_300");
    fallback.push("ott+4_40_av=off:bce=on:cond=fast:fde=none:nm=0:nwc=1:sos=all:updr=off_300");
    fallback.push("dis+1002_7_acc=on:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:ccuc=first:fsr=off:gsp=input_only:gs=on:gsem=on:nm=6:nwc=1.1:nicw=on:sos=on:sac=on:sp=occurrence:urr=ec_only_300");
    fallback.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=on_300");
    fallback.push("lrs+11_8:1_av=off:bd=off:bs=unit_only:gs=on:gsem=on:lma=on:lwlo=on:nm=0:nwc=1:sd=1:ss=axioms:sos=all:urr=on:updr=off_300");
    fallback.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("ott+1002_128_av=off:bd=off:bs=on:bsr=on:cond=on:fsr=off:nm=6:newcnf=on:nwc=1:sp=reverse_arity:updr=off_300");
    fallback.push("ott+1011_4:1_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=preordered:cond=fast:nm=32:newcnf=on:nwc=1.2:sas=z3:sac=on:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott-11_3_add=large:afp=100000:afq=1.2:anc=none:bs=on:cond=fast:fde=none:gs=on:gsem=off:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:sp=occurrence:tha=off:urr=on:uhcvi=on_300");
    fallback.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_300");
    fallback.push("lrs+10_3:1_av=off:bsr=on:cond=on:er=known:gs=on:lcm=reverse:nm=32:nwc=4:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:sd=2:ss=axioms:st=1.5:updr=off_300");
    fallback.push("dis+1010_1_acc=model:add=large:afr=on:amm=off:anc=none:bd=off:bsr=on:ccuc=small_ones:gs=on:gsem=on:nm=16:nwc=2:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:sos=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1011_1_av=off:cond=on:gs=on:lma=on:nm=4:nwc=1:sd=3:ss=axioms:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("ott+11_20_afp=10000:afq=1.1:anc=none:bs=unit_only:bsr=on:bce=on:fsr=off:gs=on:gsem=on:nwc=2.5:sas=z3:sp=occurrence:updr=off:uhcvi=on_600");
    fallback.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:sd=2:ss=axioms:sos=on:sp=reverse_arity_300");
    fallback.push("ott+11_2:1_av=off:bd=off:bsr=on:br=off:cond=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("ott+11_3_afp=1000:afq=2.0:anc=none:fsr=off:irw=on:nwc=1.7:ss=axioms:st=1.5:sac=on:updr=off_300");
    fallback.push("dis+11_3_add=off:afp=10000:afq=2.0:amm=sco:anc=none:ep=RST:gs=on:gsaa=from_current:gsem=on:inw=on:nm=64:nwc=1:sd=10:ss=axioms:st=5.0:sos=all:tha=off:updr=off:uhcvi=on_300");
    fallback.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_300");
    fallback.push("ott+1010_5:4_av=off:bd=off:fde=none:irw=on:lma=on:nm=32:nwc=2.5:sd=2:ss=axioms:st=3.0:urr=on_300");
    fallback.push("dis+11_2:1_add=large:afp=4000:afq=1.4:amm=sco:anc=none:fsr=off:nm=16:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_300");
    fallback.push("ins+1010_3_av=off:bd=off:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.3:igwr=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=1:ss=axioms:sos=on:sp=occurrence:updr=off_300");
    fallback.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_300");
    fallback.push("dis+1_3:1_acc=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:fsr=off:gs=on:inw=on:lma=on:nm=32:nwc=1:urr=on_300");
    fallback.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_4:1_add=large:afp=100000:afq=1.1:anc=none:ep=RST:fde=unused:gsp=input_only:nm=6:newcnf=on:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+10_24_add=off:afr=on:afp=1000:afq=1.4:anc=none:bs=unit_only:gs=on:gsaa=from_current:gsem=on:lma=on:nm=2:nwc=1.1:sac=on:uhcvi=on_600");
    fallback.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("dis+1002_8:1_av=off:br=off:cond=on:irw=on:lma=on:nm=32:nwc=1:sp=occurrence:urr=on_300");
    fallback.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:sac=on:urr=on_300");
    fallback.push("dis+11_1_acc=on:afp=1000:afq=1.4:amm=sco:anc=all_dependent:bs=on:ccuc=small_ones:cond=fast:fde=none:gsp=input_only:nm=64:nwc=1:sac=on:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+1002_3_av=off:cond=on:fsr=off:gs=on:gsem=off:lwlo=on:nm=64:nwc=2.5:sp=reverse_arity_300");
    fallback.push("dis+1_3:2_acc=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:ccuc=small_ones:cond=on:lma=on:nm=64:nwc=1.3:sos=all:urr=on_300");
    fallback.push("dis+1010_24_aac=none:afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:nm=6:nwc=1:sas=z3:sos=on:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+1002_1_av=off:er=filter:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1:sd=3:ss=axioms:st=1.5:sos=on_300");
    fallback.push("dis-10_3:2_aac=none:afp=1000:afq=1.1:cond=on:fsr=off:lcm=reverse:lwlo=on:nm=16:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+10_5_av=off:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("lrs+1002_8:1_add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:er=known:lwlo=on:nm=0:nwc=1.2:sd=1:ss=axioms:sp=occurrence:updr=off_300");
    fallback.push("dis+10_10_add=large:afp=4000:afq=1.1:amm=sco:anc=none:irw=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("lrs+1010_5_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:bd=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=4:sas=z3:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_50_afr=on:afp=40000:afq=1.0:amm=off:anc=all_dependent:bs=on:bsr=on:bce=on:fde=unused:gs=on:lma=on:nm=16:nwc=1.1:sp=occurrence:updr=off_600");
    fallback.push("dis+1011_8:1_aac=none:acc=on:afp=1000:afq=1.4:amm=off:anc=all:bs=unit_only:bce=on:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:lma=on:nm=16:nwc=2.5:sd=4:ss=axioms:st=1.5:sos=all:uhcvi=on_300");
    fallback.push("lrs+11_5_av=off:cond=on:fsr=off:lma=on:lwlo=on:nwc=1.2:sas=z3:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1010_3:2_av=off:lma=on:nm=2:newcnf=on:nwc=1:sd=3:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+10_8:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lcm=predicate:lwlo=on:nm=64:nwc=1:sos=all:updr=off_300");
    fallback.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_2:3_aac=none:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:gs=on:gsem=on:inw=on:newcnf=on:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+11_128_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:nwc=1.3:sac=on:uhcvi=on_300");
    fallback.push("lrs+2_3:1_afr=on:afp=10000:afq=1.0:amm=sco:anc=none:bs=unit_only:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:updr=off_300");
    fallback.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_300");
    fallback.push("dis-10_4:1_aac=none:add=off:afp=1000:afq=1.4:amm=off:anc=none:cond=fast:ep=RSTC:gs=on:gsaa=from_current:gsem=on:inw=on:lma=on:nm=64:nwc=4:sas=z3:tha=off:thi=strong:uwa=interpreted_only:updr=off:uhcvi=on_300");
    fallback.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+1_5:4_aac=none:add=off:afr=on:afp=4000:afq=1.2:amm=sco:anc=none:gsp=input_only:gs=on:irw=on:nm=64:newcnf=on:nwc=1.3:nicw=on:sas=z3:sp=occurrence:tha=off_300");
    fallback.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:sos=theory:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+4_2_av=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:sos=on:urr=on_300");
    fallback.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs-11_24_av=off:bd=off:bs=on:fsr=off:gs=on:gsem=off:lma=on:nm=4:nwc=1.7:sp=occurrence:updr=off_900");
    fallback.push("lrs+10_2:3_afp=1000:afq=1.1:amm=sco:anc=none:er=known:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=2:sd=5:ss=axioms:sos=theory:sac=on:sp=occurrence_300");
    fallback.push("lrs+1010_3_av=off:fsr=off:gs=on:gsem=off:nm=2:newcnf=on:nwc=2:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1011_3:2_av=off:er=known:lma=on:nm=2:newcnf=on:nwc=2:sd=2:ss=axioms:st=3.0:urr=on:updr=off_300");
    fallback.push("lrs+4_5:4_av=off:bd=off:er=filter:lma=on:lwlo=on:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("ott+11_20_afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bsr=on:irw=on:lma=on:nm=4:nwc=1.2:sac=on:sp=occurrence_300");
    fallback.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+1010_8:1_add=off:afp=4000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=4:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:sd=1:ss=axioms:st=2.0:sac=on:urr=on_300");
    fallback.push("ott+1_5:1_acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:lcm=predicate:nm=16:nwc=1.1:sd=1:ss=axioms:st=3.0:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:sd=1:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("dis+1010_3:2_av=off:gsp=input_only:nm=2:nwc=1:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:sd=2:ss=axioms:sos=all:urr=ec_only_300");
    fallback.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+1004_5:4_aac=none:add=large:afp=100000:afq=1.4:anc=all_dependent:bd=off:cond=fast:gsp=input_only:gs=on:gsem=off:lma=on:lwlo=on:nm=4:newcnf=on:nwc=1:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+2_128_add=large:afp=100000:afq=1.4:amm=sco:anc=none:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sos=theory:sac=on:updr=off_600");
    fallback.push("ott+10_8:1_av=off:bd=preordered:bsr=on:cond=fast:fsr=off:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1.2:sp=reverse_arity:urr=on:updr=off:uhcvi=on_1200");
    fallback.push("dis+1002_4_add=off:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1:sos=on:sac=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_300");
    fallback.push("ott+1011_3:1_aac=none:add=off:afp=1000:afq=1.2:amm=sco:bd=off:bs=on:bsr=on:cond=on:gsp=input_only:gs=on:lma=on:nm=6:newcnf=on:nwc=1.3:nicw=on:sd=3:ss=axioms:st=2.0:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+10_4_add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=predicate:lma=on:nm=64:nwc=1:sd=3:ss=axioms:sos=on:sp=reverse_arity_300");
    fallback.push("lrs+1011_4:1_av=off:fsr=off:irw=on:nwc=1:sd=4:ss=axioms:st=1.5:sp=reverse_arity_300");
    fallback.push("dis+1_2:3_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:nm=64:newcnf=on:nwc=1_300");
    fallback.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_300");
    fallback.push("lrs+1002_2:1_aac=none:afr=on:afp=1000:afq=1.2:anc=all:bd=preordered:bsr=on:cond=fast:gsp=input_only:gs=on:nm=0:nwc=2.5:nicw=on:sas=z3:sd=4:ss=axioms:st=2.0:sos=on:sac=on:urr=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1004_1_aac=none:acc=on:afp=40000:afq=1.2:anc=none:cond=on:fde=unused:gs=on:gsem=off:irw=on:nm=32:nwc=2:sd=1:ss=axioms:sos=theory:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+11_3_afp=100000:afq=2.0:amm=sco:anc=none:gs=on:gsem=off:inw=on:lma=on:nm=64:nwc=1:sas=z3:sd=10:ss=axioms:st=5.0:sp=occurrence:tha=off:updr=off_300");
    fallback.push("dis+1010_2_afr=on:afp=1000:afq=1.1:amm=off:anc=none:bs=unit_only:bce=on:cond=fast:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=5:nicw=on:sas=z3:sac=on:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+1002_16_av=off:cond=on:nwc=3_300");
    fallback.push("lrs+11_3_av=off:bce=on:cond=fast:ep=R:lcm=reverse:lma=on:newcnf=on:nwc=1.3:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence:uhcvi=on_300");
    fallback.push("dis-3_5_av=off:cond=on:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1:sos=on:sp=reverse_arity_300");
    fallback.push("ott+1_8_av=off:bd=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:lwlo=on:nwc=1:sos=on_300");
    fallback.push("lrs+10_3_add=off:afp=40000:afq=1.4:amm=off:anc=none:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=full_model:inw=on:lma=on:nm=64:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1010_4_afp=10000:afq=1.2:anc=none:irw=on:lma=on:nm=64:nwc=10:sas=z3:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis-3_4_add=off:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:inw=on:lma=on:nm=64:nwc=1.5:nicw=on:sas=z3:sp=reverse_arity:tha=off:thf=on:uhcvi=on_300");
    fallback.push("ott-2_28_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=16:newcnf=on:nwc=1:nicw=on:sp=occurrence:thf=on_300");
    fallback.push("dis+3_24_av=off:bd=off:bs=unit_only:fsr=off:fde=unused:gs=on:irw=on:lma=on:nm=0:nwc=1.1:sos=on:uhcvi=on_300");
    fallback.push("lrs+1011_14_av=off:fsr=off:irw=on:nwc=1:sos=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_300");
    fallback.push("dis+11_28_av=off:fsr=off:irw=on:lcm=predicate:nm=2:newcnf=on:nwc=5:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_3_av=off:cond=on:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+10_4_av=off:bsr=on:cond=fast:er=filter:fde=none:gsp=input_only:lcm=reverse:lma=on:nwc=4:sp=occurrence:urr=on_300");
    fallback.push("lrs+10_8:1_av=off:bs=unit_only:cond=on:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:nwc=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1011_5_afr=on:afp=100000:afq=1.0:amm=off:anc=none:cond=on:lma=on:nm=6:nwc=1:sas=z3:sac=on:urr=on_300");
    fallback.push("dis+11_3_afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=preordered:bce=on:fsr=off:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1:sd=10:ss=axioms:st=5.0:sac=on:sp=occurrence:tha=off:urr=ec_only_300");
    fallback.push("lrs+1_3:2_aac=none:add=large:anc=all_dependent:bce=on:cond=fast:ep=RST:fsr=off:lma=on:nm=2:nwc=1:sos=on:sp=occurrence:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_7_av=off:cond=fast:fde=none:gs=on:gsem=off:lcm=predicate:nm=6:nwc=1:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_3:2_aac=none:afp=10000:afq=1.2:amm=off:anc=all:cond=on:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+10_5:4_afr=on:afp=40000:afq=1.2:bd=off:gsp=input_only:gs=on:inw=on:nm=0:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:thf=on:urr=on_300");
    fallback.push("ott+11_50_add=large:afr=on:afp=40000:afq=1.1:amm=sco:anc=all:bd=off:cond=fast:gs=on:gsem=on:irw=on:lma=on:nm=0:nwc=1:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+1002_8_add=large:afp=100000:afq=1.2:amm=off:bs=on:irw=on:nm=2:newcnf=on:nwc=1.1:sos=on:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+1010_2:3_afr=on:afp=40000:afq=1.4:amm=off:anc=none:lma=on:nm=16:nwc=1:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+11_6_add=large:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=fast:gs=on:gsaa=from_current:gsem=off:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:thi=strong:updr=off_300");
    fallback.push("lrs+11_2:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=16:nwc=1:sd=1:ss=axioms:st=1.2:sp=reverse_arity_300");
    fallback.push("dis+4_8:1_add=large:afp=100000:afq=1.4:ep=RST:fde=unused:gsp=input_only:lcm=predicate:nwc=1:sos=all:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+1010_3:1_av=off:gsp=input_only:nm=6:nwc=1:sos=all:sp=occurrence_300");
    fallback.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_300");
    fallback.push("dis+4_2_av=off:bs=on:fsr=off:gsp=input_only:newcnf=on:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+1010_4_add=off:afp=100000:afq=1.0:anc=none:fsr=off:gs=on:gsem=off:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sac=on:tha=off:thf=on_300");
    fallback.push("dis+1011_5_aac=none:add=large:afp=40000:afq=1.2:amm=off:anc=none:bd=off:fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:sos=all:updr=off_300");
    fallback.push("lrs+4_1_acc=on:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bs=on:bsr=on:ccuc=first:fsr=off:fde=unused:irw=on:lma=on:nm=0:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=all:sp=occurrence:uhcvi=on_300");
    fallback.push("dis+1011_10_aac=none:add=large:afp=10000:afq=1.1:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=6:newcnf=on:nwc=2.5:sp=reverse_arity:updr=off_300");
    fallback.push("dis+10_5_add=off:afp=4000:afq=1.1:anc=none:cond=fast:ep=RSTC:fsr=off:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sp=reverse_arity:thi=all_300");
    fallback.push("ott-11_12_aac=none:afp=100000:afq=1.2:amm=sco:bs=on:bce=on:cond=fast:er=known:gs=on:gsaa=from_current:gsem=off:irw=on:nm=4:nwc=2:sas=z3:sos=all:sp=occurrence:urr=ec_only:updr=off_600");
    fallback.push("lrs+11_3:2_add=large:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:er=filter:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:sas=z3:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+11_20_afr=on:afp=100000:afq=1.2:amm=off:anc=none:bs=unit_only:bsr=on:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lma=on:lwlo=on:nm=16:newcnf=on:nwc=1:nicw=on:sas=z3:sac=on:updr=off:uhcvi=on_300");
    fallback.push("ott+10_7_av=off:bd=preordered:bs=unit_only:bsr=on:cond=fast:fsr=off:fde=unused:irw=on:lma=on:nm=32:nwc=1:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_3_afp=100000:afq=1.1:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:irw=on:nm=6:nwc=1:sas=z3:sd=2:ss=axioms:sos=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_24_add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=fast:fde=unused:gs=on:irw=on:lma=on:nm=4:nwc=1.3:nicw=on:sas=z3:sac=on:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+1010_2_add=large:afp=4000:afq=2.0:amm=off:bd=off:bs=unit_only:bsr=on:br=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=32:nwc=1.1:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs-2_3:1_add=off:afr=on:afp=1000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bs=unit_only:bsr=on:cond=on:fde=unused:gs=on:gsem=on:irw=on:lcm=reverse:nm=32:nwc=1.7:sas=z3:sos=all:sac=on_300");
    fallback.push("lrs+11_4:1_av=off:bd=off:br=off:cond=fast:fde=unused:irw=on:lcm=reverse:lma=on:lwlo=on:nm=4:nwc=1:sd=3:ss=axioms:sos=all:urr=on_600");
    fallback.push("ott+11_128_afp=4000:afq=1.1:anc=none:bs=unit_only:cond=on:gsp=input_only:lma=on:nm=4:nwc=1.5:sos=theory:sp=occurrence:updr=off_300");
    fallback.push("lrs-1_14_add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:br=off:cond=fast:fde=unused:gs=on:lcm=reverse:lma=on:nwc=1:sos=on:urr=on:updr=off_300");
    fallback.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_300");
    fallback.push("dis+10_3:2_afr=on:afp=1000:afq=1.2:bd=off:irw=on:lcm=predicate:lwlo=on:nm=0:newcnf=on:nwc=2:sos=on:tha=off:thf=on:urr=ec_only_300");
    fallback.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_300");
    fallback.push("lrs+10_128_acc=model:afp=100000:afq=2.0:anc=all_dependent:bs=on:bsr=on:cond=fast:er=filter:gs=on:gsem=off:lcm=reverse:lma=on:nm=32:nwc=3:sac=on:sp=occurrence:urr=ec_only_300");
    fallback.push("lrs+1011_2:3_add=large:afr=on:afp=100000:afq=1.2:anc=none:gs=on:gsem=off:irw=on:nm=64:nwc=1:sd=3:ss=axioms:sos=all:sp=reverse_arity_300");
    fallback.push("ott+1002_8:1_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:fsr=off:gs=on:gsem=off:nm=32:nwc=10:sas=z3:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+11_3_afr=on:afp=10000:afq=1.0:cond=fast:fsr=off:fde=none:gs=on:gsem=off:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=2.0:sos=all:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+1010_5_add=large:afp=10000:afq=1.2:amm=off:bs=unit_only:bsr=on:bce=on:cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1.3:nicw=on:sos=all:sac=on:updr=off_300");
    fallback.push("dis+2_2:1_aac=none:afp=100000:afq=1.1:amm=sco:anc=none:bsr=on:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=on_300");
    fallback.push("dis+11_8_add=off:afp=10000:afq=1.2:amm=off:anc=none:cond=fast:ep=R:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=4:sd=1:ss=axioms:sac=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_8:1_av=off:cond=fast:fde=unused:lcm=predicate:nm=16:nwc=10:sp=occurrence:urr=ec_only_600");
    fallback.push("lrs+1011_1024_av=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=32:nwc=1:sp=reverse_arity:uhcvi=on_900");
    fallback.push("dis+11_7_av=off:bd=preordered:bs=unit_only:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:nm=32:nwc=1:sd=4:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_1200");
    break;

  case Property::FNE:
    fallback.push("dis+1011_10_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=64:nwc=4:sac=on:sp=occurrence_300");
    fallback.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_300");
    fallback.push("dis+1_2_add=large:afr=on:afp=1000:afq=2.0:anc=none:gsp=input_only:lcm=predicate:nm=64:newcnf=on:nwc=5:sac=on:urr=ec_only:updr=off_300");
    fallback.push("dis+10_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:irw=on:nm=64:nwc=1:sas=z3:sac=on_300");
    fallback.push("dis+11_24_afp=40000:afq=1.1:amm=sco:anc=none:bs=on:gs=on:gsem=off:lcm=predicate:lma=on:nm=2:nwc=1:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+1011_1024_add=large:afp=4000:afq=1.1:anc=none:br=off:fsr=off:gsp=input_only:lma=on:nwc=1:sos=on:urr=on_300");
    fallback.push("dis+10_50_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:gs=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:thf=on:updr=off_300");
    fallback.push("lrs+4_32_add=large:afp=10000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lcm=predicate:lma=on:nm=2:nwc=1:sac=on:sp=occurrence:urr=on_300");
    fallback.push("dis+11_3_afr=on:afp=4000:afq=1.4:anc=none:cond=on:fsr=off:gs=on:lcm=reverse:nm=64:nwc=1:sos=on:sp=reverse_arity_300");
    fallback.push("lrs+1011_2:1_add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=64:nwc=1:sas=z3:sos=on:sp=reverse_arity:thf=on:urr=on:updr=off_300");
    fallback.push("lrs+10_4_add=off:afp=100000:afq=2.0:amm=sco:anc=none:nm=64:nwc=1:sp=occurrence:updr=off_1500");
    fallback.push("dis+10_128_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=2:nwc=1:sp=reverse_arity_300");
    fallback.push("ott+1_40_av=off:bs=unit_only:bsr=on:br=off:fsr=off:lma=on:nm=64:newcnf=on:nwc=1.5:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+1_1_av=off:bsr=on:br=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:nwc=1:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+11_5:1_afp=100000:afq=1.1:br=off:gs=on:nm=64:nwc=1:sos=all:urr=on:updr=off_300");
    fallback.push("ins+11_32_av=off:igbrr=0.4:igrr=1/64:igrpq=1.05:igwr=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1011_40_add=off:afr=on:afp=4000:afq=1.2:amm=sco:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:sos=all:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_600");
    fallback.push("ott+11_3:2_afp=40000:afq=1.0:amm=sco:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=full_model:lcm=reverse:nm=32:newcnf=on:nwc=5:nicw=on:sd=3:ss=axioms:sac=on:urr=on:updr=off_1200");
    break;
  }

  // add very long final fallback strategy
  fallback.push("dis+11_1_3600");

} // getCasc2017Schedule

/**
 * Get schedules for a problem of given property.
 * The schedules are to be executed from the bottom of the stack, i.e. in the order in which they are mentioned in the file.
 */
void Schedules::getCasc2018Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category();
  unsigned long prop = property.props();
  unsigned atoms = property.atoms();

  // for theory problems, we make the schedule before the main choice
  if (prop & (524288ul | 1048576ul | 2097152ul)) { // contains integers, rationals and reals
    if (prop & 67108864ul) { // linear integer functions
      quick.push("dis+11_6_add=large:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=fast:gs=on:gsaa=from_current:gsem=off:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:thi=strong:updr=off_2");
      quick.push("lrs+1010_4:1_aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=on:gs=on:gsem=on:irw=on:nm=0:nwc=2.5:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_11");
      quick.push("dis-10_4:1_aac=none:add=off:afp=1000:afq=1.4:amm=off:anc=none:cond=fast:ep=RSTC:gs=on:gsaa=from_current:gsem=on:inw=on:lma=on:nm=64:nwc=4:sas=z3:tha=off:thi=strong:uwa=interpreted_only:updr=off:uhcvi=on_6");
      quick.push("lrs-11_1_av=off:cond=on:gs=on:lcm=reverse:lma=on:lwlo=on:nm=16:nwc=5:stl=30:sp=reverse_arity:tha=off:thi=strong:uwa=interpreted_only_74");
      quick.push("dis+10_3:2_afr=on:afp=1000:afq=1.2:bd=off:irw=on:lcm=predicate:lwlo=on:nm=0:newcnf=on:nwc=2:sos=on:tha=off:thf=on:urr=ec_only_11");
      quick.push("dis+1010_2:3_add=off:afr=on:afp=10000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:tha=off_9");
      quick.push("lrs+1_2:3_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:lma=on:nm=64:nwc=1.3:sas=z3:stl=30:sac=on:tha=off:uwa=one_side_interpreted:updr=off_9");
      quick.push("lrs+10_5:4_aac=none:add=large:afr=on:afp=100000:afq=1.1:anc=none:bd=off:bsr=on:er=known:fsr=off:lma=on:nm=0:nwc=1:sas=z3:stl=30:sos=on:updr=off_17");
      quick.push("lrs+10_2_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:inw=on:nm=64:nwc=1:sas=z3:stl=30:sos=all:sp=occurrence:tha=off:thf=on:urr=on:updr=off:uhcvi=on_6");
      quick.push("lrs+1_5:4_aac=none:add=off:afr=on:afp=4000:afq=1.2:amm=sco:anc=none:gsp=input_only:gs=on:irw=on:nm=64:newcnf=on:nwc=1.3:nicw=on:sas=z3:stl=30:sp=occurrence:tha=off_34");
      quick.push("dis+1002_4_add=off:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1:sos=on:sac=on:sp=occurrence:tha=off:updr=off_3");
      quick.push("lrs+1011_1_add=off:afp=100000:afq=1.0:anc=none:cond=on:gs=on:gsaa=from_current:gsem=on:inw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sos=on:sp=occurrence:tha=off:uwa=ground_3");
      quick.push("dis+1_3:1_acc=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:fsr=off:gs=on:inw=on:lma=on:nm=32:nwc=1:urr=on_2");
      quick.push("dis+1010_4_add=off:afp=100000:afq=1.0:anc=none:fsr=off:gs=on:gsem=off:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sac=on:tha=off:thf=on_179");
      quick.push("dis+1002_4_add=large:afp=40000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsaa=full_model:lma=on:lwlo=on:nm=0:nwc=1.5:sas=z3:sp=reverse_arity:tha=off:thi=strong_17");
      quick.push("lrs+10_8:1_aac=none:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bd=off:cond=on:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=1.2:sas=z3:stl=30:sos=all:sp=reverse_arity:tha=off:updr=off_68");
      quick.push("dis+1011_5_aac=none:add=large:afp=40000:afq=1.2:amm=off:anc=none:bd=off:fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:updr=off_26");
      quick.push("dis-2_2:3_add=large:afp=40000:afq=1.4:amm=off:anc=none:gsp=input_only:gs=on:gsem=on:inw=on:lcm=reverse:lma=on:nm=2:nwc=1:nicw=on:sas=z3:sos=all:sp=reverse_arity:tha=off:urr=on_5");
      quick.push("ott+10_8_add=large:afp=100000:afq=1.4:amm=sco:cond=fast:fsr=off:fde=none:lcm=predicate:lma=on:nm=32:nwc=1:sos=on:sac=on:tha=off:updr=off_29");
      quick.push("dis-3_4_add=off:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:inw=on:lma=on:nm=64:nwc=1.5:nicw=on:sas=z3:sp=reverse_arity:tha=off:thf=on:uhcvi=on_13");
      quick.push("ott+10_3:2_aac=none:add=large:afp=10000:afq=2.0:amm=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=reverse:lma=on:nm=0:nwc=4:sas=z3:updr=off_11");
      quick.push("dis-1_2:1_afr=on:afp=10000:afq=2.0:anc=none:cond=on:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:inw=on:irw=on:lcm=predicate:lma=on:nm=32:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:uwa=ground_4");
      quick.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_12");
      quick.push("ott+1004_5_av=off:bd=off:bs=on:cond=on:fde=none:lma=on:nm=64:nwc=1:sos=on:sp=reverse_arity:tha=off:thi=strong:uwa=one_side_constant_143");
      quick.push("lrs+10_4:1_av=off:bd=off:bsr=on:cond=on:fde=unused:inw=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=5:stl=90:sp=reverse_arity:thi=strong:uwa=ground:updr=off:uwaf=on_359");
      quick.push("dis+1010_24_aac=none:afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:nm=6:nwc=1:sas=z3:sos=on:sp=reverse_arity:tha=off_9");
      quick.push("dis+11_3_afp=100000:afq=2.0:amm=sco:anc=none:gs=on:gsem=off:inw=on:lma=on:nm=64:nwc=1:sas=z3:sd=10:ss=axioms:st=5.0:sp=occurrence:tha=off:updr=off_202");
      quick.push("dis+1011_2:3_add=off:afr=on:afp=4000:afq=1.4:anc=none:bs=unit_only:fsr=off:gs=on:gsem=on:lwlo=on:nm=16:nwc=1.3:nicw=on:sas=z3:sac=on:tha=off_260");
      quick.push("dis+11_3_add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on:updr=off_92");
      quick.push("lrs+1011_7_aac=none:add=large:afr=on:afp=40000:afq=1.4:bd=off:bs=on:bsr=on:fsr=off:inw=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:stl=60:sos=all:sp=reverse_arity:tha=off:updr=off:uhcvi=on_541");
    }
    else {
      quick.push("lrs+1011_2:1_add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:thf=on:urr=on:updr=off_2");
      quick.push("lrs+1_2:3_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:lma=on:nm=64:nwc=1.3:sas=z3:stl=30:sac=on:tha=off:uwa=one_side_interpreted:updr=off_2");
      quick.push("lrs+1010_4:1_aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=on:gs=on:gsem=on:irw=on:nm=0:nwc=2.5:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_2");
      quick.push("dis+10_5_add=off:afp=4000:afq=1.1:anc=none:cond=fast:ep=RSTC:fsr=off:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sp=reverse_arity:thi=all_3");
      quick.push("dis+4_3:2_afr=on:afp=1000:afq=1.1:amm=sco:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsem=off:inw=on:lcm=reverse:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:updr=off_4");
      quick.push("dis+10_5_add=large:amm=sco:anc=none:fsr=off:gs=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=all:tha=off:thf=on_2");
      quick.push("dis+11_4_afp=100000:afq=1.1:anc=none:cond=on:gs=on:gsaa=full_model:nm=64:nwc=1:sac=on:sp=reverse_arity:thi=all_2");
      quick.push("dis+11_3_add=off:afp=10000:afq=2.0:amm=sco:anc=none:ep=RST:gs=on:gsaa=from_current:gsem=on:inw=on:nm=64:nwc=1:sd=10:ss=axioms:st=5.0:sos=all:tha=off:updr=off:uhcvi=on_59");
      quick.push("lrs+1_1024_av=off:bs=on:fde=none:inw=on:irw=on:nm=64:nwc=1.2:stl=60:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_595");
      quick.push("dis+10_10_add=large:afp=4000:afq=1.1:amm=sco:anc=none:irw=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=on_30");
      quick.push("ott-2_64_add=large:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:bd=off:bs=unit_only:bsr=on:bce=on:cond=on:fde=none:gs=on:gsem=on:newcnf=on:nwc=1:sac=on:sp=occurrence:thf=on:updr=off:uhcvi=on_154");
      quick.push("dis+11_3_afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=preordered:bce=on:fsr=off:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1:sd=10:ss=axioms:st=5.0:sac=on:sp=occurrence:tha=off:urr=ec_only_85");
      quick.push("ott-11_3_add=large:afp=100000:afq=1.2:anc=none:bs=on:cond=fast:fde=none:gs=on:gsem=off:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:sp=occurrence:tha=off:urr=on:uhcvi=on_268");
    }
    // add very long final fallback strategy from UFLIA problems
    fallback.push("lrs+1011_2:1_afr=on:afp=10000:afq=2.0:amm=off:gsp=input_only:gs=on:inw=on:ile=on:nm=2:nwc=1:sas=z3:tha=off_300");
    fallback.push("ott+1010_2:1_acc=on:add=large:afr=on:afp=40000:afq=1.1:anc=none:gs=on:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("dis+2_1_add=large:afr=on:afp=1000:afq=1.2:anc=none:cond=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_300");
    fallback.push("ott+10_4:1_aac=none:add=off:afp=40000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1011_2_acc=on:afp=10000:afq=1.1:amm=sco:anc=none:ccuc=small_ones:cond=fast:fde=unused:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off:updr=off:uhcvi=on_300");
    fallback.push("ott+11_5:4_aac=none:add=large:afp=4000:afq=1.4:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=on:inw=on:ile=on:nm=2:newcnf=on:nwc=1:sas=z3:sos=on:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("dis+10_14_add=large:afp=4000:afq=1.1:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=none:inw=on:irw=on:lcm=predicate:nm=4:nwc=1.1:sos=on:sac=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1002_5:4_add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:updr=off_300");
    fallback.push("lrs+11_5:1_add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:er=known:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=predicate:lwlo=on:nm=64:newcnf=on:nwc=1.1:sac=on:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("dis+1011_5:1_afp=4000:afq=1.4:amm=off:anc=none:cond=on:fde=unused:gsp=input_only:ile=on:lma=on:nm=16:nwc=1:sos=on:sac=on:tha=off:urr=ec_only:uhcvi=on_300");
    fallback.push("dis+1010_1_add=off:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=6:nwc=1.3:nicw=on:sas=z3:tha=off:urr=ec_only_300");
    fallback.push("dis+1002_2_aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=all:bsr=on:fde=unused:inw=on:ile=on:lcm=reverse:nm=4:nwc=4:nicw=on:sos=theory:sac=on:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+1011_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off_300");
    fallback.push("ott+1002_3:1_av=off:bsr=on:ep=RS:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_3_add=off:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:bce=on:cond=fast:fde=unused:ile=on:lma=on:nm=6:nwc=2:nicw=on:sas=z3:sd=3:ss=axioms:st=2.0:sp=reverse_arity:tha=off_300");
    fallback.push("ott+1011_3:2_av=off:bd=off:bs=on:bce=on:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:tha=off:updr=off_300");
    fallback.push("dis+11_3:1_av=off:br=off:ep=R:fsr=off:gsp=input_only:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("lrs-2_8:1_add=large:afp=100000:afq=1.4:amm=sco:anc=none:bs=on:bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=4:nicw=on:sas=z3:sp=reverse_arity:updr=off_600");
    fallback.push("dis+1010_2_acc=on:afr=on:afp=100000:afq=1.2:anc=none:bsr=on:fsr=off:ile=on:irw=on:nm=16:newcnf=on:nwc=4:sp=occurrence:tha=off:urr=ec_only_300");
    fallback.push("ott+1010_1_add=large:afp=1000:afq=1.2:anc=none:bd=off:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("lrs+1002_2_add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:inw=on:lwlo=on:nm=32:newcnf=on:nwc=1:sos=theory:sac=on:sp=occurrence:urr=on_300");
    fallback.push("lrs-10_3_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=unused:gs=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.2:sas=z3:tha=off:urr=ec_only_300");
    fallback.push("dis+10_3_add=off:afp=100000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sp=reverse_arity:tha=off:thf=on:updr=off_300");
    fallback.push("lrs+2_4_add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:cond=on:ep=R:gs=on:gsaa=from_current:gsem=off:ile=on:lcm=reverse:lma=on:nm=2:nwc=1.1:sos=on:sac=on:tha=off:updr=off_300");
    fallback.push("lrs+2_1024_av=off:bd=off:bsr=on:cond=fast:fsr=off:fde=none:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:tha=off:thi=overlap:uwa=one_side_constant:updr=off:uhcvi=on_300");
    fallback.push("ott+1011_8:1_afr=on:afp=1000:afq=1.4:amm=sco:anc=none:bd=off:fsr=off:fde=unused:inw=on:ile=on:nm=2:nwc=1:nicw=on:sas=z3:sos=theory:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_600");
    fallback.push("lrs+10_2:3_afr=on:afp=1000:afq=1.1:bd=off:bce=on:cond=on:gsp=input_only:gs=on:gsaa=from_current:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:uwa=interpreted_only:updr=off:uhcvi=on_300");
    fallback.push("dis+10_32_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:fde=none:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sp=occurrence:tha=off:thi=full:uhcvi=on_300");
    fallback.push("ott+1010_2:1_add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:cond=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=1.5:sac=on:tha=off:updr=off_300");
    fallback.push("lrs-11_2:1_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.5:sas=z3:sp=reverse_arity:urr=on_300");
    fallback.push("dis+11_7_add=large:afr=on:afp=10000:afq=1.2:bd=off:bsr=on:cond=on:fsr=off:fde=unused:gs=on:ile=on:lcm=predicate:lma=on:nm=2:newcnf=on:nwc=3:sos=on:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1011_1_afp=40000:afq=1.2:anc=none:cond=on:gsp=input_only:ile=on:irw=on:lma=on:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+10_3_av=off:fde=unused:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1.7:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_600");
    fallback.push("lrs+11_2:1_add=off:anc=none:bsr=on:br=off:cond=on:er=filter:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=2:nwc=1:sas=z3:sos=all:sac=on:uwa=ground:urr=on_300");
    fallback.push("lrs+1003_3:2_afp=1000:afq=2.0:amm=off:anc=none:cond=on:gs=on:ile=on:lma=on:nm=6:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=all:updr=off_300");
    fallback.push("lrs+4_8:1_av=off:cond=on:gs=on:gsem=on:irw=on:nm=64:newcnf=on:nwc=1.1:sp=occurrence:tha=off:urr=on:updr=off_300");
    fallback.push("ott-4_5:4_aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:cond=fast:ile=on:irw=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=3:thf=on:urr=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1011_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=4:nwc=1:sac=on:tha=off:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_2:1_av=off:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sp=occurrence:tha=off:urr=ec_only:uhcvi=on_300");
    fallback.push("dis+1011_4_afr=on:afp=10000:afq=1.1:amm=off:anc=none:ep=RS:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1010_1_afp=1000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:br=off:cond=on:ile=on:irw=on:nm=2:nwc=1:nicw=on:sas=z3:sos=all:sp=reverse_arity:tha=off:urr=on:updr=off_300");
    fallback.push("lrs+10_24_afp=4000:afq=2.0:bd=off:bsr=on:bce=on:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:nwc=1.3:sp=occurrence:tha=off:uwa=one_side_constant:urr=ec_only_300");
    fallback.push("lrs-2_5:1_acc=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bd=off:cond=fast:gs=on:ile=on:nm=0:newcnf=on:nwc=3:sac=on:thf=on:urr=ec_only_300");
    fallback.push("lrs+1_5:1_add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity_300");
    fallback.push("lrs+1010_1_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:fsr=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on_300");
    fallback.push("lrs+1011_2:1_aac=none:add=off:afp=1000:afq=1.0:amm=off:bs=on:gs=on:gsaa=from_current:gsem=on:ile=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off_300");
    fallback.push("dis+1010_2_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:fsr=off:fde=none:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:sp=reverse_arity_300");
    fallback.push("ott+1011_3:1_add=off:afp=100000:afq=2.0:amm=off:anc=none:bs=unit_only:gs=on:gsem=on:irw=on:newcnf=on:nwc=1:sas=z3:tha=off_300");
    fallback.push("dis-3_7_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sp=occurrence:tha=off:thi=overlap:uwa=interpreted_only:uhcvi=on_300");
    fallback.push("dis+11_32_add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:er=filter:ile=on:lcm=predicate:lma=on:newcnf=on:nwc=5:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_2:1_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=preordered:ccuc=first:cond=fast:fsr=off:fde=unused:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=2:nicw=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+2_8_av=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1002_1_add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:fsr=off:gsp=input_only:inw=on:ile=on:lcm=predicate:lwlo=on:nm=64:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("ott+10_4:1_afp=100000:afq=1.1:anc=none:bd=off:inw=on:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sac=on:sp=occurrence:tha=off:urr=on:updr=off_300");
    fallback.push("dis+1011_5:1_afr=on:afp=10000:afq=1.2:amm=sco:bd=preordered:bs=unit_only:cond=on:fsr=off:inw=on:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=1.1:sd=7:ss=axioms:st=1.2:tha=off:uhcvi=on_300");
    fallback.push("dis+1011_3_aac=none:afr=on:afp=40000:afq=1.4:amm=off:bs=on:fsr=off:fde=none:gsp=input_only:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+4_4_av=off:bd=off:bs=unit_only:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:updr=off_300");
    fallback.push("lrs+11_10_add=off:afp=40000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bce=on:cond=fast:gsp=input_only:inw=on:lma=on:nm=4:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:urr=ec_only:uhcvi=on_300");
    fallback.push("ott+1011_2:3_av=off:bs=unit_only:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thi=all:uwa=all:urr=on:uhcvi=on_300");
    fallback.push("lrs+11_6_aac=none:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:cond=fast:ile=on:nm=16:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:uhcvi=on_300");
    fallback.push("ott+1011_2:3_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:bce=on:cond=fast:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1.1:sp=reverse_arity:tha=off:urr=on:updr=off_300");
    fallback.push("lrs+4_8:1_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:tha=off_300");
    fallback.push("lrs-2_3:1_add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=on:er=filter:fde=unused:ile=on:irw=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on:thi=strong:uhcvi=on_600");
    fallback.push("dis+1011_10_av=off:bd=off:cond=fast:er=known:inw=on:ile=on:irw=on:lma=on:nwc=1.7:sp=occurrence:tha=off:uhcvi=on_300");
    fallback.push("dis+1010_3_afp=10000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:tha=off:urr=on_300");
    fallback.push("lrs+2_3:2_av=off:cond=fast:inw=on:ile=on:nm=2:nwc=1:sos=theory:urr=on_300");
    fallback.push("lrs+11_2_av=off:cond=on:fsr=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("dis+10_2:1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=on:inw=on:ile=on:irw=on:nm=2:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
    fallback.push("ott+1003_12_add=large:anc=all:bd=preordered:bce=on:fde=none:lcm=reverse:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:uwa=ground_600");
    fallback.push("dis+1011_1_aac=none:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bce=on:cond=on:er=known:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=off:ile=on:newcnf=on:nwc=1:sas=z3:ss=axioms:st=2.0:sp=occurrence:tha=off:urr=ec_only_300");
    fallback.push("dis+1002_1_add=large:afp=4000:afq=1.2:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only:uhcvi=on_300");
    fallback.push("dis+2_4_afp=10000:afq=1.1:bd=off:bs=on:cond=on:er=filter:ile=on:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("lrs+1003_8:1_av=off:fsr=off:fde=unused:gsp=input_only:gs=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on_300");
    fallback.push("lrs+11_8:1_afp=100000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:nm=64:nwc=2:nicw=on:sac=on:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs-11_8:1_afr=on:afp=1000:afq=1.4:amm=off:anc=none:bd=off:bs=on:gs=on:ile=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only_300");
    fallback.push("lrs+10_3:1_add=large:afp=10000:afq=1.1:amm=off:anc=none:cond=on:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sd=5:ss=axioms:st=1.5:tha=off:urr=on_300");
    fallback.push("lrs+1010_3:1_av=off:bd=off:bsr=on:irw=on:nm=64:newcnf=on:nwc=1.7:sos=all:updr=off_300");
    fallback.push("dis+11_5:1_afr=on:afp=40000:afq=2.0:amm=sco:anc=all_dependent:cond=fast:fde=unused:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=2:nwc=1:sos=all:urr=on:uhcvi=on_300");
    fallback.push("lrs+1004_1_aac=none:add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=all_dependent:bd=off:cond=fast:fsr=off:gs=on:gsaa=from_current:lcm=reverse:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thf=on:urr=on:updr=off_300");
    fallback.push("lrs-10_3:2_aac=none:add=off:afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=16:nwc=1:nicw=on:sas=z3:sd=2:ss=axioms:sos=on:sp=occurrence:updr=off_600");
    fallback.push("ott+1002_2:1_add=large:afr=on:afp=100000:afq=1.1:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off_300");
    fallback.push("dis-4_7_acc=on:afp=40000:afq=1.4:anc=all_dependent:bsr=on:br=off:bce=on:ccuc=first:er=filter:fsr=off:fde=unused:gsp=input_only:ile=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:thi=full:uwa=ground:urr=on:updr=off:uhcvi=on_300");
    fallback.push("ott-1_1_acc=model:add=off:afr=on:afp=4000:afq=1.2:anc=all:bd=preordered:bs=unit_only:bsr=on:ccuc=first:gs=on:gsaa=from_current:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=occurrence:tha=off:thi=strong:updr=off_300");
    fallback.push("lrs+1011_5:4_av=off:bd=off:bs=on:cond=on:er=known:gs=on:gsem=on:inw=on:ile=on:lcm=reverse:nm=6:newcnf=on:nwc=1:sp=occurrence:tha=off:uhcvi=on_300");
    fallback.push("dis+1002_14_av=off:cond=fast:fde=unused:inw=on:ile=on:lma=on:nm=0:nwc=1:sos=all:sp=reverse_arity:tha=off:uwa=one_side_interpreted:uhcvi=on_300");
    fallback.push("dis+1011_8:1_add=off:afp=10000:afq=1.1:anc=none:bce=on:er=filter:gs=on:gsaa=from_current:gsem=off:inw=on:ile=on:lma=on:nm=2:nwc=3:sac=on:urr=on:updr=off_300");
    fallback.push("dis+4_4:1_add=off:afp=4000:afq=1.2:amm=sco:anc=none:br=off:cond=fast:ep=RS:fsr=off:inw=on:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on:urr=on:uhcvi=on_300");
    fallback.push("lrs+1002_1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+1002_2:1_aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=none:cond=fast:gs=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_3:1_afp=1000:afq=1.4:amm=off:anc=none:bsr=on:inw=on:ile=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:tha=off:urr=on:updr=off_300");
    fallback.push("ott+10_2:1_av=off:bd=off:br=off:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+10_24_av=off:bs=unit_only:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sd=7:ss=axioms:st=1.2:sp=occurrence:tha=off:thf=on:uhcvi=on_600");
    fallback.push("lrs+4_28_afp=10000:afq=1.2:amm=sco:anc=none:bd=off:bce=on:cond=on:fsr=off:ile=on:irw=on:lcm=reverse:nm=16:newcnf=on:nwc=2:sas=z3:sp=occurrence:tha=off:updr=off:uhcvi=on_600");
    return;
  }

  switch (cat) {
  case Property::EPR:
    if (prop != 0UL) {
      quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_57");
      quick.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_2");
      quick.push("lrs+1_16_add=off:afp=100000:afq=1.0:amm=off:cond=fast:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=2:nicw=on:stl=60:sd=7:ss=axioms:st=5.0:sos=theory:sp=reverse_arity:urr=ec_only_344");
      quick.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:stl=30:uhcvi=on_301");
      quick.push("dis+4_5_afp=1000:afq=1.1:amm=off:anc=none:bd=off:gs=on:irw=on:lcm=predicate:lma=on:nwc=1:sas=z3:sos=all:sp=occurrence_151");
      quick.push("dis+11_2:3_add=large:afp=10000:afq=1.2:anc=none:bd=off:bce=on:cond=fast:er=filter:fsr=off:fde=unused:gsp=input_only:nwc=5:sos=theory:sac=on:urr=on_245");
      quick.push("ins+10_1_av=off:bce=on:cond=on:fde=unused:gsp=input_only:igbrr=0.7:igrr=32/1:igrp=700:igrpq=2.0:igs=1010:igwr=on:lcm=reverse:nwc=1:ss=axioms:st=3.0:sos=theory:urr=on:uhcvi=on_139");
      quick.push("lrs+1011_4_aac=none:afp=40000:afq=1.2:anc=all:bd=off:bsr=on:br=off:cond=fast:fde=none:gs=on:lma=on:nwc=1:stl=20:sp=reverse_arity:urr=on:updr=off:uhcvi=on_1");
    }
    else if (atoms < 2000) {
      quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_54");
      quick.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_283");
      quick.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:stl=30:uhcvi=on_25");
      quick.push("lrs+1011_14_afr=on:afp=100000:afq=1.1:anc=none:bd=off:bsr=on:irw=on:nwc=1:sas=z3:stl=30_90");
      quick.push("lrs+1003_10_afp=4000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:bce=on:fde=unused:lma=on:nwc=1:nicw=on:stl=120:sac=on:urr=on:updr=off:uhcvi=on_417");
      quick.push("ins+11_128_av=off:cond=on:fsr=off:irw=on:igbrr=0.5:igpr=on:igrr=1/2:igrp=2000:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:sos=all:sp=occurrence:updr=off_194");
      quick.push("dis+1011_5_add=large:anc=none:bd=preordered:cond=on:fsr=off:fde=unused:lma=on:nwc=1:sos=theory:sp=occurrence:updr=off_1327");
    }
    else {
      quick.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_2528");
    }
    break;

  case Property::FEQ:
    if (atoms > 1000000) {
      quick.push("dis+11_2_av=off:cond=fast:ep=RST:fsr=off:lma=on:nm=16:nwc=1.2:sp=occurrence:updr=off_1200");
    }
    else if (atoms > 500000) {
      quick.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_188");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_137");
      quick.push("dis+11_3_add=off:afp=10000:afq=2.0:amm=sco:anc=none:ep=RST:gs=on:gsaa=from_current:gsem=on:inw=on:nm=64:nwc=1:sd=10:ss=axioms:st=5.0:sos=all:tha=off:updr=off:uhcvi=on_59");
      quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_58");
      quick.push("ott+11_3_afp=1000:afq=2.0:anc=none:fsr=off:irw=on:nwc=1.7:ss=axioms:st=1.5:sac=on:updr=off_137");
      quick.push("dis+4_8:1_add=large:afp=100000:afq=1.4:ep=RST:fde=unused:gsp=input_only:lcm=predicate:nwc=1:sos=all:sp=occurrence:updr=off:uhcvi=on_169");
      quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_76");
      quick.push("lrs+1011_1_av=off:cond=on:gs=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity:updr=off_78");
      quick.push("lrs-10_3:2_add=large:afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:nm=16:nwc=1.3:stl=30:sd=2:ss=axioms:st=5.0:sos=on:uhcvi=on_78");
      quick.push("dis-10_3:2_aac=none:afp=1000:afq=1.1:cond=on:fsr=off:lcm=reverse:lwlo=on:nm=16:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity:updr=off_69");
      quick.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_130");
      quick.push("dis+11_8_add=off:afp=10000:afq=1.2:amm=off:anc=none:cond=fast:ep=R:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=4:sd=1:ss=axioms:sac=on:updr=off:uhcvi=on_55");
      quick.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:sos=all:sp=occurrence_66");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_136");
      quick.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_199");
    }
    else if (atoms > 240000) {
      quick.push("lrs+1011_4:1_acc=model:add=large:afp=40000:afq=1.4:ccuc=first:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:irw=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:urr=on_42");
      quick.push("dis+11_3_add=off:afp=10000:afq=2.0:amm=sco:anc=none:ep=RST:gs=on:gsaa=from_current:gsem=on:inw=on:nm=64:nwc=1:sd=10:ss=axioms:st=5.0:sos=all:tha=off:updr=off:uhcvi=on_40");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_30");
      quick.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_63");
      quick.push("ott+11_2:1_add=off:afp=10000:afq=1.1:anc=none:cond=on:gsp=input_only:lcm=reverse:lwlo=on:nm=32:nwc=5:sp=occurrence:updr=off_296");
      quick.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_38");
      quick.push("dis+10_5_av=off:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_92");
      quick.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:stl=30:sd=2:ss=axioms:sos=all:urr=ec_only_42");
      quick.push("lrs+10_4_add=off:afp=100000:afq=2.0:anc=none:bd=off:er=known:gs=on:gsem=off:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=2.5:sas=z3:stl=30:sac=on:urr=ec_only:updr=off_44");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_294");
      quick.push("dis+1011_3_afp=100000:afq=1.1:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:irw=on:nm=6:nwc=1:sas=z3:sd=2:ss=axioms:sos=on:sac=on:sp=reverse_arity:updr=off_45");
      quick.push("lrs+1002_3_aac=none:acc=on:add=off:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1.1:nicw=on:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:updr=off_26");
      quick.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_49");
      quick.push("lrs-10_3:2_add=large:afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:nm=16:nwc=1.3:stl=30:sd=2:ss=axioms:st=5.0:sos=on:uhcvi=on_54");
      quick.push("dis+10_4_add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=predicate:lma=on:nm=64:nwc=1:sd=3:ss=axioms:sos=on:sp=reverse_arity_42");
      quick.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_39");
      quick.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_175");
      quick.push("dis+11_3_afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=preordered:bce=on:fsr=off:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1:sd=10:ss=axioms:st=5.0:sac=on:sp=occurrence:tha=off:urr=ec_only_85");
    }
    else if (atoms > 160000) {
      quick.push("dis+11_8:1_afp=100000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=off:lcm=reverse:lma=on:nm=64:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on_20");
      quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_40");
      quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_28");
      quick.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_40");
      quick.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_147");
      quick.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_30");
      quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_24");
      quick.push("lrs+1002_1_av=off:er=filter:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:st=1.5:sos=on_1");
      quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_50");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_60");
      quick.push("lrs+1_2:3_aac=none:afr=on:afp=100000:afq=1.0:amm=sco:bd=off:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_1");
      quick.push("lrs-1_8:1_afp=40000:afq=1.4:amm=sco:anc=none:cond=on:gs=on:irw=on:lcm=reverse:nm=32:nwc=1:sas=z3:stl=30:sd=2:ss=axioms:st=2.0:sos=all:sac=on:sp=reverse_arity_1");
      quick.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:stl=30:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_28");
      quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_37");
      quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_32");
      quick.push("dis+11_2:1_add=large:afp=4000:afq=1.4:amm=sco:anc=none:fsr=off:nm=16:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_127");
      quick.push("lrs+1_7_av=off:cond=fast:fde=none:gs=on:gsem=off:lcm=predicate:nm=6:nwc=1:stl=30:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_23");
      quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_251");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_165");
      quick.push("ins+11_8_av=off:cond=fast:fde=none:gsp=input_only:irw=on:igbrr=0.2:igpr=on:igrr=1/8:igrp=200:igrpq=1.1:igs=1010:igwr=on:lcm=predicate:lma=on:nm=4:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:uhcvi=on_40");
      quick.push("ott+2_40_av=off:bs=unit_only:bsr=on:cond=fast:ep=RST:lma=on:nm=16:nwc=1:sp=reverse_arity_177");
      quick.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_247");
    }
    else if (atoms > 130000) {
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_43");
      quick.push("lrs+11_3_av=off:cond=on:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=5:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only_25");
      quick.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_25");
      quick.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:stl=30:sp=occurrence_58");
      quick.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_14");
      quick.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_19");
      quick.push("lrs+11_50_afp=100000:afq=1.1:amm=sco:anc=none:bs=unit_only:cond=on:irw=on:lma=on:nm=32:nwc=1.1:stl=30:sp=reverse_arity_252");
      quick.push("dis+1011_8:1_aac=none:acc=on:afp=1000:afq=1.4:amm=off:anc=all:bs=unit_only:bce=on:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:lma=on:nm=16:nwc=2.5:sd=4:ss=axioms:st=1.5:sos=all:uhcvi=on_100");
      quick.push("dis+10_4_add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=predicate:lma=on:nm=64:nwc=1:sd=3:ss=axioms:sos=on:sp=reverse_arity_17");
      quick.push("ott+11_2:1_av=off:bd=off:bsr=on:br=off:cond=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_41");
      quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_14");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_48");
      quick.push("dis+10_10_add=large:afp=4000:afq=1.1:amm=sco:anc=none:irw=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=on_30");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_290");
      quick.push("lrs-10_3:2_add=large:afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:nm=16:nwc=1.3:stl=30:sd=2:ss=axioms:st=5.0:sos=on:uhcvi=on_301");
      quick.push("ott+11_8:1_acc=model:afp=1000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=16:nwc=1.5:sac=on:sp=occurrence:urr=on_50");
      quick.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:sos=all:sp=occurrence_16");
      quick.push("ott+11_8_amm=off:anc=none:bsr=on:cond=on:irw=on:nm=2:nwc=1.1:ss=axioms:st=2.0:sac=on_39");
      quick.push("dis+1002_5_av=off:cond=on:gs=on:lma=on:nm=2:nwc=1:sd=3:ss=axioms:st=1.5:sos=on:updr=off_28");
      quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_272");
      quick.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:sos=all_172");
    }
    else if (atoms > 80000) {
      quick.push("lrs+1011_4:1_acc=model:add=large:afp=40000:afq=1.4:ccuc=first:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:irw=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:urr=on_29");
      quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_21");
      quick.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:stl=30:sd=2:ss=axioms:sos=all:urr=ec_only_23");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_13");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_25");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_15");
      quick.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_83");
      quick.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_20");
      quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_11");
      quick.push("lrs+11_1_av=off:bsr=on:gsp=input_only:gs=on:lcm=predicate:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_10");
      quick.push("lrs+4_4:1_acc=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:ccuc=first:cond=on:fsr=off:irw=on:lcm=predicate:lma=on:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all_11");
      quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_16");
      quick.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:stl=30:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_12");
      quick.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_15");
      quick.push("ins+11_8_av=off:cond=fast:fde=none:gsp=input_only:irw=on:igbrr=0.2:igpr=on:igrr=1/8:igrp=200:igrpq=1.1:igs=1010:igwr=on:lcm=predicate:lma=on:nm=4:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:uhcvi=on_73");
      quick.push("lrs-10_3_add=large:afp=100000:afq=1.2:bce=on:fde=none:lma=on:nm=32:nwc=5:stl=30:tha=off:updr=off_202");
      quick.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_15");
      quick.push("ott+2_8:1_aac=none:acc=on:add=large:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:fsr=off:lcm=predicate:lma=on:nm=2:nwc=1.7:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_25");
      quick.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_7");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_158");
      quick.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=2.0:sac=on:urr=on_115");
      quick.push("lrs+1011_50_afr=on:afp=40000:afq=1.0:amm=off:anc=all_dependent:bs=on:bsr=on:bce=on:fde=unused:gs=on:lma=on:nm=16:nwc=1.1:stl=60:sp=occurrence:updr=off_474");
      quick.push("ott+1010_5:4_av=off:bd=off:fde=none:irw=on:lma=on:nm=32:nwc=2.5:sd=2:ss=axioms:st=3.0:urr=on_190");
    }
    else if (atoms > 20000) {
      quick.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_11");
      quick.push("ott+1002_128_av=off:bd=off:bs=on:bsr=on:cond=on:fsr=off:nm=6:newcnf=on:nwc=1:sp=reverse_arity:updr=off_82");
      quick.push("dis+1011_10_av=off:cond=on:lma=on:nm=0:newcnf=on:nwc=1:sos=on:sp=occurrence:updr=off_17");
      quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_9");
      quick.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_6");
      quick.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_4");
      quick.push("lrs-10_3_add=large:afp=100000:afq=1.2:bce=on:fde=none:lma=on:nm=32:nwc=5:stl=30:tha=off:updr=off_48");
      quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_10");
      quick.push("lrs-10_3:2_add=large:afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:nm=16:nwc=1.3:stl=30:sd=2:ss=axioms:st=5.0:sos=on:uhcvi=on_10");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_26");
      quick.push("lrs+4_1_acc=on:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bs=on:bsr=on:ccuc=first:fsr=off:fde=unused:irw=on:lma=on:nm=0:nwc=1.3:stl=30:sd=10:ss=axioms:st=3.0:sos=all:sp=occurrence:uhcvi=on_26");
      quick.push("dis+10_10_add=large:afp=4000:afq=1.1:amm=sco:anc=none:irw=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=on_9");
      quick.push("dis+10_5_av=off:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_6");
      quick.push("ott+11_8_amm=off:anc=none:bsr=on:cond=on:irw=on:nm=2:nwc=1.1:ss=axioms:st=2.0:sac=on_8");
      quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_60");
      quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_76");
      quick.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_8");
      quick.push("dis-10_3:2_aac=none:afp=1000:afq=1.1:cond=on:fsr=off:lcm=reverse:lwlo=on:nm=16:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity:updr=off_27");
      quick.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_7");
      quick.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:stl=30:sd=2:ss=axioms:sos=all:urr=ec_only_13");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_7");
      quick.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_13");
      quick.push("lrs+1002_1_av=off:er=filter:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:st=1.5:sos=on_13");
      quick.push("dis+1011_2_afp=40000:afq=1.1:anc=none:nm=64:nwc=1:sp=reverse_arity:thf=on:urr=on_79");
      quick.push("dis+1011_3_afp=100000:afq=1.1:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:irw=on:nm=6:nwc=1:sas=z3:sd=2:ss=axioms:sos=on:sac=on:sp=reverse_arity:updr=off_7");
      quick.push("lrs+1011_50_afr=on:afp=40000:afq=1.0:amm=off:anc=all_dependent:bs=on:bsr=on:bce=on:fde=unused:gs=on:lma=on:nm=16:nwc=1.1:stl=60:sp=occurrence:updr=off_83");
      quick.push("lrs+1011_1_av=off:cond=on:gs=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity:updr=off_118");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_225");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_7");
      quick.push("lrs+11_4:1_av=off:bd=off:br=off:cond=fast:fde=unused:irw=on:lcm=reverse:lma=on:lwlo=on:nm=4:nwc=1:stl=60:sd=3:ss=axioms:sos=all:urr=on_69");
      quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_171");
      quick.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_29");
      quick.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:sos=all_229");
      quick.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:stl=30:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_50");
    }
    else if (prop == 2ul) {
      quick.push("dis+11_4_afr=on:afp=1000:afq=1.4:amm=off:anc=none:lcm=reverse:lma=on:lwlo=on:nm=6:newcnf=on:nwc=1:sd=2:ss=axioms:st=2.0:sp=reverse_arity_20");
      quick.push("dis+1010_3:2_acc=on:afr=on:afp=1000:afq=1.2:amm=sco:bs=on:ccuc=first:fde=none:nm=0:nwc=4:sd=3:ss=axioms:st=5.0:urr=ec_only_2");
      quick.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:stl=30:sp=occurrence_77");
      quick.push("dis+11_5:4_acc=on:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ccuc=small_ones:fsr=off:irw=on:nm=32:nwc=2.5:nicw=on:urr=on:updr=off_10");
      quick.push("dis+11_50_add=large:afp=1000:afq=1.0:amm=sco:anc=none:fsr=off:nm=16:nwc=1.5:sac=on_13");
      quick.push("lrs+11_14_add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:lma=on:nm=0:nwc=2:stl=30:sac=on:sp=reverse_arity_20");
      quick.push("ott+11_20_afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bsr=on:irw=on:lma=on:nm=4:nwc=1.2:sac=on:sp=occurrence_294");
      quick.push("lrs+11_24_add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=fast:fde=unused:gs=on:irw=on:lma=on:nm=4:nwc=1.3:nicw=on:sas=z3:stl=30:sac=on:sp=reverse_arity:uhcvi=on_136");
      quick.push("dis+1_24_av=off:cond=on:irw=on:lma=on:nm=2:nwc=1.5:sp=occurrence:updr=off_16");
      quick.push("lrs+10_24_add=off:afr=on:afp=1000:afq=1.4:anc=none:bs=unit_only:gs=on:gsaa=from_current:gsem=on:lma=on:nm=2:nwc=1.1:stl=60:sac=on:uhcvi=on_461");
      quick.push("lrs+11_5_av=off:cond=on:fsr=off:lma=on:lwlo=on:nwc=1.2:sas=z3:stl=30:sp=reverse_arity:updr=off_123");
      quick.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_122");
      quick.push("ott+11_20_afp=10000:afq=1.1:anc=none:bs=unit_only:bsr=on:bce=on:fsr=off:gs=on:gsem=on:nwc=2.5:sas=z3:sp=occurrence:updr=off:uhcvi=on_401");
      quick.push("lrs+11_128_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:nwc=1.3:stl=30:sac=on:uhcvi=on_236");
      quick.push("ott+4_4:1_acc=model:add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=preordered:bs=unit_only:ccuc=first:cond=on:gs=on:nm=32:nwc=1.5:sac=on:urr=on:updr=off_355");
      quick.push("ott+10_7_av=off:bd=preordered:bs=unit_only:bsr=on:cond=fast:fsr=off:fde=unused:irw=on:lma=on:nm=32:nwc=1:sos=all:sp=reverse_arity:updr=off_142");
      quick.push("ott+1_28_av=off:bs=unit_only:cond=on:irw=on:nm=64:nwc=2:sp=occurrence:updr=off:uhcvi=on_1052");
      quick.push("lrs+1_128_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bs=unit_only:gs=on:gsem=off:lwlo=on:nm=2:nwc=1:sas=z3:stl=90:sac=on:sp=occurrence:urr=on:updr=off:uhcvi=on_520");
      quick.push("ott-1_50_afr=on:afp=1000:afq=1.2:amm=sco:anc=none:bsr=on:cond=fast:fsr=off:fde=none:irw=on:nm=16:nwc=1.5:updr=off:uhcvi=on_1883");
      quick.push("lrs+11_64_acc=model:afp=100000:afq=1.2:anc=all_dependent:bd=off:bs=unit_only:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:irw=on:lma=on:nwc=1:stl=150:sac=on:sp=reverse_arity:urr=on:updr=off_893");
    }
    else if (prop == 0ul) {
      if (atoms > 1200) {
        quick.push("lrs+1011_4:1_acc=model:add=large:afp=40000:afq=1.4:ccuc=first:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:irw=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:urr=on_42");
        quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_28");
        quick.push("lrs+10_4:1_add=large:afp=100000:afq=1.1:anc=none:ep=RST:fde=unused:gsp=input_only:nm=6:newcnf=on:nwc=1:stl=30:sos=all:sac=on:sp=reverse_arity:urr=ec_only_53");
        quick.push("lrs+1010_32_afr=on:anc=none:bd=off:fsr=off:gs=on:gsem=on:nwc=1.3:nicw=on:sas=z3:stl=30:updr=off_63");
        quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_22");
        quick.push("ins+1010_3_av=off:bd=off:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.3:igwr=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=1:ss=axioms:sos=on:sp=occurrence:updr=off_11");
        quick.push("lrs+4_14_afr=on:afp=4000:afq=1.1:anc=none:cond=on:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1.1:sas=z3:stl=30:sd=4:ss=axioms:st=1.2:sos=all:updr=off_51");
        quick.push("lrs+1011_2_acc=on:add=off:afp=1000:afq=1.1:amm=sco:anc=none:fsr=off:lma=on:nm=64:nwc=1:stl=30:sd=1:ss=axioms:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_16");
        quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_70");
        quick.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:stl=30:sp=occurrence:urr=on_118");
        quick.push("dis+11_1_av=off:gsp=input_only:lma=on:nm=4:nwc=1:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:urr=on_246");
        quick.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=2.0:sac=on:urr=on_174");
        quick.push("lrs+1011_1024_av=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=32:nwc=1:stl=90:sp=reverse_arity:uhcvi=on_122");
        quick.push("lrs+10_128_av=off:bs=on:cond=on:gs=on:gsem=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:stl=30:updr=off_207");
        quick.push("lrs+1_7_av=off:cond=fast:fde=none:gs=on:gsem=off:lcm=predicate:nm=6:nwc=1:stl=30:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_166");
        quick.push("dis+1011_5:1_add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:bs=unit_only:bce=on:ep=RS:fsr=off:fde=none:irw=on:lwlo=on:nwc=1:sas=z3:sos=theory:sac=on:updr=off:uhcvi=on_398");
        quick.push("lrs+1011_1024_av=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=32:nwc=1:stl=90:sp=reverse_arity:uhcvi=on_426");
        quick.push("lrs+1011_7_afp=100000:afq=1.0:amm=sco:anc=all_dependent:bs=unit_only:bsr=on:ep=RS:fde=none:gsp=input_only:gs=on:gsem=off:lwlo=on:nm=4:nwc=1:stl=120:sos=theory:sp=occurrence:uhcvi=on_412");
        quick.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:stl=30:sos=all:sp=occurrence:urr=on_1");
      }
      else {
        quick.push("lrs+1011_5:1_acc=on:amm=off:anc=all_dependent:bd=off:ccuc=small_ones:fde=unused:gs=on:gsaa=full_model:gsem=off:lcm=predicate:lwlo=on:nm=6:newcnf=on:nwc=2.5:stl=30:sp=occurrence:updr=off_3");
        quick.push("dis+1002_6_add=large:afp=40000:afq=2.0:bsr=on:cond=on:irw=on:lma=on:nm=2:nwc=2.5:nicw=on:sp=reverse_arity:updr=off_2");
        quick.push("dis-10_1_aac=none:afr=on:afp=10000:afq=1.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=4:newcnf=on:nwc=2:sp=occurrence_2");
        quick.push("lrs+1010_32_afr=on:anc=none:bd=off:fsr=off:gs=on:gsem=on:nwc=1.3:nicw=on:sas=z3:stl=30:updr=off_19");
        quick.push("lrs+1011_5_afr=on:afp=100000:afq=1.0:amm=off:anc=none:cond=on:lma=on:nm=6:nwc=1:sas=z3:stl=30:sac=on:urr=on_51");
        quick.push("ott+1_28_av=off:bs=unit_only:cond=on:irw=on:nm=64:nwc=2:sp=occurrence:updr=off:uhcvi=on_94");
        quick.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:stl=30:sac=on:sp=occurrence_6");
        quick.push("lrs+11_5_afr=on:afp=100000:afq=2.0:amm=off:anc=none:bsr=on:cond=on:fsr=off:gs=on:gsem=on:nm=16:nwc=1:stl=30:sp=occurrence:urr=on:updr=off_18");
        quick.push("lrs+11_3:2_add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bs=unit_only:cond=on:gs=on:gsaa=full_model:gsem=off:irw=on:lwlo=on:nm=32:nwc=1.2:sas=z3:stl=90:sac=on_214");
        quick.push("ott+1011_4:1_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=preordered:cond=fast:nm=32:newcnf=on:nwc=1.2:sas=z3:sac=on:sp=occurrence:urr=on:updr=off_122");
        quick.push("dis+11_8:1_afp=100000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=off:lcm=reverse:lma=on:nm=64:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on_1");
        quick.push("lrs+11_128_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:nwc=1.3:stl=30:sac=on:uhcvi=on_188");
        quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_136");
        quick.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:stl=30:sac=on:urr=on_9");
        quick.push("lrs+1004_5:4_aac=none:add=large:afp=100000:afq=1.4:anc=all_dependent:bd=off:cond=fast:gsp=input_only:gs=on:gsem=off:lma=on:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sp=occurrence:updr=off:uhcvi=on_99");
        quick.push("lrs-11_24_av=off:bd=off:bs=on:fsr=off:gs=on:gsem=off:lma=on:nm=4:nwc=1.7:stl=90:sp=occurrence:updr=off_61");
        quick.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:stl=30:sp=occurrence:urr=on_91");
        quick.push("ott+2_20_add=off:afp=10000:afq=2.0:anc=none:bs=unit_only:fde=unused:irw=on:lma=on:nm=4:nwc=1:sas=z3:sac=on:urr=ec_only:uhcvi=on_420");
        quick.push("ott+10_1024_afp=40000:afq=2.0:amm=off:anc=all:bd=preordered:bs=unit_only:fde=unused:irw=on:nm=16:nwc=1:sp=reverse_arity:uhcvi=on_403");
        quick.push("ott+11_50_add=large:afr=on:afp=40000:afq=1.1:amm=sco:anc=all:bd=off:cond=fast:gs=on:gsem=on:irw=on:lma=on:nm=0:nwc=1:sp=occurrence:updr=off:uhcvi=on_102");
        quick.push("ott+10_8:1_av=off:bd=preordered:bsr=on:cond=fast:fsr=off:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1.2:sp=reverse_arity:urr=on:updr=off:uhcvi=on_713");
        quick.push("lrs+10_5:1_add=large:afp=1000:afq=1.0:amm=sco:anc=none:bs=unit_only:bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:irw=on:lwlo=on:nm=2:nwc=1:stl=210:uhcvi=on_1759");
        quick.push("ott+10_1024_afp=40000:afq=2.0:amm=off:anc=all:bd=preordered:bs=unit_only:fde=unused:irw=on:nm=16:nwc=1:sp=reverse_arity:uhcvi=on_823");
        quick.push("ott+4_1024_afr=on:afp=4000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:cond=fast:gsp=input_only:gs=on:gsaa=from_current:nm=16:nwc=1.1:nicw=on:sas=z3:sp=occurrence:updr=off_1389");
      }
    }
    else if (prop == 131073ul) {
      if (atoms > 300) {
        quick.push("dis+11_4_afr=on:afp=1000:afq=1.4:amm=off:anc=none:lcm=reverse:lma=on:lwlo=on:nm=6:newcnf=on:nwc=1:sd=2:ss=axioms:st=2.0:sp=reverse_arity_3");
        quick.push("dis+1011_10_aac=none:add=large:afp=10000:afq=1.1:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=6:newcnf=on:nwc=2.5:sp=reverse_arity:updr=off_3");
        quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_3");
        quick.push("ott+4_40_av=off:bce=on:cond=fast:fde=none:nm=0:nwc=1:sos=all:updr=off_2");
        quick.push("dis+11_10_av=off:lma=on:nm=64:nwc=5:sp=reverse_arity_3");
        quick.push("lrs+1011_3:2_aac=none:afp=10000:afq=1.2:amm=off:anc=all:cond=on:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_21");
        quick.push("lrs+2_3:1_afp=1000:afq=2.0:amm=off:anc=none:bs=unit_only:bsr=on:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:nm=6:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:sac=on:sp=reverse_arity:urr=ec_only:updr=off_3");
        quick.push("dis+11_5_afp=40000:afq=1.4:anc=none:br=off:cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1:sd=3:ss=axioms:st=2.0:urr=on:updr=off_6");
        quick.push("dis+1002_7_acc=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:newcnf=on:nwc=1.1:nicw=on:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_3");
        quick.push("ott+1002_128_av=off:bd=off:bs=on:bsr=on:cond=on:fsr=off:nm=6:newcnf=on:nwc=1:sp=reverse_arity:updr=off_10");
        quick.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:stl=30:sac=on:sp=occurrence_19");
        quick.push("lrs+1002_8_add=large:afp=40000:afq=1.0:amm=off:anc=none:cond=on:gs=on:irw=on:nm=16:newcnf=on:nwc=1:stl=30:sos=on:sp=reverse_arity:updr=off_6");
        quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_4");
        quick.push("lrs+1011_7_afp=100000:afq=1.0:amm=sco:anc=all_dependent:bs=unit_only:bsr=on:ep=RS:fde=none:gsp=input_only:gs=on:gsem=off:lwlo=on:nm=4:nwc=1:stl=120:sos=theory:sp=occurrence:uhcvi=on_17");
        quick.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_2");
        quick.push("ott+1_8_av=off:bd=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:lwlo=on:nwc=1:sos=on_10");
        quick.push("ins+11_8_av=off:cond=fast:fde=none:gsp=input_only:irw=on:igbrr=0.2:igpr=on:igrr=1/8:igrp=200:igrpq=1.1:igs=1010:igwr=on:lcm=predicate:lma=on:nm=4:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:uhcvi=on_3");
        quick.push("lrs+10_2:3_afp=1000:afq=1.1:amm=sco:anc=none:er=known:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=2:stl=30:sd=5:ss=axioms:sos=theory:sac=on:sp=occurrence_271");
        quick.push("dis+11_5_av=off:bd=off:bs=unit_only:bsr=on:cond=on:lcm=reverse:nm=0:nwc=1.2_5");
        quick.push("ott-1_3:1_av=off:irw=on:lcm=reverse:lma=on:nm=6:newcnf=on:nwc=1.2:sd=3:ss=axioms:st=3.0:sp=reverse_arity_20");
        quick.push("lrs+1011_3:2_av=off:er=known:lma=on:nm=2:newcnf=on:nwc=2:stl=30:sd=2:ss=axioms:st=3.0:urr=on:updr=off_5");
        quick.push("lrs-10_3:2_add=large:afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:nm=16:nwc=1.3:stl=30:sd=2:ss=axioms:st=5.0:sos=on:uhcvi=on_2");
        quick.push("dis-3_5_av=off:cond=on:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1:sos=on:sp=reverse_arity_3");
        quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_51");
        quick.push("lrs+1011_4:1_av=off:fsr=off:irw=on:nwc=1:stl=30:sd=4:ss=axioms:st=1.5:sp=reverse_arity_12");
        quick.push("dis+1010_2:3_afr=on:afp=40000:afq=1.4:amm=off:anc=none:lma=on:nm=16:nwc=1:sp=occurrence:updr=off:uhcvi=on_27");
        quick.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_26");
        quick.push("dis+11_28_av=off:fsr=off:irw=on:lcm=predicate:nm=2:newcnf=on:nwc=5:sp=occurrence:urr=on:updr=off_2");
        quick.push("dis+1011_24_add=large:afr=on:afp=4000:afq=1.0:anc=none:bs=unit_only:bce=on:cond=fast:gs=on:nm=32:nwc=2.5:nicw=on:sp=occurrence:updr=off_4");
        quick.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:stl=30:sos=theory:sp=occurrence:urr=ec_only:updr=off_36");
        quick.push("dis+1010_4_afp=10000:afq=1.2:anc=none:irw=on:lma=on:nm=64:nwc=10:sas=z3:sac=on:sp=reverse_arity:updr=off_2");
        quick.push("lrs+1011_14_av=off:fsr=off:irw=on:nwc=1:stl=30:sos=on:sp=occurrence:urr=ec_only:updr=off_53");
        quick.push("lrs+10_128_acc=model:afp=100000:afq=2.0:anc=all_dependent:bs=on:bsr=on:cond=fast:er=filter:gs=on:gsem=off:lcm=reverse:lma=on:nm=32:nwc=3:stl=30:sac=on:sp=occurrence:urr=ec_only_70");
        quick.push("lrs+10_6_aac=none:acc=model:add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bd=off:ccuc=small_ones:irw=on:lcm=reverse:nm=0:nwc=1:nicw=on:stl=30:sos=on:sp=reverse_arity:updr=off_2");
        quick.push("lrs+1_3:2_aac=none:add=large:anc=all_dependent:bce=on:cond=fast:ep=RST:fsr=off:lma=on:nm=2:nwc=1:stl=30:sos=on:sp=occurrence:urr=on:updr=off:uhcvi=on_5");
      }
      else if (atoms > 150) {
        quick.push("dis+1010_4_acc=on:afr=on:afp=1000:afq=2.0:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=small_ones:cond=fast:fsr=off:gs=on:gsem=off:lcm=reverse:lma=on:nm=64:nwc=2.5:nicw=on:sd=3:ss=axioms:st=3.0:sac=on:urr=ec_only:updr=off:uhcvi=on_2");
        quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_4");
        quick.push("lrs+11_2:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=16:nwc=1:stl=30:sd=1:ss=axioms:st=1.2:sp=reverse_arity_2");
        quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_2");
        quick.push("lrs+1010_3:2_afr=on:afp=100000:afq=1.1:anc=none:gsp=input_only:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1.7:stl=30:sac=on:sp=occurrence_49");
        quick.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_5");
        quick.push("lrs+1003_2_acc=on:afp=4000:afq=2.0:amm=sco:bd=off:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:nm=64:newcnf=on:nwc=2.5:nicw=on:stl=30:urr=ec_only_7");
        quick.push("ott-3_3_av=off:cond=fast:fde=none:lcm=reverse:nm=6:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:sp=reverse_arity:updr=off:uhcvi=on_17");
        quick.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_7");
        quick.push("dis+1002_8_add=large:afp=100000:afq=1.2:amm=off:bs=on:irw=on:nm=2:newcnf=on:nwc=1.1:sos=on:sp=reverse_arity:urr=ec_only:updr=off_259");
        quick.push("lrs+1_2:3_aac=none:afr=on:afp=100000:afq=1.0:amm=sco:bd=off:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_8");
        quick.push("dis+1010_1_acc=model:add=large:afr=on:amm=off:anc=none:bd=off:bsr=on:ccuc=small_ones:gs=on:gsem=on:nm=16:nwc=2:urr=ec_only:updr=off_56");
        quick.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_72");
        quick.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:stl=30:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_59");
        quick.push("lrs+1010_5:4_av=off:bd=off:bsr=on:irw=on:lwlo=on:newcnf=on:nwc=1.1:stl=90:sos=all:sp=occurrence:uhcvi=on_145");
        quick.push("ott+1_5:1_av=off:bsr=on:fsr=off:gs=on:lma=on:nm=6:nwc=1.1_173");
        quick.push("ott+11_4:1_add=large:afp=40000:afq=1.0:anc=none:br=off:cond=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sos=all:sp=reverse_arity:urr=on_8");
        quick.push("ott+1_5_afp=40000:afq=1.0:anc=all:fde=none:gs=on:irw=on:lma=on:nm=32:nwc=2:sos=all:sac=on:sp=occurrence:urr=ec_only:uhcvi=on_125");
        quick.push("ott+10_2:3_add=large:afp=40000:afq=1.1:amm=off:anc=all_dependent:bd=preordered:bs=unit_only:cond=fast:er=filter:gs=on:gsaa=from_current:lma=on:nm=32:nwc=1.1:sas=z3:sac=on:sp=occurrence:urr=ec_only:updr=off_679");
        quick.push("dis+1010_3:1_av=off:irw=on:nm=32:nwc=1:sos=all:urr=ec_only:updr=off_158");
        quick.push("ott-11_12_aac=none:afp=100000:afq=1.2:amm=sco:bs=on:bce=on:cond=fast:er=known:gs=on:gsaa=from_current:gsem=off:irw=on:nm=4:nwc=2:sas=z3:sos=all:sp=occurrence:urr=ec_only:updr=off_253");
      }
      else {
        quick.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_2");
        quick.push("lrs+4_3_av=off:br=off:nm=0:newcnf=on:nwc=1:stl=30:sp=occurrence:urr=on_32");
        quick.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_6");
        quick.push("dis+10_1_add=off:afp=4000:afq=1.2:anc=none:br=off:cond=on:gs=on:irw=on:lcm=reverse:nwc=10:sd=10:ss=axioms:sos=theory:sac=on:sp=occurrence:urr=on_2");
        quick.push("dis+11_2:1_add=large:afp=4000:afq=1.4:amm=sco:anc=none:fsr=off:nm=16:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_6");
        quick.push("dis+1011_4_av=off:cond=on:irw=on:lma=on:nm=2:nwc=1:sos=all:sp=occurrence_5");
        quick.push("lrs+1010_2:1_aac=none:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:gs=on:gsem=off:irw=on:nm=16:nwc=3:stl=30_3");
        quick.push("dis+1002_2_add=off:afr=on:afp=10000:afq=2.0:amm=off:anc=none:cond=on:er=filter:fsr=off:nm=0:nwc=1.3:sp=occurrence_4");
        quick.push("dis+11_1_acc=on:afp=1000:afq=1.4:amm=sco:anc=all_dependent:bs=on:ccuc=small_ones:cond=fast:fde=none:gsp=input_only:nm=64:nwc=1:sac=on:urr=ec_only:updr=off:uhcvi=on_105");
        quick.push("ott+4_32_av=off:bsr=on:cond=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=4:nwc=1.2:sos=theory_197");
        quick.push("lrs+1011_2:3_add=large:afr=on:afp=100000:afq=1.2:anc=none:gs=on:gsem=off:irw=on:nm=64:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity_48");
        quick.push("lrs+11_20_av=off:bs=unit_only:bsr=on:bce=on:cond=on:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=theory:sp=reverse_arity:uhcvi=on_123");
        quick.push("ott+11_8:1_acc=model:afp=1000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=16:nwc=1.5:sac=on:sp=occurrence:urr=on_104");
        quick.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_98");
        quick.push("dis+1010_5_add=large:afp=10000:afq=1.2:amm=off:bs=unit_only:bsr=on:bce=on:cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1.3:nicw=on:sos=all:sac=on:updr=off_34");
        quick.push("lrs+1002_3:1_av=off:bd=off:cond=fast:fde=none:gsp=input_only:lcm=predicate:lma=on:lwlo=on:nm=4:nwc=1.3:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity_59");
        quick.push("dis+1010_3:1_av=off:gsp=input_only:nm=6:nwc=1:sos=all:sp=occurrence_48");
        quick.push("lrs+4_14_afr=on:afp=4000:afq=1.1:anc=none:cond=on:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1.1:sas=z3:stl=30:sd=4:ss=axioms:st=1.2:sos=all:updr=off_7");
        quick.push("lrs-2_6_acc=on:afp=40000:afq=1.2:amm=sco:anc=none:bs=on:bsr=on:cond=on:fsr=off:fde=none:lcm=reverse:lma=on:nm=32:newcnf=on:nwc=2.5:nicw=on:stl=30:sp=reverse_arity:urr=ec_only_162");
        quick.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:stl=30:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_85");
        quick.push("lrs+1_5:1_afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:fde=none:gs=on:gsaa=full_model:gsem=on:irw=on:lwlo=on:nm=0:nwc=2.5:stl=30:sp=occurrence:uhcvi=on_42");
        quick.push("lrs+4_3_av=off:bd=preordered:bs=unit_only:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:lwlo=on:nm=6:nwc=1:stl=60:sp=occurrence:uhcvi=on_481");
        quick.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_145");
        quick.push("dis+11_40_aac=none:add=large:afp=10000:afq=1.2:amm=off:cond=fast:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:irw=on:nm=6:newcnf=on:nwc=1.2:sd=7:ss=axioms:st=1.5:sos=on:sp=reverse_arity:urr=on:uhcvi=on_1");
        quick.push("lrs+10_24_add=off:afr=on:afp=1000:afq=1.4:anc=none:bs=unit_only:gs=on:gsaa=from_current:gsem=on:lma=on:nm=2:nwc=1.1:stl=60:sac=on:uhcvi=on_364");
        quick.push("dis+2_128_add=large:afp=100000:afq=1.4:amm=sco:anc=none:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sos=theory:sac=on:updr=off_264");
        quick.push("lrs+3_64_add=large:afp=40000:afq=1.4:anc=none:bd=preordered:bsr=on:fde=unused:gs=on:gsaa=from_current:gsem=on:irw=on:lcm=predicate:lma=on:lwlo=on:nm=16:newcnf=on:nwc=1.2:stl=90:sos=theory_285");
      }
    }
    else if (prop == 131087ul) {
      quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_3");
      quick.push("ott+1_5:1_acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:lcm=predicate:nm=16:nwc=1.1:sd=1:ss=axioms:st=3.0:sos=on:sac=on:updr=off_18");
      quick.push("ott+10_1_av=off:bd=off:bsr=on:cond=on:fsr=off:irw=on:nm=2:nwc=1:sd=3:ss=axioms:sos=on:sp=occurrence:urr=on_26");
      quick.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_7");
      quick.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_15");
      quick.push("lrs+11_2:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=16:nwc=1:stl=30:sd=1:ss=axioms:st=1.2:sp=reverse_arity_11");
      quick.push("dis+1004_1_aac=none:acc=on:afp=40000:afq=1.2:anc=none:cond=on:fde=unused:gs=on:gsem=off:irw=on:nm=32:nwc=2:sd=1:ss=axioms:sos=theory:sp=reverse_arity:urr=ec_only_17");
      quick.push("dis+11_2_add=large:afr=on:anc=none:gs=on:gsem=on:lwlo=on:nm=16:nwc=1:sd=1:ss=axioms:st=3.0:sos=on_4");
      quick.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:stl=30:sos=all:sp=occurrence:urr=on_13");
      quick.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:stl=30:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_6");
      quick.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_9");
      quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_48");
      quick.push("dis+1011_8:1_aac=none:acc=on:afp=1000:afq=1.4:amm=off:anc=all:bs=unit_only:bce=on:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:lma=on:nm=16:nwc=2.5:sd=4:ss=axioms:st=1.5:sos=all:uhcvi=on_65");
      quick.push("lrs+1002_3_aac=none:acc=on:add=off:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1.1:nicw=on:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:updr=off_19");
      quick.push("dis+1010_2:3_afr=on:afp=40000:afq=1.4:amm=off:anc=none:lma=on:nm=16:nwc=1:sp=occurrence:updr=off:uhcvi=on_34");
      quick.push("dis+4_8:1_add=large:afp=100000:afq=1.4:ep=RST:fde=unused:gsp=input_only:lcm=predicate:nwc=1:sos=all:sp=occurrence:updr=off:uhcvi=on_33");
      quick.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_6");
      quick.push("lrs+1002_3:1_av=off:bd=off:cond=fast:fde=none:gsp=input_only:lcm=predicate:lma=on:lwlo=on:nm=4:nwc=1.3:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity_134");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_49");
      quick.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:sos=all:sp=occurrence_37");
      quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_41");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_6");
      quick.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_62");
      quick.push("ins+11_64_av=off:cond=fast:fde=none:gs=on:gsem=on:igbrr=0.7:igrr=1/2:igrp=4000:igrpq=1.2:igwr=on:lcm=predicate:lma=on:nwc=1.1:sd=2:ss=axioms:st=3.0:sos=on:sp=occurrence_38");
      quick.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:stl=30:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_8");
      quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_11");
      quick.push("lrs+1011_4:1_acc=model:add=large:afp=40000:afq=1.4:ccuc=first:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:irw=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:urr=on_21");
      quick.push("lrs-2_3:1_add=off:afr=on:afp=1000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bs=unit_only:bsr=on:cond=on:fde=unused:gs=on:gsem=on:irw=on:lcm=reverse:nm=32:nwc=1.7:sas=z3:stl=30:sos=all:sac=on_28");
      quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_9");
      quick.push("dis+1_3:2_acc=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:ccuc=small_ones:cond=on:lma=on:nm=64:nwc=1.3:sos=all:urr=on_3");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_41");
      quick.push("lrs+10_2:3_afp=1000:afq=1.1:amm=sco:anc=none:er=known:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=2:stl=30:sd=5:ss=axioms:sos=theory:sac=on:sp=occurrence_121");
      quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_8");
      quick.push("ins+10_1_av=off:fde=none:gsp=input_only:irw=on:igbrr=0.7:igpr=on:igrr=16/1:igrp=400:igrpq=2.0:igs=1003:igwr=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=3:sp=occurrence:uhcvi=on_62");
      quick.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_243");
      quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_4");
      quick.push("lrs+1_2:3_aac=none:afr=on:afp=100000:afq=1.0:amm=sco:bd=off:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_11");
      quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_254");
      quick.push("dis+1002_5_av=off:cond=on:gs=on:lma=on:nm=2:nwc=1:sd=3:ss=axioms:st=1.5:sos=on:updr=off_4");
      quick.push("lrs+1002_8:1_add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:er=known:lwlo=on:nm=0:nwc=1.2:stl=30:sd=1:ss=axioms:sp=occurrence:updr=off_51");
      quick.push("dis+10_1_add=off:afp=4000:afq=1.2:anc=none:br=off:cond=on:gs=on:irw=on:lcm=reverse:nwc=10:sd=10:ss=axioms:sos=theory:sac=on:sp=occurrence:urr=on_4");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_27");
      quick.push("dis+4_2_av=off:bs=on:fsr=off:gsp=input_only:newcnf=on:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sp=reverse_arity:urr=ec_only:updr=off_127");
      quick.push("dis+1_5:1_aac=none:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:br=off:gsp=input_only:gs=on:gsem=on:lcm=reverse:nm=6:nwc=1:nicw=on:sas=z3:sos=on:urr=on_9");
      quick.push("lrs+11_8:1_av=off:bd=preordered:br=off:cond=on:gs=on:gsem=on:lcm=reverse:lma=on:nm=0:nwc=1:stl=60:urr=on_362");
      quick.push("ins+1010_3_av=off:bd=off:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.3:igwr=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=1:ss=axioms:sos=on:sp=occurrence:updr=off_162");
      quick.push("dis+11_1_acc=on:afp=1000:afq=1.4:amm=sco:anc=all_dependent:bs=on:ccuc=small_ones:cond=fast:fde=none:gsp=input_only:nm=64:nwc=1:sac=on:urr=ec_only:updr=off:uhcvi=on_138");
      quick.push("lrs+11_2:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=16:nwc=1:stl=30:sd=1:ss=axioms:st=1.2:sp=reverse_arity_1");
      quick.push("ott-11_3_av=off:bsr=on:cond=fast:fde=unused:lcm=predicate:lma=on:nm=6:nwc=1:sos=on:updr=off_547");
    }
    else if (prop == 131075ul) {
      if (atoms > 2000) {
        quick.push("dis+11_8:1_afp=100000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=off:lcm=reverse:lma=on:nm=64:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on_11");
        quick.push("dis-1_2:3_av=off:cond=on:fsr=off:irw=on:lma=on:nwc=3:sd=3:ss=axioms:st=3.0:sp=occurrence:updr=off_9");
        quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_9");
        quick.push("lrs+2_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:gsp=input_only:lma=on:lwlo=on:newcnf=on:nwc=1:stl=30:sos=theory:sp=reverse_arity:updr=off_10");
        quick.push("lrs+1002_2:1_aac=none:afr=on:afp=1000:afq=1.2:anc=all:bd=preordered:bsr=on:cond=fast:gsp=input_only:gs=on:nm=0:nwc=2.5:nicw=on:sas=z3:stl=30:sd=4:ss=axioms:st=2.0:sos=on:sac=on:urr=on:updr=off:uhcvi=on_22");
        quick.push("dis+1002_8:1_av=off:br=off:cond=on:irw=on:lma=on:nm=32:nwc=1:sp=occurrence:urr=on_89");
        quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_11");
        quick.push("dis+1010_3:2_acc=on:afr=on:afp=1000:afq=1.2:amm=sco:bs=on:ccuc=first:fde=none:nm=0:nwc=4:sd=3:ss=axioms:st=5.0:urr=ec_only_75");
        quick.push("dis+11_3_av=off:fsr=off:lcm=predicate:lma=on:nm=4:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:updr=off_4");
        quick.push("ott-3_3_av=off:cond=fast:fde=none:lcm=reverse:nm=6:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:sp=reverse_arity:updr=off:uhcvi=on_88");
        quick.push("lrs+11_3_afr=on:afp=10000:afq=1.0:cond=fast:fsr=off:fde=none:gs=on:gsem=off:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:stl=30:sd=10:ss=axioms:st=2.0:sos=all:sp=reverse_arity:urr=on:uhcvi=on_23");
        quick.push("lrs+1002_3_aac=none:acc=on:add=off:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1.1:nicw=on:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:updr=off_2");
        quick.push("ott-1_3:1_av=off:irw=on:lcm=reverse:lma=on:nm=6:newcnf=on:nwc=1.2:sd=3:ss=axioms:st=3.0:sp=reverse_arity_51");
        quick.push("ott+1010_8:1_add=off:afp=4000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=4:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off:uhcvi=on_239");
        quick.push("lrs+1004_8:1_av=off:bd=off:fsr=off:lwlo=on:nm=4:nwc=1.5:stl=30:sd=1:ss=axioms:st=5.0:uhcvi=on_98");
        quick.push("ott+11_8:1_acc=model:afp=1000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=16:nwc=1.5:sac=on:sp=occurrence:urr=on_33");
        quick.push("lrs-1_14_add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:br=off:cond=fast:fde=unused:gs=on:lcm=reverse:lma=on:nwc=1:stl=30:sos=on:urr=on:updr=off_14");
        quick.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:stl=30:sos=theory:sp=occurrence:urr=ec_only:updr=off_223");
        quick.push("dis+1010_3:2_av=off:lma=on:nm=2:newcnf=on:nwc=1:sd=3:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_99");
      }
      else {
        quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_2");
        quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_3");
        quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_9");
        quick.push("dis+1002_2_add=off:afr=on:afp=10000:afq=2.0:amm=off:anc=none:cond=on:er=filter:fsr=off:nm=0:nwc=1.3:sp=occurrence_2");
        quick.push("lrs+1_7_av=off:irw=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=1:stl=30:sos=all:sp=occurrence:updr=off_36");
        quick.push("dis+1002_7_acc=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:newcnf=on:nwc=1.1:nicw=on:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_73");
        quick.push("ins+1010_3_av=off:bd=off:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.3:igwr=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=1:ss=axioms:sos=on:sp=occurrence:updr=off_8");
        quick.push("lrs+1010_2_add=large:afp=4000:afq=2.0:amm=off:bd=off:bs=unit_only:bsr=on:br=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=32:nwc=1.1:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_7");
        quick.push("dis+11_2_add=large:afr=on:anc=none:gs=on:gsem=on:lwlo=on:nm=16:nwc=1:sd=1:ss=axioms:st=3.0:sos=on_2");
        quick.push("dis+1010_7_afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:urr=ec_only:updr=off_83");
        quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_97");
        quick.push("dis+1002_7_acc=on:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:ccuc=first:fsr=off:gsp=input_only:gs=on:gsem=on:nm=6:nwc=1.1:nicw=on:sos=on:sac=on:sp=occurrence:urr=ec_only_217");
        quick.push("dis+1010_4_acc=on:afr=on:afp=1000:afq=2.0:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=small_ones:cond=fast:fsr=off:gs=on:gsem=off:lcm=reverse:lma=on:nm=64:nwc=2.5:nicw=on:sd=3:ss=axioms:st=3.0:sac=on:urr=ec_only:updr=off:uhcvi=on_83");
        quick.push("ott+1011_14_add=large:afr=on:afp=10000:afq=1.4:anc=none:bs=unit_only:cond=on:fde=unused:irw=on:nm=2:nwc=1.5:uhcvi=on_2");
        quick.push("lrs+4_1_acc=on:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bs=on:bsr=on:ccuc=first:fsr=off:fde=unused:irw=on:lma=on:nm=0:nwc=1.3:stl=30:sd=10:ss=axioms:st=3.0:sos=all:sp=occurrence:uhcvi=on_3");
        quick.push("lrs+1_8:1_av=off:cond=fast:fde=unused:lcm=predicate:nm=16:nwc=10:stl=60:sp=occurrence:urr=ec_only_3");
        quick.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:stl=30:sos=theory:sp=occurrence:urr=ec_only:updr=off_98");
        quick.push("lrs+10_8:1_av=off:bs=unit_only:cond=on:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:nwc=1.2:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_46");
        quick.push("lrs+1010_3:2_afr=on:afp=100000:afq=1.1:anc=none:gsp=input_only:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1.7:stl=30:sac=on:sp=occurrence_95");
        quick.push("dis+1010_3:2_acc=on:afr=on:afp=1000:afq=1.2:amm=sco:bs=on:ccuc=first:fde=none:nm=0:nwc=4:sd=3:ss=axioms:st=5.0:urr=ec_only_37");
        quick.push("dis+1002_1_av=off:bsr=on:cond=on:lma=on:nwc=2:sd=3:ss=axioms:st=3.0:updr=off_24");
        quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_91");
        quick.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:stl=30:sp=occurrence_66");
        quick.push("lrs+1010_5_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:bd=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=4:sas=z3:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_298");
        quick.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_171");
        quick.push("dis+1010_1_acc=model:add=large:afr=on:amm=off:anc=none:bd=off:bsr=on:ccuc=small_ones:gs=on:gsem=on:nm=16:nwc=2:urr=ec_only:updr=off_155");
      }
    }
    else if (prop == 131072ul) {
      quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_2");
      quick.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:stl=30:sac=on:urr=on_2");
      quick.push("lrs+1002_8_add=large:afp=40000:afq=1.0:amm=off:anc=none:cond=on:gs=on:irw=on:nm=16:newcnf=on:nwc=1:stl=30:sos=on:sp=reverse_arity:updr=off_2");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_2");
      quick.push("lrs+10_50_add=large:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gsp=input_only:gs=on:irw=on:lma=on:nm=64:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on_21");
      quick.push("lrs+1011_10_aac=none:acc=model:add=large:afp=40000:afq=2.0:anc=none:bd=off:bsr=on:fsr=off:gs=on:gsem=off:irw=on:lcm=reverse:lwlo=on:nm=64:nwc=3:nicw=on:stl=30_38");
      quick.push("dis+1010_1_acc=model:add=large:afr=on:amm=off:anc=none:bd=off:bsr=on:ccuc=small_ones:gs=on:gsem=on:nm=16:nwc=2:urr=ec_only:updr=off_3");
      quick.push("lrs+1010_2:1_aac=none:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:gs=on:gsem=off:irw=on:nm=16:nwc=3:stl=30_7");
      quick.push("lrs+1011_5_afr=on:afp=100000:afq=1.0:amm=off:anc=none:cond=on:lma=on:nm=6:nwc=1:sas=z3:stl=30:sac=on:urr=on_12");
      quick.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:stl=30:sp=occurrence:urr=on_129");
      quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_3");
      quick.push("lrs+1011_3:2_aac=none:afp=10000:afq=1.2:amm=off:anc=all:cond=on:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_5");
      quick.push("lrs+1_7_av=off:irw=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=1:stl=30:sos=all:sp=occurrence:updr=off_81");
      quick.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_9");
      quick.push("ott+1002_7_acc=on:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bs=unit_only:gsp=input_only:gs=on:nm=2:nwc=10:sos=theory:sp=reverse_arity:urr=on:updr=off_45");
      quick.push("lrs+1002_16_av=off:cond=on:nwc=3:stl=30_83");
      quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_11");
      quick.push("lrs+4_2_av=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:stl=30:sos=on:urr=on_159");
      quick.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_180");
      quick.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_169");
    }
    else if (prop & 131072ul) {
      quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_3");
      quick.push("ott+1002_8:1_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:fsr=off:gs=on:gsem=off:nm=32:nwc=10:sas=z3:sp=occurrence:urr=on:updr=off_14");
      quick.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_2");
      quick.push("dis+1_2:3_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:nm=64:newcnf=on:nwc=1_3");
      quick.push("lrs+1010_3_av=off:fsr=off:gs=on:gsem=off:nm=2:newcnf=on:nwc=2:stl=30:sp=reverse_arity:urr=on:updr=off_9");
      quick.push("dis+1010_3:2_av=off:gsp=input_only:nm=2:nwc=1:sp=reverse_arity:urr=ec_only_29");
      quick.push("dis+1002_6_add=large:afp=40000:afq=2.0:bsr=on:cond=on:irw=on:lma=on:nm=2:nwc=2.5:nicw=on:sp=reverse_arity:updr=off_10");
      quick.push("lrs+1011_7_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:cond=on:er=known:fsr=off:lma=on:nm=4:nwc=2.5:stl=30:sp=reverse_arity:updr=off_2");
      quick.push("lrs+11_4:1_aac=none:add=large:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:er=filter:fsr=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:newcnf=on:nwc=1.1:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_3");
      quick.push("dis+10_4_av=off:bsr=on:cond=fast:er=filter:fde=none:gsp=input_only:lcm=reverse:lma=on:nwc=4:sp=occurrence:urr=on_8");
      quick.push("dis+1011_24_add=large:afr=on:afp=4000:afq=1.0:anc=none:bs=unit_only:bce=on:cond=fast:gs=on:nm=32:nwc=2.5:nicw=on:sp=occurrence:updr=off_9");
      quick.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_11");
      quick.push("dis+1002_3:1_acc=model:afr=on:afp=40000:afq=1.1:anc=none:ccuc=first:fsr=off:gsp=input_only:irw=on:nm=16:nwc=1:sos=all_5");
      quick.push("dis+1011_5_add=off:afr=on:afp=10000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:nm=32:nwc=1:sas=z3:sd=3:ss=axioms:st=2.0:sp=occurrence:updr=off_2");
      quick.push("lrs+10_4:1_add=large:afp=100000:afq=1.1:anc=none:ep=RST:fde=unused:gsp=input_only:nm=6:newcnf=on:nwc=1:stl=30:sos=all:sac=on:sp=reverse_arity:urr=ec_only_22");
      quick.push("dis+1011_10_av=off:cond=on:lma=on:nm=0:newcnf=on:nwc=1:sos=on:sp=occurrence:updr=off_4");
      quick.push("ott+2_2_afp=10000:afq=1.4:amm=off:anc=none:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:nm=32:nwc=1.5:sos=on:sp=reverse_arity_18");
      quick.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_126");
      quick.push("lrs+1011_10_aac=none:acc=model:add=large:afp=40000:afq=2.0:anc=none:bd=off:bsr=on:fsr=off:gs=on:gsem=off:irw=on:lcm=reverse:lwlo=on:nm=64:nwc=3:nicw=on:stl=30_28");
      quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_130");
      quick.push("dis+11_4:1_afp=40000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:sos=on:sp=occurrence:urr=on:updr=off_83");
      quick.push("lrs+11_1_add=large:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:gs=on:gsem=on:irw=on:nm=32:newcnf=on:nwc=2.5:nicw=on:stl=30:sd=3:ss=axioms:sos=all:urr=on_34");
      quick.push("lrs+10_8:1_av=off:bs=unit_only:cond=on:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:nwc=1.2:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_12");
      quick.push("ott+1_5_afp=40000:afq=1.0:anc=all:fde=none:gs=on:irw=on:lma=on:nm=32:nwc=2:sos=all:sac=on:sp=occurrence:urr=ec_only:uhcvi=on_34");
      quick.push("lrs-11_12_av=off:nm=32:nwc=1.3:stl=30:sd=3:ss=axioms:sos=all_2");
      quick.push("dis+3_24_av=off:bd=off:bs=unit_only:fsr=off:fde=unused:gs=on:irw=on:lma=on:nm=0:nwc=1.1:sos=on:uhcvi=on_180");
      quick.push("dis+2_2:1_aac=none:afp=100000:afq=1.1:amm=sco:anc=none:bsr=on:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=on_6");
      quick.push("lrs+10_8:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lcm=predicate:lwlo=on:nm=64:nwc=1:stl=30:sos=all:updr=off_78");
      quick.push("ott+11_16_av=off:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.3:sas=z3:sp=reverse_arity_16");
      quick.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_275");
      quick.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:stl=30:sos=all:sp=occurrence:urr=on_8");
      quick.push("dis+1010_3:1_av=off:irw=on:nm=32:nwc=1:sos=all:urr=ec_only:updr=off_77");
      quick.push("lrs+10_3:1_av=off:bsr=on:cond=on:er=known:gs=on:lcm=reverse:nm=32:nwc=4:stl=30:sp=occurrence:urr=on:updr=off_73");
      quick.push("lrs+1010_3:2_afr=on:afp=100000:afq=1.1:anc=none:gsp=input_only:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1.7:stl=30:sac=on:sp=occurrence_84");
      quick.push("lrs+11_1_av=off:bsr=on:gsp=input_only:gs=on:lcm=predicate:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_245");
      quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_111");
      quick.push("lrs+2_2_aac=none:afr=on:afp=1000:afq=1.1:amm=sco:anc=all:bd=off:bce=on:cond=on:gs=on:gsaa=from_current:nm=2:nwc=2.5:stl=30:sac=on:urr=on_26");
    }
    else {
      quick.push("ott+1002_2_av=off:bd=preordered:irw=on:lma=on:nm=64:nwc=10:sp=reverse_arity:updr=off_2");
      quick.push("lrs+1002_1_av=off:er=filter:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:st=1.5:sos=on_1");
      quick.push("lrs+1010_8_add=off:afp=100000:afq=1.0:amm=off:anc=none:bce=on:irw=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:stl=30:sp=reverse_arity:urr=on_13");
      quick.push("ott+2_2_afp=10000:afq=1.4:amm=off:anc=none:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:nm=32:nwc=1.5:sos=on:sp=reverse_arity_7");
      quick.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_2");
      quick.push("lrs+11_4:1_av=off:bd=off:br=off:cond=fast:fde=unused:irw=on:lcm=reverse:lma=on:lwlo=on:nm=4:nwc=1:stl=60:sd=3:ss=axioms:sos=all:urr=on_2");
      quick.push("ott+11_20_afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bsr=on:irw=on:lma=on:nm=4:nwc=1.2:sac=on:sp=occurrence_6");
      quick.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:stl=30:sac=on:urr=on_41");
      quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_10");
      quick.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_2");
      quick.push("lrs+1011_2:3_add=large:afr=on:afp=100000:afq=1.2:anc=none:gs=on:gsem=off:irw=on:nm=64:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity_1");
      quick.push("dis+1002_3:1_acc=model:afr=on:afp=40000:afq=1.1:anc=none:ccuc=first:fsr=off:gsp=input_only:irw=on:nm=16:nwc=1:sos=all_8");
      quick.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_9");
      quick.push("lrs+11_3:2_add=large:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:er=filter:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:sas=z3:stl=30:sp=occurrence:urr=on:updr=off_100");
      quick.push("ott+4_40_av=off:bce=on:cond=fast:fde=none:nm=0:nwc=1:sos=all:updr=off_197");
      quick.push("ott+1_128_add=large:afr=on:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.1:nicw=on:sas=z3:sac=on:sp=reverse_arity_44");
      quick.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_143");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_113");
    }
    break;

  case Property::FNE:
    quick.push("lrs-11_4:1_afp=4000:afq=2.0:anc=none:br=off:gs=on:lwlo=on:nm=64:nwc=3:stl=30:urr=on_2");
    quick.push("dis+10_50_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:gs=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:thf=on:updr=off_2");
    quick.push("dis+11_3_afr=on:afp=4000:afq=1.4:anc=none:cond=on:fsr=off:gs=on:lcm=reverse:nm=64:nwc=1:sos=on:sp=reverse_arity_3");
    quick.push("lrs+4_32_add=large:afp=10000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lcm=predicate:lma=on:nm=2:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_11");
    quick.push("lrs+1_2:3_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:lma=on:nm=64:nwc=1.3:sas=z3:stl=30:sac=on:tha=off:uwa=one_side_interpreted:updr=off_2");
    quick.push("dis+10_3_afp=1000:afq=2.0:amm=off:anc=none:cond=on:gs=on:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sac=on:sp=reverse_arity:updr=off_2");
    quick.push("ins+11_32_av=off:igbrr=0.4:igrr=1/64:igrpq=1.05:igwr=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:updr=off_55");
    quick.push("lrs+10_5:4_afr=on:afp=40000:afq=1.2:bd=off:gsp=input_only:gs=on:inw=on:nm=0:nwc=1:sas=z3:stl=30:sos=all:sp=reverse_arity:tha=off:thf=on:urr=on_2");
    quick.push("dis+1011_10_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=64:nwc=4:sac=on:sp=occurrence_75");
    quick.push("lrs+10_4:1_av=off:bd=off:bsr=on:cond=on:fde=unused:inw=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=5:stl=90:sp=reverse_arity:thi=strong:uwa=ground:updr=off:uwaf=on_73");
    quick.push("dis+11_24_afp=40000:afq=1.1:amm=sco:anc=none:bs=on:gs=on:gsem=off:lcm=predicate:lma=on:nm=2:nwc=1:sos=on:sac=on:updr=off_91");
    quick.push("lrs+1011_1024_add=large:afp=4000:afq=1.1:anc=none:br=off:fsr=off:gsp=input_only:lma=on:nwc=1:stl=30:sos=on:urr=on_200");
    quick.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_125");
    quick.push("lrs+1_1_av=off:bsr=on:br=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sp=occurrence:urr=on:updr=off_152");
    quick.push("dis+1_8_afp=4000:afq=1.1:amm=sco:gsp=input_only:nm=64:newcnf=on:nwc=4:sac=on:sp=occurrence:updr=off_191");
    quick.push("ott+11_5:1_afp=100000:afq=1.1:br=off:gs=on:nm=64:nwc=1:sos=all:urr=on:updr=off_287");
    quick.push("lrs+1011_2:1_add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:thf=on:urr=on:updr=off_81");
    quick.push("dis+10_128_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=2:nwc=1:sp=reverse_arity_3");
    quick.push("ott+1_40_av=off:bs=unit_only:bsr=on:br=off:fsr=off:lma=on:nm=64:newcnf=on:nwc=1.5:sp=occurrence:urr=on:updr=off_81");
    quick.push("lrs+1011_40_add=off:afr=on:afp=4000:afq=1.2:amm=sco:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:stl=60:sos=all:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_388");
    break;

  case Property::NEQ:
    if (prop == 1) {
      quick.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_7");
      quick.push("lrs+11_2_afr=on:afp=100000:afq=1.4:amm=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:stl=30:sos=on:updr=off_3");
      quick.push("dis+10_10_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:gs=on:gsem=off:nwc=1:sas=z3:sd=3:ss=axioms:st=2.0:sp=occurrence:updr=off_2");
      quick.push("dis+1010_3:1_aac=none:add=large:afp=100000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:gs=on:lma=on:nwc=1:sos=all:sp=occurrence_18");
      quick.push("lrs+11_14_av=off:bd=off:bs=unit_only:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sos=on:sp=reverse_arity:urr=on:updr=off_46");
      quick.push("lrs+10_3_av=off:bd=off:cond=on:gs=on:gsem=off:irw=on:lwlo=on:nwc=1.2:stl=30:sp=reverse_arity:updr=off_72");
      quick.push("lrs+4_2_av=off:bs=on:er=known:gs=on:irw=on:lwlo=on:nwc=10:stl=30:sp=occurrence_243");
      quick.push("dis+4_5:1_av=off:nwc=1:sos=all:sp=reverse_arity:urr=on:updr=off_53");
      quick.push("dis-10_3:1_add=large:afr=on:afp=1000:afq=2.0:anc=none:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nwc=1:sas=z3:sd=2:ss=axioms:st=2.0:sos=all:sac=on:sp=reverse_arity:urr=on:uhcvi=on_84");
      quick.push("dis+1011_2:1_av=off:lma=on:nwc=1:sd=3:ss=axioms:st=5.0:sp=occurrence:urr=ec_only_30");
      quick.push("lrs+10_12_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=off:cond=on:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=1.3:sas=z3:stl=30:sac=on_71");
      quick.push("dis+1011_12_av=off:bd=off:bs=unit_only:fsr=off:lma=on:nwc=1:urr=ec_only:updr=off_18");
      quick.push("dis+11_5_afr=on:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:urr=ec_only:updr=off_6");
      quick.push("dis+11_4_aac=none:add=large:afp=100000:afq=1.0:amm=off:anc=none:bsr=on:cond=on:fsr=off:gs=on:lcm=reverse:lma=on:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=ec_only_3");
      quick.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bce=on:fde=unused:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_163");
      quick.push("dis-11_64_av=off:bd=off:bs=on:cond=on:fsr=off:nwc=1:sd=1:ss=axioms:urr=ec_only:updr=off_99");
      quick.push("lrs+1011_2:1_av=off:bce=on:cond=fast:fsr=off:fde=none:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:st=3.0:sp=occurrence:urr=ec_only:updr=off_144");
      quick.push("dis+1011_1_add=off:afp=100000:afq=1.4:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:nwc=1:nicw=on:sac=on:sp=occurrence:urr=on_77");
      quick.push("lrs+11_10_aac=none:acc=on:afp=4000:afq=1.2:amm=sco:anc=none:ccuc=first:fsr=off:irw=on:nwc=2:nicw=on:stl=30:sd=5:ss=axioms:st=1.2:sos=theory:urr=ec_only:updr=off_155");
      quick.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:stl=30:updr=off_2");
      quick.push("ott+10_4:1_av=off:bd=preordered:cond=fast:fde=unused:irw=on:lcm=reverse:nwc=3:sd=2:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_41");
      quick.push("lrs+1011_3:1_av=off:bs=unit_only:bsr=on:er=known:fsr=off:fde=unused:irw=on:lcm=reverse:lwlo=on:nwc=1.7:stl=30:sos=on:sp=occurrence:updr=off_229");
      quick.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_68");
      quick.push("lrs+1_20_add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=z3:stl=30:sd=3:ss=axioms:st=1.2:sp=occurrence:updr=off_136");
      quick.push("dis+10_3_acc=on:afr=on:afp=1000:afq=1.0:amm=sco:bs=unit_only:ccuc=first:fsr=off:irw=on:lcm=reverse:lma=on:nwc=1.5:updr=off_63");
      quick.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=off:anc=none:cond=on:irw=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0:sos=all:sp=occurrence_205");
      quick.push("lrs+11_5_av=off:cond=on:gs=on:gsem=on:irw=on:nwc=1:stl=30:sd=1:ss=axioms:sos=on:sp=occurrence:urr=on_4");
      quick.push("lrs-2_2:1_afr=on:afp=1000:afq=2.0:anc=none:bd=off:bce=on:cond=fast:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1.5:stl=30:sac=on:sp=reverse_arity:uhcvi=on_183");
    }
    else if (prop == 3) {
      quick.push("lrs+1_2:1_av=off:fsr=off:lma=on:nwc=1:stl=30:sd=7:ss=axioms:st=1.2:sos=on:urr=ec_only_12");
      quick.push("dis+11_5_afr=on:afp=1000:afq=1.0:amm=off:anc=none:irw=on:lcm=reverse:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sp=occurrence_2");
      quick.push("lrs+2_3:2_aac=none:acc=model:add=off:afr=on:afp=10000:afq=1.4:anc=none:bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=off:lcm=reverse:lma=on:nwc=3:stl=30:sd=3:ss=axioms:st=2.0:sac=on:urr=on_29");
      quick.push("dis-11_4:1_amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nwc=10:sac=on:sp=reverse_arity:urr=on_14");
      quick.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=off:anc=none:cond=on:irw=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0:sos=all:sp=occurrence_3");
      quick.push("dis+1011_1_add=off:afp=100000:afq=1.4:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:nwc=1:nicw=on:sac=on:sp=occurrence:urr=on_6");
      quick.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_5");
      quick.push("lrs+1011_2:1_av=off:bsr=on:cond=on:nwc=1:sas=z3:stl=30:sd=1:ss=axioms:st=1.2:sp=reverse_arity_8");
      quick.push("dis+1010_1024_afr=on:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:irw=on:lwlo=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0_3");
      quick.push("lrs+1010_3:1_av=off:cond=on:nwc=5:stl=30:sp=reverse_arity_18");
      quick.push("lrs+1011_4:1_av=off:cond=on:irw=on:lma=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_124");
      quick.push("lrs+1_3_av=off:fsr=off:lma=on:nwc=1:stl=30:sd=1:ss=axioms:sp=occurrence:urr=on_22");
      quick.push("lrs+1002_3:1_av=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity_4");
      quick.push("lrs+1011_2:1_av=off:cond=on:er=known:gs=on:gsem=on:lwlo=on:nwc=1.3:stl=30:updr=off:uhcvi=on_180");
      quick.push("dis+1011_3_av=off:nwc=1:sos=all:sp=reverse_arity_67");
      quick.push("dis-11_5:1_acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=first:cond=on:gs=on:gsem=off:nwc=1:sd=3:ss=axioms:st=1.2:sos=all_76");
      quick.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_168");
      quick.push("lrs+10_5:1_add=large:afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_5");
      quick.push("lrs+1_32_av=off:bd=off:br=off:gs=on:gsem=on:irw=on:nwc=1:stl=30:sd=1:ss=axioms:sp=occurrence:urr=on:updr=off_9");
      quick.push("lrs+1010_8:1_av=off:bs=unit_only:br=off:cond=on:fsr=off:irw=on:nwc=1.3:stl=30:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=on:updr=off_62");
      quick.push("lrs+1011_5:4_av=off:bd=off:fsr=off:gs=on:nwc=1.3:stl=30:urr=ec_only:updr=off_91");
      quick.push("dis+11_2_av=off:gs=on:gsem=on:irw=on:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=reverse_arity_6");
      quick.push("lrs+1011_40_add=off:afp=1000:afq=2.0:anc=none:bs=unit_only:fsr=off:irw=on:nwc=1:sas=z3:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_86");
      quick.push("lrs-11_3_aac=none:acc=on:afr=on:afp=10000:afq=1.1:bd=off:bs=unit_only:ccuc=first:irw=on:lcm=predicate:lma=on:nwc=1.5:stl=30:sos=all:sac=on:sp=occurrence:updr=off_111");
      quick.push("dis+1011_2:1_av=off:lma=on:nwc=1:sd=3:ss=axioms:st=5.0:sp=occurrence:urr=ec_only_154");
      quick.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:stl=30:updr=off_287");
      quick.push("lrs+1003_3_av=off:bd=off:fde=none:gs=on:lma=on:nwc=1:stl=30:sd=7:ss=axioms:st=1.2:sos=all:sp=reverse_arity:updr=off:uhcvi=on_103");
      quick.push("dis+2_5:4_add=large:afp=4000:afq=1.2:anc=all:bce=on:cond=fast:fde=none:lma=on:nwc=10:sd=1:ss=axioms:st=1.5:sac=on_9");
      quick.push("lrs+1004_20_av=off:cond=on:er=filter:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:sos=on:urr=ec_only_57");
      quick.push("dis+11_3_av=off:bd=off:bsr=on:bce=on:gsp=input_only:gs=on:gsem=on:lma=on:nwc=2.5:sp=reverse_arity:urr=ec_only:uhcvi=on_205");
      quick.push("dis+1002_5:1_av=off:cond=fast:fsr=off:fde=none:lma=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:urr=on:updr=off_19");
      quick.push("lrs-1_14_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=ec_only:updr=off_175");
      quick.push("lrs+1011_2:1_av=off:bce=on:cond=fast:fsr=off:fde=none:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:st=3.0:sp=occurrence:urr=ec_only:updr=off_102");
      quick.push("ins+11_3_av=off:bd=off:igbrr=0.6:igrr=1/8:igrp=700:igrpq=2.0:igs=1:igwr=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:updr=off_3");
    }
    else if (prop == 131079) {
      quick.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_13");
      quick.push("lrs+1011_2_acc=on:add=off:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:er=known:nwc=4:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off_13");
      quick.push("dis+11_12_av=off:bsr=on:cond=on:lcm=predicate:lma=on:nwc=5:sp=reverse_arity:updr=off_45");
      quick.push("dis+11_3_afp=100000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=10:sac=on_17");
      quick.push("lrs+4_2_av=off:bs=on:er=known:gs=on:irw=on:lwlo=on:nwc=10:stl=30:sp=occurrence_100");
      quick.push("lrs+4_1_av=off:bd=off:bsr=on:bce=on:fsr=off:irw=on:lcm=predicate:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on:uhcvi=on_39");
      quick.push("dis+10_3_av=off:cond=on:irw=on:lma=on:nwc=1.7:sp=occurrence:updr=off_12");
      quick.push("lrs+1011_2:1_av=off:cond=on:lwlo=on:nwc=1.5:stl=30_24");
      quick.push("ott+1011_2:3_av=off:cond=fast:er=filter:fde=unused:gsp=input_only:irw=on:nwc=3:sp=occurrence:updr=off:uhcvi=on_4");
      quick.push("lrs+1011_5_av=off:cond=on:er=filter:gs=on:nwc=1.7:stl=30:updr=off_17");
      quick.push("lrs+11_5_add=large:afr=on:afp=40000:afq=1.2:cond=on:er=known:gs=on:gsem=off:nwc=1.5:stl=30:sp=occurrence:updr=off_65");
      quick.push("dis+1011_3_av=off:nwc=1:sos=all:sp=reverse_arity_114");
      quick.push("lrs+11_1_av=off:fsr=off:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=reverse_arity:urr=on:updr=off_44");
      quick.push("lrs+10_2:1_av=off:bsr=on:cond=on:er=known:irw=on:lcm=predicate:nwc=4:stl=30:sp=occurrence_5");
      quick.push("lrs+4_5_av=off:bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:nwc=2.5:stl=30:sp=occurrence:updr=off_139");
      quick.push("lrs+1011_10_av=off:bs=unit_only:bsr=on:er=filter:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nwc=1.2:stl=30:sp=reverse_arity:updr=off_105");
      quick.push("lrs+1011_2_av=off:bd=preordered:bs=unit_only:cond=fast:fsr=off:fde=unused:irw=on:lma=on:lwlo=on:nwc=1.2:stl=30:sp=occurrence:uhcvi=on_90");
      quick.push("lrs+1011_3:1_av=off:bs=unit_only:bsr=on:er=known:fsr=off:fde=unused:irw=on:lcm=reverse:lwlo=on:nwc=1.7:stl=30:sos=on:sp=occurrence:updr=off_3");
      quick.push("ott+11_6_av=off:bs=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=10:sp=reverse_arity_100");
      quick.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_29");
      quick.push("dis+11_2:1_acc=model:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:er=known:gsp=input_only:irw=on:lma=on:nwc=5:sac=on:urr=ec_only_148");
      quick.push("lrs+1011_5:1_afp=100000:afq=1.0:anc=none:bd=off:gsp=input_only:gs=on:gsem=off:lwlo=on:nwc=5:nicw=on:sas=z3:stl=30:sac=on:updr=off_176");
      quick.push("ott+1011_8:1_av=off:bd=off:cond=on:nwc=1:sos=all:sp=reverse_arity_182");
    }
    else {
      quick.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:stl=30:updr=off_3");
      quick.push("dis+1011_32_aac=none:afp=10000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=3:sas=z3:sp=reverse_arity_3");
      quick.push("lrs+10_5:1_add=large:afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_5");
      quick.push("lrs+11_2_afr=on:afp=100000:afq=1.4:amm=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:stl=30:sos=on:updr=off_5");
      quick.push("dis-11_5:1_acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=first:cond=on:gs=on:gsem=off:nwc=1:sd=3:ss=axioms:st=1.2:sos=all_7");
      quick.push("dis+1011_8_av=off:bd=off:cond=fast:er=known:fde=unused:gsp=input_only:lcm=predicate:lma=on:nwc=1.2:sp=reverse_arity:updr=off:uhcvi=on_33");
      quick.push("dis+2_3_av=off:cond=on:fsr=off:lcm=reverse:lma=on:nwc=1:sos=on:sp=reverse_arity_6");
      quick.push("dis+1002_4_afr=on:afp=1000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=off:lma=on:nwc=1:sos=on:sac=on:sp=occurrence_45");
      quick.push("lrs+1011_2_acc=on:add=off:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:er=known:nwc=4:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off_3");
      quick.push("lrs+1011_5:1_afp=100000:afq=1.0:anc=none:bd=off:gsp=input_only:gs=on:gsem=off:lwlo=on:nwc=5:nicw=on:sas=z3:stl=30:sac=on:updr=off_67");
      quick.push("lrs+1011_2_av=off:bd=preordered:bs=unit_only:cond=fast:fsr=off:fde=unused:irw=on:lma=on:lwlo=on:nwc=1.2:stl=30:sp=occurrence:uhcvi=on_73");
      quick.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_8");
      quick.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_80");
      quick.push("dis+11_12_av=off:bsr=on:cond=on:lcm=predicate:lma=on:nwc=5:sp=reverse_arity:updr=off_1");
      quick.push("lrs+1010_3:2_av=off:fsr=off:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=ec_only_101");
      quick.push("dis+11_2:1_acc=model:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:er=known:gsp=input_only:irw=on:lma=on:nwc=5:sac=on:urr=ec_only_25");
      quick.push("ott+1011_4_acc=on:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:bs=unit_only:ccuc=small_ones:cond=on:fsr=off:irw=on:lwlo=on:nwc=1.3:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_23");
      quick.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_100");
      quick.push("dis+2_10_afp=100000:afq=1.2:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1_5");
      quick.push("dis+1003_3_add=off:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:bsr=on:bce=on:cond=fast:fde=none:gs=on:gsaa=from_current:gsem=off:nwc=1.2:sas=z3:sac=on:sp=reverse_arity_13");
      quick.push("lrs+4_4:1_add=large:afr=on:afp=100000:afq=1.0:anc=none:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nwc=1.5:sas=z3:stl=30:sos=all:sac=on:sp=reverse_arity_148");
      quick.push("lrs+1010_4_aac=none:acc=model:add=large:afp=10000:afq=1.4:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:cond=on:fsr=off:irw=on:nwc=2:stl=30:sac=on:sp=reverse_arity:urr=ec_only_66");
      quick.push("ott+10_28_acc=model:add=off:afp=40000:afq=2.0:amm=off:bs=unit_only:bsr=on:ccuc=small_ones:cond=on:fsr=off:gs=on:gsem=on:nwc=1.1:nicw=on:urr=on:updr=off_50");
      quick.push("lrs+1011_2:1_afp=40000:afq=1.1:amm=off:anc=none:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=on:nwc=1:sas=z3:stl=30:sos=all:sp=reverse_arity:updr=off:uhcvi=on_38");
      quick.push("ott+1011_2:3_av=off:cond=fast:er=filter:fde=unused:gsp=input_only:irw=on:nwc=3:sp=occurrence:updr=off:uhcvi=on_2");
      quick.push("lrs+1011_2:1_av=off:cond=on:lwlo=on:nwc=1.5:stl=30_187");
      quick.push("lrs+1011_2:1_av=off:cond=on:er=known:gs=on:gsem=on:lwlo=on:nwc=1.3:stl=30:updr=off:uhcvi=on_133");
      quick.push("lrs+1011_40_add=off:afp=1000:afq=2.0:anc=none:bs=unit_only:fsr=off:irw=on:nwc=1:sas=z3:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_68");
    }
    break;

  case Property::HEQ:
    quick.push("lrs+11_24_afp=100000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:irw=on:nwc=3:stl=30_3");
    quick.push("dis+10_4_aac=none:afp=10000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:irw=on:lcm=reverse:lwlo=on:nwc=2.5:sas=z3:sp=reverse_arity_4");
    quick.push("lrs+11_3:1_av=off:cond=on:fsr=off:irw=on:lma=on:nwc=10:sas=z3:stl=30:updr=off_174");
    quick.push("dis+10_6_av=off:cond=on:gs=on:lcm=reverse:lma=on:nwc=1.7:sp=reverse_arity:updr=off_7");
    quick.push("lrs+11_4_aac=none:acc=model:add=large:afp=100000:afq=1.2:amm=off:anc=none:ccuc=first:cond=fast:gs=on:lma=on:nwc=4:stl=30:sac=on:sp=occurrence_237");
    quick.push("lrs+10_40_aac=none:acc=on:add=off:afp=1000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:ccuc=first:cond=fast:fsr=off:fde=none:gs=on:gsem=on:lma=on:nwc=1.3:stl=30:sp=reverse_arity:updr=off:uhcvi=on_38");
    quick.push("ott+11_64_add=large:afp=1000:afq=2.0:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=on:nwc=2_246");
    quick.push("lrs+2_14_aac=none:add=off:afp=100000:afq=1.1:anc=none:nwc=3:sas=z3:stl=30:sac=on:sp=reverse_arity:updr=off_168");
    quick.push("lrs+11_4:1_av=off:bce=on:cond=fast:fde=none:gsp=input_only:lma=on:nwc=5:stl=30:sp=occurrence_118");
    quick.push("lrs+1011_20_acc=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:ccuc=small_ones:cond=on:gs=on:irw=on:lwlo=on:nwc=1:stl=30:sp=occurrence_38");
    quick.push("lrs+11_40_add=off:afp=10000:afq=1.2:amm=off:cond=on:fsr=off:lma=on:nwc=1.7:stl=30:sac=on:sp=occurrence_182");
    quick.push("dis+11_28_add=off:afr=on:afp=40000:afq=2.0:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off_54");
    quick.push("lrs+10_1024_av=off:bd=off:fsr=off:lma=on:nwc=1:stl=30:sp=occurrence:urr=on_205");
    quick.push("lrs+11_3:2_add=off:afr=on:afp=4000:afq=1.4:anc=none:cond=on:lma=on:lwlo=on:nwc=3:sas=z3:stl=30:sac=on:sp=reverse_arity_124");
    quick.push("ott+11_2:1_afp=1000:afq=1.0:bd=preordered:fsr=off:fde=none:lma=on:nwc=1:sos=all:sp=occurrence:uhcvi=on_171");
    quick.push("lrs+10_3:1_add=off:afp=40000:afq=1.4:anc=none:br=off:fsr=off:gs=on:gsem=on:irw=on:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:urr=on_222");
    break;

  case Property::PEQ:
    if (prop == 1) {
      quick.push("dis+11_5:4_add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:lwlo=on:nwc=1:sas=z3:sac=on:sp=reverse_arity_8");
      quick.push("lrs+11_128_aac=none:add=large:afp=4000:afq=2.0:amm=sco:bd=off:gs=on:gsem=on:nwc=1:nicw=on:stl=30:sos=all:sac=on:sp=reverse_arity:updr=off_15");
      quick.push("lrs+11_2:3_add=large:afp=4000:afq=2.0:amm=sco:anc=none:er=known:gs=on:nwc=1:sas=z3:stl=30:updr=off_4");
      quick.push("dis+10_5_add=off:afr=on:afp=10000:afq=1.4:anc=none:er=known:gs=on:gsem=off:lma=on:nwc=1:nicw=on:sas=z3:sac=on:sp=reverse_arity_5");
      quick.push("lrs+1011_7_av=off:irw=on:nwc=1:stl=30:sos=all_69");
      quick.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:stl=30:sp=occurrence_33");
      quick.push("lrs+10_5_av=off:bd=off:bs=unit_only:cond=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:stl=30:sp=occurrence_224");
      quick.push("dis+1011_2:3_av=off:irw=on:nwc=1.2:sp=reverse_arity:updr=off_157");
      quick.push("ott+11_1_afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sac=on:sp=occurrence:updr=off:uhcvi=on_624");
      quick.push("lrs+11_32_add=off:afp=10000:afq=1.1:anc=all:bs=unit_only:cond=fast:fde=none:gs=on:gsaa=from_current:irw=on:lma=on:nwc=1:nicw=on:stl=60:sos=all:sac=on:sp=occurrence:updr=off:uhcvi=on_501");
      quick.push("lrs+11_2:1_av=off:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=2:stl=60:sp=occurrence:updr=off:uhcvi=on_546");
    }
    else if (prop == 0) {
      quick.push("lrs+11_3:2_afp=10000:afq=1.0:anc=none:bsr=on:cond=on:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:stl=30:sac=on:updr=off_7");
      quick.push("lrs+11_5_acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:bs=unit_only:ccuc=first:cond=on:lma=on:lwlo=on:nwc=1:stl=30:sp=reverse_arity:updr=off_234");
      quick.push("dis+10_28_add=large:afp=4000:afq=1.0:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence_72");
      quick.push("lrs+2_2_acc=model:add=off:afp=10000:afq=1.2:anc=all:bd=off:bs=on:bsr=on:ccuc=small_ones:cond=on:fsr=off:fde=unused:gs=on:lma=on:nwc=1.2:stl=30:sos=on:updr=off:uhcvi=on_43");
      quick.push("lrs+11_3:1_add=off:afr=on:afp=100000:afq=1.4:anc=none:cond=on:gs=on:irw=on:lma=on:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sac=on:sp=occurrence:updr=off_89");
      quick.push("ott+11_14_av=off:cond=on:nwc=1.3:sp=reverse_arity:updr=off_1");
      quick.push("dis+1011_8_av=off:bsr=on:cond=on:fsr=off:fde=none:gs=on:lma=on:nwc=1.1:sos=all:sp=reverse_arity_207");
      quick.push("lrs+10_2_afr=on:afp=4000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:sac=on:sp=occurrence:updr=off_107");
      quick.push("lrs+10_5:4_add=large:afr=on:amm=sco:anc=all_dependent:bd=preordered:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1:sas=z3:stl=60:sos=all:sac=on:updr=off:uhcvi=on_142");
      quick.push("lrs+10_4:1_av=off:bd=off:bs=unit_only:bsr=on:fsr=off:gs=on:gsem=on:lwlo=on:nwc=1:stl=30:sos=all_135");
      quick.push("ott+1_3_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bsr=on:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence:uhcvi=on_138");
      quick.push("ott+1011_5_afr=on:afp=1000:afq=1.4:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=unused:gsp=input_only:gs=on:nwc=1:nicw=on:sas=z3:sos=all:updr=off:uhcvi=on_484");
      quick.push("lrs+11_3_aac=none:add=large:afp=1000:afq=1.4:anc=all_dependent:bd=off:bs=on:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=1:stl=60:sos=all:sac=on:sp=occurrence:updr=off_587");
    }
    else {
      quick.push("lrs+11_2_add=large:afp=4000:afq=1.1:anc=none:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence:updr=off_110");
      quick.push("lrs+11_1024_add=off:afp=10000:afq=1.0:anc=none:bd=off:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:stl=30:sp=occurrence:updr=off_9");
      quick.push("ott+1010_14_add=large:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:bsr=on:cond=fast:lma=on:nwc=1.2:sp=occurrence:updr=off_259");
      quick.push("ott+11_7_acc=model:afr=on:afp=40000:afq=2.0:amm=off:anc=all_dependent:bs=unit_only:ccuc=small_ones:fsr=off:gs=on:gsaa=from_current:lma=on:nwc=1.7:nicw=on:sp=occurrence:uhcvi=on_116");
      quick.push("lrs+11_5_acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:bs=unit_only:ccuc=first:cond=on:lma=on:lwlo=on:nwc=1:stl=30:sp=reverse_arity:updr=off_119");
      quick.push("ott+11_5:1_add=off:afp=10000:afq=1.4:anc=none:bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:lma=on:nwc=1_147");
      quick.push("lrs+11_3:1_add=off:afr=on:afp=100000:afq=1.4:anc=none:cond=on:gs=on:irw=on:lma=on:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sac=on:sp=occurrence:updr=off_298");
      quick.push("lrs+11_128_aac=none:add=large:afp=4000:afq=2.0:amm=sco:bd=off:gs=on:gsem=on:nwc=1:nicw=on:stl=30:sos=all:sac=on:sp=reverse_arity:updr=off_1");
    }
    break;

  case Property::HNE:
    quick.push("lrs+1011_8_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bs=on:gs=on:gsem=off:nwc=1.5:nicw=on:stl=30:sac=on:sp=reverse_arity:updr=off_32");
    quick.push("dis+11_1_add=off:afp=4000:afq=2.0:amm=sco:anc=none:br=off:cond=on:lma=on:nwc=1:sos=all:sac=on:urr=on_7");
    quick.push("lrs+10_3:2_afr=on:afp=100000:afq=2.0:anc=none:bs=unit_only:bsr=on:cond=fast:fsr=off:lwlo=on:nwc=2:nicw=on:stl=30:sp=reverse_arity_211");
    quick.push("lrs+10_2:3_av=off:bsr=on:cond=on:lwlo=on:nwc=1.7:stl=60:sp=occurrence:updr=off_171");
    quick.push("lrs+11_24_afp=4000:afq=2.0:amm=sco:anc=all:br=off:cond=fast:gsp=input_only:gs=on:gsem=on:lma=on:lwlo=on:nwc=1.7:nicw=on:stl=30:sos=theory:sac=on:sp=reverse_arity:urr=on_122");
    quick.push("lrs+1011_3_av=off:bs=unit_only:cond=on:fsr=off:lcm=predicate:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_215");
    quick.push("dis+11_3:1_aac=none:add=off:afp=4000:afq=1.4:fsr=off:nwc=3:nicw=on:sp=occurrence_101");
    quick.push("lrs+10_5:4_av=off:bsr=on:gs=on:gsem=off:lma=on:nwc=4:stl=30:uhcvi=on_146");
    quick.push("lrs+1_2:3_av=off:fsr=off:lma=on:nwc=1:sas=z3:stl=30:urr=ec_only:updr=off_72");
    quick.push("lrs+4_3_add=off:afr=on:afp=10000:afq=2.0:anc=none:lma=on:nwc=1.1:nicw=on:sas=z3:stl=30:sac=on_203");
    quick.push("dis+1_40_av=off:lwlo=on:nwc=4:sos=all:sp=occurrence:updr=off_117");
    quick.push("lrs+10_2:3_av=off:cond=on:fsr=off:lwlo=on:nwc=2.5:stl=30:sp=reverse_arity:updr=off_68");
    quick.push("lrs+1_128_afp=100000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:stl=60:sac=on_455");
    quick.push("dis+11_8:1_afr=on:afp=1000:afq=1.0:amm=off:anc=none:nwc=1:sos=all:sp=occurrence:updr=off_184");
    break;

  case Property::NNE:
    quick.push("dis+1011_5_av=off:gs=on:gsem=on:nwc=1:sos=on:updr=off_3");
    quick.push("lrs+11_14_aac=none:afp=1000:afq=2.0:fsr=off:lma=on:nwc=1:stl=30:sp=reverse_arity_26");
    quick.push("dis-1_3_av=off:cond=on:fsr=off:gs=on:gsem=on:nwc=1:updr=off_4");
    quick.push("ins+10_1_av=off:cond=on:fsr=off:gsp=input_only:igbrr=0.6:igpr=on:igrr=64/1:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:updr=off_9");
    quick.push("dis+1011_64_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:gsp=input_only:gs=on:gsem=on:lma=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off:uhcvi=on_26");
    quick.push("dis+1011_5_av=off:lma=on:nwc=1.7:sos=all:sp=reverse_arity:updr=off_429");
    quick.push("lrs+1011_50_av=off:bs=unit_only:cond=on:nwc=2:stl=30:sp=occurrence:updr=off_47");
    quick.push("dis+4_7_av=off:cond=fast:gsp=input_only:lma=on:nwc=1.3:sp=occurrence:urr=ec_only:uhcvi=on_162");
    quick.push("dis+10_5:4_afp=100000:afq=1.0:amm=sco:anc=all:cond=fast:fsr=off:gs=on:lma=on:nwc=1:sp=reverse_arity:updr=off:uhcvi=on_24");
    quick.push("lrs+1010_14_av=off:fsr=off:lma=on:lwlo=on:nwc=2.5:stl=30:updr=off:uhcvi=on_82");
    quick.push("dis+2_2_add=off:afr=on:afp=4000:afq=1.0:amm=sco:anc=none:fsr=off:lcm=predicate:lma=on:nwc=1.3:nicw=on:sos=theory:sp=reverse_arity:urr=ec_only:updr=off_26");
    quick.push("dis+1003_128_add=large:afr=on:amm=off:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:sas=z3:sos=on:sp=occurrence:updr=off:uhcvi=on_64");
    quick.push("lrs+4_20_av=off:gs=on:gsem=off:lcm=reverse:lma=on:nwc=1:stl=30:sos=on:updr=off_43");
    quick.push("lrs+1011_2_av=off:cond=on:gsp=input_only:gs=on:lwlo=on:nwc=1:stl=30:sos=all:uhcvi=on_300");
    quick.push("dis+3_64_av=off:cond=fast:lcm=reverse:lma=on:lwlo=on:nwc=1:sos=on:updr=off_68");
    quick.push("lrs+11_2_afp=1000:afq=1.4:amm=sco:anc=none:cond=on:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_155");
    quick.push("lrs+1011_3:2_av=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:lcm=predicate:lwlo=on:nwc=2.5:stl=30:sos=all:updr=off_158");
    break;

  case Property::UEQ:
    if (prop == 2) {
      quick.push("ott+10_20_av=off:bd=preordered:fde=none:ins=3:nwc=1:sp=occurrence_98");
      quick.push("lrs+10_4:1_av=off:bd=preordered:fde=none:ins=3:lwlo=on:nwc=3:stl=60:sp=reverse_arity_96");
      quick.push("ott+10_20_av=off:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_862");
      quick.push("lrs+10_1_av=off:ins=3:nwc=1.1:stl=60:sp=reverse_arity_390");
      quick.push("lrs+10_2:1_av=off:bd=off:fde=none:ins=3:lwlo=on:nwc=1:stl=90:uhcvi=on_296");
      quick.push("lrs+10_28_av=off:ins=3:nwc=1:stl=30_186");
      quick.push("ott+10_4_av=off:fde=none:ins=3:nwc=1:sos=on:sp=occurrence:uhcvi=on_122");
      quick.push("ott+10_28_av=off:bd=preordered:fde=unused:ins=3:nwc=2.5:sp=occurrence_275");
      quick.push("ott+10_5:1_av=off:bd=preordered:fde=unused:ins=3:nwc=1_268");
      quick.push("ott+10_8_av=off:bd=preordered:ins=3:nwc=1.2:sp=reverse_arity:uhcvi=on_456");
      quick.push("lrs+10_3:2_av=off:bd=preordered:ins=3:lwlo=on:nwc=1.1:stl=90:uhcvi=on_529");
      quick.push("ott+10_2_av=off:bd=off:ins=3:nwc=1.2_430");
    }
    else if (prop == 0) {
      quick.push("lrs+10_3_av=off:ins=3:lwlo=on:nwc=1:stl=90:sp=occurrence:uhcvi=on_793");
      quick.push("ott+10_2_av=off:bd=preordered:fde=none:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_580");
      quick.push("ott+10_28_av=off:bd=preordered:ins=3:nwc=3:sp=reverse_arity_966");
      quick.push("ott+10_4:1_av=off:bd=preordered:ins=3:nwc=1:sp=reverse_arity_744");
      quick.push("lrs+10_3:1_av=off:fde=none:ins=3:nwc=1:stl=120:sp=reverse_arity:uhcvi=on_350");
      quick.push("ott+10_6_av=off:bd=preordered:fde=none:ins=3:nwc=1.1:sos=on:sp=occurrence_786");
      quick.push("ott+10_4_av=off:bd=off:ins=3:nwc=1.5:sp=reverse_arity_401");
      quick.push("ott+10_2:1_av=off:ins=3:nwc=5:sp=occurrence_1");
      quick.push("ott+10_8:1_av=off:bd=preordered:fde=none:ins=3:nwc=1_442");
    }
    else {
      quick.push("lrs+10_1_av=off:bd=off:ins=3:lwlo=on:nwc=3:stl=30_64");
      quick.push("lrs+10_5:1_av=off:bd=off:ins=3:nwc=2.5:stl=60:sp=reverse_arity_138");
      quick.push("ott+10_28_av=off:bd=preordered:fde=unused:ins=3:nwc=2.5:sp=occurrence_36");
      quick.push("ott+10_128_av=off:bd=off:ins=3:nwc=1:sp=reverse_arity_45");
      quick.push("lrs+1011_8_av=off:bs=unit_only:ep=RSTC:gsp=input_only:gs=on:gsem=on:lwlo=on:nwc=1:stl=30_226");
      quick.push("ott+10_4_av=off:bd=off:ins=3:nwc=1.5:sp=reverse_arity_568");
      quick.push("lrs+10_6_av=off:fde=unused:ins=3:lwlo=on:nwc=1:stl=60:sp=reverse_arity:uhcvi=on_1");
      quick.push("lrs+10_3_av=off:ins=3:lwlo=on:nwc=1:stl=90:sp=occurrence:uhcvi=on_2");
      quick.push("lrs+10_4_av=off:ins=3:lwlo=on:nwc=4:stl=30:sp=occurrence_1");
    }
    break;
  }

  switch (cat) {
  case Property::HEQ:
  case Property::PEQ:
  case Property::NEQ:
  case Property::HNE:
  case Property::NNE:
    fallback.push("lrs+1011_5:1_afp=100000:afq=1.0:anc=none:bd=off:gsp=input_only:gs=on:gsem=off:lwlo=on:nwc=5:nicw=on:sas=z3:sac=on:updr=off_300");
    fallback.push("lrs+10_2:3_av=off:bsr=on:cond=on:lwlo=on:nwc=1.7:sp=occurrence:updr=off_600");
    fallback.push("dis+1011_5_av=off:lma=on:nwc=1.7:sos=all:sp=reverse_arity:updr=off_600");
    fallback.push("lrs+11_3:1_av=off:cond=on:fsr=off:irw=on:lma=on:nwc=10:sas=z3:updr=off_300");
    fallback.push("lrs+11_2_add=large:afp=4000:afq=1.1:anc=none:lma=on:lwlo=on:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("lrs+4_2_av=off:bs=on:er=known:gs=on:irw=on:lwlo=on:nwc=10:sp=occurrence_300");
    fallback.push("dis+1011_3_av=off:nwc=1:sos=all:sp=reverse_arity_300");
    fallback.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bce=on:fde=unused:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+11_2_afr=on:afp=100000:afq=1.4:amm=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on:updr=off_300");
    fallback.push("lrs+1011_3_av=off:bs=unit_only:cond=on:fsr=off:lcm=predicate:lma=on:lwlo=on:nwc=1:sp=occurrence_300");
    fallback.push("lrs+1011_2:1_av=off:cond=on:er=known:gs=on:gsem=on:lwlo=on:nwc=1.3:updr=off:uhcvi=on_300");
    fallback.push("ott+11_14_av=off:cond=on:nwc=1.3:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_4_aac=none:acc=model:add=large:afp=100000:afq=1.2:amm=off:anc=none:ccuc=first:cond=fast:gs=on:lma=on:nwc=4:sac=on:sp=occurrence_300");
    fallback.push("dis+2_2_add=off:afr=on:afp=4000:afq=1.0:amm=sco:anc=none:fsr=off:lcm=predicate:lma=on:nwc=1.3:nicw=on:sos=theory:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=off:anc=none:cond=on:irw=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0:sos=all:sp=occurrence_300");
    fallback.push("lrs+11_5_add=large:afr=on:afp=40000:afq=1.2:cond=on:er=known:gs=on:gsem=off:nwc=1.5:sp=occurrence:updr=off_300");
    fallback.push("lrs+1003_3_av=off:bd=off:fde=none:gs=on:lma=on:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs-1_14_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:sd=1:ss=axioms:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_6_afr=on:afp=100000:afq=1.1:anc=none:br=off:gs=on:lma=on:nwc=3:sac=on:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+4_4:1_add=large:afr=on:afp=100000:afq=1.0:anc=none:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nwc=1.5:sas=z3:sos=all:sac=on:sp=reverse_arity_300");
    fallback.push("lrs+11_3_aac=none:add=large:afp=1000:afq=1.4:anc=all_dependent:bd=off:bs=on:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=1:sos=all:sac=on:sp=occurrence:updr=off_600");
    fallback.push("ott+11_64_add=large:afp=1000:afq=2.0:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=on:nwc=2_300");
    fallback.push("lrs+1011_3:2_av=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:lcm=predicate:lwlo=on:nwc=2.5:sos=all:updr=off_300");
    fallback.push("lrs+10_3_av=off:bd=off:cond=on:gs=on:gsem=off:irw=on:lwlo=on:nwc=1.2:sp=reverse_arity:updr=off_300");
    fallback.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:sac=on:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1010_4_aac=none:acc=model:add=large:afp=10000:afq=1.4:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:cond=on:fsr=off:irw=on:nwc=2:sac=on:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+10_28_add=large:afp=4000:afq=1.0:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence_300");
    fallback.push("lrs+4_1_av=off:bd=off:bsr=on:bce=on:fsr=off:irw=on:lcm=predicate:nwc=1:sos=all:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("ins+10_1_av=off:cond=on:fsr=off:gsp=input_only:igbrr=0.6:igpr=on:igrr=64/1:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:updr=off_300");
    fallback.push("dis+10_4_aac=none:afp=10000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:irw=on:lcm=reverse:lwlo=on:nwc=2.5:sas=z3:sp=reverse_arity_300");
    fallback.push("lrs+11_5_acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:bs=unit_only:ccuc=first:cond=on:lma=on:lwlo=on:nwc=1:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1_40_av=off:lwlo=on:nwc=4:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+10_5_add=off:afr=on:afp=10000:afq=1.4:anc=none:er=known:gs=on:gsem=off:lma=on:nwc=1:nicw=on:sas=z3:sac=on:sp=reverse_arity_300");
    fallback.push("lrs+4_5_av=off:bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:nwc=2.5:sp=occurrence:updr=off_300");
    fallback.push("dis+2_3_av=off:cond=on:fsr=off:lcm=reverse:lma=on:nwc=1:sos=on:sp=reverse_arity_300");
    fallback.push("dis+1011_2:1_av=off:lma=on:nwc=1:sd=3:ss=axioms:st=5.0:sp=occurrence:urr=ec_only_300");
    fallback.push("dis+1011_8_av=off:bd=off:cond=fast:er=known:fde=unused:gsp=input_only:lcm=predicate:lma=on:nwc=1.2:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_4:1_av=off:cond=on:irw=on:lma=on:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("ott+1011_2:3_av=off:cond=fast:er=filter:fde=unused:gsp=input_only:irw=on:nwc=3:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+10_3:2_afr=on:afp=100000:afq=2.0:anc=none:bs=unit_only:bsr=on:cond=fast:fsr=off:lwlo=on:nwc=2:nicw=on:sp=reverse_arity_300");
    fallback.push("dis+11_2:1_acc=model:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:er=known:gsp=input_only:irw=on:lma=on:nwc=5:sac=on:urr=ec_only_300");
    fallback.push("lrs+11_40_add=off:afp=10000:afq=1.2:amm=off:cond=on:fsr=off:lma=on:nwc=1.7:sac=on:sp=occurrence_300");
    fallback.push("lrs+10_5:4_av=off:bsr=on:gs=on:gsem=off:lma=on:nwc=4:uhcvi=on_300");
    fallback.push("lrs+1010_3:1_av=off:cond=on:nwc=5:sp=reverse_arity_300");
    fallback.push("lrs+11_3:2_add=off:afr=on:afp=4000:afq=1.4:anc=none:cond=on:lma=on:lwlo=on:nwc=3:sas=z3:sac=on:sp=reverse_arity_300");
    fallback.push("lrs+4_3_add=off:afr=on:afp=10000:afq=2.0:anc=none:lma=on:nwc=1.1:nicw=on:sas=z3:sac=on_300");
    fallback.push("ott+11_6_av=off:bs=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=10:sp=reverse_arity_300");
    fallback.push("dis+11_12_av=off:bsr=on:cond=on:lcm=predicate:lma=on:nwc=5:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_3:1_add=off:afr=on:afp=100000:afq=1.4:anc=none:cond=on:gs=on:irw=on:lma=on:lwlo=on:nwc=1:nicw=on:sas=z3:sac=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:sp=occurrence_300");
    fallback.push("ott+1011_5_afr=on:afp=1000:afq=1.4:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=unused:gsp=input_only:gs=on:nwc=1:nicw=on:sas=z3:sos=all:updr=off:uhcvi=on_600");
    fallback.push("ott+1010_14_add=large:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:bsr=on:cond=fast:lma=on:nwc=1.2:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_2_afp=1000:afq=1.4:amm=sco:anc=none:cond=on:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+11_3:2_afp=10000:afq=1.0:anc=none:bsr=on:cond=on:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:sac=on:updr=off_300");
    fallback.push("lrs+1011_40_add=off:afp=1000:afq=2.0:anc=none:bs=unit_only:fsr=off:irw=on:nwc=1:sas=z3:sos=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_12_av=off:bd=off:bs=unit_only:fsr=off:lma=on:nwc=1:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_7_av=off:irw=on:nwc=1:sos=all_300");
    fallback.push("lrs+10_12_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=off:cond=on:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=1.3:sas=z3:sac=on_300");
    fallback.push("lrs+1_3_av=off:fsr=off:lma=on:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on_300");
    fallback.push("dis+11_5_afr=on:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_2:1_av=off:bce=on:cond=fast:fsr=off:fde=none:lwlo=on:nwc=1:sd=1:ss=axioms:st=3.0:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("dis+4_7_av=off:cond=fast:gsp=input_only:lma=on:nwc=1.3:sp=occurrence:urr=ec_only:uhcvi=on_300");
    fallback.push("lrs+1011_50_av=off:bs=unit_only:cond=on:nwc=2:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_3:1_av=off:bs=unit_only:bsr=on:er=known:fsr=off:fde=unused:irw=on:lcm=reverse:lwlo=on:nwc=1.7:sos=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_1024_av=off:bd=off:fsr=off:lma=on:nwc=1:sp=occurrence:urr=on_300");
    fallback.push("lrs+10_3:1_add=off:afp=40000:afq=1.4:anc=none:br=off:fsr=off:gs=on:gsem=on:irw=on:lwlo=on:nwc=1:nicw=on:sas=z3:sos=all:urr=on_300");
    fallback.push("lrs+1_128_afp=100000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:sac=on_600");
    fallback.push("lrs+1010_3:2_av=off:fsr=off:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:sos=all:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+1011_20_acc=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:ccuc=small_ones:cond=on:gs=on:irw=on:lwlo=on:nwc=1:sp=occurrence_300");
    fallback.push("lrs+1010_14_av=off:fsr=off:lma=on:lwlo=on:nwc=2.5:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_2:1_add=large:afr=on:afp=100000:afq=2.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:lma=on:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on_300");
    fallback.push("ott+10_4:1_av=off:bd=preordered:cond=fast:fde=unused:irw=on:lcm=reverse:nwc=3:sd=2:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+2_14_aac=none:add=off:afp=100000:afq=1.1:anc=none:nwc=3:sas=z3:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+10_5:4_add=large:afr=on:amm=sco:anc=all_dependent:bd=preordered:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1:sas=z3:sos=all:sac=on:updr=off:uhcvi=on_600");
    fallback.push("ott+1011_8:1_av=off:bd=off:cond=on:nwc=1:sos=all:sp=reverse_arity_300");
    fallback.push("dis+11_28_add=off:afr=on:afp=40000:afq=2.0:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1_20_add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=z3:sd=3:ss=axioms:st=1.2:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_14_av=off:bd=off:bs=unit_only:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lcm=reverse:lwlo=on:nwc=1:sos=on:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+3_64_av=off:cond=fast:lcm=reverse:lma=on:lwlo=on:nwc=1:sos=on:updr=off_300");
    fallback.push("dis+1011_5_av=off:gs=on:gsem=on:nwc=1:sos=on:updr=off_300");
    fallback.push("dis+1003_128_add=large:afr=on:amm=off:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:sas=z3:sos=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis-11_4:1_amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nwc=10:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("lrs+11_1_av=off:fsr=off:irw=on:lma=on:lwlo=on:nwc=1:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+2_10_afp=100000:afq=1.2:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1_300");
    fallback.push("lrs+10_2:3_av=off:cond=on:fsr=off:lwlo=on:nwc=2.5:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1010_1024_afr=on:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:irw=on:lwlo=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0_300");
    fallback.push("lrs+1002_3:1_av=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nwc=4:sas=z3:sd=1:ss=axioms:st=5.0:sp=reverse_arity_300");
    fallback.push("lrs+10_5:1_add=large:afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_5:4_av=off:bd=off:fsr=off:gs=on:nwc=1.3:urr=ec_only:updr=off_300");
    fallback.push("ins+11_3_av=off:bd=off:igbrr=0.6:igrr=1/8:igrp=700:igrpq=2.0:igs=1:igwr=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:updr=off_300");
    fallback.push("lrs+10_40_aac=none:acc=on:add=off:afp=1000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:ccuc=first:cond=fast:fsr=off:fde=none:gs=on:gsem=on:lma=on:nwc=1.3:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("dis+1011_32_aac=none:afp=10000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=3:sas=z3:sp=reverse_arity_300");
    fallback.push("dis+1011_64_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:gsp=input_only:gs=on:gsem=on:lma=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_32_av=off:bd=off:br=off:gs=on:gsem=on:irw=on:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on:updr=off_300");
    fallback.push("dis+11_3_av=off:bd=off:bsr=on:bce=on:gsp=input_only:gs=on:gsem=on:lma=on:nwc=2.5:sp=reverse_arity:urr=ec_only:uhcvi=on_300");
    fallback.push("lrs+11_24_afp=100000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:irw=on:nwc=3_300");
    fallback.push("lrs+1011_2:1_afp=40000:afq=1.1:amm=off:anc=none:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=on:nwc=1:sas=z3:sos=all:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1010_8:1_av=off:bs=unit_only:br=off:cond=on:fsr=off:irw=on:nwc=1.3:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:updr=off_300");
    fallback.push("lrs+10_2_afr=on:afp=4000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:updr=off_300");
    fallback.push("dis-11_5:1_acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=first:cond=on:gs=on:gsem=off:nwc=1:sd=3:ss=axioms:st=1.2:sos=all_300");
    fallback.push("lrs+10_4:1_av=off:bd=off:bs=unit_only:bsr=on:fsr=off:gs=on:gsem=on:lwlo=on:nwc=1:sos=all_300");
    fallback.push("dis+2_5:4_add=large:afp=4000:afq=1.2:anc=all:bce=on:cond=fast:fde=none:lma=on:nwc=10:sd=1:ss=axioms:st=1.5:sac=on_300");
    fallback.push("lrs+10_5_av=off:bd=off:bs=unit_only:cond=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sp=occurrence_300");
    fallback.push("dis+1011_2:3_av=off:irw=on:nwc=1.2:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1002_5:1_av=off:cond=fast:fsr=off:fde=none:lma=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:urr=on:updr=off_300");
    fallback.push("lrs+2_2_acc=model:add=off:afp=10000:afq=1.2:anc=all:bd=off:bs=on:bsr=on:ccuc=small_ones:cond=on:fsr=off:fde=unused:gs=on:lma=on:nwc=1.2:sos=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1003_3_add=off:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:bsr=on:bce=on:cond=fast:fde=none:gs=on:gsaa=from_current:gsem=off:nwc=1.2:sas=z3:sac=on:sp=reverse_arity_300");
    fallback.push("dis+11_3:1_aac=none:add=off:afp=4000:afq=1.4:fsr=off:nwc=3:nicw=on:sp=occurrence_300");
    fallback.push("lrs+1011_2:1_av=off:cond=on:lwlo=on:nwc=1.5_300");
    fallback.push("lrs+11_24_afp=4000:afq=2.0:amm=sco:anc=all:br=off:cond=fast:gsp=input_only:gs=on:gsem=on:lma=on:lwlo=on:nwc=1.7:nicw=on:sos=theory:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("dis+10_5:4_afp=100000:afq=1.0:amm=sco:anc=all:cond=fast:fsr=off:gs=on:lma=on:nwc=1:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs-2_2:1_afr=on:afp=1000:afq=2.0:anc=none:bd=off:bce=on:cond=fast:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1.5:sac=on:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+1_2:1_av=off:fsr=off:lma=on:nwc=1:sd=7:ss=axioms:st=1.2:sos=on:urr=ec_only_300");
    fallback.push("dis+11_4_aac=none:add=large:afp=100000:afq=1.0:amm=off:anc=none:bsr=on:cond=on:fsr=off:gs=on:lcm=reverse:lma=on:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+1011_10_av=off:bs=unit_only:bsr=on:er=filter:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nwc=1.2:sp=reverse_arity:updr=off_300");
    fallback.push("dis-1_3_av=off:cond=on:fsr=off:gs=on:gsem=on:nwc=1:updr=off_300");
    fallback.push("lrs+11_4:1_av=off:bce=on:cond=fast:fde=none:gsp=input_only:lma=on:nwc=5:sp=occurrence_300");
    fallback.push("ott+11_7_acc=model:afr=on:afp=40000:afq=2.0:amm=off:anc=all_dependent:bs=unit_only:ccuc=small_ones:fsr=off:gs=on:gsaa=from_current:lma=on:nwc=1.7:nicw=on:sp=occurrence:uhcvi=on_300");
    fallback.push("ott+10_28_acc=model:add=off:afp=40000:afq=2.0:amm=off:bs=unit_only:bsr=on:ccuc=small_ones:cond=on:fsr=off:gs=on:gsem=on:nwc=1.1:nicw=on:urr=on:updr=off_300");
    fallback.push("lrs+1011_2_av=off:bd=preordered:bs=unit_only:cond=fast:fsr=off:fde=unused:irw=on:lma=on:lwlo=on:nwc=1.2:sp=occurrence:uhcvi=on_300");
    fallback.push("dis+11_8:1_afr=on:afp=1000:afq=1.0:amm=off:anc=none:nwc=1:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+10_3_acc=on:afr=on:afp=1000:afq=1.0:amm=sco:bs=unit_only:ccuc=first:fsr=off:irw=on:lcm=reverse:lma=on:nwc=1.5:updr=off_300");
    fallback.push("ott+11_5:1_add=off:afp=10000:afq=1.4:anc=none:bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:lma=on:nwc=1_300");
    fallback.push("lrs+1004_20_av=off:cond=on:er=filter:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1:sd=3:ss=axioms:st=3.0:sos=on:urr=ec_only_300");
    fallback.push("dis+1011_8_av=off:bsr=on:cond=on:fsr=off:fde=none:gs=on:lma=on:nwc=1.1:sos=all:sp=reverse_arity_300");
    fallback.push("lrs+11_14_aac=none:afp=1000:afq=2.0:fsr=off:lma=on:nwc=1:sp=reverse_arity_300");
    fallback.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+1_2:3_av=off:fsr=off:lma=on:nwc=1:sas=z3:urr=ec_only:updr=off_300");
    fallback.push("ott+11_2:1_afp=1000:afq=1.0:bd=preordered:fsr=off:fde=none:lma=on:nwc=1:sos=all:sp=occurrence:uhcvi=on_300");
    fallback.push("ott+1_3_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bsr=on:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence:uhcvi=on_300");
    fallback.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sp=occurrence_300");
    fallback.push("dis+11_2_av=off:gs=on:gsem=on:irw=on:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=reverse_arity_300");
    fallback.push("lrs+1011_8_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bs=on:gs=on:gsem=off:nwc=1.5:nicw=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis-11_64_av=off:bd=off:bs=on:cond=on:fsr=off:nwc=1:sd=1:ss=axioms:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_1024_add=off:afp=10000:afq=1.0:anc=none:bd=off:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("lrs-11_3_aac=none:acc=on:afr=on:afp=10000:afq=1.1:bd=off:bs=unit_only:ccuc=first:irw=on:lcm=predicate:lma=on:nwc=1.5:sos=all:sac=on:sp=occurrence:updr=off_300");
    fallback.push("dis-10_3:1_add=large:afr=on:afp=1000:afq=2.0:anc=none:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nwc=1:sas=z3:sd=2:ss=axioms:st=2.0:sos=all:sac=on:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("lrs+11_2:1_av=off:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=2:sp=occurrence:updr=off:uhcvi=on_600");
    fallback.push("lrs+11_32_add=off:afp=10000:afq=1.1:anc=all:bs=unit_only:cond=fast:fde=none:gs=on:gsaa=from_current:irw=on:lma=on:nwc=1:nicw=on:sos=all:sac=on:sp=occurrence:updr=off:uhcvi=on_600");
    fallback.push("ott+11_1_afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sac=on:sp=occurrence:updr=off:uhcvi=on_900");
    break;

  case Property::EPR:
    fallback.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_300");
    fallback.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:uhcvi=on_300");
    fallback.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_3000");
    fallback.push("dis+4_5_afp=1000:afq=1.1:amm=off:anc=none:bd=off:gs=on:irw=on:lcm=predicate:lma=on:nwc=1:sas=z3:sos=all:sp=occurrence_300");
    fallback.push("lrs+1003_10_afp=4000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:bce=on:fde=unused:lma=on:nwc=1:nicw=on:sac=on:urr=on:updr=off:uhcvi=on_1200");
    fallback.push("dis+11_2:3_add=large:afp=10000:afq=1.2:anc=none:bd=off:bce=on:cond=fast:er=filter:fsr=off:fde=unused:gsp=input_only:nwc=5:sos=theory:sac=on:urr=on_300");
    fallback.push("ott-4_6_add=off:afr=on:afp=100000:afq=1.4:amm=sco:bs=on:fde=unused:gs=on:gsaa=full_model:gsem=on:irw=on:nwc=1:sas=z3:sac=on:updr=off:uhcvi=on_600");
    fallback.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_200");
    fallback.push("dis-11_32_av=off:bs=unit_only:gs=on:irw=on:lma=on:nwc=1:updr=off_300");
    fallback.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_5:1_add=off:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:gs=on:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
    fallback.push("lrs+1_16_add=off:afp=100000:afq=1.0:amm=off:cond=fast:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=2:nicw=on:sd=7:ss=axioms:st=5.0:sos=theory:sp=reverse_arity:urr=ec_only_600");
    fallback.push("dis+1011_5_add=large:anc=none:bd=preordered:cond=on:fsr=off:fde=unused:lma=on:nwc=1:sos=theory:sp=occurrence:updr=off_1800");
    fallback.push("lrs+1_8:1_add=off:anc=none:bd=preordered:br=off:bce=on:fsr=off:fde=none:nwc=1:nicw=on:sos=theory:sp=reverse_arity:urr=on_900");
    break;

  case Property::UEQ:
    fallback.push("lrs+10_3:2_av=off:bd=preordered:ins=3:lwlo=on:nwc=1.1:uhcvi=on_900");
    fallback.push("ott+10_28_av=off:bd=preordered:ins=3:nwc=3:sp=reverse_arity_1200");
    fallback.push("lrs+10_4:1_av=off:bd=preordered:fde=none:ins=3:lwlo=on:nwc=3:sp=reverse_arity_600");
    fallback.push("ott+10_4_av=off:bd=off:ins=3:nwc=1.5:sp=reverse_arity_600");
    fallback.push("ott+10_4:1_av=off:bd=preordered:ins=3:nwc=1:sp=reverse_arity_900");
    fallback.push("lrs+10_3_av=off:ins=3:lwlo=on:nwc=1:sp=occurrence:uhcvi=on_900");
    fallback.push("ott+10_128_av=off:bd=off:ins=3:nwc=1:sp=reverse_arity_300");
    fallback.push("ott+10_5:1_av=off:bd=preordered:fde=unused:ins=3:nwc=1_300");
    fallback.push("lrs+10_2:1_av=off:bd=off:fde=none:ins=3:lwlo=on:nwc=1:uhcvi=on_900");
    fallback.push("ott+10_20_av=off:bd=preordered:fde=none:ins=3:nwc=1:sp=occurrence_300");
    fallback.push("lrs+10_5:1_av=off:bd=off:ins=3:nwc=2.5:sp=reverse_arity_600");
    fallback.push("lrs+1011_8_av=off:bs=unit_only:ep=RSTC:gsp=input_only:gs=on:gsem=on:lwlo=on:nwc=1_300");
    fallback.push("ott+10_5:4_av=off:ins=3:nwc=1.7:sp=reverse_arity_300");
    fallback.push("ott+10_4_av=off:fde=none:ins=3:nwc=1:sos=on:sp=occurrence:uhcvi=on_300");
    fallback.push("ott+10_2_av=off:bd=preordered:fde=none:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_600");
    fallback.push("ott+10_8:1_av=off:bd=preordered:fde=none:ins=3:nwc=1_600");
    fallback.push("ott+10_2_av=off:bd=off:ins=3:nwc=1.2_600");
    fallback.push("ott+10_20_av=off:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_1200");
    fallback.push("lrs+10_3:1_av=off:fde=none:ins=3:nwc=1:sp=reverse_arity:uhcvi=on_1200");
    fallback.push("ott+10_8_av=off:bd=preordered:ins=3:nwc=1.2:sp=reverse_arity:uhcvi=on_600");
    fallback.push("lrs+10_1_av=off:ins=3:nwc=1.1:sp=reverse_arity_600");
    fallback.push("ott+10_6_av=off:bd=preordered:fde=none:ins=3:nwc=1.1:sos=on:sp=occurrence_900");
    break;

  case Property::FEQ:
    fallback.push("lrs+1010_3:2_afr=on:afp=100000:afq=1.1:anc=none:gsp=input_only:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1.7:sac=on:sp=occurrence_300");
    fallback.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:updr=off_300");
    fallback.push("dis+1002_4_add=large:afp=40000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsaa=full_model:lma=on:lwlo=on:nm=0:nwc=1.5:sas=z3:sp=reverse_arity:tha=off:thi=strong_300");
    fallback.push("lrs+1011_2:1_av=off:irw=on:lwlo=on:nm=16:newcnf=on:nwc=2:sd=4:ss=axioms:st=3.0:sp=occurrence_300");
    fallback.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:sp=occurrence_300");
    fallback.push("lrs+11_3:2_add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bs=unit_only:cond=on:gs=on:gsaa=full_model:gsem=off:irw=on:lwlo=on:nm=32:nwc=1.2:sas=z3:sac=on_900");
    fallback.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:sos=all:sp=occurrence:urr=on_300");
    fallback.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:sd=2:ss=axioms:st=2.0:sos=all_300");
    fallback.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+1_1024_av=off:bs=on:fde=none:inw=on:irw=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_600");
    fallback.push("lrs+11_1_av=off:bsr=on:gsp=input_only:gs=on:lcm=predicate:nm=64:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+1010_2:3_add=off:afr=on:afp=10000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:tha=off_300");
    fallback.push("dis+1010_7_afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:sp=occurrence:urr=on_300");
    fallback.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("ott+4_32_av=off:bsr=on:cond=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=4:nwc=1.2:sos=theory_300");
    fallback.push("dis+11_32_av=off:ep=RST:fsr=off:lwlo=on:nm=6:nwc=1.1:sd=5:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_1500");
    fallback.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_300");
    fallback.push("lrs+11_50_afp=100000:afq=1.1:amm=sco:anc=none:bs=unit_only:cond=on:irw=on:lma=on:nm=32:nwc=1.1:sp=reverse_arity_300");
    fallback.push("lrs+1011_7_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:cond=on:er=known:fsr=off:lma=on:nm=4:nwc=2.5:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_300");
    fallback.push("dis+1002_6_add=large:afp=40000:afq=2.0:bsr=on:cond=on:irw=on:lma=on:nm=2:nwc=2.5:nicw=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_2:3_add=off:afr=on:afp=4000:afq=1.4:anc=none:bs=unit_only:fsr=off:gs=on:gsem=on:lwlo=on:nm=16:nwc=1.3:nicw=on:sas=z3:sac=on:tha=off_300");
    fallback.push("ott+4_40_av=off:bce=on:cond=fast:fde=none:nm=0:nwc=1:sos=all:updr=off_300");
    fallback.push("dis+1002_7_acc=on:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:ccuc=first:fsr=off:gsp=input_only:gs=on:gsem=on:nm=6:nwc=1.1:nicw=on:sos=on:sac=on:sp=occurrence:urr=ec_only_300");
    fallback.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=on_300");
    fallback.push("lrs+11_8:1_av=off:bd=off:bs=unit_only:gs=on:gsem=on:lma=on:lwlo=on:nm=0:nwc=1:sd=1:ss=axioms:sos=all:urr=on:updr=off_300");
    fallback.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("ott+1002_128_av=off:bd=off:bs=on:bsr=on:cond=on:fsr=off:nm=6:newcnf=on:nwc=1:sp=reverse_arity:updr=off_300");
    fallback.push("ott+1011_4:1_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=preordered:cond=fast:nm=32:newcnf=on:nwc=1.2:sas=z3:sac=on:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott-11_3_add=large:afp=100000:afq=1.2:anc=none:bs=on:cond=fast:fde=none:gs=on:gsem=off:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:sp=occurrence:tha=off:urr=on:uhcvi=on_300");
    fallback.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_300");
    fallback.push("lrs+10_3:1_av=off:bsr=on:cond=on:er=known:gs=on:lcm=reverse:nm=32:nwc=4:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:sd=2:ss=axioms:st=1.5:updr=off_300");
    fallback.push("dis+1010_1_acc=model:add=large:afr=on:amm=off:anc=none:bd=off:bsr=on:ccuc=small_ones:gs=on:gsem=on:nm=16:nwc=2:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:sos=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1011_1_av=off:cond=on:gs=on:lma=on:nm=4:nwc=1:sd=3:ss=axioms:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("ott+11_20_afp=10000:afq=1.1:anc=none:bs=unit_only:bsr=on:bce=on:fsr=off:gs=on:gsem=on:nwc=2.5:sas=z3:sp=occurrence:updr=off:uhcvi=on_600");
    fallback.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:sd=2:ss=axioms:sos=on:sp=reverse_arity_300");
    fallback.push("ott+11_2:1_av=off:bd=off:bsr=on:br=off:cond=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("ott+11_3_afp=1000:afq=2.0:anc=none:fsr=off:irw=on:nwc=1.7:ss=axioms:st=1.5:sac=on:updr=off_300");
    fallback.push("dis+11_3_add=off:afp=10000:afq=2.0:amm=sco:anc=none:ep=RST:gs=on:gsaa=from_current:gsem=on:inw=on:nm=64:nwc=1:sd=10:ss=axioms:st=5.0:sos=all:tha=off:updr=off:uhcvi=on_300");
    fallback.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_300");
    fallback.push("ott+1010_5:4_av=off:bd=off:fde=none:irw=on:lma=on:nm=32:nwc=2.5:sd=2:ss=axioms:st=3.0:urr=on_300");
    fallback.push("dis+11_2:1_add=large:afp=4000:afq=1.4:amm=sco:anc=none:fsr=off:nm=16:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_300");
    fallback.push("ins+1010_3_av=off:bd=off:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.3:igwr=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=1:ss=axioms:sos=on:sp=occurrence:updr=off_300");
    fallback.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_300");
    fallback.push("dis+1_3:1_acc=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:fsr=off:gs=on:inw=on:lma=on:nm=32:nwc=1:urr=on_300");
    fallback.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_4:1_add=large:afp=100000:afq=1.1:anc=none:ep=RST:fde=unused:gsp=input_only:nm=6:newcnf=on:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+10_24_add=off:afr=on:afp=1000:afq=1.4:anc=none:bs=unit_only:gs=on:gsaa=from_current:gsem=on:lma=on:nm=2:nwc=1.1:sac=on:uhcvi=on_600");
    fallback.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("dis+1002_8:1_av=off:br=off:cond=on:irw=on:lma=on:nm=32:nwc=1:sp=occurrence:urr=on_300");
    fallback.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:sac=on:urr=on_300");
    fallback.push("dis+11_1_acc=on:afp=1000:afq=1.4:amm=sco:anc=all_dependent:bs=on:ccuc=small_ones:cond=fast:fde=none:gsp=input_only:nm=64:nwc=1:sac=on:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+1002_3_av=off:cond=on:fsr=off:gs=on:gsem=off:lwlo=on:nm=64:nwc=2.5:sp=reverse_arity_300");
    fallback.push("dis+1_3:2_acc=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:ccuc=small_ones:cond=on:lma=on:nm=64:nwc=1.3:sos=all:urr=on_300");
    fallback.push("dis+1010_24_aac=none:afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:nm=6:nwc=1:sas=z3:sos=on:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+1002_1_av=off:er=filter:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1:sd=3:ss=axioms:st=1.5:sos=on_300");
    fallback.push("dis-10_3:2_aac=none:afp=1000:afq=1.1:cond=on:fsr=off:lcm=reverse:lwlo=on:nm=16:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+10_5_av=off:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("lrs+1002_8:1_add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:er=known:lwlo=on:nm=0:nwc=1.2:sd=1:ss=axioms:sp=occurrence:updr=off_300");
    fallback.push("dis+10_10_add=large:afp=4000:afq=1.1:amm=sco:anc=none:irw=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("lrs+1010_5_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:bd=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=4:sas=z3:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_50_afr=on:afp=40000:afq=1.0:amm=off:anc=all_dependent:bs=on:bsr=on:bce=on:fde=unused:gs=on:lma=on:nm=16:nwc=1.1:sp=occurrence:updr=off_600");
    fallback.push("dis+1011_8:1_aac=none:acc=on:afp=1000:afq=1.4:amm=off:anc=all:bs=unit_only:bce=on:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:lma=on:nm=16:nwc=2.5:sd=4:ss=axioms:st=1.5:sos=all:uhcvi=on_300");
    fallback.push("lrs+11_5_av=off:cond=on:fsr=off:lma=on:lwlo=on:nwc=1.2:sas=z3:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1010_3:2_av=off:lma=on:nm=2:newcnf=on:nwc=1:sd=3:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+10_8:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lcm=predicate:lwlo=on:nm=64:nwc=1:sos=all:updr=off_300");
    fallback.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_2:3_aac=none:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:gs=on:gsem=on:inw=on:newcnf=on:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+11_128_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:nwc=1.3:sac=on:uhcvi=on_300");
    fallback.push("lrs+2_3:1_afr=on:afp=10000:afq=1.0:amm=sco:anc=none:bs=unit_only:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:updr=off_300");
    fallback.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_300");
    fallback.push("dis-10_4:1_aac=none:add=off:afp=1000:afq=1.4:amm=off:anc=none:cond=fast:ep=RSTC:gs=on:gsaa=from_current:gsem=on:inw=on:lma=on:nm=64:nwc=4:sas=z3:tha=off:thi=strong:uwa=interpreted_only:updr=off:uhcvi=on_300");
    fallback.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+1_5:4_aac=none:add=off:afr=on:afp=4000:afq=1.2:amm=sco:anc=none:gsp=input_only:gs=on:irw=on:nm=64:newcnf=on:nwc=1.3:nicw=on:sas=z3:sp=occurrence:tha=off_300");
    fallback.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:sos=theory:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+4_2_av=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:sos=on:urr=on_300");
    fallback.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs-11_24_av=off:bd=off:bs=on:fsr=off:gs=on:gsem=off:lma=on:nm=4:nwc=1.7:sp=occurrence:updr=off_900");
    fallback.push("lrs+10_2:3_afp=1000:afq=1.1:amm=sco:anc=none:er=known:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=2:sd=5:ss=axioms:sos=theory:sac=on:sp=occurrence_300");
    fallback.push("lrs+1010_3_av=off:fsr=off:gs=on:gsem=off:nm=2:newcnf=on:nwc=2:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1011_3:2_av=off:er=known:lma=on:nm=2:newcnf=on:nwc=2:sd=2:ss=axioms:st=3.0:urr=on:updr=off_300");
    fallback.push("lrs+4_5:4_av=off:bd=off:er=filter:lma=on:lwlo=on:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("ott+11_20_afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bsr=on:irw=on:lma=on:nm=4:nwc=1.2:sac=on:sp=occurrence_300");
    fallback.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+1010_8:1_add=off:afp=4000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=4:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:sd=1:ss=axioms:st=2.0:sac=on:urr=on_300");
    fallback.push("ott+1_5:1_acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:lcm=predicate:nm=16:nwc=1.1:sd=1:ss=axioms:st=3.0:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:sd=1:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("dis+1010_3:2_av=off:gsp=input_only:nm=2:nwc=1:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:sd=2:ss=axioms:sos=all:urr=ec_only_300");
    fallback.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+1004_5:4_aac=none:add=large:afp=100000:afq=1.4:anc=all_dependent:bd=off:cond=fast:gsp=input_only:gs=on:gsem=off:lma=on:lwlo=on:nm=4:newcnf=on:nwc=1:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+2_128_add=large:afp=100000:afq=1.4:amm=sco:anc=none:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sos=theory:sac=on:updr=off_600");
    fallback.push("ott+10_8:1_av=off:bd=preordered:bsr=on:cond=fast:fsr=off:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1.2:sp=reverse_arity:urr=on:updr=off:uhcvi=on_1200");
    fallback.push("dis+1002_4_add=off:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1:sos=on:sac=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_300");
    fallback.push("ott+1011_3:1_aac=none:add=off:afp=1000:afq=1.2:amm=sco:bd=off:bs=on:bsr=on:cond=on:gsp=input_only:gs=on:lma=on:nm=6:newcnf=on:nwc=1.3:nicw=on:sd=3:ss=axioms:st=2.0:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+10_4_add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=predicate:lma=on:nm=64:nwc=1:sd=3:ss=axioms:sos=on:sp=reverse_arity_300");
    fallback.push("lrs+1011_4:1_av=off:fsr=off:irw=on:nwc=1:sd=4:ss=axioms:st=1.5:sp=reverse_arity_300");
    fallback.push("dis+1_2:3_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:nm=64:newcnf=on:nwc=1_300");
    fallback.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_300");
    fallback.push("lrs+1002_2:1_aac=none:afr=on:afp=1000:afq=1.2:anc=all:bd=preordered:bsr=on:cond=fast:gsp=input_only:gs=on:nm=0:nwc=2.5:nicw=on:sas=z3:sd=4:ss=axioms:st=2.0:sos=on:sac=on:urr=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1004_1_aac=none:acc=on:afp=40000:afq=1.2:anc=none:cond=on:fde=unused:gs=on:gsem=off:irw=on:nm=32:nwc=2:sd=1:ss=axioms:sos=theory:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+11_3_afp=100000:afq=2.0:amm=sco:anc=none:gs=on:gsem=off:inw=on:lma=on:nm=64:nwc=1:sas=z3:sd=10:ss=axioms:st=5.0:sp=occurrence:tha=off:updr=off_300");
    fallback.push("dis+1010_2_afr=on:afp=1000:afq=1.1:amm=off:anc=none:bs=unit_only:bce=on:cond=fast:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=5:nicw=on:sas=z3:sac=on:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+1002_16_av=off:cond=on:nwc=3_300");
    fallback.push("lrs+11_3_av=off:bce=on:cond=fast:ep=R:lcm=reverse:lma=on:newcnf=on:nwc=1.3:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence:uhcvi=on_300");
    fallback.push("dis-3_5_av=off:cond=on:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1:sos=on:sp=reverse_arity_300");
    fallback.push("ott+1_8_av=off:bd=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:lwlo=on:nwc=1:sos=on_300");
    fallback.push("lrs+10_3_add=off:afp=40000:afq=1.4:amm=off:anc=none:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=full_model:inw=on:lma=on:nm=64:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1010_4_afp=10000:afq=1.2:anc=none:irw=on:lma=on:nm=64:nwc=10:sas=z3:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis-3_4_add=off:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:inw=on:lma=on:nm=64:nwc=1.5:nicw=on:sas=z3:sp=reverse_arity:tha=off:thf=on:uhcvi=on_300");
    fallback.push("ott-2_28_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=16:newcnf=on:nwc=1:nicw=on:sp=occurrence:thf=on_300");
    fallback.push("dis+3_24_av=off:bd=off:bs=unit_only:fsr=off:fde=unused:gs=on:irw=on:lma=on:nm=0:nwc=1.1:sos=on:uhcvi=on_300");
    fallback.push("lrs+1011_14_av=off:fsr=off:irw=on:nwc=1:sos=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_300");
    fallback.push("dis+11_28_av=off:fsr=off:irw=on:lcm=predicate:nm=2:newcnf=on:nwc=5:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_3_av=off:cond=on:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+10_4_av=off:bsr=on:cond=fast:er=filter:fde=none:gsp=input_only:lcm=reverse:lma=on:nwc=4:sp=occurrence:urr=on_300");
    fallback.push("lrs+10_8:1_av=off:bs=unit_only:cond=on:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:nwc=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1011_5_afr=on:afp=100000:afq=1.0:amm=off:anc=none:cond=on:lma=on:nm=6:nwc=1:sas=z3:sac=on:urr=on_300");
    fallback.push("dis+11_3_afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=preordered:bce=on:fsr=off:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1:sd=10:ss=axioms:st=5.0:sac=on:sp=occurrence:tha=off:urr=ec_only_300");
    fallback.push("lrs+1_3:2_aac=none:add=large:anc=all_dependent:bce=on:cond=fast:ep=RST:fsr=off:lma=on:nm=2:nwc=1:sos=on:sp=occurrence:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_7_av=off:cond=fast:fde=none:gs=on:gsem=off:lcm=predicate:nm=6:nwc=1:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_3:2_aac=none:afp=10000:afq=1.2:amm=off:anc=all:cond=on:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+10_5:4_afr=on:afp=40000:afq=1.2:bd=off:gsp=input_only:gs=on:inw=on:nm=0:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:thf=on:urr=on_300");
    fallback.push("ott+11_50_add=large:afr=on:afp=40000:afq=1.1:amm=sco:anc=all:bd=off:cond=fast:gs=on:gsem=on:irw=on:lma=on:nm=0:nwc=1:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+1002_8_add=large:afp=100000:afq=1.2:amm=off:bs=on:irw=on:nm=2:newcnf=on:nwc=1.1:sos=on:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+1010_2:3_afr=on:afp=40000:afq=1.4:amm=off:anc=none:lma=on:nm=16:nwc=1:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+11_6_add=large:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=fast:gs=on:gsaa=from_current:gsem=off:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:thi=strong:updr=off_300");
    fallback.push("lrs+11_2:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=16:nwc=1:sd=1:ss=axioms:st=1.2:sp=reverse_arity_300");
    fallback.push("dis+4_8:1_add=large:afp=100000:afq=1.4:ep=RST:fde=unused:gsp=input_only:lcm=predicate:nwc=1:sos=all:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+1010_3:1_av=off:gsp=input_only:nm=6:nwc=1:sos=all:sp=occurrence_300");
    fallback.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_300");
    fallback.push("dis+4_2_av=off:bs=on:fsr=off:gsp=input_only:newcnf=on:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+1010_4_add=off:afp=100000:afq=1.0:anc=none:fsr=off:gs=on:gsem=off:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sac=on:tha=off:thf=on_300");
    fallback.push("dis+1011_5_aac=none:add=large:afp=40000:afq=1.2:amm=off:anc=none:bd=off:fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:sos=all:updr=off_300");
    fallback.push("lrs+4_1_acc=on:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bs=on:bsr=on:ccuc=first:fsr=off:fde=unused:irw=on:lma=on:nm=0:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=all:sp=occurrence:uhcvi=on_300");
    fallback.push("dis+1011_10_aac=none:add=large:afp=10000:afq=1.1:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=6:newcnf=on:nwc=2.5:sp=reverse_arity:updr=off_300");
    fallback.push("dis+10_5_add=off:afp=4000:afq=1.1:anc=none:cond=fast:ep=RSTC:fsr=off:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sp=reverse_arity:thi=all_300");
    fallback.push("ott-11_12_aac=none:afp=100000:afq=1.2:amm=sco:bs=on:bce=on:cond=fast:er=known:gs=on:gsaa=from_current:gsem=off:irw=on:nm=4:nwc=2:sas=z3:sos=all:sp=occurrence:urr=ec_only:updr=off_600");
    fallback.push("lrs+11_3:2_add=large:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:er=filter:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:sas=z3:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+11_20_afr=on:afp=100000:afq=1.2:amm=off:anc=none:bs=unit_only:bsr=on:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lma=on:lwlo=on:nm=16:newcnf=on:nwc=1:nicw=on:sas=z3:sac=on:updr=off:uhcvi=on_300");
    fallback.push("ott+10_7_av=off:bd=preordered:bs=unit_only:bsr=on:cond=fast:fsr=off:fde=unused:irw=on:lma=on:nm=32:nwc=1:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_3_afp=100000:afq=1.1:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:irw=on:nm=6:nwc=1:sas=z3:sd=2:ss=axioms:sos=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_24_add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=fast:fde=unused:gs=on:irw=on:lma=on:nm=4:nwc=1.3:nicw=on:sas=z3:sac=on:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+1010_2_add=large:afp=4000:afq=2.0:amm=off:bd=off:bs=unit_only:bsr=on:br=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=32:nwc=1.1:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs-2_3:1_add=off:afr=on:afp=1000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bs=unit_only:bsr=on:cond=on:fde=unused:gs=on:gsem=on:irw=on:lcm=reverse:nm=32:nwc=1.7:sas=z3:sos=all:sac=on_300");
    fallback.push("lrs+11_4:1_av=off:bd=off:br=off:cond=fast:fde=unused:irw=on:lcm=reverse:lma=on:lwlo=on:nm=4:nwc=1:sd=3:ss=axioms:sos=all:urr=on_600");
    fallback.push("ott+11_128_afp=4000:afq=1.1:anc=none:bs=unit_only:cond=on:gsp=input_only:lma=on:nm=4:nwc=1.5:sos=theory:sp=occurrence:updr=off_300");
    fallback.push("lrs-1_14_add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:br=off:cond=fast:fde=unused:gs=on:lcm=reverse:lma=on:nwc=1:sos=on:urr=on:updr=off_300");
    fallback.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_300");
    fallback.push("dis+10_3:2_afr=on:afp=1000:afq=1.2:bd=off:irw=on:lcm=predicate:lwlo=on:nm=0:newcnf=on:nwc=2:sos=on:tha=off:thf=on:urr=ec_only_300");
    fallback.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_300");
    fallback.push("lrs+10_128_acc=model:afp=100000:afq=2.0:anc=all_dependent:bs=on:bsr=on:cond=fast:er=filter:gs=on:gsem=off:lcm=reverse:lma=on:nm=32:nwc=3:sac=on:sp=occurrence:urr=ec_only_300");
    fallback.push("lrs+1011_2:3_add=large:afr=on:afp=100000:afq=1.2:anc=none:gs=on:gsem=off:irw=on:nm=64:nwc=1:sd=3:ss=axioms:sos=all:sp=reverse_arity_300");
    fallback.push("ott+1002_8:1_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:fsr=off:gs=on:gsem=off:nm=32:nwc=10:sas=z3:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+11_3_afr=on:afp=10000:afq=1.0:cond=fast:fsr=off:fde=none:gs=on:gsem=off:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=2.0:sos=all:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+1010_5_add=large:afp=10000:afq=1.2:amm=off:bs=unit_only:bsr=on:bce=on:cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1.3:nicw=on:sos=all:sac=on:updr=off_300");
    fallback.push("dis+2_2:1_aac=none:afp=100000:afq=1.1:amm=sco:anc=none:bsr=on:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=on_300");
    fallback.push("dis+11_8_add=off:afp=10000:afq=1.2:amm=off:anc=none:cond=fast:ep=R:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=4:sd=1:ss=axioms:sac=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_8:1_av=off:cond=fast:fde=unused:lcm=predicate:nm=16:nwc=10:sp=occurrence:urr=ec_only_600");
    fallback.push("lrs+1011_1024_av=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=32:nwc=1:sp=reverse_arity:uhcvi=on_900");
    fallback.push("dis+11_7_av=off:bd=preordered:bs=unit_only:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:nm=32:nwc=1:sd=4:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_1200");
    break;

  case Property::FNE:
    fallback.push("dis+1011_10_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=64:nwc=4:sac=on:sp=occurrence_300");
    fallback.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_300");
    fallback.push("dis+1_2_add=large:afr=on:afp=1000:afq=2.0:anc=none:gsp=input_only:lcm=predicate:nm=64:newcnf=on:nwc=5:sac=on:urr=ec_only:updr=off_300");
    fallback.push("dis+10_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:irw=on:nm=64:nwc=1:sas=z3:sac=on_300");
    fallback.push("dis+11_24_afp=40000:afq=1.1:amm=sco:anc=none:bs=on:gs=on:gsem=off:lcm=predicate:lma=on:nm=2:nwc=1:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+1011_1024_add=large:afp=4000:afq=1.1:anc=none:br=off:fsr=off:gsp=input_only:lma=on:nwc=1:sos=on:urr=on_300");
    fallback.push("dis+10_50_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:gs=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:thf=on:updr=off_300");
    fallback.push("lrs+4_32_add=large:afp=10000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lcm=predicate:lma=on:nm=2:nwc=1:sac=on:sp=occurrence:urr=on_300");
    fallback.push("dis+11_3_afr=on:afp=4000:afq=1.4:anc=none:cond=on:fsr=off:gs=on:lcm=reverse:nm=64:nwc=1:sos=on:sp=reverse_arity_300");
    fallback.push("lrs+1011_2:1_add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=64:nwc=1:sas=z3:sos=on:sp=reverse_arity:thf=on:urr=on:updr=off_300");
    fallback.push("lrs+10_4_add=off:afp=100000:afq=2.0:amm=sco:anc=none:nm=64:nwc=1:sp=occurrence:updr=off_1500");
    fallback.push("dis+10_128_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=2:nwc=1:sp=reverse_arity_300");
    fallback.push("ott+1_40_av=off:bs=unit_only:bsr=on:br=off:fsr=off:lma=on:nm=64:newcnf=on:nwc=1.5:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+1_1_av=off:bsr=on:br=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:nwc=1:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+11_5:1_afp=100000:afq=1.1:br=off:gs=on:nm=64:nwc=1:sos=all:urr=on:updr=off_300");
    fallback.push("ins+11_32_av=off:igbrr=0.4:igrr=1/64:igrpq=1.05:igwr=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1011_40_add=off:afr=on:afp=4000:afq=1.2:amm=sco:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:sos=all:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_600");
    fallback.push("ott+11_3:2_afp=40000:afq=1.0:amm=sco:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=full_model:lcm=reverse:nm=32:newcnf=on:nwc=5:nicw=on:sd=3:ss=axioms:sac=on:urr=on:updr=off_1200");
    break;
  }

  // add very long final fallback strategy
  fallback.push("dis+11_1_3600");
} // getCasc2018Schedule

/**
 * Get schedules for a problem of given property.
 * The schedules are to be executed from the bottom of the stack, i.e. in the order in which they are mentioned in the file.
 */
void Schedules::getCasc2019Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category();
  unsigned long prop = property.props();
  unsigned atoms = property.atoms();

  // for theory problems, we make the schedule before the main choice
  if (prop & (524288ul | 1048576ul | 2097152ul)) { // contains integers, rationals and reals
    if ((prop & 67108864ul) == 0ul) { // no linear integer functions
      quick.push("dis+11_4_afp=100000:afq=1.1:anc=none:cond=on:gs=on:gsaa=full_model:nm=64:nwc=1:sac=on:sp=reverse_arity:thi=all_2");
      quick.push("ott+1010_2:1_awrs=decay:awrsf=512:acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:ccuc=first:fsr=off:fde=unused:gsp=input_only:gs=on:gsaa=from_current:irw=on:nm=32:newcnf=on:nwc=1:sos=theory:sp=occurrence:tha=some:uwa=interpreted_only:updr=off_8");
      quick.push("lrs+1010_4:1_aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=on:gs=on:gsem=on:irw=on:nm=0:nwc=2.5:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_2");
      quick.push("dis+10_5_add=off:afp=4000:afq=1.1:anc=none:cond=fast:ep=RSTC:fsr=off:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sp=reverse_arity:thi=all_3");
      quick.push("dis+10_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:irw=on:nm=64:nwc=1:sas=z3:sac=on_2");
      quick.push("dis+11_3_add=off:afp=10000:afq=2.0:amm=sco:anc=none:ep=RST:gs=on:gsaa=from_current:gsem=on:inw=on:nm=64:nwc=1:sd=10:ss=axioms:st=5.0:sos=all:tha=off:updr=off:uhcvi=on_59");
      quick.push("lrs+10_2_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:inw=on:nm=64:nwc=1:sas=z3:stl=30:sos=all:sp=occurrence:tha=off:thf=on:urr=on:updr=off:uhcvi=on_2");
      quick.push("lrs+1_1024_av=off:bs=on:fde=none:inw=on:irw=on:nm=64:nwc=1.2:stl=60:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_595");
      quick.push("dis+1002_8:1_awrs=converge:awrsf=256:anc=all_dependent:br=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:irw=on:nm=64:nwc=1:sas=z3:s2a=on:sp=frequency:thf=on:uwa=interpreted_only:urr=on_229");
      quick.push("dis+10_10_add=large:afp=4000:afq=1.1:amm=sco:anc=none:irw=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=on_30");
      quick.push("ott-2_64_add=large:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:bd=off:bs=unit_only:bsr=on:bce=on:cond=on:fde=none:gs=on:gsem=on:newcnf=on:nwc=1:sac=on:sp=occurrence:thf=on:updr=off:uhcvi=on_154");
      quick.push("dis+11_3_afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=preordered:bce=on:fsr=off:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1:sd=10:ss=axioms:st=5.0:sac=on:sp=occurrence:tha=off:urr=ec_only_85");
      quick.push("ott-11_3_add=large:afp=100000:afq=1.2:anc=none:bs=on:cond=fast:fde=none:gs=on:gsem=off:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:sp=occurrence:tha=off:urr=on:uhcvi=on_268");
    }
    else if (prop == 604504064ul) {
      quick.push("dis+11_4_afp=100000:afq=1.1:anc=none:cond=on:gs=on:gsaa=full_model:nm=64:nwc=1:sac=on:sp=reverse_arity:thi=all_2");
      quick.push("dis+1011_4_awrs=decay:awrsf=32:afp=40000:afq=1.0:amm=off:anc=all:bs=on:cond=on:fsr=off:gsp=input_only:lma=on:nm=16:nwc=1:nicw=on:sac=on:sp=frequency:thi=all:updr=off:uhcvi=on_670");
    }
    else {
      quick.push("ott+10_3:2_aac=none:add=large:afp=10000:afq=2.0:amm=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=reverse:lma=on:nm=0:nwc=4:sas=z3:updr=off_22");
      quick.push("ott+1_3:1_av=off:bd=off:fsr=off:fde=none:gs=on:inw=on:nm=2:nwc=1.5:sp=frequency:uwa=one_side_interpreted_22");
      quick.push("dis+1011_5_aac=none:add=large:afp=40000:afq=1.2:amm=off:anc=none:bd=off:fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:updr=off_26");
      quick.push("lrs+1_3_awrs=decay:awrsf=4:afp=10000:afq=1.0:amm=off:anc=none:bd=off:cond=on:fsr=off:fde=unused:gs=on:lwlo=on:nm=16:nwc=1:sas=z3:stl=30:ss=axioms:s2a=on:st=1.2:sos=theory:sp=frequency_29");
      quick.push("lrs-11_1_av=off:cond=on:gs=on:lcm=reverse:lma=on:lwlo=on:nm=16:nwc=5:stl=30:sp=reverse_arity:tha=off:thi=strong:uwa=interpreted_only_58");
      quick.push("dis-3_4_add=off:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:inw=on:lma=on:nm=64:nwc=1.5:nicw=on:sas=z3:sp=reverse_arity:tha=off:thf=on:uhcvi=on_13");
      quick.push("lrs+1_5:4_aac=none:add=off:afr=on:afp=4000:afq=1.2:amm=sco:anc=none:gsp=input_only:gs=on:irw=on:nm=64:newcnf=on:nwc=1.3:nicw=on:sas=z3:stl=30:sp=occurrence:tha=off_3");
      quick.push("dis+1002_8:1_awrs=converge:awrsf=256:anc=all_dependent:br=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:irw=on:nm=64:nwc=1:sas=z3:s2a=on:sp=frequency:thf=on:uwa=interpreted_only:urr=on_7");
      quick.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_12");
      quick.push("lrs+3_128_awrs=converge:awrsf=2:av=off:bs=on:cond=fast:fsr=off:fde=unused:gsp=input_only:irw=on:lma=on:nm=64:nwc=1.5:stl=30:sp=frequency:tha=some:updr=off_148");
      quick.push("dis+11_6_add=large:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=fast:gs=on:gsaa=from_current:gsem=off:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:thi=strong:updr=off_2");
      quick.push("dis+10_3:2_afr=on:afp=1000:afq=1.2:bd=off:irw=on:lcm=predicate:lwlo=on:nm=0:newcnf=on:nwc=2:sos=on:tha=off:thf=on:urr=ec_only_11");
      quick.push("dis+1010_2:3_add=off:afr=on:afp=10000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:tha=off_5");
      quick.push("dis-10_4:1_aac=none:add=off:afp=1000:afq=1.4:amm=off:anc=none:cond=fast:ep=RSTC:gs=on:gsaa=from_current:gsem=on:inw=on:lma=on:nm=64:nwc=4:sas=z3:tha=off:thi=strong:uwa=interpreted_only:updr=off:uhcvi=on_6");
      quick.push("lrs+1_2:3_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:lma=on:nm=64:nwc=1.3:sas=z3:stl=30:sac=on:tha=off:uwa=one_side_interpreted:updr=off_9");
      quick.push("lrs+10_2_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:inw=on:nm=64:nwc=1:sas=z3:stl=30:sos=all:sp=occurrence:tha=off:thf=on:urr=on:updr=off:uhcvi=on_6");
      quick.push("lrs+1011_1_add=off:afp=100000:afq=1.0:anc=none:cond=on:gs=on:gsaa=from_current:gsem=on:inw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sos=on:sp=occurrence:tha=off:uwa=ground_3");
      quick.push("dis+1010_4_add=off:afp=100000:afq=1.0:anc=none:fsr=off:gs=on:gsem=off:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sac=on:tha=off:thf=on_179");
      quick.push("lrs-2_24_awrs=converge:awrsf=64:av=off:bd=off:bs=on:bsr=on:br=off:cond=on:fde=none:gsp=input_only:inw=on:lwlo=on:nm=6:nwc=4:stl=30:s2a=on:sos=all:sp=weighted_frequency:thf=on:uwa=one_side_interpreted:urr=on:updr=off:uhcvi=on:uwaf=on_21");
      quick.push("ott+10_8_add=large:afp=100000:afq=1.4:amm=sco:cond=fast:fsr=off:fde=none:lcm=predicate:lma=on:nm=32:nwc=1:sos=on:sac=on:tha=off:updr=off_29");
      quick.push("dis-2_2:3_add=large:afp=40000:afq=1.4:amm=off:anc=none:gsp=input_only:gs=on:gsem=on:inw=on:lcm=reverse:lma=on:nm=2:nwc=1:nicw=on:sas=z3:sos=all:sp=reverse_arity:tha=off:urr=on_5");
      quick.push("dis+1_3:1_acc=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:fsr=off:gs=on:inw=on:lma=on:nm=32:nwc=1:urr=on_2");
      quick.push("dis+1011_2:3_add=off:afr=on:afp=4000:afq=1.4:anc=none:bs=unit_only:fsr=off:gs=on:gsem=on:lwlo=on:nm=16:nwc=1.3:nicw=on:sas=z3:sac=on:tha=off_260");
      quick.push("dis+1010_24_aac=none:afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:nm=6:nwc=1:sas=z3:sos=on:sp=reverse_arity:tha=off_9");
      quick.push("dis+11_3_add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on:updr=off_92");
      quick.push("lrs+1003_2_awrs=converge:awrsf=512:add=large:afp=10000:afq=1.1:amm=sco:anc=none:cond=fast:fde=unused:lma=on:nm=64:nwc=1.2:stl=30:s2a=on:sac=on:sp=reverse_arity:tha=some:thi=new:urr=on:updr=off_80");
      quick.push("ott+10_4_awrs=converge:awrsf=128:afp=100000:afq=1.4:amm=sco:anc=none:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:irw=on:nm=64:nwc=1:s2a=on:sac=on:sp=frequency:tha=off_3");
      quick.push("dis+10_6_afr=on:afp=1000:afq=1.2:anc=none:bsr=on:fsr=off:gs=on:lcm=reverse:nm=64:newcnf=on:nwc=1.7:sas=z3:tha=off_4");
      quick.push("lrs+10_8:1_aac=none:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bd=off:cond=on:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=1.2:sas=z3:stl=30:sos=all:sp=reverse_arity:tha=off:updr=off_68");
      quick.push("dis+1002_4_add=off:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1:sos=on:sac=on:sp=occurrence:tha=off:updr=off_2");
      quick.push("lrs-11_3:1_awrs=converge:awrsf=1:av=off:bce=on:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nm=16:newcnf=on:nwc=2:stl=90:s2a=on:sos=theory:sp=weighted_frequency:tha=some:uwa=one_side_constant:urr=on:updr=off:uhcvi=on_234");
      quick.push("dis-1_2:1_afr=on:afp=10000:afq=2.0:anc=none:cond=on:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:inw=on:irw=on:lcm=predicate:lma=on:nm=32:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:uwa=ground_4");
      quick.push("lrs+10_4:1_av=off:bd=off:bsr=on:cond=on:fde=unused:inw=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=5:stl=90:sp=reverse_arity:thi=strong:uwa=ground:updr=off:uwaf=on_359");
      quick.push("dis+11_3_afp=100000:afq=2.0:amm=sco:anc=none:gs=on:gsem=off:inw=on:lma=on:nm=64:nwc=1:sas=z3:sd=10:ss=axioms:st=5.0:sp=occurrence:tha=off:updr=off_202");
      quick.push("lrs-11_4_awrs=decay:awrsf=64:afp=1000:afq=2.0:amm=off:anc=none:br=off:cond=on:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:lma=on:nm=32:nwc=1.1:stl=30:s2a=on:sos=theory:sac=on:sp=frequency:tha=some:thi=new:urr=on:uhcvi=on_6");
      quick.push("ott+1004_5_av=off:bd=off:bs=on:cond=on:fde=none:lma=on:nm=64:nwc=1:sos=on:sp=reverse_arity:tha=off:thi=strong:uwa=one_side_constant_143");
      quick.push("lrs+1011_7_aac=none:add=large:afr=on:afp=40000:afq=1.4:bd=off:bs=on:bsr=on:fsr=off:inw=on:lma=on:nm=64:nwc=2:nicw=on:sas=z3:stl=60:sos=all:sp=reverse_arity:tha=off:updr=off:uhcvi=on_541");
    }
    // add very long final fallback strategy from UFLIA problems
    fallback.push("lrs+1011_2:1_afr=on:afp=10000:afq=2.0:amm=off:gsp=input_only:gs=on:inw=on:ile=on:nm=2:nwc=1:sas=z3:tha=off_300");
    fallback.push("ott+1010_2:1_acc=on:add=large:afr=on:afp=40000:afq=1.1:anc=none:gs=on:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("dis+2_1_add=large:afr=on:afp=1000:afq=1.2:anc=none:cond=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_300");
    fallback.push("ott+10_4:1_aac=none:add=off:afp=40000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1011_2_acc=on:afp=10000:afq=1.1:amm=sco:anc=none:ccuc=small_ones:cond=fast:fde=unused:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off:updr=off:uhcvi=on_300");
    fallback.push("ott+11_5:4_aac=none:add=large:afp=4000:afq=1.4:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=on:inw=on:ile=on:nm=2:newcnf=on:nwc=1:sas=z3:sos=on:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("dis+10_14_add=large:afp=4000:afq=1.1:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=none:inw=on:irw=on:lcm=predicate:nm=4:nwc=1.1:sos=on:sac=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1002_5:4_add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:updr=off_300");
    fallback.push("lrs+11_5:1_add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:er=known:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=predicate:lwlo=on:nm=64:newcnf=on:nwc=1.1:sac=on:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("dis+1011_5:1_afp=4000:afq=1.4:amm=off:anc=none:cond=on:fde=unused:gsp=input_only:ile=on:lma=on:nm=16:nwc=1:sos=on:sac=on:tha=off:urr=ec_only:uhcvi=on_300");
    fallback.push("dis+1010_1_add=off:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=6:nwc=1.3:nicw=on:sas=z3:tha=off:urr=ec_only_300");
    fallback.push("dis+1002_2_aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=all:bsr=on:fde=unused:inw=on:ile=on:lcm=reverse:nm=4:nwc=4:nicw=on:sos=theory:sac=on:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+1011_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off_300");
    fallback.push("ott+1002_3:1_av=off:bsr=on:ep=RS:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_3_add=off:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:bce=on:cond=fast:fde=unused:ile=on:lma=on:nm=6:nwc=2:nicw=on:sas=z3:sd=3:ss=axioms:st=2.0:sp=reverse_arity:tha=off_300");
    fallback.push("ott+1011_3:2_av=off:bd=off:bs=on:bce=on:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:tha=off:updr=off_300");
    fallback.push("dis+11_3:1_av=off:br=off:ep=R:fsr=off:gsp=input_only:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("lrs-2_8:1_add=large:afp=100000:afq=1.4:amm=sco:anc=none:bs=on:bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=4:nicw=on:sas=z3:sp=reverse_arity:updr=off_600");
    fallback.push("dis+1010_2_acc=on:afr=on:afp=100000:afq=1.2:anc=none:bsr=on:fsr=off:ile=on:irw=on:nm=16:newcnf=on:nwc=4:sp=occurrence:tha=off:urr=ec_only_300");
    fallback.push("ott+1010_1_add=large:afp=1000:afq=1.2:anc=none:bd=off:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("lrs+1002_2_add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:inw=on:lwlo=on:nm=32:newcnf=on:nwc=1:sos=theory:sac=on:sp=occurrence:urr=on_300");
    fallback.push("lrs-10_3_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=unused:gs=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.2:sas=z3:tha=off:urr=ec_only_300");
    fallback.push("dis+10_3_add=off:afp=100000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sp=reverse_arity:tha=off:thf=on:updr=off_300");
    fallback.push("lrs+2_4_add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:cond=on:ep=R:gs=on:gsaa=from_current:gsem=off:ile=on:lcm=reverse:lma=on:nm=2:nwc=1.1:sos=on:sac=on:tha=off:updr=off_300");
    fallback.push("lrs+2_1024_av=off:bd=off:bsr=on:cond=fast:fsr=off:fde=none:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:tha=off:thi=overlap:uwa=one_side_constant:updr=off:uhcvi=on_300");
    fallback.push("ott+1011_8:1_afr=on:afp=1000:afq=1.4:amm=sco:anc=none:bd=off:fsr=off:fde=unused:inw=on:ile=on:nm=2:nwc=1:nicw=on:sas=z3:sos=theory:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_600");
    fallback.push("lrs+10_2:3_afr=on:afp=1000:afq=1.1:bd=off:bce=on:cond=on:gsp=input_only:gs=on:gsaa=from_current:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:uwa=interpreted_only:updr=off:uhcvi=on_300");
    fallback.push("dis+10_32_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:fde=none:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sp=occurrence:tha=off:thi=full:uhcvi=on_300");
    fallback.push("ott+1010_2:1_add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:cond=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=1.5:sac=on:tha=off:updr=off_300");
    fallback.push("lrs-11_2:1_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.5:sas=z3:sp=reverse_arity:urr=on_300");
    fallback.push("dis+11_7_add=large:afr=on:afp=10000:afq=1.2:bd=off:bsr=on:cond=on:fsr=off:fde=unused:gs=on:ile=on:lcm=predicate:lma=on:nm=2:newcnf=on:nwc=3:sos=on:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1011_1_afp=40000:afq=1.2:anc=none:cond=on:gsp=input_only:ile=on:irw=on:lma=on:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+10_3_av=off:fde=unused:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1.7:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_600");
    fallback.push("lrs+11_2:1_add=off:anc=none:bsr=on:br=off:cond=on:er=filter:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=2:nwc=1:sas=z3:sos=all:sac=on:uwa=ground:urr=on_300");
    fallback.push("lrs+1003_3:2_afp=1000:afq=2.0:amm=off:anc=none:cond=on:gs=on:ile=on:lma=on:nm=6:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=all:updr=off_300");
    fallback.push("lrs+4_8:1_av=off:cond=on:gs=on:gsem=on:irw=on:nm=64:newcnf=on:nwc=1.1:sp=occurrence:tha=off:urr=on:updr=off_300");
    fallback.push("ott-4_5:4_aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:cond=fast:ile=on:irw=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=3:thf=on:urr=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1011_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=4:nwc=1:sac=on:tha=off:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_2:1_av=off:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sp=occurrence:tha=off:urr=ec_only:uhcvi=on_300");
    fallback.push("dis+1011_4_afr=on:afp=10000:afq=1.1:amm=off:anc=none:ep=RS:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1010_1_afp=1000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:br=off:cond=on:ile=on:irw=on:nm=2:nwc=1:nicw=on:sas=z3:sos=all:sp=reverse_arity:tha=off:urr=on:updr=off_300");
    fallback.push("lrs+10_24_afp=4000:afq=2.0:bd=off:bsr=on:bce=on:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:nwc=1.3:sp=occurrence:tha=off:uwa=one_side_constant:urr=ec_only_300");
    fallback.push("lrs-2_5:1_acc=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bd=off:cond=fast:gs=on:ile=on:nm=0:newcnf=on:nwc=3:sac=on:thf=on:urr=ec_only_300");
    fallback.push("lrs+1_5:1_add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity_300");
    fallback.push("lrs+1010_1_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:fsr=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on_300");
    fallback.push("lrs+1011_2:1_aac=none:add=off:afp=1000:afq=1.0:amm=off:bs=on:gs=on:gsaa=from_current:gsem=on:ile=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off_300");
    fallback.push("dis+1010_2_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:fsr=off:fde=none:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:sp=reverse_arity_300");
    fallback.push("ott+1011_3:1_add=off:afp=100000:afq=2.0:amm=off:anc=none:bs=unit_only:gs=on:gsem=on:irw=on:newcnf=on:nwc=1:sas=z3:tha=off_300");
    fallback.push("dis-3_7_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sp=occurrence:tha=off:thi=overlap:uwa=interpreted_only:uhcvi=on_300");
    fallback.push("dis+11_32_add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:er=filter:ile=on:lcm=predicate:lma=on:newcnf=on:nwc=5:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_2:1_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=preordered:ccuc=first:cond=fast:fsr=off:fde=unused:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=2:nicw=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+2_8_av=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1002_1_add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:fsr=off:gsp=input_only:inw=on:ile=on:lcm=predicate:lwlo=on:nm=64:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("ott+10_4:1_afp=100000:afq=1.1:anc=none:bd=off:inw=on:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sac=on:sp=occurrence:tha=off:urr=on:updr=off_300");
    fallback.push("dis+1011_5:1_afr=on:afp=10000:afq=1.2:amm=sco:bd=preordered:bs=unit_only:cond=on:fsr=off:inw=on:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=1.1:sd=7:ss=axioms:st=1.2:tha=off:uhcvi=on_300");
    fallback.push("dis+1011_3_aac=none:afr=on:afp=40000:afq=1.4:amm=off:bs=on:fsr=off:fde=none:gsp=input_only:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+4_4_av=off:bd=off:bs=unit_only:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:updr=off_300");
    fallback.push("lrs+11_10_add=off:afp=40000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bce=on:cond=fast:gsp=input_only:inw=on:lma=on:nm=4:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:urr=ec_only:uhcvi=on_300");
    fallback.push("ott+1011_2:3_av=off:bs=unit_only:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thi=all:uwa=all:urr=on:uhcvi=on_300");
    fallback.push("lrs+11_6_aac=none:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:cond=fast:ile=on:nm=16:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:uhcvi=on_300");
    fallback.push("ott+1011_2:3_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:bce=on:cond=fast:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1.1:sp=reverse_arity:tha=off:urr=on:updr=off_300");
    fallback.push("lrs+4_8:1_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:tha=off_300");
    fallback.push("lrs-2_3:1_add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=on:er=filter:fde=unused:ile=on:irw=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on:thi=strong:uhcvi=on_600");
    fallback.push("dis+1011_10_av=off:bd=off:cond=fast:er=known:inw=on:ile=on:irw=on:lma=on:nwc=1.7:sp=occurrence:tha=off:uhcvi=on_300");
    fallback.push("dis+1010_3_afp=10000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:tha=off:urr=on_300");
    fallback.push("lrs+2_3:2_av=off:cond=fast:inw=on:ile=on:nm=2:nwc=1:sos=theory:urr=on_300");
    fallback.push("lrs+11_2_av=off:cond=on:fsr=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on_300");
    fallback.push("dis+10_2:1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=on:inw=on:ile=on:irw=on:nm=2:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
    fallback.push("ott+1003_12_add=large:anc=all:bd=preordered:bce=on:fde=none:lcm=reverse:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:uwa=ground_600");
    fallback.push("dis+1011_1_aac=none:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bce=on:cond=on:er=known:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=off:ile=on:newcnf=on:nwc=1:sas=z3:ss=axioms:st=2.0:sp=occurrence:tha=off:urr=ec_only_300");
    fallback.push("dis+1002_1_add=large:afp=4000:afq=1.2:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only:uhcvi=on_300");
    fallback.push("dis+2_4_afp=10000:afq=1.1:bd=off:bs=on:cond=on:er=filter:ile=on:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("lrs+1003_8:1_av=off:fsr=off:fde=unused:gsp=input_only:gs=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on_300");
    fallback.push("lrs+11_8:1_afp=100000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:nm=64:nwc=2:nicw=on:sac=on:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs-11_8:1_afr=on:afp=1000:afq=1.4:amm=off:anc=none:bd=off:bs=on:gs=on:ile=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only_300");
    fallback.push("lrs+10_3:1_add=large:afp=10000:afq=1.1:amm=off:anc=none:cond=on:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sd=5:ss=axioms:st=1.5:tha=off:urr=on_300");
    fallback.push("lrs+1010_3:1_av=off:bd=off:bsr=on:irw=on:nm=64:newcnf=on:nwc=1.7:sos=all:updr=off_300");
    fallback.push("dis+11_5:1_afr=on:afp=40000:afq=2.0:amm=sco:anc=all_dependent:cond=fast:fde=unused:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=2:nwc=1:sos=all:urr=on:uhcvi=on_300");
    fallback.push("lrs+1004_1_aac=none:add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=all_dependent:bd=off:cond=fast:fsr=off:gs=on:gsaa=from_current:lcm=reverse:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thf=on:urr=on:updr=off_300");
    fallback.push("lrs-10_3:2_aac=none:add=off:afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=16:nwc=1:nicw=on:sas=z3:sd=2:ss=axioms:sos=on:sp=occurrence:updr=off_600");
    fallback.push("ott+1002_2:1_add=large:afr=on:afp=100000:afq=1.1:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off_300");
    fallback.push("dis-4_7_acc=on:afp=40000:afq=1.4:anc=all_dependent:bsr=on:br=off:bce=on:ccuc=first:er=filter:fsr=off:fde=unused:gsp=input_only:ile=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:thi=full:uwa=ground:urr=on:updr=off:uhcvi=on_300");
    fallback.push("ott-1_1_acc=model:add=off:afr=on:afp=4000:afq=1.2:anc=all:bd=preordered:bs=unit_only:bsr=on:ccuc=first:gs=on:gsaa=from_current:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=occurrence:tha=off:thi=strong:updr=off_300");
    fallback.push("lrs+1011_5:4_av=off:bd=off:bs=on:cond=on:er=known:gs=on:gsem=on:inw=on:ile=on:lcm=reverse:nm=6:newcnf=on:nwc=1:sp=occurrence:tha=off:uhcvi=on_300");
    fallback.push("dis+1002_14_av=off:cond=fast:fde=unused:inw=on:ile=on:lma=on:nm=0:nwc=1:sos=all:sp=reverse_arity:tha=off:uwa=one_side_interpreted:uhcvi=on_300");
    fallback.push("dis+1011_8:1_add=off:afp=10000:afq=1.1:anc=none:bce=on:er=filter:gs=on:gsaa=from_current:gsem=off:inw=on:ile=on:lma=on:nm=2:nwc=3:sac=on:urr=on:updr=off_300");
    fallback.push("dis+4_4:1_add=off:afp=4000:afq=1.2:amm=sco:anc=none:br=off:cond=fast:ep=RS:fsr=off:inw=on:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on:urr=on:uhcvi=on_300");
    fallback.push("lrs+1002_1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+1002_2:1_aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=none:cond=fast:gs=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_3:1_afp=1000:afq=1.4:amm=off:anc=none:bsr=on:inw=on:ile=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:tha=off:urr=on:updr=off_300");
    fallback.push("ott+10_2:1_av=off:bd=off:br=off:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+10_24_av=off:bs=unit_only:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sd=7:ss=axioms:st=1.2:sp=occurrence:tha=off:thf=on:uhcvi=on_600");
    fallback.push("lrs+4_28_afp=10000:afq=1.2:amm=sco:anc=none:bd=off:bce=on:cond=on:fsr=off:ile=on:irw=on:lcm=reverse:nm=16:newcnf=on:nwc=2:sas=z3:sp=occurrence:tha=off:updr=off:uhcvi=on_600");
    return;
  }

  switch (cat) {
  case Property::EPR:
    if (prop != 0UL) {
      quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_50");
      quick.push("lrs+1_16_add=off:afp=100000:afq=1.0:amm=off:cond=fast:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=2:nicw=on:stl=60:sd=7:ss=axioms:st=5.0:sos=theory:sp=reverse_arity:urr=ec_only_344");
      quick.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:stl=30:uhcvi=on_301");
      quick.push("dis+4_5_afp=1000:afq=1.1:amm=off:anc=none:bd=off:gs=on:irw=on:lcm=predicate:lma=on:nwc=1:sas=z3:sos=all:sp=occurrence_151");
      quick.push("dis+11_2:3_add=large:afp=10000:afq=1.2:anc=none:bd=off:bce=on:cond=fast:er=filter:fsr=off:fde=unused:gsp=input_only:nwc=5:sos=theory:sac=on:urr=on_245");
      quick.push("ins+10_1_av=off:bce=on:cond=on:fde=unused:gsp=input_only:igbrr=0.7:igrr=32/1:igrp=700:igrpq=2.0:igs=1010:igwr=on:lcm=reverse:nwc=1:ss=axioms:st=3.0:sos=theory:urr=on:uhcvi=on_139");
    }
    else if (atoms < 2000) {
      quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_47");
      quick.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_283");
      quick.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:stl=30:uhcvi=on_25");
      quick.push("lrs+1011_14_afr=on:afp=100000:afq=1.1:anc=none:bd=off:bsr=on:irw=on:nwc=1:sas=z3:stl=30_90");
      quick.push("lrs+1003_10_afp=4000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:bce=on:fde=unused:lma=on:nwc=1:nicw=on:stl=120:sac=on:urr=on:updr=off:uhcvi=on_417");
      quick.push("ins+11_128_av=off:cond=on:fsr=off:irw=on:igbrr=0.5:igpr=on:igrr=1/2:igrp=2000:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:sos=all:sp=occurrence:updr=off_194");
      quick.push("dis+1011_5_add=large:anc=none:bd=preordered:cond=on:fsr=off:fde=unused:lma=on:nwc=1:sos=theory:sp=occurrence:updr=off_1327");
    }
    else {
      quick.push("dis-11_32_av=off:bs=unit_only:gs=on:irw=on:lma=on:nwc=1:updr=off_34");
      quick.push("dis+1011_128_afp=100000:afq=1.1:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:irw=on:lma=on:nwc=1:sos=theory:sac=on:urr=on:updr=off_19");
      quick.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_16");
      quick.push("lrs+4_6_afp=10000:afq=1.2:amm=sco:anc=all_dependent:bs=on:bsr=on:fde=unused:irw=on:lma=on:nwc=1:stl=30:sos=theory:sp=occurrence:uhcvi=on_264");
      quick.push("lrs+1_8:1_add=off:anc=none:bd=preordered:br=off:bce=on:fsr=off:fde=none:nwc=1:nicw=on:stl=90:sos=theory:sp=reverse_arity:urr=on_815");
      quick.push("lrs+1_5:1_add=off:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:gs=on:nwc=1.1:nicw=on:sas=z3:stl=30:sos=theory:urr=on:updr=off_178");
      quick.push("dis+4_7_aac=none:afr=on:anc=none:bd=preordered:bce=on:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:irw=on:lma=on:nwc=1:nicw=on:sas=z3:sac=on:sp=reverse_arity:uhcvi=on_2");
    }
    fallback.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_3000");
    fallback.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_300");
    fallback.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:uhcvi=on_300");
    fallback.push("dis+4_5_afp=1000:afq=1.1:amm=off:anc=none:bd=off:gs=on:irw=on:lcm=predicate:lma=on:nwc=1:sas=z3:sos=all:sp=occurrence_300");
    fallback.push("lrs+1003_10_afp=4000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:bce=on:fde=unused:lma=on:nwc=1:nicw=on:sac=on:urr=on:updr=off:uhcvi=on_1200");
    fallback.push("dis+11_2:3_add=large:afp=10000:afq=1.2:anc=none:bd=off:bce=on:cond=fast:er=filter:fsr=off:fde=unused:gsp=input_only:nwc=5:sos=theory:sac=on:urr=on_300");
    fallback.push("ott-4_6_add=off:afr=on:afp=100000:afq=1.4:amm=sco:bs=on:fde=unused:gs=on:gsaa=full_model:gsem=on:irw=on:nwc=1:sas=z3:sac=on:updr=off:uhcvi=on_600");
    fallback.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_200");
    fallback.push("dis-11_32_av=off:bs=unit_only:gs=on:irw=on:lma=on:nwc=1:updr=off_300");
    fallback.push("ins+10_1_av=off:bce=on:cond=on:fde=unused:gsp=input_only:igbrr=0.7:igrr=32/1:igrp=700:igrpq=2.0:igs=1010:igwr=on:lcm=reverse:nwc=1:ss=axioms:st=3.0:sos=theory:urr=on:uhcvi=on_300");
    fallback.push("ott+1_3_awrs=converge:awrsf=16:av=off:bd=off:bs=on:bce=on:cond=fast:gs=on:gsem=off:irw=on:lma=on:nwc=1.5:sas=z3:sp=weighted_frequency:updr=off:uhcvi=on_300");
    fallback.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_5:1_add=off:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:gs=on:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
    fallback.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("dis-4_4_add=large:afr=on:afp=1000:afq=1.4:anc=all_dependent:bs=unit_only:fde=none:gs=on:gsaa=from_current:lma=on:nwc=1.2:sac=on:updr=off_600");
    fallback.push("lrs+1_16_add=off:afp=100000:afq=1.0:amm=off:cond=fast:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=2:nicw=on:sd=7:ss=axioms:st=5.0:sos=theory:sp=reverse_arity:urr=ec_only_600");
    fallback.push("lrs+1_8:1_add=off:anc=none:bd=preordered:br=off:bce=on:fsr=off:fde=none:nwc=1:nicw=on:sos=theory:sp=reverse_arity:urr=on_900");
    fallback.push("dis+4_2_add=large:afr=on:afp=4000:afq=1.2:anc=none:bd=preordered:bce=on:fsr=off:fde=unused:lma=on:nwc=1:sos=all:sac=on:sp=reverse_arity:updr=off_2100");
    return;

  case Property::FEQ:
    if (atoms > 1000000) {
      quick.push("dis+11_2_av=off:cond=fast:ep=RST:fsr=off:lma=on:nm=16:nwc=1.2:sp=occurrence:updr=off_1200");
      quick.push("dis+1011_3_afp=4000:afq=1.1:amm=sco:anc=none:gs=on:gsaa=from_current:gsem=off:irw=on:nm=16:nwc=1:sas=z3:ss=axioms:sos=all:sac=on:sp=reverse_arity:updr=off_1");
    }
    else if (atoms > 500000) {
      quick.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_188");
      quick.push("ott+11_4:1_awrs=converge:awrsf=16:acc=model:add=large:afr=on:afp=4000:afq=1.4:amm=off:br=off:cond=fast:fde=none:gsp=input_only:nm=64:nwc=2:nicw=on:sd=3:ss=axioms:s2a=on:sac=on:sp=frequency:urr=on:updr=off_105");
      quick.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:sos=all_119");
      quick.push("dis+11_8_add=off:afp=10000:afq=1.2:amm=off:anc=none:cond=fast:ep=R:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=4:sd=1:ss=axioms:sac=on:updr=off:uhcvi=on_61");
      quick.push("ott+11_3_afp=1000:afq=2.0:anc=none:fsr=off:irw=on:nwc=1.7:ss=axioms:st=1.5:sac=on:updr=off_137");
      quick.push("lrs-10_3:2_add=large:afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:nm=16:nwc=1.3:stl=30:sd=2:ss=axioms:st=5.0:sos=on:uhcvi=on_78");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_137");
      quick.push("dis+4_8:1_add=large:afp=100000:afq=1.4:ep=RST:fde=unused:gsp=input_only:lcm=predicate:nwc=1:sos=all:sp=occurrence:updr=off:uhcvi=on_169");
      quick.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:sos=all:sp=occurrence_66");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_136");
      quick.push("lrs+1_4:1_awrs=converge:awrsf=128:av=off:cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:lcm=predicate:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:s2a=on:st=2.0:sos=on_65");
      quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_58");
      quick.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_199");
    }
    else if (atoms > 240000) {
      quick.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_38");
      quick.push("lrs+1011_4:1_acc=model:add=large:afp=40000:afq=1.4:ccuc=first:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:irw=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:urr=on_42");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_30");
      quick.push("ott+11_2:1_add=off:afp=10000:afq=1.1:anc=none:cond=on:gsp=input_only:lcm=reverse:lwlo=on:nm=32:nwc=5:sp=occurrence:updr=off_291");
      quick.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_63");
      quick.push("lrs+1002_3_aac=none:acc=on:add=off:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1.1:nicw=on:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:updr=off_26");
      quick.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_49");
      quick.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:stl=30:sd=2:ss=axioms:sos=all:urr=ec_only_42");
      quick.push("lrs-10_3:2_add=large:afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:nm=16:nwc=1.3:stl=30:sd=2:ss=axioms:st=5.0:sos=on:uhcvi=on_54");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_294");
      quick.push("dis+10_4_add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=predicate:lma=on:nm=64:nwc=1:sd=3:ss=axioms:sos=on:sp=reverse_arity_42");
      quick.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_39");
      quick.push("lrs+10_4_add=off:afp=100000:afq=2.0:anc=none:bd=off:er=known:gs=on:gsem=off:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=2.5:sas=z3:stl=30:sac=on:urr=ec_only:updr=off_44");
      quick.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_175");
      quick.push("lrs+1011_7_afp=100000:afq=1.0:amm=sco:anc=all_dependent:bs=unit_only:bsr=on:ep=RS:fde=none:gsp=input_only:gs=on:gsem=off:lwlo=on:nm=4:nwc=1:stl=120:sos=theory:sp=occurrence:uhcvi=on_1");
      quick.push("lrs+1_4:1_awrs=converge:awrsf=128:av=off:cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:lcm=predicate:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:s2a=on:st=2.0:sos=on_63");
      quick.push("dis+1011_3_afp=100000:afq=1.1:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:irw=on:nm=6:nwc=1:sas=z3:sd=2:ss=axioms:sos=on:sac=on:sp=reverse_arity:updr=off_45");
      quick.push("dis+10_16_awrs=decay:awrsf=256:afr=on:afp=40000:afq=2.0:amm=off:bsr=on:cond=on:er=known:gsp=input_only:irw=on:lma=on:nm=6:newcnf=on:nwc=3:sas=z3:sd=4:ss=axioms:s2a=on:sp=frequency:updr=off_213");
      quick.push("dis+10_5_av=off:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_92");
    }
    else if (atoms > 200000) {
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_33");
      quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_27");
      quick.push("lrs+1_4:1_awrs=converge:awrsf=128:av=off:cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:lcm=predicate:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:s2a=on:st=2.0:sos=on_107");
      quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_37");
      quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_50");
      quick.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_30");
      quick.push("dis+11_8:1_afp=100000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=off:lcm=reverse:lma=on:nm=64:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on_20");
      quick.push("ins+11_8_av=off:cond=fast:fde=none:gsp=input_only:irw=on:igbrr=0.2:igpr=on:igrr=1/8:igrp=200:igrpq=1.1:igs=1010:igwr=on:lcm=predicate:lma=on:nm=4:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:uhcvi=on_40");
      quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_40");
      quick.push("lrs+1_7_av=off:cond=fast:fde=none:gs=on:gsem=off:lcm=predicate:nm=6:nwc=1:stl=30:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_23");
      quick.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_147");
      quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_251");
      quick.push("dis+11_2:1_add=large:afp=4000:afq=1.4:amm=sco:anc=none:fsr=off:nm=16:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_127");
      quick.push("ott-3_4:1_awrs=converge:awrsf=2:acc=model:add=large:afr=on:afp=40000:afq=1.2:anc=none:ccuc=small_ones:fde=unused:gsp=input_only:irw=on:nm=0:nwc=4:sd=4:ss=axioms:s2a=on:st=1.2:sos=on:urr=on:updr=off:uhcvi=on_155");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_165");
      quick.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_247");
    }
    else if (atoms > 130000) {
      quick.push("lrs+11_3_av=off:cond=on:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=5:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only_25");
      quick.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:sos=all:sp=occurrence_16");
      quick.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:stl=30:sp=occurrence_58");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_30");
      quick.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_25");
      quick.push("lrs+1011_10_av=off:bce=on:cond=on:fsr=off:fde=unused:gs=on:nm=2:nwc=1.1:stl=30:sd=4:ss=axioms:s2a=on:st=1.5:sos=on:sp=frequency:urr=on:updr=off:uhcvi=on_32");
      quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_24");
      quick.push("lrs+1_4:1_awrs=converge:awrsf=128:av=off:cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:lcm=predicate:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:s2a=on:st=2.0:sos=on_22");
      quick.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_14");
      quick.push("dis+1002_5_av=off:cond=on:gs=on:lma=on:nm=2:nwc=1:sd=3:ss=axioms:st=1.5:sos=on:updr=off_28");
      quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_14");
      quick.push("ott+11_4:1_awrs=converge:awrsf=16:acc=model:add=large:afr=on:afp=4000:afq=1.4:amm=off:br=off:cond=fast:fde=none:gsp=input_only:nm=64:nwc=2:nicw=on:sd=3:ss=axioms:s2a=on:sac=on:sp=frequency:urr=on:updr=off_19");
      quick.push("dis+10_16_awrs=decay:awrsf=256:afr=on:afp=40000:afq=2.0:amm=off:bsr=on:cond=on:er=known:gsp=input_only:irw=on:lma=on:nm=6:newcnf=on:nwc=3:sas=z3:sd=4:ss=axioms:s2a=on:sp=frequency:updr=off_39");
      quick.push("lrs+1010_3:2_awrs=decay:awrsf=2:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:bs=on:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=on:irw=on:nm=6:newcnf=on:nwc=1.5:nicw=on:sas=z3:stl=30:sd=4:ss=axioms:s2a=on:sos=on:sac=on:sp=weighted_frequency:urr=on_24");
      quick.push("lrs+11_50_afp=100000:afq=1.1:amm=sco:anc=none:bs=unit_only:cond=on:irw=on:lma=on:nm=32:nwc=1.1:stl=30:sp=reverse_arity_252");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_290");
      quick.push("ott+2_40_av=off:bs=unit_only:bsr=on:cond=fast:ep=RST:lma=on:nm=16:nwc=1:sp=reverse_arity_177");
      quick.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_33");
      quick.push("lrs-10_3:2_add=large:afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:nm=16:nwc=1.3:stl=30:sd=2:ss=axioms:st=5.0:sos=on:uhcvi=on_301");
      quick.push("dis+10_4_add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=predicate:lma=on:nm=64:nwc=1:sd=3:ss=axioms:sos=on:sp=reverse_arity_17");
      quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_32");
      quick.push("ott+11_8_amm=off:anc=none:bsr=on:cond=on:irw=on:nm=2:nwc=1.1:ss=axioms:st=2.0:sac=on_39");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_48");
      quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_272");
      quick.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:sos=all_172");
    }
    else if (atoms > 50000) {
      quick.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_8");
      quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_21");
      quick.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_7");
      quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_60");
      quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_11");
      quick.push("dis+1011_3_afp=100000:afq=1.1:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:irw=on:nm=6:nwc=1:sas=z3:sd=2:ss=axioms:sos=on:sac=on:sp=reverse_arity:updr=off_7");
      quick.push("ott+1002_128_av=off:bd=off:bs=on:bsr=on:cond=on:fsr=off:nm=6:newcnf=on:nwc=1:sp=reverse_arity:updr=off_82");
      quick.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_15");
      quick.push("dis+10_16_awrs=decay:awrsf=256:afr=on:afp=40000:afq=2.0:amm=off:bsr=on:cond=on:er=known:gsp=input_only:irw=on:lma=on:nm=6:newcnf=on:nwc=3:sas=z3:sd=4:ss=axioms:s2a=on:sp=frequency:updr=off_10");
      quick.push("lrs+1_4:1_awrs=converge:awrsf=128:av=off:cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:lcm=predicate:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:s2a=on:st=2.0:sos=on_88");
      quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_16");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_13");
      quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_10");
      quick.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:stl=30:sd=2:ss=axioms:sos=all:urr=ec_only_23");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_16");
      quick.push("lrs-10_3:2_add=large:afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:lma=on:nm=16:nwc=1.3:stl=30:sd=2:ss=axioms:st=5.0:sos=on:uhcvi=on_10");
      quick.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_122");
      quick.push("lrs+1011_4:1_acc=model:add=large:afp=40000:afq=1.4:ccuc=first:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:irw=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:urr=on_29");
      quick.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_20");
      quick.push("lrs+1011_10_av=off:bce=on:cond=on:fsr=off:fde=unused:gs=on:nm=2:nwc=1.1:stl=30:sd=4:ss=axioms:s2a=on:st=1.5:sos=on:sp=frequency:urr=on:updr=off:uhcvi=on_12");
      quick.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_11");
      quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_76");
      quick.push("dis-1_4_aac=none:acc=on:add=off:afr=on:afp=40000:afq=1.2:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sac=on:sp=occurrence:updr=off_26");
      quick.push("ott+1010_5:4_av=off:bd=off:fde=none:irw=on:lma=on:nm=32:nwc=2.5:sd=2:ss=axioms:st=3.0:urr=on_190");
      quick.push("dis-10_3:2_aac=none:afp=1000:afq=1.1:cond=on:fsr=off:lcm=reverse:lwlo=on:nm=16:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity:updr=off_27");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_225");
      quick.push("lrs+4_4:1_acc=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:ccuc=first:cond=on:fsr=off:irw=on:lcm=predicate:lma=on:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all_161");
      quick.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_17");
      quick.push("lrs+1010_4_awrs=decay:awrsf=64:add=large:afp=4000:afq=1.2:amm=off:anc=none:bs=on:bsr=on:fde=none:gs=on:gsaa=from_current:gsem=on:lma=on:newcnf=on:nwc=1.7:sas=z3:stl=30:sd=10:ss=axioms:s2a=on:st=1.2:sp=frequency:updr=off:uhcvi=on_239");
      quick.push("lrs+1002_1_av=off:er=filter:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:st=1.5:sos=on_13");
      quick.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=2.0:sac=on:urr=on_115");
      quick.push("ott+11_4:1_awrs=converge:awrsf=16:acc=model:add=large:afr=on:afp=4000:afq=1.4:amm=off:br=off:cond=fast:fde=none:gsp=input_only:nm=64:nwc=2:nicw=on:sd=3:ss=axioms:s2a=on:sac=on:sp=frequency:urr=on:updr=off_88");
      quick.push("lrs+1011_50_afr=on:afp=40000:afq=1.0:amm=off:anc=all_dependent:bs=on:bsr=on:bce=on:fde=unused:gs=on:lma=on:nm=16:nwc=1.1:stl=60:sp=occurrence:updr=off_474");
      quick.push("lrs+1010_3:2_awrs=decay:awrsf=2:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:bs=on:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=on:irw=on:nm=6:newcnf=on:nwc=1.5:nicw=on:sas=z3:stl=30:sd=4:ss=axioms:s2a=on:sos=on:sac=on:sp=weighted_frequency:urr=on_22");
      quick.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_21");
      quick.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:stl=30:sd=2:ss=axioms:st=2.0:sos=all_229");
    }
    else if (atoms > 20000) {
      quick.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_4");
      quick.push("lrs+1003_3_awrs=decay:awrsf=4:add=large:afr=on:afp=100000:afq=2.0:amm=sco:bd=off:cond=fast:fsr=off:fde=unused:gs=on:gsem=on:nm=6:nwc=1:stl=30:sd=1:ss=axioms:st=1.2:sos=on:sac=on:sp=frequency:urr=on:updr=off_4");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_7");
      quick.push("ott+1002_128_av=off:bd=off:bs=on:bsr=on:cond=on:fsr=off:nm=6:newcnf=on:nwc=1:sp=reverse_arity:updr=off_62");
      quick.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=2.0:sos=all:updr=off_7");
      quick.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_6");
      quick.push("dis+10_5_av=off:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_6");
      quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_11");
      quick.push("ott+11_8_amm=off:anc=none:bsr=on:cond=on:irw=on:nm=2:nwc=1.1:ss=axioms:st=2.0:sac=on_8");
      quick.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_13");
      quick.push("lrs+11_4:1_av=off:bd=off:br=off:cond=fast:fde=unused:irw=on:lcm=reverse:lma=on:lwlo=on:nm=4:nwc=1:stl=60:sd=3:ss=axioms:sos=all:urr=on_69");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_12");
      quick.push("lrs+1011_1_av=off:cond=on:gs=on:lma=on:nm=4:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity:updr=off_118");
      quick.push("lrs+4_1_acc=on:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bs=on:bsr=on:ccuc=first:fsr=off:fde=unused:irw=on:lma=on:nm=0:nwc=1.3:stl=30:sd=10:ss=axioms:st=3.0:sos=all:sp=occurrence:uhcvi=on_26");
      quick.push("ott+2_8:1_aac=none:acc=on:add=large:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:fsr=off:lcm=predicate:lma=on:nm=2:nwc=1.7:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_12");
      quick.push("lrs+1011_50_afr=on:afp=40000:afq=1.0:amm=off:anc=all_dependent:bs=on:bsr=on:bce=on:fde=unused:gs=on:lma=on:nm=16:nwc=1.1:stl=60:sp=occurrence:updr=off_83");
    }
    else if (prop == 2ul) {
      quick.push("lrs+11_24_add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=fast:fde=unused:gs=on:irw=on:lma=on:nm=4:nwc=1.3:nicw=on:sas=z3:stl=30:sac=on:sp=reverse_arity:uhcvi=on_136");
      quick.push("lrs+11_5_av=off:cond=on:fsr=off:lma=on:lwlo=on:nwc=1.2:sas=z3:stl=30:sp=reverse_arity:updr=off_123");
      quick.push("dis+11_50_add=large:afp=1000:afq=1.0:amm=sco:anc=none:fsr=off:nm=16:nwc=1.5:sac=on_13");
      quick.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:stl=30:sp=occurrence_75");
      quick.push("dis+1002_2_add=off:afr=on:afp=10000:afq=2.0:amm=off:anc=none:cond=on:er=filter:fsr=off:nm=0:nwc=1.3:sp=occurrence_3");
      quick.push("ott+1010_8:1_add=off:afp=4000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=4:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off:uhcvi=on_10");
      quick.push("dis+11_5:4_acc=on:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ccuc=small_ones:fsr=off:irw=on:nm=32:nwc=2.5:nicw=on:urr=on:updr=off_10");
      quick.push("ott+11_20_afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bsr=on:irw=on:lma=on:nm=4:nwc=1.2:sac=on:sp=occurrence_294");
      quick.push("lrs+1_128_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bs=unit_only:gs=on:gsem=off:lwlo=on:nm=2:nwc=1:sas=z3:stl=90:sac=on:sp=occurrence:urr=on:updr=off:uhcvi=on_520");
      quick.push("lrs+10_24_add=off:afr=on:afp=1000:afq=1.4:anc=none:bs=unit_only:gs=on:gsaa=from_current:gsem=on:lma=on:nm=2:nwc=1.1:stl=60:sac=on:uhcvi=on_461");
      quick.push("ott+11_20_afp=10000:afq=1.1:anc=none:bs=unit_only:bsr=on:bce=on:fsr=off:gs=on:gsem=on:nwc=2.5:sas=z3:sp=occurrence:updr=off:uhcvi=on_385");
      quick.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_122");
      quick.push("dis+1_24_av=off:cond=on:irw=on:lma=on:nm=2:nwc=1.5:sp=occurrence:updr=off_16");
      quick.push("ott+1_28_av=off:bs=unit_only:cond=on:irw=on:nm=64:nwc=2:sp=occurrence:updr=off:uhcvi=on_1052");
      quick.push("lrs+11_128_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:nwc=1.3:stl=30:sac=on:uhcvi=on_236");
      quick.push("ott+4_4:1_acc=model:add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=preordered:bs=unit_only:ccuc=first:cond=on:gs=on:nm=32:nwc=1.5:sac=on:urr=on:updr=off_355");
      quick.push("ott-1_50_afr=on:afp=1000:afq=1.2:amm=sco:anc=none:bsr=on:cond=fast:fsr=off:fde=none:irw=on:nm=16:nwc=1.5:updr=off:uhcvi=on_1883");
      quick.push("lrs+11_64_acc=model:afp=100000:afq=1.2:anc=all_dependent:bd=off:bs=unit_only:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:irw=on:lma=on:nwc=1:stl=150:sac=on:sp=reverse_arity:urr=on:updr=off_893");
    }
    else if (prop == 0ul) {
      if (atoms > 21) {
        quick.push("lrs+1011_5_afr=on:afp=100000:afq=1.0:amm=off:anc=none:cond=on:lma=on:nm=6:nwc=1:sas=z3:stl=30:sac=on:urr=on_18");
        quick.push("lrs+4_14_afr=on:afp=4000:afq=1.1:anc=none:cond=on:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1.1:sas=z3:stl=30:sd=4:ss=axioms:st=1.2:sos=all:updr=off_51");
        quick.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:stl=30:sp=occurrence:urr=on_118");
        quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_22");
        quick.push("lrs+1010_32_afr=on:anc=none:bd=off:fsr=off:gs=on:gsem=on:nwc=1.3:nicw=on:sas=z3:stl=30:updr=off_63");
        quick.push("lrs+10_4:1_add=large:afp=100000:afq=1.1:anc=none:ep=RST:fde=unused:gsp=input_only:nm=6:newcnf=on:nwc=1:stl=30:sos=all:sac=on:sp=reverse_arity:urr=ec_only_20");
        quick.push("lrs+1011_4:1_acc=model:add=large:afp=40000:afq=1.4:ccuc=first:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:irw=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:urr=on_42");
        quick.push("dis+11_5:4_acc=on:add=large:afp=40000:afq=1.2:amm=sco:anc=all_dependent:bd=off:ccuc=small_ones:irw=on:lcm=reverse:lma=on:nwc=1:sd=3:ss=axioms:sos=all:sp=occurrence:uhcvi=on_2");
        quick.push("lrs+1003_3_awrs=decay:awrsf=4:add=large:afr=on:afp=100000:afq=2.0:amm=sco:bd=off:cond=fast:fsr=off:fde=unused:gs=on:gsem=on:nm=6:nwc=1:stl=30:sd=1:ss=axioms:st=1.2:sos=on:sac=on:sp=frequency:urr=on:updr=off_71");
        quick.push("lrs+11_5_av=off:cond=on:fsr=off:lma=on:lwlo=on:nwc=1.2:sas=z3:stl=30:sp=reverse_arity:updr=off_142");
        quick.push("lrs-1_2:3_add=large:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:nm=16:nwc=1:stl=30:sd=1:ss=axioms:sos=on:sac=on_12");
        quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_136");
        quick.push("dis-10_1_aac=none:afr=on:afp=10000:afq=1.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=4:newcnf=on:nwc=2:sp=occurrence_2");
        quick.push("lrs+10_128_av=off:bs=on:cond=on:gs=on:gsem=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1:stl=30:updr=off_207");
        quick.push("lrs+11_5_afr=on:afp=100000:afq=2.0:amm=off:anc=none:bsr=on:cond=on:fsr=off:gs=on:gsem=on:nm=16:nwc=1:stl=30:sp=occurrence:urr=on:updr=off_18");
        quick.push("ott+1011_4:1_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=preordered:cond=fast:nm=32:newcnf=on:nwc=1.2:sas=z3:sac=on:sp=occurrence:urr=on:updr=off_122");
        quick.push("ott+10_1_av=off:bd=off:bsr=on:cond=on:fsr=off:irw=on:nm=2:nwc=1:sd=3:ss=axioms:sos=on:sp=occurrence:urr=on_19");
        quick.push("dis+11_1_av=off:gsp=input_only:lma=on:nm=4:nwc=1:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:urr=on_246");
        quick.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=2.0:sac=on:urr=on_174");
        quick.push("lrs+1011_1024_av=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=32:nwc=1:stl=90:sp=reverse_arity:uhcvi=on_426");
        quick.push("lrs+1_4:1_awrs=converge:awrsf=128:av=off:cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:lcm=predicate:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:s2a=on:st=2.0:sos=on_172");
        quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_70");
        quick.push("dis+1011_5:1_add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:bs=unit_only:bce=on:ep=RS:fsr=off:fde=none:irw=on:lwlo=on:nwc=1:sas=z3:sos=theory:sac=on:updr=off:uhcvi=on_398");
        quick.push("lrs+1011_7_afp=100000:afq=1.0:amm=sco:anc=all_dependent:bs=unit_only:bsr=on:ep=RS:fde=none:gsp=input_only:gs=on:gsem=off:lwlo=on:nm=4:nwc=1:stl=120:sos=theory:sp=occurrence:uhcvi=on_412");
      }
      else {
        quick.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:stl=30:sac=on:sp=occurrence_6");
        quick.push("lrs+1011_5:1_acc=on:amm=off:anc=all_dependent:bd=off:ccuc=small_ones:fde=unused:gs=on:gsaa=full_model:gsem=off:lcm=predicate:lwlo=on:nm=6:newcnf=on:nwc=2.5:stl=30:sp=occurrence:updr=off_3");
        quick.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:stl=30:sac=on:urr=on_79");
        quick.push("lrs+1004_5:4_aac=none:add=large:afp=100000:afq=1.4:anc=all_dependent:bd=off:cond=fast:gsp=input_only:gs=on:gsem=off:lma=on:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sp=occurrence:updr=off:uhcvi=on_99");
        quick.push("lrs-11_24_av=off:bd=off:bs=on:fsr=off:gs=on:gsem=off:lma=on:nm=4:nwc=1.7:stl=90:sp=occurrence:updr=off_61");
        quick.push("ott+11_50_add=large:afr=on:afp=40000:afq=1.1:amm=sco:anc=all:bd=off:cond=fast:gs=on:gsem=on:irw=on:lma=on:nm=0:nwc=1:sp=occurrence:updr=off:uhcvi=on_102");
        quick.push("lrs+11_3:2_add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bs=unit_only:cond=on:gs=on:gsaa=full_model:gsem=off:irw=on:lwlo=on:nm=32:nwc=1.2:sas=z3:stl=90:sac=on_214");
        quick.push("ott+4_1024_afr=on:afp=4000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:cond=fast:gsp=input_only:gs=on:gsaa=from_current:nm=16:nwc=1.1:nicw=on:sas=z3:sp=occurrence:updr=off_1389");
        quick.push("lrs+1010_4_awrs=decay:awrsf=64:add=large:afp=4000:afq=1.2:amm=off:anc=none:bs=on:bsr=on:fde=none:gs=on:gsaa=from_current:gsem=on:lma=on:newcnf=on:nwc=1.7:sas=z3:stl=30:sd=10:ss=axioms:s2a=on:st=1.2:sp=frequency:updr=off:uhcvi=on_211");
        quick.push("ott+2_20_add=off:afp=10000:afq=2.0:anc=none:bs=unit_only:fde=unused:irw=on:lma=on:nm=4:nwc=1:sas=z3:sac=on:urr=ec_only:uhcvi=on_420");
        quick.push("ott+10_1024_afp=40000:afq=2.0:amm=off:anc=all:bd=preordered:bs=unit_only:fde=unused:irw=on:nm=16:nwc=1:sp=reverse_arity:uhcvi=on_823");
        quick.push("lrs+10_5:1_add=large:afp=1000:afq=1.0:amm=sco:anc=none:bs=unit_only:bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:irw=on:lwlo=on:nm=2:nwc=1:stl=210:uhcvi=on_1759");
      }
    }
    else if (prop == 131072ul) {
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_2");
      quick.push("lrs+1002_8_add=large:afp=40000:afq=1.0:amm=off:anc=none:cond=on:gs=on:irw=on:nm=16:newcnf=on:nwc=1:stl=30:sos=on:sp=reverse_arity:updr=off_2");
      quick.push("lrs+1010_2:1_aac=none:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:gs=on:gsem=off:irw=on:nm=16:nwc=3:stl=30_7");
      quick.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_9");
      quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_11");
      quick.push("dis+1011_3_awrs=decay:awrsf=32:afp=10000:afq=1.1:amm=off:anc=none:cond=fast:ep=RSTC:fde=unused:lma=on:nm=16:nwc=2.5:s2a=on:sac=on:sp=frequency:urr=ec_only_2");
      quick.push("lrs+1011_10_aac=none:acc=model:add=large:afp=40000:afq=2.0:anc=none:bd=off:bsr=on:fsr=off:gs=on:gsem=off:irw=on:lcm=reverse:lwlo=on:nm=64:nwc=3:nicw=on:stl=30_38");
      quick.push("lrs+1011_5_afr=on:afp=100000:afq=1.0:amm=off:anc=none:cond=on:lma=on:nm=6:nwc=1:sas=z3:stl=30:sac=on:urr=on_12");
      quick.push("lrs+10_4_add=off:afp=100000:afq=2.0:anc=none:bd=off:er=known:gs=on:gsem=off:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=2.5:sas=z3:stl=30:sac=on:urr=ec_only:updr=off_3");
      quick.push("lrs+1011_3:2_aac=none:afp=10000:afq=1.2:amm=off:anc=all:cond=on:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_5");
      quick.push("dis+11_2:1_add=large:afp=4000:afq=1.4:amm=sco:anc=none:fsr=off:nm=16:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_2");
      quick.push("lrs+10_50_add=large:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gsp=input_only:gs=on:irw=on:lma=on:nm=64:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on_21");
      quick.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_2");
      quick.push("lrs+1_7_av=off:irw=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=1:stl=30:sos=all:sp=occurrence:updr=off_81");
      quick.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:stl=30:sp=occurrence:urr=on_129");
      quick.push("lrs+4_2_av=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:stl=30:sos=on:urr=on_159");
      quick.push("ott+1002_7_acc=on:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bs=unit_only:gsp=input_only:gs=on:nm=2:nwc=10:sos=theory:sp=reverse_arity:urr=on:updr=off_45");
      quick.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_180");
      quick.push("lrs+1002_16_av=off:cond=on:nwc=3:stl=30_83");
      quick.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_169");
      quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_3");
    }
    else if (prop == 131087ul) {
      quick.push("lrs-11_3_av=off:bs=unit_only:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:lma=on:nm=2:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_11");
      quick.push("lrs+1011_4:1_acc=model:add=large:afp=40000:afq=1.4:ccuc=first:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:irw=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:urr=on_21");
      quick.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:stl=30:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_41");
      quick.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:sos=all:sp=occurrence_8");
      quick.push("dis+4_8:1_add=large:afp=100000:afq=1.4:ep=RST:fde=unused:gsp=input_only:lcm=predicate:nwc=1:sos=all:sp=occurrence:updr=off:uhcvi=on_33");
      quick.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:stl=30:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_8");
      quick.push("dis+1004_1_aac=none:acc=on:afp=40000:afq=1.2:anc=none:cond=on:fde=unused:gs=on:gsem=off:irw=on:nm=32:nwc=2:sd=1:ss=axioms:sos=theory:sp=reverse_arity:urr=ec_only_17");
      quick.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:stl=30:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_6");
      quick.push("lrs+11_2:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=16:nwc=1:stl=30:sd=1:ss=axioms:st=1.2:sp=reverse_arity_11");
      quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_6");
      quick.push("dis+1_3:2_acc=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:ccuc=small_ones:cond=on:lma=on:nm=64:nwc=1.3:sos=all:urr=on_3");
      quick.push("dis+1010_2:3_afr=on:afp=40000:afq=1.4:amm=off:anc=none:lma=on:nm=16:nwc=1:sp=occurrence:updr=off:uhcvi=on_34");
      quick.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_15");
      quick.push("dis+4_2_av=off:bs=on:fsr=off:gsp=input_only:newcnf=on:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sp=reverse_arity:urr=ec_only:updr=off_127");
      quick.push("dis+1002_5_av=off:cond=on:gs=on:lma=on:nm=2:nwc=1:sd=3:ss=axioms:st=1.5:sos=on:updr=off_4");
      quick.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_7");
      quick.push("lrs+1010_4_awrs=decay:awrsf=64:add=large:afp=4000:afq=1.2:amm=off:anc=none:bs=on:bsr=on:fde=none:gs=on:gsaa=from_current:gsem=on:lma=on:newcnf=on:nwc=1.7:sas=z3:stl=30:sd=10:ss=axioms:s2a=on:st=1.2:sp=frequency:updr=off:uhcvi=on_23");
      quick.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_62");
      quick.push("lrs+1_2:3_aac=none:afr=on:afp=100000:afq=1.0:amm=sco:bd=off:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_11");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_6");
      quick.push("lrs+1010_4:1_aac=none:add=off:afp=40000:afq=2.0:amm=sco:anc=none:gs=on:gsem=off:lma=on:lwlo=on:nm=4:nwc=10:stl=30:sd=3:ss=axioms:sos=all:sac=on_49");
      quick.push("ott+11_4:1_awrs=converge:awrsf=16:acc=model:add=large:afr=on:afp=4000:afq=1.4:amm=off:br=off:cond=fast:fde=none:gsp=input_only:nm=64:nwc=2:nicw=on:sd=3:ss=axioms:s2a=on:sac=on:sp=frequency:urr=on:updr=off_70");
      quick.push("dis+1011_3_awrs=decay:awrsf=32:afp=10000:afq=1.1:amm=off:anc=none:cond=fast:ep=RSTC:fde=unused:lma=on:nm=16:nwc=2.5:s2a=on:sac=on:sp=frequency:urr=ec_only_157");
      quick.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_9");
      quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_9");
      quick.push("ott-3_4:1_awrs=converge:awrsf=2:acc=model:add=large:afr=on:afp=40000:afq=1.2:anc=none:ccuc=small_ones:fde=unused:gsp=input_only:irw=on:nm=0:nwc=4:sd=4:ss=axioms:s2a=on:st=1.2:sos=on:urr=on:updr=off:uhcvi=on_88");
      quick.push("dis+11_2_add=large:afr=on:anc=none:gs=on:gsem=on:lwlo=on:nm=16:nwc=1:sd=1:ss=axioms:st=3.0:sos=on_4");
      quick.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_49");
      quick.push("lrs+1011_10_av=off:bce=on:cond=on:fsr=off:fde=unused:gs=on:nm=2:nwc=1.1:stl=30:sd=4:ss=axioms:s2a=on:st=1.5:sos=on:sp=frequency:urr=on:updr=off:uhcvi=on_4");
      quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_48");
      quick.push("dis+1011_8:1_aac=none:acc=on:afp=1000:afq=1.4:amm=off:anc=all:bs=unit_only:bce=on:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:lma=on:nm=16:nwc=2.5:sd=4:ss=axioms:st=1.5:sos=all:uhcvi=on_65");
      quick.push("ott+1_5:1_acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:lcm=predicate:nm=16:nwc=1.1:sd=1:ss=axioms:st=3.0:sos=on:sac=on:updr=off_18");
      quick.push("lrs+1002_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:cond=fast:fde=none:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence_8");
      quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_4");
      quick.push("lrs+1002_3:1_av=off:bd=off:cond=fast:fde=none:gsp=input_only:lcm=predicate:lma=on:lwlo=on:nm=4:nwc=1.3:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity_134");
      quick.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_6");
      quick.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_243");
      quick.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_254");
      quick.push("ins+10_1_av=off:fde=none:gsp=input_only:irw=on:igbrr=0.7:igpr=on:igrr=16/1:igrp=400:igrpq=2.0:igs=1003:igwr=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=3:sp=occurrence:uhcvi=on_62");
      quick.push("ins+11_64_av=off:cond=fast:fde=none:gs=on:gsem=on:igbrr=0.7:igrr=1/2:igrp=4000:igrpq=1.2:igwr=on:lcm=predicate:lma=on:nwc=1.1:sd=2:ss=axioms:st=3.0:sos=on:sp=occurrence_38");
      quick.push("lrs+10_2:3_afp=1000:afq=1.1:amm=sco:anc=none:er=known:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=2:stl=30:sd=5:ss=axioms:sos=theory:sac=on:sp=occurrence_78");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_27");
      quick.push("lrs-2_3:1_add=off:afr=on:afp=1000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bs=unit_only:bsr=on:cond=on:fde=unused:gs=on:gsem=on:irw=on:lcm=reverse:nm=32:nwc=1.7:sas=z3:stl=30:sos=all:sac=on_28");
      quick.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:sos=on:sp=reverse_arity_3");
      quick.push("dis+10_1_add=off:afp=4000:afq=1.2:anc=none:br=off:cond=on:gs=on:irw=on:lcm=reverse:nwc=10:sd=10:ss=axioms:sos=theory:sac=on:sp=occurrence:urr=on_4");
      quick.push("ott-11_3_av=off:bsr=on:cond=fast:fde=unused:lcm=predicate:lma=on:nm=6:nwc=1:sos=on:updr=off_546");
      quick.push("dis+11_1_acc=on:afp=1000:afq=1.4:amm=sco:anc=all_dependent:bs=on:ccuc=small_ones:cond=fast:fde=none:gsp=input_only:nm=64:nwc=1:sac=on:urr=ec_only:updr=off:uhcvi=on_138");
      quick.push("lrs+11_8:1_av=off:bd=preordered:br=off:cond=on:gs=on:gsem=on:lcm=reverse:lma=on:nm=0:nwc=1:stl=60:urr=on_362");
      quick.push("ott+1004_12_awrs=converge:awrsf=64:aac=none:afr=on:afp=4000:afq=1.4:amm=sco:anc=none:bs=on:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=on:lma=on:nwc=5:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:urr=ec_only_113");
      quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_41");
      quick.push("dis+1010_128_afr=on:afp=10000:afq=1.1:anc=none:bsr=on:br=off:bce=on:cond=on:fsr=off:gsp=input_only:irw=on:nm=6:newcnf=on:nwc=1.5:sos=all:sac=on:sp=occurrence:urr=on:updr=off_107");
      quick.push("lrs+1002_8:1_add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:er=known:lwlo=on:nm=0:nwc=1.2:stl=30:sd=1:ss=axioms:sp=occurrence:updr=off_51");
    }
    else if (prop == 131075ul) {
      quick.push("dis+10_16_awrs=decay:awrsf=256:afr=on:afp=40000:afq=2.0:amm=off:bsr=on:cond=on:er=known:gsp=input_only:irw=on:lma=on:nm=6:newcnf=on:nwc=3:sas=z3:sd=4:ss=axioms:s2a=on:sp=frequency:updr=off_8");
      quick.push("dis+11_3_av=off:fsr=off:lcm=predicate:lma=on:nm=4:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:updr=off_4");
      quick.push("dis+1_5:1_aac=none:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:br=off:gsp=input_only:gs=on:gsem=on:lcm=reverse:nm=6:nwc=1:nicw=on:sas=z3:sos=on:urr=on_1");
      quick.push("dis+11_2:1_add=large:afp=1000:afq=1.2:amm=sco:anc=none:cond=on:gs=on:gsem=off:nm=16:newcnf=on:nwc=1:sas=z3:sd=1:ss=axioms:st=1.2:sos=on:sp=reverse_arity:updr=off_5");
      quick.push("dis+1002_8:1_av=off:br=off:cond=on:irw=on:lma=on:nm=32:nwc=1:sp=occurrence:urr=on_89");
      quick.push("dis-1_2:3_av=off:cond=on:fsr=off:irw=on:lma=on:nwc=3:sd=3:ss=axioms:st=3.0:sp=occurrence:updr=off_9");
      quick.push("dis+10_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1.3:sp=reverse_arity:updr=off_2");
      quick.push("lrs+1_8:1_av=off:cond=fast:fde=unused:lcm=predicate:nm=16:nwc=10:stl=60:sp=occurrence:urr=ec_only_3");
      quick.push("lrs+11_3_afr=on:afp=10000:afq=1.0:cond=fast:fsr=off:fde=none:gs=on:gsem=off:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:stl=30:sd=10:ss=axioms:st=2.0:sos=all:sp=reverse_arity:urr=on:uhcvi=on_23");
      quick.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:stl=30:sp=occurrence_66");
      quick.push("dis+1002_7_acc=on:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:ccuc=first:fsr=off:gsp=input_only:gs=on:gsem=on:nm=6:nwc=1.1:nicw=on:sos=on:sac=on:sp=occurrence:urr=ec_only_217");
      quick.push("lrs+4_2_av=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:stl=30:sos=on:urr=on_23");
      quick.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sd=2:ss=axioms:sos=all:sp=occurrence_6");
      quick.push("dis+1010_7_afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:urr=ec_only:updr=off_83");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_2");
      quick.push("dis+1011_3_afp=4000:afq=1.1:amm=sco:anc=none:gs=on:gsaa=from_current:gsem=off:irw=on:nm=16:nwc=1:sas=z3:ss=axioms:sos=all:sac=on:sp=reverse_arity:updr=off_17");
      quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_9");
      quick.push("lrs+1_7_av=off:irw=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=1:stl=30:sos=all:sp=occurrence:updr=off_36");
      quick.push("ott-3_3_av=off:cond=fast:fde=none:lcm=reverse:nm=6:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:sp=reverse_arity:updr=off:uhcvi=on_88");
      quick.push("lrs+1011_3:2_av=off:er=known:lma=on:nm=2:newcnf=on:nwc=2:stl=30:sd=2:ss=axioms:st=3.0:urr=on:updr=off_24");
      quick.push("ott+4_4:1_acc=model:add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=preordered:bs=unit_only:ccuc=first:cond=on:gs=on:nm=32:nwc=1.5:sac=on:urr=on:updr=off_13");
      quick.push("dis+1010_3:2_acc=on:afr=on:afp=1000:afq=1.2:amm=sco:bs=on:ccuc=first:fde=none:nm=0:nwc=4:sd=3:ss=axioms:st=5.0:urr=ec_only_75");
      quick.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:stl=30:sos=theory:sp=occurrence:urr=ec_only:updr=off_223");
      quick.push("dis+1010_4_acc=on:afr=on:afp=1000:afq=2.0:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=small_ones:cond=fast:fsr=off:gs=on:gsem=off:lcm=reverse:lma=on:nm=64:nwc=2.5:nicw=on:sd=3:ss=axioms:st=3.0:sac=on:urr=ec_only:updr=off:uhcvi=on_83");
      quick.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_1");
      quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_91");
      quick.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_11");
      quick.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_6");
      quick.push("dis+1002_7_acc=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:newcnf=on:nwc=1.1:nicw=on:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_73");
      quick.push("lrs+11_3_av=off:bce=on:cond=fast:ep=R:lcm=reverse:lma=on:newcnf=on:nwc=1.3:stl=30:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence:uhcvi=on_2");
      quick.push("ott+1010_8:1_add=off:afp=4000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=4:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off:uhcvi=on_239");
      quick.push("dis+1010_1_acc=model:add=large:afr=on:amm=off:anc=none:bd=off:bsr=on:ccuc=small_ones:gs=on:gsem=on:nm=16:nwc=2:urr=ec_only:updr=off_155");
      quick.push("lrs+11_2:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=16:nwc=1:stl=30:sd=1:ss=axioms:st=1.2:sp=reverse_arity_4");
      quick.push("lrs-1_14_add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:br=off:cond=fast:fde=unused:gs=on:lcm=reverse:lma=on:nwc=1:stl=30:sos=on:urr=on:updr=off_25");
      quick.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_171");
      quick.push("dis+1010_3:2_av=off:lma=on:nm=2:newcnf=on:nwc=1:sd=3:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_99");
      quick.push("lrs+10_2_afp=40000:afq=1.0:amm=sco:anc=none:bsr=on:br=off:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=3.0:sos=all:sac=on:sp=occurrence:urr=on:updr=off_3");
      quick.push("dis+1002_1_av=off:bsr=on:cond=on:lma=on:nwc=2:sd=3:ss=axioms:st=3.0:updr=off_24");
      quick.push("lrs+1002_2:1_aac=none:afr=on:afp=1000:afq=1.2:anc=all:bd=preordered:bsr=on:cond=fast:gsp=input_only:gs=on:nm=0:nwc=2.5:nicw=on:sas=z3:stl=30:sd=4:ss=axioms:st=2.0:sos=on:sac=on:urr=on:updr=off:uhcvi=on_22");
      quick.push("ott+1_5_afp=40000:afq=1.0:anc=all:fde=none:gs=on:irw=on:lma=on:nm=32:nwc=2:sos=all:sac=on:sp=occurrence:urr=ec_only:uhcvi=on_4");
      quick.push("lrs+1010_3:2_afr=on:afp=100000:afq=1.1:anc=none:gsp=input_only:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1.7:stl=30:sac=on:sp=occurrence_95");
      quick.push("lrs+1010_2_add=large:afp=4000:afq=2.0:amm=off:bd=off:bs=unit_only:bsr=on:br=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=32:nwc=1.1:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_80");
      quick.push("lrs+1004_8:1_av=off:bd=off:fsr=off:lwlo=on:nm=4:nwc=1.5:stl=30:sd=1:ss=axioms:st=5.0:uhcvi=on_98");
      quick.push("lrs+1_2:3_aac=none:afr=on:afp=100000:afq=1.0:amm=sco:bd=off:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_1");
      quick.push("lrs+1010_5_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:bd=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=4:sas=z3:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_298");
      quick.push("lrs+1010_3:2_awrs=decay:awrsf=2:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:bs=on:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=on:irw=on:nm=6:newcnf=on:nwc=1.5:nicw=on:sas=z3:stl=30:sd=4:ss=axioms:s2a=on:sos=on:sac=on:sp=weighted_frequency:urr=on_1");
      quick.push("ott+10_1024_afp=40000:afq=2.0:amm=off:anc=all:bd=preordered:bs=unit_only:fde=unused:irw=on:nm=16:nwc=1:sp=reverse_arity:uhcvi=on_2");
      quick.push("lrs+2_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:gsp=input_only:lma=on:lwlo=on:newcnf=on:nwc=1:stl=30:sos=theory:sp=reverse_arity:updr=off_1");
      quick.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:stl=30:sd=2:ss=axioms:st=1.5:updr=off_97");
      quick.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_1");
      quick.push("lrs+1_128_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bs=unit_only:gs=on:gsem=off:lwlo=on:nm=2:nwc=1:sas=z3:stl=90:sac=on:sp=occurrence:urr=on:updr=off:uhcvi=on_1");
    }
    else if (prop == 131073ul) {
      if (atoms > 150) {
        quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_3");
        quick.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_2");
        quick.push("ott+1010_5:4_av=off:bd=off:fde=none:irw=on:lma=on:nm=32:nwc=2.5:sd=2:ss=axioms:st=3.0:urr=on_5");
        quick.push("lrs+4_2_av=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:stl=30:sos=on:urr=on_3");
        quick.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_2");
        quick.push("dis+1011_10_aac=none:add=large:afp=10000:afq=1.1:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=6:newcnf=on:nwc=2.5:sp=reverse_arity:updr=off_2");
        quick.push("ott+1002_2_av=off:bd=preordered:irw=on:lma=on:nm=64:nwc=10:sp=reverse_arity:updr=off_22");
        quick.push("lrs+1_2:3_aac=none:afr=on:afp=100000:afq=1.0:amm=sco:bd=off:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_16");
        quick.push("dis+1011_24_add=large:afr=on:afp=4000:afq=1.0:anc=none:bs=unit_only:bce=on:cond=fast:gs=on:nm=32:nwc=2.5:nicw=on:sp=occurrence:updr=off_4");
        quick.push("dis+11_4_afr=on:afp=1000:afq=1.4:amm=off:anc=none:lcm=reverse:lma=on:lwlo=on:nm=6:newcnf=on:nwc=1:sd=2:ss=axioms:st=2.0:sp=reverse_arity_2");
        quick.push("dis-3_5_av=off:cond=on:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1:sos=on:sp=reverse_arity_3");
        quick.push("lrs+1011_3:2_av=off:er=known:lma=on:nm=2:newcnf=on:nwc=2:stl=30:sd=2:ss=axioms:st=3.0:urr=on:updr=off_5");
        quick.push("lrs+1003_2_acc=on:afp=4000:afq=2.0:amm=sco:bd=off:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:nm=64:newcnf=on:nwc=2.5:nicw=on:stl=30:urr=ec_only_8");
        quick.push("lrs+2_1_add=large:afp=100000:afq=1.0:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:gsp=input_only:lma=on:lwlo=on:newcnf=on:nwc=1:stl=30:sos=theory:sp=reverse_arity:updr=off_14");
        quick.push("ott+1_8_av=off:bd=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:lwlo=on:nwc=1:sos=on_10");
        quick.push("dis+1010_3:2_acc=on:afr=on:afp=1000:afq=1.2:amm=sco:bs=on:ccuc=first:fde=none:nm=0:nwc=4:sd=3:ss=axioms:st=5.0:urr=ec_only_4");
        quick.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_1");
        quick.push("lrs+1002_3_aac=none:acc=on:add=off:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1.1:nicw=on:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=occurrence:updr=off_24");
        quick.push("dis+1010_4_afp=10000:afq=1.2:anc=none:irw=on:lma=on:nm=64:nwc=10:sas=z3:sac=on:sp=reverse_arity:updr=off_2");
        quick.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:stl=30:sos=theory:sp=occurrence:urr=ec_only:updr=off_36");
        quick.push("ott+1004_12_awrs=converge:awrsf=64:aac=none:afr=on:afp=4000:afq=1.4:amm=sco:anc=none:bs=on:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=on:lma=on:nwc=5:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:urr=ec_only_2");
        quick.push("dis+11_28_av=off:fsr=off:irw=on:lcm=predicate:nm=2:newcnf=on:nwc=5:sp=occurrence:urr=on:updr=off_2");
        quick.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_72");
        quick.push("dis-10_1_aac=none:afr=on:afp=10000:afq=1.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=4:newcnf=on:nwc=2:sp=occurrence_2");
        quick.push("lrs+1011_4:1_av=off:fsr=off:irw=on:nwc=1:stl=30:sd=4:ss=axioms:st=1.5:sp=reverse_arity_12");
        quick.push("dis+11_5_av=off:bd=off:bs=unit_only:bsr=on:cond=on:lcm=reverse:nm=0:nwc=1.2_5");
        quick.push("ott+1002_128_av=off:bd=off:bs=on:bsr=on:cond=on:fsr=off:nm=6:newcnf=on:nwc=1:sp=reverse_arity:updr=off_9");
        quick.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_5");
        quick.push("lrs+10_6_aac=none:acc=model:add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bd=off:ccuc=small_ones:irw=on:lcm=reverse:nm=0:nwc=1:nicw=on:stl=30:sos=on:sp=reverse_arity:updr=off_2");
        quick.push("ott+11_8_amm=off:anc=none:bsr=on:cond=on:irw=on:nm=2:nwc=1.1:ss=axioms:st=2.0:sac=on_1");
        quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_4");
        quick.push("lrs+10_1_afr=on:afp=100000:afq=1.2:amm=sco:anc=none:br=off:cond=on:gs=on:gsem=on:irw=on:nm=16:nwc=1:stl=30:sac=on:sp=occurrence:urr=on:updr=off_12");
        quick.push("ott-3_3_av=off:cond=fast:fde=none:lcm=reverse:nm=6:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:sp=reverse_arity:updr=off:uhcvi=on_17");
        quick.push("lrs+10_2:3_afp=1000:afq=1.1:amm=sco:anc=none:er=known:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=2:stl=30:sd=5:ss=axioms:sos=theory:sac=on:sp=occurrence_233");
        quick.push("dis+11_32_av=off:ep=RST:fsr=off:lwlo=on:nm=6:nwc=1.1:sd=5:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_2");
        quick.push("ott+1_5_afp=40000:afq=1.0:anc=all:fde=none:gs=on:irw=on:lma=on:nm=32:nwc=2:sos=all:sac=on:sp=occurrence:urr=ec_only:uhcvi=on_125");
        quick.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_205");
        quick.push("lrs+1011_2:1_av=off:irw=on:lwlo=on:nm=16:newcnf=on:nwc=2:stl=30:sd=4:ss=axioms:st=3.0:sp=occurrence_30");
        quick.push("dis+1002_8_add=large:afp=100000:afq=1.2:amm=off:bs=on:irw=on:nm=2:newcnf=on:nwc=1.1:sos=on:sp=reverse_arity:urr=ec_only:updr=off_259");
        quick.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_5");
        quick.push("dis+11_10_av=off:lma=on:nm=64:nwc=5:sp=reverse_arity_3");
        quick.push("ott-11_12_aac=none:afp=100000:afq=1.2:amm=sco:bs=on:bce=on:cond=fast:er=known:gs=on:gsaa=from_current:gsem=off:irw=on:nm=4:nwc=2:sas=z3:sos=all:sp=occurrence:urr=ec_only:updr=off_253");
        quick.push("ott+10_2:3_add=large:afp=40000:afq=1.1:amm=off:anc=all_dependent:bd=preordered:bs=unit_only:cond=fast:er=filter:gs=on:gsaa=from_current:lma=on:nm=32:nwc=1.1:sas=z3:sac=on:sp=occurrence:urr=ec_only:updr=off_679");
        quick.push("lrs+1011_3:2_aac=none:afp=10000:afq=1.2:amm=off:anc=all:cond=on:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_21");
        quick.push("lrs+10_128_acc=model:afp=100000:afq=2.0:anc=all_dependent:bs=on:bsr=on:cond=fast:er=filter:gs=on:gsem=off:lcm=reverse:lma=on:nm=32:nwc=3:stl=30:sac=on:sp=occurrence:urr=ec_only_70");
        quick.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:stl=30:sos=all:updr=off_51");
        quick.push("lrs+1010_5:4_av=off:bd=off:bsr=on:irw=on:lwlo=on:newcnf=on:nwc=1.1:stl=90:sos=all:sp=occurrence:uhcvi=on_145");
        quick.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:stl=30:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_59");
        quick.push("lrs+1011_14_av=off:fsr=off:irw=on:nwc=1:stl=30:sos=on:sp=occurrence:urr=ec_only:updr=off_53");
        quick.push("dis+11_5_afp=40000:afq=1.4:anc=none:br=off:cond=on:fsr=off:irw=on:lma=on:nm=4:nwc=1:sd=3:ss=axioms:st=2.0:urr=on:updr=off_5");
        quick.push("lrs+1_3:2_aac=none:add=large:anc=all_dependent:bce=on:cond=fast:ep=RST:fsr=off:lma=on:nm=2:nwc=1:stl=30:sos=on:sp=occurrence:urr=on:updr=off:uhcvi=on_5");
        quick.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:stl=30:sac=on:sp=occurrence_3");
        quick.push("dis+1010_3:1_av=off:irw=on:nm=32:nwc=1:sos=all:urr=ec_only:updr=off_158");
      }
      else {
        quick.push("dis+10_1_add=off:afp=4000:afq=1.2:anc=none:br=off:cond=on:gs=on:irw=on:lcm=reverse:nwc=10:sd=10:ss=axioms:sos=theory:sac=on:sp=occurrence:urr=on_2");
        quick.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_6");
        quick.push("dis+11_1_acc=on:afp=1000:afq=1.4:amm=sco:anc=all_dependent:bs=on:ccuc=small_ones:cond=fast:fde=none:gsp=input_only:nm=64:nwc=1:sac=on:urr=ec_only:updr=off:uhcvi=on_105");
        quick.push("lrs+11_20_av=off:bs=unit_only:bsr=on:bce=on:cond=on:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=theory:sp=reverse_arity:uhcvi=on_92");
        quick.push("dis+1002_6_add=large:afp=40000:afq=2.0:bsr=on:cond=on:irw=on:lma=on:nm=2:nwc=2.5:nicw=on:sp=reverse_arity:updr=off_7");
        quick.push("lrs+1_4:1_awrs=converge:awrsf=128:av=off:cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:lcm=predicate:lwlo=on:nm=4:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:s2a=on:st=2.0:sos=on_27");
        quick.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=on_3");
        quick.push("lrs+4_3_av=off:br=off:nm=0:newcnf=on:nwc=1:stl=30:sp=occurrence:urr=on_32");
        quick.push("dis+1011_4_av=off:cond=on:irw=on:lma=on:nm=2:nwc=1:sos=all:sp=occurrence_5");
        quick.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:stl=30:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_85");
        quick.push("lrs+1_5:1_afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:fde=none:gs=on:gsaa=full_model:gsem=on:irw=on:lwlo=on:nm=0:nwc=2.5:stl=30:sp=occurrence:uhcvi=on_42");
        quick.push("lrs+1011_2:3_add=large:afr=on:afp=100000:afq=1.2:anc=none:gs=on:gsem=off:irw=on:nm=64:nwc=1:stl=30:sd=3:ss=axioms:sos=all:sp=reverse_arity_48");
        quick.push("ott+4_32_av=off:bsr=on:cond=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=4:nwc=1.2:sos=theory_197");
        quick.push("ott-3_4:1_awrs=converge:awrsf=2:acc=model:add=large:afr=on:afp=40000:afq=1.2:anc=none:ccuc=small_ones:fde=unused:gsp=input_only:irw=on:nm=0:nwc=4:sd=4:ss=axioms:s2a=on:st=1.2:sos=on:urr=on:updr=off:uhcvi=on_2");
        quick.push("dis+1011_3_awrs=decay:awrsf=32:afp=10000:afq=1.1:amm=off:anc=none:cond=fast:ep=RSTC:fde=unused:lma=on:nm=16:nwc=2.5:s2a=on:sac=on:sp=frequency:urr=ec_only_2");
        quick.push("lrs-2_6_acc=on:afp=40000:afq=1.2:amm=sco:anc=none:bs=on:bsr=on:cond=on:fsr=off:fde=none:lcm=reverse:lma=on:nm=32:newcnf=on:nwc=2.5:nicw=on:stl=30:sp=reverse_arity:urr=ec_only_162");
        quick.push("lrs+4_5:4_av=off:bd=off:er=filter:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence:updr=off_230");
        quick.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_145");
        quick.push("dis+1010_3:1_av=off:gsp=input_only:nm=6:nwc=1:sos=all:sp=occurrence_48");
        quick.push("lrs+4_24_av=off:bd=preordered:bsr=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.1:stl=30:sos=theory:updr=off:uhcvi=on_220");
        quick.push("dis+1010_5_add=large:afp=10000:afq=1.2:amm=off:bs=unit_only:bsr=on:bce=on:cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1.3:nicw=on:sos=all:sac=on:updr=off_34");
        quick.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_98");
        quick.push("lrs+3_64_add=large:afp=40000:afq=1.4:anc=none:bd=preordered:bsr=on:fde=unused:gs=on:gsaa=from_current:gsem=on:irw=on:lcm=predicate:lma=on:lwlo=on:nm=16:newcnf=on:nwc=1.2:stl=90:sos=theory_285");
        quick.push("ott+11_8:1_acc=model:afp=1000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=16:nwc=1.5:sac=on:sp=occurrence:urr=on_104");
        quick.push("dis+2_128_add=large:afp=100000:afq=1.4:amm=sco:anc=none:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sos=theory:sac=on:updr=off_288");
        quick.push("lrs+10_24_add=off:afr=on:afp=1000:afq=1.4:anc=none:bs=unit_only:gs=on:gsaa=from_current:gsem=on:lma=on:nm=2:nwc=1.1:stl=60:sac=on:uhcvi=on_364");
      }
    }
    else {
      quick.push("lrs-11_12_av=off:nm=32:nwc=1.3:stl=30:sd=3:ss=axioms:sos=all_2");
      quick.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_2");
      quick.push("dis+2_2:1_aac=none:afp=100000:afq=1.1:amm=sco:anc=none:bsr=on:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=on_6");
      quick.push("dis+1011_24_add=large:afr=on:afp=4000:afq=1.0:anc=none:bs=unit_only:bce=on:cond=fast:gs=on:nm=32:nwc=2.5:nicw=on:sp=occurrence:updr=off_39");
      quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_10");
      quick.push("lrs+11_128_av=off:bsr=on:cond=on:gs=on:lcm=reverse:lma=on:nm=32:nwc=1:stl=30:sd=5:ss=axioms:st=3.0_1");
      quick.push("ott+11_16_av=off:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.3:sas=z3:sp=reverse_arity_14");
      quick.push("lrs+11_20_av=off:bs=unit_only:bsr=on:bce=on:cond=on:fde=none:gs=on:gsem=on:irw=on:nm=4:nwc=1:stl=30:sos=theory:sp=reverse_arity:uhcvi=on_3");
      quick.push("ott+2_2_afp=10000:afq=1.4:amm=off:anc=none:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:nm=32:nwc=1.5:sos=on:sp=reverse_arity_18");
      quick.push("lrs+11_1_add=large:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:gs=on:gsem=on:irw=on:nm=32:newcnf=on:nwc=2.5:nicw=on:stl=30:sd=3:ss=axioms:sos=all:urr=on_34");
      quick.push("lrs+1003_3_awrs=decay:awrsf=4:add=large:afr=on:afp=100000:afq=2.0:amm=sco:bd=off:cond=fast:fsr=off:fde=unused:gs=on:gsem=on:nm=6:nwc=1:stl=30:sd=1:ss=axioms:st=1.2:sos=on:sac=on:sp=frequency:urr=on:updr=off_2");
      quick.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_11");
      quick.push("lrs+1011_7_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:cond=on:er=known:fsr=off:lma=on:nm=4:nwc=2.5:stl=30:sp=reverse_arity:updr=off_2");
      quick.push("lrs+10_4:1_add=large:afp=100000:afq=1.1:anc=none:ep=RST:fde=unused:gsp=input_only:nm=6:newcnf=on:nwc=1:stl=30:sos=all:sac=on:sp=reverse_arity:urr=ec_only_22");
      quick.push("lrs+1010_8_add=off:afp=100000:afq=1.0:amm=off:anc=none:bce=on:irw=on:nm=16:newcnf=on:nwc=1.1:nicw=on:sas=z3:stl=30:sp=reverse_arity:urr=on_13");
      quick.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_9");
      quick.push("dis+1_2:3_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:nm=64:newcnf=on:nwc=1_3");
      quick.push("ott+11_4_afp=100000:afq=1.2:amm=sco:anc=none:cond=fast:ep=R:fde=none:gs=on:gsaa=from_current:gsem=off:lma=on:nm=16:nwc=1:sd=3:ss=axioms:updr=off_2");
      quick.push("ott+1002_8:1_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:fsr=off:gs=on:gsem=off:nm=32:nwc=10:sas=z3:sp=occurrence:urr=on:updr=off_14");
      quick.push("lrs+1011_10_aac=none:acc=model:add=large:afp=40000:afq=2.0:anc=none:bd=off:bsr=on:fsr=off:gs=on:gsem=off:irw=on:lcm=reverse:lwlo=on:nm=64:nwc=3:nicw=on:stl=30_19");
      quick.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_126");
      quick.push("dis+1011_10_av=off:cond=on:lma=on:nm=0:newcnf=on:nwc=1:sos=on:sp=occurrence:updr=off_4");
      quick.push("dis+1002_7_acc=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:newcnf=on:nwc=1.1:nicw=on:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_8");
      quick.push("ott+4_40_av=off:bce=on:cond=fast:fde=none:nm=0:nwc=1:sos=all:updr=off_197");
      quick.push("lrs+1010_3_av=off:fsr=off:gs=on:gsem=off:nm=2:newcnf=on:nwc=2:stl=30:sp=reverse_arity:urr=on:updr=off_9");
      quick.push("dis+1_3:2_acc=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:ccuc=small_ones:cond=on:lma=on:nm=64:nwc=1.3:sos=all:urr=on_111");
      quick.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:stl=30:sos=all:sp=occurrence:urr=on_8");
      quick.push("dis+1002_6_add=large:afp=40000:afq=2.0:bsr=on:cond=on:irw=on:lma=on:nm=2:nwc=2.5:nicw=on:sp=reverse_arity:updr=off_10");
      quick.push("lrs+11_4:1_aac=none:add=large:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:er=filter:fsr=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:newcnf=on:nwc=1.1:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_3");
      quick.push("dis+1010_3:2_av=off:gsp=input_only:nm=2:nwc=1:sp=reverse_arity:urr=ec_only_29");
      quick.push("lrs+11_1_av=off:bsr=on:gsp=input_only:gs=on:lcm=predicate:nm=64:nwc=1:stl=30:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_245");
      quick.push("lrs+1010_4_awrs=decay:awrsf=64:add=large:afp=4000:afq=1.2:amm=off:anc=none:bs=on:bsr=on:fde=none:gs=on:gsaa=from_current:gsem=on:lma=on:newcnf=on:nwc=1.7:sas=z3:stl=30:sd=10:ss=axioms:s2a=on:st=1.2:sp=frequency:updr=off:uhcvi=on_20");
      quick.push("dis+1010_3:2_acc=on:afr=on:afp=1000:afq=1.2:amm=sco:bs=on:ccuc=first:fde=none:nm=0:nwc=4:sd=3:ss=axioms:st=5.0:urr=ec_only_12");
      quick.push("ott+11_20_afp=10000:afq=1.1:anc=none:bs=unit_only:bsr=on:bce=on:fsr=off:gs=on:gsem=on:nwc=2.5:sas=z3:sp=occurrence:updr=off:uhcvi=on_1");
      quick.push("dis+1010_8_acc=model:afp=4000:afq=1.0:anc=none:bd=off:bs=unit_only:bce=on:cond=fast:fde=unused:gs=on:gsem=off:lma=on:nm=0:nwc=4:sd=3:ss=axioms:st=2.0:sac=on:sp=occurrence:urr=ec_only_1");
      quick.push("lrs+11_3:2_add=large:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:er=filter:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:sas=z3:stl=30:sp=occurrence:urr=on:updr=off_100");
      quick.push("dis+10_4_av=off:bsr=on:cond=fast:er=filter:fde=none:gsp=input_only:lcm=reverse:lma=on:nwc=4:sp=occurrence:urr=on_8");
      quick.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:stl=30:sac=on:urr=on_41");
      quick.push("lrs+10_8:1_av=off:bs=unit_only:cond=on:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:nwc=1.2:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_12");
      quick.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_143");
      quick.push("lrs+10_8:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lcm=predicate:lwlo=on:nm=64:nwc=1:stl=30:sos=all:updr=off_78");
      quick.push("dis+1010_3:1_av=off:irw=on:nm=32:nwc=1:sos=all:urr=ec_only:updr=off_96");
      quick.push("dis+1011_5_add=off:afr=on:afp=10000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:nm=32:nwc=1:sas=z3:sd=3:ss=axioms:st=2.0:sp=occurrence:updr=off_2");
      quick.push("lrs+10_1_av=off:fde=unused:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity_113");
      quick.push("lrs+2_2_aac=none:afr=on:afp=1000:afq=1.1:amm=sco:anc=all:bd=off:bce=on:cond=on:gs=on:gsaa=from_current:nm=2:nwc=2.5:stl=30:sac=on:urr=on_26");
      quick.push("dis+1002_3:1_acc=model:afr=on:afp=40000:afq=1.1:anc=none:ccuc=first:fsr=off:gsp=input_only:irw=on:nm=16:nwc=1:sos=all_8");
      quick.push("dis+11_2_av=off:cond=fast:ep=RST:fsr=off:lma=on:nm=16:nwc=1.2:sp=occurrence:updr=off_1");
      quick.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_111");
      quick.push("lrs+2_4_awrs=decay:awrsf=1:afr=on:afp=10000:afq=1.0:amm=off:anc=none:bd=off:cond=on:fde=none:gs=on:lcm=predicate:nm=2:nwc=4:sas=z3:stl=30:s2a=on:sp=occurrence:urr=on:uhcvi=on_9");
      quick.push("lrs+1011_5_afr=on:afp=100000:afq=1.0:amm=off:anc=none:cond=on:lma=on:nm=6:nwc=1:sas=z3:stl=30:sac=on:urr=on_1");
      quick.push("ott+1_128_add=large:afr=on:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.1:nicw=on:sas=z3:sac=on:sp=reverse_arity_44");
      quick.push("lrs+10_3:1_av=off:bsr=on:cond=on:er=known:gs=on:lcm=reverse:nm=32:nwc=4:stl=30:sp=occurrence:urr=on:updr=off_73");
      quick.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_275");
      quick.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=2.0:sac=on:urr=on_117");
      quick.push("ott+10_8:1_av=off:bd=preordered:bsr=on:cond=fast:fsr=off:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1.2:sp=reverse_arity:urr=on:updr=off:uhcvi=on_1");
      quick.push("dis+3_24_av=off:bd=off:bs=unit_only:fsr=off:fde=unused:gs=on:irw=on:lma=on:nm=0:nwc=1.1:sos=on:uhcvi=on_180");
      quick.push("lrs+1011_10_av=off:bce=on:cond=on:fsr=off:fde=unused:gs=on:nm=2:nwc=1.1:stl=30:sd=4:ss=axioms:s2a=on:st=1.5:sos=on:sp=frequency:urr=on:updr=off:uhcvi=on_1");
      quick.push("ott+1_5_afp=40000:afq=1.0:anc=all:fde=none:gs=on:irw=on:lma=on:nm=32:nwc=2:sos=all:sac=on:sp=occurrence:urr=ec_only:uhcvi=on_34");
    }
    break;

  case Property::FNE:
    if (atoms > 1000) {
      quick.push("dis+11_3_afr=on:afp=4000:afq=1.4:anc=none:cond=on:fsr=off:gs=on:lcm=reverse:nm=64:nwc=1:sos=on:sp=reverse_arity_3");
      quick.push("dis+1011_10_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=64:nwc=4:sac=on:sp=occurrence_75");
      quick.push("ins+11_32_av=off:igbrr=0.4:igrr=1/64:igrpq=1.05:igwr=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:updr=off_55");
      quick.push("lrs+1011_1024_add=large:afp=4000:afq=1.1:anc=none:br=off:fsr=off:gsp=input_only:lma=on:nwc=1:stl=30:sos=on:urr=on_187");
      quick.push("dis+11_24_afp=40000:afq=1.1:amm=sco:anc=none:bs=on:gs=on:gsem=off:lcm=predicate:lma=on:nm=2:nwc=1:sos=on:sac=on:updr=off_52");
      quick.push("dis+11_3_awrs=decay:awrsf=256:av=off:gs=on:gsem=on:lcm=reverse:nm=0:nwc=1:sos=all:sp=frequency:updr=off_481");
      quick.push("lrs+1011_40_add=off:afr=on:afp=4000:afq=1.2:amm=sco:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:stl=60:sos=all:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_388");
      quick.push("ott+11_3:2_afp=40000:afq=1.0:amm=sco:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=full_model:lcm=reverse:nm=32:newcnf=on:nwc=5:nicw=on:sd=3:ss=axioms:sac=on:urr=on:updr=off_1019");
    }
    else {
      quick.push("dis+1011_10_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=64:nwc=4:sac=on:sp=occurrence_2");
      quick.push("dis+1002_8:1_awrs=converge:awrsf=256:anc=all_dependent:br=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:irw=on:nm=64:nwc=1:sas=z3:s2a=on:sp=frequency:thf=on:uwa=interpreted_only:urr=on_7");
      quick.push("lrs+1_3_awrs=decay:awrsf=4:afp=10000:afq=1.0:amm=off:anc=none:bd=off:cond=on:fsr=off:fde=unused:gs=on:lwlo=on:nm=16:nwc=1:sas=z3:stl=30:ss=axioms:s2a=on:st=1.2:sos=theory:sp=frequency_3");
      quick.push("lrs-11_4:1_afp=4000:afq=2.0:anc=none:br=off:gs=on:lwlo=on:nm=64:nwc=3:stl=30:urr=on_3");
      quick.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_12");
      quick.push("dis+1002_8_awrs=converge:awrsf=64:av=off:cond=fast:fsr=off:gsp=input_only:lma=on:nm=64:nwc=1.2:s2a=on:sos=on:sp=frequency:urr=on:updr=off:uhcvi=on_12");
      quick.push("dis+1_8_afp=4000:afq=1.1:amm=sco:gsp=input_only:nm=64:newcnf=on:nwc=4:sac=on:sp=occurrence:updr=off_191");
      quick.push("dis+10_128_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=2:nwc=1:sp=reverse_arity_3");
      quick.push("lrs+1003_2:3_afr=on:afp=100000:afq=1.1:amm=off:anc=none:bs=on:fde=unused:gs=on:inw=on:nm=0:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:sp=occurrence:tha=off:updr=off:uhcvi=on_2");
      quick.push("dis+11_24_afp=40000:afq=1.1:amm=sco:anc=none:bs=on:gs=on:gsem=off:lcm=predicate:lma=on:nm=2:nwc=1:sos=on:sac=on:updr=off_91");
      quick.push("lrs+10_4:1_av=off:bd=off:bsr=on:cond=on:fde=unused:inw=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=5:stl=90:sp=reverse_arity:thi=strong:uwa=ground:updr=off:uwaf=on_73");
      quick.push("dis+11_3_awrs=decay:awrsf=256:av=off:gs=on:gsem=on:lcm=reverse:nm=0:nwc=1:sos=all:sp=frequency:updr=off_4");
      quick.push("lrs+1011_2:1_add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:thf=on:urr=on:updr=off_81");
      quick.push("dis+11_40_afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sos=all:sp=reverse_arity:thf=on:urr=on:updr=off_2");
      quick.push("lrs+1_1_av=off:bsr=on:br=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sp=occurrence:urr=on:updr=off_152");
      quick.push("ott+1_40_av=off:bs=unit_only:bsr=on:br=off:fsr=off:lma=on:nm=64:newcnf=on:nwc=1.5:sp=occurrence:urr=on:updr=off_81");
      quick.push("dis+1011_4_awrs=decay:awrsf=32:afp=40000:afq=1.0:amm=off:anc=all:bs=on:cond=on:fsr=off:gsp=input_only:lma=on:nm=16:nwc=1:nicw=on:sac=on:sp=frequency:thi=all:updr=off:uhcvi=on_670");
      quick.push("ott+11_5:1_afp=100000:afq=1.1:br=off:gs=on:nm=64:nwc=1:sos=all:urr=on:updr=off_287");
      quick.push("lrs+1010_4:1_aac=none:add=off:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=on:gs=on:gsem=on:irw=on:nm=0:nwc=2.5:sas=z3:stl=30:sos=theory:sp=reverse_arity:updr=off_2");
      quick.push("lrs+10_4_add=off:afp=100000:afq=2.0:amm=sco:anc=none:nm=64:nwc=1:stl=150:sp=occurrence:updr=off_733");
      quick.push("dis+1_14_awrs=converge:awrsf=256:av=off:bs=on:bsr=on:bce=on:cond=fast:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=4:nwc=1.7:sp=weighted_frequency:urr=on_33");
    }
    break;

  case Property::NEQ:
    if (prop == 1) {
      quick.push("lrs+1011_2_acc=on:add=off:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:er=known:nwc=4:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off_2");
      quick.push("dis+10_10_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:gs=on:gsem=off:nwc=1:sas=z3:sd=3:ss=axioms:st=2.0:sp=occurrence:updr=off_2");
      quick.push("dis+1011_12_av=off:bd=off:bs=unit_only:fsr=off:lma=on:nwc=1:urr=ec_only:updr=off_18");
      quick.push("lrs+11_5_av=off:cond=on:gs=on:gsem=on:irw=on:nwc=1:stl=30:sd=1:ss=axioms:sos=on:sp=occurrence:urr=on_4");
      quick.push("lrs+11_2_afr=on:afp=100000:afq=1.4:amm=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:stl=30:sos=on:updr=off_3");
      quick.push("dis+11_4_aac=none:add=large:afp=100000:afq=1.0:amm=off:anc=none:bsr=on:cond=on:fsr=off:gs=on:lcm=reverse:lma=on:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=ec_only_3");
      quick.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_7");
      quick.push("dis+1011_2:1_av=off:lma=on:nwc=1:sd=3:ss=axioms:st=5.0:sp=occurrence:urr=ec_only_30");
      quick.push("lrs+10_3_av=off:bd=off:cond=on:gs=on:gsem=off:irw=on:lwlo=on:nwc=1.2:stl=30:sp=reverse_arity:updr=off_72");
      quick.push("lrs+10_12_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=off:cond=on:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=1.3:sas=z3:stl=30:sac=on_71");
      quick.push("dis+1010_3:1_aac=none:add=large:afp=100000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:gs=on:lma=on:nwc=1:sos=all:sp=occurrence_18");
      quick.push("lrs+11_14_av=off:bd=off:bs=unit_only:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lcm=reverse:lwlo=on:nwc=1:stl=30:sos=on:sp=reverse_arity:urr=on:updr=off_46");
      quick.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bce=on:fde=unused:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_163");
      quick.push("lrs+4_2_av=off:bs=on:er=known:gs=on:irw=on:lwlo=on:nwc=10:stl=30:sp=occurrence_243");
      quick.push("lrs+11_10_aac=none:acc=on:afp=4000:afq=1.2:amm=sco:anc=none:ccuc=first:fsr=off:irw=on:nwc=2:nicw=on:stl=30:sd=5:ss=axioms:st=1.2:sos=theory:urr=ec_only:updr=off_155");
      quick.push("dis-10_3:1_add=large:afr=on:afp=1000:afq=2.0:anc=none:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nwc=1:sas=z3:sd=2:ss=axioms:st=2.0:sos=all:sac=on:sp=reverse_arity:urr=on:uhcvi=on_84");
      quick.push("ott+10_4:1_av=off:bd=preordered:cond=fast:fde=unused:irw=on:lcm=reverse:nwc=3:sd=2:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_41");
      quick.push("lrs+1011_3:1_av=off:bs=unit_only:bsr=on:er=known:fsr=off:fde=unused:irw=on:lcm=reverse:lwlo=on:nwc=1.7:stl=30:sos=on:sp=occurrence:updr=off_229");
      quick.push("lrs+1_20_add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=z3:stl=30:sd=3:ss=axioms:st=1.2:sp=occurrence:updr=off_136");
      quick.push("dis-11_64_av=off:bd=off:bs=on:cond=on:fsr=off:nwc=1:sd=1:ss=axioms:urr=ec_only:updr=off_99");
      quick.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_68");
      quick.push("dis+11_5_afr=on:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:urr=ec_only:updr=off_6");
      quick.push("lrs+1011_2:1_av=off:bce=on:cond=fast:fsr=off:fde=none:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:st=3.0:sp=occurrence:urr=ec_only:updr=off_144");
      quick.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=off:anc=none:cond=on:irw=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0:sos=all:sp=occurrence_205");
      quick.push("lrs-2_2:1_afr=on:afp=1000:afq=2.0:anc=none:bd=off:bce=on:cond=fast:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1.5:stl=30:sac=on:sp=reverse_arity:uhcvi=on_183");
      quick.push("dis+1011_1_add=off:afp=100000:afq=1.4:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:nwc=1:nicw=on:sac=on:sp=occurrence:urr=on_77");
      quick.push("dis+4_5:1_av=off:nwc=1:sos=all:sp=reverse_arity:urr=on:updr=off_53");
      quick.push("dis+10_3_acc=on:afr=on:afp=1000:afq=1.0:amm=sco:bs=unit_only:ccuc=first:fsr=off:irw=on:lcm=reverse:lma=on:nwc=1.5:updr=off_63");
    }
    else if (prop == 3) {
      quick.push("dis-11_64_av=off:bd=off:bs=on:cond=on:fsr=off:nwc=1:sd=1:ss=axioms:urr=ec_only:updr=off_1");
      quick.push("lrs+10_5:1_add=large:afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_5");
      quick.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_5");
      quick.push("lrs+1011_2:1_av=off:bsr=on:cond=on:nwc=1:sas=z3:stl=30:sd=1:ss=axioms:st=1.2:sp=reverse_arity_8");
      quick.push("lrs+2_3:2_aac=none:acc=model:add=off:afr=on:afp=10000:afq=1.4:anc=none:bs=on:bsr=on:ccuc=first:gsp=input_only:gs=on:gsem=off:lcm=reverse:lma=on:nwc=3:stl=30:sd=3:ss=axioms:st=2.0:sac=on:urr=on_29");
      quick.push("lrs+1004_20_av=off:cond=on:er=filter:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1:stl=30:sd=3:ss=axioms:st=3.0:sos=on:urr=ec_only_57");
      quick.push("lrs+1002_3:1_av=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nwc=4:sas=z3:stl=30:sd=1:ss=axioms:st=5.0:sp=reverse_arity_4");
      quick.push("dis+11_5_afr=on:afp=1000:afq=1.0:amm=off:anc=none:irw=on:lcm=reverse:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sp=occurrence_2");
      quick.push("dis+1010_1024_afr=on:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:irw=on:lwlo=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0_3");
      quick.push("dis+2_5:4_add=large:afp=4000:afq=1.2:anc=all:bce=on:cond=fast:fde=none:lma=on:nwc=10:sd=1:ss=axioms:st=1.5:sac=on_9");
      quick.push("dis-11_4:1_amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nwc=10:sac=on:sp=reverse_arity:urr=on_14");
      quick.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=off:anc=none:cond=on:irw=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0:sos=all:sp=occurrence_3");
      quick.push("lrs+1003_3_av=off:bd=off:fde=none:gs=on:lma=on:nwc=1:stl=30:sd=7:ss=axioms:st=1.2:sos=all:sp=reverse_arity:updr=off:uhcvi=on_103");
      quick.push("lrs+1_2:1_av=off:fsr=off:lma=on:nwc=1:stl=30:sd=7:ss=axioms:st=1.2:sos=on:urr=ec_only_11");
      quick.push("lrs+1010_8:1_av=off:bs=unit_only:br=off:cond=on:fsr=off:irw=on:nwc=1.3:stl=30:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=on:updr=off_62");
      quick.push("ins+11_3_av=off:bd=off:igbrr=0.6:igrr=1/8:igrp=700:igrpq=2.0:igs=1:igwr=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:updr=off_3");
      quick.push("lrs+1011_2:1_av=off:cond=on:er=known:gs=on:gsem=on:lwlo=on:nwc=1.3:stl=30:updr=off:uhcvi=on_180");
      quick.push("lrs+1010_3:1_av=off:cond=on:nwc=5:stl=30:sp=reverse_arity_18");
      quick.push("dis+1011_3_av=off:nwc=1:sos=all:sp=reverse_arity_67");
      quick.push("lrs+1011_5:4_av=off:bd=off:fsr=off:gs=on:nwc=1.3:stl=30:urr=ec_only:updr=off_91");
      quick.push("lrs+1011_4:1_av=off:cond=on:irw=on:lma=on:nwc=1:stl=30:sd=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_124");
      quick.push("dis+1011_1_add=off:afp=100000:afq=1.4:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:nwc=1:nicw=on:sac=on:sp=occurrence:urr=on_6");
      quick.push("dis-11_5:1_acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=first:cond=on:gs=on:gsem=off:nwc=1:sd=3:ss=axioms:st=1.2:sos=all_76");
      quick.push("lrs+1011_40_add=off:afp=1000:afq=2.0:anc=none:bs=unit_only:fsr=off:irw=on:nwc=1:sas=z3:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_86");
      quick.push("dis+1002_5:1_av=off:cond=fast:fsr=off:fde=none:lma=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:urr=on:updr=off_19");
      quick.push("lrs-1_14_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:stl=30:sd=1:ss=axioms:sp=reverse_arity:urr=ec_only:updr=off_176");
      quick.push("lrs+1_32_av=off:bd=off:br=off:gs=on:gsem=on:irw=on:nwc=1:stl=30:sd=1:ss=axioms:sp=occurrence:urr=on:updr=off_9");
      quick.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:stl=30:updr=off_287");
      quick.push("dis+11_2_av=off:gs=on:gsem=on:irw=on:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=reverse_arity_6");
      quick.push("lrs+1_3_av=off:fsr=off:lma=on:nwc=1:stl=30:sd=1:ss=axioms:sp=occurrence:urr=on_22");
      quick.push("dis+11_3_av=off:bd=off:bsr=on:bce=on:gsp=input_only:gs=on:gsem=on:lma=on:nwc=2.5:sp=reverse_arity:urr=ec_only:uhcvi=on_205");
      quick.push("lrs-11_3_aac=none:acc=on:afr=on:afp=10000:afq=1.1:bd=off:bs=unit_only:ccuc=first:irw=on:lcm=predicate:lma=on:nwc=1.5:stl=30:sos=all:sac=on:sp=occurrence:updr=off_111");
      quick.push("lrs+1011_2:1_av=off:bce=on:cond=fast:fsr=off:fde=none:lwlo=on:nwc=1:stl=30:sd=1:ss=axioms:st=3.0:sp=occurrence:urr=ec_only:updr=off_102");
      quick.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_168");
      quick.push("dis+1011_2:1_av=off:lma=on:nwc=1:sd=3:ss=axioms:st=5.0:sp=occurrence:urr=ec_only_154");
    }
    else {
      quick.push("dis+11_5_afr=on:afp=1000:afq=1.0:amm=off:anc=none:irw=on:lcm=reverse:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:sac=on:sp=occurrence_6");
      quick.push("dis+1002_4_afr=on:afp=1000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=off:lma=on:nwc=1:sos=on:sac=on:sp=occurrence_45");
      quick.push("dis+1003_3_add=off:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:bsr=on:bce=on:cond=fast:fde=none:gs=on:gsaa=from_current:gsem=off:nwc=1.2:sas=z3:sac=on:sp=reverse_arity_13");
      quick.push("lrs+1011_5_av=off:cond=on:er=filter:gs=on:nwc=1.7:stl=30:updr=off_3");
      quick.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:stl=30:updr=off_3");
      quick.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_8");
      quick.push("lrs+1011_3:1_av=off:bs=unit_only:bsr=on:er=known:fsr=off:fde=unused:irw=on:lcm=reverse:lwlo=on:nwc=1.7:stl=30:sos=on:sp=occurrence:updr=off_3");
      quick.push("lrs+1011_2_acc=on:add=off:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:er=known:nwc=4:stl=30:sac=on:sp=reverse_arity:urr=ec_only:updr=off_13");
      quick.push("lrs+10_5:1_add=large:afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:stl=30:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_5");
      quick.push("ott+1011_2:3_av=off:cond=fast:er=filter:fde=unused:gsp=input_only:irw=on:nwc=3:sp=occurrence:updr=off:uhcvi=on_4");
      quick.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_79");
      quick.push("lrs+10_2:1_av=off:bsr=on:cond=on:er=known:irw=on:lcm=predicate:nwc=4:stl=30:sp=occurrence_3");
      quick.push("dis+2_3_av=off:cond=on:fsr=off:lcm=reverse:lma=on:nwc=1:sos=on:sp=reverse_arity_6");
      quick.push("lrs+11_2_afr=on:afp=100000:afq=1.4:amm=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:stl=30:sos=on:updr=off_5");
      quick.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_100");
      quick.push("dis+11_3_afp=100000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=10:sac=on_17");
      quick.push("lrs+11_1_av=off:fsr=off:irw=on:lma=on:lwlo=on:nwc=1:stl=30:sp=reverse_arity:urr=on:updr=off_44");
      quick.push("lrs+4_2_av=off:bs=on:er=known:gs=on:irw=on:lwlo=on:nwc=10:stl=30:sp=occurrence_100");
      quick.push("ott+1011_4_acc=on:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:bs=unit_only:ccuc=small_ones:cond=on:fsr=off:irw=on:lwlo=on:nwc=1.3:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_23");
      quick.push("lrs+11_5_add=large:afr=on:afp=40000:afq=1.2:cond=on:er=known:gs=on:gsem=off:nwc=1.5:stl=30:sp=occurrence:updr=off_65");
      quick.push("lrs+1011_2_av=off:bd=preordered:bs=unit_only:cond=fast:fsr=off:fde=unused:irw=on:lma=on:lwlo=on:nwc=1.2:stl=30:sp=occurrence:uhcvi=on_90");
      quick.push("ott+10_28_acc=model:add=off:afp=40000:afq=2.0:amm=off:bs=unit_only:bsr=on:ccuc=small_ones:cond=on:fsr=off:gs=on:gsem=on:nwc=1.1:nicw=on:urr=on:updr=off_50");
      quick.push("ott+11_6_av=off:bs=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=10:sp=reverse_arity_100");
      quick.push("lrs+1010_4_aac=none:acc=model:add=large:afp=10000:afq=1.4:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:cond=on:fsr=off:irw=on:nwc=2:stl=30:sac=on:sp=reverse_arity:urr=ec_only_66");
      quick.push("lrs+1010_3:2_av=off:fsr=off:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=ec_only_101");
      quick.push("dis+11_12_av=off:bsr=on:cond=on:lcm=predicate:lma=on:nwc=5:sp=reverse_arity:updr=off_33");
      quick.push("dis+1011_32_aac=none:afp=10000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=3:sas=z3:sp=reverse_arity_2");
      quick.push("dis+1011_3_av=off:nwc=1:sos=all:sp=reverse_arity_114");
      quick.push("lrs+1011_2:1_av=off:cond=on:lwlo=on:nwc=1.5:stl=30_187");
      quick.push("lrs+1011_2:1_afp=40000:afq=1.1:amm=off:anc=none:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=on:nwc=1:sas=z3:stl=30:sos=all:sp=reverse_arity:updr=off:uhcvi=on_38");
      quick.push("dis+11_2:1_acc=model:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:er=known:gsp=input_only:irw=on:lma=on:nwc=5:sac=on:urr=ec_only_148");
      quick.push("lrs+4_1_av=off:bd=off:bsr=on:bce=on:fsr=off:irw=on:lcm=predicate:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on:uhcvi=on_39");
      quick.push("ott+1011_8:1_av=off:bd=off:cond=on:nwc=1:sos=all:sp=reverse_arity_182");
      quick.push("dis+1011_8_av=off:bd=off:cond=fast:er=known:fde=unused:gsp=input_only:lcm=predicate:lma=on:nwc=1.2:sp=reverse_arity:updr=off:uhcvi=on_33");
      quick.push("lrs+1011_2:1_av=off:cond=on:er=known:gs=on:gsem=on:lwlo=on:nwc=1.3:stl=30:updr=off:uhcvi=on_133");
      quick.push("lrs+1011_40_add=off:afp=1000:afq=2.0:anc=none:bs=unit_only:fsr=off:irw=on:nwc=1:sas=z3:stl=30:sos=on:sac=on:sp=reverse_arity:updr=off_68");
      quick.push("dis+2_10_afp=100000:afq=1.2:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1_5");
      quick.push("lrs+4_4:1_add=large:afr=on:afp=100000:afq=1.0:anc=none:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nwc=1.5:sas=z3:stl=30:sos=all:sac=on:sp=reverse_arity_148");
      quick.push("lrs+1011_10_av=off:bs=unit_only:bsr=on:er=filter:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nwc=1.2:stl=30:sp=reverse_arity:updr=off_105");
      quick.push("lrs+4_5_av=off:bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:nwc=2.5:stl=30:sp=occurrence:updr=off_139");
      quick.push("lrs+1011_5:1_afp=100000:afq=1.0:anc=none:bd=off:gsp=input_only:gs=on:gsem=off:lwlo=on:nwc=5:nicw=on:sas=z3:stl=30:sac=on:updr=off_176");
    }
    break;

  case Property::HEQ:
    quick.push("dis+10_6_av=off:cond=on:gs=on:lcm=reverse:lma=on:nwc=1.7:sp=reverse_arity:updr=off_7");
    quick.push("lrs+11_24_afp=100000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:irw=on:nwc=3:stl=30_3");
    quick.push("lrs+11_3:1_av=off:cond=on:fsr=off:irw=on:lma=on:nwc=10:sas=z3:stl=30:updr=off_174");
    quick.push("lrs+11_4_aac=none:acc=model:add=large:afp=100000:afq=1.2:amm=off:anc=none:ccuc=first:cond=fast:gs=on:lma=on:nwc=4:stl=30:sac=on:sp=occurrence_237");
    quick.push("lrs+1011_20_acc=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:ccuc=small_ones:cond=on:gs=on:irw=on:lwlo=on:nwc=1:stl=30:sp=occurrence_37");
    quick.push("dis+10_4_aac=none:afp=10000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:irw=on:lcm=reverse:lwlo=on:nwc=2.5:sas=z3:sp=reverse_arity_4");
    quick.push("lrs+10_40_aac=none:acc=on:add=off:afp=1000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:ccuc=first:cond=fast:fsr=off:fde=none:gs=on:gsem=on:lma=on:nwc=1.3:stl=30:sp=reverse_arity:updr=off:uhcvi=on_38");
    quick.push("ott+11_64_add=large:afp=1000:afq=2.0:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=on:nwc=2_246");
    quick.push("lrs+2_14_aac=none:add=off:afp=100000:afq=1.1:anc=none:nwc=3:sas=z3:stl=30:sac=on:sp=reverse_arity:updr=off_167");
    quick.push("lrs+11_40_add=off:afp=10000:afq=1.2:amm=off:cond=on:fsr=off:lma=on:nwc=1.7:stl=30:sac=on:sp=occurrence_182");
    quick.push("lrs+11_3:2_add=off:afr=on:afp=4000:afq=1.4:anc=none:cond=on:lma=on:lwlo=on:nwc=3:sas=z3:stl=30:sac=on:sp=reverse_arity_124");
    quick.push("lrs+11_4:1_av=off:bce=on:cond=fast:fde=none:gsp=input_only:lma=on:nwc=5:stl=30:sp=occurrence_118");
    quick.push("dis+11_28_add=off:afr=on:afp=40000:afq=2.0:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off_53");
    quick.push("ott+11_2:1_afp=1000:afq=1.0:bd=preordered:fsr=off:fde=none:lma=on:nwc=1:sos=all:sp=occurrence:uhcvi=on_171");
    quick.push("lrs+10_3:1_add=off:afp=40000:afq=1.4:anc=none:br=off:fsr=off:gs=on:gsem=on:irw=on:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:urr=on_222");
    quick.push("lrs+10_1024_av=off:bd=off:fsr=off:lma=on:nwc=1:stl=30:sp=occurrence:urr=on_205");
    break;

  case Property::PEQ:
    if (prop == 1) {
      quick.push("lrs+11_3:2_afp=10000:afq=1.0:anc=none:bsr=on:cond=on:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:stl=30:sac=on:updr=off_7");
      quick.push("lrs+11_128_aac=none:add=large:afp=4000:afq=2.0:amm=sco:bd=off:gs=on:gsem=on:nwc=1:nicw=on:stl=30:sos=all:sac=on:sp=reverse_arity:updr=off_15");
      quick.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:stl=30:sp=occurrence_33");
      quick.push("dis+10_5_add=off:afr=on:afp=10000:afq=1.4:anc=none:er=known:gs=on:gsem=off:lma=on:nwc=1:nicw=on:sas=z3:sac=on:sp=reverse_arity_5");
      quick.push("lrs+1011_7_av=off:irw=on:nwc=1:stl=30:sos=all_69");
      quick.push("dis+1011_2:3_av=off:irw=on:nwc=1.2:sp=reverse_arity:updr=off_157");
      quick.push("lrs+11_2:3_add=large:afp=4000:afq=2.0:amm=sco:anc=none:er=known:gs=on:nwc=1:sas=z3:stl=30:updr=off_4");
      quick.push("lrs+10_5_av=off:bd=off:bs=unit_only:cond=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:stl=30:sp=occurrence_223");
      quick.push("ott+11_1_afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sac=on:sp=occurrence:updr=off:uhcvi=on_624");
      quick.push("lrs+11_32_add=off:afp=10000:afq=1.1:anc=all:bs=unit_only:cond=fast:fde=none:gs=on:gsaa=from_current:irw=on:lma=on:nwc=1:nicw=on:stl=60:sos=all:sac=on:sp=occurrence:updr=off:uhcvi=on_501");
      quick.push("lrs+11_2:1_av=off:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=2:stl=60:sp=occurrence:updr=off:uhcvi=on_546");
    }
    else {
      quick.push("lrs+11_3:2_afp=10000:afq=1.0:anc=none:bsr=on:cond=on:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:stl=30:sac=on:updr=off_7");
      quick.push("lrs+11_1024_add=off:afp=10000:afq=1.0:anc=none:bd=off:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:stl=30:sp=occurrence:updr=off_9");
      quick.push("lrs+11_5_acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:bs=unit_only:ccuc=first:cond=on:lma=on:lwlo=on:nwc=1:stl=30:sp=reverse_arity:updr=off_234");
      quick.push("ott+1010_14_add=large:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:bsr=on:cond=fast:lma=on:nwc=1.2:sp=occurrence:updr=off_259");
      quick.push("lrs+11_2:3_add=large:afp=4000:afq=2.0:amm=sco:anc=none:er=known:gs=on:nwc=1:sas=z3:stl=30:updr=off_1");
      quick.push("lrs+11_3:1_add=off:afr=on:afp=100000:afq=1.4:anc=none:cond=on:gs=on:irw=on:lma=on:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sac=on:sp=occurrence:updr=off_298");
      quick.push("dis+10_28_add=large:afp=4000:afq=1.0:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence_72");
      quick.push("dis+1011_8_av=off:bsr=on:cond=on:fsr=off:fde=none:gs=on:lma=on:nwc=1.1:sos=all:sp=reverse_arity_207");
      quick.push("lrs+2_2_acc=model:add=off:afp=10000:afq=1.2:anc=all:bd=off:bs=on:bsr=on:ccuc=small_ones:cond=on:fsr=off:fde=unused:gs=on:lma=on:nwc=1.2:stl=30:sos=on:updr=off:uhcvi=on_43");
      quick.push("lrs+10_4:1_av=off:bd=off:bs=unit_only:bsr=on:fsr=off:gs=on:gsem=on:lwlo=on:nwc=1:stl=30:sos=all_135");
      quick.push("ott+1_3_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bsr=on:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence:uhcvi=on_138");
      quick.push("ott+11_5:1_add=off:afp=10000:afq=1.4:anc=none:bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:lma=on:nwc=1_147");
      quick.push("lrs+10_5:4_add=large:afr=on:amm=sco:anc=all_dependent:bd=preordered:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1:sas=z3:stl=60:sos=all:sac=on:updr=off:uhcvi=on_142");
      quick.push("ott+11_7_acc=model:afr=on:afp=40000:afq=2.0:amm=off:anc=all_dependent:bs=unit_only:ccuc=small_ones:fsr=off:gs=on:gsaa=from_current:lma=on:nwc=1.7:nicw=on:sp=occurrence:uhcvi=on_116");
      quick.push("lrs+11_2_add=large:afp=4000:afq=1.1:anc=none:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence:updr=off_110");
      quick.push("lrs+10_2_afr=on:afp=4000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:sac=on:sp=occurrence:updr=off_107");
      quick.push("ott+1011_5_afr=on:afp=1000:afq=1.4:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=unused:gsp=input_only:gs=on:nwc=1:nicw=on:sas=z3:sos=all:updr=off:uhcvi=on_484");
      quick.push("lrs+11_3_aac=none:add=large:afp=1000:afq=1.4:anc=all_dependent:bd=off:bs=on:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=1:stl=60:sos=all:sac=on:sp=occurrence:updr=off_587");
    }
    break;

  case Property::HNE:
    quick.push("lrs+1011_8_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bs=on:gs=on:gsem=off:nwc=1.5:nicw=on:stl=30:sac=on:sp=reverse_arity:updr=off_32");
    quick.push("dis+11_1_add=off:afp=4000:afq=2.0:amm=sco:anc=none:br=off:cond=on:lma=on:nwc=1:sos=all:sac=on:urr=on_7");
    quick.push("lrs+10_2:3_av=off:bsr=on:cond=on:lwlo=on:nwc=1.7:stl=60:sp=occurrence:updr=off_171");
    quick.push("dis+1_40_av=off:lwlo=on:nwc=4:sos=all:sp=occurrence:updr=off_117");
    quick.push("lrs+4_3_add=off:afr=on:afp=10000:afq=2.0:anc=none:lma=on:nwc=1.1:nicw=on:sas=z3:stl=30:sac=on_203");
    quick.push("lrs+10_3:2_afr=on:afp=100000:afq=2.0:anc=none:bs=unit_only:bsr=on:cond=fast:fsr=off:lwlo=on:nwc=2:nicw=on:stl=30:sp=reverse_arity_211");
    quick.push("lrs+10_2:3_av=off:cond=on:fsr=off:lwlo=on:nwc=2.5:stl=30:sp=reverse_arity:updr=off_68");
    quick.push("lrs+1011_3_av=off:bs=unit_only:cond=on:fsr=off:lcm=predicate:lma=on:lwlo=on:nwc=1:stl=30:sp=occurrence_215");
    quick.push("lrs+11_24_afp=4000:afq=2.0:amm=sco:anc=all:br=off:cond=fast:gsp=input_only:gs=on:gsem=on:lma=on:lwlo=on:nwc=1.7:nicw=on:stl=30:sos=theory:sac=on:sp=reverse_arity:urr=on_122");
    quick.push("lrs+10_5:4_av=off:bsr=on:gs=on:gsem=off:lma=on:nwc=4:stl=30:uhcvi=on_146");
    quick.push("lrs+1_128_afp=100000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:stl=60:sac=on_455");
    quick.push("dis+11_3:1_aac=none:add=off:afp=4000:afq=1.4:fsr=off:nwc=3:nicw=on:sp=occurrence_101");
    quick.push("dis+11_8:1_afr=on:afp=1000:afq=1.0:amm=off:anc=none:nwc=1:sos=all:sp=occurrence:updr=off_184");
    quick.push("lrs+1_2:3_av=off:fsr=off:lma=on:nwc=1:sas=z3:stl=30:urr=ec_only:updr=off_72");
    break;

  case Property::NNE:
    quick.push("dis+1011_5_av=off:gs=on:gsem=on:nwc=1:sos=on:updr=off_3");
    quick.push("dis+1011_64_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:gsp=input_only:gs=on:gsem=on:lma=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off:uhcvi=on_26");
    quick.push("lrs+11_14_aac=none:afp=1000:afq=2.0:fsr=off:lma=on:nwc=1:stl=30:sp=reverse_arity_26");
    quick.push("dis-1_3_av=off:cond=on:fsr=off:gs=on:gsem=on:nwc=1:updr=off_4");
    quick.push("dis+10_5:4_afp=100000:afq=1.0:amm=sco:anc=all:cond=fast:fsr=off:gs=on:lma=on:nwc=1:sp=reverse_arity:updr=off:uhcvi=on_24");
    quick.push("ins+10_1_av=off:cond=on:fsr=off:gsp=input_only:igbrr=0.6:igpr=on:igrr=64/1:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:updr=off_9");
    quick.push("dis+1011_5_av=off:lma=on:nwc=1.7:sos=all:sp=reverse_arity:updr=off_430");
    quick.push("lrs+1011_50_av=off:bs=unit_only:cond=on:nwc=2:stl=30:sp=occurrence:updr=off_47");
    quick.push("dis+4_7_av=off:cond=fast:gsp=input_only:lma=on:nwc=1.3:sp=occurrence:urr=ec_only:uhcvi=on_162");
    quick.push("dis+3_64_av=off:cond=fast:lcm=reverse:lma=on:lwlo=on:nwc=1:sos=on:updr=off_68");
    quick.push("lrs+1011_2_av=off:cond=on:gsp=input_only:gs=on:lwlo=on:nwc=1:stl=30:sos=all:uhcvi=on_300");
    quick.push("lrs+4_20_av=off:gs=on:gsem=off:lcm=reverse:lma=on:nwc=1:stl=30:sos=on:updr=off_43");
    quick.push("lrs+1010_14_av=off:fsr=off:lma=on:lwlo=on:nwc=2.5:stl=30:updr=off:uhcvi=on_82");
    quick.push("lrs+1011_3:2_av=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:lcm=predicate:lwlo=on:nwc=2.5:stl=30:sos=all:updr=off_158");
    quick.push("dis+2_2_add=off:afr=on:afp=4000:afq=1.0:amm=sco:anc=none:fsr=off:lcm=predicate:lma=on:nwc=1.3:nicw=on:sos=theory:sp=reverse_arity:urr=ec_only:updr=off_26");
    quick.push("dis+1003_128_add=large:afr=on:amm=off:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:sas=z3:sos=on:sp=occurrence:updr=off:uhcvi=on_64");
    quick.push("lrs+11_2_afp=1000:afq=1.4:amm=sco:anc=none:cond=on:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on:updr=off_155");
    break;

  case Property::UEQ:
    if (prop == 4096) {
      quick.push("lrs+10_4:1_av=off:bd=preordered:fde=none:ins=3:lwlo=on:nwc=3:stl=60:sp=reverse_arity_113");
      quick.push("ott+10_128_av=off:bd=off:ins=3:nwc=1:sp=reverse_arity_45");
      quick.push("ott+10_4_av=off:bd=off:ins=3:nwc=1.5:sp=reverse_arity_568");
      quick.push("ott+10_8_awrs=converge:awrsf=8:av=off:bd=preordered:ins=3:nwc=1:s2a=on_1858");
    }
    else if (prop == 0) {
      if (atoms > 12) {
        quick.push("ott+10_16_av=off:ins=3:nwc=1.2_1139");
        quick.push("lrs+10_3:1_awrs=decay:awrsf=512:av=off:fde=unused:ins=3:nwc=1:stl=150:s2a=on:sp=frequency:uhcvi=on_103");
        quick.push("ott+10_12_awrs=decay:awrsf=32:av=off:bd=preordered:ins=3:nwc=1:s2a=on:sp=frequency:uhcvi=on_107");
        quick.push("lrs+10_14_awrs=decay:awrsf=128:av=off:bd=off:fde=none:ins=3:nwc=1:stl=60:s2a=on_208");
        quick.push("ott+10_28_av=off:bd=preordered:ins=3:nwc=3:sp=reverse_arity_966");
      }
      else if (atoms > 10) {
        quick.push("lrs+10_3_av=off:fde=none:ins=3:lwlo=on:nwc=1.1:stl=60:sp=frequency_93");
        quick.push("ott+10_4_awrs=decay:awrsf=4:av=off:ins=3:nwc=1.2:sp=weighted_frequency:uhcvi=on_709");
        quick.push("ott+10_1024_awrs=converge:awrsf=512:av=off:bd=off:fde=none:ins=3:nwc=4_1341");
        quick.push("ott+10_6_av=off:bd=preordered:fde=none:ins=3:nwc=1.1:sos=on:sp=occurrence_786");
        quick.push("ott+10_12_awrs=decay:awrsf=16:av=off:bd=preordered:ins=3:nwc=1.2:sp=frequency_1980");
      }
      else {
        quick.push("ott+10_10_av=off:bd=preordered:fde=unused:ins=3:nwc=1.5_37");
        quick.push("lrs+10_1024_awrs=converge:awrsf=256:av=off:fde=unused:ins=3:lwlo=on:nwc=1.3:stl=90:sp=weighted_frequency:uhcvi=on_643");
        quick.push("lrs+10_3:1_av=off:bd=preordered:fde=unused:ins=3:lwlo=on:nwc=1:stl=150:sp=weighted_frequency_597");
        quick.push("lrs+10_4_awrs=decay:awrsf=512:av=off:bd=preordered:fde=unused:ins=3:nwc=2:stl=120:sp=frequency:uhcvi=on_1");
      }
    }
    else if (prop == 2) {
      if (atoms < 15) {
        quick.push("lrs+10_28_av=off:ins=3:nwc=1:stl=30_188");
        quick.push("ott+10_5:1_av=off:bd=preordered:fde=unused:ins=3:nwc=1_268");
        quick.push("ott+10_40_av=off:bd=preordered:fde=none:ins=3:nwc=1.2:sp=weighted_frequency:uhcvi=on_975");
        quick.push("ott+10_32_av=off:ins=3:nwc=1:sp=reverse_arity_171");
        quick.push("lrs+10_2:1_av=off:bd=off:fde=none:ins=3:lwlo=on:nwc=1:stl=90:uhcvi=on_292");
      }
      else if (atoms < 18) {
        quick.push("lrs+10_7_av=off:ins=3:lwlo=on:nwc=1.1:stl=150_80");
        quick.push("lrs+10_3_awrs=decay:awrsf=32:av=off:bd=preordered:ins=3:lwlo=on:nwc=1.2:stl=30:sp=reverse_arity_203");
        quick.push("ott+10_28_av=off:bd=preordered:fde=unused:ins=3:nwc=2.5:sp=occurrence_275");
        quick.push("ott+10_4_av=off:fde=none:ins=3:nwc=1:sos=on:sp=occurrence:uhcvi=on_122");
        quick.push("ott+10_8_av=off:bd=preordered:ins=3:nwc=1.2:sp=reverse_arity:uhcvi=on_456");
        quick.push("lrs+10_4:1_awrs=converge:awrsf=1:av=off:bd=preordered:ins=3:lwlo=on:nwc=1.3:stl=120:s2a=on:sp=weighted_frequency:uhcvi=on_363");
        quick.push("ott+10_2_av=off:bd=off:ins=3:nwc=1.2_430");
        quick.push("ott+10_4_av=off:bd=off:ins=3:nwc=1.5:sp=reverse_arity_1");
      }
      else {
        quick.push("ott+10_20_av=off:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_862");
        quick.push("ott+10_12_awrs=converge:awrsf=64:av=off:bd=preordered:fde=unused:ins=3:nwc=1.5:sp=frequency:uhcvi=on_2028");
        quick.push("lrs+10_3:2_av=off:bd=preordered:ins=3:lwlo=on:nwc=1.1:stl=90:uhcvi=on_529");
      }
    }
    else {
      quick.push("lrs+10_5:1_av=off:bd=off:ins=3:nwc=2.5:stl=60:sp=reverse_arity_138");
      quick.push("lrs+10_128_awrs=converge:awrsf=16:av=off:bd=off:fde=unused:ins=3:lwlo=on:nwc=5:stl=120:s2a=on:sp=frequency_349");
      quick.push("ott+10_28_av=off:bd=preordered:fde=unused:ins=3:nwc=2.5:sp=occurrence_36");
      quick.push("lrs+1011_8_av=off:bs=unit_only:ep=RSTC:gsp=input_only:gs=on:gsem=on:lwlo=on:nwc=1:stl=30_226");
    }
    break;
  }

  switch (cat) {
  case Property::HEQ:
  case Property::PEQ:
  case Property::NEQ:
  case Property::HNE:
  case Property::NNE:
    fallback.push("lrs+1011_5:1_afp=100000:afq=1.0:anc=none:bd=off:gsp=input_only:gs=on:gsem=off:lwlo=on:nwc=5:nicw=on:sas=z3:sac=on:updr=off_300");
    fallback.push("lrs+10_2:3_av=off:bsr=on:cond=on:lwlo=on:nwc=1.7:sp=occurrence:updr=off_600");
    fallback.push("dis+1011_5_av=off:lma=on:nwc=1.7:sos=all:sp=reverse_arity:updr=off_600");
    fallback.push("lrs+11_3:1_av=off:cond=on:fsr=off:irw=on:lma=on:nwc=10:sas=z3:updr=off_300");
    fallback.push("lrs+11_2_add=large:afp=4000:afq=1.1:anc=none:lma=on:lwlo=on:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("lrs+4_2_av=off:bs=on:er=known:gs=on:irw=on:lwlo=on:nwc=10:sp=occurrence_300");
    fallback.push("dis+1011_3_av=off:nwc=1:sos=all:sp=reverse_arity_300");
    fallback.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bce=on:fde=unused:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+11_2_afr=on:afp=100000:afq=1.4:amm=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on:updr=off_300");
    fallback.push("lrs+1011_3_av=off:bs=unit_only:cond=on:fsr=off:lcm=predicate:lma=on:lwlo=on:nwc=1:sp=occurrence_300");
    fallback.push("lrs+1011_2:1_av=off:cond=on:er=known:gs=on:gsem=on:lwlo=on:nwc=1.3:updr=off:uhcvi=on_300");
    fallback.push("ott+11_14_av=off:cond=on:nwc=1.3:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_4_aac=none:acc=model:add=large:afp=100000:afq=1.2:amm=off:anc=none:ccuc=first:cond=fast:gs=on:lma=on:nwc=4:sac=on:sp=occurrence_300");
    fallback.push("dis+2_2_add=off:afr=on:afp=4000:afq=1.0:amm=sco:anc=none:fsr=off:lcm=predicate:lma=on:nwc=1.3:nicw=on:sos=theory:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=off:anc=none:cond=on:irw=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0:sos=all:sp=occurrence_300");
    fallback.push("lrs+11_5_add=large:afr=on:afp=40000:afq=1.2:cond=on:er=known:gs=on:gsem=off:nwc=1.5:sp=occurrence:updr=off_300");
    fallback.push("lrs+1003_3_av=off:bd=off:fde=none:gs=on:lma=on:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs-1_14_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:sd=1:ss=axioms:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_6_afr=on:afp=100000:afq=1.1:anc=none:br=off:gs=on:lma=on:nwc=3:sac=on:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+4_4:1_add=large:afr=on:afp=100000:afq=1.0:anc=none:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nwc=1.5:sas=z3:sos=all:sac=on:sp=reverse_arity_300");
    fallback.push("lrs+11_3_aac=none:add=large:afp=1000:afq=1.4:anc=all_dependent:bd=off:bs=on:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=1:sos=all:sac=on:sp=occurrence:updr=off_600");
    fallback.push("ott+11_64_add=large:afp=1000:afq=2.0:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=on:nwc=2_300");
    fallback.push("lrs+1011_3:2_av=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:lcm=predicate:lwlo=on:nwc=2.5:sos=all:updr=off_300");
    fallback.push("lrs+10_3_av=off:bd=off:cond=on:gs=on:gsem=off:irw=on:lwlo=on:nwc=1.2:sp=reverse_arity:updr=off_300");
    fallback.push("lrs-11_10_afr=on:afp=1000:afq=1.0:amm=off:anc=none:gs=on:irw=on:lma=on:nwc=2.5:sac=on:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1010_4_aac=none:acc=model:add=large:afp=10000:afq=1.4:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:cond=on:fsr=off:irw=on:nwc=2:sac=on:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+10_28_add=large:afp=4000:afq=1.0:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence_300");
    fallback.push("lrs+4_1_av=off:bd=off:bsr=on:bce=on:fsr=off:irw=on:lcm=predicate:nwc=1:sos=all:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("ins+10_1_av=off:cond=on:fsr=off:gsp=input_only:igbrr=0.6:igpr=on:igrr=64/1:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:updr=off_300");
    fallback.push("dis+10_4_aac=none:afp=10000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:irw=on:lcm=reverse:lwlo=on:nwc=2.5:sas=z3:sp=reverse_arity_300");
    fallback.push("lrs+11_5_acc=on:add=large:afr=on:afp=100000:afq=1.0:anc=none:bs=unit_only:ccuc=first:cond=on:lma=on:lwlo=on:nwc=1:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1_40_av=off:lwlo=on:nwc=4:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+10_5_add=off:afr=on:afp=10000:afq=1.4:anc=none:er=known:gs=on:gsem=off:lma=on:nwc=1:nicw=on:sas=z3:sac=on:sp=reverse_arity_300");
    fallback.push("lrs+4_5_av=off:bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:nwc=2.5:sp=occurrence:updr=off_300");
    fallback.push("dis+2_3_av=off:cond=on:fsr=off:lcm=reverse:lma=on:nwc=1:sos=on:sp=reverse_arity_300");
    fallback.push("dis+1011_2:1_av=off:lma=on:nwc=1:sd=3:ss=axioms:st=5.0:sp=occurrence:urr=ec_only_300");
    fallback.push("dis+1011_8_av=off:bd=off:cond=fast:er=known:fde=unused:gsp=input_only:lcm=predicate:lma=on:nwc=1.2:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_4:1_av=off:cond=on:irw=on:lma=on:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("ott+1011_2:3_av=off:cond=fast:er=filter:fde=unused:gsp=input_only:irw=on:nwc=3:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+10_3:2_afr=on:afp=100000:afq=2.0:anc=none:bs=unit_only:bsr=on:cond=fast:fsr=off:lwlo=on:nwc=2:nicw=on:sp=reverse_arity_300");
    fallback.push("dis+11_2:1_acc=model:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:ccuc=first:er=known:gsp=input_only:irw=on:lma=on:nwc=5:sac=on:urr=ec_only_300");
    fallback.push("lrs+11_40_add=off:afp=10000:afq=1.2:amm=off:cond=on:fsr=off:lma=on:nwc=1.7:sac=on:sp=occurrence_300");
    fallback.push("lrs+10_5:4_av=off:bsr=on:gs=on:gsem=off:lma=on:nwc=4:uhcvi=on_300");
    fallback.push("lrs+1010_3:1_av=off:cond=on:nwc=5:sp=reverse_arity_300");
    fallback.push("lrs+11_3:2_add=off:afr=on:afp=4000:afq=1.4:anc=none:cond=on:lma=on:lwlo=on:nwc=3:sas=z3:sac=on:sp=reverse_arity_300");
    fallback.push("lrs+4_3_add=off:afr=on:afp=10000:afq=2.0:anc=none:lma=on:nwc=1.1:nicw=on:sas=z3:sac=on_300");
    fallback.push("ott+11_6_av=off:bs=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=10:sp=reverse_arity_300");
    fallback.push("dis+11_12_av=off:bsr=on:cond=on:lcm=predicate:lma=on:nwc=5:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_3:1_add=off:afr=on:afp=100000:afq=1.4:anc=none:cond=on:gs=on:irw=on:lma=on:lwlo=on:nwc=1:nicw=on:sas=z3:sac=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_1_av=off:bsr=on:fde=none:irw=on:lma=on:lwlo=on:nwc=1:sp=occurrence_300");
    fallback.push("ott+1011_5_afr=on:afp=1000:afq=1.4:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=unused:gsp=input_only:gs=on:nwc=1:nicw=on:sas=z3:sos=all:updr=off:uhcvi=on_600");
    fallback.push("ott+1010_14_add=large:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:bsr=on:cond=fast:lma=on:nwc=1.2:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_2_afp=1000:afq=1.4:amm=sco:anc=none:cond=on:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+11_3:2_afp=10000:afq=1.0:anc=none:bsr=on:cond=on:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:sac=on:updr=off_300");
    fallback.push("lrs+1011_40_add=off:afp=1000:afq=2.0:anc=none:bs=unit_only:fsr=off:irw=on:nwc=1:sas=z3:sos=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_12_av=off:bd=off:bs=unit_only:fsr=off:lma=on:nwc=1:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_7_av=off:irw=on:nwc=1:sos=all_300");
    fallback.push("lrs+10_12_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=off:cond=on:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=1.3:sas=z3:sac=on_300");
    fallback.push("lrs+1_3_av=off:fsr=off:lma=on:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on_300");
    fallback.push("dis+11_5_afr=on:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:irw=on:lma=on:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_2:1_av=off:bce=on:cond=fast:fsr=off:fde=none:lwlo=on:nwc=1:sd=1:ss=axioms:st=3.0:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("dis+4_7_av=off:cond=fast:gsp=input_only:lma=on:nwc=1.3:sp=occurrence:urr=ec_only:uhcvi=on_300");
    fallback.push("lrs+1011_50_av=off:bs=unit_only:cond=on:nwc=2:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_3:1_av=off:bs=unit_only:bsr=on:er=known:fsr=off:fde=unused:irw=on:lcm=reverse:lwlo=on:nwc=1.7:sos=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_1024_av=off:bd=off:fsr=off:lma=on:nwc=1:sp=occurrence:urr=on_300");
    fallback.push("lrs+10_3:1_add=off:afp=40000:afq=1.4:anc=none:br=off:fsr=off:gs=on:gsem=on:irw=on:lwlo=on:nwc=1:nicw=on:sas=z3:sos=all:urr=on_300");
    fallback.push("lrs+1_128_afp=100000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:sac=on_600");
    fallback.push("lrs+1010_3:2_av=off:fsr=off:gs=on:gsem=off:irw=on:lwlo=on:nwc=1:sos=all:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+1011_20_acc=on:add=large:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:ccuc=small_ones:cond=on:gs=on:irw=on:lwlo=on:nwc=1:sp=occurrence_300");
    fallback.push("lrs+1010_14_av=off:fsr=off:lma=on:lwlo=on:nwc=2.5:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_2:1_add=large:afr=on:afp=100000:afq=2.0:anc=none:bd=off:bsr=on:cond=on:gs=on:gsem=off:lma=on:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on_300");
    fallback.push("ott+10_4:1_av=off:bd=preordered:cond=fast:fde=unused:irw=on:lcm=reverse:nwc=3:sd=2:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+2_14_aac=none:add=off:afp=100000:afq=1.1:anc=none:nwc=3:sas=z3:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+10_5:4_add=large:afr=on:amm=sco:anc=all_dependent:bd=preordered:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1:sas=z3:sos=all:sac=on:updr=off:uhcvi=on_600");
    fallback.push("ott+1011_8:1_av=off:bd=off:cond=on:nwc=1:sos=all:sp=reverse_arity_300");
    fallback.push("dis+11_28_add=off:afr=on:afp=40000:afq=2.0:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1_20_add=large:afr=on:afp=4000:afq=1.2:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sas=z3:sd=3:ss=axioms:st=1.2:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_14_av=off:bd=off:bs=unit_only:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lcm=reverse:lwlo=on:nwc=1:sos=on:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+3_64_av=off:cond=fast:lcm=reverse:lma=on:lwlo=on:nwc=1:sos=on:updr=off_300");
    fallback.push("dis+1011_5_av=off:gs=on:gsem=on:nwc=1:sos=on:updr=off_300");
    fallback.push("dis+1003_128_add=large:afr=on:amm=off:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:sas=z3:sos=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis-11_4:1_amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nwc=10:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("lrs+11_1_av=off:fsr=off:irw=on:lma=on:lwlo=on:nwc=1:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("dis+2_10_afp=100000:afq=1.2:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nwc=1_300");
    fallback.push("lrs+10_2:3_av=off:cond=on:fsr=off:lwlo=on:nwc=2.5:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1010_1024_afr=on:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:irw=on:lwlo=on:nwc=1:sas=z3:sd=1:ss=axioms:st=3.0_300");
    fallback.push("lrs+1002_3:1_av=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nwc=4:sas=z3:sd=1:ss=axioms:st=5.0:sp=reverse_arity_300");
    fallback.push("lrs+10_5:1_add=large:afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_5:4_av=off:bd=off:fsr=off:gs=on:nwc=1.3:urr=ec_only:updr=off_300");
    fallback.push("ins+11_3_av=off:bd=off:igbrr=0.6:igrr=1/8:igrp=700:igrpq=2.0:igs=1:igwr=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:updr=off_300");
    fallback.push("lrs+10_40_aac=none:acc=on:add=off:afp=1000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:ccuc=first:cond=fast:fsr=off:fde=none:gs=on:gsem=on:lma=on:nwc=1.3:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("dis+1011_32_aac=none:afp=10000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=3:sas=z3:sp=reverse_arity_300");
    fallback.push("dis+1011_64_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:gsp=input_only:gs=on:gsem=on:lma=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_32_av=off:bd=off:br=off:gs=on:gsem=on:irw=on:nwc=1:sd=1:ss=axioms:sp=occurrence:urr=on:updr=off_300");
    fallback.push("dis+11_3_av=off:bd=off:bsr=on:bce=on:gsp=input_only:gs=on:gsem=on:lma=on:nwc=2.5:sp=reverse_arity:urr=ec_only:uhcvi=on_300");
    fallback.push("lrs+11_24_afp=100000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:irw=on:nwc=3_300");
    fallback.push("lrs+1011_2:1_afp=40000:afq=1.1:amm=off:anc=none:cond=on:ep=RST:fsr=off:gs=on:gsaa=full_model:gsem=on:nwc=1:sas=z3:sos=all:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1010_8:1_av=off:bs=unit_only:br=off:cond=on:fsr=off:irw=on:nwc=1.3:sd=3:ss=axioms:st=3.0:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1011_2_av=off:bs=unit_only:bsr=on:gs=on:gsem=on:nwc=3:updr=off_300");
    fallback.push("lrs+10_2_afr=on:afp=4000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:updr=off_300");
    fallback.push("dis-11_5:1_acc=model:add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=first:cond=on:gs=on:gsem=off:nwc=1:sd=3:ss=axioms:st=1.2:sos=all_300");
    fallback.push("lrs+10_4:1_av=off:bd=off:bs=unit_only:bsr=on:fsr=off:gs=on:gsem=on:lwlo=on:nwc=1:sos=all_300");
    fallback.push("dis+2_5:4_add=large:afp=4000:afq=1.2:anc=all:bce=on:cond=fast:fde=none:lma=on:nwc=10:sd=1:ss=axioms:st=1.5:sac=on_300");
    fallback.push("lrs+10_5_av=off:bd=off:bs=unit_only:cond=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sp=occurrence_300");
    fallback.push("dis+1011_2:3_av=off:irw=on:nwc=1.2:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1002_5:1_av=off:cond=fast:fsr=off:fde=none:lma=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:urr=on:updr=off_300");
    fallback.push("lrs+2_2_acc=model:add=off:afp=10000:afq=1.2:anc=all:bd=off:bs=on:bsr=on:ccuc=small_ones:cond=on:fsr=off:fde=unused:gs=on:lma=on:nwc=1.2:sos=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1003_3_add=off:afp=100000:afq=2.0:amm=sco:anc=none:bs=on:bsr=on:bce=on:cond=fast:fde=none:gs=on:gsaa=from_current:gsem=off:nwc=1.2:sas=z3:sac=on:sp=reverse_arity_300");
    fallback.push("dis+11_3:1_aac=none:add=off:afp=4000:afq=1.4:fsr=off:nwc=3:nicw=on:sp=occurrence_300");
    fallback.push("lrs+1011_2:1_av=off:cond=on:lwlo=on:nwc=1.5_300");
    fallback.push("lrs+11_24_afp=4000:afq=2.0:amm=sco:anc=all:br=off:cond=fast:gsp=input_only:gs=on:gsem=on:lma=on:lwlo=on:nwc=1.7:nicw=on:sos=theory:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("dis+10_5:4_afp=100000:afq=1.0:amm=sco:anc=all:cond=fast:fsr=off:gs=on:lma=on:nwc=1:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs-2_2:1_afr=on:afp=1000:afq=2.0:anc=none:bd=off:bce=on:cond=fast:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1.5:sac=on:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+1_2:1_av=off:fsr=off:lma=on:nwc=1:sd=7:ss=axioms:st=1.2:sos=on:urr=ec_only_300");
    fallback.push("dis+11_4_aac=none:add=large:afp=100000:afq=1.0:amm=off:anc=none:bsr=on:cond=on:fsr=off:gs=on:lcm=reverse:lma=on:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+1011_10_av=off:bs=unit_only:bsr=on:er=filter:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nwc=1.2:sp=reverse_arity:updr=off_300");
    fallback.push("dis-1_3_av=off:cond=on:fsr=off:gs=on:gsem=on:nwc=1:updr=off_300");
    fallback.push("lrs+11_4:1_av=off:bce=on:cond=fast:fde=none:gsp=input_only:lma=on:nwc=5:sp=occurrence_300");
    fallback.push("ott+11_7_acc=model:afr=on:afp=40000:afq=2.0:amm=off:anc=all_dependent:bs=unit_only:ccuc=small_ones:fsr=off:gs=on:gsaa=from_current:lma=on:nwc=1.7:nicw=on:sp=occurrence:uhcvi=on_300");
    fallback.push("ott+10_28_acc=model:add=off:afp=40000:afq=2.0:amm=off:bs=unit_only:bsr=on:ccuc=small_ones:cond=on:fsr=off:gs=on:gsem=on:nwc=1.1:nicw=on:urr=on:updr=off_300");
    fallback.push("lrs+1011_2_av=off:bd=preordered:bs=unit_only:cond=fast:fsr=off:fde=unused:irw=on:lma=on:lwlo=on:nwc=1.2:sp=occurrence:uhcvi=on_300");
    fallback.push("dis+11_8:1_afr=on:afp=1000:afq=1.0:amm=off:anc=none:nwc=1:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+10_3_acc=on:afr=on:afp=1000:afq=1.0:amm=sco:bs=unit_only:ccuc=first:fsr=off:irw=on:lcm=reverse:lma=on:nwc=1.5:updr=off_300");
    fallback.push("ott+11_5:1_add=off:afp=10000:afq=1.4:anc=none:bs=unit_only:bsr=on:cond=on:fsr=off:gs=on:lma=on:nwc=1_300");
    fallback.push("lrs+1004_20_av=off:cond=on:er=filter:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1:sd=3:ss=axioms:st=3.0:sos=on:urr=ec_only_300");
    fallback.push("dis+1011_8_av=off:bsr=on:cond=on:fsr=off:fde=none:gs=on:lma=on:nwc=1.1:sos=all:sp=reverse_arity_300");
    fallback.push("lrs+11_14_aac=none:afp=1000:afq=2.0:fsr=off:lma=on:nwc=1:sp=reverse_arity_300");
    fallback.push("dis+11_3_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:irw=on:lma=on:lwlo=on:nwc=1:sas=z3:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+1_2:3_av=off:fsr=off:lma=on:nwc=1:sas=z3:urr=ec_only:updr=off_300");
    fallback.push("ott+11_2:1_afp=1000:afq=1.0:bd=preordered:fsr=off:fde=none:lma=on:nwc=1:sos=all:sp=occurrence:uhcvi=on_300");
    fallback.push("ott+1_3_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bsr=on:cond=fast:fsr=off:gs=on:gsem=on:nwc=1:nicw=on:sas=z3:sos=all:sp=occurrence:uhcvi=on_300");
    fallback.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sp=occurrence_300");
    fallback.push("dis+11_2_av=off:gs=on:gsem=on:irw=on:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=reverse_arity_300");
    fallback.push("lrs+1011_8_add=large:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bs=on:gs=on:gsem=off:nwc=1.5:nicw=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis-11_64_av=off:bd=off:bs=on:cond=on:fsr=off:nwc=1:sd=1:ss=axioms:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_1024_add=off:afp=10000:afq=1.0:anc=none:bd=off:fsr=off:gs=on:gsem=on:irw=on:nwc=1.5:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("lrs-11_3_aac=none:acc=on:afr=on:afp=10000:afq=1.1:bd=off:bs=unit_only:ccuc=first:irw=on:lcm=predicate:lma=on:nwc=1.5:sos=all:sac=on:sp=occurrence:updr=off_300");
    fallback.push("dis-10_3:1_add=large:afr=on:afp=1000:afq=2.0:anc=none:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:lcm=reverse:lma=on:nwc=1:sas=z3:sd=2:ss=axioms:st=2.0:sos=all:sac=on:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("lrs+11_2:1_av=off:cond=fast:fde=none:gs=on:gsem=off:lwlo=on:nwc=2:sp=occurrence:updr=off:uhcvi=on_600");
    fallback.push("lrs+11_32_add=off:afp=10000:afq=1.1:anc=all:bs=unit_only:cond=fast:fde=none:gs=on:gsaa=from_current:irw=on:lma=on:nwc=1:nicw=on:sos=all:sac=on:sp=occurrence:updr=off:uhcvi=on_600");
    fallback.push("ott+11_1_afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:lma=on:nwc=1:sac=on:sp=occurrence:updr=off:uhcvi=on_900");
    break;

  case Property::EPR:
    fallback.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_300");
    fallback.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:uhcvi=on_300");
    fallback.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_3000");
    fallback.push("dis+4_5_afp=1000:afq=1.1:amm=off:anc=none:bd=off:gs=on:irw=on:lcm=predicate:lma=on:nwc=1:sas=z3:sos=all:sp=occurrence_300");
    fallback.push("lrs+1003_10_afp=4000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:bce=on:fde=unused:lma=on:nwc=1:nicw=on:sac=on:urr=on:updr=off:uhcvi=on_1200");
    fallback.push("dis+11_2:3_add=large:afp=10000:afq=1.2:anc=none:bd=off:bce=on:cond=fast:er=filter:fsr=off:fde=unused:gsp=input_only:nwc=5:sos=theory:sac=on:urr=on_300");
    fallback.push("ott-4_6_add=off:afr=on:afp=100000:afq=1.4:amm=sco:bs=on:fde=unused:gs=on:gsaa=full_model:gsem=on:irw=on:nwc=1:sas=z3:sac=on:updr=off:uhcvi=on_600");
    fallback.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_200");
    fallback.push("dis-11_32_av=off:bs=unit_only:gs=on:irw=on:lma=on:nwc=1:updr=off_300");
    fallback.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
    fallback.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_5:1_add=off:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:gs=on:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
    fallback.push("lrs+1_16_add=off:afp=100000:afq=1.0:amm=off:cond=fast:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=2:nicw=on:sd=7:ss=axioms:st=5.0:sos=theory:sp=reverse_arity:urr=ec_only_600");
    fallback.push("dis+1011_5_add=large:anc=none:bd=preordered:cond=on:fsr=off:fde=unused:lma=on:nwc=1:sos=theory:sp=occurrence:updr=off_1800");
    fallback.push("lrs+1_8:1_add=off:anc=none:bd=preordered:br=off:bce=on:fsr=off:fde=none:nwc=1:nicw=on:sos=theory:sp=reverse_arity:urr=on_900");
    break;

  case Property::UEQ:
    fallback.push("lrs+10_7_av=off:ins=3:lwlo=on:nwc=1.1_1500");
    fallback.push("lrs+10_4:1_av=off:bd=preordered:fde=none:ins=3:lwlo=on:nwc=3:sp=reverse_arity_600");
    fallback.push("ott+10_12_awrs=converge:awrsf=64:av=off:bd=preordered:fde=unused:ins=3:nwc=1.5:sp=frequency:uhcvi=on_2100");
    fallback.push("lrs+10_3:1_awrs=decay:awrsf=512:av=off:fde=unused:ins=3:nwc=1:s2a=on:sp=frequency:uhcvi=on_1500");
    fallback.push("ott+10_5_av=off:ins=3:nwc=1.1:sp=occurrence_2100");
    fallback.push("ott+10_8_awrs=converge:awrsf=8:av=off:bd=preordered:ins=3:nwc=1:s2a=on_2100");
    fallback.push("ott+10_4_av=off:fde=none:ins=3:nwc=1:sos=on:sp=occurrence:uhcvi=on_300");
    fallback.push("lrs+10_2:1_av=off:bd=off:fde=none:ins=3:lwlo=on:nwc=1:uhcvi=on_900");
    fallback.push("ott+10_5:1_av=off:bd=preordered:fde=unused:ins=3:nwc=1_300");
    fallback.push("ott+10_20_av=off:ins=3:nwc=1.5:sp=reverse_arity:uhcvi=on_1200");
    fallback.push("ott+10_4_awrs=decay:awrsf=4:av=off:ins=3:nwc=1.2:sp=weighted_frequency:uhcvi=on_900");
    fallback.push("lrs+10_5:1_av=off:bd=off:ins=3:nwc=2.5:sp=reverse_arity_600");
    fallback.push("ott+10_8:1_awrs=converge:awrsf=8:av=off:bd=preordered:fde=unused:ins=3:nwc=1.2:sp=weighted_frequency:uhcvi=on_2400");
    fallback.push("lrs+10_4:1_awrs=converge:awrsf=1:av=off:bd=preordered:ins=3:lwlo=on:nwc=1.3:s2a=on:sp=weighted_frequency:uhcvi=on_1200");
    fallback.push("lrs+1011_8_av=off:bs=unit_only:ep=RSTC:gsp=input_only:gs=on:gsem=on:lwlo=on:nwc=1_300");
    fallback.push("ott+10_8_av=off:bd=preordered:ins=3:nwc=1.2:sp=reverse_arity:uhcvi=on_600");
    fallback.push("lrs+10_1024_awrs=converge:awrsf=256:av=off:fde=unused:ins=3:lwlo=on:nwc=1.3:sp=weighted_frequency:uhcvi=on_900");
    fallback.push("lrs+10_3:1_av=off:bd=preordered:fde=unused:ins=3:lwlo=on:nwc=1:sp=weighted_frequency_1500");
    fallback.push("lrs+10_128_awrs=converge:awrsf=16:av=off:bd=off:fde=unused:ins=3:lwlo=on:nwc=5:s2a=on:sp=frequency_1200");
    fallback.push("ott+10_40_av=off:bd=preordered:fde=none:ins=3:nwc=1.2:sp=weighted_frequency:uhcvi=on_1200");
    fallback.push("ott+10_8:1_av=off:bd=off:ins=3:nwc=1.5:sp=occurrence_1200");
    fallback.push("ott+10_6_av=off:bd=preordered:fde=none:ins=3:nwc=1.1:sos=on:sp=occurrence_900");
    fallback.push("lrs+10_4_awrs=decay:awrsf=512:av=off:bd=preordered:fde=unused:ins=3:nwc=2:sp=frequency:uhcvi=on_1200");
    break;

  case Property::FEQ:
    fallback.push("lrs+1010_3:2_afr=on:afp=100000:afq=1.1:anc=none:gsp=input_only:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1.7:sac=on:sp=occurrence_300");
    fallback.push("lrs+1_4_afp=100000:afq=1.2:anc=none:bd=off:cond=on:gs=on:gsem=off:nm=64:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:updr=off_300");
    fallback.push("dis+1002_4_add=large:afp=40000:afq=1.0:anc=none:cond=fast:fde=none:gs=on:gsaa=full_model:lma=on:lwlo=on:nm=0:nwc=1.5:sas=z3:sp=reverse_arity:tha=off:thi=strong_300");
    fallback.push("lrs+1011_2:1_av=off:irw=on:lwlo=on:nm=16:newcnf=on:nwc=2:sd=4:ss=axioms:st=3.0:sp=occurrence_300");
    fallback.push("lrs+1002_1_av=off:fde=unused:lwlo=on:nm=16:nwc=4:sp=occurrence_300");
    fallback.push("lrs+11_3:2_add=large:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bs=unit_only:cond=on:gs=on:gsaa=full_model:gsem=off:irw=on:lwlo=on:nm=32:nwc=1.2:sas=z3:sac=on_900");
    fallback.push("lrs+1011_3:1_add=off:afr=on:afp=10000:afq=1.1:amm=off:bce=on:cond=on:ep=R:fsr=off:nm=16:nwc=1:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_8:1_afr=on:afp=1000:afq=2.0:br=off:gsp=input_only:gs=on:nm=16:nwc=1:sos=all:sp=occurrence:urr=on_300");
    fallback.push("lrs+1011_1_afp=40000:afq=1.4:bd=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=16:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("dis+1011_5:4_acc=model:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:ccuc=small_ones:cond=fast:fde=unused:gs=on:nm=2:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_5:4_av=off:cond=on:fde=unused:gs=on:gsem=on:lcm=reverse:lma=on:lwlo=on:nm=32:nwc=1.7:sd=2:ss=axioms:st=2.0:sos=all_300");
    fallback.push("lrs+2_3:1_add=large:afr=on:afp=10000:afq=1.1:amm=off:anc=none:er=known:fde=unused:gs=on:gsaa=from_current:gsem=on:lma=on:nm=32:newcnf=on:nwc=4:sas=z3:sd=1:ss=axioms:st=5.0:sac=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+1_1024_av=off:bs=on:fde=none:inw=on:irw=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_600");
    fallback.push("lrs+11_1_av=off:bsr=on:gsp=input_only:gs=on:lcm=predicate:nm=64:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+1010_2:3_add=off:afr=on:afp=10000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:tha=off_300");
    fallback.push("dis+1010_7_afr=on:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:cond=on:fsr=off:lma=on:nm=32:newcnf=on:nwc=1:urr=ec_only:updr=off_300");
    fallback.push("lrs+11_1024_afr=on:afp=40000:afq=2.0:anc=none:br=off:ep=RSTC:gs=on:nm=16:nwc=1:sp=occurrence:urr=on_300");
    fallback.push("lrs+1011_5_add=large:afp=1000:afq=1.2:anc=none:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("ott+4_32_av=off:bsr=on:cond=on:er=known:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=4:nwc=1.2:sos=theory_300");
    fallback.push("dis+11_32_av=off:ep=RST:fsr=off:lwlo=on:nm=6:nwc=1.1:sd=5:ss=axioms:st=5.0:sp=reverse_arity:uhcvi=on_1500");
    fallback.push("lrs+1002_8:1_av=off:cond=on:gsp=input_only:gs=on:irw=on:lma=on:nm=0:nwc=1.7:sd=2:ss=axioms:sos=on:sp=occurrence:urr=on_300");
    fallback.push("lrs+11_50_afp=100000:afq=1.1:amm=sco:anc=none:bs=unit_only:cond=on:irw=on:lma=on:nm=32:nwc=1.1:sp=reverse_arity_300");
    fallback.push("lrs+1011_7_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:cond=on:er=known:fsr=off:lma=on:nm=4:nwc=2.5:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1_1_aac=none:acc=model:add=large:afp=100000:afq=1.2:anc=none:bd=off:bs=on:bsr=on:ccuc=first:cond=on:fde=unused:irw=on:nm=2:newcnf=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:uhcvi=on_300");
    fallback.push("dis+1002_6_add=large:afp=40000:afq=2.0:bsr=on:cond=on:irw=on:lma=on:nm=2:nwc=2.5:nicw=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_2:3_add=off:afr=on:afp=4000:afq=1.4:anc=none:bs=unit_only:fsr=off:gs=on:gsem=on:lwlo=on:nm=16:nwc=1.3:nicw=on:sas=z3:sac=on:tha=off_300");
    fallback.push("ott+4_40_av=off:bce=on:cond=fast:fde=none:nm=0:nwc=1:sos=all:updr=off_300");
    fallback.push("dis+1002_7_acc=on:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:ccuc=first:fsr=off:gsp=input_only:gs=on:gsem=on:nm=6:nwc=1.1:nicw=on:sos=on:sac=on:sp=occurrence:urr=ec_only_300");
    fallback.push("lrs+1011_8_add=large:afp=100000:afq=1.1:er=filter:gsp=input_only:gs=on:gsem=on:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=on_300");
    fallback.push("lrs+11_8:1_av=off:bd=off:bs=unit_only:gs=on:gsem=on:lma=on:lwlo=on:nm=0:nwc=1:sd=1:ss=axioms:sos=all:urr=on:updr=off_300");
    fallback.push("ott+11_8:1_av=off:fde=unused:nm=2:newcnf=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:sp=reverse_arity:updr=off:uhcvi=on_300");
    fallback.push("ott+1002_128_av=off:bd=off:bs=on:bsr=on:cond=on:fsr=off:nm=6:newcnf=on:nwc=1:sp=reverse_arity:updr=off_300");
    fallback.push("ott+1011_4:1_add=off:afr=on:afp=10000:afq=1.0:anc=none:bd=preordered:cond=fast:nm=32:newcnf=on:nwc=1.2:sas=z3:sac=on:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott-11_3_add=large:afp=100000:afq=1.2:anc=none:bs=on:cond=fast:fde=none:gs=on:gsem=off:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:sp=occurrence:tha=off:urr=on:uhcvi=on_300");
    fallback.push("dis-11_3_add=off:afp=40000:afq=1.0:amm=sco:anc=none:gs=on:irw=on:lcm=reverse:nm=6:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on_300");
    fallback.push("lrs+10_3:1_av=off:bsr=on:cond=on:er=known:gs=on:lcm=reverse:nm=32:nwc=4:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs-2_3:2_av=off:bce=on:cond=on:gsp=input_only:gs=on:gsem=on:lcm=predicate:lma=on:newcnf=on:nwc=1:sd=2:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+1010_2:3_afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:gs=on:gsem=off:nm=16:nwc=1:nicw=on:sas=z3:sd=2:ss=axioms:st=1.5:updr=off_300");
    fallback.push("dis+1010_1_acc=model:add=large:afr=on:amm=off:anc=none:bd=off:bsr=on:ccuc=small_ones:gs=on:gsem=on:nm=16:nwc=2:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_5:4_acc=on:add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:bsr=on:ccuc=first:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:irw=on:nm=2:newcnf=on:nwc=1.2:sos=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1011_1_av=off:cond=on:gs=on:lma=on:nm=4:nwc=1:sd=3:ss=axioms:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("ott+11_20_afp=10000:afq=1.1:anc=none:bs=unit_only:bsr=on:bce=on:fsr=off:gs=on:gsem=on:nwc=2.5:sas=z3:sp=occurrence:updr=off:uhcvi=on_600");
    fallback.push("lrs+1002_2:1_acc=on:add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ccuc=first:fsr=off:gs=on:irw=on:nm=32:newcnf=on:nwc=1:sd=2:ss=axioms:sos=on:sp=reverse_arity_300");
    fallback.push("ott+11_2:1_av=off:bd=off:bsr=on:br=off:cond=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("ott+11_3_afp=1000:afq=2.0:anc=none:fsr=off:irw=on:nwc=1.7:ss=axioms:st=1.5:sac=on:updr=off_300");
    fallback.push("dis+11_3_add=off:afp=10000:afq=2.0:amm=sco:anc=none:ep=RST:gs=on:gsaa=from_current:gsem=on:inw=on:nm=64:nwc=1:sd=10:ss=axioms:st=5.0:sos=all:tha=off:updr=off:uhcvi=on_300");
    fallback.push("ott+11_8:1_av=off:bs=on:bce=on:fde=none:gsp=input_only:gs=on:gsem=on:irw=on:lcm=predicate:nm=6:nwc=1.5:sd=2:ss=axioms:st=1.2:sos=theory:urr=on:updr=off_300");
    fallback.push("ott+1010_5:4_av=off:bd=off:fde=none:irw=on:lma=on:nm=32:nwc=2.5:sd=2:ss=axioms:st=3.0:urr=on_300");
    fallback.push("dis+11_2:1_add=large:afp=4000:afq=1.4:amm=sco:anc=none:fsr=off:nm=16:nwc=1:sd=3:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_300");
    fallback.push("ins+1010_3_av=off:bd=off:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.3:igwr=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=1:ss=axioms:sos=on:sp=occurrence:updr=off_300");
    fallback.push("ott+1_8:1_av=off:bd=preordered:bsr=on:lma=on:nm=64:newcnf=on:nwc=1.2:uhcvi=on_300");
    fallback.push("dis+1_3:1_acc=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:fsr=off:gs=on:inw=on:lma=on:nm=32:nwc=1:urr=on_300");
    fallback.push("dis+10_3_av=off:irw=on:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_4:1_add=large:afp=100000:afq=1.1:anc=none:ep=RST:fde=unused:gsp=input_only:nm=6:newcnf=on:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs+10_24_add=off:afr=on:afp=1000:afq=1.4:anc=none:bs=unit_only:gs=on:gsaa=from_current:gsem=on:lma=on:nm=2:nwc=1.1:sac=on:uhcvi=on_600");
    fallback.push("lrs+1002_4:1_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:lma=on:nm=16:nwc=1:sas=z3:sd=7:ss=axioms:sos=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+4_4_av=off:fsr=off:gs=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("dis+1002_8:1_av=off:br=off:cond=on:irw=on:lma=on:nm=32:nwc=1:sp=occurrence:urr=on_300");
    fallback.push("lrs+10_12_add=off:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:lma=on:nm=64:nwc=1.3:sac=on:urr=on_300");
    fallback.push("dis+11_1_acc=on:afp=1000:afq=1.4:amm=sco:anc=all_dependent:bs=on:ccuc=small_ones:cond=fast:fde=none:gsp=input_only:nm=64:nwc=1:sac=on:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+1002_3_av=off:cond=on:fsr=off:gs=on:gsem=off:lwlo=on:nm=64:nwc=2.5:sp=reverse_arity_300");
    fallback.push("dis+1_3:2_acc=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:ccuc=small_ones:cond=on:lma=on:nm=64:nwc=1.3:sos=all:urr=on_300");
    fallback.push("dis+1010_24_aac=none:afr=on:anc=none:cond=on:fsr=off:gs=on:gsem=on:nm=6:nwc=1:sas=z3:sos=on:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+1002_1_av=off:er=filter:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=4:nwc=1:sd=3:ss=axioms:st=1.5:sos=on_300");
    fallback.push("dis-10_3:2_aac=none:afp=1000:afq=1.1:cond=on:fsr=off:lcm=reverse:lwlo=on:nm=16:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+10_5_av=off:cond=on:gs=on:gsem=off:irw=on:lcm=predicate:lma=on:lwlo=on:nm=6:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("lrs+1002_8:1_add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:bsr=on:er=known:lwlo=on:nm=0:nwc=1.2:sd=1:ss=axioms:sp=occurrence:updr=off_300");
    fallback.push("dis+10_10_add=large:afp=4000:afq=1.1:amm=sco:anc=none:irw=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:sp=reverse_arity:urr=on_300");
    fallback.push("lrs+1010_5_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:bd=off:gsp=input_only:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=4:sas=z3:sos=on:sac=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+1011_50_afr=on:afp=40000:afq=1.0:amm=off:anc=all_dependent:bs=on:bsr=on:bce=on:fde=unused:gs=on:lma=on:nm=16:nwc=1.1:sp=occurrence:updr=off_600");
    fallback.push("dis+1011_8:1_aac=none:acc=on:afp=1000:afq=1.4:amm=off:anc=all:bs=unit_only:bce=on:ccuc=small_ones:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:lma=on:nm=16:nwc=2.5:sd=4:ss=axioms:st=1.5:sos=all:uhcvi=on_300");
    fallback.push("lrs+11_5_av=off:cond=on:fsr=off:lma=on:lwlo=on:nwc=1.2:sas=z3:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1010_3:2_av=off:lma=on:nm=2:newcnf=on:nwc=1:sd=3:ss=axioms:st=5.0:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+10_8:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lcm=predicate:lwlo=on:nm=64:nwc=1:sos=all:updr=off_300");
    fallback.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("lrs+10_2:3_aac=none:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:bd=off:gs=on:gsem=on:inw=on:newcnf=on:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off_300");
    fallback.push("lrs+11_128_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:nwc=1.3:sac=on:uhcvi=on_300");
    fallback.push("lrs+2_3:1_afr=on:afp=10000:afq=1.0:amm=sco:anc=none:bs=unit_only:lcm=reverse:lma=on:nm=64:nwc=1.7:sas=z3:updr=off_300");
    fallback.push("lrs-3_4:1_afp=1000:afq=1.4:amm=sco:fde=none:gs=on:lcm=reverse:lma=on:nwc=1.5:sd=1:ss=axioms:sp=reverse_arity:urr=on:updr=off:uhcvi=on_300");
    fallback.push("dis-10_4:1_aac=none:add=off:afp=1000:afq=1.4:amm=off:anc=none:cond=fast:ep=RSTC:gs=on:gsaa=from_current:gsem=on:inw=on:lma=on:nm=64:nwc=4:sas=z3:tha=off:thi=strong:uwa=interpreted_only:updr=off:uhcvi=on_300");
    fallback.push("lrs-4_5:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bs=unit_only:bsr=on:irw=on:lcm=reverse:newcnf=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+1_5:4_aac=none:add=off:afr=on:afp=4000:afq=1.2:amm=sco:anc=none:gsp=input_only:gs=on:irw=on:nm=64:newcnf=on:nwc=1.3:nicw=on:sas=z3:sp=occurrence:tha=off_300");
    fallback.push("lrs+1011_2:3_av=off:gs=on:gsem=off:nwc=1.5:sos=theory:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("lrs+4_2_av=off:gs=on:gsem=on:lma=on:nm=16:nwc=1:sas=z3:sos=on:urr=on_300");
    fallback.push("lrs+11_4_av=off:gsp=input_only:irw=on:lma=on:nm=0:nwc=1.2:sd=2:ss=axioms:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs-11_24_av=off:bd=off:bs=on:fsr=off:gs=on:gsem=off:lma=on:nm=4:nwc=1.7:sp=occurrence:updr=off_900");
    fallback.push("lrs+10_2:3_afp=1000:afq=1.1:amm=sco:anc=none:er=known:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=2:sd=5:ss=axioms:sos=theory:sac=on:sp=occurrence_300");
    fallback.push("lrs+1010_3_av=off:fsr=off:gs=on:gsem=off:nm=2:newcnf=on:nwc=2:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1011_3:2_av=off:er=known:lma=on:nm=2:newcnf=on:nwc=2:sd=2:ss=axioms:st=3.0:urr=on:updr=off_300");
    fallback.push("lrs+4_5:4_av=off:bd=off:er=filter:lma=on:lwlo=on:nwc=1:sp=occurrence:updr=off_300");
    fallback.push("ott+11_20_afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bsr=on:irw=on:lma=on:nm=4:nwc=1.2:sac=on:sp=occurrence_300");
    fallback.push("dis+10_5:4_add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:br=off:cond=on:fsr=off:gs=on:nm=64:nwc=1:sas=z3:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+1010_8:1_add=off:afp=4000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:fsr=off:fde=unused:irw=on:lma=on:nwc=4:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1011_3_afp=1000:afq=1.1:anc=none:bd=off:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=6:nwc=4:sas=z3:sd=1:ss=axioms:st=2.0:sac=on:urr=on_300");
    fallback.push("ott+1_5:1_acc=on:add=off:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:lcm=predicate:nm=16:nwc=1.1:sd=1:ss=axioms:st=3.0:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+2_5:4_av=off:bce=on:cond=fast:ep=R:fde=none:gs=on:lcm=reverse:lwlo=on:nwc=1:sd=1:ss=axioms:sos=all:sp=occurrence_300");
    fallback.push("dis+1010_3:2_av=off:gsp=input_only:nm=2:nwc=1:sp=reverse_arity:urr=ec_only_300");
    fallback.push("lrs-1_5:4_add=off:afp=100000:afq=1.4:amm=sco:anc=all_dependent:fde=none:gs=on:irw=on:lma=on:nm=0:nwc=1:sd=2:ss=axioms:sos=all:urr=ec_only_300");
    fallback.push("dis+10_128_acc=on:add=off:afp=4000:afq=1.4:amm=off:bd=preordered:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1.2:nicw=on:sos=on:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("lrs+1004_5:4_aac=none:add=large:afp=100000:afq=1.4:anc=all_dependent:bd=off:cond=fast:gsp=input_only:gs=on:gsem=off:lma=on:lwlo=on:nm=4:newcnf=on:nwc=1:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+2_128_add=large:afp=100000:afq=1.4:amm=sco:anc=none:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sos=theory:sac=on:updr=off_600");
    fallback.push("ott+10_8:1_av=off:bd=preordered:bsr=on:cond=fast:fsr=off:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1.2:sp=reverse_arity:urr=on:updr=off:uhcvi=on_1200");
    fallback.push("dis+1002_4_add=off:afp=10000:afq=2.0:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:gsem=on:irw=on:lwlo=on:nm=2:newcnf=on:nwc=1:sos=on:sac=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("dis+10_3_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gsp=input_only:gs=on:gsem=off:lcm=reverse:nwc=1:sos=on_300");
    fallback.push("ott+1011_3:1_aac=none:add=off:afp=1000:afq=1.2:amm=sco:bd=off:bs=on:bsr=on:cond=on:gsp=input_only:gs=on:lma=on:nm=6:newcnf=on:nwc=1.3:nicw=on:sd=3:ss=axioms:st=2.0:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+10_4_add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:lcm=predicate:lma=on:nm=64:nwc=1:sd=3:ss=axioms:sos=on:sp=reverse_arity_300");
    fallback.push("lrs+1011_4:1_av=off:fsr=off:irw=on:nwc=1:sd=4:ss=axioms:st=1.5:sp=reverse_arity_300");
    fallback.push("dis+1_2:3_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:nm=64:newcnf=on:nwc=1_300");
    fallback.push("ins+11_3_av=off:irw=on:igbrr=0.1:igpr=on:igrr=1/8:igrp=1400:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:lma=on:nm=16:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sp=reverse_arity_300");
    fallback.push("lrs+1002_2:1_aac=none:afr=on:afp=1000:afq=1.2:anc=all:bd=preordered:bsr=on:cond=fast:gsp=input_only:gs=on:nm=0:nwc=2.5:nicw=on:sas=z3:sd=4:ss=axioms:st=2.0:sos=on:sac=on:urr=on:updr=off:uhcvi=on_300");
    fallback.push("dis+1004_1_aac=none:acc=on:afp=40000:afq=1.2:anc=none:cond=on:fde=unused:gs=on:gsem=off:irw=on:nm=32:nwc=2:sd=1:ss=axioms:sos=theory:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+11_3_afp=100000:afq=2.0:amm=sco:anc=none:gs=on:gsem=off:inw=on:lma=on:nm=64:nwc=1:sas=z3:sd=10:ss=axioms:st=5.0:sp=occurrence:tha=off:updr=off_300");
    fallback.push("dis+1010_2_afr=on:afp=1000:afq=1.1:amm=off:anc=none:bs=unit_only:bce=on:cond=fast:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=5:nicw=on:sas=z3:sac=on:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+1002_16_av=off:cond=on:nwc=3_300");
    fallback.push("lrs+11_3_av=off:bce=on:cond=fast:ep=R:lcm=reverse:lma=on:newcnf=on:nwc=1.3:sd=3:ss=axioms:st=1.2:sos=on:sp=occurrence:uhcvi=on_300");
    fallback.push("dis-3_5_av=off:cond=on:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1:sos=on:sp=reverse_arity_300");
    fallback.push("ott+1_8_av=off:bd=off:bs=on:cond=on:gsp=input_only:gs=on:gsem=off:irw=on:lcm=predicate:lwlo=on:nwc=1:sos=on_300");
    fallback.push("lrs+10_3_add=off:afp=40000:afq=1.4:amm=off:anc=none:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=full_model:inw=on:lma=on:nm=64:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:updr=off_300");
    fallback.push("dis+1010_4_afp=10000:afq=1.2:anc=none:irw=on:lma=on:nm=64:nwc=10:sas=z3:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis-3_4_add=off:afp=40000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:gs=on:inw=on:lma=on:nm=64:nwc=1.5:nicw=on:sas=z3:sp=reverse_arity:tha=off:thf=on:uhcvi=on_300");
    fallback.push("ott-2_28_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:irw=on:nm=16:newcnf=on:nwc=1:nicw=on:sp=occurrence:thf=on_300");
    fallback.push("dis+3_24_av=off:bd=off:bs=unit_only:fsr=off:fde=unused:gs=on:irw=on:lma=on:nm=0:nwc=1.1:sos=on:uhcvi=on_300");
    fallback.push("lrs+1011_14_av=off:fsr=off:irw=on:nwc=1:sos=on:sp=occurrence:urr=ec_only:updr=off_300");
    fallback.push("dis+1010_10_afr=on:afp=100000:afq=2.0:amm=sco:anc=none:ep=RS:fde=unused:gs=on:nwc=1:sos=on:sac=on:sp=occurrence_300");
    fallback.push("dis+11_28_av=off:fsr=off:irw=on:lcm=predicate:nm=2:newcnf=on:nwc=5:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+1002_1_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:er=known:fsr=off:gs=on:gsem=off:lma=on:nm=2:newcnf=on:nwc=2:sas=z3:sd=1:ss=axioms:st=5.0:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+11_3_av=off:cond=on:er=filter:fsr=off:gsp=input_only:gs=on:gsem=off:lcm=reverse:newcnf=on:nwc=1:sd=5:ss=axioms:st=3.0:sp=reverse_arity:urr=ec_only_300");
    fallback.push("dis+10_4_av=off:bsr=on:cond=fast:er=filter:fde=none:gsp=input_only:lcm=reverse:lma=on:nwc=4:sp=occurrence:urr=on_300");
    fallback.push("lrs+10_8:1_av=off:bs=unit_only:cond=on:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:nwc=1.2:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs+1011_5_afr=on:afp=100000:afq=1.0:amm=off:anc=none:cond=on:lma=on:nm=6:nwc=1:sas=z3:sac=on:urr=on_300");
    fallback.push("dis+11_3_afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=preordered:bce=on:fsr=off:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=6:nwc=1:sd=10:ss=axioms:st=5.0:sac=on:sp=occurrence:tha=off:urr=ec_only_300");
    fallback.push("lrs+1_3:2_aac=none:add=large:anc=all_dependent:bce=on:cond=fast:ep=RST:fsr=off:lma=on:nm=2:nwc=1:sos=on:sp=occurrence:urr=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_7_av=off:cond=fast:fde=none:gs=on:gsem=off:lcm=predicate:nm=6:nwc=1:sd=3:ss=axioms:sos=on:sp=occurrence:updr=off_300");
    fallback.push("lrs+1011_3:2_aac=none:afp=10000:afq=1.2:amm=off:anc=all:cond=on:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_300");
    fallback.push("lrs+10_5:4_afr=on:afp=40000:afq=1.2:bd=off:gsp=input_only:gs=on:inw=on:nm=0:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:thf=on:urr=on_300");
    fallback.push("ott+11_50_add=large:afr=on:afp=40000:afq=1.1:amm=sco:anc=all:bd=off:cond=fast:gs=on:gsem=on:irw=on:lma=on:nm=0:nwc=1:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+1002_8_add=large:afp=100000:afq=1.2:amm=off:bs=on:irw=on:nm=2:newcnf=on:nwc=1.1:sos=on:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+1010_2:3_afr=on:afp=40000:afq=1.4:amm=off:anc=none:lma=on:nm=16:nwc=1:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+11_6_add=large:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=fast:gs=on:gsaa=from_current:gsem=off:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:thi=strong:updr=off_300");
    fallback.push("lrs+11_2:1_av=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=16:nwc=1:sd=1:ss=axioms:st=1.2:sp=reverse_arity_300");
    fallback.push("dis+4_8:1_add=large:afp=100000:afq=1.4:ep=RST:fde=unused:gsp=input_only:lcm=predicate:nwc=1:sos=all:sp=occurrence:updr=off:uhcvi=on_300");
    fallback.push("dis+1010_3:1_av=off:gsp=input_only:nm=6:nwc=1:sos=all:sp=occurrence_300");
    fallback.push("ott+11_2:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:br=off:cond=on:irw=on:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:urr=on:updr=off_300");
    fallback.push("dis+4_2_av=off:bs=on:fsr=off:gsp=input_only:newcnf=on:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sp=reverse_arity:urr=ec_only:updr=off_300");
    fallback.push("dis+1010_4_add=off:afp=100000:afq=1.0:anc=none:fsr=off:gs=on:gsem=off:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sac=on:tha=off:thf=on_300");
    fallback.push("dis+1011_5_aac=none:add=large:afp=40000:afq=1.2:amm=off:anc=none:bd=off:fsr=off:gsp=input_only:inw=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:updr=off_300");
    fallback.push("lrs+1011_3:1_av=off:cond=on:irw=on:lma=on:nm=16:nwc=1:sos=all:updr=off_300");
    fallback.push("lrs+4_1_acc=on:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bs=on:bsr=on:ccuc=first:fsr=off:fde=unused:irw=on:lma=on:nm=0:nwc=1.3:sd=10:ss=axioms:st=3.0:sos=all:sp=occurrence:uhcvi=on_300");
    fallback.push("dis+1011_10_aac=none:add=large:afp=10000:afq=1.1:amm=off:anc=none:fsr=off:gs=on:gsem=off:irw=on:nm=6:newcnf=on:nwc=2.5:sp=reverse_arity:updr=off_300");
    fallback.push("dis+10_5_add=off:afp=4000:afq=1.1:anc=none:cond=fast:ep=RSTC:fsr=off:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:sp=reverse_arity:thi=all_300");
    fallback.push("ott-11_12_aac=none:afp=100000:afq=1.2:amm=sco:bs=on:bce=on:cond=fast:er=known:gs=on:gsaa=from_current:gsem=off:irw=on:nm=4:nwc=2:sas=z3:sos=all:sp=occurrence:urr=ec_only:updr=off_600");
    fallback.push("lrs+11_3:2_add=large:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:er=filter:irw=on:lcm=predicate:lma=on:nm=6:nwc=1:sas=z3:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+11_20_afr=on:afp=100000:afq=1.2:amm=off:anc=none:bs=unit_only:bsr=on:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=off:lma=on:lwlo=on:nm=16:newcnf=on:nwc=1:nicw=on:sas=z3:sac=on:updr=off:uhcvi=on_300");
    fallback.push("ott+10_7_av=off:bd=preordered:bs=unit_only:bsr=on:cond=fast:fsr=off:fde=unused:irw=on:lma=on:nm=32:nwc=1:sos=all:sp=reverse_arity:updr=off_300");
    fallback.push("dis+1011_3_afp=100000:afq=1.1:amm=off:anc=none:fsr=off:gsp=input_only:gs=on:irw=on:nm=6:nwc=1:sas=z3:sd=2:ss=axioms:sos=on:sac=on:sp=reverse_arity:updr=off_300");
    fallback.push("dis+10_5_av=off:bce=on:cond=on:gsp=input_only:gs=on:lcm=predicate:lma=on:nm=6:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:sp=occurrence:updr=off_300");
    fallback.push("lrs+11_24_add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:cond=fast:fde=unused:gs=on:irw=on:lma=on:nm=4:nwc=1.3:nicw=on:sas=z3:sac=on:sp=reverse_arity:uhcvi=on_300");
    fallback.push("lrs+1010_2_add=large:afp=4000:afq=2.0:amm=off:bd=off:bs=unit_only:bsr=on:br=off:fsr=off:gs=on:gsem=off:irw=on:lma=on:nm=32:nwc=1.1:sos=all:sp=reverse_arity:urr=on:updr=off_300");
    fallback.push("lrs-2_3:1_add=off:afr=on:afp=1000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bs=unit_only:bsr=on:cond=on:fde=unused:gs=on:gsem=on:irw=on:lcm=reverse:nm=32:nwc=1.7:sas=z3:sos=all:sac=on_300");
    fallback.push("lrs+11_4:1_av=off:bd=off:br=off:cond=fast:fde=unused:irw=on:lcm=reverse:lma=on:lwlo=on:nm=4:nwc=1:sd=3:ss=axioms:sos=all:urr=on_600");
    fallback.push("ott+11_128_afp=4000:afq=1.1:anc=none:bs=unit_only:cond=on:gsp=input_only:lma=on:nm=4:nwc=1.5:sos=theory:sp=occurrence:updr=off_300");
    fallback.push("lrs-1_14_add=off:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:br=off:cond=fast:fde=unused:gs=on:lcm=reverse:lma=on:nwc=1:sos=on:urr=on:updr=off_300");
    fallback.push("dis+11_3_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:cond=on:nm=16:nwc=1:ss=axioms:st=5.0:sos=all:sp=reverse_arity_300");
    fallback.push("dis+10_3:2_afr=on:afp=1000:afq=1.2:bd=off:irw=on:lcm=predicate:lwlo=on:nm=0:newcnf=on:nwc=2:sos=on:tha=off:thf=on:urr=ec_only_300");
    fallback.push("dis+1003_64_add=off:afr=on:afp=100000:afq=1.1:anc=none:cond=fast:fde=none:irw=on:nm=6:newcnf=on:nwc=1.3:uhcvi=on_300");
    fallback.push("lrs+10_128_acc=model:afp=100000:afq=2.0:anc=all_dependent:bs=on:bsr=on:cond=fast:er=filter:gs=on:gsem=off:lcm=reverse:lma=on:nm=32:nwc=3:sac=on:sp=occurrence:urr=ec_only_300");
    fallback.push("lrs+1011_2:3_add=large:afr=on:afp=100000:afq=1.2:anc=none:gs=on:gsem=off:irw=on:nm=64:nwc=1:sd=3:ss=axioms:sos=all:sp=reverse_arity_300");
    fallback.push("ott+1002_8:1_add=off:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:fsr=off:gs=on:gsem=off:nm=32:nwc=10:sas=z3:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+11_3_afr=on:afp=10000:afq=1.0:cond=fast:fsr=off:fde=none:gs=on:gsem=off:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:sd=10:ss=axioms:st=2.0:sos=all:sp=reverse_arity:urr=on:uhcvi=on_300");
    fallback.push("dis+11_3:1_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bd=off:bs=unit_only:irw=on:lcm=predicate:lma=on:nm=2:nwc=1:sd=2:ss=axioms:sos=all:sp=occurrence:updr=off_300");
    fallback.push("dis+1010_5_add=large:afp=10000:afq=1.2:amm=off:bs=unit_only:bsr=on:bce=on:cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1.3:nicw=on:sos=all:sac=on:updr=off_300");
    fallback.push("dis+2_2:1_aac=none:afp=100000:afq=1.1:amm=sco:anc=none:bsr=on:fsr=off:gs=on:gsem=on:lcm=reverse:lma=on:nm=64:nwc=1:sos=on_300");
    fallback.push("dis+11_8_add=off:afp=10000:afq=1.2:amm=off:anc=none:cond=fast:ep=R:gs=on:gsaa=from_current:gsem=on:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=4:sd=1:ss=axioms:sac=on:updr=off:uhcvi=on_300");
    fallback.push("lrs+1_8:1_av=off:cond=fast:fde=unused:lcm=predicate:nm=16:nwc=10:sp=occurrence:urr=ec_only_600");
    fallback.push("lrs+1011_1024_av=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:nm=32:nwc=1:sp=reverse_arity:uhcvi=on_900");
    fallback.push("dis+11_7_av=off:bd=preordered:bs=unit_only:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:nm=32:nwc=1:sd=4:ss=axioms:st=1.2:sos=all:urr=ec_only:updr=off_1200");
    break;

  case Property::FNE:
    fallback.push("dis+1011_10_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=64:nwc=4:sac=on:sp=occurrence_300");
    fallback.push("dis+1_3_add=large:afp=4000:afq=1.0:anc=none:gs=on:gsem=off:inw=on:lcm=reverse:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:thi=all:uwa=all:updr=off:uhcvi=on_300");
    fallback.push("dis+1_2_add=large:afr=on:afp=1000:afq=2.0:anc=none:gsp=input_only:lcm=predicate:nm=64:newcnf=on:nwc=5:sac=on:urr=ec_only:updr=off_300");
    fallback.push("dis+10_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:irw=on:nm=64:nwc=1:sas=z3:sac=on_300");
    fallback.push("dis+11_24_afp=40000:afq=1.1:amm=sco:anc=none:bs=on:gs=on:gsem=off:lcm=predicate:lma=on:nm=2:nwc=1:sos=on:sac=on:updr=off_300");
    fallback.push("lrs+1011_1024_add=large:afp=4000:afq=1.1:anc=none:br=off:fsr=off:gsp=input_only:lma=on:nwc=1:sos=on:urr=on_300");
    fallback.push("dis+10_50_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:gs=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:thf=on:updr=off_300");
    fallback.push("lrs+4_32_add=large:afp=10000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lcm=predicate:lma=on:nm=2:nwc=1:sac=on:sp=occurrence:urr=on_300");
    fallback.push("dis+11_3_afr=on:afp=4000:afq=1.4:anc=none:cond=on:fsr=off:gs=on:lcm=reverse:nm=64:nwc=1:sos=on:sp=reverse_arity_300");
    fallback.push("lrs+1011_2:1_add=large:afr=on:afp=4000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:nm=64:nwc=1:sas=z3:sos=on:sp=reverse_arity:thf=on:urr=on:updr=off_300");
    fallback.push("lrs+10_4_add=off:afp=100000:afq=2.0:amm=sco:anc=none:nm=64:nwc=1:sp=occurrence:updr=off_1500");
    fallback.push("dis+10_128_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:lma=on:nm=2:nwc=1:sp=reverse_arity_300");
    fallback.push("ott+1_40_av=off:bs=unit_only:bsr=on:br=off:fsr=off:lma=on:nm=64:newcnf=on:nwc=1.5:sp=occurrence:urr=on:updr=off_300");
    fallback.push("lrs+1_1_av=off:bsr=on:br=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:nwc=1:sp=occurrence:urr=on:updr=off_300");
    fallback.push("ott+11_5:1_afp=100000:afq=1.1:br=off:gs=on:nm=64:nwc=1:sos=all:urr=on:updr=off_300");
    fallback.push("ins+11_32_av=off:igbrr=0.4:igrr=1/64:igrpq=1.05:igwr=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:updr=off_300");
    fallback.push("lrs+1011_40_add=off:afr=on:afp=4000:afq=1.2:amm=sco:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:sos=all:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_600");
    fallback.push("ott+11_3:2_afp=40000:afq=1.0:amm=sco:bs=unit_only:cond=on:fsr=off:gs=on:gsaa=full_model:lcm=reverse:nm=32:newcnf=on:nwc=5:nicw=on:sd=3:ss=axioms:sac=on:urr=on:updr=off_1200");
    break;
  }

  // add very long final fallback strategy
  fallback.push("dis+11_1_3600");
} // getCasc2019Schedule

// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------

void Schedules::getCascSat2017Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category(); // currently unused
  unsigned long prop = property.props();
  unsigned atoms = property.atoms();

  (void)prop; (void)atoms; // to make "unused warning" go away

  switch (cat) {
  case Property::FNE:
    quick.push("fmb+10_1_av=off:bce=on:nm=6_1461");
    quick.push("dis+10_3_add=large:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nm=16:nwc=1:sac=on:updr=off_197");
    quick.push("dis+1_3_av=off:cond=on:nm=64:newcnf=on:nwc=1_87");
    break;
  case Property::FEQ:
    quick.push("fmb+10_1_av=off:bce=on:fmbsr=1.8:nm=4_1");
    quick.push("ott+11_3_aac=none:afr=on:afp=4000:afq=1.4:amm=off:anc=all:bs=unit_only:bsr=on:bce=on:fde=unused:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:uhcvi=on_31");
    quick.push("fmb+10_1_av=off:fmbsr=1.1:newcnf=on_266");
    quick.push("ott+11_3:1_afp=4000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:lma=on:nm=64:newcnf=on:nwc=1:updr=off_83");
    quick.push("ott+10_3:1_afp=1000:afq=2.0:anc=none:fde=none:gs=on:gsaa=full_model:irw=on:nm=64:nwc=1:sas=z3:sac=on:updr=off_68");
    quick.push("ott+10_128_av=off:bs=on:gsp=input_only:irw=on:lcm=predicate:lma=on:nm=0:nwc=1:sp=occurrence:urr=on:updr=off:uhcvi=on_231");
    quick.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:stl=30:sac=on:sp=occurrence_109");
    quick.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_169");
    quick.push("ott+4_5_add=large:afr=on:afp=40000:afq=1.1:amm=sco:bd=off:bs=unit_only:bsr=on:gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity_234");
    break;
  case Property::NEQ:
    quick.push("lrs+4_5_av=off:bd=off:bs=on:bsr=on:fsr=off:gs=on:gsem=off:nwc=2.5:stl=30:sp=occurrence:updr=off_2");
    quick.push("fmb+10_1_av=off:fmbsr=2.0:fde=none:updr=off_3000");
    break;
  case Property::UEQ:
    quick.push("dis+10_3_av=off:ins=3:nwc=1:sp=reverse_arity_2");
    quick.push("fmb+10_1_av=off:fmbsr=1.6_3000");
    break;
  case Property::HNE:
  case Property::HEQ:
  case Property::PEQ:
  case Property::NNE:
    quick.push("fmb+10_1_av=off:fmbsr=1.1:updr=off_7");
    quick.push("fmb+10_1_av=off:fmbsr=1.5:updr=off_43");
    quick.push("fmb+10_1_av=off:fmbsr=1.4_57");
    quick.push("dis-1_3_av=off:cond=on:fsr=off:gs=on:gsem=on:nwc=1:updr=off_2");
    quick.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:stl=30:sp=occurrence_271");
    quick.push("dis+11_5:4_add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:lwlo=on:nwc=1:sas=z3:sac=on:sp=reverse_arity_36");
    quick.push("dis+11_3_add=large:afp=1000:afq=1.4:amm=off:anc=none:gs=on:lma=on:nwc=1:sas=z3:sac=on:sp=occurrence:updr=off_218");
    quick.push("fmb+10_1_av=off:fmbsr=1.2:fde=unused:updr=off_3000");
    break;
  case Property::EPR:
    quick.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_12");
    quick.push("fmb+10_1_av=off:fmbsr=1.1:updr=off_81");
    quick.push("dis-1_64_acc=on:add=large:afp=100000:afq=1.1:anc=none:bd=preordered:ccuc=small_ones:gs=on:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_83");
    quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_57");
    quick.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_90");
    quick.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_233");
    quick.push("ott-4_6_add=off:afr=on:afp=100000:afq=1.4:amm=sco:bs=on:fde=unused:gs=on:gsaa=full_model:gsem=on:irw=on:nwc=1:sas=z3:sac=on:updr=off:uhcvi=on_334");
    quick.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_2528");
    break;
  }
  fallback.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_3000");
  fallback.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sp=occurrence_300");
  fallback.push("ott+10_3:1_afp=40000:afq=1.4:amm=off:anc=none:bs=on:irw=on:nm=64:nwc=1:sac=on:sp=reverse_arity_600");
  fallback.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("dis+11_3_afp=100000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=10:sac=on_300");
  fallback.push("ott+10_128_av=off:bs=on:gsp=input_only:irw=on:lcm=predicate:lma=on:nm=0:nwc=1:sp=occurrence:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+10_3_add=large:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nm=16:nwc=1:sac=on:updr=off_300");
  fallback.push("ins+10_1_av=off:igbrr=0.6:igrr=32/1:igrp=700:igrpq=1.2:igwr=on:nwc=1:sp=occurrence_300");
  fallback.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_300");
  fallback.push("ott-4_6_add=off:afr=on:afp=100000:afq=1.4:amm=sco:bs=on:fde=unused:gs=on:gsaa=full_model:gsem=on:irw=on:nwc=1:sas=z3:sac=on:updr=off:uhcvi=on_600");
  fallback.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_300");
  fallback.push("dis+11_3_add=large:afp=1000:afq=1.4:amm=off:anc=none:gs=on:lma=on:nwc=1:sas=z3:sac=on:sp=occurrence:updr=off_300");
  fallback.push("dis+11_5:4_add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:lwlo=on:nwc=1:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("ott+11_3:1_afp=4000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:lma=on:nm=64:newcnf=on:nwc=1:updr=off_300");
  fallback.push("ott+11_3_aac=none:afr=on:afp=4000:afq=1.4:amm=off:anc=all:bs=unit_only:bsr=on:bce=on:fde=unused:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_200");
  fallback.push("ott+4_5_add=large:afr=on:afp=40000:afq=1.1:amm=sco:bd=off:bs=unit_only:bsr=on:gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("dis-1_64_acc=on:add=large:afp=100000:afq=1.1:anc=none:bd=preordered:ccuc=small_ones:gs=on:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:sac=on:sp=occurrence_300");
  fallback.push("dis+1_3_av=off:cond=on:nm=64:newcnf=on:nwc=1_300");
  fallback.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_300");
} // getCascSat2017Schedule

// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------

void Schedules::getCascSat2018Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category(); // currently unused
  unsigned long prop = property.props();
  unsigned atoms = property.atoms();

  (void)prop; (void)atoms; // to make "unused warning" go away

  switch (cat) {
  case Property::FNE:
    quick.push("fmb+10_1_av=off:bce=on:nm=6_1461");
    quick.push("dis+10_3_add=large:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nm=16:nwc=1:sac=on:updr=off_197");
    quick.push("dis+1_3_av=off:cond=on:nm=64:newcnf=on:nwc=1_87");
    quick.push("ott+2_5_afp=4000:afq=2.0:anc=none:bce=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sp=reverse_arity_315");
    break;
  case Property::FEQ:
    quick.push("ott+11_3_aac=none:afr=on:afp=4000:afq=1.4:amm=off:anc=all:bs=unit_only:bsr=on:bce=on:fde=unused:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:uhcvi=on_31");
    quick.push("fmb+10_1_av=off:fmbsr=1.1:newcnf=on_266");
    quick.push("ott+11_3:1_afp=4000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:lma=on:nm=64:newcnf=on:nwc=1:updr=off_83");
    quick.push("ott+10_128_av=off:bs=on:gsp=input_only:irw=on:lcm=predicate:lma=on:nm=0:nwc=1:sp=occurrence:urr=on:updr=off:uhcvi=on_231");
    quick.push("ott+10_3:1_afp=1000:afq=2.0:anc=none:fde=none:gs=on:gsaa=full_model:irw=on:nm=64:nwc=1:sas=z3:sac=on:updr=off_68");
    quick.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:stl=30:sac=on:sp=occurrence_109");
    quick.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_169");
    quick.push("ott+4_5_add=large:afr=on:afp=40000:afq=1.1:amm=sco:bd=off:bs=unit_only:bsr=on:gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity_234");
    break;
  case Property::NEQ:
    quick.push("dis+11_1_afp=10000:afq=1.4:anc=none:er=filter:gs=on:gsem=off:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_2");
    quick.push("fmb+10_1_av=off:fmbsr=2.0:fde=none:updr=off_1793");
    break;
  case Property::UEQ:
    quick.push("ins+10_1_av=off:igbrr=0.6:igrr=32/1:igrp=700:igrpq=1.2:igwr=on:nwc=1:sp=occurrence_2");
    quick.push("fmb+10_1_av=off:fmbsr=1.6_1165");
    break;
  case Property::HNE:
  case Property::HEQ:
  case Property::PEQ:
  case Property::NNE:
    quick.push("fmb+10_1_av=off:fmbsr=1.1:updr=off_7");
    quick.push("fmb+10_1_av=off:fmbsr=1.5:updr=off_43");
    quick.push("dis+11_5:4_add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:lwlo=on:nwc=1:sas=z3:sac=on:sp=reverse_arity_36");
    quick.push("dis-1_3_av=off:cond=on:fsr=off:gs=on:gsem=on:nwc=1:updr=off_2");
    quick.push("dis+11_3_afp=100000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=10:sac=on_2");
    quick.push("fmb+10_1_av=off:fmbsr=1.4_57");
    quick.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:stl=30:sp=occurrence_271");
    quick.push("fmb+10_1_av=off:fmbsr=1.2:fde=unused:updr=off_1755");
    quick.push("dis+11_3_add=large:afp=1000:afq=1.4:amm=off:anc=none:gs=on:lma=on:nwc=1:sas=z3:sac=on:sp=occurrence:updr=off_218");
    break;
  case Property::EPR:
    if (prop != 0UL) {
      quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_57");
      quick.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_2");
      quick.push("lrs+1_16_add=off:afp=100000:afq=1.0:amm=off:cond=fast:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=2:nicw=on:stl=60:sd=7:ss=axioms:st=5.0:sos=theory:sp=reverse_arity:urr=ec_only_344");
      quick.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:stl=30:uhcvi=on_301");
      quick.push("dis+4_5_afp=1000:afq=1.1:amm=off:anc=none:bd=off:gs=on:irw=on:lcm=predicate:lma=on:nwc=1:sas=z3:sos=all:sp=occurrence_151");
      quick.push("dis+11_2:3_add=large:afp=10000:afq=1.2:anc=none:bd=off:bce=on:cond=fast:er=filter:fsr=off:fde=unused:gsp=input_only:nwc=5:sos=theory:sac=on:urr=on_245");
      quick.push("ins+10_1_av=off:bce=on:cond=on:fde=unused:gsp=input_only:igbrr=0.7:igrr=32/1:igrp=700:igrpq=2.0:igs=1010:igwr=on:lcm=reverse:nwc=1:ss=axioms:st=3.0:sos=theory:urr=on:uhcvi=on_139");
      quick.push("lrs+1011_4_aac=none:afp=40000:afq=1.2:anc=all:bd=off:bsr=on:br=off:cond=fast:fde=none:gs=on:lma=on:nwc=1:stl=20:sp=reverse_arity:urr=on:updr=off:uhcvi=on_1");
    }
    else if (atoms < 2000) {
      quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_54");
      quick.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_283");
      quick.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:stl=30:uhcvi=on_25");
      quick.push("lrs+1011_14_afr=on:afp=100000:afq=1.1:anc=none:bd=off:bsr=on:irw=on:nwc=1:sas=z3:stl=30_90");
      quick.push("lrs+1003_10_afp=4000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:bce=on:fde=unused:lma=on:nwc=1:nicw=on:stl=120:sac=on:urr=on:updr=off:uhcvi=on_417");
      quick.push("ins+11_128_av=off:cond=on:fsr=off:irw=on:igbrr=0.5:igpr=on:igrr=1/2:igrp=2000:igrpq=1.5:igs=1010:igwr=on:lma=on:nwc=1:sos=all:sp=occurrence:updr=off_194");
      quick.push("dis+1011_5_add=large:anc=none:bd=preordered:cond=on:fsr=off:fde=unused:lma=on:nwc=1:sos=theory:sp=occurrence:updr=off_1327");
    }
    else {
      quick.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_2528");
    }
    break;
  }
  fallback.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_3000");
  fallback.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sp=occurrence_300");
  fallback.push("ott+11_3:1_afp=4000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:lma=on:nm=64:newcnf=on:nwc=1:updr=off_300");
  fallback.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("dis+11_12_av=off:bsr=on:cond=on:lcm=predicate:lma=on:nwc=5:sp=reverse_arity:updr=off_300");
  fallback.push("ott+10_128_av=off:bs=on:gsp=input_only:irw=on:lcm=predicate:lma=on:nm=0:nwc=1:sp=occurrence:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+10_3_add=large:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nm=16:nwc=1:sac=on:updr=off_300");
  fallback.push("ins+10_1_av=off:igbrr=0.6:igrr=32/1:igrp=700:igrpq=1.2:igwr=on:nwc=1:sp=occurrence_300");
  fallback.push("ott-4_6_add=off:afr=on:afp=100000:afq=1.4:amm=sco:bs=on:fde=unused:gs=on:gsaa=full_model:gsem=on:irw=on:nwc=1:sas=z3:sac=on:updr=off:uhcvi=on_600");
  fallback.push("dis+11_3_add=large:afp=1000:afq=1.4:amm=off:anc=none:gs=on:lma=on:nwc=1:sas=z3:sac=on:sp=occurrence:updr=off_300");
  fallback.push("lrs+10_64_add=large:afp=10000:afq=1.0:amm=off:anc=none:bs=unit_only:bsr=on:fsr=off:irw=on:lma=on:lwlo=on:nm=16:nwc=1.2:sac=on_300");
  fallback.push("dis+11_5:4_add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:lwlo=on:nwc=1:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_300");
  fallback.push("ott+11_16_av=off:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.3:sas=z3:sp=reverse_arity_300");
  fallback.push("dis-1_64_acc=on:add=large:afp=100000:afq=1.1:anc=none:bd=preordered:ccuc=small_ones:gs=on:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("ott+10_3:1_afp=40000:afq=1.4:amm=off:anc=none:bs=on:irw=on:nm=64:nwc=1:sac=on:sp=reverse_arity_600");
  fallback.push("dis+1_3_av=off:cond=on:nm=64:newcnf=on:nwc=1_300");
  fallback.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_300");
  fallback.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:sac=on:sp=occurrence_300");
  fallback.push("ott+4_5_add=large:afr=on:afp=40000:afq=1.1:amm=sco:bd=off:bs=unit_only:bsr=on:gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_200");
  fallback.push("ott+11_3_aac=none:afr=on:afp=4000:afq=1.4:amm=off:anc=all:bs=unit_only:bsr=on:bce=on:fde=unused:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("dis-4_4_add=large:afr=on:afp=1000:afq=1.4:anc=all_dependent:bs=unit_only:fde=none:gs=on:gsaa=from_current:lma=on:nwc=1.2:sac=on:updr=off_600");
  fallback.push("ott+2_5_afp=4000:afq=2.0:anc=none:bce=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sp=reverse_arity_600");
  fallback.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_300");
  fallback.push("ins+10_1_av=off:igbrr=0.2:igpr=on:igrp=400:igrpq=2.0:igs=1:nwc=2.5:sos=theory_300");
  fallback.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:uhcvi=on_300");
  fallback.push("dis+4_5_afp=1000:afq=1.1:amm=off:anc=none:bd=off:gs=on:irw=on:lcm=predicate:lma=on:nwc=1:sas=z3:sos=all:sp=occurrence_300");
  fallback.push("lrs+1003_10_afp=4000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:bce=on:fde=unused:lma=on:nwc=1:nicw=on:sac=on:urr=on:updr=off:uhcvi=on_1200");
  fallback.push("dis+11_2:3_add=large:afp=10000:afq=1.2:anc=none:bd=off:bce=on:cond=fast:er=filter:fsr=off:fde=unused:gsp=input_only:nwc=5:sos=theory:sac=on:urr=on_300");
  fallback.push("ott-4_6_add=off:afr=on:afp=100000:afq=1.4:amm=sco:bs=on:fde=unused:gs=on:gsaa=full_model:gsem=on:irw=on:nwc=1:sas=z3:sac=on:updr=off:uhcvi=on_600");
  fallback.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_200");
  fallback.push("dis-11_32_av=off:bs=unit_only:gs=on:irw=on:lma=on:nwc=1:updr=off_300");
  fallback.push("lrs+1_5:1_add=off:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:cond=on:gs=on:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
  fallback.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("ins+10_1_av=off:bce=on:cond=on:fde=unused:gsp=input_only:igbrr=0.7:igrr=32/1:igrp=700:igrpq=2.0:igs=1010:igwr=on:lcm=reverse:nwc=1:ss=axioms:st=3.0:sos=theory:urr=on:uhcvi=on_300");
  fallback.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_300");
  fallback.push("dis-4_4_add=large:afr=on:afp=1000:afq=1.4:anc=all_dependent:bs=unit_only:fde=none:gs=on:gsaa=from_current:lma=on:nwc=1.2:sac=on:updr=off_600");
  fallback.push("lrs+1_16_add=off:afp=100000:afq=1.0:amm=off:cond=fast:er=filter:lcm=predicate:lma=on:lwlo=on:nwc=2:nicw=on:sd=7:ss=axioms:st=5.0:sos=theory:sp=reverse_arity:urr=ec_only_600");
  fallback.push("dis+4_2_add=large:afr=on:afp=4000:afq=1.2:anc=none:bd=preordered:bce=on:fsr=off:fde=unused:lma=on:nwc=1:sos=all:sac=on:sp=reverse_arity:updr=off_2100");
  fallback.push("lrs+1_8:1_add=off:anc=none:bd=preordered:br=off:bce=on:fsr=off:fde=none:nwc=1:nicw=on:sos=theory:sp=reverse_arity:urr=on_900");
} // getCascSat2018Schedule

void Schedules::getCascSat2019Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  Property::Category cat = property.category(); // currently unused
  unsigned long prop = property.props();
  unsigned atoms = property.atoms();

  (void)prop; (void)atoms; // to make "unused warning" go away

  switch (cat) {
  case Property::FNE:
    quick.push("ott+10_128_av=off:bs=on:gsp=input_only:irw=on:lcm=predicate:lma=on:nm=0:nwc=1:sp=occurrence:urr=on:updr=off:uhcvi=on_4");
    quick.push("fmb+10_1_av=off:bce=on:nm=6_1461");
    quick.push("dis+10_3_add=large:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nm=16:nwc=1:sac=on:updr=off_197");
    quick.push("dis+1_3_av=off:cond=on:nm=64:newcnf=on:nwc=1_87");
    quick.push("ott-3_5_awrs=decay:awrsf=128:afr=on:afp=40000:afq=1.0:amm=off:anc=none:bce=on:cond=fast:lma=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sp=frequency:updr=off_146");
    quick.push("ott+2_5_afp=4000:afq=2.0:anc=none:bce=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sp=reverse_arity_315");
    break;
  case Property::FEQ:
    quick.push("ott+11_3_aac=none:afr=on:afp=4000:afq=1.4:amm=off:anc=all:bs=unit_only:bsr=on:bce=on:fde=unused:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:uhcvi=on_31");
    quick.push("fmb+10_1_av=off:fmbsr=1.6:lma=on:nm=64:nwc=3:sp=reverse_arity:urr=on_258");
    quick.push("ott-4_4_awrs=decay:awrsf=64:add=off:afr=on:afp=40000:afq=1.0:amm=off:bs=on:gs=on:gsaa=from_current:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=frequency:updr=off_36");
    quick.push("ott+10_128_av=off:bs=on:gsp=input_only:irw=on:lcm=predicate:lma=on:nm=0:nwc=1:sp=occurrence:urr=on:updr=off:uhcvi=on_231");
    quick.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_169");
    quick.push("ott-3_5_awrs=decay:awrsf=128:afr=on:afp=40000:afq=1.0:amm=off:anc=none:bce=on:cond=fast:lma=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sp=frequency:updr=off_60");
    quick.push("ott+11_3:1_afp=4000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:lma=on:nm=64:newcnf=on:nwc=1:updr=off_83");
    quick.push("ott+10_3:1_afp=1000:afq=2.0:anc=none:fde=none:gs=on:gsaa=full_model:irw=on:nm=64:nwc=1:sas=z3:sac=on:updr=off_68");
    quick.push("dis-3_5_awrs=decay:awrsf=2:add=off:afr=on:afp=100000:afq=2.0:amm=sco:anc=none:bs=unit_only:bce=on:fsr=off:fde=none:gs=on:gsem=on:irw=on:lma=on:nm=64:nwc=1:sp=frequency:updr=off:uhcvi=on_119");
    quick.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:stl=30:sac=on:sp=occurrence_109");
    quick.push("ott+4_5_add=large:afr=on:afp=40000:afq=1.1:amm=sco:bd=off:bs=unit_only:bsr=on:gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity_234");
    break;
  case Property::PEQ:
    quick.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:stl=30:sp=occurrence_271");
    quick.push("dis+11_5:4_add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:lwlo=on:nwc=1:sas=z3:sac=on:sp=reverse_arity_36");
    quick.push("fmb+10_1_av=off:fmbsr=1.2:fde=unused:updr=off_1755");
    break;
  case Property::NEQ:
  case Property::UEQ:
  case Property::HEQ:
    quick.push("dis+11_3_afp=100000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=10:sac=on_2");
    quick.push("dis+10_3_av=off:ins=3:nwc=1:sp=reverse_arity_2");
    quick.push("fmb+10_1_av=off:fmbsr=1.5:updr=off_43");
    quick.push("fmb+10_1_av=off:fmbsr=1.6_1165");
    quick.push("fmb+10_1_av=off:fmbsr=2.0:fde=none:updr=off_1793");
    break;
  case Property::HNE:
  case Property::NNE:
    quick.push("fmb+10_1_av=off:fmbsr=1.1:updr=off_7");
    quick.push("fmb+10_1_av=off:fmbsr=1.4_57");
    quick.push("dis-1_3_av=off:cond=on:fsr=off:gs=on:gsem=on:nwc=1:updr=off_2");
    quick.push("dis+11_3_add=large:afp=1000:afq=1.4:amm=off:anc=none:gs=on:lma=on:nwc=1:sas=z3:sac=on:sp=occurrence:updr=off_218");
    break;
  case Property::EPR:
    if (atoms > 10000) {
      quick.push("lrs+2_64_add=large:afp=40000:afq=1.1:bd=off:bs=on:bsr=on:bce=on:fde=unused:irw=on:lma=on:lwlo=on:nwc=1:stl=30:uhcvi=on_50");
      quick.push("dis-4_4_add=large:afr=on:afp=1000:afq=1.4:anc=all_dependent:bs=unit_only:fde=none:gs=on:gsaa=from_current:lma=on:nwc=1.2:sac=on:updr=off_353");
      quick.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_2528");
    }
    else {
      quick.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_25");
      quick.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_12");
      quick.push("dis-1_64_acc=on:add=large:afp=100000:afq=1.1:anc=none:bd=preordered:ccuc=small_ones:gs=on:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_11");
      quick.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_57");
      quick.push("ott+1_3_awrs=converge:awrsf=16:av=off:bd=off:bs=on:bce=on:cond=fast:gs=on:gsem=off:irw=on:lma=on:nwc=1.5:sas=z3:sp=weighted_frequency:updr=off:uhcvi=on_80");
      quick.push("dis+4_7_aac=none:afr=on:anc=none:bd=preordered:bce=on:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:irw=on:lma=on:nwc=1:nicw=on:sas=z3:sac=on:sp=reverse_arity:uhcvi=on_176");
      quick.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_90");
      quick.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_233");
    }
    break;
  }
  fallback.push("fmb+10_1_av=off:bce=on:fmbes=smt:fmbsr=1.4:fde=none:ile=on:updr=off_600");
  fallback.push("fmb+10_1_av=off:bce=on:fmbsr=1.3:nm=2:newcnf=on_1200");
  fallback.push("fmb+10_1_av=off:fmbsr=1.1:updr=off_300");
  fallback.push("fmb+10_1_av=off:fmbsr=1.6_1200");
  fallback.push("fmb+10_1_av=off:fde=unused:ile=on:irw=on:lcm=predicate:lma=on:nm=16:nwc=1.7:sos=all:sp=reverse_arity_300");
  fallback.push("fmb+10_1_av=off:fmbsr=1.5:updr=off_300");
  fallback.push("lrs+11_6_aac=none:add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sp=occurrence_300");
  fallback.push("fmb+10_1_av=off:fmbsr=1.4_300");
  fallback.push("dis+11_5_add=large:afr=on:afp=1000:afq=1.0:anc=none:bsr=on:fsr=off:nm=64:newcnf=on:nwc=1:updr=off_300");
  fallback.push("ott+11_3:1_afp=4000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:lma=on:nm=64:newcnf=on:nwc=1:updr=off_300");
  fallback.push("dis+4_7_aac=none:afr=on:anc=none:bd=preordered:bce=on:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:irw=on:lma=on:nwc=1:nicw=on:sas=z3:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("fmb+10_1_av=off:fmbes=contour:fmbsr=1.6:ile=on:nm=64:updr=off_600");
  fallback.push("fmb+10_1_av=off:fmbsr=1.2:fde=unused:updr=off_1800");
  fallback.push("dis+11_1_afp=10000:afq=1.4:anc=none:er=filter:gs=on:gsem=off:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("ott+10_6_add=off:afr=on:afp=1000:afq=1.0:amm=off:bsr=on:cond=on:fsr=off:fde=none:gs=on:gsem=on:ile=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:updr=off_300");
  fallback.push("ott+2_4:1_aac=none:add=off:afp=10000:afq=1.1:amm=off:anc=none:bs=on:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity_300");
  fallback.push("ott+10_128_av=off:bs=on:gsp=input_only:irw=on:lcm=predicate:lma=on:nm=0:nwc=1:sp=occurrence:urr=on:updr=off:uhcvi=on_300");
  fallback.push("fmb+10_1_av=off:bce=on:nm=6_1500");
  fallback.push("ott+1_8:1_add=large:afp=10000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:fsr=off:fde=unused:ile=on:irw=on:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:updr=off:uhcvi=on_300");
  fallback.push("ott+11_3_afr=on:afp=10000:afq=1.4:amm=off:anc=none:bsr=on:cond=on:er=known:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on_300");
  fallback.push("dis+10_3_add=large:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:lma=on:nm=16:nwc=1:sac=on:updr=off_300");
  fallback.push("lrs+1011_64_add=off:afr=on:afp=1000:afq=1.2:amm=off:anc=all_dependent:bsr=on:bce=on:cond=on:fsr=off:gs=on:inw=on:ile=on:nm=2:newcnf=on:nwc=1.1:sas=z3:sac=on:sp=occurrence:tha=off:thi=overlap:updr=off:uhcvi=on_300");
  fallback.push("dis+11_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:bce=on:cond=on:fsr=off:ile=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:thf=on_300");
  fallback.push("dis+10_3_av=off:ins=3:nwc=1:sp=reverse_arity_300");
  fallback.push("fmb+10_1_av=off:fmbas=predicate:fmbes=contour:fmbsr=1.4:fde=unused:nm=64:newcnf=on:updr=off_600");
  fallback.push("fmb+10_1_av=off:fmbsr=2.0:fde=none:updr=off_1800");
  fallback.push("dis+11_1024_av=off:bd=off:bs=on:cond=on:gs=on:lma=on:nm=16:nwc=1:sp=occurrence:updr=off_300");
  fallback.push("dis-1_64_acc=on:add=large:afp=100000:afq=1.1:anc=none:bd=preordered:ccuc=small_ones:gs=on:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("ott+10_3:1_afp=40000:afq=1.4:amm=off:anc=none:bs=on:irw=on:nm=64:nwc=1:sac=on:sp=reverse_arity_600");
  fallback.push("ott+10_64_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:lcm=predicate:lma=on:nwc=1:sac=on:sp=reverse_arity:urr=on_300");
  fallback.push("dis+11_3_add=large:afp=1000:afq=1.4:amm=off:anc=none:gs=on:lma=on:nwc=1:sas=z3:sac=on:sp=occurrence:updr=off_300");
  fallback.push("fmb+10_1_av=off:bce=on:fmbsr=1.3:fde=none:nm=64:newcnf=on_900");
  fallback.push("dis+11_5:4_add=large:afr=on:afp=1000:afq=2.0:amm=off:anc=none:lwlo=on:nwc=1:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("ott+10_1_add=large:afp=1000:afq=1.2:amm=off:anc=none:bd=off:bs=on:fsr=off:gs=on:gsem=on:irw=on:lma=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("ott+11_14_add=large:afp=1000:afq=1.4:amm=off:anc=none:fde=unused:gs=on:gsem=on:irw=on:nm=4:newcnf=on:nwc=1:sac=on:sp=occurrence_300");
  fallback.push("dis+4_16_acc=model:add=large:afr=on:afp=40000:afq=2.0:amm=off:anc=none:bs=on:ccuc=small_ones:fsr=off:ile=on:nm=4:newcnf=on:nwc=1:nicw=on:sp=reverse_arity_300");
  fallback.push("ott+2_6_add=large:afr=on:afp=4000:afq=2.0:amm=sco:anc=all:bs=on:bce=on:cond=fast:fde=none:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:sac=on:urr=on:updr=off_300");
  fallback.push("dis+1_3_av=off:cond=on:nm=64:newcnf=on:nwc=1_300");
  fallback.push("ott+1_3_add=large:afp=10000:afq=1.4:amm=off:bd=preordered:bs=on:bsr=on:bce=on:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lma=on:nwc=1:sas=z3:sp=reverse_arity:updr=off:uhcvi=on_300");
  fallback.push("lrs-11_8_afr=on:afp=100000:afq=2.0:anc=none:bd=off:cond=on:gs=on:lma=on:nm=2:newcnf=on:nwc=3:sac=on:sp=occurrence_300");
  fallback.push("ott+4_5_add=large:afr=on:afp=40000:afq=1.1:amm=sco:bd=off:bs=unit_only:bsr=on:gs=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("ott+11_3_aac=none:afr=on:afp=4000:afq=1.4:amm=off:anc=all:bs=unit_only:bsr=on:bce=on:fde=unused:irw=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("dis+11_50_aac=none:acc=model:add=large:afr=on:afp=4000:afq=2.0:anc=none:ccuc=first:er=known:fde=unused:gsp=input_only:gs=on:gsaa=full_model:ile=on:irw=on:nm=64:newcnf=on:nwc=4:sac=on:sp=occurrence:updr=off_300");
  fallback.push("ott+11_50_aac=none:add=off:afp=1000:afq=1.4:anc=none:bs=unit_only:fde=none:gs=on:gsem=off:lma=on:nwc=1:sas=z3:sac=on:uhcvi=on_200");
  fallback.push("ott+2_5_afp=4000:afq=2.0:anc=none:bce=on:fsr=off:gsp=input_only:lma=on:nm=32:nwc=1:sp=reverse_arity_600");
  fallback.push("dis-4_4_add=large:afr=on:afp=1000:afq=1.4:anc=all_dependent:bs=unit_only:fde=none:gs=on:gsaa=from_current:lma=on:nwc=1.2:sac=on:updr=off_600");
  fallback.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=2.0:ile=on:nm=2_600");
  fallback.push("fmb+10_1_av=off:fmbsr=1.6:fde=none:updr=off_3000");
  fallback.push("fmb+10_1_av=off:bce=on:fmbes=smt:fmbsr=1.6:fde=none:ile=on:nm=64:updr=off_900");
  fallback.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=1.4:fde=unused:updr=off_900");
} // getCascSat2019Schedule

// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------

void Schedules::getSmtcomp2017Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  switch (property.getSMTLIBLogic()) {
  case SMT_AUFDTLIA:
    quick.push("lrs+1010_5:4_afp=100000:afq=1.2:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=64:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_9");
    quick.push("dis+11_5_add=large:afr=on:afp=10000:afq=1.2:anc=none:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:urr=on:updr=off_273");
    quick.push("dis+1011_2:3_add=large:afr=on:afp=40000:afq=1.0:anc=none:br=off:cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nwc=1:sos=on:sac=on:sp=occurrence:tac=axiom:tar=off:urr=on:updr=off_264");
    break;

  case SMT_UFDTLIA:
    quick.push("lrs+11_7_add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bsr=on:br=off:fde=unused:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=1:stl=30:sos=all:sac=on:urr=on:updr=off:uhcvi=on_5");
    quick.push("dis+1004_1_add=off:afr=on:afp=1000:afq=1.1:amm=off:anc=none:bd=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tac=light:tar=off:tha=off:thi=all:urr=on:uhcvi=on_6");
    quick.push("dis+1011_1_add=off:afp=1000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:sp=occurrence:urr=on_4");
    quick.push("lrs+1010_5:4_afp=100000:afq=1.2:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=64:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_88");
    quick.push("lrs+10_2:1_add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:tac=axiom:tar=off:tha=off:uwa=ground:uhcvi=on_145");
    break;

  case SMT_UFDT:
    quick.push("lrs+11_8:1_add=large:afp=100000:afq=2.0:amm=off:anc=none:bsr=on:cond=fast:gs=on:gsaa=full_model:inw=on:ile=on:lcm=predicate:nm=4:newcnf=on:nwc=1:stl=30:sp=reverse_arity:tha=off:urr=on_47");
    quick.push("dis+1003_8_afr=on:anc=none:bd=preordered:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=reverse_arity:updr=off:uhcvi=on_161");
    quick.push("dis+1_8:1_av=off:br=off:fsr=off:fde=none:gsp=input_only:ile=on:lma=on:nm=2:nwc=1:sos=on:sp=reverse_arity:urr=on:updr=off_13");
    quick.push("lrs-1_3:1_av=off:bd=off:cond=on:gs=on:ile=on:lcm=reverse:lma=on:nm=32:nwc=1.2:stl=30:urr=on:updr=off_45");
    quick.push("dis+1011_12_afp=100000:afq=1.0:amm=sco:anc=none:fsr=off:fde=unused:gsp=input_only:ile=on:irw=on:nm=64:nwc=1.2:sac=on:sp=occurrence:tac=axiom:tar=off:uhcvi=on_73");
    quick.push("ott+1011_3:1_add=off:afp=10000:afq=1.4:amm=off:anc=none:br=off:bce=on:cond=on:fde=unused:gs=on:ile=on:lma=on:nm=4:nwc=1:sp=occurrence:tar=off:urr=on:updr=off_226");
    quick.push("dis+1004_8_av=off:cond=on:er=filter:fde=unused:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity_23");
    quick.push("lrs+11_2:1_av=off:bd=off:br=off:bce=on:cond=on:fde=none:irw=on:lma=on:nm=2:newcnf=on:nwc=1.1:stl=30:sp=occurrence:urr=on:updr=off:uhcvi=on_20");
    quick.push("dis+1003_2:1_afr=on:afp=100000:afq=1.1:anc=none:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:sp=occurrence:tar=off:uhcvi=on_284");
    quick.push("lrs+10_3:2_av=off:bce=on:cond=on:er=filter:fsr=off:fde=unused:gs=on:gsem=on:ile=on:irw=on:nm=6:nwc=1:stl=30:sos=all:tac=light:tar=off:updr=off:uhcvi=on_10");
    quick.push("lrs+11_8:1_av=off:cond=on:fde=none:ile=on:nm=16:nwc=1.3:stl=30:sp=reverse_arity:urr=on:updr=off_300");
    quick.push("dis+1011_1_av=off:fsr=off:fde=unused:gsp=input_only:ile=on:irw=on:lma=on:nwc=1:sos=on:sp=reverse_arity:urr=ec_only_47");
    quick.push("dis+1010_1_av=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:sp=occurrence:tar=off:urr=on_172");
    quick.push("ott+1003_14_av=off:fsr=off:fde=unused:ile=on:lcm=predicate:nm=0:newcnf=on:nwc=1:sp=occurrence:uhcvi=on_11");
    quick.push("lrs+11_1_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:cond=on:gsp=input_only:gs=on:ile=on:irw=on:nm=6:nwc=1:stl=30:sos=all:sac=on:urr=on_299");
    quick.push("ott+1_5:1_afr=on:afp=4000:afq=1.2:amm=off:anc=none:bs=unit_only:br=off:cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:urr=on:updr=off_573");
    quick.push("lrs+1011_3_add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:tar=off_291");
    quick.push("dis-11_4:1_aac=none:add=large:afp=4000:afq=1.2:anc=none:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sp=occurrence_25");
    quick.push("lrs+10_5:1_av=off:fde=unused:ile=on:lwlo=on:nwc=1.1:stl=90:sp=occurrence:urr=on_343");
    quick.push("ott+1_8:1_add=large:afp=10000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:fsr=off:fde=unused:ile=on:irw=on:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:updr=off:uhcvi=on_99");
    quick.push("lrs+1_4_add=off:afp=100000:afq=2.0:anc=none:bsr=on:br=off:cond=on:fde=unused:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=2:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:urr=on:updr=off_122");
    quick.push("lrs-11_4:1_afp=100000:afq=1.2:amm=off:anc=all_dependent:bs=unit_only:fsr=off:fde=none:gs=on:gsem=on:ile=on:lma=on:nm=64:nwc=1:stl=30:sp=reverse_arity:updr=off:uhcvi=on_232");
    quick.push("lrs+10_8:1_acc=model:add=large:afr=on:afp=10000:afq=1.4:anc=none:bs=unit_only:ccuc=first:cond=on:er=known:gs=on:gsaa=from_current:ile=on:lcm=reverse:nm=2:nwc=1.2:stl=30:sac=on:urr=on:uhcvi=on_245");
    quick.push("lrs-11_4_acc=on:afr=on:afp=40000:afq=1.4:amm=off:anc=none:br=off:bce=on:cond=fast:fde=none:gs=on:ile=on:irw=on:nm=0:newcnf=on:nwc=1.1:nicw=on:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_197");
    quick.push("dis+1011_4_add=large:afr=on:afp=4000:afq=1.4:anc=none:cond=on:ep=RS:fsr=off:gs=on:gsaa=from_current:ile=on:lwlo=on:nm=64:nwc=1:sos=all:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_143");
    quick.push("lrs+10_3:1_av=off:cond=on:fde=none:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sp=reverse_arity:tar=off:uhcvi=on_57");
    quick.push("dis+1010_5_av=off:bsr=on:cond=fast:fde=unused:ile=on:nm=6:nwc=1:uhcvi=on_410");
    quick.push("lrs+1010_5:4_afp=100000:afq=1.2:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=64:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_94");
    quick.push("dis+1004_16_av=off:fsr=off:fde=unused:ile=on:irw=on:nm=0:newcnf=on:nwc=1.1:sp=reverse_arity:urr=on_181");
    break;

  case SMT_LIA:
    quick.push("dis+1011_8_afp=10000:afq=1.2:amm=sco:anc=none:bce=on:gs=on:gsem=off:ile=on:lma=on:nm=16:newcnf=on:nwc=2.5:sas=z3:sos=all:sac=on:sp=occurrence:updr=off_12");
    quick.push("lrs+10_20_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bs=unit_only:bce=on:fde=unused:gs=on:gsaa=full_model:gsem=on:ile=on:nm=16:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence:tha=off:thi=all:updr=off_161");
    quick.push("lrs+1002_128_aac=none:acc=model:add=large:afr=on:afp=1000:afq=1.1:anc=all_dependent:bd=preordered:ccuc=first:cond=fast:fde=unused:ile=on:irw=on:lma=on:newcnf=on:nwc=1:nicw=on:stl=30:sos=all:sac=on_15");
    break;

  case SMT_UFNIA:
    quick.push("lrs+11_5:4_av=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=32:newcnf=on:nwc=1.3:stl=30:sp=reverse_arity:updr=off_63");
    quick.push("lrs+1002_1_av=off:bd=off:bsr=on:cond=on:ile=on:lma=on:nm=64:nwc=1:stl=30:sos=on:sp=reverse_arity_18");
    quick.push("dis+1011_8_av=off:bd=off:bsr=on:bce=on:cond=on:fsr=off:fde=unused:ile=on:nm=64:nwc=1.1:sd=10:ss=axioms:st=1.5:sos=all:sp=reverse_arity:tha=off_569");
    quick.push("lrs-11_4:1_aac=none:add=off:afp=10000:afq=1.2:anc=none:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:urr=on:updr=off_16");
    quick.push("dis+1002_1_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:ile=on:lma=on:nm=4:nwc=1:tha=off:updr=off_6");
    quick.push("lrs-4_5:1_add=large:afp=1000:afq=1.0:amm=sco:anc=none:cond=on:fde=unused:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sos=all:sp=occurrence:uwa=ground:urr=on:updr=off:uhcvi=on_132");
    quick.push("lrs+4_3:1_add=off:afp=1000:afq=2.0:anc=none:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=5:sas=z3:stl=30:sac=on:sp=occurrence:updr=off_8");
    quick.push("ott+1010_7_av=off:fsr=off:fde=none:lma=on:nm=2:newcnf=on:nwc=1.3:sos=on:sp=reverse_arity:uhcvi=on_194");
    quick.push("ott+1_10_av=off:ep=RSTC:fsr=off:ile=on:lma=on:newcnf=on:nwc=1:sos=on:tha=off:updr=off_227");
    quick.push("ott+1002_5:1_add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:br=off:cond=on:fsr=off:gs=on:inw=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sos=all:tha=off:urr=on_74");
    quick.push("lrs+1011_3:2_add=off:afr=on:afp=10000:afq=1.2:amm=off:anc=all:bd=off:bsr=on:br=off:fde=unused:gs=on:inw=on:irw=on:nm=64:newcnf=on:nwc=1:stl=30:sp=reverse_arity:tha=off:uwa=interpreted_only:urr=on_6");
    quick.push("dis+10_2_add=off:amm=off:anc=none:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off:uhcvi=on_75");
    quick.push("lrs+11_5:1_afr=on:afp=10000:afq=1.2:amm=off:anc=none:fsr=off:gs=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:sp=occurrence:tha=off_150");
    quick.push("lrs+1002_1_av=off:bd=off:fsr=off:fde=none:nm=2:newcnf=on:nwc=1:stl=30:sp=reverse_arity:uhcvi=on_26");
    quick.push("ott+11_2_av=off:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=6:nwc=1.5:sp=occurrence:updr=off_158");
    quick.push("lrs+11_1_add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:fsr=off:gs=on:gsem=on:inw=on:ile=on:nm=64:newcnf=on:nwc=1:stl=30:sp=reverse_arity:tha=off:thf=on:updr=off_197");
    quick.push("dis+11_2_afp=40000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:inw=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:updr=off_12");
    quick.push("lrs+1004_5:1_av=off:cond=on:fde=none:irw=on:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=1:stl=60:sos=on:sp=reverse_arity:updr=off:uhcvi=on_215");
    quick.push("ott+1_5:1_av=off:bs=unit_only:br=off:gs=on:gsem=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=4:ss=axioms:st=1.5:sp=occurrence:tha=off:urr=on:uhcvi=on_173");
    quick.push("dis+11_5:1_av=off:br=off:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:tha=off:urr=on_45");
    quick.push("lrs+1011_3:1_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:bce=on:ep=RS:gs=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1.2:stl=30:sp=occurrence:tha=off_5");
    quick.push("dis+1003_28_acc=model:add=large:afp=10000:afq=1.1:amm=off:anc=none:bd=off:ccuc=first:fsr=off:gs=on:gsaa=from_current:ile=on:lma=on:nm=2:nwc=1:sos=all:sp=occurrence:tha=off:uwa=ground:uhcvi=on_86");
    quick.push("dis+1_4:1_acc=on:add=large:afp=4000:afq=1.2:amm=sco:anc=none:ccuc=small_ones:ile=on:lwlo=on:nm=64:nwc=1:tha=off:urr=ec_only:updr=off_228");
    quick.push("lrs+10_5:4_av=off:bd=off:fsr=off:fde=none:lcm=reverse:lma=on:newcnf=on:nwc=1:stl=30:tha=off:urr=on:updr=off_173");
    quick.push("lrs+11_8:1_add=off:afr=on:amm=off:anc=none:fsr=off:inw=on:ile=on:nm=2:newcnf=on:nwc=1.2:sas=z3:stl=30:sp=occurrence_1");
    quick.push("lrs+1011_2:3_av=off:bsr=on:cond=fast:fsr=off:gsp=input_only:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:tha=off:updr=off_87");
    quick.push("lrs+1010_1_av=off:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_128");
    quick.push("lrs+10_10_av=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:stl=30:updr=off_8");
    quick.push("lrs+1002_8_afp=10000:afq=2.0:amm=sco:anc=none:bs=on:cond=on:fsr=off:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1.3:sas=z3:stl=30:sp=reverse_arity:urr=on_37");
    quick.push("dis+11_3_av=off:cond=on:fsr=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:updr=off:uhcvi=on_117");
    quick.push("ott-1_3:1_av=off:bsr=on:br=off:bce=on:cond=on:fsr=off:fde=unused:irw=on:nm=2:newcnf=on:nwc=2.5:sos=on:sp=occurrence:tha=off:thi=overlap:urr=on:updr=off:uhcvi=on_148");
    quick.push("lrs+1003_4:1_av=off:bd=preordered:cond=on:fde=unused:gs=on:ile=on:irw=on:nm=64:nwc=1.2:stl=90:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_322");
    quick.push("lrs+2_8:1_add=off:afp=40000:afq=1.0:anc=none:fde=unused:gs=on:ile=on:irw=on:lcm=reverse:nm=64:nwc=3:sas=z3:stl=30:sp=occurrence:urr=on:uhcvi=on_13");
    quick.push("dis+1010_1_av=off:lma=on:newcnf=on:nwc=1:sd=4:ss=axioms:sos=on:sp=reverse_arity_196");
    quick.push("ott+1011_4_afp=4000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:nm=32:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off_45");
    quick.push("dis+1002_2:3_av=off:bs=on:bce=on:cond=fast:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:tha=off:thi=strong_79");
    quick.push("lrs+1002_5:4_add=large:afr=on:afp=40000:afq=2.0:anc=none:cond=on:inw=on:ile=on:nm=64:nwc=1:sas=z3:stl=30:sd=10:ss=axioms:tha=off:updr=off_109");
    quick.push("dis+2_2_afr=on:afp=100000:afq=1.2:amm=off:anc=none:bsr=on:cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=1.1:sas=z3:sac=on:tha=off:updr=off_58");
    quick.push("lrs+10_50_av=off:cond=fast:fde=none:lcm=reverse:nm=64:newcnf=on:nwc=1:stl=30:sp=occurrence:tha=off:uhcvi=on_264");
    quick.push("dis+10_3_add=large:afp=4000:afq=1.4:amm=off:anc=none:cond=on:ep=RS:gs=on:gsaa=from_current:inw=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:tha=off:updr=off_3");
    quick.push("lrs+1010_8:1_av=off:br=off:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:inw=on:ile=on:irw=on:lma=on:nm=4:nwc=5:stl=30:sos=on:tha=off:thf=on:urr=on_44");
    quick.push("dis-1_128_av=off:bs=on:fde=unused:ile=on:irw=on:nm=32:nwc=1.1:sos=all:tha=off:thi=full:uwa=one_side_constant:uhcvi=on_355");
    quick.push("lrs+4_3_av=off:bd=preordered:fde=none:inw=on:ile=on:nm=64:newcnf=on:nwc=1:stl=30:tha=off:thf=on:updr=off:uhcvi=on_8");
    quick.push("ott+11_5:4_aac=none:add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:bce=on:cond=on:fsr=off:fde=unused:ile=on:irw=on:lma=on:nm=6:newcnf=on:nwc=1:nicw=on:sas=z3:tha=off:updr=off_31");
    quick.push("dis+1010_1_afr=on:afp=40000:afq=2.0:amm=off:anc=none:gs=on:ile=on:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off_10");
    quick.push("dis+1010_2:1_afp=40000:afq=1.1:anc=none:gsp=input_only:ile=on:irw=on:nm=6:nwc=1:sac=on:tha=off:updr=off_8");
    quick.push("lrs-1_2:3_afr=on:afp=100000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sp=reverse_arity:tha=off:thf=on:urr=on_210");
    quick.push("lrs+11_2:1_afp=1000:afq=1.4:amm=off:anc=none:inw=on:ile=on:irw=on:nm=64:nwc=1:stl=30:sac=on:tha=off:urr=on_73");
    quick.push("dis+10_28_av=off:cond=on:fsr=off:gsp=input_only:gs=on:ile=on:lma=on:nm=64:nwc=1:sos=all:sp=occurrence:tha=off:thf=on:updr=off_17");
    quick.push("dis+1010_6_av=off:cond=on:er=filter:fsr=off:nm=64:newcnf=on:nwc=1.3:sp=reverse_arity_222");
    quick.push("lrs+4_5:4_av=off:bd=off:cond=on:fsr=off:irw=on:lwlo=on:nm=0:newcnf=on:nwc=1:stl=30:sos=on:sp=reverse_arity:updr=off_143");
    quick.push("dis+1002_1_av=off:bd=off:br=off:cond=on:fsr=off:fde=unused:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_8");
    quick.push("dis+1003_4:1_add=large:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=fast:fsr=off:fde=none:gs=on:ile=on:lma=on:nm=64:nwc=1.2:sas=z3:sp=reverse_arity:tha=off:urr=ec_only_19");
    quick.push("lrs+1010_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:lcm=reverse:lwlo=on:nm=64:nwc=3:nicw=on:stl=30:sp=occurrence_158");
    break;

  case SMT_ALIA:
    quick.push("lrs+2_4_add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:bd=off:bce=on:fde=none:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1.3:sas=z3:stl=30:tha=off:thi=strong:uwa=one_side_interpreted:urr=on:updr=off:uhcvi=on_3");
    quick.push("lrs-1_128_aac=none:add=off:afp=40000:afq=1.0:amm=off:anc=none:fsr=off:inw=on:ile=on:lcm=reverse:lma=on:nm=16:nwc=10:sas=z3:stl=30:sac=on:updr=off_195");
    quick.push("ott-3_2:1_add=off:afr=on:afp=100000:afq=2.0:bsr=on:fsr=off:fde=none:gs=on:ile=on:irw=on:lma=on:lwlo=on:nm=4:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:urr=ec_only_211");
    break;

  case SMT_UFLIA:
    quick.push("lrs+1002_1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:stl=30:sac=on:sp=occurrence:tha=off:updr=off_25");
    quick.push("lrs+1010_1_afp=1000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:br=off:cond=on:ile=on:irw=on:nm=2:nwc=1:nicw=on:sas=z3:stl=30:sos=all:sp=reverse_arity:tha=off:urr=on:updr=off_48");
    quick.push("lrs+11_8:1_afp=100000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:nm=64:nwc=2:nicw=on:stl=30:sac=on:sp=occurrence:urr=on:updr=off_25");
    quick.push("lrs+1011_1_add=large:afr=on:afp=1000:afq=1.4:anc=none:bd=off:cond=on:ile=on:irw=on:nm=2:nwc=1.7:stl=30:tha=off_144");
    quick.push("dis+1_2_av=off:bd=off:cond=on:fsr=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:urr=ec_only:updr=off_21");
    quick.push("dis+2_1_add=large:afr=on:afp=1000:afq=1.2:anc=none:cond=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_49");
    quick.push("ott+1011_3:1_add=off:afp=100000:afq=2.0:amm=off:anc=none:bs=unit_only:gs=on:gsem=on:irw=on:newcnf=on:nwc=1:sas=z3:tha=off_98");
    quick.push("lrs+10_2:3_afr=on:afp=1000:afq=1.1:bd=off:bce=on:cond=on:gsp=input_only:gs=on:gsaa=from_current:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sp=reverse_arity:tha=off:uwa=interpreted_only:updr=off:uhcvi=on_263");
    quick.push("ott+10_4:1_afp=100000:afq=1.1:anc=none:bd=off:inw=on:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sac=on:sp=occurrence:tha=off:urr=on:updr=off_6");
    quick.push("dis+1002_5:4_add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:updr=off_132");
    quick.push("dis+1010_2:3_add=off:afr=on:afp=10000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:tha=off_1");
    quick.push("dis+11_5:1_afr=on:afp=40000:afq=2.0:amm=sco:anc=all_dependent:cond=fast:fde=unused:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=2:nwc=1:sos=all:urr=on:uhcvi=on_7");
    quick.push("lrs+1003_3:2_afp=1000:afq=2.0:amm=off:anc=none:cond=on:gs=on:ile=on:lma=on:nm=6:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence:tha=off:thi=all:updr=off_75");
    quick.push("lrs-11_2:1_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.5:sas=z3:stl=30:sp=reverse_arity:urr=on_22");
    quick.push("ott+10_4:1_aac=none:add=off:afp=40000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_242");
    quick.push("ott+1010_1_add=large:afp=1000:afq=1.2:anc=none:bd=off:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:updr=off_260");
    quick.push("lrs+11_2:1_add=off:anc=none:bsr=on:br=off:cond=on:er=filter:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=2:nwc=1:sas=z3:stl=30:sos=all:sac=on:uwa=ground:urr=on_27");
    quick.push("dis+10_3_add=off:afp=100000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sp=reverse_arity:tha=off:thf=on:updr=off_59");
    quick.push("dis+10_14_add=large:afp=4000:afq=1.1:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=none:inw=on:irw=on:lcm=predicate:nm=4:nwc=1.1:sos=on:sac=on:updr=off:uhcvi=on_268");
    quick.push("dis+1011_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=4:nwc=1:sac=on:tha=off:updr=off:uhcvi=on_278");
    quick.push("dis+1011_1_afp=40000:afq=1.2:anc=none:cond=on:gsp=input_only:ile=on:irw=on:lma=on:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_249");
    quick.push("dis-4_7_acc=on:afp=40000:afq=1.4:anc=all_dependent:bsr=on:br=off:bce=on:ccuc=first:er=filter:fsr=off:fde=unused:gsp=input_only:ile=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:thi=full:uwa=ground:urr=on:updr=off:uhcvi=on_12");
    quick.push("dis+1002_1_add=large:afp=4000:afq=1.2:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only:uhcvi=on_33");
    quick.push("lrs+11_5:1_add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:er=known:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=predicate:lwlo=on:nm=64:newcnf=on:nwc=1.1:stl=30:sac=on:sp=reverse_arity:tha=off:thf=on_255");
    quick.push("dis+1011_8:1_add=off:afp=10000:afq=1.1:anc=none:bce=on:er=filter:gs=on:gsaa=from_current:gsem=off:inw=on:ile=on:lma=on:nm=2:nwc=3:sac=on:urr=on:updr=off_5");
    quick.push("lrs+1011_5:4_av=off:bd=off:bs=on:cond=on:er=known:gs=on:gsem=on:inw=on:ile=on:lcm=reverse:nm=6:newcnf=on:nwc=1:stl=30:sp=occurrence:tha=off:uhcvi=on_74");
    quick.push("lrs+1_32_aac=none:add=off:afr=on:afp=100000:afq=1.1:anc=none:bd=off:gsp=input_only:gs=on:lcm=predicate:nm=16:nwc=3:stl=30:sac=on:sp=occurrence:urr=on_1");
    quick.push("lrs+1011_2:1_afr=on:afp=10000:afq=2.0:amm=off:gsp=input_only:gs=on:inw=on:ile=on:nm=2:nwc=1:sas=z3:stl=30:tha=off_296");
    quick.push("lrs+1002_2:1_aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=none:cond=fast:gs=on:nm=64:newcnf=on:nwc=1.5:sas=z3:stl=30:sp=occurrence:updr=off_85");
    quick.push("ott+1002_2:1_add=large:afr=on:afp=100000:afq=1.1:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off_90");
    quick.push("dis+2_4_afp=10000:afq=1.1:bd=off:bs=on:cond=on:er=filter:ile=on:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_107");
    quick.push("dis+10_32_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:fde=none:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sp=occurrence:tha=off:thi=full:uhcvi=on_202");
    quick.push("ott+1011_2:3_av=off:bs=unit_only:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thi=all:uwa=all:urr=on:uhcvi=on_235");
    quick.push("lrs-2_5:1_acc=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bd=off:cond=fast:gs=on:ile=on:nm=0:newcnf=on:nwc=3:stl=30:sac=on:thf=on:urr=ec_only_296");
    quick.push("dis+1010_3_afp=10000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:tha=off:urr=on_46");
    quick.push("lrs+2_3:2_av=off:cond=fast:inw=on:ile=on:nm=2:nwc=1:stl=30:sos=theory:urr=on_24");
    quick.push("lrs+1002_2_add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:inw=on:lwlo=on:nm=32:newcnf=on:nwc=1:stl=30:sos=theory:sac=on:sp=occurrence:urr=on_84");
    quick.push("lrs+2_4_add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:cond=on:ep=R:gs=on:gsaa=from_current:gsem=off:ile=on:lcm=reverse:lma=on:nm=2:nwc=1.1:stl=30:sos=on:sac=on:tha=off:updr=off_120");
    quick.push("dis+1011_1_aac=none:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bce=on:cond=on:er=known:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=off:ile=on:newcnf=on:nwc=1:sas=z3:ss=axioms:st=2.0:sp=occurrence:tha=off:urr=ec_only_231");
    quick.push("dis+1011_3_aac=none:afr=on:afp=40000:afq=1.4:amm=off:bs=on:fsr=off:fde=none:gsp=input_only:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off_110");
    quick.push("dis+1011_10_av=off:bd=off:cond=fast:er=known:inw=on:ile=on:irw=on:lma=on:nwc=1.7:sp=occurrence:tha=off:uhcvi=on_192");
    quick.push("lrs+1011_3_add=off:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:bce=on:cond=fast:fde=unused:ile=on:lma=on:nm=6:nwc=2:nicw=on:sas=z3:stl=30:sd=3:ss=axioms:st=2.0:sp=reverse_arity:tha=off_261");
    quick.push("lrs+4_28_afp=10000:afq=1.2:amm=sco:anc=none:bd=off:bce=on:cond=on:fsr=off:ile=on:irw=on:lcm=reverse:nm=16:newcnf=on:nwc=2:sas=z3:stl=60:sp=occurrence:tha=off:updr=off:uhcvi=on_350");
    quick.push("lrs+10_3:1_afp=1000:afq=1.4:amm=off:anc=none:bsr=on:inw=on:ile=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:tha=off:urr=on:updr=off_291");
    quick.push("lrs+1002_1_add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:fsr=off:gsp=input_only:inw=on:ile=on:lcm=predicate:lwlo=on:nm=64:nwc=1.7:sas=z3:stl=30:sac=on:sp=reverse_arity:tha=off:thf=on_81");
    quick.push("lrs+1011_3:2_add=large:afp=10000:afq=1.4:amm=sco:anc=none:cond=fast:fde=unused:gsp=input_only:gs=on:ile=on:irw=on:lma=on:nwc=1:stl=30:sac=on:tha=off:updr=off:uhcvi=on_118");
    quick.push("dis+1_2_afp=10000:afq=1.1:amm=off:anc=none:bd=off:gsp=input_only:gs=on:gsem=off:ile=on:lcm=reverse:nm=32:newcnf=on:nwc=1:sas=z3:tha=off:urr=ec_only:updr=off_57");
    quick.push("dis+11_7_add=large:afr=on:afp=10000:afq=1.2:bd=off:bsr=on:cond=on:fsr=off:fde=unused:gs=on:ile=on:lcm=predicate:lma=on:nm=2:newcnf=on:nwc=3:sos=on:sp=reverse_arity:tha=off:updr=off_38");
    quick.push("lrs+1011_2:1_av=off:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sp=occurrence:tha=off:urr=ec_only:uhcvi=on_79");
    quick.push("ott+1002_3:1_av=off:bsr=on:ep=RS:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:updr=off:uhcvi=on_160");
    quick.push("dis-3_7_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sp=occurrence:tha=off:thi=overlap:uwa=interpreted_only:uhcvi=on_128");
    quick.push("ott+1010_2:1_acc=on:add=large:afr=on:afp=40000:afq=1.1:anc=none:gs=on:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=on_5");
    quick.push("lrs+2_8_av=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1.2:stl=30:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_171");
    quick.push("ott-1_1_acc=model:add=off:afr=on:afp=4000:afq=1.2:anc=all:bd=preordered:bs=unit_only:bsr=on:ccuc=first:gs=on:gsaa=from_current:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=occurrence:tha=off:thi=strong:updr=off_80");
    quick.push("dis-11_5:4_add=large:afp=40000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:fde=unused:gsp=input_only:ile=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:urr=ec_only:uhcvi=on_72");
    quick.push("dis+10_2:1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=on:inw=on:ile=on:irw=on:nm=2:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_75");
    quick.push("lrs-10_3:2_aac=none:add=off:afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=16:nwc=1:nicw=on:sas=z3:stl=60:sd=2:ss=axioms:sos=on:sp=occurrence:updr=off_83");
    quick.push("ott+1011_8:1_afr=on:afp=1000:afq=1.4:amm=sco:anc=none:bd=off:fsr=off:fde=unused:inw=on:ile=on:nm=2:nwc=1:nicw=on:sas=z3:sos=theory:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_280");
    quick.push("lrs+11_2_av=off:cond=on:fsr=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sp=reverse_arity:tha=off:thf=on_66");
    quick.push("ott+10_2:1_av=off:bd=off:br=off:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on:updr=off:uhcvi=on_194");
    quick.push("dis+1002_14_av=off:cond=fast:fde=unused:inw=on:ile=on:lma=on:nm=0:nwc=1:sos=all:sp=reverse_arity:tha=off:uwa=one_side_interpreted:uhcvi=on_22");
    quick.push("dis+11_3:1_av=off:br=off:ep=R:fsr=off:gsp=input_only:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:urr=on:uhcvi=on_16");
    quick.push("dis+11_5_add=off:afp=1000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:sp=occurrence:tha=off:updr=off_1");
    quick.push("dis+10_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:irw=on:nm=64:nwc=1:sas=z3:sac=on_1");
    quick.push("lrs-2_3:1_add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=on:er=filter:fde=unused:ile=on:irw=on:nm=64:newcnf=on:nwc=1.1:sas=z3:stl=60:sac=on:sp=reverse_arity:tha=off:thf=on:thi=strong:uhcvi=on_41");
    quick.push("lrs-10_5:4_add=off:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.1:stl=30:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence_28");
    quick.push("lrs+1011_2:1_aac=none:add=off:afp=1000:afq=1.0:amm=off:bs=on:gs=on:gsaa=from_current:gsem=on:ile=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sp=reverse_arity:tha=off_200");
    quick.push("ott-4_5:4_aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:cond=fast:ile=on:irw=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=3:thf=on:urr=on:updr=off:uhcvi=on_170");
    quick.push("lrs+10_24_afp=4000:afq=2.0:bd=off:bsr=on:bce=on:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:nwc=1.3:stl=30:sp=occurrence:tha=off:uwa=one_side_constant:urr=ec_only_282");
    quick.push("dis+1011_5:1_afp=4000:afq=1.4:amm=off:anc=none:cond=on:fde=unused:gsp=input_only:ile=on:lma=on:nm=16:nwc=1:sos=on:sac=on:tha=off:urr=ec_only:uhcvi=on_248");
    quick.push("dis+1002_2_aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=all:bsr=on:fde=unused:inw=on:ile=on:lcm=reverse:nm=4:nwc=4:nicw=on:sos=theory:sac=on:sp=reverse_arity:uhcvi=on_85");
    quick.push("lrs+1_5:1_add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1.2:stl=30:sp=reverse_arity_269");
    quick.push("lrs+1010_1_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:fsr=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sos=all:urr=on_126");
    quick.push("dis+1010_2_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:fsr=off:fde=none:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:sp=reverse_arity_193");
    quick.push("lrs+10_3:1_add=large:afp=10000:afq=1.1:amm=off:anc=none:cond=on:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sd=5:ss=axioms:st=1.5:tha=off:urr=on_183");
    quick.push("dis+1011_2_acc=on:afp=10000:afq=1.1:amm=sco:anc=none:ccuc=small_ones:cond=fast:fde=unused:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off:updr=off:uhcvi=on_266");
    quick.push("lrs+1010_3:1_av=off:bd=off:bsr=on:irw=on:nm=64:newcnf=on:nwc=1.7:stl=30:sos=all:updr=off_18");
    quick.push("lrs+11_10_add=off:afp=40000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bce=on:cond=fast:gsp=input_only:inw=on:lma=on:nm=4:newcnf=on:nwc=1:stl=30:sp=occurrence:tha=off:thf=on:urr=ec_only:uhcvi=on_44");
    quick.push("ott+1010_2:1_add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:cond=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=1.5:sac=on:tha=off:updr=off_193");
    quick.push("dis+11_1_av=off:br=off:cond=on:gsp=input_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:urr=on_82");
    quick.push("lrs+4_8:1_av=off:cond=on:gs=on:gsem=on:irw=on:nm=64:newcnf=on:nwc=1.1:stl=30:sp=occurrence:tha=off:urr=on:updr=off_63");
    quick.push("dis+1010_1_add=off:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=6:nwc=1.3:nicw=on:sas=z3:tha=off:urr=ec_only_281");
    quick.push("lrs+1011_2:1_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=preordered:ccuc=first:cond=fast:fsr=off:fde=unused:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=2:nicw=on:stl=30:sp=occurrence:urr=ec_only:updr=off_190");
    quick.push("ott+1011_3:2_av=off:bd=off:bs=on:bce=on:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:tha=off:updr=off_221");
    quick.push("lrs+10_24_av=off:bs=unit_only:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=60:sd=7:ss=axioms:st=1.2:sp=occurrence:tha=off:thf=on:uhcvi=on_343");
    quick.push("lrs-11_8:1_afr=on:afp=1000:afq=1.4:amm=off:anc=none:bd=off:bs=on:gs=on:ile=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only_56");
    quick.push("ott+1011_2:3_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:bce=on:cond=fast:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1.1:sp=reverse_arity:tha=off:urr=on:updr=off_148");
    quick.push("lrs+1011_8:1_av=off:bs=on:cond=on:fsr=off:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on_117");
    quick.push("dis+1011_4_afr=on:afp=10000:afq=1.1:amm=off:anc=none:ep=RS:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:updr=off:uhcvi=on_55");
    quick.push("dis+1011_5:1_afr=on:afp=10000:afq=1.2:amm=sco:bd=preordered:bs=unit_only:cond=on:fsr=off:inw=on:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=1.1:sd=7:ss=axioms:st=1.2:tha=off:uhcvi=on_267");
    quick.push("dis+4_4:1_add=off:afp=4000:afq=1.2:amm=sco:anc=none:br=off:cond=fast:ep=RS:fsr=off:inw=on:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on:urr=on:uhcvi=on_29");
    quick.push("dis+1002_10_afp=4000:afq=1.4:amm=sco:bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:nm=6:newcnf=on:nwc=1:nicw=on:sos=all:sp=occurrence:urr=ec_only_31");
    quick.push("lrs+11_6_aac=none:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:cond=fast:ile=on:nm=16:newcnf=on:nwc=1:sas=z3:stl=30:sp=reverse_arity:tha=off:uhcvi=on_165");
    quick.push("lrs+1004_1_aac=none:add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=all_dependent:bd=off:cond=fast:fsr=off:gs=on:gsaa=from_current:lcm=reverse:nm=0:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence:tha=off:thf=on:urr=on:updr=off_16");
    quick.push("ott+1003_12_add=large:anc=all:bd=preordered:bce=on:fde=none:lcm=reverse:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:uwa=ground_293");
    quick.push("lrs+10_3_av=off:fde=unused:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1.7:stl=60:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_277");
    quick.push("lrs+4_8:1_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:stl=30:sos=all:tha=off_6");
    quick.push("lrs+2_1024_av=off:bd=off:bsr=on:cond=fast:fsr=off:fde=none:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:stl=30:tha=off:thi=overlap:uwa=one_side_constant:updr=off:uhcvi=on_195");
    quick.push("lrs-10_3_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=unused:gs=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.2:sas=z3:stl=30:tha=off:urr=ec_only_42");
    quick.push("lrs+4_4_av=off:bd=off:bs=unit_only:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sp=occurrence:tha=off:thf=on:updr=off_234");
    quick.push("ott-11_3_add=off:afr=on:afp=100000:afq=1.0:anc=none:bs=unit_only:bsr=on:cond=on:gs=on:inw=on:ile=on:nm=2:nwc=1:sas=z3:sos=on:tha=off_35");
    quick.push("dis+1010_2_acc=on:afr=on:afp=100000:afq=1.2:anc=none:bsr=on:fsr=off:ile=on:irw=on:nm=16:newcnf=on:nwc=4:sp=occurrence:tha=off:urr=ec_only_202");
    quick.push("lrs+1011_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:stl=30:sp=reverse_arity:tha=off:urr=on:updr=off_7");
    quick.push("dis+11_32_add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:er=filter:ile=on:lcm=predicate:lma=on:newcnf=on:nwc=5:sp=occurrence:updr=off_286");
    quick.push("lrs-11_4:1_add=large:afp=1000:afq=1.1:amm=sco:bs=on:cond=on:gs=on:gsem=on:ile=on:nm=2:newcnf=on:nwc=1:sas=z3:stl=30:sos=on:sp=occurrence:updr=off_128");
    break;

  case SMT_UFIDL:
    quick.push("dis+10_3_afr=on:afp=1000:afq=1.0:anc=none:cond=on:fsr=off:gs=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:sp=occurrence:urr=on_3");
    quick.push("dis+11_3_add=large:afp=100000:afq=1.4:amm=off:anc=none:fsr=off:gs=on:ile=on:irw=on:lma=on:nm=32:nwc=1:tha=off:updr=off_4");
    break;

  case SMT_LRA:
    quick.push("ott-4_1_add=off:afp=40000:afq=1.0:anc=none:bce=on:cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lcm=reverse:lma=on:nm=6:newcnf=on:nwc=1.5:sas=z3:sos=all:sp=occurrence:updr=off:uhcvi=on_2");
    quick.push("dis+1_5:1_add=off:afp=40000:afq=1.2:anc=none:bd=off:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=4:sas=z3:updr=off_59");
    quick.push("dis+4_5_av=off:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lwlo=on:nm=6:nwc=1:sos=on:sp=reverse_arity:updr=off_5");
    quick.push("dis+1002_1_afp=1000:afq=1.0:amm=sco:anc=none:fsr=off:gs=on:gsem=on:ile=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:updr=off_8");
    quick.push("ott+11_4_av=off:ile=on:lma=on:nm=64:nwc=1:sos=all:sp=occurrence:uwa=interpreted_only:updr=off:uhcvi=on_37");
    quick.push("dis+11_2_add=large:afr=on:afp=1000:afq=1.1:anc=none:gs=on:gsaa=full_model:ile=on:irw=on:lma=on:nm=16:nwc=1:sas=z3:sos=on:sac=on:sp=occurrence:thi=strong:uhcvi=on_72");
    quick.push("dis+10_50_add=large:afr=on:afp=4000:afq=1.0:amm=off:anc=none:cond=on:fsr=off:gs=on:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:thf=on:updr=off_1");
    quick.push("dis+1011_5_add=off:afp=4000:afq=2.0:anc=none:fsr=off:inw=on:lwlo=on:nm=64:nwc=1:sos=all:sac=on:sp=reverse_arity_1");
    break;

  case SMT_NIA:
    quick.push("dis+11_10_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=10:sas=z3:sac=on:sp=reverse_arity_2");
    break;

  case SMT_UFLRA:
    quick.push("dis+11_4_afp=4000:afq=1.4:amm=sco:anc=none:gs=on:ile=on:lma=on:nm=64:nwc=1.7:sas=z3:sac=on:sp=occurrence_2");
    break;

  case SMT_NRA:
    quick.push("dis+11_4_add=off:afp=1000:afq=2.0:amm=sco:anc=none:fsr=off:gs=on:gsem=off:ile=on:nm=64:nwc=1.7:sas=z3:urr=on_2");
    quick.push("dis+11_5_av=off:cond=on:fsr=off:ile=on:lwlo=on:nm=64:nwc=3:sp=reverse_arity:updr=off_2");
    quick.push("dis+1011_4:1_anc=none:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence_9");
    quick.push("dis+11_3_afr=on:afp=40000:afq=2.0:anc=none:fsr=off:gs=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:urr=on_8");
    quick.push("lrs+1011_3_add=large:afp=1000:afq=1.1:anc=none:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:sac=on_182");
    break;

  case SMT_AUFLIA:
    quick.push("dis+10_4_add=off:afp=4000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:tha=off:urr=on:updr=off_2");
    break;

  case SMT_AUFNIRA:
    quick.push("dis+1002_5_add=large:afr=on:afp=4000:afq=1.4:amm=off:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:sp=occurrence:updr=off_6");
    quick.push("dis+1011_24_av=off:fsr=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sos=all:tha=off:updr=off_8");
    quick.push("lrs+10_8_afr=on:afp=4000:afq=1.1:amm=sco:anc=none:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sac=on:tha=off:urr=on:updr=off_2");
    quick.push("lrs+1002_3_afr=on:afp=40000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:irw=on:lma=on:nm=2:nwc=1.1:nicw=on:sas=z3:stl=30:sac=on:updr=off:uhcvi=on_7");
    quick.push("lrs+1_3:2_aac=none:afr=on:afp=40000:afq=1.0:anc=none:bs=unit_only:lma=on:nm=64:newcnf=on:nwc=3:sas=z3:stl=30:sd=3:ss=axioms:st=3.0:sp=occurrence:updr=off_15");
    quick.push("lrs+1_3:1_acc=model:add=large:afp=40000:afq=1.2:anc=none:bd=off:bsr=on:ccuc=first:fsr=off:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=2:stl=30:sp=reverse_arity:updr=off_13");
    quick.push("lrs+11_3:2_add=off:afp=40000:afq=1.0:anc=none:fsr=off:fde=none:gs=on:inw=on:irw=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sos=all:sp=reverse_arity:thi=strong_2");
    quick.push("lrs+11_3_afr=on:afp=40000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:inw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:sac=on:sp=occurrence:updr=off_2");
    quick.push("lrs+10_24_add=off:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=1:stl=30:sp=occurrence:tha=off:thf=on_45");
    quick.push("dis+1010_8_add=large:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1.5:sas=z3:sac=on:sp=reverse_arity_3");
    quick.push("dis+11_2_add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:fde=unused:gs=on:gsaa=full_model:ile=on:nm=64:nwc=1:sas=z3:sos=all:sac=on_6");
    quick.push("lrs+11_5:1_add=large:afr=on:afp=1000:afq=1.0:amm=off:anc=none:bd=off:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:sp=occurrence:tha=off:urr=ec_only_192");
    quick.push("lrs+10_24_av=off:bd=off:cond=on:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=2.5:stl=30:sp=occurrence_3");
    quick.push("lrs+11_4:1_add=large:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:tha=off:urr=on:updr=off_288");
    quick.push("lrs+1010_24_afp=40000:afq=2.0:amm=off:anc=none:cond=fast:gs=on:gsem=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:tha=off:thf=on:updr=off:uhcvi=on_2");
    quick.push("lrs+1011_7_av=off:cond=on:gs=on:ile=on:nm=64:nwc=3:stl=30:updr=off_166");
    quick.push("lrs+1003_3:1_av=off:bsr=on:cond=fast:fde=unused:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sos=all:sp=occurrence:tha=off:updr=off:uhcvi=on_125");
    quick.push("lrs+10_3:1_afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity_93");
    quick.push("ott+1004_8:1_afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:fde=unused:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_146");
    break;

  case SMT_UF:
    quick.push("ott+11_8:1_acc=model:add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:ccuc=first:cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:lwlo=on:nm=2:nwc=1:sos=all:urr=on_27");
    quick.push("fmb+10_1_av=off:fmbes=contour:fmbsr=1.3:ile=on:nm=2:newcnf=on:updr=off_45");
    quick.push("lrs+11_2_av=off:br=off:ep=R:ile=on:lma=on:nm=64:nwc=1:stl=30:urr=on_41");
    quick.push("ott+11_1_add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:urr=ec_only_149");
    quick.push("lrs+11_2_add=large:afr=on:amm=sco:anc=none:bsr=on:gs=on:gsem=off:irw=on:lma=on:nm=16:newcnf=on:nwc=1:stl=30:sac=on:sp=occurrence:urr=on:updr=off_270");
    quick.push("dis+1002_2_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:ep=RS:ile=on:nm=64:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_22");
    quick.push("dis+11_24_acc=on:afr=on:amm=sco:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:newcnf=on:nwc=1:updr=off_8");
    quick.push("lrs+10_3:1_afr=on:afp=100000:afq=1.4:amm=sco:anc=none:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:urr=on:uhcvi=on_212");
    quick.push("lrs+11_2:1_av=off:cond=on:er=known:gsp=input_only:ile=on:lma=on:nm=64:nwc=1:stl=30:sos=theory:sp=reverse_arity:urr=on:updr=off_7");
    quick.push("ott+4_4_av=off:cond=on:gs=on:gsem=on:nm=2:nwc=1.7:updr=off_40");
    quick.push("dis+1011_8_av=off:bd=off:bs=unit_only:bsr=on:cond=on:irw=on:nm=64:newcnf=on:nwc=1_35");
    quick.push("dis+1011_8:1_add=off:afp=10000:afq=1.0:amm=off:anc=none:bd=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:updr=off_91");
    quick.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=2.0:ile=on:nm=2_595");
    quick.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.0:anc=none:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1.1:sas=z3:stl=30:sac=on:updr=off_115");
    quick.push("dis+2_4_acc=on:add=large:afp=100000:afq=1.1:amm=sco:anc=none:ccuc=first:cond=on:fsr=off:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1.1:nicw=on_280");
    quick.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=sco:anc=none:cond=on:ep=RSTC:gs=on:gsem=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity_25");
    quick.push("dis+4_16_acc=model:add=large:afr=on:afp=40000:afq=2.0:amm=off:anc=none:bs=on:ccuc=small_ones:fsr=off:ile=on:nm=4:newcnf=on:nwc=1:nicw=on:sp=reverse_arity_13");
    quick.push("ott+2_6_add=large:afr=on:afp=4000:afq=2.0:amm=sco:anc=all:bs=on:bce=on:cond=fast:fde=none:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:sac=on:urr=on:updr=off_4");
    quick.push("dis+10_1_afp=4000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:er=known:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=on:sac=on:sp=occurrence:urr=ec_only_39");
    quick.push("dis+11_5_add=large:afr=on:afp=1000:afq=1.0:anc=none:bsr=on:fsr=off:nm=64:newcnf=on:nwc=1:updr=off_3");
    quick.push("lrs+11_3:2_av=off:cond=on:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sd=3:ss=axioms:st=5.0:updr=off_78");
    quick.push("lrs+1_32_av=off:bd=off:bs=unit_only:er=known:gsp=input_only:gs=on:nm=64:newcnf=on:nwc=1.1:stl=30:sos=on:sp=reverse_arity:urr=ec_only_88");
    quick.push("lrs+1002_3_acc=on:amm=sco:anc=none:ccuc=small_ones:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:stl=30:urr=on_239");
    quick.push("lrs+11_4:1_av=off:bd=off:bs=unit_only:cond=on:fsr=off:fde=none:ile=on:irw=on:lwlo=on:nm=4:nwc=1.1:stl=30:sp=reverse_arity_95");
    quick.push("lrs+11_3:1_av=off:bsr=on:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=64:nwc=1.1:stl=30:sp=reverse_arity:updr=off_22");
    quick.push("ott+1011_8:1_add=off:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bd=off:fsr=off:ile=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sp=reverse_arity:updr=off_55");
    quick.push("ott+11_3_afr=on:afp=10000:afq=1.4:amm=off:anc=none:bsr=on:cond=on:er=known:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on_2");
    quick.push("lrs+10_3:2_av=off:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:nm=64:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=all:urr=on_163");
    quick.push("lrs+10_2_av=off:bd=off:er=known:fsr=off:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sos=theory:sp=reverse_arity:urr=ec_only_27");
    quick.push("dis+2_4_add=large:afr=on:afp=40000:afq=2.0:anc=none:bd=off:ile=on:irw=on:nm=16:nwc=1:nicw=on:sas=z3:sos=on:sp=reverse_arity:updr=off_45");
    quick.push("dis+1011_4:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bsr=on:fsr=off:ile=on:nm=64:nwc=5:sas=z3:sp=reverse_arity:urr=ec_only:updr=off_255");
    quick.push("ott-10_4_av=off:bd=preordered:fsr=off:fde=none:ile=on:irw=on:nm=2:newcnf=on:nwc=1:updr=off:uhcvi=on_244");
    quick.push("dis+11_50_aac=none:acc=model:add=large:afr=on:afp=4000:afq=2.0:anc=none:ccuc=first:er=known:fde=unused:gsp=input_only:gs=on:gsaa=full_model:ile=on:irw=on:nm=64:newcnf=on:nwc=4:sac=on:sp=occurrence:updr=off_5");
    quick.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=1.4:fde=unused:updr=off_808");
    quick.push("dis+1002_5_av=off:cond=on:fsr=off:ile=on:nm=64:newcnf=on:nwc=1.1:sp=reverse_arity_20");
    quick.push("lrs+1010_1_add=off:afp=1000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:ile=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence_192");
    quick.push("dis+11_50_add=large:afp=10000:afq=1.2:anc=none:fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sac=on_4");
    quick.push("lrs+11_5_av=off:cond=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:updr=off_22");
    quick.push("lrs+1010_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:gsp=input_only:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:nwc=2:stl=30:updr=off_128");
    quick.push("ott+1_5:1_afp=4000:afq=1.1:anc=none:bd=off:cond=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:urr=on:updr=off_154");
    quick.push("dis+1011_2_acc=model:add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:ccuc=first:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence_132");
    quick.push("dis+1004_4:1_av=off:br=off:cond=on:ep=RST:fsr=off:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:sp=occurrence:urr=on_71");
    quick.push("ott+10_2:1_av=off:bce=on:cond=fast:fde=none:irw=on:nm=32:newcnf=on:nwc=1:sos=theory:updr=off_207");
    quick.push("lrs+10_3_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:cond=on:er=known:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sos=all:sac=on:updr=off_118");
    quick.push("dis+2_3_acc=on:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bs=unit_only:br=off:ccuc=first:cond=on:er=filter:ile=on:nm=6:nwc=1:urr=on_53");
    quick.push("lrs+11_5:1_add=off:afr=on:afp=4000:afq=1.1:anc=none:bsr=on:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_172");
    quick.push("lrs+10_128_add=off:afr=on:amm=sco:anc=none:bsr=on:cond=on:ile=on:irw=on:nm=2:nwc=2:nicw=on:sas=z3:stl=30:updr=off_96");
    quick.push("lrs+1002_3_av=off:bs=unit_only:bsr=on:ile=on:nm=64:nwc=1:stl=30:sos=theory:sp=reverse_arity_216");
    quick.push("dis+1010_5:1_av=off:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:urr=on:updr=off_74");
    quick.push("ott+2_4:1_aac=none:add=off:afp=10000:afq=1.1:amm=off:anc=none:bs=on:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity_130");
    quick.push("lrs+10_2:1_av=off:cond=on:fde=none:gs=on:gsem=off:ile=on:irw=on:nm=64:nwc=1:stl=30:sp=occurrence:urr=on_167");
    quick.push("fmb+10_1_av=off:bce=on:fmbsr=1.3:fde=none:nm=64:newcnf=on_761");
    quick.push("dis+1_4_av=off:bd=off:fsr=off:nm=64:newcnf=on:nwc=1:sp=reverse_arity_243");
    quick.push("lrs-1_2:3_aac=none:add=off:afr=on:afp=40000:afq=2.0:amm=off:cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lwlo=on:nm=2:nwc=1.2:stl=60:sos=theory:sp=occurrence_120");
    quick.push("ott+4_4_av=off:bd=off:er=filter:ile=on:irw=on:lma=on:nm=64:nwc=1:sos=on:sp=reverse_arity:updr=off_140");
    quick.push("dis+1011_8:1_av=off:ile=on:lma=on:nm=32:newcnf=on:nwc=1:sp=occurrence_161");
    quick.push("dis+10_3:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:ile=on:nm=2:nwc=2.5:sas=z3:sac=on:sp=occurrence_91");
    quick.push("lrs+1011_3:2_av=off:bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=64:nwc=1.5:sas=z3:stl=30:sp=reverse_arity_222");
    quick.push("dis+4_6_av=off:bd=off:bs=on:ile=on:irw=on:lma=on:nm=64:nwc=1_229");
    quick.push("lrs+1011_1_av=off:bd=off:ile=on:irw=on:lma=on:lwlo=on:nm=16:newcnf=on:nwc=1:stl=30:sp=occurrence_108");
    quick.push("ott+1003_5_av=off:bd=off:bs=on:er=known:fde=none:gs=on:gsem=off:ile=on:nwc=2.5:sos=all:sp=occurrence:urr=on_237");
    break;

  case SMT_AUFLIRA:
    quick.push("lrs+1010_2_anc=none:fsr=off:gs=on:irw=on:newcnf=on:nwc=1:sas=z3:stl=30:sos=on:sp=occurrence:updr=off_4");
    quick.push("lrs+1010_4_av=off:bd=off:bs=unit_only:bsr=on:gs=on:inw=on:ile=on:lma=on:newcnf=on:nwc=2.5:stl=30:sp=occurrence:updr=off_6");
    quick.push("dis+1011_4_afr=on:afp=4000:afq=1.4:anc=none:fsr=off:gs=on:gsem=on:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:tha=off:updr=off_8");
    quick.push("dis+2_3_add=off:afp=40000:afq=1.1:anc=none:cond=on:gs=on:inw=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off_43");
    quick.push("lrs+1010_20_add=large:afr=on:afp=4000:afq=1.4:amm=sco:anc=none:bd=preordered:bs=unit_only:fsr=off:fde=unused:gs=on:ile=on:lcm=reverse:nm=2:nwc=4:sas=z3:stl=120:sac=on:sp=occurrence:tha=off:urr=on:updr=off:uhcvi=on_791");
    quick.push("dis+1002_5:4_afr=on:afp=1000:afq=1.2:anc=none:cond=on:ile=on:irw=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:tha=off:updr=off_33");
    quick.push("dis+1011_3_aac=none:afp=1000:afq=1.2:anc=all:fde=none:gs=on:gsem=on:inw=on:ile=on:lcm=predicate:lma=on:nm=6:newcnf=on:nwc=1.5:nicw=on:sas=z3:tha=off:urr=on_344");
    break;

  case SMT_QF_ABV:
  case SMT_QF_ALIA:
  case SMT_QF_ANIA:
  case SMT_QF_AUFBV:
  case SMT_QF_AUFLIA:
  case SMT_QF_AUFNIA:
  case SMT_QF_AX:
  case SMT_QF_BV:
  case SMT_QF_IDL:
  case SMT_QF_LIA:
  case SMT_QF_LIRA:
  case SMT_QF_LRA:
  case SMT_QF_NIA:
  case SMT_QF_NIRA:
  case SMT_QF_NRA:
  case SMT_QF_RDL:
  case SMT_QF_UF:
  case SMT_QF_UFBV:
  case SMT_QF_UFIDL:
  case SMT_QF_UFLIA:
  case SMT_QF_UFLRA:
  case SMT_QF_UFNIA:
  case SMT_QF_UFNRA:
    throw UserErrorException("Kein Kinderspiel, Bruder, use Z3 for quantifier-free problems!");

  case SMT_ALL:
    break;
  case SMT_BV:
  case SMT_UFBV:
    throw UserErrorException("Sorry, we don't deal with bit-vectors!");
  default:
    throw UserErrorException("This version cannot be used with this logic!");
  }

  fallback.push("dis+1002_5:1_aac=none:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:bsr=on:br=off:cond=on:fsr=off:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1.1:sas=z3:sp=reverse_arity:tha=off:urr=on_600");
  fallback.push("lrs+1011_2:1_afr=on:afp=10000:afq=2.0:amm=off:gsp=input_only:gs=on:inw=on:ile=on:nm=2:nwc=1:sas=z3:tha=off_300");
  fallback.push("dis+1011_4:1_anc=none:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence_300");
  fallback.push("lrs+1010_5:4_afp=100000:afq=1.2:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=64:nwc=1:sac=on:sp=occurrence:urr=on_300");
  fallback.push("lrs-11_2:3_av=off:bd=off:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=1:sp=reverse_arity_600");
  fallback.push("lrs+10_3:1_afr=on:afp=100000:afq=1.4:amm=sco:anc=none:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:sac=on:urr=on:uhcvi=on_300");
  fallback.push("lrs+1_3:1_acc=model:add=large:afp=40000:afq=1.2:anc=none:bd=off:bsr=on:ccuc=first:fsr=off:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=2:sp=reverse_arity:updr=off_300");
  fallback.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=1.4:fde=unused:updr=off_900");
  fallback.push("ott+1010_2:1_acc=on:add=large:afr=on:afp=40000:afq=1.1:anc=none:gs=on:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=on_300");
  fallback.push("dis+1_5:1_add=off:afp=40000:afq=1.2:anc=none:bd=off:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=4:sas=z3:updr=off_300");
  fallback.push("dis+2_1_add=large:afr=on:afp=1000:afq=1.2:anc=none:cond=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_300");
  fallback.push("lrs+10_20_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bs=unit_only:bce=on:fde=unused:gs=on:gsaa=full_model:gsem=on:ile=on:nm=16:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=all:updr=off_300");
  fallback.push("lrs+1002_5:4_add=large:afr=on:afp=40000:afq=2.0:anc=none:cond=on:inw=on:ile=on:nm=64:nwc=1:sas=z3:sd=10:ss=axioms:tha=off:updr=off_300");
  fallback.push("lrs+10_5:1_av=off:fde=unused:ile=on:lwlo=on:nwc=1.1:sp=occurrence:urr=on_900");
  fallback.push("dis-11_7_add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=6:nwc=10:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("ott+10_4:1_aac=none:add=off:afp=40000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_300");
  fallback.push("dis+1011_32_av=off:bd=off:inw=on:irw=on:lwlo=on:nm=16:nwc=3:sd=2:ss=axioms:st=5.0:sp=occurrence:tha=off_600");
  fallback.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.0:anc=none:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1.1:sas=z3:sac=on:updr=off_300");
  fallback.push("dis+1011_2_acc=on:afp=10000:afq=1.1:amm=sco:anc=none:ccuc=small_ones:cond=fast:fde=unused:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off:updr=off:uhcvi=on_300");
  fallback.push("ott+1010_7_av=off:fsr=off:fde=none:lma=on:nm=2:newcnf=on:nwc=1.3:sos=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("ott+11_5:4_aac=none:add=large:afp=4000:afq=1.4:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=on:inw=on:ile=on:nm=2:newcnf=on:nwc=1:sas=z3:sos=on:sp=reverse_arity:urr=on:uhcvi=on_300");
  fallback.push("ott+1_5:1_afr=on:afp=4000:afq=1.2:amm=off:anc=none:bs=unit_only:br=off:cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:urr=on:updr=off_600");
  fallback.push("dis+10_3_afr=on:afp=1000:afq=1.0:anc=none:cond=on:fsr=off:gs=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:sp=occurrence:urr=on_300");
  fallback.push("dis+11_5_add=large:afr=on:afp=1000:afq=1.0:anc=none:bsr=on:fsr=off:nm=64:newcnf=on:nwc=1:updr=off_300");
  fallback.push("dis+10_14_add=large:afp=4000:afq=1.1:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=none:inw=on:irw=on:lcm=predicate:nm=4:nwc=1.1:sos=on:sac=on:updr=off:uhcvi=on_300");
  fallback.push("dis+1002_5:4_add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:updr=off_300");
  fallback.push("dis+1003_2:1_afr=on:afp=100000:afq=1.1:anc=none:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:sp=occurrence:tar=off:uhcvi=on_300");
  fallback.push("lrs+1004_5:1_av=off:cond=on:fde=none:irw=on:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on:sp=reverse_arity:updr=off:uhcvi=on_600");
  fallback.push("dis+10_4:1_add=off:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("lrs+11_5:1_add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:er=known:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=predicate:lwlo=on:nm=64:newcnf=on:nwc=1.1:sac=on:sp=reverse_arity:tha=off:thf=on_300");
  fallback.push("ott-3_2:1_add=off:afr=on:afp=100000:afq=2.0:bsr=on:fsr=off:fde=none:gs=on:ile=on:irw=on:lma=on:lwlo=on:nm=4:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:urr=ec_only_300");
  fallback.push("dis+1011_5:1_afp=4000:afq=1.4:amm=off:anc=none:cond=on:fde=unused:gsp=input_only:ile=on:lma=on:nm=16:nwc=1:sos=on:sac=on:tha=off:urr=ec_only:uhcvi=on_300");
  fallback.push("lrs-1_3:1_av=off:bd=off:cond=on:gs=on:ile=on:lcm=reverse:lma=on:nm=32:nwc=1.2:urr=on:updr=off_300");
  fallback.push("lrs-4_5:1_add=large:afp=1000:afq=1.0:amm=sco:anc=none:cond=on:fde=unused:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sp=occurrence:uwa=ground:urr=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+10_8:1_aac=none:afr=on:afp=100000:afq=1.0:amm=off:cond=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1.7:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("lrs+1011_3_add=off:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:bce=on:cond=fast:fde=unused:ile=on:lma=on:nm=6:nwc=2:nicw=on:sas=z3:sd=3:ss=axioms:st=2.0:sp=reverse_arity:tha=off_300");
  fallback.push("ott+11_3:2_aac=none:add=large:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:bs=on:bsr=on:br=off:cond=on:fsr=off:gsp=input_only:gs=on:irw=on:lcm=predicate:lma=on:nm=16:nwc=1.5:nicw=on:sas=z3:sac=on:sp=reverse_arity:tha=off:thi=all:urr=on:updr=off_1200");
  fallback.push("dis+1002_2_aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=all:bsr=on:fde=unused:inw=on:ile=on:lcm=reverse:nm=4:nwc=4:nicw=on:sos=theory:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("lrs+11_5:1_add=large:afr=on:afp=1000:afq=1.0:amm=off:anc=none:bd=off:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off:urr=ec_only_300");
  fallback.push("dis+11_4_afp=4000:afq=1.4:amm=sco:anc=none:gs=on:ile=on:lma=on:nm=64:nwc=1.7:sas=z3:sac=on:sp=occurrence_300");
  fallback.push("lrs+1011_3_add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:tar=off_300");
  fallback.push("ott+1002_3:1_av=off:bsr=on:ep=RS:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:updr=off:uhcvi=on_300");
  fallback.push("lrs+11_4:1_add=large:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:tha=off:urr=on:updr=off_300");
  fallback.push("ott+11_1_add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:urr=ec_only_300");
  fallback.push("dis+1010_1_add=off:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=6:nwc=1.3:nicw=on:sas=z3:tha=off:urr=ec_only_300");
  fallback.push("lrs+1011_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off_300");
  fallback.push("dis+1011_8_afp=10000:afq=1.2:amm=sco:anc=none:bce=on:gs=on:gsem=off:ile=on:lma=on:nm=16:newcnf=on:nwc=2.5:sas=z3:sos=all:sac=on:sp=occurrence:updr=off_300");
  fallback.push("lrs+1010_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:lcm=reverse:lwlo=on:nm=64:nwc=3:nicw=on:sp=occurrence_300");
  fallback.push("ott+2_4:1_aac=none:add=off:afp=10000:afq=1.1:amm=off:anc=none:bs=on:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity_300");
  fallback.push("ott+1011_3:2_av=off:bd=off:bs=on:bce=on:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:tha=off:updr=off_300");
  fallback.push("dis+4_16_acc=model:add=large:afr=on:afp=40000:afq=2.0:amm=off:anc=none:bs=on:ccuc=small_ones:fsr=off:ile=on:nm=4:newcnf=on:nwc=1:nicw=on:sp=reverse_arity_300");
  fallback.push("ott+11_8:1_acc=model:add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:ccuc=first:cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:lwlo=on:nm=2:nwc=1:sos=all:urr=on_300");
  fallback.push("lrs+2_8:1_add=off:afp=40000:afq=1.0:anc=none:fde=unused:gs=on:ile=on:irw=on:lcm=reverse:nm=64:nwc=3:sas=z3:sp=occurrence:urr=on:uhcvi=on_300");
  fallback.push("dis+11_3:1_av=off:br=off:ep=R:fsr=off:gsp=input_only:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:urr=on:uhcvi=on_300");
  fallback.push("dis+1011_4:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bsr=on:fsr=off:ile=on:nm=64:nwc=5:sas=z3:sp=reverse_arity:urr=ec_only:updr=off_300");
  fallback.push("lrs-2_8:1_add=large:afp=100000:afq=1.4:amm=sco:anc=none:bs=on:bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=4:nicw=on:sas=z3:sp=reverse_arity:updr=off_600");
  fallback.push("dis+1002_3_add=off:afr=on:amm=off:anc=none:cond=on:ile=on:lma=on:nm=64:nwc=1:nicw=on:sac=on:sp=reverse_arity:tac=axiom:tar=off:updr=off_300");
  fallback.push("dis+1010_2_acc=on:afr=on:afp=100000:afq=1.2:anc=none:bsr=on:fsr=off:ile=on:irw=on:nm=16:newcnf=on:nwc=4:sp=occurrence:tha=off:urr=ec_only_300");
  fallback.push("ott+1010_1_add=large:afp=1000:afq=1.2:anc=none:bd=off:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:updr=off_300");
  fallback.push("lrs+1010_2_anc=none:fsr=off:gs=on:irw=on:newcnf=on:nwc=1:sas=z3:sos=on:sp=occurrence:updr=off_300");
  fallback.push("lrs+1002_2_add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:inw=on:lwlo=on:nm=32:newcnf=on:nwc=1:sos=theory:sac=on:sp=occurrence:urr=on_300");
  fallback.push("ott+1_10_av=off:ep=RSTC:fsr=off:ile=on:lma=on:newcnf=on:nwc=1:sos=on:tha=off:updr=off_300");
  fallback.push("lrs-10_3_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=unused:gs=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.2:sas=z3:tha=off:urr=ec_only_300");
  fallback.push("lrs+1010_5:4_aac=none:add=large:afp=10000:afq=1.1:amm=off:bs=on:bsr=on:ile=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=on:thf=on:updr=off_300");
  fallback.push("lrs+1010_1_add=off:afp=1000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:ile=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence_300");
  fallback.push("lrs+11_5:1_afr=on:afp=10000:afq=1.2:amm=off:anc=none:fsr=off:gs=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off_300");
  fallback.push("lrs+1011_3:2_av=off:bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=64:nwc=1.5:sas=z3:sp=reverse_arity_300");
  fallback.push("dis+1004_8_av=off:cond=on:er=filter:fde=unused:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity_300");
  fallback.push("lrs+2_4_add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:cond=on:ep=R:gs=on:gsaa=from_current:gsem=off:ile=on:lcm=reverse:lma=on:nm=2:nwc=1.1:sos=on:sac=on:tha=off:updr=off_300");
  fallback.push("dis+2_2_afr=on:afp=100000:afq=1.2:amm=off:anc=none:bsr=on:cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=1.1:sas=z3:sac=on:tha=off:updr=off_300");
  fallback.push("dis+11_5_add=large:afr=on:afp=10000:afq=1.2:anc=none:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("lrs+10_2:3_afr=on:afp=1000:afq=1.1:bd=off:bce=on:cond=on:gsp=input_only:gs=on:gsaa=from_current:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:uwa=interpreted_only:updr=off:uhcvi=on_300");
  fallback.push("dis+1010_8_add=large:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:cond=on:fsr=off:gs=on:ile=on:irw=on:nm=64:nwc=1.5:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("ott+1_8:1_add=large:afp=10000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:fsr=off:fde=unused:ile=on:irw=on:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:updr=off:uhcvi=on_300");
  fallback.push("lrs+2_1024_av=off:bd=off:bsr=on:cond=fast:fsr=off:fde=none:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:tha=off:thi=overlap:uwa=one_side_constant:updr=off:uhcvi=on_300");
  fallback.push("ott+1003_5_av=off:bd=off:bs=on:er=known:fde=none:gs=on:gsem=off:ile=on:nwc=2.5:sos=all:sp=occurrence:urr=on_300");
  fallback.push("lrs-1_2:1_add=off:afp=4000:afq=1.1:amm=sco:anc=none:bd=off:fsr=off:ile=on:irw=on:lcm=reverse:nm=64:nwc=1:sas=z3:sos=all:sp=occurrence:urr=on_300");
  fallback.push("dis+1002_2_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:ep=RS:ile=on:nm=64:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("dis+11_4_add=off:afp=1000:afq=2.0:amm=sco:anc=none:fsr=off:gs=on:gsem=off:ile=on:nm=64:nwc=1.7:sas=z3:urr=on_300");
  fallback.push("ott+11_3_afr=on:afp=10000:afq=1.4:amm=off:anc=none:bsr=on:cond=on:er=known:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on_300");
  fallback.push("ott+1011_8:1_afr=on:afp=1000:afq=1.4:amm=sco:anc=none:bd=off:fsr=off:fde=unused:inw=on:ile=on:nm=2:nwc=1:nicw=on:sas=z3:sos=theory:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_600");
  fallback.push("lrs+4_5:4_av=off:bd=off:cond=on:fsr=off:irw=on:lwlo=on:nm=0:newcnf=on:nwc=1:sos=on:sp=reverse_arity:updr=off_300");
  fallback.push("dis+11_7_add=large:afr=on:afp=10000:afq=1.2:bd=off:bsr=on:cond=on:fsr=off:fde=unused:gs=on:ile=on:lcm=predicate:lma=on:nm=2:newcnf=on:nwc=3:sos=on:sp=reverse_arity:tha=off:updr=off_300");
  fallback.push("lrs-11_2:1_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.5:sas=z3:sp=reverse_arity:urr=on_300");
  fallback.push("dis+1011_1_afp=40000:afq=1.2:anc=none:cond=on:gsp=input_only:ile=on:irw=on:lma=on:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
  fallback.push("dis+1011_8:1_add=off:afp=10000:afq=1.0:amm=off:anc=none:bd=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:updr=off_300");
  fallback.push("dis+10_32_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:fde=none:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sp=occurrence:tha=off:thi=full:uhcvi=on_300");
  fallback.push("dis+1002_5_add=large:afr=on:afp=4000:afq=1.4:amm=off:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:sp=occurrence:updr=off_300");
  fallback.push("dis+1011_4_afr=on:afp=4000:afq=1.4:anc=none:fsr=off:gs=on:gsem=on:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:tha=off:updr=off_300");
  fallback.push("lrs+11_5:1_add=off:afr=on:afp=4000:afq=1.1:anc=none:bsr=on:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("fmb+10_1_av=off:bce=on:fmbsr=1.3:fde=none:nm=64:newcnf=on_900");
  fallback.push("lrs+1011_2:1_av=off:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sp=occurrence:tha=off:urr=ec_only:uhcvi=on_300");
  fallback.push("dis+1011_2:3_add=large:afr=on:afp=40000:afq=1.0:anc=none:br=off:cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nwc=1:sos=on:sac=on:sp=occurrence:tac=axiom:tar=off:urr=on:updr=off_300");
  fallback.push("dis+1011_4_afr=on:afp=10000:afq=1.1:amm=off:anc=none:ep=RS:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:updr=off:uhcvi=on_300");
  fallback.push("dis+10_2_add=off:amm=off:anc=none:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off:uhcvi=on_300");
  fallback.push("dis+1011_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=4:nwc=1:sac=on:tha=off:updr=off:uhcvi=on_300");
  fallback.push("dis+11_10_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=10:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("lrs+11_1_av=off:bd=off:bsr=on:cond=on:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_300");
  fallback.push("lrs+1002_5_av=off:cond=fast:fsr=off:fde=unused:gs=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1.7:sp=reverse_arity_300");
  fallback.push("dis+1011_4_add=large:afr=on:afp=4000:afq=1.4:anc=none:cond=on:ep=RS:fsr=off:gs=on:gsaa=from_current:ile=on:lwlo=on:nm=64:nwc=1:sos=all:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_300");
  fallback.push("lrs+1010_1_afp=1000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:br=off:cond=on:ile=on:irw=on:nm=2:nwc=1:nicw=on:sas=z3:sos=all:sp=reverse_arity:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+1003_3:2_afp=1000:afq=2.0:amm=off:anc=none:cond=on:gs=on:ile=on:lma=on:nm=6:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=all:updr=off_300");
  fallback.push("lrs+2_8:1_acc=on:afr=on:amm=off:bd=off:ccuc=first:cond=on:gsp=input_only:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1.2:sos=on:sp=occurrence_300");
  fallback.push("lrs+11_2:1_add=off:anc=none:bsr=on:br=off:cond=on:er=filter:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=2:nwc=1:sas=z3:sos=all:sac=on:uwa=ground:urr=on_300");
  fallback.push("lrs+10_3_av=off:fde=unused:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1.7:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_600");
  fallback.push("ott+1010_2:1_add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:cond=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=1.5:sac=on:tha=off:updr=off_300");
  fallback.push("ott-4_5:4_aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:cond=fast:ile=on:irw=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=3:thf=on:urr=on:updr=off:uhcvi=on_300");
  fallback.push("ott+1011_3:1_add=off:afp=100000:afq=2.0:amm=off:anc=none:bs=unit_only:gs=on:gsem=on:irw=on:newcnf=on:nwc=1:sas=z3:tha=off_300");
  fallback.push("dis-1_128_av=off:bs=on:fde=unused:ile=on:irw=on:nm=32:nwc=1.1:sos=all:tha=off:thi=full:uwa=one_side_constant:uhcvi=on_600");
  fallback.push("ott+1_1_av=off:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:lma=on:nm=4:newcnf=on:nwc=1:sp=occurrence:urr=on_600");
  fallback.push("lrs+11_2_add=large:afr=on:amm=sco:anc=none:bsr=on:gs=on:gsem=off:irw=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:sp=occurrence:urr=on:updr=off_300");
  fallback.push("dis+1_8:1_av=off:br=off:fsr=off:fde=none:gsp=input_only:ile=on:lma=on:nm=2:nwc=1:sos=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("dis+1004_16_av=off:fsr=off:fde=unused:ile=on:irw=on:nm=0:newcnf=on:nwc=1.1:sp=reverse_arity:urr=on_300");
  fallback.push("lrs+11_3_afr=on:afp=40000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:inw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:updr=off_300");
  fallback.push("lrs+1002_1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
  fallback.push("dis+1011_2_add=large:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:cond=on:gs=on:ile=on:irw=on:nm=64:newcnf=on:nwc=1:urr=on_300");
  fallback.push("lrs+11_5:1_afr=on:afp=100000:afq=1.4:anc=none:cond=fast:fsr=off:ile=on:irw=on:nm=64:nwc=5:nicw=on:sas=z3:sp=reverse_arity:tha=off:thi=all:uwa=one_side_interpreted:updr=off_600");
  fallback.push("ott+1011_2:3_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:bce=on:cond=fast:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1.1:sp=reverse_arity:tha=off:urr=on:updr=off_300");
  fallback.push("ott+4_4_av=off:cond=on:gs=on:gsem=on:nm=2:nwc=1.7:updr=off_300");
  fallback.push("dis+10_3_add=large:afp=4000:afq=1.4:amm=off:anc=none:cond=on:ep=RS:gs=on:gsaa=from_current:inw=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:tha=off:updr=off_300");
  fallback.push("lrs+11_8:1_add=large:afp=100000:afq=2.0:amm=off:anc=none:bsr=on:cond=fast:gs=on:gsaa=full_model:inw=on:ile=on:lcm=predicate:nm=4:newcnf=on:nwc=1:sp=reverse_arity:tha=off:urr=on_300");
  fallback.push("dis+1004_4:1_av=off:br=off:cond=on:ep=RST:fsr=off:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:sp=occurrence:urr=on_300");
  fallback.push("dis+1010_1_av=off:lma=on:newcnf=on:nwc=1:sd=4:ss=axioms:sos=on:sp=reverse_arity_300");
  fallback.push("lrs+1010_1_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:fsr=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on_300");
  fallback.push("lrs-2_5:1_acc=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bd=off:cond=fast:gs=on:ile=on:nm=0:newcnf=on:nwc=3:sac=on:thf=on:urr=ec_only_300");
  fallback.push("lrs+11_5:4_av=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=32:newcnf=on:nwc=1.3:sp=reverse_arity:updr=off_300");
  fallback.push("lrs+10_50_av=off:cond=fast:fde=none:lcm=reverse:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:uhcvi=on_300");
  fallback.push("lrs+1_5:1_add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity_300");
  fallback.push("lrs+10_3:1_av=off:cond=on:fde=none:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sp=reverse_arity:tar=off:uhcvi=on_300");
  fallback.push("dis+1010_2_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:fsr=off:fde=none:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:sp=reverse_arity_300");
  fallback.push("dis+1011_8:1_av=off:ile=on:lma=on:nm=32:newcnf=on:nwc=1:sp=occurrence_300");
  fallback.push("lrs+11_8_afr=on:afp=1000:afq=2.0:amm=sco:anc=none:bsr=on:fsr=off:gs=on:gsem=off:inw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:tha=off:urr=ec_only:updr=off_300");
  fallback.push("dis+1002_5:4_afr=on:afp=1000:afq=1.2:anc=none:cond=on:ile=on:irw=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:tha=off:updr=off_300");
  fallback.push("ott+1_5:1_av=off:bs=unit_only:br=off:gs=on:gsem=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=4:ss=axioms:st=1.5:sp=occurrence:tha=off:urr=on:uhcvi=on_300");
  fallback.push("dis+1003_8_afr=on:anc=none:bd=preordered:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=reverse_arity:updr=off:uhcvi=on_300");
  fallback.push("dis+2_3_add=off:afp=40000:afq=1.1:anc=none:cond=on:gs=on:inw=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+10_10_av=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:updr=off_300");
  fallback.push("dis+11_2_add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:fde=unused:gs=on:gsaa=full_model:ile=on:nm=64:nwc=1:sas=z3:sos=all:sac=on_300");
  fallback.push("ott+1002_5:1_add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:br=off:cond=on:fsr=off:gs=on:inw=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sos=all:tha=off:urr=on_300");
  fallback.push("lrs+1011_2:1_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=preordered:ccuc=first:cond=fast:fsr=off:fde=unused:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=2:nicw=on:sp=occurrence:urr=ec_only:updr=off_300");
  fallback.push("lrs-2_3:1_add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=on:er=filter:fde=unused:ile=on:irw=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on:thi=strong:uhcvi=on_600");
  fallback.push("lrs+1002_1_add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:fsr=off:gsp=input_only:inw=on:ile=on:lcm=predicate:lwlo=on:nm=64:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on_300");
  fallback.push("lrs+10_24_afp=4000:afq=2.0:bd=off:bsr=on:bce=on:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:nwc=1.3:sp=occurrence:tha=off:uwa=one_side_constant:urr=ec_only_300");
  fallback.push("dis+1011_5:1_afr=on:afp=10000:afq=1.2:amm=sco:bd=preordered:bs=unit_only:cond=on:fsr=off:inw=on:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=1.1:sd=7:ss=axioms:st=1.2:tha=off:uhcvi=on_300");
  fallback.push("lrs+2_8_av=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+11_2_av=off:cond=on:fsr=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on_300");
  fallback.push("ott+4_4_av=off:bd=off:er=filter:ile=on:irw=on:lma=on:nm=64:nwc=1:sos=on:sp=reverse_arity:updr=off_300");
  fallback.push("dis+1011_10_av=off:bd=off:cond=fast:er=known:inw=on:ile=on:irw=on:lma=on:nwc=1.7:sp=occurrence:tha=off:uhcvi=on_300");
  fallback.push("dis-11_4:1_aac=none:add=large:afp=4000:afq=1.2:anc=none:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sp=occurrence_300");
  fallback.push("dis+10_2:1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=on:inw=on:ile=on:irw=on:nm=2:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
  fallback.push("ott+1003_12_add=large:anc=all:bd=preordered:bce=on:fde=none:lcm=reverse:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:uwa=ground_600");
  fallback.push("lrs-11_4_acc=on:afr=on:afp=40000:afq=1.4:amm=off:anc=none:br=off:bce=on:cond=fast:fde=none:gs=on:ile=on:irw=on:nm=0:newcnf=on:nwc=1.1:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("dis+11_3_afp=10000:afq=1.0:anc=none:cond=on:gs=on:ile=on:irw=on:lma=on:nm=64:nwc=1.1:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("dis+2_4_acc=on:add=large:afp=100000:afq=1.1:amm=sco:anc=none:ccuc=first:cond=on:fsr=off:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1.1:nicw=on_300");
  fallback.push("lrs+1003_3:1_av=off:bsr=on:cond=fast:fde=unused:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sp=occurrence:tha=off:updr=off:uhcvi=on_300");
  fallback.push("lrs-11_4:1_afp=100000:afq=1.2:amm=off:anc=all_dependent:bs=unit_only:fsr=off:fde=none:gs=on:gsem=on:ile=on:lma=on:nm=64:nwc=1:sp=reverse_arity:updr=off:uhcvi=on_300");
  fallback.push("lrs+10_8_afr=on:afp=4000:afq=1.1:amm=sco:anc=none:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sac=on:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+10_24_add=off:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=1:sp=occurrence:tha=off:thf=on_300");
  fallback.push("dis+1004_1_add=off:afr=on:afp=1000:afq=1.1:amm=off:anc=none:bd=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tac=light:tar=off:tha=off:thi=all:urr=on:uhcvi=on_300");
  fallback.push("lrs+1011_2:1_aac=none:add=off:afp=1000:afq=1.0:amm=off:bs=on:gs=on:gsaa=from_current:gsem=on:ile=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off_300");
  fallback.push("ott+11_5:1_add=off:afp=1000:afq=1.4:anc=none:bd=off:bs=unit_only:bsr=on:br=off:gsp=input_only:ile=on:irw=on:lma=on:nm=64:nwc=1:sos=all:tha=off:urr=on_300");
  fallback.push("ott-4_1_add=off:afp=40000:afq=1.0:anc=none:bce=on:cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lcm=reverse:lma=on:nm=6:newcnf=on:nwc=1.5:sas=z3:sos=all:sp=occurrence:updr=off:uhcvi=on_300");
  fallback.push("lrs-11_5:4_afp=4000:afq=1.4:amm=sco:anc=none:bd=off:br=off:gs=on:gsem=off:inw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sp=occurrence:urr=on_300");
  fallback.push("dis+1011_12_afp=100000:afq=1.0:amm=sco:anc=none:fsr=off:fde=unused:gsp=input_only:ile=on:irw=on:nm=64:nwc=1.2:sac=on:sp=occurrence:tac=axiom:tar=off:uhcvi=on_300");
  fallback.push("lrs+1010_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:gsp=input_only:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:nwc=2:updr=off_300");
  fallback.push("dis+10_4_add=off:afp=4000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+1010_1_av=off:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:sp=reverse_arity:updr=off_300");
  fallback.push("dis+11_3_av=off:cond=on:fsr=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:updr=off:uhcvi=on_300");
  fallback.push("lrs+11_1_add=off:afp=100000:afq=1.4:amm=off:anc=none:bsr=on:fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence:updr=off_300");
  fallback.push("dis+1010_3_afp=10000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:tha=off:urr=on_300");
  fallback.push("lrs+2_3:2_av=off:cond=fast:inw=on:ile=on:nm=2:nwc=1:sos=theory:urr=on_300");
  fallback.push("lrs+4_8:1_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:tha=off_300");
  fallback.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=sco:anc=none:cond=on:ep=RSTC:gs=on:gsem=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity_300");
  fallback.push("ott+1011_2:3_av=off:bs=unit_only:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thi=all:uwa=all:urr=on:uhcvi=on_300");
  fallback.push("lrs+4_8:1_av=off:cond=on:gs=on:gsem=on:irw=on:nm=64:newcnf=on:nwc=1.1:sp=occurrence:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+10_3_add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:cond=on:er=known:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sac=on:updr=off_300");
  fallback.push("lrs+11_2_av=off:br=off:ep=R:ile=on:lma=on:nm=64:nwc=1:urr=on_300");
  fallback.push("lrs+1002_3_afr=on:afp=40000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:irw=on:lma=on:nm=2:nwc=1.1:nicw=on:sas=z3:sac=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+11_4:1_av=off:bd=off:bs=unit_only:cond=on:fsr=off:fde=none:ile=on:irw=on:lwlo=on:nm=4:nwc=1.1:sp=reverse_arity_300");
  fallback.push("lrs+11_10_add=off:afp=40000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bce=on:cond=fast:gsp=input_only:inw=on:lma=on:nm=4:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:urr=ec_only:uhcvi=on_300");
  fallback.push("dis-3_7_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sp=occurrence:tha=off:thi=overlap:uwa=interpreted_only:uhcvi=on_300");
  fallback.push("dis+1011_3_aac=none:afr=on:afp=40000:afq=1.4:amm=off:bs=on:fsr=off:fde=none:gsp=input_only:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off_300");
  fallback.push("lrs+4_4_av=off:bd=off:bs=unit_only:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:updr=off_300");
  fallback.push("lrs+11_3:2_av=off:bd=off:br=off:fde=none:gs=on:gsem=off:inw=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:urr=on:uhcvi=on_600");
  fallback.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=2.0:ile=on:nm=2_600");
  fallback.push("dis+1_4:1_acc=on:add=large:afp=4000:afq=1.2:amm=sco:anc=none:ccuc=small_ones:ile=on:lwlo=on:nm=64:nwc=1:tha=off:urr=ec_only:updr=off_300");
  fallback.push("lrs+1010_24_afp=40000:afq=2.0:amm=off:anc=none:cond=fast:gs=on:gsem=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:thf=on:updr=off:uhcvi=on_300");
  fallback.push("ott+11_2_av=off:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=6:nwc=1.5:sp=occurrence:updr=off_300");
  fallback.push("dis+10_3:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:ile=on:nm=2:nwc=2.5:sas=z3:sac=on:sp=occurrence_300");
  fallback.push("lrs+1011_5:4_av=off:bd=off:bs=on:cond=on:er=known:gs=on:gsem=on:inw=on:ile=on:lcm=reverse:nm=6:newcnf=on:nwc=1:sp=occurrence:tha=off:uhcvi=on_300");
  fallback.push("ott+1002_2:1_add=large:afr=on:afp=100000:afq=1.1:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off_300");
  fallback.push("dis+4_5_av=off:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lwlo=on:nm=6:nwc=1:sos=on:sp=reverse_arity:updr=off_300");
  fallback.push("ott+1_5:1_afp=4000:afq=1.1:anc=none:bd=off:cond=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("lrs+1010_3:1_av=off:bd=off:bsr=on:irw=on:nm=64:newcnf=on:nwc=1.7:sos=all:updr=off_300");
  fallback.push("dis+11_5:1_afr=on:afp=40000:afq=2.0:amm=sco:anc=all_dependent:cond=fast:fde=unused:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=2:nwc=1:sos=all:urr=on:uhcvi=on_300");
  fallback.push("ott-10_4_av=off:bd=preordered:fsr=off:fde=none:ile=on:irw=on:nm=2:newcnf=on:nwc=1:updr=off:uhcvi=on_600");
  fallback.push("lrs+4_2_add=off:afr=on:afp=1000:afq=1.0:amm=off:anc=none:bsr=on:br=off:cond=on:er=known:fsr=off:gs=on:ile=on:lma=on:nm=4:nwc=1:sos=on:sp=occurrence:tar=off:urr=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+1011_2:3_av=off:bsr=on:cond=fast:fsr=off:gsp=input_only:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_300");
  fallback.push("lrs+4_3_av=off:bd=preordered:fde=none:inw=on:ile=on:nm=64:newcnf=on:nwc=1:tha=off:thf=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+1011_3:1_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:bce=on:ep=RS:gs=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1.2:sp=occurrence:tha=off_300");
  fallback.push("fmb+10_1_av=off:fmbes=contour:fmbsr=1.3:ile=on:nm=2:newcnf=on:updr=off_300");
  fallback.push("lrs-11_5_add=off:afr=on:afp=100000:afq=1.0:anc=all:bs=unit_only:cond=fast:fsr=off:ile=on:irw=on:lcm=reverse:lwlo=on:nwc=1.7:nicw=on:sos=on:sac=on:sp=reverse_arity:tha=off:urr=on_300");
  fallback.push("dis+11_24_acc=on:afr=on:amm=sco:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:newcnf=on:nwc=1:updr=off_300");
  fallback.push("lrs+1011_3:2_add=off:afr=on:afp=10000:afq=1.2:amm=off:anc=all:bd=off:bsr=on:br=off:fde=unused:gs=on:inw=on:irw=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:uwa=interpreted_only:urr=on_300");
  fallback.push("dis+1011_8_av=off:bd=off:bs=unit_only:bsr=on:cond=on:irw=on:nm=64:newcnf=on:nwc=1_300");
  fallback.push("ott+10_2:1_av=off:bce=on:cond=fast:fde=none:irw=on:nm=32:newcnf=on:nwc=1:sos=theory:updr=off_300");
  fallback.push("dis+11_2_add=large:afr=on:afp=1000:afq=1.1:anc=none:gs=on:gsaa=full_model:ile=on:irw=on:lma=on:nm=16:nwc=1:sas=z3:sos=on:sac=on:sp=occurrence:thi=strong:uhcvi=on_300");
  fallback.push("ott-1_3:1_av=off:bsr=on:br=off:bce=on:cond=on:fsr=off:fde=unused:irw=on:nm=2:newcnf=on:nwc=2.5:sos=on:sp=occurrence:tha=off:thi=overlap:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+4_4:1_add=off:afp=4000:afq=1.2:amm=sco:anc=none:br=off:cond=fast:ep=RS:fsr=off:inw=on:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on:urr=on:uhcvi=on_300");
  fallback.push("lrs+1004_1_aac=none:add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=all_dependent:bd=off:cond=fast:fsr=off:gs=on:gsaa=from_current:lcm=reverse:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thf=on:urr=on:updr=off_300");
  fallback.push("dis+1011_24_av=off:fsr=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sos=all:tha=off:updr=off_300");
  fallback.push("dis+2_3_acc=on:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bs=unit_only:br=off:ccuc=first:cond=on:er=filter:ile=on:nm=6:nwc=1:urr=on_300");
  fallback.push("dis+1011_1_aac=none:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=all:bce=on:cond=on:er=known:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=off:ile=on:newcnf=on:nwc=1:sas=z3:ss=axioms:st=2.0:sp=occurrence:tha=off:urr=ec_only_300");
  fallback.push("dis+11_5:1_av=off:br=off:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:tha=off:urr=on_300");
  fallback.push("lrs+11_1_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:cond=on:gsp=input_only:gs=on:ile=on:irw=on:nm=6:nwc=1:sos=all:sac=on:urr=on_300");
  fallback.push("lrs-10_5:4_add=off:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.1:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence_300");
  fallback.push("lrs+1002_2:1_aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=none:cond=fast:gs=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("lrs+10_3:2_av=off:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=5.0:sos=all:urr=on_300");
  fallback.push("lrs+1_3:2_aac=none:afr=on:afp=40000:afq=1.0:anc=none:bs=unit_only:lma=on:nm=64:newcnf=on:nwc=3:sas=z3:sd=3:ss=axioms:st=3.0:sp=occurrence:updr=off_300");
  fallback.push("lrs+10_128_add=off:afr=on:amm=sco:anc=none:bsr=on:cond=on:ile=on:irw=on:nm=2:nwc=2:nicw=on:sas=z3:updr=off_300");
  fallback.push("dis+1_4_av=off:bd=off:fsr=off:nm=64:newcnf=on:nwc=1:sp=reverse_arity_300");
  fallback.push("ins+11_3_av=off:gs=on:gsem=on:ile=on:igbrr=0.9:igrr=1/64:igrp=1400:igs=1004:igwr=on:nm=6:nwc=2.5:sp=occurrence:urr=on_300");
  fallback.push("lrs+1_32_av=off:bd=off:bs=unit_only:er=known:gsp=input_only:gs=on:nm=64:newcnf=on:nwc=1.1:sos=on:sp=reverse_arity:urr=ec_only_300");
  fallback.push("dis+1002_14_av=off:cond=fast:fde=unused:inw=on:ile=on:lma=on:nm=0:nwc=1:sos=all:sp=reverse_arity:tha=off:uwa=one_side_interpreted:uhcvi=on_300");
  fallback.push("lrs+1002_3_av=off:bs=unit_only:bsr=on:ile=on:nm=64:nwc=1:sos=theory:sp=reverse_arity_300");
  fallback.push("dis+11_32_add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:er=filter:ile=on:lcm=predicate:lma=on:newcnf=on:nwc=5:sp=occurrence:updr=off_300");
  fallback.push("dis+1003_4:1_add=large:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=fast:fsr=off:fde=none:gs=on:ile=on:lma=on:nm=64:nwc=1.2:sas=z3:sp=reverse_arity:tha=off:urr=ec_only_300");
  fallback.push("dis+4_6_av=off:bd=off:bs=on:ile=on:irw=on:lma=on:nm=64:nwc=1_300");
  fallback.push("dis+11_3_afr=on:afp=40000:afq=2.0:anc=none:fsr=off:gs=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:urr=on_300");
  fallback.push("lrs+11_6_aac=none:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:cond=fast:ile=on:nm=16:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:uhcvi=on_300");
  fallback.push("ott+2_6_add=large:afr=on:afp=4000:afq=2.0:amm=sco:anc=all:bs=on:bce=on:cond=fast:fde=none:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:sac=on:urr=on:updr=off_300");
  fallback.push("lrs+1011_3_add=large:afp=1000:afq=1.1:anc=none:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on_300");
  fallback.push("lrs-1_128_aac=none:add=off:afp=40000:afq=1.0:amm=off:anc=none:fsr=off:inw=on:ile=on:lcm=reverse:lma=on:nm=16:nwc=10:sas=z3:sac=on:updr=off_300");
  fallback.push("lrs+10_3:1_afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:sp=reverse_arity_300");
  fallback.push("dis+10_1_afp=4000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:er=known:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=on:sac=on:sp=occurrence:urr=ec_only_300");
  fallback.push("dis-4_7_acc=on:afp=40000:afq=1.4:anc=all_dependent:bsr=on:br=off:bce=on:ccuc=first:er=filter:fsr=off:fde=unused:gsp=input_only:ile=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:thi=full:uwa=ground:urr=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+11_3:1_av=off:bsr=on:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=64:nwc=1.1:sp=reverse_arity:updr=off_300");
  fallback.push("dis+1002_2:3_av=off:bs=on:bce=on:cond=fast:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:tha=off:thi=strong_300");
  fallback.push("lrs+1002_1_av=off:bd=off:fsr=off:fde=none:nm=2:newcnf=on:nwc=1:sp=reverse_arity:uhcvi=on_300");
  fallback.push("lrs+11_2:1_afp=1000:afq=1.4:amm=off:anc=none:inw=on:ile=on:irw=on:nm=64:nwc=1:sac=on:tha=off:urr=on_300");
  fallback.push("dis+10_3_add=off:afp=100000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sp=reverse_arity:tha=off:thf=on:updr=off_300");
  fallback.push("lrs+10_3:2_av=off:bce=on:cond=on:er=filter:fsr=off:fde=unused:gs=on:gsem=on:ile=on:irw=on:nm=6:nwc=1:sos=all:tac=light:tar=off:updr=off:uhcvi=on_300");
  fallback.push("dis+11_50_aac=none:acc=model:add=large:afr=on:afp=4000:afq=2.0:anc=none:ccuc=first:er=known:fde=unused:gsp=input_only:gs=on:gsaa=full_model:ile=on:irw=on:nm=64:newcnf=on:nwc=4:sac=on:sp=occurrence:updr=off_300");
  fallback.push("dis+1002_1_add=large:afp=4000:afq=1.2:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only:uhcvi=on_300");
  fallback.push("lrs+1002_1_av=off:bd=off:bsr=on:cond=on:ile=on:lma=on:nm=64:nwc=1:sos=on:sp=reverse_arity_300");
  fallback.push("dis+1010_6_av=off:cond=on:er=filter:fsr=off:nm=64:newcnf=on:nwc=1.3:sp=reverse_arity_300");
  fallback.push("ott+1004_8:1_afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:fde=unused:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_300");
  fallback.push("lrs+10_3:1_afp=1000:afq=1.4:amm=off:anc=none:bsr=on:inw=on:ile=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+10_3:1_add=large:afp=10000:afq=1.1:amm=off:anc=none:cond=on:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sd=5:ss=axioms:st=1.5:tha=off:urr=on_300");
  fallback.push("ott+1011_4_afp=4000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:nm=32:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off_300");
  fallback.push("lrs-11_8:1_afr=on:afp=1000:afq=1.4:amm=off:anc=none:bd=off:bs=on:gs=on:ile=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only_300");
  fallback.push("lrs+2_4_add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:bd=off:bce=on:fde=none:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1.3:sas=z3:tha=off:thi=strong:uwa=one_side_interpreted:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+1010_5:1_av=off:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:urr=on:updr=off_300");
  fallback.push("dis+1011_8:1_add=off:afp=10000:afq=1.1:anc=none:bce=on:er=filter:gs=on:gsaa=from_current:gsem=off:inw=on:ile=on:lma=on:nm=2:nwc=3:sac=on:urr=on:updr=off_300");
  fallback.push("lrs+1002_3_acc=on:amm=sco:anc=none:ccuc=small_ones:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:urr=on_300");
  fallback.push("ott-1_1_acc=model:add=off:afr=on:afp=4000:afq=1.2:anc=all:bd=preordered:bs=unit_only:bsr=on:ccuc=first:gs=on:gsaa=from_current:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=occurrence:tha=off:thi=strong:updr=off_300");
  fallback.push("dis+1003_28_acc=model:add=large:afp=10000:afq=1.1:amm=off:anc=none:bd=off:ccuc=first:fsr=off:gs=on:gsaa=from_current:ile=on:lma=on:nm=2:nwc=1:sos=all:sp=occurrence:tha=off:uwa=ground:uhcvi=on_300");
  fallback.push("lrs+1011_7_av=off:cond=on:gs=on:ile=on:nm=64:nwc=3:updr=off_300");
  fallback.push("dis+2_4_afp=10000:afq=1.1:bd=off:bs=on:cond=on:er=filter:ile=on:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_300");
  fallback.push("ott+10_2:1_av=off:bd=off:br=off:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+1010_1_av=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:sp=occurrence:tar=off:urr=on_300");
  fallback.push("lrs-10_3:2_aac=none:add=off:afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=16:nwc=1:nicw=on:sas=z3:sd=2:ss=axioms:sos=on:sp=occurrence:updr=off_600");
  fallback.push("lrs+10_2:1_av=off:cond=on:fde=none:gs=on:gsem=off:ile=on:irw=on:nm=64:nwc=1:sp=occurrence:urr=on_300");
  fallback.push("ott+11_5:4_aac=none:add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:bce=on:cond=on:fsr=off:fde=unused:ile=on:irw=on:lma=on:nm=6:newcnf=on:nwc=1:nicw=on:sas=z3:tha=off:updr=off_300");
  fallback.push("lrs+4_28_afp=10000:afq=1.2:amm=sco:anc=none:bd=off:bce=on:cond=on:fsr=off:ile=on:irw=on:lcm=reverse:nm=16:newcnf=on:nwc=2:sas=z3:sp=occurrence:tha=off:updr=off:uhcvi=on_600");
  fallback.push("dis+1011_3_aac=none:afp=1000:afq=1.2:anc=all:fde=none:gs=on:gsem=on:inw=on:ile=on:lcm=predicate:lma=on:nm=6:newcnf=on:nwc=1.5:nicw=on:sas=z3:tha=off:urr=on_900");
  fallback.push("lrs+10_24_av=off:bs=unit_only:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sd=7:ss=axioms:st=1.2:sp=occurrence:tha=off:thf=on:uhcvi=on_600");
  fallback.push("lrs+1003_4:1_av=off:bd=preordered:cond=on:fde=unused:gs=on:ile=on:irw=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_900");
} // getSmtcomp2017Schedule

// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------

void Schedules::getSmtcomp2018Schedule(const Property& property, Schedule& quick, Schedule& fallback)
{
  switch (property.getSMTLIBLogic()) {
  case SMT_AUFDTLIA:
  case SMT_AUFDTLIRA: // Add new logic here even though probably not best schedule
  case SMT_AUFDTNIRA: // Add new logic here even though probably not best schedule
    quick.push("lrs+1010_5:4_afp=100000:afq=1.2:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=64:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_9");
    quick.push("dis+11_5_add=large:afr=on:afp=10000:afq=1.2:anc=none:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:urr=on:updr=off_273");
    quick.push("dis+1011_2:3_add=large:afr=on:afp=40000:afq=1.0:anc=none:br=off:cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nwc=1:sos=on:sac=on:sp=occurrence:tac=axiom:tar=off:urr=on:updr=off_264");
    break;

  case SMT_UFDTLIA:
  case SMT_UFDTLIRA: // Add new logic here even though probably not best schedule
  case SMT_UFDTNIA:
  case SMT_UFDTNIRA: // Add new logic here even though probably not best schedule
    quick.push("dis+1011_2:3_add=large:afr=on:afp=40000:afq=1.0:anc=none:br=off:cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nwc=1:sos=on:sac=on:sp=occurrence:tac=axiom:tar=off:urr=on:updr=off_2");
    quick.push("lrs+11_7_add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bsr=on:br=off:fde=unused:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=1:stl=30:sos=all:sac=on:urr=on:updr=off:uhcvi=on_5");
    quick.push("dis+1004_1_add=off:afr=on:afp=1000:afq=1.1:amm=off:anc=none:bd=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tac=light:tar=off:tha=off:thi=all:urr=on:uhcvi=on_6");
    quick.push("lrs+1010_5:4_afp=100000:afq=1.2:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=64:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_88");
    quick.push("lrs+10_2:1_add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:tac=axiom:tar=off:tha=off:uwa=ground:uhcvi=on_145");
    break;

  case SMT_UFDT:
    quick.push("lrs+11_8:1_av=off:cond=on:fde=none:ile=on:nm=16:nwc=1.3:stl=30:sp=reverse_arity:urr=on:updr=off_135");
    quick.push("ott+1003_14_av=off:fsr=off:fde=unused:ile=on:lcm=predicate:nm=0:newcnf=on:nwc=1:sp=occurrence:uhcvi=on_12");
    quick.push("lrs+10_3:1_av=off:cond=on:fde=none:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sp=reverse_arity:tar=off:uhcvi=on_148");
    quick.push("dis+1003_8_afr=on:anc=none:bd=preordered:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=reverse_arity:updr=off:uhcvi=on_271");
    quick.push("dis+1011_12_afp=100000:afq=1.0:amm=sco:anc=none:fsr=off:fde=unused:gsp=input_only:ile=on:irw=on:nm=64:nwc=1.2:sac=on:sp=occurrence:tac=axiom:tar=off:uhcvi=on_72");
    quick.push("dis+1011_4_add=large:afr=on:afp=4000:afq=1.4:anc=none:cond=on:ep=RS:fsr=off:gs=on:gsaa=from_current:ile=on:lwlo=on:nm=64:nwc=1:sos=all:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_145");
    quick.push("lrs+1010_5:4_afp=100000:afq=1.2:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=64:nwc=1:stl=30:sac=on:sp=occurrence:urr=on_94");
    quick.push("lrs-1_3:1_av=off:bd=off:cond=on:gs=on:ile=on:lcm=reverse:lma=on:nm=32:nwc=1.2:stl=30:urr=on:updr=off_64");
    quick.push("lrs+11_8:1_add=large:afp=100000:afq=2.0:amm=off:anc=none:bsr=on:cond=fast:gs=on:gsaa=full_model:inw=on:ile=on:lcm=predicate:nm=4:newcnf=on:nwc=1:stl=30:sp=reverse_arity:tha=off:urr=on_201");
    quick.push("lrs+11_2:1_av=off:bd=off:br=off:bce=on:cond=on:fde=none:irw=on:lma=on:nm=2:newcnf=on:nwc=1.1:stl=30:sp=occurrence:urr=on:updr=off:uhcvi=on_228");
    quick.push("dis+1003_2:1_afr=on:afp=100000:afq=1.1:anc=none:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:sp=occurrence:tar=off:uhcvi=on_284");
    quick.push("dis+1011_2:3_add=large:afr=on:afp=40000:afq=1.0:anc=none:br=off:cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nwc=1:sos=on:sac=on:sp=occurrence:tac=axiom:tar=off:urr=on:updr=off_20");
    quick.push("dis+1_8:1_av=off:br=off:fsr=off:fde=none:gsp=input_only:ile=on:lma=on:nm=2:nwc=1:sos=on:sp=reverse_arity:urr=on:updr=off_13");
    quick.push("lrs+11_1_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:cond=on:gsp=input_only:gs=on:ile=on:irw=on:nm=6:nwc=1:stl=30:sos=all:sac=on:urr=on_299");
    quick.push("lrs+10_3:2_av=off:bce=on:cond=on:er=filter:fsr=off:fde=unused:gs=on:gsem=on:ile=on:irw=on:nm=6:nwc=1:stl=30:sos=all:tac=light:tar=off:updr=off:uhcvi=on_10");
    quick.push("lrs+10_8:1_acc=model:add=large:afr=on:afp=10000:afq=1.4:anc=none:bs=unit_only:ccuc=first:cond=on:er=known:gs=on:gsaa=from_current:ile=on:lcm=reverse:nm=2:nwc=1.2:stl=30:sac=on:urr=on:uhcvi=on_245");
    quick.push("lrs-11_4:1_afp=100000:afq=1.2:amm=off:anc=all_dependent:bs=unit_only:fsr=off:fde=none:gs=on:gsem=on:ile=on:lma=on:nm=64:nwc=1:stl=30:sp=reverse_arity:updr=off:uhcvi=on_232");
    quick.push("ott+1_8:1_add=large:afp=10000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:fsr=off:fde=unused:ile=on:irw=on:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:updr=off:uhcvi=on_90");
    quick.push("dis+1004_8_av=off:cond=on:er=filter:fde=unused:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity_24");
    quick.push("lrs-11_4_acc=on:afr=on:afp=40000:afq=1.4:amm=off:anc=none:br=off:bce=on:cond=fast:fde=none:gs=on:ile=on:irw=on:nm=0:newcnf=on:nwc=1.1:nicw=on:stl=30:sac=on:sp=reverse_arity:urr=on:updr=off_209");
    quick.push("dis+1011_1_av=off:fsr=off:fde=unused:gsp=input_only:ile=on:irw=on:lma=on:nwc=1:sos=on:sp=reverse_arity:urr=ec_only_89");
    quick.push("lrs+1_4_add=off:afp=100000:afq=2.0:anc=none:bsr=on:br=off:cond=on:fde=unused:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=2:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:urr=on:updr=off_122");
    quick.push("dis+1004_16_av=off:fsr=off:fde=unused:ile=on:irw=on:nm=0:newcnf=on:nwc=1.1:sp=reverse_arity:urr=on_191");
    quick.push("lrs+1011_3_add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:tar=off_291");
    quick.push("dis-11_4:1_aac=none:add=large:afp=4000:afq=1.2:anc=none:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sp=occurrence_99");
    quick.push("ott+1_5:1_afr=on:afp=4000:afq=1.2:amm=off:anc=none:bs=unit_only:br=off:cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:urr=on:updr=off_573");
    quick.push("dis+1010_5_add=off:afp=100000:afq=1.0:amm=sco:anc=none:fsr=off:fde=none:gsp=input_only:gs=on:gsaa=from_current:ile=on:nm=64:nwc=1:sas=z3:tar=off:updr=off_5");
    quick.push("dis+1010_5_av=off:bsr=on:cond=fast:fde=unused:ile=on:nm=6:nwc=1:uhcvi=on_411");
    quick.push("dis+1010_1_av=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:sp=occurrence:tar=off:urr=on_177");
    quick.push("lrs+11_1_add=off:afp=100000:afq=1.4:amm=off:anc=none:bsr=on:fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:stl=30:sac=on:sp=occurrence:updr=off_267");
    quick.push("lrs+10_5:1_av=off:fde=unused:ile=on:lwlo=on:nwc=1.1:stl=90:sp=occurrence:urr=on_343");
    break;

  case SMT_LIA:
    quick.push("dis+1011_8_afp=10000:afq=1.2:amm=sco:anc=none:bce=on:gs=on:gsem=off:ile=on:lma=on:nm=16:newcnf=on:nwc=2.5:sas=z3:sos=all:sac=on:sp=occurrence:updr=off_2");
    quick.push("dis+11_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:bce=on:cond=on:fsr=off:ile=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:thf=on_2");
    quick.push("dis-2_4_add=large:afp=1000:afq=1.0:amm=sco:anc=none:bce=on:gs=on:inw=on:ile=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:thi=all_13");
    quick.push("dis+10_1_afp=10000:afq=1.0:amm=sco:anc=none:bce=on:fde=none:gs=on:gsem=off:inw=on:ile=on:irw=on:lma=on:nm=16:newcnf=on:nwc=1:sas=z3:sos=on:sac=on:sp=occurrence:tha=off:thi=full_145");
    quick.push("lrs+10_20_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bs=unit_only:bce=on:fde=unused:gs=on:gsaa=full_model:gsem=on:ile=on:nm=16:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence:tha=off:thi=all:updr=off_87");
    quick.push("lrs-11_4:1_afp=4000:afq=2.0:anc=none:br=off:gs=on:lwlo=on:nm=64:nwc=3:stl=30:urr=on_1");
    break;

  case SMT_UFNIA:
    quick.push("lrs+11_5:4_av=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=32:newcnf=on:nwc=1.3:stl=30:sp=reverse_arity:updr=off_63");
    quick.push("ott+1010_7_av=off:fsr=off:fde=none:lma=on:nm=2:newcnf=on:nwc=1.3:sos=on:sp=reverse_arity:uhcvi=on_194");
    quick.push("dis+1011_8_av=off:bd=off:bsr=on:bce=on:cond=on:fsr=off:fde=unused:ile=on:nm=64:nwc=1.1:sd=10:ss=axioms:st=1.5:sos=all:sp=reverse_arity:tha=off_569");
    quick.push("dis+2_2_afr=on:afp=100000:afq=1.2:amm=off:anc=none:bsr=on:cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=1.1:sas=z3:sac=on:tha=off:updr=off_47");
    quick.push("lrs-4_5:1_add=large:afp=1000:afq=1.0:amm=sco:anc=none:cond=on:fde=unused:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sos=all:sp=occurrence:uwa=ground:urr=on:updr=off:uhcvi=on_132");
    quick.push("dis+10_2_add=off:amm=off:anc=none:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off:uhcvi=on_75");
    quick.push("dis+1010_2:1_afp=40000:afq=1.1:anc=none:gsp=input_only:ile=on:irw=on:nm=6:nwc=1:sac=on:tha=off:updr=off_8");
    quick.push("lrs+1002_1_av=off:bd=off:fsr=off:fde=none:nm=2:newcnf=on:nwc=1:stl=30:sp=reverse_arity:uhcvi=on_26");
    quick.push("dis+10_3_add=large:afp=4000:afq=1.4:amm=off:anc=none:cond=on:ep=RS:gs=on:gsaa=from_current:inw=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:tha=off:updr=off_3");
    quick.push("dis+1010_2:1_add=large:afp=10000:afq=2.0:amm=off:anc=all_dependent:bce=on:cond=fast:ep=R:fsr=off:ile=on:lma=on:nm=64:nwc=1:sac=on:urr=on_11");
    quick.push("lrs+1010_1_av=off:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sos=all:sp=reverse_arity:updr=off_128");
    quick.push("dis+1010_1_afr=on:afp=40000:afq=2.0:amm=off:anc=none:gs=on:ile=on:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off_10");
    quick.push("dis+11_5:1_av=off:br=off:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:tha=off:urr=on_45");
    quick.push("ott+1002_5:1_add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:br=off:cond=on:fsr=off:gs=on:inw=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sos=all:tha=off:urr=on_74");
    quick.push("lrs+11_1_add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:fsr=off:gs=on:gsem=on:inw=on:ile=on:nm=64:newcnf=on:nwc=1:stl=30:sp=reverse_arity:tha=off:thf=on:updr=off_197");
    quick.push("lrs+11_2:1_afp=1000:afq=1.4:amm=off:anc=none:inw=on:ile=on:irw=on:nm=64:nwc=1:stl=30:sac=on:tha=off:urr=on_73");
    quick.push("dis+1002_2:3_av=off:bs=on:bce=on:cond=fast:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:tha=off:thi=strong_79");
    quick.push("lrs+1011_3:1_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:bce=on:ep=RS:gs=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1.2:stl=30:sp=occurrence:tha=off_5");
    quick.push("dis+11_2_afp=40000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:inw=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:updr=off_12");
    quick.push("lrs+1010_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:lcm=reverse:lwlo=on:nm=64:nwc=3:nicw=on:stl=30:sp=occurrence_158");
    quick.push("dis+11_3_av=off:cond=on:fsr=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:updr=off:uhcvi=on_117");
    quick.push("lrs+4_3_av=off:bd=preordered:fde=none:inw=on:ile=on:nm=64:newcnf=on:nwc=1:stl=30:tha=off:thf=on:updr=off:uhcvi=on_8");
    quick.push("lrs+1004_5:1_av=off:cond=on:fde=none:irw=on:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=1:stl=60:sos=on:sp=reverse_arity:updr=off:uhcvi=on_215");
    quick.push("ott-1_3:1_av=off:bsr=on:br=off:bce=on:cond=on:fsr=off:fde=unused:irw=on:nm=2:newcnf=on:nwc=2.5:sos=on:sp=occurrence:tha=off:thi=overlap:urr=on:updr=off:uhcvi=on_148");
    quick.push("dis-1_128_av=off:bs=on:fde=unused:ile=on:irw=on:nm=32:nwc=1.1:sos=all:tha=off:thi=full:uwa=one_side_constant:uhcvi=on_355");
    quick.push("lrs+11_5:1_afr=on:afp=10000:afq=1.2:amm=off:anc=none:fsr=off:gs=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:sp=occurrence:tha=off_150");
    quick.push("lrs+1002_2:1_add=large:afp=100000:afq=1.2:amm=off:anc=none:cond=fast:fde=unused:gs=on:gsaa=from_current:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sd=5:ss=axioms:st=1.2:tha=off:uwa=ground_6");
    quick.push("dis+1002_1_av=off:bd=off:br=off:cond=on:fsr=off:fde=unused:newcnf=on:nwc=1:sd=5:ss=axioms:st=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_8");
    quick.push("lrs+1011_2:3_av=off:bsr=on:cond=fast:fsr=off:gsp=input_only:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:tha=off:updr=off_87");
    quick.push("lrs+10_50_av=off:cond=fast:fde=none:lcm=reverse:nm=64:newcnf=on:nwc=1:stl=30:sp=occurrence:tha=off:uhcvi=on_264");
    quick.push("dis+1_4:1_acc=on:add=large:afp=4000:afq=1.2:amm=sco:anc=none:ccuc=small_ones:ile=on:lwlo=on:nm=64:nwc=1:tha=off:urr=ec_only:updr=off_228");
    quick.push("lrs+1010_8:1_av=off:br=off:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:inw=on:ile=on:irw=on:lma=on:nm=4:nwc=5:stl=30:sos=on:tha=off:thf=on:urr=on_44");
    quick.push("ott+1_5:1_av=off:bs=unit_only:br=off:gs=on:gsem=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=4:ss=axioms:st=1.5:sp=occurrence:tha=off:urr=on:uhcvi=on_173");
    quick.push("ott+11_5:4_aac=none:add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:bce=on:cond=on:fsr=off:fde=unused:ile=on:irw=on:lma=on:nm=6:newcnf=on:nwc=1:nicw=on:sas=z3:tha=off:updr=off_31");
    quick.push("lrs+1003_4:1_av=off:bd=preordered:cond=on:fde=unused:gs=on:ile=on:irw=on:nm=64:nwc=1.2:stl=90:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_322");
    quick.push("lrs+10_3_av=off:bs=unit_only:bce=on:cond=on:fde=unused:gsp=input_only:gs=on:inw=on:irw=on:nm=0:newcnf=on:nwc=1.1:stl=30:tha=off:uhcvi=on_44");
    quick.push("lrs+1002_8_afp=10000:afq=2.0:amm=sco:anc=none:bs=on:cond=on:fsr=off:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1.3:sas=z3:stl=30:sp=reverse_arity:urr=on_37");
    quick.push("lrs+1002_1_av=off:bd=off:bsr=on:cond=on:ile=on:lma=on:nm=64:nwc=1:stl=30:sos=on:sp=reverse_arity_18");
    quick.push("lrs-1_2:3_afr=on:afp=100000:afq=1.1:amm=off:anc=none:cond=on:fsr=off:gs=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sp=reverse_arity:tha=off:thf=on:urr=on_210");
    quick.push("ott+1011_4_afp=4000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:nm=32:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off_45");
    quick.push("dis+1010_1_av=off:lma=on:newcnf=on:nwc=1:sd=4:ss=axioms:sos=on:sp=reverse_arity_196");
    quick.push("lrs+1002_5:4_add=large:afr=on:afp=40000:afq=2.0:anc=none:cond=on:inw=on:ile=on:nm=64:nwc=1:sas=z3:stl=30:sd=10:ss=axioms:tha=off:updr=off_109");
    quick.push("lrs-11_4:1_aac=none:add=off:afp=10000:afq=1.2:anc=none:fsr=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:urr=on:updr=off_16");
    quick.push("lrs+10_5:4_av=off:bd=off:fsr=off:fde=none:lcm=reverse:lma=on:newcnf=on:nwc=1:stl=30:tha=off:urr=on:updr=off_173");
    quick.push("dis+1003_4:1_add=large:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=fast:fsr=off:fde=none:gs=on:ile=on:lma=on:nm=64:nwc=1.2:sas=z3:sp=reverse_arity:tha=off:urr=ec_only_19");
    quick.push("dis+1002_1_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:cond=on:fsr=off:ile=on:lma=on:nm=4:nwc=1:tha=off:updr=off_6");
    quick.push("dis+1010_6_av=off:cond=on:er=filter:fsr=off:nm=64:newcnf=on:nwc=1.3:sp=reverse_arity_222");
    quick.push("lrs+10_10_av=off:gs=on:gsem=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:stl=30:updr=off_8");
    quick.push("lrs+2_8:1_add=off:afp=40000:afq=1.0:anc=none:fde=unused:gs=on:ile=on:irw=on:lcm=reverse:nm=64:nwc=3:sas=z3:stl=30:sp=occurrence:urr=on:uhcvi=on_13");
    quick.push("lrs+1011_10_av=off:cond=fast:er=filter:fsr=off:fde=none:gs=on:gsem=on:ile=on:lma=on:nm=4:nwc=1:stl=30:sos=all:sp=reverse_arity:tha=off:thi=new:uwa=ground:updr=off:uhcvi=on_118");
    quick.push("ott+1004_3_av=off:fde=none:gs=on:gsem=on:ile=on:nm=0:nwc=1.3:sp=reverse_arity:tha=off:thi=overlap:urr=ec_only:updr=off_106");
    quick.push("ott+11_2_av=off:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=6:nwc=1.5:sp=occurrence:updr=off_158");
    quick.push("lrs+11_6_av=off:bd=off:cond=fast:fde=none:lma=on:lwlo=on:nm=0:newcnf=on:nwc=1:stl=30:sos=on:sp=reverse_arity:updr=off:uhcvi=on_84");
    quick.push("ott+1_10_av=off:ep=RSTC:fsr=off:ile=on:lma=on:newcnf=on:nwc=1:sos=on:tha=off:updr=off_227");
    quick.push("dis+1003_28_acc=model:add=large:afp=10000:afq=1.1:amm=off:anc=none:bd=off:ccuc=first:fsr=off:gs=on:gsaa=from_current:ile=on:lma=on:nm=2:nwc=1:sos=all:sp=occurrence:tha=off:uwa=ground:uhcvi=on_86");
    quick.push("ott-1_24_av=off:bd=off:cond=fast:er=known:fsr=off:fde=unused:gsp=input_only:irw=on:lma=on:lwlo=on:nm=0:newcnf=on:nwc=1.3:sp=occurrence:tha=off:thi=new:uhcvi=on_88");
    quick.push("lrs+4_3:1_add=off:afp=1000:afq=2.0:anc=none:gs=on:gsem=on:ile=on:lma=on:nm=2:nwc=5:sas=z3:stl=30:sac=on:sp=occurrence:updr=off_8");
    break;

  case SMT_ALIA:
    quick.push("lrs+2_4_add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:bd=off:bce=on:fde=none:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1.3:sas=z3:stl=30:tha=off:thi=strong:uwa=one_side_interpreted:urr=on:updr=off:uhcvi=on_3");
    quick.push("ott-3_2:3_add=off:afr=on:afp=40000:afq=1.0:bsr=on:cond=fast:fsr=off:fde=none:gs=on:ile=on:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1.2:nicw=on:sas=z3:sos=all:sp=reverse_arity:urr=ec_only:updr=off_44");
    quick.push("lrs-1_128_aac=none:add=off:afp=40000:afq=1.0:amm=off:anc=none:fsr=off:inw=on:ile=on:lcm=reverse:lma=on:nm=16:nwc=10:sas=z3:stl=30:sac=on:updr=off_195");
    break;

  case SMT_UFLIA:
    quick.push("lrs-11_2:1_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.5:sas=z3:stl=30:sp=reverse_arity:urr=on_9");
    quick.push("lrs-10_3_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=unused:gs=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.2:sas=z3:stl=30:tha=off:urr=ec_only_42");
    quick.push("lrs+1011_2:1_afr=on:afp=10000:afq=2.0:amm=off:gsp=input_only:gs=on:inw=on:ile=on:nm=2:nwc=1:sas=z3:stl=30:tha=off_296");
    quick.push("dis+10_14_add=large:afp=4000:afq=1.1:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=none:inw=on:irw=on:lcm=predicate:nm=4:nwc=1.1:sos=on:sac=on:updr=off:uhcvi=on_268");
    quick.push("ott+1011_3:2_av=off:bd=off:bs=on:bce=on:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:tha=off:updr=off_124");
    quick.push("dis+1011_1_afp=40000:afq=1.2:anc=none:cond=on:gsp=input_only:ile=on:irw=on:lma=on:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_249");
    quick.push("lrs+1011_2:1_av=off:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sp=occurrence:tha=off:urr=ec_only:uhcvi=on_79");
    quick.push("lrs+4_28_afp=10000:afq=1.2:amm=sco:anc=none:bd=off:bce=on:cond=on:fsr=off:ile=on:irw=on:lcm=reverse:nm=16:newcnf=on:nwc=2:sas=z3:stl=60:sp=occurrence:tha=off:updr=off:uhcvi=on_350");
    quick.push("lrs+4_8:1_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:stl=30:sos=all:tha=off_6");
    quick.push("ott+1011_8:1_afr=on:afp=1000:afq=1.4:amm=sco:anc=none:bd=off:fsr=off:fde=unused:inw=on:ile=on:nm=2:nwc=1:nicw=on:sas=z3:sos=theory:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_280");
    quick.push("lrs+11_2_av=off:cond=on:fsr=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sp=reverse_arity:tha=off:thf=on_66");
    quick.push("dis+1002_5:4_add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:updr=off_132");
    quick.push("lrs-11_8:1_afr=on:afp=1000:afq=1.4:amm=off:anc=none:bd=off:bs=on:gs=on:ile=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only_56");
    quick.push("lrs+4_4:1_add=off:afp=10000:afq=1.2:anc=none:bd=off:bsr=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=2:nwc=1:sas=z3:stl=30:sac=on:sp=occurrence:tha=off_1");
    quick.push("dis+1011_3:1_add=off:afr=on:afp=40000:afq=1.1:amm=sco:bd=off:bce=on:cond=fast:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=4:nwc=1.5:sas=z3:sos=all:sp=occurrence:tha=off:uwa=interpreted_only:uhcvi=on_93");
    quick.push("dis+1011_2_acc=on:afp=10000:afq=1.1:amm=sco:anc=none:ccuc=small_ones:cond=fast:fde=unused:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off:updr=off:uhcvi=on_267");
    quick.push("dis+10_2:1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=on:inw=on:ile=on:irw=on:nm=2:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_75");
    quick.push("dis+1010_2_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:fsr=off:fde=none:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:sp=reverse_arity_118");
    quick.push("lrs+2_4_add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:cond=on:ep=R:gs=on:gsaa=from_current:gsem=off:ile=on:lcm=reverse:lma=on:nm=2:nwc=1.1:stl=30:sos=on:sac=on:tha=off:updr=off_120");
    quick.push("ott+1010_2:1_add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:cond=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=1.5:sac=on:tha=off:updr=off_193");
    quick.push("lrs+4_8:1_av=off:cond=on:gs=on:gsem=on:irw=on:nm=64:newcnf=on:nwc=1.1:stl=30:sp=occurrence:tha=off:urr=on:updr=off_133");
    quick.push("dis+1011_5:1_afp=4000:afq=1.4:amm=off:anc=none:cond=on:fde=unused:gsp=input_only:ile=on:lma=on:nm=16:nwc=1:sos=on:sac=on:tha=off:urr=ec_only:uhcvi=on_248");
    quick.push("lrs+11_5:1_add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:er=known:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=predicate:lwlo=on:nm=64:newcnf=on:nwc=1.1:stl=30:sac=on:sp=reverse_arity:tha=off:thf=on_255");
    quick.push("dis-3_7_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sp=occurrence:tha=off:thi=overlap:uwa=interpreted_only:uhcvi=on_128");
    quick.push("lrs+1011_3:1_aac=none:add=large:afp=1000:afq=2.0:fsr=off:gs=on:gsaa=from_current:gsem=on:ile=on:nm=4:nwc=1.5:sas=z3:stl=30:sp=reverse_arity:tha=off:uwa=interpreted_only:uhcvi=on_158");
    quick.push("ott+1010_1_add=large:afp=1000:afq=1.2:anc=none:bd=off:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:updr=off_221");
    quick.push("ott+10_4:1_aac=none:add=off:afp=40000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_210");
    quick.push("dis+1010_3_afp=10000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:tha=off:urr=on_46");
    quick.push("lrs+1002_2_add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:inw=on:lwlo=on:nm=32:newcnf=on:nwc=1:stl=30:sos=theory:sac=on:sp=occurrence:urr=on_74");
    quick.push("ott+1011_3:1_add=off:afp=100000:afq=2.0:amm=off:anc=none:bs=unit_only:gs=on:gsem=on:irw=on:newcnf=on:nwc=1:sas=z3:tha=off_67");
    quick.push("lrs+2_1024_av=off:bd=off:bsr=on:cond=fast:fsr=off:fde=none:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:stl=30:tha=off:thi=overlap:uwa=one_side_constant:updr=off:uhcvi=on_195");
    quick.push("lrs+1011_5:4_av=off:bd=off:bs=on:cond=on:er=known:gs=on:gsem=on:inw=on:ile=on:lcm=reverse:nm=6:newcnf=on:nwc=1:stl=30:sp=occurrence:tha=off:uhcvi=on_136");
    quick.push("lrs+11_2:1_add=off:anc=none:bsr=on:br=off:cond=on:er=filter:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=2:nwc=1:sas=z3:stl=30:sos=all:sac=on:uwa=ground:urr=on_27");
    quick.push("dis+1011_8:1_add=off:afp=10000:afq=1.1:anc=none:bce=on:er=filter:gs=on:gsaa=from_current:gsem=off:inw=on:ile=on:lma=on:nm=2:nwc=3:sac=on:urr=on:updr=off_5");
    quick.push("dis+1_2_av=off:bd=off:cond=on:fsr=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:urr=ec_only:updr=off_21");
    quick.push("dis-11_5:4_add=large:afp=40000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:fde=unused:gsp=input_only:ile=on:lcm=reverse:lma=on:nm=6:nwc=1:sos=all:sac=on:urr=ec_only:uhcvi=on_72");
    quick.push("lrs-2_5:1_acc=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bd=off:cond=fast:gs=on:ile=on:nm=0:newcnf=on:nwc=3:stl=30:sac=on:thf=on:urr=ec_only_296");
    quick.push("lrs+1011_3_add=off:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:bce=on:cond=fast:fde=unused:ile=on:lma=on:nm=6:nwc=2:nicw=on:sas=z3:stl=30:sd=3:ss=axioms:st=2.0:sp=reverse_arity:tha=off_261");
    quick.push("dis+11_1_av=off:br=off:cond=on:gsp=input_only:gs=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:urr=on_82");
    quick.push("lrs+1002_1_add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:fsr=off:gsp=input_only:inw=on:ile=on:lcm=predicate:lwlo=on:nm=64:nwc=1.7:sas=z3:stl=30:sac=on:sp=reverse_arity:tha=off:thf=on_89");
    quick.push("ott+1011_2:3_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:bce=on:cond=fast:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1.1:sp=reverse_arity:tha=off:urr=on:updr=off_148");
    quick.push("dis+11_7_add=large:afr=on:afp=10000:afq=1.2:bd=off:bsr=on:cond=on:fsr=off:fde=unused:gs=on:ile=on:lcm=predicate:lma=on:nm=2:newcnf=on:nwc=3:sos=on:sp=reverse_arity:tha=off:updr=off_22");
    quick.push("lrs+1_5:1_add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1.2:stl=30:sp=reverse_arity_269");
    quick.push("ott+10_4:1_afp=100000:afq=1.1:anc=none:bd=off:inw=on:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sac=on:sp=occurrence:tha=off:urr=on:updr=off_6");
    quick.push("dis+11_5:1_afr=on:afp=40000:afq=2.0:amm=sco:anc=all_dependent:cond=fast:fde=unused:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=2:nwc=1:sos=all:urr=on:uhcvi=on_7");
    quick.push("dis+1002_1_add=large:afp=4000:afq=1.2:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only:uhcvi=on_33");
    quick.push("ott+1011_5_av=off:fde=unused:gsp=input_only:gs=on:gsem=off:ile=on:nm=32:nwc=1.3:sas=z3:sp=reverse_arity:tha=off:uwa=ground_145");
    quick.push("lrs+10_24_av=off:bs=unit_only:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:stl=60:sd=7:ss=axioms:st=1.2:sp=occurrence:tha=off:thf=on:uhcvi=on_343");
    quick.push("dis+1002_2_aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=all:bsr=on:fde=unused:inw=on:ile=on:lcm=reverse:nm=4:nwc=4:nicw=on:sos=theory:sac=on:sp=reverse_arity:uhcvi=on_85");
    quick.push("ott+1002_2:1_add=large:afr=on:afp=100000:afq=1.1:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off_90");
    quick.push("lrs+1003_3:2_afp=1000:afq=2.0:amm=off:anc=none:cond=on:gs=on:ile=on:lma=on:nm=6:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence:tha=off:thi=all:updr=off_75");
    quick.push("lrs+1002_2:1_aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=none:cond=fast:gs=on:nm=64:newcnf=on:nwc=1.5:sas=z3:stl=30:sp=occurrence:updr=off_85");
    quick.push("dis+10_3_add=off:afp=100000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sp=reverse_arity:tha=off:thf=on:updr=off_59");
    quick.push("lrs+11_10_add=off:afp=40000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bce=on:cond=fast:gsp=input_only:inw=on:lma=on:nm=4:newcnf=on:nwc=1:stl=30:sp=occurrence:tha=off:thf=on:urr=ec_only:uhcvi=on_44");
    quick.push("lrs+1002_8:1_add=off:afp=1000:afq=1.2:amm=sco:anc=none:bce=on:cond=on:ep=RS:gs=on:gsaa=from_current:ile=on:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence:tha=off:uwa=interpreted_only:updr=off_199");
    quick.push("dis+1010_1_add=off:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=6:nwc=1.3:nicw=on:sas=z3:tha=off:urr=ec_only_276");
    quick.push("dis+1011_4_afr=on:afp=10000:afq=1.1:amm=off:anc=none:ep=RS:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:updr=off:uhcvi=on_55");
    quick.push("ott+1003_12_add=large:anc=all:bd=preordered:bce=on:fde=none:lcm=reverse:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:uwa=ground_293");
    quick.push("lrs+1011_5:4_aac=none:add=off:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:bd=off:bs=on:bsr=on:cond=on:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:inw=on:ile=on:nm=16:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:tha=off:uwa=interpreted_only:uhcvi=on_146");
    quick.push("lrs+1011_2:1_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=preordered:ccuc=first:cond=fast:fsr=off:fde=unused:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=2:nicw=on:stl=30:sp=occurrence:urr=ec_only:updr=off_190");
    quick.push("lrs-11_4:1_add=large:afp=1000:afq=1.1:amm=sco:bs=on:cond=on:gs=on:gsem=on:ile=on:nm=2:newcnf=on:nwc=1:sas=z3:stl=30:sos=on:sp=occurrence:updr=off_128");
    quick.push("lrs-10_3:2_aac=none:add=off:afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=16:nwc=1:nicw=on:sas=z3:stl=60:sd=2:ss=axioms:sos=on:sp=occurrence:updr=off_83");
    quick.push("ott+1011_3:1_aac=none:acc=on:afr=on:afp=4000:afq=1.2:amm=off:anc=none:bd=off:bs=on:bsr=on:ccuc=first:fsr=off:gs=on:gsem=on:inw=on:ile=on:nm=6:nwc=1:sos=on:thf=on:urr=on_20");
    quick.push("lrs+1010_3:1_av=off:bd=off:bsr=on:irw=on:nm=64:newcnf=on:nwc=1.7:stl=30:sos=all:updr=off_18");
    quick.push("lrs+11_8_afr=on:afp=1000:afq=2.0:amm=sco:anc=none:bsr=on:fsr=off:gs=on:gsem=off:inw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:tha=off:urr=ec_only:updr=off_120");
    quick.push("lrs+1011_2:1_aac=none:add=off:afp=1000:afq=1.0:amm=off:bs=on:gs=on:gsaa=from_current:gsem=on:ile=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sp=reverse_arity:tha=off_200");
    quick.push("lrs+10_3:1_add=large:afp=10000:afq=1.1:amm=off:anc=none:cond=on:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sd=5:ss=axioms:st=1.5:tha=off:urr=on_183");
    quick.push("dis+10_3:2_add=large:afr=on:afp=1000:afq=1.1:anc=none:bd=off:fsr=off:inw=on:ile=on:lma=on:nm=2:nwc=1:sas=z3:sd=1:ss=axioms:sos=all:sp=occurrence:tha=off:updr=off_157");
    quick.push("lrs+1010_1_afp=1000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:br=off:cond=on:ile=on:irw=on:nm=2:nwc=1:nicw=on:sas=z3:stl=30:sos=all:sp=reverse_arity:tha=off:urr=on:updr=off_48");
    quick.push("lrs-2_3:1_add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=on:er=filter:fde=unused:ile=on:irw=on:nm=64:newcnf=on:nwc=1.1:sas=z3:stl=60:sac=on:sp=reverse_arity:tha=off:thf=on:thi=strong:uhcvi=on_41");
    quick.push("dis+1011_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=4:nwc=1:sac=on:tha=off:updr=off:uhcvi=on_205");
    quick.push("ott-1_1_acc=model:add=off:afr=on:afp=4000:afq=1.2:anc=all:bd=preordered:bs=unit_only:bsr=on:ccuc=first:gs=on:gsaa=from_current:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=occurrence:tha=off:thi=strong:updr=off_80");
    quick.push("dis+1002_10_afp=4000:afq=1.4:amm=sco:bd=off:bsr=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:nm=6:newcnf=on:nwc=1:nicw=on:sos=all:sp=occurrence:urr=ec_only_32");
    quick.push("ott+1011_5:4_aac=none:add=large:afp=100000:afq=2.0:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:cond=on:gs=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:nicw=on:sas=z3:sos=on:sp=occurrence:tha=off:updr=off:uhcvi=on_305");
    quick.push("lrs+1_20_add=off:afp=40000:afq=1.4:anc=none:bd=off:bsr=on:gsp=input_only:inw=on:ile=on:newcnf=on:nwc=1:stl=30:sac=on:sp=reverse_arity:tha=off_23");
    quick.push("lrs+1004_1_aac=none:add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=all_dependent:bd=off:cond=fast:fsr=off:gs=on:gsaa=from_current:lcm=reverse:nm=0:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence:tha=off:thf=on:urr=on:updr=off_16");
    quick.push("lrs+4_4_av=off:bd=off:bs=unit_only:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sp=occurrence:tha=off:thf=on:updr=off_234");
    quick.push("dis+1011_10_av=off:bd=off:cond=fast:er=known:inw=on:ile=on:irw=on:lma=on:nwc=1.7:sp=occurrence:tha=off:uhcvi=on_192");
    quick.push("lrs+10_3:1_afp=1000:afq=1.4:amm=off:anc=none:bsr=on:inw=on:ile=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:tha=off:urr=on:updr=off_291");
    quick.push("dis+1011_5:1_afr=on:afp=10000:afq=1.2:amm=sco:bd=preordered:bs=unit_only:cond=on:fsr=off:inw=on:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=1.1:sd=7:ss=axioms:st=1.2:tha=off:uhcvi=on_267");
    quick.push("dis+2_1_add=large:afr=on:afp=1000:afq=1.2:anc=none:cond=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_49");
    quick.push("dis+10_4_afp=1000:afq=1.2:amm=sco:anc=none:gs=on:gsem=on:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=4:nicw=on:sas=z3_32");
    quick.push("dis+1002_14_av=off:cond=fast:fde=unused:inw=on:ile=on:lma=on:nm=0:nwc=1:sos=all:sp=reverse_arity:tha=off:uwa=one_side_interpreted:uhcvi=on_22");
    quick.push("dis+11_32_add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:er=filter:ile=on:lcm=predicate:lma=on:newcnf=on:nwc=5:sp=occurrence:updr=off_286");
    quick.push("lrs+2_8_av=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1.2:stl=30:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_171");
    quick.push("lrs+10_3_av=off:fde=unused:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1.7:stl=60:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_277");
    quick.push("lrs-10_5:4_add=off:afr=on:afp=10000:afq=1.2:amm=off:anc=none:bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.1:stl=30:sd=3:ss=axioms:st=3.0:sos=all:sp=occurrence_28");
    quick.push("dis+4_4:1_add=off:afp=4000:afq=1.2:amm=sco:anc=none:br=off:cond=fast:ep=RS:fsr=off:inw=on:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on:urr=on:uhcvi=on_29");
    quick.push("dis+1010_2_acc=on:afr=on:afp=100000:afq=1.2:anc=none:bsr=on:fsr=off:ile=on:irw=on:nm=16:newcnf=on:nwc=4:sp=occurrence:tha=off:urr=ec_only_202");
    quick.push("lrs+1002_1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:stl=30:sac=on:sp=occurrence:tha=off:updr=off_25");
    quick.push("lrs+2_3:2_av=off:cond=fast:inw=on:ile=on:nm=2:nwc=1:stl=30:sos=theory:urr=on_23");
    quick.push("ott-4_5:4_aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:cond=fast:ile=on:irw=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=3:thf=on:urr=on:updr=off:uhcvi=on_171");
    quick.push("ott+1011_2:3_av=off:bs=unit_only:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thi=all:uwa=all:urr=on:uhcvi=on_225");
    quick.push("lrs+1011_2:3_aac=none:acc=on:add=large:afr=on:afp=40000:afq=1.2:amm=off:ccuc=small_ones:cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:irw=on:lcm=reverse:lma=on:nm=32:newcnf=on:nwc=3:nicw=on:stl=30:sp=occurrence:tha=off:thf=on:updr=off:uhcvi=on_32");
    quick.push("dis-4_7_acc=on:afp=40000:afq=1.4:anc=all_dependent:bsr=on:br=off:bce=on:ccuc=first:er=filter:fsr=off:fde=unused:gsp=input_only:ile=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:thi=full:uwa=ground:urr=on:updr=off:uhcvi=on_12");
    quick.push("ott+10_2:1_av=off:bd=off:br=off:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on:updr=off:uhcvi=on_194");
    quick.push("dis+10_32_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:fde=none:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sp=occurrence:tha=off:thi=full:uhcvi=on_202");
    quick.push("lrs+1011_8:1_av=off:bs=on:cond=on:fsr=off:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:sos=all:sp=reverse_arity:urr=on_117");
    quick.push("lrs+10_24_afp=4000:afq=2.0:bd=off:bsr=on:bce=on:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:nwc=1.3:stl=30:sp=occurrence:tha=off:uwa=one_side_constant:urr=ec_only_282");
    quick.push("lrs+1011_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:stl=30:sp=reverse_arity:tha=off:urr=on:updr=off_7");
    quick.push("dis+2_4_afp=10000:afq=1.1:bd=off:bs=on:cond=on:er=filter:ile=on:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_106");
    quick.push("lrs+1011_3_add=large:afp=10000:afq=1.1:amm=off:fde=unused:ile=on:irw=on:lma=on:nwc=1.7:stl=30:sp=reverse_arity:tha=off:thf=on:updr=off_218");
    quick.push("lrs+1010_1_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:fsr=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sos=all:urr=on_126");
    quick.push("lrs+10_2:3_afr=on:afp=1000:afq=1.1:bd=off:bce=on:cond=on:gsp=input_only:gs=on:gsaa=from_current:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sp=reverse_arity:tha=off:uwa=interpreted_only:updr=off:uhcvi=on_263");
    quick.push("dis+11_5:4_aac=none:acc=on:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:cond=fast:fsr=off:fde=none:lcm=reverse:nm=2:newcnf=on:nwc=1.1:tha=off:thi=strong:uwa=interpreted_only:uhcvi=on_232");
    quick.push("lrs+1011_3:2_add=large:afp=10000:afq=1.4:amm=sco:anc=none:cond=fast:fde=unused:gsp=input_only:gs=on:ile=on:irw=on:lma=on:nwc=1:stl=30:sac=on:tha=off:updr=off:uhcvi=on_118");
    break;

  case SMT_UFIDL:
    quick.push("dis+11_3_add=large:afp=100000:afq=1.4:amm=off:anc=none:fsr=off:gs=on:ile=on:irw=on:lma=on:nm=32:nwc=1:tha=off:updr=off_2");
    quick.push("dis+10_3_afr=on:afp=1000:afq=1.0:anc=none:cond=on:fsr=off:gs=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:sp=occurrence:urr=on_3");
    break;

  case SMT_LRA:
    quick.push("dis+1011_2:1_add=off:afp=40000:afq=1.1:amm=sco:anc=none:fsr=off:fde=unused:gsp=input_only:ile=on:irw=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sos=all:sp=occurrence:updr=off:uhcvi=on_298");
    quick.push("dis+4_5_av=off:cond=on:fsr=off:gs=on:gsem=on:ile=on:irw=on:lwlo=on:nm=6:nwc=1:sos=on:sp=reverse_arity:updr=off_5");
    quick.push("ott+11_4_av=off:ile=on:lma=on:nm=64:nwc=1:sos=all:sp=occurrence:uwa=interpreted_only:updr=off:uhcvi=on_37");
    quick.push("dis+1_5:1_add=off:afp=40000:afq=1.2:anc=none:bd=off:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=4:sas=z3:updr=off_59");
    quick.push("dis+11_2_add=large:afr=on:afp=1000:afq=1.1:anc=none:gs=on:gsaa=full_model:ile=on:irw=on:lma=on:nm=16:nwc=1:sas=z3:sos=on:sac=on:sp=occurrence:thi=strong:uhcvi=on_72");
    break;

  case SMT_NIA:
    quick.push("dis+11_10_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=10:sas=z3:sac=on:sp=reverse_arity_2");
    break;

  case SMT_UFLRA:
    quick.push("dis+11_4_afp=4000:afq=1.4:amm=sco:anc=none:gs=on:ile=on:lma=on:nm=64:nwc=1.7:sas=z3:sac=on:sp=occurrence_2");
    break;

  case SMT_NRA:
    quick.push("dis+1011_4:1_anc=none:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence_9");
    quick.push("dis+11_2_add=large:afp=10000:afq=1.0:amm=sco:anc=none:gs=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:uwa=all:updr=off_2");
    quick.push("dis+11_3_afr=on:afp=40000:afq=2.0:anc=none:fsr=off:gs=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:urr=on_2");
    quick.push("dis+11_5_av=off:cond=on:fsr=off:ile=on:lwlo=on:nm=64:nwc=3:sp=reverse_arity:updr=off_4");
    quick.push("lrs+1011_3_add=large:afp=1000:afq=1.1:anc=none:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:sac=on_182");
    break;

  case SMT_AUFLIA:
  case SMT_AUFNIA:
    quick.push("lrs+1011_1_add=off:afr=on:afp=1000:afq=1.1:amm=off:anc=none:br=off:bce=on:er=filter:gsp=input_only:gs=on:gsaa=full_model:inw=on:ile=on:nm=32:nwc=1.2:sas=z3:stl=30:uwa=one_side_constant:urr=on_7");
    quick.push("dis+11_3_add=off:afp=1000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:gsaa=full_model:inw=on:ile=on:lcm=predicate:lma=on:nm=6:nwc=1:sas=z3:sos=all:sp=occurrence:tha=off:uhcvi=on_13");
    quick.push("fmb+10_1_av=off:fde=unused:ile=on:irw=on:lcm=predicate:lma=on:nm=16:nwc=1.7:sos=all:sp=reverse_arity_2");
    quick.push("dis+1011_3:2_afp=1000:afq=1.2:anc=none:bd=off:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=off:ile=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:nicw=on:sas=z3:sos=on:sac=on:sp=reverse_arity:urr=ec_only_11");
    quick.push("lrs+1011_12_afr=on:afp=100000:afq=1.4:amm=off:anc=none:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:stl=30:sac=on:tha=off:thf=on:urr=on_9");
    quick.push("dis+1011_3_add=large:afr=on:afp=10000:afq=1.0:anc=all_dependent:bd=off:cond=fast:gsp=input_only:ile=on:irw=on:lma=on:nm=2:newcnf=on:nwc=1.3:nicw=on:sas=z3:sp=reverse_arity:updr=off_16");
    quick.push("dis-10_3_aac=none:acc=model:add=off:afp=100000:afq=1.1:anc=none:bs=unit_only:bce=on:ccuc=small_ones:cond=on:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:lcm=predicate:lma=on:lwlo=on:nm=2:nwc=1.5:sos=on:sp=occurrence:uwa=ground:urr=ec_only:uhcvi=on_9");
    quick.push("lrs+1011_2_add=off:afr=on:afp=4000:afq=1.1:amm=off:bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:irw=on:nm=32:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity:tha=off:urr=ec_only:uhcvi=on_4");
    quick.push("lrs+1_3:2_afr=on:afp=100000:afq=1.0:anc=all_dependent:cond=on:fde=none:gs=on:inw=on:ile=on:irw=on:nm=6:nwc=1:stl=30:sos=theory:updr=off:uhcvi=on_4");
    quick.push("lrs+10_14_add=large:afp=40000:afq=1.1:amm=sco:fde=unused:gs=on:gsem=on:ile=on:lma=on:nm=6:newcnf=on:nwc=1:nicw=on:stl=30:sp=reverse_arity:tha=off:uwa=one_side_interpreted:updr=off:uhcvi=on_28");
    quick.push("dis+11_3_afp=40000:afq=1.4:anc=none:bce=on:fsr=off:gs=on:gsaa=full_model:gsem=off:ile=on:lma=on:nm=64:nwc=1:uhcvi=on_20");
    quick.push("lrs+11_3_av=off:br=off:fsr=off:gs=on:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=1:stl=30:sos=all:sp=reverse_arity:uwa=one_side_interpreted:urr=on:updr=off:uhcvi=on_11");
    quick.push("lrs+1011_5:1_aac=none:add=off:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:bd=preordered:bsr=on:fde=none:gs=on:gsaa=from_current:inw=on:ile=on:irw=on:lcm=predicate:nm=2:newcnf=on:nwc=1:sas=z3:stl=30:tha=off:uwa=ground:updr=off:uhcvi=on_46");
    quick.push("dis-4_4_add=large:afp=1000:afq=1.4:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:ile=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:updr=off:uhcvi=on_10");
    quick.push("lrs+1011_8_aac=none:acc=model:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:ccuc=first:cond=on:fde=none:gs=on:gsaa=from_current:inw=on:ile=on:nm=2:nwc=1:stl=30:sos=on:sp=reverse_arity:tha=off:urr=on_135");
    quick.push("lrs+1011_5_add=large:afp=1000:afq=1.2:amm=off:anc=none:br=off:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=32:nwc=1:sas=z3:stl=30:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_9");
    quick.push("dis+1010_4_afp=40000:afq=1.1:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=on:ile=on:irw=on:nm=4:nwc=1:sp=reverse_arity:uhcvi=on_140");
    quick.push("dis+11_4:1_add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=none:br=off:cond=fast:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=2:nwc=1:sas=z3:ss=axioms:st=3.0:sos=all:sp=occurrence:tha=off:urr=on:updr=off:uhcvi=on_12");
    quick.push("lrs+10_5:1_afp=100000:afq=1.0:bd=preordered:inw=on:ile=on:irw=on:lcm=predicate:nm=6:nwc=1:stl=30:sos=all:sp=reverse_arity:tha=off:uwa=interpreted_only:urr=on:updr=off:uhcvi=on_255");
    quick.push("dis+11_8:1_afp=100000:afq=1.4:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:nwc=1:sos=all:sac=on:urr=on:uhcvi=on_145");
    quick.push("dis+1004_3_acc=on:add=large:afr=on:afp=4000:afq=2.0:amm=off:bs=unit_only:bsr=on:bce=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:urr=ec_only_19");
    quick.push("dis+2_3_aac=none:afr=on:afp=1000:afq=1.1:bsr=on:cond=on:gs=on:gsem=off:lma=on:nm=64:nwc=1:sas=z3:sos=on:sp=occurrence:tha=off:thi=new:urr=ec_only:updr=off:uhcvi=on_23");
    quick.push("lrs+1011_64_add=off:afr=on:afp=1000:afq=1.2:amm=off:anc=all_dependent:bsr=on:bce=on:cond=on:fsr=off:gs=on:inw=on:ile=on:nm=2:newcnf=on:nwc=1.1:sas=z3:stl=30:sac=on:sp=occurrence:tha=off:thi=overlap:updr=off:uhcvi=on_129");
    quick.push("lrs+1003_2_av=off:cond=on:fsr=off:ile=on:nm=2:nwc=1.3:stl=30:sos=on:sp=occurrence:tha=off:updr=off:uhcvi=on_35");
    quick.push("lrs+1010_2:3_aac=none:acc=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bs=unit_only:bce=on:ccuc=first:fsr=off:fde=unused:gs=on:gsem=off:ile=on:nm=4:nwc=1:stl=30:sos=on:sp=reverse_arity:uhcvi=on_148");
    break;

    
  case SMT_AUFNIRA:
    quick.push("dis+11_2_add=large:afp=1000:afq=1.1:anc=none:fsr=off:fde=none:ile=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=all:sac=on_5");
    quick.push("lrs+10_8_afr=on:afp=4000:afq=1.1:amm=sco:anc=none:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sac=on:tha=off:urr=on:updr=off_2");
    quick.push("dis+1002_5_add=large:afr=on:afp=4000:afq=1.4:amm=off:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:sp=occurrence:updr=off_6");
    quick.push("lrs+11_3_afr=on:afp=40000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:inw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sos=all:sac=on:sp=occurrence:updr=off_2");
    quick.push("lrs+1002_3_afr=on:afp=40000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:irw=on:lma=on:nm=2:nwc=1.1:nicw=on:sas=z3:stl=30:sac=on:updr=off:uhcvi=on_7");
    quick.push("lrs+1_3:2_aac=none:afr=on:afp=40000:afq=1.0:anc=none:bs=unit_only:lma=on:nm=64:newcnf=on:nwc=3:sas=z3:stl=30:sd=3:ss=axioms:st=3.0:sp=occurrence:updr=off_15");
    quick.push("lrs+1_3:1_acc=model:add=large:afp=40000:afq=1.2:anc=none:bd=off:bsr=on:ccuc=first:fsr=off:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=2:stl=30:sp=reverse_arity:updr=off_13");
    quick.push("dis+1011_24_av=off:fsr=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sos=all:tha=off:updr=off_8");
    quick.push("lrs+10_24_av=off:bd=off:cond=on:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=2.5:stl=30:sp=occurrence_3");
    quick.push("dis+11_4_add=large:afr=on:afp=40000:afq=1.0:anc=none:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off_2");
    quick.push("lrs+1010_24_afp=40000:afq=2.0:amm=off:anc=none:cond=fast:gs=on:gsem=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:tha=off:thf=on:updr=off:uhcvi=on_2");
    quick.push("lrs+10_3:1_afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=reverse_arity_93");
    quick.push("lrs+11_5:1_add=large:afr=on:afp=1000:afq=1.0:amm=off:anc=none:bd=off:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sac=on:sp=occurrence:tha=off:urr=ec_only_192");
    quick.push("lrs+1004_2_av=off:cond=fast:fsr=off:fde=none:gs=on:gsem=off:lwlo=on:nm=0:nwc=1:stl=30:sp=occurrence:tha=off:thi=new:updr=off:uhcvi=on_79");
    quick.push("lrs+11_2:1_add=large:afr=on:afp=1000:afq=1.4:anc=none:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:stl=30:tha=off:urr=on:uhcvi=on_246");
    quick.push("lrs+1011_8:1_av=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:stl=30:sp=reverse_arity:tha=off:thi=strong:uwa=ground:urr=on:updr=off_74");
    quick.push("lrs+1011_7_av=off:cond=on:gs=on:ile=on:nm=64:nwc=3:stl=30:updr=off_166");
    quick.push("lrs+10_24_add=off:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=1:stl=30:sp=occurrence:tha=off:thf=on_45");
    quick.push("lrs+1003_3:1_av=off:bsr=on:cond=fast:fde=unused:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sos=all:sp=occurrence:tha=off:updr=off:uhcvi=on_125");
    quick.push("ott+1004_8:1_afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:fde=unused:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_146");
    break;

  case SMT_UF:
    quick.push("lrs+11_5_av=off:cond=on:lma=on:nm=64:newcnf=on:nwc=1:stl=30:updr=off_22");
    quick.push("fmb+10_1_av=off:fmbes=contour:fmbsr=1.5:ile=on:updr=off_28");
    quick.push("dis+11_50_add=large:afp=10000:afq=1.2:anc=none:fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sac=on_4");
    quick.push("dis+1010_5:1_av=off:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:urr=on:updr=off_74");
    quick.push("ott+11_1_add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:urr=ec_only_149");
    quick.push("lrs+10_128_add=off:afr=on:amm=sco:anc=none:bsr=on:cond=on:ile=on:irw=on:nm=2:nwc=2:nicw=on:sas=z3:stl=30:updr=off_96");
    quick.push("dis+4_16_acc=model:add=large:afr=on:afp=40000:afq=2.0:amm=off:anc=none:bs=on:ccuc=small_ones:fsr=off:ile=on:nm=4:newcnf=on:nwc=1:nicw=on:sp=reverse_arity_13");
    quick.push("dis+1010_2:1_add=off:afp=10000:afq=2.0:anc=none:cond=on:fde=none:gs=on:gsaa=from_current:gsem=off:ile=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sas=z3:sp=occurrence:uhcvi=on_233");
    quick.push("lrs+11_4:1_av=off:bd=off:bs=unit_only:cond=on:fsr=off:fde=none:ile=on:irw=on:lwlo=on:nm=4:nwc=1.1:stl=30:sp=reverse_arity_127");
    quick.push("ott-10_4_av=off:bd=preordered:fsr=off:fde=none:ile=on:irw=on:nm=2:newcnf=on:nwc=1:updr=off:uhcvi=on_244");
    quick.push("lrs+1010_1_add=off:afp=1000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:ile=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:stl=30:sp=occurrence_192");
    quick.push("ott+1003_5_av=off:bd=off:bs=on:er=known:fde=none:gs=on:gsem=off:ile=on:nwc=2.5:sos=all:sp=occurrence:urr=on_237");
    quick.push("dis+1011_4:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bsr=on:fsr=off:ile=on:nm=64:nwc=5:sas=z3:sp=reverse_arity:urr=ec_only:updr=off_182");
    quick.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=sco:anc=none:cond=on:ep=RSTC:gs=on:gsem=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity_3");
    quick.push("lrs+10_5_add=off:afp=10000:afq=1.0:amm=off:anc=none:bsr=on:fde=unused:gs=on:irw=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:stl=30:sos=on:sac=on:sp=occurrence:updr=off:uhcvi=on_283");
    quick.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=1.4:fde=unused:updr=off_808");
    quick.push("dis+1011_8_av=off:bd=off:bs=unit_only:bsr=on:cond=on:irw=on:nm=64:newcnf=on:nwc=1_250");
    quick.push("dis+10_4:1_afp=10000:afq=1.4:anc=none:bd=off:fsr=off:gsp=input_only:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:urr=on:updr=off_222");
    quick.push("lrs+10_3:1_afr=on:afp=100000:afq=1.4:amm=sco:anc=none:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:stl=30:sac=on:urr=on:uhcvi=on_212");
    quick.push("ott+11_8:1_acc=model:add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:ccuc=first:cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:lwlo=on:nm=2:nwc=1:sos=all:urr=on_155");
    quick.push("lrs+10_3:2_av=off:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:nm=64:newcnf=on:nwc=1:stl=30:sd=2:ss=axioms:st=5.0:sos=all:urr=on_278");
    quick.push("dis+2_3_acc=on:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bs=unit_only:br=off:ccuc=first:cond=on:er=filter:ile=on:nm=6:nwc=1:urr=on_53");
    quick.push("ott+2_4:1_aac=none:add=off:afp=10000:afq=1.1:amm=off:anc=none:bs=on:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity_130");
    quick.push("lrs+1011_3:2_av=off:bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=64:nwc=1.5:sas=z3:stl=30:sp=reverse_arity_222");
    quick.push("dis+11_24_acc=on:afr=on:amm=sco:bsr=on:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:newcnf=on:nwc=1:updr=off_8");
    quick.push("ott+11_3_afr=on:afp=10000:afq=1.4:amm=off:anc=none:bsr=on:cond=on:er=known:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on_2");
    quick.push("lrs+10_8:1_aac=none:afr=on:afp=100000:afq=1.0:amm=off:cond=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1.7:stl=30:sp=reverse_arity:urr=on:updr=off_91");
    quick.push("lrs+11_3:1_av=off:bsr=on:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=64:nwc=1.1:stl=30:sp=reverse_arity:updr=off_22");
    quick.push("lrs+1002_3_av=off:bs=unit_only:bsr=on:ile=on:nm=64:nwc=1:stl=30:sos=theory:sp=reverse_arity_273");
    quick.push("lrs-1_2:3_aac=none:add=off:afr=on:afp=40000:afq=2.0:amm=off:cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:ile=on:irw=on:lwlo=on:nm=2:nwc=1.2:stl=60:sos=theory:sp=occurrence_120");
    quick.push("dis+11_50_aac=none:acc=model:add=large:afr=on:afp=4000:afq=2.0:anc=none:ccuc=first:er=known:fde=unused:gsp=input_only:gs=on:gsaa=full_model:ile=on:irw=on:nm=64:newcnf=on:nwc=4:sac=on:sp=occurrence:updr=off_5");
    quick.push("dis+1011_8:1_av=off:ile=on:lma=on:nm=32:newcnf=on:nwc=1:sp=occurrence_161");
    quick.push("dis+10_3:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:ile=on:nm=2:nwc=2.5:sas=z3:sac=on:sp=occurrence_91");
    quick.push("dis+4_6_av=off:bd=off:bs=on:ile=on:irw=on:lma=on:nm=64:nwc=1_229");
    quick.push("lrs+11_2_add=large:afr=on:amm=sco:anc=none:bsr=on:gs=on:gsem=off:irw=on:lma=on:nm=16:newcnf=on:nwc=1:stl=30:sac=on:sp=occurrence:urr=on:updr=off_270");
    quick.push("dis+1002_5_av=off:cond=on:fsr=off:ile=on:nm=64:newcnf=on:nwc=1.1:sp=reverse_arity_20");
    quick.push("dis+11_5_add=large:afr=on:afp=1000:afq=1.0:anc=none:bsr=on:fsr=off:nm=64:newcnf=on:nwc=1:updr=off_3");
    quick.push("dis+1004_4:1_av=off:br=off:cond=on:ep=RST:fsr=off:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:sp=occurrence:urr=on_69");
    quick.push("lrs+1011_3:2_add=large:afp=100000:afq=1.1:anc=none:br=off:fsr=off:ile=on:irw=on:lwlo=on:nm=64:nwc=1:sas=z3:stl=30:sos=on:sp=occurrence:urr=on_1");
    quick.push("lrs+1011_1_av=off:bd=off:ile=on:irw=on:lma=on:lwlo=on:nm=16:newcnf=on:nwc=1:stl=30:sp=occurrence_110");
    quick.push("lrs+11_3:2_av=off:cond=on:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=64:nwc=1:stl=30:sd=3:ss=axioms:st=5.0:updr=off_78");
    quick.push("dis+1011_8:1_add=off:afp=10000:afq=1.0:amm=off:anc=none:bd=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:updr=off_91");
    quick.push("lrs+11_2_av=off:br=off:ep=R:ile=on:lma=on:nm=64:nwc=1:stl=30:urr=on_72");
    quick.push("ott+10_2:1_av=off:bce=on:cond=fast:fde=none:irw=on:nm=32:newcnf=on:nwc=1:sos=theory:updr=off_207");
    quick.push("ott+11_14_add=large:afp=1000:afq=1.4:amm=off:anc=none:fde=unused:gs=on:gsem=on:irw=on:nm=4:newcnf=on:nwc=1:sac=on:sp=occurrence_292");
    quick.push("dis+1010_4_av=off:bd=off:lma=on:nm=2:newcnf=on:nwc=1:sp=occurrence:updr=off_72");
    quick.push("ott+4_4_av=off:bd=off:er=filter:ile=on:irw=on:lma=on:nm=64:nwc=1:sos=on:sp=reverse_arity:updr=off_140");
    quick.push("dis+1_4_av=off:bd=off:fsr=off:nm=64:newcnf=on:nwc=1:sp=reverse_arity_243");
    quick.push("lrs+10_2:1_av=off:cond=on:fde=none:gs=on:gsem=off:ile=on:irw=on:nm=64:nwc=1:stl=30:sp=occurrence:urr=on_167");
    quick.push("dis+1011_2_acc=model:add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:ccuc=first:cond=on:er=known:fsr=off:ile=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sos=on:sac=on:sp=occurrence_132");
    quick.push("dis+1002_2_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:ep=RS:ile=on:nm=64:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_81");
    quick.push("dis+2_4_acc=on:add=large:afp=100000:afq=1.1:amm=sco:anc=none:ccuc=first:cond=on:fsr=off:gs=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1.1:nicw=on_6");
    quick.push("ins+11_2:3_av=off:cond=fast:fsr=off:gsp=input_only:ile=on:irw=on:igbrr=0.3:igpr=on:igrr=128/1:igrp=200:igrpq=1.3:igs=1003:igwr=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:updr=off:uhcvi=on_195");
    quick.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.0:anc=none:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1.1:sas=z3:stl=30:sac=on:updr=off_221");
    quick.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=2.0:ile=on:nm=2_595");
    quick.push("ott+10_6_add=off:afr=on:afp=1000:afq=1.0:amm=off:bsr=on:cond=on:fsr=off:fde=none:gs=on:gsem=on:ile=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:updr=off_6");
    quick.push("lrs+1_32_av=off:bd=off:bs=unit_only:er=known:gsp=input_only:gs=on:nm=64:newcnf=on:nwc=1.1:stl=30:sos=on:sp=reverse_arity:urr=ec_only_88");
    quick.push("fmb+10_1_av=off:bce=on:fmbes=smt:fmbsr=1.6:fde=none:ile=on:nm=64:updr=off_848");
    quick.push("lrs+1010_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:gsp=input_only:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:nwc=2:stl=30:updr=off_128");
    quick.push("ott+1_5:1_afp=4000:afq=1.1:anc=none:bd=off:cond=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:urr=on:updr=off_154");
    quick.push("ott+2_6_add=large:afr=on:afp=4000:afq=2.0:amm=sco:anc=all:bs=on:bce=on:cond=fast:fde=none:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:sac=on:urr=on:updr=off_4");
    quick.push("fmb+10_1_av=off:bce=on:fmbsr=1.3:fde=none:nm=64:newcnf=on_761");
    quick.push("lrs+1002_3_acc=on:amm=sco:anc=none:ccuc=small_ones:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:stl=30:urr=on_295");
    quick.push("ott+1011_8:1_add=off:afr=on:afp=40000:afq=1.2:amm=off:anc=none:bd=off:fsr=off:ile=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sp=reverse_arity:updr=off_55");
    break;

  case SMT_AUFLIRA:
    quick.push("lrs+1010_2_anc=none:fsr=off:gs=on:irw=on:newcnf=on:nwc=1:sas=z3:stl=30:sos=on:sp=occurrence:updr=off_4");
    quick.push("lrs+1010_4_av=off:bd=off:bs=unit_only:bsr=on:gs=on:inw=on:ile=on:lma=on:newcnf=on:nwc=2.5:stl=30:sp=occurrence:updr=off_6");
    quick.push("dis+2_3_add=off:afp=40000:afq=1.1:anc=none:cond=on:gs=on:inw=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off_43");
    quick.push("dis+1011_4_afr=on:afp=4000:afq=1.4:anc=none:fsr=off:gs=on:gsem=on:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:tha=off:updr=off_8");
    quick.push("lrs+1010_20_add=large:afr=on:afp=4000:afq=1.4:amm=sco:anc=none:bd=preordered:bs=unit_only:fsr=off:fde=unused:gs=on:ile=on:lcm=reverse:nm=2:nwc=4:sas=z3:stl=120:sac=on:sp=occurrence:tha=off:urr=on:updr=off:uhcvi=on_791");
    quick.push("dis+1002_5:4_afr=on:afp=1000:afq=1.2:anc=none:cond=on:ile=on:irw=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:tha=off:updr=off_33");
    quick.push("dis+1011_3_aac=none:afp=1000:afq=1.2:anc=all:fde=none:gs=on:gsem=on:inw=on:ile=on:lcm=predicate:lma=on:nm=6:newcnf=on:nwc=1.5:nicw=on:sas=z3:tha=off:urr=on_344");
    quick.push("dis+1011_32_aac=none:add=off:afr=on:afp=40000:afq=1.0:amm=sco:bs=on:bsr=on:br=off:fde=unused:gs=on:gsaa=full_model:ile=on:lcm=predicate:nm=6:newcnf=on:nwc=1.5:sas=z3:sos=all:sac=on:tha=off:thi=all:uwa=one_side_constant:urr=on_1");
    break;

  case SMT_QF_ABV:
  case SMT_QF_ALIA:
  case SMT_QF_ANIA:
  case SMT_QF_AUFBV:
  case SMT_QF_AUFLIA:
  case SMT_QF_AUFNIA:
  case SMT_QF_AX:
  case SMT_QF_BV:
  case SMT_QF_IDL:
  case SMT_QF_LIA:
  case SMT_QF_LIRA:
  case SMT_QF_LRA:
  case SMT_QF_NIA:
  case SMT_QF_NIRA:
  case SMT_QF_NRA:
  case SMT_QF_RDL:
  case SMT_QF_UF:
  case SMT_QF_UFBV:
  case SMT_QF_UFIDL:
  case SMT_QF_UFLIA:
  case SMT_QF_UFLRA:
  case SMT_QF_UFNIA:
  case SMT_QF_UFNRA:
    throw UserErrorException("Kein Kinderspiel, Bruder, use Z3 for quantifier-free problems!");

  case SMT_ALL:
    break;
  case SMT_BV:
  case SMT_UFBV:
    throw UserErrorException("Sorry, we don't deal with bit-vectors!");
  case SMT_UNDEFINED:
    throw UserErrorException("This version cannot be used with this logic!");
  }

  fallback.push("dis+1002_5:1_aac=none:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:bsr=on:br=off:cond=on:fsr=off:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=32:newcnf=on:nwc=1.1:sas=z3:sp=reverse_arity:tha=off:urr=on_600");
  fallback.push("lrs+1011_3:1_aac=none:add=large:afp=1000:afq=2.0:fsr=off:gs=on:gsaa=from_current:gsem=on:ile=on:nm=4:nwc=1.5:sas=z3:sp=reverse_arity:tha=off:uwa=interpreted_only:uhcvi=on_300");
  fallback.push("ott+1_5:1_afr=on:afp=4000:afq=1.2:amm=off:anc=none:bs=unit_only:br=off:cond=on:fsr=off:gs=on:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:urr=on:updr=off_600");
  fallback.push("dis+1011_4:1_anc=none:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence_300");
  fallback.push("lrs+10_5_add=off:afp=10000:afq=1.0:amm=off:anc=none:bsr=on:fde=unused:gs=on:irw=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sos=on:sac=on:sp=occurrence:updr=off:uhcvi=on_300");
  fallback.push("dis+10_4_add=off:afp=4000:afq=1.1:amm=sco:anc=none:fsr=off:gs=on:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:tha=off:urr=on:updr=off_300");
  fallback.push("lrs-11_2:3_av=off:bd=off:lcm=predicate:lma=on:nm=4:newcnf=on:nwc=1:sp=reverse_arity_600");
  fallback.push("lrs+1_3:1_acc=model:add=large:afp=40000:afq=1.2:anc=none:bd=off:bsr=on:ccuc=first:fsr=off:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=2:sp=reverse_arity:updr=off_300");
  fallback.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=1.4:fde=unused:updr=off_900");
  fallback.push("dis+1011_2:1_add=off:afp=40000:afq=1.1:amm=sco:anc=none:fsr=off:fde=unused:gsp=input_only:ile=on:irw=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sos=all:sp=occurrence:updr=off:uhcvi=on_300");
  fallback.push("ott+1010_2:1_acc=on:add=large:afr=on:afp=40000:afq=1.1:anc=none:gs=on:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=on_300");
  fallback.push("lrs+10_3:1_afr=on:afp=100000:afq=1.4:amm=sco:anc=none:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:sac=on:urr=on:uhcvi=on_300");
  fallback.push("dis+1002_3_add=off:afr=on:amm=off:anc=none:cond=on:ile=on:lma=on:nm=64:nwc=1:nicw=on:sac=on:sp=reverse_arity:tac=axiom:tar=off:updr=off_300");
  fallback.push("dis+2_1_add=large:afr=on:afp=1000:afq=1.2:anc=none:cond=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_300");
  fallback.push("lrs+10_20_add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:bs=unit_only:bce=on:fde=unused:gs=on:gsaa=full_model:gsem=on:ile=on:nm=16:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=all:updr=off_300");
  fallback.push("lrs+1002_5:4_add=large:afr=on:afp=40000:afq=2.0:anc=none:cond=on:inw=on:ile=on:nm=64:nwc=1:sas=z3:sd=10:ss=axioms:tha=off:updr=off_300");
  fallback.push("dis-11_7_add=large:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=6:nwc=10:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("fmb+10_1_av=off:fde=unused:ile=on:irw=on:lcm=predicate:lma=on:nm=16:nwc=1.7:sos=all:sp=reverse_arity_300");
  fallback.push("lrs+10_5:1_av=off:fde=unused:ile=on:lwlo=on:nwc=1.1:sp=occurrence:urr=on_900");
  fallback.push("ott+10_4:1_aac=none:add=off:afp=40000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_300");
  fallback.push("dis+1011_32_av=off:bd=off:inw=on:irw=on:lwlo=on:nm=16:nwc=3:sd=2:ss=axioms:st=5.0:sp=occurrence:tha=off_600");
  fallback.push("dis+1011_2_acc=on:afp=10000:afq=1.1:amm=sco:anc=none:ccuc=small_ones:cond=fast:fde=unused:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:tha=off:updr=off:uhcvi=on_300");
  fallback.push("lrs+1011_3:1_add=large:afr=on:afp=40000:afq=1.0:anc=none:cond=on:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1.1:sas=z3:sac=on:updr=off_300");
  fallback.push("lrs+1011_2_add=off:afr=on:afp=4000:afq=1.1:amm=off:bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:irw=on:nm=32:nwc=1:sas=z3:sos=on:sp=reverse_arity:tha=off:urr=ec_only:uhcvi=on_300");
  fallback.push("ott+1010_7_av=off:fsr=off:fde=none:lma=on:nm=2:newcnf=on:nwc=1.3:sos=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("ott+11_14_add=large:afp=1000:afq=1.4:amm=off:anc=none:fde=unused:gs=on:gsem=on:irw=on:nm=4:newcnf=on:nwc=1:sac=on:sp=occurrence_300");
  fallback.push("ott+11_5:4_aac=none:add=large:afp=4000:afq=1.4:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=on:inw=on:ile=on:nm=2:newcnf=on:nwc=1:sas=z3:sos=on:sp=reverse_arity:urr=on:uhcvi=on_300");
  fallback.push("dis+10_3_afr=on:afp=1000:afq=1.0:anc=none:cond=on:fsr=off:gs=on:ile=on:irw=on:lwlo=on:nm=32:nwc=1:sos=all:sp=occurrence:urr=on_300");
  fallback.push("dis+10_14_add=large:afp=4000:afq=1.1:amm=sco:bs=unit_only:bsr=on:cond=fast:fde=none:inw=on:irw=on:lcm=predicate:nm=4:nwc=1.1:sos=on:sac=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+10_8:1_acc=model:add=large:afr=on:afp=10000:afq=1.4:anc=none:bs=unit_only:ccuc=first:cond=on:er=known:gs=on:gsaa=from_current:ile=on:lcm=reverse:nm=2:nwc=1.2:sac=on:urr=on:uhcvi=on_300");
  fallback.push("lrs+1002_8:1_add=off:afp=1000:afq=1.2:amm=sco:anc=none:bce=on:cond=on:ep=RS:gs=on:gsaa=from_current:ile=on:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:uwa=interpreted_only:updr=off_300");
  fallback.push("lrs+1011_2:1_afr=on:afp=10000:afq=2.0:amm=off:gsp=input_only:gs=on:inw=on:ile=on:nm=2:nwc=1:sas=z3:tha=off_300");
  fallback.push("dis+1003_8_afr=on:anc=none:bd=preordered:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=reverse_arity:updr=off:uhcvi=on_300");
  fallback.push("lrs+1004_5:1_av=off:cond=on:fde=none:irw=on:lcm=reverse:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on:sp=reverse_arity:updr=off:uhcvi=on_600");
  fallback.push("dis+10_4:1_add=off:afp=10000:afq=1.4:amm=sco:anc=none:bd=off:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("ott-3_2:3_add=off:afr=on:afp=40000:afq=1.0:bsr=on:cond=fast:fsr=off:fde=none:gs=on:ile=on:lma=on:lwlo=on:nm=2:newcnf=on:nwc=1.2:nicw=on:sas=z3:sos=all:sp=reverse_arity:urr=ec_only:updr=off_300");
  fallback.push("lrs+1010_2:3_aac=none:acc=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bs=unit_only:bce=on:ccuc=first:fsr=off:fde=unused:gs=on:gsem=off:ile=on:nm=4:nwc=1:sos=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("dis+1002_5:4_add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:updr=off_300");
  fallback.push("lrs+10_8:1_aac=none:afr=on:afp=100000:afq=1.0:amm=off:cond=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1.7:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("lrs+11_1_afr=on:afp=1000:afq=1.1:amm=sco:anc=none:bd=off:cond=on:gsp=input_only:gs=on:ile=on:irw=on:nm=6:nwc=1:sos=all:sac=on:urr=on_300");
  fallback.push("dis+1010_2:1_add=off:afp=10000:afq=2.0:anc=none:cond=on:fde=none:gs=on:gsaa=from_current:gsem=off:ile=on:irw=on:lma=on:lwlo=on:nm=2:nwc=1:sas=z3:sp=occurrence:uhcvi=on_300");
  fallback.push("dis+1011_5:1_afp=4000:afq=1.4:amm=off:anc=none:cond=on:fde=unused:gsp=input_only:ile=on:lma=on:nm=16:nwc=1:sos=on:sac=on:tha=off:urr=ec_only:uhcvi=on_300");
  fallback.push("lrs-4_5:1_add=large:afp=1000:afq=1.0:amm=sco:anc=none:cond=on:fde=unused:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sp=occurrence:uwa=ground:urr=on:updr=off:uhcvi=on_300");
  fallback.push("ott+11_3:2_aac=none:add=large:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:bs=on:bsr=on:br=off:cond=on:fsr=off:gsp=input_only:gs=on:irw=on:lcm=predicate:lma=on:nm=16:nwc=1.5:nicw=on:sas=z3:sac=on:sp=reverse_arity:tha=off:thi=all:urr=on:updr=off_1200");
  fallback.push("dis+1002_2_aac=none:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=all:bsr=on:fde=unused:inw=on:ile=on:lcm=reverse:nm=4:nwc=4:nicw=on:sos=theory:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("lrs+11_5:1_add=large:afr=on:afp=1000:afq=1.0:amm=off:anc=none:bd=off:gs=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off:urr=ec_only_300");
  fallback.push("dis+11_4_afp=4000:afq=1.4:amm=sco:anc=none:gs=on:ile=on:lma=on:nm=64:nwc=1.7:sas=z3:sac=on:sp=occurrence_300");
  fallback.push("lrs-1_1_aac=none:add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:gsp=input_only:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1.5:sas=z3:urr=on_300");
  fallback.push("lrs+1011_3_add=off:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:bce=on:cond=fast:fde=unused:ile=on:lma=on:nm=6:nwc=2:nicw=on:sas=z3:sd=3:ss=axioms:st=2.0:sp=reverse_arity:tha=off_300");
  fallback.push("lrs+11_4:1_add=large:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+11_5:1_add=off:afp=100000:afq=1.1:amm=off:anc=none:bd=off:cond=on:er=known:gs=on:gsem=off:inw=on:ile=on:irw=on:lcm=predicate:lwlo=on:nm=64:newcnf=on:nwc=1.1:sac=on:sp=reverse_arity:tha=off:thf=on_300");
  fallback.push("lrs+10_3:1_av=off:cond=on:fde=none:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sp=reverse_arity:tar=off:uhcvi=on_300");
  fallback.push("dis+1_5:1_add=off:afp=40000:afq=1.2:anc=none:bd=off:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=4:sas=z3:updr=off_300");
  fallback.push("ott+10_6_add=off:afr=on:afp=1000:afq=1.0:amm=off:bsr=on:cond=on:fsr=off:fde=none:gs=on:gsem=on:ile=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:updr=off_300");
  fallback.push("lrs+11_5:1_add=off:afr=on:afp=4000:afq=1.1:anc=none:bsr=on:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("dis+1011_8_afp=10000:afq=1.2:amm=sco:anc=none:bce=on:gs=on:gsem=off:ile=on:lma=on:nm=16:newcnf=on:nwc=2.5:sas=z3:sos=all:sac=on:sp=occurrence:updr=off_300");
  fallback.push("dis+1010_1_add=off:afp=10000:afq=1.1:amm=sco:anc=none:bd=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=6:nwc=1.3:nicw=on:sas=z3:tha=off:urr=ec_only_300");
  fallback.push("lrs+1011_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off_300");
  fallback.push("dis+11_3_add=off:afp=1000:afq=2.0:amm=off:anc=none:fsr=off:gs=on:gsaa=full_model:inw=on:ile=on:lcm=predicate:lma=on:nm=6:nwc=1:sas=z3:sos=all:sp=occurrence:tha=off:uhcvi=on_300");
  fallback.push("ott+11_1_add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:cond=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sp=occurrence:urr=ec_only_300");
  fallback.push("lrs+1010_1_add=off:afp=40000:afq=1.1:amm=off:anc=none:bd=off:fsr=off:inw=on:ile=on:lcm=reverse:lwlo=on:nm=64:nwc=3:nicw=on:sp=occurrence_300");
  fallback.push("ott+1011_3:1_add=off:afp=10000:afq=1.4:amm=off:anc=none:br=off:bce=on:cond=on:fde=unused:gs=on:ile=on:lma=on:nm=4:nwc=1:sp=occurrence:tar=off:urr=on:updr=off_300");
  fallback.push("dis+1003_2:1_afr=on:afp=100000:afq=1.1:anc=none:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:sp=occurrence:tar=off:uhcvi=on_300");
  fallback.push("ott+1011_3:2_av=off:bd=off:bs=on:bce=on:cond=on:fde=unused:ile=on:lma=on:newcnf=on:nwc=1:tha=off:updr=off_300");
  fallback.push("ott+2_4:1_aac=none:add=off:afp=10000:afq=1.1:amm=off:anc=none:bs=on:gs=on:gsem=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity_300");
  fallback.push("dis+1002_2_add=large:afr=on:afp=10000:afq=1.2:amm=sco:anc=none:bs=unit_only:cond=on:ep=RS:ile=on:nm=64:nwc=1:sac=on:sp=reverse_arity:uhcvi=on_300");
  fallback.push("lrs+1011_3_add=large:afr=on:afp=100000:afq=1.0:amm=off:anc=none:fsr=off:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:tar=off_300");
  fallback.push("lrs+1010_1_add=off:afp=1000:afq=1.0:amm=sco:anc=none:cond=on:fsr=off:gsp=input_only:gs=on:ile=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence_300");
  fallback.push("dis+11_8_aac=none:add=large:afp=10000:afq=1.0:amm=sco:anc=none:bs=on:bsr=on:cond=on:er=known:fsr=off:fde=none:ile=on:lcm=predicate:lma=on:nm=32:nwc=1.7:nicw=on:sas=z3:sac=on:sp=occurrence:tha=off:updr=off_900");
  fallback.push("lrs+2_8:1_add=off:afp=40000:afq=1.0:anc=none:fde=unused:gs=on:ile=on:irw=on:lcm=reverse:nm=64:nwc=3:sas=z3:sp=occurrence:urr=on:uhcvi=on_300");
  fallback.push("lrs+10_5:1_afp=100000:afq=1.0:bd=preordered:inw=on:ile=on:irw=on:lcm=predicate:nm=6:nwc=1:sos=all:sp=reverse_arity:tha=off:uwa=interpreted_only:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+1011_4:1_add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:bsr=on:fsr=off:ile=on:nm=64:nwc=5:sas=z3:sp=reverse_arity:urr=ec_only:updr=off_300");
  fallback.push("ott+1010_1_add=large:afp=1000:afq=1.2:anc=none:bd=off:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:updr=off_300");
  fallback.push("ott+1011_3:1_add=off:afp=100000:afq=2.0:amm=off:anc=none:bs=unit_only:gs=on:gsem=on:irw=on:newcnf=on:nwc=1:sas=z3:tha=off_300");
  fallback.push("ott+1_8:1_add=large:afp=10000:afq=1.0:amm=sco:anc=none:bd=off:bsr=on:fsr=off:fde=unused:ile=on:irw=on:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:updr=off:uhcvi=on_300");
  fallback.push("dis+1010_2_acc=on:afr=on:afp=100000:afq=1.2:anc=none:bsr=on:fsr=off:ile=on:irw=on:nm=16:newcnf=on:nwc=4:sp=occurrence:tha=off:urr=ec_only_300");
  fallback.push("lrs+1002_2_add=large:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:inw=on:lwlo=on:nm=32:newcnf=on:nwc=1:sos=theory:sac=on:sp=occurrence:urr=on_300");
  fallback.push("ott+1_10_av=off:ep=RSTC:fsr=off:ile=on:lma=on:newcnf=on:nwc=1:sos=on:tha=off:updr=off_300");
  fallback.push("dis+1004_8_av=off:cond=on:er=filter:fde=unused:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity_300");
  fallback.push("lrs+1010_2_anc=none:fsr=off:gs=on:irw=on:newcnf=on:nwc=1:sas=z3:sos=on:sp=occurrence:updr=off_300");
  fallback.push("lrs+1011_1_add=off:afr=on:afp=1000:afq=1.1:amm=off:anc=none:br=off:bce=on:er=filter:gsp=input_only:gs=on:gsaa=full_model:inw=on:ile=on:nm=32:nwc=1.2:sas=z3:uwa=one_side_constant:urr=on_300");
  fallback.push("ins+11_2:3_av=off:cond=fast:fsr=off:gsp=input_only:ile=on:irw=on:igbrr=0.3:igpr=on:igrr=128/1:igrp=200:igrpq=1.3:igs=1003:igwr=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:updr=off:uhcvi=on_300");
  fallback.push("lrs+11_6_av=off:bd=off:cond=fast:fde=none:lma=on:lwlo=on:nm=0:newcnf=on:nwc=1:sos=on:sp=reverse_arity:updr=off:uhcvi=on_300");
  fallback.push("dis+1010_4_afp=40000:afq=1.1:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=on:ile=on:irw=on:nm=4:nwc=1:sp=reverse_arity:uhcvi=on_300");
  fallback.push("ott+1010_2:1_add=off:afr=on:afp=1000:afq=2.0:amm=off:anc=none:bsr=on:cond=on:ile=on:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=1.5:sac=on:tha=off:updr=off_300");
  fallback.push("lrs+1011_3:2_av=off:bd=off:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=64:nwc=1.5:sas=z3:sp=reverse_arity_300");
  fallback.push("lrs+10_2:1_add=off:afp=4000:afq=2.0:amm=sco:anc=none:bs=unit_only:br=off:cond=on:inw=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:urr=on:updr=off_300");
  fallback.push("lrs+4_6_av=off:bd=off:bs=unit_only:br=off:fsr=off:gsp=input_only:ile=on:irw=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("dis+2_2_afr=on:afp=100000:afq=1.2:amm=off:anc=none:bsr=on:cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=64:nwc=1.1:sas=z3:sac=on:tha=off:updr=off_300");
  fallback.push("lrs+1011_64_add=off:afr=on:afp=1000:afq=1.2:amm=off:anc=all_dependent:bsr=on:bce=on:cond=on:fsr=off:gs=on:inw=on:ile=on:nm=2:newcnf=on:nwc=1.1:sas=z3:sac=on:sp=occurrence:tha=off:thi=overlap:updr=off:uhcvi=on_300");
  fallback.push("lrs+2_4_add=large:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:cond=on:ep=R:gs=on:gsaa=from_current:gsem=off:ile=on:lcm=reverse:lma=on:nm=2:nwc=1.1:sos=on:sac=on:tha=off:updr=off_300");
  fallback.push("dis+11_2_add=large:afp=1000:afq=1.1:anc=none:fsr=off:fde=none:ile=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sos=all:sac=on_300");
  fallback.push("ott+11_3_afr=on:afp=10000:afq=1.4:amm=off:anc=none:bsr=on:cond=on:er=known:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on_300");
  fallback.push("lrs+2_1024_av=off:bd=off:bsr=on:cond=fast:fsr=off:fde=none:ile=on:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:tha=off:thi=overlap:uwa=one_side_constant:updr=off:uhcvi=on_300");
  fallback.push("dis+1011_4_add=large:afr=on:afp=4000:afq=1.4:anc=none:cond=on:ep=RS:fsr=off:gs=on:gsaa=from_current:ile=on:lwlo=on:nm=64:nwc=1:sos=all:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_300");
  fallback.push("lrs+11_2:1_av=off:bd=off:br=off:bce=on:cond=on:fde=none:irw=on:lma=on:nm=2:newcnf=on:nwc=1.1:sp=occurrence:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+11_4_add=off:afp=1000:afq=2.0:amm=sco:anc=none:fsr=off:gs=on:gsem=off:ile=on:nm=64:nwc=1.7:sas=z3:urr=on_300");
  fallback.push("dis+11_5_add=large:afr=on:afp=1000:afq=1.0:anc=none:bsr=on:fsr=off:nm=64:newcnf=on:nwc=1:updr=off_300");
  fallback.push("ott+1011_8:1_afr=on:afp=1000:afq=1.4:amm=sco:anc=none:bd=off:fsr=off:fde=unused:inw=on:ile=on:nm=2:nwc=1:nicw=on:sas=z3:sos=theory:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_600");
  fallback.push("lrs+1_4_add=off:afp=100000:afq=2.0:anc=none:bsr=on:br=off:cond=on:fde=unused:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=2:nwc=1:sas=z3:sos=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("dis+1011_1_afp=40000:afq=1.2:anc=none:cond=on:gsp=input_only:ile=on:irw=on:lma=on:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
  fallback.push("lrs+1011_3_add=large:afp=10000:afq=1.1:amm=off:fde=unused:ile=on:irw=on:lma=on:nwc=1.7:sp=reverse_arity:tha=off:thf=on:updr=off_300");
  fallback.push("dis+1011_12_afp=100000:afq=1.0:amm=sco:anc=none:fsr=off:fde=unused:gsp=input_only:ile=on:irw=on:nm=64:nwc=1.2:sac=on:sp=occurrence:tac=axiom:tar=off:uhcvi=on_300");
  fallback.push("lrs+1011_2:1_av=off:fsr=off:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sp=occurrence:tha=off:urr=ec_only:uhcvi=on_300");
  fallback.push("dis+1011_8:1_add=off:afp=10000:afq=1.0:amm=off:anc=none:bd=off:bsr=on:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sac=on:sp=reverse_arity:updr=off_300");
  fallback.push("dis+1011_4_afr=on:afp=4000:afq=1.4:anc=none:fsr=off:gs=on:gsem=on:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:tha=off:updr=off_300");
  fallback.push("lrs-11_2:1_add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1.5:sas=z3:sp=reverse_arity:urr=on_300");
  fallback.push("dis+1002_5_add=large:afr=on:afp=4000:afq=1.4:amm=off:anc=none:fsr=off:gs=on:gsem=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sos=all:sac=on:sp=occurrence:updr=off_300");
  fallback.push("lrs+1003_3:2_afp=1000:afq=2.0:amm=off:anc=none:cond=on:gs=on:ile=on:lma=on:nm=6:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=all:updr=off_300");
  fallback.push("dis+10_32_add=large:afp=40000:afq=1.0:anc=none:bd=off:bsr=on:fde=none:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sp=occurrence:tha=off:thi=full:uhcvi=on_300");
  fallback.push("dis+11_8:1_afp=100000:afq=1.4:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:nwc=1:sos=all:sac=on:urr=on:uhcvi=on_300");
  fallback.push("lrs+11_1_add=off:afp=100000:afq=1.4:amm=off:anc=none:bsr=on:fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence:updr=off_300");
  fallback.push("ott+11_8:1_acc=model:add=off:afr=on:afp=100000:afq=2.0:amm=off:anc=none:ccuc=first:cond=on:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:lwlo=on:nm=2:nwc=1:sos=all:urr=on_300");
  fallback.push("lrs-2_5:1_acc=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bd=off:cond=fast:gs=on:ile=on:nm=0:newcnf=on:nwc=3:sac=on:thf=on:urr=ec_only_300");
  fallback.push("fmb+10_1_av=off:bce=on:fmbsr=1.3:fde=none:nm=64:newcnf=on_900");
  fallback.push("lrs+11_3_afr=on:afp=40000:afq=1.1:anc=none:fsr=off:gs=on:gsem=off:inw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:updr=off_300");
  fallback.push("ott-4_5:4_aac=none:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:bd=off:bsr=on:br=off:cond=fast:ile=on:irw=on:lcm=reverse:lma=on:nm=64:newcnf=on:nwc=3:thf=on:urr=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+11_5:1_afr=on:afp=10000:afq=1.2:amm=off:anc=none:fsr=off:gs=on:inw=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off_300");
  fallback.push("lrs+1002_5_av=off:cond=fast:fsr=off:fde=unused:gs=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1.7:sp=reverse_arity_300");
  fallback.push("lrs+1010_1_afp=1000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:br=off:cond=on:ile=on:irw=on:nm=2:nwc=1:nicw=on:sas=z3:sos=all:sp=reverse_arity:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+10_8_afr=on:afp=4000:afq=1.1:amm=sco:anc=none:bsr=on:cond=on:gs=on:gsem=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sac=on:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+11_1_av=off:bd=off:bsr=on:cond=on:fsr=off:ile=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_300");
  fallback.push("lrs+1011_2:1_aac=none:add=off:afp=1000:afq=1.0:amm=off:bs=on:gs=on:gsaa=from_current:gsem=on:ile=on:lcm=reverse:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off_300");
  fallback.push("lrs+10_2:3_afr=on:afp=1000:afq=1.1:bd=off:bce=on:cond=on:gsp=input_only:gs=on:gsaa=from_current:inw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:uwa=interpreted_only:updr=off:uhcvi=on_300");
  fallback.push("dis+11_10_add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:cond=on:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=10:sas=z3:sac=on:sp=reverse_arity_300");
  fallback.push("ott+11_4_av=off:ile=on:lma=on:nm=64:nwc=1:sos=all:sp=occurrence:uwa=interpreted_only:updr=off:uhcvi=on_300");
  fallback.push("dis+11_3:1_av=off:br=off:ep=R:fsr=off:gsp=input_only:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:urr=on:uhcvi=on_300");
  fallback.push("dis+10_2_add=off:amm=off:anc=none:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lcm=reverse:nm=2:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off:uhcvi=on_300");
  fallback.push("lrs+1011_1_add=large:afr=on:afp=1000:afq=1.4:anc=none:bd=off:cond=on:ile=on:irw=on:nm=2:nwc=1.7:tha=off_300");
  fallback.push("lrs+11_2:1_add=off:anc=none:bsr=on:br=off:cond=on:er=filter:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=2:nwc=1:sas=z3:sos=all:sac=on:uwa=ground:urr=on_300");
  fallback.push("dis+1004_16_av=off:fsr=off:fde=unused:ile=on:irw=on:nm=0:newcnf=on:nwc=1.1:sp=reverse_arity:urr=on_300");
  fallback.push("lrs+11_4:1_av=off:bd=off:bs=unit_only:cond=on:fsr=off:fde=none:ile=on:irw=on:lwlo=on:nm=4:nwc=1.1:sp=reverse_arity_300");
  fallback.push("dis-1_128_av=off:bs=on:fde=unused:ile=on:irw=on:nm=32:nwc=1.1:sos=all:tha=off:thi=full:uwa=one_side_constant:uhcvi=on_600");
  fallback.push("dis+10_3_add=large:afp=4000:afq=1.4:amm=off:anc=none:cond=on:ep=RS:gs=on:gsaa=from_current:inw=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:sac=on:tha=off:updr=off_300");
  fallback.push("lrs+11_2_av=off:br=off:ep=R:ile=on:lma=on:nm=64:nwc=1:urr=on_300");
  fallback.push("lrs-1_3:1_av=off:bd=off:cond=on:gs=on:ile=on:lcm=reverse:lma=on:nm=32:nwc=1.2:urr=on:updr=off_300");
  fallback.push("ott+1011_5:4_aac=none:add=large:afp=100000:afq=2.0:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:cond=on:gs=on:inw=on:ile=on:irw=on:lma=on:nm=32:nwc=1:nicw=on:sas=z3:sos=on:sp=occurrence:tha=off:updr=off:uhcvi=on_300");
  fallback.push("lrs+1010_1_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:fsr=off:inw=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on_300");
  fallback.push("lrs-11_5:4_afp=4000:afq=1.4:amm=sco:anc=none:bd=off:br=off:gs=on:gsem=off:inw=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sp=occurrence:urr=on_300");
  fallback.push("lrs+10_24_afp=4000:afq=2.0:bd=off:bsr=on:bce=on:cond=fast:fsr=off:gsp=input_only:gs=on:gsem=on:inw=on:ile=on:nwc=1.3:sp=occurrence:tha=off:uwa=one_side_constant:urr=ec_only_300");
  fallback.push("ott+1004_3_av=off:fde=none:gs=on:gsem=on:ile=on:nm=0:nwc=1.3:sp=reverse_arity:tha=off:thi=overlap:urr=ec_only:updr=off_300");
  fallback.push("lrs+10_24_av=off:bd=off:cond=on:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=2.5:sp=occurrence_300");
  fallback.push("lrs+11_5:1_afr=on:afp=100000:afq=1.4:anc=none:cond=fast:fsr=off:ile=on:irw=on:nm=64:nwc=5:nicw=on:sas=z3:sp=reverse_arity:tha=off:thi=all:uwa=one_side_interpreted:updr=off_600");
  fallback.push("lrs+11_2_add=large:afr=on:amm=sco:anc=none:bsr=on:gs=on:gsem=off:irw=on:lma=on:nm=16:newcnf=on:nwc=1:sac=on:sp=occurrence:urr=on:updr=off_300");
  fallback.push("dis+1010_2_add=off:afr=on:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:fsr=off:fde=none:ile=on:lcm=reverse:lma=on:nm=64:nwc=1:nicw=on:sas=z3:sp=reverse_arity_300");
  fallback.push("lrs-10_3_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=unused:gs=on:inw=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.2:sas=z3:tha=off:urr=ec_only_300");
  fallback.push("dis-10_3_aac=none:acc=model:add=off:afp=100000:afq=1.1:anc=none:bs=unit_only:bce=on:ccuc=small_ones:cond=on:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:lcm=predicate:lma=on:lwlo=on:nm=2:nwc=1.5:sos=on:sp=occurrence:uwa=ground:urr=ec_only:uhcvi=on_300");
  fallback.push("dis-11_4:1_aac=none:add=large:afp=4000:afq=1.2:anc=none:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:nm=64:nwc=1:sas=z3:sp=occurrence_300");
  fallback.push("lrs+1_5:1_add=off:afr=on:afp=40000:afq=2.0:amm=off:anc=none:cond=on:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity_300");
  fallback.push("lrs+1002_3_av=off:bs=unit_only:bsr=on:ile=on:nm=64:nwc=1:sos=theory:sp=reverse_arity_300");
  fallback.push("dis+1011_8_av=off:bd=off:bs=unit_only:bsr=on:cond=on:irw=on:nm=64:newcnf=on:nwc=1_300");
  fallback.push("lrs+10_2:1_add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bs=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sac=on:tac=axiom:tar=off:tha=off:uwa=ground:uhcvi=on_300");
  fallback.push("lrs-1_128_aac=none:add=off:afp=40000:afq=1.0:amm=off:anc=none:fsr=off:inw=on:ile=on:lcm=reverse:lma=on:nm=16:nwc=10:sas=z3:sac=on:updr=off_300");
  fallback.push("dis+1010_1_av=off:lma=on:newcnf=on:nwc=1:sd=4:ss=axioms:sos=on:sp=reverse_arity_300");
  fallback.push("dis+1004_1_add=off:afr=on:afp=1000:afq=1.1:amm=off:anc=none:bd=off:fde=none:gs=on:gsaa=from_current:gsem=on:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tac=light:tar=off:tha=off:thi=all:urr=on:uhcvi=on_300");
  fallback.push("lrs+11_8:1_add=large:afp=100000:afq=2.0:amm=off:anc=none:bsr=on:cond=fast:gs=on:gsaa=full_model:inw=on:ile=on:lcm=predicate:nm=4:newcnf=on:nwc=1:sp=reverse_arity:tha=off:urr=on_300");
  fallback.push("dis+10_3_add=off:afp=100000:afq=1.4:amm=sco:anc=none:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sp=reverse_arity:tha=off:thf=on:updr=off_300");
  fallback.push("dis+1011_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:cond=fast:fde=none:gs=on:gsem=off:ile=on:lma=on:lwlo=on:nm=4:nwc=1:sac=on:tha=off:updr=off:uhcvi=on_300");
  fallback.push("dis+1011_1_av=off:fsr=off:fde=unused:gsp=input_only:ile=on:irw=on:lma=on:nwc=1:sos=on:sp=reverse_arity:urr=ec_only_300");
  fallback.push("lrs+11_5:4_av=off:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=32:newcnf=on:nwc=1.3:sp=reverse_arity:updr=off_300");
  fallback.push("dis+11_5_add=large:afr=on:afp=10000:afq=1.2:anc=none:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=all:sac=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("dis+1011_8:1_av=off:ile=on:lma=on:nm=32:newcnf=on:nwc=1:sp=occurrence_300");
  fallback.push("dis+1010_5_av=off:bsr=on:cond=fast:fde=unused:ile=on:nm=6:nwc=1:uhcvi=on_600");
  fallback.push("ott+1_1_av=off:bsr=on:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:ile=on:lma=on:nm=4:newcnf=on:nwc=1:sp=occurrence:urr=on_600");
  fallback.push("ott+1003_5_av=off:bd=off:bs=on:er=known:fde=none:gs=on:gsem=off:ile=on:nwc=2.5:sos=all:sp=occurrence:urr=on_300");
  fallback.push("lrs-11_4:1_afp=100000:afq=1.2:amm=off:anc=all_dependent:bs=unit_only:fsr=off:fde=none:gs=on:gsem=on:ile=on:lma=on:nm=64:nwc=1:sp=reverse_arity:updr=off:uhcvi=on_300");
  fallback.push("dis+11_7_add=large:afr=on:afp=10000:afq=1.2:bd=off:bsr=on:cond=on:fsr=off:fde=unused:gs=on:ile=on:lcm=predicate:lma=on:nm=2:newcnf=on:nwc=3:sos=on:sp=reverse_arity:tha=off:updr=off_300");
  fallback.push("lrs+1011_10_av=off:cond=fast:er=filter:fsr=off:fde=none:gs=on:gsem=on:ile=on:lma=on:nm=4:nwc=1:sos=all:sp=reverse_arity:tha=off:thi=new:uwa=ground:updr=off:uhcvi=on_300");
  fallback.push("lrs+1010_20_add=large:afr=on:afp=4000:afq=1.4:amm=sco:anc=none:bd=preordered:bs=unit_only:fsr=off:fde=unused:gs=on:ile=on:lcm=reverse:nm=2:nwc=4:sas=z3:sac=on:sp=occurrence:tha=off:urr=on:updr=off:uhcvi=on_1200");
  fallback.push("dis+11_3_add=large:afp=100000:afq=1.4:amm=off:anc=none:fsr=off:gs=on:ile=on:irw=on:lma=on:nm=32:nwc=1:tha=off:updr=off_300");
  fallback.push("lrs+1011_8_aac=none:acc=model:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:ccuc=first:cond=on:fde=none:gs=on:gsaa=from_current:inw=on:ile=on:nm=2:nwc=1:sos=on:sp=reverse_arity:tha=off:urr=on_300");
  fallback.push("dis+11_3_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:bce=on:cond=on:fsr=off:ile=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:thf=on_300");
  fallback.push("lrs+4_8:1_afr=on:afp=4000:afq=2.0:amm=sco:anc=none:er=filter:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:lma=on:nm=2:nwc=1:sos=all:tha=off_300");
  fallback.push("lrs+1003_8:1_av=off:fsr=off:fde=unused:gsp=input_only:gs=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on_300");
  fallback.push("lrs+11_10_add=off:afp=40000:afq=1.2:amm=sco:anc=all_dependent:bd=off:bce=on:cond=fast:gsp=input_only:inw=on:lma=on:nm=4:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:urr=ec_only:uhcvi=on_300");
  fallback.push("dis+1010_3_afp=10000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=off:inw=on:ile=on:irw=on:nm=64:nwc=1:sas=z3:tha=off:urr=on_300");
  fallback.push("ott+1003_12_add=large:anc=all:bd=preordered:bce=on:fde=none:lcm=reverse:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:uwa=ground_600");
  fallback.push("dis-3_7_av=off:bs=unit_only:bsr=on:cond=on:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sp=occurrence:tha=off:thi=overlap:uwa=interpreted_only:uhcvi=on_300");
  fallback.push("lrs+11_5_av=off:cond=on:lma=on:nm=64:newcnf=on:nwc=1:updr=off_300");
  fallback.push("lrs+1011_7_av=off:cond=on:gs=on:ile=on:nm=64:nwc=3:updr=off_300");
  fallback.push("ott-1_24_av=off:bd=off:cond=fast:er=known:fsr=off:fde=unused:gsp=input_only:irw=on:lma=on:lwlo=on:nm=0:newcnf=on:nwc=1.3:sp=occurrence:tha=off:thi=new:uhcvi=on_300");
  fallback.push("dis+1011_2:3_add=large:afr=on:afp=40000:afq=1.0:anc=none:br=off:cond=on:gs=on:gsem=on:ile=on:irw=on:lma=on:lwlo=on:nwc=1:sos=on:sac=on:sp=occurrence:tac=axiom:tar=off:urr=on:updr=off_300");
  fallback.push("lrs+1002_1_add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:fsr=off:gsp=input_only:inw=on:ile=on:lcm=predicate:lwlo=on:nm=64:nwc=1.7:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on_300");
  fallback.push("fmb+10_1_av=off:fmbes=contour:fmbsr=1.3:ile=on:nm=2:newcnf=on:updr=off_300");
  fallback.push("lrs+1010_8:1_av=off:br=off:cond=on:fsr=off:gsp=input_only:gs=on:gsem=off:inw=on:ile=on:irw=on:lma=on:nm=4:nwc=5:sos=on:tha=off:thf=on:urr=on_300");
  fallback.push("lrs+1002_3_afr=on:afp=40000:afq=2.0:anc=none:cond=fast:fsr=off:gs=on:gsaa=full_model:gsem=off:inw=on:ile=on:irw=on:lma=on:nm=2:nwc=1.1:nicw=on:sas=z3:sac=on:updr=off:uhcvi=on_300");
  fallback.push("dis+10_2:1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:cond=on:fsr=off:gs=on:gsem=on:inw=on:ile=on:irw=on:nm=2:nwc=1.1:nicw=on:sas=z3:sos=theory:urr=on:updr=off_300");
  fallback.push("lrs+11_8:1_av=off:cond=on:fde=none:ile=on:nm=16:nwc=1.3:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("lrs+1002_3_acc=on:amm=sco:anc=none:ccuc=small_ones:gs=on:gsem=on:ile=on:lma=on:nm=2:newcnf=on:nwc=1:nicw=on:urr=on_300");
  fallback.push("lrs+1011_12_afr=on:afp=100000:afq=1.4:amm=off:anc=none:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:tha=off:thf=on:urr=on_300");
  fallback.push("lrs+2_8_av=off:bsr=on:cond=on:fsr=off:ile=on:lma=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+1_8:1_av=off:br=off:fsr=off:fde=none:gsp=input_only:ile=on:lma=on:nm=2:nwc=1:sos=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("dis+1011_3:2_afp=1000:afq=1.2:anc=none:bd=off:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=off:ile=on:irw=on:lma=on:lwlo=on:nm=6:nwc=1:nicw=on:sas=z3:sos=on:sac=on:sp=reverse_arity:urr=ec_only_300");
  fallback.push("lrs+1010_24_afp=40000:afq=2.0:amm=off:anc=none:cond=fast:gs=on:gsem=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:tha=off:thf=on:updr=off:uhcvi=on_300");
  fallback.push("dis+10_1_add=off:afp=4000:afq=1.4:amm=sco:anc=none:cond=on:ep=RSTC:gs=on:gsem=on:ile=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity_300");
  fallback.push("lrs+1004_2_av=off:cond=fast:fsr=off:fde=none:gs=on:gsem=off:lwlo=on:nm=0:nwc=1:sp=occurrence:tha=off:thi=new:updr=off:uhcvi=on_300");
  fallback.push("dis+1004_4:1_av=off:br=off:cond=on:ep=RST:fsr=off:ile=on:lma=on:nm=2:newcnf=on:nwc=1.1:sp=occurrence:urr=on_300");
  fallback.push("lrs+11_2_av=off:cond=on:fsr=off:gs=on:gsem=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on_300");
  fallback.push("ott+1011_2:3_av=off:bs=unit_only:bce=on:cond=on:fsr=off:fde=unused:gs=on:gsem=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thi=all:uwa=all:urr=on:uhcvi=on_300");
  fallback.push("lrs+10_3_av=off:bs=unit_only:bce=on:cond=on:fde=unused:gsp=input_only:gs=on:inw=on:irw=on:nm=0:newcnf=on:nwc=1.1:tha=off:uhcvi=on_300");
  fallback.push("dis+1011_10_av=off:bd=off:cond=fast:er=known:inw=on:ile=on:irw=on:lma=on:nwc=1.7:sp=occurrence:tha=off:uhcvi=on_300");
  fallback.push("lrs+10_14_add=large:afp=40000:afq=1.1:amm=sco:fde=unused:gs=on:gsem=on:ile=on:lma=on:nm=6:newcnf=on:nwc=1:nicw=on:sp=reverse_arity:tha=off:uwa=one_side_interpreted:updr=off:uhcvi=on_300");
  fallback.push("ott+1011_2:3_add=large:afr=on:afp=40000:afq=2.0:anc=none:br=off:bce=on:cond=fast:fde=unused:irw=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1.1:sp=reverse_arity:tha=off:urr=on:updr=off_300");
  fallback.push("dis+2_3_add=off:afp=40000:afq=1.1:anc=none:cond=on:gs=on:inw=on:ile=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+1011_2:1_acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=preordered:ccuc=first:cond=fast:fsr=off:fde=unused:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=2:nicw=on:sp=occurrence:urr=ec_only:updr=off_300");
  fallback.push("ott+1011_5_av=off:fde=unused:gsp=input_only:gs=on:gsem=off:ile=on:nm=32:nwc=1.3:sas=z3:sp=reverse_arity:tha=off:uwa=ground_300");
  fallback.push("lrs+10_50_av=off:cond=fast:fde=none:lcm=reverse:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:uhcvi=on_300");
  fallback.push("dis+1011_3:1_add=off:afr=on:afp=40000:afq=1.1:amm=sco:bd=off:bce=on:cond=fast:gsp=input_only:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=4:nwc=1.5:sas=z3:sos=all:sp=occurrence:tha=off:uwa=interpreted_only:uhcvi=on_300");
  fallback.push("dis+10_3:1_add=large:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:cond=on:ile=on:nm=2:nwc=2.5:sas=z3:sac=on:sp=occurrence_300");
  fallback.push("lrs+1003_3:1_av=off:bsr=on:cond=fast:fde=unused:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sos=all:sp=occurrence:tha=off:updr=off:uhcvi=on_300");
  fallback.push("ott+10_4:1_afp=100000:afq=1.1:anc=none:bd=off:inw=on:ile=on:irw=on:lma=on:nm=4:nwc=1:sos=all:sac=on:sp=occurrence:tha=off:urr=on:updr=off_300");
  fallback.push("dis+11_3_av=off:cond=on:fsr=off:ile=on:irw=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:updr=off:uhcvi=on_300");
  fallback.push("lrs+4_4_av=off:bd=off:bs=unit_only:cond=fast:fsr=off:fde=unused:gs=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=occurrence:tha=off:thf=on:updr=off_300");
  fallback.push("dis+11_32_add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=none:bd=off:er=filter:ile=on:lcm=predicate:lma=on:newcnf=on:nwc=5:sp=occurrence:updr=off_300");
  fallback.push("lrs+2_4_add=off:afr=on:afp=40000:afq=1.0:amm=off:anc=none:bd=off:bce=on:fde=none:gs=on:gsem=on:lma=on:nm=64:newcnf=on:nwc=1.3:sas=z3:tha=off:thi=strong:uwa=one_side_interpreted:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+1010_1_av=off:ile=on:irw=on:nm=2:nwc=1:sas=z3:sp=occurrence:tar=off:urr=on_300");
  fallback.push("lrs+1010_1_av=off:fde=unused:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sos=all:sp=reverse_arity:updr=off_300");
  fallback.push("lrs+10_3:2_av=off:bce=on:cond=on:er=filter:fsr=off:fde=unused:gs=on:gsem=on:ile=on:irw=on:nm=6:nwc=1:sos=all:tac=light:tar=off:updr=off:uhcvi=on_300");
  fallback.push("lrs+11_7_add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:bsr=on:br=off:fde=unused:gs=on:gsem=on:inw=on:ile=on:irw=on:lma=on:nm=64:nwc=1:sos=all:sac=on:urr=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+10_24_add=off:afp=100000:afq=1.2:amm=sco:anc=none:cond=on:fsr=off:gs=on:ile=on:nm=64:nwc=1:sp=occurrence:tha=off:thf=on_300");
  fallback.push("lrs-2_3:1_add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=on:er=filter:fde=unused:ile=on:irw=on:nm=64:newcnf=on:nwc=1.1:sas=z3:sac=on:sp=reverse_arity:tha=off:thf=on:thi=strong:uhcvi=on_600");
  fallback.push("dis+11_5:4_aac=none:acc=on:afp=40000:afq=2.0:amm=sco:anc=none:bd=off:cond=fast:fsr=off:fde=none:lcm=reverse:nm=2:newcnf=on:nwc=1.1:tha=off:thi=strong:uwa=interpreted_only:uhcvi=on_300");
  fallback.push("lrs+2_3:2_av=off:cond=fast:inw=on:ile=on:nm=2:nwc=1:sos=theory:urr=on_300");
  fallback.push("lrs+1011_5_add=large:afp=1000:afq=1.2:amm=off:anc=none:br=off:fsr=off:gs=on:gsem=on:inw=on:ile=on:lma=on:nm=32:nwc=1:sas=z3:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_300");
  fallback.push("dis+1011_5:1_afr=on:afp=10000:afq=1.2:amm=sco:bd=preordered:bs=unit_only:cond=on:fsr=off:inw=on:irw=on:lcm=reverse:nm=4:newcnf=on:nwc=1.1:sd=7:ss=axioms:st=1.2:tha=off:uhcvi=on_300");
  fallback.push("ott+4_4_av=off:bd=off:er=filter:ile=on:irw=on:lma=on:nm=64:nwc=1:sos=on:sp=reverse_arity:updr=off_300");
  fallback.push("fmb+10_1_av=off:bce=on:fmbes=contour:fmbsr=2.0:ile=on:nm=2_600");
  fallback.push("ott+2_6_add=large:afr=on:afp=4000:afq=2.0:amm=sco:anc=all:bs=on:bce=on:cond=fast:fde=none:gs=on:gsem=off:ile=on:nm=64:newcnf=on:nwc=1:sac=on:urr=on:updr=off_300");
  fallback.push("dis+10_4:1_afp=10000:afq=1.4:anc=none:bd=off:fsr=off:gsp=input_only:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("lrs+11_8_afr=on:afp=1000:afq=2.0:amm=sco:anc=none:bsr=on:fsr=off:gs=on:gsem=off:inw=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:tha=off:urr=ec_only:updr=off_300");
  fallback.push("lrs+10_5:4_av=off:bd=off:fsr=off:fde=none:lcm=reverse:lma=on:newcnf=on:nwc=1:tha=off:urr=on:updr=off_300");
  fallback.push("lrs+1011_5:4_aac=none:add=off:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:bd=off:bs=on:bsr=on:cond=on:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:inw=on:ile=on:nm=16:nwc=1:sas=z3:sos=on:sp=reverse_arity:tha=off:uwa=interpreted_only:uhcvi=on_300");
  fallback.push("dis+1_4:1_acc=on:add=large:afp=4000:afq=1.2:amm=sco:anc=none:ccuc=small_ones:ile=on:lwlo=on:nm=64:nwc=1:tha=off:urr=ec_only:updr=off_300");
  fallback.push("lrs+10_2:1_av=off:cond=on:fde=none:gs=on:gsem=off:ile=on:irw=on:nm=64:nwc=1:sp=occurrence:urr=on_300");
  fallback.push("ott+11_5:4_aac=none:add=large:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:bce=on:cond=on:fsr=off:fde=unused:ile=on:irw=on:lma=on:nm=6:newcnf=on:nwc=1:nicw=on:sas=z3:tha=off:updr=off_300");
  fallback.push("dis+11_3_afr=on:afp=40000:afq=2.0:anc=none:fsr=off:gs=on:lma=on:nm=64:newcnf=on:nwc=1:nicw=on:sas=z3:sos=all:sac=on:sp=occurrence:urr=on_300");
  fallback.push("lrs+1002_2:1_aac=none:add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=none:cond=fast:gs=on:nm=64:newcnf=on:nwc=1.5:sas=z3:sp=occurrence:updr=off_300");
  fallback.push("lrs-11_5_add=off:afr=on:afp=100000:afq=1.0:anc=all:bs=unit_only:cond=fast:fsr=off:ile=on:irw=on:lcm=reverse:lwlo=on:nwc=1.7:nicw=on:sos=on:sac=on:sp=reverse_arity:tha=off:urr=on_300");
  fallback.push("dis+1003_4:1_add=large:afp=10000:afq=1.4:amm=off:anc=none:bd=off:cond=fast:fsr=off:fde=none:gs=on:ile=on:lma=on:nm=64:nwc=1.2:sas=z3:sp=reverse_arity:tha=off:urr=ec_only_300");
  fallback.push("lrs+1002_1_av=off:bd=off:bsr=on:cond=on:ile=on:lma=on:nm=64:nwc=1:sos=on:sp=reverse_arity_300");
  fallback.push("lrs+1010_8:1_add=off:afp=10000:afq=1.0:amm=sco:anc=none:cond=on:gsp=input_only:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=64:nwc=2:updr=off_300");
  fallback.push("dis+1004_3_acc=on:add=large:afr=on:afp=4000:afq=2.0:amm=off:bs=unit_only:bsr=on:bce=on:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:urr=ec_only_300");
  fallback.push("ott-10_4_av=off:bd=preordered:fsr=off:fde=none:ile=on:irw=on:nm=2:newcnf=on:nwc=1:updr=off:uhcvi=on_600");
  fallback.push("lrs+11_8:1_afp=100000:afq=1.4:amm=off:anc=all:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:lcm=reverse:nm=64:nwc=2:nicw=on:sac=on:sp=occurrence:urr=on:updr=off_300");
  fallback.push("dis+1003_28_acc=model:add=large:afp=10000:afq=1.1:amm=off:anc=none:bd=off:ccuc=first:fsr=off:gs=on:gsaa=from_current:ile=on:lma=on:nm=2:nwc=1:sos=all:sp=occurrence:tha=off:uwa=ground:uhcvi=on_300");
  fallback.push("lrs+1002_1_aac=none:add=large:afr=on:afp=4000:afq=2.0:amm=off:anc=none:ile=on:irw=on:lma=on:nm=4:newcnf=on:nwc=1:sac=on:sp=occurrence:tha=off:updr=off_300");
  fallback.push("dis+1002_14_av=off:cond=fast:fde=unused:inw=on:ile=on:lma=on:nm=0:nwc=1:sos=all:sp=reverse_arity:tha=off:uwa=one_side_interpreted:uhcvi=on_300");
  fallback.push("lrs+1_3:2_aac=none:afr=on:afp=40000:afq=1.0:anc=none:bs=unit_only:lma=on:nm=64:newcnf=on:nwc=3:sas=z3:sd=3:ss=axioms:st=3.0:sp=occurrence:updr=off_300");
  fallback.push("lrs+1010_3:1_av=off:bd=off:bsr=on:irw=on:nm=64:newcnf=on:nwc=1.7:sos=all:updr=off_300");
  fallback.push("lrs-11_8:1_afr=on:afp=1000:afq=1.4:amm=off:anc=none:bd=off:bs=on:gs=on:ile=on:lcm=predicate:lma=on:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only_300");
  fallback.push("dis-4_4_add=large:afp=1000:afq=1.4:amm=off:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:ile=on:irw=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:updr=off:uhcvi=on_300");
  fallback.push("ott+1002_2:1_add=large:afr=on:afp=100000:afq=1.1:anc=none:cond=on:fsr=off:fde=none:gs=on:gsaa=from_current:irw=on:lcm=reverse:nm=64:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:updr=off_300");
  fallback.push("lrs-11_4_acc=on:afr=on:afp=40000:afq=1.4:amm=off:anc=none:br=off:bce=on:cond=fast:fde=none:gs=on:ile=on:irw=on:nm=0:newcnf=on:nwc=1.1:nicw=on:sac=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("dis+1010_1_afr=on:afp=40000:afq=2.0:amm=off:anc=none:gs=on:ile=on:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off_300");
  fallback.push("lrs+1010_5:4_afp=100000:afq=1.2:anc=none:cond=on:fsr=off:ile=on:irw=on:nm=64:nwc=1:sac=on:sp=occurrence:urr=on_300");
  fallback.push("ott+1_5:1_av=off:bs=unit_only:br=off:gs=on:gsem=off:inw=on:ile=on:nm=64:newcnf=on:nwc=1:sd=4:ss=axioms:st=1.5:sp=occurrence:tha=off:urr=on:uhcvi=on_300");
  fallback.push("dis+1002_1_add=large:afp=4000:afq=1.2:anc=none:cond=on:fsr=off:gs=on:gsem=off:ile=on:lma=on:nm=64:nwc=1:sas=z3:sac=on:sp=occurrence:tha=off:thi=strong:uwa=interpreted_only:uhcvi=on_300");
  fallback.push("dis+11_5_av=off:cond=on:fsr=off:ile=on:lwlo=on:nm=64:nwc=3:sp=reverse_arity:updr=off_300");
  fallback.push("lrs+1011_2:3_av=off:bsr=on:cond=fast:fsr=off:gsp=input_only:ile=on:irw=on:lwlo=on:nm=64:newcnf=on:nwc=1:tha=off:updr=off_300");
  fallback.push("lrs+1002_2:1_add=large:afp=100000:afq=1.2:amm=off:anc=none:cond=fast:fde=unused:gs=on:gsaa=from_current:gsem=on:ile=on:nm=64:newcnf=on:nwc=1:sas=z3:sd=5:ss=axioms:st=1.2:tha=off:uwa=ground_300");
  fallback.push("lrs+1_32_av=off:bd=off:bs=unit_only:er=known:gsp=input_only:gs=on:nm=64:newcnf=on:nwc=1.1:sos=on:sp=reverse_arity:urr=ec_only_300");
  fallback.push("ott+1_5:1_afp=4000:afq=1.1:anc=none:bd=off:cond=on:ile=on:nm=64:nwc=1:sas=z3:sac=on:sp=reverse_arity:urr=on:updr=off_300");
  fallback.push("lrs+1011_1_av=off:bd=off:ile=on:irw=on:lma=on:lwlo=on:nm=16:newcnf=on:nwc=1:sp=occurrence_300");
  fallback.push("lrs+11_3:1_av=off:bsr=on:cond=on:fsr=off:ile=on:irw=on:lma=on:nm=64:nwc=1.1:sp=reverse_arity:updr=off_300");
  fallback.push("dis+2_4_afp=10000:afq=1.1:bd=off:bs=on:cond=on:er=filter:ile=on:newcnf=on:nwc=1:sos=on:sac=on:sp=reverse_arity:urr=ec_only:updr=off_300");
  fallback.push("ott-1_3:1_av=off:bsr=on:br=off:bce=on:cond=on:fsr=off:fde=unused:irw=on:nm=2:newcnf=on:nwc=2.5:sos=on:sp=occurrence:tha=off:thi=overlap:urr=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+10_3:2_av=off:bd=off:bsr=on:cond=on:fsr=off:gs=on:gsem=off:nm=64:newcnf=on:nwc=1:sd=2:ss=axioms:st=5.0:sos=all:urr=on_300");
  fallback.push("ott+10_2:1_av=off:bd=off:br=off:cond=fast:fsr=off:fde=none:gs=on:gsem=off:irw=on:nm=64:newcnf=on:nwc=1:sos=all:urr=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+4_3_av=off:bd=preordered:fde=none:inw=on:ile=on:nm=64:newcnf=on:nwc=1:tha=off:thf=on:updr=off:uhcvi=on_300");
  fallback.push("dis+1011_8:1_add=off:afp=10000:afq=1.1:anc=none:bce=on:er=filter:gs=on:gsaa=from_current:gsem=off:inw=on:ile=on:lma=on:nm=2:nwc=3:sac=on:urr=on:updr=off_300");
  fallback.push("dis+10_1_afp=10000:afq=1.0:amm=sco:anc=none:bce=on:fde=none:gs=on:gsem=off:inw=on:ile=on:irw=on:lma=on:nm=16:newcnf=on:nwc=1:sas=z3:sos=on:sac=on:sp=occurrence:tha=off:thi=full_300");
  fallback.push("lrs+1004_1_aac=none:add=off:afr=on:afp=10000:afq=1.0:amm=sco:anc=all_dependent:bd=off:cond=fast:fsr=off:gs=on:gsaa=from_current:lcm=reverse:nm=0:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off:thf=on:urr=on:updr=off_300");
  fallback.push("lrs-10_3:2_aac=none:add=off:afr=on:afp=4000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:fsr=off:ile=on:irw=on:lcm=reverse:lma=on:lwlo=on:nm=16:nwc=1:nicw=on:sas=z3:sd=2:ss=axioms:sos=on:sp=occurrence:updr=off_600");
  fallback.push("dis+11_50_aac=none:acc=model:add=large:afr=on:afp=4000:afq=2.0:anc=none:ccuc=first:er=known:fde=unused:gsp=input_only:gs=on:gsaa=full_model:ile=on:irw=on:nm=64:newcnf=on:nwc=4:sac=on:sp=occurrence:updr=off_300");
  fallback.push("lrs+4_8:1_av=off:cond=on:gs=on:gsem=on:irw=on:nm=64:newcnf=on:nwc=1.1:sp=occurrence:tha=off:urr=on:updr=off_300");
  fallback.push("lrs-11_4:1_add=large:afp=1000:afq=1.1:amm=sco:bs=on:cond=on:gs=on:gsem=on:ile=on:nm=2:newcnf=on:nwc=1:sas=z3:sos=on:sp=occurrence:updr=off_300");
  fallback.push("dis+1010_6_av=off:cond=on:er=filter:fsr=off:nm=64:newcnf=on:nwc=1.3:sp=reverse_arity_300");
  fallback.push("lrs+10_128_add=off:afr=on:amm=sco:anc=none:bsr=on:cond=on:ile=on:irw=on:nm=2:nwc=2:nicw=on:sas=z3:updr=off_300");
  fallback.push("ott-1_1_acc=model:add=off:afr=on:afp=4000:afq=1.2:anc=all:bd=preordered:bs=unit_only:bsr=on:ccuc=first:gs=on:gsaa=from_current:ile=on:nm=64:newcnf=on:nwc=1:nicw=on:sac=on:sp=occurrence:tha=off:thi=strong:updr=off_300");
  fallback.push("dis+4_6_av=off:bd=off:bs=on:ile=on:irw=on:lma=on:nm=64:nwc=1_300");
  fallback.push("lrs+10_3:1_afp=1000:afq=1.4:amm=off:anc=none:bsr=on:inw=on:ile=on:lma=on:nm=0:newcnf=on:nwc=1:sas=z3:sac=on:tha=off:urr=on:updr=off_300");
  fallback.push("dis+11_5:1_av=off:br=off:cond=on:fsr=off:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:tha=off:urr=on_300");
  fallback.push("lrs+1011_3:1_add=off:afr=on:afp=4000:afq=2.0:amm=off:anc=none:bce=on:ep=RS:gs=on:ile=on:lma=on:nm=64:newcnf=on:nwc=1.2:sp=occurrence:tha=off_300");
  fallback.push("lrs+11_2:1_add=large:afr=on:afp=1000:afq=1.4:anc=none:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:ile=on:irw=on:lma=on:nm=64:newcnf=on:nwc=1.1:tha=off:urr=on:uhcvi=on_300");
  fallback.push("dis+1010_5:1_av=off:cond=on:gsp=input_only:gs=on:gsem=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:urr=on:updr=off_300");
  fallback.push("ott+10_2:1_av=off:bce=on:cond=fast:fde=none:irw=on:nm=32:newcnf=on:nwc=1:sos=theory:updr=off_300");
  fallback.push("dis+4_16_acc=model:add=large:afr=on:afp=40000:afq=2.0:amm=off:anc=none:bs=on:ccuc=small_ones:fsr=off:ile=on:nm=4:newcnf=on:nwc=1:nicw=on:sp=reverse_arity_300");
  fallback.push("ott+1011_5:1_add=off:afr=on:afp=10000:afq=1.4:amm=off:anc=none:bd=off:bsr=on:inw=on:ile=on:lma=on:nm=2:nwc=1.5:sas=z3:sos=theory:thf=on:updr=off_300");
  fallback.push("lrs+1011_5:4_av=off:bd=off:bs=on:cond=on:er=known:gs=on:gsem=on:inw=on:ile=on:lcm=reverse:nm=6:newcnf=on:nwc=1:sp=occurrence:tha=off:uhcvi=on_300");
  fallback.push("dis+11_5:1_afr=on:afp=40000:afq=2.0:amm=sco:anc=all_dependent:cond=fast:fde=unused:gs=on:gsem=off:ile=on:irw=on:lma=on:nm=2:nwc=1:sos=all:urr=on:uhcvi=on_300");
  fallback.push("dis+1_4_av=off:bd=off:fsr=off:nm=64:newcnf=on:nwc=1:sp=reverse_arity_300");
  fallback.push("dis+1002_2:3_av=off:bs=on:bce=on:cond=fast:ile=on:nm=2:newcnf=on:nwc=1:sp=occurrence:tha=off:thi=strong_300");
  fallback.push("dis+4_4:1_add=off:afp=4000:afq=1.2:amm=sco:anc=none:br=off:cond=fast:ep=RS:fsr=off:inw=on:lma=on:nm=2:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thf=on:urr=on:uhcvi=on_300");
  fallback.push("dis+1010_2:1_add=large:afp=10000:afq=2.0:amm=off:anc=all_dependent:bce=on:cond=fast:ep=R:fsr=off:ile=on:lma=on:nm=64:nwc=1:sac=on:urr=on_300");
  fallback.push("dis+11_4:1_add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=none:br=off:cond=fast:gs=on:gsem=on:ile=on:irw=on:lma=on:nm=2:nwc=1:sas=z3:ss=axioms:st=3.0:sos=all:sp=occurrence:tha=off:urr=on:updr=off:uhcvi=on_300");
  fallback.push("ott+1004_8:1_afp=10000:afq=1.1:amm=sco:anc=none:bd=off:bsr=on:fde=unused:ile=on:nm=2:newcnf=on:nwc=1:nicw=on:sas=z3:sp=reverse_arity:tha=off:updr=off_300");
  fallback.push("ott+11_2_av=off:inw=on:ile=on:irw=on:lcm=reverse:lma=on:nm=6:nwc=1.5:sp=occurrence:updr=off_300");
  fallback.push("lrs+10_3:1_afr=on:afp=40000:afq=1.0:amm=sco:anc=none:bd=off:ile=on:irw=on:lma=on:lwlo=on:nm=64:nwc=1:sas=z3:sos=on:sp=reverse_arity_300");
  fallback.push("lrs+11_2:1_afp=1000:afq=1.4:amm=off:anc=none:inw=on:ile=on:irw=on:nm=64:nwc=1:sac=on:tha=off:urr=on_300");
  fallback.push("lrs+10_3:1_add=large:afp=10000:afq=1.1:amm=off:anc=none:cond=on:gs=on:gsem=off:inw=on:ile=on:lma=on:lwlo=on:nm=64:nwc=1:sd=5:ss=axioms:st=1.5:tha=off:urr=on_300");
  fallback.push("lrs+1011_8:1_av=off:cond=fast:fde=unused:gs=on:gsem=on:irw=on:lma=on:lwlo=on:nm=64:newcnf=on:nwc=1:sp=reverse_arity:tha=off:thi=strong:uwa=ground:urr=on:updr=off_300");
  fallback.push("ott+1011_4_afp=4000:afq=1.1:amm=off:anc=none:bs=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:ile=on:irw=on:nm=32:newcnf=on:nwc=1:sas=z3:sp=occurrence:tha=off_300");
  fallback.push("dis+2_3_acc=on:add=off:afr=on:afp=100000:afq=1.2:amm=off:anc=none:bs=unit_only:br=off:ccuc=first:cond=on:er=filter:ile=on:nm=6:nwc=1:urr=on_300");
  fallback.push("ott+1002_5:1_add=large:afr=on:afp=100000:afq=1.1:amm=off:anc=none:bd=off:bs=unit_only:bsr=on:br=off:cond=on:fsr=off:gs=on:inw=on:irw=on:lma=on:nm=0:newcnf=on:nwc=1:nicw=on:sos=all:tha=off:urr=on_300");
  fallback.push("dis-4_7_acc=on:afp=40000:afq=1.4:anc=all_dependent:bsr=on:br=off:bce=on:ccuc=first:er=filter:fsr=off:fde=unused:gsp=input_only:ile=on:lcm=reverse:lma=on:nm=4:newcnf=on:nwc=1:nicw=on:sac=on:sp=reverse_arity:tha=off:thi=full:uwa=ground:urr=on:updr=off:uhcvi=on_300");
  fallback.push("lrs+1002_1_av=off:bd=off:fsr=off:fde=none:nm=2:newcnf=on:nwc=1:sp=reverse_arity:uhcvi=on_300");
  fallback.push("dis-2_4_add=large:afp=1000:afq=1.0:amm=sco:anc=none:bce=on:gs=on:inw=on:ile=on:lwlo=on:nm=64:newcnf=on:nwc=1:sas=z3:sos=all:sp=reverse_arity:tha=off:thi=all_300");
  fallback.push("dis+1011_3_aac=none:afp=1000:afq=1.2:anc=all:fde=none:gs=on:gsem=on:inw=on:ile=on:lcm=predicate:lma=on:nm=6:newcnf=on:nwc=1.5:nicw=on:sas=z3:tha=off:urr=on_900");
  fallback.push("lrs+10_24_av=off:bs=unit_only:fsr=off:irw=on:lma=on:nm=64:newcnf=on:nwc=1:sd=7:ss=axioms:st=1.2:sp=occurrence:tha=off:thf=on:uhcvi=on_600");
  fallback.push("lrs+4_28_afp=10000:afq=1.2:amm=sco:anc=none:bd=off:bce=on:cond=on:fsr=off:ile=on:irw=on:lcm=reverse:nm=16:newcnf=on:nwc=2:sas=z3:sp=occurrence:tha=off:updr=off:uhcvi=on_600");
  fallback.push("lrs+1003_4:1_av=off:bd=preordered:cond=on:fde=unused:gs=on:ile=on:irw=on:nm=64:nwc=1.2:sp=reverse_arity:tha=off:urr=on:updr=off:uhcvi=on_900");
  fallback.push("fmb+10_1_av=off:bce=on:fmbes=smt:fmbsr=1.6:fde=none:ile=on:nm=64:updr=off_900");
} // getSmtcomp2018Schedule

// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------

void Schedules::getLtb2017Hh4Schedule(const Property& property, Schedule& sched) {
  sched.push("lrs+10_3_ep=RS:gs=on:gsem=off:nm=1024:nwc=1:stl=300:sd=2:ss=priority:sos=all:av=off_60"); // HH4 1 (4899)
  sched.push("dis+11_4_cond=on:gsp=input_only:gs=on:nm=0:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:av=off:urr=on:updr=off:uhcvi=on_60"); // HH4 2 (1018)
  sched.push("lrs+11_2:3_br=off:cond=on:fde=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:stl=300:sd=1:ss=axioms:st=2.0:sos=all:av=off:sp=occurrence:urr=on:updr=off_60"); // HH4 3 (356)
  sched.push("dis+1002_4_cond=fast:ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:sac=on:add=large:afp=100000:afq=1.0:amm=sco:anc=none:updr=off:uhcvi=on_60"); // HH4 4 (230)
  sched.push("lrs+1011_1_cond=fast:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:gsssp=full:nm=0:nwc=10:stl=300:sd=1:ss=axioms:st=5.0:av=off:sp=occurrence:urr=on_60"); // HH4 5 (179)
  sched.push("ott+2_2:1_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:sd=3:ss=priority:st=1.5:sos=on:av=off:sp=occurrence:updr=off_60"); // HH4 6 (138)
  sched.push("lrs+11_5:4_bd=off:bsr=unit_only:br=off:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:nm=0:nwc=1:stl=300:sd=1:ss=axioms:sos=on:add=off:afp=40000:afq=1.4:amm=sco:urr=on:updr=off:uhcvi=on_60"); // HH4 7 (120)
  sched.push("ott+1011_1_bd=preordered:cond=on:gsp=input_only:nm=64:nwc=1:sd=3:ss=priority:av=off:sp=reverse_arity:urr=on_60"); // HH4 8 (90)
  sched.push("ins+11_5_cond=fast:ep=RST:gs=on:gsem=on:igbrr=0.4:igpr=on:igrr=1/64:igrp=4000:igrpq=1.3:igwr=on:lcm=reverse:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // HH4 9 (81)
  sched.push("lrs+11_5_cond=on:ep=RST:fde=none:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:stl=300:sd=2:ss=axioms:st=3.0:sos=all:sac=on:add=large:afp=40000:afq=1.4:amm=off:anc=none:urr=ec_only:uhcvi=on_60"); // HH4 10 (70)
  sched.push("lrs+1011_3_bd=off:bsr=on:cond=fast:fde=none:gs=on:gsssp=full:nm=0:nwc=1:stl=300:sd=2:ss=axioms:sos=all:av=off:sp=occurrence_60"); // HH4 11 (58)
  sched.push("lrs-4_5:4_cond=on:gs=on:gsem=on:gsssp=full:nm=64:nwc=1:stl=300:sd=2:ss=axioms:st=2.0:sos=on:sac=on:afp=100000:afq=1.1:amm=sco:anc=none:urr=on_60"); // HH4 12 (44)
  sched.push("dis+1011_3:1_br=off:nm=0:nwc=5:sd=1:ss=axioms:sac=on:afp=40000:afq=1.4:amm=sco:anc=none:urr=on:uhcvi=on_60"); // HH4 13 (38)
  sched.push("lrs+11_3:1_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:stl=300:sd=2:ss=priority:sac=on:amm=sco:anc=none:sp=reverse_arity:updr=off:uhcvi=on_60"); // HH4 14 (36)
  sched.push("dis+4_3_bd=off:cond=on:fde=unused:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sd=3:ss=axioms:st=3.0:sos=on:add=off:afr=on:afp=10000:afq=1.0:amm=off:anc=none:urr=ec_only:updr=off:uhcvi=on_60"); // HH4 15 (32)
  sched.push("dis+1010_5_cond=fast:nm=0:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // HH4 16 (32)
  sched.push("lrs+11_4:1_bd=off:bsr=unit_only:br=off:fsr=off:fde=unused:gsp=input_only:gs=on:nm=0:nwc=1:stl=300:sd=1:ss=axioms:sos=on:av=off:sp=reverse_arity:urr=on_60"); // HH4 17 (29)
  sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_60"); // HH4 18 (28)
  sched.push("lrs+11_2:3_cond=on:fde=unused:gs=on:gsaa=full_model:nwc=4:stl=300:sd=2:ss=priority:st=5.0:sac=on:add=off:afr=on:amm=off:anc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HH4 19 (24)
  sched.push("ott-11_8:1_bsr=unit_only:cond=on:gs=on:gsem=off:gsssp=full:nwc=1.1:sd=2:ss=axioms:sos=on:sac=on:acc=model:add=large:aer=off:afp=40000:afq=2.0:anc=none:sp=reverse_arity:urr=on_60"); // HH4 20 (23)
  sched.push("lrs+1010_2:1_gs=on:lwlo=on:nm=0:nwc=3:stl=300:sd=4:ss=axioms:av=off_60"); // HH4 21 (22)
  sched.push("lrs+1010_4_bsr=unit_only:cond=fast:fsr=off:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:stl=300:sd=1:ss=axioms:st=1.5:sos=on:add=off:aer=off:afr=on:afp=10000:afq=1.0:anc=none:sp=occurrence:urr=on_60"); // HH4 22 (20)
  sched.push("dis+2_1_fsr=off:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // HH4 23 (17)
  sched.push("ott+2_2:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HH4 24 (17)
  sched.push("lrs+1003_8:1_br=off:cond=on:fde=none:gs=on:gsem=off:nm=0:nwc=1:stl=300:sd=1:ss=axioms:sos=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:sp=occurrence:urr=on_60"); // HH4 25 (14)
  sched.push("dis+11_2:1_bd=off:cond=fast:fde=unused:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=all:av=off:sp=occurrence_60"); // HH4 26 (14)
  sched.push("lrs+1011_3:1_bd=off:cond=fast:fsr=off:fde=unused:gs=on:nm=0:nwc=5:stl=300:sd=2:ss=axioms:afp=100000:afq=1.4:amm=off:anc=none:sp=occurrence:urr=on_60"); // HH4 27 (14)
  sched.push("dis+1011_3_cond=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:afr=on:afp=1000:afq=1.4:anc=none:sp=occurrence:updr=off:uhcvi=on_60"); // HH4 28 (13)
  sched.push("dis+11_2:1_fde=none:gsp=input_only:nwc=1:sd=2:ss=axioms:sos=all:av=off:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HH4 29 (13)
  sched.push("lrs+11_3_bsr=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:nwc=5:stl=300:sd=3:ss=priority:st=2.0:av=off:updr=off:uhcvi=on_60"); // HH4 30 (12)
  sched.push("lrs+11_5:1_cond=on:gs=on:gsssp=full:lwlo=on:nwc=1:stl=300:sd=1:ss=axioms:st=3.0:av=off:urr=on_60"); // HH4 31 (11)
  sched.push("dis+1_5_bd=preordered:bs=unit_only:ccuc=small_ones:cond=on:fsr=off:gs=on:gsem=on:nm=0:nwc=1:sd=1:ss=axioms:st=1.5:sos=all:aac=none:acc=model:add=off:aer=off:afr=on:afp=100000:afq=1.2:anc=all_dependent:urr=ec_only:updr=off:uhcvi=on_60"); // HH4 32 (10)
  sched.push("lrs+11_4_bd=off:cond=fast:fde=unused:gs=on:nwc=1:stl=300:sd=3:ss=axioms:st=5.0:sos=all:afr=on:afp=100000:afq=1.2:amm=sco:anc=none:updr=off:uhcvi=on_60"); // HH4 33 (10)
  sched.push("lrs+11_8:1_br=off:cond=fast:fde=none:gs=on:gsaa=from_current:gsem=on:nm=0:nwc=2:stl=300:sd=2:ss=axioms:st=1.5:sac=on:add=off:afp=40000:afq=1.4:anc=none:sp=reverse_arity:urr=on_60"); // HH4 34 (9)
  sched.push("dis+1003_1_ccuc=first:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:nm=64:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:acc=model:add=large:sp=reverse_arity:uhcvi=on_60"); // HH4 35 (9)
  sched.push("dis+11_3_br=off:cond=on:ep=RST:fsr=off:fde=none:gsp=input_only:gs=on:nwc=1:sd=2:ss=axioms:sos=all:sac=on:add=off:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_60"); // HH4 36 (8)
  sched.push("dis+10_5:4_ep=R:gs=on:gsaa=from_current:nm=64:nwc=1:sd=1:ss=axioms:sos=on:add=large:aer=off:afp=4000:afq=1.1:anc=none:updr=off:uhcvi=on_60"); // HH4 37 (8)
  sched.push("dis+11_3:1_br=off:cond=fast:fde=unused:gs=on:gsem=off:nm=0:nwc=1.7:sd=1:ss=axioms:st=1.5:sac=on:add=off:aer=off:afr=on:afp=10000:afq=1.4:anc=none:urr=on:uhcvi=on_60"); // HH4 38 (8)
  sched.push("dis+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=off:gsssp=full:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.1:amm=off:anc=none:sp=occurrence:urr=on:uhcvi=on_60"); // HH4 39 (8)
  sched.push("ott+11_4_cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsssp=full:lcm=predicate:nm=64:nwc=1.7:sd=2:ss=priority:st=1.2:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // HH4 40 (8)
  sched.push("ott+11_2:1_bd=preordered:ccuc=small_ones:cond=fast:fde=unused:gs=on:gsem=on:nm=1024:nwc=3:sd=3:ss=priority:st=1.2:acc=model:add=large:afr=on:afp=100000:afq=1.4:amm=off:anc=none:sp=occurrence:uhcvi=on_60"); // HH4 41 (8)
  sched.push("ott+1011_5:4_fde=unused:gs=on:gsem=off:nwc=1.3:sd=4:ss=priority:st=5.0:add=off:afp=1000:afq=1.2:amm=sco:sp=reverse_arity:urr=on_60"); // HH4 42 (7)
  sched.push("lrs+11_5:4_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:stl=300:sd=2:ss=axioms:st=2.0:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // HH4 43 (7)
  sched.push("lrs+11_2_bd=off:cond=fast:fde=unused:gsp=input_only:nwc=1:nicw=on:stl=300:sd=2:ss=priority:st=1.2:sos=all:sac=on:aac=none:aer=off:afr=on:afp=10000:afq=2.0:anc=all:sp=reverse_arity_60"); // HH4 44 (7)
  sched.push("dis+1003_5:4_fde=none:gs=on:gsem=on:gsssp=full:nm=64:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:av=off:sp=occurrence:urr=ec_only_60"); // HH4 45 (7)
  sched.push("dis+11_1_br=off:cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1.1:sd=1:ss=axioms:add=large:aer=off:afr=on:anc=none:urr=on:updr=off_60"); // HH4 46 (6)
  sched.push("dis-2_5_bd=off:cond=on:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=1024:nwc=1:sd=5:ss=axioms:sos=on:sac=on:add=off:aer=off:afr=on:afp=1000:afq=1.2:anc=none:urr=ec_only:updr=off_60"); // HH4 47 (6)
  sched.push("lrs-1_5_cond=fast:fde=none:gs=on:nm=0:nwc=1.1:stl=300:sd=1:ss=axioms:st=3.0:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // HH4 48 (6)
  sched.push("dis+1010_4_cond=on:fde=unused:gs=on:gsem=on:nm=0:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:av=off:updr=off_60"); // HH4 49 (6)
  sched.push("lrs+10_5:4_fde=unused:gs=on:gsem=on:gsssp=full:nm=0:nwc=5:stl=300:sd=1:ss=axioms:av=off:updr=off:uhcvi=on_60"); // HH4 50 (6)
  sched.push("dis+11_4:1_bsr=unit_only:ccuc=small_ones:fsr=off:nm=64:nwc=3:sd=1:ss=axioms:st=1.2:sos=on:acc=on:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:updr=off_60"); // HH4 51 (6)
  sched.push("ott+1011_1_cond=on:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=10:sd=1:ss=axioms:st=2.0:av=off:sp=occurrence:urr=on:updr=off_60"); // HH4 52 (6)
  sched.push("dis+1002_1_cond=on:ep=RSTC:fsr=off:gs=on:nwc=1:sd=1:ss=axioms:sos=on:av=off:sp=reverse_arity_60"); // HH4 53 (5)
  sched.push("lrs+1003_6_cond=on:gs=on:gsem=off:gsssp=full:nm=0:nwc=2.5:stl=300:sd=3:ss=priority:av=off:sp=reverse_arity:updr=off_60"); // HH4 54 (5)
  sched.push("dis-2_3_bd=off:cond=fast:fsr=off:gs=on:gsem=off:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:av=off:sp=occurrence:urr=ec_only:uhcvi=on_60"); // HH4 55 (5)
  sched.push("dis+1011_4_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nm=1024:nwc=1:sd=3:ss=axioms:sos=on:sac=on:add=large:afp=100000:afq=1.2:amm=sco:anc=all_dependent:urr=on:uhcvi=on_60"); // HH4 56 (5)
  sched.push("lrs+10_4_bsr=unit_only:cond=fast:ep=RST:gs=on:gsem=on:gsssp=full:nm=0:nwc=1:stl=300:sd=2:ss=priority:st=1.5:sos=all:av=off:sp=reverse_arity_60"); // HH4 57 (5)
  sched.push("lrs+4_5:4_bd=off:cond=on:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:nm=0:nwc=1:stl=300:sd=2:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity_60"); // HH4 58 (5)
  sched.push("ott+11_3:1_bs=unit_only:bsr=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:nm=64:nwc=1.2:sd=5:ss=priority:st=1.2:av=off:sp=reverse_arity:updr=off_60"); // HH4 59 (5)
  sched.push("dis+10_3:1_cond=on:fsr=off:fde=unused:nwc=1:sd=1:ss=priority:st=5.0:sos=all:add=large:aer=off:afr=on:afp=40000:afq=1.4_60"); // HH4 60 (4)
  sched.push("lrs+11_4:1_cond=fast:fde=none:nm=0:nwc=1:stl=300:sd=3:ss=priority:st=2.0:av=off:sp=occurrence:urr=ec_only_60"); // HH4 61 (4)
  sched.push("lrs-3_4_bd=off:bsr=unit_only:fde=unused:gs=on:gsaa=full_model:nwc=1:stl=300:sd=3:ss=axioms:sos=on:sac=on:aer=off:afr=on:afp=40000:afq=1.4:anc=none:updr=off:uhcvi=on_60"); // HH4 62 (4)
  sched.push("lrs+1011_2:3_bsr=unit_only:fsr=off:fde=none:gs=on:gsem=on:nm=64:nwc=2.5:stl=300:sd=3:ss=priority:st=3.0:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // HH4 63 (4)
  sched.push("ott+1010_3:1_cond=fast:fde=unused:nm=64:nwc=1.7:sd=3:ss=priority:av=off:sp=occurrence:updr=off_60"); // HH4 64 (4)
  sched.push("dis+2_4:1_bsr=unit_only:br=off:cond=fast:fde=none:lcm=reverse:lwlo=on:nm=0:nwc=1:sd=1:ss=axioms:sos=on:av=off:sp=occurrence:urr=on_60"); // HH4 65 (3)
  sched.push("dis+11_1_cond=fast:ep=RST:fde=none:nm=1024:nwc=2:sd=2:ss=priority:st=1.5:add=off:afp=100000:afq=1.1:amm=off:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HH4 66 (3)
  sched.push("lrs+4_5:4_bd=off:bsr=unit_only:fsr=off:gs=on:nm=0:nwc=1:stl=300:sd=2:ss=axioms:st=1.5:sos=on:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HH4 67 (3)
  sched.push("dis+1010_3_bsr=unit_only:gs=on:gsaa=from_current:gsem=off:gsssp=full:nm=64:nwc=1:sd=1:ss=axioms:sos=on:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity_60"); // HH4 68 (3)
  sched.push("lrs+11_3_cond=fast:gs=on:nm=0:nwc=1.3:stl=300:sd=2:ss=priority:av=off:sp=reverse_arity:updr=off_60"); // HH4 69 (3)
  sched.push("dis+11_3_cond=fast:gsp=input_only:gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1:sd=3:ss=priority:sos=on:add=off:afp=10000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // HH4 70 (3)
  sched.push("lrs+11_3_bd=off:bsr=on:cond=fast:fde=unused:gs=on:gsem=on:nwc=1:stl=300:sd=4:ss=axioms:st=1.5:sos=all:av=off:sp=reverse_arity:uhcvi=on_60"); // HH4 71 (3)
  sched.push("dis+10_3:1_bsr=unit_only:cond=fast:fde=none:nm=64:nwc=1:sd=2:ss=axioms:sos=all:av=off:sp=reverse_arity:updr=off_60"); // HH4 72 (3)
  sched.push("lrs+11_4_ep=RST:fde=unused:gs=on:gsaa=from_current:gsssp=full:nwc=1:stl=300:sd=1:ss=axioms:st=5.0:sos=all:sac=on:aer=off:afr=on:afp=40000:afq=2.0:anc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // HH4 73 (3)
  sched.push("dis+11_2_bd=preordered:bs=unit_only:fsr=off:fde=none:gs=on:gsem=on:nm=0:nwc=3:sd=3:ss=axioms:st=1.5:sac=on:acc=on:add=off:afr=on:afp=100000:afq=1.2:amm=sco:anc=all:sp=occurrence:uhcvi=on_60"); // HH4 74 (3)
  sched.push("lrs+11_4_fde=unused:gsp=input_only:lcm=predicate:nm=0:nwc=1.3:stl=300:sd=1:ss=axioms:st=2.0:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // HH4 75 (3)
  sched.push("lrs+11_3:1_bd=off:bsr=unit_only:fsr=off:fde=unused:gs=on:gsem=on:nm=0:nwc=1:stl=300:sd=2:ss=axioms:add=large:afp=40000:afq=1.1:sp=reverse_arity_60"); // HH4 76 (3)
  sched.push("lrs+11_3:1_bd=off:bsr=unit_only:cond=on:fsr=off:gs=on:gsem=on:nm=64:nwc=1:stl=300:sd=1:ss=axioms:st=1.5:av=off:sp=reverse_arity:updr=off_60"); // HH4 77 (2)
  sched.push("dis+11_3_ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:gsssp=full:nm=0:nwc=1:sd=3:ss=axioms:st=3.0:sos=all:sac=on:amm=sco:anc=none:urr=ec_only:updr=off_60"); // HH4 78 (2)
  sched.push("ott+11_5:1_br=off:cond=on:ep=RST:fsr=off:gs=on:gsem=on:gsssp=full:nwc=1:sd=3:ss=priority:sos=all:av=off:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HH4 79 (2)
  sched.push("lrs+11_5_bd=off:bsr=unit_only:gs=on:gsaa=from_current:gsem=off:nwc=1:stl=300:sd=2:ss=axioms:st=3.0:sos=all:sac=on:afr=on:afp=100000:afq=1.1:amm=off:anc=none:sp=occurrence:updr=off_60"); // HH4 80 (2)
  sched.push("dis+11_5_cond=on:ep=RST:gs=on:lwlo=on:nm=64:nwc=2:sd=1:ss=axioms:st=2.0:sac=on:afp=10000:afq=1.2:anc=none:updr=off_60"); // HH4 81 (2)
  sched.push("ott+1010_3_cond=on:fsr=off:fde=unused:nm=0:nwc=1.2:sd=2:ss=priority:st=1.2:av=off:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // HH4 82 (2)
  sched.push("lrs+1010_2_bs=unit_only:bsr=unit_only:ccuc=first:cond=on:fsr=off:fde=unused:gs=on:gsssp=full:nm=0:nwc=1.5:nicw=on:stl=300:sd=2:ss=axioms:st=5.0:sos=on:sac=on:acc=on:add=off:afr=on:afp=40000:afq=1.2:amm=sco:anc=none:sp=reverse_arity:uhcvi=on_60"); // HH4 83 (2)
  sched.push("lrs+11_5_bd=off:cond=fast:gs=on:gsssp=full:nwc=1.1:stl=300:sd=2:ss=axioms:st=1.2:sos=all:av=off:uhcvi=on_60"); // HH4 84 (2)
  sched.push("lrs+11_5_fde=unused:gs=on:gsem=off:nwc=1:stl=300:sd=1:ss=priority:st=1.2:sos=all:sac=on:aer=off:afp=40000:afq=1.2:anc=all:sp=occurrence_60"); // HH4 85 (2)
  sched.push("ins+11_5_cond=on:fsr=off:fde=none:gs=on:gsem=on:gsssp=full:igbrr=0.4:igrr=1/8:igrp=1400:igrpq=1.5:igwr=on:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HH4 86 (2)
  sched.push("lrs+11_3_fsr=off:fde=unused:gs=on:nm=64:nwc=1.7:stl=300:sd=3:ss=axioms:st=2.0:av=off_60"); // HH4 87 (2)
  sched.push("lrs+11_3_bd=off:bsr=on:cond=fast:fsr=off:fde=none:gs=on:gsem=on:gsssp=full:nm=0:nwc=1:stl=300:sd=3:ss=priority:st=1.5:av=off:sp=reverse_arity_60"); // HH4 88 (2)
  sched.push("ott+1004_4:1_bs=on:cond=on:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1.5:sd=2:ss=axioms:st=5.0:sac=on:add=large:afr=on:afp=1000:afq=1.1:anc=none:urr=on:updr=off_60"); // HH4 89 (2)
  sched.push("lrs+10_8:1_bd=off:fsr=off:fde=none:gs=on:gsem=on:lwlo=on:nwc=1:stl=300:sd=10:ss=priority:add=off:aer=off:afp=100000:afq=1.4:sp=reverse_arity:urr=on:uhcvi=on_60"); // HH4 90 (2)
  sched.push("dis+10_24_cond=fast:ep=RST:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=on:gsssp=full:nm=0:nwc=1:sd=3:ss=axioms:st=5.0:sos=on:sac=on:add=off:afp=1000:afq=1.4:amm=sco:anc=none:sp=occurrence_60"); // HH4 91 (1)
  sched.push("lrs+11_4:1_cond=on:ep=RS:fsr=off:lwlo=on:nm=0:nwc=1:stl=300:sos=all:av=off:sp=reverse_arity:uhcvi=on_60"); // HH4 92 (1)
  sched.push("dis+2_5_bsr=unit_only:cond=on:fsr=off:nwc=1:sd=10:ss=priority:st=2.0:sos=on:av=off:sp=occurrence:urr=ec_only_60"); // HH4 93 (1)
  sched.push("lrs-3_8:1_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsem=off:nm=0:nwc=1:stl=300:sd=4:ss=priority:st=3.0:av=off:sp=occurrence_60"); // HH4 94 (1)
  sched.push("lrs+10_2:3_fsr=off:fde=unused:gs=on:gsem=on:nm=64:nwc=1:stl=300:sd=5:ss=priority:st=1.2:sos=on:av=off:sp=reverse_arity:updr=off_60"); // HH4 95 (1)
  sched.push("lrs+1010_8:1_bsr=on:fsr=off:fde=unused:nm=0:nwc=1:stl=300:sd=2:ss=axioms:av=off:sp=reverse_arity:uhcvi=on_60"); // HH4 96 (1)
  sched.push("lrs+1003_4:1_bd=off:bs=unit_only:bsr=unit_only:fde=none:gsp=input_only:nm=0:nwc=1:stl=300:sd=3:ss=axioms:sos=all:sac=on:aac=none:add=large:afr=on:afp=4000:afq=1.1:amm=sco:anc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HH4 97 (1)
  sched.push("lrs+1003_10_bsr=unit_only:br=off:fsr=off:fde=none:nm=64:nwc=2:stl=300:sd=4:ss=axioms:st=1.2:sos=all:sac=on:add=large:afr=on:afp=40000:afq=1.1:amm=off:anc=all:urr=on_60"); // HH4 98 (1)
  sched.push("dis+1004_5_cond=fast:fde=unused:gs=on:lwlo=on:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // HH4 99 (1)
  sched.push("dis+11_4_bd=off:cond=on:fde=unused:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sd=2:ss=axioms:st=2.0:sos=all:afr=on:afp=1000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // HH4 100 (1)
  sched.push("lrs+11_2:3_bd=off:bs=unit_only:bsr=unit_only:cond=fast:fsr=off:gsp=input_only:lcm=reverse:nm=0:nwc=1.1:stl=300:sd=1:ss=axioms:st=1.5:sos=all:av=off_60"); // HH4 101 (1)
  sched.push("lrs+11_5:4_bd=off:bsr=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1:stl=300:sd=2:ss=axioms:av=off:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HH4 102 (1)
  sched.push("ins+11_4_ep=RST:fsr=off:igrr=1/16:igrp=400:igrpq=2.0:igs=1:igwr=on:nm=0:nwc=1.3:sd=1:ss=axioms:st=5.0:av=off_60"); // HH4 103 (1)
  sched.push("ins+11_3_fsr=off:fde=none:gs=on:gsem=off:igbrr=0.5:igpr=on:igrr=1/4:igrp=2000:igrpq=1.2:igs=1003:igwr=on:nwc=10:sd=1:ss=axioms:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // HH4 104 (1)
  sched.push("dis+4_4_br=off:cond=fast:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=1.2:add=off:afp=1000:afq=2.0:anc=none:urr=on:updr=off_60"); // HH4 105 (1)
  sched.push("lrs+1004_8_bd=off:bsr=unit_only:br=off:cond=fast:fde=unused:nm=64:nwc=1.5:stl=300:sd=3:ss=priority:st=3.0:av=off:sp=reverse_arity:urr=on_60"); // HH4 106 (1)
  sched.push("lrs+11_2_bsr=unit_only:fsr=off:fde=none:gsp=input_only:gs=on:nwc=1:stl=300:sd=1:ss=axioms:st=3.0:sos=on:add=off:afp=10000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HH4 107 (1)
  sched.push("dis+10_4_bd=off:fsr=off:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=64:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:add=large:aer=off:afp=4000:afq=1.1:anc=none:sp=occurrence:urr=ec_only:updr=off_60"); // HH4 108 (1)
  sched.push("dis+10_4_cond=fast:ep=RST:fsr=off:nwc=1:sd=3:ss=axioms:sos=all:av=off_60"); // HH4 109 (1)
  sched.push("dis+1_5_cond=on:ep=RST:fsr=off:fde=none:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sd=2:ss=priority:st=3.0:sos=all:av=off:urr=on_60"); // HH4 110 (1)
  sched.push("dis+2_7_bd=off:cond=fast:fsr=off:gs=on:gsem=off:nm=64:nwc=1.1:sd=3:ss=axioms:st=3.0:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // HH4 111 (1)
  sched.push("lrs-3_4_bsr=unit_only:gs=on:nwc=1:stl=300:sd=2:ss=axioms:st=3.0:sos=on:sac=on:add=off:afr=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:uhcvi=on_60"); // HH4 112 (1)
  sched.push("dis+1004_3:1_fsr=off:gs=on:gsem=off:gsssp=full:nm=0:nwc=1:sd=2:ss=axioms:sos=on:sac=on:add=large:afp=40000:afq=1.2:urr=ec_only:uhcvi=on_60"); // HH4 113 (1)
  sched.push("lrs+1003_4_bd=off:bsr=unit_only:cond=on:gs=on:gsem=off:nm=0:nwc=1:stl=300:sd=2:ss=axioms:sos=on:av=off:sp=occurrence:urr=on:updr=off_60"); // HH4 114 (1)
  sched.push("lrs+11_5_bd=off:cond=fast:fsr=off:fde=none:nm=0:nwc=1:stl=300:sd=2:ss=axioms:st=3.0:sos=all:av=off:updr=off:uhcvi=on_60"); // HH4 115 (1)
  sched.push("dis+4_5_bd=off:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:afr=on:afp=10000:afq=1.2:anc=none:urr=on_60"); // HH4 116 (1)
  sched.push("lrs+11_5_fsr=off:fde=unused:gs=on:gsaa=from_current:gsssp=full:nwc=1:stl=300:sd=4:ss=axioms:sos=on:add=off:afp=10000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:urr=on:uhcvi=on_60"); // HH4 117 (1)
  sched.push("dis+11_3:1_fsr=off:fde=none:nwc=1:sd=1:ss=priority:st=5.0:add=off:aer=off:afr=on:afp=100000:afq=1.1:sp=reverse_arity:urr=on:updr=off_60"); // HH4 118 (1)
  sched.push("dis+11_4:1_cond=fast:gs=on:gsem=on:nm=64:nwc=1.1:sd=1:ss=axioms:st=2.0:av=off:sp=occurrence:urr=on_60"); // HH4 119 (1)
  sched.push("lrs+1010_5:1_cond=fast:fde=none:gs=on:gsaa=from_current:gsem=on:lwlo=on:nwc=1.2:stl=300:sd=2:ss=priority:st=3.0:sac=on:add=large:aer=off:afr=on:afp=40000:afq=1.0:anc=all:uhcvi=on_60"); // HH4 120 (1)
  sched.push("ins+11_4_cond=on:fde=none:gs=on:gsem=off:gsssp=full:igbrr=0.5:igpr=on:igrr=1/4:igrp=4000:igrpq=1.05:igs=1:igwr=on:nm=0:nwc=1:sd=1:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // HH4 121 (1)
  sched.push("lrs+1002_3:1_bd=preordered:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=3:stl=300:sd=4:ss=priority:sac=on:add=large:afp=10000:afq=1.0:amm=off:anc=none:uhcvi=on_60"); // HH4 122 (1)
  sched.push("dis+4_5:4_cond=on:fsr=off:fde=none:lwlo=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=on:av=off:sp=occurrence:urr=ec_only:updr=off:uhcvi=on_60"); // HH4 123 (1)
  sched.push("ins+11_3_cond=fast:fsr=off:fde=none:igbrr=0.6:igrr=1/16:igrp=1400:igrpq=1.1:igs=1002:igwr=on:nm=0:nwc=1:sd=2:ss=priority:av=off:urr=ec_only:uhcvi=on_60"); // HH4 124 (1)
  sched.push("ott+11_2:1_gs=on:gsem=on:gsssp=full:nm=0:nwc=1:sd=2:ss=axioms:st=1.5:sac=on:add=large:afp=100000:afq=1.2:amm=sco:anc=all:sp=occurrence:updr=off:uhcvi=on_60"); // HH4 125 (1)
  sched.push("dis+1002_8_bd=off:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sd=4:ss=priority:sos=on:av=off:sp=occurrence_60"); // HH4 126 (1)
  sched.push("lrs+11_3_cond=fast:ep=RST:fde=unused:gs=on:gsem=on:nwc=1:stl=300:sd=3:ss=axioms:st=5.0:sos=all:av=off:updr=off:uhcvi=on_60"); // HH4 127 (1)
  sched.push("lrs+4_3:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:lwlo=on:nm=0:nwc=1.7:stl=300:av=off:sp=reverse_arity:updr=off_60"); // HH4 128 (1)
  sched.push("ins+11_4_cond=on:fde=none:gsp=input_only:igbrr=0.8:igrp=1400:igs=1004:igwr=on:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:av=off:sp=reverse_arity_60"); // HH4 129 (1)
  sched.push("lrs-3_1_bd=off:cond=on:fde=none:gs=on:lcm=reverse:nm=0:nwc=1.1:stl=300:sd=2:ss=axioms:st=1.5:av=off:updr=off:uhcvi=on_60"); // HH4 130 (1)
  sched.push("lrs+11_5_bd=off:ccuc=first:fde=none:gs=on:lcm=reverse:nm=0:nwc=1:stl=300:sd=2:ss=priority:st=1.2:sos=all:aac=none:acc=model:afr=on:afp=1000:afq=1.1:anc=none:updr=off:uhcvi=on_60"); // HH4 131 (1)
  sched.push("lrs+11_5:4_cond=fast:fde=none:gs=on:gsaa=from_current:gsem=on:nwc=1:stl=300:sd=7:ss=axioms:st=3.0:sos=all:afp=10000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:updr=off:uhcvi=on_60"); // HH4 132 (1)
  sched.push("dis+11_6_fsr=off:fde=none:gs=on:gsem=on:gsssp=full:lwlo=on:nm=0:nwc=1:sd=3:ss=axioms:sos=all:add=off:afr=on:afp=4000:afq=1.0:amm=sco:anc=all:sp=occurrence:urr=ec_only:uhcvi=on_60"); // HH4 133 (1)
  sched.push("lrs+1010_3:2_bd=off:bsr=unit_only:cond=fast:nwc=4:stl=300:sd=1:ss=axioms:st=3.0:sac=on:add=large:afp=10000:afq=1.2:amm=sco:sp=reverse_arity:updr=off:uhcvi=on_60"); // HH4 134 (1)
  sched.push("dis+1004_3:1_cond=fast:fde=unused:nm=0:nwc=1:sd=1:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // HH4 135 (1)
  sched.push("dis+4_3_bd=off:cond=fast:fsr=off:fde=unused:gs=on:gsssp=full:lwlo=on:nm=64:nwc=1:ss=axioms:st=2.0:sos=on:av=off:sp=occurrence:updr=off_60"); // HH4 136 (1)
  sched.push("lrs-10_24_bd=off:fsr=off:lcm=reverse:nm=0:nwc=1:stl=300:sd=1:ss=axioms:sos=on:av=off:sp=occurrence_60"); // HH4 137 (1)
  sched.push("ins+11_4_fde=none:igbrr=0.6:igpr=on:igrr=1/128:igrp=700:igrpq=1.05:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=1:ss=axioms:sos=on:av=off:sp=occurrence:updr=off_60"); // HH4 138 (1)
  sched.push("ott+11_2:1_cond=fast:nm=0:nwc=2.5:sd=2:ss=priority:st=1.2:av=off:sp=occurrence:urr=on:updr=off_60"); // HH4 139 (1)
}

// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------

void Schedules::getLtb2017IsaSchedule(const Property& property, Schedule& sched) {
  sched.push("dis+1002_3_gs=on:gsem=off:nwc=1.2:sd=2:ss=axioms:st=3.0:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // ISA 1 (1149)
  sched.push("dis+1011_3:2_bsr=unit_only:cond=fast:nwc=3:nicw=on:sd=3:ss=priority:add=off:afr=on:afp=10000:afq=1.2:uhcvi=on_60"); // ISA 2 (347)
  sched.push("lrs+1010_1_cond=on:fde=none:gs=on:gsem=off:nwc=1:stl=300:sd=1:ss=axioms:st=3.0:sos=on:sac=on:afp=10000:afq=1.1:amm=sco:anc=none:urr=on:updr=off_60"); // ISA 3 (174)
  sched.push("lrs-2_3_ep=RS:gs=on:gsaa=from_current:nwc=1:stl=300:sd=2:ss=axioms:sos=on:sac=on:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:uhcvi=on_60"); // ISA 4 (93)
  sched.push("dis+1011_5_fsr=off:fde=unused:nm=64:nwc=3:sd=2:ss=priority:av=off:sp=occurrence:uhcvi=on_60"); // ISA 5 (58)
  sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=1:ss=axioms:sos=on:sac=on:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:sp=occurrence:uhcvi=on_60"); // ISA 6 (52)
  sched.push("dis+1002_4_ep=RST:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sd=4:ss=axioms:st=1.5:sos=on:aer=off:afr=on:afp=40000:afq=1.2:anc=none_60"); // ISA 7 (39)
  sched.push("lrs+1011_3:2_bd=off:cond=on:gsp=input_only:gs=on:gsem=on:nm=0:nwc=4:stl=300:sd=1:ss=axioms:aer=off:afr=on:afp=40000:afq=1.1:anc=all_dependent:sp=reverse_arity:updr=off_60"); // ISA 8 (34)
  sched.push("dis+1011_1_bsr=on:ccuc=first:nm=0:nwc=4:sd=2:ss=priority:acc=model:add=large:afr=on:amm=off:anc=none:updr=off:uhcvi=on_60"); // ISA 9 (32)
  sched.push("lrs+1002_4_bd=off:fde=none:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:stl=300:sd=3:ss=axioms:st=5.0:sos=on:aer=off:afp=100000:afq=1.1:anc=none:sp=reverse_arity_60"); // ISA 10 (29)
  sched.push("dis+1002_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:add=large:afp=40000:afq=1.1:amm=off:anc=none:sp=reverse_arity:updr=off_60"); // ISA 11 (25)
  sched.push("dis+1011_3_fde=unused:nm=64:nwc=1:sd=2:ss=axioms:st=5.0:add=off:aer=off:afp=10000:afq=1.0:sp=occurrence_60"); // ISA 12 (20)
  sched.push("dis+1011_3:1_cond=fast:ep=RS:nm=0:nwc=1.7:sd=2:ss=priority:st=1.2:add=off:afp=10000:afq=1.2:amm=sco:anc=all:sp=occurrence:updr=off:uhcvi=on_60"); // ISA 13 (16)
  sched.push("dis+1002_5_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:add=off:afr=on:amm=sco:anc=none:updr=off:uhcvi=on_60"); // ISA 14 (16)
  sched.push("dis+1011_5_cond=on:er=filter:fde=none:nm=64:nwc=3:sd=2:ss=priority:st=2.0:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // ISA 15 (12)
  sched.push("lrs+10_3:1_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1.2:stl=300:sd=1:ss=priority:sos=on:sac=on:add=off:afp=1000:afq=1.4:amm=sco:anc=all:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // ISA 16 (12)
  sched.push("lrs+11_5_br=off:cond=on:fde=none:gs=on:nwc=1:stl=300:sd=2:ss=axioms:st=3.0:sos=all:add=off:afr=on:afp=40000:afq=1.4:anc=none:sp=reverse_arity:urr=on_60"); // ISA 17 (10)
  sched.push("dis+1002_3_bd=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:afr=on:amm=sco:anc=none:sp=occurrence_60"); // ISA 18 (10)
  sched.push("ins+11_4_cond=fast:fde=none:igbrr=0.4:igrr=1/32:igrp=200:igrpq=1.2:igs=1003:igwr=on:nwc=10:sd=1:ss=axioms:st=5.0:av=off_60"); // ISA 19 (10)
  sched.push("lrs+1011_4:1_fsr=off:fde=unused:gs=on:gsem=on:gsssp=full:nm=64:nwc=4:stl=300:sd=1:ss=priority:st=3.0:aac=none:acc=on:afr=on:afp=40000:afq=1.2:amm=sco:anc=all:sp=reverse_arity:updr=off:uhcvi=on_60"); // ISA 20 (9)
  sched.push("dis+1002_50_fde=unused:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // ISA 21 (8)
  sched.push("ott+11_4_cond=fast:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // ISA 22 (8)
  sched.push("dis-3_3_ep=RST:fde=none:nm=64:nwc=1:sd=4:ss=axioms:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // ISA 23 (7)
  sched.push("dis+1010_7_fsr=off:fde=unused:nm=0:nwc=1.3:nicw=on:sd=3:ss=priority:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:updr=off:uhcvi=on_60"); // ISA 24 (7)
  sched.push("dis+1002_4_cond=fast:ep=RST:fsr=off:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:afp=10000:afq=1.1:amm=sco:sp=occurrence:uhcvi=on_60"); // ISA 25 (6)
  sched.push("ott+1011_2_bd=off:ccuc=first:cond=on:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1.3:sd=3:ss=priority:st=1.2:sac=on:acc=on:add=off:afp=4000:afq=1.4:amm=sco:anc=none:urr=ec_only:updr=off:uhcvi=on_60"); // ISA 26 (6)
  sched.push("dis+1002_3_bd=off:gs=on:gsem=on:nwc=1.1:sd=7:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:updr=off_60"); // ISA 27 (5)
  sched.push("dis+11_2:3_cond=on:gs=on:gsem=off:gsssp=full:lcm=reverse:nwc=1:sd=1:ss=axioms:st=1.5:add=off:aer=off:afr=on:afp=1000:afq=2.0:anc=all_dependent:sp=reverse_arity:updr=off:uhcvi=on_60"); // ISA 28 (5)
  sched.push("ins+11_10_cond=on:gs=on:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.1:igs=1010:igwr=on:lcm=reverse:nwc=1.3:sd=1:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // ISA 29 (5)
  sched.push("dis+1002_3_cond=fast:ep=RSTC:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // ISA 30 (4)
  sched.push("lrs+11_4_ccuc=small_ones:fde=none:gs=on:gsaa=from_current:gsem=off:gsssp=full:nm=0:nwc=1.2:stl=300:sd=1:ss=axioms:st=3.0:sac=on:acc=model:add=off:aer=off:afr=on:afp=4000:afq=1.4:anc=none:urr=on:updr=off:uhcvi=on_60"); // ISA 31 (4)
  sched.push("dis-11_3_cond=on:fsr=off:gs=on:gsem=on:lcm=reverse:lwlo=on:nwc=1:sd=2:ss=axioms:sos=on:av=off_60"); // ISA 32 (4)
  sched.push("lrs+10_4:1_bd=off:ccuc=small_ones:gs=on:nwc=1:stl=300:sd=2:ss=priority:sos=all:sac=on:acc=model:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none_60"); // ISA 33 (4)
  sched.push("dis+11_5_br=off:ccuc=small_ones:cond=fast:fsr=off:gs=on:nwc=1:sd=1:ss=axioms:st=1.2:sos=on:acc=on:add=large:afp=100000:afq=1.2:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_60"); // ISA 34 (4)
  sched.push("ott+1_8:1_bd=off:br=off:cond=on:nm=64:nwc=1.2:sd=2:ss=priority:st=2.0:av=off:sp=occurrence:urr=on_60"); // ISA 35 (4)
  sched.push("dis+10_2:1_fde=none:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1.1:sd=1:ss=axioms:st=3.0:sos=on:sac=on:acc=on:afr=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:uhcvi=on_60"); // ISA 36 (4)
  sched.push("dis+10_4_cond=on:fsr=off:fde=unused:gs=on:gsem=off:gsssp=full:lcm=reverse:nm=1024:nwc=1:sd=2:ss=priority:sac=on:aer=off:afr=on:afp=4000:afq=1.1:anc=all:updr=off_60"); // ISA 37 (3)
  sched.push("lrs+11_6_cond=fast:fsr=off:fde=unused:gs=on:gsssp=full:nwc=1.1:stl=300:sd=1:ss=axioms:st=5.0:sos=on:sac=on:afp=4000:afq=1.4:amm=off:anc=none:sp=occurrence:urr=on_60"); // ISA 38 (3)
  sched.push("dis-4_3:1_ep=RST:nwc=1:sos=on:av=off:updr=off:uhcvi=on_60"); // ISA 39 (3)
  sched.push("lrs+1003_3_cond=fast:fde=unused:gs=on:gsaa=from_current:nwc=1:stl=300:sd=1:ss=axioms:sos=on:sac=on:add=off:afr=on:afp=1000:afq=1.4:anc=none:sp=reverse_arity:updr=off:uhcvi=on_60"); // ISA 40 (3)
  sched.push("dis-11_3:1_bd=off:fsr=off:fde=unused:lcm=reverse:nm=64:nwc=2.5:sd=5:ss=priority:st=3.0:av=off_60"); // ISA 41 (3)
  sched.push("dis+10_2_cond=fast:fsr=off:fde=none:gs=on:gsem=off:lcm=reverse:nwc=1.1:sd=1:ss=axioms:st=5.0:sos=on:av=off_60"); // ISA 42 (3)
  sched.push("dis+1011_1_fsr=off:fde=unused:nm=64:nwc=1.7:sd=2:ss=priority:av=off:updr=off_60"); // ISA 43 (3)
  sched.push("lrs+2_4_bd=off:cond=on:lcm=predicate:nwc=1:stl=300:sd=3:ss=priority:sos=on:add=off:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:urr=ec_only_60"); // ISA 44 (3)
  sched.push("dis+10_2_bd=off:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:nm=64:nwc=1:sd=3:ss=axioms:st=5.0:sos=on:sac=on:afp=10000:afq=1.4:anc=none:updr=off:uhcvi=on_60"); // ISA 45 (3)
  sched.push("dis+1002_32_bs=on:fde=none:nm=64:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:sac=on:acc=model:add=large:aer=off:afr=on:afp=10000:afq=1.2:anc=none_60"); // ISA 46 (3)
  sched.push("dis+1003_3_cond=on:ep=RST:fde=none:gs=on:gsem=off:lwlo=on:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:sac=on:aer=off:afr=on:afp=1000:afq=1.0:anc=none:updr=off_60"); // ISA 47 (2)
  sched.push("dis+11_24_br=off:cond=fast:ep=RST:fsr=off:fde=none:gsp=input_only:gs=on:gsaa=full_model:gsem=off:nwc=1.1:sd=1:ss=axioms:sac=on:add=off:afr=on:afp=4000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // ISA 48 (2)
  sched.push("dis+3_1_cond=on:fde=unused:nwc=1:sd=1:ss=axioms:st=1.2:sos=on:sac=on:add=off:afp=40000:afq=1.4:anc=none_60"); // ISA 49 (2)
  sched.push("dis+10_3_bd=off:fsr=off:gs=on:gsaa=full_model:gsssp=full:lcm=reverse:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:sac=on:add=large:afr=on:afp=10000:afq=1.4:anc=none:sp=occurrence_60"); // ISA 50 (2)
  sched.push("dis+10_2_bd=off:fde=unused:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:av=off:uhcvi=on_60"); // ISA 51 (2)
  sched.push("lrs+11_12_bd=off:bs=unit_only:ccuc=small_ones:cond=fast:fde=none:nwc=2.5:stl=300:sd=5:ss=priority:st=1.2:sos=all:acc=model:aer=off:afp=100000:afq=1.4:anc=none_60"); // ISA 52 (2)
  sched.push("dis+11_3_br=off:ccuc=small_ones:cond=fast:gsp=input_only:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:acc=on:afr=on:afp=1000:afq=2.0:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_60"); // ISA 53 (2)
  sched.push("ins+11_4_bd=off:fde=unused:gs=on:igbrr=0.6:igrr=1/16:igrp=4000:igrpq=1.3:igwr=on:lcm=predicate:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:av=off:urr=on_60"); // ISA 54 (2)
  sched.push("ott+1011_10_fsr=off:fde=unused:nm=0:nwc=1:sd=3:ss=priority:st=1.2:av=off:uhcvi=on_60"); // ISA 55 (2)
  sched.push("ott+11_5:1_bd=off:cond=fast:nm=64:nwc=1.1:sd=3:ss=priority:st=2.0:av=off:sp=reverse_arity:urr=on:updr=off_60"); // ISA 56 (2)
  sched.push("lrs-3_3:1_cond=fast:ep=R:gsp=input_only:gs=on:gsem=on:gsssp=full:lcm=predicate:nwc=1:stl=300:sd=1:ss=axioms:st=3.0:sac=on:add=off:afr=on:afp=40000:afq=1.1:amm=sco:anc=none:uhcvi=on_60"); // ISA 57 (1)
  sched.push("dis+1011_2_cond=fast:ep=RST:fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=4:sd=2:ss=priority:sac=on:add=large:afr=on:afp=40000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_60"); // ISA 58 (1)
  sched.push("dis+1002_4_cond=on:fde=none:gs=on:gsem=on:nwc=1:sd=4:ss=axioms:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // ISA 59 (1)
  sched.push("dis-1_4_cond=fast:fsr=off:gs=on:gsem=off:gsssp=full:lcm=reverse:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:av=off:updr=off:uhcvi=on_60"); // ISA 60 (1)
  sched.push("dis+10_4:1_fsr=off:gs=on:gsem=on:lcm=reverse:nm=64:nwc=1:sd=4:ss=priority:st=3.0:av=off:updr=off:uhcvi=on_60"); // ISA 61 (1)
  sched.push("dis+10_2_fsr=off:fde=none:lcm=reverse:lwlo=on:nm=64:nwc=1.2:sd=4:ss=priority:st=5.0:av=off:uhcvi=on_60"); // ISA 62 (1)
  sched.push("lrs+10_4_bd=off:cond=on:fde=unused:gs=on:gsem=off:lcm=predicate:nwc=1:stl=300:sd=2:ss=axioms:st=5.0:sos=on:av=off:urr=ec_only_60"); // ISA 63 (1)
  sched.push("ins+11_5_cond=on:ep=RST:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:igrr=1/8:igrpq=1.1:igs=1003:igwr=on:lcm=reverse:nwc=1:sd=1:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // ISA 64 (1)
  sched.push("dis+10_3_ep=RST:fde=unused:gs=on:gsem=off:nwc=1:sos=on:afp=100000:afq=1.1:amm=sco:anc=none:sp=occurrence:updr=off_60"); // ISA 65 (1)
  sched.push("dis+10_4_cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:nwc=1.5:sd=1:ss=axioms:st=3.0:sac=on:add=off:afr=on:afp=10000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:updr=off:uhcvi=on_60"); // ISA 66 (1)
  sched.push("ott+10_8_cond=on:gs=on:gsem=off:nm=64:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_60"); // ISA 67 (1)
  sched.push("dis+1003_2:1_cond=fast:fde=none:nwc=1:sd=1:ss=axioms:st=2.0:sos=all:av=off:sp=reverse_arity:urr=ec_only_60"); // ISA 68 (1)
  sched.push("lrs+10_3_bd=off:cond=on:gs=on:gsem=off:nwc=1:stl=300:sd=3:ss=priority:st=3.0:sos=all:av=off:uhcvi=on_60"); // ISA 69 (1)
  sched.push("lrs+10_3_bd=off:cond=fast:fde=unused:lcm=reverse:nwc=1:stl=300:sd=5:ss=axioms:st=1.5:sos=on:av=off:sp=occurrence:urr=ec_only_60"); // ISA 70 (1)
  sched.push("dis+1011_4_cond=fast:ep=RST:fsr=off:gs=on:nm=64:nwc=1:sd=3:ss=axioms:st=3.0:sos=on:av=off_60"); // ISA 71 (1)
  sched.push("dis+11_1_cond=fast:gsp=input_only:lcm=predicate:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // ISA 72 (1)
  sched.push("dis-11_4_bd=off:fde=none:gs=on:gsem=on:lwlo=on:nwc=1:sd=3:ss=axioms:st=5.0:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_60"); // ISA 73 (1)
  sched.push("dis+10_1_cond=fast:ep=RST:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:sd=4:ss=axioms:st=3.0:sos=on:sac=on:add=off:afr=on:afp=40000:afq=2.0:anc=none:sp=occurrence:urr=ec_only_60"); // ISA 74 (1)
  sched.push("lrs+11_40_bs=unit_only:cond=fast:gs=on:gsem=on:gsssp=full:lcm=reverse:nm=64:nwc=1.3:stl=300:sd=3:ss=priority:av=off:sp=reverse_arity:updr=off_60"); // ISA 75 (1)
  sched.push("dis+1002_7_gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:sac=on:add=large:afr=on:afp=40000:afq=1.0:amm=off:anc=none_60"); // ISA 76 (1)
  sched.push("lrs+11_1_cond=fast:ep=RST:lwlo=on:nwc=1:stl=300:sd=2:ss=axioms:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // ISA 77 (1)
  sched.push("lrs+11_4_bd=off:br=off:cond=on:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=300:sd=2:ss=axioms:st=5.0:sos=all:sac=on:add=large:aer=off:afp=1000:afq=1.4:anc=none:sp=occurrence:urr=on:uhcvi=on_60"); // ISA 78 (1)
  sched.push("lrs+1010_8:1_bs=unit_only:cond=fast:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=off:lwlo=on:nwc=4:stl=300:sd=3:ss=priority:st=2.0:sac=on:aac=none:add=off:aer=off:afp=1000:afq=2.0:sp=occurrence:urr=ec_only:uhcvi=on_60"); // ISA 79 (1)
  sched.push("lrs+1011_4_ep=RST:fsr=off:gs=on:gsssp=full:nwc=1.1:stl=300:sos=on:av=off:updr=off_60"); // ISA 80 (1)
  sched.push("lrs+11_24_bd=off:bsr=unit_only:cond=on:gs=on:gsssp=full:nm=0:nwc=1.1:stl=300:sd=1:ss=axioms:st=3.0:sos=all:av=off:sp=occurrence:urr=ec_only_60"); // ISA 81 (1)
}

// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------


void Schedules::getLtb2017HllSchedule(const Property& property, Schedule& sched) {
  sched.push("ins+11_3_ep=RST:fde=unused:gsp=input_only:igbrr=0.4:igrr=1/8:igrpq=1.5:igs=1:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:updr=off:uhcvi=on_60"); // HLL 1 (1373)
  sched.push("lrs-4_5:4_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1.1:nicw=on:stl=300:sd=1:ss=axioms:st=2.0:sos=on:sac=on:afr=on:afp=10000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_60"); // HLL 2 (382)
  sched.push("dis+1002_1_ep=RST:gs=on:gsaa=full_model:gsem=on:nm=64:nwc=1:sd=7:ss=axioms:st=1.2:sos=on:aer=off:afp=40000:afq=1.2:anc=none:updr=off:uhcvi=on_60"); // HLL 3 (170)
  sched.push("dis+1002_1_gs=on:gsem=off:nwc=1:sd=3:ss=axioms:sos=on:sac=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:urr=on_60"); // HLL 4 (148)
  sched.push("lrs+1011_4:1_bd=off:bsr=unit_only:ccuc=small_ones:fsr=off:fde=unused:gs=on:gsssp=full:nm=64:nwc=4:stl=300:sd=1:ss=priority:sac=on:acc=model:add=large:aer=off:afr=on:afp=100000:afq=1.2:anc=all:uhcvi=on_60"); // HLL 5 (68)
  sched.push("ins+11_5_br=off:ep=RST:fde=none:gsp=input_only:gs=on:gsem=on:igbrr=0.5:igpr=on:igrp=1400:igrpq=1.3:igs=1:igwr=on:nm=0:nwc=1:sd=1:ss=axioms:st=2.0:sos=all:av=off:urr=on:updr=off_60"); // HLL 6 (64)
  sched.push("lrs+10_1_bsr=unit_only:cond=fast:gs=on:gsem=off:gsssp=full:lcm=reverse:nwc=1:stl=300:sd=2:ss=axioms:st=5.0:sos=on:sac=on:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_60"); // HLL 7 (62)
  sched.push("dis+10_3:1_ep=RST:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1.1:sd=2:ss=priority:st=2.0:sos=on:sac=on:add=large:aer=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity_60"); // HLL 8 (42)
  sched.push("lrs+1011_3:1_bd=off:bsr=on:cond=fast:gs=on:gsem=on:lwlo=on:nwc=10:stl=300:sd=1:ss=axioms:st=3.0:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HLL 9 (35)
  sched.push("lrs+1011_5:1_fde=none:gs=on:gsem=on:nwc=4:stl=300:sd=1:ss=axioms:st=5.0:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // HLL 10 (25)
  sched.push("ins+11_4_fsr=off:fde=unused:gsp=input_only:gs=on:igbrr=0.3:igrr=1/4:igrp=700:igrpq=1.3:igs=1:igwr=on:lcm=reverse:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // HLL 11 (22)
  sched.push("ott+11_2:3_cond=on:ep=RST:fsr=off:fde=none:gsp=input_only:lcm=predicate:nwc=1:sd=3:ss=priority:sos=all:sac=on:aac=none:aer=off:afp=100000:afq=1.2:anc=all_dependent:urr=ec_only_60"); // HLL 12 (21)
  sched.push("dis+1011_5:1_ep=RSTC:fde=unused:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sd=1:ss=axioms:st=3.0:sos=on:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HLL 13 (16)
  sched.push("dis+1011_5:1_cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:nwc=3:sd=2:ss=axioms:sac=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_60"); // HLL 14 (14)
  sched.push("lrs+11_3_cond=fast:gsp=input_only:gs=on:gsem=on:gsssp=full:lcm=predicate:nwc=1:stl=300:sd=2:ss=axioms:sos=on:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:updr=off:uhcvi=on_60"); // HLL 15 (14)
  sched.push("dis+1011_2:1_cond=fast:ep=RST:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sd=1:ss=axioms:sos=on:add=large:aer=off:afr=on:afp=4000:afq=1.1:anc=none:sp=reverse_arity_60"); // HLL 16 (13)
  sched.push("dis+1011_1_cond=fast:ep=RST:gs=on:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:sac=on:amm=sco:anc=none:urr=ec_only_60"); // HLL 17 (12)
  sched.push("lrs+10_4_bd=off:bsr=unit_only:fde=none:gs=on:lcm=reverse:nwc=1:stl=300:sd=3:ss=axioms:st=3.0:sos=on:av=off:uhcvi=on_60"); // HLL 18 (11)
  sched.push("dis+1002_7_bsr=unit_only:cond=fast:nm=64:nwc=1:sd=1:ss=axioms:sos=on:sac=on:afr=on:afp=100000:afq=1.4:anc=none:uhcvi=on_60"); // HLL 19 (11)
  sched.push("lrs+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:stl=300:sd=2:ss=axioms:sos=on:av=off:urr=on:updr=off:uhcvi=on_60"); // HLL 20 (10)
  sched.push("dis+2_4_bd=off:cond=fast:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:lwlo=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // HLL 21 (9)
  sched.push("lrs+1010_5:1_bs=unit_only:bsr=on:fde=none:gs=on:gsem=off:gsssp=full:lcm=reverse:nm=0:nwc=4:stl=300:sd=3:ss=priority:st=1.2:sos=on:aac=none:acc=model:afr=on:afp=1000:afq=1.0:amm=off:urr=on:uhcvi=on_60"); // HLL 22 (8)
  sched.push("lrs-11_8:1_bsr=on:cond=on:fde=none:lcm=reverse:nm=0:nwc=1.5:stl=300:sd=2:ss=priority:av=off:sp=occurrence_60"); // HLL 23 (7)
  sched.push("dis+1002_3_cond=on:ep=RS:fsr=off:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:afp=4000:afq=1.4:amm=off:anc=none:updr=off_60"); // HLL 24 (7)
  sched.push("dis+1003_5_cond=on:fsr=off:fde=none:gs=on:gsem=off:nwc=1:sos=on:add=large:aer=off:afr=on:afp=100000:afq=1.0:anc=all_dependent:sp=reverse_arity:urr=ec_only:uhcvi=on_60"); // HLL 25 (6)
  sched.push("lrs+10_5:4_bd=off:gs=on:gsssp=full:lcm=reverse:nwc=1:stl=300:sd=1:ss=axioms:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:updr=off:uhcvi=on_60"); // HLL 26 (6)
  sched.push("lrs+11_4:1_br=off:cond=on:fsr=off:fde=unused:gsp=input_only:gs=on:gsssp=full:lcm=predicate:nm=0:nwc=1:stl=300:sd=1:ss=axioms:av=off:sp=occurrence:urr=on_60"); // HLL 27 (6)
  sched.push("lrs+1010_5_cond=fast:ep=RST:gs=on:gsaa=from_current:gsem=on:nwc=1:stl=300:sd=4:ss=axioms:st=1.5:sos=on:sac=on:add=off:afp=4000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:uhcvi=on_60"); // HLL 28 (6)
  sched.push("lrs+11_3_bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:stl=300:sd=1:ss=axioms:sos=all:add=large:aer=off:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence:urr=on:updr=off_60"); // HLL 29 (5)
  sched.push("lrs+4_5:4_bd=off:bs=on:bsr=unit_only:cond=fast:fde=unused:gs=on:gsem=off:nwc=1:stl=300:sd=2:ss=axioms:st=2.0:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // HLL 30 (5)
  sched.push("lrs+11_5:1_bd=off:br=off:cond=fast:gsp=input_only:gs=on:gsem=on:gsssp=full:lcm=predicate:nwc=1.1:stl=300:sd=2:ss=priority:st=3.0:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // HLL 31 (5)
  sched.push("dis+1011_3:2_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sd=1:ss=priority:sos=all:add=large:anc=all:sp=occurrence_60"); // HLL 32 (5)
  sched.push("lrs+11_3:1_br=off:cond=fast:ep=R:fsr=off:gs=on:nwc=1:stl=300:sd=2:ss=priority:st=2.0:sos=all:av=off:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HLL 33 (4)
  sched.push("lrs+11_3_bsr=unit_only:cond=on:ep=RST:gsp=input_only:nwc=1.7:stl=300:sd=3:ss=axioms:st=5.0:sos=all:sac=on:afr=on:afp=100000:afq=1.1:anc=all_dependent:sp=reverse_arity:updr=off:uhcvi=on_60"); // HLL 34 (4)
  sched.push("ins+11_5_ep=RS:fsr=off:gs=on:igbrr=0.4:igpr=on:igrr=1/2:igrp=4000:igrpq=1.2:igs=1010:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_60"); // HLL 35 (3)
  sched.push("dis+1010_2:3_bs=unit_only:bsr=unit_only:cond=fast:fsr=off:fde=unused:gs=on:gsem=on:gsssp=full:nm=0:nwc=3:sd=4:ss=priority:st=1.5:sos=on:acc=on:add=off:aer=off:afr=on:afp=100000:afq=1.0:sp=reverse_arity:uhcvi=on_60"); // HLL 36 (3)
  sched.push("dis+1010_5:4_bd=off:fsr=off:fde=unused:gs=on:nm=64:nwc=1:sd=4:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // HLL 37 (3)
  sched.push("lrs+1011_8:1_bd=off:bsr=unit_only:fde=none:gs=on:lwlo=on:nm=0:nwc=1.5:stl=300:sd=1:ss=axioms:st=1.2:av=off:sp=occurrence:updr=off_60"); // HLL 38 (3)
  sched.push("dis+4_5:4_bd=off:fsr=off:fde=unused:gs=on:nwc=1:sd=5:ss=axioms:st=1.5:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // HLL 39 (3)
  sched.push("dis+1011_3_cond=fast:ep=R:gs=on:gsem=off:lwlo=on:nm=0:nwc=1:sd=5:ss=axioms:st=1.5:sos=on:sac=on:add=large:afr=on:afp=1000:afq=1.1:anc=none:uhcvi=on_60"); // HLL 40 (2)
  sched.push("lrs+1004_3:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:stl=300:sd=2:ss=axioms:sos=on:av=off:updr=off_60"); // HLL 41 (2)
  sched.push("lrs+10_1_bd=off:bsr=unit_only:cond=fast:fsr=off:fde=unused:gs=on:gsem=off:lcm=reverse:nwc=1:stl=300:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_60"); // HLL 42 (2)
  sched.push("lrs+10_4_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:stl=300:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // HLL 43 (2)
  sched.push("ins+11_4_ep=RS:igbrr=0.6:igpr=on:igrr=1/128:igrp=2000:igrpq=2.0:igs=1002:igwr=on:nwc=1:sd=1:ss=axioms:sos=all:av=off:uhcvi=on_60"); // HLL 44 (2)
  sched.push("lrs+1010_2:3_bsr=unit_only:ccuc=small_ones:cond=on:fde=unused:gs=on:gsaa=full_model:nwc=1:stl=300:sd=2:ss=axioms:st=1.5:sos=on:sac=on:acc=model:add=off:aer=off:afr=on:afp=1000:afq=2.0:sp=occurrence:uhcvi=on_60"); // HLL 45 (2)
  sched.push("dis+10_1_bd=preordered:bs=unit_only:cond=on:fde=none:lcm=predicate:nwc=1:sd=2:ss=axioms:sos=all:sac=on:afr=on:amm=sco:anc=none:updr=off:uhcvi=on_60"); // HLL 46 (2)
  sched.push("lrs+11_5_bd=off:bsr=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsssp=full:nwc=1:stl=300:sd=1:ss=axioms:st=5.0:sos=all:add=off:afp=4000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HLL 47 (2)
  sched.push("dis+11_2:1_br=off:ep=RST:fde=unused:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nwc=1:sd=1:ss=priority:st=1.2:sos=all:add=large:aer=off:afp=100000:afq=1.1:anc=none:sp=occurrence:urr=on_60"); // HLL 48 (2)
  sched.push("dis+1011_2:3_cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:sd=2:ss=axioms:sos=on:sac=on:add=large:afr=on:afp=1000:afq=2.0:anc=none:sp=reverse_arity:urr=ec_only:uhcvi=on_60"); // HLL 49 (2)
  sched.push("lrs+1003_4_bsr=unit_only:cond=fast:fsr=off:gsp=input_only:gs=on:gsaa=from_current:nm=0:nwc=1:stl=300:sos=on:sac=on:add=large:afp=10000:afq=1.1:anc=none:urr=ec_only:uhcvi=on_60"); // HLL 50 (1)
  sched.push("ins+11_4:1_cond=on:ep=RSTC:fde=none:gs=on:gsem=on:igbrr=0.2:igpr=on:igrr=32/1:igrp=2000:igrpq=1.3:igs=1002:igwr=on:nm=0:nwc=1:sd=2:ss=axioms:sos=all:av=off:sp=reverse_arity:uhcvi=on_60"); // HLL 51 (1)
  sched.push("dis+11_20_cond=fast:ep=R:fde=none:lwlo=on:nm=0:nwc=10:sd=4:ss=axioms:st=2.0:add=large:amm=off:sp=occurrence_60"); // HLL 52 (1)
  sched.push("lrs-2_3_bd=off:bs=unit_only:cond=on:fde=none:nwc=1:stl=300:sd=1:ss=axioms:st=1.2:sos=all:av=off:sp=occurrence:urr=ec_only:updr=off_60"); // HLL 53 (1)
  sched.push("lrs+11_3_bsr=unit_only:cond=on:ep=RST:fsr=off:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:stl=300:sd=10:ss=axioms:st=1.5:sos=all:add=off:afp=40000:afq=1.0:anc=none:sp=occurrence:urr=on_60"); // HLL 54 (1)
  sched.push("lrs+1004_4_cond=on:fde=unused:gsp=input_only:gs=on:nwc=1:stl=300:sd=3:ss=axioms:st=5.0:sos=on:av=off:sp=occurrence:urr=on:updr=off_60"); // HLL 55 (1)
  sched.push("lrs+10_4_bsr=unit_only:cond=fast:fsr=off:fde=unused:gs=on:gsssp=full:nwc=1:stl=300:sd=2:ss=axioms:st=2.0:sos=on:afp=10000:afq=1.0:amm=sco:anc=all_dependent:sp=occurrence:uhcvi=on_60"); // HLL 56 (1)
  sched.push("dis+1011_3:1_cond=fast:fsr=off:fde=unused:lwlo=on:nwc=1:sd=2:ss=axioms:st=1.2:av=off:sp=reverse_arity:uhcvi=on_60"); // HLL 57 (1)
  sched.push("lrs-10_3:1_cond=fast:fde=unused:gs=on:gsaa=from_current:gsem=off:lcm=predicate:nwc=1:stl=300:sd=1:ss=axioms:sos=on:sac=on:add=off:afp=100000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:uhcvi=on_60"); // HLL 58 (1)
  sched.push("lrs-10_3:1_bd=off:cond=fast:fsr=off:fde=unused:gs=on:gsem=off:gsssp=full:lcm=reverse:nm=0:nwc=1:stl=300:sd=4:ss=axioms:st=1.5:sos=on:av=off:urr=ec_only:updr=off_60"); // HLL 59 (1)
  sched.push("ins+11_5_fde=unused:gsp=input_only:gs=on:igbrr=0.4:igrr=1/16:igrp=100:igrpq=1.5:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:av=off:urr=ec_only_60"); // HLL 60 (1)
  sched.push("lrs+4_5_bd=off:fde=none:nwc=1.1:stl=300:sd=2:ss=axioms:st=5.0:sos=on:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HLL 61 (1)
  sched.push("lrs-2_5:4_bd=off:bsr=unit_only:fsr=off:nm=0:nwc=1:stl=300:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity_60"); // HLL 62 (1)
  sched.push("lrs+11_5_bd=off:fde=none:gsp=input_only:gs=on:gsaa=full_model:gsssp=full:nwc=1:stl=300:sd=2:ss=priority:st=2.0:sos=all:sac=on:add=large:aer=off:afp=40000:afq=1.2:anc=none:uhcvi=on_60"); // HLL 63 (1)
  sched.push("lrs+11_24_bsr=unit_only:gsp=input_only:gs=on:gsem=off:gsssp=full:nm=0:nwc=1:stl=300:sd=2:ss=axioms:sos=all:sac=on:afp=1000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:updr=off_60"); // HLL 64 (1)
  sched.push("ins+11_6_fsr=off:igbrr=0.4:igrr=1/64:igrp=100:igrpq=1.5:igs=1010:igwr=on:lcm=reverse:nwc=1:sd=2:ss=priority:av=off:sp=occurrence:updr=off_60"); // HLL 65 (1)
  sched.push("dis+11_2_bd=off:cond=fast:gs=on:gsaa=full_model:nwc=1:sd=7:ss=axioms:st=1.2:sos=all:sac=on:add=off:afr=on:afp=40000:afq=1.2:anc=all_dependent_60"); // HLL 66 (1)
  sched.push("lrs+1004_2:3_bd=off:ccuc=small_ones:cond=fast:fsr=off:fde=none:lwlo=on:nm=0:nwc=1.5:stl=300:sac=on:aac=none:acc=on:add=large:afp=40000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_60"); // HLL 67 (1)
  sched.push("dis+11_1_cond=fast:fde=none:gs=on:gsssp=full:nwc=1.1:sd=1:ss=axioms:sac=on:add=large:afp=1000:afq=2.0:amm=sco:anc=none:urr=on:updr=off:uhcvi=on_60"); // HLL 68 (1)
  sched.push("dis+2_5_bd=off:cond=fast:gs=on:lcm=reverse:nwc=1:sd=3:ss=axioms:sos=on:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HLL 69 (1)
  sched.push("lrs+4_3_bsr=unit_only:cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:stl=300:sd=1:ss=axioms:st=1.5:sos=on:sac=on:add=off:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // HLL 70 (1)
  sched.push("lrs+11_2:3_bd=off:cond=on:fde=none:nwc=1:stl=300:sd=10:ss=axioms:st=1.2:sos=all:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HLL 71 (1)
  sched.push("dis+11_4_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsaa=from_current:nwc=1:sd=2:ss=axioms:sos=all:add=off:aer=off:anc=none:sp=occurrence:uhcvi=on_60"); // HLL 72 (1)
  sched.push("lrs+1003_3_bd=off:bsr=unit_only:cond=on:nwc=1:stl=300:sd=2:ss=priority:av=off_60"); // HLL 73 (1)
}

// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------

void Schedules::getLtb2017MzrSchedule(const Property& property, Schedule& sched) {
  sched.push("ott-11_8:1_bsr=unit_only:lcm=predicate:nwc=1:sd=2:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity_60"); // MZR 1 (828)
  sched.push("ott+1010_3:1_bs=unit_only:bsr=unit_only:br=off:ccuc=first:cond=fast:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:sos=on:sac=on:aac=none:acc=on:aer=off:afp=1000:afq=2.0:anc=all_dependent:sp=reverse_arity:urr=on:updr=off_60"); // MZR 2 (112)
  sched.push("ins+11_3_cond=fast:igbrr=0.7:igpr=on:igrr=1/32:igrp=700:igrpq=1.5:igs=1003:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // MZR 3 (81)
  sched.push("lrs+1004_2_bd=off:ccuc=small_ones:gs=on:gsem=off:gsssp=full:lwlo=on:nm=0:nwc=1:stl=300:sd=4:ss=priority:st=5.0:sos=all:sac=on:acc=on:add=off:aer=off:afp=100000:afq=1.2:anc=none:sp=occurrence:updr=off:uhcvi=on_60"); // MZR 4 (47)
  sched.push("dis+10_5_bsr=unit_only:cond=on:ep=RS:fde=unused:nm=0:nwc=1:sd=1:ss=axioms:sos=all:av=off_60"); // MZR 5 (37)
  sched.push("lrs+11_5:1_br=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:lcm=predicate:nm=0:nwc=1:nicw=on:stl=300:sd=1:ss=axioms:st=1.2:sac=on:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=all:urr=on_60"); // MZR 6 (32)
  sched.push("lrs+1011_8:1_cond=on:fde=none:gsp=input_only:lwlo=on:nwc=1:stl=300:sd=2:ss=axioms:sos=all:av=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_60"); // MZR 7 (22)
  sched.push("lrs+11_3_br=off:cond=fast:gs=on:gsem=off:nwc=1:stl=300:sd=3:ss=priority:st=1.5:sos=all:sac=on:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=on:uhcvi=on_60"); // MZR 8 (21)
  sched.push("dis+10_2:1_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:add=off:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:urr=on:updr=off:uhcvi=on_60"); // MZR 9 (19)
  sched.push("lrs+1002_1_bsr=unit_only:nwc=1:stl=300:sd=1:ss=axioms:st=1.5:sos=all:av=off:updr=off:uhcvi=on_60"); // MZR 10 (14)
  sched.push("lrs+1_1_bs=on:bsr=on:br=off:cond=fast:fsr=off:gs=on:gsem=off:lwlo=on:nwc=3:stl=300:sd=3:ss=priority:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:urr=on:updr=off_60"); // MZR 11 (11)
  sched.push("lrs-2_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsssp=full:lcm=predicate:nwc=1:stl=300:sd=4:ss=axioms:st=3.0:sos=on:sac=on:afr=on:afp=10000:afq=1.1:anc=none:updr=off_60"); // MZR 12 (11)
  sched.push("lrs+10_8:1_bsr=unit_only:br=off:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:nm=0:nwc=1:stl=300:sd=2:ss=axioms:st=1.2:sos=on:sac=on:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // MZR 13 (10)
  sched.push("dis+11_12_cond=fast:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // MZR 14 (8)
  sched.push("dis+1010_3_bsr=unit_only:cond=fast:fde=none:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // MZR 15 (8)
  sched.push("dis+1002_2:3_fde=none:gsp=input_only:nm=0:nwc=1:sd=3:ss=axioms:sos=on:sac=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=occurrence:updr=off_60"); // MZR 16 (7)
  sched.push("lrs+10_3:1_fde=unused:lcm=reverse:nwc=1:stl=300:sd=3:ss=priority:st=2.0:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // MZR 17 (6)
  sched.push("lrs+10_2:3_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=300:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity_60"); // MZR 18 (6)
  sched.push("dis+1004_3:1_bsr=unit_only:ep=R:fde=unused:gs=on:gsssp=full:nm=0:nwc=1:sos=all:sac=on:afr=on:afp=10000:afq=2.0:anc=all:sp=reverse_arity:urr=on:updr=off_60"); // MZR 19 (5)
  sched.push("ott+4_5:1_br=off:cond=fast:fsr=off:nwc=1.3:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // MZR 20 (5)
  sched.push("dis+1010_1_cond=fast:fsr=off:nwc=1.3:sd=2:ss=axioms:st=1.5:sos=on:acc=model:add=off:afp=4000:afq=2.0:uhcvi=on_60"); // MZR 21 (5)
  sched.push("lrs+11_2_bd=off:bsr=unit_only:cond=on:lcm=predicate:lwlo=on:nm=64:nwc=1.1:stl=300:sd=1:ss=axioms:st=2.0:sos=all:av=off_60"); // MZR 22 (5)
  sched.push("lrs+10_4:1_bd=off:cond=fast:fde=unused:lcm=reverse:nm=0:nwc=1.2:stl=300:sd=2:ss=axioms:sos=all:av=off_60"); // MZR 23 (5)
  sched.push("dis+10_5_ep=RST:fsr=off:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sd=4:ss=axioms:sos=on:afr=on:afp=40000:afq=1.1:amm=off:anc=none:uhcvi=on_60"); // MZR 24 (4)
  sched.push("dis+1002_3_ep=RST:fde=unused:gs=on:gsaa=full_model:gsem=off:nwc=1:sd=1:ss=axioms:st=2.0:sos=on:afp=100000:afq=1.1:anc=none:sp=occurrence:uhcvi=on_60"); // MZR 25 (4)
  sched.push("dis+10_5_cond=on:fsr=off:fde=none:gs=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:av=off_60"); // MZR 26 (4)
  sched.push("ott-11_8:1_bd=preordered:ccuc=first:er=known:fsr=off:fde=unused:gsp=input_only:lcm=predicate:nm=0:nwc=2:sd=3:ss=axioms:acc=on:afp=10000:afq=2.0:amm=sco:sp=occurrence:updr=off_60"); // MZR 27 (4)
  sched.push("ins+11_4_bd=off:fsr=off:gsp=input_only:gs=on:gsem=off:igbrr=0.6:igpr=on:igrr=1/128:igrp=700:igrpq=1.2:igs=1004:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:av=off:uhcvi=on_60"); // MZR 28 (3)
  sched.push("lrs+1011_1_cond=on:fsr=off:gs=on:nwc=1:stl=300:sd=4:ss=priority:st=1.2:sos=on:av=off:sp=reverse_arity:urr=on_60"); // MZR 29 (3)
  sched.push("lrs+11_3:1_bsr=unit_only:br=off:cond=on:ep=RST:fsr=off:gs=on:gsaa=from_current:gsem=off:nwc=3:stl=300:sd=2:ss=axioms:st=2.0:sac=on:add=large:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:urr=on_60"); // MZR 30 (3)
  sched.push("ins+11_3_cond=fast:fde=unused:igbrr=0.6:igrr=1/2:igrp=1400:igrpq=2.0:igs=1003:igwr=on:nwc=1:sos=on:av=off:updr=off_60"); // MZR 31 (3)
  sched.push("dis+11_5:4_ccuc=first:cond=on:er=known:fde=none:gs=on:nwc=1:sd=2:ss=priority:st=1.2:sos=all:aac=none:acc=model:add=large:afr=on:afp=10000:afq=1.2:anc=all_dependent:sp=reverse_arity:urr=on:uhcvi=on_60"); // MZR 32 (3)
  sched.push("lrs+1010_2_cond=on:fde=none:gs=on:gsem=off:nwc=1:stl=300:sd=3:ss=priority:st=1.2:sos=all:av=off:uhcvi=on_60"); // MZR 33 (3)
  sched.push("ins+11_4_fde=unused:gs=on:igbrr=0.7:igpr=on:igrr=1/4:igrp=100:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:nwc=1:sd=3:ss=axioms:st=1.5:sos=all:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // MZR 34 (3)
  sched.push("lrs+10_5:4_bd=off:ccuc=small_ones:cond=on:fde=none:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:stl=300:sd=2:ss=priority:sos=on:acc=model:add=large:aer=off:afp=100000:afq=1.4:anc=none:urr=on_60"); // MZR 35 (2)
  sched.push("dis+11_1_ccuc=first:cond=on:fsr=off:fde=none:gs=on:gsem=off:lcm=predicate:nm=0:nwc=1:nicw=on:sd=3:ss=priority:acc=model:add=large:afp=4000:afq=1.4:anc=all:sp=occurrence:updr=off:uhcvi=on_60"); // MZR 36 (2)
  sched.push("dis+1_1_fsr=off:gs=on:gsem=on:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:acc=on:afr=on:afp=10000:afq=1.0:amm=sco:anc=none:sp=reverse_arity_60"); // MZR 37 (2)
  sched.push("dis+1004_2_bs=unit_only:bsr=unit_only:fde=unused:gs=on:nwc=1:sos=on:add=large:afr=on_60"); // MZR 38 (2)
  sched.push("dis+11_4_ep=RS:fde=none:gs=on:gsaa=full_model:gsem=off:nwc=1:sd=1:ss=priority:st=1.2:sos=all:sac=on:afp=10000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:uhcvi=on_60"); // MZR 39 (2)
  sched.push("lrs+11_8_br=off:cond=on:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:nicw=on:stl=300:sd=1:ss=axioms:st=5.0:sos=all:sac=on:add=off:afp=100000:afq=1.4:amm=off:anc=all:sp=reverse_arity:urr=on:uhcvi=on_60"); // MZR 40 (2)
  sched.push("ott+1_28_cond=fast:er=filter:fde=none:gsp=input_only:lcm=reverse:nwc=1.1:sos=on:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // MZR 41 (2)
  sched.push("ins+10_5_cond=fast:fde=none:gsp=input_only:gs=on:gsem=on:igbrr=0.8:igrr=1/32:igrpq=1.5:igs=1003:igwr=on:nwc=1:sd=3:ss=priority:st=1.2:sos=all:av=off:sp=occurrence:urr=ec_only_60"); // MZR 42 (2)
  sched.push("dis+10_14_cond=fast:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1.5:sd=1:ss=axioms:st=1.5:afp=40000:afq=1.1:amm=sco:anc=none:sp=occurrence:updr=off_60"); // MZR 43 (2)
  sched.push("ins+11_5_cond=fast:fsr=off:fde=none:gsp=input_only:gs=on:gsem=off:igbrr=1.0:igrr=1/64:igrp=4000:igrpq=1.3:igs=1002:igwr=on:lcm=predicate:nm=0:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:av=off:uhcvi=on_60"); // MZR 44 (2)
  sched.push("lrs+11_5_fde=none:gsp=input_only:gs=on:gsem=on:nwc=1:stl=300:sd=3:ss=axioms:st=3.0:sos=on:av=off:sp=occurrence:urr=on_60"); // MZR 45 (2)
  sched.push("lrs-10_4:1_cond=on:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=on:nwc=1:stl=300:sd=3:ss=axioms:sos=on:av=off:urr=on_60"); // MZR 46 (2)
  sched.push("lrs+3_3:1_bd=preordered:bs=on:bsr=unit_only:fsr=off:fde=unused:gs=on:gsem=off:nwc=1:nicw=on:stl=300:sd=2:ss=axioms:st=3.0:sos=all:sac=on:add=large:afp=40000:afq=1.2:amm=sco:anc=all_dependent:sp=reverse_arity:urr=ec_only_60"); // MZR 47 (2)
  sched.push("dis+11_3_cond=fast:fsr=off:nwc=1:sd=1:ss=axioms:st=5.0:add=off:afr=on:afp=4000:afq=1.1:anc=none:sp=occurrence:updr=off_60"); // MZR 48 (1)
  sched.push("dis+11_4_bd=off:fsr=off:fde=unused:gs=on:gsaa=full_model:gsem=on:nwc=1:sd=1:ss=axioms:sac=on:add=large:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity_60"); // MZR 49 (1)
  sched.push("dis+1010_2_bs=on:cond=fast:ep=RSTC:fde=unused:lwlo=on:nwc=1:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:sp=reverse_arity:uhcvi=on_60"); // MZR 50 (1)
  sched.push("dis+10_5_fsr=off:fde=unused:gs=on:gsem=on:gsssp=full:lcm=reverse:nwc=1:sd=2:ss=axioms:sos=on:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:sp=occurrence:updr=off:uhcvi=on_60"); // MZR 51 (1)
  sched.push("lrs+1_4:1_br=off:cond=on:er=known:fsr=off:fde=unused:gs=on:nm=0:nwc=1:stl=600:sd=1:ss=priority:st=1.5:sos=on:sac=on:add=off:afp=4000:afq=1.1:amm=sco:anc=none:urr=on:updr=off:uhcvi=on_60"); // MZR 52 (1)
  sched.push("ins+11_10_cond=fast:fsr=off:gs=on:gsem=on:igbrr=0.5:igrr=1/2:igrpq=1.3:igs=1003:igwr=on:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity_60"); // MZR 53 (1)
  sched.push("dis+1010_5_cond=fast:fde=unused:gs=on:gsem=on:nm=0:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // MZR 54 (1)
  sched.push("lrs+11_8:1_bd=off:fde=unused:nwc=1:stl=300:sd=2:ss=axioms:sos=on:av=off:updr=off:uhcvi=on_60"); // MZR 55 (1)
  sched.push("lrs+1003_4_cond=on:fsr=off:fde=none:gs=on:gsem=off:nwc=1:stl=300:sd=3:ss=priority:sos=on:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // MZR 56 (1)
  sched.push("dis-10_2:3_cond=on:fde=none:nwc=1:sd=2:ss=axioms:st=2.0:sos=on:av=off:updr=off:uhcvi=on_60"); // MZR 57 (1)
  sched.push("dis+10_3_cond=fast:fde=unused:gs=on:gsem=off:lwlo=on:nwc=1:sd=3:ss=axioms:sos=on:add=large:afp=10000:afq=2.0:anc=none:sp=reverse_arity_60"); // MZR 58 (1)
  sched.push("lrs+11_4_cond=on:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:stl=300:sd=1:ss=axioms:st=2.0:sac=on:add=off:aer=off:afp=100000:afq=1.4:anc=none:sp=occurrence:updr=off:uhcvi=on_60"); // MZR 59 (1)
  sched.push("ott+10_5:1_bsr=unit_only:er=known:fsr=off:fde=none:gsp=input_only:lcm=reverse:nwc=2:av=off:sp=reverse_arity:urr=ec_only:uhcvi=on_60"); // MZR 60 (1)
  sched.push("ott+11_3:1_bs=unit_only:bsr=unit_only:br=off:cond=on:fsr=off:fde=none:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:av=off:sp=occurrence:urr=on_60"); // MZR 61 (1)
  sched.push("dis-11_5:1_bd=off:bsr=on:ccuc=small_ones:cond=on:fsr=off:fde=none:gs=on:lcm=predicate:nwc=10:acc=on:aer=off:afp=1000:afq=1.1:sp=occurrence:uhcvi=on_60"); // MZR 62 (1)
  sched.push("dis+11_5_60"); // MZR 63 (1)
  sched.push("lrs+1_8:1_bsr=on:fde=none:gs=on:lcm=reverse:nwc=1:stl=300:sd=2:ss=priority:st=1.5:sos=all:av=off:sp=reverse_arity_60"); // MZR 64 (1)
  sched.push("lrs+11_12_cond=fast:fde=unused:nwc=1:stl=600:sd=2:ss=priority:sos=all:av=off:sp=reverse_arity:uhcvi=on_60"); // MZR 65 (1)
  sched.push("lrs+11_5:1_bsr=unit_only:br=off:cond=fast:fsr=off:gs=on:nwc=1.2:stl=300:sd=1:ss=axioms:st=5.0:sos=all:av=off:sp=reverse_arity:urr=on_60"); // MZR 66 (1)
  sched.push("lrs+1004_5:1_ep=RST:fsr=off:nm=64:nwc=1.2:stl=600:add=off:aer=off:afr=on:afp=100000:afq=1.4:sp=occurrence:updr=off:uhcvi=on_60"); // MZR 67 (1)
  sched.push("dis-11_1_cond=fast:nm=0:nwc=1:sd=2:ss=axioms:sac=on:acc=model:afr=on:afp=100000:afq=1.2:amm=off:anc=all_dependent:sp=reverse_arity:uhcvi=on_60"); // MZR 68 (1)
}

// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------
// ---------- ---------- ---------- ---------- ---------- ---------- ---------- ---------- ----------

void Schedules::getLtb2017DefaultSchedule(const Property& property, Schedule& sched) {
  sched.push("ins+11_3_ep=RST:fde=unused:gsp=input_only:igbrr=0.4:igrr=1/8:igrpq=1.5:igs=1:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=3.0:sos=all:av=off:updr=off:uhcvi=on_60"); // HLL 1 (1373)
  sched.push("lrs-4_5:4_bd=off:bs=unit_only:bsr=on:cond=on:fde=none:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1.1:nicw=on:stl=300:sd=1:ss=axioms:st=2.0:sos=on:sac=on:afr=on:afp=10000:afq=1.0:amm=off:anc=none:sp=reverse_arity:urr=on:updr=off_60"); // HLL 2 (382)
  sched.push("dis+1002_1_ep=RST:gs=on:gsaa=full_model:gsem=on:nm=64:nwc=1:sd=7:ss=axioms:st=1.2:sos=on:aer=off:afp=40000:afq=1.2:anc=none:updr=off:uhcvi=on_60"); // HLL 3 (170)
  sched.push("dis+1002_1_gs=on:gsem=off:nwc=1:sd=3:ss=axioms:sos=on:sac=on:afp=1000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:urr=on_60"); // HLL 4 (148)
  sched.push("lrs+1011_4:1_bd=off:bsr=unit_only:ccuc=small_ones:fsr=off:fde=unused:gs=on:gsssp=full:nm=64:nwc=4:stl=300:sd=1:ss=priority:sac=on:acc=model:add=large:aer=off:afr=on:afp=100000:afq=1.2:anc=all:uhcvi=on_60"); // HLL 5 (68)
  sched.push("ins+11_5_br=off:ep=RST:fde=none:gsp=input_only:gs=on:gsem=on:igbrr=0.5:igpr=on:igrp=1400:igrpq=1.3:igs=1:igwr=on:nm=0:nwc=1:sd=1:ss=axioms:st=2.0:sos=all:av=off:urr=on:updr=off_60"); // HLL 6 (64)
  sched.push("lrs+10_1_bsr=unit_only:cond=fast:gs=on:gsem=off:gsssp=full:lcm=reverse:nwc=1:stl=300:sd=2:ss=axioms:st=5.0:sos=on:sac=on:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_60"); // HLL 7 (62)
  sched.push("dis+10_3:1_ep=RST:gsp=input_only:gs=on:gsem=on:lcm=reverse:nwc=1.1:sd=2:ss=priority:st=2.0:sos=on:sac=on:add=large:aer=off:afp=10000:afq=1.1:anc=none:sp=reverse_arity_60"); // HLL 8 (42)
  sched.push("lrs+1011_3:1_bd=off:bsr=on:cond=fast:gs=on:gsem=on:lwlo=on:nwc=10:stl=300:sd=1:ss=axioms:st=3.0:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HLL 9 (35)
  sched.push("lrs+1011_5:1_fde=none:gs=on:gsem=on:nwc=4:stl=300:sd=1:ss=axioms:st=5.0:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // HLL 10 (25)
  sched.push("ins+11_4_fsr=off:fde=unused:gsp=input_only:gs=on:igbrr=0.3:igrr=1/4:igrp=700:igrpq=1.3:igs=1:igwr=on:lcm=reverse:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // HLL 11 (22)
  sched.push("ott+11_2:3_cond=on:ep=RST:fsr=off:fde=none:gsp=input_only:lcm=predicate:nwc=1:sd=3:ss=priority:sos=all:sac=on:aac=none:aer=off:afp=100000:afq=1.2:anc=all_dependent:urr=ec_only_60"); // HLL 12 (21)
  sched.push("dis+1011_5:1_ep=RSTC:fde=unused:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sd=1:ss=axioms:st=3.0:sos=on:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HLL 13 (16)
  sched.push("dis+1011_5:1_cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:nwc=3:sd=2:ss=axioms:sac=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off_60"); // HLL 14 (14)
  sched.push("lrs+11_3_cond=fast:gsp=input_only:gs=on:gsem=on:gsssp=full:lcm=predicate:nwc=1:stl=300:sd=2:ss=axioms:sos=on:sac=on:add=large:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:updr=off:uhcvi=on_60"); // HLL 15 (14)
  sched.push("dis+1011_2:1_cond=fast:ep=RST:fsr=off:gs=on:gsem=off:gsssp=full:nwc=1:sd=1:ss=axioms:sos=on:add=large:aer=off:afr=on:afp=4000:afq=1.1:anc=none:sp=reverse_arity_60"); // HLL 16 (13)
  sched.push("dis+1011_1_cond=fast:ep=RST:gs=on:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:sac=on:amm=sco:anc=none:urr=ec_only_60"); // HLL 17 (12)
  sched.push("lrs+10_4_bd=off:bsr=unit_only:fde=none:gs=on:lcm=reverse:nwc=1:stl=300:sd=3:ss=axioms:st=3.0:sos=on:av=off:uhcvi=on_60"); // HLL 18 (11)
  sched.push("dis+1002_7_bsr=unit_only:cond=fast:nm=64:nwc=1:sd=1:ss=axioms:sos=on:sac=on:afr=on:afp=100000:afq=1.4:anc=none:uhcvi=on_60"); // HLL 19 (11)
  sched.push("lrs+10_5_bd=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:nwc=1:stl=300:sd=2:ss=axioms:sos=on:av=off:urr=on:updr=off:uhcvi=on_60"); // HLL 20 (10)
  sched.push("dis+1002_3_gs=on:gsem=off:nwc=1.2:sd=2:ss=axioms:st=3.0:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // ISA 1 (1149)
  sched.push("lrs+10_3_ep=RS:gs=on:gsem=off:nm=1024:nwc=1:stl=300:sd=2:ss=priority:sos=all:av=off_60"); // HH4 1 (4899)
  sched.push("ott-11_8:1_bsr=unit_only:lcm=predicate:nwc=1:sd=2:ss=axioms:st=1.2:sos=all:av=off:sp=reverse_arity_60"); // MZR 1 (828)
  sched.push("dis+2_4_bd=off:cond=fast:fsr=off:fde=none:gs=on:gsem=on:lcm=reverse:lwlo=on:nwc=1:sd=3:ss=axioms:st=1.5:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // HLL 21 (9)
  sched.push("dis+1011_3:2_bsr=unit_only:cond=fast:nwc=3:nicw=on:sd=3:ss=priority:add=off:afr=on:afp=10000:afq=1.2:uhcvi=on_60"); // ISA 2 (347)
  sched.push("dis+11_4_cond=on:gsp=input_only:gs=on:nm=0:nwc=1:sd=2:ss=axioms:st=1.5:sos=on:av=off:urr=on:updr=off:uhcvi=on_60"); // HH4 2 (1018)
  sched.push("ott+1010_3:1_bs=unit_only:bsr=unit_only:br=off:ccuc=first:cond=fast:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:sos=on:sac=on:aac=none:acc=on:aer=off:afp=1000:afq=2.0:anc=all_dependent:sp=reverse_arity:urr=on:updr=off_60"); // MZR 2 (112)
  sched.push("lrs+1010_5:1_bs=unit_only:bsr=on:fde=none:gs=on:gsem=off:gsssp=full:lcm=reverse:nm=0:nwc=4:stl=300:sd=3:ss=priority:st=1.2:sos=on:aac=none:acc=model:afr=on:afp=1000:afq=1.0:amm=off:urr=on:uhcvi=on_60"); // HLL 22 (8)
  sched.push("lrs+1010_1_cond=on:fde=none:gs=on:gsem=off:nwc=1:stl=300:sd=1:ss=axioms:st=3.0:sos=on:sac=on:afp=10000:afq=1.1:amm=sco:anc=none:urr=on:updr=off_60"); // ISA 3 (174)
  sched.push("lrs+11_2:3_br=off:cond=on:fde=none:gs=on:gsem=on:lwlo=on:nm=64:nwc=1:stl=300:sd=1:ss=axioms:st=2.0:sos=all:av=off:sp=occurrence:urr=on:updr=off_60"); // HH4 3 (356)
  sched.push("ins+11_3_cond=fast:igbrr=0.7:igpr=on:igrr=1/32:igrp=700:igrpq=1.5:igs=1003:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // MZR 3 (81)
  sched.push("lrs-11_8:1_bsr=on:cond=on:fde=none:lcm=reverse:nm=0:nwc=1.5:stl=300:sd=2:ss=priority:av=off:sp=occurrence_60"); // HLL 23 (7)
  sched.push("lrs-2_3_ep=RS:gs=on:gsaa=from_current:nwc=1:stl=300:sd=2:ss=axioms:sos=on:sac=on:afr=on:afp=40000:afq=1.0:amm=off:anc=none:sp=reverse_arity:uhcvi=on_60"); // ISA 4 (93)
  sched.push("dis+1002_4_cond=fast:ep=RST:fde=unused:gs=on:gsaa=from_current:gsem=off:nm=0:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:sac=on:add=large:afp=100000:afq=1.0:amm=sco:anc=none:updr=off:uhcvi=on_60"); // HH4 4 (230)
  sched.push("lrs+1004_2_bd=off:ccuc=small_ones:gs=on:gsem=off:gsssp=full:lwlo=on:nm=0:nwc=1:stl=300:sd=4:ss=priority:st=5.0:sos=all:sac=on:acc=on:add=off:aer=off:afp=100000:afq=1.2:anc=none:sp=occurrence:updr=off:uhcvi=on_60"); // MZR 4 (47)
  sched.push("dis+1002_3_cond=on:ep=RS:fsr=off:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=5:ss=axioms:st=2.0:sos=on:afp=4000:afq=1.4:amm=off:anc=none:updr=off_60"); // HLL 24 (7)
  sched.push("dis+1011_5_fsr=off:fde=unused:nm=64:nwc=3:sd=2:ss=priority:av=off:sp=occurrence:uhcvi=on_60"); // ISA 5 (58)
  sched.push("lrs+1011_1_cond=fast:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:gsssp=full:nm=0:nwc=10:stl=300:sd=1:ss=axioms:st=5.0:av=off:sp=occurrence:urr=on_60"); // HH4 5 (179)
  sched.push("dis+10_5_bsr=unit_only:cond=on:ep=RS:fde=unused:nm=0:nwc=1:sd=1:ss=axioms:sos=all:av=off_60"); // MZR 5 (37)
  sched.push("dis+1003_5_cond=on:fsr=off:fde=none:gs=on:gsem=off:nwc=1:sos=on:add=large:aer=off:afr=on:afp=100000:afq=1.0:anc=all_dependent:sp=reverse_arity:urr=ec_only:uhcvi=on_60"); // HLL 25 (6)
  sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=1:ss=axioms:sos=on:sac=on:afr=on:afp=1000:afq=1.2:amm=sco:anc=none:sp=occurrence:uhcvi=on_60"); // ISA 6 (52)
  sched.push("ott+2_2:1_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:sd=3:ss=priority:st=1.5:sos=on:av=off:sp=occurrence:updr=off_60"); // HH4 6 (138)
  sched.push("lrs+11_5:1_br=off:cond=fast:fde=unused:gsp=input_only:gs=on:gsem=on:gsssp=full:lcm=predicate:nm=0:nwc=1:nicw=on:stl=300:sd=1:ss=axioms:st=1.2:sac=on:add=large:afr=on:afp=40000:afq=1.4:amm=sco:anc=all:urr=on_60"); // MZR 6 (32)
  sched.push("lrs+10_5:4_bd=off:gs=on:gsssp=full:lcm=reverse:nwc=1:stl=300:sd=1:ss=axioms:sos=on:sac=on:add=off:afr=on:afp=10000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:updr=off:uhcvi=on_60"); // HLL 26 (6)
  sched.push("dis+1002_4_ep=RST:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sd=4:ss=axioms:st=1.5:sos=on:aer=off:afr=on:afp=40000:afq=1.2:anc=none_60"); // ISA 7 (39)
  sched.push("lrs+11_5:4_bd=off:bsr=unit_only:br=off:fsr=off:fde=none:gsp=input_only:gs=on:gsem=on:nm=0:nwc=1:stl=300:sd=1:ss=axioms:sos=on:add=off:afp=40000:afq=1.4:amm=sco:urr=on:updr=off:uhcvi=on_60"); // HH4 7 (120)
  sched.push("lrs+1011_8:1_cond=on:fde=none:gsp=input_only:lwlo=on:nwc=1:stl=300:sd=2:ss=axioms:sos=all:av=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_60"); // MZR 7 (22)
  sched.push("lrs+11_4:1_br=off:cond=on:fsr=off:fde=unused:gsp=input_only:gs=on:gsssp=full:lcm=predicate:nm=0:nwc=1:stl=300:sd=1:ss=axioms:av=off:sp=occurrence:urr=on_60"); // HLL 27 (6)
  sched.push("lrs+1011_3:2_bd=off:cond=on:gsp=input_only:gs=on:gsem=on:nm=0:nwc=4:stl=300:sd=1:ss=axioms:aer=off:afr=on:afp=40000:afq=1.1:anc=all_dependent:sp=reverse_arity:updr=off_60"); // ISA 8 (34)
  sched.push("ott+1011_1_bd=preordered:cond=on:gsp=input_only:nm=64:nwc=1:sd=3:ss=priority:av=off:sp=reverse_arity:urr=on_60"); // HH4 8 (90)
  sched.push("lrs+11_3_br=off:cond=fast:gs=on:gsem=off:nwc=1:stl=300:sd=3:ss=priority:st=1.5:sos=all:sac=on:afr=on:afp=1000:afq=2.0:amm=sco:anc=none:sp=occurrence:urr=on:uhcvi=on_60"); // MZR 8 (21)
  sched.push("lrs+1010_5_cond=fast:ep=RST:gs=on:gsaa=from_current:gsem=on:nwc=1:stl=300:sd=4:ss=axioms:st=1.5:sos=on:sac=on:add=off:afp=4000:afq=2.0:amm=sco:anc=none:sp=reverse_arity:uhcvi=on_60"); // HLL 28 (6)
  sched.push("dis+1011_1_bsr=on:ccuc=first:nm=0:nwc=4:sd=2:ss=priority:acc=model:add=large:afr=on:amm=off:anc=none:updr=off:uhcvi=on_60"); // ISA 9 (32)
  sched.push("ins+11_5_cond=fast:ep=RST:gs=on:gsem=on:igbrr=0.4:igpr=on:igrr=1/64:igrp=4000:igrpq=1.3:igwr=on:lcm=reverse:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // HH4 9 (81)
  sched.push("dis+10_2:1_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsaa=full_model:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:add=off:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:urr=on:updr=off:uhcvi=on_60"); // MZR 9 (19)
  sched.push("lrs+11_3_bd=off:cond=fast:fde=none:gsp=input_only:gs=on:gsaa=from_current:gsem=on:gsssp=full:nwc=1:stl=300:sd=1:ss=axioms:sos=all:add=large:aer=off:afr=on:afp=4000:afq=2.0:anc=none:sp=occurrence:urr=on:updr=off_60"); // HLL 29 (5)
  sched.push("lrs+1002_4_bd=off:fde=none:gs=on:gsaa=from_current:gsem=off:gsssp=full:nwc=1:stl=300:sd=3:ss=axioms:st=5.0:sos=on:aer=off:afp=100000:afq=1.1:anc=none:sp=reverse_arity_60"); // ISA 10 (29)
  sched.push("lrs+11_5_cond=on:ep=RST:fde=none:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:stl=300:sd=2:ss=axioms:st=3.0:sos=all:sac=on:add=large:afp=40000:afq=1.4:amm=off:anc=none:urr=ec_only:uhcvi=on_60"); // HH4 10 (70)
  sched.push("lrs+1002_1_bsr=unit_only:nwc=1:stl=300:sd=1:ss=axioms:st=1.5:sos=all:av=off:updr=off:uhcvi=on_60"); // MZR 10 (14)
  sched.push("lrs+4_5:4_bd=off:bs=on:bsr=unit_only:cond=fast:fde=unused:gs=on:gsem=off:nwc=1:stl=300:sd=2:ss=axioms:st=2.0:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // HLL 30 (5)
  sched.push("dis+1002_5_cond=fast:fsr=off:fde=none:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sd=1:ss=axioms:st=5.0:sos=on:sac=on:add=large:afp=40000:afq=1.1:amm=off:anc=none:sp=reverse_arity:updr=off_60"); // ISA 11 (25)
  sched.push("lrs+1011_3_bd=off:bsr=on:cond=fast:fde=none:gs=on:gsssp=full:nm=0:nwc=1:stl=300:sd=2:ss=axioms:sos=all:av=off:sp=occurrence_60"); // HH4 11 (58)
  sched.push("lrs+1_1_bs=on:bsr=on:br=off:cond=fast:fsr=off:gs=on:gsem=off:lwlo=on:nwc=3:stl=300:sd=3:ss=priority:add=large:afr=on:afp=40000:afq=1.4:amm=off:anc=none:sp=occurrence:urr=on:updr=off_60"); // MZR 11 (11)
  sched.push("lrs+11_5:1_bd=off:br=off:cond=fast:gsp=input_only:gs=on:gsem=on:gsssp=full:lcm=predicate:nwc=1.1:stl=300:sd=2:ss=priority:st=3.0:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // HLL 31 (5)
  sched.push("dis+1011_3_fde=unused:nm=64:nwc=1:sd=2:ss=axioms:st=5.0:add=off:aer=off:afp=10000:afq=1.0:sp=occurrence_60"); // ISA 12 (20)
  sched.push("lrs-4_5:4_cond=on:gs=on:gsem=on:gsssp=full:nm=64:nwc=1:stl=300:sd=2:ss=axioms:st=2.0:sos=on:sac=on:afp=100000:afq=1.1:amm=sco:anc=none:urr=on_60"); // HH4 12 (44)
  sched.push("lrs-2_1_cond=on:fde=unused:gs=on:gsaa=from_current:gsssp=full:lcm=predicate:nwc=1:stl=300:sd=4:ss=axioms:st=3.0:sos=on:sac=on:afr=on:afp=10000:afq=1.1:anc=none:updr=off_60"); // MZR 12 (11)
  sched.push("dis+1011_3:2_cond=fast:ep=RST:fsr=off:fde=unused:gsp=input_only:gs=on:gsem=off:nm=0:nwc=1:sd=1:ss=priority:sos=all:add=large:anc=all:sp=occurrence_60"); // HLL 32 (5)
  sched.push("dis+1011_3:1_cond=fast:ep=RS:nm=0:nwc=1.7:sd=2:ss=priority:st=1.2:add=off:afp=10000:afq=1.2:amm=sco:anc=all:sp=occurrence:updr=off:uhcvi=on_60"); // ISA 13 (16)
  sched.push("dis+1011_3:1_br=off:nm=0:nwc=5:sd=1:ss=axioms:sac=on:afp=40000:afq=1.4:amm=sco:anc=none:urr=on:uhcvi=on_60"); // HH4 13 (38)
  sched.push("lrs+10_8:1_bsr=unit_only:br=off:cond=on:fsr=off:gsp=input_only:gs=on:gsaa=from_current:nm=0:nwc=1:stl=300:sd=2:ss=axioms:st=1.2:sos=on:sac=on:add=large:afr=on:afp=1000:afq=1.1:amm=sco:anc=none:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // MZR 13 (10)
  sched.push("lrs+11_3:1_br=off:cond=fast:ep=R:fsr=off:gs=on:nwc=1:stl=300:sd=2:ss=priority:st=2.0:sos=all:av=off:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HLL 33 (4)
  sched.push("dis+1002_5_cond=on:ep=RST:fsr=off:fde=unused:gs=on:gsem=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:add=off:afr=on:amm=sco:anc=none:updr=off:uhcvi=on_60"); // ISA 14 (16)
  sched.push("lrs+11_3:1_bd=off:bsr=unit_only:fsr=off:gs=on:gsaa=from_current:gsem=off:nm=64:nwc=1:stl=300:sd=2:ss=priority:sac=on:amm=sco:anc=none:sp=reverse_arity:updr=off:uhcvi=on_60"); // HH4 14 (36)
  sched.push("dis+11_12_cond=fast:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // MZR 14 (8)
  sched.push("lrs+11_3_bsr=unit_only:cond=on:ep=RST:gsp=input_only:nwc=1.7:stl=300:sd=3:ss=axioms:st=5.0:sos=all:sac=on:afr=on:afp=100000:afq=1.1:anc=all_dependent:sp=reverse_arity:updr=off:uhcvi=on_60"); // HLL 34 (4)
  sched.push("dis+1011_5_cond=on:er=filter:fde=none:nm=64:nwc=3:sd=2:ss=priority:st=2.0:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // ISA 15 (12)
  sched.push("dis+4_3_bd=off:cond=on:fde=unused:gs=on:gsaa=full_model:gsem=off:gsssp=full:nwc=1:sd=3:ss=axioms:st=3.0:sos=on:add=off:afr=on:afp=10000:afq=1.0:amm=off:anc=none:urr=ec_only:updr=off:uhcvi=on_60"); // HH4 15 (32)
  sched.push("dis+1010_3_bsr=unit_only:cond=fast:fde=none:nwc=1:sd=2:ss=axioms:st=1.5:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // MZR 15 (8)
  sched.push("ins+11_5_ep=RS:fsr=off:gs=on:igbrr=0.4:igpr=on:igrr=1/2:igrp=4000:igrpq=1.2:igs=1010:igwr=on:nwc=1:sd=1:ss=axioms:st=3.0:sos=all:av=off:sp=reverse_arity:urr=on:updr=off_60"); // HLL 35 (3)
  sched.push("lrs+10_3:1_cond=on:fde=none:gs=on:gsem=off:gsssp=full:nwc=1.2:stl=300:sd=1:ss=priority:sos=on:sac=on:add=off:afp=1000:afq=1.4:amm=sco:anc=all:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // ISA 16 (12)
  sched.push("dis+1010_5_cond=fast:nm=0:nwc=1:sd=1:ss=axioms:st=1.5:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // HH4 16 (32)
  sched.push("dis+1002_2:3_fde=none:gsp=input_only:nm=0:nwc=1:sd=3:ss=axioms:sos=on:sac=on:afp=100000:afq=1.0:amm=sco:anc=none:sp=occurrence:updr=off_60"); // MZR 16 (7)
  sched.push("dis+1010_2:3_bs=unit_only:bsr=unit_only:cond=fast:fsr=off:fde=unused:gs=on:gsem=on:gsssp=full:nm=0:nwc=3:sd=4:ss=priority:st=1.5:sos=on:acc=on:add=off:aer=off:afr=on:afp=100000:afq=1.0:sp=reverse_arity:uhcvi=on_60"); // HLL 36 (3)
  sched.push("lrs+11_5_br=off:cond=on:fde=none:gs=on:nwc=1:stl=300:sd=2:ss=axioms:st=3.0:sos=all:add=off:afr=on:afp=40000:afq=1.4:anc=none:sp=reverse_arity:urr=on_60"); // ISA 17 (10)
  sched.push("lrs+11_4:1_bd=off:bsr=unit_only:br=off:fsr=off:fde=unused:gsp=input_only:gs=on:nm=0:nwc=1:stl=300:sd=1:ss=axioms:sos=on:av=off:sp=reverse_arity:urr=on_60"); // HH4 17 (29)
  sched.push("lrs+10_3:1_fde=unused:lcm=reverse:nwc=1:stl=300:sd=3:ss=priority:st=2.0:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // MZR 17 (6)
  sched.push("dis+1010_5:4_bd=off:fsr=off:fde=unused:gs=on:nm=64:nwc=1:sd=4:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // HLL 37 (3)
  sched.push("dis+1002_3_bd=off:fde=unused:gs=on:gsaa=from_current:gsem=off:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:afr=on:amm=sco:anc=none:sp=occurrence_60"); // ISA 18 (10)
  sched.push("dis+1002_4_cond=on:gs=on:gsem=off:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity:urr=ec_only:updr=off:uhcvi=on_60"); // HH4 18 (28)
  sched.push("lrs+10_2:3_bsr=unit_only:cond=on:fde=none:gs=on:nwc=1:stl=300:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity_60"); // MZR 18 (6)
  sched.push("lrs+1011_8:1_bd=off:bsr=unit_only:fde=none:gs=on:lwlo=on:nm=0:nwc=1.5:stl=300:sd=1:ss=axioms:st=1.2:av=off:sp=occurrence:updr=off_60"); // HLL 38 (3)
  sched.push("ins+11_4_cond=fast:fde=none:igbrr=0.4:igrr=1/32:igrp=200:igrpq=1.2:igs=1003:igwr=on:nwc=10:sd=1:ss=axioms:st=5.0:av=off_60"); // ISA 19 (10)
  sched.push("lrs+11_2:3_cond=on:fde=unused:gs=on:gsaa=full_model:nwc=4:stl=300:sd=2:ss=priority:st=5.0:sac=on:add=off:afr=on:amm=off:anc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HH4 19 (24)
  sched.push("dis+1004_3:1_bsr=unit_only:ep=R:fde=unused:gs=on:gsssp=full:nm=0:nwc=1:sos=all:sac=on:afr=on:afp=10000:afq=2.0:anc=all:sp=reverse_arity:urr=on:updr=off_60"); // MZR 19 (5)
  sched.push("dis+4_5:4_bd=off:fsr=off:fde=unused:gs=on:nwc=1:sd=5:ss=axioms:st=1.5:sos=all:av=off:sp=occurrence:uhcvi=on_60"); // HLL 39 (3)
  sched.push("lrs+1011_4:1_fsr=off:fde=unused:gs=on:gsem=on:gsssp=full:nm=64:nwc=4:stl=300:sd=1:ss=priority:st=3.0:aac=none:acc=on:afr=on:afp=40000:afq=1.2:amm=sco:anc=all:sp=reverse_arity:updr=off:uhcvi=on_60"); // ISA 20 (9)
  sched.push("ott-11_8:1_bsr=unit_only:cond=on:gs=on:gsem=off:gsssp=full:nwc=1.1:sd=2:ss=axioms:sos=on:sac=on:acc=model:add=large:aer=off:afp=40000:afq=2.0:anc=none:sp=reverse_arity:urr=on_60"); // HH4 20 (23)
  sched.push("ott+4_5:1_br=off:cond=fast:fsr=off:nwc=1.3:av=off:sp=occurrence:urr=on:uhcvi=on_60"); // MZR 20 (5)
  sched.push("dis+1011_3_cond=fast:ep=R:gs=on:gsem=off:lwlo=on:nm=0:nwc=1:sd=5:ss=axioms:st=1.5:sos=on:sac=on:add=large:afr=on:afp=1000:afq=1.1:anc=none:uhcvi=on_60"); // HLL 40 (2)
  sched.push("dis+1002_50_fde=unused:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity:uhcvi=on_60"); // ISA 21 (8)
  sched.push("lrs+1010_2:1_gs=on:lwlo=on:nm=0:nwc=3:stl=300:sd=4:ss=axioms:av=off_60"); // HH4 21 (22)
  sched.push("dis+1010_1_cond=fast:fsr=off:nwc=1.3:sd=2:ss=axioms:st=1.5:sos=on:acc=model:add=off:afp=4000:afq=2.0:uhcvi=on_60"); // MZR 21 (5)
  sched.push("lrs+1004_3:1_bd=off:bsr=unit_only:cond=fast:fde=none:gs=on:gsem=off:gsssp=full:nwc=1:stl=300:sd=2:ss=axioms:sos=on:av=off:updr=off_60"); // HLL 41 (2)
  sched.push("ott+11_4_cond=fast:fde=none:gs=on:gsem=on:gsssp=full:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // ISA 22 (8)
  sched.push("lrs+1010_4_bsr=unit_only:cond=fast:fsr=off:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:stl=300:sd=1:ss=axioms:st=1.5:sos=on:add=off:aer=off:afr=on:afp=10000:afq=1.0:anc=none:sp=occurrence:urr=on_60"); // HH4 22 (20)
  sched.push("lrs+11_2_bd=off:bsr=unit_only:cond=on:lcm=predicate:lwlo=on:nm=64:nwc=1.1:stl=300:sd=1:ss=axioms:st=2.0:sos=all:av=off_60"); // MZR 22 (5)
  sched.push("lrs+10_1_bd=off:bsr=unit_only:cond=fast:fsr=off:fde=unused:gs=on:gsem=off:lcm=reverse:nwc=1:stl=300:sd=3:ss=axioms:st=1.5:av=off:sp=reverse_arity:urr=on_60"); // HLL 42 (2)
  sched.push("dis-3_3_ep=RST:fde=none:nm=64:nwc=1:sd=4:ss=axioms:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // ISA 23 (7)
  sched.push("dis+2_1_fsr=off:nwc=1:sd=2:ss=axioms:sos=on:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // HH4 23 (17)
  sched.push("lrs+10_4:1_bd=off:cond=fast:fde=unused:lcm=reverse:nm=0:nwc=1.2:stl=300:sd=2:ss=axioms:sos=all:av=off_60"); // MZR 23 (5)
  sched.push("lrs+10_4_bd=off:bsr=unit_only:cond=on:gs=on:nwc=1:stl=300:sd=4:ss=axioms:st=5.0:sos=all:av=off:sp=reverse_arity:updr=off:uhcvi=on_60"); // HLL 43 (2)
  sched.push("dis+1010_7_fsr=off:fde=unused:nm=0:nwc=1.3:nicw=on:sd=3:ss=priority:afr=on:afp=100000:afq=1.0:amm=sco:anc=none:updr=off:uhcvi=on_60"); // ISA 24 (7)
  sched.push("ott+2_2:1_cond=fast:fsr=off:fde=unused:gs=on:gsem=off:nm=0:nwc=1:sd=1:ss=axioms:st=5.0:sos=all:av=off:sp=occurrence:updr=off:uhcvi=on_60"); // HH4 24 (17)
  sched.push("dis+10_5_ep=RST:fsr=off:gs=on:gsssp=full:lwlo=on:nm=0:nwc=1:sd=4:ss=axioms:sos=on:afr=on:afp=40000:afq=1.1:amm=off:anc=none:uhcvi=on_60"); // MZR 24 (4)
  sched.push("ins+11_4_ep=RS:igbrr=0.6:igpr=on:igrr=1/128:igrp=2000:igrpq=2.0:igs=1002:igwr=on:nwc=1:sd=1:ss=axioms:sos=all:av=off:uhcvi=on_60"); // HLL 44 (2)
  sched.push("dis+1002_4_cond=fast:ep=RST:fsr=off:nwc=1:sd=3:ss=axioms:st=2.0:sos=on:afp=10000:afq=1.1:amm=sco:sp=occurrence:uhcvi=on_60"); // ISA 25 (6)
  sched.push("lrs+1003_8:1_br=off:cond=on:fde=none:gs=on:gsem=off:nm=0:nwc=1:stl=300:sd=1:ss=axioms:sos=on:add=off:afr=on:afp=40000:afq=1.1:amm=off:anc=none:sp=occurrence:urr=on_60"); // HH4 25 (14)
  sched.push("dis+1002_3_ep=RST:fde=unused:gs=on:gsaa=full_model:gsem=off:nwc=1:sd=1:ss=axioms:st=2.0:sos=on:afp=100000:afq=1.1:anc=none:sp=occurrence:uhcvi=on_60"); // MZR 25 (4)
  sched.push("lrs+1010_2:3_bsr=unit_only:ccuc=small_ones:cond=on:fde=unused:gs=on:gsaa=full_model:nwc=1:stl=300:sd=2:ss=axioms:st=1.5:sos=on:sac=on:acc=model:add=off:aer=off:afr=on:afp=1000:afq=2.0:sp=occurrence:uhcvi=on_60"); // HLL 45 (2)
  sched.push("ott+1011_2_bd=off:ccuc=first:cond=on:fsr=off:fde=unused:gs=on:gsaa=from_current:gsem=on:nm=64:nwc=1.3:sd=3:ss=priority:st=1.2:sac=on:acc=on:add=off:afp=4000:afq=1.4:amm=sco:anc=none:urr=ec_only:updr=off:uhcvi=on_60"); // ISA 26 (6)
  sched.push("dis+11_2:1_bd=off:cond=fast:fde=unused:gs=on:gsem=on:nwc=1:sd=3:ss=axioms:st=2.0:sos=all:av=off:sp=occurrence_60"); // HH4 26 (14)
  sched.push("dis+10_5_cond=on:fsr=off:fde=none:gs=on:nwc=1:sd=2:ss=axioms:st=3.0:sos=on:av=off_60"); // MZR 26 (4)
  sched.push("dis+10_1_bd=preordered:bs=unit_only:cond=on:fde=none:lcm=predicate:nwc=1:sd=2:ss=axioms:sos=all:sac=on:afr=on:amm=sco:anc=none:updr=off:uhcvi=on_60"); // HLL 46 (2)
  sched.push("dis+1002_3_bd=off:gs=on:gsem=on:nwc=1.1:sd=7:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:updr=off_60"); // ISA 27 (5)
  sched.push("lrs+1011_3:1_bd=off:cond=fast:fsr=off:fde=unused:gs=on:nm=0:nwc=5:stl=300:sd=2:ss=axioms:afp=100000:afq=1.4:amm=off:anc=none:sp=occurrence:urr=on_60"); // HH4 27 (14)
  sched.push("ott-11_8:1_bd=preordered:ccuc=first:er=known:fsr=off:fde=unused:gsp=input_only:lcm=predicate:nm=0:nwc=2:sd=3:ss=axioms:acc=on:afp=10000:afq=2.0:amm=sco:sp=occurrence:updr=off_60"); // MZR 27 (4)
  sched.push("lrs+11_5_bd=off:bsr=unit_only:cond=on:fsr=off:gs=on:gsaa=from_current:gsssp=full:nwc=1:stl=300:sd=1:ss=axioms:st=5.0:sos=all:add=off:afp=4000:afq=2.0:amm=off:anc=none:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HLL 47 (2)
  sched.push("dis+11_2:3_cond=on:gs=on:gsem=off:gsssp=full:lcm=reverse:nwc=1:sd=1:ss=axioms:st=1.5:add=off:aer=off:afr=on:afp=1000:afq=2.0:anc=all_dependent:sp=reverse_arity:updr=off:uhcvi=on_60"); // ISA 28 (5)
  sched.push("dis+1011_3_cond=on:nwc=1:sd=2:ss=axioms:st=1.2:sos=on:sac=on:afr=on:afp=1000:afq=1.4:anc=none:sp=occurrence:updr=off:uhcvi=on_60"); // HH4 28 (13)
  sched.push("ins+11_4_bd=off:fsr=off:gsp=input_only:gs=on:gsem=off:igbrr=0.6:igpr=on:igrr=1/128:igrp=700:igrpq=1.2:igs=1004:igwr=on:lcm=predicate:nwc=1:sd=2:ss=axioms:st=5.0:sos=on:av=off:uhcvi=on_60"); // MZR 28 (3)
  sched.push("dis+11_2:1_br=off:ep=RST:fde=unused:gsp=input_only:gs=on:gsaa=from_current:gsem=off:nwc=1:sd=1:ss=priority:st=1.2:sos=all:add=large:aer=off:afp=100000:afq=1.1:anc=none:sp=occurrence:urr=on_60"); // HLL 48 (2)
  sched.push("ins+11_10_cond=on:gs=on:igbrr=0.3:igpr=on:igrr=1/32:igrp=100:igrpq=1.1:igs=1010:igwr=on:lcm=reverse:nwc=1.3:sd=1:ss=axioms:st=1.2:sos=on:av=off:sp=reverse_arity:urr=on:updr=off:uhcvi=on_60"); // ISA 29 (5)
  sched.push("dis+11_2:1_fde=none:gsp=input_only:nwc=1:sd=2:ss=axioms:sos=all:av=off:sp=occurrence:urr=on:updr=off:uhcvi=on_60"); // HH4 29 (13)
  sched.push("lrs+1011_1_cond=on:fsr=off:gs=on:nwc=1:stl=300:sd=4:ss=priority:st=1.2:sos=on:av=off:sp=reverse_arity:urr=on_60"); // MZR 29 (3)
  sched.push("dis+1011_2:3_cond=fast:fsr=off:fde=none:gs=on:gsaa=from_current:gsem=on:gsssp=full:nm=0:nwc=1:sd=2:ss=axioms:sos=on:sac=on:add=large:afr=on:afp=1000:afq=2.0:anc=none:sp=reverse_arity:urr=ec_only:uhcvi=on_60"); // HLL 49 (2)
  sched.push("dis+1002_3_cond=fast:ep=RSTC:fsr=off:gs=on:gsem=off:lwlo=on:nwc=1:sd=3:ss=axioms:st=1.2:sos=on:av=off:sp=occurrence:uhcvi=on_60"); // ISA 30 (4)
  sched.push("lrs+11_3_bsr=unit_only:cond=fast:fsr=off:fde=none:gsp=input_only:nwc=5:stl=300:sd=3:ss=priority:st=2.0:av=off:updr=off:uhcvi=on_60"); // HH4 30 (12)
  sched.push("lrs+11_3:1_bsr=unit_only:br=off:cond=on:ep=RST:fsr=off:gs=on:gsaa=from_current:gsem=off:nwc=3:stl=300:sd=2:ss=axioms:st=2.0:sac=on:add=large:afr=on:afp=100000:afq=1.4:amm=sco:anc=none:sp=reverse_arity:urr=on_60"); // MZR 30 (3)
  sched.push("lrs+1003_4_bsr=unit_only:cond=fast:fsr=off:gsp=input_only:gs=on:gsaa=from_current:nm=0:nwc=1:stl=300:sos=on:sac=on:add=large:afp=10000:afq=1.1:anc=none:urr=ec_only:uhcvi=on_60"); // HLL 50 (1)
}
